/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.chart;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Stroke;

import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;

import java.text.NumberFormat;

import java.util.List;
import java.util.StringTokenizer;

import org.ujac.util.BeanUtils;

/**
 * Name: Base2DScaleChart<br>
 * Description: Base class for 2D scale charts like bar and line charts.
 * 
 * @author lauerc
 */
public abstract class Base2DScaleChart extends BaseChart {

  /** The segment mark style for centered lines. */
  public static final int SEGMENT_MARK_STYLE_CENTER = 1;
  /** The segment mark style for border lines. */
  public static final int SEGMENT_MARK_STYLE_BORDER = 2;
  /** The default segment mark style. */
  public static final int DEFAULT_SEGMENT_MARK_STYLE = SEGMENT_MARK_STYLE_BORDER;

  /** The arrowhead style for open arrows. */
  public static final int ARROWHEAD_STYLE_OPEN = 1;
  /** The arrowhead style for filled arrows. */
  public static final int ARROWHEAD_STYLE_FILLED = 2;
  /** The default arrowhead style. */
  public static final int DEFAULT_ARROWHEAD_STYLE = ARROWHEAD_STYLE_OPEN;

  /** The default line width. */
  public static final float DEFAULT_LINE_WIDTH = 1.0F;
  /** The default line width. */
  public static final float DEFAULT_SCALE_MARK_LENGTH = 6.0F;
  /** The default scale arrowhead length. */
  public static final float DEFAULT_SCALE_ARROWHEAD_LENGTH = 10.0F;
  /** The default scale arrowhead style. */
  public static final String DEFAULT_SCALE_ARROWHEAD_STYLE = "open";
  /** The default grid color. */
  public static final Color DEFAULT_GRID_COLOR = Color.gray;
  /** The default grid background color. */
  public static final Color DEFAULT_GRID_BACKGROUND_COLOR = Color.lightGray;

  /** The default description item spacing. */
  public static final float DEFAULT_DESCRIPTION_ITEM_SPACING = 1.5F;
  /** The default description line spacing. */
  public static final float DEFAULT_DESCRIPTION_LINE_SPACING = 1.0F;
  
  /** The Font to use for the grid. */
  protected Font scaleFont = null;
  /** The effective viewport (is determined whenever draw scale is called). */
  protected Rectangle effectiveViewport = null;
  /** The vertical anchor from which to draw the chart. */
  protected float verticalAnchor = 0;
  /** The minimum vertical position. */
  protected float verticalMinimum = 0;
  /** The maximum vertical position. */
  protected float verticalMaximum = 0;
  /** The vertical position of the zero coordinate. */
  protected float verticalZero = 0;

  /** The maximum value. */
  protected double max; 
  /** The minimum value. */
  protected double min; 
  /** The value range. */
  protected double range;
  
  /** The horizontal offset for the chart. */
  protected float horizontalOffset = 0.0F;
  /** The horizontal scale for the chart. */
  protected float horizontalScale = 0.0F;
  /** The vertical scale for the chart values. */
  protected float verticalScale = 0.0F;
  /** The horizontal positions for the values. */
  protected float[] horizontalPositions = null;
  
  /** The maximum segment label height, calculated at method calcViewport. */
  private float maxSegmentLabelHeight = 0.0F;
  /** The segment metrics, calculated at method calcViewport. */
  private Rectangle2D[] segmentMetrics = null;

  /** The maximum area label line height, calculated at method calcViewport. */
  private float maxAreaLabelLineHeight = 0.0F;
  /** The maximum area label height, calculated at method calcViewport. */
  private float maxAreaLabelHeight = 0.0F;
  /** The legend metrics, calculated at method calcViewport. */
  private Rectangle2D legendMetrics = null;
  
  /** The value scale description area. */
  private Rectangle2D valueScaleDescriptionArea = null;
  /** The segment scale description area. */
  private Rectangle2D segmentScaleDescriptionArea = null;
  
  /**
   * Gets the line width.
   * @return The line width according to the attribute ATTR_LINE_WIDTH.
   */
  public float getLineWidth() {
    if (attributes.isDefined(ATTR_LINE_WIDTH)) {
      return attributes.getFloat(ATTR_LINE_WIDTH);      
    }
    return DEFAULT_LINE_WIDTH;
  }

  /**
   * Gets the scale mark length.
   * @return The length of the scale mark lines according to the attribute ATTR_SCALE_MARK_LENGTH.
   */
  public float getScaleMarkLength() {
    if (attributes.isDefined(ATTR_SCALE_MARK_LENGTH)) {
      return attributes.getFloat(ATTR_SCALE_MARK_LENGTH);      
    }
    return DEFAULT_SCALE_MARK_LENGTH;
  }

  /**
   * Gets the value step size according to the attribute ATTR_VALUE_STEP_SIZE
   * or calculates if it's not specified.
   * @param descriptionHeight The description height.
   * @return The value step size.
   */
  public float getValueStepSize(double descriptionHeight) {
    if (attributes.isDefined(ATTR_VALUE_STEP_SIZE)) {
      return attributes.getFloat(ATTR_VALUE_STEP_SIZE);      
    }
    double valueRange = getMaxValue() - getMinValue();
    float stepSize = 1;
    // maximum 50 steps 
    while (valueRange / stepSize > 50) {
      stepSize = stepSize * 10;
    }
    // minimum 10 steps
    if (valueRange / stepSize < 10 && stepSize > 1 &&
      verticalScale * stepSize >= 3 * descriptionHeight) {
      stepSize = stepSize / 2;
    }
    // minimum 1.5*descriptionHeight between steps
    if (verticalScale * stepSize < 3 * descriptionHeight / 20) {
      stepSize = stepSize * 20;
    } else if (verticalScale * stepSize < 3 * descriptionHeight / 10) {
      stepSize = stepSize * 10;
    } else if (verticalScale * stepSize < 3 * descriptionHeight / 5) {
      stepSize = stepSize * 5;
    } else if (verticalScale * stepSize < 3 * descriptionHeight / 2) {
      stepSize = stepSize * 2;
    }
    return stepSize;
  }
  
  /**
   * Gets the 'view-legend' flag.
   * @return The 'view-legend' flag according to the attribute ATTR_VIEW_LEGEND.
   */
  public boolean isViewLegend() {
    if (!attributes.isDefined(ATTR_VIEW_LEGEND)) {
      return true;
    }
    return attributes.getBoolean(ATTR_VIEW_LEGEND);      
  }

  /**
   * Gets the 'view-scale-arrows' flag.
   * @return The 'view-grid' flag according to the attribute ATTR_VIEW_SCALE_ARROWS.
   */
  public boolean isViewScaleArrows() {
    if (!attributes.isDefined(ATTR_VIEW_SCALE_ARROWS)) {
      return true;
    }
    return attributes.getBoolean(ATTR_VIEW_GRID);      
  }

  /**
   * Gets the scale arrowhead length.
   * @return The length of the scale arrowhead according to the attribute ATTR_SCALE_ARROWHEAD_LENGTH.
   */
  public float getScaleArrowheadLength() {
    if (attributes.isDefined(ATTR_SCALE_ARROWHEAD_LENGTH)) {
      return attributes.getFloat(ATTR_SCALE_ARROWHEAD_LENGTH);      
    }
    return DEFAULT_SCALE_ARROWHEAD_LENGTH;
  }

  /**
   * Gets the scale arrowhead width.
   * @return The width of the scale arrowhead according to the attribute ATTR_SCALE_ARROWHEAD_WIDTH.
   */
  public float getScaleArrowheadWidth() {
    if (attributes.isDefined(ATTR_SCALE_ARROWHEAD_WIDTH)) {
      return attributes.getFloat(ATTR_SCALE_ARROWHEAD_WIDTH);      
    }
    return getScaleArrowheadLength() * 0.4F;
  }

  /**
   * Gets the scale arrowhead style.
   * @return The style of the scale arrowhead according to the attribute ATTR_SCALE_ARROWHEAD_STYLE.
   */
  public int getScaleArrowheadStyle() {
    if (attributes.isDefined(ATTR_SCALE_ARROWHEAD_STYLE)) {
      String style = attributes.getString(ATTR_SCALE_ARROWHEAD_STYLE);
      if ("filled".equals(style)) {
        return ARROWHEAD_STYLE_FILLED;      
      }
    }
    return DEFAULT_ARROWHEAD_STYLE;
  }

  /**
   * Gets the number format for scale value output.
   * @return The scale value format.
   */
  public NumberFormat getScaleValueFormat() {
    if (attributes.isDefined(ATTR_SCALE_VALUE_FORMAT)) {
      return (NumberFormat) attributes.getObject(ATTR_SCALE_VALUE_FORMAT);
    }
    if (attributes.isDefined(ATTR_VALUE_FORMAT)) {
      return (NumberFormat) attributes.getObject(ATTR_VALUE_FORMAT);
    }
    return defaultValueFormat;
  }
  
  /**
   * Format a number for scale value output.
   * @param number The number to format.
   * @param stepSize The step size to use.
   * @return The string value of given number.
   */
  public String formatScaleValue(double number, double stepSize) {
    NumberFormat format = defaultValueFormat;
    if (attributes.isDefined(ATTR_SCALE_VALUE_FORMAT)) {
      format = (NumberFormat) attributes.getObject(ATTR_SCALE_VALUE_FORMAT);
    } else if (attributes.isDefined(ATTR_VALUE_FORMAT)) {
      format = (NumberFormat) attributes.getObject(ATTR_VALUE_FORMAT);
    }
    if (stepSize >= 1000 && format == defaultValueFormat) {
      return format.format(number/1000) + "K";
    }
    return format.format(number);
  }

  /**
   * Gets the segment mark style.
   * @return The style of the segment mark lines according to the attribute ATTR_SEGMENT_MARK_STYLE.
   */
  public int getSegmentMarkStyle() {
    if (attributes.isDefined(ATTR_SEGMENT_MARK_STYLE)) {
      String style = attributes.getString(ATTR_SEGMENT_MARK_STYLE);
      if ("border".equals(style)) {
        return SEGMENT_MARK_STYLE_BORDER;      
      }
      return SEGMENT_MARK_STYLE_CENTER;      
    }
    return DEFAULT_SEGMENT_MARK_STYLE;
  }
  /**
   * Gets the 'view-grid' flag.
   * @return The 'view-grid' flag according to the attribute ATTR_VIEW_GRID.
   */
  public boolean isViewGrid() {
    if (!attributes.isDefined(ATTR_VIEW_GRID)) {
      return true;
    }
    return attributes.getBoolean(ATTR_VIEW_GRID);      
  }

  /**
   * Gets the grid color.
   * @return The current grid color.
   */
  public Color getGridColor() {
    if (attributes.isDefined(ATTR_GRID_COLOR)) {
      return attributes.getColor(ATTR_GRID_COLOR);      
    }
    return DEFAULT_GRID_COLOR;
  }
  
  /**
   * Gets the grid background color.
   * @return The current grid background color.
   */
  public Color getGridBackgroundColor() {
    if (attributes.isDefined(ATTR_GRID_BACKGROUND_COLOR)) {
      return attributes.getColor(ATTR_GRID_BACKGROUND_COLOR);      
    }
    return DEFAULT_GRID_BACKGROUND_COLOR;
  }

  /**
   * Gets the inner padding.
   * @return The inner padding.
   */
  protected int getInnerPadding() {
    return 10;
  }
  
  /**
   * Gets the value scale description.
   * @return The ialue scale description text.
   */
  protected TextChartItem getValueScaleDescription() {
    return attributes.getTextItem(ATTR_VALUE_SCALE_DESCRIPTION);      
  }

  /**
   * Gets the segment scale description.
   * @return The ialue scale description text.
   */
  protected TextChartItem getSegmentScaleDescription() {
    return attributes.getTextItem(ATTR_SEGMENT_SCALE_DESCRIPTION);      
  }

  /**
   * Draws the scale.
   * @param viewport The viewport.
   * @param effectiveViewport The effective viewport for the scale.
   * @param graphics The graphics object to use.
   */
  protected void drawScale(Rectangle2D viewport, Rectangle2D effectiveViewport, Graphics2D graphics) {

    Color scaleColor = getDescriptionColor();
    Color backgroundColor = getGridBackgroundColor();
    Color gridColor = getGridColor();

    float halfScaleLength = (getScaleMarkLength() / 2.0F);
    float x = (float) effectiveViewport.getX();
    float width = (float) effectiveViewport.getWidth();
  
    float arrowHeadLength = getScaleArrowheadLength();
    float arrowHeadWidth = getScaleArrowheadWidth();
    int arrowHeadStyle = getScaleArrowheadStyle();
    int segmentMarkStyle = getSegmentMarkStyle();
    
    List chartData = model.getChartData();
    int numSegments = chartData.size();

    Font oldFont = graphics.getFont();
    Stroke oldStroke = graphics.getStroke();
    Stroke gridStroke = new BasicStroke(getDescriptionLineWidth() / 2);
    Stroke scaleStroke = new BasicStroke(getDescriptionLineWidth(), BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
    graphics.setStroke(scaleStroke);
    
    String[] segmentLabels = getSegmentLabels();

    TextChartItem valueScaleDescription = getValueScaleDescription();
    if (valueScaleDescription != null) {
      valueScaleDescription.draw(valueScaleDescriptionArea, graphics);
    }

    TextChartItem segmentScaleDescription = getSegmentScaleDescription();
    if (segmentScaleDescription != null) {
      segmentScaleDescription.draw(segmentScaleDescriptionArea, graphics);
    }

    Font segmentDescriptionFont = getDescriptionFont();
    segmentDescriptionFont = segmentDescriptionFont.deriveFont(AffineTransform.getRotateInstance(Math.PI * 1.5));
    graphics.setFont(segmentDescriptionFont);
    
    Rectangle2D segmentDescriptionArea = new Rectangle2D.Float(
        x, verticalMinimum + halfScaleLength, width, (int) maxSegmentLabelHeight
    );

    boolean viewGrid = isViewGrid();
    if (viewGrid) {
      // drawing grid background
      graphics.setColor(backgroundColor);
      graphics.fill(new Rectangle2D.Float(
          x, verticalMaximum, width, 
          verticalMinimum - verticalMaximum
      ));
    }
    
    graphics.setColor(scaleColor);
    // drawing the horizontal scale
    float vsMaxX = (float) (viewport.getX() + viewport.getWidth());
    float maxGridX = x + width;
    if (isViewScaleArrows()) {
      if (arrowHeadStyle == ARROWHEAD_STYLE_FILLED) {
        GeneralPath arrowPoly = new GeneralPath();
        arrowPoly.moveTo(vsMaxX - arrowHeadLength, verticalMinimum - arrowHeadWidth);
        arrowPoly.lineTo(vsMaxX, verticalMinimum);
        arrowPoly.lineTo(vsMaxX - arrowHeadLength, verticalMinimum + arrowHeadWidth);
        graphics.fill(arrowPoly);
        graphics.draw(arrowPoly);
      } else {
        graphics.draw(new Line2D.Float(
            vsMaxX - arrowHeadLength, verticalMinimum - arrowHeadWidth, 
            vsMaxX, verticalMinimum
        ));
        graphics.draw(new Line2D.Float(
            vsMaxX - arrowHeadLength, verticalMinimum + arrowHeadWidth, 
            vsMaxX, verticalMinimum
        ));
      }
      graphics.draw(new Line2D.Float(
          x, verticalMinimum, 
          vsMaxX, verticalMinimum
      ));
    } else {
      graphics.draw(new Line2D.Float(
          x, verticalMinimum, maxGridX, verticalMinimum
      ));
    }
    
    // drawing the vertical scale
    int vsMinY = (int) viewport.getY();
    if (isViewScaleArrows()) {
      graphics.draw(new Line2D.Float(
          x, vsMinY, x, verticalMinimum
      ));
      if (arrowHeadStyle == ARROWHEAD_STYLE_FILLED) {
        GeneralPath arrowPoly = new GeneralPath();
        arrowPoly.moveTo(x - arrowHeadWidth, vsMinY + arrowHeadLength);
        arrowPoly.lineTo(x, vsMinY);
        arrowPoly.lineTo(x + arrowHeadWidth, vsMinY + arrowHeadLength);
        graphics.fill(arrowPoly);
        graphics.draw(arrowPoly);
      } else {
        graphics.draw(new Line2D.Float(x, vsMinY, x - arrowHeadWidth, vsMinY + arrowHeadLength));
        graphics.draw(new Line2D.Float(x, vsMinY, x + arrowHeadWidth, vsMinY + arrowHeadLength));
      }
    } else {
      graphics.draw(new Line2D.Float(x, verticalMaximum, x, verticalMinimum));
    }
    
    // drawing the horizontal descriptions
    this.horizontalPositions = new float[numSegments];
    float hScaleYfrom = verticalMinimum - halfScaleLength;
    float hScaleYto = verticalMinimum + halfScaleLength;
    float halfHorizontalScale = horizontalScale / 2.0F;
    float lastPos = horizontalOffset + halfHorizontalScale;
    
    double segXPos = 0.0;
    switch (segmentMarkStyle) {
      case SEGMENT_MARK_STYLE_BORDER:
        segXPos = (int) effectiveViewport.getMinX();
        break;
        
      case SEGMENT_MARK_STYLE_CENTER:
        segXPos = (int) lastPos;
        break;
        
      default: // same as center
        segXPos = (int) lastPos;
        break;
    }
    
    graphics.setFont(segmentDescriptionFont);
    for (int i = 0; i < numSegments; i++) {
      String segmentLabel = null;
      if ((segmentLabels != null) && (i < segmentLabels.length)) {
        segmentLabel = segmentLabels[i];
      }
      int xPos = (int) lastPos;  
      int xSegPos = (int) segXPos;  
      horizontalPositions[i] = xPos;

      if (BeanUtils.isEmpty(segmentLabel)) {
        lastPos += horizontalScale;
        segXPos += horizontalScale;
        continue;
      }
      
      if (viewGrid && ((i > 0) || (segmentMarkStyle != SEGMENT_MARK_STYLE_BORDER))) {
        graphics.setStroke(gridStroke);
        graphics.setColor(gridColor);
        graphics.draw(new Line2D.Float(xSegPos, verticalMinimum, xSegPos, verticalMaximum));
      }
      graphics.setStroke(scaleStroke);
      graphics.setColor(scaleColor);
      graphics.draw(new Line2D.Float(xSegPos, hScaleYfrom, xSegPos, hScaleYto));

      lastPos += horizontalScale;
      segXPos += horizontalScale;
      Rectangle2D segmentMetric = segmentMetrics[i];
      graphics.drawString(
          segmentLabel,
        (float) (xPos + segmentMetric.getHeight() / 3.0), 
        (float) (segmentDescriptionArea.getMinY() + segmentMetric.getWidth() + halfScaleLength * 2)
      );
    }

    if (segmentMarkStyle == SEGMENT_MARK_STYLE_BORDER) {
      int xSegPos = (int) segXPos;  
      if (viewGrid) {
        graphics.setStroke(gridStroke);
        graphics.setColor(gridColor);
        graphics.draw(new Line2D.Float(xSegPos, verticalMinimum, xSegPos, verticalMaximum));
      }
      graphics.setStroke(scaleStroke);
      graphics.setColor(scaleColor);
      graphics.draw(new Line2D.Float(xSegPos, hScaleYfrom, xSegPos, hScaleYto));
    }
    
    // drawing the vertical descriptions
    segmentDescriptionFont = getDescriptionFont();
    graphics.setFont(segmentDescriptionFont);
    FontMetrics fm = graphics.getFontMetrics();

    double descriptionHeight = fm.getStringBounds("1", graphics).getHeight();
    
    float stepSize = getValueStepSize(descriptionHeight);
    double maxValue = getMaxValue();
    double minValue = getMinValue();

    float base = (float) minValue;
    if (base < 0.0F) {
      base = 0.0F;
    }

    // drawing the zero position
    float xPos = (int) effectiveViewport.getMinX();
    float yPos = calculateVerticalPosition(0.0);
    float vScaleXfrom = xPos - halfScaleLength;
    float vScaleXto = xPos + halfScaleLength;

    graphics.setColor(scaleColor);
    graphics.draw(new Line2D.Float(vScaleXfrom, yPos, vScaleXto, yPos));

    String strValue = null;
    Rectangle2D textMetrics = null;
    
    for (float value = base; value <= maxValue; value += stepSize) {
      yPos = calculateVerticalPosition(value);
      if (viewGrid) {
        graphics.setStroke(gridStroke);
        graphics.setColor(gridColor);
        if (value != base) {
          graphics.draw(new Line2D.Float(x, yPos, maxGridX, yPos));
        }
      }
      
      graphics.setStroke(scaleStroke);
      graphics.setColor(scaleColor);
      graphics.draw(new Line2D.Float(vScaleXfrom, yPos, vScaleXto, yPos));
  
      strValue = formatScaleValue(value, stepSize);
      textMetrics = fm.getStringBounds(strValue, graphics);
      
      graphics.drawString(
        strValue,
        (float) (xPos - halfScaleLength * 2 - textMetrics.getWidth()), 
        (float) (yPos + textMetrics.getHeight() / 3)
      );
    }

    for (float value = -stepSize; value >= minValue; value -= stepSize) {
      yPos = calculateVerticalPosition(value);
      if (viewGrid) {
        graphics.setStroke(gridStroke);
        graphics.setColor(gridColor);
        graphics.draw(new Line2D.Float(x, yPos, maxGridX, yPos));
      }
      
      graphics.setStroke(scaleStroke);
      graphics.setColor(scaleColor);
      graphics.draw(new Line2D.Float(vScaleXfrom, yPos, vScaleXto, yPos));
  
      strValue = formatScaleValue(value, stepSize);
      textMetrics = fm.getStringBounds(strValue, graphics);
      
      graphics.drawString(
        strValue,
        (float) (xPos - halfScaleLength * 2 - textMetrics.getWidth()), 
        (float) (yPos + textMetrics.getHeight() / 3)
      );
    }

    graphics.setStroke(oldStroke);
    graphics.setFont(oldFont);
  }

  /**
   * Draws the legend.
   * @param viewport The viewport.
   * @param graphics The graphics object to use.
   */
  protected void drawLegend(Rectangle2D viewport, Graphics2D graphics) {

    int padding = getInnerPadding();

    if (isViewLegend()) {
      Font oldFont = graphics.getFont();
      Stroke oldStroke = graphics.getStroke();
      graphics.setStroke(new BasicStroke(getDescriptionLineWidth(), BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
      
      double labelBoxSize = maxAreaLabelLineHeight * 0.75;
      double labelBoxXPos = legendMetrics.getMinX() + padding;
      double labelXPos = labelBoxXPos + labelBoxSize * 2;
      double labelYPos = legendMetrics.getMinY();

      Color[] segmentColors = attributes.getColorArray(ATTR_SEGMENT_COLORS); 

      Font segmentDescriptionFont = getDescriptionFont();
      graphics.setFont(segmentDescriptionFont);
        
      String[] areaLabels = getAreaLabels();
      for (int i = 0; i < areaLabels.length; i++) {
        String label = areaLabels[i];
        labelYPos += maxAreaLabelLineHeight * 1.5;      

        // setting fill color
        Color oldColor = graphics.getColor();        
        graphics.setColor(segmentColors[i]);

        graphics.fillRect(
          (int) labelBoxXPos, 
          (int) (labelYPos - labelBoxSize), 
          (int) (labelBoxSize),
          (int) (labelBoxSize)
        ); 

        // restoring old color
        graphics.setColor(oldColor);
        graphics.drawRect(
          (int) labelBoxXPos, 
          (int) (labelYPos - labelBoxSize), 
          (int) (labelBoxSize),
          (int) (labelBoxSize)
        ); 

        boolean firstLine = true;
        StringTokenizer tkn = new StringTokenizer(label, "\r\n");
        while (tkn.hasMoreTokens()) {
          String ll = tkn.nextToken();
          if (!firstLine) {
            labelYPos += maxAreaLabelLineHeight * DEFAULT_DESCRIPTION_LINE_SPACING;
          }
          graphics.drawString(ll, (float) labelXPos, (float) labelYPos);
          firstLine = false;
        }
      }
      
      graphics.draw(legendMetrics);
      
      // setting old font again
      graphics.setFont(oldFont);
      graphics.setStroke(oldStroke);
    }    
  }

  /**
   * Calculates the viewport.
   * @param viewport The original viewport.
   * @param graphics The graphics interface needed for font metrics calculation.
   * @return The determined effective viewport.
   */
  protected Rectangle calcViewport(Rectangle2D viewport, Graphics2D graphics) {

    int padding = getInnerPadding();
    int x = (int) (viewport.getX() + padding);
    int y = (int) (viewport.getY() + padding);
    int width = (int) viewport.getWidth() - padding * 2;
    int height = (int) viewport.getHeight() - padding * 2;

    if ((model == null) || (model.getChartData() == null)) {
      effectiveViewport = new Rectangle(x, y, width, height);
      return effectiveViewport;
    }
    
    TextChartItem title = model.getTitle();
    double topSpace = 0.0;
    if (title != null) {
      Rectangle2D titleArea = title.getOutputArea(viewport, graphics);
      topSpace = (int) titleArea.getMaxY(); 
    }
    
    List chartData = model.getChartData();
    int numSegments = chartData.size();
    this.min = getMinValue();
    if (min < 0.0) {
      min -= 5;
    }
    this.max = getMaxValue() * 1.05;
    this.range = max - min;

    Font oldFont = graphics.getFont();

    Font segmentDescriptionFont = getDescriptionFont();
    graphics.setFont(segmentDescriptionFont);
    FontMetrics fm = graphics.getFontMetrics();
    double descriptionHeight = fm.getStringBounds("1", graphics).getHeight();
    float stepSize = getValueStepSize(descriptionHeight);
    
    /* calculating value description metrics. */
    String maxValue = formatScaleValue(getMaxValue(), stepSize);
    String minValue = formatScaleValue(getMinValue(), stepSize);
    Rectangle2D maxValueMetrics = null;
    if (maxValue.length() > minValue.length()) {
      maxValueMetrics = fm.getStringBounds(maxValue, graphics); 
    } else {
      maxValueMetrics = fm.getStringBounds(minValue, graphics); 
    }
    double maxValueLabelWidth = maxValueMetrics.getMaxX();
    
    // calculating value scale description metrics
    TextChartItem valueScaleDescription = getValueScaleDescription();
    int valueScaleDescriptionVerticalPositionMode = -1;
    if (valueScaleDescription != null) {
      double maxWidth = valueScaleDescription.getMaxWidth();
      valueScaleDescriptionArea = valueScaleDescription.getOutputArea(
        new Rectangle(padding, padding, (int) maxWidth, (int) (viewport.getHeight() - padding * 2)),
        graphics
      );
      valueScaleDescriptionVerticalPositionMode = valueScaleDescription.getPosition().getVerticalPositionMode();
      if (valueScaleDescriptionVerticalPositionMode == ChartItemPosition.MIDDLE) {
        maxValueLabelWidth = maxValueLabelWidth + valueScaleDescriptionArea.getWidth() + padding / 2.0F;
      } else {
        topSpace = Math.max(topSpace, valueScaleDescriptionArea.getHeight());
        maxValueLabelWidth = Math.max(maxValueLabelWidth, valueScaleDescriptionArea.getWidth());
      }
    }
    height -= (int) topSpace;
    y += topSpace;
    effectiveViewport = new Rectangle(x, y, width, height);
    
    width -= (int) (maxValueLabelWidth + padding);
    x += (int) (maxValueLabelWidth + padding);

    float rightDescriptionSpace = 0.0F;
    float legendHeight = padding;
    if (isViewLegend()) {
      /* calculating legend metrics. */     
      maxAreaLabelHeight = 0.0F;
      float areaLabelWidth = 0.0F;
      String[] areaLabels = getAreaLabels();
      for (int i = 0; i < areaLabels.length; i++) {
        String label = areaLabels[i];
        StringTokenizer tkn = new StringTokenizer(label, "\r\n");
        float labelHeight = 0.0F;
        while (tkn.hasMoreTokens()) {
          String ll = tkn.nextToken();
          Rectangle2D textMetrics = fm.getStringBounds(ll, graphics);
          if (labelHeight > 0.0) {
            labelHeight += textMetrics.getHeight() * DEFAULT_DESCRIPTION_LINE_SPACING;
          } else {
            labelHeight += textMetrics.getHeight() * DEFAULT_DESCRIPTION_ITEM_SPACING;
          }
          areaLabelWidth = Math.max(areaLabelWidth, (float) textMetrics.getWidth());
          maxAreaLabelLineHeight = Math.max(maxAreaLabelLineHeight, (float) textMetrics.getHeight());      
          maxAreaLabelHeight = Math.max(maxAreaLabelHeight, labelHeight);      
        }
        legendHeight += labelHeight; 
      }
      
      areaLabelWidth += maxAreaLabelLineHeight * 1.5 + (padding) * 2;

      // setting legend metrics
      this.legendMetrics = new Rectangle(
        (int) (x + width - (areaLabelWidth + padding * 2.0F) + padding), 
        y + padding,
        (int) areaLabelWidth,
        (int) legendHeight
      );
      
      rightDescriptionSpace = areaLabelWidth;
    }

    // calculating segment scale description metrics
    float segmentScaleDescriptionHeight = 0;
    TextChartItem segmentScaleDescription = getSegmentScaleDescription();
    if (segmentScaleDescription != null) {
      double maxWidth = segmentScaleDescription.getMaxWidth();
      segmentScaleDescriptionArea = segmentScaleDescription.getOutputArea(
          new Rectangle(padding, padding, (int) maxWidth, (int) (viewport.getHeight() / 2 - padding * 2)),
          graphics
      );
      if (segmentScaleDescription.getPosition().getHorizontalPositionMode() == ChartItemPosition.CENTER) {
        segmentScaleDescriptionHeight = (float) segmentScaleDescriptionArea.getHeight() + padding;
      } else {
        maxSegmentLabelHeight = Math.max(maxSegmentLabelHeight, (float) segmentScaleDescriptionArea.getHeight());
        rightDescriptionSpace = Math.max(rightDescriptionSpace, (float) segmentScaleDescriptionArea.getWidth());
      }
    }
    // reducing effective viewport
    width -= (rightDescriptionSpace + padding * 2.0F);

    effectiveViewport = new Rectangle(x, y, width, height);
    
    /* calculating segment description metrics. */     
    String[] segmentLabels = getSegmentLabels();
    this.maxSegmentLabelHeight = 0.0F;
    this.segmentMetrics = new Rectangle2D[numSegments];

    this.horizontalScale = (float) (effectiveViewport.getWidth() / (model.getChartData().size()));
    this.horizontalOffset = (float) effectiveViewport.getX(); /* + horizontalScale * 0.1*/;

    if (segmentLabels != null) {
      String label = null;
      for (int i = 0; i < numSegments; i++) {
        if (i < segmentLabels.length) {
          label = segmentLabels[i];
        } else {
          continue;
        }
        if (label == null) {
          continue;
        }
        Rectangle2D textMetrics = fm.getStringBounds(label, graphics);
        segmentMetrics[i] = textMetrics;
        maxSegmentLabelHeight = Math.max(maxSegmentLabelHeight, (float) textMetrics.getWidth());
      }
    }
    
    double vertDescriptionDelta = maxSegmentLabelHeight + segmentScaleDescriptionHeight;
    this.verticalScale = (float) ((effectiveViewport.getHeight() - vertDescriptionDelta) / range);
    this.verticalAnchor = (int) (viewport.getMaxY() - padding * 2);

    this.verticalZero = calculateVerticalPosition(0.0);
    if (vertDescriptionDelta > 0) {
      verticalAnchor -= vertDescriptionDelta;
      verticalZero = calculateVerticalPosition(0.0);
    }
    this.verticalScale = (float) ((verticalZero - effectiveViewport.getY() - padding) / max);

    this.horizontalPositions = new float[numSegments];
    float lastPos = horizontalOffset + (horizontalScale / 2.0F);
    
    graphics.setFont(segmentDescriptionFont);
    for (int i = 0; i < numSegments; i++) {
      int xPos = (int) lastPos;  
      horizontalPositions[i] = xPos;
      lastPos += horizontalScale;
    }

    this.verticalMinimum = verticalAnchor;
    this.verticalMaximum = calculateVerticalPosition(max);
    if (min < 0.0) {
      verticalMinimum = calculateVerticalPosition(min);
    } 
    this.verticalZero = calculateVerticalPosition(0.0);

    if (segmentScaleDescriptionArea != null) {
      if (segmentScaleDescription.getPosition().getHorizontalPositionMode() == ChartItemPosition.CENTER) {
        segmentScaleDescriptionArea = new Rectangle(
            (int) (effectiveViewport.getMinX()),
            (int) (verticalZero + maxSegmentLabelHeight + padding),
            (int) (effectiveViewport.getMaxX() - effectiveViewport.getMinX()),
            (int) (segmentScaleDescriptionArea.getHeight())
        );
      } else {
        segmentScaleDescriptionArea = new Rectangle(
            (int) (effectiveViewport.getMaxX() + padding), (int) (verticalZero + padding),
            (int) (viewport.getMaxX() - padding * 2.0 - effectiveViewport.getMaxX()), 
            (int) (maxSegmentLabelHeight)
        );
      }
      segmentScaleDescription.alignItem(segmentScaleDescriptionArea);
    }

    if (valueScaleDescriptionArea != null) {
      if (valueScaleDescriptionVerticalPositionMode == ChartItemPosition.MIDDLE) {
        valueScaleDescriptionArea = new Rectangle(
            (int) (viewport.getX() + padding), (int) verticalMinimum,
            (int) maxValueLabelWidth, (int) (verticalMaximum - verticalMinimum)
        );
      } else {
        valueScaleDescriptionArea = new Rectangle(
            (int) (viewport.getX() + padding), (int) (viewport.getY() + padding),
            (int) maxValueLabelWidth, (int) topSpace
        );
      }
      valueScaleDescription.alignItem(valueScaleDescriptionArea);
    }

    graphics.setFont(oldFont);
    
    return effectiveViewport;
  }

  /**
   * Calculates the vertical position for the given value.
   * @param value The value we are interrested in the vertical position for
   * @return The vertical position.
   */
  protected float calculateVerticalPosition(double value) {
    return verticalAnchor - calculateVerticalExtent(value);
  }

  /**
   * Calculates the vertical extent for the given value.
   * @param value The value we are interrested in the vertical position for
   * @return The vertical extent.
   */
  protected float calculateVerticalExtent(double value) {
    return (float) ((value - min) * verticalScale);
  }

  /**
   * Calculates the base scale line position for the chart.
   * @return The vertical position of the base line.
   */
  protected float calculateBaseLinePosition() {
    double nn = 0.0;
    if (model.getMinValue() > 0.0) {
      nn = model.getMinValue();
    }
    return calculateVerticalPosition(nn);
  }

}
