/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.chart;

import java.awt.Color;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.Stroke;
import java.awt.Graphics2D;
import java.awt.BasicStroke;
import java.awt.RenderingHints;

import java.awt.geom.Rectangle2D;

import java.text.NumberFormat;

import java.util.ArrayList;
import java.util.List;
import org.ujac.util.CollectionUtils;
import org.ujac.util.UjacTypes;

/**
 * Name: BaseChart<br>
 * Description: Base Chart implementation.
 * 
 * @author lauerc
 */
public abstract class BaseChart implements Chart {

  /** The default border width. */
  public static final float DEFAULT_BORDER_WIDTH = 0.0F;
  /** The default padding. */
  public static final float DEFAULT_PADDING = 2.0F;
  /** The default title font. */
  public static final Font DEFAULT_TITLE_FONT = new Font("Helvetica", Font.BOLD, 20);
  /** The default description line width. */
  public static final float DEFAULT_DESCRIPTION_LINE_WIDTH = 1.0F;
  /** The default description color. */
  public static final Color DEFAULT_DESCRIPTION_COLOR = Color.black;
  /** The default description pattern. */
  public static final String DEFAULT_DESCRIPTION_PATTERN = "%v";
  /** The default grid color. */
  public static final Color DEFAULT_SHADOW_COLOR = Color.darkGray;
  /** The default shadow offset. */
  public static final float DEFAULT_SHADOW_OFFSET = 4.0F;
  /** The default description font. */
  public static final Font DEFAULT_DESCRIPTION_FONT = new Font("Helvetica", 0, 14);
  
  /** The default value format. */
  public transient NumberFormat defaultValueFormat = NumberFormat.getNumberInstance();
  /** The default percent format. */
  public transient NumberFormat defaultPercentFormat = NumberFormat.getNumberInstance();

  /** The chart model. */
  protected ChartModel model;
  /** The chart attributes. */
  protected ChartAttributes attributes = null;
  
  /**
   * Gets a list of supported attributes for the chart.
   * @return The list of supported attributes for the chart.
   */
  public List getSupportedAttributes() {
    List attributes = new ArrayList();
    fillSupportedAttributes(attributes);
    return attributes;
  }

  /**
   * Adds an attribute definition to the list of supported attributes for the chart.
   * @param attributeDefinitions The list of supported attributes for the chart to fill.
   * @param name The name of the attribute definition.
   * @param type The attribute type.
   * @param list The list flag.
   * @param optional The optional flag.
   */
  protected void addAttributeDefinition(List attributeDefinitions, String name, 
                                        int type, boolean list, boolean optional) {
    attributeDefinitions.add(
      new ChartAttributeDefinition(
        name, 
        type,
        list,
        optional
      )
    );
  }

  /**
   * Fills the list of supported attributes for the chart.
   * @param attrDefs The list of supported attributes for the chart to fill.
   */
  protected void fillSupportedAttributes(List attrDefs) {
    addAttributeDefinition(attrDefs, ATTR_BACKGROUND_COLOR, ChartAttributeDefinition.TYPE_COLOR, false, true);
    addAttributeDefinition(attrDefs, ATTR_BORDER_WIDTH, UjacTypes.TYPE_FLOAT, false, true);
    addAttributeDefinition(attrDefs, ATTR_TITLE_FONT, ChartAttributeDefinition.TYPE_FONT, false, true);
    addAttributeDefinition(attrDefs, ATTR_SEGMENT_LABELS, UjacTypes.TYPE_STRING, true, true);
    addAttributeDefinition(attrDefs, ATTR_AREA_LABELS, UjacTypes.TYPE_STRING, true, true);
    addAttributeDefinition(attrDefs, ATTR_DESCRIPTION_FONT, ChartAttributeDefinition.TYPE_FONT, false, true);
    addAttributeDefinition(attrDefs, ATTR_DESCRIPTION_PATTERN, UjacTypes.TYPE_STRING, false, true);
    addAttributeDefinition(attrDefs, ATTR_DESCRIPTION_PATTERN, UjacTypes.TYPE_STRING, false, true);
    addAttributeDefinition(attrDefs, ATTR_DESCRIPTION_LINE_WIDTH, UjacTypes.TYPE_FLOAT, false, true);
    addAttributeDefinition(attrDefs, ATTR_DESCRIPTION_COLOR, ChartAttributeDefinition.TYPE_COLOR, false, true);
    addAttributeDefinition(attrDefs, ATTR_VALUE_FORMAT, ChartAttributeDefinition.TYPE_FORMAT, false, true);
    addAttributeDefinition(attrDefs, ATTR_PERCENT_FORMAT, ChartAttributeDefinition.TYPE_FORMAT, false, true);
    addAttributeDefinition(attrDefs, ATTR_SEGMENT_COLORS, ChartAttributeDefinition.TYPE_COLOR, true, true);
    addAttributeDefinition(attrDefs, ATTR_PIE_HEIGHT, UjacTypes.TYPE_FLOAT, false, true);
    addAttributeDefinition(attrDefs, ATTR_LINE_WIDTH, UjacTypes.TYPE_FLOAT, false, true);
    addAttributeDefinition(attrDefs, ATTR_SCALE_MARK_LENGTH, UjacTypes.TYPE_FLOAT, false, true);
    addAttributeDefinition(attrDefs, ATTR_SHADOW_COLOR, ChartAttributeDefinition.TYPE_COLOR, false, true);
    addAttributeDefinition(attrDefs, ATTR_SHADOW_OFFSET, UjacTypes.TYPE_FLOAT, false, true);
    addAttributeDefinition(attrDefs, ATTR_VIEW_LEGEND, UjacTypes.TYPE_BOOLEAN, false, true);
    addAttributeDefinition(attrDefs, ATTR_VIEW_GRID, UjacTypes.TYPE_BOOLEAN, false, true);
    addAttributeDefinition(attrDefs, ATTR_GRID_COLOR, ChartAttributeDefinition.TYPE_COLOR, false, true);
    addAttributeDefinition(attrDefs, ATTR_GRID_BACKGROUND_COLOR, ChartAttributeDefinition.TYPE_COLOR, false, true);
    addAttributeDefinition(attrDefs, ATTR_FILL_AREA, UjacTypes.TYPE_BOOLEAN, false, true);
    addAttributeDefinition(attrDefs, ATTR_VIEW_SCALE_ARROWS, UjacTypes.TYPE_BOOLEAN, false, true);
    addAttributeDefinition(attrDefs, ATTR_SCALE_ARROWHEAD_LENGTH, UjacTypes.TYPE_FLOAT, false, true);
    addAttributeDefinition(attrDefs, ATTR_SCALE_ARROWHEAD_WIDTH, UjacTypes.TYPE_FLOAT, false, true);
    addAttributeDefinition(attrDefs, ATTR_SCALE_ARROWHEAD_STYLE, UjacTypes.TYPE_STRING, false, true);
    addAttributeDefinition(attrDefs, ATTR_SCALE_VALUE_FORMAT, ChartAttributeDefinition.TYPE_FORMAT, false, true);
    addAttributeDefinition(attrDefs, ATTR_VALUE_STEP_SIZE, UjacTypes.TYPE_DOUBLE, false, true);
    addAttributeDefinition(attrDefs, ATTR_VALUE_SCALE_DESCRIPTION, ChartAttributeDefinition.TYPE_TEXT_ITEM, false, true);
    addAttributeDefinition(attrDefs, ATTR_SEGMENT_SCALE_DESCRIPTION, ChartAttributeDefinition.TYPE_TEXT_ITEM, false, true);
  }

  /**
   * Gets the chart attributes.
   * @return The current border width.
   */
  public ChartAttributes getAttributes() {
    return attributes;
  }
  /**
   * Sets the chart attributes.
   * @param attributes The chart attributes to set.
   */
  public void setAttributes(ChartAttributes attributes) {
    this.attributes = attributes;
  }

  /**
   * Gets the background color.
   * @return The current background color.
   */
  public Color getBackgroundColor() {
    if (attributes.isDefined(ATTR_BACKGROUND_COLOR)) {
      return attributes.getColor(ATTR_BACKGROUND_COLOR);      
    }
    return null;
  }

  /**
   * Gets the border width.
   * @return The current border width.
   */
  public float getBorderWidth() {
    if (attributes.isDefined(ATTR_BORDER_WIDTH)) {
      return attributes.getFloat(ATTR_BORDER_WIDTH);      
    }
    return DEFAULT_BORDER_WIDTH;
  }

  /**
   * Gets the description line width.
   * @return The current description line width.
   */
  public float getDescriptionLineWidth() {
    if (attributes.isDefined(ATTR_DESCRIPTION_LINE_WIDTH)) {
      return attributes.getFloat(ATTR_DESCRIPTION_LINE_WIDTH);      
    }
    return DEFAULT_DESCRIPTION_LINE_WIDTH;
  }

  /**
   * Gets the description line color.
   * @return The current description line color.
   */
  public Color getDescriptionColor() {
    if (attributes.isDefined(ATTR_DESCRIPTION_COLOR)) {
      return attributes.getColor(ATTR_DESCRIPTION_COLOR);      
    }
    return DEFAULT_DESCRIPTION_COLOR;
  }
  
  /**
   * Gets the description pattern.
   * @return The description pattern.
   */
  public String getDescriptionPattern() {
    String descriptionPattern = attributes.getString(ATTR_DESCRIPTION_PATTERN);
    if (descriptionPattern != null) {
      return descriptionPattern;
    }
    
    return DEFAULT_DESCRIPTION_PATTERN;
  }

  /**
   * Computes the fill color for value area.
   * @return The transformed color.
   */
  public Color getShadowColor()  {
    Color color = DEFAULT_SHADOW_COLOR;
    if (attributes.isDefined(ATTR_SHADOW_COLOR)) {
      color = attributes.getColor(ATTR_SHADOW_COLOR);      
    }
    int r = color.getRed();
    int g = color.getGreen();
    int b = color.getBlue();
    return new Color(r, g, b, 192);
  }

  /**
   * Gets the shadow length.
   * @return The length of the shadow to draw according to the attribute ATTR_SHADOW_OFFSET.
   */
  public float getShadowLength() {
    if (attributes.isDefined(ATTR_SHADOW_OFFSET)) {
      return attributes.getFloat(ATTR_SHADOW_OFFSET);      
    }
    return DEFAULT_SHADOW_OFFSET;
  }

  /**
   * Gets the number format for value output.
   * @return The value format.
   */
  public NumberFormat getValueFormat() {
    if (attributes.isDefined(ATTR_VALUE_FORMAT)) {
      return (NumberFormat) attributes.getObject(ATTR_VALUE_FORMAT);
    }
    return defaultValueFormat;
  }

  /**
   * Gets the number format for percent output.
   * @return The percent format.
   */
  public NumberFormat getPercentFormat() {
    if (attributes.isDefined(ATTR_PERCENT_FORMAT)) {
      return (NumberFormat) attributes.getObject(ATTR_PERCENT_FORMAT);
    }
    return defaultPercentFormat;
  }

  /**
   * Gets the left padding.
   * @return The current left padding.
   */
  public float getLeftPadding() {
    if (attributes.isDefined(ATTR_PADDING_LEFT)) {
      return attributes.getFloat(ATTR_PADDING_LEFT);      
    } else if (attributes.isDefined(ATTR_PADDING)) {
      return attributes.getFloat(ATTR_PADDING);
    }
    return DEFAULT_PADDING;
  }

  /**
   * Gets the right padding.
   * @return The current right padding.
   */
  public float getRightPadding() {
    if (attributes.isDefined(ATTR_PADDING_RIGHT)) {
      return attributes.getFloat(ATTR_PADDING_RIGHT);      
    } else if (attributes.isDefined(ATTR_PADDING)) {
      return attributes.getFloat(ATTR_PADDING);
    }
    return DEFAULT_PADDING;
  }

  /**
   * Gets the top padding.
   * @return The current top padding.
   */
  public float getTopPadding() {
    if (attributes.isDefined(ATTR_PADDING_TOP)) {
      return attributes.getFloat(ATTR_PADDING_TOP);      
    } else if (attributes.isDefined(ATTR_PADDING)) {
      return attributes.getFloat(ATTR_PADDING);
    }
    return DEFAULT_PADDING;
  }

  /**
   * Gets the top padding.
   * @return The current top padding.
   */
  public float getBottomPadding() {
    if (attributes.isDefined(ATTR_PADDING_BOTTOM)) {
      return attributes.getFloat(ATTR_PADDING_BOTTOM);      
    } else if (attributes.isDefined(ATTR_PADDING)) {
      return attributes.getFloat(ATTR_PADDING);
    }
    return DEFAULT_PADDING;
  }

  /**
   * Gets the title font.
   * @return The current title font.
   */
  public Font getTitleFont() {
    if (attributes.isDefined(ATTR_TITLE_FONT)) {
      return attributes.getFont(ATTR_TITLE_FONT);      
    }
    return DEFAULT_TITLE_FONT;
  }

  /**
   * Gets the description font.
   * @return The current description font.
   */
  public Font getDescriptionFont() {
    if (attributes.isDefined(ATTR_DESCRIPTION_FONT)) {
      return attributes.getFont(ATTR_DESCRIPTION_FONT);      
    }
    return DEFAULT_DESCRIPTION_FONT;
  }

  /**
   * Gets the segment labels.
   * @return The current segment labels.
   */
  public String[] getSegmentLabels() {
    List labelList = model.getSegmentLabels();
    if (labelList != null) {
      int numLabels = CollectionUtils.getSize(labelList);
      String[] strLabels = new String[numLabels];
      Object cw = null;
      for (int i = 0; i < numLabels; i++) {
        cw = labelList.get(i);
        if (cw == null) {
          continue;
        }
        strLabels[i] = cw.toString();
      }
      return strLabels;
    }
    if (attributes.isDefined(ATTR_SEGMENT_LABELS)) {
      return attributes.getStringArray(ATTR_SEGMENT_LABELS);
    }
    return null;
  }

  /**
   * Gets the area labels.
   * @return The current area labels.
   */
  public String[] getAreaLabels() {
    List labelList = model.getAreaLabels();
    if (labelList != null) {
      int numLabels = CollectionUtils.getSize(labelList);
      String[] strLabels = new String[numLabels];
      Object cw = null;
      for (int i = 0; i < numLabels; i++) {
        cw = labelList.get(i);
        if (cw == null) {
          continue;
        }
        strLabels[i] = cw.toString();
      }
      return strLabels;
    }
    if (attributes.isDefined(ATTR_AREA_LABELS)) {
      return attributes.getStringArray(ATTR_AREA_LABELS);
    }
    return null;
  }

  /**
   * Gets the chart model.
   * @return The chart model.
   */
  public ChartModel getModel() {
    return model;
  }
  /**
   * Sets the chart model.
   * @param model The chart model to set.
   * @exception ChartException In case the model contained invalid data for the chart.
   */
  public void setModel(ChartModel model) throws ChartException {
    this.model = model;
    if (!supportsNegativeValues() && (model != null) && (model.getMinValue() < 0.0)) {
      throw new ChartException("This chart type doesn't support negative values.");
    }
  }

  /**
   * Tells, whether the chart supports negative values or not.
   * @return true, in case the chart supports negative values else false.
   */
  public boolean supportsNegativeValues() {
    return true;
  }

  /**
   * Gets the minimum value from the chart data.
   * @return The minimum chart data value.
   */
  protected double getMinValue() {
    return model.getMinValue();
  }

  /**
   * Gets the maximum value from the chart data.
   * @return The maximum chart data value.
   */
  protected double getMaxValue() {
    return model.getMaxValue();
  }

  /**
   * Draws the chart to the given chart output.
   * @param viewport The viewport for chart output.
   * @param graphics The graphics object used for chart output.
   * @return The effective drawing area for the chart.
   * @exception ChartException In case the rendering failed.
   */
  public Rectangle2D draw(Rectangle2D viewport, Graphics2D graphics)
    throws ChartException {
    
    graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    float borderWidth = getBorderWidth();
    if (borderWidth > 0.0) {
      Stroke oldStroke = graphics.getStroke();
      graphics.setStroke(new BasicStroke(borderWidth));
      graphics.drawRect((int) viewport.getX(), (int) viewport.getY(), 
                        (int) viewport.getWidth(), (int) viewport.getHeight());
      graphics.setStroke(oldStroke);
    }
    
    Color backgroundColor = getBackgroundColor();
    if (backgroundColor != null) {
      Color c = graphics.getColor();
      graphics.setColor(backgroundColor);
      graphics.fillRect(
          (int) viewport.getMinX(), 
          (int) viewport.getMinY(), 
          (int) viewport.getWidth(), 
          (int) viewport.getHeight()
      );
      graphics.setColor(c);
    }
    
    Rectangle2D localViewport = new Rectangle(
      (int) (viewport.getMinX() + getLeftPadding()),
      (int) (viewport.getMinY() + getTopPadding()),
      (int) (viewport.getWidth() - getLeftPadding() - getRightPadding()),
      (int) (viewport.getHeight() - getTopPadding() - getBottomPadding())
    );
    TextChartItem title = model.getTitle();
    if (title != null) {
      // drawing the title
      title.resetMetrics();
      title.draw(localViewport, graphics);
    }
    return localViewport;
  }
}
