/*
 * Copyright (C) 2003 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.chart;

import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;

/**
 * Name: BaseChartItem<br>
 * Description: Base ChartItem implementation.
 * @author lauerc
 */
public abstract class BaseChartItem implements ChartItem {

  /** The position of the chart item. */
  protected ChartItemPosition position = null;
  /** The output area. */
  protected Rectangle2D outputArea = null;
  
  /**
   * Constructs a BaseChartItem instance with specific attributes.
   * @param position The position of the chart item.
   */
  public BaseChartItem(ChartItemPosition position) {
    this.position = position;
  }

  /**
   * Gets the position of the chart item.
   * @return The current position.
   */
  public ChartItemPosition getPosition() {
    return position;
  }
  
  /**
   * Gets the output area ot the chart item.
   * @param viewport The viewport for chart output.
   * @param graphics The graphics object, needed to calculate the metrics.
   * @return The output area the item will be rendered to.
   */
  public Rectangle2D getOutputArea(Rectangle2D viewport, Graphics2D graphics) {
    if (outputArea == null) {
      outputArea = calculateOutputArea(viewport, graphics);
    }
    return outputArea;
  }

  /**
   * Resets the calculated metrics to enforce re-calculation of the
   * item's output area.
   */
  public void resetMetrics() {
    this.outputArea = null;
  }

  /**
   * Calculates the position of the item.
   * @param viewport The viewport for chart output.
   * @param graphics The graphics object, needed to calculate the metrics.
   * @return The calculated output area.
   */
  protected abstract Rectangle2D calculateOutputArea(Rectangle2D viewport, Graphics2D graphics);


  /**
   * Gets the width of the item box.
   * @param viewport The viewport for chart output.
   * @param graphics The graphics object, used for output.
   * @return The width of the item box.
   */
  public double getWidth(Rectangle2D viewport, Graphics2D graphics) {
    return getOutputArea(viewport, graphics).getWidth();
  }

  /**
   * Gets the height of the item box.
   * @param viewport The viewport for chart output.
   * @param graphics The graphics object, used for output.
   * @return The height of the item box.
   */
  public double getHeight(Rectangle2D viewport, Graphics2D graphics) {
    return getOutputArea(viewport, graphics).getHeight();
  }

  /**
   * Moves the item position.
   * @param x The number of pixels by which to move the item horizontally.
   * @param y The number of pixels by which to move the item vertically.
   */
  public void move(double x, double y) {
    double curX = 0.0;
    double curY = 0.0;
    double curWidth = 0.0;
    double curHeight = 0.0;

    if (outputArea != null) {
      curX = outputArea.getX();
      curY = outputArea.getY();
      curWidth = outputArea.getWidth();
      curHeight = outputArea.getHeight();
    }
    
    if (x != 0.0) {
      if (position.getHorizontalPositionMode() == ChartItemPosition.ABSOLUTE) {
        position.setX(position.getX() + x);
        curX = position.getX();
      }
    }
    
    if (y != 0.0) {
      if (position.getVerticalPositionMode() == ChartItemPosition.ABSOLUTE) {
        position.setY(position.getY() + y);
        curY = position.getY();
      }
    }

    if (outputArea != null) {
      outputArea.setFrame(curX, curY, curWidth, curHeight);
    }
  }

  /**
   * Moves the item to the desired position.
   * @param x The new horizontal position.
   * @param y The new vertical position.
   */
  public void moveTo(double x, double y) {
    double curX = 0.0;
    double curY = 0.0;
    double curWidth = 0.0;
    double curHeight = 0.0;

    if (outputArea != null) {
      curX = outputArea.getX();
      curY = outputArea.getY();
      curWidth = outputArea.getWidth();
      curHeight = outputArea.getHeight();
    }
    
    if (x != 0.0) {
      if (position.getHorizontalPositionMode() == ChartItemPosition.ABSOLUTE) {
        position.setX(x);
        curX = position.getX();
      }
    }
    
    if (y != 0.0) {
      if (position.getVerticalPositionMode() == ChartItemPosition.ABSOLUTE) {
        position.setY(y);
        curY = position.getY();
      }
    }

    if (outputArea != null) {
      outputArea.setFrame(curX, curY, curWidth, curHeight);
    }
  }

  /**
   * Aligns the item inside the given viewport.
   * @param viewport The area to place the item.
   */
  public void alignItem(Rectangle2D viewport) {
    double xPos = outputArea.getX();
    double yPos = outputArea.getY();
    double width = outputArea.getWidth();
    double height = outputArea.getHeight();
    
    switch (position.getHorizontalPositionMode()) {
      case ChartItemPosition.ABSOLUTE:
        // nothing 2 do here!
        break;
      case ChartItemPosition.CENTER:
        xPos = (viewport.getCenterX()  - width / 2.0);
        break;
      case ChartItemPosition.RIGHT:
        xPos = (viewport.getX() + viewport.getWidth() - width);
        break;
      default: // ChartItemPosition.LEFT
        xPos = viewport.getX();
        break;
    }
    
    switch (position.getVerticalPositionMode()) {
      case ChartItemPosition.ABSOLUTE:
        // nothing 2 do here!
        break;
      case ChartItemPosition.MIDDLE:
        yPos = (viewport.getCenterY()  - height / 2.0);
        break;
      case ChartItemPosition.BOTTOM:
        yPos = (viewport.getY() + viewport.getHeight() - height);
        break;
      default: // ChartItemPosition.TOP:
        yPos = (viewport.getY());
        break;
    }
    
    // adjusting output area
    outputArea.setFrame(xPos, yPos, width, height);
  }

}
