/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.chart;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Stroke;
import java.awt.font.LineMetrics;
import java.awt.geom.AffineTransform;
import java.awt.geom.Line2D;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import java.text.NumberFormat;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Name: BasePieChart<br>
 * Description: A base class for the rendering of pie charts.
 * 
 * @author lauerc
 */
abstract public class BasePieChart extends BaseChart {

  /** The default description pattern. */
  public static final String DEFAULT_DESCRIPTION_PATTERN = "%d (%v, %p)";

  /** Constant for border aligned pie description layout mode. */
  public static final int PIE_DESCRIPTION_LAYOUT_BORDER = 1;
  /** Constant for round pie description layout mode. */
  public static final int PIE_DESCRIPTION_LAYOUT_ROUND = 2;
  /** The default pie description layout mode. */
  public static final int DEFAULT_PIE_DESCRIPTION_LAYOUT = PIE_DESCRIPTION_LAYOUT_ROUND;
  /** The default pie description anchor ratio. */
  public static final float DEFAULT_PIE_DESCRIPTION_ANCHOR_RATIO = 1.15F;
  /** The default pie description anchor ratio. */
  public static final float DEFAULT_PIE_DESCRIPTION_BASELINE_RATIO = 2.2F;

  /** The pie center. */
  private Point pieCenter = null;
  /** The pie's size. */
  private Dimension pieSize = null;

  /** The half pie width. */
  protected double halfPieWidth = 0.0;
  /** The half pie height. */
  protected double halfPieHeight = 0.0;
  
  /** The array holding the percentages for the slices. */
  protected double[] percentages = null;
  
  /** The array holding the start angles. */
  protected double[] startAngles = null;
  /** The array holding the end angles. */
  protected double[] endAngles = null;
  /** The array holding the arc differences. */
  protected double[] angleDiffs = null;

  /** The array holding the start angles. */
  protected double[] startArcs = null;
  /** The array holding the end angles. */
  protected double[] endArcs = null;
  
  /** Moves the given object to the center of the pie. */
  protected AffineTransform moveToCenter = null;
  /** Moves the given description line to its correct position. */
  protected AffineTransform moveDescriptionLine = null;
  
  /** The description line definitions. */
  protected Line2D.Double[] descriptionLines = null;
  
  /**
   * Tells, whether the chart supports negative values or not.
   * @return true, in case the chart supports negative values else false.
   */
  public boolean supportsNegativeValues() {
    return false;
  }
  
  /**
   * Gets the number format for value output.
   * @return The value format.
   */
  public int getPieDescriptionLayout() {
    if (attributes.isDefined(ATTR_PIE_DESCRIPTION_LAYOUT)) {
      String strLayout = (String) attributes.getObject(ATTR_PIE_DESCRIPTION_LAYOUT);
      if ("BORDER".equals(strLayout)) {
        return PIE_DESCRIPTION_LAYOUT_BORDER;
      }
    }
    return DEFAULT_PIE_DESCRIPTION_LAYOUT;
  }
  
  /**
   * Gets the border width.
   * @return The current border width.
   */
  public float getPieDescriptionAnchorRatio() {
    if (attributes.isDefined(ATTR_PIE_DESCRIPTION_ANCHOR_RATIO)) {
      return attributes.getFloat(ATTR_PIE_DESCRIPTION_ANCHOR_RATIO);      
    }
    return DEFAULT_PIE_DESCRIPTION_ANCHOR_RATIO;
  }

  /**
   * Gets the border width.
   * @return The current border width.
   */
  public float getPieDescriptionBaselineRatio() {
    if (attributes.isDefined(ATTR_PIE_DESCRIPTION_BASELINE_RATIO)) {
      return attributes.getFloat(ATTR_PIE_DESCRIPTION_BASELINE_RATIO);      
    }
    return DEFAULT_PIE_DESCRIPTION_BASELINE_RATIO;
  }
  
  /**
   * Gets the pie center.
   * @return The pie center, specified by the method draw.
   */
  protected Point getPieCenter() {
    return pieCenter;
  }

  /**
   * Gets the pie size.
   * @return The pie size.
   */
  protected Dimension getPieSize() {
    return pieSize;
  }
  
  /**
   * Gets the vertical delta factor for the pie height.
   * @return The vertival delta factor for the pie size.
   */
  protected double getDeltaY() {
    return 1.0;
  }

  /**
   * Gets the height of the slices.
   * @return The height of the slices.
   */
  protected int getSliceHeight() {
    return 0;
  }

  /**
   * @see org.ujac.chart.BaseChart#draw(java.awt.geom.Rectangle2D, java.awt.Graphics2D)
   */
  public Rectangle2D draw(Rectangle2D viewport, Graphics2D graphics) 
    throws ChartException {
    Rectangle2D localViewport = super.draw(viewport, graphics);
    
    if ((model == null) || (model.getChartData() == null)) {
      return localViewport;
    }

    List chartData = model.getChartData();
    int numSegments = chartData.size();
    
    // calculating percentage for each value
    double sum = 0.0;
    for (int i = 0; i < numSegments; i++) {
      double[] row = (double[]) chartData.get(i);
      double value = row[0];
      sum += value;
    }

    this.pieCenter = new Point(
      (int) (localViewport.getX() + localViewport.getWidth() / 2.0),
      (int) (localViewport.getY() + localViewport.getHeight() / 2.0)
    );
    
    double pieSize = localViewport.getHeight() * 0.60;
    int pieHeight = (int) (pieSize * getDeltaY());
    int pieWidth = (int) pieSize;
    this.pieSize = new Dimension(pieWidth, pieHeight);
    double maxArc = 360.0;
    
    // initializing chart
    init(numSegments);
    
    // The segment colors.
    Color[] segmentColors = attributes.getColorArray(ATTR_SEGMENT_COLORS); 
    Color[] sliceColors = new Color[numSegments]; 

    // calculating the metrics of the slices
    double lastArc = 0.0;
    int nextColorIdx = 0;
    for (int i = 0; i < numSegments; i++) {
      double[] row = (double[]) chartData.get(i);
      double value = row[0];
      double percentage = value / sum;
      percentages[i] = percentage * 100.0;
      double arc = percentage * maxArc;
      Color color = segmentColors[nextColorIdx];
      sliceColors[i] = color;
      ++nextColorIdx;
      if (nextColorIdx >= segmentColors.length) {
        nextColorIdx = 0;
      }
      // calculating the slice metrics
      calculateSliceMetrics(i, lastArc, lastArc + arc);
      lastArc += arc;
    }

    int shadowOffsetX = (int) getShadowLength();
    int shadowOffsetY = shadowOffsetX + 1;

    Stroke oldStroke = graphics.getStroke();
    graphics.setStroke(new BasicStroke(0.125F, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));

    int numDrawingSteps = getNumDrawingSteps();    
    if (shadowOffsetX > 0)  {
      AffineTransform moveToCenterOld = this.moveToCenter;
      this.moveToCenter = new AffineTransform();
      this.moveToCenter.translate(pieCenter.getX() + shadowOffsetX, pieCenter.getY() + shadowOffsetY);

      Color shadowColor = getShadowColor();
      for (int i = 0; i < numSegments; i++) {
        // drawing shadow of pie slice
        drawSliceShadow(graphics, shadowColor, i);
      }
      this.moveToCenter = moveToCenterOld;
    }
    
    for (int j = 0; j < numDrawingSteps; j++) {
      if (j == getDescriptionDrawingStep()) {
        // drawing the chart description
        drawDescription(localViewport, graphics);
      }
      for (int i = 0; i < numSegments; i++) {
        // drawing surface of pie slice
        drawSlice(graphics, sliceColors[i], i, j);
      }
    }

    graphics.setStroke(oldStroke);
    
    return localViewport;
  }
  
  /**
   * Initializes the data structures for the chart metrics.
   * @param numSegments The number of metrics for which to 
   */
  protected void init(int numSegments) {
    // calculating common values
    Dimension pieSize = getPieSize();
    this.halfPieWidth = pieSize.getWidth() / 2.0;
    this.halfPieHeight = pieSize.getHeight() / 2.0;

    // initializing data structures
    this.percentages = new double[numSegments];
    this.startAngles = new double[numSegments];
    this.endAngles = new double[numSegments];
    this.angleDiffs = new double[numSegments];
    this.startArcs = new double[numSegments];
    this.endArcs = new double[numSegments];

    this.descriptionLines = new Line2D.Double[numSegments];

    moveToCenter = new AffineTransform();
    moveToCenter.translate(pieCenter.getX(), pieCenter.getY());

    moveDescriptionLine = new AffineTransform();
    moveDescriptionLine.translate(pieCenter.getX(), pieCenter.getY() + getSliceHeight() / 2.0);
  }

  /**
   * Calculates the metrics for the given  slice
   * @param sliceIdx The index of the slice to calculate.
   * @param startAngle The start angle in degrees.
   * @param endAngle The end angle in degrees.
   */
  protected void calculateSliceMetrics(int sliceIdx,  
                                       double startAngle, double endAngle) {
    this.startAngles[sliceIdx] = startAngle;
    this.endAngles[sliceIdx] = endAngle;
    double angleDiff = endAngle - startAngle;
    this.angleDiffs[sliceIdx] = angleDiff;
    this.startArcs[sliceIdx] = Math.PI * (startAngle + 90) / 180;
    this.endArcs[sliceIdx] = Math.PI * (endAngle + 90) / 180;

    double sliceCenterArc = Math.PI * (startAngle + (angleDiff / 2.0) + 90) / 180;
    AffineTransform rotate = new AffineTransform();
    rotate.rotate(-sliceCenterArc);
    Line2D.Double startLine = new Line2D.Double(
      0, halfPieHeight, 
      0, halfPieHeight * 1.25
    );
    PathIterator pIterLine = startLine.getPathIterator(rotate);
    double[] lineStart = new double[2];
    double[] lineEnd = new double[2];
    pIterLine.currentSegment(lineStart);
    pIterLine.next();
    pIterLine.currentSegment(lineEnd);
    double deltaY = getDeltaY();
    lineStart[0] /= deltaY;
    lineEnd[0] /= deltaY;

    moveDescriptionLine.transform(lineStart, 0, lineStart, 0, 1);
    moveDescriptionLine.transform(lineEnd, 0, lineEnd, 0, 1);
    this.descriptionLines[sliceIdx] = new Line2D.Double(
      lineStart[0], lineStart[1],
      lineEnd[0], lineEnd[1]
    );
  }
  
  /**
   * Gets the number of drawing steps to perform to draw the slices.
   * @return The number of drawing steps.
   */
  protected abstract int getNumDrawingSteps();

  /**
   * Gets the index of the step before which to draw the chart
   * description.
   * @return The index of the step before which to draw the description.
   */
  protected int getDescriptionDrawingStep() {
    return 0;
  }

  /**
   * Draws a pie slice.
   * @param graphics The graphics object to draw width.
   * @param color The slice's color.
   * @param sliceIdx The index of the slice to draw.
   * @param stepIdx The index of the drawing step to perform.
   */
  protected abstract void drawSlice(Graphics2D graphics, Color color,
                                    int sliceIdx, int stepIdx);

  /**
   * Draws the shadow of a pie slice.
   * @param graphics The graphics object to draw width.
   * @param color The slice's color.
   * @param sliceIdx The index of the slice to draw.
   */
  protected abstract void drawSliceShadow(Graphics2D graphics, Color color, int sliceIdx);
                                                                        
  /**
   * Draws the chart description.
   * @param viewport The viewport for chart output.
   * @param graphics The graphics object used for chart output.
   * @exception ChartException In case the rendering failed.
   */
  public void drawDescription(Rectangle2D viewport, Graphics2D graphics) 
    throws ChartException {
    
    float pieDescriptionAnchorRatio = getPieDescriptionAnchorRatio();
    double pieDescriptionBaselineRatio = getPieDescriptionBaselineRatio();
    
    graphics.setColor(getDescriptionColor());
    Stroke oldStroke = graphics.getStroke();
    graphics.setStroke(new BasicStroke(getDescriptionLineWidth(), BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
    for (int i = 0; i < descriptionLines.length; i++) {
      // first drawing descriptions
      graphics.draw(descriptionLines[i]);      
    }

    Font descriptionFont = getDescriptionFont();
    String[] segmentLabels = getSegmentLabels();
    
    LineMetrics lm = descriptionFont.getLineMetrics("XYZ", graphics.getFontRenderContext());
    float lineHeight = lm.getHeight();
    float halfLineHeight = lineHeight / 2.0F;
      
    List leftItems = new ArrayList(); 
    List rightItems = new ArrayList(); 
    double centerX = getPieCenter().getX();
    double centerY = getPieCenter().getY();
    
    List chartData = model.getChartData();
    NumberFormat valueFormat = getValueFormat();
    NumberFormat percentFormat = getPercentFormat();
    String patternText = getDescriptionPattern();
    
    double pieSizePlusPadding = halfPieHeight * pieDescriptionAnchorRatio / getDeltaY();
    int maxWidthLeft = (int) (centerX - pieSizePlusPadding - viewport.getMinX() - halfLineHeight - getLeftPadding());
    int maxWidthRight = maxWidthLeft;
    
    DescriptionPattern descriptionPattern = new DescriptionPattern(patternText, valueFormat, percentFormat);
    
    for (int i = 0; i < descriptionLines.length; i++) {
      double percent = percentages[i];
      double value = ((double[]) chartData.get(i))[0];
      String segmentLabel = null;
      if ((segmentLabels != null) && (i < segmentLabels.length)) {
        segmentLabel = segmentLabels[i];
      }
      String itemText = descriptionPattern.apply(value, percent, segmentLabel, null);

      Line2D.Double line = descriptionLines[i];
      PieChartDescriptionItem item = null;
      if (line.getX2() >= centerX) {
        item = new PieChartDescriptionItem(
          i, 
          new ChartItemPosition(
            ChartItemPosition.RIGHT, 
            line.getY2()
          ),
          maxWidthLeft,
          ChartItemPosition.RIGHT,
          itemText,
          descriptionFont
        );
        rightItems.add(item);
      } else {
        item = new PieChartDescriptionItem(
          i, 
          new ChartItemPosition(
            ChartItemPosition.LEFT, 
            line.getY2()
          ), 
          maxWidthRight,
          ChartItemPosition.LEFT,
          itemText,
          descriptionFont
        );
        leftItems.add(item);
      }
      item.move(0.0, -item.getOutputArea(viewport, graphics).getHeight() / 2.0 - viewport.getMinY());
    }
    
    int numLeftItems = leftItems.size();
    int numRightItems = rightItems.size();
    
    // sorting item lists vertically
    Collections.sort(leftItems);
    Collections.sort(rightItems);
    
    // adjusting item positions vertically
    int maxIterations = 1;
    int numIterations = 0;
    boolean adjustmentRequired = true;
    while (adjustmentRequired) {
      adjustmentRequired = false;
      if (numIterations > maxIterations) {
        // breaking loop to avoid endless iteration
        break;
      }
      
      // adjusting left items
      adjustDescriptionItems(viewport, graphics, leftItems, centerY);
      
      // adjusting right items
      adjustDescriptionItems(viewport, graphics, rightItems, centerY);
    }

    double leftBorder = viewport.getMinX() + getLeftPadding();
    double rightBorder = viewport.getMaxX() - getRightPadding();
    if (getPieDescriptionLayout() == PIE_DESCRIPTION_LAYOUT_ROUND) {
      // finally drawing description items
      for (int i = 0; i < numLeftItems; i++) {
        PieChartDescriptionItem item = (PieChartDescriptionItem) leftItems.get(i);
        int itemIdx = item.getItemIdx();
        Line2D.Double line = descriptionLines[itemIdx];
        double deltaX = Math.abs(centerX - line.x2);
        deltaX *= pieDescriptionBaselineRatio;
        ChartItemPosition itemPos = item.getPosition();
        itemPos.setHorizontalPositionMode(ChartItemPosition.ABSOLUTE);
        double y = itemPos.getY();
        double x = deltaX;
        x = centerX - x - maxWidthLeft;
        if (x < leftBorder) {
          x = leftBorder;
        }
        item.moveTo(x, y);
        item.resetMetrics();
        item.draw(viewport, graphics);
        Rectangle2D itemBox = item.getOutputArea(viewport, graphics);
        graphics.draw(new Line2D.Double(
          descriptionLines[item.getItemIdx()].getP2(), 
          new Point2D.Double(itemBox.getMaxX() + halfLineHeight, itemBox.getCenterY())
        ));
      }
      for (int i = 0; i < numRightItems; i++) {
        PieChartDescriptionItem item = (PieChartDescriptionItem) rightItems.get(i);
        int itemIdx = item.getItemIdx();
        Line2D.Double line = descriptionLines[itemIdx];
        double deltaX = Math.abs(centerX - line.x2);
        deltaX *= pieDescriptionBaselineRatio;
        ChartItemPosition itemPos = item.getPosition();
        itemPos.setHorizontalPositionMode(ChartItemPosition.ABSOLUTE);
        double y = itemPos.getY();
        double x = deltaX;
        double width = item.getWidth(viewport, graphics);
        x += centerX;
        if (x + width > rightBorder) {
          x = rightBorder - width;
        }
        item.moveTo(x, y);
        item.resetMetrics();
        item.draw(viewport, graphics);
        Rectangle2D itemBox = item.getOutputArea(viewport, graphics);
        graphics.draw(new Line2D.Double(
          descriptionLines[item.getItemIdx()].getP2(), 
          new Point2D.Double(itemBox.getX() - halfLineHeight, itemBox.getCenterY())
        ));
      }
    } else {
      // finally drawing description items
      for (int i = 0; i < numLeftItems; i++) {
        PieChartDescriptionItem item = (PieChartDescriptionItem) leftItems.get(i);
        item.resetMetrics();
        item.draw(viewport, graphics);
        Rectangle2D itemBox = item.getOutputArea(viewport, graphics);
        graphics.draw(new Line2D.Double(
          descriptionLines[item.getItemIdx()].getP2(), 
          new Point2D.Double(itemBox.getMaxX() + halfLineHeight, itemBox.getCenterY())
        ));
      }
      for (int i = 0; i < numRightItems; i++) {
        PieChartDescriptionItem item = (PieChartDescriptionItem) rightItems.get(i);
        item.resetMetrics();
        item.draw(viewport, graphics);
        Rectangle2D itemBox = item.getOutputArea(viewport, graphics);
        graphics.draw(new Line2D.Double(
          descriptionLines[item.getItemIdx()].getP2(), 
          new Point2D.Double(itemBox.getX() - halfLineHeight, itemBox.getCenterY())
        ));
      }
    }
    graphics.setStroke(oldStroke);
  }  
  /**
   * Adjusting description items to fit vertically into the viewport without overlapping themselves.
   * @param viewport The viewport to render to.
   * @param graphics The graphics instance to use.
   * @param items The items to adjust.
   * @param centerY The vertical pie center. 
   */
  private void adjustDescriptionItems(Rectangle2D viewport, Graphics2D graphics, List items, double centerY) {

    int numItems = items.size();
    int lastItemIdx = 0;
    
    if (numItems < 1) {
      return;
    }
    
    // adjusting right items
    for (int i = 0; i < numItems; i++) {
      PieChartDescriptionItem item = (PieChartDescriptionItem) items.get(i);
      ChartItemPosition itemPos = item.getPosition();
      if (itemPos.getY() > centerY) {
        break;
      }
      PieChartDescriptionItem nextItem = null;
      if (i < numItems - 1) {
        nextItem = (PieChartDescriptionItem) items.get(i + 1);
        ChartItemPosition nextPos = nextItem.getPosition();
        if (itemPos.getY() + item.getHeight(viewport, graphics) >= nextPos.getY()) {
          item.moveTo(0.0, nextPos.getY() - (item.getHeight(viewport, graphics) + 1));
        }
      }
      lastItemIdx = i;
    }
    
    for (int i = numItems - 1; i > lastItemIdx; i--) {
      PieChartDescriptionItem item = (PieChartDescriptionItem) items.get(i);
      ChartItemPosition itemPos = item.getPosition();
      PieChartDescriptionItem nextItem = null;
      if (i > lastItemIdx) {
        nextItem = (PieChartDescriptionItem) items.get(i - 1);
        ChartItemPosition nextPos = nextItem.getPosition();
        double delta = (nextPos.getY() + nextItem.getHeight(viewport, graphics)) - itemPos.getY() + 1;
        if (delta > 0) {
          item.move(0.0, delta);
          if (i < numItems - 1) {
            for (int j = i + 1; j < numItems; j++) {
              PieChartDescriptionItem itm = (PieChartDescriptionItem) items.get(j);
              ChartItemPosition itmPos = itm.getPosition();
              itm.moveTo(0.0, itmPos.getY() + delta);
            }
          }
        }
      }
    }

    PieChartDescriptionItem lastItem = (PieChartDescriptionItem) items.get(numItems - 1);
    double bottomPosDelta = (lastItem.getPosition().getY() + lastItem.getHeight(viewport, graphics)) - viewport.getHeight();
    double topPos = lastItem.getPosition().getY() - bottomPosDelta;
    if (bottomPosDelta > 0) {
      for (int i = numItems - 1; i >= lastItemIdx; i--) {
        PieChartDescriptionItem item = (PieChartDescriptionItem) items.get(i);
        item.move(0.0, -bottomPosDelta);
        topPos = item.getPosition().getY();
      }
    }

    for (int i = lastItemIdx - 1; i >= 0; i--) {
      PieChartDescriptionItem item = (PieChartDescriptionItem) items.get(i);
      ChartItemPosition itemPos = item.getPosition();
      double delta = topPos - (itemPos.getY() + item.getHeight(viewport, graphics));
      if (delta < 0) {
        item.move(0, delta);
      }
      topPos = item.getPosition().getY();
    }
  }
  
}
