/*
 * Copyright (C) 2003 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.chart;

import java.awt.Color;
import java.awt.Font;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Name: ChartAttributes<br>
 * Description: A class managing chart attributes.
 * 
 * @author lauerc
 */
public class ChartAttributes {

  /** The chart attributes. */
  private Map attributes = null;

  /**
   * Constructs a ChartException instance with specific attributes.
   */
  public ChartAttributes() {
    this.attributes = new HashMap();
  }

  /**
   * Constructs a ChartException instance with specific attributes.
   * @param attributes The chart attributes.
   */
  public ChartAttributes(Map attributes) {
    this.attributes = attributes;
  }

  /**
   * Gets a integer attribute.
   * @param name The attribute name.
   * @return The associated value.
   */
  public boolean isDefined(String name) {
    return attributes.get(name) != null;
  }

  /**
   * Gets a boolean attribute.
   * @param name The attribute name.
   * @return The associated value.
   */
  public boolean getBoolean(String name) {
    Boolean value = (Boolean) attributes.get(name);
    if (value == null) {
      return false;
    }
    return value.booleanValue();
  }
  /**
   * Sets a boolean attribute.
   * @param name The attribute name.
   * @param value The value to associate with the given name.
   */
  public void setBoolean(String name, boolean value) {
    attributes.put(name, new Boolean(value));
  }

  /**
   * Gets a integer attribute.
   * @param name The attribute name.
   * @return The associated value.
   */
  public int getInt(String name) {
    Number value = (Number) attributes.get(name);
    if (value == null) {
      return 0;
    }
    return value.intValue();
  }
  /**
   * Sets a integer attribute.
   * @param name The attribute name.
   * @param value The value to associate with the given name.
   */
  public void setInt(String name, int value) {
    attributes.put(name, new Integer(value));
  }

  /**
   * Gets a long attribute.
   * @param name The attribute name.
   * @return The associated value.
   */
  public double getLong(String name) {
    Number value = (Number) attributes.get(name);
    if (value == null) {
      return 0L;
    }
    return value.longValue();
  }
  /**
   * Sets a long attribute.
   * @param name The attribute name.
   * @param value The value to associate with the given name.
   */
  public void setLong(String name, long value) {
    attributes.put(name, new Long(value));
  }

  /**
   * Gets a float attribute.
   * @param name The attribute name.
   * @return The associated value.
   */
  public float getFloat(String name) {
    Number value = (Number) attributes.get(name);
    if (value == null) {
      return 0.0F;
    }
    return value.floatValue();
  }
  /**
   * Sets a float attribute.
   * @param name The attribute name.
   * @param value The value to associate with the given name.
   */
  public void setFloat(String name, float value) {
    attributes.put(name, new Float(value));
  }

  /**
   * Gets a double attribute.
   * @param name The attribute name.
   * @return The associated value.
   */
  public double getDouble(String name) {
    Number value = (Number) attributes.get(name);
    if (value == null) {
      return 0.0;
    }
    return value.doubleValue();
  }
  /**
   * Sets a double attribute.
   * @param name The attribute name.
   * @param value The value to associate with the given name.
   */
  public void setDouble(String name, double value) {
    attributes.put(name, new Double(value));
  }

  /**
   * Gets a String attribute.
   * @param name The attribute name.
   * @return The associated value.
   */
  public String getString(String name) {
    Object value = attributes.get(name);
    if (value == null) {
      return null;
    }
    return value.toString();
  }
  /**
   * Sets a String attribute.
   * @param name The attribute name.
   * @param value The value to associate with the given name.
   */
  public void setString(String name, String value) {
    attributes.put(name, value);
  }

  /**
   * Gets a String array attribute.
   * @param name The attribute name.
   * @return The associated value.
   */
  public String[] getStringArray(String name) {
    List values = getList(name);
    int numValues = values.size();
    String[] strings = new String[numValues];
    for (int i = 0; i < numValues; i++) {
      Object value = values.get(i);
      if (value != null) {
        strings[i] = value.toString();
      }      
    }
    return strings;
  }

  /**
   * Gets a Font attribute.
   * @param name The attribute name.
   * @return The associated value.
   */
  public Font getFont(String name) {
    Font value = (Font) attributes.get(name);
    return value;
  }
  /**
   * Sets a Font attribute.
   * @param name The attribute name.
   * @param value The value to associate with the given name.
   */
  public void setFont(String name, Font value) {
    attributes.put(name, value);
  }

  /**
   * Gets a Color attribute.
   * @param name The attribute name.
   * @return The associated value.
   */
  public Color getColor(String name) {
    Color value = (Color) attributes.get(name);
    return value;
  }
  /**
   * Sets a Color attribute.
   * @param name The attribute name.
   * @param value The value to associate with the given name.
   */
  public void setColor(String name, Color value) {
    attributes.put(name, value);
  }

  /**
   * Gets a Color array attribute.
   * @param name The attribute name.
   * @return The associated value.
   */
  public Color[] getColorArray(String name) {
    List values = getList(name);
    int numValues = values.size();
    Color[] colors = new Color[numValues];
    for (int i = 0; i < numValues; i++) {
      colors[i] = (Color) values.get(i);
    }
    return colors;
  }

  /**
   * Gets a Object attribute.
   * @param name The attribute name.
   * @return The associated value.
   */
  public Object getObject(String name) {
    Object value = attributes.get(name);
    return value;
  }
  /**
   * Sets a Object attribute.
   * @param name The attribute name.
   * @param value The value to associate with the given name.
   */
  public void setObject(String name, Object value) {
    attributes.put(name, value);
  }

  /**
   * Gets a list attribute.
   * @param name The attribute name.
   * @return The associated value.
   */
  public List getList(String name) {
    List value = (List) attributes.get(name);
    return value;
  }
  /**
   * Sets a list attribute.
   * @param name The attribute name.
   * @param value The value to associate with the given name.
   */
  public void setList(String name, List value) {
    attributes.put(name, value);
  }

  /**
   * Gets a TextItem attribute.
   * @param name The attribute name.
   * @return The associated value.
   */
  public TextChartItem getTextItem(String name) {
    Object value = attributes.get(name);
    return (TextChartItem) value;
  }
  /**
   * Sets a TextItem attribute.
   * @param name The attribute name.
   * @param value The value to associate with the given name.
   */
  public void setTextItem(String name, TextChartItem value) {
    attributes.put(name, value);
  }

}
