/*
 * Copyright (C) 2003 2004, 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.chart;

import java.util.HashMap;
import java.util.Map;

/**
 * Name: PieChart2D<br>
 * Description: A class rendering 2D pie charts.
 * 
 * @author lauerc
 */
public class ChartFactory {

  /** The chart type registry. */
  private static final Map CHART_TYPES = new HashMap();
  
  /**
   * Registers the given chart class.
   * @param name The chart's name.
   * @param clazz The chart class to register.
   */
  public static final void registerChart(String name, Class clazz) {
    synchronized (CHART_TYPES) {
      CHART_TYPES.put(name, clazz);
    }
  }
  
  static {
    // registering the chart types
    registerChart("bar-2d", BarChart2D.class);
    registerChart("stacked-bar-2d", StackedBarChart2D.class);
    registerChart("line-2d", LineChart2D.class);
    registerChart("stacked-area-2d", StackedAreaChart2D.class);
    registerChart("pie-2d", PieChart2D.class);
    registerChart("pie-3d", PieChart3D.class);
    registerChart("donut-3d", DonutChart3D.class);
  }

  /**
   * Creates a chart instance.
   * @param type The name of the chart type to instanciate.
   * @return The created chart, or null if the chart type is not registered.
   * @exception ChartException In case the chart construction or configuration failed.
   */
  public static Chart createChart(String type) 
    throws ChartException {
    return createChart(type, null, null);
  }

  /**
   * Creates a chart instance.
   * @param type The name of the chart type to instanciate.
   * @param model The chart model.
   * @param attributes The chart attributes.
   * @return The created chart, or null if the chart type is not registered.
   * @exception ChartException In case the chart construction or configuration failed.
   */
  public static Chart createChart(String type, ChartModel model,
                                  ChartAttributes attributes) 
    throws ChartException {
    try {
      Class clazz = (Class) CHART_TYPES.get(type);
      // creating chart instance
      Chart chart = (Chart) clazz.newInstance();
      // configuring chart
      chart.setModel(model);
      chart.setAttributes(attributes);
      return chart;
    } catch (IllegalAccessException ex) {
      throw new ChartException("The chart type '" + type + "' is not allowed to be accessed: " + ex.getMessage(), ex);
    } catch (InstantiationException ex) {
      throw new ChartException("The chart type '" + type + "' cannot get instanciated: " + ex.getMessage(), ex);
    }
  }

}
