/*
 * Copyright (C) 2003, 2004 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.chart;

import java.util.List;

/**
 * Name: DefaultChartModel<br>
 * Description: Default implementation for chart models.
 * 
 * @author lauerc
 */
public class DefaultChartModel implements ChartModel {

  /** The chart's title item. */
  protected TextChartItem title = null; 
  /** The chart data. */
  protected List chartData = null; 
  /** The chart area labels. */
  protected List areaLabels = null; 
  /** The chart segment labels. */
  protected List segmentLabels = null; 
  /** The minimum data value. */
  protected double minValue = 0.0;
  /** The maximum data value. */
  protected double maxValue = 0.0;
   
  /**
   * Constructs a DefaultChartModel instance with no specific attributes.
   */
  public DefaultChartModel() {
  }

  /**
   * @see org.ujac.chart.ChartModel#getTitle()
   */
  public TextChartItem getTitle() {
    return title;
  }
  /**
   * @see org.ujac.chart.ChartModel#setTitle(org.ujac.chart.TextChartItem)
   */
  public void setTitle(TextChartItem title) {
    this.title = title;
    
  }

  /**
   * @see org.ujac.chart.ChartModel#getChartData()
   */
  public List getChartData() {
    return chartData;
  }
  /**
   * @see org.ujac.chart.ChartModel#setChartData(java.util.List)
   */
  public void setChartData(List chartData) {
    this.chartData = chartData;
    if (chartData == null) {
      return;
    }
    int numSegments = chartData.size();
    double min = minValue;
    double max = maxValue;
    for (int i = 0; i < numSegments; i++) {
      double[] row = (double[]) chartData.get(i);
      for (int j = 0; j < row.length; j++) {
        double val = row[j];
        min = Math.min(min, val);
        max = Math.max(max, val);
      }
    }
    this.minValue = Math.min(minValue, min);
    this.maxValue = Math.max(maxValue, max);
  }

  /**
   * Gets the chart area labels as a list of Strings.
   * @return A list of Strings.
   */
  public List getAreaLabels() {
    return areaLabels;
  }
  /**
   * Sets the chart area labels as a list of Strings.
   * @param areaLabels A list of Strings.
   */
  public void setAreaLabels(List areaLabels) {
    this.areaLabels = areaLabels;
  }

  /**
   * Gets the chart segment labels as a list of Strings.
   * @return A list of Strings.
   */
  public List getSegmentLabels() {
    return segmentLabels;
  }
  /**
   * Sets the chart segment labels as a list of Strings.
   * @param segmentLabels A list of Strings.
   */
  public void setSegmentLabels(List segmentLabels) {
    this.segmentLabels = segmentLabels;
  }

  /**
   * Gets the minimum value from the chartData.
   * @return The minimum chart data value.
   */
  public double getMinValue() {
    return minValue;
  }
  /**
   * Sets the minimum value from the chart data.
   * @param minValue The minimum chart data value.
   */
  public void setMinValue(double minValue) {
    this.minValue = minValue;
  }

  /**
   * Gets the maximum value from the chartData.
   * @return The maximum chart data value.
   */
  public double getMaxValue() {
    return maxValue;
  }
  /**
   * Sets the maximum value from the chart data.
   * @param maxValue The maximum chart data value.
   */
  public void setMaxValue(double maxValue) {
    this.maxValue = maxValue;
  }

  /**
   * Gets the number of segments.
   * @return The number of segments.
   */
  public int getNumSegments() {
    return chartData.size();
  }

  /**
   * Gets the number of areas.
   * @return The number of areas (the segment lengths).
   */
  public int getNumAreas() {
    int numAreas = 0;
    int numSegments = chartData.size();
    for (int i = 0; i < numSegments; i++) {
      double[] row = (double[]) chartData.get(i);
      numAreas = Math.max(numAreas, row.length);
    }
    return numAreas;
  }
}
