/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.chart;

import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.List;

/**
 * Name: DescriptionPattern<br>
 * Description: Class for more flexible item descriptions.
 * A class processing patterns for chart item descriptions.
 * Valid pattern symbols are: 
 *   %s: The segment label for the item to process.
 *   %a: The area label for the item to process.
 *   %p: The relative value in percent.
 *   %v: The absolute value of the item to process.
 *   %%: The percent sign
 * 
 * @author lauerc
 */
public class DescriptionPattern {

  /**
   * Name: PatternItem<br>
   * Description: Interface for pattern items.
   */
  private interface PatternItem {
    /**
     * Gets the content of the item.
     * @return The content of the item.
     */
    String getContent();
  }
  
  /**
   * Name: StaticItem<br>
   * Description: A class for items that hold a simple static content.
   */
  private class StaticItem implements PatternItem {
    
    /** The static content. */
    private String content = null;
    
    /**
     * Constructs a new StaticItem instance with no specific attributes.
     * @param content The intial content.
     */
    public StaticItem(String content) {
      this.content = content;
    }
    
    /**
     * @see org.ujac.chart.DescriptionPattern.PatternItem#getContent()
     */
    public String getContent() {
      return content;
    }
  }

  /**
   * Name: SegmentLabelItem<br>
   * Description: A class for items that hold the item's segment label.
   */
  private class SegmentLabelItem implements PatternItem {
    
    /**
     * Constructs a SegmentLabelItem instance with no specific attributes.
     */
    public SegmentLabelItem() {
    }

    /**
     * @see org.ujac.chart.DescriptionPattern.PatternItem#getContent()
     */
    public String getContent() {
      return getSegmentLabel();
    }
  }

  /**
   * Name: AreaLabelItem<br>
   * Description: A class for items that hold the item's area label.
   */
  private class AreaLabelItem implements PatternItem {
    
    /**
     * Constructs a AreaLabelItem instance with no specific attributes.
     */
    public AreaLabelItem() {
    }
    
    /**
     * @see org.ujac.chart.DescriptionPattern.PatternItem#getContent()
     */
    public String getContent() {
      return getAreaLabel();
    }
  }

  /**
   * Name: ValueItem<br>
   * Description: A class for items that hold the item value.
   */
  private class ValueItem implements PatternItem {
    
    /**
     * Constructs a ValueItem instance with no specific attributes.
     */
    public ValueItem() {
    }

    /**
     * @see org.ujac.chart.DescriptionPattern.PatternItem#getContent()
     */
    public String getContent() {
      return getValue();
    }
  }

  /**
   * Name: PercentItem<br>
   * Description: A class for items that hold the item percentage.
   */
  private class PercentItem implements PatternItem {
    
    /**
     * Constructs a PercentItem instance with no specific attributes.
     */
    public PercentItem() {
    }

    /**
     * @see org.ujac.chart.DescriptionPattern.PatternItem#getContent()
     */
    public String getContent() {
      return getPercent();
    }
  }

  /** The pattern string. */
  private String pattern = null;
  /** The compiled pattern. */
  private PatternItem[] patternItems = null;
  /** The value number format. */
  private NumberFormat valueNumberFormat = null;
  /** The value number format. */
  private NumberFormat percentNumberFormat = null;

  /** The current value. */
  private double value = 0.0;
  /** The current percentage. */
  private double percent = 0.0;
  /** The current segment label. */
  private String segmentLabel = null;
  /** The current area label. */
  private String areaLabel = null;

  /**
   * Constructs a new DescriptionPattern instance with specific attributes.
   * @param pattern The pattern string.
   * @param valueNumberFormat The format for value output.
   * @param percentNumberFormat The format for percent output.
   * @throws ChartException in case the parsing of the pattern failed.
   */
  public DescriptionPattern(String pattern, NumberFormat valueNumberFormat,
                            NumberFormat percentNumberFormat)
    throws ChartException {
    
    this.pattern = pattern;
    this.valueNumberFormat = valueNumberFormat;
    this.percentNumberFormat = percentNumberFormat;
    // compiling pattern
    compile();
  }
  
  /**
   * 'Compiles' the pattern.
   * @throws ChartException in case the parsing of the pattern failed.
   */
  private void compile() throws ChartException {
    List patternItemList = new ArrayList();
    char[] patternChars = pattern.toCharArray();
    int lastOffset = 0;
    for (int i = 0; i < patternChars.length; i++) {
      switch (patternChars[i]) {
        case '%':
          if (i < patternChars.length - 1) {
            char c = patternChars[i + 1];
            switch (c) {
              case '%':
                // OK, a real percent sign was requested
                patternItemList.add(new StaticItem(new String(patternChars, lastOffset, i - lastOffset + 1)));
                ++i;
                lastOffset = i + 1;
                break;

              case 's':
              case 'a':
              case 'p':
              case 'v':
                // adding previous content as static item
                if (lastOffset < i) {
                  patternItemList.add(new StaticItem(new String(patternChars, lastOffset, i - lastOffset)));
                }
                ++i;
                lastOffset = i + 1;
                // adding special item
                if (c == 's') {
                  patternItemList.add(new SegmentLabelItem());
                } else if (c == 'a') {
                    patternItemList.add(new AreaLabelItem());
                } else if (c == 'v') {
                  patternItemList.add(new ValueItem());
                } else if (c == 'p') {
                  patternItemList.add(new PercentItem());
                }
                break;
              
              default:
                throw new ChartException("Unsupported token in pattern '" + pattern + "' at pos " + i + ".");
            }
          }
          break;
           
        default:
      }
      
    }
    if (lastOffset < patternChars.length) {
      patternItemList.add(new StaticItem(new String(patternChars, lastOffset, patternChars.length - lastOffset)));        
    }
    
    patternItems = new PatternItem[patternItemList.size()];
    for (int i = 0; i < patternItems.length; i++) {
      patternItems[i] = (PatternItem) patternItemList.get(i);
    }
  }

  /**
   * Applies the given values to the pattern.
   * @param value The current value.
   * @param percent The current percentage.
   * @param segmentLabel The item's segment label.
   * @param areaLabel The item's area label.
   * @return The formatted string.
   */
  public String apply(double value, double percent, String segmentLabel, String areaLabel) {
    this.value = value;
    this.percent = percent;
    this.segmentLabel = segmentLabel;
    this.areaLabel = areaLabel;
    
    StringBuffer outBuf = new StringBuffer();
    for (int i = 0; i < patternItems.length; i++) {
      PatternItem patternItem = patternItems[i];
      outBuf.append(patternItem.getContent());
    }
    
    return outBuf.toString();
  }  
  
  /**
   * Gets the current segment label
   * @return The current segment label.
   */
  public String getSegmentLabel() {
    return this.segmentLabel;
  } 

  /**
   * Gets the current area label
   * @return The current area label.
   */
  public String getAreaLabel() {
    return this.areaLabel;
  } 

  /**
   * Gets the current value.
   * @return The current item value.
   */
  public String getValue() {
    return this.valueNumberFormat.format(this.value);
  } 

  /**
   * Gets the current percentage.
   * @return The current item percentage.
   */
  public String getPercent() {
    return this.percentNumberFormat.format(this.percent);
  }

}
