/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.chart;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Stroke;

import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;

import java.util.List;

/**
 * Name: LineChart2D<br>
 * Description: A class rendering two dimensional line charts.
 * 
 * @author lauerc
 */
public class LineChart2D extends Base2DScaleChart {

  /**
   * Rules, whether the area below the line is filled or not.
   * @return The flag, ruling wheter the area below the line is filled or not.
   */
  public boolean getFillArea() {
    if (attributes.isDefined(ATTR_FILL_AREA)) {
      return attributes.getBoolean(ATTR_FILL_AREA);      
    }
    return false;
  }

  /**
   * @see org.ujac.chart.Chart#draw(java.awt.geom.Rectangle2D, java.awt.Graphics2D)
   */
  public Rectangle2D draw(Rectangle2D viewport, Graphics2D graphics) 
    throws ChartException {
    Rectangle2D localViewport = super.draw(viewport, graphics);
    if ((model == null) || (model.getChartData() == null)) {
      return localViewport;
    }

    // calculating the effective viewport
    Rectangle effectiveViewport = calcViewport(localViewport, graphics);
    // drawing the scale
    drawScale(localViewport, effectiveViewport, graphics);
    // drawing the legend
    drawLegend(localViewport, graphics);

    Stroke oldStroke = graphics.getStroke();
    graphics.setStroke(new BasicStroke(getLineWidth(), BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));

    /** The segment colors. */
    Color[] segmentColors = attributes.getColorArray(ATTR_SEGMENT_COLORS); 

    List chartData = model.getChartData();
    int numSegments = chartData.size();
    
    boolean fillArea = getFillArea();
    float baseLineY = calculateBaseLinePosition();

    
    GeneralPath[] segmentPaths = new GeneralPath[model.getNumAreas()];
    GeneralPath[] segmentAreas = new GeneralPath[model.getNumAreas()];

    for (int i = 0; i < numSegments; i++) {
      double[] row = (double[]) chartData.get(i);
      float curPos = horizontalPositions[i];
      for (int j = 0; j < row.length; j++) {
        if (i == 0) {
          float yPos = calculateVerticalPosition(row[j]);
          segmentPaths[j] = new GeneralPath();
          segmentPaths[j].moveTo(curPos, yPos);
          if (fillArea) {
            segmentAreas[j] = new GeneralPath();
            segmentAreas[j].moveTo(curPos, baseLineY);
            segmentAreas[j].lineTo(curPos, yPos);
          }
        } else {
          float yPos = calculateVerticalPosition(row[j]);
          segmentPaths[j].lineTo(curPos, yPos);
          if (fillArea) {
            segmentAreas[j].lineTo(curPos, yPos);
            if (i == numSegments - 1) {
              // closing area segment
              segmentAreas[j].lineTo(curPos, baseLineY);
            }
          }
        }
      }
    }

    for (int j = 0; j < segmentPaths.length; j++) {
      Color color = segmentColors[j];
      if (fillArea) {
        //graphics.setColor(new Color(color.getRed(), color.getGreen(), color.getBlue(), 128));
        graphics.setColor(getFillColor(color));
        graphics.fill(segmentAreas[j]);
      }
      graphics.setColor(color);
      graphics.draw(segmentPaths[j]);
    }
    
    graphics.setStroke(oldStroke);
    
    return localViewport;
  }
  
  /**
   * Computes the fill color for value area.
   * @param color The color to transform.
   * @return The transformed color.
   */
  private Color getFillColor(Color color)  {
    int r = color.getRed();
    int g = color.getGreen();
    int b = color.getBlue();
    return new Color(r, g, b, 128);
  }
}
