/*
 * Copyright (C) 2003 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.chart;

import java.awt.Color;
import java.awt.Graphics2D;

import java.awt.geom.AffineTransform;
import java.awt.geom.Arc2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.geom.PathIterator;

/**
 * Name: PieChart2D<br>
 * Description: A class rendering 2D pie charts.
 * 
 * @author lauerc
 */
public class PieChart2D extends BasePieChart {

  /**
   * Gets the number of drawing steps to perform to draw the slices.
   * @return The number of drawing steps.
   */
  protected int getNumDrawingSteps() {
    return 1;
  }

  /**
   * Draws a pie slice.
   * @param graphics The graphics object to draw width.
   * @param color The slice's color.
   * @param sliceIdx The index of the slice to draw.
   * @param stepIdx The index of the drawing step to perform.
   */
  protected void drawSlice(Graphics2D graphics, Color color,
                           int sliceIdx, int stepIdx) {

    double deltaY = getDeltaY();

    double startAngle = startAngles[sliceIdx];
    double angleDiff = angleDiffs[sliceIdx];
    double startArc = startArcs[sliceIdx];
    double endArc = endArcs[sliceIdx];
  
    double pieWidth = getPieSize().getWidth();
    double pieHeight = getPieSize().getHeight();

    AffineTransform rotate = new AffineTransform();
    Line2D.Double startLine = new Line2D.Double(0, halfPieHeight, 
                                                0, halfPieHeight);
    PathIterator pIterStart = startLine.getPathIterator(rotate);
    rotate.rotate(-startArc);
    double[] coordFrom = new double[2];
    pIterStart.currentSegment(coordFrom);
    coordFrom[0] /= deltaY;

    rotate = new AffineTransform();
    rotate.rotate(-endArc);
    double[] coordTo = new double[2];
    pIterStart.currentSegment(coordTo);
    coordTo[0] /= deltaY;

    GeneralPath path = new GeneralPath();
    path.append(new Line2D.Double(coordTo[0], coordTo[1], 0.0, 0.0), true);
    path.append(new Arc2D.Double(-halfPieWidth, -halfPieHeight, 
                                 pieWidth, pieHeight, startAngle, angleDiff,
                                 Arc2D.OPEN), true);
    path.append(new Line2D.Double(0.0, 0.0, coordFrom[0], coordFrom[1]), true);
    path.transform(moveToCenter);
    graphics.setColor(color);
    graphics.fill(path);    
  }

  /**
   * Draws the shadow of a pie slice.
   * @param graphics The graphics object to draw width.
   * @param color The slice's color.
   * @param sliceIdx The index of the slice to draw.
   */
  protected void drawSliceShadow(Graphics2D graphics, Color color, int sliceIdx) {

    double deltaY = getDeltaY();

    double startAngle = startAngles[sliceIdx];
    double angleDiff = angleDiffs[sliceIdx];
    double startArc = startArcs[sliceIdx];
    double endArc = endArcs[sliceIdx];
  
    double pieWidth = getPieSize().getWidth();
    double pieHeight = getPieSize().getHeight();

    AffineTransform rotate = new AffineTransform();
    Line2D.Double startLine = new Line2D.Double(0, halfPieHeight, 
                                                0, halfPieHeight);
    PathIterator pIterStart = startLine.getPathIterator(rotate);
    rotate.rotate(-startArc);
    double[] coordFrom = new double[2];
    pIterStart.currentSegment(coordFrom);
    coordFrom[0] /= deltaY;

    rotate = new AffineTransform();
    rotate.rotate(-endArc);
    double[] coordTo = new double[2];
    pIterStart.currentSegment(coordTo);
    coordTo[0] /= deltaY;

    GeneralPath path = new GeneralPath();
    path.append(new Line2D.Double(coordTo[0], coordTo[1], 0.0, 0.0), true);
    path.append(new Arc2D.Double(-halfPieWidth, -halfPieHeight, 
                                 pieWidth, pieHeight, startAngle, angleDiff,
                                 Arc2D.OPEN), true);
    path.append(new Line2D.Double(0.0, 0.0, coordFrom[0], coordFrom[1]), true);
    path.transform(moveToCenter);
    graphics.setColor(color);
    graphics.fill(path);    
  }
}
