/*
 * Copyright (C) 2003 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.chart;

import java.awt.Color;
import java.awt.Graphics2D;

import java.awt.geom.AffineTransform;
import java.awt.geom.Arc2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.geom.PathIterator;

/**
 * Name: PieChart3D<br>
 * Description: A class rendering 3D pie charts.
 * 
 * @author lauerc
 */
public class PieChart3D extends BasePieChart3D {

  /**
   * Gets the number of drawing steps to perform to draw the slices.
   * @return The number of drawing steps.
   */
  protected int getNumDrawingSteps() {
    return 2;
  }

  /**
   * Gets the index of the step before which to draw the chart
   * description.
   * @return The index of the step before which to draw the description.
   */
  protected int getDescriptionDrawingStep() {
    return 1;
  }

  /**
   * Draws a pie slice.
   * @param graphics The graphics object to draw width.
   * @param color The slice's color.
   * @param sliceIdx The index of the slice to draw.
   * @param stepIdx The index of the drawing step to perform.
   */
  protected void drawSlice(Graphics2D graphics, Color color,
                           int sliceIdx, int stepIdx) {

    double deltaY = getDeltaY();

    double startAngle = startAngles[sliceIdx];
    double endAngle = endAngles[sliceIdx];
    double angleDiff = angleDiffs[sliceIdx];
    double startArc = startArcs[sliceIdx];
    double endArc = endArcs[sliceIdx];
  
    double pieWidth = getPieSize().getWidth();
    double pieHeight = getPieSize().getHeight();

    double sliceHeight = getSliceHeight();
  
    AffineTransform rotate = new AffineTransform();
    Line2D.Double startLine = new Line2D.Double(0, halfPieHeight, 
                                                0, halfPieHeight + sliceHeight);

    PathIterator pIterStart = startLine.getPathIterator(rotate);
    rotate.rotate(-startArc);
    double[] coordFrom = new double[2];
    pIterStart.currentSegment(coordFrom);
    coordFrom[0] /= deltaY;
    
    rotate = new AffineTransform();
    rotate.rotate(-endArc);
    double[] coordTo = new double[2];
    pIterStart.currentSegment(coordTo);
    coordTo[0] /= deltaY;

    GeneralPath path = null;
    
    if (stepIdx == 0) {
      if ((startAngle > 0) && (startAngle < 180)) {
        double angleDiff180 = startAngle - 180;
        double angleDiffCut = angleDiff180 + angleDiff;
        if (angleDiffCut > 0.0) {
          double startAngleCut = Math.PI * 1.5;
          rotate = new AffineTransform();
          PathIterator pIterStartCut = startLine.getPathIterator(rotate);
          rotate.rotate(-startAngleCut);
          double[] coordFromCut = new double[2];
          pIterStartCut.currentSegment(coordFromCut);
          coordFromCut[0] /= deltaY;
          
          path = new GeneralPath();
          path.append(new Arc2D.Double(-halfPieWidth, -halfPieHeight, 
                                       pieWidth, pieHeight, 180, angleDiffCut,
                                       Arc2D.OPEN), true);
          path.append(new Arc2D.Double(-halfPieWidth, -halfPieHeight + sliceHeight, 
                                       pieWidth, pieHeight, endAngle, -angleDiffCut,
                                       Arc2D.OPEN), true);
          path.append(new Line2D.Double(coordFromCut[0], coordFromCut[1],
                                        coordFromCut[0], 
                                        coordFromCut[1] + sliceHeight), true);
        }
      } else {
        path = new GeneralPath();
        path.append(new Arc2D.Double(-halfPieWidth, -halfPieHeight, 
                                     pieWidth, pieHeight, startAngle, angleDiff,
                                     Arc2D.OPEN), true);
        path.append(new Arc2D.Double(-halfPieWidth, -halfPieHeight + sliceHeight, 
                                     pieWidth, pieHeight, endAngle, -angleDiff,
                                     Arc2D.OPEN), true);
        path.append(new Line2D.Double(coordFrom[0], coordFrom[1],
                                      coordFrom[0], 
                                      coordFrom[1] + sliceHeight), true);
      }
          
      if (path != null) {
        path.transform(moveToCenter);
        graphics.setColor(color.darker());
        graphics.fill(path);
      }
    } else {
      // drawing surface
      path = new GeneralPath();
      path.append(new Line2D.Double(coordTo[0], coordTo[1], 0.0, 0.0), true);
      path.append(new Arc2D.Double(-halfPieWidth, -halfPieHeight, 
                                   pieWidth, pieHeight, startAngle, angleDiff,
                                   Arc2D.OPEN), true);
      path.append(new Line2D.Double(0.0, 0.0, coordFrom[0], coordFrom[1]), true);
      path.transform(moveToCenter);
      graphics.setColor(color);
      graphics.fill(path);
    }
  }

  /**
   * Draws the shadow of a pie slice.
   * @param graphics The graphics object to draw width.
   * @param color The slice's color.
   * @param sliceIdx The index of the slice to draw.
   */
  protected void drawSliceShadow(Graphics2D graphics, Color color, int sliceIdx) {

    double deltaY = getDeltaY();

    double startAngle = startAngles[sliceIdx];
    double angleDiff = angleDiffs[sliceIdx];
    double startArc = startArcs[sliceIdx];
    double endArc = endArcs[sliceIdx];
  
    double pieWidth = getPieSize().getWidth();
    double pieHeight = getPieSize().getHeight();

    double sliceHeight = getSliceHeight();
  
    AffineTransform rotate = new AffineTransform();
    Line2D.Double startLine = new Line2D.Double(0, halfPieHeight, 
                                                0, halfPieHeight + sliceHeight);

    PathIterator pIterStart = startLine.getPathIterator(rotate);
    rotate.rotate(-startArc);
    double[] coordFrom = new double[2];
    pIterStart.currentSegment(coordFrom);
    coordFrom[0] /= deltaY;
    
    rotate = new AffineTransform();
    rotate.rotate(-endArc);
    double[] coordTo = new double[2];
    pIterStart.currentSegment(coordTo);
    coordTo[0] /= deltaY;

    // drawing shadow
    GeneralPath path = new GeneralPath();
    path.append(new Line2D.Double(coordTo[0], coordTo[1] + sliceHeight, 0.0, 0.0 + sliceHeight), true);
    path.append(new Arc2D.Double(-halfPieWidth, -halfPieHeight + sliceHeight, 
                                 pieWidth, pieHeight, startAngle, angleDiff,
                                 Arc2D.OPEN), true);
    path.append(new Line2D.Double(0.0, 0.0 + sliceHeight, coordFrom[0], coordFrom[1] + sliceHeight), true);
    path.transform(moveToCenter);
    graphics.setColor(color);
    graphics.fill(path);
  }    

}
