/*
 * Copyright (C) 2005 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more etails.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.chart;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Stroke;

import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;

import java.util.List;

/**
 * Name: StackedBarChart2D<br>
 * Description: A class rendering two dimensional stacked area charts.
 * 
 * @author lauerc
 */
public class StackedAreaChart2D extends Base2DStackedChart {
  
  /**
   * Rules, whether the area below the line is filled or not.
   * @return The flag, ruling wheter the area below the line is filled or not.
   */
  public boolean getFillArea() {
    if (attributes.isDefined(ATTR_FILL_AREA)) {
      return attributes.getBoolean(ATTR_FILL_AREA);      
    }
    return true;
  }
  
  /**
   * Computes the fill color for value area.
   * @param color The color to transform.
   * @return The transformed color.
   */
  private Color getFillColor(Color color)  {
    int r = color.getRed();
    int g = color.getGreen();
    int b = color.getBlue();
    return new Color(r, g, b, 128);
  }

  /**
   * @see org.ujac.chart.Chart#draw(java.awt.geom.Rectangle2D, java.awt.Graphics2D)
   */
  public Rectangle2D draw(Rectangle2D viewport, Graphics2D graphics) 
    throws ChartException {
    
    Rectangle2D localViewport = super.draw(viewport, graphics);
    if ((model == null) || (model.getChartData() == null)) {
      return localViewport;
    }

    /** The segment colors. */
    Color[] segmentColors = attributes.getColorArray(ATTR_SEGMENT_COLORS); 

    List chartData = model.getChartData();
    int numSegments = chartData.size();
    
    // calculating the effective viewport
    Rectangle effectiveViewport = calcViewport(localViewport, graphics);
    // drawing the scale
    drawScale(localViewport, effectiveViewport, graphics);
    // drawing the legend
    drawLegend(localViewport, graphics);
    
    Stroke oldStroke = graphics.getStroke();
    graphics.setStroke(new BasicStroke(getLineWidth(), BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));

    float[][] yPositions = new float[model.getNumAreas()][];
    GeneralPath[] segmentPaths = new GeneralPath[model.getNumAreas()];
    GeneralPath[] segmentAreas = new GeneralPath[model.getNumAreas()];

    boolean fillArea = getFillArea();
    float nullPos = calculateBaseLinePosition();

    int numAreas = model.getNumAreas();
    for (int i = 0; i < numSegments; i++) {
      double[] row = (double[]) chartData.get(i);
      float curPos = horizontalPositions[i];
      float yOffset = nullPos; 
      for (int j = 0; j < numAreas; j++) {
        float verticalExtent = calculateVerticalExtent(row[j]);
        yOffset -= verticalExtent;
        float yPos = yOffset;
        if (i == 0) {
          yPositions[j] = new float[numSegments];
          yPositions[j][i] = yPos;
          segmentPaths[j] = new GeneralPath();
          segmentPaths[j].moveTo(curPos, yPos);
          if (fillArea) {
            segmentAreas[j] = new GeneralPath();
            if (j == 0) {
              segmentAreas[j].moveTo(curPos, nullPos);
              segmentAreas[j].lineTo(curPos, yPos);
            } else {
              segmentAreas[j].moveTo(curPos, yPos);
            }
          }
        } else {
          yPositions[j][i] = yPos;
          segmentPaths[j].lineTo(curPos, yPos);
          if (fillArea) {
            segmentAreas[j].lineTo(curPos, yPos);
            if ((j == 0) && (i == numSegments - 1)) {
              // closing area segment
              segmentAreas[j].lineTo(curPos, nullPos);
            }
          }
        }
      }
    }

    if (fillArea) {
      for (int j = 1; j < numAreas; j++) {
        GeneralPath segmentArea = segmentAreas[j];
        for (int i = numSegments - 1; i >=0; i--) {
          float[] yPos = yPositions[j - 1];
          float curPos = horizontalPositions[i];
          segmentArea.lineTo(curPos, yPos[i]);
        }
      }
    }
    
    if (fillArea) {
      for (int j = 0; j < segmentPaths.length; j++) {
        Color color = segmentColors[j];
        //graphics.setColor(new Color(color.getRed(), color.getGreen(), color.getBlue(), 128));
        graphics.setColor(getFillColor(color));
        graphics.fill(segmentAreas[j]);
      }
    }

    for (int j = 0; j < segmentPaths.length; j++) {
      Color color = segmentColors[j];
      graphics.setColor(color);
      graphics.setStroke(new BasicStroke(getLineWidth(), BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));

      graphics.draw(segmentPaths[j]);
    }

    graphics.setStroke(oldStroke);

    return localViewport;
  }

}
