/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.chart;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Rectangle;

import java.awt.geom.Rectangle2D;

import java.util.List;
import java.util.ArrayList;

/**
 * Name: ChartTextItem<br>
 * Description: Holds the data, required to precisely place a texual chart item.
 * 
 * @author lauerc
 */
public class TextChartItem extends BaseChartItem {

  /** The default text color. */
  public static final Color DEFAULT_TEXT_COLOR = Color.black;
  
  /**
   * A class holding informations about each single line of the text item.
   */
  private class TextSegment {
    /** The text of the line. */
    private String segmentText;
    /** The horizontzal offset. */
    private double xOffset;
    /** The vertical offset. */
    private double yOffset;
    /** The segment width. */
    private double width;
    /** The line height. */
    private double height;
    
    /**
     * Constructs a TextSegment instance with specific attributes.
     * @param segmentText The text of the line.
     * @param xOffset The horizontal offset.
     * @param yOffset The vertical offset.
     * @param width The segment width.
     * @param height The line height.
     */
    public TextSegment(String segmentText, double xOffset, double yOffset, double width, double height) {
      this.segmentText = segmentText;
      this.xOffset = xOffset;
      this.yOffset = yOffset;
      this.width = width;
      this.height = height;
    }
    
    /**
     * Getter method for the the property height.
     * @return The current value of property height.
     */
    public double getHeight() {
      return height;
    }
    /**
     * Getter method for the the property segmentText.
     * @return The current value of property segmentText.
     */
    public String getSegmentText() {
      return segmentText;
    }
    /**
     * Getter method for the the property width.
     * @return The current value of property width.
     */
    public double getWidth() {
      return width;
    }
    /**
     * Getter method for the the property xOffset.
     * @return The current value of property xOffset.
     */
    public double getXOffset() {
      return xOffset;
    }
    /**
     * Getter method for the the property yOffset.
     * @return The current value of property yOffset.
     */
    public double getYOffset() {
      return yOffset;
    }
  }

  /** The text of the chart item. */
  private String text = null;
  /** A list of text segments. */
  private List textSegments = new ArrayList();
  /** The maximum with, zero means no limit. */
  private int maxWidth = 0;
  /** The font for the chart item. */
  private Font font = null;
  /** The text color to use. */
  private Color color = null;
  /** The horizontal alignment of the text. */
  private int hAlign = ChartItemPosition.LEFT;
  /** The rotation flag. */
  private boolean rotate = false;
  
  /**
   * Constructs a ChartTextItem instance with specific attributes.
   * @param position The position of the chart item.
   * @param text The text of the chart item.
   * @param font The font to render with.
   */
  public TextChartItem(ChartItemPosition position, String text, Font font) {
    this(position, position.getHorizontalPositionMode(), text, font, null);
  }
  
  /**
   * Constructs a ChartTextItem instance with specific attributes.
   * @param position The position of the chart item.
   * @param text The text of the chart item.
   * @param font The font to render with.
   * @param color The text color to use.
   * @param rotate The rotation angle of the text.
   */
  public TextChartItem(ChartItemPosition position, String text, Font font, Color color, boolean rotate) {
    this(position, position.getHorizontalPositionMode(), text, font, color);
    this.rotate = rotate;
  }

  /**
   * Constructs a ChartTextItem instance with specific attributes.
   * @param position The position of the chart item.
   * @param text The text of the chart item.
   * @param font The font to render with.
   * @param color The text color to use.
   */
  public TextChartItem(ChartItemPosition position, String text, Font font, Color color) {
    this(position, position.getHorizontalPositionMode(), text, font, color);
  }

  /**
   * Constructs a ChartTextItem instance with specific attributes.
   * @param position The position of the chart item.
   * @param hAlign The horizontal alignment of the text.
   * @param text The text of the chart item.
   * @param font The font to render with.
   */
  public TextChartItem(ChartItemPosition position, int hAlign, String text, Font font) {
    this(position, 0, hAlign, text, font, null);
  }

  /**
   * Constructs a ChartTextItem instance with specific attributes.
   * @param position The position of the chart item.
   * @param hAlign The horizontal alignment of the text.
   * @param text The text of the chart item.
   * @param font The font to render with.
   * @param color The text color to use.
   */
  public TextChartItem(ChartItemPosition position, int hAlign, String text, Font font, Color color) {
    this(position, 0, hAlign, text, font, color);
  }

  /**
   * Constructs a ChartTextItem instance with specific attributes.
   * @param position The position of the chart item.
   * @param maxWidth The maximm width of the text item.
   * @param hAlign The horizontal alignment of the text.
   * @param text The text of the chart item.
   * @param font The font to render with.
   */
  public TextChartItem(ChartItemPosition position, int maxWidth, int hAlign,
                       String text, Font font) {
    this(position, maxWidth, hAlign, text, font, null);
  }
  
  /**
   * Constructs a ChartTextItem instance with specific attributes.
   * @param position The position of the chart item.
   * @param maxWidth The maximm width of the text item.
   * @param hAlign The horizontal alignment of the text.
   * @param text The text of the chart item.
   * @param font The font to render with.
   * @param color The text color to use.
   */
  public TextChartItem(ChartItemPosition position, int maxWidth, int hAlign,
                       String text, Font font, Color color) {
    super(position);
    this.maxWidth = maxWidth;
    this.hAlign = hAlign;
    this.text = text;
    this.font = font;
    if (color == null) {
      this.color = DEFAULT_TEXT_COLOR;
    } else {
      this.color = color;
    }
  }

  /**
   * Gets the item's text.
   * @return The text of the chart item.
   */
  public String getText() {
    return text;
  }

  /**
   * Gets the maximum width of the item box.
   * @return The maximum width of the item box.
   */
  public int getMaxWidth() {
    return maxWidth;
  }

  /**
   * Gets the item's horizontal alignment.
   * @return The horizontal alignment of the chart item text.
   */
  public int getHAlign() {
    return hAlign;
  }

  /**
   * Gets the item's font.
   * @return The font for the chart item.
   */
  public Font getFont() {
    return font;
  }

  /**
   * Getter method for the the property color.
   * @return The current value of property color.
   */
  public Color getColor() {
    return color;
  }
  
  /**
   * Getter method for the the property angle.
   * @return The current value of property angle.
   */
  public boolean getRotate() {
    return rotate;
  }
   
  /**
   * Setter method for the the property color.
   * @param rotate The new value for the property color.
   * @return this instance.
   */
  public TextChartItem setRotate(boolean rotate) {
    this.rotate = rotate;
    return this;
  }
  
  /**
   * Calculates the position of the item.
   * @param viewport The viewport for chart output.
   * @param graphics The graphics object, needed to calculate the metrics.
   * @return The calculated output area.
   */
  public Rectangle2D calculateOutputArea(Rectangle2D viewport, Graphics2D graphics) {
    int xPos = 0;
    int yPos = 0;
    
    Font oldFont = graphics.getFont();
    graphics.setFont(this.font);
    Rectangle2D textMetrics = graphics.getFontMetrics().getStringBounds(text, graphics);

    int maxWidth = this.maxWidth;
    if ((textMetrics.getWidth() > viewport.getWidth()) && 
        ((maxWidth == 0) || (maxWidth > viewport.getWidth()))) {
      maxWidth = (int) viewport.getWidth();
    }

    double width = 0.0;
    double height = 0.0;
    if (((maxWidth > 0) && (textMetrics.getWidth() > maxWidth)) || 
        (text.indexOf('\n') != -1)) {
      // splitting text segments into parts to fit into maximum space
      char[] textChars = text.toCharArray();
      int segmentOffset = 0;
      int lastWhitespace = 0;
      double yOffset = 0.0;
      for (int i = 0; i < textChars.length; i++) {
        char c = textChars[i];
        switch (c) {
          case '\t':
          case ' ': {
            String segmentText = new String(textChars, segmentOffset, i - segmentOffset);
            Rectangle2D segmentMetrics = graphics.getFontMetrics().getStringBounds(segmentText, graphics);
            if ((maxWidth > 0) && (segmentMetrics.getWidth() > maxWidth)) {
              if (lastWhitespace > 0) {
                // going back to last whitespace
                for (int j = lastWhitespace; j > segmentOffset; j--) {
                  if ((textChars[j] == '(') || (textChars[j] == '[')) {
                    // break before last paranthesis or bracket
                    lastWhitespace = j - 1;
                    break;
                  }
                }
                segmentText = new String(textChars, segmentOffset, lastWhitespace - segmentOffset + 1).trim();
                segmentMetrics = graphics.getFontMetrics().getStringBounds(segmentText, graphics);
                this.textSegments.add(new TextSegment(segmentText, 0.0, yOffset, segmentMetrics.getWidth(), segmentMetrics.getHeight()));
                yOffset += segmentMetrics.getHeight();
                height += segmentMetrics.getHeight();
                width = Math.max(width, segmentMetrics.getWidth());
                --i;
                segmentOffset = lastWhitespace + 1;
                lastWhitespace = 0;
              } else {
                // don't know where to break, so adding (too) long segment.
                this.textSegments.add(new TextSegment(segmentText, 0.0, yOffset, segmentMetrics.getWidth(), segmentMetrics.getHeight()));
                yOffset += segmentMetrics.getHeight();
                height += segmentMetrics.getHeight();
                width = Math.max(width, segmentMetrics.getWidth());
                segmentOffset = i + 1;
                lastWhitespace = 0;
              }
            }
            lastWhitespace = i;
          }
          break;
          
          case '\r':
            break;

          case '\n': {
            // line break found, breaking line
            String segmentText = new String(textChars, segmentOffset, i - segmentOffset).trim();
            Rectangle2D segmentMetrics = graphics.getFontMetrics().getStringBounds(segmentText, graphics);
            this.textSegments.add(new TextSegment(segmentText, 0.0, yOffset, segmentMetrics.getWidth(), segmentMetrics.getHeight()));
            yOffset += segmentMetrics.getHeight();
            height += segmentMetrics.getHeight();
            width = Math.max(width, segmentMetrics.getWidth());
            segmentOffset = i + 1;
            lastWhitespace = 0;
          }
          break;
        }
      }
      if (segmentOffset < textChars.length) {
        String segmentText = new String(textChars, segmentOffset, textChars.length - segmentOffset).trim();
        Rectangle2D segmentMetrics = graphics.getFontMetrics().getStringBounds(segmentText, graphics);
        if ((maxWidth > 0) && (segmentMetrics.getWidth() > maxWidth)) {
          if (lastWhitespace > 0) {
            // going back to last whitespace
            for (int j = lastWhitespace; j > segmentOffset; j--) {
              if ((textChars[j] == '(') || (textChars[j] == '[')) {
                // break before last paranthesis or bracket
                lastWhitespace = j - 1;
                break;
              }
            }
            segmentText = new String(textChars, segmentOffset, lastWhitespace - segmentOffset + 1).trim();
            segmentMetrics = graphics.getFontMetrics().getStringBounds(segmentText, graphics);
            this.textSegments.add(new TextSegment(segmentText, 0.0, yOffset, segmentMetrics.getWidth(), segmentMetrics.getHeight()));
            yOffset += segmentMetrics.getHeight();
            height += segmentMetrics.getHeight();
            width = Math.max(width, segmentMetrics.getWidth());
            // adding the rest
            segmentOffset = lastWhitespace + 1;
            segmentText = new String(textChars, segmentOffset, textChars.length - segmentOffset);
            segmentMetrics = graphics.getFontMetrics().getStringBounds(segmentText, graphics);
            this.textSegments.add(new TextSegment(segmentText, 0.0, yOffset, segmentMetrics.getWidth(), segmentMetrics.getHeight()));
            height += segmentMetrics.getHeight();
            width = Math.max(width, segmentMetrics.getWidth());
          } else {
            // don't know where to break, so adding (too) long segment.
            this.textSegments.add(new TextSegment(segmentText, 0.0, yOffset, segmentMetrics.getWidth(), segmentMetrics.getHeight()));
            yOffset += segmentMetrics.getHeight();
            height += segmentMetrics.getHeight();
            width = Math.max(width, segmentMetrics.getWidth());
          }
        } else {
          this.textSegments.add(new TextSegment(segmentText, 0.0, yOffset, segmentMetrics.getWidth(), segmentMetrics.getHeight()));
          width = Math.max(width, segmentMetrics.getWidth());
          yOffset += segmentMetrics.getHeight();
          height += segmentMetrics.getHeight();
        }
      }      
    } else {
      // no line breaks necessary
      width = textMetrics.getWidth();
      height = textMetrics.getHeight();
      this.textSegments.add(new TextSegment(text, 0.0, 0.0, width, height));
    }
    
    switch (position.getHorizontalPositionMode()) {
      case ChartItemPosition.ABSOLUTE:
        xPos = (int) (position.getX());
        break;
      case ChartItemPosition.CENTER:
        xPos = (int) (viewport.getCenterX()  - width / 2.0);
        break;
      case ChartItemPosition.RIGHT:
        xPos = (int) (viewport.getX() + viewport.getWidth() - width);
        break;
      default: // ChartItemPosition.LEFT:
        xPos = (int) viewport.getX();
        break;
    }
    
    switch (position.getVerticalPositionMode()) {
      case ChartItemPosition.ABSOLUTE:
        yPos = (int) (viewport.getY() + position.getY());
        break;
      case ChartItemPosition.MIDDLE:
        yPos = (int) (viewport.getCenterY()  - height / 2.0);
        break;
      case ChartItemPosition.BOTTOM:
        yPos = (int) (viewport.getY() + viewport.getHeight() - height);
        break;
      default: // ChartItemPosition.TOP:
        yPos = (int) (viewport.getY());
        break;
    }

    graphics.setFont(oldFont);    
    
    if (rotate) {
      return new Rectangle(xPos, yPos, (int) height, (int) width);
    }
    return new Rectangle(xPos, yPos, (int) width, (int) height);
  }

  /** 
   * Draws the item to the given graphic instance.
   * @param viewport The viewport for chart output.
   * @param graphics The graphics object, used for output.
   */
  public void draw(Rectangle2D viewport, Graphics2D graphics) {
    Rectangle2D area = getOutputArea(viewport, graphics);

    // determining the vertical offset
    int xOffset = 0;
    int yOffset = 0;
    xOffset = (int) area.getX();
    TextSegment firstSegment = (TextSegment) textSegments.get(0);

    switch (position.getHorizontalPositionMode()) {
      case ChartItemPosition.ABSOLUTE:
        xOffset = (int) area.getX();
        break;
      case ChartItemPosition.CENTER:
        xOffset = (int) (viewport.getCenterX() - area.getWidth() / 2.0);
        break;
      case ChartItemPosition.RIGHT:
        xOffset = (int) (viewport.getMinX() + viewport.getWidth() - area.getWidth());
        break;
      default: // ChartItemPosition.LEFT:
        xOffset = (int) viewport.getMinX();
        break;
    }

    switch (position.getVerticalPositionMode()) {
      case ChartItemPosition.TOP:
        if (rotate) {
          yOffset = (int) (area.getCenterY() - (area.getWidth() / 2.0) + firstSegment.getHeight());
        } else {
          yOffset = (int) (viewport.getMinY() + firstSegment.getHeight());
        }
        break;
      case ChartItemPosition.BOTTOM:
        yOffset = (int) (viewport.getMaxY() - area.getHeight() + firstSegment.getHeight());
        break;
      case ChartItemPosition.MIDDLE:
        if (rotate) {
          yOffset = (int) (area.getCenterY() - (area.getWidth() / 2.0) + firstSegment.getHeight());
        } else {
          yOffset = (int) (area.getCenterY() - (area.getHeight() / 2.0) + firstSegment.getHeight());
        }
        break;
      case ChartItemPosition.ABSOLUTE:
        yOffset = (int) (area.getY() + firstSegment.getHeight());
        break;
      default: // ChartItemPosition.BOTTOM:
        yOffset = (int) (viewport.getMaxY() - area.getHeight() + firstSegment.getHeight());
        break;
    }

    Graphics2D g = graphics;
    if (rotate) {
      g = (Graphics2D) graphics.create();
      g.rotate(Math.PI * 1.5, area.getCenterX(), area.getCenterY());
    }
    
    g.setFont(this.font);

    g.setColor(this.color);
    int xPos = 0;
    int yPos = 0;
    int numSegments = textSegments.size();
    for (int i = 0; i < numSegments; i++) {
      TextSegment segment = (TextSegment) textSegments.get(i);
      xPos = (int) (xOffset + segment.getXOffset());
      yPos = (int) (yOffset + segment.getYOffset());
      // adjusting horizontal offset
      switch (hAlign) {
        case ChartItemPosition.RIGHT:
          if (rotate) {
            xPos += (int) (area.getHeight() / 2.0 + area.getWidth() / 2.0 - segment.getWidth());
          } else {
            xPos += (int) (area.getWidth() - segment.getWidth());
          }
          break;
        case ChartItemPosition.CENTER:
          xPos += (int) ((area.getWidth() - segment.getWidth()) / 2.0);
          break;
      }
      g.drawString(segment.getSegmentText(), xPos, yPos);
    }
    if (rotate) {
      g.dispose();
    }
  }

}
