/*
 * Copyright (C) 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print;

import java.awt.Color;

/**
 * Name: AbstractTagTest<br>
 * Description: An abstract tag test implementation, used at class <code>TestTagInterceptor</code>
 *   to define single tag tests during document processing.
 * 
 * @author lauerc
 */
public abstract class AbstractTagTest {
  
  /** Tells to perform tests when opening the tag. */
  public static final int TEST_OPEN = 1;
  /** Tells to perform tests when closing the tag. */
  public static final int TEST_CLOSE = 2;
  
  /** The test mode, default is <code>TEST_CLOSE</code>s. */
  private int testMode = TEST_CLOSE;

  /** Tells, whether or not the test has been passed. */
  private boolean testPassed = false;

  /** The tag to test. */
  protected BaseDocumentTag tag = null;

  /**
   * Constructs a AbstractTagTest instance with no specific attributes.
   */
  public AbstractTagTest() {
  }

  /**
   * Constructs a AbstractTagTest instance with specific attributes.
   * @param testMode The initial test mode.
   */
  public AbstractTagTest(int testMode) {
    this.testMode = testMode;
  }
  
  /**
   * Sets up the test.
   * @param tag The tag to test.
   */
  protected void setup(BaseDocumentTag tag) {
    this.tag = tag;
  }

  /**
   * Performs the test.
   * @throws DocumentHandlerException In case a document handler problem occurred, during the tests.
   */
  public abstract void performTest() throws DocumentHandlerException;

  /**
   * Performs the tests, sets the <code>testPassed</code> flag,
   * in case the test was successful.
   * @throws DocumentHandlerException In case a document handler problem occurred, during the tests.
   */
  protected void doIt() throws DocumentHandlerException {
    performTest();
    testPassed = true;
  }

  /**
   * Gets the attached document tag instance.
   * @return The current document tag instance.
   */
  public BaseDocumentTag getTag() {
    return tag;
  }

  /**
   * Gets the current test mode.
   * @return The current test mode.
   */
  public int getTestMode() {
    return testMode;
  }

  /**
   * Tells, whether or not the test has been passed.
   * @return true, if the test has been successfully passed, else false.
   */
  public boolean isTestPassed() {
    return testPassed;
  }
  
  /**
   * Gets a string attribute.
   * @param name The attribute name.
   * @return The according value.
   * @exception TagAttributeException If the item contains an invalid value 
   *    or an required attribute was not defined.
   */
  protected String getStringAttribute(String name) throws TagAttributeException {
    AttributeDefinition ad = tag.getAttributeDefinition(name);
    return tag.getStringAttribute(ad, true, null);
  }
  /**
   * Gets a string style attribute.
   * @param name The attribute name.
   * @return The according value.
   * @exception TagAttributeException If the item contains an invalid value 
   *    or an required attribute was not defined.
   */
  protected String getStringStyleAttribute(String name) throws TagAttributeException {
    AttributeDefinition ad = tag.getStyleAttributeDefinition(name);
    return tag.getStringAttribute(null, true, ad);
  }

  /**
   * Gets a float attribute.
   * @param name The attribute name.
   * @return The according value.
   * @exception TagAttributeException If the item contains an invalid value 
   *    or an required attribute was not defined.
   */
  protected float getFloatAttribute(String name) throws TagAttributeException {
    AttributeDefinition ad = tag.getAttributeDefinition(name);
    return tag.getFloatAttribute(ad, true, null);
  }
  /**
   * Gets a float style attribute.
   * @param name The attribute name.
   * @return The according value.
   * @exception TagAttributeException If the item contains an invalid value 
   *    or an required attribute was not defined.
   */
  protected float getFloatStyleAttribute(String name) throws TagAttributeException {
    AttributeDefinition ad = tag.getStyleAttributeDefinition(name);
    return tag.getFloatAttribute(null, true, ad);
  }

  /**
   * Gets a integer style attribute.
   * @param name The attribute name.
   * @return The according value.
   * @exception TagAttributeException If the item contains an invalid value 
   *    or an required attribute was not defined.
   */
  protected int getIntegerAttribute(String name) throws TagAttributeException {
    AttributeDefinition ad = tag.getAttributeDefinition(name);
    return tag.getIntegerAttribute(ad, true, null);
  }
  /**
   * Gets a integer style attribute.
   * @param name The attribute name.
   * @return The according value.
   * @exception TagAttributeException If the item contains an invalid value 
   *    or an required attribute was not defined.
   */
  protected int getIntegerStyleAttribute(String name) throws TagAttributeException {
    AttributeDefinition ad = tag.getStyleAttributeDefinition(name);
    return tag.getIntegerAttribute(null, true, ad);
  }

  /**
   * Gets a dimension attribute.
   * @param name The attribute name.
   * @return The according value.
   * @exception TagAttributeException If the item contains an invalid value 
   *    or an required attribute was not defined.
   */
  protected float getDimensionAttribute(String name) throws TagAttributeException {
    AttributeDefinition ad = tag.getAttributeDefinition(name);
    return tag.getDimensionAttribute(ad, true, null);
  }
  /**
   * Gets a dimension style attribute.
   * @param name The attribute name.
   * @return The according value.
   * @exception TagAttributeException If the item contains an invalid value 
   *    or an required attribute was not defined.
   */
  protected float getDimensionStyleAttribute(String name) throws TagAttributeException {
    AttributeDefinition ad = tag.getStyleAttributeDefinition(name);
    return tag.getDimensionAttribute(null, true, ad);
  }

  /**
   * Gets a boolean attribute.
   * @param name The attribute name.
   * @return The according value.
   * @exception TagAttributeException If the item contains an invalid value 
   *    or an required attribute was not defined.
   */
  protected boolean getBooleanAttribute(String name) throws TagAttributeException {
    AttributeDefinition ad = tag.getAttributeDefinition(name);
    return tag.getBooleanAttribute(ad, true, null);
  }
  /**
   * Gets a boolean style attribute.
   * @param name The attribute name.
   * @return The according value.
   * @exception TagAttributeException If the item contains an invalid value 
   *    or an required attribute was not defined.
   */
  protected boolean getBooleanStyleAttribute(String name) throws TagAttributeException {
    AttributeDefinition ad = tag.getStyleAttributeDefinition(name);
    return tag.getBooleanAttribute(null, true, ad);
  }

  /**
   * Gets a color attribute.
   * @param name The attribute name.
   * @return The according value.
   * @exception TagAttributeException If the item contains an invalid value 
   *    or an required attribute was not defined.
   */
  protected Color getColorAttribute(String name) throws TagAttributeException {
    AttributeDefinition ad = tag.getAttributeDefinition(name);
    return tag.getColorAttribute(ad, true, null);
  }
  /**
   * Gets a color style attribute.
   * @param name The attribute name.
   * @return The according value.
   * @exception TagAttributeException If the item contains an invalid value 
   *    or an required attribute was not defined.
   */
  protected Color getColorStyleAttribute(String name) throws TagAttributeException {
    AttributeDefinition ad = tag.getStyleAttributeDefinition(name);
    return tag.getColorAttribute(null, true, ad);
  }

  /**
   * Gets a horizontal alignment attribute.
   * @param name The attribute name.
   * @return The according value.
   * @exception TagAttributeException If the item contains an invalid value 
   *    or an required attribute was not defined.
   */
  protected int getHorizontalAlignmentAttribute(String name) throws TagAttributeException {
    AttributeDefinition ad = tag.getAttributeDefinition(name);
    return tag.getHorizontalAlignmentAttribute(ad, true, null);
  }
  /**
   * Gets a horizontal alignment style attribute.
   * @param name The attribute name.
   * @return The according value.
   * @exception TagAttributeException If the item contains an invalid value 
   *    or an required attribute was not defined.
   */
  protected int getHorizontalAlignmentStyleAttribute(String name) throws TagAttributeException {
    AttributeDefinition ad = tag.getStyleAttributeDefinition(name);
    return tag.getHorizontalAlignmentAttribute(null, true, ad);
  }

  /**
   * Gets a vertical alignment attribute.
   * @param name The attribute name.
   * @return The according value.
   * @exception TagAttributeException If the item contains an invalid value 
   *    or an required attribute was not defined.
   */
  protected int getVerticalAlignmentAttribute(String name) throws TagAttributeException {
    AttributeDefinition ad = tag.getAttributeDefinition(name);
    return tag.getVerticalAlignmentAttribute(ad, true, null);
  }
  /**
   * Gets a vertical alignment style attribute.
   * @param name The attribute name.
   * @return The according value.
   * @exception TagAttributeException If the item contains an invalid value 
   *    or an required attribute was not defined.
   */
  protected int getVerticalAlignmentStyleAttribute(String name) throws TagAttributeException {
    AttributeDefinition ad = tag.getStyleAttributeDefinition(name);
    return tag.getVerticalAlignmentAttribute(null, true, ad);
  }

  /**
   * Gets the tags textual content.
   * @return The tags textual content.
   */
  protected String getContent() {
    String content = tag.getActualContent();
    return content;
  }

}
