/*
 * Copyright (C) 2003 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

/**
 * Name: AttributeDefinition<br>
 * Description: A class for attribute definitions.<br>
 * 
 * @author lauerc
 */
public class AttributeDefinition {

  /** Constant for integer attribute type. */
  public static final int TYPE_INT = 1;
  /** Constant for float attribute type. */
  public static final int TYPE_FLOAT = 2;
  /** Constant for boolean attribute type. */
  public static final int TYPE_BOOLEAN = 3;
  /** Constant for string attribute type. */
  public static final int TYPE_STRING = 4;
  /** Constant for color attribute type. */
  public static final int TYPE_COLOR = 5;
  /** Constant for border attribute type. */
  public static final int TYPE_BORDER = 6;
  /** Constant for horizontal alignment attribute type. */
  public static final int TYPE_HORIZONTAL_ALIGNMENT = 7;
  /** Constant for vertical alignment attribute type. */
  public static final int TYPE_VERTICAL_ALIGNMENT = 8;
  /** Constant for page size attribute type. */
  public static final int TYPE_PAGE_SIZE = 9;
  /** Constant for font family attribute type. */
  public static final int TYPE_FONT_FAMILY = 10;
  /** Constant for font style attribute type. */
  public static final int TYPE_FONT_STYLE = 11;
  /** Constant for name attribute type. */
  public static final int TYPE_NAME = 12;
  /** Constant for enumeration attribute type. */
  public static final int TYPE_ENUM = 13;
  /** Constant for float attribute type. */
  public static final int TYPE_DIMENSION = 14;
  
  /** The attribute name. */
  private String name = null;
  /** The attribute type .*/
  private int type;
  /** The required flag .*/
  private boolean required;
  /** The attribute's description. */
  private String description = null;
  /** The deprecation message. */
  private String deprecation = null;
  /** The list of possible values for an enumeration. */
  private String[] possibleValues = null;
  /** The set of aliases for the attribute. */
  private Set aliases = new HashSet();

  /**
   * Constructs a AttributeDefinition instance with specific arguments.
   * @param name The attribute name.
   * @param type The attribute type
   * @param required The required flag.
   * @param description The attribute description.
   */
  public AttributeDefinition(String name, int type, boolean required, String description) {
    this.name = name;
    this.type = type;
    this.required = required;    
    this.description = description;
  }

  /**
   * Constructs a AttributeDefinition instance with specific arguments.
   * @param name The attribute name.
   * @param type The attribute type
   * @param required The required flag.
   * @param description The attribute description.
   * @param aliases The aliases to support.
   */
  public AttributeDefinition(String name, int type, boolean required, String description, String[] aliases) {
    this(name, type, required, description);
    addAliases(aliases);
  }
  
  /**
   * Constructs a AttributeDefinition instance with specific arguments.
   * @param name The attribute name.
   * @param type The attribute type
   * @param required The required flag.
   * @param description The attribute description.
   * @param deprecation The deprecation message.
   */
  public AttributeDefinition(String name, int type, boolean required, String description, String deprecation) {
    this(name, type, required, description);
    this.deprecation = deprecation;
  }

  /**
   * Constructs a AttributeDefinition instance with specific arguments.
   * @param name The attribute name.
   * @param possibleValues The enumeration values.
   * @param required The required flag.
   * @param description The attribute description.
   */
  public AttributeDefinition(String name, String[] possibleValues, boolean required, String description) {
    this(name, TYPE_ENUM, required, description);
    this.possibleValues = possibleValues;
  }

  /**
   * Constructs a AttributeDefinition instance with specific arguments.
   * @param name The attribute name.
   * @param possibleValues The enumeration values.
   * @param required The required flag.
   * @param description The attribute description.
   * @param deprecation The deprecation message.
   */
  public AttributeDefinition(String name, String[] possibleValues, boolean required, String description, 
                             String deprecation) {
    this(name, TYPE_ENUM, required, description);
    this.possibleValues = possibleValues;
    this.deprecation = deprecation;
  }

  /**
   * Gets the attribute's name.
   * @return The attribute name.
   */
  public String getName() {
    return name;
  }
  
  /**
   * Gets the attribute's type.
   * @return The attribute type.
   */
  public int getType() {
    return type;
  }

  /**
   * Gets the attribute's required flag.
   * @return true, if the attribute is required, else false.
   */
  public boolean isRequired() {
    return required;
  }
  /**
   * Sets the attribute's required flag.
   * @param required true, if the attribute is required, else false.
   * @return The instance itself.
   */
  public AttributeDefinition setRequired(boolean required) {
    this.required = required;
    return this;
  }

  /**
   * Gets the attribute's description.
   * @return The attribute's description.
   */
  public String getDescription() {
    return description;
  }

  /**
   * Setter method for the the property deprecation.
   * @param deprecation The value to set for the property deprecation.
   * @return The instance itself.
   */
  public AttributeDefinition setDeprecation(String deprecation) {
    this.deprecation = deprecation;
    return this;
  }
  /**
   * Getter method for the the property deprecation.
   * @return The current value of property deprecation.
   */
  public String getDeprecation() {
    return deprecation;
  }

  /**
   * Gets the list of possible values for the attribute.
   * @return The list of possible values.
   */
  public String[] getPossibleValues() {
    return possibleValues;
  }
  
  /**
   * Adds the given aliases to the alias set.
   * @param aliases The aliases to add.
   * @return A reference to this instance.
   */
  public AttributeDefinition addAliases(String[] aliases) {
    if ((aliases == null) || (aliases.length < 1)) {
      return this;
    }
    for (int i = 0; i < aliases.length; i++) {
      this.aliases.add(aliases[i]);
    }
    return this;
  }
  
  /**
   * Adds the given alias to the alias set.
   * @param alias The alias to add.
   * @return A reference to this instance.
   */
  public AttributeDefinition addAlias(String alias) {
    if ((alias == null) || (alias.length() < 1)) {
      return this;
    }
    this.aliases.add(alias);
    return this;
  }

  /**
   * Checks whether or not the given name is a registered alias.
   * @param alias The alias to check.
   * @return true if the given name is a registered alias, else false.
   */
  public boolean checkAlias(String alias) {
    if ((alias == null) || (alias.length() < 1)) {
      return false;
    }
    return this.aliases.contains(alias);
  }

  /**
   * Gets an iterator for the alias set.
   * @return An iterator for the alias set.
   */
  public Collection getAliases() {
    return this.aliases;
  }

  /**
   * Creates a clone from the current instance.
   * @return The cloned instance.
   */
  public AttributeDefinition cloneAttrDef() {
    AttributeDefinition clone = new AttributeDefinition(name, type, required, description);
    clone.deprecation = this.deprecation;
    clone.possibleValues = this.possibleValues;
    clone.aliases.addAll(this.aliases);
    return clone;
  }

  /**
   * Creates a clone from the current instance.
   * @param description The description to set for the clone.
   * @return The cloned instance.
   */
  public AttributeDefinition cloneAttrDef(String description) {
    AttributeDefinition clone = cloneAttrDef();
    clone.description = description;
    return clone;
  }

}
