/*
 * Copyright (C) 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

/**
 * Name: AttributeDefinitionMap<br>
 * Description: An optimized container for AttributeDefinition instances.
 * 
 * @author lauerc
 */
public class AttributeDefinitionMap extends HashMap {
  
  /** The serial version UID. */
  private static final long serialVersionUID = 6712410303870062267L;

  /**
   * Constructs a AttributeDefinitionMap instance with specific attributes.
   */
  public AttributeDefinitionMap() {
    super();
  }
  
  /**
   * Constructs a AttributeDefinitionMap instance with specific attributes.
   * @param attrDef The first attribute definition to be added to the map.
   * @see #addDefinition(AttributeDefinition) 
   */
  public AttributeDefinitionMap(AttributeDefinition attrDef) {
    super();
    addDefinition(attrDef);
  }

  /**
   * Adds the given AttributeDefinition instance to the map by using the instances 
   * property 'name' as key.
   * @param attrDef The attribute definition to be added to the map. 
   * @return The container itself.
   */
  public AttributeDefinitionMap addDefinition(AttributeDefinition attrDef) {
    //if (this.containsKey(attrDef.getName())) {
      //RuntimeException ex = new RuntimeException("Attribute '" + attrDef.getName() + "' redefined, that's illegal!");
      //System.err.println("Attribute '" + attrDef.getName() + "' redefined, that's illegal!");
    //}
    put(attrDef.getName(), attrDef);
    Iterator iterLocalAliases = attrDef.getAliases().iterator();
    String aliasName = null;
    while (iterLocalAliases.hasNext()) {
      aliasName = iterLocalAliases.next().toString();
      put(aliasName, attrDef);
    }
    return this;
  }

  /**
   * Removes the given AttributeDefinition instance from the map by using the instances 
   * property 'name' as key.
   * @param attrDef The attribute definition to be removed from to the map. 
   * @return The container itself.
   */
  public AttributeDefinitionMap removeDefinition(AttributeDefinition attrDef) {
    remove(attrDef.getName());
    Iterator iterLocalAliases = attrDef.getAliases().iterator();
    String aliasName = null;
    while (iterLocalAliases.hasNext()) {
      aliasName = iterLocalAliases.next().toString();
      remove(aliasName);
    }
    return this;
  }

  
  /**
   * Constructs a AttributeDefinitionMap instance with specific attributes.
   * @param attrDefMap The attribute definition map to copy.
   * @see #addDefinition(AttributeDefinition) 
   */
  public AttributeDefinitionMap(AttributeDefinitionMap attrDefMap) {
    super();
    this.putAll(attrDefMap);
  }
  
  /**
   * Gets a sorted list of attribute names including all attribute aliases.
   * @return The sorted attribute name list.
   */
  public List getSortedAttributeNames() {
    List attributeNames = new ArrayList(this.keySet());
    Collections.sort(attributeNames, new Comparator() {
      public int compare(Object arg0, Object arg1) {
        String a1 = (String) arg0;
        String a2 = (String) arg1;
        return a1.compareTo(a2);
      }
    });
    return attributeNames;
  }
  
  /**
   * Gets a list of all attributes, sorted by their primary names.
   * @return The sorted attribute list.
   */
  public List getSortedAttributes() {
    List attributes = new ArrayList(this.values());
    Collections.sort(attributes, new Comparator() {
      public int compare(Object arg0, Object arg1) {
        AttributeDefinition a1 = (AttributeDefinition) arg0;
        AttributeDefinition a2 = (AttributeDefinition) arg1;
        return a1.getName().compareTo(a2.getName());
      }
    });
    return attributes;
  }
  
}
