/*
 * Copyright (C) 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print;

import java.awt.Color;

import org.xml.sax.Locator;

/**
 * Name: AttributeParser<br>
 * Description: Parses argument values from given strings.
 * 
 * @author lauerc
 */
public interface AttributeHandler {

  /**
   * Sets the document handler.
   * @param documentHandler The document handler to set.
   */
  void setDocumentHandler(DocumentHandler documentHandler);
  
  /**
   * Casts the given object value into a the correct type for the specified attribute definition.
   * @param locator The source locator.
   * @param attrDef The attribute definition.
   * @param attrName The attribute name.
   * @param value The value to convert.
   * @return The converted value.
   * @throws DocumentHandlerException In case the type conversion failed.
   */
  Object typeCastAttributeValue(Locator locator, AttributeDefinition attrDef, String attrName, Object value)
    throws DocumentHandlerException;

  /**
   * Gets the value for the given attribute's name.
   * Works like method <code>getStringAttribute(java.lang.String, java.lang.String, boolean)</code> but takes care of escape sequences.
   * @param locator The tag source locator.
   * @param attrName The attribute's name.
   * @param attrValue The argument value.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @param translateEscapeSequences Tells whether or not to translate escape sequences.
   * @return The determined value.
   * @exception TagAttributeException if the item contains an invalid value or an required attribute was not defined.
   */
  String parseTextAttribute(Locator locator, String attrName, Object attrValue, 
                            boolean evaluate, boolean translateEscapeSequences)
    throws TagAttributeException;

  /**
   * Parses a float value from the given string.
   * @param locator The tag source locator.
   * @param attrName The attribute's name.
   * @param attrValue The argument value.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @return The determined value.
   * @exception TagAttributeException Thrown in case the item contained an invalid value.
   */
  float parseFloatAttribute(Locator locator, String attrName, Object attrValue, boolean evaluate)
    throws TagAttributeException;

  /**
   * Parses a dimension value from the given string.
   * @param locator The tag source locator.
   * @param attrName The attribute's name.
   * @param attrValue The argument value.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @return The determined value.
   * @exception TagAttributeException Thrown in case the item contained an invalid value.
   */
  float parseDimensionAttribute(Locator locator, String attrName, Object attrValue, boolean evaluate)
    throws TagAttributeException;

  /**
   * Parses a double value from the given string.
   * @param locator The tag source locator.
   * @param attrName The attribute's name.
   * @param attrValue The argument value.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @return The determined value.
   * @exception TagAttributeException Thrown in case the item contained an invalid value.
   */
  double parseDoubleAttribute(Locator locator, String attrName, Object attrValue, boolean evaluate)
    throws TagAttributeException;


  /**
   * Parses an integer value from the given string.
   * @param locator The tag source locator.
   * @param attrName The attribute's name.
   * @param attrValue The argument value.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @return The determined value.
   * @exception TagAttributeException Thrown in case the item contained an invalid value.
   */
  int parseIntAttribute(Locator locator, String attrName, Object attrValue, boolean evaluate)
    throws TagAttributeException;
  
  /**
   * Gets the boolean value for the given attribute's name.
   * @param locator The tag source locator.
   * @param attrName The attribute's name.
   * @param attrValue The argument value.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @return The determined value.
   * @exception TagAttributeException if the item contains an invalid value or an required attribute was not defined.
   */
  boolean parseBooleanAttribute(Locator locator, String attrName, Object attrValue, boolean evaluate)
    throws TagAttributeException;

  /**
   * Parses the color value from the given attribute.
   * @param locator The tag source locator.
   * @param attrName The name of the attribute to parse.
   * @param attrValue The attribute value to parse.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @return The resulting border style.
   * @exception TagAttributeException Thrown in case the item contained an invalid value.
   */
  Color parseColorAttribute(Locator locator, String attrName, Object attrValue, boolean evaluate)
    throws TagAttributeException;
  
  /**
   * Parses the border value from the given attribute.
   * @param locator The tag source locator.
   * @param attrName The name of the attribute to parse.
   * @param attrValue The attribute value to parse.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @return The resulting border style.
   * @exception TagAttributeException Thrown in case the item contained an invalid value.
   */
  int parseBorderAttribute(Locator locator, String attrName, Object attrValue, boolean evaluate)
    throws TagAttributeException;

  /**
   * Parses the horizontal alignment value from the given attribute.
   * @param locator The tag source locator.
   * @param attrName The name of the attribute to parse.
   * @param attrValue The attribute value to parse.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @return The resulting horizontal alignment.
   * @exception TagAttributeException Thrown in case the item contained an invalid value.
   */
  int parseHorizontalAlignmentAttribute(Locator locator, String attrName, Object attrValue, boolean evaluate)
    throws TagAttributeException;

  /**
   * Parses the vertical alignment value from the given attribute.
   * @param locator The tag source locator.
   * @param attrName The name of the attribute to parse.
   * @param attrValue The attribute value to parse.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @return The resulting vertical alignment.
   * @exception TagAttributeException Thrown in case the item contained an invalid value.
   */
  int parseVerticalAlignmentAttribute(Locator locator, String attrName, Object attrValue, boolean evaluate)
    throws TagAttributeException;
  
  /**
   * Gets the integer attribute handler.
   * @return The integer attribute handler
   */
  TypedAttributeHandler getTextHandler();
  /**
   * Gets the integer attribute handler.
   * @return The integer attribute handler
   */
  TypedAttributeHandler getEscapeSequenceTextHandler();
  /**
   * Gets the integer attribute handler.
   * @return The integer attribute handler
   */
  TypedAttributeHandler getIntegerHandler();
  /**
   * Gets the float attribute handler.
   * @return The integer attribute handler
   */
  TypedAttributeHandler getFloatHandler();
  /**
   * Gets the dimension attribute handler.
   * @return The integer attribute handler
   */
  TypedAttributeHandler getDimensionHandler();
  /**
   * Gets the double attribute handler.
   * @return The double attribute handler
   */
  TypedAttributeHandler getDoubleHandler();
  /**
   * Gets the boolean attribute handler.
   * @return The boolean attribute handler
   */
  TypedAttributeHandler getBooleanHandler();
  /**
   * Gets the color attribute handler.
   * @return The color attribute handler
   */
  TypedAttributeHandler getColorHandler();
  /**
   * Gets the boolean attribute handler.
   * @return The boolean attribute handler
   */
  TypedAttributeHandler getBorderHandler();
  /**
   * Gets the horizontal alignment attribute handler.
   * @return The horizontal alignment attribute handler
   */
  TypedAttributeHandler getHorizontalAlignmentHandler();
  /**
   * Gets the vertical alignment attribute handler.
   * @return The vertical alignment attribute handler
   */
  TypedAttributeHandler getVerticalAlignmentHandler();

}
