/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print;

import java.awt.Color;

import java.io.IOException;
import java.io.StringReader;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.List;
import java.util.HashMap;

import org.apache.commons.logging.Log;

import org.ujac.print.tag.CommonAttributes;
import org.ujac.print.tag.CommonStyleAttributes;
import org.ujac.print.tag.TagAttributes;

import org.ujac.util.BeanUtils;
import org.ujac.util.CollectionUtils;
import org.ujac.util.StringUtils;
import org.ujac.util.UjacTypes;

import org.ujac.util.exi.ExpressionException;

import org.ujac.util.template.TemplateException;

import org.ujac.util.xml.XmlUtils;

import org.xml.sax.Locator;
import org.xml.sax.XMLReader;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import org.xml.sax.helpers.LocatorImpl;

import com.lowagie.text.BadElementException;
import com.lowagie.text.Cell;
import com.lowagie.text.DocumentException;
import com.lowagie.text.Font;
import com.lowagie.text.Chunk;
import com.lowagie.text.FontFactory;
import com.lowagie.text.Image;
import com.lowagie.text.Phrase;
import com.lowagie.text.Element;
import com.lowagie.text.Rectangle;

import com.lowagie.text.pdf.BaseFont;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfPCell;
import com.lowagie.text.pdf.PdfTemplate;
import com.lowagie.text.pdf.PdfWriter;

/**
 * Name: BaseDocumentTag<br>
 * Description: Base implementation for tags, processed by the DocumentHandler class.<br>
 * 
 * @author lauerc
 */
public abstract class BaseDocumentTag {
  
  /** The begin of a CDATA section. */
  private static final String CDATA_BEGIN = "<![CDATA["; 
  /** The end of a CDATA section. */
  private static final String CDATA_END = "]]>"; 

  /** The IMAGE chunk attribute. */
  public static final String IMAGE_CHUNK_ATTR = "IMAGE";
  
  /** The resource name of the number of pages template. */
  public static final String NUMBER_OF_PAGES_TEMPLATES = "UJAC_NUMBER_OF_PAGES_TEMPLATES";

  /** The default font family. */
  public static final String DEFAULT_FONT_FAMILY = "Helvetica";
  /** The default font encoding. */
  public static final String DEFAULT_FONT_ENCODING = BaseFont.WINANSI;
  /** The default font size. */
  public static final float DEFAULT_FONT_SIZE = 10f;
  /** The default font color. */
  public static final Color DEFAULT_FONT_COLOR = Color.black;
  /** The default font style. */
  public static final int DEFAULT_FONT_STYLE = Font.NORMAL;
  /** The default font. */
  public static DocumentFont DEFAULT_FONT = new DocumentFont(
    FontFactory.getFont(DEFAULT_FONT_FAMILY, DEFAULT_FONT_ENCODING, DEFAULT_FONT_SIZE, 
                        DEFAULT_FONT_STYLE, DEFAULT_FONT_COLOR),
    DEFAULT_FONT_FAMILY, DEFAULT_FONT_ENCODING, DEFAULT_FONT_SIZE, 
    DEFAULT_FONT_STYLE, DEFAULT_FONT_COLOR
  );

  /** The default fixed vertical leading distance. */
  public static final float DEFAULT_LEADING = 0.0F;
  /** The factor by which to multiply the font size to get the default leading. */
  public static final float DEFAULT_LINE_SPACING = 1.2F;

  /** A map holding the most common colors. */
  private static final Map colorMap = new HashMap();
  
  static {
    // filling the color map
    colorMap.put("null", null); 
    colorMap.put("NULL", null); 
    colorMap.put("none", null); 
    colorMap.put("NONE", null);
    colorMap.put("black", Color.black); 
    colorMap.put("BLACK", Color.black); 
    colorMap.put("blue", Color.blue); 
    colorMap.put("BLUE", Color.blue); 
    colorMap.put("cyan", Color.cyan); 
    colorMap.put("CYAN", Color.cyan); 
    colorMap.put("darkGray", Color.darkGray); 
    colorMap.put("DARK_GRAY", Color.darkGray);  
    colorMap.put("gray", Color.gray); 
    colorMap.put("GRAY", Color.gray); 
    colorMap.put("green", Color.green); 
    colorMap.put("GREEN", Color.green); 
    colorMap.put("lightGray", Color.lightGray); 
    colorMap.put("LIGHT_GRAY", Color.lightGray);  
    colorMap.put("magenta", Color.magenta); 
    colorMap.put("MAGENTA", Color.magenta); 
    colorMap.put("orange", Color.orange); 
    colorMap.put("ORANGE", Color.orange); 
    colorMap.put("pink", Color.pink); 
    colorMap.put("PINK", Color.pink); 
    colorMap.put("red", Color.red); 
    colorMap.put("RED", Color.red); 
    colorMap.put("white", Color.white); 
    colorMap.put("WHITE", Color.white); 
    colorMap.put("yellow", Color.yellow); 
    colorMap.put("YELLOW", Color.yellow); 
  }

  /** The logger. */
  protected Log log = null;
  /** The document handler. */
  protected DocumentHandler documentHandler = null;
  /** The tag factory. */
  protected DocumentTagFactory tagFactory = null;
  
  /** The item's depth in the document tree. */
  protected int depth = 0;
  /** The supported attribute map. */
  protected AttributeDefinitionMap supportedAttributeMap = null;
  /** The supported attribute map. */
  protected AttributeDefinitionMap supportedStyleAttributeMap = null;
  /** The supported child map. */
  protected ChildDefinitionMap supportedChildMap = null;

  /** The tag name. */
  private String name = null;
  /** The item's attributes. */
  protected Attributes attributes = null;
  /** The item's custom attributes. */
  protected Map customAttributes = null;

  /** The item's actual content, used for debugging issues. */
  protected String actualContent = null;
  /** The item's content. */
  protected StringBuffer content = null;
  
  /** The parent item. */
  protected BaseDocumentTag parentItem = null;
  /** The previous item. */
  protected BaseDocumentTag previousItem = null;
  /** The condition. */
  protected Condition condition = null;
  /** The document locator for better error handling. */
  private Locator locator = null;
  /** The font holder. */
  private FontHolder fontHolder = null;
  /** The ignore flush flag. */
  private boolean ignoreFlush = false;
  /** The dynamic content XML reader. */
  private XMLReader dynamicContentXmlReader = null;
  /** Flag indicating whether the current item has just been opened or not. */
  private boolean itemOpened = false;
  /** Flag indicating whether the current item has already been closed or not. */
  private boolean itemClosed = false;
  /** Flag telling whether the body shall be trimmed or not. */
  private boolean trimBody = true;
  /** Flag telling whether or not to trimm of the head of the next content part. */
  private boolean trimNextHead = false;
  /** 
   * Flag telling whether to evaluate XML tags, which are embedded in the tags content or not.
   * Default is true, in case &lt; or &gt; characters are explicitely added to the content, 
   * this flag is switched to false.
   */
  private boolean evaluateXml = true;
  /** The locally defined font. */
  private DocumentFont localFont = null;
  /** The background color. */
  private Color backgroundColor = null;
  /** The style class. */
  private String styleClass = null;
  /** The inline style attributes. */
  private String style = null;
  /** The rendered atttibute. */
  private String renderedAttr = null;
  /** The rendered evaluation result. */
  private boolean rendered = true;

  /** The parent style holder. */
  protected StyleHolder styleParent = null;
  /** The style attribute map. */
  private Map styleAttributes = null;

  /** The style parser. */
  private StyleParser styleParser = null;
  /** The attributeHandler. */
  private AttributeHandler attributeHandler = null;

  /** The common attributes holder. */
  private CommonAttributesHolder commonAttributesHolder = null;


  /**
   * Constructs a BaseDocumentTag instance with no specific attributes.
   */
  public BaseDocumentTag() {
  }
  /**
   * Constructs a BaseDocumentTag instance with specific attributes.
   * @param name The tag name.
   */
  public BaseDocumentTag(String name) {
    this.name = name;
  }
  
  /**
   * Gets the item's name.
   * @return The item's name.
   */
  public String getName() {
    return name;
  }
  /**
   * Sets the item's name.
   * @param name The name to set.
   */
  protected void setName(String name) {
    this.name = name;
  }

  /**
   * Gets a description for the item.
   * @return The item's description.
   */
  public abstract String getDescription();

  /**
   * Gets examples for the item.
   * @return one ore more examples for the item's usage..
   */
  public String getExamples() {
    return null;
  }

  /**
   * Gets the map of supported attributes.
   * @return The attribute definitions.
   */
  public AttributeDefinitionMap getSupportedAttributes() {
    if (supportedAttributeMap != null) {
      return supportedAttributeMap;
    }
    AttributeDefinitionMap attrDefs = documentHandler.getSupportedAttributes(this);
    supportedAttributeMap = attrDefs;
    return attrDefs;
  }

  /**
   * Gets the map of supported style attributes.
   * @return The attribute definitions.
   */
  public AttributeDefinitionMap getSupportedStyleAttributes() {
    if (supportedStyleAttributeMap != null) {
      return supportedStyleAttributeMap;
    }
    AttributeDefinitionMap attrDefs = documentHandler.getSupportedStyleAttributes(this);
    supportedStyleAttributeMap = attrDefs;
    return attrDefs;
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap getSupportedChilds() {
    if (supportedChildMap != null) {
      return supportedChildMap;
    }
    ChildDefinitionMap childDefs = documentHandler.getSupportedChilds(this);
    supportedChildMap = childDefs;
    return childDefs;
  }

  /**
   * Builds the map of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return new AttributeDefinitionMap()
      .addDefinition(CommonAttributes.ID);
  }

  /**
   * Builds the map of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return new AttributeDefinitionMap();
  }

  /**
   * Builds the map of supported child definitions.
   * @return The child definitions.
   */
  protected ChildDefinitionMap buildSupportedChilds() {
    return new ChildDefinitionMap();
  }

  /**
   * Gets the sorted list of supported attributes.
   * @return The attribute definitions.
   */
  public List getSortedAttributeDefinitions() {
    AttributeDefinitionMap attrMap = getSupportedAttributes();
    if (attrMap == null) {
      return null;
    }
    return attrMap.getSortedAttributes();
  }

  /**
   * Gets the sorted list of supported style attributes.
   * @return The attribute definitions.
   */
  public List getSortedStyleAttributeDefinitions() {
    AttributeDefinitionMap attrMap = getSupportedStyleAttributes();
    if (attrMap == null) {
      return null;
    }
    return attrMap.getSortedAttributes();
  }

  /**
   * Is the item virtual container, such as &lt;if&gt;, &lt;else&gt; or &lt;foreach&gt;.
   * @return true if the item is such a container, else false.
   */
  public boolean isVirtualContainer() {
    return false;
  }

  /**
   * Is the item a possible document root tag, such as &lt;document&gt; or &lt;module&gt;.
   * @return true if the item is a possible root tag, else false.
   */
  public boolean isDocumentRoot() {
    return false;
  }

  /**
   * Tells whether textual content is allowed for the tag or not.
   * @return true if the item is allowed to contain textual content, else false.
   */
  public boolean isTextBodyAllowed() {
    return false;
  }

  /**
   * Tells whether or not to disable checks which test if this tag fits into its parent tag.
   * This method should be overwritten by custom classes which extent the default tag 
   * base of UJAC.
   * @return true if structure checks are disabled for this tag.
   */
  public boolean isStructureChecksDisabled() {
    return false;
  }

  /**
   * Finds the supported child tags from the DocumentTagFactory using the informations
   * from the methods getSupportedChilds and getExcludedChilds.
   * @return The child definitions.
   * @exception TagInstantiationException In case the method <code>DocumentTagFactory.getAllPossibleChilds</code> failed.
   */
  public List getSupportedChildTags() throws TagInstantiationException {
    return tagFactory.getSupportedChildTags(this);
  }
  

  /**
   * Gets the document handler.
   * @return The document handler.
   */
  public DocumentHandler getDocumentHandler() {
    return documentHandler;
  }
  /**
   * Sets the document handler.
   * @param documentHandler The document handler to set.
   */
  protected void setDocumentHandler(DocumentHandler documentHandler) {
    this.documentHandler = documentHandler;
    this.attributeHandler = documentHandler.getAttributeHandler();
    this.styleParser = documentHandler.getStyleParser();
    this.log = documentHandler.getLog();
  }

  /**
   * Gets the document tag factory.
   * @return The document tag factory.
   */
  public DocumentTagFactory getTagFactory() {
    return tagFactory;
  }
  /**
   * Sets the document tag factory.
   * @param tagFactory The document tag factory to set.
   */
  protected void setTagFactory(DocumentTagFactory tagFactory) {
    this.tagFactory = tagFactory;
  }

  /**
   * Gets the item's depth in the document tree.
   * @return The item's depth.
   */
  public int getDepth() {
    return depth;
  }
  /**
   * Sets the item's depht in the document tree.
   * @param depth The depth to set.
   */
  protected void setDepth(int depth) {
    this.depth = depth;
  }

  /**
   * Gets the item's attributes.
   * @return The item's attributes.
   */
  public Attributes getAttributes() {
    return attributes;
  }
  /**
   * Sets the item's attributes.
   * @param attributes The attributes to set.
   */
  protected void setAttributes(Attributes attributes) {
    this.attributes = attributes;
  }

  /**
   * Gets the item's custom attributes.
   * @return The item's custom attributes.
   */
  public Map getCustomAttributes() {
    return customAttributes;
  }
  /**
   * Sets the item's custom attributes.
   * @param customAttributes The custom attributes to set.
   */
  public void setCustomAttributes(Map customAttributes) {
    this.customAttributes = customAttributes;
  }

  /**
   * Gets the parent item.
   * @return The parent item.
   */
  public BaseDocumentTag getParentItem() {
    return parentItem;
  }
  /**
   * Sets the parent item.
   * @param item The parent item to set.
   */
  public void setParentItem(BaseDocumentTag item) {
    this.parentItem = item;
    if (item != null)  { 
      // the parent item contains at least one item, so it can't be just have been opened!
      item.setItemOpened(false);
    }
  }
  
  /**
   * Gets the previous item.
   * @return The previous item.
   */
  public BaseDocumentTag getPreviousItem() {
    return previousItem;
  }
  /**
   * Sets the previous item.
   * @param item The previous item to set.
   */
  protected void setPreviousItem(BaseDocumentTag item) {
    this.previousItem = item;
  }
  
  /**
   * Sets the item opened flag.
   * @param itemOpened The item opened flag to set.
   */
  public void setItemOpened(boolean itemOpened) {
    this.itemOpened = itemOpened;
  }
  /**
   * Gets the item opened flag.
   * @return The item opened flag.
   */
  public boolean isItemOpened() {
    return itemOpened;
  }

  /**
   * Getter method for the the property trimBody.
   * @return The current value of property trimBody.
   */
  public boolean isTrimBody() {
    return trimBody;
  }
  /**
   * Setter method for the the property trimBody.
   * @param trimBody The value to set for the property trimBody.
   */
  public void setTrimBody(boolean trimBody) {
    this.trimBody = trimBody;
  }
  
  /**
   * Sets the 'trim next head' flag in dependency of the 'trim body' flag.
   * @param trimNextHead The 'trim next head' flag to set.
   */
  public void setTrimNextHead(boolean trimNextHead) {
    this.trimNextHead = trimNextHead && trimBody;
  }
  /**
   * Gets the 'trim next head' flag.
   * @return The 'trim next head' flag.
   */
  public boolean isTrimNextHead() {
    return trimNextHead;
  }

  /**
   * Sets the item closed flag.
   * @param itemClosed The item closed flag to set.
   */
  public void setItemClosed(boolean itemClosed) {
    this.itemClosed = itemClosed;
  }
  /**
   * Gets the item closed flag.
   * @return The item closed flag.
   */
  public boolean isItemClosed() {
    return itemClosed;
  }

  /** 
   * Gets the document locator, which describes the position of this item.
   * @return This item's locator.
   */
  public Locator locator() {
    return locator;
  }

  /**
   * Pre-initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void preInitialize() throws DocumentHandlerException {
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    // building map of supported attributes
    supportedAttributeMap = getSupportedAttributes();
    if (supportedAttributeMap == null) {
      supportedAttributeMap = new AttributeDefinitionMap();
    }
    // building map of supported style attributes
    supportedStyleAttributeMap = getSupportedStyleAttributes();
    if (supportedStyleAttributeMap == null) {
      supportedStyleAttributeMap = new AttributeDefinitionMap();
    } else {
      if (supportedStyleAttributeMap.containsKey(CommonStyleAttributes.FONT_NAME.getName())) {
        try {
          supportedStyleAttributeMap.addDefinition(CommonStyleAttributes.FONT_COLOR);
        } catch (RuntimeException ex) {
        }
        try {
          supportedStyleAttributeMap.addDefinition(CommonStyleAttributes.FONT_ENCODING);
        } catch (RuntimeException ex) {
        }
        try {
          supportedStyleAttributeMap.addDefinition(CommonStyleAttributes.FONT_FAMILY);
        } catch (RuntimeException ex) {
        }
        try {
          supportedStyleAttributeMap.addDefinition(CommonStyleAttributes.FONT_NAME);
        } catch (RuntimeException ex) {
        }
        try {
          supportedStyleAttributeMap.addDefinition(CommonStyleAttributes.FONT_SIZE);
        } catch (RuntimeException ex) {
        }
        try {
          supportedStyleAttributeMap.addDefinition(CommonStyleAttributes.FONT_STYLE);
        } catch (RuntimeException ex) {
        }
      }
    }
    // getting the rendered attribute
    this.renderedAttr = getStringAttribute(CommonAttributes.RENDERED, false, null);

    // setting the condition.
    this.condition = documentHandler.latestCondition();
    // determinig the surrounding style holder.
    this.styleParent = (StyleHolder) documentHandler.latestOfType(StyleHolder.class);
    // determining the surrounding common attributes holder
    this.commonAttributesHolder = documentHandler.latestCommonAttributesHolder();

    if (this.locator == null) {
      // setting the document locator
      this.locator = new LocatorImpl(documentHandler.locator());
    }
    
    // setting the font holder
    this.fontHolder = documentHandler.latestFontHolder();
    // checking for unsupported attributes
    checkAttributes();
  }
  
  /**
   * Checks for unsupported attributes. 
   * @exception TagAttributeException If an unsupported attribute was detected.
   */
  protected void checkAttributes() throws TagAttributeException {
    // checking for unsupported attributes
    int numAttributes = attributes.getLength();
    for (int i = 0; i < numAttributes; i++) {
      String attrName = attributes.getLocalName(i);
      AttributeDefinition attrDef = (AttributeDefinition) supportedAttributeMap.get(attrName);
      if (attrDef == null) {
        StringBuffer messageBuf = new StringBuffer("Unsupported attribute '")
          .append(attrName).append("' for item ").append(getName());
        if (supportedAttributeMap.size() == 0) {
          messageBuf.append(". This tag doesn't support any attribute.");
        } else {
          messageBuf.append(". Supported attributes are: ");
          List attributeNames = supportedAttributeMap.getSortedAttributeNames();
          int numAttributeNames = attributeNames.size();
          for (int j = 0; j < numAttributeNames; j++) {
            if (j > 0) {
              messageBuf.append(", ");
            }
            messageBuf.append(attributeNames.get(j));
          }
          messageBuf.append(".");
        }
        throw new TagAttributeException(locator(), messageBuf.toString());
      }
      
      String deprecation = attrDef.getDeprecation();
      if (deprecation != null) {
        log.warn("At line " + locator().getLineNumber() + ": Detected usage of deprecated attribute '" + 
                 attrName + "' for tag <" + getName() + ">: " + deprecation);
      }
    }
  }

  /**
   * Clears the current content.
   */
  public void resetContent() {
    if (content != null) {
      actualContent = content.toString();
      this.content.setLength(0);
    }
  }

  /**
   * Adds the specified subset of the given character array to the content buffer.
   * @param content The given content.
   * @param offset The offset from which to add the content.
   * @param length The number of characters to copy.
   */
  public void addContent(char[] content, int offset, int length) {
    
    if (this.content == null) {
      this.content = new StringBuffer();
    }    
    if (length == 1) {
      char first = content[offset];
      if ((first == '<') ||
          (first == '>')) {
        evaluateXml = false;    
      }
    }
    this.content.append(content, offset, length);
  }
  
  /**
   * Adds the given String to the content buffer.
   * @param content The content to add.
   */
  public void addContent(String content) {
    
    if (this.content == null) {
      this.content = new StringBuffer();
    }
    if (content.length() == 1) {
      char first = content.charAt(0);
      if ((first == '<') ||
          (first == '>')) {
        evaluateXml = false;    
      }
    }
    this.content.append(content);
  }
  
  /**
   * Gets the current content.
   * @return The current content as StringBuffer.
   */
  public boolean hasContent() {
    if (content == null) {
      return false;
    }
    return content.length() > 0;
  }

  /**
   * Gets the current content.
   * @return The current content as StringBuffer.
   */
  public String getContent() {
    if (content == null) {
      return null;
    }
    return content.toString();
  }
  /**
   * Gets the actual content.
   * This method is intended for testing purposes. 
   * @return The last content.
   */
  public String getActualContent() {
    return actualContent;
  }
  
  /**
   * Gets the trimmed content.
   * @return The current content as trimmed String.
   */
  public String getTrimmedContent() {
    if (content == null) {
      return "";
    }
    return content.toString().trim();
  }

  /**
   * Flushes the content buffer.
   * @exception DocumentHandlerException In case a problem occured while processing the content.
   */
  public void flushContent() throws DocumentHandlerException {
  }

  /**
   * Trims off the heading white spaces of the content buffer.
   */
  protected void trimContentHead() {
    if (content == null) {
      // no sense to go on!
      return;
    }
    int length = content.length();
    for (int i = 0; i < length; i++) {
      if (!Character.isWhitespace(content.charAt(i))) {
        if (i > 0) {
          content.delete(0, i);
        }
        break;
      }
    }
  }
  
  /**
   * Trims off the trailing white spaces of the content buffer.
   */
  protected void trimContentTail() {
    if (content == null) {
      // no sense to go on!
      return;
    }
    int length = content.length();
    while ((length > 0) && (Character.isWhitespace(content.charAt(length - 1)))) {
      --length;
    }
    if ((length > 0) && documentHandler.isTranslateEscapeSequences()) {
      // taking care of escape sequences here
      int escapeCount = 0;
      for (int i = length - 1; i >= 0; i--) {
        if (content.charAt(i) == '\\') {
          ++escapeCount;
        } else {
          break;
        }
      }
      if ((escapeCount % 2) != 0) {
        ++length;
      }
    }
    content.setLength(length);
  }

  /**
   * Gets the ignore flush flag.
   * @return The ignore flush flag.
   */
  protected boolean isIgnoreFlush() {
    return ignoreFlush;
  }

  /**
   * Checks, if the given attribute is supported.
   * @param name The attribute name.
   * @return true if the attribute is supported, else false.
   */
  public boolean isAttributeSupported(String name) {
    AttributeDefinition attrDef = (AttributeDefinition) supportedAttributeMap.get(name);
    if (attrDef == null) {
      return false;
    }
    return true;
  }
  /**
   * Checks, if the given attribute is supported.
   * @param attrDef The attribute definition.
   * @return true if the attribute is supported, else false.
   */
  public boolean isAttributeSupported(AttributeDefinition attrDef) {
    AttributeDefinition ad = (AttributeDefinition) supportedAttributeMap.get(attrDef.getName());
    if (ad == null) {
      return false;
    }
    return true;
  }

  /**
   * Checks, if the given attribute or style attribute is supported.
   * @param name The attribute name.
   * @param styleName The style attribute name.
   * @return true if the attribute is supported, else false.
   */
  public boolean isAttributeSupported(String name, String styleName) {
    AttributeDefinition attrDef = null;
    if (name != null) {
      attrDef = (AttributeDefinition) supportedAttributeMap.get(name);
    }
    if (attrDef != null) {
      return true;
    }
    if (styleName != null) {
      attrDef = (AttributeDefinition) supportedStyleAttributeMap.get(styleName);
    }
    return (attrDef != null);
  }
  /**
   * Checks, if the given attribute or style attribute is supported.
   * @param attrDef The attribute definition.
   * @param styleAttrDef The style attribute definition.
   * @return true if the attribute is supported, else false.
   */
  public boolean isAttributeSupported(AttributeDefinition attrDef, AttributeDefinition styleAttrDef) {
    AttributeDefinition ad = null;
    if (attrDef != null) {
      ad = (AttributeDefinition) supportedAttributeMap.get(attrDef.getName());
    }
    if (ad != null) {
      return true;
    }
    
    if (styleAttrDef != null) {
      ad = (AttributeDefinition) supportedStyleAttributeMap.get(styleAttrDef.getName());
    }
    return (ad != null);
  }

  /**
   * Gets the attribute definition for the given name
   * @param name The attribute name.
   * @return The attribute definition.
   */
  public AttributeDefinition getAttributeDefinition(String name) {
    if (name == null) {
      return null;
    }
    AttributeDefinition attrDef = (AttributeDefinition) supportedAttributeMap.get(name);
    return attrDef;
  }
  /**
   * Adds the given attribute definition to the set of supported attributes.
   * @param definition The attribute definition to register with the tag.
   */
  protected void addAttributeDefinition(AttributeDefinition definition) {
    supportedAttributeMap.put(definition.getName(), definition);
  }
  /**
   * Checks whether the given attribute is defined or not.
   * @param attrDef The attribute definition.
   * @return true, if the attribute is defined or not.
   */
  public boolean isAttributeDefined(AttributeDefinition attrDef) {
    String name = attrDef.getName();
    String attrValue = attributes.getValue(name);
    if (attrValue != null) {
      return true;
    }
    if (customAttributes != null) {
      if (customAttributes.containsKey(name)) {
        return true;
      }
    }
    return false;
  }
  /**
   * Checks whether the given attribute is defined or not.
   * @param name The attribute's name.
   * @return true, if the attribute is defined or not.
   */
  public boolean isAttributeDefined(String name) {
    String attrValue = attributes.getValue(name);
    if (attrValue != null) {
      return true;
    }
    if (customAttributes != null) {
      if (customAttributes.containsKey(name)) {
        return true;
      }
    }
    return false;
  }
  /**
   * Checks whether the given attribute is defined or not.
   * @param name The attribute's name.
   * @param styleName The according style name.
   * @return true, if the attribute is defined or not.
   */
  public boolean isAttributeDefined(String name, String styleName) {
    String attrValue = attributes.getValue(name);
    if (attrValue != null) {
      return true;
    }
    if (customAttributes != null) {
      if (customAttributes.containsKey(name)) {
        return true;
      }
    }
    return isStyleAttributeDefined(styleName);
  }
  /**
   * Checks whether the given attribute is defined or not.
   * @param attrDef The attribute definition.
   * @param styleAttrDef The style attribute definition.
   * @return true, if the attribute is defined or not.
   */
  public boolean isAttributeDefined(AttributeDefinition attrDef, AttributeDefinition styleAttrDef) {
    String styleAttrName = null;
    if (styleAttrDef == null) {
      return isAttributeDefined(attrDef);
    }
    styleAttrName = styleAttrDef.getName();
    String attrName = null;
    if (attrDef != null) {
      attrName = attrDef.getName();
    }
    return isAttributeDefined(attrName, styleAttrName);
  }
  
  /**
   * Gets the attribute definition for the given name
   * @param name The attribute name.
   * @return The attribute definition.
   */
  public AttributeDefinition getStyleAttributeDefinition(String name) {
    if (name == null) {
      return null;
    }
    AttributeDefinition attrDef = (AttributeDefinition) supportedStyleAttributeMap.get(name);
    return attrDef;
  }
  /**
   * Adds the given attribute definition to the set of supported attributes.
   * @param definition The attribute definition to register with the tag.
   */
  protected void addStyleAttributeDefinition(AttributeDefinition definition) {
    supportedStyleAttributeMap.put(definition.getName(), definition);
  }
  /**
   * Checks whether the given attribute is defined or not.
   * @param name The attribute's name.
   * @return true, if the attribute is defined or not.
   */
  public boolean isStyleAttributeDefined(String name) {
    Object attrValue = styleAttributes.get(name);
    if (attrValue != null) {
      return true;
    }
    return false;
  }

  /**
   * Gets the value for the given attribute's name.
   * @param name The attribute's name.
   * @return The determined value.
   * @exception TagAttributeException if the item contains an invalid value or an required attribute was not defined.
   */
  public Object evalAttribute(String name) throws TagAttributeException {
    AttributeDefinition attrDef = getAttributeDefinition(name);
    if (attrDef == null) {
      throw new TagAttributeException(locator(), "The attribute '" + name + "' is not supported for element '" + getName() + "'.");
    }
    String attrValue = attributes.getValue(name);
    try {
      return documentHandler.evalObject(attrValue);
    } catch (ExpressionException ex) {
      throw new TagAttributeException(locator(), "Failed to evaluate attribute '" + name + "' at the element '" + getName() + "'.", ex);
    }
  }

  /**
   * Gets the raw value for the given attribute's name.
   * @param name The attribute's name.
   * @return The determined value.
   * @exception TagAttributeException if the item contains an invalid value or an required attribute was not defined.
   */
  public String getRawAttribute(String name) throws TagAttributeException {
    AttributeDefinition attrDef = getAttributeDefinition(name);
    if (attrDef == null) {
      throw new TagAttributeException(locator(), "The attribute '" + name + "' is not supported for element '" + getName() + "'.");
    }
    String attrValue = attributes.getValue(name);
    if (attrValue == null) {
      attrValue = (String) customAttributes.get(name);
    }
    return attrValue;
  }
  /**
   * Gets the value for the given attribute definition.
   * @param attrDef The attribute definition.
   * @return The determined value.
   * @exception TagAttributeException if the item contains an invalid value or an required attribute was not defined.
   */
  public Object evalAttribute(AttributeDefinition attrDef) throws TagAttributeException {
    String attrName = attrDef.getName();
    AttributeDefinition ad = getAttributeDefinition(attrName);
    if (ad == null) {
      throw new TagAttributeException(locator(), "The attribute '" + attrName + "' is not supported for element '" + getName() + "'.");
    }
    String attrValue = attributes.getValue(attrName);
    try {
      return documentHandler.evalObject(attrValue);
    } catch (ExpressionException ex) {
      throw new TagAttributeException(locator(), "Failed to evaluate attribute '" + attrName + "' at the element '" + getName() + "'.", ex);
    }
  }

  /**
   * Gets the value for the given attribute's name.
   * @param attrDef The attribute definition.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @param styleAttrDef The style attribute to consider.
   * @param attrHandler The attribute handler to use.
   * @return The determined value.
   * @exception TagAttributeException if the item contains an invalid value or an required attribute was not defined.
   */
  public Object getAttribute(AttributeDefinition attrDef, boolean evaluate, AttributeDefinition styleAttrDef, TypedAttributeHandler attrHandler) throws TagAttributeException {
    return getAttribute(attrDef, null, evaluate, styleAttrDef, attrHandler);
  }

  /**
   * Gets the value for the given attribute's name.
   * @param attrDef The attribute definition.
   * @param defaultValue The default value in case the attribute was not defined.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @param styleAttrDef The style attribute to consider.
   * @param attrHandler The attribute handler to use.
   * @return The determined value.
   * @exception TagAttributeException if the item contains an invalid value or an required attribute was not defined.
   */
  public Object getAttribute(AttributeDefinition attrDef, Object defaultValue, boolean evaluate, AttributeDefinition styleAttrDef, TypedAttributeHandler attrHandler)
    throws TagAttributeException {

    if (attrDef != null) {
      String name = attrDef.getName();
      //AttributeDefinition attrDef = getAttributeDefinition(name);
      //if (attrDef == null) {
      //  throw new TagAttributeException(locator(), "The attribute '" + name + "' is not supported for element '" + getName() + "'.");
      //}
      String attrValue = attributes.getValue(name);
      if ((attrValue == null) && (customAttributes != null)) {
        attrValue = (String) customAttributes.get(name);        
      }
      if (attrValue == null) {
        Iterator aliases = attrDef.getAliases().iterator();
        while (aliases.hasNext() && attrValue == null) {
          String aliasName = aliases.next().toString();
          attrValue = attributes.getValue(aliasName);
          if ((attrValue == null) && (customAttributes != null)) {
            attrValue = (String) customAttributes.get(aliasName);        
          }
        }
        if (attrValue == null) {
          if (attrDef.isRequired()) {
            throw new TagAttributeException(locator(), "The required attribute '" + name + "' was not defined for element '" + getName() + "'.");
          }
        }
      }
      if (attrValue != null) {
        return attrHandler.parseAttribute(locator, name, attrValue, evaluate);
      }
    }
    if (styleAttrDef != null) {
      Object value = getStyleAttribute(styleAttrDef);
      if (value != null) {
        return attrHandler.parseAttribute(locator, styleAttrDef.getName(), value, false);
      }
    }
    return defaultValue;
  }

  /**
   * Gets the local value for the given attribute's name.
   * @param name The attribute's name.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @return The determined value.
   * @exception TagAttributeException if the item contains an invalid value or an required attribute was not defined.
   */
  public Object getLocalAttribute(String name, boolean evaluate)
    throws TagAttributeException {
    
    String attrValue = attributes.getValue(name);
    if ((attrValue == null) && (customAttributes != null)) {
      attrValue = (String) customAttributes.get(name);        
    }
    try {
      Object value = attrValue;
      if (evaluate && (attrValue != null)) {
        return documentHandler.evalObject(attrValue);
      }
      return value;
    } catch (ExpressionException ex) {
      throw new TagAttributeException(locator(), "Expression evaluation failed: " + ex.getMessage(), ex);
    }
  }

  /**
   * Tells whether or not this tag does respect style attributes or not.
   * @return always <code>false</code> as default.
   */
  public boolean isStyleable() {
    return false;
  }
  
  /**
   * Gets the style attribute with the given name.
   * @param name The name of the style attribute to get.
   * @return The style value or null, if not defined.
   * @exception TagAttributeException In case the tag attribute evaluation failed.
   */
  public Object getStyleAttribute(String name) throws TagAttributeException {
    AttributeDefinition attrDef = getStyleAttributeDefinition(name);
    if (attrDef == null) {
      return null;
    }
    return getLocalStyleAttribute(name);
  }
  
  /**
   * Gets the style attribute with the given name.
   * @param styleAttrDef The definition of the style attribute to get.
   * @return The style value or null, if not defined.
   * @exception TagAttributeException In case the tag attribute evaluation failed.
   */
  public Object getStyleAttribute(AttributeDefinition styleAttrDef) throws TagAttributeException {
    return getLocalStyleAttribute(styleAttrDef.getName());
  }
  
  /**
   * Gets the style attribute with the given name.
   * @param name The name of the style attribute to get.
   * @return The style value or null, if not defined.
   * @exception TagAttributeException In case the tag attribute evaluation failed.
   */
  public Object getLocalStyleAttribute(String name) throws TagAttributeException {
    if (!isStyleable()) {
      return null;
    }
    if (styleAttributes != null) {
      Object value = styleAttributes.get(name);
      if (value != null) {
        return value;
      }
    }
    if (styleParent != null) {
        // Changed by Kiran
      return styleParent.getStyleAttribute(name);
    }
    return null;
  }

  /**
   * Gets the style attribute with the given name.
   * @param name The name of the style attribute to get.
   * @return The style value or null, if not defined.
   * @exception TagAttributeException In case the tag attribute evaluation failed.
   */
  public Map getStyleAttributes() {
    return styleAttributes;
  }

  /**
   * Getter method for the the property attributeHandler.
   * @return The current value of property attributeHandler.
   */
  public AttributeHandler getAttributeHandler() {
    return attributeHandler;
  }

  /**
   * Gets the value for the given attribute's name.
   * Works like method <code>getStringAttribute(java.lang.String, boolean)</code> but takes care of escape sequences.
   * @param attrDef The attribute definition.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @param styleAttrDef The definition of the style to consider.
   * @return The determined value.
   * @exception TagAttributeException if the item contains an invalid value or an required attribute was not defined.
   */
  public String getTextAttribute(AttributeDefinition attrDef, boolean evaluate, AttributeDefinition styleAttrDef) throws TagAttributeException {
    return getTextAttribute(attrDef, null, evaluate, styleAttrDef);
  }
  /**
   * Gets the value for the given attribute's name.
   * Works like method <code>getStringAttribute(java.lang.String, java.lang.String, boolean)</code> but takes care of escape sequences.
   * @param attrDef The attribute definition.
   * @param defaultValue The default value in case the attribute was not defined.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @param styleAttrDef The definition of the style to consider.
   * @return The determined value.
   * @exception TagAttributeException if the item contains an invalid value or an required attribute was not defined.
   */
  public String getTextAttribute(AttributeDefinition attrDef, String defaultValue, boolean evaluate, AttributeDefinition styleAttrDef)
    throws TagAttributeException {
    
    String rawText = getStringAttribute(attrDef, defaultValue, evaluate, styleAttrDef);
    if (rawText == null) {
      return null;
    }
    return attributeHandler.parseTextAttribute(locator(), attrDef.getName(), rawText, evaluate, true);
  }

  /**
   * Gets the value for the given attribute's name.
   * @param attrDef The attribute definition.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @param styleAttrDef The definition of the style to consider.
   * @return The determined value.
   * @exception TagAttributeException if the item contains an invalid value or an required attribute was not defined.
   */
  public String getStringAttribute(AttributeDefinition attrDef, boolean evaluate, AttributeDefinition styleAttrDef) throws TagAttributeException {
    return getStringAttribute(attrDef, null, evaluate, styleAttrDef);
  }
  /**
   * Gets the value for the given attribute's name.
   * @param attrDef The attribute definition.
   * @param defaultValue The default value in case the attribute was not defined.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @param styleAttrDef The definition of the style to consider.
   * @return The determined value.
   * @exception TagAttributeException if the item contains an invalid value or an required attribute was not defined.
   */
  public String getStringAttribute(AttributeDefinition attrDef, String defaultValue, boolean evaluate, AttributeDefinition styleAttrDef)
    throws TagAttributeException {
    
    Object attr = getAttribute(attrDef, defaultValue, evaluate, styleAttrDef, attributeHandler.getTextHandler());
    return (String) attr;
  }
  /**
   * Gets the value for the given attribute's name.
   * @param attrName The attribute name.
   * @param defaultValue The default value in case the attribute was not defined.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @param styleAttrName The name of the style to consider.
   * @return The determined value.
   * @exception TagAttributeException if the item contains an invalid value or an required attribute was not defined.
   */
  public String getStringAttribute(String attrName, String defaultValue, boolean evaluate, String styleAttrName)
    throws TagAttributeException {
    
    AttributeDefinition attrDef = getAttributeDefinition(attrName);
    AttributeDefinition styleAttrDef = getStyleAttributeDefinition(styleAttrName);
    Object attr = getAttribute(attrDef, defaultValue, evaluate, styleAttrDef, attributeHandler.getTextHandler());
    return (String) attr;
  }
  
  /**
   * Gets the float value for the given attribute's name.
   * @param attrDef The attribute definition.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @param styleAttrDef The definition of the style to consider.
   * @return The determined value.
   * @exception TagAttributeException Thrown in case the item contained an invalid value.
   */
  public float getFloatAttribute(AttributeDefinition attrDef, boolean evaluate, AttributeDefinition styleAttrDef) throws TagAttributeException {
    return getFloatAttribute(attrDef, 0.0f, evaluate, styleAttrDef);
  }
  /**
   * Gets the float value for the given attribute's name.
   * @param attrDef The attribute definition.
   * @param defaultValue The default value in case the attribute was not defined.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @param styleAttrDef The definition of the style to consider.
   * @return The determined value.
   * @exception TagAttributeException Thrown in case the item contained an invalid value.
   */
  public float getFloatAttribute(AttributeDefinition attrDef, float defaultValue, boolean evaluate, AttributeDefinition styleAttrDef)
    throws TagAttributeException {
    
    Object attr = getAttribute(attrDef, new Float(defaultValue), evaluate, styleAttrDef, attributeHandler.getFloatHandler());
    return ((Number) attr).floatValue();
  }

  /**
   * Gets the dimension value for the given attribute's name.
   * @param attrDef The attribute definition.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @param styleAttrDef The definition of the style to consider.
   * @return The determined value.
   * @exception TagAttributeException Thrown in case the item contained an invalid value.
   */
  public float getDimensionAttribute(AttributeDefinition attrDef, boolean evaluate, AttributeDefinition styleAttrDef) throws TagAttributeException {
    return getDimensionAttribute(attrDef, 0.0f, evaluate, styleAttrDef);
  }

  /**
   * Gets the dimension value for the given attribute's name.
   * @param attrDef The attribute definition.
   * @param defaultValue The default value in case the attribute was not defined.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @param styleAttrDef The definition of the style to consider.
   * @return The determined value.
   * @exception TagAttributeException Thrown in case the item contained an invalid value.
   */
  public float getDimensionAttribute(AttributeDefinition attrDef, float defaultValue, boolean evaluate, AttributeDefinition styleAttrDef)
    throws TagAttributeException {
    
    return getDimensionAttribute(attrDef, new Float(defaultValue), evaluate, styleAttrDef);
  }

  /**
   * Gets the dimension value for the given attribute's name.
   * @param attrDef The attribute definition.
   * @param defaultValue The default value in case the attribute was not defined.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @param styleAttrDef The definition of the style to consider.
   * @return The determined value.
   * @exception TagAttributeException Thrown in case the item contained an invalid value.
   */
  public float getDimensionAttribute(AttributeDefinition attrDef, Float defaultValue, boolean evaluate, AttributeDefinition styleAttrDef)
    throws TagAttributeException {
    
    Object attr = getAttribute(attrDef, defaultValue, evaluate, styleAttrDef, attributeHandler.getDimensionHandler());
    return ((Number) attr).floatValue();
  }

  /**
   * Gets the integer value for the given attribute's name.
   * @param attrDef The attribute definition.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @param styleAttrDef The definition of the style to consider.
   * @return The determined value.
   * @exception TagAttributeException Thrown in case the item contained an invalid value.
   */
  public int getIntegerAttribute(AttributeDefinition attrDef, boolean evaluate, AttributeDefinition styleAttrDef) throws TagAttributeException {
    return getIntegerAttribute(attrDef, 0, evaluate, styleAttrDef);
  }
  /**
   * Gets the integer value for the given attribute's name.
   * @param attrDef The attribute definition.
   * @param defaultValue The default value in case the attribute was not defined.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @param styleAttrDef The definition of the style to consider.
   * @return The determined value.
   * @exception TagAttributeException Thrown in case the item contained an invalid value.
   */
  public int getIntegerAttribute(AttributeDefinition attrDef, int defaultValue, boolean evaluate, AttributeDefinition styleAttrDef)
    throws TagAttributeException {
    
    Object attr = getAttribute(attrDef, new Integer(defaultValue), evaluate, styleAttrDef, attributeHandler.getIntegerHandler());
    return ((Number) attr).intValue();
  }

  /**
   * Gets the boolean value for the given attribute's name.
   * @param attrDef The attribute definition.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @param styleAttrDef The definition of the style to consider.
   * @return The determined value.
   * @exception TagAttributeException if the item contains an invalid value or an required attribute was not defined.
   */
  public boolean getBooleanAttribute(AttributeDefinition attrDef, boolean evaluate, AttributeDefinition styleAttrDef)
    throws TagAttributeException {
    
    return getBooleanAttribute(attrDef, false, evaluate, styleAttrDef);
  }
  /**
   * Gets the boolean value for the given attribute's name.
   * @param attrDef The attribute definition.
   * @param defaultValue The default value in case the attribute was not defined.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @param styleAttrDef The definition of the style to consider.
   * @return The determined value.
   * @exception TagAttributeException if the item contains an invalid value or an required attribute was not defined.
   */
  public boolean getBooleanAttribute(AttributeDefinition attrDef, boolean defaultValue, boolean evaluate, AttributeDefinition styleAttrDef)
    throws TagAttributeException {
    
    Object attr = getAttribute(attrDef, new Boolean(defaultValue), evaluate, styleAttrDef, attributeHandler.getBooleanHandler());
    return ((Boolean) attr).booleanValue();
  }

  /**
   * Gets the color value for the given attribute's name.
   * @param attrDef The attribute definition.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @param styleAttrDef The definition of the style to consider.
   * @return The determined value.
   * @exception TagAttributeException Thrown in case the item contained an invalid value.
   */
  public Color getColorAttribute(AttributeDefinition attrDef, boolean evaluate, AttributeDefinition styleAttrDef)
    throws TagAttributeException {
    
    return getColorAttribute(attrDef, null, evaluate, styleAttrDef);
  }
  /**
   * Gets the color value for the given attribute's name.
   * @param attrDef The attribute definition.
   * @param defaultValue The default value in case the attribute was not defined.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @param styleAttrDef The definition of the style to consider.
   * @return The determined value.
   * @exception TagAttributeException Thrown in case the item contained an invalid value.
   */
  public Color getColorAttribute(AttributeDefinition attrDef, Color defaultValue, boolean evaluate, AttributeDefinition styleAttrDef)
    throws TagAttributeException {
    
    Object attr = getAttribute(attrDef, defaultValue, evaluate, styleAttrDef, attributeHandler.getColorHandler());
    return (Color) attr;
  }

  /**
   * Gets the border value for the given attribute's name.
   * @param attrDef The attribute definition.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @param styleAttrDef The definition of the style to consider.
   * @return The determined value (default is NO_BORDER).
   * @exception TagAttributeException Thrown in case the item contained an invalid value.
   */
  public int getBorderAttribute(AttributeDefinition attrDef, boolean evaluate, AttributeDefinition styleAttrDef) throws TagAttributeException {
    return getBorderAttribute(attrDef, Rectangle.NO_BORDER, evaluate, styleAttrDef);
  }
  /**
   * Gets the border value for the given attribute's name.
   * @param attrDef The attribute definition.
   * @param defaultValue The default value in case the attribute was not defined.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @param styleAttrDef The definition of the style to consider.
   * @return The determined value (default is NO_BORDER).
   * @exception TagAttributeException Thrown in case the item contained an invalid value.
   */
  public int getBorderAttribute(AttributeDefinition attrDef, int defaultValue, boolean evaluate, AttributeDefinition styleAttrDef)
    throws TagAttributeException {
    
    Object attr = getAttribute(attrDef, new Integer(defaultValue), evaluate, styleAttrDef, attributeHandler.getBorderHandler());
    return ((Number) attr).intValue();
  }
  
  /**
   * Gets the horizontal alignment value for the given attribute's name.
   * @param attrDef The attribute definition.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @param styleAttrDef The definition of the style to consider.
   * @return The determined value.
   * @exception TagAttributeException Thrown in case the item contained an invalid value.
   */
  public int getHorizontalAlignmentAttribute(AttributeDefinition attrDef, boolean evaluate, AttributeDefinition styleAttrDef) throws TagAttributeException {
    return getHorizontalAlignmentAttribute(attrDef, Element.ALIGN_LEFT, evaluate, styleAttrDef);
  }
  /**
   * Gets the horizontal alignment value for the given attribute's name.
   * @param attrDef The attribute definition.
   * @param defaultValue The default value in case the attribute was not defined.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @param styleAttrDef The definition of the style to consider.
   * @return The determined value.
   * @exception TagAttributeException Thrown in case the item contained an invalid value.
   */
  public int getHorizontalAlignmentAttribute(AttributeDefinition attrDef, int defaultValue, boolean evaluate, AttributeDefinition styleAttrDef)
    throws TagAttributeException {
    
    Object attr = getAttribute(attrDef, new Integer(defaultValue), evaluate, styleAttrDef, attributeHandler.getHorizontalAlignmentHandler());
    return ((Number) attr).intValue();
  }
  
  /**
   * Gets the vertical alignment value for the given attribute's name.
   * @param attrDef The attribute definition.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @param styleAttrDef The definition of the style to consider.
   * @return The determined value.
   * @exception TagAttributeException Thrown in case the item contained an invalid value.
   */
  public int getVerticalAlignmentAttribute(AttributeDefinition attrDef, boolean evaluate, AttributeDefinition styleAttrDef) throws TagAttributeException {
    return getVerticalAlignmentAttribute(attrDef, Element.ALIGN_BASELINE, evaluate, styleAttrDef);
  }  
  /**
   * Gets the vertical alignment value for the given attribute's name.
   * @param attrDef The attribute definition.
   * @param defaultValue The default value to use in case the attribute was not defined.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @param styleAttrDef The definition of the style to consider.
   * @return The determined value.
   * @exception TagAttributeException Thrown in case the item contained an invalid value.
   */
  public int getVerticalAlignmentAttribute(AttributeDefinition attrDef, int defaultValue, boolean evaluate, AttributeDefinition styleAttrDef)
    throws TagAttributeException {
    
    Object attr = getAttribute(attrDef, new Integer(defaultValue), evaluate, styleAttrDef, attributeHandler.getVerticalAlignmentHandler());
    return ((Number) attr).intValue();
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    // resetting content controlling flags
    itemOpened = true;
    itemClosed = false;
    evaluateXml = true;
    rendered = true;

    resetContent();

    if (!isValid()) {
      return;
    }

    if (isAttributeSupported(CommonAttributes.RENDERED)) {
      if (!BeanUtils.isEmpty(renderedAttr)) {
        try {
          rendered = documentHandler.evalBoolean(renderedAttr);
        } catch (ExpressionException ex) {
          throw new DocumentHandlerException(locator(), "Failed to evaluate value '" + renderedAttr + "' of attribute 'rendered': " + ex.getMessage(), ex);
        }
      }
    }

    if (!rendered) {
      return;
    }

    // getting the parent's class
    styleAttributes = new HashMap();
    styleClass = null;
    style = null;
    if (parentItem != null) {
      Map parentStyles = parentItem.getStyleAttributes();
      mergeStyleAttributes(parentStyles);
    }
    if (isAttributeSupported(CommonAttributes.CLASS)) {
      // getting the style attribute
      styleClass = getStringAttribute(CommonAttributes.CLASS, styleClass, true, null);
    }
    Map styles = documentHandler.getStyle(this, styleClass);
    if (styles != null) {
      mergeStyleAttributes(styles);
    }
    if (isAttributeSupported(CommonAttributes.STYLE)) {
      // getting the style attribute
      String localStyle = getStringAttribute(CommonAttributes.STYLE, null, true, null);
      if (localStyle != null) {
        style = localStyle;
        // parsing style attribute
        Map localStyles = new HashMap();
        styleParser.parseStyle(locator(), style, supportedStyleAttributeMap, localStyles);
        mergeStyleAttributes(localStyles);
      }
    }
    
    // processing background color
    if (isAttributeSupported(CommonAttributes.BACKGROUND_COLOR, CommonStyleAttributes.BACKGROUND_COLOR)) {
      setBackgroundColor(getColorAttribute(CommonAttributes.BACKGROUND_COLOR, true, CommonStyleAttributes.BACKGROUND_COLOR));
    }
    
    handleFontAttributes();
  }
  
  /**
   * Merges the given styles into the tags style attributes.
   * @param styleAttrs The style attributes to merge.
   * @throws TagAttributeException In case the attribute handling failed.
   */
  private void mergeStyleAttributes(Map styleAttrs) throws TagAttributeException {
    String fn = (String) CollectionUtils.get(styleAttrs, TagAttributes.ATTR_FONT_NAME);
    if (fn != null) {
      DocumentFont df = documentHandler.getFont(fn);
      styleAttrs.remove(TagAttributes.ATTR_FONT_NAME);
      Color color = df.getColor();
      if (color != null) {
        styleAttributes.put(TagAttributes.ATTR_FONT_COLOR, color);
      }
      styleAttributes.put(TagAttributes.ATTR_FONT_ENCODING, df.getEncoding());
      styleAttributes.put(TagAttributes.ATTR_FONT_FAMILY, df.getFamily());
      styleAttributes.put(TagAttributes.ATTR_FONT_SIZE, Float.toString(df.getSize()));

      String fontStyle = int2FontStyle(df.getStyle());
      styleAttributes.put(TagAttributes.ATTR_FONT_STYLE, fontStyle);
      styleAttributes.put(TagAttributes.ATTR_TEXT_RISE, Float.toString(df.getTextRise()));
    }
    CollectionUtils.putAll(styleAttributes, styleAttrs);
  }
  
  /**
   * Handles the font attributes.
   * @throws DocumentHandlerException
   */
  protected void handleFontAttributes() throws DocumentHandlerException {
    // setting the local font
    if (isAttributeSupported(CommonAttributes.FONT, CommonStyleAttributes.FONT_NAME))  {
      // getting attributes
      String fontDef = getStringAttribute(CommonAttributes.FONT, true, null);
      DocumentFont font = null;
      if (fontDef != null) {
        font = documentHandler.getFont(fontDef);
        setLocalFont(font);
        styleAttributes.remove(TagAttributes.ATTR_FONT_COLOR);
        styleAttributes.remove(TagAttributes.ATTR_FONT_ENCODING);
        styleAttributes.remove(TagAttributes.ATTR_FONT_FAMILY);
        styleAttributes.remove(TagAttributes.ATTR_FONT_SIZE);
        styleAttributes.remove(TagAttributes.ATTR_FONT_STYLE);
        styleAttributes.remove(TagAttributes.ATTR_TEXT_RISE);
        return;
      }
      fontDef = getStringAttribute(null, true, CommonStyleAttributes.FONT_NAME);
      if (fontDef != null) {
        font = documentHandler.getFont(fontDef);
      }
      
      Color color = getColorAttribute(null, false, CommonStyleAttributes.FONT_COLOR);
      if ((color == null) && (font != null)) {
        color = font.getColor();
      }
      String family = getStringAttribute(null, false, CommonStyleAttributes.FONT_FAMILY);
      if ((family == null) && (font != null)) {
        family = font.getFamily();
      }
      String encoding = getStringAttribute(null, false, CommonStyleAttributes.FONT_ENCODING);
      if ((encoding == null) && (font != null)) {
        encoding = font.getEncoding();
      }
      if (encoding == null) {
        encoding = BaseFont.CP1252;
      }
      float size = -1;
      if (font != null) {
        size = font.getSize();
      }
      if (isAttributeDefined(null, CommonStyleAttributes.FONT_SIZE)) {
        size = getDimensionAttribute(null, false, CommonStyleAttributes.FONT_SIZE);
      } else if (font != null) {
        size = font.getSize();
      }
      String styleAttribute = getStringAttribute(null, false, CommonStyleAttributes.FONT_STYLE);
      int style = 0;
      if (styleAttribute != null) {
        style = Font.getStyleValue(styleAttribute);
      } else if (font != null) {
        style = font.getStyle();
      }
      float textRise = 0.0F;
      if (isAttributeDefined(null, CommonStyleAttributes.TEXT_RISE)) {
        textRise = getDimensionAttribute(null, false, CommonStyleAttributes.TEXT_RISE);
      } else if (font != null) {
        textRise = font.getTextRise();
      }
      
      // configuring font
      if (family != null) {
        font = createFont(family, encoding, size, style, textRise, color);
        setLocalFont(font);
      }
      
    }
  }
  
  /**
   * Gets the font style from the given integer.
   * @param fontStyle The font style to transform into text.
   * @return The font style as textual attribute.
   */
  protected String int2FontStyle(int fontStyle) {
    StringBuffer styleBuf = new StringBuffer();
    if ((fontStyle & Font.BOLD) != 0) {
      if (styleBuf.length() > 0) {
        styleBuf.append('|');
      }
      styleBuf.append("bold");
    }
    if ((fontStyle & Font.ITALIC) != 0) {
      if (styleBuf.length() > 0) {
        styleBuf.append('|');
      }
      styleBuf.append("italic");
    }
    if ((fontStyle & Font.UNDERLINE) != 0) {
      if (styleBuf.length() > 0) {
        styleBuf.append('|');
      }
      styleBuf.append("underline");
    }
    if ((fontStyle & Font.STRIKETHRU) != 0) {
      if (styleBuf.length() > 0) {
        styleBuf.append('|');
      }
      styleBuf.append("strikethru");
    }
    if (styleBuf.length() == 0) {
      styleBuf.append("normal");
    }
    return styleBuf.toString();
  }
  
  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while closing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
  }

  /**
   * Adds the current content to the given root phrase while taking care of ${pageCount} expressions.
   * @param phrase The phrase to add the content to.
   * @param fixedLeading The fixed vertical leading distance.
   * @param lineSpacing The line spacing factor.
   * @return The phrase to which the chunk was added.
   * @exception DocumentHandlerException In case the document processing failed.
   */
  public Phrase addChunk(Phrase phrase, float fixedLeading, float lineSpacing)
    throws DocumentHandlerException {

    try {
      DocumentFont docFont = getFont();
      float textRise = docFont.getTextRise();
      String content = getContent();
      int pageCountPos = checkPageCount(content);
      if (pageCountPos < 0) {
        return addChunk(this, phrase, documentHandler.executeTemplate(content), docFont, fixedLeading, lineSpacing);
      }
      
      String part1 = content.substring(0, pageCountPos);
      Phrase localPhrase = addChunk(this, phrase, documentHandler.executeTemplate(part1), docFont, fixedLeading, lineSpacing);
      PdfWriter writer = (PdfWriter) documentHandler.getDocumentWriter();
      try {
        Chunk ch = new Chunk(Image.getInstance(ensureNumberOfPagesTemplate(getFont(), writer.getDirectContent())), 0, -1);
        if (backgroundColor != null) {
          ch.setBackground(backgroundColor);
        }
        if (textRise != 0.0F) {
          ch.setTextRise(textRise);
        }
        localPhrase.add(ch);
      } catch (BadElementException ex) {
        throw new DocumentHandlerException(locator(), "Unable to add ${pageCount} to phase.");
      }
      String part2 = content.substring(pageCountPos + 12);
      return addChunk(this, localPhrase, documentHandler.executeTemplate(part2), docFont, fixedLeading, lineSpacing);
    } catch (TemplateException ex) {
      throw new DocumentHandlerException(locator(), "Template evaluation failed: " + ex.getMessage(), ex);
    }
  }

  /**
   * Adds the current content to the given PhraseHolder while taking care of ${pageCount} expressions.
   * @param phraseHolder The PhraseHolder to submit the content to.
   * @exception DocumentHandlerException In case the content processing failed.
   */
  public void handleContent(PhraseHolder phraseHolder)
    throws DocumentHandlerException {

    try {
      DocumentFont docFont = getFont();
      float textRise = docFont.getTextRise();
      String content = getContent();
      int pageCountPos = checkPageCount(content);
      if (pageCountPos < 0) {
        phraseHolder.addChunk(this, documentHandler.executeTemplate(content), docFont);
        return;
      }
      
      String part1 = content.substring(0, pageCountPos);
      phraseHolder.addChunk(this, documentHandler.executeTemplate(part1), docFont);
      PdfWriter writer = (PdfWriter) documentHandler.getDocumentWriter();
      try {
        Chunk ch = new Chunk(Image.getInstance(ensureNumberOfPagesTemplate(getFont(), writer.getDirectContent())), 0, -1);
        if (backgroundColor != null) {
          ch.setBackground(backgroundColor);
        }
        if (textRise != 0.0F) {
          ch.setTextRise(textRise);
        }
        phraseHolder.addChunk(this, ch);
      } catch (BadElementException ex) {
        throw new DocumentHandlerException(locator(), "Unable to add ${pageCount} to phase.");
      }
      String part2 = content.substring(pageCountPos + 12);
      phraseHolder.addChunk(this, documentHandler.executeTemplate(part2), docFont);
    } catch (TemplateException ex) {
      throw new DocumentHandlerException(locator(), "Template evaluation failed: " + ex.getMessage(), ex);
    }
  }
  
  /**
   * Checks, whether the content contains a '${pageCount}' "expression".  
   * @param content The content to check.
   * @return The position of the ${pageCount} string or -1.
   */
  private int checkPageCount(String content) {
    if (content == null) {
      return -1;
    }
    return content.indexOf("${pageCount}");
  }

  /**
   * Adds the given chunk to the root phrase.
   * @param phrase The phrase to add the chunk to.
   * @param chunk The chunk to add to the root phrase.
   * @param fixedLeading The fixed vertical leading distance.
   * @param lineSpacing The line spacing factor.
   * @return The phrase the chunk has been added to.
   */
  public Phrase addChunk(Phrase phrase, Chunk chunk, float fixedLeading, float lineSpacing) {
    //Added by Kiran
  	//If there is background color to be set for the text
  	//then bgcolor would have been created in its respective tag
    if (this.backgroundColor != null) {
      chunk.setBackground(backgroundColor);
    }
    // End Kiran

    phrase.add(chunk);
    extendLeading(phrase, chunk, fixedLeading, lineSpacing);
    return phrase;
  }
  
  /**
   * Adds the given chunk to the root phrase.
   * @param tag The document tag instance.
   * @param phrase The phrase to add the chunk to.
   * @param chunk The chunk to add to the root phrase.
   * @param font The font to use.
   * @param fixedLeading The fixed vertical leading distance.
   * @param lineSpacing The line spacing factor.
   * @return The phrase to which the chunk was added.
   * @exception DocumentHandlerException In case the document processing failed.
   */
  public Phrase addChunk(BaseDocumentTag tag, Phrase phrase, String chunk, DocumentFont font, float fixedLeading, float lineSpacing)
    throws DocumentHandlerException {
    
    Phrase localPhrase = phrase;
    if (localPhrase == null) {
      localPhrase = new Phrase(0.0F);
    }
    if ((localPhrase.getChunks().size() == 0) && isEmptyString(chunk)) {
      return localPhrase;
    }
    
    processContents(tag, localPhrase, chunk, font, fixedLeading, lineSpacing);
    return localPhrase;
  }

  /**
   * Creates a chunk for the current content.
   * @param fixedLeading The fixed vertical leading distance.
   * @param lineSpacing The line spacing factor.
   * @return The created chunk.
   * @exception DocumentHandlerException In case the document processing failed.
   */
  public Phrase processContents(float fixedLeading, float lineSpacing)
    throws DocumentHandlerException {
    
    try {
      DocumentFont docFont = getFont();
      String content = getContent();
      int pageCountPos = checkPageCount(content);
      if (pageCountPos < 0) {
        return processContents(this, null, documentHandler.executeTemplate(content), docFont, fixedLeading, lineSpacing);
      }
      
      String part1 = content.substring(0, pageCountPos);
      Phrase phrase = processContents(this, null, documentHandler.executeTemplate(part1), docFont, fixedLeading, lineSpacing);
      if (phrase == null) {
        phrase = new Phrase(0);
      }
      PdfWriter writer = (PdfWriter) documentHandler.getDocumentWriter();
      try {
        Chunk ch = new Chunk(Image.getInstance(ensureNumberOfPagesTemplate(getFont(), writer.getDirectContent())), 0, -1);
        if (backgroundColor != null) {
          ch.setBackground(backgroundColor);
        }
        phrase.add(ch);
      } catch (BadElementException ex) {
        throw new DocumentHandlerException(locator(), "Unable to add ${pageCount} to phase.");
      }
      String part2 = content.substring(pageCountPos + 12);
      return processContents(this, phrase, documentHandler.executeTemplate(part2), docFont, fixedLeading, lineSpacing);
    } catch (TemplateException ex) {
      throw new TagAttributeException(locator(), "Template evaluation failed: " + ex.getMessage(), ex);
    }
  }

  /**
   * Processes the given contents an adds the computed elements to the given phrase.
   * @param tag The document tag.
   * @param phrase The phrase to add the contents to. In case null is given, an empty phrase is created.
   * @param contents The contents to process.
   * @param font The font to use.
   * @param fixedLeading The fixed vertical leading distance.
   * @param lineSpacing The line spacing factor.
   * @return The phrase to which the chunk was added.
   * @exception DocumentHandlerException In case the document processing failed.
   */
  public Phrase processContents(BaseDocumentTag tag, Phrase phrase, String contents, DocumentFont font, 
                                float fixedLeading, float lineSpacing)
    throws DocumentHandlerException {
    
    if (isIgnoreFlush()) {
      // avoid flushing the same item twice
      return null;
    }

    if ((contents == null) || (contents.length() == 0)) {
      // no sense to go on!
      return phrase;
    }

    Phrase localPhrase = phrase;
    if (localPhrase == null) {
      localPhrase = new Phrase(0.0F);
    }
    
    // scanning phrase text for xml tokens
    int textLen = contents.length();
    char c = 0;
    boolean parsePhrase = false;

    // decoding explicite white spaces
    StringBuffer chunkBuf = new StringBuffer(textLen);
    filterContents(tag, contents, chunkBuf, isTrimNextHead());
    
    textLen = chunkBuf.length();
    int cDataBegin = -1;
    int cDataEnd = -1;
    if (documentHandler.isParseDynamicContent() && evaluateXml) {
     outerLoop:
      for (int i = 0; i < textLen; i++) {
        c = chunkBuf.charAt(i);
        if (c == '<') {
          if (textLen > i+1 && chunkBuf.charAt(i+1) == '!') { 
            // checking for CDATA
            int cDataBeginLen = CDATA_BEGIN.length();
            for (int j = 0; j < cDataBeginLen; j++) {
              char cc = chunkBuf.charAt(i + j);
              char cdc = CDATA_BEGIN.charAt(j);
              if (cc != cdc) {
                cDataBegin = -1;
                cDataEnd = -1;
                break;
              }
              if (j == cDataBeginLen - 1) {
                cDataBegin = i + cDataBeginLen;
                parsePhrase = false; 
              }
            }
            
            if (cDataBegin != -1) {
              for (i = cDataBegin; i < textLen; i++) {
                c = chunkBuf.charAt(i);
                if (c == ']' && chunkBuf.charAt(i+1) == ']') {
                  int cDataEndLen = CDATA_END.length();
                  for (int j = 0; j < cDataEndLen; j++) {
                    char cc = chunkBuf.charAt(i + j);
                    char cdc = CDATA_END.charAt(j);
                    if (cc != cdc) {
                      cDataEnd = -1;
                      break;
                    }
                    if (j == cDataEndLen - 1) {
                      cDataEnd = i;
                      break outerLoop;
                    }
                  }
                }
              }
            }
            break outerLoop;
          } 
          int startIdx = i;
          for (; i < textLen; i++) {
            c = chunkBuf.charAt(i);
            if (c == '>') {
              if (i - startIdx < 2) {
                // empty tags are not allowed
                parsePhrase = false;
              } else {
                // OK, we should parse the phrase
                parsePhrase = true;
              }
              break outerLoop;
            }
            if (Character.isWhitespace(c)) {
              // whitespaces in XML tags are not allowed
              parsePhrase = false;
              break outerLoop;
            }
          }
        } else if (c == '&') {
          int curPos = i;
          if ((textLen - curPos) > 3) {
            if (chunkBuf.charAt(i + 3) == ';') {
              // seems to be an XML encoded character
              parsePhrase = true;
              break outerLoop;
            }
          }
        }
      }
    }
    if (cDataBegin != - 1  && cDataEnd != -1) { 
      chunkBuf = new StringBuffer(chunkBuf.substring(cDataBegin, cDataEnd)); 
    }
    if (parsePhrase) {
      // letting the parser do the stuff!
      try {
        this.ignoreFlush = true;
        // setting up the dynami content container
        DynamicContentContainer dynamicContentContainer = documentHandler.getDynamicContentContainer();
        dynamicContentContainer.setContents(localPhrase);
        
        // setting up the XML parser
        if (dynamicContentXmlReader == null) {
          dynamicContentXmlReader = XmlUtils.createXmlReader(documentHandler.getXmlReaderClass());
        }
        dynamicContentXmlReader.setContentHandler(documentHandler);

        // avoiding that the internally processed stuff is copied too.
        documentHandler.disableTemplateCopyWriter();
        
        // parsing the dynamic content
        String dynaContent = "<" + DynamicContentContainer.TAG_NAME + ">" + chunkBuf.toString() + "</" + DynamicContentContainer.TAG_NAME + ">";
        dynamicContentXmlReader.parse(new InputSource(new StringReader(dynaContent)));
        this.ignoreFlush = false;
        
        // allowing the copying of XML events again.
        documentHandler.enableTemplateCopyWriter();
        
        return dynamicContentContainer.getContents();
      } catch (SAXException ex) {
        throw new DocumentHandlerException(tag.locator(), "Error while parsing expression result: " + ex.getMessage(), ex);
      } catch (IOException ex) {
        throw new DocumentHandlerException(tag.locator(), "Error while parsing expression result: " + ex.getMessage(), ex);
      }
    }

    // processing static content
    if (chunkBuf.length() > 0) {
      Chunk chnk = null;
      if (font == null) {
        chnk = new Chunk(chunkBuf.toString());
      } else {
        chnk = new Chunk(chunkBuf.toString(), font.getFont());
      }
      if (backgroundColor != null) {
        chnk.setBackground(backgroundColor);
      }
      float textRise = font.getTextRise();
      if (textRise != 0.0F) {
        chnk.setTextRise(textRise);
      }
      addChunk(localPhrase, chnk, fixedLeading, lineSpacing);
    }
    return localPhrase;
  }

  /**
   * Processes the escape sequences from the given content string and writes
   * the result into the given buffer. 
   * @param tag The tag used to tell the document position.
   * @param contents The contents to parse for escape sequences
   * @param buffer The buffer to write to.
   * @param trimHead Tells whether or not to trim the leading white spaces
   * @throws DocumentHandlerException
   */
  protected void filterContents(BaseDocumentTag tag, String contents, 
                                StringBuffer buffer, boolean trimHead) 
    throws DocumentHandlerException {

    int textLen = StringUtils.getLength(contents);
    int offset = 0;
    if (trimHead) {
      // decoding explicite white spaces
      while (offset < textLen) {
        // removing all leading white spaces in case 'trimHead' is switched on
        // also removing all extra white spaces
        char c = contents.charAt(offset);
        if (!Character.isWhitespace(c)) {
          break;
        }
        ++offset;
      }
    }
    
    ContentFilter contentFilter = documentHandler.getContentFilter();
    if (contentFilter != null) {
      contentFilter.filterContents(this, contents, offset, buffer);
    } else {
      if (offset > 0) {
        buffer.append(contents.substring(offset));
      } else {
        buffer.append(contents);
      }
    }
  }
  
  /**
   * Extends the leading of the given phrase.
   * @param phrase The phrase to alter.
   * @param leading The size up to which to extend the leading.
   */
  protected void extendLeading(Phrase phrase, float leading) {
    float currentLeading = phrase.getLeading();
    phrase.setLeading(Math.max(currentLeading, leading));
  }
  
  /**
   * Extends the leading of the given phrase.
   * @param phrase The phrase to alter.
   * @param font The font that wants to extend the leading of the given phrase.
   * @param fixedLeading The fixed vertical leading distance.
   * @param lineSpacing The line spacing factor.
   */
  protected void extendLeading(Phrase phrase, Font font, float fixedLeading, float lineSpacing) {
    float currentLeading = phrase.getLeading();
    phrase.setLeading(Math.max(currentLeading, fixedLeading + font.getSize() * lineSpacing));
  }

  /**
   * Extends the leading of the given phrase.
   * @param phrase The phrase to alter.
   * @param extender The chunk that wants to extend the leading of the given phrase.
   * @param fixedLeading The fixed vertical leading distance.
   * @param lineSpacing The line spacing factor.
   */
  protected void extendLeading(Phrase phrase, Chunk extender, float fixedLeading, float lineSpacing) {
    Map chunkAttributes = extender.getAttributes();
    if (chunkAttributes != null) {
      Object image = chunkAttributes.get(IMAGE_CHUNK_ATTR);
      if (image != null) {
        Object[] imgData = (Object[]) image; 
        Image img = (Image) imgData[0];
        extendLeading(phrase, fixedLeading + img.getScaledHeight() * lineSpacing);
        return;
      }
    }
    extendLeading(phrase, extender.getFont(), fixedLeading, lineSpacing);
  }

  /**
   * Extends the leading of the given phrase.
   * @param phrase The phrase to alter.
   * @param extender The phrase that wants to extend the leading of the given phrase.
   */
  protected void extendLeading(Phrase phrase, Phrase extender) {
    float currentLeading = phrase.getLeading();
    phrase.setLeading(Math.max(currentLeading, extender.getLeading()));
  }

  /**
   * Creates a new chunk.
   * @param text The text for the chunk to be created.
   * @param font The font to use.
   * @return The created chunk.
   */
  protected Chunk createChunk(String text, DocumentFont font) {
    Chunk chunk = new Chunk(text, font.getFont());
    //Added by Kiran
    //If there is background color to be set for the text
    //then bgcolor would have been created in its respective tag
    if (this.backgroundColor != null) {
      chunk.setBackground(backgroundColor);
    }
    // End Kiran
    float textRise = font.getTextRise();
    if (textRise != 0.0F) {
      chunk.setTextRise(textRise);
    }
    return chunk;
  }

  /**
   * Gets the color from the given String.
   * @param attrName The attribute name.
   * @param color The color given as String.
   * @param evaluate If this flag is set to true, the attribute will get evaluated.
   * @return The determined color.
   * @exception TagAttributeException In case the given string doesn't resolve to a color 
   *   in the color map and is a invalid color code.
   */
  protected Color colorFromString(String attrName, Object color, boolean evaluate)
    throws TagAttributeException {
    
    if (color == null) {
      return null;
    }
    
    return attributeHandler.parseColorAttribute(locator(), attrName, color, evaluate);
  }
  
  /**
   * Gets the font from the surrounding FontHolder tag.
   * @return The external font.
   */
  public DocumentFont getFont() {
    if (localFont != null) {
      return localFont;
    }
    if (fontHolder != null) {
      return fontHolder.getFont();
    }
    return DEFAULT_FONT;
  }
  
  /**
   * Gets the locally defined font.
   * @return The locally defined font.
   */
  public DocumentFont getLocalFont() {
    return localFont;
  }
  /**
   * Sets the local font.
   * @param localFont The local font to set.
   */
  protected void setLocalFont(DocumentFont localFont) {
    this.localFont = localFont;
  }
  /**
   * Sets the local font.
   * @param fontDef The name of the attribute that font definition to use.
   * @exception TagAttributeException In case the given font does not exist.
   */
  protected void setLocalFont(String fontDef) throws TagAttributeException {
    if (fontDef != null) {
      setLocalFont(documentHandler.getFont(fontDef));
    }
  }

  /**
   * Getter method for the the property backgroundColor.
   * @return The current value of property backgroundColor.
   */
  public Color getBackgroundColor() {
    return backgroundColor;
  }
  /**
   * Setter method for the the property backgroundColor.
   * @param backgroundColor The value to set for the property backgroundColor.
   */
  protected void setBackgroundColor(Color backgroundColor) {
    this.backgroundColor = backgroundColor;
  }

  /**
   * Gets the style class.
   * @return The current value of property styleClass.
   */
  public String getStyleClass() {
    return styleClass;
  }

  /**
   * Gets the inline style attributes.
   * @return The current value of property style.
   */
  public String getStyle() {
    return style;
  }

  /**
   * Adds the element at the latest parent item. 
   * @param element The element to add at the parent.
   * @exception DocumentHandlerException In case the inserting of the given element has failed.
   */
  protected void addToLatestElementContainer(Element element) throws DocumentHandlerException {
    documentHandler.latestElementContainer().addElement(this, element);
  }

  /**
   * Adds the cell at the latest parent item. 
   * @param cell The cell to add at the parent.
   * @exception DocumentHandlerException In case the inserting of the given cell has failed.
   */
  protected void addToLatestCellContainer(Cell cell) throws DocumentHandlerException {
    documentHandler.latestCellContainer().addCell(this, cell);
  }

  /**
   * Adds the cell at the latest parent item. 
   * @param cell The cell to add at the parent.
   * @exception DocumentHandlerException In case the inserting of the given cell has failed.
   */
  protected void addToLatestPdfCellContainer(PdfPCell cell) throws DocumentHandlerException {
    documentHandler.latestPdfCellContainer().addCell(this, cell);
  }

  /**
   * Checks if the content contains no or no real data.
   * @return true if the content is null, has zero length or 
   *   consists of whitespaces only.
   */
  protected boolean isEmptyContent() {
    StringBuffer cnt = content;
    if (cnt == null) {
      return true;
    }
    int len = cnt.length();
    for (int i = 0; i < len; i++) {
      char c = cnt.charAt(i);
      if (!Character.isWhitespace(c)) {
        return false;
      }
    }
    return true;
  }
  
  /**
   * Checks if the given string contains no or no real data.
   * @param text The string to check.
   * @return true if the string's content is null, has zero length or 
   *   consists of whitespaces only.
   */
  protected boolean isEmptyString(String text) {
    if (text == null) {
      return true;
    }
    int len = text.length();
    for (int i = 0; i < len; i++) {
      char c = text.charAt(i);
      if (!Character.isWhitespace(c)) {
        return false;
      }
    }
    return true;
  }
  
  /**
   * Checks whether the condition is valid or not.
   * @return true if the condition is not defined or evaluates to true, else false.
   * @exception DocumentHandlerException In case a document handling problem occurred.
   */
  protected boolean isValid()
    throws DocumentHandlerException {
    
    if ((parentItem != null) && !parentItem.isValid()) {
      return false;
    }
    if (condition != null) {
      return condition.isTrue();
    }
    return rendered;
  }
  
  /**
   * Tells whether or not this tag is rendered.
   * @return true if the tag is rendered, else false.
   */
  protected boolean isRendered() {
    return rendered;
  }

  /**
   * Creates the item's font according to the item's attributes.
   * @param family The font family.
   * @param encoding The font encoding.
   * @param size The font size.
   * @param style The font style.
   * @param textRise The text displacement relative to the baseline. 
   *    Positive values rise the text, negative values lower the text.
   * @param color The font color.
   * @return The created font.
   * @exception DocumentHandlerException In case the font could not be created.
   */
  protected DocumentFont createFont(String family, String encoding, float size, int style, 
                                    float textRise, Color color)
    throws DocumentHandlerException { 

    // creating font
    return documentHandler.createFont(family, encoding, size, style, textRise, color);
  }

  /**
   * Gets the width of the given phrase.
   * @param item The phrase to determine the width for.
   * @return The calculated width.
   */
  protected float getPhraseWidth(Phrase item) {
    float width = 0.0F;
    if (item == null) {
      return width;
    }
    List chunks = item.getChunks();
    int numChunks = chunks.size();
    for (int i = 0; i < numChunks; i++) {
      Chunk chunk = (Chunk) chunks.get(i);
      Map chunkAttributes = chunk.getAttributes();
      if (chunkAttributes != null) {
        Object image = chunkAttributes.get(IMAGE_CHUNK_ATTR);
        if (image != null) {
          Object[] imgData = (Object[]) image; 
          Image img = (Image) imgData[0];
          width += img.getScaledWidth();
          continue;
        }
      }
      Font font = chunk.getFont();
      BaseFont bf = font.getBaseFont();
      String content = chunk.getContent();        
      if (bf != null) {
        float chunkWidth = bf.getWidthPoint(content, font.getSize());
        width += chunkWidth;
      }
    }
    return width;
  }

  /**
   * Gets the height of the given phrase.
   * @param item The phrase to determine the width for.
   * @return The calculated width.
   */
  protected float getPhraseHeight(Phrase item) {
    float height = 0.0F;
    List chunks = item.getChunks();
    int numChunks = chunks.size();
    for (int i = 0; i < numChunks; i++) {
      Chunk chunk = (Chunk) chunks.get(i);
      Map chunkAttributes = chunk.getAttributes();
      if (chunkAttributes != null) { 
        Object image = chunkAttributes.get(IMAGE_CHUNK_ATTR);
        if (image != null) {
          Object[] imgData = (Object[]) image; 
          Image img = (Image) imgData[0];
          height = Math.max(height, img.getScaledWidth());
          continue;
        }
      }
      Font font = chunk.getFont();
      height = Math.max(height, font.getSize());
    }
    return height;
  }

  /**
   * Gets the content of the given phrase.
   * @param item The phrase to get the content for.
   * @return The content of the given phrase.
   */
  protected String getPhraseContent(Phrase item) {
    StringBuffer contentBuffer = new StringBuffer();
    List chunks = item.getChunks();
    int numChunks = chunks.size();
    for (int i = 0; i < numChunks; i++) {
      Chunk chunk = (Chunk) chunks.get(i);
      Map chunkAttributes = chunk.getAttributes();
      if (chunkAttributes != null) { 
        Object image = chunkAttributes.get(IMAGE_CHUNK_ATTR);
        if (image != null) {
          continue;
        }
      }
      contentBuffer.append(chunk.getContent());
    }
    return contentBuffer.toString();
  }

  /**
   * Rotates the given phrase.
   * @param phrase The phrase to rotate.
   * @param rotate The rotation angle.
   * @return The rotated phrase.
   * @exception DocumentHandlerException Thrown in case something went wrong while rotating the phrase.
   */
  protected Phrase rotatePhrase(Phrase phrase, float rotate) throws DocumentHandlerException {
    try {
      PdfContentByte cb = ((PdfWriter) documentHandler.getDocumentWriter()).getDirectContent();
      cb.saveState();
      float phraseWidth = getPhraseWidth(phrase);
      float phraseHeight = getPhraseHeight(phrase);
      if ((phraseWidth == 0.0F) || (phraseHeight == 0.0F)) {
        return phrase;
      }
      PdfTemplate template = cb.createTemplate(phraseWidth, phraseHeight);
      if (backgroundColor != null) {
        template.setColorFill(backgroundColor);
        template.rectangle(0, 0, phraseWidth, phraseHeight);
        template.fill();
      }
      template.beginText();
      //TODO: Fix calculation of baseline offset!
      printPhrase(template, phrase, 0.0F, phraseHeight * 0.25F);
      template.endText();
      cb.restoreState(); 
      Image img = Image.getInstance(template);
      img.setRotation((float) Math.PI * rotate / 180);
      Chunk ch = new Chunk(img, 0.0F, 0.0F);
      Phrase localPhrase = new Phrase(ch);
      localPhrase.setLeading(img.getScaledHeight());
      return localPhrase;
    } catch (BadElementException ex) {
      throw new DocumentHandlerException(locator(), "Failed to rotate phrase: " + ex.getMessage(), ex);
    }
  }
  
  /**
   * Prints a phrase into the given template.
   * @param template The template to output the phrase to.
   * @param phrase The phrase to output.
   * @param xPos The horizontal position releative to the template. 
   * @param yPos The vertical position releative to the template.
   * @exception DocumentHandlerException Thrown in case something went wrong while rotating the phrase.
   */
  protected void printPhrase(PdfTemplate template, Phrase phrase, float xPos, float yPos)
    throws DocumentHandlerException {
    
    float localXPos = xPos;
    ArrayList chunks = phrase.getChunks();
    int numChunks = chunks.size();
    for (int i = 0; i < numChunks; i++) {
      Element elem = (Element) chunks.get(i);
      if (elem instanceof Chunk) {
        Chunk chunk = (Chunk) elem;
        Map chunkAttributes = chunk.getAttributes();
        if (chunkAttributes != null) { 
          Object image = chunkAttributes.get(IMAGE_CHUNK_ATTR);
          if (image != null) {
            Object[] imgData = (Object[]) image; 
            Image img = (Image) imgData[0];
            try {
              template.endText();
              template.addImage(img, img.getScaledWidth(), 0, 0, img.getScaledHeight(), localXPos, yPos);
              template.beginText();
            } catch (DocumentException ex) {
              throw new DocumentHandlerException(locator(), "Unable to handle image: " + ex.getMessage(), ex);
            }
            localXPos += img.getScaledWidth();
            continue;
          }
        }
        Font chunkFont = chunk.getFont();
        BaseFont bf = chunkFont.getBaseFont();
        Color color = chunkFont.getColor();
        if (color == null) {
          template.setColorStroke(Color.black);
          template.setColorFill(Color.black);
        } else {
          template.setColorStroke(color);
          template.setColorFill(color);
        }
        template.setFontAndSize(bf, chunkFont.getSize());
        template.setTextMatrix(localXPos, yPos);
        template.showText(chunk.getContent());
        String content = chunk.getContent();
        float chunkWidth = bf.getWidthPoint(content, chunkFont.getSize());
        localXPos += chunkWidth;
      } else if (elem instanceof Phrase) {
        printPhrase(template, (Phrase) elem, localXPos, yPos);
      }
    }
  }
 
  /**
   * Gets a well typed value for the given one.
   * @param text The text to parse the value from.
   * @param type The type name to cast to.
   * @return The correctly typed value.
   * @throws DocumentHandlerException In case the type conversion failed.
   */
  protected Object typeCastValue(String text, String type) throws DocumentHandlerException {
    try {
      if (type.equalsIgnoreCase(UjacTypes.TYPE_NAME_BOOLEAN)) {
        return new Boolean(documentHandler.evalBoolean(text));
      } else if (type.equalsIgnoreCase(UjacTypes.TYPE_NAME_INT)) {
        return new Integer(documentHandler.evalInt(text));
      } else if (type.equalsIgnoreCase(UjacTypes.TYPE_NAME_LONG)) {
        return new Long(documentHandler.evalLong(text));
      } else if (type.equalsIgnoreCase(UjacTypes.TYPE_NAME_FLOAT)) {
        return new Float(documentHandler.evalFloat(text));
      } else if (type.equalsIgnoreCase(UjacTypes.TYPE_NAME_DOUBLE)) {
        return new Double(documentHandler.evalDouble(text));
      } else if (type.equalsIgnoreCase(UjacTypes.TYPE_NAME_DATE)) {
        return documentHandler.evalDate(text);
      } else if (type.equalsIgnoreCase(UjacTypes.TYPE_NAME_TIME)) {
        return documentHandler.evalTime(text);
      } else if (type.equalsIgnoreCase(UjacTypes.TYPE_NAME_TIMESTAMP)) {
        return documentHandler.evalTimestamp(text);
      } else if (type.equalsIgnoreCase(UjacTypes.TYPE_NAME_OBJECT)) {
        return documentHandler.evalObject(text);
      }
      // if all else failes, getting the value as a String
      return documentHandler.evalString(text);
    } catch (ExpressionException ex) {
      throw new TagAttributeException(locator(), ex.getMessage(), ex);
    }
  }

  /**
   * Ensures the 'number of pages' template for the given font.
   * @param font The font to use for the number of pages output.
   * @param cb The PDF content byte.
   * @return The according template.
   */
  protected PdfTemplate ensureNumberOfPagesTemplate(DocumentFont font, PdfContentByte cb) {
    float numberOfPagesWidth = 0.0F; 

    Font numberOfPagesFont = font.getFont();
    BaseFont bf = numberOfPagesFont.getBaseFont();
    String expectedNumberOfPages = Integer.toString(documentHandler.getExpectedNumberOfPages());
    numberOfPagesWidth = bf.getWidthPoint(expectedNumberOfPages, numberOfPagesFont.getSize());

    Map numberOfPagesTemplates = (Map) documentHandler.getResource(NUMBER_OF_PAGES_TEMPLATES);
    if (numberOfPagesTemplates == null) {
      numberOfPagesTemplates = new HashMap();
      documentHandler.registerResource(NUMBER_OF_PAGES_TEMPLATES, numberOfPagesTemplates);
    }
    PdfTemplate numberOfPagesTemplate = (PdfTemplate) numberOfPagesTemplates.get(font);
    if (numberOfPagesTemplate == null) {
      float height = font.getSize();
      numberOfPagesTemplate = cb.createTemplate(numberOfPagesWidth, height + 1);
      numberOfPagesTemplates.put(font, numberOfPagesTemplate);
    }
    return numberOfPagesTemplate;
  }
 
  /**
   * Tells whether the given phrase is empty or not.
   * @param phrase The phrase to check.
   * @return true if the phrase contains only chunks with zero length, 
   *   is null or contains no chunks at all, else false. 
   */
  protected boolean isPhraseEmpty(Phrase phrase) {
    if (phrase == null) {
      return true;
    }
    int numChunks = phrase.size();
    
    for (int i = 0; i < numChunks; i++) {
      Object element = phrase.get(i);
      if (element instanceof Phrase) {
        if (!isPhraseEmpty((Phrase) element)) {
          return false;
        }
        continue;
      }
      Chunk chunk = (Chunk) element;
      if (chunk.getContent().length() > 0) {
        return false;
      }
    }
    return true;
  }
  
  /**
   * Gets the element container.
   * In case no element container has been set so far, it 
   * obtains the root element container from the DocumentHandler.
   * @return The current element container.
   */
  public CommonAttributesHolder getCommonAttributesHolder() {
    return commonAttributesHolder;
  }
}


