/*
 * Copyright (C) 2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

/**
 * Name: ChildDefinitionMap<br>
 * Description: An optimized container for ChildDefinitionMap instances.
 * 
 * @author lauerc
 */
public class ChildDefinitionMap extends HashMap {

  /** The serial version UID. */
  private static final long serialVersionUID = -2184710357984808726L;

  /**
   * Constructs a SupportedChildMap instance with no specific attributes.
   */
  public ChildDefinitionMap() {
    super();
  }

  /**
   * Constructs a SupportedChildMap instance with specific attributes.
   * @param childDef The first child definition to be added to the map.
   * @see #addDefinition(ChildDefinition) 
   */
  public ChildDefinitionMap(ChildDefinition childDef) {
    super();
    addDefinition(childDef);
  }

  /**
   * Adds the given AttributeDefinition instance to the map by using the instances 
   * property 'name' as key.
   * @param childDef The child definition to be added to the map. 
   * @return The container itself.
   */
  public ChildDefinitionMap addDefinition(ChildDefinition childDef) {
    if (containsKey(childDef.getType())) {
      throw new RuntimeException("Key " + childDef.getType().getName() + " already exists");
    }
    put(childDef.getType(), childDef);
    return this;
  }

  /**
   * Removes the given AttributeDefinition instance from the map by using the instances 
   * property 'name' as key.
   * @param childDef The child definition to be removed from to the map. 
   * @return The container itself.
   */
  public ChildDefinitionMap removeDefinition(ChildDefinition childDef) {
    remove(childDef.getType());
    return this;
  }

  
  /**
   * Constructs a AttributeDefinitionMap instance with specific attributes.
   * @param childDefMap The child definition map to copy.
   * @see #addDefinition(AttributeDefinition) 
   */
  public ChildDefinitionMap(ChildDefinitionMap childDefMap) {
    super();
    this.putAll(childDefMap);
  }
  
  /**
   * Tells whether or not the given tag class is supported.
   * @param tag The tag to be checked.
   * @return True if the tag is supported, else false.
   */
  public ChildDefinition getCompatibleDefinition(BaseDocumentTag tag) {
    ChildDefinition cd = (ChildDefinition) this.get(tag.getClass());
    if (cd != null) {
      return cd;
    }
    Collection childDefinitions = this.values();
    Iterator cdIter = childDefinitions.iterator();
    while (cdIter.hasNext()) {
      cd = (ChildDefinition) cdIter.next();
      if (cd.getType().isInstance(tag)) {
        return cd;
      }
    }
    // not compatible with any supported type!
    return null;
  }
  
  /**
   * Gets a sorted list of attribute names including all attribute aliases.
   * @return The sorted attribute name list.
   */
  public List getSortedChildNames() {
    List childNames = new ArrayList(this.keySet());
    Collections.sort(childNames, new Comparator() {
      public int compare(Object arg0, Object arg1) {
        String a1 = ((Class) arg0).getName();
        String a2 = ((Class) arg1).getName();
        return a1.compareTo(a2);
      }
    });
    return childNames;
  }
  
  /**
   * Gets a list of all attributes, sorted by their primary names.
   * @return The sorted attribute list.
   */
  public List getSortedChilds() {
    List childs = new ArrayList(this.values());
    Collections.sort(childs, new Comparator() {
      public int compare(Object arg0, Object arg1) {
        ChildDefinition a1 = (ChildDefinition) arg0;
        ChildDefinition a2 = (ChildDefinition) arg1;
        return a1.getClass().getName().compareTo(a2.getClass().getName());
      }
    });
    return childs;
  }

}
