/*
 * Copyright (C) 2005 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
package org.ujac.print;

import java.awt.Color;

import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

import org.ujac.util.BeanUtils;

import org.ujac.util.exi.ExpressionException;

import org.ujac.util.template.TemplateException;

import org.xml.sax.Locator;

import com.lowagie.text.Element;
import com.lowagie.text.Rectangle;

/**
 * Name: DefaultAttributeParser<br>
 * Description: Default AttributeHandler implementation. <br>
 * 
 * @author lauerc
 */
public class DefaultAttributeHandler implements AttributeHandler {

  /** A map holding the most common colors. */
  private static final Map colorMap = new HashMap();
  
  static {
    // filling the color map
    colorMap.put("null", null);
    colorMap.put("NULL", null);
    colorMap.put("none", null);
    colorMap.put("NONE", null);
    colorMap.put("black", Color.black);
    colorMap.put("BLACK", Color.black);
    colorMap.put("blue", Color.blue);
    colorMap.put("BLUE", Color.blue);
    colorMap.put("cyan", Color.cyan);
    colorMap.put("CYAN", Color.cyan);
    colorMap.put("darkGray", Color.darkGray);
    colorMap.put("DARK_GRAY", Color.darkGray);
    colorMap.put("gray", Color.gray);
    colorMap.put("GRAY", Color.gray);
    colorMap.put("green", Color.green);
    colorMap.put("GREEN", Color.green);
    colorMap.put("lightGray", Color.lightGray);
    colorMap.put("LIGHT_GRAY", Color.lightGray);
    colorMap.put("magenta", Color.magenta);
    colorMap.put("MAGENTA", Color.magenta);
    colorMap.put("orange", Color.orange);
    colorMap.put("ORANGE", Color.orange);
    colorMap.put("pink", Color.pink);
    colorMap.put("PINK", Color.pink);
    colorMap.put("red", Color.red);
    colorMap.put("RED", Color.red);
    colorMap.put("white", Color.white);
    colorMap.put("WHITE", Color.white);
    colorMap.put("yellow", Color.yellow);
    colorMap.put("YELLOW", Color.yellow);
  }
  
  /** The document handler. */
  private DocumentHandler documentHandler = null;
 
  /** The text attribute handler. */
  private TypedAttributeHandler textHandler = null;
  /** The escape sequence text attribute handler. */
  private TypedAttributeHandler escapeSequenceTextHandler = null;
  /** The integer attribute handler. */
  private TypedAttributeHandler integerHandler = null;
  /** The integer attribute handler. */
  public TypedAttributeHandler floatHandler = null;
  /** The integer attribute handler. */
  public TypedAttributeHandler dimensionHandler = null;
  /**  The double attribute handler. */
  public TypedAttributeHandler doubleHandler = null;
  /** The boolean attribute handler. */
  public TypedAttributeHandler booleanHandler = null;
  /** The color attribute handler. */
  public TypedAttributeHandler colorHandler = null;
  /** The boolean attribute handler. */
  public TypedAttributeHandler borderHandler = null;
  /** The horizontal alignment attribute handler. */
  public TypedAttributeHandler horizontalAlignmentHandler = null;
  /** The vertical alignment attribute handler. */
  public TypedAttributeHandler verticalAlignmentHandler = null;

  /**
   * Constructs a DefaultAttributeHandler instance with no specific attributes.
   */
  public DefaultAttributeHandler() {
    textHandler = new TextAttributeHandler(this);
    escapeSequenceTextHandler = new EscapeSequenceTextAttributeHandler(this);
    integerHandler = new IntegerAttributeHandler(this);
    floatHandler = new FloatAttributeHandler(this);
    dimensionHandler = new DimensionAttributeHandler(this);
    doubleHandler = new DoubleAttributeHandler(this);
    booleanHandler = new BooleanAttributeHandler(this);
    colorHandler = new ColorAttributeHandler(this);
    borderHandler = new BorderAttributeHandler(this);
    horizontalAlignmentHandler = new HorizontalAlignmentAttributeHandler(this);
    verticalAlignmentHandler = new VerticalAlignmentAttributeHandler(this);
  }
  
  /**
   * Sets the document handler.
   * @param documentHandler The document handler to set.
   */
  public void setDocumentHandler(DocumentHandler documentHandler) {
    this.documentHandler = documentHandler;
  }

  /**
   * Casts the given object value into a the correct type for the specified
   * attribute definition.
   * @param locator The source locator.
   * @param attrDef The attribute definition.
   * @param attrName The attribute name.
   * @param value The value to convert.
   * @return The converted value.
   * @throws DocumentHandlerException
   *           In case the type conversion failed.
   */
  public Object typeCastAttributeValue(Locator locator, AttributeDefinition attrDef, 
                                       String attrName, Object value)
    throws DocumentHandlerException {
    
    Object localValue = null;
    switch (attrDef.getType()) {
      case AttributeDefinition.TYPE_INT :
        localValue = new Integer(parseIntAttribute(locator, attrName, value, true));
        break;
      case AttributeDefinition.TYPE_FLOAT :
        localValue = new Float(parseFloatAttribute(locator, attrName, value, true));
        break;
      case AttributeDefinition.TYPE_DIMENSION :
        localValue = new Float(parseDimensionAttribute(locator, attrName, value, true));
        break;
      case AttributeDefinition.TYPE_BOOLEAN :
        localValue = new Boolean(parseBooleanAttribute(locator, attrName, value, true));
        break;
      case AttributeDefinition.TYPE_COLOR :
        localValue = parseColorAttribute(locator, attrName, value, true);
        break;
      case AttributeDefinition.TYPE_BORDER :
        localValue = new Integer(parseBorderAttribute(locator, attrName, value, true));
        break;
      case AttributeDefinition.TYPE_HORIZONTAL_ALIGNMENT :
        localValue = new Integer(parseHorizontalAlignmentAttribute(locator, attrName, value, true));
        break;
      case AttributeDefinition.TYPE_VERTICAL_ALIGNMENT :
        localValue = new Integer(parseVerticalAlignmentAttribute(locator, attrName, value, true));
        break;
      default :
        localValue = value.toString();
        break;
    }
    return localValue;
  }
  
  /**
   * Gets the value for the given attribute's name. Works like method
   * <code>getStringAttribute(java.lang.String, java.lang.String, boolean)</code>
   * but takes care of escape sequences.
   * 
   * @param locator
   *          The tag source locator.
   * @param attrName
   *          The attribute's name.
   * @param attrValue
   *          The argument value.
   * @param evaluate
   *          If this flag is set to true, the attribute will get evaluated.
   * @param translateEscapeSequences
   *          Tells whether or not to translate escape sequences.
   * @return The determined value.
   * @exception TagAttributeException
   *              if the item contains an invalid value or an required attribute
   *              was not defined.
   */
  public String parseTextAttribute(Locator locator, String attrName,
                                   Object attrValue, boolean evaluate, 
                                   boolean translateEscapeSequences)
    throws TagAttributeException {
    
    if (attrValue == null) {
      return null;
    }
    String strValue = attrValue.toString();
    if (evaluate) {
      try {
        strValue = documentHandler.executeTemplate(strValue);
      } catch (TemplateException ex) {
        throw new TagAttributeException(locator,
            "Failed to evaluate expression " + attrValue + " for Attribute "
                + attrName + ".", ex);
      }
    }
    if (!translateEscapeSequences) {
      return strValue;
    }
    StringBuffer resultBuf = new StringBuffer();
    try {
      processEscapeSequences(locator, attrName, strValue, resultBuf, false);
    } catch (TagAttributeException ex) {
      throw ex;
    } catch (DocumentHandlerException ex) {
      throw new TagAttributeException(ex.getLocator(), ex.getMessage(), ex);
    }
    return resultBuf.toString();
  }
  
  /**
   * Parses an integer value from the given string.
   * 
   * @param locator
   *          The tag source locator.
   * @param attrName
   *          The attribute's name.
   * @param attrValue
   *          The argument value.
   * @param evaluate
   *          If this flag is set to true, the attribute will get evaluated.
   * @return The determined value.
   * @exception TagAttributeException
   *              Thrown in case the item contained an invalid value.
   */
  public int parseIntAttribute(Locator locator, String attrName,
      Object attrValue, boolean evaluate) throws TagAttributeException {
    if (attrValue == null) {
      return 0;
    }
    if (attrValue instanceof Number) {
      return ((Number) attrValue).intValue();
    }
    if (!evaluate) {
      try {
        return Integer.parseInt(attrValue.toString());
      } catch (NumberFormatException ex) {
        throw new TagAttributeException(locator, "Attribute " + attrName
            + " must be a valid float value (was '" + attrValue + "').", ex);
      }
    }
    try {
      return documentHandler.evalInt(attrValue.toString());
    } catch (ExpressionException ex) {
      throw new TagAttributeException(locator, "Attribute " + attrName
          + " must be a valid float value (was '" + attrValue + "').", ex);
    }
  }
  
  /**
   * Parses a float value from the given string.
   * 
   * @param locator
   *          The tag source locator.
   * @param attrName
   *          The attribute's name.
   * @param attrValue
   *          The argument value.
   * @param evaluate
   *          If this flag is set to true, the attribute will get evaluated.
   * @return The determined value.
   * @exception TagAttributeException
   *              Thrown in case the item contained an invalid value.
   */
  public float parseFloatAttribute(Locator locator, String attrName,
      Object attrValue, boolean evaluate) throws TagAttributeException {
    if (attrValue == null) {
      return 0.0F;
    }
    if (attrValue instanceof Number) {
      return ((Number) attrValue).floatValue();
    }
    if (!evaluate) {
      try {
        return Float.parseFloat(attrValue.toString());
      } catch (NumberFormatException ex) {
        throw new TagAttributeException(locator, "Attribute " + attrName
            + " must be a valid float value (was '" + attrValue + "').", ex);
      }
    }
    try {
      return documentHandler.evalFloat(attrValue.toString());
    } catch (ExpressionException ex) {
      throw new TagAttributeException(locator, "Attribute " + attrName
          + " must be a valid float value (was '" + attrValue + "').", ex);
    }
  }
  
  /**
   * Parses a dimension value from the given string.
   * 
   * @param locator
   *          The tag source locator.
   * @param attrName
   *          The attribute's name.
   * @param attrValue
   *          The argument value.
   * @param evaluate
   *          If this flag is set to true, the attribute will get evaluated.
   * @return The determined value.
   * @exception TagAttributeException
   *              Thrown in case the item contained an invalid value.
   */
  public float parseDimensionAttribute(Locator locator, String attrName,
      Object attrValue, boolean evaluate) throws TagAttributeException {
    if (attrValue == null) {
      return 0.0F;
    }
    if (attrValue instanceof Number) {
      return ((Number) attrValue).floatValue();
    }

    String strVal = attrValue.toString();
    if (evaluate) {
      try {
        strVal = documentHandler.evalString(strVal);
      } catch (ExpressionException ex) {
        throw new TagAttributeException(locator, "Attribute " + attrName + " must be a valid float value (was '" + attrValue + "').", ex);
      }
    }
    try {
      int ptIdx = strVal.indexOf("pt");
      if (ptIdx >= 0) {
        strVal = strVal.substring(0, ptIdx);
      }
      return Float.parseFloat(strVal);
    } catch (NumberFormatException ex) {
      throw new TagAttributeException(locator, "Attribute " + attrName + " must be a valid float value (was '" + attrValue + "').", ex);
    }
  }

  /**
   * Parses a double value from the given string.
   * 
   * @param locator
   *          The tag source locator.
   * @param attrName
   *          The attribute's name.
   * @param attrValue
   *          The argument value.
   * @param evaluate
   *          If this flag is set to true, the attribute will get evaluated.
   * @return The determined value.
   * @exception TagAttributeException
   *              Thrown in case the item contained an invalid value.
   */
  public double parseDoubleAttribute(Locator locator, String attrName,
      Object attrValue, boolean evaluate) throws TagAttributeException {
    if (attrValue == null) {
      return 0.0;
    }
    if (attrValue instanceof Number) {
      return ((Number) attrValue).doubleValue();
    }
    if (!evaluate) {
      try {
        return Double.parseDouble(attrValue.toString());
      } catch (NumberFormatException ex) {
        throw new TagAttributeException(locator, "Attribute " + attrName
            + " must be a valid double value (was '" + attrValue + "').", ex);
      }
    }
    try {
      return documentHandler.evalDouble(attrValue.toString());
    } catch (ExpressionException ex) {
      throw new TagAttributeException(locator, "Attribute " + attrName
          + " must be a valid double value (was '" + attrValue + "').", ex);
    }
  }
  
  /**
   * Gets the boolean value for the given attribute's name.
   * 
   * @param locator
   *          The tag source locator.
   * @param attrName
   *          The attribute's name.
   * @param attrValue
   *          The argument value.
   * @param evaluate
   *          If this flag is set to true, the attribute will get evaluated.
   * @return The determined value.
   * @exception TagAttributeException
   *              if the item contains an invalid value or an required attribute
   *              was not defined.
   */
  public boolean parseBooleanAttribute(Locator locator, String attrName,
      Object attrValue, boolean evaluate) throws TagAttributeException {
    if (attrValue == null) {
      return false;
    }
    if (attrValue instanceof Boolean) {
      return ((Boolean) attrValue).booleanValue();
    }
    if (!evaluate) {
      try {
        return new Boolean(attrValue.toString()).booleanValue();
      } catch (NumberFormatException ex) {
        throw new TagAttributeException(locator, "Attribute " + attrName
            + " must be a valid float value (was '" + attrValue + "').", ex);
      }
    }
    try {
      return documentHandler.evalBoolean(attrValue.toString());
    } catch (ExpressionException ex) {
      throw new TagAttributeException(locator, "Attribute " + attrName
          + " must be a valid float value (was '" + attrValue + "').", ex);
    }
  }
  
  /**
   * Parses the color value from the given attribute.
   * 
   * @param locator
   *          The tag source locator.
   * @param attrName
   *          The name of the attribute to parse.
   * @param attrValue
   *          The attribute value to parse.
   * @param evaluate
   *          If this flag is set to true, the attribute will get evaluated.
   * @return The resulting border style.
   * @exception TagAttributeException
   *              Thrown in case the item contained an invalid value.
   */
  public Color parseColorAttribute(Locator locator, String attrName,
      Object attrValue, boolean evaluate) throws TagAttributeException {
    if (attrValue == null) {
      return null;
    }
    if (attrValue instanceof Color) {
      return (Color) attrValue;
    }
    String colVal = attrValue.toString();
    if (evaluate) {
      try {
        colVal = documentHandler.evalString(colVal);
      } catch (ExpressionException ex) {
        throw new TagAttributeException(locator, "Attribute " + attrName
            + " must be a valid color value (was '" + attrValue + "').", ex);
      }
    }
    if (colVal == null) {
      return null;
    }
    // first probing the color registry ...
    Color colorValue = documentHandler.getColor(colVal);
    if (colorValue != null) {
      return colorValue;
    }
    // ... then giving the built in color map a try
    colorValue = (Color) colorMap.get(colVal);
    if (colorValue != null) {
      return colorValue;
    }
    // ... now it has to be an RGB value
    // Added by Kiran
    if (colVal.length() >= 6) {
      try {
        if (colVal.length() == 7) {
          colVal = colVal.substring(1);
        }
        return new Color(java.lang.Integer.parseInt(colVal, 16));
      } catch (NumberFormatException ex) {
        throw new TagAttributeException(locator, "The color value '" + colVal
            + "' has to define a registered color, a built in one "
            + "or a 6 digit RGB value.");
      }
    }
    return null;
  }
  
  /**
   * Parses the border value from the given attribute.
   * 
   * @param locator
   *          The tag source locator.
   * @param attrName
   *          The name of the attribute to parse.
   * @param attrValue
   *          The attribute value to parse.
   * @param evaluate
   *          If this flag is set to true, the attribute will get evaluated.
   * @return The resulting border style.
   * @exception TagAttributeException
   *              Thrown in case the item contained an invalid value.
   */
  public int parseBorderAttribute(Locator locator, String attrName,
      Object attrValue, boolean evaluate) throws TagAttributeException {
    if (attrValue == null) {
      return 0;
    }
    if (attrValue instanceof Number) {
      return ((Number) attrValue).intValue();
    }
    String strValue = attrValue.toString();
    if (evaluate) {
      try {
        strValue = documentHandler.evalString(attrValue.toString());
      } catch (ExpressionException ex) {
        throw new TagAttributeException(locator, "Attribute " + attrName
            + " must be a legal border value (was '" + attrValue + "').", ex);
      }
    }
    StringTokenizer tkn = new StringTokenizer(strValue, "|");
    int border = Rectangle.NO_BORDER;
    while (tkn.hasMoreTokens()) {
      String token = tkn.nextToken();
      if (token.equalsIgnoreCase("none")) {
        border = Rectangle.NO_BORDER;
        // OK, we can't get more than this!
        break;
      }
      if (token.equalsIgnoreCase("box")) {
        border = Rectangle.BOX;
        // OK, we can't get more than this!
        break;
      }
      if (token.equalsIgnoreCase("left")) {
        border |= Rectangle.LEFT;
        continue;
      }
      if (token.equalsIgnoreCase("right")) {
        border |= Rectangle.RIGHT;
        continue;
      }
      if (token.equalsIgnoreCase("top")) {
        border |= Rectangle.TOP;
        continue;
      }
      if (token.equalsIgnoreCase("bottom")) {
        border |= Rectangle.BOTTOM;
        continue;
      }
      throw new TagAttributeException(locator, "Attribute " + attrName
          + " must hold a list of border styles "
          + "(none, top, bottom, left, right, box), separated by '|' (was '"
          + token + "').");
    }
    return border;
  }
  
  /**
   * Parses the horizontal alignment value from the given attribute.
   * 
   * @param locator
   *          The tag source locator.
   * @param attrName
   *          The name of the attribute to parse.
   * @param attrValue
   *          The attribute value to parse.
   * @param evaluate
   *          If this flag is set to true, the attribute will get evaluated.
   * @return The resulting horizontal alignment.
   * @exception TagAttributeException
   *              Thrown in case the item contained an invalid value.
   */
  public int parseHorizontalAlignmentAttribute(Locator locator,
      String attrName, Object attrValue, boolean evaluate)
      throws TagAttributeException {
    if (attrValue == null) {
      return 0;
    }
    if (attrValue instanceof Number) {
      return ((Number) attrValue).intValue();
    }
    String strValue = attrValue.toString();
    if (evaluate) {
      try {
        strValue = documentHandler.evalString(attrValue.toString());
      } catch (ExpressionException ex) {
        throw new TagAttributeException(locator, "Attribute " + attrName
            + " must be a legal horizontal alignment value (was '" + attrValue
            + "').", ex);
      }
    }
    if (BeanUtils.isEmpty(strValue)) {
      return 0;
    }
    if (strValue.equalsIgnoreCase("left")) {
      return Element.ALIGN_LEFT;
    }
    if (strValue.equalsIgnoreCase("right")) {
      return Element.ALIGN_RIGHT;
    }
    if (strValue.equalsIgnoreCase("center")) {
      return Element.ALIGN_CENTER;
    }
    if (strValue.equalsIgnoreCase("justified")) {
      return Element.ALIGN_JUSTIFIED;
    }
    if (strValue.equalsIgnoreCase("justified-all")) {
      return Element.ALIGN_JUSTIFIED_ALL;
    }
    throw new TagAttributeException(
        locator,
        "Invalid value for attribute "
            + attrName
            + ": '"
            + attrValue
            + "'. Allowed values are: left, right, center, justified, justified-all.");
  }
  
  /**
   * Parses the vertical alignment value from the given attribute.
   * 
   * @param locator
   *          The tag source locator.
   * @param attrName
   *          The name of the attribute to parse.
   * @param attrValue
   *          The attribute value to parse.
   * @param evaluate
   *          If this flag is set to true, the attribute will get evaluated.
   * @return The resulting vertical alignment.
   * @exception TagAttributeException
   *              Thrown in case the item contained an invalid value.
   */
  public int parseVerticalAlignmentAttribute(Locator locator, String attrName,
      Object attrValue, boolean evaluate) throws TagAttributeException {
    if (attrValue == null) {
      return 0;
    }
    if (attrValue instanceof Number) {
      return ((Number) attrValue).intValue();
    }
    String strValue = attrValue.toString();
    if (evaluate) {
      try {
        strValue = documentHandler.evalString(strValue);
      } catch (ExpressionException ex) {
        throw new TagAttributeException(locator, "Attribute " + attrName
            + " must be a legal vertical alignment value (was '" + attrValue
            + "').", ex);
      }
    }
    if (BeanUtils.isEmpty(strValue)) {
      return 0;
    }
    if (strValue.equalsIgnoreCase("top")) {
      return Element.ALIGN_TOP;
    }
    if (strValue.equalsIgnoreCase("bottom")) {
      return Element.ALIGN_BOTTOM;
    }
    if (strValue.equalsIgnoreCase("middle")) {
      return Element.ALIGN_MIDDLE;
    }
    if (strValue.equalsIgnoreCase("baseline")) {
      return Element.ALIGN_BASELINE;
    }
    throw new TagAttributeException(locator, "Invalid value for attribute "
        + attrName + ": '" + attrValue
        + "'. Allowed values are: top, bottom, middle, baseline.");
  }
  
  /**
   * Processes the escape sequences from the given content string and writes the
   * result into the given buffer.
   * 
   * @param locator
   *          The tag source locator.
   * @param attrName
   *          The attribute name.
   * @param contents
   *          The contents to parse for escape sequences
   * @param buffer
   *          The buffer to write to.
   * @param trimHead
   *          Tells whether or not to trim the leading white spaces
   * @throws DocumentHandlerException
   */
  protected void processEscapeSequences(Locator locator, String attrName, String contents, 
                                        StringBuffer buffer, boolean trimHead)
      throws DocumentHandlerException {
    // decoding explicite white spaces
    int textLen = contents.length();
    for (int i = 0; i < textLen; i++) {
      char c = contents.charAt(i);
      if (c == '\\') {
        // processing escape sequences
        ++i;
        if (i >= textLen) {
          throw new DocumentHandlerException(locator,
              "Syntax expression: empty escape sequence at chunk '" + contents
                  + "'.");
        }
        c = contents.charAt(i);
        switch (c) {
          case 'n' :
            buffer.append('\n');
            // removing all following white spaces
            for (++i; i < textLen; i++) {
              if (!Character.isWhitespace(contents.charAt(i))) {
                --i;
                break;
              }
            }
            break;
          case 'u' :
            // unicode character detected ...
            String code = contents.substring(i + 1, i + 5);
            int unicodeValue = Integer.parseInt(code, 16);
            i += 4;
            buffer.append((char) unicodeValue);
            break;
          case ' ' :
            buffer.append(" ");
            break;
          case 't' :
            buffer.append("    ");
            break;
          case '\\' :
            buffer.append("\\");
            break;
          case '{' :
            buffer.append("{");
            break;
          case '}' :
            buffer.append("}");
            break;
          default :
            throw new DocumentHandlerException(locator,
                "Syntax expression: unknown escape sequence '\\" + c
                    + "' at chunk '" + contents + "'.");
        }
      } else {
        // handling white spaces
        if (Character.isWhitespace(c)) {
          // removing all leading white spaces in case 'trimHead' is switched on
          // also removing all extra white spaces
          if ((trimHead && (buffer.length() == 0))
              || ((buffer.length() > 0) && Character.isWhitespace(buffer
                  .charAt(buffer.length() - 1)))) {
            continue;
          }
        }
        // character seems to be a regular one
        buffer.append(c);
      }
    }
  }
  
  /**
   * Gets the integer attribute handler.
   * @return The integer attribute handler
   */
  public TypedAttributeHandler getTextHandler() {
    return textHandler;
  }
  /**
   * Gets the integer attribute handler.
   * @return The integer attribute handler
   */
  public TypedAttributeHandler getEscapeSequenceTextHandler() {
    return escapeSequenceTextHandler;
  }
  /**
   * Gets the integer attribute handler.
   * @return The integer attribute handler
   */
  public TypedAttributeHandler getIntegerHandler() {
    return integerHandler;
  }
  /**
   * Gets the float attribute handler.
   * @return The integer attribute handler
   */
  public TypedAttributeHandler getFloatHandler() {
    return floatHandler;
  }
  /**
   * Gets the dimension attribute handler.
   * @return The integer attribute handler
   */
  public TypedAttributeHandler getDimensionHandler() {
    return dimensionHandler;
  }
  /**
   * Gets the double attribute handler.
   * @return The double attribute handler
   */
  public TypedAttributeHandler getDoubleHandler() {
    return doubleHandler;
  }
  /**
   * Gets the boolean attribute handler.
   * @return The boolean attribute handler
   */
  public TypedAttributeHandler getBooleanHandler() {
    return booleanHandler;
  }
  /**
   * Gets the color attribute handler.
   * @return The color attribute handler
   */
  public TypedAttributeHandler getColorHandler() {
    return colorHandler;
  }
  /**
   * Gets the boolean attribute handler.
   * @return The boolean attribute handler
   */
  public TypedAttributeHandler getBorderHandler() {
    return borderHandler;
  }
  /**
   * Gets the horizontal alignment attribute handler.
   * @return The horizontal alignment attribute handler
   */
  public TypedAttributeHandler getHorizontalAlignmentHandler() {
    return horizontalAlignmentHandler;
  }
  /**
   * Gets the vertical alignment attribute handler.
   * @return The vertical alignment attribute handler
   */
  public TypedAttributeHandler getVerticalAlignmentHandler() {
    return verticalAlignmentHandler;    
  }

}
