/*
 * Copyright (C) 2005 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print;

/**
 * Name: DefaultContentFilter<br>
 * Description: Default ContentFilter implementation.
 * 
 * @author lauerc
 */
public class DefaultContentFilter implements ContentFilter {
  
  /**
   * @see org.ujac.print.ContentFilter#filterContents(java.lang.String, int, java.lang.StringBuffer)
   */
  public void filterContents(BaseDocumentTag tag, String contents, int offset, StringBuffer outputBuffer)
    throws DocumentHandlerException {
    
    boolean translateEscapeSequences = tag.getDocumentHandler().isTranslateEscapeSequences();
    int textLen = contents.length();
    int numConnectedWhiteSpaces = 0;
    for (int i = offset; i < textLen; i++) {
      char c = contents.charAt(i);
      if (translateEscapeSequences && (c == '\\')) {
        // processing escape sequences
        i = processEscapeSequence(tag, contents, i, outputBuffer);
        continue;
      }
          
      if (Character.isWhitespace(c)) {
        ++numConnectedWhiteSpaces;
        if (numConnectedWhiteSpaces > 1) {
          continue;
        }
      } else {
        numConnectedWhiteSpaces = 0;
      }
      // character seems to be a regular one
      outputBuffer.append(c);
    }
  }
  
  /**
   * Processes the escape sequence at offset position in the given contents string and
   * prints the translated characters to the given output buffer. 
   * @param tag The tag for which to filter the contents.
   * @param contents The contents to filter.
   * @param offset The offset where to start filtering the content string.
   * @param outputBuffer The buffer holding the content processing result.
   * @return The offset after the terminated escape sequence.
   * @exception DocumentHandlerException In case the escape sequence could not be processed.
   */
  protected int processEscapeSequence(BaseDocumentTag tag, String contents, 
                                      int offset, StringBuffer outputBuffer)
    throws DocumentHandlerException {

    // decoding explicite white spaces
    int textLen = contents.length();
    // processing escape sequences
    int localOffset = offset + 1;
    if (localOffset >= textLen) {
      throw new DocumentHandlerException(tag.locator(), "Syntax expression: empty escape sequence at chunk '" + contents + "'.");
    }
    char c = contents.charAt(localOffset);
    switch (c) {
      case 'n':
        outputBuffer.append('\n');
        // removing all following white spaces
        for (++localOffset; localOffset < textLen; localOffset++) {
          if (!Character.isWhitespace(contents.charAt(localOffset))) {
            --localOffset;
            break;
          }
        }
        return localOffset;

      case 'u':
        // unicode character detected ...
        String code = contents.substring(localOffset + 1, localOffset + 5);
        int unicodeValue = Integer.parseInt(code, 16);
        localOffset += 4;
        outputBuffer.append((char) unicodeValue);
        return localOffset;

      case ' ':
        outputBuffer.append(" ");
        return localOffset;

      case 't':
        outputBuffer.append("    ");
        return localOffset;

      case '\\':
        outputBuffer.append("\\");
        return localOffset;

      case '{':
        outputBuffer.append("{");
        return localOffset;

      case '}':
        outputBuffer.append("}");
        return localOffset;

      default:
        throw new DocumentHandlerException(
            tag.locator(), "Syntax expression: unknown escape sequence '\\" + 
            c + "' at chunk '" + contents + "'."
        );
    }
  }
  
}
