/*
 * Copyright (C) 2003 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.xml.sax.Locator;

/**
 * Name: DefaultTagBuilder<br>
 * Description: The default TagBuilder implementation.<br/>
 *   builds a document tag by simply creating an instance of the 
 *   specified class, which has to be a sub class of BaseDocumentTag
 *   of course.
 * 
 * @author lauerc
 */
public class DefaultTagBuilder implements TagBuilder {

  /** The tag name. */
  private String name;
  /** The tag handler class. */
  private Class clazz;
  /** The alias set. */
  private Set aliases = null;
  
  /**
   * Constructs a DefaultTagBuilder instance with specific attributes.
   * @param name The name of the tag.
   * @param clazz The class of the tag for which to build instances.
   */
  public DefaultTagBuilder(String name, Class clazz) {
    this.name = name;
    this.clazz = clazz;
  }
  
  /**
   * @see org.ujac.print.TagBuilder#getName()
   */
  public String getName() {
    return name;
  }
  
  /**
   * @see org.ujac.print.TagBuilder#createTag(DocumentHandler)
   */
  public BaseDocumentTag createTag(DocumentHandler documentHandler) throws TagInstantiationException {
    Locator locator = null;
    if (documentHandler != null) {
      locator = documentHandler.locator();
    }
    try {
      return (BaseDocumentTag) clazz.newInstance();
    } catch (IllegalAccessException ex) {
      throw new TagInstantiationException(locator, "The tag type '" + name + "' is not allowed to be accessed: " + ex.getMessage(), ex);
    } catch (InstantiationException ex) {
      throw new TagInstantiationException(locator, "The tag type '" + name + "' cannot get instanciated: " + ex.getMessage(), ex);
    }
  }
  
  
  /**
   * Registers an alias for this tag.
   * @param alias The alias to register.
   */
  public void registerAlias(String alias) {
    if (aliases == null) {
      aliases = new HashSet();
    }
    aliases.add(alias);
  }

  /**
   * Unregisters an alias for this tag.
   * @param alias The alias to unregister.
   */
  public void unregisterAlias(String alias) {
    if (aliases == null) {
      return;
    }
    aliases.remove(alias);
  }

  /**
   * Gets all registered aliases.
   * @return The registered aliases.
   */
  public Collection getAliases() {
    return aliases;
  }

}
