/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print;

import java.awt.Color;

import org.ujac.util.BeanUtils;

import com.lowagie.text.Font;

/**
 * Name: DocumentFont<br>
 * Description: A class holding font informations.
 * 
 * @author lauerc
 */
public class DocumentFont {

  /** The font itself. */
  private Font font = null;
  /** The font family. */
  private String family;
  /** The font encoding. */
  private String encoding;
  /** The font size. */
  private float size;
  /** The font style. */
  private int style;
  /**
   * The text displacement relative to the baseline. 
   *  Positive values rise the text, negative values lower the text.
   */
  private float textRise = 0.0F;
  /** The font color. */
  private Color color;
  
  /**
   * Constructs a DocumentFont instance with specific attributes.
   * @param font The font itself.
   * @param family The font family.
   * @param encoding The font encoding.
   * @param size The font size.
   * @param style The font style.
   * @param color The font color.
   */
  public DocumentFont(Font font, String family, String encoding, float size, int style, Color color) {
    this.font = font;
    this.family = family;
    this.encoding = encoding;
    this.size = size;
    this.style = style;
    this.color = color;
   }
  
  /**
   * Constructs a DocumentFont instance with specific attributes.
   * @param font The font itself.
   * @param family The font family.
   * @param encoding The font encoding.
   * @param size The font size.
   * @param style The font style.
   * @param textRise The text displacement relative to the baseline. 
   *    Positive values rise the text, negative values lower the text.
   * @param color The font color.
   */
  public DocumentFont(Font font, String family, String encoding, float size, int style, 
                      float textRise, Color color) {
    this.font = font;
    this.family = family;
    this.encoding = encoding;
    this.size = size;
    this.style = style;
    this.textRise = textRise;
    this.color = color;
   }

  /**
   * Getter method for the the property color.
   * @return The current value of property color.
   */
  public Color getColor() {
    return color;
  }

  /**
   * Getter method for the the property family.
   * @return The current value of property family.
   */
  public String getFamily() {
    return family;
  }


  /**
   * Getter method for the the property encoding.
   * @return The current value of property encoding.
   */
  public String getEncoding() {
    return encoding;
  }

  /**
   * Getter method for the the property font.
   * @return The current value of property font.
   */
  public Font getFont() {
    return font;
  }

  /**
   * Getter method for the the property size.
   * @return The current value of property size.
   */
  public float getSize() {
    return size;
  }

  /**
   * Getter method for the the property style.
   * @return The current value of property style.
   */
  public int getStyle() {
    return style;
  }

  /**
   * Getter method for the the property textRise.
   * @return The current value of property textRise.
   */
  public float getTextRise() {
    return textRise;
  }

  
  /**
   * Checks if the font is declared as bold.
   * @return true if the font is declared as bold, else false.
   */
  public boolean isBold() {
    return (style & Font.BOLD) != 0; 
  }

  /**
   * Checks if the font is declared as italic.
   * @return true if the font is declared as italic, else false.
   */
  public boolean isItalic() {
    return (style & Font.ITALIC) != 0; 
  }
 
  /**
   * Checks if the font is declared as underline.
   * @return true if the font is declared as underline, else false.
   */
  public boolean isUnderline() {
    return (style & Font.UNDERLINE) != 0; 
  }
 
  /**
   * Checks if the font is declared as strikethru.
   * @return true if the font is declared as strikethru, else false.
   */
  public boolean isStrikethru() {
    return (style & Font.STRIKETHRU) != 0; 
  }
  
  /**
   * @see java.lang.Object#equals(java.lang.Object)
   */
  public boolean equals(Object obj) {
    if (obj == null) {
      return false;
    }
    if (!(obj instanceof DocumentFont)) {
      return false;
    }
    DocumentFont other = (DocumentFont) obj;
    boolean familiesEqual = BeanUtils.equals(family, other.family);
    boolean colorsEqual = BeanUtils.equals(color, other.color);
    return familiesEqual && (size == other.size) && 
      (style == other.style) && colorsEqual;  
  }

  
  /**
   * @see java.lang.Object#hashCode()
   */
  public int hashCode() {
    int colorHash = 0;
    if (color != null) {
      colorHash = color.hashCode() * 23;
    }
    return family.hashCode() + (int) (size * 113) + style * 17 + colorHash;
  }

}
