/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print;

import java.awt.Color;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import java.sql.Time;
import java.sql.Timestamp;
import java.text.Format;

import java.util.Map;
import java.util.Date;
import java.util.List;
import java.util.HashMap;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.StringTokenizer;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.logging.impl.SimpleLog;

import org.ujac.print.tag.MacroDefTag;

import org.ujac.util.template.DefaultTemplateInterpreterFactory;
import org.ujac.util.template.TemplateContext;
import org.ujac.util.template.TemplateException;
import org.ujac.util.template.TemplateInterpreter;


import org.ujac.util.exi.ExpressionException;
import org.ujac.util.io.ResourceLoader;

import org.ujac.util.text.FormatHelper;

import org.xml.sax.Locator;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.LocatorImpl;

import com.lowagie.text.Document;
import com.lowagie.text.DocWriter;
import com.lowagie.text.DocumentException;
import com.lowagie.text.Font;
import com.lowagie.text.FontFactory;
import com.lowagie.text.Image;
import com.lowagie.text.Paragraph;
import com.lowagie.text.Rectangle;

import com.lowagie.text.pdf.BaseFont;
import com.lowagie.text.pdf.FontMapper;
import com.lowagie.text.pdf.PdfDestination;
import com.lowagie.text.pdf.PdfDictionary;
import com.lowagie.text.pdf.PdfName;
import com.lowagie.text.pdf.PdfPageLabels;
import com.lowagie.text.pdf.PdfString;
import com.lowagie.text.pdf.PdfTemplate;
import com.lowagie.text.pdf.PdfWriter;
import com.lowagie.text.pdf.PdfPageEvent;
import com.lowagie.text.pdf.PdfPageEventHelper;
import com.lowagie.text.xml.xmp.XmpWriter;

/**
 * Name: DocumentHandler<br>
 * Description: XML content handler, managing core tasks of the XML to iText processing framework.<br>
 * 
 * @author lauerc
 */
public class DocumentHandler extends DefaultHandler implements ColorRegistry {
  
  /** The begin of a CDATA section. */
  private static final String CDATA_BEGIN = "<![CDATA[";
  
  /** The logger. */
  private Log log = null;
  /** The tag factory. */
  private DocumentTagFactory tagFactory = null;
  /** The XML Reader class to use. */
  private String xmlReaderClass = null;

  /** The document stack. */
  private List documentStack = new ArrayList();
  /** The top document elements. */
  private Map topDocumentElements = new HashMap();
  /** The top document elements. */
  private BaseDocumentTag topRealTag = null;
  /** The event recorder stack. */
  private List eventRecorderStack = new ArrayList();
  /** The top event recorder stack. */
  private DocumentEventRecorder topEventRecorder = null;
  
  /** The current document. */
  private Document document = null;
  /** The stream where to write the document to. */
  private OutputStream outputStream = null;
  /** The template interpreter. */
  private TemplateInterpreter templateInterpreter = null;
  /** The template context. */
  private TemplateContext templateContext = null;
  /** The document writer. */
  private DocWriter documentWriter = null;
  /** The document locator. */
  private Locator locator = null;
  /** The font map. */
  private Map fontMap = new HashMap();
  /** The last item. */
  private BaseDocumentTag lastItem = null;
  /** The event handler. */
  private PageEventHandler pageEventHandler = new PageEventHandler();
  /** The resource loader. */
  private ResourceLoader resourceLoader = null;
  /** The macro map. */
  private Map macroDefinitionMap = new HashMap();
  
  /** The style registry. */
  private Map styleRegistry = new HashMap();
  /** The color registry. */
  private Map colorRegistry = new HashMap();
  /** The true type font registry. */
  private Map fontRegistry = new HashMap();

  /** The document handler component registry. */
  private DocumentHandlerComponentRegistry documentHandlerComponentRegistry = DocumentHandlerComponentRegistry.getInstance();

  /** The font mapper. */
  private FontMapper fontMapper = null;
  /** Holds the table of contents items. */
  private List tocItems = null;
  /** Holds the table of contents items. */
  private Map tocItemMap = new HashMap();
  /** Holds the outline items. */
  private List outlineItems = null;
  /** Holds the currently unnumbered table of contents items. */
  private List unnumberedTocItems = null;
  /** The data accessor. */
  private DocumentDataAccessor dataAccessor = null;
  /** The document tag interceptor. */
  private DocumentTagInterceptor tagInterceptor = null;
  /** The content filter. */
  private ContentFilter contentFilter = null;
  /** The document resource registry. */
  private Map resourceRegistry = new HashMap();
  /** The current dynamic content container. */
  private DynamicContentContainer dynamicContentContainer = new DynamicContentContainer();
  
  /** Tells whether or not to translate escape sequences (defaults to true). */
  private boolean translateEscapeSequences = true;
  /** Tells whether or not to parse dyamic content for XML contents. */
  private boolean parseDynamicContent = true;
  /** Tells whether or not to evaluate expressions (defaults to true). */
  private boolean evaluateExpressions = true;
  /** Tells whether or not to cache images (defaults to true). */
  private boolean cacheImages = true;
  /** 
   * Tells whether or not to strictly check document structure (defaults to true).
   * In case this flag is set, any document structure violation will cause an exception. 
   */
  private boolean strictlyCheckDocumentStructure = true;
  /** The expected number of pages (needed to define the size for the number of pages template). */
  private int expectedNumberOfPages = 999;
  
  /** Tells whether or not to include the head section. */ 
  private boolean includeHeadSection = false;
  /** The writer to copy the incoming XML stream to. */ 
  private PrintWriter templateCopyWriter = null;
  /** The disabled writer to copy the incoming XML stream to. */ 
  private PrintWriter disabledTemplateCopyWriter = null;

  // Added by Kiran. This will set the PdfPageLabels for the generated PDF
  /** The Pdf Page Labels can be seen on the pages tab in acrobat. */
  protected PdfPageLabels pdfLabels = new PdfPageLabels();
  /** The document meta data. */
  private DocumentMetaData metaData = null;
  /** The XMP meta data (generated from attribute metaData). */
  private byte[] xmpMetaData = null;
  
  /** The style parser. */
  private StyleParser styleParser = null;
  /** The attribute handler. */
  private AttributeHandler attributeHandler = null;
  
  /** The logical logicalPageNumberStyle attribute for the generated PDF. */
  private int logicalPageNumberStyle = 0;  
  /** The logical logicalPageNumberText attribute for the generated PDF. */
  private String logicalPageNumberText = "";

  /** The page number offset. */
  private int pageNumberOffset = 0;
  /** Tells whether or not to freeze the toc. */
  private boolean tocFreezed = false;
  /** Tells whether or not to support destinations. */
  private boolean destinationsSupported = true;

  /**
   * Name: PageEventHandler<br>
   * Description: The page event handler class, intended to handler stuff 
   *              like headers and footers.
   */
  class PageEventHandler extends PdfPageEventHelper {

    /** The event handler map. */
    private Map eventHandlers = new HashMap();
    
    /**
     * Sets the given handler to the event handler map.
     * @param key The event handler key.
     * @param handler The event handler.
     */
    public void setHandler(Object key, PdfPageEvent handler) {
      eventHandlers.put(key, handler);
    } 
    
    /**
     * Removes the handler with the given name from the event handler map. 
     * @param key The key of the handler
     */
    public void removeHandler(Object key) {
      eventHandlers.remove(key);
    }
    
    /**
     * Removess all event handlers. 
     */
    public void clearHandlers() {
      eventHandlers.clear();
    }
    
    /**
     * @see com.lowagie.text.pdf.PdfPageEvent#onChapter(com.lowagie.text.pdf.PdfWriter, com.lowagie.text.Document, float, com.lowagie.text.Paragraph)
     */
    public void onChapter(PdfWriter writer, Document document, float paragraphPosition, Paragraph title) {
      flushTocChapter(writer, paragraphPosition);
      Iterator iter = eventHandlers.values().iterator();
      while (iter.hasNext()) {
        DocumentPageEventHandler evtHandler = (DocumentPageEventHandler) iter.next();
        evtHandler.onChapter(writer, document, paragraphPosition, title);
      }
    }

    /**
     * @see com.lowagie.text.pdf.PdfPageEvent#onChapterEnd(com.lowagie.text.pdf.PdfWriter, com.lowagie.text.Document, float)
     */
    public void onChapterEnd(PdfWriter writer, Document document, float position) {
      Iterator iter = eventHandlers.values().iterator();
      while (iter.hasNext()) {
        DocumentPageEventHandler evtHandler = (DocumentPageEventHandler) iter.next();
        evtHandler.onChapterEnd(writer, document, position);
      }
    }

    /**
     * @see com.lowagie.text.pdf.PdfPageEvent#onStartPage(com.lowagie.text.pdf.PdfWriter, com.lowagie.text.Document)
     */
    public void onStartPage(PdfWriter writer, Document document) {
      DocumentHandler.this.getTemplateContext().setProperty("pageNumber", new Integer(getPageNumber(writer)));
      // applying the current logical page number style
      applyCurrentLogicalPageNumberStyle();
      
      Iterator iter = eventHandlers.values().iterator();
      while (iter.hasNext()) {
        DocumentPageEventHandler evtHandler = (DocumentPageEventHandler) iter.next();
        evtHandler.onStartPage(writer, document);
      }
    }

    /**
     * @see com.lowagie.text.pdf.PdfPageEvent#onEndPage(com.lowagie.text.pdf.PdfWriter, com.lowagie.text.Document)
     */
    public void onEndPage(PdfWriter writer, Document document) {
      Iterator iter = eventHandlers.entrySet().iterator();
      // Added by Kiran. since the pageNumber was not printing correctly when segments are used
      DocumentHandler.this.getTemplateContext().setProperty("pageNumber", new Integer(getPageNumber(writer)));
      while (iter.hasNext()) {
        Map.Entry entry = (Map.Entry) iter.next();
        DocumentPageEventHandler evtHandler = (DocumentPageEventHandler) entry.getValue();
        evtHandler.onEndPage(writer, document);
        if (evtHandler.isTemporary()) {
          // removing temporary handler
          iter.remove();
        }
      }
    }

    /**
     * @see com.lowagie.text.pdf.PdfPageEvent#onGenericTag(com.lowagie.text.pdf.PdfWriter, com.lowagie.text.Document, com.lowagie.text.Rectangle, java.lang.String)
     */
    public void onGenericTag(PdfWriter writer, Document document, Rectangle rect, String text) {
      Iterator iter = eventHandlers.values().iterator();
      while (iter.hasNext()) {
        DocumentPageEventHandler evtHandler = (DocumentPageEventHandler) iter.next();
        evtHandler.onGenericTag(writer, document, rect, text);
      }
    }

    /**
     * @see com.lowagie.text.pdf.PdfPageEvent#onOpenDocument(com.lowagie.text.pdf.PdfWriter, com.lowagie.text.Document)
     */
    public void onOpenDocument(PdfWriter writer, Document document) {
      // applying the current logical page number style
      applyCurrentLogicalPageNumberStyle();

      Iterator iter = eventHandlers.values().iterator();
      while (iter.hasNext()) {
        DocumentPageEventHandler evtHandler = (DocumentPageEventHandler) iter.next();
        evtHandler.onOpenDocument(writer, document);
      }
    }

    /**
     * @see com.lowagie.text.pdf.PdfPageEvent#onCloseDocument(com.lowagie.text.pdf.PdfWriter, com.lowagie.text.Document)
     */
    public void onCloseDocument(PdfWriter writer, Document document) {
      flushTocItems(writer);
      Iterator iter = eventHandlers.values().iterator();
      while (iter.hasNext()) {
        DocumentPageEventHandler evtHandler = (DocumentPageEventHandler) iter.next();
        evtHandler.onCloseDocument(writer, document);
        // removing handler
        iter.remove();
      }

      /** 
       * Added by Kiran
       * this will put any PDF Page labels if created.
       */
      writer.setPageLabels(DocumentHandler.this.pdfLabels);

      Map numberOfPagesTemplates = (Map) getResource(BaseDocumentTag.NUMBER_OF_PAGES_TEMPLATES);
      if (numberOfPagesTemplates != null) {
        Iterator iterNumberOfPageTemplates = numberOfPagesTemplates.entrySet().iterator();
        while (iterNumberOfPageTemplates.hasNext()) {
          Map.Entry entry = (Map.Entry) iterNumberOfPageTemplates.next();
          DocumentFont font = (DocumentFont) entry.getKey();

          PdfTemplate numberOfPagesTemplate = (PdfTemplate) entry.getValue();
          Font numberOfPagesFont = font.getFont();
          if ((numberOfPagesTemplate != null) && (numberOfPagesFont != null)) {
            BaseFont bf = numberOfPagesFont.getBaseFont();
            if (numberOfPagesTemplate == null) {
              // no template has been built, skipping number of pages output!
              return;
            }
            numberOfPagesTemplate.beginText();
            numberOfPagesTemplate.setTextMatrix(0, 1);
            numberOfPagesTemplate.setFontAndSize(bf, numberOfPagesFont.getSize());
            Color fontColor = numberOfPagesFont.getColor();
            if (fontColor != null) {
              numberOfPagesTemplate.setColorStroke(fontColor);
              numberOfPagesTemplate.setColorFill(fontColor);
            }
            String numberOfPages = String.valueOf(getPageNumber(writer) - 1);
            float numberOfPagesWidth = bf.getWidthPoint(numberOfPages, numberOfPagesFont.getSize());
            numberOfPagesTemplate.setWidth(numberOfPagesWidth + 1);
            numberOfPagesTemplate.showText(numberOfPages);
            numberOfPagesTemplate.endText();
          }

        }
        // removing resources again
        removeResource(BaseDocumentTag.NUMBER_OF_PAGES_TEMPLATES);
      }
    }

    /**
     * @see com.lowagie.text.pdf.PdfPageEvent#onParagraph(com.lowagie.text.pdf.PdfWriter, com.lowagie.text.Document, float)
     */
    public void onParagraph(PdfWriter writer, Document document, float paragraphPosition) {
      Iterator iter = eventHandlers.values().iterator();
      while (iter.hasNext()) {
        DocumentPageEventHandler evtHandler = (DocumentPageEventHandler) iter.next();
        evtHandler.onParagraph(writer, document, paragraphPosition);
      }
    }

    /**
     * @see com.lowagie.text.pdf.PdfPageEvent#onParagraphEnd(com.lowagie.text.pdf.PdfWriter, com.lowagie.text.Document, float)
     */
    public void onParagraphEnd(PdfWriter writer, Document document, float paragraphPosition) {
      Iterator iter = eventHandlers.values().iterator();
      while (iter.hasNext()) {
        DocumentPageEventHandler evtHandler = (DocumentPageEventHandler) iter.next();
        evtHandler.onParagraphEnd(writer, document, paragraphPosition);
      }
    }

    /**
     * @see com.lowagie.text.pdf.PdfPageEvent#onSection(com.lowagie.text.pdf.PdfWriter, com.lowagie.text.Document, float, int, com.lowagie.text.Paragraph)
     */
    public void onSection(PdfWriter writer, Document document, float paragraphPosition, int depth, Paragraph title) {
      flushTocSection(writer, paragraphPosition);
      Iterator iter = eventHandlers.values().iterator();
      while (iter.hasNext()) {
        DocumentPageEventHandler evtHandler = (DocumentPageEventHandler) iter.next();
        evtHandler.onSection(writer, document, paragraphPosition, depth, title);
      }
    }

    /**
     * @see com.lowagie.text.pdf.PdfPageEvent#onSectionEnd(com.lowagie.text.pdf.PdfWriter, com.lowagie.text.Document, float)
     */
    public void onSectionEnd(PdfWriter writer, Document document, float position) {
      Iterator iter = eventHandlers.values().iterator();
      while (iter.hasNext()) {
        DocumentPageEventHandler evtHandler = (DocumentPageEventHandler) iter.next();
        evtHandler.onSectionEnd(writer, document, position);
      }
    }
  }

  /** 
   * Sets the current logical page numbering style.
   * @param logicalPageNumberStyle The style value for the logical page numbering.
   * this value can be DECIMAL_ARABIC_NUMERALS = 0 which is the default
   * UPPERCASE_ROMAN_NUMERALS = 1 (I, II, III, IV etc)
   * LOWERCASE_ROMAN_NUMERALS = 2 (i, ii, iii, iv etc)
   * UPPERCASE_LETTERS = 3 (A,B,C,D etc)
   * LOWERCASE_LETTERS = 4 (a,b,c,d etc)
   * EMPTY = 5 (No logical page numbers are generated but fixed text may still exist)
   */    
  public void setLogicalPageNumberStyle(int logicalPageNumberStyle) {
    this.logicalPageNumberStyle = logicalPageNumberStyle;
  }

  /** 
   * Sets the current logical page numbering text.
   * @param logicalPageNumberText The text that is prepended to the logical page numbering style.
   */    
  public void setLogicalPageNumberText(String logicalPageNumberText) {
    this.logicalPageNumberText = logicalPageNumberText;
  }
  
  /** 
   * Applies the logical page Numbering style for the generated PDF.
   * @see #setLogicalPageNumberStyle(int, String)
   * @see #logicalPageNumberStyle
   * @see #logicalPageNumberText
   */    
  public void applyCurrentLogicalPageNumberStyle() {
    PdfWriter writer = (PdfWriter) getDocumentWriter(false);
    //Added by Kiran
    //this adds logical page numbers to the generated PDF
    if (isDestinationsSupported()) {
      int pageNumber = getPageNumber(writer);
      if (pageNumber > 0) {
        pdfLabels.addPageLabel(pageNumber, logicalPageNumberStyle, logicalPageNumberText, pageNumber);
      }
      //writer.setPageLabels(pdfLabels);
    }
  }
  
  /**
   * Gets the current page number.
   * @param writer The document writer.
   * @return The current page number.
   */
  public int getPageNumber(PdfWriter writer) {
    return writer.getPageNumber() + pageNumberOffset;
  }

  /**
   * Constructs a DocumentHandler instance with specific arguments.
   * @param outputStream The stream where to write the document to.
   * @param templateContext The template context to use.
   * @param tagFactory The special tag factory to use.
   */
  public DocumentHandler(OutputStream outputStream, TemplateContext templateContext, 
                         DocumentTagFactory tagFactory) {
    init(outputStream, null, templateContext, tagFactory);
  }

  /**
   * Constructs a DocumentHandler instance with specific arguments.
   * @param outputStream The stream where to write the document to.
   * @param templateInterpreter The template interpreter to use.
   * @param templateContext The template context to use.
   * @param tagFactory The special tag factory to use.
   */
  public DocumentHandler(OutputStream outputStream, TemplateInterpreter templateInterpreter, 
                         TemplateContext templateContext, DocumentTagFactory tagFactory) {
    init(outputStream, templateInterpreter, templateContext, tagFactory);
  }
  
  /**
   * Initializes the document handler.
   * @param outputStream The stream where to write the document to.
   * @param templateInterpreter The template interpreter to use.
   * @param templateContext The template context to use.
   * @param tagFactory The special tag factory to use.
   */  
  private void init(OutputStream outputStream, TemplateInterpreter templateInterpreter,
                    TemplateContext templateContext, DocumentTagFactory tagFactory) {
    try {
      this.log = LogFactory.getLog("DocumentHandler");
    } catch (Throwable ex) {
      this.log = new SimpleLog("DocumentHandler");
      this.log.warn("Unable to obtain log from the jakarta.commons log factory, using SimpleLog instead.", ex);
    }
    this.outputStream = outputStream;
    if (templateInterpreter != null) {
      this.templateInterpreter = templateInterpreter;
    } else {
      this.templateInterpreter = new DefaultTemplateInterpreterFactory().createTemplateInterpreter();
    }
    this.templateContext = templateContext;
    this.tagFactory = tagFactory;
    this.fontMapper = new DocumentFontMapper(this);
    
    // setting up style parser and attribute handler.
    if (styleParser == null) {
      setStyleParser(new DefaultStyleParser());
    }
    if (attributeHandler == null) {
      setAttributeHandler(new DefaultAttributeHandler());
    }
  }

  /**
   * Resets the document handler.
   * @param outputStream The stream where to write the document to.
   */  
  protected void reset(OutputStream outputStream) {
    this.outputStream = outputStream;
    // resetting the document writer
    this.documentWriter = null;
  }

  
  /**
   * Getter method for the the property log.
   * @return The current value of property log.
   */
  public Log getLog() {
    return log;
  }
  /**
   * Setter method for the the property log.
   * @param log The value to set for the property log.
   */
  public void setLog(Log log) {
    this.log = log;
  }
  
  /**
   * Getter method for the the property xmlReaderClass.
   * @return The current value of property xmlReaderClass.
   */
  public String getXmlReaderClass() {
    return xmlReaderClass;
  }
  /**
   * Setter method for the the property xmlReaderClass.
   * @param xmlReaderClass The value to set for the property xmlReaderClass.
   */
  public void setXmlReaderClass(String xmlReaderClass) {
    this.xmlReaderClass = xmlReaderClass;
  }

  /**
   * Gets the format helper.
   * @return The format helper.
   */
  public FormatHelper getFormatHelper() {
    return templateContext.getFormatHelper();
  }
  
  /**
   * Adds a resource loader.
   * @param resourceLoader The resource loader to add. 
   */
  public void setResourceLoader(ResourceLoader resourceLoader) {
    this.resourceLoader = resourceLoader;
  } 
  
  /**
   * Loads the resource from the given loacation.
   * @param location The location from which to load the resource.
   * @return The loaded resource as a byte array.
   * @exception IOException In case the loading of the resource failed.
   */
  public byte[] loadResource(String location) throws IOException {
    if (resourceLoader == null) {
      throw new IOException("No resource loaders specified!");
    }
    return resourceLoader.loadResource(location);
  } 

  /**
   * Registers the specified style attributes for the given name.
   * @param styleName The style name.
   * @param attributes The attributes to register.
   */
  public void registerStyle(String styleName, Map attributes) {
    styleRegistry.put(styleName, attributes);
  }
  /**
   * Gets the style attributes for the given style name.
   * @param styleName The style name.
   * @return The attributes from the style registry or null.
   */
  public Map getStyle(String styleName) {
    return (Map) styleRegistry.get(styleName);
  }
  /**
   * Gets the style attributes for the given style class name.
   * @param tag The tag to retrive the style for.
   * @param clazz The style class names, separated by blank.
   * @return The attributes from the style registry or null.
   */
  public Map getStyle(BaseDocumentTag tag, String clazz) {
    Map styleMap = new HashMap();
    Map styles = (Map) styleRegistry.get(tag.getName());
    if (styles != null) {
      styleMap.putAll(styles);
    }
    if (clazz == null) {
      return styleMap;
    }
    
    StringTokenizer tknClass = new StringTokenizer(clazz, " ");
    while (tknClass.hasMoreTokens()) {
      String styleClazz = tknClass.nextToken();
      getStyleAttributes(styleMap, tag, styleClazz);
    }
    return styleMap;
  }
  
  /**
   * Gets the styles for the given tag and class name.
   * @param styleMap The map to add the detected styles to.
   * @param tag The tag to process.
   * @param clazz The class name.
   */
  private void getStyleAttributes(Map styleMap, BaseDocumentTag tag, String clazz) {
    String qName = "." + clazz;
    Map style = (Map) styleRegistry.get(qName);
    if (style != null) {
      styleMap.putAll(style);
    }

    String fqName = tag.getName() + qName;
    style = (Map) styleRegistry.get(fqName);
    if (style != null) {
      styleMap.putAll(style);
    }
  }

  /**
   * Registers the given color for the also given name.
   * @param colorName The color name.
   * @param color The color to register.
   */
  public void registerColor(String colorName, Color color) {
    colorRegistry.put(colorName, color);
  }
  /**
   * Gets a color from the color registry.
   * @param colorName The color name.
   * @return The color from the color registry or null.
   */
  public Color getColor(String colorName) {
    return (Color) colorRegistry.get(colorName);
  }
  
  /**
   * Registers the given true type font for the also given font name and style.
   * @param fontName The font name.
   * @param fontBytes The TTF font data.
   * @param style The base font style (NORMAL,BOLD,ITALIC,BOLD|ITALIC).
   * @param embed Tells whether or not to embed the font into the PDF (Added by Kiran).
   * @param cached Tells whether or not to cache the font (Added by Kiran).
   * @exception IOException In case the base font creation failed.
   * @exception DocumentException In case the base font creation failed.
   */
  public void registerFont(String fontName, byte[] fontBytes, int style, boolean embed, boolean cached) 
    throws IOException, DocumentException {
    
    TrueTypeFontHolder fontHolder = (TrueTypeFontHolder) fontRegistry.get(fontName); 
    if (fontHolder == null) {
      fontHolder = new TrueTypeFontHolder(fontName);
      fontRegistry.put(fontName, fontHolder);
    }

    int baseFontStyle = style & Font.BOLDITALIC;

    switch (baseFontStyle) {
      case Font.NORMAL:
        fontHolder.setNormalFont(fontBytes);
        break;
      case Font.BOLD:
        fontHolder.setBoldFont(fontBytes);
        break;
      case Font.ITALIC:
        fontHolder.setItalicFont(fontBytes);
        break;
      case Font.BOLDITALIC:
        fontHolder.setBoldItalicFont(fontBytes);
        break;
      default: // assuming normal font  
        fontHolder.setNormalFont(fontBytes);
        break;
    }
    
    //Added by Kiran
    fontHolder.setEmbedded(embed);
    fontHolder.setCached(cached);
  } 

  /**
   * Creates a font according to the given attributes.
   * First it checks the local font registry. If no matching font has been
   * found, it asks the iText FontFactory. 
   * @param fontName The font name. This may be a comma separated list of font names.     
   * @param encoding The font encoding.
   * @param size The desired font size.
   * @param style The font style (NORMAL, BOLD, ITALIC ...).
   * @param color The font color.
   * @param textRise The text displacement relative to the baseline. 
   *    Positive values rise the text, negative values lower the text.
   * @return The created document font.
   * @exception DocumentHandlerException In case the font could not be created.
   */
  public DocumentFont createFont(String fontName, String encoding, float size, int style, 
                                 float textRise, Color color) 
    throws DocumentHandlerException {
    
    StringTokenizer tkn = new StringTokenizer(fontName, ", ");
    while (tkn.hasMoreTokens()) {
      String name = tkn.nextToken();
      TrueTypeFontHolder fontHolder = getRegisteredFont(name); 
      if (fontHolder == null) {
        Font font = FontFactory.getFont(name, encoding, size, style, color);
        if ((font.getFamily() < 0) && (font.getBaseFont() == null)) {
          // trying next family
          continue;
        }
        return new DocumentFont(
            font, name, encoding, size, 
            style, textRise, color
        );
      }

      BaseFont baseFont = null;
      int fontStyle = style & (Font.STRIKETHRU | Font.UNDERLINE);
      int baseFontStyle = style & Font.BOLDITALIC;
      switch (baseFontStyle) {
        case Font.NORMAL:
          baseFont = fontHolder.getNormalFont();
          break;
        case Font.BOLDITALIC:
          baseFont = fontHolder.getBoldItalicFont();
          if (baseFont == null) {
            fontStyle = style;
            if (fontHolder.getBoldFont() != null) {
              baseFont = fontHolder.getBoldFont();
              fontStyle ^= Font.BOLD;
            }
          } else if (baseFont.getFontDescriptor(BaseFont.ITALICANGLE, size) == 0.0F) {
            fontStyle |= Font.ITALIC;
          }
          break;
        case Font.BOLD:
          baseFont = fontHolder.getBoldFont();
          if (baseFont == null) {
            fontStyle = style;
          }
          break;
        case Font.ITALIC:
          baseFont = fontHolder.getItalicFont();
          if ((baseFont == null) || (baseFont.getFontDescriptor(BaseFont.ITALICANGLE, size) == 0.0F)) {
            fontStyle = style;
          }
          break;
        default: // assuming normal font
          baseFont = fontHolder.getNormalFont();
          break;
      }
  
      if (baseFont == null) {
        baseFont = fontHolder.getNormalFont();
      }
      return new DocumentFont(
          new Font(baseFont, size, fontStyle, color), 
          fontName, encoding, size, style, textRise, color
      );
    }
    
    // Failed to create font! 
    throw new DocumentHandlerException(
        locator(), "Unable to create font '" + fontName + 
        "', make sure that the family has been registered properly."
    );
  }
    
  /**
   * Gets the BaseFont for the given font name and style.
   * @param fontName The font name. 
   * @param style The font style (NORMAL, BOLD, ITALIC ...).
   * @return The according BaseFont.
   */
  public BaseFont getBaseFont(String fontName, int style) {
    
    TrueTypeFontHolder fontHolder = (TrueTypeFontHolder) fontRegistry.get(fontName); 
    if (fontHolder == null) {
      //return BaseFont.createFont(fontName, BaseFont.WINANSI, false);
      return null;
    }

    BaseFont baseFont = null;
    int baseFontStyle = style & Font.BOLDITALIC;
    switch (baseFontStyle) {
      case Font.NORMAL:
        baseFont = fontHolder.getNormalFont();
        break;
      case Font.BOLD:
        baseFont = fontHolder.getBoldFont();
        break;
      case Font.ITALIC:
        baseFont = fontHolder.getItalicFont();
        break;
      case Font.BOLDITALIC:
        baseFont = fontHolder.getBoldItalicFont();
        break;
      default: // assuming normal font
        baseFont = fontHolder.getNormalFont();
        break;
    }

    if (baseFont == null) {
      baseFont = fontHolder.getNormalFont();
    }
    return baseFont;
  }

  /**
   * Gets the BaseFont for the given font name and style.
   * @param font The document font for which to create the AWT font. 
   * @return The according AWT font.
   */
  public java.awt.Font getAwtFont(DocumentFont font) {

    String name = font.getFamily();
    int baseFontStyle = font.getStyle() & Font.BOLDITALIC;
    int style = java.awt.Font.PLAIN;
    switch (baseFontStyle) {
      case Font.NORMAL:
        break;
      case Font.BOLD:
        style |= java.awt.Font.BOLD;
        break;
      case java.awt.Font.ITALIC:
        style |= java.awt.Font.ITALIC;
        break;
      case Font.BOLDITALIC:
        style |= java.awt.Font.BOLD | java.awt.Font.ITALIC;
        break;
      default:
        break;
    }
    return new AwtFont(name, style, (int) font.getSize());
  }

  /**
   * Gets an array holding all registerd fonts.
   * @return An array of TrueTypeFontHolder instances holding the registered BaseFont variations
   *   for the specific font names.
   */
  public TrueTypeFontHolder[] getRegisteredFonts() {
    int numRegisterdFonts = fontRegistry.size();
    TrueTypeFontHolder[] fonts = new TrueTypeFontHolder[numRegisterdFonts];
    Iterator iterFonts = fontRegistry.entrySet().iterator();
    int idx = 0;
    while (iterFonts.hasNext()) {
      TrueTypeFontHolder font = (TrueTypeFontHolder) iterFonts.next();
      fonts[idx] = font;
      ++idx;
    }
    return fonts; 
  }

  /**
   * Gets a specific registered font.
   * @param name The name of the desired font.
   * @return The according TrueTypeFontHolder instances.
   */
  public TrueTypeFontHolder getRegisteredFont(String name) {
    TrueTypeFontHolder fontHolder = (TrueTypeFontHolder) fontRegistry.get(name); 
    return fontHolder; 
  }

  /**
   * Gets the font mapper, required for PDF rendering using the Graphics2D interface.
   * @return The font mapper.
   */
  public FontMapper getFontMapper() {
    return fontMapper;
  }

  /**
   * Getting image from cache.
   * @param source The image source.
   * @return The cached image or null, if not found.
   */
  public Image getCachedImage(Object source) {
    Image img = (Image) getResource(source);
    if (!cacheImages || (img == null)) {
      return null;
    }
    return new ImageCopy(img);
  }
  /**
   * Putting image into cache.
   * @param source The image source.
   * @param img The image to put into cache.
   */
  public void cacheImage(Object source, Image img) {
    if (cacheImages) {
      registerResource(source, img);
    }
  }

  
  /**
   * Gets the supported attribute definition map for the given tag instance.
   * In case the attribute definitions are already defined, the value from the registry is returned,
   * else the tags buildSupportedAttributes() is called and the created map is registerd and
   * returned.
   * @param tag The tag to get the supported attributes for.
   * @return The determined supported attribute definitios. 
   */
  public AttributeDefinitionMap getSupportedAttributes(BaseDocumentTag tag) {
    return documentHandlerComponentRegistry.getSupportedAttributes(tag);
  }
  
  /**
   * Gets the supported style attribute definition map for the given tag instance.
   * In case the attribute definitions are already defined, the value from the registry is returned,
   * else the tags buildSupportedStyleAttributes() is called and the created map is registerd and
   * returned.
   * @param tag The tag to get the supported attributes for.
   * @return The determined supported attribute definitios. 
   */
  public AttributeDefinitionMap getSupportedStyleAttributes(BaseDocumentTag tag) {
    return documentHandlerComponentRegistry.getSupportedStyleAttributes(tag);
  } 
  
  /**
   * Gets the supported style attribute definition map for the given tag instance.
   * In case the attribute definitions are already defined, the value from the registry is returned,
   * else the tags buildSupportedStyleAttributes() is called and the created map is registerd and
   * returned.
   * @param tag The tag to get the supported attributes for.
   * @return The determined supported attribute definitios. 
   */
  public ChildDefinitionMap getSupportedChilds(BaseDocumentTag tag) {
    return documentHandlerComponentRegistry.getSupportedChilds(tag);
  } 


  /**
   * Gets the current page event handler.
   * @return The current page event handler.
   */
  public PdfPageEvent getPageEventHandler() {
    return pageEventHandler;
  }
  /**
   * Creates a new page event handler.
   * @return The created page event handler.
   */
  public PdfPageEvent createPageEventHandler() {
    return new PageEventHandler();
  }

  
  /**
   * Sets the given handler to the event handler map. 
   * In case true is passed for the attribute <code>queue</code> and a handler
   * for the given key is currently assigned, it puts the handler to the queue map.   
   * @param key The event handler key.
   * @param handler The event handler.
   */
  public void setPageEventHandler(Object key, PdfPageEvent handler) {
    pageEventHandler.setHandler(key, handler);
  }  
  /**
   * Removes the handler with the given name from the event handler map. 
   * @param key The event handler key.
   */
  public void removePageEventHandler(Object key) {
    pageEventHandler.removeHandler(key);
  } 
  /**
   * Removes all current page event handlers. 
   */
  public void clearPageEventHandlers() {
    pageEventHandler.clearHandlers();
  } 
  
  /**
   * Gets the template context.
   * @return The current template context.
   */
  public TemplateContext getTemplateContext() {
    return templateContext;
  }
  /**
   * Sets the template context.
   * @param templateContext The template context to set.
   */
  public void setTemplateContext(TemplateContext templateContext) {
    this.templateContext = templateContext;
  }

  /**
   * Gets the tag factory.
   * @return The tag factory.
   */
  public DocumentTagFactory getTagFactory() {
    return tagFactory;
  }

  /**
   * Getter method for the the property translateEscapeSequences.
   * @return The current value of property translateEscapeSequences.
   */
  public boolean isTranslateEscapeSequences() {
    return translateEscapeSequences;
  }
  /**
   * Setter method for the the property translateEscapeSequences.
   * @param translateEscapeSequences The value to set for the property translateEscapeSequences.
   */
  public void setTranslateEscapeSequences(boolean translateEscapeSequences) {
    this.translateEscapeSequences = translateEscapeSequences;
  }

  /**
   * Getter method for the the property parseDynamicContent.
   * @return The current value of property parseDynamicContent.
   */
  public boolean isParseDynamicContent() {
    return parseDynamicContent;
  }
  /**
   * Setter method for the the property parseDynamicContent.
   * @param parseDynamicContent The value to set for the property parseDynamicContent.
   */
  public void setParseDynamicContent(boolean parseDynamicContent) {
    this.parseDynamicContent = parseDynamicContent;
  }

  /**
   * Getter method for the the property evaluateExpressions.
   * @return The current value of property evaluateExpressions.
   */
  public boolean isEvaluateExpressions() {
    return evaluateExpressions;
  }
  /**
   * Setter method for the the property evaluateExpressions.
   * @param evaluateExpressions The value to set for the property evaluateExpressions.
   */
  public void setEvaluateExpressions(boolean evaluateExpressions) {
    this.evaluateExpressions = evaluateExpressions;
  }
  
  /**
   * Getter method for the the property cacheImages.
   * @return The current value of property cacheImages.
   */
  public boolean isCacheImages() {
    return cacheImages;
  }
  /**
   * Setter method for the the property cacheImages.
   * @param cacheImages The value to set for the property cacheImages.
   */
  public void setCacheImages(boolean cacheImages) {
    this.cacheImages = cacheImages;
  }

  /**
   * Getter method for the the property strictlyCheckDocumentStructure.
   * @return The current value of property strictlyCheckDocumentStructure.
   */
  public boolean isStrictlyCheckDocumentStructure() {
    return strictlyCheckDocumentStructure;
  }
  /**
   * Setter method for the the property strictlyCheckDocumentStructure.
   * @param strictlyCheckDocumentStructure The value to set for the property strictlyCheckDocumentStructure.
   */
  public void setStrictlyCheckDocumentStructure(
      boolean strictlyCheckDocumentStructure) {
    this.strictlyCheckDocumentStructure = strictlyCheckDocumentStructure;
  }

  /**
   * Getter method for the the property expectedNumberOfPages.
   * @return The current value of property expectedNumberOfPages.
   */
  public int getExpectedNumberOfPages() {
    return expectedNumberOfPages;
  }
  /**
   * Setter method for the the property expectedNumberOfPages.
   * @param expectedNumberOfPages The value to set for the property expectedNumberOfPages.
   */
  public void setExpectedNumberOfPages(int expectedNumberOfPages) {
    this.expectedNumberOfPages = expectedNumberOfPages;
  }


  /**
   * Getter method for the the property includeHeadSection.
   * @return The current value of property includeHeadSection.
   */
  public boolean isIncludeHeadSection() {
    return includeHeadSection;
  }
  /**
   * Setter method for the the property includeHeadSection.
   * @param includeHeadSection The value to set for the property includeHeadSection.
   */
  protected void setIncludeHeadSection(boolean includeHeadSection) {
    this.includeHeadSection = includeHeadSection;
  }


  /**
   * Executes the given template source.
   * @param templateSource The template source to execute.
   * @return The template execution result.
   * @exception TemplateException If the expression could not be successfuly resolved
   */
  public String executeTemplate(String templateSource) throws TemplateException {
    if (!evaluateExpressions || templateSource == null || templateSource.indexOf(CDATA_BEGIN) != -1) {
      return templateSource;
    }
    return templateInterpreter.execute(templateSource, templateContext);
  }
  
  /**
   * Evaluates a text, which is likely to contain expressions as a boolean.
   * @param expression The expression to evaluate.
   * @return The computed boolean value.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public boolean evalBoolean(String expression) throws ExpressionException {
    return templateInterpreter.evalBooleanExpr(expression, templateContext);
  }

  /**
   * Evaluates a text, which is likely to contain expressions as an int.
   * @param expression The expression to evaluate.
   * @return The computed int value.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public int evalInt(String expression) throws ExpressionException {
    return templateInterpreter.evalIntExpr(expression, templateContext);
  }

  /**
   * Evaluates a text, which is likely to contain expressions as an long.
   * @param expression The expression to evaluate.
   * @return The computed long value.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public long evalLong(String expression) throws ExpressionException {
    return templateInterpreter.evalLongExpr(expression, templateContext);
  }

  /**
   * Evaluates a text, which is likely to contain expressions as a float.
   * @param expression The expression to evaluate.
   * @return The computed float value.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public float evalFloat(String expression) throws ExpressionException {
    return templateInterpreter.evalFloatExpr(expression, templateContext);
  }

  /**
   * Evaluates a text, which is likely to contain expressions as a double.
   * @param expression The expression to evaluate.
   * @return The computed boolean value.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public double evalDouble(String expression) throws ExpressionException {
    return templateInterpreter.evalDoubleExpr(expression, templateContext);
  }

  /**
   * Evaluates a text, which is likely to contain expressions as a Date.
   * @param expression The expression to evaluate.
   * @return The computed Date value.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public Date evalDate(String expression) throws ExpressionException {
    return templateInterpreter.evalDateExpr(expression, templateContext);
  }

  /**
   * Evaluates a text, which is likely to contain expressions as a Time.
   * @param expression The expression to evaluate.
   * @return The computed Time value.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public Time evalTime(String expression) throws ExpressionException {
    return templateInterpreter.evalTimeExpr(expression, templateContext);
  }

  /**
   * Evaluates a text, which is likely to contain expressions as a Timestamp.
   * @param expression The expression to evaluate.
   * @return The computed Timestamp value.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public Timestamp evalTimestamp(String expression) throws ExpressionException {
    return templateInterpreter.evalTimestampExpr(expression, templateContext);
  }

  /**
   * Evaluates a text, which is likely to contain expressions as a String.
   * @param expression The expression to evaluate.
   * @return The computed String value.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public String evalString(String expression) throws ExpressionException {
    if (!evaluateExpressions) {
      return expression;
    }
    return templateInterpreter.evalStringExpr(expression, templateContext);
  }

  /**
   * Evaluates a text, which is likely to contain expressions as am Object.
   * @param expression The expression to evaluate.
   * @return The computed Object.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public Object evalObject(String expression) throws ExpressionException {
    return templateInterpreter.evalObjectExpr(expression, templateContext);
  }

  /**
   * Registers the given format instance for the also given format string.
   * @param formatString The format string.
   * @param formatInstance The format instance.
   */
  public void registerFormat(String formatString, Format formatInstance) {
    getFormatHelper().registerFormat(formatString, formatInstance);
  }
  /**
   * Looks up the macro definition for the given macro name.
   * @param formatString The format string.
   * @return The according format instance or null.
   */
  public Format lookupFormat(String formatString) {
    return getFormatHelper().lookupFormat(formatString);
  }

  /**
   * Registers the given macro definition instance for the also given macro name.
   * @param macroName The macro name.
   * @param macroDefinition The macro definition instance.
   */
  public void registerMacroDefinition(String macroName, MacroDefTag macroDefinition) {
    macroDefinitionMap.put(macroName, macroDefinition);
  }
  /**
   * Looks up the macro definition for the given macro name.
   * @param macroName The macro name.
   * @return The according macro instance or null.
   */
  public MacroDefTag lookupMacroDefinition(String macroName) {
    return (MacroDefTag) macroDefinitionMap.get(macroName);
  }

  /**
   * Pushes an item on the document stack.
   * @param item The new item on top of the stack.
   */
  public void pushItem(BaseDocumentTag item) {
    String tagBody = item.getContent();
    if ((tagBody != null) && (tagBody.length() > 0)) {
      templateContext.setProperty("body", tagBody);
    }
    documentStack.add(item);
    registerTopDocumentElement(item);
  }
  /**
   * Pops the top item from the document stack.
   * @return The old top item on top of the stack.
   */
  public BaseDocumentTag popItem() {
    BaseDocumentTag pop = (BaseDocumentTag) documentStack.remove(documentStack.size()-1);
    updateTopDocumentElements();
    BaseDocumentTag top = topItem();
    if (top != null) {
      String tagBody = top.getContent();
      if ((tagBody != null) && (tagBody.length() > 0)) {
        templateContext.setProperty("body", tagBody);
      }
    }
    return pop;
  }

  /**
   * Updates the top document elements.
   */
  private void updateTopDocumentElements() {
    topDocumentElements.clear();
    topRealTag = null;
    BaseDocumentTag item = null;
    int stackSize = documentStack.size();
    for (int i = 0; i < stackSize; i++) {
      item = (BaseDocumentTag) documentStack.get(i);
      registerTopDocumentElement(item);
    }
  }
  /**
   * Registers the given item as new top level item. 
   * @param item The item to register.
   */
  private void registerTopDocumentElement(BaseDocumentTag item) {
    topDocumentElements.put(item.getName(), item);
    if (item instanceof ElementContainer) {
      topDocumentElements.put(ElementContainer.class, item);
    }
    if (item instanceof CellContainer) {
      topDocumentElements.put(CellContainer.class, item);
    }
    if (item instanceof PdfCellContainer) {
      topDocumentElements.put(PdfCellContainer.class, item);
    }
    if (item instanceof Condition) {
      topDocumentElements.put(Condition.class, item);
    }
    if (item instanceof FontHolder) {
      topDocumentElements.put(FontHolder.class, item);
    }
    if (item instanceof PhraseHolder) {
      topDocumentElements.put(PhraseHolder.class, item);
    }
    if (item instanceof CommonAttributesHolder) {
      topDocumentElements.put(CommonAttributesHolder.class, item);
    }
    if (!item.isVirtualContainer()) {
      topRealTag = item;
    }
  }
  
  /**
   * Gets the top item from the document stack.
   * @return The top item on top of the stack.
   */
  public BaseDocumentTag topItem() {
    try {
      return (BaseDocumentTag) documentStack.get(documentStack.size()-1);
    } catch (IndexOutOfBoundsException ex) {
      return null;
    }
  }

  /**
   * Pushes an item on the event recorder stack.
   * @param recorder The new item on top of the stack.
   */
  public void pushEventRecorder(DocumentEventRecorder recorder) {
    eventRecorderStack.add(recorder);
    topEventRecorder = recorder;
  }
  /**
   * Pops the top item from the event recorder stack.
   * @return The old top item on top of the stack.
   */
  public DocumentEventRecorder popEventRecorder() {
    DocumentEventRecorder top = (DocumentEventRecorder) eventRecorderStack.remove(eventRecorderStack.size() - 1);
    int stackSize = eventRecorderStack.size();
    if (stackSize > 0) {
      topEventRecorder = (DocumentEventRecorder) eventRecorderStack.get(stackSize - 1);
    } else {
      topEventRecorder = null;
    }
    return top;
  }
  /**
   * Gets the top item from the event recorder stack.
   * @return The top item on top of the stack.
   */
  public DocumentEventRecorder topEventRecorder() {
    return topEventRecorder;
  }

  /**
   * Gets the first item from the top of the stack that matches the given type name.
   * @param type The name of the tag to look for.
   * @return The detected item or null.
   */
  public BaseDocumentTag latestOfType(String type) {
    return (BaseDocumentTag) topDocumentElements.get(type);
  }
  /**
   * Gets the first item from the top of the stack that is no virtual container.
   * @return The detected item or null.
   */
  public BaseDocumentTag latestRealTag() {
    return topRealTag;
  }
  /**
   * Gets the first item from the top of the stack that matches the given type name.
   * @param type The class of the tag to look for.
   * @return The detected item or null.
   */
  public BaseDocumentTag latestOfType(Class type) {
    BaseDocumentTag item = null;
    int stackSize = documentStack.size();
    for (int i = stackSize - 1; i >= 0; i--) {
      item = (BaseDocumentTag) documentStack.get(i);
      // checking type of instance
      if (type.isInstance(item)) {
        return item;
      }
    }
    return null;
  }
  /**
   * Gets the first item from the top of the stack that implements the ElementContainer interface.
   * @return The detected item or null.
   */
  public ElementContainer latestElementContainer() {
    return (ElementContainer) topDocumentElements.get(ElementContainer.class);
  }
  /**
   * Gets the first item from the top of the stack that implements the CellContainer interface.
   * @return The detected item or null.
   */
  public CellContainer latestCellContainer() {
    return (CellContainer) topDocumentElements.get(CellContainer.class);
  }
  
  /**
   * Gets the first item from the top of the stack that implements the PdfCellContainer interface.
   * @return The detected item or null.
   */
  public PdfCellContainer latestPdfCellContainer() {
    return (PdfCellContainer) topDocumentElements.get(PdfCellContainer.class);
  }
  /**
   * Gets the first item from the top of the stack that implements the Condition interface.
   * @return The detected item or null.
   */
  public Condition latestCondition() {
    return (Condition) topDocumentElements.get(Condition.class);
  }
  /**
   * Gets the first item from the top of the stack that implements the FontHolder interface.
   * @return The detected item or null.
   */
  public FontHolder latestFontHolder() {
    return (FontHolder) topDocumentElements.get(FontHolder.class);
  }
  /**
   * Gets the first item from the top of the stack that implements the PhraseHolder interface.
   * @return The detected item or null.
   */
  public PhraseHolder latestPhraseHolder() {
    return (PhraseHolder) topDocumentElements.get(PhraseHolder.class);
  }
  /**
   * Gets the first item from the top of the stack that implements the CommonAttributesHolder interface.
   * @return The detected item or null.
   */
  public CommonAttributesHolder latestCommonAttributesHolder() {
    return (CommonAttributesHolder) topDocumentElements.get(CommonAttributesHolder.class);
  }
  
  /**
   * Gets number of items, currently on the document stack.
   * @return The number of items on the the stack.
   */
  public int numItems() {
    return documentStack.size();
  }

  /**
   * Gets the document.
   * @return The iText document.
   */
  public Document getDocument() {
    return document;
  }
  /**
   * Gets the document. In case the openDocumentRequired is true, 
   * it opens the document before returning it.
   * @param openDocumentRequired True, if an open document is required, else false. 
   * @return The iText document.
   * @exception DocumentHandlerException In case no document has been set so far.
   */
  public Document getDocument(boolean openDocumentRequired) throws DocumentHandlerException {
    if (document == null) {
      throw new DocumentHandlerException(locator, "No document format has been specified so far. " +
          "Make sure that you have eigher defined the format at the &lt;document&gt; tag or at a " +
          " &lt;page-format&gt; tag before the occurrence of this tag!");
    }
    
    if (openDocumentRequired && !document.isOpen()) {
      document.open();
    }
    return document;
  }
  /**
   * Sets the document and initializes the document writer.
   * @param document The document.
   * @return The document writer.
   * @exception DocumentHandlerException In case the document writer could not be initialized.
   */
  public DocWriter setDocument(Document document) throws DocumentHandlerException {
    this.document = document;
    try {
      documentWriter = PdfWriter.getInstance(document, outputStream);
      ((PdfWriter) documentWriter).setPageEvent(pageEventHandler);
      if (metaData != null) {
        ((PdfWriter) documentWriter).setPDFXConformance(metaData.getPdfxConformance());
        ((PdfWriter) documentWriter).setPdfVersion(metaData.getPdfVersion());
        ((PdfWriter) documentWriter).setXmpMetadata(getXmpMetaData());
      }
      return documentWriter;
    } catch (DocumentException ex) {
      throw new DocumentHandlerException(locator, "Unable to initialize the document writer:" + ex.getMessage(), ex);
    }
  }
  /**
   * Sets the document and the document writer.
   * @param document The document.
   * @param writer The document writer.
   * @return The old document writer.
   */
  public DocWriter setDocumentAndWriter(Document document, DocWriter writer) {
    DocWriter oldWriter = documentWriter;
    this.document = document;
    this.documentWriter = writer;
    return oldWriter;
  }
  /**
   * Sets the document and the document writer.
   * @param document The document.
   * @param writer The document writer.
   * @param resourceRegistry The resource registry to set.
   * @param eventHandler The page event handler to set.
   * @return The old document writer.
   */
  public DocWriter setDocumentAndWriter(Document document, DocWriter writer, 
                                        Map resourceRegistry, PdfPageEvent eventHandler) {
    DocWriter oldWriter = documentWriter;
    this.document = document;
    this.documentWriter = writer;
    this.resourceRegistry = resourceRegistry;
    this.pageEventHandler = (PageEventHandler) eventHandler;
    ((PdfWriter) documentWriter).setPageEvent(pageEventHandler);
    return oldWriter;
  }

  /**
   * Gets the document writer. In case the document is not open yet, 
   * it opens the document before returning the writer.
   * @return The document writer.
   */
  public DocWriter getDocumentWriter() {
    return getDocumentWriter(true);
  }
  /**
   * Gets the document writer. In case the openDocumentRequired is true, 
   * it opens the document before returning the writer.
   * @param openDocumentRequired True, if an open document is required, else false. 
   * @return The document writer.
   */
  public DocWriter getDocumentWriter(boolean openDocumentRequired) {
    // ensuring that the document is open
    if (openDocumentRequired && !document.isOpen()) {
      document.open();
    }

    return documentWriter;
  }

  /**
   * Gets the document locator.
   * @return The document locator.
   */
  public Locator locator() {
    if (locator == null) {
      locator = new LocatorImpl();
    }
    return locator;
  }
  /**
   * Sets the document locator.
   * @param locator The document locator.
   */
  public void setDocumentLocator(Locator locator) {
    if (this.locator != null) {
      if (locator.getLineNumber() < this.locator.getLineNumber()) {
        // this was probably sent by the parser of a nested evaluation, ignoring it.
        return;
      }
    }
    this.locator = locator;
  }

  /**
   * Registers the font with the given name.
   * @param name The font name.
   * @param font The font to be registered.
   */
  public void registerFont(String name, DocumentFont font) {
    fontMap.put(name, font);
  }
  /**
   * Gets the font with the given name.
   * @param name The font name.
   * @return The detected font.
   * @exception TagAttributeException If the desired font was not defined.
   */
  public DocumentFont getFont(String name) throws TagAttributeException {
    DocumentFont font = (DocumentFont) fontMap.get(name);
    if (font == null) {
      throw new TagAttributeException(locator, "The font '" + name + "' is not defined at this position in the document.");
    }
    return font;
  }

  /**
   * Gets the list of TOC items.
   * Note: It makes only sense to call this method <b>after</b> the document has been
   * processed. In case no TOC item has been added the result will be <code>null</code>. 
   * @return The list of TOC items that has been filled during the document processing.
   */
  public List getTocItems() {
    return tocItems;
  }
  
  /**
   * Adds a TOC item.
   * @param tocItem The TOC item to add.
   */
  public void addTocItem(TocItem tocItem) {
    if (tocItems == null) {
      tocItems = new ArrayList();
    }
    if (unnumberedTocItems == null) {
      unnumberedTocItems = new ArrayList();
    }

    if (tocFreezed) {
      if (tocItem.getPosition() < 0.0F) {
        unnumberedTocItems.add(tocItem);
      }
    } else {
      TocItem ti = null;
      if ("index".equals(tocItem.getTocRelevance())) {
        ti = (TocItem) tocItemMap.get(tocItem.getTitle());
      } else {
        ti = (TocItem) tocItemMap.get(tocItem.getNumber());
      }
      if (ti != null) {
        tocItems.set(ti.getIndex(), ti);
        if (tocItem.getPosition() < 0.0F) {
          unnumberedTocItems.add(ti);
        }
      } else {
        tocItem.setIndex(tocItems.size());
        tocItems.add(tocItem);
        tocItemMap.put(tocItem.getNumber(), tocItem);
        if (tocItem.getPosition() < 0.0F) {
          unnumberedTocItems.add(tocItem);
        }
      }
    }
  }

  /**
   * Gets the list of outline items.
   * Note: It makes only sense to call this method <b>after</b> the document has been
   * processed. In case no TOC item has been added the result will be <code>null</code>. 
   * @return The list of TOC items that has been filled during the document processing.
   */
  public List getOutlines() {
    return outlineItems;
  }
  
  /**
   * Adds an outline item.
   * @param outlineItem The outline item to add.
   */
  public void addOutline(OutlineItem outlineItem) {
    if (outlineItems == null) {
      outlineItems = new ArrayList();
    }
    outlineItems.add(outlineItem);
  }

  /**
   * Processes the next unnumbered TOC chapter. 
   * @param writer The PDF writer to use.
   * @param verticalPosition The vertical position of the chapter title.
   * @return The processed TOC item.
   */
  protected TocItem flushTocChapter(PdfWriter writer, float verticalPosition) {
    if (unnumberedTocItems != null) {
      int numUnnumberedTocItems = unnumberedTocItems.size();
      for (int i = 0; i < numUnnumberedTocItems; i++) {
        TocItem tocItem = (TocItem) unnumberedTocItems.get(i);
        if (tocItem.getLevel() == 1) {
          tocItem.setPage(getPageNumber(writer));
          tocItem.setPosition(verticalPosition);
          // generating outline item for chapter
          String itemName = "chapter_" + tocItem.getNumber();
          float position = tocItem.getPosition();
          OutlineItem outline = new OutlineItem(
              itemName, tocItem.getPage(), 
              0.0F, position, null, tocItem.getTitle()
          );
          // creating anchor for outline
          if (isDestinationsSupported()) {
            writer.getDirectContent().localDestination(itemName, new PdfDestination(PdfDestination.FITH, position));
          }
          tocItem.setOutline(outline);
          addOutline(outline);
          unnumberedTocItems.remove(i);
          return tocItem;
        }
      }
    }
    return null;
  }

  /**
   * Processes the next unnumbered TOC section. 
   * @param writer The PDF writer to use.
   * @param verticalPosition The vertical position of the chapter title.
   * @return The processed TOC item.
   */
  protected TocItem flushTocSection(PdfWriter writer, float verticalPosition) {
    if (unnumberedTocItems != null) {
      int numUnnumberedTocItems = unnumberedTocItems.size();
      for (int i = 0; i < numUnnumberedTocItems; i++) {
        TocItem tocItem = (TocItem) unnumberedTocItems.get(i);
        int tocLevel = tocItem.getLevel();
        if (tocLevel > 1) {
          tocItem.setPage(getPageNumber(writer));
          tocItem.setPosition(verticalPosition);
          // determining superiour item
          TocItem parent = null;
          for (int j = tocItems.size() - 1; j >= 0; j--) {
            TocItem ti = (TocItem) tocItems.get(j);
            if ((ti.getPage() > 0) && ti.getLevel() == (tocLevel - 1)) {
              parent = ti;
              break;
            }
          }
          // generating outline item for chapter
          String itemName = "section_" + tocItem.getNumber();
          float position = tocItem.getPosition();
          OutlineItem outline = new OutlineItem(
              itemName, tocItem.getPage(), 
              0.0F, position, parent.getOutline().getName(), tocItem.getTitle()
          );
          // creating anchor for outline
          if (isDestinationsSupported()) {
            writer.getDirectContent().localDestination(itemName, new PdfDestination(PdfDestination.FITH, position));
          }
          tocItem.setOutline(outline);
          addOutline(outline);
          unnumberedTocItems.remove(i);
          return tocItem;
        }
      }
    }
    return null;
  }

  /**
   * Processes all currently unnumbered TOC items. 
   * @param writer The PDF writer to use.
   */
  protected void flushTocItems(PdfWriter writer) {
    if (unnumberedTocItems != null) {
      int numUnnumberedTocItems = unnumberedTocItems.size();
      for (int i = 0; i < numUnnumberedTocItems; i++) {
        TocItem tocItem = (TocItem) unnumberedTocItems.get(i);
        tocItem.setPage(getPageNumber(writer));
      }
      unnumberedTocItems.clear();
    }
  }
  
  /**
   * Getter method for the the property dataAccessor.
   * @return The current value of property dataAccessor.
   */
  public DocumentDataAccessor getDataAccessor() {
    return dataAccessor;
  }
  /**
   * Setter method for the the property dataAccessor.
   * @param dataAccessor The value to set for the property dataAccessor.
   */
  protected void setDataAccessor(DocumentDataAccessor dataAccessor) {
    this.dataAccessor = dataAccessor;
  }

  /**
   * Getter method for the the property tagInterceptor.
   * @return The current value of property tagInterceptor.
   */
  public DocumentTagInterceptor getTagInterceptor() {
    return tagInterceptor;
  }
  /**
   * Setter method for the the property tagInterceptor.
   * @param tagInterceptor The value to set for the property tagInterceptor.
   */
  public void setTagInterceptor(DocumentTagInterceptor tagInterceptor) {
    this.tagInterceptor = tagInterceptor;
  }

  /**
   * Getter method for the the property contentFilter.
   * @return The current value of property contentFilter.
   */
  public ContentFilter getContentFilter() {
    return contentFilter;
  }
  /**
   * Setter method for the the property contentFilter.
   * @param contentFilter The value to set for the property contentFilter.
   */
  public void setContentFilter(ContentFilter contentFilter) {
    this.contentFilter = contentFilter;
  }

  /**
   * Gets a the resource registry.
   * @return The current resource registry.
   */
  public Map getResourceRegistry() {
    return resourceRegistry;
  }
  
  /**
   * Gets a document resource from the registry.
   * @param key The key of the document resource to get. 
   * @return The desired resource or null.
   */
  public Object getResource(Object key) {
    return resourceRegistry.get(key);
  }
  /**
   * Registers a document resource from the registry.
   * @param key The key of the document resource to register. 
   * @param resource The resource to register.
   */
  public void registerResource(Object key, Object resource) {
    resourceRegistry.put(key, resource);
  }
  /**
   * Removes a document resource from the registry.
   * @param key The key of the document resource to remove. 
   * @return The desired resource or null.
   */
  public Object removeResource(Object key) {
    return resourceRegistry.remove(key);
  }


  /**
   * Getter method for the the property dynamicContentContainer.
   * @return The current value of property dynamicContentContainer.
   */
  public DynamicContentContainer getDynamicContentContainer() {
    return dynamicContentContainer;
  }
  /**
   * Setter method for the the property dynamicContentContainer.
   * @param dynamicContentContainer The value to set for the property dynamicContentContainer.
   */
  public void setDynamicContentContainer(DynamicContentContainer dynamicContentContainer) {
    this.dynamicContentContainer = dynamicContentContainer;
  }

  /**
   * Gets the vertical position on the current page. 
   * @return The vertical position on the current page.
   */
  public float getVerticalPosition() {
    return ((PdfWriter) documentWriter).getVerticalPosition(true);
  }
  
  /**
   * @see org.xml.sax.ContentHandler#startDocument()
   */
  public void startDocument() throws SAXException {
    try {
      if (dataAccessor != null) {
        dataAccessor.startup();
      }
    } catch (DocumentHandlerException ex) {
      // rethrow the exception as SAX exception
      throw new SAXException(ex.getMessage(), ex);
    }
    
    if (templateCopyWriter != null) {
      // copying template for later use
      templateCopyWriter.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
    }
  }

  /**
   * @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
   */
  public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
    try {
      if (templateCopyWriter != null) {
        // copying template for later use
        templateCopyWriter.write('<');
        templateCopyWriter.print(localName);
        int numAttribs = attributes.getLength();
        for (int i = 0; i < numAttribs; i++) {
          templateCopyWriter.write(' ');
          templateCopyWriter.print(attributes.getLocalName(i));
          templateCopyWriter.print("=\"");
          templateCopyWriter.print(attributes.getValue(i));
          templateCopyWriter.write('\"');
        }
        templateCopyWriter.write('>');
      }

      BaseDocumentTag item = null;
      if (DynamicContentContainer.TAG_NAME.equals(localName)) {
        item = getDynamicContentContainer();
      } else {
        item = tagFactory.createTagInstance(this, localName);
      }
      BaseDocumentTag topItem = topItem();
      item.setDocumentHandler(this);
      item.setAttributes(new AttributesImpl(attributes));
      item.setDepth(documentStack.size());
      item.setParentItem(topItem);
      item.setPreviousItem(lastItem);
      
      // checking tag structure
      if ((topItem != null) && !topItem.isVirtualContainer() && !item.isDocumentRoot() &&
          !item.isStructureChecksDisabled()) {
        boolean legalChild = false;
        ChildDefinitionMap childDefs = topItem.getSupportedChilds();
        ChildDefinition childDef = childDefs.getCompatibleDefinition(item);
        if (childDef != null) {
          legalChild = true;
        }
        if (!legalChild) {
          if (strictlyCheckDocumentStructure) {
            log.error("At line " + locator().getLineNumber() + ": Detected illegal nested tag <" + item.getName() + "> below tag <" + topItem.getName() + ">.");
            throw new DocumentHandlerException(locator(), "Detected illegal nested tag <" + item.getName() + "> below tag <" + topItem.getName() + ">.");
          }
          log.warn("At line " + locator().getLineNumber() + ": Detected illegal nested tag <" + item.getName() + "> below tag <" + topItem.getName() + ">.");
        }
      }
      
      // flushing the content of the current top item
      if (topItem != null) {
        topItem.flushContent();
      }

      DocumentEventRecorder recorder = topEventRecorder();
      if (recorder != null) {
        // recording event
        recorder.recordEvent(new DocumentEvent(DocumentEvent.OPEN, item, null));
      }

      // initializes the item
      item.preInitialize();
      item.initialize();
      // checking item attributes
      item.checkAttributes();
      if ((tagInterceptor == null) || tagInterceptor.beforeOpen(item)) {
        // opening the item.
        item.openItem();
        if (tagInterceptor != null) {
          tagInterceptor.afterOpen(item);
        }
      }
      // resetting the previous item to give the garbage collector a chance to clean up
      item.setPreviousItem(null);
      // putting the item on top of the stack.
      pushItem(item);
      // setting the last item
      lastItem = item;
    } catch (DocumentHandlerException ex) {
      // re-throwing the exception as SAX exception
      throw new SAXException(ex.getMessage(), ex);
    }
  }

  /**
   * @see org.xml.sax.ContentHandler#characters(char[], int, int)
   */
  public void characters(char[] ch, int start, int length) throws SAXException {
    if (templateCopyWriter != null) {
      // copying template for later use
      int offset = start;
      int endIdx = start + length;
      char c = 0;
      String entity = null;
      for (int i = start; i < endIdx; i++) {
        c = ch[i];
        // recreating common XML entities
        switch (c) {
          case '&':
            entity = "&amp;";
            break;

          case '<':
            entity = "&lt;";
            break;
 
          case '>':
            entity = "&gt;";
            break;

          case '\"':
            entity = "&quot;";
            break;

          case '\'':
            entity = "&apos;";
            break;
        }
        
        if (entity != null) {
          templateCopyWriter.write(ch, offset, i - offset);
          offset = i + 1;
          templateCopyWriter.write(entity);
          entity = null;
        }
      }
      templateCopyWriter.write(ch, offset, endIdx - offset);
    }
    BaseDocumentTag item = topItem();
    int effectiveStart = start;
    int effectiveLength = length;
    if (item.isItemOpened()) {
      // ignoring all leading whitespaces
      while ((effectiveLength > 0) && Character.isWhitespace(ch[effectiveStart])) {
        ++effectiveStart;
        --effectiveLength;
      }
      
      if (!item.isTrimBody()) {
        if ((effectiveStart - start) >= 1) {
          --effectiveStart;
          ++effectiveLength;
        }
      }

      if (effectiveLength == 0) {
        // ignoring completely empty content
        return;
      }
    }
    
    // resets the opened flag
    item.setItemOpened(false);
    
    // trimming the content
    String trimmedContent = trimContent(ch, effectiveStart, effectiveLength);
    // adding the content to the item
    item.addContent(trimmedContent);

    DocumentEventRecorder recorder = topEventRecorder();
    if (recorder != null) {
      // recording event
      recorder.recordEvent(new DocumentEvent(DocumentEvent.CHARACTERS, item, trimmedContent));
    }
  }

  /**
   * Trims the content.
   * @param ch The character set.
   * @param start The start index of the interresting area.
   * @param length The end index of the interresting area.
   * @return The trimmed text.
   */
  private String trimContent(char[] ch, int start, int length) {  
    StringBuffer buf = new StringBuffer();
    for (int i = 0; i < length; i++) {
      char c = ch[start + i];
      boolean whitespaceDetected = Character.isWhitespace(c);
      while (whitespaceDetected) {
        ++i;
        if (i >= length) {
          --i;
          break;
        }
        c = ch[start + i];
        if (!Character.isWhitespace(c)) {
          --i;
          break;
        }
      }
      if (whitespaceDetected) {
        // replacing each whitespace sequence with a single blank
        buf.append(' ');
        continue;
      }
      // character seems to be a regular one 
      buf.append(c); 
    }
    return buf.toString();
  }

  /**
   * @see org.xml.sax.ContentHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
   */
  public void endElement(String uri, String localName, String qName) throws SAXException {
    BaseDocumentTag item = topItem();
    try {
      if (templateCopyWriter != null) {
        // copying template for later use
        templateCopyWriter.print("</");
        templateCopyWriter.print(localName);
        templateCopyWriter.print(">");
      }
      DocumentEventRecorder recorder = topEventRecorder();
      if (recorder != null) {
        // recording event
        recorder.recordEvent(new DocumentEvent(DocumentEvent.CLOSE, item, item.getContent()));
      }
      // closing the item again
      item.setItemClosed(true);
      
      Map customAttributes = item.getCustomAttributes();
      String customBody = null;
      if (customAttributes != null) {
        customBody = (String) customAttributes.get("body");
      }
      String tagBody = item.getContent();
      if ((tagBody != null) && (tagBody.length() > 0)) {
        templateContext.setProperty("body", tagBody);
      }
      if (customBody == null) {
        item.flushContent();
      } else {
        item.resetContent();
        item.addContent(customBody);
        item.flushContent();
      }
      if ((tagInterceptor == null) || tagInterceptor.beforeClose(item)) {
        item.closeItem();
        if (tagInterceptor != null) {
          tagInterceptor.afterClose(item);
        }
      }
      popItem();
    } catch (DocumentHandlerException ex) {
      // rethrow the exception as SAX exception
      throw new SAXException(ex.getMessage(), ex);
    }
    // setting the last item
    lastItem = item;
  }

  /**
   * @see org.xml.sax.ContentHandler#endDocument()
   */
  public void endDocument() throws SAXException {
    if ((documentWriter != null) && documentStack.isEmpty()) {
      // finally closing document writer, to make sure everything 
      // has been written properly to the output stream when the 
      // DocumentPrinter has finished its Job.
      documentWriter.close();
    }
    
    if (templateCopyWriter != null) {
      templateCopyWriter.flush();
    }

    try {
      if (dataAccessor != null) {
        dataAccessor.shutdown();
      }
    } catch (DocumentHandlerException ex) {
      // rethrow the exception as SAX exception
      throw new SAXException(ex.getMessage(), ex);
    }
  }

  /**
   * Getter method for the the property templateCopyWriter.
   * @return The current value of property templateCopyWriter.
   */
  public PrintWriter getTemplateCopyWriter() {
    return templateCopyWriter;
  }
  /**
   * Setter method for the the property templateCopyWriter.
   * @param templateCopyWriter The value to set for the property templateCopyWriter.
   */
  protected void setTemplateCopyWriter(PrintWriter templateCopyWriter) {
    this.templateCopyWriter = templateCopyWriter;
  }
  
  /**
   * Disables the template copy writer to avoid the copy of internally processed XML stuff.
   */
  public void disableTemplateCopyWriter() {
    this.disabledTemplateCopyWriter = this.templateCopyWriter;
  }
  /**
   * Enables the template copy writer again.
   */
  public void enableTemplateCopyWriter() {
    this.templateCopyWriter = this.disabledTemplateCopyWriter;
  }

  /**
   * Getter method for the the property attributeHandler.
   * @return The current value of property attributeHandler.
   */
  public AttributeHandler getAttributeHandler() {
    return attributeHandler;
  }
  /**
   * Setter method for the the property attributeHandler.
   * @param attributeHandler The value to set for the property attributeHandler.
   */
  public void setAttributeHandler(AttributeHandler attributeHandler) {
    this.attributeHandler = attributeHandler;
    attributeHandler.setDocumentHandler(this);
    if (styleParser != null) {
      styleParser.setAttributeHandler(attributeHandler);
    }
  }

  /**
   * Getter method for the the property styleParser.
   * @return The current value of property styleParser.
   */
  public StyleParser getStyleParser() {
    return styleParser;
  }
  /**
   * Setter method for the the property styleParser.
   * @param styleParser The value to set for the property styleParser.
   */
  public void setStyleParser(StyleParser styleParser) {
    this.styleParser = styleParser;
    styleParser.setAttributeHandler(attributeHandler);
  }

  /**
   * Getter method for the the property pageNumberOffset.
   * @return The current value of property pageNumberOffset.
   */
  public int getPageNumberOffset() {
    return pageNumberOffset;
  }

  /**
   * Setter method for the the property pageNumberOffset.
   * @param pageNumberOffset The value to set for the property pageNumberOffset.
   */
  public void setPageNumberOffset(int pageNumberOffset) {
    this.pageNumberOffset = pageNumberOffset;
  }
  
  /**
   * Clears all references.
   */
  public void clearReferences() {
    if (outlineItems != null) {
      this.outlineItems.clear();
    }
    if (unnumberedTocItems != null) {
      this.unnumberedTocItems.clear();
    }
    if (tocItems != null) {
      this.tocItems.clear();
    }
    if (tocItemMap != null) {
      this.tocItemMap.clear();
    }
  }

  /**
   * Resets all references.
   */
  public void resetReferences() {
    if (unnumberedTocItems != null) {
      this.unnumberedTocItems.clear();
    }
    //this.tocItems.clear();
    if (outlineItems != null) {
      this.outlineItems.clear();
    }
  }

  /**
   * Getter method for the the property tocFreezed.
   * @return The current value of property tocFreezed.
   */
  public boolean isTocFreezed() {
    return tocFreezed;
  }
  /**
   * Setter method for the the property tocFreezed.
   * @param tocFreezed The value to set for the property tocFreezed.
   */
  public void setTocFreezed(boolean tocFreezed) {
    this.tocFreezed = tocFreezed;
  }

  /**
   * Getter method for the the property destinationsSupported.
   * @return The current value of property destinationsSupported.
   */
  public boolean isDestinationsSupported() {
    return destinationsSupported;
  }
  /**
   * Setter method for the the property destinationsSupported.
   * @param destinationsSupported The value to set for the property destinationsSupported.
   */
  public void setDestinationsSupported(boolean destinationsSupported) {
    this.destinationsSupported = destinationsSupported;
  }

  /**
   * Gets the meta data.
   * @return The meta data.
   */
  public DocumentMetaData getMetaData() {
    return metaData;
  }
  /**
   * Gets the XMP meta data.
   * @return The XMP meta data.
   */
  private byte[] getXmpMetaData() {
    if (xmpMetaData != null) {
      return xmpMetaData;
    }
    if (metaData == null) {
      return null;
    }
    
    ByteArrayOutputStream os = new ByteArrayOutputStream();
    PdfDictionary dict = new PdfDictionary();
    if (metaData.getTitle() != null) {
      dict.put(PdfName.TITLE, new PdfString(metaData.getTitle()));
    }
    if (metaData.getAuthor() != null) {
      dict.put(PdfName.AUTHOR, new PdfString(metaData.getAuthor()));
    }
    if (metaData.getSubject() != null) {
      dict.put(PdfName.SUBJECT, new PdfString(metaData.getSubject()));
    }
    if (metaData.getKeywords() != null) {
      dict.put(PdfName.KEYWORDS, new PdfString(metaData.getKeywords()));
    }
    try {
      XmpWriter xmp = new XmpWriter(os, dict, metaData.getPdfxConformance());
      xmp.close();
    } catch (IOException ex) {
      // this should never happen!
      log.error("Failed to generate XMP meta data", ex);
    }
    xmpMetaData = os.toByteArray();
    return xmpMetaData;
  }
  /**
   * Sets the meta data.
   * @param metaData The meta data to set.
   */
  public void setMetaData(DocumentMetaData metaData) {
    this.metaData = metaData;
    this.xmpMetaData = null;
  }

}
