/*
 * Copyright (C) 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print;

import java.util.HashMap;
import java.util.Map;

/**
 * Name: DocumentHandlerComponentRegistry<br>
 * Description: A singleton maintaining static resources for the document handler.
 * 
 * @author lauerc
 */
public class DocumentHandlerComponentRegistry {

  /** The one and only instance. */
  private static DocumentHandlerComponentRegistry theInstance = new DocumentHandlerComponentRegistry(); 
  
  /** A map holding the known supported attribute maps. */
  private Map supportedAttributeRegistry = new HashMap();
  /** A map holding the known supported style attribute maps. */
  private Map supportedStyleAttributeRegistry = new HashMap();
  /** A map holding the known supported child definition maps. */
  private Map childDefinitionMapRegistry = new HashMap();

  /**
   * Gets the singleton. 
   * @return The one and only instance.
   */
  public static final DocumentHandlerComponentRegistry getInstance() {
    return theInstance;
  }
  
  /**
   * Gets the supported attribute definition map for the given tag instance.
   * In case the attribute definitions are already defined, the value from the registry is returned,
   * else the tags buildSupportedAttributes() is called and the created map is registerd and
   * returned.
   * @param tag The tag to get the supported attributes for.
   * @return The determined supported attribute definitios. 
   */
  public AttributeDefinitionMap getSupportedAttributes(BaseDocumentTag tag) {
    Class tagClass = tag.getClass();
    AttributeDefinitionMap attrDefs = (AttributeDefinitionMap) supportedAttributeRegistry.get(tagClass);
    if (attrDefs == null) {
      synchronized (supportedAttributeRegistry) {
        attrDefs = tag.buildSupportedAttributes();
        supportedAttributeRegistry.put(tagClass, attrDefs);
      }
    }
    return attrDefs;
  }
  
  /**
   * Gets the supported style attribute definition map for the given tag instance.
   * In case the attribute definitions are already defined, the value from the registry is returned,
   * else the tags buildSupportedStyleAttributes() is called and the created map is registerd and
   * returned.
   * @param tag The tag to get the supported attributes for.
   * @return The determined supported attribute definitios. 
   */
  public AttributeDefinitionMap getSupportedStyleAttributes(BaseDocumentTag tag) {
    Class tagClass = tag.getClass();
    AttributeDefinitionMap attrDefs = (AttributeDefinitionMap) supportedStyleAttributeRegistry.get(tagClass);
    if (attrDefs == null) {
      synchronized (supportedStyleAttributeRegistry) {
        attrDefs = tag.buildSupportedStyleAttributes();
        supportedStyleAttributeRegistry.put(tagClass, attrDefs);
      }
    }
    return attrDefs;
  } 
  
  /**
   * Gets the supported style attribute definition map for the given tag instance.
   * In case the attribute definitions are already defined, the value from the registry is returned,
   * else the tags buildSupportedStyleAttributes() is called and the created map is registerd and
   * returned.
   * @param tag The tag to get the supported attributes for.
   * @return The determined supported attribute definitios. 
   */
  public ChildDefinitionMap getSupportedChilds(BaseDocumentTag tag) {
    Class tagClass = tag.getClass();
    ChildDefinitionMap childDefs = (ChildDefinitionMap) childDefinitionMapRegistry.get(tagClass);
    if (childDefs == null) {
      synchronized (childDefinitionMapRegistry) {
        childDefs = tag.buildSupportedChilds();
        childDefinitionMapRegistry.put(tagClass, childDefs);
      }
    }
    return childDefs;
  } 

}
