/*
 * Copyright (C) 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print;

/**
 * Title:       DocumentMetaData<br>
 * Description: Holds meta data for PDF documents.<br>
 *
 * @author lauerc
 */
public class DocumentMetaData {

  /** The document title. */
  private String title;
  /** The document author. */
  private String author;
  /** The document subject. */
  private String subject;
  /** The document's keywords. */
  private String keywords;
  /**
   * The PDF version, defaults to Version 1.4.
   * @see com.lowagie.text.pdf.PdfWriter#VERSION_1_2
   * @see com.lowagie.text.pdf.PdfWriter#VERSION_1_3
   * @see com.lowagie.text.pdf.PdfWriter#VERSION_1_4
   * @see com.lowagie.text.pdf.PdfWriter#VERSION_1_5
   * @see com.lowagie.text.pdf.PdfWriter#VERSION_1_6
   * @see com.lowagie.text.pdf.PdfWriter#VERSION_1_7
   */
  private char pdfVersion = com.lowagie.text.pdf.PdfWriter.VERSION_1_4;
  /**
   * The PDF-X Conformance, defaults to PDF/A-1A.
   * @see com.lowagie.text.pdf.PdfWriter#PDFXNONE
   * @see com.lowagie.text.pdf.PdfWriter#PDFX1A2001
   * @see com.lowagie.text.pdf.PdfWriter#PDFX32002
   * @see com.lowagie.text.pdf.PdfWriter#PDFA1A
   * @see com.lowagie.text.pdf.PdfWriter#PDFA1B
   */
  private int pdfxConformance = com.lowagie.text.pdf.PdfWriter.PDFA1A;

  /**
   * Constructs a DocumentMetaData instance with no specific attributes.
   */
  public DocumentMetaData() {
  }

  /**
   * Constructs a DocumentMetaData instance with specific attributes.
   * @param title The initial title.
   * @param author The initial author.
   * @param subject The initial subject.
   * @param keywords The initial keywords.
   * @param pdfxConformance The initial PDF-X conformance.
   */
  public DocumentMetaData(String title, String author, String subject, String keywords, int pdfxConformance) {
    this.title = title;
    this.author = author;
    this.subject = subject;
    this.keywords = keywords;
    this.pdfxConformance = pdfxConformance;
  }

  /**
   * Getter method for the the property title.
   * @return The current value of property title.
   */
  public String getTitle() {
    return title;
  }
  /**
   * Setter method for the the property title.
   * @param title The value to set for the property title.
   * @return A reference to the current instance.
   */
  public DocumentMetaData setTitle(String title) {
    this.title = title;
    return this;
  }
  
  /**
   * Getter method for the the property author.
   * @return The current value of property author.
   */
  public String getAuthor() {
    return author;
  }
  /**
   * Setter method for the the property author.
   * @param author The value to set for the property author.
   * @return A reference to the current instance.
   */
  public DocumentMetaData setAuthor(String author) {
    this.author = author;
    return this;
  }
  
  /**
   * Getter method for the the property subject.
   * @return The current value of property subject.
   */
  public String getSubject() {
    return subject;
  }
  /**
   * Setter method for the the property subject.
   * @param subject The value to set for the property subject.
   * @return A reference to the current instance.
   */
  public DocumentMetaData setSubject(String subject) {
    this.subject = subject;
    return this;
  }

  /**
   * Getter method for the the property keywords.
   * @return The current value of property keywords.
   */
  public String getKeywords() {
    return keywords;
  }
  /**
   * Setter method for the the property keywords.
   * @param keywords The value to set for the property keywords.
   * @return A reference to the current instance.
   */
  public DocumentMetaData setKeywords(String keywords) {
    this.keywords = keywords;
    return this;
  }
  
  /**
   * Getter method for the the property pdfVersion.
   * @return The current value of property pdfVersion.
   */
  public char getPdfVersion() {
    return pdfVersion;
  }
  /**
   * Setter method for the the property pdfVersion.
   * @param pdfVersion The value to set for the property pdfVersion.
   * @return A reference to the current instance.
   */
  public DocumentMetaData setPdfVersion(char pdfVersion) {
    this.pdfVersion = pdfVersion;
    return this;
  }
  
  /**
   * Getter method for the the property pdfxConformance.
   * @return The current value of property pdfxConformance.
   */
  public int getPdfxConformance() {
    return pdfxConformance;
  }
  /**
   * Setter method for the the property pdfxConformance.
   * @param pdfxConformance The value to set for the property pdfxConformance.
   * @return A reference to the current instance.
   */
  public DocumentMetaData setPdfxConformance(int pdfxConformance) {
    this.pdfxConformance = pdfxConformance;
    return this;
  }
  
}
