/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.Reader;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;

import java.sql.Timestamp;
import java.text.SimpleDateFormat;

import java.util.Date;
import java.util.Locale;
import java.util.Map;
import java.util.List;
import java.util.HashMap;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Properties;
import java.util.StringTokenizer;

import org.ujac.util.UjacTypes;
import org.ujac.util.console.CommandLineHelper;

import org.ujac.util.io.FileResourceLoader;
import org.ujac.util.io.NullDeviceOutputStream;
import org.ujac.util.io.ResourceLoader;

import org.ujac.util.template.TemplateContext;
import org.ujac.util.template.TemplateInterpreter;

import org.ujac.util.table.Column;
import org.ujac.util.table.LayoutHints;
import org.ujac.util.table.Row;
import org.ujac.util.table.RowNotDefinedException;
import org.ujac.util.table.DataTable;
import org.ujac.util.table.TableException;

import org.ujac.util.text.FormatHelper;

import org.ujac.util.xml.XmlUtils;

import org.xml.sax.XMLReader;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Name: DocumentPrinter<br>
 * Description: A class printing xml based documents.<br>
 * 
 * @author lauerc
 */
public class DocumentPrinter {

  /** The template source. */
  private InputSource templateSource = null;
  /** The document tag factory. */
  private DocumentTagFactory tagFactory = null;
  /** The template interpreter to use. */
  private TemplateInterpreter templateInterpeter = null;
  /** The format helper to use. */
  private FormatHelper formatHelper = null;
  /** The properties. */
  private Map properties = null;
  /** The XML reader implementation. */
  private String xmlReaderClass = null;
  /** The resource loader. */
  private ResourceLoader resourceLoader = null;
  /** The data accessor. */
  private DocumentDataAccessor dataAccessor = null;
  /** The document tag interceptor. */
  private DocumentTagInterceptor tagInterceptor = null;
  /** The content filter. */
  private ContentFilter contentFilter = new DefaultContentFilter();
  /** The module list. */
  private List modules = null;
  /** The document meta data. */
  private DocumentMetaData metaData = null;
  
  
  /** Tells whether or not to translate escape sequences (defaults to true). */
  private boolean translateEscapeSequences = true;
  /** Tells whether or not to parse dyamic content for XML contents (defaults to true). */
  private boolean parseDynamicContent = true;
  /** Tells whether or not to evaluate expressions (defaults to true). */
  private boolean evaluateExpressions = true;
  /** Tells whether or not to cache images (defaults to true). */
  private boolean cacheImages = true;
  /** 
   * Tells whether or not to strictly check document structure (defaults to true).
   * In case this flag is set, any document structure violation will cause an exception. 
   */
  private boolean strictlyCheckDocumentStructure = true;
  /** The expected number of pages (needed to define the size for the number of pages template). */
  private int expectedNumberOfPages = 999;
  /** Tells whether or not the page numering starts after the head section. */
  private boolean pageNumberingStartsAfterHead = true;
  
  /**
   * Constructs a DocumentPrinter instance with specific attributes.
   * In case the system propery 'org.xml.sax.driver' is defined,
   * overwrites the attribute xmlReaderClassName with that value.
   */
  public DocumentPrinter() {
    this(null);
  }

  /**
   * Constructs a DocumentPrinter instance with specific attributes.
   * In case the system propery 'org.xml.sax.driver' is defined,
   * overwrites the attribute xmlReaderClassName with that value.
   * @param tagFactory The document tag factory.
   */
  public DocumentPrinter(DocumentTagFactory tagFactory) {
    this(null, tagFactory);
  }
  
  /**
   * Constructs a DocumentPrinter instance with specific arguments.
   * @param templateStream The template input stream.
   * @param properties The document's properties.
   */
  public DocumentPrinter(InputStream templateStream, Map properties) {
    this(properties, null);
    this.templateSource = new InputSource(templateStream);
  }
  /**
   * Constructs a DocumentPrinter instance with specific arguments.
   * @param templateStream The template input stream.
   * @param properties The document's properties.
   * @param tagFactory The document tag factory.
   */
  public DocumentPrinter(InputStream templateStream, Map properties, 
                         DocumentTagFactory tagFactory) {
    
    this(properties, tagFactory);
    this.templateSource = new InputSource(templateStream);
  }

  /**
   * Constructs a DocumentPrinter instance with specific arguments.
   * @param templateReader The template reader.
   * @param properties The document's properties.
   */
  public DocumentPrinter(Reader templateReader, Map properties) {
    this(properties, null);
    this.templateSource = new InputSource(templateReader);
  }
  /**
   * Constructs a DocumentPrinter instance with specific arguments.
   * @param templateReader The template reader.
   * @param properties The document's properties.
   * @param tagFactory The document tag factory.
   */
  public DocumentPrinter(Reader templateReader, Map properties,
                         DocumentTagFactory tagFactory) {
    
    this(properties, tagFactory);
    this.templateSource = new InputSource(templateReader);
  }

  /**
   * Constructs a DocumentPrinter instance with specific attributes.
   * In case the system propery 'org.xml.sax.driver' is defined,
   * overwrites the attribute xmlReaderClassName with that value.
   * @param properties The document properties.
   * @param tagFactory The document tag factory.
   */
  private DocumentPrinter(Map properties, DocumentTagFactory tagFactory) {
    this.properties = properties;
    if (tagFactory == null) {
      this.tagFactory = DefaultDocumentTagFactory.getInstance();
    } else {
      this.tagFactory = tagFactory;
    }
  }

  /**
   * Getter method for the the property templateInterpeter.
   * @return The current value of property templateInterpeter.
   */
  public TemplateInterpreter getTemplateInterpreter() {
    return templateInterpeter;
  }
  /**
   * Setter method for the the property templateInterpeter.
   * @param templateInterpeter The value to set for the property templateInterpeter.
   */
  public void setTemplateInterpreter(TemplateInterpreter templateInterpeter) {
    this.templateInterpeter = templateInterpeter;
  }

  /**
   * Gets the format helper.
   * @return The format helper.
   */
  public FormatHelper getFormatHelper() {
    return formatHelper;
  }
  /**
   * Sets the format helper.
   * @param formatHelper The format helper to set.
   */
  public void setFormatHelper(FormatHelper formatHelper) {
    this.formatHelper = formatHelper;
  }
  
  /**
   * Sets the template source.
   * @param templateStream The input stream to read from.
   */
  public void setTemplateSource(InputStream templateStream) {
    this.templateSource = new InputSource(templateStream);
  }

  /**
   * Sets the template source.
   * @param templateReader The reader to read from.
   */
  public void setTemplateSource(Reader templateReader) {
    this.templateSource = new InputSource(templateReader);
  }

  /**
   * Gets the document properties.
   * @return The document properties.
   */
  public Map getProperties() {
    return properties;
  }
  /**
   * Sets the document properties.
   * @param properties The document's properties.
   */
  public void setProperties(Map properties) {
    this.properties = properties;
  }

  /**
   * Gets the XML reader implementation.
   * @return The class name of the current XML reader implementation.
   */
  public String getXmlReaderClass() {
    return xmlReaderClass;
  }
  /**
   * Sets the XML reader implementation.
   * @param xmlReaderClass The class name of the XML reader implementation 
   *    to set, only non null values are accepted, null values are ignored.
   */
  public void setXmlReaderClass(String xmlReaderClass) {
    if (xmlReaderClass != null) {
      this.xmlReaderClass = xmlReaderClass;
    }
  }
  
  /**
   * Creates a XMLReader instance.
   * @return The created XMLReader instance.
   * @exception SAXException If no default XMLReader class
   *  can be identified and instantiated or the class cannot be
   *  loaded, instantiated, and cast to XMLReader. 
   */
  protected XMLReader createXmlReader() throws SAXException {
    return XmlUtils.createXmlReader(getXmlReaderClass());
  }
  
  /**
   * Gets the resource loader.
   * @return The current resource loader.
   */
  public ResourceLoader getResourceLoader() {
    return this.resourceLoader;
  }
  /**
   * Sets the resource loader.
   * @param resourceLoader The resource loader to set.
   */
  public void setResourceLoader(ResourceLoader resourceLoader) {
    this.resourceLoader = resourceLoader;
  }

  /**
   * Sets the data accessor component.
   * @param dataAccessor The data accessor to set.
   */
  public void setDataAccessor(DocumentDataAccessor dataAccessor) {
    this.dataAccessor = dataAccessor;
  }

  /**
   * Getter method for the the property tagInterceptor.
   * @return The current value of property tagInterceptor.
   */
  public DocumentTagInterceptor getTagInterceptor() {
    return tagInterceptor;
  }
  /**
   * Setter method for the the property tagInterceptor.
   * @param tagInterceptor The value to set for the property tagInterceptor.
   */
  public void setTagInterceptor(DocumentTagInterceptor tagInterceptor) {
    this.tagInterceptor = tagInterceptor;
  }
  
  /**
   * Getter method for the the property contentFilter.
   * @return The current value of property contentFilter.
   */
  public ContentFilter getContentFilter() {
    return contentFilter;
  }
  /**
   * Setter method for the the property contentFilter.
   * @param contentFilter The value to set for the property contentFilter.
   */
  public void setContentFilter(ContentFilter contentFilter) {
    this.contentFilter = contentFilter;
  }

  /**
   * Getter method for the the property translateEscapeSequences.
   * @return The current value of property translateEscapeSequences.
   */
  public boolean isTranslateEscapeSequences() {
    return translateEscapeSequences;
  }
  /**
   * Setter method for the the property translateEscapeSequences.
   * @param translateEscapeSequences The value to set for the property translateEscapeSequences.
   */
  public void setTranslateEscapeSequences(boolean translateEscapeSequences) {
    this.translateEscapeSequences = translateEscapeSequences;
  }

  /**
   * Getter method for the the property parseDynamicContent.
   * @return The current value of property parseDynamicContent.
   */
  public boolean isParseDynamicContent() {
    return parseDynamicContent;
  }
  /**
   * Setter method for the the property parseDynamicContent.
   * @param parseDynamicContent The value to set for the property parseDynamicContent.
   */
  public void setParseDynamicContent(boolean parseDynamicContent) {
    this.parseDynamicContent = parseDynamicContent;
  }

  /**
   * Getter method for the the property evaluateExpressions.
   * @return The current value of property evaluateExpressions.
   */
  public boolean isEvaluateExpressions() {
    return evaluateExpressions;
  }
  /**
   * Setter method for the the property evaluateExpressions.
   * @param evaluateExpressions The value to set for the property evaluateExpressions.
   */
  public void setEvaluateExpressions(boolean evaluateExpressions) {
    this.evaluateExpressions = evaluateExpressions;
  }
  
  /**
   * Getter method for the the property cacheImages.
   * @return The current value of property cacheImages.
   */
  public boolean isCacheImages() {
    return cacheImages;
  }
  /**
   * Setter method for the the property cacheImages.
   * @param cacheImages The value to set for the property cacheImages.
   */
  public void setCacheImages(boolean cacheImages) {
    this.cacheImages = cacheImages;
  }


  /**
   * Getter method for the the property strictlyCheckDocumentStructure.
   * @return The current value of property strictlyCheckDocumentStructure.
   */
  public boolean isStrictlyCheckDocumentStructure() {
    return strictlyCheckDocumentStructure;
  }
  /**
   * Setter method for the the property strictlyCheckDocumentStructure.
   * @param strictlyCheckDocumentStructure The value to set for the property strictlyCheckDocumentStructure.
   */
  public void setStrictlyCheckDocumentStructure(
      boolean strictlyCheckDocumentStructure) {
    this.strictlyCheckDocumentStructure = strictlyCheckDocumentStructure;
  }

  /**
   * Getter method for the the property expectedNumberOfPages.
   * @return The current value of property expectedNumberOfPages.
   */
  public int getExpectedNumberOfPages() {
    return expectedNumberOfPages;
  }
  /**
   * Setter method for the the property expectedNumberOfPages.
   * @param expectedNumberOfPages The value to set for the property expectedNumberOfPages.
   */
  public void setExpectedNumberOfPages(int expectedNumberOfPages) {
    this.expectedNumberOfPages = expectedNumberOfPages;
  }

  /**
   * Getter method for the the property pageNumberingStartsAfterHead.
   * @return The current value of property pageNumberingStartsAfterHead.
   */
  public boolean isPageNumberingStartsAfterHead() {
    return pageNumberingStartsAfterHead;
  }
  /**
   * Setter method for the the property pageNumberingStartsAfterHead.
   * @param pageNumberingStartsAfterHead The value to set for the property pageNumberingStartsAfterHead.
   */
  public void setPageNumberingStartsAfterHead(boolean pageNumberingStartsAfterHead) {
    this.pageNumberingStartsAfterHead = pageNumberingStartsAfterHead;
  }

  
  /**
   * Loads a module from a resource loader.
   * @param modulePath The path that leads to the module.
   * @exception IOException In case the given directory couldn't be accessed.
   */
  public void loadModule(String modulePath) throws IOException {
    byte[] moduleCode = resourceLoader.loadResource(modulePath);
    if (modules == null) {
      modules = new ArrayList();
    }
    modules.add(moduleCode);
  }

  /**
   * Prints the document to the given output stream.
   * @param outputStream The output stream to which to write the document.
   * @return Reference of the document handler, used for the document processing.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public DocumentHandler printDocument(OutputStream outputStream) 
    throws DocumentHandlerException, IOException {
    
    return printDocument(outputStream, false);
  }
  
  /**
   * Prints the document to the given output stream.
   * @param outputStream The output stream to which to write the document.
   * @param supportToc Tells whether or not to support the output of a leading table of contents.
   *    Notice: if you use this option, the document will be generated in up to three document 
   *    processing steps which has negative performance impacts.
   * @return Reference of the document handler, used for the document processing.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public DocumentHandler printDocument(OutputStream outputStream, boolean supportToc) 
    throws DocumentHandlerException, IOException {
    
    // removing properties, which are internally used by the DocumentHandler.
    removeInternalDocumentProperties();
    
    if (!supportToc) {
      return printDocument(createDocumentHandler(outputStream), this.templateSource, this.modules);
    }

    File tmpSourceFile = null;
    OutputStream mainDocumentStream = null;
    // pass 1: collecting the TOC informations
    DocumentHandler documentHandler = null;
    
    int numPages = 0;
    // writing template to temporary file, to keep memory consumption low
    PrintWriter templateCopyWriter = null;
    try {
      // writing template to temporary file, to keep memory consumption low
      tmpSourceFile = File.createTempFile("ujac_print", "xml");
      mainDocumentStream = new NullDeviceOutputStream();
      documentHandler = createDocumentHandler(mainDocumentStream);
      documentHandler.setDestinationsSupported(false);
      documentHandler.clearReferences();
      documentHandler.setPageNumberOffset(0);
      documentHandler.setIncludeHeadSection(false);
      // the template needs to be interpreted twice or more ...
      templateCopyWriter = new PrintWriter(new FileWriter(tmpSourceFile));
      documentHandler.setTemplateCopyWriter(templateCopyWriter);
      // executing first processing step!
      printDocument(documentHandler, this.templateSource, this.modules);
      numPages = ((Integer) documentHandler.getTemplateContext().get("pageNumber")).intValue();
    } finally {
      documentHandler.setTemplateCopyWriter(null);
      if (templateCopyWriter != null) {
        templateCopyWriter.close();
      }
    }

    try {
      // pass 2: printing the document together with the TOC 
      documentHandler.setIncludeHeadSection(true);
      removeInternalDocumentProperties();
      if (pageNumberingStartsAfterHead) {
        documentHandler.setTocFreezed(true);
        documentHandler.resetReferences();
      } else {
        documentHandler.setTocFreezed(false);
        documentHandler.clearReferences();
      }
      documentHandler.reset(new NullDeviceOutputStream());
      printDocument(documentHandler, new InputSource(new FileReader(tmpSourceFile)), null);
      int numPagesInclToc = ((Integer) documentHandler.getTemplateContext().get("pageNumber")).intValue();

      // pass 3: recalculating the TOC numbers
      if (!pageNumberingStartsAfterHead) {
        removeInternalDocumentProperties();
        documentHandler.setTocFreezed(false);
        documentHandler.resetReferences();
        documentHandler.reset(new NullDeviceOutputStream());
        printDocument(documentHandler, new InputSource(new FileReader(tmpSourceFile)), null);
      }

      // pass 4: finally generating the destination document
      removeInternalDocumentProperties();
      if (pageNumberingStartsAfterHead) {
        documentHandler.setTocFreezed(true);
        documentHandler.resetReferences();
      } else {
        documentHandler.setTocFreezed(false);
        documentHandler.resetReferences();
      }
      documentHandler.reset(outputStream);
      int offset = 0;
      if (pageNumberingStartsAfterHead) {
        offset = numPages - numPagesInclToc; 
      }
      documentHandler.setPageNumberOffset(offset);
      documentHandler.setDestinationsSupported(true);
      return printDocument(documentHandler, new InputSource(new FileReader(tmpSourceFile)), null);
    } finally {
      tmpSourceFile.delete();
    }
  }

  /**
   * Removes document properties, which are internally used by the DocumentHandler.
   */
  protected void removeInternalDocumentProperties() {
    if (properties != null) {
      // removing properties, which are internally used by the DocumentHandler.
      properties.remove("chapterId");
      properties.remove("main_document");
      properties.remove("body");
      properties.remove("tocItem");
      properties.remove("pageNumber");
    }
  }
  
  /**
   * Prints the document to the given output stream.
   * @param documentHandler The document handler to use for XML processing.
   * @param templateSource The template to process.
   * @param modules The modules to use.
   * @return Reference of the document handler, used for the document processing.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  protected DocumentHandler printDocument(DocumentHandler documentHandler, InputSource templateSource, List modules) 
    throws DocumentHandlerException, IOException {

    try {
      long startTime = System.currentTimeMillis();

      XMLReader xmlReader = createXmlReader();
      
      xmlReader.setContentHandler(documentHandler);
      
      // disabling DTD/Schema based XML validation to avoid performance problems
      // and problems on systems that doesn't have internet access. 
      XmlUtils.disableEntityResolver(xmlReader);
        
      if (modules != null) {
        // loading modules into document handler
        int numModules = modules.size();
        for (int i = 0; i < numModules; i++) {
          byte[] moduleCode = (byte[]) modules.get(i);
          xmlReader.parse(new InputSource(new ByteArrayInputStream(moduleCode)));
        }
      }

      // parsing the main document
      xmlReader.parse(templateSource);
      long endTime = System.currentTimeMillis();
      if (System.getProperty("ujac.profile") != null) {
        System.out.println("Needed " + (endTime - startTime) + " ms to process document.");
      }
      
      return documentHandler;
    } catch (SAXException ex) {
      Exception cause = ex.getException();
      if (cause != null) {
        if (cause instanceof DocumentHandlerException) {
          throw ((DocumentHandlerException) cause);
        }
        String causeMsg = cause.getMessage();
        if ((cause instanceof ClassNotFoundException) && (causeMsg.indexOf(xmlReaderClass) >= 0)) {
          throw new DocumentHandlerException(documentHandler.locator(), "Unable to instantiate XML reader '" + xmlReaderClass + 
              "', check your class path or specify a different implementation by calling the method setXmlReaderClass or by " +
              "defining the system property 'org.xml.sax.driver'.", cause);
        }
        throw new DocumentHandlerException(documentHandler.locator(), causeMsg, cause);
      }
      throw new DocumentHandlerException(documentHandler.locator(), ex.getMessage(), ex);
    }
  }

  /**
   * Creates a DocumentHandler instance for the given output stream.
   * @param outputStream The output stream to use.
   * @return The created DocumentHandler instance.
   */
  protected DocumentHandler createDocumentHandler(OutputStream outputStream) {

    // ensuring the existance of document properties
    if (properties == null) {
      properties = new HashMap();
    }
    
    // adding default properties
    addDefaultProperties();

    DocumentHandler documentHandler = null;
    if (formatHelper == null) {
      formatHelper = FormatHelper.createInstance();
    }
    if (templateInterpeter == null) {
      documentHandler = new DocumentHandler(outputStream, templateInterpeter, new TemplateContext(properties, formatHelper), tagFactory);
    } else {
      documentHandler = new DocumentHandler(outputStream, templateInterpeter, new TemplateContext(properties, formatHelper), tagFactory);
    }
    
    documentHandler.setXmlReaderClass(xmlReaderClass);
    documentHandler.setDataAccessor(dataAccessor);
    documentHandler.setTagInterceptor(tagInterceptor);
    documentHandler.setContentFilter(contentFilter);
    documentHandler.setResourceLoader(resourceLoader);
    documentHandler.setMetaData(metaData);
    
    documentHandler.setTranslateEscapeSequences(translateEscapeSequences);
    documentHandler.setParseDynamicContent(parseDynamicContent);
    documentHandler.setEvaluateExpressions(evaluateExpressions);
    documentHandler.setCacheImages(cacheImages);
    documentHandler.setStrictlyCheckDocumentStructure(strictlyCheckDocumentStructure);
    documentHandler.setExpectedNumberOfPages(expectedNumberOfPages);

    return documentHandler;
  }
  
  /**
   * Loads document properties from the given location.
   * @param location The file system location to load the properties from.
   * @return The map holding the parsed properties.
   * @throws IOException In case a problem occurred while reading the properties.
   */
  public static Map loadProperties(String location) throws IOException {
    Map properties = new HashMap();
    try {
      Properties props = new Properties();
      File propsFile = new File(location);
      if (!propsFile.exists()) {
        System.err.println("The property file '" + location + "' does not exist.");
      }
      FileInputStream propsInputStream = new FileInputStream(propsFile);
      props.load(propsInputStream);
      // checking properties for sequences.
      Iterator propsIter = props.entrySet().iterator();
      while (propsIter.hasNext()) {
        Map.Entry propsEntry = (Map.Entry) propsIter.next();
        String key = (String) propsEntry.getKey();
        int lastDotIdx = key.lastIndexOf('.');
        if (lastDotIdx > 0) {
          String suffix = key.substring(lastDotIdx + 1);
          try {
            int idx = Integer.parseInt(suffix);
            String sequenceName = key.substring(0, lastDotIdx);
  
            Object sequence = properties.get(sequenceName);
            if (sequence == null) {
              // looking for fields entry.
              String fieldsProp = props.getProperty(sequenceName + ".fields");
              if (fieldsProp != null) {
                // fields property found -> creating a table
                DataTable sequenceTab = new DataTable();
                // parsing fields
                List fieldNames = parseRow(fieldsProp);
                int numFields = fieldNames.size();
                for (int i = 0; i < numFields; i ++) {
                  StringTokenizer fieldDefTok = new StringTokenizer((String) fieldNames.get(i), ":");
                  String fieldName = fieldDefTok.nextToken();
                  String fieldType = fieldDefTok.nextToken();
                  Column col = null;
                  if (fieldType.equalsIgnoreCase("int")) {
                    col = sequenceTab.addColumn(fieldName, UjacTypes.TYPE_INT);
                  } else if (fieldType.equalsIgnoreCase("float")) {
                    col = sequenceTab.addColumn(fieldName, UjacTypes.TYPE_FLOAT);
                  } else if (fieldType.equalsIgnoreCase("double")) {
                    col = sequenceTab.addColumn(fieldName, UjacTypes.TYPE_DOUBLE);
                  } else if (fieldType.equalsIgnoreCase("string")) {
                    col = sequenceTab.addColumn(fieldName, UjacTypes.TYPE_STRING);
                  } else if (fieldType.equalsIgnoreCase("date")) {
                    col = sequenceTab.addColumn(fieldName, UjacTypes.TYPE_DATE);
                    col.setFormat(new SimpleDateFormat("yyyy-MM-dd"));
                  }
                  if (fieldDefTok.hasMoreTokens()) {
                    String fieldTitle = fieldDefTok.nextToken();
                    float fieldWidth = 0.0F;
                    int fieldAlign = LayoutHints.LEFT;
                    if (fieldDefTok.hasMoreTokens()) {
                      String strFieldWidth = fieldDefTok.nextToken();
                      fieldWidth = Float.parseFloat(strFieldWidth);
                    }
                    if (fieldDefTok.hasMoreTokens()) {
                      String strFieldAlign = fieldDefTok.nextToken();
                      if (strFieldAlign.equalsIgnoreCase("left")) {
                        fieldAlign = LayoutHints.LEFT;
                      } else if (strFieldAlign.equalsIgnoreCase("right")) {
                        fieldAlign = LayoutHints.RIGHT;
                      } else if (strFieldAlign.equalsIgnoreCase("center")) {
                        fieldAlign = LayoutHints.CENTER;
                      }
                    }
                    col.setLayoutHints(new LayoutHints(fieldWidth, fieldTitle, fieldAlign)); 
                  }
                }
                sequence = sequenceTab;
                properties.put(sequenceName, sequenceTab);
              } else {
                // using plain list
                List sequenceList = new ArrayList();
                sequence = sequenceList;
                properties.put(sequenceName, sequenceList);
              }
            }
  
            if (sequence instanceof DataTable) {
              DataTable sequenceTab = (DataTable) sequence;
              Row row = ensureTableSize(sequenceTab, idx);
              // parsing fields
              List fieldValues = parseRow((String) propsEntry.getValue());
              int numFields = fieldValues.size();
              for (int i = 0; i < numFields; i ++) {
                row.setString(i, (String) fieldValues.get(i));
              }
            } else {
              List sequenceList = (List) sequence;
              ensureListSize(sequenceList, idx);
              sequenceList.set(idx, propsEntry.getValue());
            }
            continue;
          } catch (NumberFormatException ex) {
             // no sequence!
          }
        }
        // regular property!
        properties.put(key, propsEntry.getValue());
      }
    } catch (TableException ex) {
      throw new IOException("Table propblem encountered: " + ex.getMessage());
    }
    return properties;
  }

  /**
   * Clears the document property map.
   */
  public void clearProperties() {
    properties.clear();
  }
  
  /**
   * Adds default properties.
   */
  protected void addDefaultProperties() {
    Date currentDate = new Date();
    if (!properties.containsKey("today")) {
      properties.put("today", currentDate);
    }
    if (!properties.containsKey("now")) {
      properties.put("now", new Timestamp(currentDate.getTime()));
    }
  }

  /**
   * Getter method for the the property modules.
   * @return The current value of property modules.
   */
  protected List getModules() {
    return modules;
  }

  
  /**
   * The main method for command line access.
   * @param args The command line arguments.
   */
  public static void main(String[] args) {
    Map options = new HashMap();
    int argStart = CommandLineHelper.parseOptions(args, options);
    int numArgs = 0;
    if (argStart >= 0) {
      numArgs = args.length - argStart;
    }
    if (numArgs < 2) {
      System.err.println("Usage: org.ujac.print.DocumentPrinter [options] <template> <outputfile> [<properties file>]");
      System.err.println("Available options: -language=<language code>");
      System.err.println("                   -country=<country code>");
      System.err.println("                   -resource-dir=<resource directory>");
      System.err.println("                   -module=<module location in resource directory>");
      System.err.println("                   -support-toc=<true/false>");
      System.exit(1);
    }

    File templateFile = new File(args[argStart]);
    if (!templateFile.exists()) {
      System.err.println("The template file '" + templateFile + "' does not exist.");
    }
    try {
      
      FileInputStream templateInputStream;
      templateInputStream = new FileInputStream(templateFile);

      File outputFile = new File(args[argStart + 1]);
      FileOutputStream outputStream = new FileOutputStream(outputFile);

      Map properties = null;
      
      if (numArgs >= 3) {
        properties = loadProperties(args[argStart + 2]);
      }

      DocumentPrinter docPrinter = new DocumentPrinter(templateInputStream, properties);
      String resourceDir = (String) options.get("resource-dir");
      if (resourceDir == null) {
        resourceDir = ".";
      }
      docPrinter.setResourceLoader(new FileResourceLoader(resourceDir));

      String modulePath = (String) options.get("module");
      if (modulePath != null) {
        docPrinter.loadModule(modulePath);
      }
      
      String language = (String) options.get("language");
      String country = (String) options.get("country");
      if (language != null) {
        if (country != null) {
          docPrinter.setFormatHelper(FormatHelper.createInstance(new Locale(language, country)));
        } else {
          docPrinter.setFormatHelper(FormatHelper.createInstance(new Locale(language, "")));
        }
      }
      String strSupportToc = (String) options.get("support-toc");
      if (strSupportToc != null) {
        docPrinter.printDocument(outputStream, new Boolean(strSupportToc).booleanValue());
      } else {
        docPrinter.printDocument(outputStream);
      }      
    } catch (IOException e) {
      e.printStackTrace();
    } catch (DocumentHandlerException e) {
      e.printStackTrace();
    }
  }

  /**
   * Ensures the size of the given list, by filling the missing entries with null references. 
   * @param list The list to provess.
   * @param index The desired index to ensure.
   */
  private static void ensureListSize(List list, int index) {
    while (list.size() < index + 1) {
      list.add(null);
    }
  }

  /**
   * Ensures the size of the given table, by filling the missing entries with null references. 
   * @param table The table to provess.
   * @param index The desired index to ensure.
   * @return The desired row.
   */
  private static Row ensureTableSize(DataTable table, int index) {
    while (table.getRowCount() < index + 1) {
      table.addRow();
    }
    try {
      return table.getRow(index);
    } catch (RowNotDefinedException ex) {
      // this should never happen
      return null;
    }
  }
  
  /**
   * Parses the given row text for items (comma separated).
   * @param row The row to parse.
   * @return The detected items.
   */
  private static List parseRow(String row) {
    List items = new ArrayList();
    int rowLength = row.length();
    int lastItemStart = 0;
    for (int i = 0; i < rowLength; i++) {
      char c = row.charAt(i);
      if (c == '\'') {
        lastItemStart = i + 1;
        for (i = lastItemStart; i < rowLength; i++) {
          c = row.charAt(i);
          if (c == '\'') {
            // OK, field completed
            items.add(row.substring(lastItemStart, i));
            // skipping next comma!
            ++i;
            lastItemStart = i + 1;
            break;
          }
        }
      } else if (c == ',') {
        // OK, field completed
        items.add(row.substring(lastItemStart, i));
        lastItemStart = i + 1;
      }
    }
    if (lastItemStart < rowLength) {
      // OK, field completed
      items.add(row.substring(lastItemStart, rowLength));
    }
    return items;
  }

  /**
   * Getter method for the the property tagFactory.
   * @return The current value of property tagFactory.
   */
  public DocumentTagFactory getTagFactory() {
    return tagFactory;
  }
  /**
   * Setter method for the the property tagFactory.
   * @param tagFactory The value to set for the property tagFactory.
   */
  public void setTagFactory(DocumentTagFactory tagFactory) {
    this.tagFactory = tagFactory;
  }

  /**
   * Gets the document meta data.
   * @return The document meta data.
   */
  public DocumentMetaData getMetaData() {
    return metaData;
  }
  /**
   * Sets the document meta data.
   * @param metaData The XMP meta data to set.
   */
  public void setMetaData(DocumentMetaData metaData) {
    this.metaData = metaData;
  }

}