/*
 * Copyright (C) 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print;

import java.util.List;
import java.util.Map;

/**
 * Name: DocumentTagFactory<br>
 * Description: Interface for document tag factories.
 * 
 * @author lauerc
 */
public interface DocumentTagFactory {
  
  /**
   * Gets a new tag instance for the given tag name.
   * @param documentHandler The document handler for which the item is created.
   * @param name The tag name.
   * @return The newly generated tag.
   * @throws TagInstantiationException If the no tag type with the requested name is defined.
   */
  BaseDocumentTag createTagInstance(DocumentHandler documentHandler, String name) 
    throws TagInstantiationException;

  /**
   * Checks wheter the given tag is supported or not.
   * @param name The tag name to check.
   * @return true if a build was found for the given tag name, else false.
   */
  boolean isTagSupported(String name);
  
  /**
   * Registers the given document tag.
   * @param name The name of the tag to register.
   * @param tagClazz The tag class to register.
   */
  void registerTag(String name, Class tagClazz);

  /**
   * Registers the given document tag.
   * @param name The name of the tag to register.
   * @param tagBuilder The tag builder, which will build the tag handlers for the given name.
   */
  void registerTag(String name, TagBuilder tagBuilder);

  /**
   * Unregisters the given document tag.
   * @param tagName The name of the tag to unregister
   */
  void unregisterTag(String tagName);

  /**
   * Registers an alias for the given document tag.
   * @param tagName The name of the tag to register the alias for.
   * @param alias The name of the alias to register.
   * @exception TagInstantiationException In case the original tag does not exist 
   *     or has not been specified so far.
   */
  void registerTagAlias(String tagName, String alias) throws TagInstantiationException;

  /**
   * Gets all registered tag types.
   * @return A map containing all registered tag types.
   */
  Map getTagTypes();

  /**
   * Gets all possible childs
   * @return all possible childs.
   * @exception TagInstantiationException In case the instantiation of a tag has been failed.
   */
  List getAllPossibleChilds() throws TagInstantiationException;

  /**
   * Finds the supported child tags using the tag informations
   * from the methods getSupportedChilds and getExcludedChilds.
   * @param tag The tag to find the childs for.
   * @return The child definitions.
   * @exception TagInstantiationException In case the method <code>DefaultDocumentTagFactory.getAllPossibleChilds</code> failed.
   */
  List getSupportedChildTags(BaseDocumentTag tag) throws TagInstantiationException;

}
