/*
 * Copyright (C) 2004 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import java.util.List;
import java.util.Map;

import javax.xml.transform.Result;

import javax.xml.transform.sax.SAXResult;

import org.ujac.util.io.NullDeviceOutputStream;

import org.ujac.util.xml.XmlUtils;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

/**
 * Name: DocumentTransformHelper<br>
 * Description: A class which enables more efficient XML transform result processing.
 * 
 * @author lauerc
 */
public class DocumentTransformHelper extends DocumentPrinter {

  /** The used document handler. */
  private DocumentHandler documentHandler = null;
  /** The output stream. */
  private OutputStream outputStream = null;
  /** Tells whether or not to support TOC output. */
  private boolean supportToc = false;
  /** The temporary file to write to. */
  private File tmpSourceFile = null;

  /**
   * Constructs a DocumentTransformHelper instance with specific attributes.
   * In case the system propery 'org.xml.sax.driver' is defined,
   * overwrites the attribute xmlReaderClassName with that value.
   * Switches off all dynamic content processing features, because these are not
   * necessary for generated XML contents.
   */
  public DocumentTransformHelper() {
    configure();
  }

  /**
   * Constructs a DocumentTransformHelper instance with specific attributes.
   * In case the system propery 'org.xml.sax.driver' is defined,
   * overwrites the attribute xmlReaderClassName with that value.
   * Switches off all dynamic content processing features, because these are not
   * necessary for generated XML contents.
   * @param tagFactory The document tag factory.
   */
  public DocumentTransformHelper(DocumentTagFactory tagFactory) {
    super(tagFactory);
    configure();
  }

  /**
   * Constructs a DocumentTransformHelper instance with specific arguments.
   * Switches off all dynamic content processing features, because these are not
   * necessary for generated XML contents.
   * @param properties The document's properties.
   * @param tagFactory The document tag factory.
   */
  public DocumentTransformHelper(Map properties, DocumentTagFactory tagFactory) {
    super(tagFactory);
    setProperties(properties);
    configure();
  }

  /**
   * Switches off all dynamic content processing features, because these are not
   * necessary for generated XML contents.
   */
  private void configure() {
    setTranslateEscapeSequences(false);
    setParseDynamicContent(false);
    setEvaluateExpressions(false);
  }

  /**
   * Prepares for XSLT transformation using SAXResult.
   * @param outputStream The output stream to which to write the document.
   * @param supportToc Tells whether or not to support the output of a leading table of contents.
   *    Notice: if you use this option, the main document will be completely generated in memory, 
   *    this may lead to problems if you process huge documents.
   * @return Reference of the document handler, used for the document processing.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public Result prepare(OutputStream outputStream, boolean supportToc) 
    throws DocumentHandlerException, IOException {
    
    this.supportToc = supportToc;
    this.outputStream = outputStream;
    
    // removing properties, which are internally used by the DocumentHandler.
    removeInternalDocumentProperties();

    if (supportToc) {
      File tmpSourceFile = null;
      // pass 1: collecting the TOC informations
      DocumentHandler documentHandler = null;
      
      // writing template to temporary file, to keep memory consumption low
      PrintWriter templateCopyWriter = null;
      // writing template to temporary file, to keep memory consumption low
      tmpSourceFile = File.createTempFile("ujac_print", "xml");
      documentHandler = createDocumentHandler(new NullDeviceOutputStream());
      documentHandler.setDestinationsSupported(false);
      documentHandler.clearReferences();
      documentHandler.setPageNumberOffset(0);
      documentHandler.setIncludeHeadSection(false);
      // the template needs to be interpreted twice or more ...
      templateCopyWriter = new PrintWriter(new FileWriter(tmpSourceFile));
      documentHandler.setTemplateCopyWriter(templateCopyWriter);
    } else {
      this.documentHandler = createDocumentHandler(outputStream);
    }
    // executing first processing step!
    return createTransformationResult(documentHandler, getModules());
  }

  /**
   * Finishes the document processing. This is necessary in case TOC support 
   * has been switched on. Otherwise this method will do nothing.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void finish()
    throws DocumentHandlerException, IOException {
    
    if (!supportToc) {
      // nothing 2 do for us here!
      return;
    }
    
    int numPages = ((Integer) documentHandler.getTemplateContext().get("pageNumber")).intValue();
    PrintWriter templateCopyWriter = documentHandler.getTemplateCopyWriter();
    documentHandler.setTemplateCopyWriter(null);
    if (templateCopyWriter != null) {
      templateCopyWriter.close();
    }

    boolean pageNumberingStartsAfterHead = isPageNumberingStartsAfterHead();
    try {
      // pass 2: printing the document together with the TOC 
      documentHandler.setIncludeHeadSection(true);
      removeInternalDocumentProperties();
      if (pageNumberingStartsAfterHead) {
        documentHandler.setTocFreezed(true);
        documentHandler.resetReferences();
      } else {
        documentHandler.setTocFreezed(false);
        documentHandler.clearReferences();
      }
      documentHandler.reset(new NullDeviceOutputStream());
      printDocument(documentHandler, new InputSource(new FileReader(tmpSourceFile)), null);
      int numPagesInclToc = ((Integer) documentHandler.getTemplateContext().get("pageNumber")).intValue();

      // pass 3: recalculating the TOC numbers
      if (!pageNumberingStartsAfterHead) {
        removeInternalDocumentProperties();
        documentHandler.setTocFreezed(false);
        documentHandler.resetReferences();
        documentHandler.reset(new NullDeviceOutputStream());
        printDocument(documentHandler, new InputSource(new FileReader(tmpSourceFile)), null);
      }

      // pass 4: finally generating the destination document
      removeInternalDocumentProperties();
      if (pageNumberingStartsAfterHead) {
        documentHandler.setTocFreezed(true);
        documentHandler.resetReferences();
      } else {
        documentHandler.setTocFreezed(false);
        documentHandler.resetReferences();
      }
      documentHandler.reset(outputStream);
      int offset = 0;
      if (pageNumberingStartsAfterHead) {
        offset = numPages - numPagesInclToc; 
      }
      documentHandler.setPageNumberOffset(offset);
      documentHandler.setDestinationsSupported(true);
      printDocument(documentHandler, new InputSource(new FileReader(tmpSourceFile)), null);
    } finally {
      tmpSourceFile.delete();
    }
  }


  /**
   * Creates a transformation result, based on the given document handler.
   * @param documentHandler The document handler to use for XML processing.
   * @param modules The modules to use.
   * @return Reference of the document handler, used for the document processing.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  private Result createTransformationResult(DocumentHandler documentHandler, List modules) 
    throws DocumentHandlerException, IOException {

    try {
      if ((modules != null) && (modules.size() > 0)) {
        XMLReader xmlReader = createXmlReader();
        
        xmlReader.setContentHandler(documentHandler);
        
        // disabling DTD/Schema based XML validation to avoid performance problems
        // and problems on systems that doesn't have internet access. 
        XmlUtils.disableEntityResolver(xmlReader);
          
        if (modules != null) {
          // loading modules into document handler
          int numModules = modules.size();
          for (int i = 0; i < numModules; i++) {
            byte[] moduleCode = (byte[]) modules.get(i);
            xmlReader.parse(new InputSource(new ByteArrayInputStream(moduleCode)));
          }
        }
      }
      
      Result result = new SAXResult(documentHandler);
      return result;
    } catch (SAXException ex) {
      Exception cause = ex.getException();
      if (cause != null) {
        if (cause instanceof DocumentHandlerException) {
          throw ((DocumentHandlerException) cause);
        }
        throw new DocumentHandlerException(documentHandler.locator(), ex.getMessage(), cause);
      }
      throw new DocumentHandlerException(documentHandler.locator(), ex.getMessage(), ex);
    }
  }

}
