/*
 * Copyright (C) 2003 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.FileOutputStream;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.HashMap;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Map;

/**
 * Name: DocumentationGenerator<br>
 * Description: A class generating the documentation for the item set
 *              according to the meta data, served by the DocumentTagFactory
 *              and the items themselves.
 * 
 * @author lauerc
 */
public class DocumentationGenerator {

  /** The tag factory. */
  private DocumentTagFactory tagFactory = null;
  
  /**
   * Name: TagNameComarator<br>
   * Description: Compares BaseDocumentTag instances by their names.
   */
  private static class TagNameComarator implements Comparator {

    /**
     * Constructs a TagNameComarator instance with no specific attributes.
     */
    public TagNameComarator() {
    }
    
    /**
     * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
     */
    public int compare(Object o1, Object o2) {
      BaseDocumentTag item1 = (BaseDocumentTag) o1;
      BaseDocumentTag item2 = (BaseDocumentTag) o2;
      return item1.getName().compareTo(item2.getName());
    }
    
  }
  
  /**
   * Constructs a DocumentationGenerator instance with specific attributes.
   * @param tagFactory The tag factory to use.
   */
  public DocumentationGenerator(DocumentTagFactory tagFactory) {
    this.tagFactory = tagFactory;
  }
  
  /**
   * Generates the documentation using the given documentaion template.
   * @param documentationTempleteStream  The documentation template stream.
   * @param outputStream  The output stream.
   * @exception DocumentHandlerException If something went wrong while generating the documentation.
   * @exception IOException In case the writing of the document or the reading of the template has been failed.
   */
  private void generateDocumentation(InputStream documentationTempleteStream, 
                                     OutputStream outputStream) 
    throws DocumentHandlerException, IOException {
   
    try { 
      DocumentHandler docHandler = new DocumentHandler(null, null, tagFactory);
      List tags = new ArrayList();
      Iterator typeIter = tagFactory.getTagTypes().entrySet().iterator();
      while (typeIter.hasNext()) {
        Map.Entry typeEntry = (Map.Entry) typeIter.next();
        TagBuilder builder = (TagBuilder) typeEntry.getValue();
        BaseDocumentTag tag = builder.createTag(docHandler);
        tag.setDocumentHandler(docHandler);
        tag.setTagFactory(tagFactory);
        tags.add(tag);      
      }
      // sorting the item list
      Collections.sort(tags, new TagNameComarator());
      
      Map properties = new HashMap();
      properties.put("tags", tags);
      DocumentPrinter docPrinter = new DocumentPrinter(documentationTempleteStream, properties);
      docPrinter.printDocument(outputStream);
    } catch (TagInstantiationException ex) {
      throw new IOException(ex.getMessage());
    }
  }

  /**
   * The main method for command line access.
   * @param args The command line attributes.
   */
  public static void main(String[] args) {
    if (args.length > 1) {
      System.err.println("usage: org.ujac.print.DocumentationGenerator [<output file> <tag factory class>]");
      System.exit(1);
    }
    try {
      OutputStream outputStream = null;
      DocumentTagFactory tagFactory = null;
      if (args.length < 1)  {
        outputStream = System.out;
        tagFactory = new DocumentPrinter().getTagFactory();
      } else {
        outputStream = new FileOutputStream(args[0]);
        if (args.length > 1) {
          tagFactory = (DocumentTagFactory) Class.forName(args[1]).newInstance();
        } else {
          tagFactory = new DocumentPrinter().getTagFactory();
        }
      }
      DocumentationGenerator dg = new DocumentationGenerator(tagFactory);
      InputStream documentationTempleteStream = DocumentationGenerator.class.getClassLoader().getResourceAsStream("org/ujac/print/documentation-template.xml");
      dg.generateDocumentation(documentationTempleteStream, outputStream);
    } catch (IOException ex) {
      ex.printStackTrace();
    } catch (DocumentHandlerException ex) {
      ex.printStackTrace();
    } catch (InstantiationException e) {
      e.printStackTrace();
    } catch (IllegalAccessException e) {
      e.printStackTrace();
    } catch (ClassNotFoundException e) {
      e.printStackTrace();
    }
  }
}
