/*
 * Copyright (C) 2003, 2004, 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print;

import java.io.Writer;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;

import java.util.HashSet;
import java.util.Map;
import java.util.List;
import java.util.Iterator;
import java.util.Set;
import org.ujac.util.CollectionUtils;

/**
 * Title:       DtdGenerator<br>
 * Description: Provides command line access to the static method generateDtd. 
 *              at class DocumentTagFactory .<br>
 * 
 * @author lauerc
 */
public class DtdGenerator {

  /** The tag factory. */
  private DocumentTagFactory tagFactory = null;

  /**
   * Constructs a DtdGenerator instance with specific attributes.
   * @param tagFactory The tag factory to use.
   */
  public DtdGenerator(DocumentTagFactory tagFactory) {
    this.tagFactory = tagFactory;
  }

  /**
   * Generates the DTD according to the meta informations, provided by the registered items.
   * @param out The writer, needed for output.
   * @exception IOException In case the writing of the DTD has been failed. 
   */
  public void generateDtd(Writer out) throws IOException {

    try {
      List allPossibleChilds = tagFactory.getAllPossibleChilds();
      int numPossibleChilds = allPossibleChilds.size();
      
      Iterator typeIter = tagFactory.getTagTypes().entrySet().iterator();
      while (typeIter.hasNext()) {
        Map.Entry typeEntry = (Map.Entry) typeIter.next();
        String name = (String) typeEntry.getKey();
        TagBuilder builder = (TagBuilder) typeEntry.getValue();
        BaseDocumentTag tag = builder.createTag(null);
        tag.setTagFactory(tagFactory);
        out.write("<!ELEMENT " + name);
        
        // declaring child sequence
        ChildDefinitionMap supportedChilds = tag.buildSupportedChilds();
        if (tag.isVirtualContainer()) {
          out.write(" (");
          boolean first = true;
          if (tag.isTextBodyAllowed()) {
            out.write("#PCDATA");
            first = false;
          }
          // all possible childs are eventually supported.
          for (int i = 0; i < numPossibleChilds; i++) {
            BaseDocumentTag childTag = (BaseDocumentTag) allPossibleChilds.get(i);
            if (childTag.isDocumentRoot()) {
              // skipping document roots
              continue;
            }
            if (first) {
              first = false;
            } else {
              out.write(" | ");
            }
            out.write(childTag.getName());
          }            
          out.write(")*>\n");
        } else if ((supportedChilds != null) && (supportedChilds.size() > 0)) {
          out.write(" (");
          if (tag.isTextBodyAllowed()) {
            out.write("#PCDATA | ");
          }
          Set childSet = new HashSet();
          int numSupportedChilds = supportedChilds.size();
          List supportedChildList = supportedChilds.getSortedChilds();
          for (int i = 0; i < numSupportedChilds; i++) {
            ChildDefinition child = (ChildDefinition) supportedChildList.get(i);
            Class childType = child.getType();
            for (int j = 0; j < numPossibleChilds; j++) {
              BaseDocumentTag childTag = (BaseDocumentTag) allPossibleChilds.get(j);
              if (childType.isInstance(childTag)) {
                childSet.add(childTag.getName());
              }
            }
          }
          Iterator iterChilds = childSet.iterator();
          int childIdx = 0;
          while (iterChilds.hasNext()) {
            if (childIdx > 0)  {
              out.write(" | ");
            }
            out.write(iterChilds.next().toString());
            ++childIdx;
          }
          out.write(")*>\n");
        } else {
          if (tag.isTextBodyAllowed()) {
            out.write(" (#PCDATA)>\n");
          } else {   
            out.write(" EMPTY>\n");
          }
        }
        
        String aliasName = null;
        AttributeDefinitionMap supportedAttributes = tag.buildSupportedAttributes();
        if (!CollectionUtils.isEmpty(supportedAttributes)) {
          out.write("<!ATTLIST " + name + "\n");
          List attributes = supportedAttributes.getSortedAttributes();
          int numSortedAttributes = supportedAttributes.size();
          for (int i = 0; i < numSortedAttributes; i++) {
            AttributeDefinition attribute = (AttributeDefinition) attributes.get(i);
            int numAliases = 0;
            Iterator iterLocalAliases = attribute.getAliases().iterator();
            while (iterLocalAliases.hasNext()) {
              aliasName = iterLocalAliases.next().toString();
              ++numAliases;
              writeAttribute(out, aliasName, attribute.getType(), false);
            }
            writeAttribute(out, attribute.getName(), attribute.getType(), attribute.isRequired() && numAliases == 0);
          }
          out.write(">\n");
        }
        
        out.write("\n");
      }
    } catch (TagInstantiationException ex) {
      throw new IOException(ex.getMessage());
    }
    out.flush();
  }

  /**
   * Writes a single attribute to the DTD.
   * @param out The writer to use.
   * @param name The attribute name.
   * @param type The attribute type.
   * @param required The required flag.
   * @throws IOException In case the DTD output failed.
   */
  private void writeAttribute(Writer out, String name, int type, boolean required) throws IOException {
    out.write("  " + name + " " + attributeTypeToDtdType(type));
    out.write(" #");
    if (required) {
      out.write("REQUIRED"); 
    } else {
      out.write("IMPLIED"); 
    }
    out.write("\n");
  }

  /**
   * Gets the DTD type name for the given type ID.
   * @param attributeType The type ID.
   * @return The according schema type.
   */
  protected static String attributeTypeToDtdType(int attributeType) {
    switch (attributeType) {
      case AttributeDefinition.TYPE_BOOLEAN: return "CDATA";
      case AttributeDefinition.TYPE_BORDER: return "CDATA";
      case AttributeDefinition.TYPE_COLOR: return "CDATA";
      case AttributeDefinition.TYPE_FLOAT: return "CDATA";
      case AttributeDefinition.TYPE_DIMENSION: return "CDATA";
      case AttributeDefinition.TYPE_FONT_FAMILY: return "CDATA";
      case AttributeDefinition.TYPE_FONT_STYLE: return "CDATA";
      case AttributeDefinition.TYPE_HORIZONTAL_ALIGNMENT: return "CDATA";
      case AttributeDefinition.TYPE_VERTICAL_ALIGNMENT: return "CDATA";
      case AttributeDefinition.TYPE_INT: return "CDATA";
      case AttributeDefinition.TYPE_PAGE_SIZE: return "(LETTER | NOTE | LEGAL | A0 | A1 | A2 | A3 | A4 | A5 | A6 | A7 | A8 | A9 | A10 | B0 | B1 | B2 | B3 | B4 | B5)";
      case AttributeDefinition.TYPE_STRING: return "CDATA";
      case AttributeDefinition.TYPE_NAME: return "CDATA";
      case AttributeDefinition.TYPE_ENUM: return "CDATA";
      default: return "unsupported";
    }
  }

  /**
   * The main method for command line access.
   * @param args The command line attributes.
   */
  public static void main(String[] args) {
    if (args.length > 1) {
      System.err.println("usage: org.ujac.print.DtdGenerator [<output file>]");
      System.exit(1);
    }
    try {
      Writer writer = null;
      DocumentTagFactory tagFactory = null;
      if (args.length < 1)  {
        writer = new OutputStreamWriter(System.out);
        tagFactory = new DocumentPrinter().getTagFactory();
      } else {
        writer = new FileWriter(args[0]);
        if (args.length > 1) {
          tagFactory = (DocumentTagFactory) Class.forName(args[1]).newInstance();
        } else {
          tagFactory = new DocumentPrinter().getTagFactory();
        }
      }
      DtdGenerator dg = new DtdGenerator(tagFactory);
      dg.generateDtd(writer);
    } catch (IOException ex) {
      ex.printStackTrace();
    } catch (InstantiationException e) {
      e.printStackTrace();
    } catch (IllegalAccessException e) {
      e.printStackTrace();
    } catch (ClassNotFoundException e) {
      e.printStackTrace();
    }
  }
}
