/*
 * Copyright (C) 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.ujac.util.CollectionUtils;

/**
 * Name: TestTagInterceptor<br>
 * Description: Implementation of the DocumentTagInterceptor interface for testing use.
 * 
 * @author lauerc
 */
public class TestTagInterceptor implements DocumentTagInterceptor {

  /** The level of tag nesting; 1 -> top level. */
  private int level = 0;
  
  /** Holds the tests to perform for the document processing. */
  private Map testMap = new HashMap();
  
  /**
   * Adds a test to the given 
   * @param tagId The ID of the tag to test.
   * @param test The test to perform.
   */
  public void addTest(String tagId, AbstractTagTest test) {
    List tagTests = (List) testMap.get(tagId);
    if (tagTests == null) {
      tagTests = new ArrayList();
      testMap.put(tagId, tagTests); 
    }
    tagTests.add(test);
  }
  
  /**
   * Tells, whether or not all test have been passed.
   * @return true in case all tests have been performed successfully, else falses.
   */
  public boolean isAllTestsPassed() {
    Collection tagTestSet = testMap.values();
    Iterator iterTagTests = tagTestSet.iterator();
    while (iterTagTests.hasNext()) {
      List testCases = (List) iterTagTests.next();
      int numTests = CollectionUtils.getSize(testCases);
      for (int i = 0; i < numTests; i++) {
        AbstractTagTest testCase = (AbstractTagTest) testCases.get(i);
        if (!testCase.isTestPassed()) {
          return false;
        }
      }
    }
    return true;
  }
  
  /**
   * Removes all tests. 
   */
  public void clearTests() {
    testMap.clear();
  }
  
  /**
   * @see org.ujac.print.DocumentTagInterceptor#beforeOpen(org.ujac.print.BaseDocumentTag)
   */
  public boolean beforeOpen(BaseDocumentTag tag)
      throws DocumentHandlerException {

    ++level;
    return true;
  }

  /**
   * @see org.ujac.print.DocumentTagInterceptor#afterOpen(org.ujac.print.BaseDocumentTag)
   */
  public void afterOpen(BaseDocumentTag tag) throws DocumentHandlerException {
  }

  /**
   * @see org.ujac.print.DocumentTagInterceptor#beforeClose(org.ujac.print.BaseDocumentTag)
   */
  public boolean beforeClose(BaseDocumentTag tag)
    throws DocumentHandlerException {
    
    doTests(tag, AbstractTagTest.TEST_OPEN);
    return true;
  }
  
  /**
   * @see org.ujac.print.DocumentTagInterceptor#afterClose(org.ujac.print.BaseDocumentTag)
   */
  public void afterClose(BaseDocumentTag tag)
    throws DocumentHandlerException {
    
    doTests(tag, AbstractTagTest.TEST_CLOSE);
   --level;
  }
  
  /**
   * Does the tests.
   * @param tag The tag to test.
   * @param mode The test mode.
   * @throws DocumentHandlerException In case a document handler problem occurred, during the tests.
   */
  private void doTests(BaseDocumentTag tag, int mode)
    throws DocumentHandlerException {
    
    if (!tag.isValid()) {
      return;
    }
    
    String tagId = tag.getStringAttribute("id", "", true, null); 
    List tagTests = (List) testMap.get(tagId);
    int numTagTests = CollectionUtils.getSize(tagTests);
    for (int i = 0; i < numTagTests; i++) {
      AbstractTagTest tagTest = (AbstractTagTest) tagTests.get(i);
      if (tagTest.getTestMode() == mode) {
        tagTest.setup(tag);
        tagTest.doIt();
      }
    }
  }
  
}
