/*
 * Copyright (C) 2003, 2004, 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print;

import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.ujac.util.CollectionUtils;

/**
 * Name: XmlSchemaGenerator<br>
 * Description: Provides command line access to the static method generateXmlSchema 
 *              at class DocumentTagFactory .<br>
 * 
 * @author lauerc
 */
public class XmlSchemaGenerator {

  /** The tag factory. */
  private DocumentTagFactory tagFactory = null;

  /**
   * Constructs a XmlSchemaGenerator instance with specific attributes.
   * @param tagFactory The tag factory to use.
   */
  public XmlSchemaGenerator(DocumentTagFactory tagFactory) {
    this.tagFactory = tagFactory;
  }

  /**
   * Generates the XML schema according to the meta informations, provided by the registered items.
   * @param out The writer, needed for output.
   * @exception IOException In case the writing of the DTD has been failed. 
   */
  public void generateXmlSchema(Writer out) throws IOException {
    out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
    out.write("<xsd:schema targetNamespace=\"http:///UJAC/org/ujac/print/document.xsd\"\n");
    out.write("            xmlns:this=\"http:///UJAC/org/ujac/print/document.xsd\"\n");
    out.write("            xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">\n");
    out.write("    <xsd:annotation>\n");
    out.write("        <xsd:documentation xml:lang=\"en\">Document definition for print.ujac.org.</xsd:documentation>\n");
    out.write("    </xsd:annotation>\n");
    out.write("    <!-- specifying the attribute types -->\n");
    out.write("    <xsd:simpleType name=\"pageSize\">\n");
    out.write("        <xsd:annotation>The supported page sizes</xsd:annotation>\n");
    out.write("        <xsd:restriction base=\"xsd:string\">\n");
    out.write("            <xsd:enumeration value=\"LETTER\"/>\n");
    out.write("            <xsd:enumeration value=\"NOTE\"/>\n");
    out.write("            <xsd:enumeration value=\"LEGAL\"/>\n");
    out.write("            <xsd:enumeration value=\"A0\"/>\n");
    out.write("            <xsd:enumeration value=\"A1\"/>\n");
    out.write("            <xsd:enumeration value=\"A2\"/>\n");
    out.write("            <xsd:enumeration value=\"A3\"/>\n");
    out.write("            <xsd:enumeration value=\"A4\"/>\n");
    out.write("            <xsd:enumeration value=\"A5\"/>\n");
    out.write("            <xsd:enumeration value=\"A6\"/>\n");
    out.write("            <xsd:enumeration value=\"A7\"/>\n");
    out.write("            <xsd:enumeration value=\"A8\"/>\n");
    out.write("            <xsd:enumeration value=\"A9\"/>\n");
    out.write("            <xsd:enumeration value=\"A10\"/>\n");
    out.write("            <xsd:enumeration value=\"B0\"/>\n");
    out.write("            <xsd:enumeration value=\"B1\"/>\n");
    out.write("            <xsd:enumeration value=\"B2\"/>\n");
    out.write("            <xsd:enumeration value=\"B3\"/>\n");
    out.write("            <xsd:enumeration value=\"B4\"/>\n");
    out.write("            <xsd:enumeration value=\"B5\"/>\n");
    out.write("        </xsd:restriction>\n");
    out.write("    </xsd:simpleType>\n");
    out.write("    <xsd:simpleType name=\"borderStyle\">\n");
    out.write("        <xsd:annotation>The supported border styles</xsd:annotation>\n");
    out.write("        <xsd:restriction base=\"xsd:string\">\n");
    out.write("            <xsd:pattern value=\"none|box|((left|right|top|bottom)\\|)+\"/>\n");
    out.write("        </xsd:restriction>\n");
    out.write("    </xsd:simpleType>\n");
    out.write("    <xsd:simpleType name=\"color\">\n");
    out.write("        <xsd:annotation>The color attribute type</xsd:annotation>\n");
    out.write("        <xsd:restriction base=\"xsd:string\">\n");
    out.write("            <xsd:pattern value=\"[0-9a-fA-F]{6}\"/>\n");
    out.write("        </xsd:restriction>\n");
    out.write("    </xsd:simpleType>\n");
    out.write("    <xsd:simpleType name=\"dimension\">\n");
    out.write("        <xsd:annotation>The dimension alignment attribute type</xsd:annotation>\n");
    out.write("        <xsd:restriction base=\"xsd:string\">\n");
    out.write("            <xsd:pattern value=\"-?[0-9]+\\.?[0-9]+(\\p\\t)?\"/>\n");
    out.write("        </xsd:restriction>\n");
    out.write("    </xsd:simpleType>\n");
    out.write("    <xsd:simpleType name=\"hAlign\">\n");
    out.write("        <xsd:annotation>The horizontal alignment attribute type</xsd:annotation>\n");
    out.write("        <xsd:restriction base=\"xsd:string\">\n");
    out.write("            <xsd:enumeration value=\"left\"/>\n");
    out.write("            <xsd:enumeration value=\"right\"/>\n");
    out.write("            <xsd:enumeration value=\"center\"/>\n");
    out.write("        </xsd:restriction>\n");
    out.write("    </xsd:simpleType>\n");
    out.write("    <xsd:simpleType name=\"vAlign\">\n");
    out.write("        <xsd:annotation>The vertical alignment attribute type</xsd:annotation>\n");
    out.write("        <xsd:restriction base=\"xsd:string\">\n");
    out.write("            <xsd:enumeration value=\"top\"/>\n");
    out.write("            <xsd:enumeration value=\"bottom\"/>\n");
    out.write("            <xsd:enumeration value=\"middle\"/>\n");
    out.write("            <xsd:enumeration value=\"baseline\"/>\n");
    out.write("        </xsd:restriction>\n");
    out.write("    </xsd:simpleType>\n");
    out.write("    <xsd:simpleType name=\"fontFamily\">\n");
    out.write("        <xsd:annotation>The supported font families</xsd:annotation>\n");
    out.write("        <xsd:restriction base=\"xsd:string\"/>\n");
    out.write("    </xsd:simpleType>\n");
    out.write("    <xsd:simpleType name=\"fontStyle\">\n");
    out.write("        <xsd:annotation>The font style attribute type</xsd:annotation>\n");
    out.write("        <xsd:restriction base=\"xsd:string\">\n");
    out.write("            <xsd:pattern value=\"normal|(bold|italic|underline|line-through)(\\|(bold|italic|underline|line-through))*\"/>\n");
    out.write("        </xsd:restriction>\n");
    out.write("    </xsd:simpleType>\n");
    out.write("    <!-- specifying the item types -->\n");

    try {
      List allPossibleChilds = tagFactory.getAllPossibleChilds();

      Map tagMap = tagFactory.getTagTypes();
      Iterator typeIter = tagMap.entrySet().iterator();
      while (typeIter.hasNext()) {
        Map.Entry typeEntry = (Map.Entry) typeIter.next();
        TagBuilder builder = (TagBuilder) typeEntry.getValue();
        BaseDocumentTag tag = builder.createTag(null);
        // writing complex type to the schema
        printType(out, tag, allPossibleChilds);
        typeIter.remove();
      }
    } catch (Exception ex) {
      ex.printStackTrace();
    }
    out.write("</xsd:schema>\n");
    out.flush();
  }

  /**
   * Generates the XML schema according to the meta informations, provided by the registered items.
   * @param out The writer, needed for output.
   * @param tag The tag to write the type definition for.
   * @param allPossibleChilds The list holding all possible childs.
   * @exception IOException In case the writing of the DTD has been failed. 
   */
  private void printType(Writer out, BaseDocumentTag tag, List allPossibleChilds) 
    throws IOException {
    
    int numPossibleChilds = allPossibleChilds.size();
    // declaring child sequence
    ChildDefinitionMap supportedChilds = tag.buildSupportedChilds();
    AttributeDefinitionMap supportedAttributes = tag.buildSupportedAttributes();
    if (tag.isVirtualContainer() || 
        ((supportedChilds != null) && (supportedChilds.size() > 0)) ||
        !CollectionUtils.isEmpty(supportedAttributes)) {
      out.write("\t<xsd:element name=\"" + tag.getName() + "\">\n");
      out.write("\t\t<xsd:complexType>\n");
      if (tag.isVirtualContainer()) {
        out.write("\t\t\t<xsd:choice minOccurs=\"0\" maxOccurs=\"unbounded\">\n");
        // all possible childs are eventually supported.
        for (int i = 0; i < numPossibleChilds; i++) {
          BaseDocumentTag childTag = (BaseDocumentTag) allPossibleChilds.get(i);
          if (childTag.isDocumentRoot()) {
            // skipping document roots
            continue;
          }
          out.write("\t\t\t\t<xsd:element ref=\"this:" + childTag.getName() + "\" minOccurs=\"0\" maxOccurs=\"unbounded\"/>\n");
        }            
        out.write("\t\t\t</xsd:choice>\n");
      } else if ((supportedChilds != null) && (supportedChilds.size() > 0)) {
        out.write("\t\t\t<xsd:choice minOccurs=\"0\" maxOccurs=\"unbounded\">\n");
        Map childMap = new HashMap();
        int numSupportedChilds = supportedChilds.size();
        List supportedChildList = supportedChilds.getSortedChilds();
        for (int i = 0; i < numSupportedChilds; i++) {
          ChildDefinition child = (ChildDefinition) supportedChildList.get(i);
          Class childType = child.getType();
          for (int j = 0; j < numPossibleChilds; j++) {
            BaseDocumentTag childTag = (BaseDocumentTag) allPossibleChilds.get(j);
            if (childType.isInstance(childTag)) {
              childMap.put(childTag.getName(), child);
            }
          }
        }
        Iterator iterChilds = childMap.entrySet().iterator();
        while (iterChilds.hasNext()) {
          Map.Entry entry = (Map.Entry) iterChilds.next();
          String name = (String) entry.getKey();
          ChildDefinition child = (ChildDefinition) entry.getValue();
          String maxOccurs = null;
          if (child.getMaxOccurrences() == ChildDefinition.UNLIMITED) {
            maxOccurs = "unbounded";
          } else {
            maxOccurs = Integer.toString(child.getMaxOccurrences());
          }
          out.write("\t\t\t\t<xsd:element ref=\"this:" + name + "\" minOccurs=\"" + child.getMinOccurrences() + "\" maxOccurs=\"" + maxOccurs + "\"/>\n");
        }
        out.write("\t\t\t</xsd:choice>\n");
      }
      // declaring type attributes
      if (!CollectionUtils.isEmpty(supportedAttributes)) {
        String aliasName = null;
        List attributes = supportedAttributes.getSortedAttributes();
        int numSortedAttributes = supportedAttributes.size();
        for (int i = 0; i < numSortedAttributes; i++) {
          AttributeDefinition attribute = (AttributeDefinition) attributes.get(i);
          int numAliases = 0;
          Iterator iterLocalAliases = attribute.getAliases().iterator();
          while (iterLocalAliases.hasNext()) {
            aliasName = iterLocalAliases.next().toString();
            ++numAliases;
            writeAttribute(out, aliasName, attribute.getType(), false);
          }
          writeAttribute(out, attribute.getName(), attribute.getType(), attribute.isRequired() && numAliases == 0);
        }
      }
      out.write("\t\t</xsd:complexType>\n");
      out.write("\t</xsd:element>\n");
    } else {
      out.write("\t<xsd:element name=\"" + tag.getName() + "\"/>\n");
    }
  }

  /**
   * Writes a single attribute to the XML schema.
   * @param out The writer to use.
   * @param name The attribute name.
   * @param type The attribute type.
   * @param required The required flag.
   * @throws IOException In case the DTD output failed.
   */
  private void writeAttribute(Writer out, String name, int type, boolean required) throws IOException {
    String useType = null;
    if (required) {
      useType = "required";
    } else {
      useType = "optional";
    }
    out.write("\t\t\t<xsd:attribute name=\"" + name + "\" type=\"" + attributeTypeToSchemaType(type) + "\" use=\"" + useType + "\"/>\n");
  }
  
  /**
   * Gets the schema type name for the given type ID.
   * @param attributeType The type ID.
   * @return The according schema type.
   */
  protected static String attributeTypeToSchemaType(int attributeType) {
    switch (attributeType) {
      case AttributeDefinition.TYPE_BOOLEAN: return "xsd:boolean";
      case AttributeDefinition.TYPE_BORDER: return "this:borderStyle";
      case AttributeDefinition.TYPE_COLOR: return "this:color";
      case AttributeDefinition.TYPE_FLOAT: return "xsd:float";
      case AttributeDefinition.TYPE_DIMENSION: return "this:hAlign";
      case AttributeDefinition.TYPE_FONT_FAMILY: return "this:fontFamily";
      case AttributeDefinition.TYPE_FONT_STYLE: return "this:fontStyle";
      case AttributeDefinition.TYPE_HORIZONTAL_ALIGNMENT: return "this:hAlign";
      case AttributeDefinition.TYPE_VERTICAL_ALIGNMENT: return "this:vAlign";
      case AttributeDefinition.TYPE_INT: return "xsd:integer";
      case AttributeDefinition.TYPE_PAGE_SIZE: return "this:pageSize";
      case AttributeDefinition.TYPE_STRING: return "xsd:string";
      case AttributeDefinition.TYPE_NAME: return "xsd:NMTOKEN";
      case AttributeDefinition.TYPE_ENUM: return "xsd:string";
      default: return "unsupported";
    }
  }

  /**
   * The main method for command line access.
   * @param args The command line attributes.
   */
  public static void main(String[] args) {
    if (args.length > 1) {
      System.err.println("usage: org.ujac.print.XmlSchemaGenerator [<output file>]");
      System.exit(1);
    }
    try {
      Writer writer = null;
      DocumentTagFactory tagFactory = null;
      if (args.length < 1)  {
        writer = new OutputStreamWriter(System.out);
        tagFactory = new DocumentPrinter().getTagFactory();
      } else {
        writer = new FileWriter(args[0]);
        if (args.length > 1) {
          tagFactory = (DocumentTagFactory) Class.forName(args[1]).newInstance();
        } else {
          tagFactory = new DocumentPrinter().getTagFactory();
        }
      }
      XmlSchemaGenerator xg = new XmlSchemaGenerator(tagFactory);
      xg.generateXmlSchema(writer);
    } catch (IOException ex) {
      ex.printStackTrace();
    } catch (InstantiationException e) {
      e.printStackTrace();
    } catch (IllegalAccessException e) {
      e.printStackTrace();
    } catch (ClassNotFoundException e) {
      e.printStackTrace();
    }
  }
}
