/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.ChildDefinition;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.TagAttributeException;

import org.ujac.util.BeanUtils;
import org.ujac.util.UjacTypes;

import org.ujac.util.table.Column;
import org.ujac.util.table.Row;
import org.ujac.util.table.Table;
import org.ujac.util.table.TableException;

import org.ujac.util.template.TemplateException;

/**
 * Name: AddRowTag<br>
 * Description: Class handling the &lt;add-row&gt; tag.
 * 
 * @author lauerc
 */
public class AddRowTag extends BaseDocumentTag {

  /** The item's name. */
  public static final String TAG_NAME = "add-row";

  /** The property name. */
  private String tableName = null;
  /** The distinct flag. */
  private boolean distinct = false;
  /** The table. */
  private Table table = null;
  /** The row. */
  private Row row = null;
  /** The map to hold values for distinct checks. */
  private Map fieldValues;
  
  /**
   * Constructs a AddRowTag instance with no specific attributes.
   */
  public AddRowTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Adds a row to the specified table." ;
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.ALTER_TABLE)
      .addDefinition(CommonAttributes.ALTER_DISTINCT);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition( new ChildDefinition(SetFieldTag.class, 0, ChildDefinition.UNLIMITED)); 
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
  }
  
  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    // getting attributes
    this.tableName = getStringAttribute(CommonAttributes.ALTER_TABLE, false, null);
    this.distinct = getBooleanAttribute(CommonAttributes.ALTER_DISTINCT, true, null);
    
    // getting the table
    Object to = documentHandler.getTemplateContext().getProperty(tableName);
    if (to == null) {
      throw new TagAttributeException(locator(), "Can't add row to undefined table '" + tableName + "'.");
    }
    this.table = (Table) to;
    try {
      if (distinct) {
        this.fieldValues = new HashMap();
      } else {
        // creating the row.
        this.row = table.addRow();
      }
    } catch (TableException ex) {
      throw new DocumentHandlerException(
          locator(),
          "Failed to add row to table '" + tableName + "': " + ex.getMessage(), 
          ex
      );
    }
  }

  /**
   * Closes the tag.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the macro-param item.
   */
  public void closeItem() throws DocumentHandlerException {

    if (!isValid()) {
      return;
    }
   
    if (distinct) {
      try {
        int numRows = table.getRowCount();
        int numValues = fieldValues.size();
        int[] fieldIds = new int[numValues];
        Object[] fieldVals = new Object[numValues];
        Set valueEntries = fieldValues.entrySet();
        Iterator iterValueEntries = valueEntries.iterator();
        int idx = 0;
        while (iterValueEntries.hasNext()) {
          Map.Entry entry = (Map.Entry) iterValueEntries.next();
          String key = (String) entry.getKey();
          Column col = table.getColumn(key);
          fieldIds[idx] = col.getIndex();
          fieldVals[idx] = entry.getValue();
          ++idx;
        }
        
        boolean rowMatched = false;
       compareLoop:
        for (int i = 0; i < numRows; i++) {
          Row row = table.getRow(i);
          for (int j = 0; j < numValues; j++) {
            Object rowVal = row.getObject(fieldIds[j]);
            Object val = fieldVals[j];
            if (!BeanUtils.equals(rowVal, val)) {
              continue compareLoop;
            }
            // matching row found!
            rowMatched = true;
            break compareLoop;
          }
        }
        
        if (!rowMatched) {
          // adding values to table
          row = table.addRow();
          for (int j = 0; j < numValues; j++) {
            row.setObject(fieldIds[j], fieldVals[j]);
          }
        }
      } catch (TableException ex) {
        throw new DocumentHandlerException(locator(), "Failed to perform distinct checks: " + ex.getMessage(), ex);
      }
    }
    
    // adding table to document properties
    documentHandler.getTemplateContext().setProperty(this.tableName, this.table);
  }
  
  /**
   * Gets the table to fill the values in.
   * @return The table to fill the values in.
   */
  public Table getTable() {
    return table;
  }
  
  /**
   * Sets a value for the new row. 
   * @param fieldName The name of the field to set.
   * @param value The value to set.
   * @exception DocumentHandlerException Thrown in case a type mismatch problem occurred.
   */
  public void setFieldValue(String fieldName, String value)
    throws DocumentHandlerException {
    
    try {
      Column column = table.getColumn(fieldName);
      Object val = null;
      switch (column.getType()) {
        case UjacTypes.TYPE_INT:
          val = new Integer(documentHandler.evalInt(value));
          break;
        case UjacTypes.TYPE_FLOAT:
          val = new Float(documentHandler.evalFloat(value));
          break;
        case UjacTypes.TYPE_LONG:
          val = new Long(documentHandler.evalLong(value));
          break;
        case UjacTypes.TYPE_DOUBLE:
          val = new Double(documentHandler.evalDouble(value));
          break;
        case UjacTypes.TYPE_STRING:
          val = documentHandler.executeTemplate(value);
          break;
        case UjacTypes.TYPE_DATE:
          val = documentHandler.evalDate(value);
          break;
        case UjacTypes.TYPE_TIME:
          val = documentHandler.evalTime(value);
          break;
        case UjacTypes.TYPE_TIMESTAMP:
          val = documentHandler.evalTimestamp(value);
          break;
        default: // assuming type 'Object'
          val = documentHandler.evalObject(value);
          break;
      }

      if (distinct) {
        fieldValues.put(fieldName, val);
      } else {
        row.setObject(fieldName, val);
      }
    } catch (TableException ex) {
      throw new DocumentHandlerException(locator(), "Failed to set value '" + value + "' for field '" + fieldName + "'.", ex);
    } catch (TemplateException ex) {
      throw new DocumentHandlerException(locator(), "Failed to evaluate value '" + value + "' for field '" + fieldName + "'.", ex);
    }
  }
  
}
