/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import java.net.MalformedURLException;
import java.net.URL;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.ElementContainer;
import org.ujac.print.PhraseHolder;
import org.ujac.print.TagAttributeException;

import org.ujac.util.template.TemplateException;

import com.lowagie.text.Anchor;
import com.lowagie.text.Chunk;

/**
 * Name: AnchorTag<br>
 * Description: A tag handling document anchors, needed for navigation in documents.
 * 
 * @author lauerc
 */
public class AnchorTag extends BaseElementTag {

  /** Definition of the 'name' attribute. */
  private static final AttributeDefinition ANCHOR_NAME = CommonAttributes.NAME.cloneAttrDef("The anchor name.")
      .setRequired(false);
  /** Definition of the 'reference' attribute. */
  private static final AttributeDefinition REFERENCE = new AttributeDefinition(TagAttributes.ATTR_REFERENCE, AttributeDefinition.TYPE_STRING, false, 
      "The anchor reference.");

  /** The item's name. */
  public static final String TAG_NAME = "anchor";
  
  /** The phrase holder. */
  protected PhraseHolder phraseHolder = null;
  
  /**
   * Constructs a AnchorTag instance with no specific attributes.
   */
  public AnchorTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Adds an anchor to the document.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.CLASS)
      .addDefinition(CommonAttributes.STYLE)
      .addDefinition(ANCHOR_NAME)
      .addDefinition(REFERENCE)
      .addDefinition(CommonAttributes.FONT);
  }

  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.FONT_NAME);
  }
  
  /**
   * Tells whether textual content is allowed for the tag or not.
   * @return true if the item is allowed to contain textual content, else false.
   */
  public boolean isTextBodyAllowed() {
    return true;
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // setting the phrase holder
    this.phraseHolder = documentHandler.latestPhraseHolder();
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    String content = getContent();
    if (content == null) {
      content = "";
    } else {
      try {
        content = documentHandler.executeTemplate(getContent());
      } catch (TemplateException ex) {
        throw new DocumentHandlerException(locator(), "Content evaluation failed: " + ex.getMessage(), ex);
      }
    }
    // processing escape sequences
    StringBuffer cntBuf = new StringBuffer();
    filterContents(this, content, cntBuf, false);
    content = cntBuf.toString();
    
    String name = getStringAttribute(ANCHOR_NAME, true, null);
    String reference = getStringAttribute(REFERENCE, true, null);
    if ((name != null) && (reference != null)) {
      throw new TagAttributeException(locator(), "Please define eighter the name or the reference of an anchor, not both!");
    }
    
    ElementContainer elementContainer = getElementContainer();
    if (name != null) {
      // creating anchor
      Anchor anchor = new Anchor(content, getFont().getFont());
      anchor.setName(name);
      elementContainer.addElement(this, anchor);
    }
    if (reference != null) {
      // creating reference
      if (phraseHolder != null) {
        try {
          Chunk chunk = createChunk(content, getFont());
          if (documentHandler.isDestinationsSupported()) {
            if (reference.startsWith("#")){
              chunk.setLocalGoto(reference.substring(1));
            } else {
              chunk.setAnchor(new URL(reference));
            } 
          }
          phraseHolder.addChunk(this, chunk);
        } catch (MalformedURLException ex) {
          throw new TagAttributeException(locator(), "Detected illegal URL '" + reference + "' as reference for an anchor.", ex);
        }
      } else {
        Anchor anchor = new Anchor(content, getFont().getFont());
        anchor.setReference(reference);
        elementContainer.addElement(this, anchor);
      }
    }
    
  }
 
}
