/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.TagAttributeException;
import org.ujac.print.TagBuilder;

/**
 * Name: AttributeTag<br>
 * Description: Specifies attributes for the surrounding attribute holder.
 * 
 * @author lauerc
 */
public class AttributeTag extends BaseDocumentTag {

  /** Definition of the 'name' attribute. */
  private static final AttributeDefinition ATTR_NAME =  
      CommonAttributes.NAME.cloneAttrDef("The name of the attribute to set.");
  /** Definition of the 'value' attribute. */
  private static final AttributeDefinition ATTR_VALUE = new AttributeDefinition(TagAttributes.ATTR_VALUE, 
      AttributeDefinition.TYPE_STRING, false, "The attribute value.");

  /** The item's name. */
  public static final String TAG_NAME = "attribute";
  
  /** The custom tag definition item. */
  private CustomTagTag customTag = null;

  /** The name attribute. */
  private String name = null;
  /** The name attribute. */
  private String value = null;

  /**
   * Constructs a AttributeTag instance with no specific attributes.
   */
  public AttributeTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Specifies an attribute for the enclosing custom tag definition.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(ATTR_NAME)
      .addDefinition(ATTR_VALUE);                         
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // getting the surrounding item
    this.customTag = (CustomTagTag) documentHandler.latestOfType(CustomTagTag.class);
    // getting attributes
    this.name = getStringAttribute(CommonAttributes.NAME, false, null);
    if (TagBuilder.ATTR_TAG_NAME.equals(name)) {
      throw new TagAttributeException(locator(), "The attribute '" + TagBuilder.ATTR_TAG_NAME + "' is no legal custom attribute.");
    }
    this.value = getStringAttribute(ATTR_VALUE, false, null);
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the image item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }
    
    customTag.setAttribute(name, value);
  }
  
}
