/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.DocumentPageEventHandler;
import org.ujac.print.PagePosition;

import com.lowagie.text.Element;
import com.lowagie.text.Image;
import com.lowagie.text.Document;
import com.lowagie.text.DocumentException;
import com.lowagie.text.Rectangle;

import com.lowagie.text.pdf.PdfWriter;
import com.lowagie.text.pdf.PdfContentByte;

/**
 * Name: BackgroundImageTag<br>
 * Description: A class handling &lt;background-image&gt; items.<br>
 * 
 * @author lauerc
 */
public class BackgroundImageTag extends BaseImageTag {

  /** Definition of the 'source' attribute. */
  private static final AttributeDefinition IMAGE_SOURCE = CommonAttributes.IMAGE_SOURCE.cloneAttrDef().setRequired(false);

  /** The item's name. */
  public static final String TAG_NAME = "background-image";
  
  /**
   * Name: ImagePageEventHandler<br>
   * Description: The page event handler, handling the output of absolutely positioned images.
   */
  class ImagePageEventHandler extends DocumentPageEventHandler {
    
    /** The image to display. */
    private Image image = null;
    
    /**
     * Constructs a ImagePageEventHandler instance with specific attributes.
     * @param image The image to display.
     */
    public ImagePageEventHandler(Image image) {
      this.image = image;
    }
    
    /**
     * We override the onEndPage method.
     * @param writer The document writer.
     * @param document The iText document.
     */
    public void onEndPage(PdfWriter writer, Document document) {
      insertImage(writer, document, image);
    }
    
    /**
     * We override the onCloseDocument method.
     * @param writer The document writer.
     * @param document The iText document.
     */
    public void onCloseDocument(PdfWriter writer, Document document) {
    }

    /**
     * Tells whether this event handler is temporary, and should be removed after the 
     * first usage or not.
     * @return true, if this handler should be executed only once, else false.
     */
    public boolean isTemporary() {
      return false;
    }
  }
  
  /** The horizontal position. */
  private PagePosition pos = null;
  /** The scale-to-fit flag. */
  private boolean scaleToFit = false;
  /** The rotation angle. */
  private float rotate = 0;
  /** The image's horizontal alignment. */
  private int hAlign = 0;
  /** The image's vertical alignment. */
  private int vAlign = 0;
  /** The antialiasing flag. */
  protected boolean antialiasing = false;
  
  /** The surrounding &lt;sticky&gt; tag. */
  private StickyTag stickyTag = null;
  
  /**
   * Constructs a BackgroundImageTag instance with no specific attributes.
   */
  public BackgroundImageTag() {
    super(TAG_NAME);
  }
  /**
   * Constructs a BackgroundImageTag instance with specific attributes.
   * @param name The tag name.
   */
  public BackgroundImageTag(String name) {
    super(name);
  }
  
  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Inserts a image into the document. Images may be places absolutely on the page or " +
      "relative to its parent item. The image gets loaded by the resource loader, which is " +
      "specified at the document handler.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.CLASS)
      .addDefinition(CommonAttributes.STYLE)
      .addDefinition(IMAGE_SOURCE)
      .addDefinition(CommonAttributes.IMAGE_X)
      .addDefinition(CommonAttributes.IMAGE_Y)
      .addDefinition(CommonAttributes.IMAGE_WIDTH)
      .addDefinition(CommonAttributes.IMAGE_HEIGHT)
      .addDefinition(CommonAttributes.ROTATE)
      .addDefinition(CommonAttributes.SCALE_TO_FIT)
      .addDefinition(CommonAttributes.IMAGE_PAGE)
      .addDefinition(CommonAttributes.IMAGE_HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonAttributes.IMAGE_VERTICAL_ALIGNMENT);
  }

  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.IMAGE_X)
      .addDefinition(CommonStyleAttributes.IMAGE_Y)
      .addDefinition(CommonStyleAttributes.IMAGE_WIDTH)
      .addDefinition(CommonStyleAttributes.IMAGE_HEIGHT)
      .addDefinition(CommonStyleAttributes.SCALE_TO_FIT)
      .addDefinition(CommonStyleAttributes.HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.VERTICAL_ALIGNMENT);
  }

  /**
   * Gets the image's page position.
   * @return The image's page position.
   */
  public PagePosition getPosition() {
    return pos;
  }

  /**
   * Gets the image width.
   * @return The image width.
   */
  public float getWidth() {
    return width;
  }
  
  /**
   * Gets the image height.
   * @return The image height.
   */
  public float getHeight() {
    return height;
  }
  
  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    
    stickyTag = (StickyTag) documentHandler.latestOfType(StickyTag.TAG_NAME);
    if (stickyTag == null) {
      throw new DocumentHandlerException(locator(), "The <" + getName() + "> tag needs to be nested within a <" + StickyTag.TAG_NAME + "> tag.");
    }
  }
  
  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the image item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    // setting the attributes
    float x = getDimensionAttribute(CommonAttributes.IMAGE_X, true, CommonStyleAttributes.IMAGE_X);
    float y = getDimensionAttribute(CommonAttributes.IMAGE_Y, true, CommonStyleAttributes.IMAGE_Y);
    pos = new PagePosition(x, y);
    width = getDimensionAttribute(CommonAttributes.IMAGE_WIDTH, true, CommonStyleAttributes.IMAGE_WIDTH);
    height = getDimensionAttribute(CommonAttributes.IMAGE_HEIGHT, true, CommonStyleAttributes.IMAGE_HEIGHT);
    if (isAttributeSupported(CommonAttributes.SCALE_TO_FIT, CommonStyleAttributes.SCALE_TO_FIT)) {
      scaleToFit = getBooleanAttribute(CommonAttributes.SCALE_TO_FIT, false, true, CommonStyleAttributes.SCALE_TO_FIT);
    }
    rotate = getFloatAttribute(CommonAttributes.IMAGE_ROTATE, true, CommonStyleAttributes.IMAGE_ROTATE);
    if (isAttributeSupported(IMAGE_SOURCE)) {
      sourceAttr = getStringAttribute(IMAGE_SOURCE, true, null);
    }
    if (isAttributeSupported(CommonAttributes.IMAGE_PAGE)) {
      pageAttr = getStringAttribute(CommonAttributes.IMAGE_PAGE, true, null);
    }
    hAlign = getHorizontalAlignmentAttribute(CommonAttributes.IMAGE_HORIZONTAL_ALIGNMENT, true, CommonStyleAttributes.IMAGE_HORIZONTAL_ALIGNMENT);
    vAlign = getVerticalAlignmentAttribute(CommonAttributes.IMAGE_VERTICAL_ALIGNMENT, true, CommonStyleAttributes.IMAGE_VERTICAL_ALIGNMENT);
  }

  /**
   * Processes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    Image image = getImage();
    
    float width = this.width;
    float height = this.height;
    if (image != null) {
      if ((width > 0) && (height > 0)) {
        if (scaleToFit) {
          image.scaleToFit(width, height);
        } else {
          image.scaleAbsolute(width, height);
        }
      } else if (width > 0) {
        float pw = image.getPlainWidth();
        float ph = image.getPlainHeight();
        image.scaleAbsolute(width, (width / pw) * ph);
      } else if (height > 0) {
        float pw = image.getPlainWidth();
        float ph = image.getPlainHeight();
        image.scaleAbsolute((height / ph) * pw, height);
      }
      if (rotate != 0.0F) {
        image.setRotation((float)Math.PI * rotate / 180);
      }
      width = image.getScaledWidth();
      height = image.getScaledHeight();
      
      Document document = documentHandler.getDocument();
      Rectangle pageSize = document.getPageSize();
      if (!isAttributeDefined(CommonAttributes.IMAGE_Y, CommonStyleAttributes.IMAGE_Y)) {
        switch (vAlign) {
          case Element.ALIGN_TOP:
            pos.setY(pageSize.getHeight() - document.bottomMargin() - height);
            break;
          case Element.ALIGN_BOTTOM:
            pos.setY(document.topMargin());
            break;
          case Element.ALIGN_MIDDLE:
            pos.setY(pageSize.getHeight() / 2.0F - height / 2.0F);
            break;
          default: // same as bottom
            pos.setY(document.topMargin());
            break;
        }
      }
      
      if (!isAttributeDefined(CommonAttributes.IMAGE_X, CommonStyleAttributes.IMAGE_X)) {
        switch (hAlign) {
          case Element.ALIGN_LEFT:
            pos.setX(document.leftMargin());
            break;
          case Element.ALIGN_RIGHT:
            pos.setX(pageSize.getWidth() - document.rightMargin() - width);
            break;
          case Element.ALIGN_CENTER:
            pos.setX(pageSize.getWidth() / 2.0F - width / 2.0F);
            break;
          default: // same as left
            pos.setX(document.leftMargin());
            break;
        }
      }
      
      // adding page event handler, which inserts the PDF image at the end of the document.
      stickyTag.addStickyElementHandler(new ImagePageEventHandler(image));
    }
  }
  
  /**
   * Inserts the given image into the document.
   * @param writer The pdf writer.
   * @param document The document to add the image to.
   * @param image The image to insert.
   */
  protected void insertImage(PdfWriter writer, Document document, Image image) {
    try {
      // calculating the correct image position
      Rectangle pageSize = document.getPageSize();
      float x = pos.calcX(pageSize.getRight(), image.getScaledWidth());
      float y = pos.calcY(pageSize.getTop(), image.getScaledHeight());
      // placing the image absolutely into the page
      image.setAbsolutePosition(x, y);
      PdfContentByte cb = writer.getDirectContentUnder();
      cb.addImage(image);      
    } catch (DocumentException ex) {
      log.error("Image output failed: " + ex.getMessage(), ex);
      throw new RuntimeException(ex.getMessage());
    }
  }
  
}
