/*
 * Copyright (C) 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.ElementContainer;
import org.ujac.print.PhraseHolder;
import org.ujac.print.TagAttributeException;

import org.ujac.util.exi.ExpressionException;

import com.lowagie.text.Chunk;
import com.lowagie.text.Image;
import com.lowagie.text.pdf.BarcodePDF417;

/**
 * Name: BarcodePDF417Tag<br>
 * Description: A class creating PDF-417 barcodes.
 * 
 * @author lauerc
 */
public class BarcodePdf417Tag extends BaseElementTag {

  /** Definition of the 'type' attribute. */
  private static final AttributeDefinition OPTIONS_TYPE = new AttributeDefinition(
      "options", new String[] {
          "USE_ASPECT_RATIO", "FIXED_RECTANGLE", "FIXED_COLUMNS", "FIXED_ROWS", "AUTO_ERROR_LEVEL", 
          "USE_ERROR_LEVEL", "FORCE_BINARY", "INVERT_BITMAP", "USE_MACRO"
      }, false, "The encodation type.");
  /** Definition of the 'aspect-ratio' attribute. */
  private static final AttributeDefinition ASPECT_RATIO = new AttributeDefinition(
      "aspect-ratio", AttributeDefinition.TYPE_FLOAT, false, "The barcode aspect ratio."
  );
  /** Definition of the 'code-columns' attribute. */
  private static final AttributeDefinition CODE_COLUMNS = new AttributeDefinition(
      "code-columns", AttributeDefinition.TYPE_INT, false, "The number of barcode data columns."
  );
  /** Definition of the 'code-rows' attribute. */
  private static final AttributeDefinition CODE_ROWS = new AttributeDefinition(
      "code-rows", AttributeDefinition.TYPE_INT, false, "The number of barcode data rows."
  );
  /** Definition of the 'error-level' attribute. */
  private static final AttributeDefinition ERROR_LEVEL = new AttributeDefinition(
      "error-level", AttributeDefinition.TYPE_INT, false, "The error level correction used for the barcode."
  );
  /** Definition of the 'len-codewords' attribute. */
  private static final AttributeDefinition LEN_CODEWORDS = new AttributeDefinition(
      "len-codewords", AttributeDefinition.TYPE_INT, false, "The length of the codewords."
  );
  /** Definition of the 'macro-file-id' attribute. */
  private static final AttributeDefinition MACRO_FILE_ID = new AttributeDefinition(
      "macro-file-id", AttributeDefinition.TYPE_INT, false, "The File ID for macro PDF417 encoding."
  );
  /** Definition of the 'macro-segment-count' attribute. */
  private static final AttributeDefinition MACRO_SEGMENT_COUNT = new AttributeDefinition(
      "macro-segment-count", AttributeDefinition.TYPE_INT, false, "The segment count for macro PDF417 encoding."
  );
  /** Definition of the 'y-height' attribute. */
  private static final AttributeDefinition Y_HEIGHT = new AttributeDefinition(
      "y-height", AttributeDefinition.TYPE_FLOAT, false, "The Y pixel height relative to X. This is usually 3."
  );

  /** The item's name. */
  public static final String TAG_NAME = "barcode-pdf-417";
  
  /** The barcode type map. */
  private static Map barcodeTypes = null;
  
  static {
    // setting up barcode type map
    barcodeTypes = new HashMap();
    barcodeTypes.put("USE_ASPECT_RATIO", new Integer(BarcodePDF417.PDF417_USE_ASPECT_RATIO));
    barcodeTypes.put("FIXED_RECTANGLE", new Integer(BarcodePDF417.PDF417_FIXED_RECTANGLE));
    barcodeTypes.put("FIXED_COLUMNS", new Integer(BarcodePDF417.PDF417_FIXED_COLUMNS));
    barcodeTypes.put("FIXED_ROWS", new Integer(BarcodePDF417.PDF417_FIXED_ROWS));
    barcodeTypes.put("AUTO_ERROR_LEVEL", new Integer(BarcodePDF417.PDF417_AUTO_ERROR_LEVEL));
    barcodeTypes.put("USE_ERROR_LEVEL", new Integer(BarcodePDF417.PDF417_USE_ERROR_LEVEL));
    barcodeTypes.put("FORCE_BINARY", new Integer(BarcodePDF417.PDF417_FORCE_BINARY));
    barcodeTypes.put("INVERT_BITMAP", new Integer(BarcodePDF417.PDF417_INVERT_BITMAP));
    barcodeTypes.put("USE_MACRO", new Integer(BarcodePDF417.PDF417_USE_MACRO));
  }
  
  /** The latest phrase holder. */
  private PhraseHolder phraseHolder = null;

  /** The options to generate the barcode. */
  private int options = 0;
  /** The aspect ratio. */
  private float aspectRatio = 0.0F;
  /** The code columns. */
  private int codeColumns = 0;
  /** The code rows. */
  private int codeRows = 0;
  /** The level of error correction. */
  private int errorLevel = 0;
  /** The length of the codewords. */
  private int lenCodewords = 0;
  /** The File ID for macro PDF417 encoding . */
  private String marcoFileId = null;
  /** The segment count for macro PDF417 encoding. */
  private int macroSegmentCount = 1;
  /** The segment ID for macro PDF417 encoding. */
  private int macroSegmentId = 0;
  /** The text that will form the barcode. */
  private String text = null;
  /** The Y pixel height relative to X. */
  private float yHeight = 3.0F;
  /** The width of the barcode area. */
  private float width = 0.0F;
  /** The height of the barcode area. */
  private float height = 0.0F;

  /**
   * Constructs a DatamatrixTag instance with no specific attributes.
   */
  public BarcodePdf417Tag() {
    super(TAG_NAME);
  }
  
  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Prints a 2d PDF barcode (PDF-417). The tag content defines the text, that will form the barcode.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(OPTIONS_TYPE)
      .addDefinition(ASPECT_RATIO)
      .addDefinition(CODE_COLUMNS)
      .addDefinition(CODE_ROWS)
      .addDefinition(ERROR_LEVEL)
      .addDefinition(LEN_CODEWORDS)
      .addDefinition(MACRO_FILE_ID)
      .addDefinition(MACRO_SEGMENT_COUNT)
      .addDefinition(Y_HEIGHT)
      .addDefinition(CommonAttributes.IMAGE_WIDTH)
      .addDefinition(CommonAttributes.IMAGE_HEIGHT);
  }

  /**
   * Tells whether textual content is allowed for the tag or not.
   * @return true if the item is allowed to contain textual content, else false.
   */
  public boolean isTextBodyAllowed() {
    return true;
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // determining the latest phrase holder
    phraseHolder = documentHandler.latestPhraseHolder();
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }
  }

  /**
   * Processes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    // getting attributes
    String optionsStr = getStringAttribute(OPTIONS_TYPE, true, null);
    if (optionsStr != null) {
      StringTokenizer optionTkn = new StringTokenizer(optionsStr, "|");
      while (optionTkn.hasMoreTokens()) {
        String strOption = optionTkn.nextToken();
        Integer option = (Integer) barcodeTypes.get(strOption);
        if (option == null) {
          throw new TagAttributeException(
              locator(), "Unsupported option '" + strOption + "', supported options are: " + 
              "USE_ASPECT_RATIO, FIXED_RECTANGLE, FIXED_COLUMNS, FIXED_ROWS, AUTO_ERROR_LEVEL, USE_ERROR_LEVEL, " + 
              "FORCE_BINARY, INVERT_BITMAP, USE_MACRO."
          );
        }
        options |= option.intValue();
      }
    }
    this.aspectRatio = getFloatAttribute(ASPECT_RATIO, true, null);
    this.codeColumns = getIntegerAttribute(CODE_COLUMNS, true, null);
    this.codeRows = getIntegerAttribute(CODE_ROWS, true, null);
    this.errorLevel = getIntegerAttribute(ERROR_LEVEL, true, null);
    this.lenCodewords = getIntegerAttribute(LEN_CODEWORDS, true, null);
    this.marcoFileId = getStringAttribute(MACRO_FILE_ID, true, null);
    this.macroSegmentCount = getIntegerAttribute(MACRO_SEGMENT_COUNT, true, null);
    this.yHeight = getFloatAttribute(Y_HEIGHT, true, null);
    this.width = getDimensionAttribute(CommonAttributes.IMAGE_WIDTH, true, CommonStyleAttributes.IMAGE_WIDTH);
    this.height = getDimensionAttribute(CommonAttributes.IMAGE_HEIGHT, true, CommonStyleAttributes.IMAGE_HEIGHT);

    // evaluating the code
    this.text = null;
    try {
      text = documentHandler.evalString(getTrimmedContent());
    } catch (ExpressionException ex) {
      throw new DocumentHandlerException(locator(), "Content evaluation failed: " + ex.getMessage(), ex);
    }

    ElementContainer elementContainer = getElementContainer();
    try {
      BarcodePDF417 pdf417 = new BarcodePDF417();
      pdf417.setOptions(options);
      pdf417.setAspectRatio(aspectRatio);
      pdf417.setCodeColumns(codeColumns);
      pdf417.setCodeRows(codeRows);
      pdf417.setErrorLevel(errorLevel);
      pdf417.setLenCodewords(lenCodewords);
      pdf417.setMacroFileId(marcoFileId);
      pdf417.setMacroSegmentId(macroSegmentId);
      pdf417.setMacroSegmentCount(macroSegmentCount);
      pdf417.setYHeight(yHeight);
      pdf417.setText(text);
      pdf417.paintCode();
      
      Image barcodeImage = pdf417.getImage();
      barcodeImage.scaleAbsolute(width, height);
      if (phraseHolder != null) {
        phraseHolder.addChunk(this, new Chunk(barcodeImage, 0, 0));
      } else {
        elementContainer.addElement(this, barcodeImage);
      }
    } catch (DocumentHandlerException ex) {
      throw ex;
    } catch (Throwable ex) {
      throw new TagAttributeException(locator(), "Creation of 2D barcode failed: " + ex.getMessage(), ex);
    }
  }
}
