/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import java.awt.Color;

import java.util.Map;
import java.util.HashMap;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.AttributeDefinition;
import org.ujac.print.ElementContainer;
import org.ujac.print.TagAttributeException;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.PhraseHolder;

import org.ujac.util.exi.ExpressionException;

import com.lowagie.text.Chunk;
import com.lowagie.text.Image;

import com.lowagie.text.pdf.Barcode;
import com.lowagie.text.pdf.Barcode39;
import com.lowagie.text.pdf.BarcodeCodabar;
import com.lowagie.text.pdf.PdfWriter;
import com.lowagie.text.pdf.Barcode128;
import com.lowagie.text.pdf.BarcodeEAN;
import com.lowagie.text.pdf.BarcodeInter25;
import com.lowagie.text.pdf.BarcodePostnet;
import com.lowagie.text.pdf.PdfContentByte;

/**
 * Name: BarcodeTag<br>
 * Description: A class handling barcode tags.
 * 
 * @author lauerc
 */
public class BarcodeTag extends BaseElementTag {

  /** Definition of the 'type' attribute. */
  private static final AttributeDefinition BARCODE_TYPE = new AttributeDefinition(
      TagAttributes.ATTR_TYPE, new String[] {
          "CODABAR", "CODE39", "CODE39EXT", "INTER25", "CODE128", "CODE128UCC", "CODE128RAW", 
          "EAN13", "EAN8", "PLANET", "POSTNET", "SUPP2", "SUPP5", "UPCA", "UPCE"
      }, true, "The barcode type.");
  /** Definition of the 'bar-width' attribute. */
  private static final AttributeDefinition BARCODE_BAR_WIDTH = new AttributeDefinition(
      TagAttributes.ATTR_BAR_WIDTH, AttributeDefinition.TYPE_DIMENSION, false, "The width of one single bar.");
  /** Definition of the 'bar-height' attribute. */
  private static final AttributeDefinition BARCODE_BAR_HEIGHT = new AttributeDefinition(
      TagAttributes.ATTR_BAR_HEIGHT, AttributeDefinition.TYPE_DIMENSION, false, "The height of a bar.");
  /** Definition of the 'bar-color' attribute. */
  private static final AttributeDefinition BARCODE_BAR_COLOR = new AttributeDefinition(
      TagAttributes.ATTR_BAR_COLOR, AttributeDefinition.TYPE_COLOR, false, "The bar color.");
  /** Definition of the 'text-color' attribute. */
  private static final AttributeDefinition BARCODE_TEXT_COLOR = new AttributeDefinition(
      TagAttributes.ATTR_TEXT_COLOR, AttributeDefinition.TYPE_COLOR, false, "The color of the text below the bar.");
  /** Definition of the 'rotate' attribute. */
  private static final AttributeDefinition BARCODE_ROTATE = new AttributeDefinition(
      TagAttributes.ATTR_ROTATE, AttributeDefinition.TYPE_BOOLEAN, false, "The rotation flag.");
  /** Definition of the 'start-stop-text' attribute. */
  private static final AttributeDefinition BARCODE_START_STOP_TEXT = new AttributeDefinition(
      TagAttributes.ATTR_START_STOP_TEXT, AttributeDefinition.TYPE_BOOLEAN, false, "The start-stop text flag.");
  /** Definition of the 'generate-checksum' attribute. */
  private static final AttributeDefinition BARCODE_GENERATE_CHECKSUM = new AttributeDefinition(
      TagAttributes.ATTR_GENERATE_CHECKSUM, AttributeDefinition.TYPE_BOOLEAN, false, "The flag deciding, whether to generate a checksum or not.");
  /** Definition of the 'baseline' attribute. */
  private static final AttributeDefinition BARCODE_BASELINE = new AttributeDefinition(
      TagAttributes.ATTR_BASELINE, AttributeDefinition.TYPE_FLOAT, false, 
      "If positive the text distance above the bars, if negative, the text " +
      "distance below the bars.");

  /** Definition of the 'font' attribute. */
  private static final AttributeDefinition BARCODE_FONT = 
      CommonAttributes.FONT.cloneAttrDef("The name of the font to use for the barcode label.");
  /** Definition of the 'font' style attribute. */
  private static final AttributeDefinition BARCODE_FONT_STYLE = 
    CommonStyleAttributes.FONT.cloneAttrDef("The name of the font to use for the barcode label.");

  /** The item's name. */
  public static final String TAG_NAME = "barcode";
  
  /** Constant for the code 39 barcode. */
  public static final int CODE39 = 30;
  /** Constant for the code 39 extended barcode. */
  public static final int CODE39EXT = 31;
  /** Constant for the Interleave 2 of 5 barcode. */
  public static final int INTER25 = 32;
  /** The barcode type map. */
  private static Map barcodeTypes = null;
  
  static {
    // setting up barcode type map
    barcodeTypes = new HashMap();
    barcodeTypes.put("CODABAR", new Integer(Barcode.CODABAR));
    barcodeTypes.put("CODE39", new Integer(CODE39));
    barcodeTypes.put("CODE39EXT", new Integer(CODE39EXT));
    barcodeTypes.put("INTER25", new Integer(INTER25));
    barcodeTypes.put("CODE128", new Integer(Barcode.CODE128));
    barcodeTypes.put("CODE128UCC", new Integer(Barcode.CODE128_UCC));
    barcodeTypes.put("CODE128RAW", new Integer(Barcode.CODE128_RAW));
    barcodeTypes.put("EAN13", new Integer(Barcode.EAN13));
    barcodeTypes.put("EAN8", new Integer(Barcode.EAN8));
    barcodeTypes.put("PLANET", new Integer(Barcode.PLANET));
    barcodeTypes.put("POSTNET", new Integer(Barcode.POSTNET));
    barcodeTypes.put("SUPP2", new Integer(Barcode.SUPP2));
    barcodeTypes.put("SUPP5", new Integer(Barcode.SUPP5));
    barcodeTypes.put("UPCA", new Integer(Barcode.UPCA));
    barcodeTypes.put("UPCE", new Integer(Barcode.UPCE));
  }
  
  /** The latest phrase holder. */
  private PhraseHolder phraseHolder = null;

  /** The barcode type attribute. */
  private String typeAttr = null;
  /** The bar width attribute. */
  private float barWidth = 0.0F;
  /** The bar height attribute. */
  private float barHeight = 0.0F;
  /** The bar color. */
  private Color barColor = null;
  /** The text color. */
  private Color textColor = null;
  /** The rotate flag. */
  private boolean rotate = false;
  /** The start stop text flag. */
  private boolean startStopTextAttr = false;
  /** The generate checksum flag. */
  private String generateChecksumAttr = null;
  /** The baseline flag. */
  private String baselineAttr = null;
  
  /**
   * Constructs a BarcodeTag instance with no specific attributes.
   */
  public BarcodeTag() {
    super(TAG_NAME);
  }
  
  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Prints a barcode.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(BARCODE_TYPE)
      .addDefinition(BARCODE_FONT)
      .addDefinition(BARCODE_BAR_WIDTH)
      .addDefinition(BARCODE_BAR_HEIGHT)
      .addDefinition(BARCODE_BAR_COLOR)
      .addDefinition(BARCODE_TEXT_COLOR)
      .addDefinition(BARCODE_ROTATE)
      .addDefinition(BARCODE_START_STOP_TEXT)
      .addDefinition(BARCODE_GENERATE_CHECKSUM)
      .addDefinition(BARCODE_BASELINE);
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(BARCODE_FONT_STYLE)
      .addDefinition(BARCODE_BAR_WIDTH)
      .addDefinition(BARCODE_BAR_HEIGHT)
      .addDefinition(BARCODE_BAR_COLOR)
      .addDefinition(BARCODE_TEXT_COLOR);
  }

  /**
   * Tells whether textual content is allowed for the tag or not.
   * @return true if the item is allowed to contain textual content, else false.
   */
  public boolean isTextBodyAllowed() {
    return true;
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // determining the latest phrase holder
    phraseHolder = documentHandler.latestPhraseHolder();
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }
  }

  /**
   * Processes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    // getting attributes
    this.typeAttr = getStringAttribute(BARCODE_TYPE, true, null);
    this.barWidth = getDimensionAttribute(BARCODE_BAR_WIDTH, true, BARCODE_BAR_WIDTH);
    this.barHeight = getDimensionAttribute(BARCODE_BAR_HEIGHT, true, BARCODE_BAR_HEIGHT);
    this.barColor = getColorAttribute(BARCODE_BAR_COLOR, true, BARCODE_BAR_COLOR);
    this.textColor = getColorAttribute(BARCODE_TEXT_COLOR, true, BARCODE_TEXT_COLOR);
    this.rotate = getBooleanAttribute(BARCODE_ROTATE, true, null);
    this.startStopTextAttr = getBooleanAttribute(BARCODE_START_STOP_TEXT, false, true, null);
    this.generateChecksumAttr = getStringAttribute(BARCODE_GENERATE_CHECKSUM, true, null);
    this.baselineAttr = getStringAttribute(BARCODE_BASELINE, true, null);

    // evaluating the code
    String code = null;
    try {
      code = documentHandler.evalString(getTrimmedContent());
    } catch (ExpressionException ex) {
      throw new DocumentHandlerException(locator(), "Content evaluation failed: " + ex.getMessage(), ex);
    }

    PdfContentByte cb = ((PdfWriter) documentHandler.getDocumentWriter()).getDirectContent();
    Barcode barcode = null;
    String typeString = null;
    try {
      typeString = documentHandler.evalString(typeAttr).toUpperCase();
    } catch (ExpressionException ex) {
      throw new TagAttributeException(locator(), "Attribute evaluation failed: " + ex.getMessage(), ex);
    }
      
    Integer type = (Integer) barcodeTypes.get(typeString);
    if (type == null) {
      throw new TagAttributeException(locator(), "Unsupported barcode type '" + typeString + "'.");
    }
    switch (type.intValue()) {
      case CODE39: 
      case CODE39EXT: 
        barcode = new Barcode39();
        if (type.intValue() == CODE39EXT) {
          barcode.setExtended(true);
        }
        break;

      case INTER25: 
        barcode = new BarcodeInter25();
        break;

      case Barcode.CODABAR:
        barcode = new BarcodeCodabar();
      break;
      
      case Barcode.EAN13: 
        if (code.length() != 13) {
          throw new TagAttributeException(locator(), "Creation of barcode '" + typeAttr + "' failed: The length must be excactly 13 characters!");
        }
        barcode = new BarcodeEAN();
        break;
      case Barcode.EAN8: 
        if (code.length() != 8) {
          throw new TagAttributeException(locator(), "Creation of barcode '" + typeAttr + "' failed: The length must be excactly 8 characters!");
        }
        barcode = new BarcodeEAN();
        break;
      case Barcode.UPCA: 
        if (code.length() != 12) {
          throw new TagAttributeException(locator(), "Creation of barcode '" + typeAttr + "' failed: The length must be excactly 12 characters!");
        }
        barcode = new BarcodeEAN();
        break;
      case Barcode.UPCE: 
        if (code.length() != 8) {
          throw new TagAttributeException(locator(), "Creation of barcode '" + typeAttr + "' failed: The length must be excactly 8 characters!");
        }
        barcode = new BarcodeEAN();
        break;
      case Barcode.SUPP2:
      case Barcode.SUPP5: 
        barcode = new BarcodeEAN();
        break;

      case Barcode.CODE128: 
      case Barcode.CODE128_UCC: 
      case Barcode.CODE128_RAW: 
        barcode = new Barcode128();
        break;

      case Barcode.PLANET: 
      case Barcode.POSTNET: 
        barcode = new BarcodePostnet();
        break;
        
      default: 
        throw new TagAttributeException(locator(), "Unsupported barcode type '" + typeAttr + "'");
    }
    barcode.setCodeType(type.intValue());

    try {
      if (baselineAttr != null) {
        float baseLine = documentHandler.evalFloat(baselineAttr);
        barcode.setBaseline(baseLine);
      }
      if ((generateChecksumAttr != null) && documentHandler.evalBoolean(generateChecksumAttr)) {
        barcode.setGenerateChecksum(true);
      }
      if (isAttributeDefined(BARCODE_BAR_WIDTH)) {
        barcode.setX(barWidth);  
      }
      if (isAttributeDefined(BARCODE_BAR_HEIGHT)) {
        barcode.setBarHeight(barHeight);
      }
    } catch (ExpressionException ex) {
      throw new TagAttributeException(locator(), "Attribute evaluation failed: " + ex.getMessage(), ex);
    }
    barcode.setFont(getFont().getFont().getBaseFont());
    barcode.setCodeType(type.intValue());
    barcode.setStartStopText(startStopTextAttr);
    barcode.setCode(code);
    
    ElementContainer elementContainer = getElementContainer();
    try {
      Image barcodeImage = barcode.createImageWithBarcode(cb, barColor, textColor);
      if (rotate) {
        barcodeImage.setRotation((float) Math.PI/2);
      }
      if (phraseHolder != null) {
        phraseHolder.addChunk(this, new Chunk(barcodeImage, 0, 0));
      } else {
        elementContainer.addElement(this, barcodeImage);
      }
    } catch (DocumentHandlerException ex) {
      throw ex;
    } catch (Throwable ex) {
      throw new TagAttributeException(locator(), "Creation of barcode '" + typeAttr + "' failed: " + ex.getMessage(), ex);
    }
  }
}
