/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import java.util.ArrayList;
import java.util.List;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.ChildDefinition;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.DocumentFont;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.ElementContainer;
import org.ujac.print.FontHolder;
import org.ujac.print.PdfCellContainer;
import org.ujac.print.PdfTableHolder;
import org.ujac.print.PhraseHolder;
import org.ujac.print.StyleHolder;
import org.ujac.print.tag.acroform.BaseAcroFieldTag;
import org.ujac.util.StringUtils;

import com.lowagie.text.Chunk;
import com.lowagie.text.Element;
import com.lowagie.text.Image;
import com.lowagie.text.Paragraph;
import com.lowagie.text.Phrase;
import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfPCell;
import com.lowagie.text.pdf.PdfPCellEvent;
import com.lowagie.text.pdf.PdfPTable;

/**
 * Name: BaseCellTag<br>
 * Description: A base class for cell tags.
 * 
 * @author lauerc
 */
public abstract class BaseCellTag 
  extends BaseDocumentTag 
  implements ElementContainer, PdfCellContainer, PhraseHolder, FontHolder, StyleHolder {

  /** The default horizontal alignment for cell contents. */
  public static final int DEFAULT_HALIGN = Element.ALIGN_LEFT; 
  /** The default vertical alignment for cell contents. */
  public static final int DEFAULT_VALIGN = Element.ALIGN_BOTTOM; 
  
  /** Definition of the 'rotate' attribute. */
  private static final AttributeDefinition CELL_ROTATE = 
    CommonAttributes.ROTATE.cloneAttrDef("The rotation angle of the cell.");
  /** Definition of the 'rotate' style attribute. */
  private static final AttributeDefinition CELL_ROTATE_STYLE = 
    CommonStyleAttributes.ROTATE.cloneAttrDef("The rotation angle of the cell.");
  
  /**
   * An event handler, aggregating all event handlers, which have been added by child tags.
   */
  class CellEventHandler implements PdfPCellEvent {
    
    /** The PDF cell event handlers. */
    private List cellEvents = null;
    
    /**
     * Constructs a BaseCellTag.CellEventHandler instance with specific attributes.
     * @param cellEventHandlers The event handler list.
     */
    public CellEventHandler(List cellEventHandlers) {
      this.cellEvents = cellEventHandlers;
    }
    
    /**
     * @see com.lowagie.text.pdf.PdfPCellEvent#cellLayout(com.lowagie.text.pdf.PdfPCell, com.lowagie.text.Rectangle, com.lowagie.text.pdf.PdfContentByte[])
     */
    public void cellLayout(PdfPCell cell, Rectangle position, PdfContentByte[] canvases) {
      int numCellEventHandlers = cellEvents.size();
      for (int i = 0; i < numCellEventHandlers; i++) {
        PdfPCellEvent eventHandler = (PdfPCellEvent) cellEvents.get(i);
        eventHandler.cellLayout(cell, position, canvases);
      }
    }
    
  }
  
  /** The current cell attributes. */
  private CellAttributes cellAttributes = null;

  /** The iText element this tag represents. */
  private PdfPCell cell = null;
  /** The nested cell. */
  private PdfPCell nestedCell = null;

  /** The PDF cell event handlers. */
  private List cellEventHandlers = null;
  
  /** The column span of the cell. */ 
  protected int colspan = 0;
  /** Tells whether to fill the current row or not. */ 
  protected boolean fillRow = false;

  /** The phrase. */
  private Phrase phrase = null;
  /** The item closed flag. */
  protected boolean itemClosed = false;

  /** Tells whether or not the PdfPCell is or may be in composite mode. */
  protected boolean composite = false;
  //added by Kiran
  /** The rotation angle. */
  private float rotate = 0.0f;
  
  /**
   * Constructs a BaseCellTag instance with specific attributes.
   * @param name The tag name.
   */
  public BaseCellTag(String name) {
    super(name);
  }
  
  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Adds a cell to its surrounding table or surrounding cell.";
  }
  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.RENDERED)
      .addDefinition(CommonAttributes.CLASS)
      .addDefinition(CommonAttributes.STYLE)
      .addDefinition(CommonAttributes.FONT)
      .addDefinition(CommonAttributes.LEADING)
      .addDefinition(CommonAttributes.LINE_SPACING)
      .addDefinition(CommonAttributes.BACKGROUND_COLOR)
      .addDefinition(CommonAttributes.COLSPAN)
      .addDefinition(CommonAttributes.FILL_ROW)
      .addDefinition(CommonAttributes.BORDER_STYLE)
      .addDefinition(CommonAttributes.BORDER_WIDTH)
      .addDefinition(CommonAttributes.BORDER_COLOR)
      .addDefinition(CommonAttributes.HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonAttributes.VERTICAL_ALIGNMENT)
      .addDefinition(CommonAttributes.PADDING)
      .addDefinition(CommonAttributes.PADDING_LEFT)
      .addDefinition(CommonAttributes.PADDING_RIGHT)
      .addDefinition(CommonAttributes.PADDING_TOP)
      .addDefinition(CommonAttributes.PADDING_BOTTOM)
      .addDefinition(CommonAttributes.CELL_HEIGHT)
      .addDefinition(CELL_ROTATE)
      .addDefinition(CommonAttributes.NO_WRAP);
  }
  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.FONT_NAME)
      .addDefinition(CommonStyleAttributes.LEADING)
      .addDefinition(CommonStyleAttributes.LINE_SPACING)
      .addDefinition(CommonStyleAttributes.BACKGROUND_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_STYLE)
      .addDefinition(CommonStyleAttributes.BORDER_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_LEFT_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_RIGHT_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_TOP_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_BOTTOM_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_LEFT_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_RIGHT_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_TOP_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_BOTTOM_COLOR)
      .addDefinition(CommonStyleAttributes.HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.VERTICAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.PADDING)
      .addDefinition(CommonStyleAttributes.PADDING_LEFT)
      .addDefinition(CommonStyleAttributes.PADDING_RIGHT)
      .addDefinition(CommonStyleAttributes.PADDING_TOP)
      .addDefinition(CommonStyleAttributes.PADDING_BOTTOM)
      .addDefinition(CommonStyleAttributes.CELL_HEIGHT)
      .addDefinition(CELL_ROTATE_STYLE)
      .addDefinition(CommonStyleAttributes.NO_WRAP);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(BaseFontTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(MacroTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(IfTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ElseTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(LogTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ForeachTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(SwitchTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(PhraseTag.class, 0, 1))
      .addDefinition(new ChildDefinition(ParagraphTag.class, 0, 1))
      .addDefinition(new ChildDefinition(PdfCellTag.class, 0, 1))
      .addDefinition(new ChildDefinition(PdfTableTag.class, 0, 1))
      .addDefinition(new ChildDefinition(PrintTableTag.class, 0, 1))
      .addDefinition(new ChildDefinition(ImageTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(SpaceTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(NewLineTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(NumberFormatTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(DateFormatTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BarcodeTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(DatamatrixTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BarcodePdf417Tag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(AnchorTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(OutlineTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ListTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BaseAcroFieldTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ImportTag.class, 0, ChildDefinition.UNLIMITED));
  }

  /**
   * Tells whether or not this tag does respect style attributes or not.
   * @return always <code>true</code>.
   */
  public boolean isStyleable() {
    return true;
  }

  /**
   * Tells whether textual content is allowed for the tag or not.
   * @return true if the item is allowed to contain textual content, else false.
   */
  public boolean isTextBodyAllowed() {
    return true;
  }
  
  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    cellEventHandlers = null;
    composite = false;
    
    if (!isValid()) {
      return;
    }
    
    //added by Kiran
    rotate = getFloatAttribute(CommonAttributes.ROTATE, 0.0f,  false, CommonStyleAttributes.ROTATE);
   
    // inherriting the surrounding cell's attributes
    cellAttributes = new CellAttributes(getCommonAttributesHolder());
    cellAttributes.parseAttributes(this, getParentAttributes());
    
    if (isAttributeDefined(CommonAttributes.COLSPAN)) {
      colspan = getIntegerAttribute(CommonAttributes.COLSPAN, true, null);
    }
    if (isAttributeDefined(CommonAttributes.FILL_ROW)) {
      fillRow = getBooleanAttribute(CommonAttributes.FILL_ROW, true, null);
    }

    itemClosed = false;
    nestedCell = null;
    cell = null;
    phrase = null;
  }
  
  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while closing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }
    // adding an existing phrase before adding the element
    flushContentPhrase();
  }

  /**
   * Gets the parent attributes.
   * @return The parent attributes.
   */
  protected abstract CellAttributes getParentAttributes();
  
  /**
   * Builds a table cell.
   * @param embeddedImage The image to place in the cell.
   * @return The created cell
   * @exception DocumentHandlerException In case the cell could not be set up.
   */
  private PdfPCell buildCell(Image embeddedImage) throws DocumentHandlerException {
    // creating the cell    
    cell = new PdfPCell(embeddedImage);
    // setting up the cell
    setupCell(cell);
    return cell;
  }

  /**
   * Builds a table cell.
   * @param embeddedCell The cell to place in the cell.
   * @return The created cell
   * @exception DocumentHandlerException In case the cell could not be set up.
   */
  private PdfPCell buildCell(PdfPCell embeddedCell) throws DocumentHandlerException {
    // embedding the table into a table, otherwise iText will only replace the contents of
    // this cell with the contents of the embedded cell.
    PdfPTable nestedTable = new PdfPTable(1);
    nestedTable.setWidthPercentage(100);
    nestedTable.addCell(embeddedCell);
    composite = true;
    // creating the cell    
    cell = new PdfPCell(nestedTable);
    // setting up the cell
    setupCell(cell);
    return cell;
  }

  /**
   * Builds a table cell.
   * @param table The table to place in the cell.
   * @return The created cell
   * @exception DocumentHandlerException In case the cell could not be set up.
   */
  private PdfPCell buildCell(PdfPTable table) throws DocumentHandlerException {
    composite = true;
    // creating the cell    
    cell = new PdfPCell();
    // adding the table to the cell
    cell.addElement(table);
    // setting up the cell
    setupCell(cell);
    return cell;
  }

  /**
   * Builds a table cell.
   * @param embeddedParagraph The paragraph to place in the cell.
   * @return The created cell
   * @exception DocumentHandlerException In case the cell could not be set up.
   */
  private PdfPCell buildCell(Paragraph embeddedParagraph) throws DocumentHandlerException {
    composite = true;
    // creating the cell    
    cell = new PdfPCell();
    // adding an existing phrase before adding the element
    flushContentPhrase();
    cell.addElement(embeddedParagraph);
    // setting up the cell
    setupCell(cell);
    return cell;
  }

  /**
   * Builds a table cell.
   * @param embeddedPhrase The phrase to place in the cell.
   * @return The created cell
   * @exception DocumentHandlerException In case the cell could not be set up.
   */
  private PdfPCell buildCell(Phrase embeddedPhrase) throws DocumentHandlerException {
    // creating the cell    
    cell = new PdfPCell(embeddedPhrase);
    setupCell(cell);
    return cell;
  }

  /**
   * Builds a table cell.
   * @param element The element to place in the cell.
   * @return The created cell
   */
  private PdfPCell buildCell(Element element) throws DocumentHandlerException {
    composite = true;
    // creating the cell    
    cell = new PdfPCell();
    // adding an existing phrase before adding the element
    flushContentPhrase();
    cell.addElement(element);
    // setting up the cell
    setupCell(cell);
    return cell;
  }

  /**
   * Builds a table cell according to the tags contents and attributes.
   * @return The created cell
   * @exception DocumentHandlerException In case the cell could not be set up.
   */
  protected PdfPCell buildCell() throws DocumentHandlerException {
    if (nestedCell != null) {
      // creating the cell    
      cell = new PdfPCell(nestedCell);
    } else {
      cell = new PdfPCell();
    }
    // setting up the cell
    setupCell(cell);
    return cell;
  }

  /**
   * Sets up the given table cell according to the tags contents and attributes.
   * @param cell The cell to set up.
   * @exception DocumentHandlerException In case the cell could not be rotated.
   */
  private void setupCell(PdfPCell cell)
    throws DocumentHandlerException {

    cellAttributes.setupCell(cell);
    
    if (cell != null) {
      if (cellEventHandlers != null) {
        cell.setCellEvent(new CellEventHandler(cellEventHandlers));
      }
      
      // Added by Kiran: Adding rotate to a cell.
      Phrase phrase = cell.getPhrase();
      if (phrase != null && rotate != 0.0F) {
        cell.setPhrase(rotatePhrase(cell.getPhrase(), rotate));
      }
    }
  }

  /**
   * Adds a cell event handler.
   * @param eventHandler The cell event handler to add.
   */
  public void addCellEventHandler(PdfPCellEvent eventHandler) {
    if (cellEventHandlers == null) {
      cellEventHandlers = new ArrayList();
    }
    cellEventHandlers.add(eventHandler);
  }

  /**
   * Adds a element to the container.
   * @param item The document item.
   * @param element The element to be added.
   * @exception DocumentHandlerException If something went wrong when adding the element.
   */
  public void addElement(BaseDocumentTag item, Element element) throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    if (element == null) {
      return;
    }
    if (nestedCell != null) {
      throw new DocumentHandlerException(locator(), "Can't hold elements and cells within a cell.");
    }
    
    if (cell == null) {  
      // setting up the cell
      switch (element.type()) {
        case Element.PHRASE:
          if (phrase == null) {
            phrase = (Phrase) element;
          } else {
            phrase.add(element);
          }
          break;
          
        case Element.PARAGRAPH:
          cell = buildCell((Paragraph) element);
          break;
          
        case Element.IMGRAW:
        case Element.IMGTEMPLATE:
        case Element.JPEG:
          cell = buildCell((Image) element);
          break;
          
        case Element.LIST:
          composite = true;
          cell = buildCell(element);
          break;
          
        case Element.PTABLE:
          cell = buildCell((PdfPTable) element);
          break;

        default:
          if (element instanceof PdfPCell) {
            cell = buildCell((PdfPCell) element);
          } else if (element instanceof Phrase) {
            if (phrase == null) {
              phrase = (Phrase) element;
            } else {
              phrase.add(element);
            }
          } else {
            throw new DocumentHandlerException(locator(), "Cells musten't contain elements other than " +
                "tables, phrases, images, lists, paragraphs or cells.");
          }
      }
      return;
    }

    // adding the element to the iText PDF cell
    switch (element.type()) {
      case Element.PHRASE:
        if (phrase == null) {
          phrase = (Phrase) element;
        } else {
          phrase.add(element);
        }
        break;
        
      case Element.PARAGRAPH:
      case Element.IMGRAW:
      case Element.IMGTEMPLATE:
      case Element.JPEG:
      case Element.LIST:
      case Element.PTABLE:
        // adding an existing phrase before adding the element
        flushContentPhrase();
        // adding the element to the cell
        cell.addElement(element);
        break;
        
      default:
       if (element instanceof PdfPCell) {
         cell.addElement(element);
       } else if (element instanceof Phrase) {
         if (phrase == null) {
           phrase = (Phrase) element;
         } else {
           phrase.add(element);
         }
       } else {
         throw new DocumentHandlerException(locator(), "Cells musten't contain elements other than " +
             "tables, phrases, images, lists, paragraphs or cells.");
       }
    }
  }

  /**
   * Adds the content phrase element to the PDF cell, 
   * it has been created and is not empty.
   * @exception DocumentHandlerException In case the cell could not be built.
   */
  private void flushContentPhrase() throws DocumentHandlerException {
    if (phrase != null) {
      if (!isPhraseEmpty(phrase)) {
        if (cell == null) {
          cell = buildCell(phrase);
        } else {
          if (composite) {
            // we have to embed the phrase in a paragraph
            Paragraph p = new Paragraph(phrase);
            p.setAlignment(cellAttributes.getHAlign());
            cell.addElement(p);
          } else {
            // simply addint the phrase to the cells textual content
            cell.getPhrase().add(phrase);
          }
        }
      }
      phrase = null;
    }
  }
  
  /**
   * Gets the PDF cell.
   * @return The iText element this tag represents.
   */
  protected PdfPCell getCell() {
    return cell;
  }
  
  /**
   * Tells whether the element container is a top level container or not.
   * @return true if the element container is a top level container, else false.
   */
  public boolean isTopLevel() {
    return false;
  }
  
  /**
   * Adds a PDF cell to the container.
   * @param item The document item.
   * @param cell The PDF cell to be added.
   * @exception DocumentHandlerException If something went wrong when adding the cell.
   */
  public void addCell(BaseDocumentTag item, PdfPCell cell) throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    if (cell == null) {
      return;
    }
    if (nestedCell != null) {
      throw new DocumentHandlerException(locator(), "Can't hold more than one cell within a cell.");
    }
    if (this.cell == null) {
      nestedCell = cell;
      this.cell = buildCell(nestedCell);
      return;
    }
    throw new DocumentHandlerException(locator(), "Can't hold elements and cells within a cell.");
  }

  /**
   * Gets the number of remaining columns of the current row.
   * @return The number of remaining column to the end of the current row.
   */
  public int getRemainingPdfColumns() {
    return 1;
  }


  /**
   * Gets the PDF table holder.
   * @return The component that really holds a PDF table.
   */
  public PdfTableHolder getPdfTable() {
    return null;
  }
 
  /**
   * Gets the actual content.
   * This method is intended for testing purposes. 
   * @return The last content.
   */
  public String getActualContent() {
    if (phrase == null) {
      if (this.cell != null) {
        Phrase cellPhrase = this.cell.getPhrase();
        if (cellPhrase != null) {
          return getPhraseContent(cellPhrase);
        }
      }
      return super.getActualContent();
    }
    return getPhraseContent(phrase);
  }

  /**
   * Flushes the content buffer.
   * @exception DocumentHandlerException If something went wrong while flushing the content buffer.
   */
  public void flushContent() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    if (itemClosed) {
      return;
    }

    if (isEmptyContent()) {
      resetContent();
      return;
    }

    if (isTrimNextHead()) {
      trimContentHead();
      setTrimNextHead(false);
    }
    if (isItemClosed() && isTrimBody()) {
      trimContentTail();
    }

    if (phrase == null) {
      phrase = new Phrase(0.0F);
    }
    phrase = addChunk(phrase, getLeading(), getLineSpacing());
    resetContent();
  }

  /**
   * Adds the given chunk to the root phrase.
   * @param item The document item.
   * @param chunk The chunk to add to the root phrase.
   * @param font The font to use.
   * @exception DocumentHandlerException In case the document processing failed.
   */
  public void addChunk(BaseDocumentTag item, String chunk, DocumentFont font) 
    throws DocumentHandlerException {
    
    if (!isValid()) {
      return;
    }
    if (phrase == null) {
      phrase = new Phrase(0.0F);
      if (isTrimBody()) {
        chunk = StringUtils.trimHead(chunk);
      }
    }
    addChunk(item, phrase, chunk, font, getLeading(), getLineSpacing());
  }

  /**
   * Adds the given chunk to the root phrase.
   * @param item The document item.
   * @param chunk The chunk to add to the root phrase.
   * @exception DocumentHandlerException In case the document processing failed.
   */
  public void addChunk(BaseDocumentTag item, Chunk chunk)
    throws DocumentHandlerException {
    
    if (!isValid()) {
      return;
    }
    if (phrase == null) {
      phrase = new Phrase(0.0F);
    }
    addChunk(phrase, chunk, getLeading(), getLineSpacing());
  }

  /**
   * Gets the fixed vertical leading distance.
   * @return The current fixed vertical leading distance.
   */
  public float getLeading() {
    return cellAttributes.getLeading();
  }

  /**
   * Gets the line spacing.
   * @return The current line spacing.
   */
  public float getLineSpacing() {
    return cellAttributes.getLineSpacing();
  }

  /**
   * Gets the local cell attributes.
   * @return The local cell attributes.
   */
  public CellAttributes getLocalCellAttributes() {
    return cellAttributes;
  }

  /**
   * Gets the cell attributes.
   * @return The cell attributes, note that the attribute 'fixedHeigt' is neutralized here!
   */
  public CellAttributes getCellAttributes() {
    if (cellAttributes.getFixedHeight() <= 0.0F) {
      return cellAttributes;
    }
    CellAttributes safeCopy = (CellAttributes) cellAttributes.clone();
    safeCopy.setFixedHeight(-1.0F);
    return safeCopy;
  }

}
