/*
 * Copyright (C) 2003 - 2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import java.awt.Color;

import java.util.List;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.DocumentFont;
import org.ujac.print.ElementContainer;
import org.ujac.print.FontHolder;
import org.ujac.print.PhraseHolder;
import org.ujac.print.ChildDefinition;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.DocumentHandlerException;

import org.ujac.util.template.TemplateException;

import com.lowagie.text.Chunk;
import com.lowagie.text.Phrase;

/**
 * Name: BaseFontTag.java<br>
 * Description: Base class for font handling tags.<br>
 * 
 * @author lauerc
 */
public abstract class BaseFontTag extends BaseElementTag implements FontHolder, PhraseHolder {

  /** The surrounding phrase holder. */
  protected PhraseHolder phraseHolder = null;
  /** The surrounding font. */
  protected FontHolder parentFont = null;
  /** The font. */
  protected DocumentFont font = null;
  /** The font definition. */
  protected String fontDef = null;
  /** The font's color. */
  protected Color color = null;
  /** The font's family. */
  protected String family = DEFAULT_FONT_FAMILY;
  /** The font encoding. */
  protected String encoding = DEFAULT_FONT_ENCODING;
  /** The font's size. */
  protected float size = DEFAULT_FONT_SIZE;
  /** The font's style. */
  protected int style = DEFAULT_FONT_STYLE;
  /** The text raise amount. */
  protected float textRise = 0.0F;
  
  /**
   * Constructs a BaseFontTag instance with specific attributes.
   * @param name The tag name.
   */
  public BaseFontTag(String name) {
    super(name);
  }
  
  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.TRIM_BODY);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(IfTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ElseTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(SwitchTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(LogTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ForeachTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(PrintTableTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(TocTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(IndexTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(CellTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(PdfCellTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(TableTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(PdfTableTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(PdfTableHeaderTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(PhraseTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ParagraphTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(HeaderFooterTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BaseFontTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(MacroTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(NewLineTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(NewPageTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(SetPropertyTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(DefineTableTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(AddRowTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(AssertTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ImportTag.class, 0, ChildDefinition.UNLIMITED));
  }

  /**
   * Is the item virtual container, such as if, else or foreach.
   * @return true if the item is such a container, else false.
   */
  public boolean isVirtualContainer() {
    return true;
  }

  /**
   * Tells whether textual content is allowed for the tag or not.
   * @return true if the item is allowed to contain textual content, else false.
   */
  public boolean isTextBodyAllowed() {
    return true;
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // determining surrounding tags
    phraseHolder = documentHandler.latestPhraseHolder();
    parentFont = documentHandler.latestFontHolder();
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();
    if (parentFont != null) {
      DocumentFont font = parentFont.getFont();
      if (font != null) {
        family = font.getFamily();
        encoding = font.getEncoding();
        size = font.getSize();
        style = font.getStyle();
        textRise = font.getTextRise();
        color = font.getColor();
      }
    }
    // getting attributes
    setTrimBody(getBooleanAttribute(CommonAttributes.TRIM_BODY, false, true, null));
  }
  
  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while closing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }
    
    if (phraseHolder != null) {
      String content = getTrimmedContent();
      if (content.length() > 0) {
        try {
          Chunk chunk = new Chunk(documentHandler.executeTemplate(content), getFont().getFont());
          if (textRise != 0.0F) {
            chunk.setTextRise(textRise);
          }
          Color bgColor = getBackgroundColor();
          if (bgColor != null) {
            chunk.setBackground(bgColor);
          }
          phraseHolder.addChunk(this, chunk);
        } catch (TemplateException ex) {
          throw new DocumentHandlerException(locator(), "Content evaluation failed: " + ex.getMessage(), ex);
        }
      }
    }
  }

  /**
   * Creates the item's font according to the item's attributes.
   * @exception DocumentHandlerException In case the font could not be created.
   */
  protected void createFont() throws DocumentHandlerException {
    // creating font
    font = createFont(family, encoding, size, style, textRise, color);
  }

  /**
   * Gets the font.
   * @return The item's font.
   */
  public DocumentFont getFont() {
    return font;
  }

  /**
   * Flushes the content buffer.
   * @exception DocumentHandlerException In case the content processing failed.
   */
  public void flushContent() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    if (isEmptyContent()) {
      resetContent();
      return;
    }

    if (isTrimNextHead()) {
      trimContentHead();
      setTrimNextHead(false);
    }
    if (isItemClosed() && isTrimBody()) {
      trimContentTail();
    }
    
    if (phraseHolder != null) {
      Phrase contents = processContents(getLeading(), getLineSpacing());
      applyTextRiseAndAddToPhraseHolder(contents, (textRise != 0.0F));
    }
    resetContent();
  }

  /**
   * Recursively applies the given text rise attibute to the chunks,
   * nested inside the given phrase and adds the chunks to the current chunk holder.
   * @param phrase The phrase to process.
   * @param setTextRise The textRise flag.
   * @exception DocumentHandlerException In case the chunk couldn't be added to the phrase holder.
   */
  private void applyTextRiseAndAddToPhraseHolder(Phrase phrase, boolean setTextRise)
    throws DocumentHandlerException {
    
    if (phrase == null) {
      return;
    }
    
    int contentSize = phrase.size();
    for (int i = 0; i < contentSize; i++) {
      Object item = phrase.get(i);
      if (item instanceof Phrase) {
        applyTextRiseAndAddToPhraseHolder((Phrase) item, setTextRise);
        continue;
      }
      Chunk chunk = (Chunk) item;
      chunk.setTextRise(textRise);
      Color bgColor = getBackgroundColor();
      if (bgColor != null) {
        chunk.setBackground(bgColor);
      }
      phraseHolder.addChunk(this, chunk);
    }
  }
  
  /**
   * Adds the given chunk to the root phrase.
   * @param item The document item.
   * @param chunk The chunk to add to the root phrase.
   * @param font The font to use.
   * @exception DocumentHandlerException In case the document processing failed.
   */
  public void addChunk(BaseDocumentTag item, String chunk, DocumentFont font) 
    throws DocumentHandlerException {
    
    if (phraseHolder != null) {
      Phrase contents = addChunk(item, null, chunk, font, getLeading(), getLineSpacing());
      applyTextRiseAndAddToPhraseHolder(contents, (textRise != 0.0F));
    }
  }

  /**
   * Adds the given chunk to the root phrase.
   * @param item The document item.
   * @param chunk The chunk to add to the root phrase.
   * @exception DocumentHandlerException In case the document processing failed.
   */
  public void addChunk(BaseDocumentTag item, Chunk chunk)
    throws DocumentHandlerException {
    
    if (phraseHolder != null) {
      if (textRise != 0.0F) {
        chunk.setTextRise(textRise);
      }
      Color bgColor = getBackgroundColor();
      if (bgColor != null) {
        chunk.setBackground(bgColor);
      }
      phraseHolder.addChunk(this, chunk);
    }
  }

  /**
   * Gets the fixed vertical leading distance.
   * @return The current fixed vertical leading distance.
   */
  public float getLeading() {
    if (phraseHolder == null) {
      return DEFAULT_LEADING;
    }
    return phraseHolder.getLeading();
  }

  /**
   * Gets the line spacing.
   * @return The current line spacing.
   */
  public float getLineSpacing() {
    if (phraseHolder == null) {
      ElementContainer elementContainer = getElementContainer();
      if (elementContainer == null) {
        return DEFAULT_LINE_SPACING;
      }
      return elementContainer.getLineSpacing();
    }
    return phraseHolder.getLineSpacing();
  }

  /**
   * Sets the textRise for all chunks of the given phrase.
   * @param item The phrase to set the textRise for.
   */
  protected void setTextRise(Phrase item) {
    List chunks = item.getChunks();
    int numChunks = chunks.size();
    for (int i = 0; i < numChunks; i++) {
      Chunk chunk = (Chunk) chunks.get(i);
      chunk.setTextRise(this.textRise);
    }
  }
 
}
