/*
 * Copyright (C) 2005 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import java.io.File;
import java.io.IOException;

import java.util.ArrayList;
import java.util.List;

import org.ujac.print.DocumentHandlerException;
import org.ujac.print.TagAttributeException;

import org.ujac.util.exi.ExpressionException;

import org.ujac.util.io.FileUtils;

import com.lowagie.text.DocumentException;
import com.lowagie.text.Image;

/**
 * Name: BaseImageTag<br>
 * Description: A base class for image handling tags.
 * 
 * @author lauerc
 */
public abstract class BaseImageTag extends BaseElementTag {
  
  /** The builders for the supported image types. */
  protected static ImageBuilder[] IMAGE_BUILDERS = null; 
    
  static {
    List builders = new ArrayList();
    builders.add(new PdfImageBuilder());
    try {
      builders.add(Class.forName("org.ujac.print.tag.SvgImageBuilder").newInstance());
    } catch (Throwable ex) {
      // SVG image builder seems to be not available,
      // maybe batik is missing, see README-batik.txt ...
    }
    builders.add(new RasterImageBuilder());
    
    int numBuilders = builders.size();
    IMAGE_BUILDERS = new ImageBuilder[numBuilders];
    for (int i = 0; i < numBuilders; i++) {
      IMAGE_BUILDERS[i] = (ImageBuilder) builders.get(i);
    }
  }

  /** The image source attribute. */
  protected String sourceAttr = null;
  /** The image page attribute. */
  protected String pageAttr = null;

  /** The image width. */
  protected float width = 0;
  /** The image height. */
  protected float height = 0;

  /** The image data. */
  protected byte[] imageData = null;
  
  /**
   * Constructs a BaseImageTag instance with specific attributes.
   * @param name The tag name.
   */
  public BaseImageTag(String name) {
    super(name);
  }
  
  /**
   * Gets the image to insert into the document.
   * @return The created image.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public Image getImage() throws DocumentHandlerException {
    try {
      // evaluating the image source
      Object imageSource = null;
      try {
        imageSource = documentHandler.evalObject(sourceAttr);
      } catch (ExpressionException ex) {
        throw new TagAttributeException(locator(), "Attribute evaluation failed: " + ex.getMessage(), ex);
      }

      imageData = null;
      String path = null;
      if (imageSource instanceof String) {
        // it seems we've found a resource location
        String location = (String) imageSource;
        int lastSlash = location.lastIndexOf('/');
        if (lastSlash >= 0) {
          path = location.substring(0, lastSlash + 1);
        }
        Image img = documentHandler.getCachedImage(imageSource);
        if (img != null) {
          return img;
        }
        imageData = documentHandler.loadResource(location);
      } else if (imageSource instanceof File) {
        path = ((File) imageSource).getAbsolutePath();
        Image img = documentHandler.getCachedImage(imageSource);
        if (img != null) {
          return img;
        }
        // we've definitely found a file
        imageData = FileUtils.loadFile((File) imageSource);
      } else if (imageSource instanceof byte[]) {
        path = new String((byte[]) imageSource);
        Image img = documentHandler.getCachedImage(imageSource);
        if (img != null) {
          return img;
        }
        imageData = (byte[]) imageSource;
      } else if (imageSource instanceof java.awt.Image) {
        path = new String(((java.awt.Image) imageSource).toString());
        Image img = documentHandler.getCachedImage(imageSource);
        if (img != null) {
          return img;
        }
        img = Image.getInstance((java.awt.Image) imageSource, null);
        documentHandler.cacheImage(imageSource, img);
        return img;
      } else if (imageSource instanceof Image) {
        Image img = (Image) imageSource;
        Image cachedImg = documentHandler.getCachedImage("IMAGE_" + img.getMySerialId());
        if (cachedImg != null) {
          return cachedImg;
        }
        documentHandler.cacheImage(imageSource, img);
        return img;
      }
      if (imageData != null) {
        int page = 1;
        if (pageAttr != null) {
          try {
            page = Integer.parseInt(pageAttr);
          } catch (NumberFormatException ex) {
            throw new TagAttributeException(locator(), "Invalid value '" + pageAttr + "' for attribute 'page' at tag " + getName() + ".");
          }
        }
        for (int i = 0; i < IMAGE_BUILDERS.length; i++) {
          ImageBuilder builder = IMAGE_BUILDERS[i];
          if (builder.checkFormat(imageData)) {
            Image img = builder.buildImage(documentHandler, imageData, path, width, height, page);
            documentHandler.cacheImage(imageSource, img);
            return img;
          }
        }
      }
      throw new DocumentHandlerException(locator(), "No valid builder detected for image '" + sourceAttr + "'.");
    } catch (DocumentException ex) {
      throw new DocumentHandlerException(locator(), "Unable to build image: " + ex.getMessage(), ex);
    } catch (IOException ex) {
      throw new DocumentHandlerException(locator(), "Unable to load image: " + ex.getMessage(), ex);
    }
  }


}
