/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.ujac.print.DocumentEvent;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.DocumentEventRecorder;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.DocumentTagInterceptor;
import org.ujac.print.TagAttributeException;
import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.SequenceIndex;
import org.ujac.util.table.Table;
import org.ujac.util.template.TemplateContext;

/**
 * Name: BaseLoopTag<br>
 * Description: A base class for loop tags like the &lt;foreach&gt; and &lt;print-table&gt; tag.
 * 
 * @author lauerc
 */
public abstract class BaseLoopTag 
  extends BaseElementTag 
  implements DocumentEventRecorder {

  /** The iterator method's attribute declaration. */
  private static final Class[] iteratorAttributes = {};
  /** The iterator method's parameter list. */
  private static final Object[] iteratorParameters = {};
  
  /** Flag which rules, whether the nested items should be added to the document or not. */
  protected boolean insertItems = false;  
  /** Flag which tells, whether we're in the first iteration loop or not. */
  private boolean firstLoop = false;  
  /** The loop variable. */
  private String loopVariable = null;
  /** The loop index variable (loop variable + 'Idx'). */
  private String loopIdxVariable = null;
  /** The sequence expression. */
  private String sequenceExpression = null;
  /** The sequence holder. */
  protected Object sequenceHolder = null;
  /** The sequence iterator. */
  protected Iterator sequenceIterator = null;
  /** The document event list. */
  private List documentEvents = new ArrayList();
  /** The parent document event recorder. */
  private DocumentEventRecorder parentEventRecorder = null;
  /** The iteration index. */
  protected SequenceIndex iterationIdx = null;
  /** The current value. */
  private Object currentValue = null;
  
  /**
   * Constructs a BaseLoopTag instance with specific attributes.
   * @param name The tag name.
   */
  public BaseLoopTag(String name) {
    super(name);
  }
  
  /**
   * Gets the the sequence holder.
   * @return The sequence holder instance.
   */
  public Object getSequenceHolder() {
    return sequenceHolder;
  }
  /**
   * Sets the the sequence holder.
   * @param sequenceHolder The sequence holder instance to set.
   */
  protected void setSequenceHolder(Object sequenceHolder) {
    this.sequenceHolder = sequenceHolder;
  }

  /**
   * Gets the the current value.
   * @return The current value instance.
   */
  public Object getCurrentValue() {
    return currentValue;
  }
  /**
   * Sets the the current value.
   * @param currentValue The value instance to set.
   */
  public void setCurrentValue(Object currentValue) {
    this.currentValue = currentValue;
  }
  
  /**
   * Gets the the loop variable.
   * @return The loop variable name.
   */
  public String getLoopVariable() {
    return loopVariable;
  }
  /**
   * Sets the the loop variable.
   * @param loopVariable The loop variable name to set.
   */
  protected void setLoopVariable(String loopVariable) {
    this.loopVariable = loopVariable;
    this.loopIdxVariable = loopVariable + "Idx";
  }

  /**
   * Gets the the loop index variable.
   * @return The loop index variable name.
   */
  public String getLoopIdxVariable() {
    return loopIdxVariable;
  }
  
  /**
   * Tells whether or not the sequence iteration has more elements.
   * @return true if the sequence iterator has more elements, else false.
   */
  public boolean hasNext() {
    if (sequenceIterator == null) {
      return false;
    }
    return sequenceIterator.hasNext();
  }
  
  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    setLoopVariable(getStringAttribute(TagAttributes.ATTR_LOOP_VARIABLE, null, false, null));
    if (isAttributeDefined(TagAttributes.ATTR_SEQUENCE)) {
      this.sequenceExpression = getStringAttribute(TagAttributes.ATTR_SEQUENCE, null, false, null);
    }
    this.parentEventRecorder = documentHandler.topEventRecorder();
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the foreach item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    documentHandler.pushEventRecorder(this);

    if (!isValid()) {
      return;
    }
    
    if (sequenceExpression != null) {
      try {
        this.sequenceHolder = documentHandler.evalObject(sequenceExpression);
      } catch (ExpressionException ex) {
        throw new TagAttributeException(locator(), "Attribute evaluation failed: " + ex.getMessage(), ex);
      }
    }
    if (sequenceHolder == null) {
      insertItems = false;
      return;
    }

    if (sequenceHolder instanceof Collection) {
      sequenceIterator = ((Collection) sequenceHolder).iterator();
    } else if (sequenceHolder instanceof Iterator) {
      sequenceIterator = ((Iterator) sequenceHolder);
    } else if (sequenceHolder instanceof Map) {
      sequenceIterator = ((Map) sequenceHolder).keySet().iterator();
    } else if (sequenceHolder instanceof Table) {
      sequenceIterator = ((Table) sequenceHolder).iterator();
    } else if (sequenceHolder instanceof Object[]) {
      List sequenceList = Arrays.asList((Object[]) sequenceHolder);
      sequenceIterator = sequenceList.iterator();
    } else if (sequenceHolder instanceof String) {
      // sequence can be a string like 1-10 which means, we will iterate from 1 to 10 in steps of 1.
      StringTokenizer strTkn = new StringTokenizer(sequenceHolder.toString(), "-");
      if (strTkn.countTokens() == 2) {
        String strFrom = strTkn.nextToken();
        String strTo = strTkn.nextToken();
        
        int from = -1;
        try {
          from = Integer.parseInt(strFrom);
        } catch (NumberFormatException ex) {
          throw new TagAttributeException(locator(), "The attribute 'sequence' at tag '" + getName() + "' doesn't evaluate to a valid sequence.", ex);
        }

        int to = -1;
        try {
          to = Integer.parseInt(strTo);
        } catch (NumberFormatException ex) {
          throw new TagAttributeException(locator(), "The attribute 'sequence' at tag '" + getName() + "' doesn't evaluate to a valid sequence.", ex);
        }
        
        List sequence = new ArrayList();
        for (int i = from;  i <= to; i++) {
          sequence.add(new Integer(i));
        }
        sequenceIterator = sequence.iterator();
      } else {
        // sequence can be a string like 1,2,3,4,5 which means, we will iterate over the list of comma separated values.
        strTkn = new StringTokenizer(sequenceHolder.toString(), ",");
        if (strTkn.countTokens() < 2) {
          throw new TagAttributeException(locator(), "The attribute 'sequence' at tag '" + getName() + "' doesn't evaluate to a valid sequence.");
        }
        List sequence = new ArrayList();
        while (strTkn.hasMoreTokens()) {
          sequence.add(strTkn.nextToken());
        }
        sequenceIterator = sequence.iterator();
      }
    } else {
      try {
        Method iteratorMethod = sequenceHolder.getClass().getMethod("iterator", iteratorAttributes);
        sequenceIterator = (Iterator) iteratorMethod.invoke(sequenceHolder, iteratorParameters);
      } catch (SecurityException ex) {
        throw new TagAttributeException(locator(), "The attribute 'sequence' at tag '" + getName() + "' doesn't evaluate to a valid sequence.", ex);
      } catch (NoSuchMethodException ex) {
        throw new TagAttributeException(locator(), "The attribute 'sequence' at tag '" + getName() + "' doesn't evaluate to a valid sequence.", ex);
      } catch (ClassCastException ex) {
        throw new TagAttributeException(locator(), "The attribute 'sequence' at tag '" + getName() + "' doesn't evaluate to a valid sequence.", ex);
      } catch (IllegalArgumentException ex) {
        throw new TagAttributeException(locator(), "The attribute 'sequence' at tag '" + getName() + "' doesn't evaluate to a valid sequence.", ex);
      } catch (IllegalAccessException ex) {
        throw new TagAttributeException(locator(), "The attribute 'sequence' at tag '" + getName() + "' doesn't evaluate to a valid sequence.", ex);
      } catch (InvocationTargetException ex) {
        throw new TagAttributeException(locator(), "The attribute 'sequence' at tag '" + getName() + "' doesn't evaluate to a valid sequence.", ex);
      }
    }

    // resetting iteration index
    this.iterationIdx = null;

    insertItems = false;
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while closing the foreach item.
   */
  public void closeItem() throws DocumentHandlerException {
    documentHandler.popEventRecorder();

    if (!isValid()) {
      return;
    }

    boolean insertItemsOld = insertItems;
    insertItems = true;
    firstLoop = true;
    
    preLoopChildItems();
    
    if ((sequenceIterator != null) && sequenceIterator.hasNext()) {
      TemplateContext ctx = documentHandler.getTemplateContext();
      iterationIdx = new SequenceIndex(0);
      ctx.setProperty(loopIdxVariable, iterationIdx);
      while (sequenceIterator.hasNext()) {
        Object nextValue = sequenceIterator.next();
        ctx.setProperty(loopVariable, nextValue);
        this.currentValue = nextValue;
        if (!preRepeatChildItems()) {
          continue;
        }
        repeatChildItems();
        postRepeatChildItems();
        iterationIdx.increment();
      }
    }
    
    postLoopChildItems();
    
    insertItems = insertItemsOld;
  }
  
  /**
   * Method called, before the child items are printed.
   * @exception DocumentHandlerException Thrown in case something went wrong.
   */
  protected void preLoopChildItems() throws DocumentHandlerException {
  }
  
  /**
   * Method called, after the child items are printed.
   * @exception DocumentHandlerException Thrown in case something went wrong.
   */
  protected void postLoopChildItems() throws DocumentHandlerException {
  }

  /**
   * Checks whether the condition is true or not.
   * @return true, if the condition evaluates to true, else false.
   */
  public boolean isTrue() {
    return insertItems;
  }

  /**
   * Tells, whether this is the first iteration loop or not.
   * @return true if is the first iteration loop, else false.
   */
  public boolean isFirstLoop() {
    return firstLoop;
  }

  /**
   * Pre-processes the child items of the foreach item.
   * @return Tells, whether or not to repeat the child items, returning false skips the execution
   *   for the current item.
   * @exception DocumentHandlerException Thrown in case something went wrong.
   */
  protected boolean preRepeatChildItems() throws DocumentHandlerException {
    return true;
  }

  /**
   * Post-processes the child items of the foreach item.
   * @exception DocumentHandlerException Thrown in case something went wrong.
   */
  protected void postRepeatChildItems() throws DocumentHandlerException {
  }

  /**
   * Repeating the child items of the foreach item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening ot closing a child item.
   */
  protected void repeatChildItems() throws DocumentHandlerException {

    this.setItemClosed(false);
    DocumentTagInterceptor tagInterceptor = documentHandler.getTagInterceptor();
    
    int numEvents = documentEvents.size();
    for (int i = 0; i < numEvents; i++) {
      DocumentEvent event = (DocumentEvent) documentEvents.get(i);
      BaseDocumentTag item = event.getItem();
      if (i == numEvents -1) {
        this.setItemClosed(true);
      }
      switch (event.getType()) {
        case DocumentEvent.OPEN:
          // opening item
          BaseDocumentTag topItem = item.getParentItem();
          if (topItem != null) {
            topItem.flushContent();
          }
          documentHandler.pushItem(item);
          if ((tagInterceptor == null) || tagInterceptor.beforeOpen(item)) {
            // opening the item.
            item.openItem();
            if (tagInterceptor != null) {
              tagInterceptor.afterOpen(item);
            }
          }
          break;
        case DocumentEvent.CHARACTERS:
          String content = event.getContent();
          if (content.length() > 0) {
            item.setItemOpened(false);
            item.addContent(content);
            if (i == (numEvents - 1)) {
              // flushing content of last item
              item.flushContent();
            }
          }
          break;
        case DocumentEvent.CLOSE:
          // closing item
          item.setItemClosed(true);
          item.flushContent();
          if ((tagInterceptor == null) || tagInterceptor.beforeClose(item)) {
            item.closeItem();
            if (tagInterceptor != null) {
              tagInterceptor.afterClose(item);
            }
          }
          documentHandler.popItem();
          break;
          
        default:
          throw new DocumentHandlerException(locator(), "Encountered illegal event type '" + event.getType() + "!");
      }
    }
    firstLoop = false;
  }

  /**
   * @see org.ujac.print.DocumentEventRecorder#recordEvent(org.ujac.print.DocumentEvent)
   */
  public void recordEvent(DocumentEvent event) {
    if (event.getItem() != this) {
      documentEvents.add(event);
    } else if (event.getType() == DocumentEvent.CHARACTERS) {
      documentEvents.add(event);
      resetContent();
    } else if (parentEventRecorder != null) {
      parentEventRecorder.recordEvent(event);
    }
  }

}
