/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.DocumentPageEventHandler;
import org.ujac.print.ElementContainer;
import org.ujac.print.FontHolder;
import org.ujac.print.PagePosition;
import org.ujac.print.PhraseHolder;

import com.lowagie.text.Document;
import com.lowagie.text.DocumentException;
import com.lowagie.text.Image;
import com.lowagie.text.Rectangle;

import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfPCell;
import com.lowagie.text.pdf.PdfPTable;
import com.lowagie.text.pdf.PdfTemplate;
import com.lowagie.text.pdf.PdfWriter;

/**
 * Name: BoxTag<br>
 * Description: Places a text box at a fix position on the current page.
 * 
 * @author lauerc
 */
public class BoxTag extends BaseCellTag implements PhraseHolder, FontHolder, ElementContainer {

  /** The item's name. */
  public static final String TAG_NAME = "box";
      
  /** Definition of the 'rotate' attribute. */
  private static final AttributeDefinition BOX_ROTATE = 
    CommonAttributes.ROTATE.cloneAttrDef("The rotation angle of the box.");
  /** Definition of the 'rotate' style attribute. */
  private static final AttributeDefinition BOX_ROTATE_STYLE = 
    CommonStyleAttributes.ROTATE.cloneAttrDef("The rotation angle of the box.");

  /** The surrounding element container. */
  private ElementContainer elementContainer = null;
  
  /**
   * Name: BoxPageEventHandler<br>
   * Description: The page event handler, handling the output of boxes.
   */
  class BoxPageEventHandler extends DocumentPageEventHandler {
    
    /** The image to display. */
    private Image image = null;
    
    /**
     * Constructs a ImagePageEventHandler instance with specific attributes.
     * @param image The image to display.
     */
    public BoxPageEventHandler(Image image) {
      this.image = image;
    }
    
    /**
     * We override the onEndPage method.
     * @param writer The document writer.
     * @param document The iText document.
     */
    public void onEndPage(PdfWriter writer, Document document) {
      insertImage(writer, document, image);
    }
    
    /**
     * We override the onCloseDocument method.
     * @param writer The document writer.
     * @param document The iText document.
     */
    public void onCloseDocument(PdfWriter writer, Document document) {
    }

    /**
     * Tells whether this event handler is temporary, and should be removed after the 
     * first usage or not.
     * @return true, if this handler should be executed only once, else false.
     */
    public boolean isTemporary() {
      return false;
    }
  }

  /** The position of the box. */
  private PagePosition pos = null;
  /** The width of the box. */
  private float width;
  /** The height of the box. */
  private float height;
  /** The rotation angle. */
  private float angle;
  
  /** The surrounding &lt;sticky&gt; tag. */
  private StickyTag stickyTag = null;
  
  /**
   * Constructs a BoxTag instance with no specific attributes.
   */
  public BoxTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Places a text box at a fix position on the current page.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.BOX_X)
      .addDefinition(CommonAttributes.BOX_Y)
      .addDefinition(CommonAttributes.BOX_WIDTH)
      .addDefinition(CommonAttributes.BOX_HEIGHT)
      .addDefinition(BOX_ROTATE);
    }

  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.BOX_X)
      .addDefinition(CommonStyleAttributes.BOX_Y)
      .addDefinition(CommonStyleAttributes.BOX_WIDTH)
      .addDefinition(CommonAttributes.BOX_HEIGHT)
      .addDefinition(BOX_ROTATE_STYLE);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds();
  }
  
  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // determining the latest surrounding element container
    elementContainer = documentHandler.latestElementContainer();
    // determining the optional surrounding sticky tag
    stickyTag = (StickyTag) documentHandler.latestOfType(StickyTag.TAG_NAME);
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    pos = new PagePosition(
        getDimensionAttribute(CommonAttributes.BOX_X, 0.0F, true, CommonStyleAttributes.BOX_X), 
        getDimensionAttribute(CommonAttributes.BOX_Y, 0.0F, true, CommonStyleAttributes.BOX_Y)
    );
    this.width = getDimensionAttribute(CommonAttributes.BOX_WIDTH, true, CommonStyleAttributes.BOX_WIDTH);
    this.height = getDimensionAttribute(CommonAttributes.BOX_HEIGHT, true, CommonStyleAttributes.BOX_HEIGHT);
    this.angle = getFloatAttribute(CommonAttributes.ROTATE, 0.0F, true, CommonStyleAttributes.ROTATE);
  }

  /**
   * Gets the parent attributes.
   * @return The parent attributes.
   */
  protected CellAttributes getParentAttributes() {
    return null;
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while closing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }
    super.closeItem();
    
    PdfWriter writer = (PdfWriter) documentHandler.getDocumentWriter();
    PdfContentByte cb = writer.getDirectContent();
    cb.saveState();
    PdfTemplate template = cb.createTemplate(width, height);

    PdfPTable table = new PdfPTable(1);
    table.setWidthPercentage(100.0F);
    table.setTotalWidth(width);

    PdfPCell cell = getCell();
    if (cell == null) {
      // forcing creation of the table cell
      cell = buildCell();
    }
    //float totalHeight = height + cell.getPaddingBottom() + cell.getPaddingTop();
    //cell.setFixedHeight(totalHeight);
    cell.setFixedHeight(height);
    table.addCell(cell);
    table.writeSelectedRows(0, 1, 0, height/* + cell.getPaddingTop() + cell.getPaddingBottom()*/, template);
    cb.restoreState(); 

    // creating image from template
    Image img = null;
    try {
      img = Image.getInstance(template);
      if (angle != 0.0F) {
        img.setRotation((float) Math.PI * angle / 180);
      }
    } catch (DocumentException ex) {
      throw new DocumentHandlerException(locator(), "Unable to write box: " + ex.getMessage(), ex);
    }
    
    if (stickyTag != null) {
      // adding page event handler, which inserts the PDF image at the end of the document.
      stickyTag.addStickyElementHandler(new BoxPageEventHandler(img));
    } else {
      insertImage((PdfWriter) documentHandler.getDocumentWriter(), documentHandler.getDocument(), img);
    }
  }

  /**
   * Inserts the given image into the document.
   * @param writer The pdf writer.
   * @param document The document to add the image to.
   * @param image The image to insert.
   */
  protected void insertImage(PdfWriter writer, Document document, Image image) {
    try {
      // calculating the correct image position
      Rectangle pageSize = document.getPageSize();
      float x = pos.calcX(pageSize.getRight(), image.getScaledWidth());
      float y = pos.calcY(pageSize.getTop(), image.getScaledHeight());
      // placing the image absolutely into the page
      image.setAbsolutePosition(x, y);
      PdfContentByte cb = writer.getDirectContent();
      cb.addImage(image);      
    } catch (DocumentException ex) {
      ex.printStackTrace();
      throw new RuntimeException(ex.getMessage());
    }
  }

  /**
   * Gets the line spacing.
   * @return The current line spacing.
   */
  public float getLineSpacing() {
    ElementContainer elementContainer = getElementContainer();
    return elementContainer.getLineSpacing();
  }

  /**
   * Gets the element container.
   * In case no element container has been set so far, it 
   * obtains the root element container from the DocumentHandler.
   * @return The current element container.
   */
  public ElementContainer getElementContainer() {
    if (elementContainer == null) {
      elementContainer = documentHandler.latestElementContainer();
    }
    return elementContainer;
  }

}
