/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import java.awt.Color;

import org.ujac.print.AttributeHandler;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.CommonAttributesHolder;
import org.ujac.print.TagAttributeException;

import org.ujac.util.BeanUtils;

import com.lowagie.text.Element;
import com.lowagie.text.pdf.PdfPCell;

/**
 * Title:       CellAttributeHolder<br>
 * Description: Holds cell attributes.<br>
 * 
 * @author lauerc
 */
public class CellAttributes implements Cloneable {
  
  /** The default padding. */
  public static final float DEFAULT_PADDING = 2.0F;
  /** The default horizontal alignment for cell contents. */
  public static final int DEFAULT_HALIGN = Element.ALIGN_LEFT; 
  /** The default vertical alignment for cell contents. */
  public static final int DEFAULT_VALIGN = Element.ALIGN_BOTTOM; 

  /** The fixed leading amount for contents of embedded cells. */
  private float leading = BaseDocumentTag.DEFAULT_LEADING;
  /** The line spacing factor for contents of embedded cells. */
  private float lineSpacing = BaseDocumentTag.DEFAULT_LINE_SPACING;
  /** The horizontal alignment. */ 
  private int hAlign = -1;
  /** The vertical alignment. */ 
  private int vAlign = -1;
  /** The default cell background color. */ 
  protected Color bgColor = null;
  /** The style of the cell's border. */ 
  protected int borderStyle = 0;
  
  /** The color of the cell's border. */ 
  protected Color borderColor = null;
  /** The width of the cell's border. */
  protected float borderWidth = 0;

  /** The color of the cell's left border. */ 
  protected Color borderLeftColor = null;
  /** The width of the cell's left border. */
  protected float borderLeftWidth = 0;
  /** The color of the cell's left border. */ 
  protected Color borderRightColor = null;
  /** The width of the cell's left border. */
  protected float borderRightWidth = 0;
  /** The color of the cell's top border. */ 
  protected Color borderTopColor = null;
  /** The width of the cell's top border. */
  protected float borderTopWidth = 0;
  /** The color of the cell's bottom border. */ 
  protected Color borderBottomColor = null;
  /** The width of the cell's bottom border. */
  protected float borderBottomWidth = 0;

  /** The overall padding. */ 
  protected float padding = DEFAULT_PADDING;
  /** The left padding. */ 
  protected float paddingLeft = DEFAULT_PADDING;
  /** The right padding. */ 
  protected float paddingRight = DEFAULT_PADDING;
  /** The top padding. */ 
  protected float paddingTop = DEFAULT_PADDING;
  /** The bottom padding. */ 
  protected float paddingBottom = DEFAULT_PADDING;
  /** The fixed height size. */ 
  protected float fixedHeight = -1.0F;
  /** The no-wrap flag. */ 
  protected boolean noWrap = false;

  /** The colspan amount. */ 
  protected int colspan = 1;
  /** The fill-row flag. */ 
  protected boolean fillRow = false;

  /**
   * Constructs a CellAttributes instance with specific attributes.
   * @param commonAttributesHolder The initial common attributes holder.
   */
  public CellAttributes(CommonAttributesHolder commonAttributesHolder) {
    if (commonAttributesHolder != null) {
      this.lineSpacing = commonAttributesHolder.getLineSpacing();
      this.leading = commonAttributesHolder.getLeading();
    }
  }
 
  /**
   * Reads the supported attributes from the given tag and copies the values into the 
   * according member variables.
   * @param tag The tag to read the attributes from.
   * @param parentAttributes The parent attributes to inherit.
   * @exception TagAttributeException In case the attribute handling failed.
   */
  public void parseAttributes(BaseDocumentTag tag, CellAttributes parentAttributes)
    throws TagAttributeException {
    
    parseCommonAttributes(tag, parentAttributes);
  }

  /**
   * Does the same as <code>parseAttributes(BaseDocumentTag)</code>, but processes the
   * attributes 'cell-halign' and 'cell-valign' instead of 'halign' and 'valign'. 
   * @param tag The tag to read the attributes from.
   * @param parentAttributes The parent attributes to inherit.
   * @exception TagAttributeException In case the attribute handling failed.
   */
  public void parseTableAttributes(BaseDocumentTag tag, CellAttributes parentAttributes)
    throws TagAttributeException {
    
    parseCommonAttributes(tag, parentAttributes);
    
    AttributeHandler attrHandler = tag.getAttributeHandler();
    Object attrVal = tag.getAttribute(
        CommonAttributes.CELL_HORIZONTAL_ALIGNMENT, null, true, 
        CommonAttributes.CELL_HORIZONTAL_ALIGNMENT, attrHandler.getHorizontalAlignmentHandler()
    );
    if (attrVal != null) {
      hAlign = ((Number) attrVal).intValue();
    } else {
      // respecting the horizontal alignment of the table itself may lead to weired layout
      hAlign = -1;
    }

    attrVal = tag.getAttribute(
        CommonAttributes.CELL_VERTICAL_ALIGNMENT, null, true, 
        CommonAttributes.CELL_VERTICAL_ALIGNMENT, attrHandler.getVerticalAlignmentHandler()
    );
    if (attrVal != null) {
      vAlign = ((Number) attrVal).intValue();
    } else {
      // respecting the vertical alignment of the table itself may lead to weired layout
      vAlign = -1;
    }
  }

  /**
   * Does the same as <code>parseAttributes(BaseDocumentTag)</code>, but inherits the
   * the <code>fixedHeight</code attribute. 
   * @param tag The tag to read the attributes from.
   * @param parentAttributes The parent attributes to inherit.
   * @exception TagAttributeException In case the attribute handling failed.
   */
  public void parseRowAttributes(BaseDocumentTag tag, CellAttributes parentAttributes)
    throws TagAttributeException {
    
    parseCommonAttributes(tag, parentAttributes);
  }

  /**
   * Reads the supported attributes from the given tag and copies the values into the 
   * according member variables.
   * @param tag The tag to read the attributes from.
   * @param parentAttributes The parent attributes to inherit.
   * @exception TagAttributeException In case the attribute handling failed.
   */
  private void parseCommonAttributes(BaseDocumentTag tag, CellAttributes parentAttributes)
    throws TagAttributeException {
    
    if (parentAttributes != null) {
      leading = parentAttributes.getLeading();
      lineSpacing = parentAttributes.getLineSpacing();
      bgColor = parentAttributes.getBgColor();
      borderStyle = parentAttributes.getBorderStyle();
      borderWidth = parentAttributes.getBorderWidth();
      borderColor = parentAttributes.getBorderColor();
      borderLeftWidth = parentAttributes.getBorderLeftWidth();
      borderLeftColor = parentAttributes.getBorderLeftColor();
      borderRightWidth = parentAttributes.getBorderRightWidth();
      borderRightColor = parentAttributes.getBorderRightColor();
      borderTopWidth = parentAttributes.getBorderTopWidth();
      borderTopColor = parentAttributes.getBorderTopColor();
      borderBottomWidth = parentAttributes.getBorderBottomWidth();
      borderBottomColor = parentAttributes.getBorderBottomColor();
      fixedHeight = parentAttributes.getFixedHeight();
      noWrap = parentAttributes.isNoWrap();
    }

    AttributeHandler attrHandler = tag.getAttributeHandler();
    Object attrVal = tag.getAttribute(
      CommonAttributes.LEADING, null, true, 
      CommonStyleAttributes.LEADING, attrHandler.getDimensionHandler()
    );
    if (attrVal != null) {
      leading = ((Number) attrVal).floatValue();
    }

    attrVal = tag.getAttribute(
        CommonAttributes.LINE_SPACING, null, true, 
        CommonStyleAttributes.LINE_SPACING, attrHandler.getDimensionHandler()
    );
    if (attrVal != null) {
      lineSpacing = ((Number) attrVal).floatValue();
    }
    
    attrVal = tag.getAttribute(
        CommonAttributes.BACKGROUND_COLOR, null, true, 
        CommonStyleAttributes.BACKGROUND_COLOR, attrHandler.getColorHandler()
    );
    if (attrVal != null) {
      bgColor = (Color) attrVal;
    }

    
    attrVal = tag.getAttribute(
        CommonAttributes.BORDER_STYLE, null, true, 
        CommonStyleAttributes.BORDER_STYLE, attrHandler.getBorderHandler()
    );
    if (attrVal != null) {
      borderStyle = ((Number) attrVal).intValue();
    }

    attrVal = tag.getAttribute(
        CommonAttributes.BORDER_WIDTH, null, true, 
        CommonStyleAttributes.BORDER_WIDTH, attrHandler.getDimensionHandler()
    );
    if (attrVal != null) {
      borderWidth = ((Number) attrVal).floatValue();
      borderLeftWidth = borderWidth;
      borderRightWidth = borderWidth;
      borderTopWidth = borderWidth;
      borderBottomWidth = borderWidth;
    }

    attrVal = tag.getAttribute(
        CommonAttributes.BORDER_COLOR, null, true, 
        CommonStyleAttributes.BORDER_COLOR, attrHandler.getColorHandler()
    );
    if (attrVal != null) {
      borderColor = (Color) attrVal;
      borderLeftColor = borderColor;
      borderRightColor = borderColor;
      borderTopColor = borderColor;
      borderBottomColor = borderColor;
    }

    attrVal = tag.getAttribute(
        null, null, true, 
        CommonStyleAttributes.BORDER_LEFT_WIDTH, attrHandler.getDimensionHandler()
    );
    if (attrVal != null) {
      borderLeftWidth = ((Number) attrVal).floatValue();
    }
    attrVal = tag.getAttribute(
        null, null, true, 
        CommonStyleAttributes.BORDER_RIGHT_WIDTH, attrHandler.getDimensionHandler()
    );
    if (attrVal != null) {
      borderRightWidth = ((Number) attrVal).floatValue();
    }
    attrVal = tag.getAttribute(
        null, null, true, 
        CommonStyleAttributes.BORDER_TOP_WIDTH, attrHandler.getDimensionHandler()
    );
    if (attrVal != null) {
      borderTopWidth = ((Number) attrVal).floatValue();
    }
    attrVal = tag.getAttribute(
        null, null, true, 
        CommonStyleAttributes.BORDER_BOTTOM_WIDTH, attrHandler.getDimensionHandler()
    );
    if (attrVal != null) {
      borderBottomWidth = ((Number) attrVal).floatValue();
    }

    attrVal = tag.getAttribute(
        null, null, true, 
        CommonStyleAttributes.BORDER_LEFT_COLOR, attrHandler.getColorHandler()
    );
    if (attrVal != null) {
      borderLeftColor = (Color) attrVal;
    }
    attrVal = tag.getAttribute(
        null, null, true, 
        CommonStyleAttributes.BORDER_RIGHT_COLOR, attrHandler.getColorHandler()
    );
    if (attrVal != null) {
      borderRightColor = (Color) attrVal;
    }
    attrVal = tag.getAttribute(
        null, null, true, 
        CommonStyleAttributes.BORDER_TOP_COLOR, attrHandler.getColorHandler()
    );
    if (attrVal != null) {
      borderTopColor = (Color) attrVal;
    }
    attrVal = tag.getAttribute(
        null, null, true, 
        CommonStyleAttributes.BORDER_BOTTOM_COLOR, attrHandler.getColorHandler()
    );
    if (attrVal != null) {
      borderBottomColor = (Color) attrVal;
    }

    // initializing padding
    padding = DEFAULT_PADDING;
    paddingLeft = DEFAULT_PADDING;
    paddingRight = DEFAULT_PADDING;
    paddingTop = DEFAULT_PADDING;
    paddingBottom = DEFAULT_PADDING;
    
    if (parentAttributes != null) {
      padding = parentAttributes.getPadding();
      paddingLeft = parentAttributes.getPaddingLeft();
      paddingRight = parentAttributes.getPaddingRight();
      paddingTop = parentAttributes.getPaddingTop();
      paddingBottom = parentAttributes.getPaddingBottom();
    }
    
    attrVal = tag.getAttribute(
        CommonAttributes.PADDING, null, true, 
        CommonAttributes.PADDING, attrHandler.getDimensionHandler()
    );
    if (attrVal != null) {
      padding = ((Number) attrVal).floatValue();
      paddingLeft = padding;
      paddingRight = padding;
      paddingTop = padding;
      paddingBottom = padding;
    }
    
    attrVal = tag.getAttribute(
        CommonAttributes.PADDING_LEFT, null, true, 
        CommonStyleAttributes.PADDING_LEFT, attrHandler.getDimensionHandler()
    );
    if (attrVal != null) {
      paddingLeft = ((Number) attrVal).floatValue();
    }
    
    attrVal = tag.getAttribute(
        CommonAttributes.PADDING_RIGHT, null, true, 
        CommonStyleAttributes.PADDING_RIGHT, attrHandler.getDimensionHandler()
    );
    if (attrVal != null) {
      paddingRight = ((Number) attrVal).floatValue();
    }
    
    attrVal = tag.getAttribute(
        CommonAttributes.PADDING_TOP, null, true, 
        CommonStyleAttributes.PADDING_TOP, attrHandler.getDimensionHandler()
    );
    if (attrVal != null) {
      paddingTop = ((Number) attrVal).floatValue();
    }

    attrVal = tag.getAttribute(
        CommonAttributes.PADDING_BOTTOM, null, true, 
        CommonStyleAttributes.PADDING_BOTTOM, attrHandler.getDimensionHandler()
    );
    if (attrVal != null) {
      paddingBottom = ((Number) attrVal).floatValue();
    }
    
    if (tag.isAttributeSupported(CommonAttributes.NO_WRAP)) {
      attrVal = tag.getAttribute(
          CommonAttributes.NO_WRAP, null, true, 
          CommonStyleAttributes.NO_WRAP, attrHandler.getBooleanHandler()
      );
      if (attrVal != null) {
        noWrap = ((Boolean) attrVal).booleanValue();
      }
    }

    if (tag.isAttributeSupported(CommonAttributes.CELL_HEIGHT)) {
      attrVal = tag.getAttribute(
          CommonAttributes.CELL_HEIGHT, null, true, 
          CommonStyleAttributes.CELL_HEIGHT, attrHandler.getDimensionHandler()
      );
      if (attrVal != null) {
        fixedHeight = ((Number) attrVal).floatValue();
      }
    }
    
    if (tag.isAttributeSupported(CommonAttributes.COLSPAN)) {
      attrVal = tag.getAttribute(
          CommonAttributes.COLSPAN, null, true, 
          null, attrHandler.getIntegerHandler()
      );
      if (attrVal != null) {
        colspan = ((Number) attrVal).intValue();
      }
    }
    
    if (tag.isAttributeSupported(CommonAttributes.FILL_ROW)) {
      attrVal = tag.getAttribute(
          CommonAttributes.FILL_ROW, null, true, 
          null, attrHandler.getBooleanHandler()
      );
      if (attrVal != null) {
        fillRow = ((Boolean) attrVal).booleanValue();
      }
    }
    
    attrVal = null;
    if (tag.isAttributeSupported(CommonAttributes.CELL_HORIZONTAL_ALIGNMENT)) {
      attrVal = tag.getAttribute(
          CommonAttributes.CELL_HORIZONTAL_ALIGNMENT, null, true, 
          CommonStyleAttributes.CELL_HORIZONTAL_ALIGNMENT, attrHandler.getHorizontalAlignmentHandler()
      );
    }
    if ((attrVal == null) && tag.isAttributeSupported(CommonAttributes.HORIZONTAL_ALIGNMENT)) {
      attrVal = tag.getAttribute(
          CommonAttributes.HORIZONTAL_ALIGNMENT, null, true, 
          CommonStyleAttributes.HORIZONTAL_ALIGNMENT, attrHandler.getHorizontalAlignmentHandler()
      );
    }
    if (attrVal != null) {
      hAlign = ((Number) attrVal).intValue();
    } else if (parentAttributes != null) {
      hAlign = parentAttributes.getHAlign();
    }

    attrVal = null;
    if (tag.isAttributeSupported(CommonAttributes.CELL_VERTICAL_ALIGNMENT)) {
      attrVal = tag.getAttribute(
          CommonAttributes.CELL_VERTICAL_ALIGNMENT, null, true, 
          CommonStyleAttributes.CELL_VERTICAL_ALIGNMENT, attrHandler.getVerticalAlignmentHandler()
      );
    }
    if ((attrVal == null) && tag.isAttributeSupported(CommonAttributes.VERTICAL_ALIGNMENT)) {
      attrVal = tag.getAttribute(
          CommonAttributes.VERTICAL_ALIGNMENT, null, true, 
          CommonStyleAttributes.VERTICAL_ALIGNMENT, attrHandler.getVerticalAlignmentHandler()
      );
    }
    if (attrVal != null) {
      vAlign = ((Number) attrVal).intValue();
    } else if (parentAttributes != null) {
      vAlign = parentAttributes.getVAlign();
    }
  }
  
  /**
   * Merges this attribute sett with the given cell attributes.
   * @param attr The attributes to merge.
   */
  public void merge(CellAttributes attr) {
    float floatVal = attr.getLeading();
    if (floatVal >= 0.0F) {
      this.leading = floatVal;
    }
    floatVal = attr.getLineSpacing();
    if (floatVal >= 0.0F) {
      this.lineSpacing = floatVal;
    }
    Color colorVal = attr.getBgColor();
    if (colorVal != null) {
      this.bgColor = colorVal;
    }
    
    floatVal = attr.getBorderWidth();
    if (floatVal >= 0.0F) {
      borderWidth = floatVal;
      borderLeftWidth = borderWidth;
      borderRightWidth = borderWidth;
      borderTopWidth = borderWidth;
      borderBottomWidth = borderWidth;
    }

    int intVal = attr.getBorderStyle();
    if (intVal >= 0) {
      borderStyle = intVal;
    }

    colorVal = attr.getBorderColor();
    if (colorVal != null) {
      borderColor = colorVal;
      borderLeftColor = borderColor;
      borderRightColor = borderColor;
      borderTopColor = borderColor;
      borderBottomColor = borderColor;
    }

    floatVal = attr.getBorderLeftWidth();
    if (floatVal >= 0.0F) {
      borderLeftWidth = floatVal;
    }
    floatVal = attr.getBorderRightWidth();
    if (floatVal >= 0.0F) {
      borderRightWidth = floatVal;
    }
    floatVal = attr.getBorderTopWidth();
    if (floatVal >= 0.0F) {
      borderTopWidth = floatVal;
    }
    floatVal = attr.getBorderBottomWidth();
    if (floatVal >= 0.0F) {
      borderBottomWidth = floatVal;
    }

    colorVal = attr.getBorderLeftColor();
    if (colorVal != null) {
      borderLeftColor = colorVal;
    }
    colorVal = attr.getBorderRightColor();
    if (colorVal != null) {
      borderRightColor = colorVal;
    }
    colorVal = attr.getBorderTopColor();
    if (colorVal != null) {
      borderTopColor = colorVal;
    }
    colorVal = attr.getBorderBottomColor();
    if (colorVal != null) {
      borderBottomColor = colorVal;
    }

    floatVal = attr.getPadding();
    if (floatVal >= 0.0F) {
      padding = floatVal;
      paddingLeft = padding;
      paddingRight = padding;
      paddingTop = padding;
      paddingBottom = padding;
    }
    
    floatVal = attr.getPaddingLeft();
    if (floatVal >= 0.0F) {
      paddingLeft = floatVal;
    }
    floatVal = attr.getPaddingRight();
    if (floatVal >= 0.0F) {
      paddingRight = floatVal;
    }
    floatVal = attr.getPaddingTop();
    if (floatVal >= 0.0F) {
      paddingTop = floatVal;
    }
    floatVal = attr.getPaddingBottom();
    if (floatVal >= 0.0F) {
      paddingBottom = floatVal;
    }

    boolean boolValue = attr.isNoWrap();
    noWrap = boolValue;
    boolValue = attr.isFillRow();
    fillRow = boolValue;

    floatVal = attr.getFixedHeight();
    if (floatVal >= 0.0F) {
      fixedHeight = floatVal;
    }
    intVal = attr.getColspan();
    if (intVal >= 0) {
      colspan = intVal;
    }

    intVal = attr.getHAlign();
    if (intVal >= 0) {
      hAlign = intVal;
    }
    intVal = attr.getVAlign();
    if (intVal >= 0) {
      vAlign = intVal;
    }
  }
  
  /**
   * Sets up the given table cell according to the tags contents and attributes.
   * @param cell The cell to set up.
   */
  public void setupCell(PdfPCell cell) {

    float leading = this.getLeading();
    float lineSpacing = this.getLineSpacing();
    cell.setLeading(leading, lineSpacing);
    
    if (cell != null) {
      // configuring the cell
      Color bgColor = this.getBgColor();
      if (bgColor != null) {
        cell.setBackgroundColor(bgColor);
      }
      float borderWidth = this.getBorderWidth();
      if (borderWidth > 0.0f) {
        cell.setBorderWidth(borderWidth);
      }
      float borderLeftWidth = this.getBorderLeftWidth();
      if (borderLeftWidth != borderWidth) {
        cell.setBorderWidthLeft(borderLeftWidth);
        cell.setUseVariableBorders(true);
      }
      float borderRightWidth = this.getBorderRightWidth();
      if (borderRightWidth != borderWidth) {
        cell.setBorderWidthRight(borderRightWidth);
        cell.setUseVariableBorders(true);
      }
      float borderTopWidth = this.getBorderTopWidth();
      if (borderTopWidth != borderWidth) {
        cell.setBorderWidthTop(borderTopWidth);
        cell.setUseVariableBorders(true);
      }
      float borderBottomWidth = this.getBorderBottomWidth();
      if (borderBottomWidth != borderWidth) {
        cell.setBorderWidthBottom(borderBottomWidth);
        cell.setUseVariableBorders(true);
      }

      if (colspan > 0) {
        cell.setColspan(colspan);
      }
      cell.setBorder(this.getBorderStyle());
      Color borderColor = this.getBorderColor();
      if (borderColor != null) {
        cell.setBorderColor(borderColor);
      }
      Color borderLeftColor = this.getBorderLeftColor();
      if (!BeanUtils.equals(borderLeftColor, borderColor)) {
        cell.setBorderColorLeft(borderLeftColor);
        cell.setUseVariableBorders(true);
      }
      Color borderRightColor = this.getBorderRightColor();
      if (!BeanUtils.equals(borderRightColor, borderColor)) {
        cell.setBorderColorRight(borderRightColor);
        cell.setUseVariableBorders(true);
      }
      Color borderTopColor = this.getBorderTopColor();
      if (!BeanUtils.equals(borderTopColor, borderColor)) {
        cell.setBorderColorTop(borderTopColor);
        cell.setUseVariableBorders(true);
      }
      Color borderBottomColor = this.getBorderBottomColor();
      if (!BeanUtils.equals(borderBottomColor, borderColor)) {
        cell.setBorderColorBottom(borderBottomColor);
        cell.setUseVariableBorders(true);
      }
      
      int hAlign = this.getHAlign();
      if (hAlign < 0) {
        hAlign = DEFAULT_HALIGN;
      }
      cell.setHorizontalAlignment(hAlign);
      int vAlign = this.getVAlign();
      if (vAlign < 0) {
        vAlign = DEFAULT_VALIGN;
      }
      cell.setVerticalAlignment(vAlign);
      cell.setPaddingLeft(this.getPaddingLeft());
      cell.setPaddingRight(this.getPaddingRight());
      cell.setPaddingTop(this.getPaddingTop());
      cell.setPaddingBottom(this.getPaddingBottom());
      cell.setNoWrap(this.isNoWrap());
      
      float fixedHeigth = this.getFixedHeight();
      if (fixedHeigth > 0.0F) {
        cell.setFixedHeight(fixedHeigth);
      }
    }
  }

  
  /**
   * Clones the attribute holder instance.
   * @return The cloned attribute holder. 
   */
  public Object clone() {
    CellAttributes clone = null;
    try {
      clone = (CellAttributes) super.clone();
    } catch (CloneNotSupportedException ex) {
      throw new RuntimeException("Failed to clone CellAttributes instance: " + ex.getMessage(), ex);
    }
    return clone;
  }


  /**
   * Getter method for the the property leading.
   * @return The current value of property leading.
   */
  public float getLeading() {
    return leading;
  }
  /**
   * Setter method for the the property leading.
   * @param leading The value to set for the property leading.
   */
  public void setLeading(float leading) {
    this.leading = leading;
  }

  /**
   * Getter method for the the property lineSpacing.
   * @return The current value of property lineSpacing.
   */
  public float getLineSpacing() {
    return lineSpacing;
  }
  /**
   * Setter method for the the property lineSpacing.
   * @param lineSpacing The value to set for the property lineSpacing.
   */
  public void setLineSpacing(float lineSpacing) {
    this.lineSpacing = lineSpacing;
  }

  /**
   * Getter method for the the property hAlign.
   * @return The current value of property hAlign.
   */
  public int getHAlign() {
    return hAlign;
  }
  /**
   * Setter method for the the property hAlign.
   * @param hAlign The value to set for the property hAlign.
   */
  public void setHAlign(int hAlign) {
    this.hAlign = hAlign;
  }

  /**
   * Getter method for the the property vAlign.
   * @return The current value of property vAlign.
   */
  public int getVAlign() {
    return vAlign;
  }
  /**
   * Setter method for the the property vAlign.
   * @param vAlign The value to set for the property vAlign.
   */
  public void setVAlign(int vAlign) {
    this.vAlign = vAlign;
  }
  
  
  /**
   * Getter method for the the property bgColor.
   * @return The current value of property bgColor.
   */
  public Color getBgColor() {
    return bgColor;
  }
  /**
   * Setter method for the the property bgColor.
   * @param bgColor The value to set for the property bgColor.
   */
  public void setBgColor(Color bgColor) {
    this.bgColor = bgColor;
  }
  
  /**
   * Getter method for the the property borderColor.
   * @return The current value of property borderColor.
   */
  public Color getBorderColor() {
    return borderColor;
  }
  /**
   * Setter method for the the property borderColor.
   * @param borderColor The value to set for the property borderColor.
   */
  public void setBorderColor(Color borderColor) {
    this.borderColor = borderColor;
  }
  
  /**
   * Getter method for the the property borderStyle.
   * @return The current value of property borderStyle.
   */
  public int getBorderStyle() {
    return borderStyle;
  }
  /**
   * Setter method for the the property borderStyle.
   * @param borderStyle The value to set for the property borderStyle.
   */
  public void setBorderStyle(int borderStyle) {
    this.borderStyle = borderStyle;
  }
  
  /**
   * Getter method for the the property borderWidth.
   * @return The current value of property borderWidth.
   */
  public float getBorderWidth() {
    return borderWidth;
  }
  /**
   * Setter method for the the property borderWidth.
   * @param borderWidth The value to set for the property borderWidth.
   */
  public void setBorderWidth(float borderWidth) {
    this.borderWidth = borderWidth;
  }
  
  /**
   * Getter method for the the property borderLeftColor.
   * @return The current value of property borderLeftColor.
   */
  public Color getBorderLeftColor() {
    return borderLeftColor;
  }
  /**
   * Setter method for the the property borderLeftColor.
   * @param borderLeftColor The value to set for the property borderLeftColor.
   */
  public void setBorderLeftColor(Color borderLeftColor) {
    this.borderLeftColor = borderLeftColor;
  }
  /**
   * Getter method for the the property borderLeftWidth.
   * @return The current value of property borderLeftWidth.
   */
  public float getBorderLeftWidth() {
    return borderLeftWidth;
  }
  /**
   * Setter method for the the property borderLeftWidth.
   * @param borderLeftWidth The value to set for the property borderLeftWidth.
   */
  public void setBorderLeftWidth(float borderLeftWidth) {
    this.borderLeftWidth = borderLeftWidth;
  }

  /**
   * Getter method for the the property borderRightColor.
   * @return The current value of property borderRightColor.
   */
  public Color getBorderRightColor() {
    return borderRightColor;
  }
  /**
   * Setter method for the the property borderRightColor.
   * @param borderRightColor The value to set for the property borderRightColor.
   */
  public void setBorderRightColor(Color borderRightColor) {
    this.borderRightColor = borderRightColor;
  }
  /**
   * Getter method for the the property borderRightWidth.
   * @return The current value of property borderRightWidth.
   */
  public float getBorderRightWidth() {
    return borderRightWidth;
  }
  /**
   * Setter method for the the property borderRightWidth.
   * @param borderRightWidth The value to set for the property borderRightWidth.
   */
  public void setBorderRightWidth(float borderRightWidth) {
    this.borderRightWidth = borderRightWidth;
  }
  
  /**
   * Getter method for the the property borderTopColor.
   * @return The current value of property borderTopColor.
   */
  public Color getBorderTopColor() {
    return borderTopColor;
  }
  /**
   * Setter method for the the property borderTopColor.
   * @param borderTopColor The value to set for the property borderTopColor.
   */
  public void setBorderTopColor(Color borderTopColor) {
    this.borderTopColor = borderTopColor;
  }
  /**
   * Getter method for the the property borderTopWidth.
   * @return The current value of property borderTopWidth.
   */
  public float getBorderTopWidth() {
    return borderTopWidth;
  }
  /**
   * Setter method for the the property borderTopWidth.
   * @param borderTopWidth The value to set for the property borderTopWidth.
   */
  public void setBorderTopWidth(float borderTopWidth) {
    this.borderTopWidth = borderTopWidth;
  }
  
  /**
   * Getter method for the the property borderBottomColor.
   * @return The current value of property borderBottomColor.
   */
  public Color getBorderBottomColor() {
    return borderBottomColor;
  }
  /**
   * Setter method for the the property borderBottomColor.
   * @param borderBottomColor The value to set for the property borderBottomColor.
   */
  public void setBorderBottomColor(Color borderBottomColor) {
    this.borderBottomColor = borderBottomColor;
  }
  /**
   * Getter method for the the property borderBottomWidth.
   * @return The current value of property borderBottomWidth.
   */
  public float getBorderBottomWidth() {
    return borderBottomWidth;
  }
  /**
   * Setter method for the the property borderBottomWidth.
   * @param borderBottomWidth The value to set for the property borderBottomWidth.
   */
  public void setBorderBottomWidth(float borderBottomWidth) {
    this.borderBottomWidth = borderBottomWidth;
  }


  /**
   * Getter method for the the property padding.
   * @return The current value of property padding.
   */
  public float getPadding() {
    return padding;
  }
  /**
   * Setter method for the the property padding.
   * @param padding The value to set for the property padding.
   */
  public void setPadding(float padding) {
    this.padding = padding;
  }
  
  /**
   * Getter method for the the property paddingBottom.
   * @return The current value of property paddingBottom.
   */
  public float getPaddingBottom() {
    return paddingBottom;
  }
  /**
   * Setter method for the the property paddingBottom.
   * @param paddingBottom The value to set for the property paddingBottom.
   */
  public void setPaddingBottom(float paddingBottom) {
    this.paddingBottom = paddingBottom;
  }
  
  /**
   * Getter method for the the property paddingLeft.
   * @return The current value of property paddingLeft.
   */
  public float getPaddingLeft() {
    return paddingLeft;
  }
  /**
   * Setter method for the the property paddingLeft.
   * @param paddingLeft The value to set for the property paddingLeft.
   */
  public void setPaddingLeft(float paddingLeft) {
    this.paddingLeft = paddingLeft;
  }
  
  /**
   * Getter method for the the property paddingRight.
   * @return The current value of property paddingRight.
   */
  public float getPaddingRight() {
    return paddingRight;
  }
  /**
   * Setter method for the the property paddingRight.
   * @param paddingRight The value to set for the property paddingRight.
   */
  public void setPaddingRight(float paddingRight) {
    this.paddingRight = paddingRight;
  }
  
  /**
   * Getter method for the the property paddingTop.
   * @return The current value of property paddingTop.
   */
  public float getPaddingTop() {
    return paddingTop;
  }
  /**
   * Setter method for the the property paddingTop.
   * @param paddingTop The value to set for the property paddingTop.
   */
  public void setPaddingTop(float paddingTop) {
    this.paddingTop = paddingTop;
  }

  
  /**
   * Getter method for the the property noWrap.
   * @return The current value of property noWrap.
   */
  public boolean isNoWrap() {
    return noWrap;
  }
  /**
   * Setter method for the the property noWrap.
   * @param noWrap The value to set for the property noWrap.
   */
  public void setNoWrap(boolean noWrap) {
    this.noWrap = noWrap;
  }
  
  /**
   * Getter method for the the property fixedHeight.
   * @return The current value of property fixedHeight.
   */
  public float getFixedHeight() {
    return fixedHeight;
  }
  /**
   * Setter method for the the property fixedHeight.
   * @param fixedHeight The value to set for the property fixedHeight.
   */
  public void setFixedHeight(float fixedHeight) {
    this.fixedHeight = fixedHeight;
  }

  /**
   * Getter method for the the property colspan.
   * @return The current value of property colspan.
   */
  public int getColspan() {
    return colspan;
  }
  /**
   * Setter method for the the property colspan.
   * @param colspan The value to set for the property colspan.
   */
  public void setColspan(int colspan) {
    this.colspan = colspan;
  }
  
  /**
   * Getter method for the the property fillRow.
   * @return The current value of property fillRow.
   */
  public boolean isFillRow() {
    return fillRow;
  }
  /**
   * Setter method for the the property fillRow.
   * @param fillRow The value to set for the property fillRow.
   */
  public void setFillRow(boolean fillRow) {
    this.fillRow = fillRow;
  }
  
}