/*
 * Copyright (C) 2003 - 2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import java.awt.Color;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.CellContainer;
import org.ujac.print.ChildDefinition;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.ElementContainer;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.FontHolder;
import org.ujac.print.CommonAttributesHolder;
import org.ujac.print.StyleHolder;

import com.lowagie.text.Cell;
import com.lowagie.text.Phrase;
import com.lowagie.text.Element;
import com.lowagie.text.DocumentException;

/**
 * Name: CellTag<br>
 * Description: An item handling table cells.<br>
 * 
 * @author lauerc
 */
public class CellTag extends BaseDocumentTag implements ElementContainer, CellContainer, FontHolder, StyleHolder {

  /** The item's name. */
  public static final String TAG_NAME = "cell";
  
  /** The current cell attributes. */
  private CellAttributes cellAttributes = null;

  /** The cell container. */
  protected CellContainer cellContainer = null;
  /** The iText table cell. */
  private Cell cell = null;
  /** The element count. */
  private int elementCount = 0;
  /** The cell count. */
  private int cellCount = 0;

  /** The column span of the cell. */ 
  private int colspan = 0;
  /** Tells whether to fill the current row or not. */ 
  private boolean fillRow = false;
  
  /**
   * Constructs a CellTag instance with no specific attributes.
   */
  public CellTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Adds a cell to the surrounding table.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.RENDERED)
      .addDefinition(CommonAttributes.CLASS)
      .addDefinition(CommonAttributes.STYLE)
      .addDefinition(CommonAttributes.FONT)
      .addDefinition(CommonAttributes.LEADING)
      .addDefinition(CommonAttributes.LINE_SPACING)
      .addDefinition(CommonAttributes.BACKGROUND_COLOR)
      .addDefinition(CommonAttributes.COLSPAN)
      .addDefinition(CommonAttributes.FILL_ROW)
      .addDefinition(CommonAttributes.BORDER_STYLE)
      .addDefinition(CommonAttributes.BORDER_WIDTH)
      .addDefinition(CommonAttributes.BORDER_COLOR)
      .addDefinition(CommonAttributes.HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonAttributes.VERTICAL_ALIGNMENT)
      .addDefinition(CommonAttributes.NO_WRAP);
  }

  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.FONT_NAME)
      .addDefinition(CommonStyleAttributes.LEADING)
      .addDefinition(CommonStyleAttributes.LINE_SPACING)
      .addDefinition(CommonStyleAttributes.BACKGROUND_COLOR)
      .addDefinition(CommonStyleAttributes.COLSPAN)
      .addDefinition(CommonStyleAttributes.FILL_ROW)
      .addDefinition(CommonStyleAttributes.BORDER_STYLE)
      .addDefinition(CommonStyleAttributes.BORDER_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_COLOR)
      .addDefinition(CommonStyleAttributes.HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.VERTICAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.NO_WRAP);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(BaseFontTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(MacroTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(IfTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ElseTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(LogTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ForeachTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(SwitchTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(PhraseTag.class, 0, 1))
      .addDefinition(new ChildDefinition(ParagraphTag.class, 0, 1))
      .addDefinition(new ChildDefinition(CellTag.class, 0, 1))
      .addDefinition(new ChildDefinition(TableTag.class, 0, 1))
      .addDefinition(new ChildDefinition(ImageTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(SpaceTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(NewLineTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(NumberFormatTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(DateFormatTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BarcodeTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(DatamatrixTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BarcodePdf417Tag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(AnchorTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ListTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ImportTag.class, 0, ChildDefinition.UNLIMITED));
  }

  /**
   * Tells whether or not this tag does respect style attributes or not.
   * @return always <code>true</code>.
   */
  public boolean isStyleable() {
    return true;
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // setting the cell container.
    cellContainer = documentHandler.latestCellContainer();
    CommonAttributesHolder lineSpacingHolder = getCommonAttributesHolder();
    
    cellAttributes = new CellAttributes(lineSpacingHolder);
    cellAttributes.parseAttributes(this, null);

    if (isAttributeDefined(CommonAttributes.COLSPAN)) {
      colspan = getIntegerAttribute(CommonAttributes.COLSPAN, true, null);
    }
    if (isAttributeDefined(CommonAttributes.FILL_ROW)) {
      fillRow = getBooleanAttribute(CommonAttributes.FILL_ROW, true, null);
      if (fillRow) {
        colspan = cellContainer.getRemainingColumns();
      }
    }
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    // creating the cell    
    cell = new Cell();
    
    // configuring the table
    Color bgColor = cellAttributes.getBgColor();
    if (bgColor != null) {
      cell.setBackgroundColor(bgColor);
    }
    float borderWidth = cellAttributes.getBorderWidth();
    if (borderWidth > 0.0f) {
      cell.setBorderWidth(borderWidth);
    }
    if (colspan > 0) {
      cell.setColspan(colspan);
    }
    cell.setBorder(cellAttributes.getBorderStyle());
    Color borderColor = cellAttributes.getBorderColor();
    if (borderColor != null) {
      cell.setBorderColor(borderColor);
    }
    cell.setHorizontalAlignment(cellAttributes.getHAlign());
    cell.setVerticalAlignment(cellAttributes.getVAlign());
    
    // handling the 'no wrap' flag.
    cell.setMaxLines(cellAttributes.isNoWrap() ? 1 : 0);
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while closing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    if ((elementCount == 0) && (cellCount == 0)) {
      try {
        cell.addElement(addChunk(new Phrase(0.0F), getLeading(), getLineSpacing()));
      } catch (DocumentException ex) {  
        throw new DocumentHandlerException(locator(), "Unable to add phrase to the " + getName() + ": " + ex.getMessage(), ex);
      }
    }
    // adding table to latest element container
    cellContainer.addCell(this, cell);
  }

  /**
   * Adds a element to the container.
   * @param item The document item.
   * @param element The element to be added.
   * @exception DocumentHandlerException If something went wrong when adding the element.
   */
  public void addElement(BaseDocumentTag item, Element element) throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    if (cellCount > 0) {
      throw new DocumentHandlerException(locator(), "Can't hold elements and cells within a cell.");
    }
    ++cellCount;
    try {
      cell.addElement(element);    
    } catch (DocumentException ex) {
      throw new DocumentHandlerException(locator(), "Unable to add element to the " + getName() + ": " + ex.getMessage(), ex);
    }
  }

  /**
   * Tells whether the element container is a top level container or not.
   * @return true if the element container is a top level container, else false.
   */
  public boolean isTopLevel() {
    return false;
  }

  /**
   * Gets the fixed vertical leading distance.
   * @return The current fixed vertical leading distance.
   */
  public float getLeading() {
    return cellAttributes.getLeading();
  }

  /**
   * Gets the line spacing.
   * @return The current line spacing.
   */
  public float getLineSpacing() {
    return cellAttributes.getLineSpacing();
  }

  /**
   * Adds a cell to the container.
   * @param item The document item.
   * @param cell The cell to be added.
   * @exception DocumentHandlerException If something went wrong when adding the cell.
   */
  public void addCell(BaseDocumentTag item, Cell cell) throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    if (elementCount > 0) {
      throw new DocumentHandlerException(locator(), "Can't hold elements and cells within a cell.");
    }
    ++cellCount;
    try {
      cell.addElement(cell);    
    } catch (DocumentException ex) {
      throw new DocumentHandlerException(locator(), "Unable to add cell to the " + getName() + ": " + ex.getMessage(), ex);
    }
  }

  /**
   * Gets the number of remaining columns of the current row.
   * @return The number of remaining column to the end of the current row.
   */
  public int getRemainingColumns() {
    return 1; 
  }

  /**
   * Gets the cell attributes.
   * @return The cell attributes.
   */
  public CellAttributes getCellAttributes() {
    return cellAttributes;
  }

}
