/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.DocumentFont;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.ElementContainer;
import org.ujac.print.TocItem;
import org.ujac.util.template.TemplateContext;

import com.lowagie.text.Chapter;
import com.lowagie.text.Chunk;
import com.lowagie.text.Document;
import com.lowagie.text.Font;
import com.lowagie.text.Paragraph;
import com.lowagie.text.Rectangle;
import com.lowagie.text.Section;

/**
 * Name: ChapterTag<br>
 * Description: Handler class for the chapter tag.
 * 
 * @author lauerc
 */
public class ChapterTag extends SectionTag {

  /** Constant for the property name for the chapter ID. */
  public static final String PROP_CAPTER_ID = "chapterId";
  
  /** The item's name. */
  public static final String TAG_NAME = "chapter";

  /** Definition of the 'font' attribute. */
  private static final AttributeDefinition CHAPTER_FONT = 
      CommonAttributes.FONT.cloneAttrDef("The name of the font definition to use for the chapter contents.");
  /** Definition of the 'font-name' style attribute. */
  private static final AttributeDefinition CHAPTER_FONT_NAME_STYLE = 
      CommonStyleAttributes.FONT_NAME.cloneAttrDef("The name of the font definition to use for the chapter contents.");
  /** Definition of the 'number-format' attribute. */ 
  private static final AttributeDefinition SECTION_NUMBER_STYLE = new AttributeDefinition(
      TagAttributes.ATTR_NUMBER_STYLE, AttributeDefinition.TYPE_INT, false, 
      "The style of the chapter number: 0 = trailing dot, 1 = no trailing dot"
  ); 
  private static final AttributeDefinition SECTION_NUMBER_STYLE_STYLE = new AttributeDefinition(
      TagAttributes.ATTR_NUMBER_STYLE, AttributeDefinition.TYPE_INT, false, 
      "The style of the chapter number: 0 = trailing dot, 1 = no trailing dot"
  ); 
   
  /** The number format. */ 
  private int numberStyle = 0; 
   
  /** The title paragraph. */
  private Paragraph title = null;
  
  /**
   * Constructs a ChapterTag instance with no specific attributes.
   */
  public ChapterTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Adds a chapter to the document.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.CHAPTER_NUMBER)
      .addDefinition(CommonAttributes.CHAPTER_TITLE)
      .addDefinition(CommonAttributes.CHAPTER_TITLE_FONT)
      .addDefinition(CHAPTER_FONT)
      .addDefinition(CommonAttributes.SPACING_BEFORE)
      .addDefinition(CommonAttributes.SPACING_AFTER)
      .addDefinition(CommonAttributes.LEADING)
      .addDefinition(CommonAttributes.LINE_SPACING)
      .addDefinition(CommonAttributes.TOC_RELEVANCE)
      .addDefinition(SECTION_NUMBER_STYLE);
  }

  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.INDENTATION) 
      .addDefinition(CommonStyleAttributes.CHART_TITLE_FONT)
      .addDefinition(CHAPTER_FONT_NAME_STYLE)
      .addDefinition(CommonStyleAttributes.SPACING_BEFORE)
      .addDefinition(CommonStyleAttributes.SPACING_AFTER)
      .addDefinition(CommonStyleAttributes.LEADING)
      .addDefinition(CommonStyleAttributes.LINE_SPACING)
      .addDefinition(CommonStyleAttributes.TOC_RELEVANCE)
      .addDefinition(SECTION_NUMBER_STYLE);
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
  }
  
  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    numberStyle = Section.NUMBERSTYLE_DOTTED;
    if (isAttributeDefined(SECTION_NUMBER_STYLE, SECTION_NUMBER_STYLE_STYLE)) { 
      numberStyle = getIntegerAttribute(SECTION_NUMBER_STYLE, Section.NUMBERSTYLE_DOTTED, true, SECTION_NUMBER_STYLE_STYLE); 
    } 
  }

  /**
   * Creates the iText section element.
   * @exception DocumentHandlerException In case the creation of the section failed.
   */
  protected void createSection() throws DocumentHandlerException {
    String titleStr = getTextAttribute(CommonAttributes.CHAPTER_TITLE, true, null);
    DocumentFont titleDocFont = getTitleFont();
    Font titleFont = null;
    if (titleDocFont != null) {
      titleFont = titleDocFont.getFont();
    } else {
      titleFont = new Font();
    }
    this.title = new Paragraph(titleStr, titleFont);
    // workaround for the missing multipliedLeading (aka line-spacing) support in iText
    extendLeading(title, titleFont, getLeading(), getLineSpacing());
    title.setSpacingBefore(getSpacingBefore());
    title.setSpacingAfter(getSpacingAfter());
    int chapterId = 1;
    if (isAttributeDefined(CommonAttributes.CHAPTER_NUMBER)) {
      chapterId = getIntegerAttribute(CommonAttributes.CHAPTER_NUMBER, 1, true, null);
    } else {
      TemplateContext ctx = documentHandler.getTemplateContext();
      Integer chapId = (Integer) ctx.getProperty(PROP_CAPTER_ID);
      if (chapId == null) {
        chapId = new Integer(1);
      } else {
        chapId = new Integer(chapId.intValue() + 1);
      }
      ctx.setProperty(PROP_CAPTER_ID, chapId);
      chapterId = chapId.intValue();
    }
    this.section = new Chapter(title, chapterId);
    this.section.setNumberStyle(numberStyle); 
    getDocumentHandler().addTocItem(new TocItem(1, Integer.toString(chapterId), titleStr, tocRelevance));
  }

  /**
   * Processes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }
    ElementContainer elementContainer = getElementContainer();
    elementContainer.addElement(this, section);
  }

  /**
   * Handles a new page request.
   * @param size The new document size, null if the size should remain the same.
   * @param margins The new document margins, null if the margins should remain the same.
   * @return A reference to the document. 
   * @exception DocumentHandlerException In case something went wrong.
   */
  public Document handleNewPage(Rectangle size, Rectangle margins) 
    throws DocumentHandlerException {
    
    this.section.add(Chunk.NEXTPAGE);

    return documentHandler.getDocument();
  }

}
