/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import java.awt.Font;
import java.awt.Rectangle;
import java.awt.Graphics2D;

import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;

import java.util.HashMap;
import java.util.List;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;

import org.ujac.chart.Chart;
import org.ujac.chart.ChartAttributeDefinition;
import org.ujac.chart.ChartAttributes;
import org.ujac.chart.ChartException;
import org.ujac.chart.ChartFactory;
import org.ujac.chart.ChartModel;
import org.ujac.chart.TextChartItem;
import org.ujac.chart.ChartItemPosition;
import org.ujac.chart.DefaultChartModel;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.AttributeHandler;
import org.ujac.print.ChildDefinition;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.DocumentFont;
import org.ujac.print.ParameterDefinition;
import org.ujac.print.ParameterHolder;
import org.ujac.print.TagAttributeException;
import org.ujac.print.DocumentHandlerException;

import org.ujac.util.UjacTypes;
import org.ujac.util.exi.ExpressionException;

import org.ujac.util.table.Row;
import org.ujac.util.table.Table;
import org.ujac.util.table.Column;
import org.ujac.util.table.TypeMismatchException;
import org.ujac.util.table.ColumnNotDefinedException;
import org.ujac.util.template.TemplateException;

import com.lowagie.text.BadElementException;
import com.lowagie.text.Image;

import com.lowagie.text.pdf.PdfWriter;
import com.lowagie.text.pdf.FontMapper;
import com.lowagie.text.pdf.PdfTemplate;
import com.lowagie.text.pdf.PdfContentByte;

/**
 * Name: ChartTag<br>
 * Description: A class handling 'chart' items.
 * 
 * @author lauerc
 */
public class ChartTag extends ImageTag implements ParameterHolder {

  /** The item's name. */
  public static final String TAG_NAME = "chart";

  /** The iterator method's attribute declaration. */
  private static final Class[] iteratorAttributes = {};
  /** The iterator method's parameter list. */
  private static final Object[] iteratorParameters = {};
  
  /** The title attribute. */
  private String titleAttr = null;
  /** The title font attribute. */
  private String titleFontAttr = null;
  /** The type. */
  private String type = null;
  
  /** The overall padding. */ 
  private float padding = 0.0f;
  /** The left padding. */ 
  private float paddingLeft = 0.0f;
  /** The right padding. */ 
  private float paddingRight = 0.0f;
  /** The top padding. */ 
  private float paddingTop = 0.0f;
  /** The bottom padding. */ 
  private float paddingBottom = 1.0f;

  /** The chart values to use. */
  private Object valuesAttr = null;
  /** The min chart values to use. */
  private Double minValueAttr = null;
  /** The max chart values to use. */
  private Double maxValueAttr = null;
  /** The value fields to use (required when the values attribute evaluates to a table). */
  private Object valueFieldsAttr = null;
  /** The segment labels. */
  private Object segmentLabelsAttr = null;
  /** The segment label field to use (required when the segment-labels attribute evaluates to a table). */
  private Object segmentFieldAttr = null;
  /** The area labels. */
  private Object areaLabelsAttr = null;
  /** The area label field to use (required when the area-labels attribute evaluates to a table). */
  private Object areaFieldAttr = null;

  /** The title item. */
  private TextChartItem titleItem = null;
  /** The value scale description item. */
  private TextChartItem valueScaleDescriptionItem = null;
  /** The segment scale description item. */
  private TextChartItem segmentScaleDescriptionItem = null;
  
  /** The map of supported chart attributes. */
  private Map chartAttributeDefinitions = null;
  /** The chart instance. */
  private Chart chart = null;
  /** The chart attributes. */
  private ChartAttributes chartAttributes = null;
  /** Tells whether or not a external chart instance has been defined. */
  private boolean externalInstance = false;
  
  /**
   * Constructs a ChartTag instance with no specific attributes.
   */
  public ChartTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Places a chart into the document.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.CHART_TITLE)
      .addDefinition(CommonAttributes.CHART_TITLE_FONT)
      .addDefinition(CommonAttributes.CHART_TYPE)
      .addDefinition(CommonAttributes.CHART_INSTANCE)
      .addDefinition(CommonAttributes.PADDING)
      .addDefinition(CommonAttributes.PADDING_LEFT)
      .addDefinition(CommonAttributes.PADDING_RIGHT)
      .addDefinition(CommonAttributes.PADDING_TOP)
      .addDefinition(CommonAttributes.PADDING_BOTTOM)
      .removeDefinition(CommonAttributes.IMAGE_SOURCE);
  }

  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.CHART_TITLE_FONT)
      .addDefinition(CommonStyleAttributes.CHART_TYPE)
      .addDefinition(CommonStyleAttributes.PADDING)
      .addDefinition(CommonStyleAttributes.PADDING_LEFT)
      .addDefinition(CommonStyleAttributes.PADDING_RIGHT)
      .addDefinition(CommonStyleAttributes.PADDING_TOP)
      .addDefinition(CommonStyleAttributes.PADDING_BOTTOM)
      .removeDefinition(CommonStyleAttributes.IMAGE_SOURCE);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(ParameterTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(TextChartItemTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(IfTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ElseTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ForeachTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(LogTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(SwitchTag.class, 0, ChildDefinition.UNLIMITED));
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();
    
    if (!isValid()) {
      return;
    }

    try {
      this.chart = (Chart) evalAttribute(CommonAttributes.CHART_INSTANCE);
    } catch (ClassCastException ex) {
      throw new TagAttributeException(
          locator(), "The expression '" + getStringAttribute(CommonAttributes.CHART_INSTANCE, false, null) + 
          "' doesn't resolve to a Chart instance."
      );
    }
    this.externalInstance = (chart != null);
    
    // setting the attributes
    this.titleAttr = getTextAttribute(CommonAttributes.CHART_TITLE, false, null);
    this.titleFontAttr = getStringAttribute(CommonAttributes.CHART_TITLE_FONT, true, CommonStyleAttributes.CHART_TITLE_FONT);
    this.type = getStringAttribute(CommonAttributes.CHART_TYPE, false, CommonStyleAttributes.CHART_TYPE);
    
    AttributeHandler attrHandler = documentHandler.getAttributeHandler();
    Object attrVal = getAttribute(CommonAttributes.PADDING, true, CommonStyleAttributes.PADDING, attrHandler.getFloatHandler());
    if (attrVal != null) {
      float padding = ((Number) attrVal).floatValue();
      paddingLeft = padding;
      paddingRight = padding;
      paddingTop = padding;
      paddingBottom = padding;
    }
    attrVal = getAttribute(CommonAttributes.PADDING_LEFT, true, CommonStyleAttributes.PADDING_LEFT, attrHandler.getFloatHandler());
    if (attrVal != null) {
      paddingLeft = ((Number) attrVal).floatValue();
    }
    attrVal = getAttribute(CommonAttributes.PADDING_RIGHT, true, CommonStyleAttributes.PADDING_RIGHT, attrHandler.getFloatHandler());
    if (attrVal != null) {
      paddingRight = ((Number) attrVal).floatValue();
    }
    attrVal = getAttribute(CommonAttributes.PADDING_TOP, true, CommonStyleAttributes.PADDING_TOP, attrHandler.getFloatHandler());
    if (attrVal != null) {
      paddingTop = ((Number) attrVal).floatValue();
    }
    attrVal = getAttribute(CommonAttributes.PADDING_BOTTOM, true, CommonStyleAttributes.PADDING_BOTTOM, attrHandler.getFloatHandler());
    if (attrVal != null) {
      paddingBottom = ((Number) attrVal).floatValue();
    }

    if (!externalInstance) {
      try {
        if (type == null) {
          throw new TagAttributeException(locator(), "No chart type and no external instance was given, unable to render chart.");
        }
        String chartType = null;
        try {
          chartType = documentHandler.evalString(type).toLowerCase();
        } catch (ExpressionException ex) {
          throw new TagAttributeException(locator(), "Attribute evaluation failed: " + ex.getMessage(), ex);
        }
        this.chart = ChartFactory.createChart(chartType);
        this.chartAttributes = new ChartAttributes();
  
        List supportedChartAttributes = chart.getSupportedAttributes();
        int numSupportedChartAttributes =  supportedChartAttributes.size();
        chartAttributeDefinitions = new HashMap();
        chartAttributeDefinitions.put("values", new ParameterDefinition(
          "values",
          UjacTypes.TYPE_DOUBLE,
          true,
          false,
          null
        ));
        chartAttributeDefinitions.put("min-value", new ParameterDefinition(
          "min-value",
          UjacTypes.TYPE_DOUBLE,
          false,
          true,
          null
        ));
        chartAttributeDefinitions.put("max-value", new ParameterDefinition(
          "max-value",
          UjacTypes.TYPE_DOUBLE,
          false,
          true,
          null
        ));
        chartAttributeDefinitions.put("value-fields", new ParameterDefinition(
          "value-fields",
          UjacTypes.TYPE_STRING,
          true,
          false,
          null
        ));
        chartAttributeDefinitions.put("segment-field", new ParameterDefinition(
          "segment-field",
          UjacTypes.TYPE_STRING,
          false,
          false,
          null
        ));
        for (int i = 0; i < numSupportedChartAttributes; i++) {
          ChartAttributeDefinition cad = (ChartAttributeDefinition) supportedChartAttributes.get(i);
          chartAttributeDefinitions.put(cad.getName(), new ParameterDefinition(
            cad.getName(),
            chartAttributeTypeToParameterType(cad),
            cad.isList(),
            cad.isOptional(),
            null
          ));
        }
      } catch (ChartException ex) {
        throw new DocumentHandlerException(locator(), "Unable to create chart: " + ex.getMessage(), ex);
      }
    }
  }
  
  /**
   * Maps a chart attribute type to the according parameter type. 
   * @param chartAttributeDef The chart attribute definition for which to convert the type.
   * @return The according parameter type.
   */
  private int chartAttributeTypeToParameterType(ChartAttributeDefinition chartAttributeDef) { 
    
    switch (chartAttributeDef.getType()) {
      case UjacTypes.TYPE_BOOLEAN:
      case UjacTypes.TYPE_INT:
      case UjacTypes.TYPE_FLOAT:
      case UjacTypes.TYPE_DOUBLE:
      case UjacTypes.TYPE_STRING:
      case UjacTypes.TYPE_DATE:
      case UjacTypes.TYPE_OBJECT:
        return chartAttributeDef.getType();
      case ChartAttributeDefinition.TYPE_COLOR:
        return ParameterDefinition.TYPE_COLOR;
      case ChartAttributeDefinition.TYPE_FONT:
        return ParameterDefinition.TYPE_FONT;
      case ChartAttributeDefinition.TYPE_FORMAT:
        return ParameterDefinition.TYPE_FORMAT;
      default:
        return UjacTypes.TYPE_OBJECT;
        //throw new DocumentHandlerException(locator(), "Unable to convert chart attribute type of attribute " + chartAttributeDef.getName() + ".");
    }
  }
  
  /**
   * Gets the image to insert into the document.
   * @return The created image.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public Image getImage() throws DocumentHandlerException {
    float width = getWidth();
    float height = getHeight();
    
    PdfWriter writer = (PdfWriter) documentHandler.getDocumentWriter();
    PdfContentByte cb = writer.getDirectContent();
    PdfTemplate tp = cb.createTemplate(width, height);
    
    FontMapper fontMapper = documentHandler.getFontMapper();
    Graphics2D graphics = tp.createGraphics(width, height, fontMapper);
    
    try {
      if (!externalInstance) {
        // adding cell to latest cell container
        ChartModel cm = new DefaultChartModel();
        if (minValueAttr != null) {
          cm.setMinValue(minValueAttr.doubleValue());
        }
        if (maxValueAttr != null) {
          cm.setMaxValue(maxValueAttr.doubleValue());
        }
        if (titleAttr != null) {
          DocumentFont titleFont = null;
          if (titleFontAttr != null) {
            titleFont = documentHandler.getFont(titleFontAttr);
          }
          if (titleFont == null) {
            titleFont = getFont();
          }
          try {
            titleItem = new TextChartItem(
                new ChartItemPosition(ChartItemPosition.CENTER, ChartItemPosition.TOP), 
                documentHandler.executeTemplate(titleAttr), awtFontFromDocumentFont(titleFont)
            );
          } catch (TemplateException ex) {
            throw new TagAttributeException(locator(), "Attribute evaluation failed: " + ex.getMessage(), ex);
          }
        }
        if (titleItem != null) {
          cm.setTitle(titleItem);
        }
        cm.setChartData(getChartValues());
        cm.setAreaLabels(getAreaLabels());
        cm.setSegmentLabels(getSegmentLabels());
    
        chartAttributes.setFloat(Chart.ATTR_PADDING, this.padding);
        chartAttributes.setFloat(Chart.ATTR_PADDING_LEFT, this.paddingLeft);
        chartAttributes.setFloat(Chart.ATTR_PADDING_RIGHT, this.paddingRight);
        chartAttributes.setFloat(Chart.ATTR_PADDING_TOP, this.paddingTop);
        chartAttributes.setFloat(Chart.ATTR_PADDING_BOTTOM, this.paddingBottom);
        if (valueScaleDescriptionItem != null) {
          chartAttributes.setTextItem(Chart.ATTR_VALUE_SCALE_DESCRIPTION, valueScaleDescriptionItem);
        }
        if (segmentScaleDescriptionItem != null) {
          chartAttributes.setTextItem(Chart.ATTR_SEGMENT_SCALE_DESCRIPTION, segmentScaleDescriptionItem);
        }
  
        try {
          // configuring chart
          chart.setModel(cm);
          chart.setAttributes(chartAttributes);
        } catch (ChartException ex) {
          throw new DocumentHandlerException(locator(), "Failed to render chart: " + 
              ex.getMessage(), ex);
        }
      }

      try {
        // drawing chart
        chart.draw(new Rectangle(0, 0, (int) width, (int) height), graphics);
        // OK, imagwe drawn
        return Image.getInstance(tp);
      } catch (ChartException ex) {
        throw new DocumentHandlerException(locator(), "Failed to render chart: " + 
            ex.getMessage(), ex);
      } catch (BadElementException ex) {
        throw new DocumentHandlerException(locator(), "Failed to output chart: " + ex.getMessage(), ex);
      }
    } finally {
      // disposing graphics context, important to make future output to 
      // pdf content byte work.
      graphics.dispose();
    }
  }
  
  /**
   * Gets the chart values.
   * @return A list of double arrays.
   * @throws DocumentHandlerException
   */
  private List getChartValues() throws DocumentHandlerException {
    List chartData = new ArrayList();
    Iterator iter = getValuesIterator();
    Object first = iter.next();
    if (first instanceof Row) {
      Row row = (Row) first;
      Table table = row.getTable();
      // parsing the columns attribute
      if (valueFieldsAttr == null) {
        throw new TagAttributeException(locator(), "The attribute 'value-fields' at item 'chart' must be specified for table values.");
      }
      List valueFieldNames = null;
      if (valueFieldsAttr instanceof List) {
        valueFieldNames = (List) valueFieldsAttr;
      } else {
        valueFieldNames = new ArrayList();
        valueFieldNames.add(valueFieldsAttr.toString());
      }
      int[] colIndices = new int[valueFieldNames.size()];
      int idx = 0;
      for (int i = 0; i < colIndices.length; i++) {
        String colName = (String) valueFieldNames.get(i);
        Column col = null;
        try {
          col = table.getColumn(colName);
        } catch (ColumnNotDefinedException ex) {
          try  {
            int colIdx = Integer.parseInt(colName);
            col = table.getColumn(colIdx);
          } catch (ColumnNotDefinedException cndex) {
            throw new TagAttributeException(locator(), "The value '" + colName + "' from attribute 'value-fields' is no valid column index within table '" + valuesAttr + "'.", ex);
          } catch (NumberFormatException nfex) {
            throw new TagAttributeException(locator(), "The value '" + colName + "' from attribute 'value-fields' is no valid column within table '" + valuesAttr + "'.", ex);
          }
        }
        colIndices[idx] = col.getIndex();
        ++idx;
      }
      int numColumns = colIndices.length;
      while (row != null) {
        double[] valueRow = new double[numColumns];
        for (int i = 0; i < numColumns; i++) {
          int colIdx = colIndices[i];
          try {
            valueRow[i] = row.getDouble(colIdx);
          } catch (ColumnNotDefinedException cndex) {
            throw new TagAttributeException(locator(), cndex.getMessage(), cndex);
          } catch (TypeMismatchException tmex) {
            throw new TagAttributeException(locator(), tmex.getMessage(), tmex);
          }
        }
        chartData.add(valueRow);
        row = null;
        if (iter.hasNext()) {
          row = (Row) iter.next();
        }
      }
    } else if (first instanceof Number) {
      Number number = (Number) first;
      while (number != null) {
        double[] valueRow = new double[] {number.doubleValue()};
        chartData.add(valueRow);
        number = null;
        if (iter.hasNext()) {
          number = (Number) iter.next();
        }
      } 
    } else if (first instanceof Number[]) {
      Number[] numbers = (Number[]) first;
      while (numbers != null) {
        double[] valueRow = new double[numbers.length];
        for (int i = 0; i < numbers.length; i++) {
          if (numbers[i] != null) {
            valueRow[i] = numbers[i].doubleValue();
          }
        }
        chartData.add(valueRow);
        numbers = null;
        if (iter.hasNext()) {
          numbers = (Number[]) iter.next();
        }
      } 
    } else if (first instanceof double[]) {
      double[] numbers = (double[]) first;
      while (numbers != null) {
        double[] valueRow = numbers;
        chartData.add(valueRow);
        numbers = null;
        if (iter.hasNext()) {
          numbers = (double[]) iter.next();
        }
      } 
    } else if (first instanceof List) {
      List numbers = (List) first;
      while (numbers != null) {
        int numValues = numbers.size();
        double[] valueRow = new double[numValues];
        for (int i = 0; i < numValues; i++) {
          valueRow[i] = ((Number) numbers.get(i)).doubleValue();
        }
        chartData.add(valueRow);
        numbers = null;
        if (iter.hasNext()) {
          numbers = (List) iter.next();
        }
      } 
    } else if (first instanceof String) {
      String number = (String) first;
      while (number != null) {
        double[] valueRow = new double[] {Double.parseDouble(number)};
        chartData.add(valueRow);
        number = null;
        if (iter.hasNext()) {
          number = (String) iter.next();
        }
      } 
    } else {
      Object cur = first;
      // parsing the columns attribute
      if (valueFieldsAttr == null) {
        throw new TagAttributeException(locator(), "The attribute 'value-fields' at item 'chart' must be specified for bean values.");
      }
      List valueFieldNames = null;
      if (valueFieldsAttr instanceof List) {
        valueFieldNames = (List) valueFieldsAttr;
      } else {
        valueFieldNames = new ArrayList();
        valueFieldNames.add(valueFieldsAttr.toString());
      }
      int numValueFields = valueFieldNames.size();
      Method[] getters = new Method[numValueFields];
      Class beanClazz = cur.getClass();
      Class[] getterParams = {};
      for (int i = 0; i < numValueFields; i++) {
        String fieldName = (String) valueFieldNames.get(i);
        Method getter = null;
        try {
          getter = beanClazz.getMethod("get" + Character.toUpperCase(fieldName.charAt(0)) + fieldName.substring(1), getterParams);
        } catch (NoSuchMethodException ex) {
          throw new TagAttributeException(locator(), "The value '" + fieldName + "' from attribute 'value-fields' is no valid attribute for class '" + beanClazz.getName() + "'.", ex);
        } catch (SecurityException ex) {
          throw new TagAttributeException(locator(), "The value '" + fieldName + "' from attribute 'value-fields' is no valid attribute for class '" + beanClazz.getName() + "'.", ex);
        }
        getters[i] = getter;
      }
      Object[] getterArgs = {};
      while (cur != null) {
        double[] valueRow = new double[numValueFields];
        for (int i = 0; i < numValueFields; i++) {
          Method getter = getters[i];
          try {
            valueRow[i] = ((Number) getter.invoke(cur, getterArgs)).doubleValue();
          } catch (ClassCastException ex) {
            throw new TagAttributeException(locator(), "The value '" + valueFieldNames.get(i) + "' from attribute 'value-fields' is no valid numeric attribute for class '" + cur.getClass().getName() + "'.", ex);
          } catch (IllegalAccessException ex) {
            throw new TagAttributeException(locator(), "The value '" + valueFieldNames.get(i) + "' from attribute 'value-fields' is not accessible for class '" + cur.getClass().getName() + "'.", ex);
          } catch (InvocationTargetException ex) {
            throw new TagAttributeException(locator(), "The value '" + valueFieldNames.get(i) + "' from attribute 'value-fields' is not accessible for class '" + cur.getClass().getName() + "'.", ex);
          }
        }
        chartData.add(valueRow);
        cur = null;
        if (iter.hasNext()) {
          cur = iter.next();
        }
      }
    }
    return chartData;
  }

  /**
   * Gets the area labels.
   * @return A list of Strings.
   * @throws DocumentHandlerException In case the label list processing failed.
   */
  private List getAreaLabels() throws DocumentHandlerException {
    String areaFieldName = null;
    if (areaFieldAttr != null) {
      areaFieldName = areaFieldAttr.toString();
    }
    return getLabelList(getAreaLabelsIterator(), areaFieldName, "area-field");
  }

  /**
   * Gets the segment labels.
   * @return A list of Strings.
   * @throws DocumentHandlerException In case the label list processing failed.
   */
  private List getSegmentLabels() throws DocumentHandlerException {
    String segmentFieldName = null;
    if (segmentFieldAttr != null) {
      segmentFieldName = segmentFieldAttr.toString();
    }
    return getLabelList(getSegmentLabelsIterator(), segmentFieldName, "segment-field");
  }
  
  /**
   * Gets a list of labels.
   * @param labelIterator The label iterator.
   * @param fieldName The name of the table column of bean property.
   * @param attrAttrName The attribute name, needed to build error messages.
   * @return A list of Strings.
   * @throws DocumentHandlerException In case the label list processing failed.
   */
  private List getLabelList(Iterator labelIterator, String fieldName, String attrAttrName) throws DocumentHandlerException {
    Iterator iter = labelIterator;
    if (iter == null) {
      return null;
    }
    List segmentLabels = new ArrayList();
    Object first = iter.next();
    if (first instanceof Row) {
      Row row = (Row) first;
      Table table = row.getTable();
      // parsing the columns attribute
      if (fieldName == null) {
        throw new TagAttributeException(locator(), "The attribute '" + attrAttrName + "' at tag 'chart' must be specified for segment-labels.");
      }
      int colIndex = -1;
      Column col = null;
      try {
        col = table.getColumn(fieldName);
      } catch (ColumnNotDefinedException ex) {
        try  {
          int colIdx = Integer.parseInt(fieldName);
          col = table.getColumn(colIdx);
        } catch (ColumnNotDefinedException cndex) {
          throw new TagAttributeException(locator(), "The value '" + fieldName + "' from attribute '" + attrAttrName + "' is no valid column index within table '" + valuesAttr + "'.", ex);
        } catch (NumberFormatException nfex) {
          throw new TagAttributeException(locator(), "The value '" + fieldName + "' from attribute '" + attrAttrName + "' is no valid column within table '" + valuesAttr + "'.", ex);
        }
      }
      colIndex = col.getIndex();

      while (row != null) {
        try {
          segmentLabels.add(row.getString(colIndex));
          row = null;
          if (iter.hasNext()) {
            row = (Row) iter.next();
          }
        } catch (ColumnNotDefinedException cndex) {
          throw new TagAttributeException(locator(), cndex.getMessage(), cndex);
        } catch (TypeMismatchException tmex) {
          throw new TagAttributeException(locator(), tmex.getMessage(), tmex);
        }
      }
    } else if (first instanceof String) {
      String label = (String) first;
      while (label != null) {
        segmentLabels.add(label);
        label = null;
        if (iter.hasNext()) {
          label = (String) iter.next();
        }
      } 
    } else {
      Object cur = first;
      // parsing the columns attribute
      if (segmentFieldAttr == null) {
        throw new TagAttributeException(locator(), "The attribute '" + attrAttrName + "' at item 'chart' must be specified for segment-labels.");
      }
      Method getter = null;
      Class beanClazz = cur.getClass();
      Class[] getterParams = {};
      try {
        getter = beanClazz.getMethod("get" + Character.toUpperCase(fieldName.charAt(0)) + fieldName.substring(1), getterParams);
      } catch (NoSuchMethodException ex) {
        throw new TagAttributeException(locator(), "The value '" + fieldName + "' from attribute '" + attrAttrName + "' is no valid attribute for class '" + beanClazz.getName() + "'.", ex);
      } catch (SecurityException ex) {
        throw new TagAttributeException(locator(), "The value '" + fieldName + "' from attribute '" + attrAttrName + "' is no valid attribute for class '" + beanClazz.getName() + "'.", ex);
      }
      Object[] getterArgs = {};
      while (cur != null) {
        String text = null;
        try {
          Object value = getter.invoke(cur, getterArgs);
          if (value != null) {
            text = value.toString();
          }
        } catch (IllegalAccessException ex) {
          throw new TagAttributeException(locator(), "The value '" + fieldName + "' from attribute '" + attrAttrName + "' is not accessible for class '" + cur.getClass().getName() + "'.", ex);
        } catch (InvocationTargetException ex) {
          throw new TagAttributeException(locator(), "The value '" + fieldName + "' from attribute '" + attrAttrName + "' is not accessible for class '" + cur.getClass().getName() + "'.", ex);
        }
        segmentLabels.add(text);
        cur = null;
        if (iter.hasNext()) {
          cur = iter.next();
        }
      }
    }
    return segmentLabels;
  }

  /**
   * Converts the given iText font into a AWT font.
   * @param font the document font to convert.
   * @return The according AWT font.
   */
  protected Font awtFontFromDocumentFont(DocumentFont font) {
    return documentHandler.getAwtFont(font);
  }

  /**
   * Gets the iterator by which to obtain the values for the chart.
   * @return The desired iterator.
   * @throws DocumentHandlerException If an exception occured while evaluating the value expression.
   * @throws TagAttributeException If an exception occured while fechting the iterator.
   */  
  private Iterator getValuesIterator() throws DocumentHandlerException, TagAttributeException {
    Object valuesHolder = valuesAttr;
    if (valuesHolder instanceof String) {
      try {
        valuesHolder = documentHandler.evalObject("${" + valuesHolder.toString() + "}");
      } catch (ExpressionException ex) {
        throw new TagAttributeException(locator(), "Attribute evaluation failed: " + ex.getMessage(), ex);
      }
    }
    if (valuesHolder == null) {
      return null;
    }

    if (valuesHolder instanceof Collection) {
      return ((Collection) valuesHolder).iterator();
    }
    if (valuesHolder instanceof Iterator) {
      return (Iterator) valuesHolder;
    }
    try {
      Method iteratorMethod = valuesHolder.getClass().getMethod("iterator", iteratorAttributes);
      return (Iterator) iteratorMethod.invoke(valuesHolder, iteratorParameters);
    } catch (SecurityException ex) {
      throw new TagAttributeException(locator(), "The attribute 'values' at item 'chart' doesn't evaluate to a valid sequence.", ex);
    } catch (NoSuchMethodException ex) {
      throw new TagAttributeException(locator(), "The attribute 'values' at item 'chart' doesn't evaluate to a valid sequence.", ex);
    } catch (ClassCastException ex) {
      throw new TagAttributeException(locator(), "The attribute 'values' at item 'chart' doesn't evaluate to a valid sequence.", ex);
    } catch (IllegalArgumentException ex) {
      throw new TagAttributeException(locator(), "The attribute 'values' at item 'chart' doesn't evaluate to a valid sequence.", ex);
    } catch (IllegalAccessException ex) {
      throw new TagAttributeException(locator(), "The attribute 'values' at item 'chart' doesn't evaluate to a valid sequence.", ex);
    } catch (InvocationTargetException ex) {
      throw new TagAttributeException(locator(), "The attribute 'values' at item 'chart' doesn't evaluate to a valid sequence.", ex);
    }
  }

  /**
   * Gets the iterator by which to obtain the area labels for the chart.
   * @return The desired iterator.
   * @throws DocumentHandlerException If an exception occured while evaluating the value expression.
   * @throws TagAttributeException If an exception occured while fechting the iterator.
   */  
  private Iterator getAreaLabelsIterator() throws DocumentHandlerException, TagAttributeException {
    Object valuesHolder = areaLabelsAttr;
    if (valuesHolder instanceof String) {
      try {
        valuesHolder = documentHandler.evalObject("${" + valuesHolder.toString() + "}");
      } catch (ExpressionException ex) {
        throw new TagAttributeException(locator(), "Evaluation of attribute 'area-labels' failed: " + ex.getMessage(), ex);
      }
    }
    if (valuesHolder == null) {
      return null;
    }

    if (valuesHolder instanceof Collection) {
      return ((Collection) valuesHolder).iterator();
    }
    if (valuesHolder instanceof Iterator) {
      return (Iterator) valuesHolder;
    }
    try {
      Method iteratorMethod = valuesHolder.getClass().getMethod("iterator", iteratorAttributes);
      return (Iterator) iteratorMethod.invoke(valuesHolder, iteratorParameters);
    } catch (SecurityException ex) {
      throw new TagAttributeException(locator(), "The attribute 'area-labels' at item 'chart' doesn't evaluate to a valid sequence.", ex);
    } catch (NoSuchMethodException ex) {
      throw new TagAttributeException(locator(), "The attribute 'area-labels' at item 'chart' doesn't evaluate to a valid sequence.", ex);
    } catch (ClassCastException ex) {
      throw new TagAttributeException(locator(), "The attribute 'area-labels' at item 'chart' doesn't evaluate to a valid sequence.", ex);
    } catch (IllegalArgumentException ex) {
      throw new TagAttributeException(locator(), "The attribute 'area-labels' at item 'chart' doesn't evaluate to a valid sequence.", ex);
    } catch (IllegalAccessException ex) {
      throw new TagAttributeException(locator(), "The attribute 'area-labels' at item 'chart' doesn't evaluate to a valid sequence.", ex);
    } catch (InvocationTargetException ex) {
      throw new TagAttributeException(locator(), "The attribute 'area-labels' at item 'chart' doesn't evaluate to a valid sequence.", ex);
    }
  }

  /**
   * Gets the iterator by which to obtain the segment labels for the chart.
   * @return The desired iterator.
   * @throws DocumentHandlerException If an exception occured while evaluating the value expression.
   * @throws TagAttributeException If an exception occured while fechting the iterator.
   */  
  private Iterator getSegmentLabelsIterator() throws DocumentHandlerException, TagAttributeException {
    Object valuesHolder = segmentLabelsAttr;
    if (valuesHolder instanceof String) {
      try {
        valuesHolder = documentHandler.evalObject("${" + valuesHolder.toString() + "}");
      } catch (ExpressionException ex) {
        throw new TagAttributeException(locator(), "Evaluation of attribute 'segment-labels' failed: " + ex.getMessage(), ex);
      }
    }
    if (valuesHolder == null) {
      return null;
    }

    if (valuesHolder instanceof Collection) {
      return ((Collection) valuesHolder).iterator();
    }
    if (valuesHolder instanceof Iterator) {
      return (Iterator) valuesHolder;
    }
    try {
      Method iteratorMethod = valuesHolder.getClass().getMethod("iterator", iteratorAttributes);
      return (Iterator) iteratorMethod.invoke(valuesHolder, iteratorParameters);
    } catch (SecurityException ex) {
      throw new TagAttributeException(locator(), "The attribute 'segment-labels' at item 'chart' doesn't evaluate to a valid sequence.", ex);
    } catch (NoSuchMethodException ex) {
      throw new TagAttributeException(locator(), "The attribute 'segment-labels' at item 'chart' doesn't evaluate to a valid sequence.", ex);
    } catch (ClassCastException ex) {
      throw new TagAttributeException(locator(), "The attribute 'segment-labels' at item 'chart' doesn't evaluate to a valid sequence.", ex);
    } catch (IllegalArgumentException ex) {
      throw new TagAttributeException(locator(), "The attribute 'segment-labels' at item 'chart' doesn't evaluate to a valid sequence.", ex);
    } catch (IllegalAccessException ex) {
      throw new TagAttributeException(locator(), "The attribute 'segment-labels' at item 'chart' doesn't evaluate to a valid sequence.", ex);
    } catch (InvocationTargetException ex) {
      throw new TagAttributeException(locator(), "The attribute 'segment-labels' at item 'chart' doesn't evaluate to a valid sequence.", ex);
    }
  }

  /**
   * @see org.ujac.print.ParameterHolder#setParameter(java.lang.String, java.lang.Object)
   */
  public void setParameter(String name, Object value) throws DocumentHandlerException {
    if ("values".equals(name)) {
      this.valuesAttr = value;
      return;
    }
    if ("min-value".equals(name)) {
      this.minValueAttr = (Double) value;
      return;
    }
    if ("max-value".equals(name)) {
      this.maxValueAttr = (Double) value;
      return;
    }
    if ("value-fields".equals(name)) {
      this.valueFieldsAttr = value;
      return;
    }
    if ("segment-labels".equals(name)) {
      this.segmentLabelsAttr = value;
      return;
    }
    if ("segment-field".equals(name)) {
      this.segmentFieldAttr = value;
      return;
    }
    if ("area-labels".equals(name)) {
      this.areaLabelsAttr = value;
      return;
    }
    if ("area-field".equals(name)) {
      this.areaFieldAttr = value;
      return;
    }
    if ("title".equals(name)) {
      this.titleItem = (TextChartItem) value;
      return;
    }
    if (Chart.ATTR_VALUE_SCALE_DESCRIPTION.equals(name)) {
      this.valueScaleDescriptionItem = (TextChartItem) value;
      return;
    }
    if (Chart.ATTR_SEGMENT_SCALE_DESCRIPTION.equals(name)) {
      this.segmentScaleDescriptionItem = (TextChartItem) value;
      return;
    }

    ParameterDefinition pd = (ParameterDefinition) chartAttributeDefinitions.get(name);
    if (pd.isList()) {
      chartAttributes.setList(name, (List) value);
    } else {
      switch (pd.getType()) {
        case UjacTypes.TYPE_BOOLEAN:
          chartAttributes.setBoolean(name, ((Boolean) value).booleanValue());
          break;
        case UjacTypes.TYPE_INT:
          chartAttributes.setInt(name, ((Integer) value).intValue());
          break;
        case UjacTypes.TYPE_FLOAT:
          chartAttributes.setFloat(name, ((Float) value).floatValue());
          break;
        case UjacTypes.TYPE_DOUBLE:
          chartAttributes.setDouble(name, ((Double) value).doubleValue());
          break;
        case UjacTypes.TYPE_STRING:
          chartAttributes.setString(name, (String) value);
          break;
        case UjacTypes.TYPE_DATE:
          throw new DocumentHandlerException(locator(), "Unsupported attribute type Date for attribute " + name + ".");

        case ParameterDefinition.TYPE_COLOR:
          chartAttributes.setColor(name, colorFromString(name, value, true));
          break;
        case ParameterDefinition.TYPE_FONT:
          try {
            DocumentFont descriptionFont = documentHandler.getFont((String) value);
            chartAttributes.setFont(name, awtFontFromDocumentFont(descriptionFont));
          } catch (TagAttributeException ex) {
            // Font was not defined
          }
          break;
        case ParameterDefinition.TYPE_FORMAT:
          chartAttributes.setObject(name, documentHandler.lookupFormat((String) value));
          break;
        case UjacTypes.TYPE_OBJECT:
          break;
        default:
          break;
      }
    }
  }

  /**
   * @see org.ujac.print.ParameterHolder#getParameterDefinition(java.lang.String)
   */
  public ParameterDefinition getParameterDefinition(String name) {
    return (ParameterDefinition) chartAttributeDefinitions.get(name);
  }

}
