/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import java.awt.Color;
import java.awt.color.ColorSpace;

import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.TagAttributeException;

import com.lowagie.text.pdf.CMYKColor;
import com.lowagie.text.pdf.PdfSpotColor;
import com.lowagie.text.pdf.SpotColor;

/**
 * Name: ColorDefItem<br>
 * Description: A class which handles color definitions.<br>
 * 
 * @author lauerc
 */
public class ColorDefTag extends BaseDocumentTag {

  /** Definition of the 'color-space' attribute. */
  private static final AttributeDefinition COLOR_SPACE = new AttributeDefinition(TagAttributes.ATTR_COLOR_SPACE, 
      new String[] {"RGB", "sRGB", "LINEAR_RGB", "GRAY", "CIEXYZ", "PYCC", "CMYK"}, 
      true, "The name of the color space.");
  /** Definition of the 'value' attribute. */
  private static final AttributeDefinition VALUE = new AttributeDefinition(TagAttributes.ATTR_VALUE, AttributeDefinition.TYPE_STRING, true,
      "The values of the color components in comma separated form. " +
      "The values can be entered in three forms: float in range from 0.0 to 1.0," +
      " integer in range from 0 to 255 and hexadecimal in range from 0x00 to 0xff.");
  /** Definition of the 'spot-color' attribute. */
  private static final AttributeDefinition SPOT_COLOR = new AttributeDefinition(TagAttributes.ATTR_SPOT_COLOR, AttributeDefinition.TYPE_BOOLEAN, false,
      "Tells wether or not to define the color as a SpotColor or Pantone Color.");
  /** Definition of the 'name' attribute. */
  private static final AttributeDefinition COLOR_NAME =
      CommonAttributes.NAME.cloneAttrDef("The name of the color definition.");
  
  /** The item's name. */
  public static final String TAG_NAME = "color-def";
  
  /** Defines the valid color spaces. */
  private static final Map COLOR_SPACES = new HashMap();
  static {
    COLOR_SPACES.put("sRGB", new Integer(ColorSpace.CS_sRGB));
    COLOR_SPACES.put("LINEAR_RGB", new Integer(ColorSpace.CS_LINEAR_RGB));
    COLOR_SPACES.put("GRAY", new Integer(ColorSpace.CS_GRAY));
    COLOR_SPACES.put("CIEXYZ", new Integer(ColorSpace.CS_CIEXYZ));
    COLOR_SPACES.put("PYCC", new Integer(ColorSpace.CS_PYCC));
  }
  
  /** The font's name. */
  private String name = null;
  /** The color space. */
  private ColorSpace colorSpace = null;
  /** The spot color flag. */
  private boolean spotColor = false;
  /** The color components. */
  private float[] components = null;
  
  /**
   * Constructs a ColorDefTag instance with no specific attributes.
   */
  public ColorDefTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Defines a logical color for the document. This defined color can be used by every tag, " +
      " which defines a color attribute." ;
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(COLOR_NAME)    
      .addDefinition(COLOR_SPACE)
      .addDefinition(VALUE)
      .addDefinition(SPOT_COLOR);
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
  }
  
  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    // getting attributes
    name = getStringAttribute(CommonAttributes.NAME, false, null);
    String colorSpaceName = getStringAttribute(COLOR_SPACE, true, null);
    String strValue = getStringAttribute(VALUE, true, null);
    spotColor = getBooleanAttribute(SPOT_COLOR, false, true, null);
    
    StringTokenizer valueTkn = new StringTokenizer(strValue, ",");
    int numComponents = valueTkn.countTokens();
    components = new float[numComponents];
    boolean intValues = false;
    for (int i = 0; i < numComponents; i++) {
      String component = valueTkn.nextToken();
      try {
        if (component.startsWith("0x")) {
          int comp = Integer.parseInt(component.substring(2), 16);
          if ((comp < 0) || (comp > 255)) {
            throw new TagAttributeException(locator(), "Illegal value '" + strValue + "' for attribute '" + 
                TagAttributes.ATTR_VALUE + "', all components must be in range from 0x00 to 0xff.");
          }
          components[i] = comp; 
          intValues = true;
          continue;
        }
        float comp = Float.parseFloat(component);
        if (comp > 1.0) {
          intValues = true;
        }
        if ((comp < 0) || (comp > 255)) {
          throw new TagAttributeException(locator(), "Illegal value '" + strValue + "' for attribute '" + 
              TagAttributes.ATTR_VALUE + "', all components must be in range from 0 to 255 or in range from 0.0 to 1.0 for float values.");
        }
        components[i] = comp;
      } catch (NumberFormatException ex) {
        throw new TagAttributeException(locator(), "Illegal value '" + strValue + "' for attribute '" + 
            TagAttributes.ATTR_VALUE + "', must define a comma separated list of float, integer or hexadecimal values.");
      }
    }
    
    Color color = null;
    if ("CMYK".equals(colorSpaceName)) {  
      // processing CYMK color
      if (numComponents != 4) {
        throw new TagAttributeException(locator(), "Illegal value '" + strValue + "' for attribute '" + 
            TagAttributes.ATTR_VALUE + "', must define 4 values for components C,M,Y,K.");
      }
      if (intValues) {
        color = new CMYKColor((int) components[0], (int) components[1], (int) components[2], (int) components[3]);
      } else {
        color = new CMYKColor(components[0], components[1], components[2], components[3]);
      }
    } else if ("RGB".equals(colorSpaceName)) {
      // processing RGB color
      if (numComponents != 3) {
        throw new TagAttributeException(locator(), "Illegal value '" + strValue + "' for attribute '" + 
            TagAttributes.ATTR_VALUE + "', must define 3 values for components R,G,B.");
      }
      if (intValues) {
        color = new Color((int) components[0], (int) components[1], (int) components[2]);
      } else {
        color = new Color(components[0], components[1], components[2]);
      }
    } else {
      if (intValues) {
        // transforming integer values in rang 0..255 to float values in range 0.0f..1.0f
        for (int i = 0; i < numComponents; i++) {
          components[i] /= 255.0F;
        }
      }
      // processing color using color space
      colorSpace = getColorSpace(colorSpaceName);
      if (colorSpace == null) {
        throw new TagAttributeException(locator(), "Illegal value '" + colorSpaceName + "' for attribute '" + 
            TagAttributes.ATTR_COLOR_SPACE + "', must be one of RGB, LINEAR_RGB, GRAY, CIEXYZ, PYCC or CMYK.");
      }
      if (numComponents != colorSpace.getNumComponents()) {
        throw new TagAttributeException(locator(), "Illegal value '" + strValue + "' for attribute '" + 
            TagAttributes.ATTR_VALUE + "', must define '" + colorSpace.getNumComponents() + " components for " +
            " color space " + colorSpaceName + ".");
      }
      // creating color
      color = new Color(colorSpace, components, 1.0F);
    }
    
    if (spotColor) {
      // defining color as SpotColor
      color = new SpotColor(new PdfSpotColor(name, 0, color), 1.0F);
    }
    
    // registering color at document handler
    documentHandler.registerColor(name, color);
  }
  
  /**
   * Gets the color space for the given name.
   * @param colorSpaceName The name of the color space to retrieve.
   * @return The according color space.
   * @throws TagAttributeException In case there was a problem in instantiating the color space. 
   */
  private ColorSpace getColorSpace(String colorSpaceName)
    throws TagAttributeException {
    
    Object colorSpaceObj = COLOR_SPACES.get(colorSpaceName);
    if (colorSpaceObj == null) {
      return null;
    }
    if (colorSpaceObj instanceof Integer) {
      // color space instance not loaded yet
      Integer colorSpaceId = (Integer) colorSpaceObj;
      synchronized (COLOR_SPACES) {
        try {
          ColorSpace cs = ColorSpace.getInstance(colorSpaceId.intValue());
          COLOR_SPACES.put(colorSpaceName, cs);
          return cs;
        } catch (IllegalArgumentException ex) {
          throw new TagAttributeException(locator(), "Unable to load profile for color space '" + 
              colorSpaceName + "', check if the according profile file '" + colorSpaceName +
              ".pf' is right in place (should be in directory 'cmm' of your JRE installation).");
        }
      }
    }
    return (ColorSpace) colorSpaceObj;
  }

}
