/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.ChildDefinition;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.CommonAttributesHolder;
import org.ujac.print.DocumentFont;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.ElementContainer;
import org.ujac.print.FontHolder;
import org.ujac.print.NewPageHandler;
import org.ujac.print.PhraseHolder;
import org.ujac.util.StringUtils;

import com.lowagie.text.Chunk;
import com.lowagie.text.Document;
import com.lowagie.text.DocumentException;
import com.lowagie.text.Element;
import com.lowagie.text.Image;
import com.lowagie.text.Paragraph;
import com.lowagie.text.Phrase;
import com.lowagie.text.Rectangle;

import com.lowagie.text.pdf.ColumnText;
import com.lowagie.text.pdf.MultiColumnText;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfPTable;
import com.lowagie.text.pdf.PdfWriter;

/**
 * Name: ColumnTextTag<br>
 * Description: A class handling &lt;column-text&gt; tags.
 * 
 * @author lauerc
 */
public class ColumnTextTag 
  extends BaseElementTag 
  implements PhraseHolder, ElementContainer, FontHolder, NewPageHandler {

  /** Definition of the 'columns' attribute. */
  private static final AttributeDefinition COLUMNS = new AttributeDefinition(TagAttributes.ATTR_COLUMNS, 
      AttributeDefinition.TYPE_INT, true, "The number of columns to use.");
  /** Definition of the 'height' attribute. */
  private static final AttributeDefinition HEIGHT = new AttributeDefinition(TagAttributes.ATTR_HEIGHT, 
      AttributeDefinition.TYPE_STRING, false, "The fixed height of the column text area.");

  /** Definition of the 'padding' attribute. */
  private static final AttributeDefinition PADDING =  
      CommonAttributes.PADDING.cloneAttrDef("The horizontal padding between the columns.");
  /** Definition of the 'padding' style attribute. */
  private static final AttributeDefinition PADDING_STYLE =  
      CommonStyleAttributes.PADDING.cloneAttrDef("The horizontal padding between the columns.");
  /** Definition of the 'halign' attribute. */
  private static final AttributeDefinition HORIZONTAL_ALIGNMENT =  
      CommonAttributes.HORIZONTAL_ALIGNMENT.cloneAttrDef("The horizontal alignment of the column contents.");
  /** Definition of the 'horizontal-align' style attribute. */
  private static final AttributeDefinition HORIZONTAL_ALIGNMENT_STYLE =  
      CommonStyleAttributes.HORIZONTAL_ALIGNMENT.cloneAttrDef("The horizontal alignment of the column contents.");

  /** The item's name. */
  public static final String TAG_NAME = "column-text";
  
  /** The column text element. */
  private MultiColumnText columnText = null;
  /** The number of columns to use. */
  private int numColumns = 0;
  /** The horizontal padding between the columns. */
  private float columnPadding = 0.0F;
  /** The horizontal alignment of the column contents. */
  private int columnAlignment = Element.ALIGN_JUSTIFIED;
  /** The fixed vertical leading distance. */
  private float leading = 0.0f;
  /** The line spacing attribute. */
  private float lineSpacing = 0.0f;
  /** The fixed height. */
  private float height = -1.0f;
  
  /** The number of added elements. */
  private int numElements = 0;
  
  /** The phrase. */
  private Phrase phrase = null;

  /**
   * Constructs a ColumnTextTag instance with no specific attributes.
   */
  public ColumnTextTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Splits the output area in several columns.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.CLASS)
      .addDefinition(CommonAttributes.STYLE)
      .addDefinition(CommonAttributes.FONT) 
      .addDefinition(COLUMNS)
      .addDefinition(PADDING)
      .addDefinition(HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonAttributes.LEADING)
      .addDefinition(CommonAttributes.LINE_SPACING) 
      .addDefinition(HEIGHT);
  }

  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.FONT_NAME) 
      .addDefinition(COLUMNS)
      .addDefinition(PADDING_STYLE)
      .addDefinition(HORIZONTAL_ALIGNMENT_STYLE)
      .addDefinition(CommonStyleAttributes.LEADING)
      .addDefinition(CommonStyleAttributes.LINE_SPACING) 
      .addDefinition(HEIGHT);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(BaseFontTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(IfTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(ElseTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(LogTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(SwitchTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ForeachTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(PhraseTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(ParagraphTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(SetFormatTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(SetPropertyTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(DefineTableTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(AddRowTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(AssertTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(AnchorTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(AnnotationTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ImageTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BarcodeTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(DatamatrixTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BarcodePdf417Tag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ImportTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(MacroTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(NewLineTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(NewPageTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(OutlineTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ListTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(PdfTableTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(TocTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(IndexTag.class, 0, ChildDefinition.UNLIMITED));
  }

  /**
   * Tells whether or not this tag does respect style attributes or not.
   * @return always <code>true</code>.
   */
  public boolean isStyleable() {
    return true;
  }

  /**
   * Tells whether textual content is allowed for the tag or not.
   * @return true if the item is allowed to contain textual content, else false.
   */
  public boolean isTextBodyAllowed() {
    return true;
  }
  
  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();
    
    if (!isValid()) {
      return;
    }

    CommonAttributesHolder commonAttributesHolder = getCommonAttributesHolder();
    if (isAttributeDefined(CommonAttributes.LEADING, CommonStyleAttributes.LEADING)) {
      this.leading = getDimensionAttribute(CommonAttributes.LEADING, true, CommonStyleAttributes.LEADING);
    } else {
      this.leading = commonAttributesHolder.getLeading();
    }    
    if (isAttributeDefined(CommonAttributes.LINE_SPACING, CommonStyleAttributes.LINE_SPACING)) {
      this.lineSpacing = getDimensionAttribute(CommonAttributes.LINE_SPACING, true, CommonStyleAttributes.LINE_SPACING);
    } else {
      this.lineSpacing = commonAttributesHolder.getLineSpacing();
    }
    this.height = getDimensionAttribute(HEIGHT, -1.0F, true, HEIGHT);
    
    this.numColumns = getIntegerAttribute(COLUMNS, true, null);
    this.columnPadding = getIntegerAttribute(CommonAttributes.PADDING, true, CommonStyleAttributes.PADDING);
    this.columnAlignment = getHorizontalAlignmentAttribute(CommonAttributes.HORIZONTAL_ALIGNMENT, Element.ALIGN_JUSTIFIED, true, CommonStyleAttributes.HORIZONTAL_ALIGNMENT);
    
    // setting up the MultiColumnText element.
    setupMultiColumnText();
  }

  /**
   * Sets up the MultiColumnText element.
   */
  private void setupMultiColumnText() {
    this.columnText = new MultiColumnText(height);
    // configuring the MultiColumnText
    PdfWriter pdfWriter = (PdfWriter) getDocumentHandler().getDocumentWriter();
    PdfContentByte cb = pdfWriter.getDirectContent();
    ColumnText confText = new ColumnText(cb);
    confText.setAlignment(this.columnAlignment);
    confText.setLeading(this.leading, this.lineSpacing);
    columnText.useColumnParams(confText);

    // defining columns
    Document document = documentHandler.getDocument();
    Rectangle ouputSize = document.getPageSize();
    float ouputWidth = ouputSize.getWidth() - document.leftMargin() - document.rightMargin();
    float columnWidth = (ouputWidth - (columnPadding * (numColumns - 1))) / numColumns;
    float hPos = document.left();
    for (int i = 0; i < numColumns; i++) {
      columnText.addSimpleColumn(hPos, hPos + columnWidth);
      hPos += columnWidth + columnPadding;
    }

    // resetting the element counter
    phrase = null;
    numElements = 0;
  }
  
  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while closing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    // closing the column text
    if (!isValid()) {
      return;
    }

    // adds the column text to the element container
    ElementContainer elementContainer = getElementContainer();
    elementContainer.addElement(this, columnText);
  }

  /**
   * Adds a element to the container.
   * @param item The document item.
   * @param element The element to be added.
   * @exception DocumentHandlerException If something went wrong when adding the element.
   */
  public void addElement(BaseDocumentTag item, Element element) throws DocumentHandlerException {
    if (element == null) {
      return;
    }

    // adding an existing phrase before adding the element
    addContentPhrase();
    
    numElements += 1;

    try {
      if (element instanceof Paragraph) {
        columnText.addElement(element);
      } else if (element instanceof Phrase) {
        columnText.addElement(new Paragraph((Phrase) element));
      } else if (element instanceof Image) {
        columnText.addElement(element);
      } else if (element instanceof com.lowagie.text.List) {
        columnText.addElement(element);
      } else if (element instanceof PdfPTable) {
        columnText.addElement(element);
      } else {
        throw new DocumentHandlerException(locator(), "ColumnTexts musten't contain elements other than " +
            "tables, phrases, images, lists or paragraphs.");
      }
    } catch (DocumentException ex) {
      throw new DocumentHandlerException(locator(), "Unable to add element '" + element.getClass().getName() + " <column-text> element.");
    }
  }

  /**
   * Tells whether the element container is a top level container or not.
   * @return true if the element container is a top level container, else false.
   */
  public boolean isTopLevel() {
    return true;
  }
  
  /**
   * Flushes the content buffer.
   * @exception DocumentHandlerException If something went wrong while flushing the content buffer.
   */
  public void flushContent() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    if (isEmptyContent()) {
      resetContent();
      return;
    }

    if (isTrimNextHead()) {
      trimContentHead();
      setTrimNextHead(false);
    }
    if (isItemClosed() && isTrimBody()) {
      trimContentTail();
    }

    if (phrase == null) {
      phrase = new Phrase(0.0F);
    }
    phrase = addChunk(phrase, getLeading(), getLineSpacing());
    resetContent();
  }
  

  /**
   * Adds the content phrase element to the PDF cell, 
   * it has been created and is not empty.
   * @exception DocumentHandlerException If the content could not be added to the column text.
   */
  private void addContentPhrase() throws DocumentHandlerException {
    try {
      if (phrase != null) {
        if (!phrase.isEmpty()) {
          numElements += 1;
          columnText.addElement(new Paragraph(phrase));
        }
        phrase = null;
      }
    } catch (DocumentException ex) {
      throw new DocumentHandlerException(locator(), "Unable to add content to the <colmn-text> element.");
    }
  }


  /**
   * Adds the given chunk to the root phrase.
   * @param item The document item.
   * @param chunk The chunk to add to the root phrase.
   * @param font The font to use.
   * @exception DocumentHandlerException In case the document processing failed.
   */
  public void addChunk(BaseDocumentTag item, String chunk, DocumentFont font) 
    throws DocumentHandlerException {
    
    if (!isValid()) {
      return;
    }
    if (phrase == null) {
      phrase = new Phrase(0.0F);
      if (isTrimBody()) {
        chunk = StringUtils.trimHead(chunk);
      }
    }
    addChunk(item, phrase, chunk, font, getLeading(), getLineSpacing());
  }

  /**
   * Adds the given chunk to the root phrase.
   * @param item The document item.
   * @param chunk The chunk to add to the root phrase.
   * @exception DocumentHandlerException In case the document processing failed.
   */
  public void addChunk(BaseDocumentTag item, Chunk chunk)
    throws DocumentHandlerException {

    if (!isValid()) {
      return;
    }
    if (phrase == null) {
      phrase = new Phrase(0.0F);
    }
    addChunk(phrase, chunk, getLeading(), getLineSpacing());
  }

  /**
   * Gets the fixed vertical leading distance.
   * @return The current fixed vertical leading distance.
   */
  public float getLeading() {
    return leading;
  }

  /**
   * Gets the line spacing.
   * @return The current line spacing.
   */
  public float getLineSpacing() {
    return lineSpacing;
  }
  
  /**
   * Handles a new page request.
   * @param size The new document size, null if the size should remain the same.
   * @param margins The new document margins, null if the margins should remain the same.
   * @return A reference to the document. 
   * @exception DocumentHandlerException In case something went wrong.
   */
  public Document handleNewPage(Rectangle size, Rectangle margins) 
    throws DocumentHandlerException {
    
    Document document = documentHandler.getDocument();
    if (!isValid()) {
      return document;
    }

    // adds the column text to the element container
    ElementContainer elementContainer = getElementContainer();
    elementContainer.addElement(this, columnText);
    
    // doing the new-page
    try {
      if (size != null) {
        document.setPageSize(size);
      }
      if (margins != null) {
        document.setMargins(
            margins.getBorderWidthLeft(), 
            margins.getBorderWidthRight(),
            margins.getBorderWidthTop(),
            margins.getBorderWidthBottom()
        );
      }
      document.newPage();
      // setting up the MultiColumnText element for the next fragment.
      setupMultiColumnText();
    } catch (Exception ex) {
      throw new DocumentHandlerException(locator(), "Unable to start a new page: " + ex.getMessage(), ex);
    }

    return document;
  }
  
}
