/*
 * Copyright (C) 2003 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
package org.ujac.print.tag;

import java.util.Collection;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.ujac.print.BaseDocumentTag;
import org.ujac.print.DocumentHandler;
import org.ujac.print.TagBuilder;
import org.ujac.print.TagInstantiationException;

/**
 * Name: CustomTagBuilder <br>
 * Description: TagBuilder implementation for the creation of custom tags. This
 * one creates the according original tag and attaches the specified attributes
 * to it. <br>
 * 
 * @author lauerc
 */
public class CustomTagBuilder implements TagBuilder {

  /** The name of the tag to call. */
  private String tagName;
  /** The alias name for the custom tag. */
  private String alias;
  /** The attributes to pass to the called tag. */
  private Map attributes;
  /** The alias set. */
  private Set aliases = null;

  /**
   * Constructs a CustomTagBuilder instance with specific attributes.
   * @param tagName The name of the tag to call.
   * @param alias The alias for the custom tag.
   * @param attributes The attributes to pass to the called tag.
   */
  public CustomTagBuilder(String tagName, String alias, Map attributes) {
    this.tagName = tagName;
    this.alias = alias;
    if (!attributes.containsKey(ATTR_TAG_NAME)) {
      attributes.put(ATTR_TAG_NAME, tagName);
    }
    this.attributes = attributes;
  }
  
  /**
   * @see org.ujac.print.TagBuilder#getName()
   */
  public String getName() {
    return alias;
  }
  
  /**
   * @see org.ujac.print.TagBuilder#createTag(org.ujac.print.DocumentHandler)
   */
  public BaseDocumentTag createTag(DocumentHandler documentHandler)
    throws TagInstantiationException {
    
    BaseDocumentTag tag = documentHandler.getTagFactory().createTagInstance(documentHandler, tagName);
    tag.setCustomAttributes(attributes);
    return tag;
  }

  /**
   * Registers an alias for this tag.
   * @param alias The alias to register.
   */
  public void registerAlias(String alias) {
    if (aliases == null) {
      aliases = new HashSet();
    }
    aliases.add(alias);
  }

  /**
   * Unregisters an alias for this tag.
   * @param alias The alias to unregister.
   */
  public void unregisterAlias(String alias) {
    if (aliases == null) {
      return;
    }
    aliases.remove(alias);
  }

  /**
   * Gets all registered aliases.
   * @return The registered aliases.
   */
  public Collection getAliases() {
    return aliases;
  }

}