/*
 * Copyright (C) 2003 - 2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import java.util.HashMap;
import java.util.Map;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.ChildDefinition;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.DocumentHandlerException;

/**
 * Name: CustomTagTag<br>
 * Description: Tag handler, defining custom tags for more convenience.
 * 
 * @author lauerc
 */
public class CustomTagTag
  extends BaseDocumentTag {

  /** Definition of the 'alias' attribute. */
  private static final AttributeDefinition ALIAS = new AttributeDefinition(TagAttributes.ATTR_ALIAS, AttributeDefinition.TYPE_NAME, true, 
      "The alias for the custom tag definition.");
  /** Definition of the 'tag' attribute. */
  private static final AttributeDefinition TAG = new AttributeDefinition(TagAttributes.ATTR_TAG, AttributeDefinition.TYPE_NAME, false, 
      "The name of the original tag.");

  /** The item's name. */
  public static final String TAG_NAME = "custom-tag";
  
  /** The name of the original tag to call. */
  private String tagName = null;
  /** The alias for the custom tag. */
  private String alias = null;

  /** The attribute map. */
  private Map attributes = null;

  /**
   * Constructs a CustomTagTag instance with no specific attributes.
   */
  public CustomTagTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Defines a custom document tag for more convenience and less typing.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(ALIAS)                    
      .addDefinition(TAG);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(AttributeTag.class, 0, ChildDefinition.UNLIMITED));
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // getting attributes
    this.alias = getStringAttribute(ALIAS, false, null);
    this.tagName = getStringAttribute(TAG, false, null);
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the image item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    // creating attribute map
    attributes = new HashMap();    
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the macro-param item.
   */
  public void closeItem() throws DocumentHandlerException {

    if (!isValid()) {
      return;
    }
   
    documentHandler.getTagFactory().registerTag(alias, new CustomTagBuilder(tagName, alias, attributes));
  }

  /**
   * Sets a attribute. 
   * @param name The attribute name.
   * @param value The attribute value.
   */
  public void setAttribute(String name, String value) {
    attributes.put(name, value);
  }

}
