/*
 * Copyright (C) 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import java.util.HashMap;
import java.util.Map;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.ElementContainer;
import org.ujac.print.PhraseHolder;
import org.ujac.print.TagAttributeException;

import org.ujac.util.exi.ExpressionException;

import com.lowagie.text.Chunk;
import com.lowagie.text.Image;
import com.lowagie.text.pdf.BarcodeDatamatrix;

/**
 * Name: DatamatrixTag<br>
 * Description: A class creating datamatrix codes.
 * 
 * @author lauerc
 */
public class DatamatrixTag extends BaseElementTag {

  /** Definition of the 'type' attribute. */
  private static final AttributeDefinition ENCODATION_TYPE = new AttributeDefinition(
      TagAttributes.ATTR_TYPE, new String[] {
          "AUTO", "ASCII", "B256", "C40", "TEXT", "EDIFACT", "X21", "RAW"
      }, true, "The encodation type.");
  /** Definition of the 'width' attribute. */
  private static final AttributeDefinition WIDTH = new AttributeDefinition(
      TagAttributes.ATTR_WIDTH, AttributeDefinition.TYPE_FLOAT, true, "The output width."
  );
  /** Definition of the 'height' attribute. */
  private static final AttributeDefinition HEIGHT = new AttributeDefinition(
      TagAttributes.ATTR_HEIGHT, AttributeDefinition.TYPE_FLOAT, true, "The output height."
  );
  /** Definition of the 'matrix-width' attribute. */
  private static final AttributeDefinition MATRIX_WIDTH = new AttributeDefinition(
      "matrix-width", AttributeDefinition.TYPE_FLOAT, false, "The datamatrix width."
  );
  /** Definition of the 'matrix-height' attribute. */
  private static final AttributeDefinition MATRIX_HEIGHT = new AttributeDefinition(
      "matrix-height", AttributeDefinition.TYPE_FLOAT, false, "The datamatrix height."
  );

  /** The item's name. */
  public static final String TAG_NAME = "datamatrix";
  
  /** The barcode type map. */
  private static Map barcodeTypes = null;
  
  static {
    // setting up barcode type map
    barcodeTypes = new HashMap();
    barcodeTypes.put("AUTO", new Integer(BarcodeDatamatrix.DM_AUTO));
    barcodeTypes.put("ASCII", new Integer(BarcodeDatamatrix.DM_ASCII));
    barcodeTypes.put("B256", new Integer(BarcodeDatamatrix.DM_B256));
    barcodeTypes.put("C40", new Integer(BarcodeDatamatrix.DM_C40));
    barcodeTypes.put("TEXT", new Integer(BarcodeDatamatrix.DM_TEXT));
    barcodeTypes.put("EDIFACT", new Integer(BarcodeDatamatrix.DM_EDIFACT));
    barcodeTypes.put("X21", new Integer(BarcodeDatamatrix.DM_X21));
    barcodeTypes.put("RAW", new Integer(BarcodeDatamatrix.DM_RAW));
  }
  
  /** The latest phrase holder. */
  private PhraseHolder phraseHolder = null;

  /** The barcode type attribute. */
  private String typeAttr = null;
  /** The image width attribute. */
  private float width = 0.0F;
  /** The image height attribute. */
  private float height = 0.0F;
  /** The datamatrix width attribute. */
  private int matrixWidth = 0;
  /** The datamatrix height attribute. */
  private int matrixHeight = 0;
  /** The padding size. */
  private int padding = 0;
  /** The rotate flag. */
  private boolean rotate = false;

  /**
   * Constructs a DatamatrixTag instance with no specific attributes.
   */
  public DatamatrixTag() {
    super(TAG_NAME);
  }
  
  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Prints a datmatrix code.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(ENCODATION_TYPE)
      .addDefinition(WIDTH)
      .addDefinition(HEIGHT)
      .addDefinition(CommonAttributes.PADDING)
      .addDefinition(CommonAttributes.ROTATE);
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(WIDTH)
      .addDefinition(HEIGHT)
      .addDefinition(CommonStyleAttributes.PADDING);
  }

  /**
   * Tells whether textual content is allowed for the tag or not.
   * @return true if the item is allowed to contain textual content, else false.
   */
  public boolean isTextBodyAllowed() {
    return true;
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // determining the latest phrase holder
    phraseHolder = documentHandler.latestPhraseHolder();
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }
  }

  /**
   * Processes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    // getting attributes
    this.typeAttr = getStringAttribute(ENCODATION_TYPE, true, null);
    this.width = getDimensionAttribute(WIDTH, true, WIDTH);
    this.height = getDimensionAttribute(HEIGHT, true, HEIGHT);
    this.matrixWidth = getIntegerAttribute(MATRIX_WIDTH, true, null);
    this.matrixHeight = getIntegerAttribute(MATRIX_HEIGHT, true, null);
    this.padding = getIntegerAttribute(CommonAttributes.PADDING, true, CommonStyleAttributes.PADDING);
    this.rotate = getBooleanAttribute(CommonAttributes.ROTATE, true, null);

    // evaluating the code
    String code = null;
    try {
      code = documentHandler.evalString(getTrimmedContent());
    } catch (ExpressionException ex) {
      throw new DocumentHandlerException(locator(), "Content evaluation failed: " + ex.getMessage(), ex);
    }

    String typeString = null;
    try {
      typeString = documentHandler.evalString(typeAttr).toUpperCase();
    } catch (ExpressionException ex) {
      throw new TagAttributeException(locator(), "Attribute evaluation failed: " + ex.getMessage(), ex);
    }
      
    Integer type = (Integer) barcodeTypes.get(typeString);
    if (type == null) {
      throw new TagAttributeException(locator(), "Unsupported datamatrix code type '" + typeString + "'.");
    }
    
    BarcodeDatamatrix dm = new BarcodeDatamatrix();
    dm.setOptions(type.intValue());
    if (matrixWidth > 0) {
      dm.setWidth(matrixWidth);
    }
    if (matrixHeight > 0) {
      dm.setHeight(matrixHeight);
    }
    dm.setWs(padding);
    
    ElementContainer elementContainer = getElementContainer();
    try {
      int result = dm.generate(code);
      switch (result) {
        case BarcodeDatamatrix.DM_ERROR_TEXT_TOO_BIG:
          throw new DocumentHandlerException(locator(), "Could not generate datamatrix code: The text is too big for the symbology capabilities.");
        case BarcodeDatamatrix.DM_ERROR_INVALID_SQUARE:
          throw new DocumentHandlerException(locator(), "Could not generate datamatrix code: The dimensions given for the symbol are illegal.");
        case BarcodeDatamatrix.DM_ERROR_EXTENSION:
          throw new DocumentHandlerException(locator(), "Could not generate datamatrix code: An error was while parsing an extension.");
      }
      Image barcodeImage = dm.createImage();
      if (rotate) {
        barcodeImage.setRotation((float) Math.PI/2);
      }
      barcodeImage.scaleAbsolute(width, height);
      if (phraseHolder != null) {
        phraseHolder.addChunk(this, new Chunk(barcodeImage, 0, 0));
      } else {
        elementContainer.addElement(this, barcodeImage);
      }
    } catch (DocumentHandlerException ex) {
      throw ex;
    } catch (Throwable ex) {
      throw new TagAttributeException(locator(), "Creation of barcode '" + typeAttr + "' failed: " + ex.getMessage(), ex);
    }
  }
}
