/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import java.text.Format;

import java.util.Date;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.PhraseHolder;
import org.ujac.print.TagAttributeException;

import org.ujac.util.exi.ExpressionException;
import org.ujac.util.text.FormatHelper;

/**
 * Title:       DateFormatTag<br>
 * Description: A class handling date-format items.<br>
 * 
 * @author lauerc
 */
public class DateFormatTag extends BaseDocumentTag {

  /** Definition of the 'format' attribute. */
  private static final AttributeDefinition FORMAT = new AttributeDefinition(TagAttributes.ATTR_FORMAT, 
      AttributeDefinition.TYPE_STRING, true, "The format string according to the definition of class " +
      "java.text.DecimalFormat.");
  /** Definition of the 'value' attribute. */
  private static final AttributeDefinition VALUE = new AttributeDefinition(TagAttributes.ATTR_VALUE, 
      AttributeDefinition.TYPE_STRING, false, "The value to be formatted");

  /** The item's name. */
  public static final String TAG_NAME = "date-format";
  
  /**
   * Constructs a DateFormatTag instance with no specific attributes.
   */
  public DateFormatTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Formats the given value using the specified number format. The given number format " +
      "must be specified according to the specifications of the SDK class java.text.DecimalFormat.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(FORMAT)
      .addDefinition(VALUE);
  }

  
  /** The phrase holder. */
  private PhraseHolder phraseHolder = null;
  /** The format attribute. */
  private String formatAttr = null;
  /** The number format. */
  private Format format = null;
  /** The value attribute. */
  private String valueAttr = null;

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // getting parents
    this.phraseHolder = documentHandler.latestPhraseHolder();
    if (phraseHolder == null) {
      throw new DocumentHandlerException(locator(), "Item '" + getName() + "' is not supported here.");
    }
    // getting attributes
    this.valueAttr = getStringAttribute(VALUE, false, null);
  }
  
  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    this.formatAttr = getStringAttribute(FORMAT, true, null);
    this.format = documentHandler.lookupFormat(formatAttr);
    FormatHelper formatHelper = documentHandler.getFormatHelper();
    if (format == null) {
      format = formatHelper.createDateFormat(formatAttr);
      documentHandler.registerFormat(formatAttr, format);
    }

    Object value = null;
    try {
      value = documentHandler.evalObject(valueAttr);
    } catch (ExpressionException ex) {
      throw new TagAttributeException(locator(), "Attribute evaluation failed: " + ex.getMessage(), ex);
    }
    
    if (!(value instanceof Date)) {
      throw new DocumentHandlerException(locator(), "Unable to format non-date values at date-format item.");
    }

    phraseHolder.addChunk(this, format.format(value), getFont());
  }
}