/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.ChildDefinition;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.TagAttributeException;
import org.ujac.util.table.DataTable;
import org.ujac.util.table.Table;
import org.ujac.util.table.TableException;
import org.ujac.util.table.TableHelper;

/**
 * Name: DefineTableTag<br>
 * Description: Class handling the &lt;define-table&gt; tag.
 * 
 * @author lauerc
 */
public class DefineTableTag extends BaseDocumentTag {

  /** The item's name. */
  public static final String TAG_NAME = "define-table";
  
  /** The property name. */
  private String name = null;
  /** The table. */
  private Table table = null;
    
  /**
   * Constructs a DefineTableTag instance with no specific attributes.
   */
  public DefineTableTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Defines a table and adds it to the document properties.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.TABLE_NAME);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(AddColumnTag.class, 0, ChildDefinition.UNLIMITED));
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
  }
  
  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    // getting attributes
    this.name = getStringAttribute(CommonAttributes.TABLE_NAME, false, null);

    // creating the table
    this.table = new DataTable(documentHandler.getFormatHelper());
  }

  /**
   * Closes the tag.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the macro-param item.
   */
  public void closeItem() throws DocumentHandlerException {

    if (!isValid()) {
      return;
    }
   
    // adding table to document properties
    documentHandler.getTemplateContext().setProperty(this.name, this.table);
  }
  
  /**
   * Defines a column at the current table.
   * @param name The column name.
   * @param type The column type.
   * @exception TagAttributeException Thrown in case the column definition failed.
   */
  public void defineColumn(String name, String type)
    throws TagAttributeException {
    
    try {
      int typeId = TableHelper.getColumnTypeId(type);
      this.table.addColumn(name, typeId);
    } catch (TableException ex) {
      throw new TagAttributeException(locator(), "Failed to define column '" + name + "': " + ex.getMessage(), ex);
    }
  }
  
}
