/*
 * Copyright (C) 2003 - 2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import java.awt.Color;

import java.lang.reflect.Field;

import java.sql.Time;
import java.sql.Timestamp;

import java.util.Date;

import com.lowagie.text.Element;
import com.lowagie.text.Document;
import com.lowagie.text.PageSize;
import com.lowagie.text.DocumentException;
import com.lowagie.text.Paragraph;

import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.PdfWriter;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.ChildDefinition;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.ElementContainer;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.JavascriptContainer;
import org.ujac.print.NewPageHandler;
import org.ujac.print.ParameterDefHolder;
import org.ujac.print.ParameterDefinition;
import org.ujac.print.StyleHolder;

import org.ujac.print.tag.acroform.BaseAcroFieldTag;
import org.ujac.print.tag.graphics.GraphicsTag;

import org.ujac.util.UjacTypes;
import org.ujac.util.template.TemplateContext;

/**
 * Name: DocumentTag<br>
 * Description: A class handling &lt;document&gt; tags.<br>
 * 
 * @author lauerc
 */
public class DocumentTag
  extends BaseDocumentTag 
  implements ElementContainer, JavascriptContainer, ParameterDefHolder, NewPageHandler, StyleHolder {

  /** The item's name. */
  public static final String TAG_NAME = "document";

  /** The iText document. */
  private Document document = null;
  /** The line spacing attribute. */
  private float lineSpacing = DEFAULT_LINE_SPACING;
  //Added by Kiran
  /** The pdf Version attribute. */
  private char pdfVersion = '4';

  /**
   * Constructs a DocumentTag instance with no specific attributes.
   */
  public DocumentTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Root element of the document.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.PAGE_SIZE)
      .addDefinition(CommonAttributes.PAGE_WIDTH)
      .addDefinition(CommonAttributes.PAGE_HEIGHT)
      .addDefinition(CommonAttributes.PAGE_ROTATE)
      .addDefinition(CommonAttributes.MARGIN_LEFT)
      .addDefinition(CommonAttributes.MARGIN_RIGHT)
      .addDefinition(CommonAttributes.MARGIN_TOP)
      .addDefinition(CommonAttributes.MARGIN_BOTTOM)
      .addDefinition(CommonAttributes.BACKGROUND_COLOR)
      .addDefinition(CommonAttributes.LINE_SPACING)
      .addDefinition(CommonAttributes.PDF_VERSION)
      .addDefinition(CommonAttributes.LOGICAL_PAGENUMBERING)
      .addDefinition(CommonAttributes.LOGICAL_PAGETEXT);
  }

  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.PAGE_SIZE)
      .addDefinition(CommonStyleAttributes.PAGE_WIDTH)
      .addDefinition(CommonStyleAttributes.PAGE_HEIGHT)
      .addDefinition(CommonStyleAttributes.PAGE_ROTATE)
      .addDefinition(CommonStyleAttributes.MARGIN_LEFT)
      .addDefinition(CommonStyleAttributes.MARGIN_RIGHT)
      .addDefinition(CommonStyleAttributes.MARGIN_TOP)
      .addDefinition(CommonStyleAttributes.MARGIN_BOTTOM)
      .addDefinition(CommonStyleAttributes.BACKGROUND_COLOR)
      .addDefinition(CommonStyleAttributes.LINE_SPACING);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(ViewerPreferencesTag.class, 0, 1))
      .addDefinition(new ChildDefinition(EncryptionTag.class, 0, 1))
      .addDefinition(new ChildDefinition(RegisterFontTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(RegisterTagTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(StyleDefTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(ColorDefTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(FontDefTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(ModuleTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(ParameterDefTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(CustomTagTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(MacroDefTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(HeadTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(PageFormatTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(HeaderTag.class, 0, 1)) 
      .addDefinition(new ChildDefinition(FooterTag.class, 0, 1)) 
      .addDefinition(new ChildDefinition(StickyTag.class, 0, 1)) 
      .addDefinition(new ChildDefinition(SegmentTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(BaseFontTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(IfTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(ElseTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(LogTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(SwitchTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(ForeachTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(TocTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(IndexTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(ChapterTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(PhraseTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(ParagraphTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(TableTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(PdfTableTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(PrintTableTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(SetFormatTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ResourceBundleTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(AnchorTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(AnnotationTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ImageTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(MetaTag.class, 0, 1))
      .addDefinition(new ChildDefinition(ListTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(OverlayTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BoxTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(SetPropertyTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(DefineTableTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(AddRowTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(AssertTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ImportTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(MacroTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(NewLineTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(NewPageTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(OutlineTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ColumnTextTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BaseAcroFieldTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(GraphicsTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BarcodeTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(DatamatrixTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BarcodePdf417Tag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(InsertDocumentTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(JavascriptTag.class, 0, ChildDefinition.UNLIMITED));
  }

  /**
   * Tells whether or not this tag does respect style attributes or not.
   * @return always <code>true</code>.
   */
  public boolean isStyleable() {
    return true;
  }

  /**
   * Is the item a possible document root tag, such as &lt;document&gt; or &lt;module&gt;.
   * @return true if the item is a possible root tag, else false.
   */
  public boolean isDocumentRoot() {
    return true;
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    Rectangle size = null;
    String sizeAttr = getStringAttribute(CommonAttributes.PAGE_SIZE, true, CommonStyleAttributes.PAGE_SIZE);
    if (sizeAttr != null) {
      try {
        Field sizeField = PageSize.class.getField(sizeAttr);
        size = (Rectangle) sizeField.get(null);
      } catch (Exception e) {
        throw new DocumentHandlerException(locator(), "Unable to set page size: " + e.getMessage(), e);
      }
    } else {
      if (isAttributeDefined(CommonAttributes.PAGE_WIDTH, CommonStyleAttributes.PAGE_WIDTH) && 
          isAttributeDefined(CommonAttributes.PAGE_HEIGHT, CommonStyleAttributes.PAGE_HEIGHT)) {
        float width = getDimensionAttribute(CommonAttributes.PAGE_WIDTH, true, CommonStyleAttributes.PAGE_WIDTH);
        float height = getDimensionAttribute(CommonAttributes.PAGE_HEIGHT, true, CommonStyleAttributes.PAGE_HEIGHT);
        size = new Rectangle(width, height);
      }
    }

    this.lineSpacing = getDimensionAttribute(CommonAttributes.LINE_SPACING, DEFAULT_LINE_SPACING, true, CommonStyleAttributes.LINE_SPACING);

    if (size != null) {
      String rotateAttr = getStringAttribute(CommonAttributes.ROTATE, true, CommonStyleAttributes.ROTATE);
      boolean rotate = false;
      if (rotateAttr != null) {
        rotate = new Boolean(rotateAttr).booleanValue(); 
      }
      
      Color bgColor = getColorAttribute(CommonAttributes.BACKGROUND_COLOR, true, CommonStyleAttributes.BACKGROUND_COLOR);
      
      float marginLeft = getDimensionAttribute(CommonAttributes.MARGIN_LEFT, true, CommonStyleAttributes.MARGIN_LEFT);
      float marginRight = getDimensionAttribute(CommonAttributes.MARGIN_RIGHT, true, CommonStyleAttributes.MARGIN_RIGHT);
      float marginTop = getDimensionAttribute(CommonAttributes.MARGIN_TOP, true, CommonStyleAttributes.MARGIN_TOP);
      float marginBottom = getDimensionAttribute(CommonAttributes.MARGIN_BOTTOM, true, CommonStyleAttributes.MARGIN_BOTTOM);
                
      //Adding 48 becoz the char '0' ascii value is 48.
      this.pdfVersion = (char) (this.getIntegerAttribute(CommonAttributes.PDF_VERSION, 4, true, null) + 48);

      if (rotate) {
        // rotating document size.
        size = size.rotate();
      }
      if (bgColor != null) {
        size.setBackgroundColor(bgColor);
      }

      if (isAttributeDefined(CommonAttributes.LOGICAL_PAGENUMBERING)) {
        // Gets the value for the Logical Page Numbering Attribute.
        int logicalPageNumberStyle = this.getIntegerAttribute(CommonAttributes.LOGICAL_PAGENUMBERING, true, null);
        // setting the page number style at document handler
        documentHandler.setLogicalPageNumberStyle(logicalPageNumberStyle);
      }
      
      if (isAttributeDefined(CommonAttributes.LOGICAL_PAGETEXT)) {
        // Gets the value for the Logical Page Numbering Text Attribute.
        String logicalPageNumberText = this.getStringAttribute(CommonAttributes.LOGICAL_PAGETEXT, "", true, null);
        // setting the page number text at document handler
        documentHandler.setLogicalPageNumberText(logicalPageNumberText);
      }

      // creating document
      document = new Document(size, marginLeft, marginRight, marginTop, marginBottom);
      // setting document at document handler
      documentHandler.setDocument(document);
      
      
      if (isAttributeDefined(CommonAttributes.PDF_VERSION)) {
        //Added by Kiran
        //This will set the Acrobat version for the generated PDF.
        ((PdfWriter) documentHandler.getDocumentWriter(false)).setPdfVersion(this.pdfVersion);
      }
    }
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while closing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (document != null) {
      document.close();
    }
  }

  /**
   * Adds a element to the container.
   * @param item The document item.
   * @param element The element to be added.
   * @exception DocumentHandlerException If something went wrong when adding the element.
   */
  public void addElement(BaseDocumentTag item, Element element) throws DocumentHandlerException {
    try {
      Document document = getDocument(true);
      // workaround for the missing multipliedLeading (aka line-spacing) support in iText
      if (item instanceof ParagraphTag) {
        ParagraphTag tag = (ParagraphTag) item;
        float leading = tag.getLeading();
        float lineSpacing = tag.getLineSpacing();
        extendLeading((Paragraph) element, item.getFont().getFont(), leading, lineSpacing);
      }
      document.add(element);
    } catch (DocumentException ex) {
      throw new DocumentHandlerException(locator(), "Unable to add element <" + item.getName() + "> to the " + getName() + ": " + ex.getMessage(), ex);
    }
  }

  /**
   * Tells whether the element container is a top level container or not.
   * @return true if the element container is a top level container, else false.
   */
  public boolean isTopLevel() {
    return true;
  }

  /**
   * Gets the line spacing.
   * @return The current line spacing.
   */
  public float getLineSpacing() {
    return lineSpacing;
  }

  /**
   * Gets the leading distance.
   * @return The current leading distance.
   */
  public float getLeading() {
    return DEFAULT_LEADING;
  }

  /**
   * Gets the iText document.
   * @return The iText document.
   */
  public Document getDocument() {
    return getDocument(true);
  }
  /**
   * Setter method for property document.
   * @param document The document to set.
   */
  protected void setDocument(Document document) {
    this.document = document;
  }

  /**
   * Gets the iText document.
   * @param openDocumentRequired True, if an open document is required, else false. 
   * @return The iText document.
   */
  public Document getDocument(boolean openDocumentRequired) {
    if (openDocumentRequired) {
      if (!document.isOpen()) {
        document.open();
      }
    }
    return document;
  }

  /**
   * Sets a parameter definition. 
   * @param parameterDef  The parameter definition.
   * @exception DocumentHandlerException In case the a problem was detected with the parameter definition.
   */
  public void setParameterDef(ParameterDefinition parameterDef) throws DocumentHandlerException {
    TemplateContext ctx = documentHandler.getTemplateContext();
    String paramName = parameterDef.getName();
    Object paramValue = ctx.getProperty(paramName);
    if (paramValue == null) {
      if (!parameterDef.isOptional()) {
        throw new DocumentHandlerException(locator(), "The mandatory document parameter '" + paramName + "' was not defined in the document properties.");
      }
      paramValue = parameterDef.getDefaultValue();
      if (paramValue != null) {
        // using default value
        ctx.setProperty(paramName, parameterDef.getDefaultValue());
      }
    } else {
      // checking parameter type
      String attr = (String) parameterDef.getDefaultValue();
      if (attr != null) {
        switch (parameterDef.getType()) {
          case UjacTypes.TYPE_BOOLEAN:
            if (!(paramValue instanceof Boolean)) {
              throw new DocumentHandlerException(locator(), "The document parameter '" + paramName + "' was declared as boolean but was given as instance of class '" + paramValue.getClass().getName() + "'.");
            }
            break;
    
          case UjacTypes.TYPE_INT:
            if (!(paramValue instanceof Number)) {
              throw new DocumentHandlerException(locator(), "The document parameter '" + paramName + "' was declared as int but was given as instance of class '" + paramValue.getClass().getName() + "'.");
            }
            break;
    
          case UjacTypes.TYPE_FLOAT:
            if (!(paramValue instanceof Number)) {
              throw new DocumentHandlerException(locator(), "The document parameter '" + paramName + "' was declared as float but was given as instance of class '" + paramValue.getClass().getName() + "'.");
            }
            break;
    
          case UjacTypes.TYPE_DOUBLE:
            if (!(paramValue instanceof Number)) {
              throw new DocumentHandlerException(locator(), "The document parameter '" + paramName + "' was declared as double but was given as instance of class '" + paramValue.getClass().getName() + "'.");
            }
            break;
    
          case UjacTypes.TYPE_DATE:
            if (!(paramValue instanceof Date)) {
              throw new DocumentHandlerException(locator(), "The document parameter '" + paramName + "' was declared as Date but was given as instance of class '" + paramValue.getClass().getName() + "'.");
            }
            break;

          case UjacTypes.TYPE_TIME:
            if (!(paramValue instanceof Date)) {
              throw new DocumentHandlerException(locator(), "The document parameter '" + paramName + "' was declared as Time but was given as instance of class '" + paramValue.getClass().getName() + "'.");
            }
            if (!(paramValue instanceof Time)) {
              paramValue = new Time(((Date) paramValue).getTime());
            }
            break;

          case UjacTypes.TYPE_TIMESTAMP:
            if (!(paramValue instanceof Date)) {
              throw new DocumentHandlerException(locator(), "The document parameter '" + paramName + "' was declared as Timestamp but was given as instance of class '" + paramValue.getClass().getName() + "'.");
            }
            if (!(paramValue instanceof Timestamp)) {
              paramValue = new Timestamp(((Timestamp) paramValue).getTime());
            }
            break;

          case UjacTypes.TYPE_STRING:
            if (!(paramValue instanceof String)) {
              throw new DocumentHandlerException(locator(), "The document parameter '" + paramName + "' was declared as String but was given as instance of class '" + paramValue.getClass().getName() + "'.");
            }
            break;

          case ParameterDefinition.TYPE_COLOR:
            if (!(paramValue instanceof Color) && !(paramValue instanceof String)) {
              throw new DocumentHandlerException(locator(), "The document parameter '" + paramName + "' was declared as Color but was given as instance of class '" + paramValue.getClass().getName() + "'.");
            }
            break;

          case ParameterDefinition.TYPE_FONT:
            if (!(paramValue instanceof String)) {
              throw new DocumentHandlerException(locator(), "The document parameter '" + paramName + "' was declared as Font but was given as instance of class '" + paramValue.getClass().getName() + "'.");
            }
            break;

          case ParameterDefinition.TYPE_FORMAT:
            if (!(paramValue instanceof String)) {
              throw new DocumentHandlerException(locator(), "The document parameter '" + paramName + "' was declared as Format but was given as instance of class '" + paramValue.getClass().getName() + "'.");
            }
            break;
        }
      }
    }
  }

  /**
   * Handles a new page request.
   * @param size The new document size, null if the size should remain the same.
   * @param margins The new document margins, null if the margins should remain the same.
   * @return A reference to the document. 
   * @exception DocumentHandlerException In case something went wrong.
   */
  public Document handleNewPage(Rectangle size, Rectangle margins) 
    throws DocumentHandlerException {
    
    try {
      if (size != null) {
        document.setPageSize(size);
      }
      if (margins != null) {
        document.setMargins(
            margins.getBorderWidthLeft(), 
            margins.getBorderWidthRight(),
            margins.getBorderWidthTop(),
            margins.getBorderWidthBottom()
        );
      }
      document.newPage();
    } catch (Exception ex) {
      throw new DocumentHandlerException(locator(), "Unable to start a new page: " + ex.getMessage(), ex);
    }
    return document;
  }

  /**
   * @see org.ujac.print.JavascriptContainer#addJavaScript(java.lang.String)
   */
  public void addJavaScript(String code) {
    PdfWriter pdfWriter = (PdfWriter) documentHandler.getDocumentWriter();
    pdfWriter.addJavaScript(code);
  }

}
