/*
 * Copyright (C) 2005 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.DocumentHandlerException;

import com.lowagie.text.DocWriter;
import com.lowagie.text.DocumentException;
import com.lowagie.text.pdf.PdfWriter;

/**
 * Name: EncryptionTag<br>
 * Description: A tag implementation defining the encrpytion for the generated document.
 * 
 * @author lauerc
 */
public class EncryptionTag extends BaseDocumentTag {

  /** Definition of the 'encryption-type' attribute. */
  private static final AttributeDefinition ENCRYPTION_TYPE = new AttributeDefinition(TagAttributes.ATTR_ENCRYPTION_TYPE,
      new String[] {"40bit", "128bit", "AES128"}, false, "Defines the encryption bit strength, defaults to '128bit'.");
  /** Definition of the 'owner-password' attribute. */
  private static final AttributeDefinition OWNER_PASSWORD = new AttributeDefinition(TagAttributes.ATTR_OWNER_PASSWORD, 
      AttributeDefinition.TYPE_STRING, false, "Defines the user password to be added to the document.");
  /** Definition of the 'user-password' attribute. */
  private static final AttributeDefinition USER_PASSWORD = new AttributeDefinition(TagAttributes.ATTR_USER_PASSWORD, 
      AttributeDefinition.TYPE_STRING, false, "Defines the owner password to be added to the document.");
  /** Definition of the 'allow-printing' attribute. */
  private static final AttributeDefinition ALLOW_PRINTING = new AttributeDefinition(TagAttributes.ATTR_ALLOW_PRINTING, 
      AttributeDefinition.TYPE_BOOLEAN, false, "Defines permission to print the document.");
  /** Definition of the 'allow-modify-contents' attribute. */
  private static final AttributeDefinition ALLOW_MODIFY_CONTENTS = new AttributeDefinition(TagAttributes.ATTR_ALLOW_MODIFY_CONTENTS, 
      AttributeDefinition.TYPE_BOOLEAN, false, "Defines permission to modify the contents of the document.");
  /** Definition of the 'allow-copy' attribute. */
  private static final AttributeDefinition ALLOW_COPY = new AttributeDefinition(TagAttributes.ATTR_ALLOW_COPY, 
      AttributeDefinition.TYPE_BOOLEAN, false, "Defines permission to copy the document.");
  /** Definition of the 'allow-modify-annotations' attribute. */
  private static final AttributeDefinition ALLOW_MODIFY_ANNOTATIONS = new AttributeDefinition(TagAttributes.ATTR_ALLOW_MODIFY_ANNOTATIONS, 
      AttributeDefinition.TYPE_BOOLEAN, false, "Defines permission to modify annotions of the document.");
  /** Definition of the 'allow-fillin' attribute. */
  private static final AttributeDefinition ALLOW_FILLIN = new AttributeDefinition(TagAttributes.ATTR_ALLOW_FILLIN, 
      AttributeDefinition.TYPE_BOOLEAN, false, "Defines permission to fill in the document as a form.");
  /** Definition of the 'allow-screen-readers' attribute. */
  private static final AttributeDefinition ALLOW_SCREEN_READERS = new AttributeDefinition(TagAttributes.ATTR_ALLOW_SCREEN_READERS, 
      AttributeDefinition.TYPE_BOOLEAN, false, "Defines permission to use screen readers with document.");
  /** Definition of the 'allow-assembly' attribute. */
  private static final AttributeDefinition ALLOW_ASSEMBLY = new AttributeDefinition(TagAttributes.ATTR_ALLOW_ASSEMBLY, 
      AttributeDefinition.TYPE_BOOLEAN, false, "Defines permission to do assembly with the document.");
  /** Definition of the 'allow-degraded-printing' attribute. */
  private static final AttributeDefinition ALLOW_DEGRADED_PRINTING = new AttributeDefinition(TagAttributes.ATTR_ALLOW_DEGRADED_PRINTING, 
      AttributeDefinition.TYPE_BOOLEAN, false, "Defines permission to print a degraded rendition of the document.");

  /** The item's name. */
  public static final String TAG_NAME = "encryption";

  /**
   * Constructs a EncryptionTag instance with no specific attributes.
   */
  public EncryptionTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Specifies the encryption for the generated document.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(ENCRYPTION_TYPE)
      .addDefinition(USER_PASSWORD)
      .addDefinition(OWNER_PASSWORD)
      .addDefinition(ALLOW_PRINTING)
      .addDefinition(ALLOW_MODIFY_CONTENTS)
      .addDefinition(ALLOW_COPY)
      .addDefinition(ALLOW_MODIFY_ANNOTATIONS)
      .addDefinition(ALLOW_FILLIN)
      .addDefinition(ALLOW_SCREEN_READERS)
      .addDefinition(ALLOW_ASSEMBLY)
      .addDefinition(ALLOW_DEGRADED_PRINTING);
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
  }
  
  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    // setting the document encryption
    DocWriter writer = getDocumentHandler().getDocumentWriter(false);
    if (!(writer instanceof PdfWriter)) {
      throw new DocumentHandlerException(locator(), "Encryption is only supported for PDF documents.", null);
    }
    PdfWriter pdfWriter = (PdfWriter) writer;
    
    int encryptionType = PdfWriter.STANDARD_ENCRYPTION_128;
    int encryptionPermissions = 0;
    String userPassword = null;
    String ownerPassword = null;

    if (isAttributeDefined(ENCRYPTION_TYPE)) {
      String attr = getStringAttribute(ENCRYPTION_TYPE, true, null);
      if ("40bit".equals(attr)) {
        encryptionType = PdfWriter.STANDARD_ENCRYPTION_40;
      } else if ("AES128".equals(attr)) {
        encryptionType = PdfWriter.ENCRYPTION_AES_128;
      }
    }
    if (isAttributeDefined(USER_PASSWORD)) {
      userPassword = getStringAttribute(USER_PASSWORD, false, null); 
    }
    if (isAttributeDefined(OWNER_PASSWORD)) {
      ownerPassword = getStringAttribute(OWNER_PASSWORD, false, null); 
    }
    if (getBooleanAttribute(ALLOW_PRINTING, false, true, null)) {
       encryptionPermissions |= PdfWriter.AllowPrinting;
    }
    if (getBooleanAttribute(ALLOW_MODIFY_CONTENTS, false, true, null)) {
      encryptionPermissions |= PdfWriter.AllowModifyContents;
    }
    if (getBooleanAttribute(ALLOW_COPY, false, true, null)) {
      encryptionPermissions |= PdfWriter.AllowCopy;
    }
    if (getBooleanAttribute(ALLOW_MODIFY_ANNOTATIONS, false, true, null)) {
      encryptionPermissions |= PdfWriter.AllowModifyAnnotations;
    }
    if (getBooleanAttribute(ALLOW_FILLIN, false, true, null)) {
      encryptionPermissions |= PdfWriter.AllowFillIn;
    }
    if (getBooleanAttribute(ALLOW_SCREEN_READERS, false, true, null)) {
      encryptionPermissions |= PdfWriter.AllowScreenReaders;
    }
    if (getBooleanAttribute(ALLOW_ASSEMBLY, false, true, null)) {
      encryptionPermissions |= PdfWriter.AllowAssembly;
    }
    if (getBooleanAttribute(ALLOW_DEGRADED_PRINTING, false, true, null)) {
      encryptionPermissions |= PdfWriter.AllowDegradedPrinting;
    }
    
    try {
      // finally setting the document encryption
      pdfWriter.setEncryption(
          userPassword != null ? userPassword.getBytes() : null, 
          ownerPassword != null ? ownerPassword.getBytes() : null,
          encryptionPermissions, encryptionType
      );
    } catch (DocumentException ex) {
      throw new DocumentHandlerException(locator(), "Unable to set document encryption: " + ex.getMessage(), ex);
    }
  }

}
