/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.ChildDefinition;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.Condition;
import org.ujac.print.DocumentFont;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.FontHolder;
import org.ujac.print.StyleHolder;

import org.ujac.util.table.Column;
import org.ujac.util.table.LayoutHints;
import org.ujac.util.table.Row;
import org.ujac.util.table.TableException;

import com.lowagie.text.Element;
import com.lowagie.text.Phrase;

import com.lowagie.text.pdf.PdfPCell;

/**
 * Name: ExtraRowTag<br>
 * Description: Implementation of the handler for a tag which adds a extra rows at the 
 *   surrounding 'print-row' tag.
 * 
 * @author lauerc
 */
public class ExtraRowTag
  extends BaseDocumentTag 
  implements FontHolder, Condition, RowCellContainer, StyleHolder {

  /** Definition of the 'direction' attribute. */
  private static final AttributeDefinition DIRECTION = new AttributeDefinition(TagAttributes.ATTR_DIRECTION, 
      AttributeDefinition.TYPE_NAME, false, "The insert direction: 'before' | 'after'.");
  /** Definition of the 'rotate' attribute. */
  private static final AttributeDefinition CELL_ROTATE = 
      CommonAttributes.ROTATE.cloneAttrDef("The rotation angle for all cells of the row.");
  
  /** The item's name. */
  public static final String TAG_NAME = "extra-row";
  
  /** The surrounding print-table tag. */
  protected PrintTableTag rowContainer = null;
  /** The surrounding row-cell tag. */
  private PrintRowTag printRowTag = null;
  /** The current cell attributes. */
  private CellAttributes cellAttributes = null;
  
  /** The rotation angle. */
  protected float rotate = 0.0f;
  
  /** The cell style item map. */
  protected RowCellTag[] cellStyles = null;

  /**
   * Constructs a ExtraRowTag instance with no specific attributes.
   */
  public ExtraRowTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Adds an extra row to the surrounding &lt;print-row&gt; tag.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.RENDERED)
      .addDefinition(CommonAttributes.CLASS) 
      .addDefinition(CommonAttributes.STYLE) 
      .addDefinition(CommonAttributes.FONT)
      .addDefinition(DIRECTION)
      .addDefinition(CommonAttributes.LEADING)
      .addDefinition(CommonAttributes.LINE_SPACING)
      .addDefinition(CommonAttributes.BACKGROUND_COLOR)
      .addDefinition(CommonAttributes.BORDER_STYLE)
      .addDefinition(CommonAttributes.BORDER_WIDTH)
      .addDefinition(CommonAttributes.BORDER_COLOR)
      .addDefinition(CommonAttributes.VERTICAL_ALIGNMENT)
      .addDefinition(CommonAttributes.PADDING)
      .addDefinition(CommonAttributes.PADDING_LEFT)
      .addDefinition(CommonAttributes.PADDING_RIGHT)
      .addDefinition(CommonAttributes.PADDING_TOP)
      .addDefinition(CommonAttributes.PADDING_BOTTOM) 
      .addDefinition(CommonAttributes.NO_WRAP)
      .addDefinition(CommonAttributes.CELL_HEIGHT)
      .addDefinition(CommonAttributes.ROTATE)
      .addDefinition(CELL_ROTATE);
  }
  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.FONT_NAME)
      .addDefinition(CommonStyleAttributes.LEADING)
      .addDefinition(CommonStyleAttributes.LINE_SPACING)
      .addDefinition(CommonStyleAttributes.BACKGROUND_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_STYLE)
      .addDefinition(CommonStyleAttributes.BORDER_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_LEFT_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_RIGHT_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_TOP_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_BOTTOM_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_LEFT_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_RIGHT_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_TOP_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_BOTTOM_COLOR)
      .addDefinition(CommonStyleAttributes.HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.VERTICAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.PADDING)
      .addDefinition(CommonStyleAttributes.PADDING_LEFT)
      .addDefinition(CommonStyleAttributes.PADDING_RIGHT)
      .addDefinition(CommonStyleAttributes.PADDING_TOP)
      .addDefinition(CommonStyleAttributes.PADDING_BOTTOM)
      .addDefinition(CommonStyleAttributes.CELL_HEIGHT)
      .addDefinition(CommonStyleAttributes.ROTATE)
      .addDefinition(CommonStyleAttributes.NO_WRAP);
  }
  
  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(IfTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(ElseTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(LogTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(SwitchTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BaseFontTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(MacroTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(RowCellTag.class, 0, ChildDefinition.UNLIMITED));
  }
  
  /**
   * Tells whether or not this tag does respect style attributes or not.
   * @return always <code>true</code>.
   */
  public boolean isStyleable() {
    return true;
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // setting the cell container.
    rowContainer = (PrintTableTag) documentHandler.latestOfType(PrintTableTag.class);
    printRowTag = (PrintRowTag) documentHandler.latestOfType(PrintRowTag.class);
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the foreach item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }
    
    // inherriting the surrounding cell's attributes
    cellAttributes = new CellAttributes(getCommonAttributesHolder());
    cellAttributes.parseRowAttributes(this, rowContainer.getCellAttributes());
    
    rotate = getFloatAttribute(CommonAttributes.ROTATE, 0.0f, true, CommonStyleAttributes.ROTATE);
  
    cellStyles = new RowCellTag[rowContainer.getNumColumns()];
  }
  
  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while closing the foreach item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    String strDirection = getStringAttribute(DIRECTION, "after", true, DIRECTION).toLowerCase();

    int direction = PrintTableTag.DIRECTION_AFTER;
    if ("before".equals(strDirection)) {
      direction = PrintTableTag.DIRECTION_BEFORE;
    }

    // inserting column into table
    printRowTag.insertExtraRow(this, direction);
  }

  /**
   * Performs the row output.
   * @exception DocumentHandlerException Thrown in case something went wrong while closing the foreach item.
   */
  public void outputRow() throws DocumentHandlerException {

    Row currentRow = (Row) rowContainer.getCurrentValue();

    try {
      // creating the cells
      Column[] columns = rowContainer.getVisibleColumns();
      for (int i = 0; i < columns.length; i++) {
        Column column = columns[i];
        
        int columnIdx = column.getIndex();
        
        int colspan = 1;
        boolean fillRow = false;
        CellAttributes attrs = (CellAttributes) cellAttributes.clone();
        int hAlign = -1;
        if (columnIdx != -1) {
          switch (currentRow.getAlign(columnIdx)) {
          case LayoutHints.LEFT:
            hAlign = Element.ALIGN_LEFT;
            break;
          case LayoutHints.RIGHT:
            hAlign = Element.ALIGN_RIGHT;
            break;
          case LayoutHints.CENTER:
            hAlign = Element.ALIGN_CENTER;
            break;
          default: // assuming left alignment
            hAlign = Element.ALIGN_LEFT;
            break;
          }
        }
        attrs.setHAlign(hAlign);
        DocumentFont font = getFont();
        
        PdfPCell cell = null;
        RowCellTag rowCell = cellStyles[i]; 

        if (rowCell != null) {
          CellAttributes rowCellAttributes = rowCell.getCellAttributes();
          attrs.merge(rowCellAttributes);
          if (rowCell.getCell() != null) {
            cell = rowCell.getCell();
            colspan = cell.getColspan();
            fillRow = rowCellAttributes.isFillRow();
            if (fillRow) {
              colspan = rowContainer.getRemainingPdfColumns();
            }
            attrs.setColspan(colspan);
            attrs.setupCell(cell);
          } else {
            colspan = rowCellAttributes.getColspan();
            fillRow = rowCellAttributes.isFillRow();
            if (fillRow) {
              colspan = rowContainer.getRemainingPdfColumns();
            }
            font = rowCell.getFont();
            rotate = rowCell.getRotate();
          }
        }

        if (cell == null) {
          Phrase cellPhrase = new Phrase("", font.getFont());
          if (rotate != 0.0F) {
            cellPhrase = rotatePhrase(cellPhrase, rotate);
          }
          cell = new PdfPCell(cellPhrase);
        
          // configuring the cell
          attrs.setLeading(0.0F);
          attrs.setColspan(colspan);
          attrs.setupCell(cell);
        }
        if (colspan > 1) {
          // colspan has been defined, skipping the spanned columns
          i += colspan - 1;
        }
        // adding cell to latest cell container
        if (isHeaderRow()) {
          rowContainer.addHeaderCell(this, cell);
        } else {
          rowContainer.addCell(this, cell);
        }
      }
    } catch (TableException ex) {
      throw new DocumentHandlerException(locator(), "Failed to access table cell: " + ex.getMessage(), ex);
    }
  }

  /**
   * Registers the given cell style for the specified column name.
   * @param rowCell The row cell to register.
   */
  public void registerRowCell(RowCellTag rowCell) {
    int index = rowCell.getColumnIndex();
    if (index < 0) {
      index = rowContainer.getColumnIndex(rowCell.getColumnName());
    }
    cellStyles[index] = rowCell;
  }

  /**
   * @see org.ujac.print.Condition#isTrue()
   */
  public boolean isTrue() {
    return condition != null ? condition.isTrue() && printRowTag.isTrue() : printRowTag.isTrue();
  }

  /**
   * Tells whether the row is a header row or not.
   * @return true in case the row is a header row, else false.
   */
  protected boolean isHeaderRow() {
    return printRowTag.isHeaderRow();
  }

  /**
   * Gets the cell attributes.
   * @return The cell attributes.
   */
  public CellAttributes getCellAttributes() {
    return cellAttributes;
  }

}
