/*
 * Copyright (C) 2003 - 2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.DocumentHandlerException;

import com.lowagie.text.Font;

/**
 * Name: FontTag<br>
 * Description: A class handling font items.<br>
 * 
 * @author lauerc
 */
public class FontTag extends BaseFontTag {

  /** The item's name. */
  public static final String TAG_NAME = "font";
  
  /** Definition of the 'name' attribute. */
  public static final AttributeDefinition NAME = new AttributeDefinition(TagAttributes.ATTR_NAME, 
      AttributeDefinition.TYPE_NAME, false, "The font definition name, use this in case you want to make use " +
      "of predefined fonts (see font-def item).").addAlias(TagAttributes.ATTR_FONT_DEF);
  
  /**
   * Constructs a FontTag instance with no specific attributes.
   */
  public FontTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Sets the current font for the document. You can either hardcode a font here, or use a " +
      " predefined font definition (see 'font-def' item)." ;
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(NAME)
      .addDefinition(CommonAttributes.FONT_COLOR)
      .addDefinition(CommonAttributes.FONT_FAMILY)
      .addDefinition(CommonAttributes.FONT_ENCODING)
      .addDefinition(CommonAttributes.FONT_SIZE)
      .addDefinition(CommonAttributes.FONT_STYLE)
      .addDefinition(CommonAttributes.TEXT_RISE)
      .addDefinition(CommonAttributes.STYLE);
  }

  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.FONT_NAME)
      .addDefinition(CommonStyleAttributes.FONT_COLOR)
      .addDefinition(CommonStyleAttributes.FONT_FAMILY)
      .addDefinition(CommonStyleAttributes.FONT_ENCODING)
      .addDefinition(CommonStyleAttributes.FONT_SIZE)
      .addDefinition(CommonStyleAttributes.FONT_STYLE)
      .addDefinition(CommonStyleAttributes.TEXT_RISE);
  }

  /**
   * Is the item virtual container, such as if, else or foreach.
   * @return true if the item is such a container, else false.
   */
  public boolean isVirtualContainer() {
    return true;
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    // getting attributes
    String fontDef = getStringAttribute(NAME, true, NAME);
    if (fontDef != null) {
      font = documentHandler.getFont(fontDef);
      return;
    }
    color = getColorAttribute(CommonAttributes.FONT_COLOR, true, CommonStyleAttributes.FONT_COLOR);
    String familyAttribute = getStringAttribute(CommonAttributes.FONT_FAMILY, true, CommonStyleAttributes.FONT_FAMILY);
    if (familyAttribute != null) {
      family = familyAttribute;
    }
    String encodingAttribute = getStringAttribute(CommonAttributes.FONT_ENCODING, true, CommonStyleAttributes.FONT_ENCODING);
    if (encodingAttribute != null) {
      encoding = encodingAttribute;
    }
    if (isAttributeDefined(CommonAttributes.FONT_SIZE, CommonStyleAttributes.FONT_SIZE)) {
      size = getDimensionAttribute(CommonAttributes.FONT_SIZE, true, CommonStyleAttributes.FONT_SIZE);
    }
    String styleAttribute = getStringAttribute(CommonAttributes.FONT_STYLE, true, CommonStyleAttributes.FONT_STYLE);
    if (styleAttribute != null) {
      style = Font.getStyleValue(styleAttribute);
    } else {
      // for compatibility with old style handling
      styleAttribute = getStringAttribute(CommonAttributes.STYLE, true, null);
      if (styleAttribute != null) {
        style = Font.getStyleValue(styleAttribute);
      }
    }
    if (isAttributeDefined(CommonAttributes.TEXT_RISE, CommonStyleAttributes.TEXT_RISE)) {
      textRise = getDimensionAttribute(CommonAttributes.TEXT_RISE, true, CommonStyleAttributes.TEXT_RISE);
    }

    // configuring font
    font = createFont(family, encoding, size, style, textRise, color);
  }

  /**
   * Handles the font attributes.
   * @throws DocumentHandlerException
   */
  protected void handleFontAttributes() throws DocumentHandlerException {
  }

}
