/*
 * Copyright (C) 2003 - 2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.ChildDefinition;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.DocumentHandlerException;

import com.lowagie.text.Document;
import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfWriter;

/**
 * Name: FooterTag<br>
 * Description: A class handling footer items.<br>
 * 
 * @author lauerc
 */
public class FooterTag extends HeaderFooterTag {

  /** The item's name. */
  public static final String TAG_NAME = "footer";
  
  /** Definition of the 'padding-top' attribute. */
  private static final AttributeDefinition PADDING_TOP =
      CommonStyleAttributes.PADDING_TOP.cloneAttrDef("The padding between the horizontal ruler line and the content");
  /** Definition of the 'padding-top' style attribute. */
  private static final AttributeDefinition PADDING_TOP_STYLE =
      CommonStyleAttributes.PADDING_TOP.cloneAttrDef("The padding between the horizontal ruler line and the content");
  
  /** The top padding to the document's content area. */
  private float paddingTop = DEFAULT_RULE_PADDING;

  /**
   * Constructs a FooterTag instance with no specific attributes.
   */
  public FooterTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Defines the footer for the document.";
  }

  /**
   * @see org.ujac.print.BaseDocumentTag#buildSupportedAttributes()
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.CLASS)
      .addDefinition(CommonAttributes.STYLE)
      .addDefinition(CommonAttributes.FONT)
      .addDefinition(RULE_WIDTH)
      .addDefinition(RULE_COLOR)
      .addDefinition(PADDING_TOP);
  }
  /**
   * @see org.ujac.print.BaseDocumentTag#buildSupportedStyleAttributes()
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.FONT_NAME)
      .addDefinition(RULE_WIDTH)
      .addDefinition(RULE_COLOR)
      .addDefinition(PADDING_TOP_STYLE);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(IfTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ElseTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(LogTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(SwitchTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BaseFontTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(MacroTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(FooterPartTag.class, 0, ChildDefinition.UNLIMITED)); 
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    this.paddingTop = getDimensionAttribute(CommonAttributes.PADDING_TOP, DEFAULT_RULE_PADDING, true, CommonStyleAttributes.PADDING_TOP);
  }

  /**
   * Draws the horizontal rule line.
   * @param writer The PDF writer.
   * @param document The document.
   */
  protected void drawRule(PdfWriter writer, Document document) {
    float ruleWidth = getRuleWidth();
    if (ruleWidth <= 0.0F) {
      return;
    }
    
    Rectangle pageSize = document.getPageSize();
    float left = document.left();
    float right = document.right();
    float bottom = pageSize.getBottom() + document.bottomMargin() - paddingTop;
    PdfContentByte cb = writer.getDirectContent();
    
    cb.saveState();
    cb.setLineWidth(ruleWidth);
    cb.setColorStroke(getRuleColor());
    cb.moveTo(left, bottom);
    cb.lineTo(right, bottom);
    cb.closePathStroke();
    cb.restoreState();
  }

  /**
   * Getter method for the the property paddingTop.
   * @return The current value of property paddingTop.
   */
  public float getPaddingTop() {
    return paddingTop;
  }

}
