/*
 * Copyright (C) 2003, 2004 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.ElementContainer;

import com.lowagie.text.Element;

/**
 * Name: HeadTag<br>
 * Description: Handler class for tag &lt;head&gt;. <br>
 *   This tag defines an area, which is recorded during the main
 *   document processing for later execution at a document, which
 *   will get inserted at the top of the main document.
 * 
 * @author lauerc
 */
public class HeadTag extends ConditionTag {

  /** The item's name. */
  public static final String TAG_NAME = "head";

  /**
   * Constructs a HeadTag instance with no specific attributes.
   */
  public HeadTag() {
    super(TAG_NAME);
  }


  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return null;
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Defines a document macro block. This macro can be used in serveral parts of the " +
      "document using the 'macro' item.";
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    
    // defining name of the macro definition
    //this.name = "head_definition";
  }
  
  /**
   * In case the document handler's <code>includeHeaderSection</code> is set, 
   * the head will be executed here. 
   * @see org.ujac.print.BaseDocumentTag#closeItem()
   */
  public void closeItem() throws DocumentHandlerException {
    if (documentHandler.isIncludeHeadSection()) {
      try {
        // starting a new page
        documentHandler.getDocument().newPage();
      } catch (Exception ex) {
        throw new DocumentHandlerException(locator(), "Failed to start new page.");
      }
    } else {
      super.closeItem();
    }
  }
  
  /**
   * @see org.ujac.print.Condition#isTrue()
   */
  public boolean isTrue() {
    return documentHandler.isIncludeHeadSection();
  }

  /**
   * Adds a element to the container.
   * @param item The document item.
   * @param element The element to be added.
   * @exception DocumentHandlerException If something went wrong when adding the element.
   */
  public void addElement(BaseDocumentTag item, Element element) throws DocumentHandlerException {
    if (documentHandler.isIncludeHeadSection()) {
      ((ElementContainer) parentItem).addElement(item, element);
    }
  }

}
