/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import java.awt.Color;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.ChildDefinition;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.Condition;
import org.ujac.print.DocumentEvent;
import org.ujac.print.DocumentEventRecorder;
import org.ujac.print.DocumentFont;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.ElementContainer;
import org.ujac.print.PagePosition;
import org.ujac.print.PhraseHolder;
import org.ujac.print.StyleHolder;
import org.ujac.print.TagAttributeException;

import org.ujac.util.StringUtils;
import org.ujac.util.template.TemplateException;

import com.lowagie.text.Chunk;
import com.lowagie.text.Document;
import com.lowagie.text.Element;
import com.lowagie.text.Phrase;
import com.lowagie.text.Rectangle;

import com.lowagie.text.pdf.PdfPCell;
import com.lowagie.text.pdf.PdfPTable;
import com.lowagie.text.pdf.PdfWriter;
import com.lowagie.text.pdf.PdfTemplate;
import com.lowagie.text.pdf.PdfContentByte;

/**
 * Name: HeaderFooterPartTag.java<br>
 * Description: Base class for header and footer part tags.<br>
 * 
 * @author lauerc
 */
public abstract class HeaderFooterPartTag 
  extends BaseDocumentTag 
  implements PhraseHolder, ElementContainer, DocumentEventRecorder, Condition, StyleHolder {

  /** Declaration of specific attribute 'x' */
  public static final AttributeDefinition X = new AttributeDefinition(TagAttributes.ATTR_X, AttributeDefinition.TYPE_DIMENSION, false,
      "The fixed horizontal position of this part. In case you've specified " +
      "a negative value, the part will be placed relative to the right " +
      "page border");
  /** Declaration of specific attribute 'width' */
  public static final AttributeDefinition WIDTH = new AttributeDefinition(TagAttributes.ATTR_WIDTH, AttributeDefinition.TYPE_DIMENSION, false,
      "The fixed width of this part.");

  /** The fixed horizontal position of this part. */
  protected PagePosition pos = null;
  /** The fixed width of this part. */
  protected float width = 0.0F;

  /** The nested table.  */
  protected PdfPTable nestedTable = null;

  /** The parent document event recorder. */
  private DocumentEventRecorder parentEventRecorder = null;

  /** Flag which rules, whether the nested items should be added to the document or not. */
  private boolean insertItems = false;  
  /** The document event list. */
  private List documentEvents = new ArrayList();
  
  /**
   * Title:       ContentPart<br/>
   * Description: The class holding the parts of the content.<br/> 
   */
  class ContentPart {
    /** Type for the phrase item. */
    public static final int TYPE_PHRASE = 1;
    /** Type for the page number item. */
    public static final int TYPE_PAGE_NUMBER = 2;
    /** Type for the number of pages item. */
    public static final int TYPE_NUMBER_OF_PAGES = 3;

    /** The content part type. */
    private int type;
    /** The content part. */
    private Phrase contentPhrase = null;

    /**
     * Constructs a ContentPart instance with specific attributes.
     * @param type The content type.
     */
    public ContentPart(int type) {
      this.type = type;
    }

    /**
     * Constructs a ContentPart instance with specific attributes.
     * @param contentPhrase The phrase holding the content.
     */
    public ContentPart(Phrase contentPhrase) {
      this.type = TYPE_PHRASE;
      this.contentPhrase = contentPhrase;
    }

    /**
     * Gets the type.
     * @return The content part type.
     */
    public int getType() {
      return type;
    }

    /**
     * Gets the content.
     * @return The content.
     */
    public Phrase getContent() {
      return contentPhrase;
    }
  }
  /**
   * Constructs a HeaderFooterPartTag instance with specific attributes.
   * @param name The tag name.
   */
  public HeaderFooterPartTag(String name) {
    super(name);
  }
  
  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.RENDERED);
  }

  /**
   * @see org.ujac.print.BaseDocumentTag#buildSupportedChilds()
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(IfTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ElseTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(LogTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(SwitchTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ForeachTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BaseFontTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(MacroTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(NumberFormatTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(DateFormatTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ImageTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BarcodeTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(DatamatrixTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BarcodePdf417Tag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(PdfTableTag.class, 0, 1));
  }

  /** The phrase that holds the content. */
  private Phrase phrase = null;

  /** The horizontal alignment. */
  protected int hAlign = 0;
  /** The item list. */
  protected List items = null;

  /**
   * Tells whether or not this tag does respect style attributes or not.
   * @return always <code>true</code>.
   */
  public boolean isStyleable() {
    return true;
  }

  /**
   * Tells whether textual content is allowed for the tag or not.
   * @return true if the item is allowed to contain textual content, else false.
   */
  public boolean isTextBodyAllowed() {
    return true;
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    this.parentEventRecorder = documentHandler.topEventRecorder();
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    documentHandler.pushEventRecorder(this);

    if (!isValid()) {
      return;
    }
    
    insertItems = false;
    
    // getting attributes
    if (isAttributeDefined(X)) {
      float xPos = getDimensionAttribute(X, true, X);
      this.pos = new PagePosition(xPos, 0.0F);
    }
    this.width = getDimensionAttribute(WIDTH, 0.0F, true, WIDTH);

    // creating item list
    this.items = new ArrayList();
    this.nestedTable = null;
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while closing the document item.
   */
  public void closeItem() throws DocumentHandlerException {

    documentHandler.popEventRecorder();

    if (!isValid()) {
      return;
    }

    // parsing the item's content
    flushContent();

    if (this.phrase != null) {
      items.add(new ContentPart(this.phrase));
      this.phrase = null;
    }

    if ((nestedTable != null) && (width <= 0.0F)) {
      throw new TagAttributeException(locator(), "You need to define a fixed width if nested tables are used within your header or footer part.");
    }
  }


  /**
   * Gets the vertical position for the header/footer part for the current page of the given document.
   * @param document The document to place the header/footer part on.
   * @return The calculated position.
   */
  protected abstract float getVerticalPosition(Document document);

  /**
   * Clones the given phrase.
   * @param phrase The phrase to clone.
   * @return The cloned phrase.
   */
  private Phrase clonePhrase(Phrase phrase) {
    try {
      if (phrase == null) {
        return null;
      }
      Phrase clone = null;
      List chunks = phrase.getChunks();
      int numChunks = chunks.size();
      for (int i = 0; i < numChunks; i++) {
        Chunk chunk = (Chunk) chunks.get(i);
        Chunk clonedChunk = null;
        Map attribs = chunk.getAttributes();
        if ((attribs != null) && (attribs.get(IMAGE_CHUNK_ATTR) != null)) {
          // chunk holds an image!
          clonedChunk = chunk;
        } else {
          clonedChunk = new Chunk(documentHandler.executeTemplate(chunk.getContent()), chunk.getFont());
        }
        if (clone == null) {
          clone = new Phrase(clonedChunk);
        } else {
          clone.add(clonedChunk);
        }
      }
      return clone;
    } catch (TemplateException ex) {
      throw new RuntimeException("Content evaluation failed: " + ex.getMessage());
    }
  }

  /** 
   * We override the onEndPage method.
   * @param writer The document writer.
   * @param document The iText document.
   */
  public void onEndPage(PdfWriter writer, Document document) {
    int pageNumber = documentHandler.getPageNumber(writer);
    
    float verticalPosition = getVerticalPosition(document);

    boolean addNumberOfPages = false;
    
    // removing old content parts again
    this.phrase = null;
    this.items.clear();

    try {
      insertItems = true;
      repeatChildItems();
    } catch (DocumentHandlerException ex) {
      log.error(getName() + " output failed:" + ex.getMessage(), ex);
      throw new RuntimeException(getName() + " output failed:" + ex.getMessage());
    }

    DocumentFont font = getFont();
    float textWidth = 0.0F;
    Phrase contentPhrase = null;
    int numItems = items.size();
    for (int i = 0; i < numItems; i++) {
      ContentPart item = (ContentPart) items.get(i);
      switch (item.getType()) {
        case ContentPart.TYPE_PHRASE:
          Phrase tmpPhrase = clonePhrase(item.getContent());
          textWidth += getPhraseWidth(tmpPhrase);
          if (contentPhrase == null) {
            contentPhrase = tmpPhrase;
          } else if (tmpPhrase != null) {
            contentPhrase.add(tmpPhrase);
          }
          break;
        case ContentPart.TYPE_PAGE_NUMBER:
          Phrase pageNumberItem = new Phrase(Integer.toString(pageNumber), font.getFont());
          textWidth += getPhraseWidth(pageNumberItem);
          if (contentPhrase == null) {
            contentPhrase = pageNumberItem;
          } else {
            contentPhrase.add(pageNumberItem);
          }
          break;
        case ContentPart.TYPE_NUMBER_OF_PAGES:
          addNumberOfPages = true;
          break;
      }
    }

    PdfContentByte cb = writer.getDirectContent();
    PdfTemplate numberOfPagesTemplate = null;
    float numberOfPagesWidth = 0.0F; 
    float totalWidth = textWidth;
    if (addNumberOfPages) {
      numberOfPagesTemplate = ensureNumberOfPagesTemplate(font, cb);
      numberOfPagesWidth = numberOfPagesTemplate.getWidth();
      totalWidth += numberOfPagesWidth;
    }

    float effectiveWidth = totalWidth;
    if (nestedTable != null) {
      effectiveWidth = width;
    }
    
    float horizontalPosition = -1.0f;
    if (pos == null) {
      switch (hAlign) {
        case Element.ALIGN_LEFT:
          horizontalPosition = document.left();
          break;
        case Element.ALIGN_RIGHT:
          horizontalPosition = document.right() - effectiveWidth;
          break;
        case Element.ALIGN_CENTER:
          horizontalPosition = document.left() + ((document.right() - document.left()) / 2.0f) - (effectiveWidth / 2.0f);
          break;
        default:
          horizontalPosition = document.left();
          break;
      }
    } else {
      Rectangle pageSize = document.getPageSize();
      horizontalPosition = pos.calcX(pageSize.getRight(), width);
    }
    
    Color bgColor = getBackgroundColor();
    PdfPTable table = new PdfPTable(1);

    PdfPCell contentCell = null;
    if (nestedTable == null) {
      contentCell = new PdfPCell(contentPhrase);
      contentCell.setPadding(0.0F);
      contentCell.setBorder(0);
      contentCell.setBorderWidth(0.0F);
      if ((pos != null) && (width >= 0.0F)) {
        contentCell.setHorizontalAlignment(hAlign);
      }
      if (bgColor != null) {
        contentCell.setBackgroundColor(bgColor);
      }
      table.addCell(contentCell);
      table.setTotalWidth(Math.max(effectiveWidth + 1, width));
    } else {
      contentCell = new PdfPCell(nestedTable);
      contentCell.setPadding(0.0F);
      contentCell.setBorder(0);
      contentCell.setBorderWidth(0.0F);
      if ((pos != null) && (width >= 0.0F)) {
        contentCell.setHorizontalAlignment(hAlign);
      }
      if (bgColor != null) {
        contentCell.setBackgroundColor(bgColor);
      }
      table.addCell(contentCell);
      table.setTotalWidth(width);
    }
    table.writeSelectedRows(0, 1, horizontalPosition, verticalPosition + table.getRowHeight(0), cb);

    if (addNumberOfPages) {
      cb.addTemplate(numberOfPagesTemplate, horizontalPosition + textWidth, verticalPosition - 1);
    }
  }

  /**
   * We override the onCloseDocument method.
   * @param writer The document writer.
   * @param document The iText document.
   */
  public void onCloseDocument(PdfWriter writer, Document document) {
  }

  /**
   * Parses the item's content.
   * @param content The content to parse.
   * @exception DocumentHandlerException In case the pasrsing of the content has been failed.
   */
  protected void parseContent(String content) throws DocumentHandlerException {
   
    if (content == null) {
      return;
    }
    
    if (this.phrase != null) {
      items.add(new ContentPart(this.phrase));
      this.phrase = null;
    }
    
    char[] contentChars = content.toCharArray();
    char c = 0;
    int itemStart = 0;
    for (int i = 0; i < contentChars.length; i++) {
      c = contentChars[i];
      if (c == '$') {
        int startIdx = i;
        ++i;
        c = contentChars[i];
        if (c == '{') {
          if ((i - itemStart) > 2) {
            String phraseText = new String(contentChars, itemStart, i - itemStart - 1);
            if (phraseText.length() > 0) {
              items.add(new ContentPart(addChunk(this, null, phraseText, getFont(), getLeading(), getLineSpacing())));
              itemStart = i - 1;
            }
          }
          for (; i < contentChars.length; i++) {
            c = contentChars[i];
            if (c == '}') {
              String exprToken = new String(contentChars, startIdx, i - startIdx + 1);
              if ("${pageNumber}".equals(exprToken)) {
                // adding content part
                items.add(new ContentPart(ContentPart.TYPE_PAGE_NUMBER));
                // setting new start index
                itemStart = i + 1;
              } else if ("${pageCount}".equals(exprToken)) {
                // adding content part
                items.add(new ContentPart(ContentPart.TYPE_NUMBER_OF_PAGES));
                // setting new start index
                itemStart = i + 1;
              }
              break;
            }
          }
        }
      }
    }
    if (itemStart < contentChars.length) {
      // adding final phrase text
      String lastPhraseText = new String(contentChars, itemStart, contentChars.length - itemStart);
      items.add(new ContentPart(addChunk(this, null, lastPhraseText, getFont(), getLeading(), getLineSpacing())));
    }
  }

  /**
   * Flushes the content buffer.
   * @exception DocumentHandlerException If something went wrong while flushing the content buffer.
   */
  public void flushContent() throws DocumentHandlerException {
    if (!isTrue()) {
      return;
    }

    if (isItemClosed()) {
      trimContentTail();
    }

    parseContent(getContent());
        
    resetContent();
  }
  
  /**
   * Adds the given chunk to the root phrase.
   * @param item The document item.
   * @param chunk The chunk to add to the root phrase.
   * @param font The font to use.
   * @exception DocumentHandlerException In case the document processing failed.
   */
  public void addChunk(BaseDocumentTag item, String chunk, DocumentFont font) 
    throws DocumentHandlerException {
    
    if (!isTrue()) {
      return;
    }
    if (phrase == null) {
      this.phrase = new Phrase(0.0F);
      if (isTrimBody()) {
        chunk = StringUtils.trimHead(chunk);
      }
    }
    addChunk(item, phrase, chunk, font, getLeading(), getLineSpacing());
    items.add(new ContentPart(phrase));
    this.phrase = null;
  }

  /**
   * Adds the given chunk to the root phrase.
   * @param item The document item.
   * @param chunk The chunk to add to the root phrase.
   */
  public void addChunk(BaseDocumentTag item, Chunk chunk) {
    if (!isTrue()) {
      return;
    }
    if (phrase == null) {
      phrase = new Phrase(0.0F);
    }
    addChunk(phrase, chunk, getLeading(), getLineSpacing());
    items.add(new ContentPart(phrase));
    this.phrase = null;
  }

  /**
   * Gets the fixed vertical leading distance.
   * @return The current fixed vertical leading distance.
   */
  public float getLeading() {
    return DEFAULT_LEADING;
  }

  /**
   * Gets the line spacing.
   * @return The current line spacing.
   */
  public abstract float getLineSpacing();

  /**
   * Adds a element to the container.
   * @param item The document item.
   * @param element The element to be added.
   * @exception DocumentHandlerException If something went wrong when adding the element.
   */
  public void addElement(BaseDocumentTag item, Element element) throws DocumentHandlerException {
    if (!isTrue()) {
      return;
    }
    if (!(element instanceof PdfPTable)) {
      throw new DocumentHandlerException(locator(), "Only tables are allowed here.");
    }
    if (width == 0.0F) {
      throw new TagAttributeException(
          locator(), 
          "You need to define the absolute 'width' for <" + getName() + 
          "> tags with nested <table> elements."); 
    }
    nestedTable = (PdfPTable) element;
  }

  /**
   * Tells whether the element container is a top level container or not.
   * @return true if the element container is a top level container, else false.
   */
  public boolean isTopLevel() {
    return false;
  }


  /**
   * Repeating the child items of the foreach item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening ot closing a child item.
   */
  protected void repeatChildItems() throws DocumentHandlerException {

    this.setItemClosed(false);
    int numEvents = documentEvents.size();
    for (int i = 0; i < numEvents; i++) {
      DocumentEvent event = (DocumentEvent) documentEvents.get(i);
      BaseDocumentTag item = event.getItem();
      if (i == numEvents -1) {
        this.setItemClosed(true);
      }
      switch (event.getType()) {
        case DocumentEvent.OPEN:
          // opening item
          BaseDocumentTag topItem = item.getParentItem();
          if (topItem != null) {
            topItem.flushContent();
          }
          documentHandler.pushItem(item);
          item.openItem();
          
          break;
        case DocumentEvent.CHARACTERS:
          String content = event.getContent();
          if (content.length() > 0) {
            item.setItemOpened(false);
            item.addContent(content);
            if (i == (numEvents - 1)) {
              // flushing content of last item
              item.flushContent();
            }
          }
          break;
        case DocumentEvent.CLOSE:
          // closing item
          item.setItemClosed(true);
          item.flushContent();
          item.closeItem();
          documentHandler.popItem();
          break;
          
        default:
          throw new DocumentHandlerException(locator(), "Encountered illegal event type '" + event.getType() + "!");
      }
    }
  }

  /**
   * @see org.ujac.print.DocumentEventRecorder#recordEvent(org.ujac.print.DocumentEvent)
   */
  public void recordEvent(DocumentEvent event) {
    if (event.getItem() != this) {
      documentEvents.add(event);
    } else if (event.getType() == DocumentEvent.CHARACTERS) {
      documentEvents.add(event);
      resetContent();
    } else if (parentEventRecorder != null) {
      parentEventRecorder.recordEvent(event);
    }
  }

  /**
   * Checks whether the condition is true or not.
   * @return true, if the condition evaluates to true, else false.
   */
  public boolean isTrue() {
    return insertItems;
  }

}
