/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import java.awt.Color;

import java.util.List;
import java.util.ArrayList;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.DocumentFont;
import org.ujac.print.DocumentHandler;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.DocumentPageEventHandler;
import org.ujac.print.ElementContainer;
import org.ujac.print.FontHolder;
import org.ujac.print.StyleHolder;

import com.lowagie.text.DocWriter;
import com.lowagie.text.Document;

import com.lowagie.text.pdf.PdfWriter;

/**
 * Name: HeaderFooterTag<br>
 * Description: Base class for header and footer tags.<br>
 * 
 * @author lauerc
 */
public abstract class HeaderFooterTag extends BaseElementTag implements FontHolder, StyleHolder {

  /** The width of the horizontal ruler line. */
  public static final float DEFAULT_RULE_PADDING = 2.0F;

  public static final AttributeDefinition RULE_WIDTH = new AttributeDefinition(TagAttributes.ATTR_RULE_WIDTH, AttributeDefinition.TYPE_DIMENSION, false,
      "The width of the horizontal ruler line");
  public static final AttributeDefinition RULE_COLOR = new AttributeDefinition(TagAttributes.ATTR_RULE_COLOR, AttributeDefinition.TYPE_COLOR, false,
      "The color of the horizontal ruler line");

  /** The width of the horizontal ruler line. */
  private float ruleWidth = 0.0F;
  /** The color of the horizontal ruler line. */
  private Color ruleColor = null;
  
  /**
   * Name: PageEventHandler<br>
   * Description: The page event handler, handling the header/footer line output.<br>
   */
  class PageEventHandler extends DocumentPageEventHandler {
    
    /**
     * We override the onEndPage method.
     * @param writer The document writer.
     * @param document The iText document.
     */
    public void onEndPage(PdfWriter writer, Document document) {
      DocumentHandler documentHandler = getDocumentHandler();
      Document oldDocument = documentHandler.getDocument();
      DocWriter oldWriter = documentHandler.setDocumentAndWriter(document, writer);
      try {
        int numItems = headerFooterParts.size();
        for (int i = 0; i < numItems; i++) {
          HeaderFooterPartTag tag = (HeaderFooterPartTag) headerFooterParts.get(i);
          tag.onEndPage(writer, document);
        }
        drawRule(writer, document);
      } finally {
        documentHandler.setDocumentAndWriter(oldDocument, oldWriter);
      }
    }
    
    /**
     * We override the onCloseDocument method.
     * @param writer The document writer.
     * @param document The iText document.
     */
    public void onCloseDocument(PdfWriter writer, Document document) {
      int numItems = headerFooterParts.size();
      for (int i = 0; i < numItems; i++) {
        HeaderFooterPartTag tag = (HeaderFooterPartTag) headerFooterParts.get(i);
        tag.onCloseDocument(writer, document);
      }
      drawRule(writer, document);
    }
  }

  /** The new-page tag this tag may be nested within. */
  protected NewPageTag newPageTag = null;
  /** The vertical position of the header/footer line. */ 
  protected List headerFooterParts = new ArrayList();
 
  /**
   * Constructs a HeaderFooterTag instance with specific attributes.
   * @param name The tag name.
   */
  public HeaderFooterTag(String name) {
    super(name);
  }
  
  /**
   * Tells whether or not this tag does respect style attributes or not.
   * @return always <code>true</code>.
   */
  public boolean isStyleable() {
    return true;
  }

  /**
   * Adds the given part to the header/footer part list.
   * @param part The part which to add.
   */
  public void addPart(HeaderFooterPartTag part) {
    headerFooterParts.add(part);
  }
  
  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // determining the supprounding tags
    this.newPageTag = (NewPageTag) documentHandler.latestOfType(NewPageTag.TAG_NAME);
  }
  
  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();
    
    // resetting header/footer parts
    headerFooterParts.clear();

    if (!isValid()) {
      return;
    }

    // adding page event handler
    if (isInNewPageTag()) {
      newPageTag.queuePageEventHandler(getName(), new PageEventHandler());
    } else {
      documentHandler.setPageEventHandler(getName(), new PageEventHandler());
    }

    ruleWidth = getDimensionAttribute(RULE_WIDTH, 0.0F, true, RULE_WIDTH);
    ruleColor = getColorAttribute(RULE_COLOR, Color.black, true, RULE_COLOR);
  }

  /**
   * Gets the font.
   * @return The item's font.
   */
  public DocumentFont getFont() {
    return super.getFont();
  }

  /**
   * @return Returns the ruleWidth.
   */
  public float getRuleWidth() {
    return ruleWidth;
  }

  /**
   * @return Returns the ruleColor.
   */
  public Color getRuleColor() {
    return ruleColor;
  }
  
  /**
   * Draws the horizontal rule line.
   * @param writer The PDF writer.
   * @param document The document.
   */
  protected abstract void drawRule(PdfWriter writer, Document document);

  /**
   * Tells whether this tag is embedded within a new-page tag or not.
   * @return true if a surrounding new-page tag exists, else false.
   */
  protected boolean isInNewPageTag() {
    return this.newPageTag != null;
  }

  /**
   * Gets the line spacing.
   * @return The current line spacing.
   */
  public float getLineSpacing() {
    ElementContainer elementContainer = getElementContainer();
    return elementContainer.getLineSpacing();
  }

}
