/*
 * Copyright (C) 2003 - 2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.DocumentHandlerException;

import com.lowagie.text.Document;

/**
 * Name: HeaderPartTag.java<br>
 * Description: Class handling &lt;header-part&gt; tags.<br>
 * 
 * @author lauerc
 */
public class HeaderPartTag extends HeaderFooterPartTag {

  /** The item's name. */
  public static final String TAG_NAME = "header-part";

  /** The default padding to the document's content area. */
  private static final float DEFAULT_PADDING = 3.0f; 
  
  /** The bottom padding to the document's content area. */
  private float paddingBottom = DEFAULT_PADDING;

  /** The parent tag. */
  protected HeaderTag headerTag = null;

  /**
   * Constructs a HeaderPartTag instance with specific attributes.
   */
  public HeaderPartTag() {
    super(TAG_NAME);
  }
  
  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Defines a part of the surrounding header.";
  }

  /**
   * @see org.ujac.print.BaseDocumentTag#buildSupportedAttributes()
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.CLASS)
      .addDefinition(CommonAttributes.STYLE)
      .addDefinition(CommonAttributes.FONT)
      .addDefinition(CommonAttributes.HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonAttributes.PADDING_BOTTOM)
      .addDefinition(X)
      .addDefinition(WIDTH)
      .addDefinition(CommonAttributes.BACKGROUND_COLOR);
  }
  /**
   * @see org.ujac.print.BaseDocumentTag#buildSupportedStyleAttributes()
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.FONT_NAME)
      .addDefinition(CommonStyleAttributes.HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.PADDING_BOTTOM)
      .addDefinition(X)
      .addDefinition(WIDTH)
      .addDefinition(CommonStyleAttributes.BACKGROUND_COLOR);
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // setting the conditions
    headerTag = (HeaderTag) documentHandler.latestOfType(HeaderTag.class);
    if (headerTag == null) {
      throw new DocumentHandlerException(locator(), "The header-part tag must be located below a header tag!");
    }
  }
  
  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the tag.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }
    
    // setting the attributes.
    hAlign = getHorizontalAlignmentAttribute(CommonAttributes.HORIZONTAL_ALIGNMENT, true, CommonStyleAttributes.HORIZONTAL_ALIGNMENT);
    // adding page event handler
    headerTag.addPart(this);
    // setting the vertical position
    paddingBottom = getDimensionAttribute(CommonAttributes.PADDING_BOTTOM, DEFAULT_PADDING, true, CommonStyleAttributes.PADDING_BOTTOM);
  }
  
  /**
   * Gets the vertical position for the header part for the current page of the given document.
   * @param document The document to place the header part on.
   * @return The calculated position.
   */
  protected float getVerticalPosition(Document document) {
    return document.getPageSize().getTop() - document.topMargin() + paddingBottom + headerTag.getPaddingBottom();
  }

  /**
   * Gets the line spacing.
   * @return The current line spacing.
   */
  public float getLineSpacing() {
    return headerTag.getLineSpacing();
  }

}
