/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.DocumentPageEventHandler;
import org.ujac.print.ElementContainer;
import org.ujac.print.PagePosition;
import org.ujac.print.PhraseHolder;

import com.lowagie.text.Chunk;
import com.lowagie.text.Element;
import com.lowagie.text.Image;
import com.lowagie.text.Document;
import com.lowagie.text.DocumentException;
import com.lowagie.text.Rectangle;

import com.lowagie.text.pdf.PdfWriter;
import com.lowagie.text.pdf.PdfContentByte;

/**
 * Name: ImageTag<br>
 * Description: A class handling image items.<br>
 * 
 * @author lauerc
 */
public class ImageTag extends BaseImageTag {

  /** The item's name. */
  public static final String TAG_NAME = "image";
   
  /**
   * Name: ImagePageEventHandler<br>
   * Description: The page event handler, handling the output of absolutely positioned images.
   */
  class ImagePageEventHandler extends DocumentPageEventHandler {
    
    /** The image to display. */
    private Image image = null;
    /** Tells whether to draw the image in the foreground or the background. */
    private boolean foreground = true;
    
    /**
     * Constructs a ImagePageEventHandler instance with specific attributes.
     * @param image The image to display.
     * @param foreground Tells whether to draw the image in the foreground or the background.
     */
    public ImagePageEventHandler(Image image, boolean foreground) {
      this.image = image;
      this.foreground = foreground;
    }
    
    /**
     * We override the onEndPage method.
     * @param writer The document writer.
     * @param document The iText document.
     */
    public void onEndPage(PdfWriter writer, Document document) {
      insertImage(writer, document, image, foreground);
    }
    
    /**
     * We override the onCloseDocument method.
     * @param writer The document writer.
     * @param document The iText document.
     */
    public void onCloseDocument(PdfWriter writer, Document document) {
    }

    /**
     * Tells whether this event handler is temporary, and should be removed after the 
     * first usage or not.
     * @return true, if this handler should be executed only once, else false.
     */
    public boolean isTemporary() {
      return true;
    }
  }

  /** The surrounding phrase holder. */
  private PhraseHolder phraseHolder = null;
  
  /** The horizontal position. */
  private PagePosition pos = null;
  /** The scale-to-fit flag. */
  private boolean scaleToFit = false;
  /** The rotation angle. */
  private float rotate = 0;
  /** The image's horizontal alignment. */
  private int hAlign = 0;
  /** The image's vertical alignment. */
  private int vAlign = 0;
  
  /** The surrounding &lt;sticky&gt; tag. */
  private StickyTag stickyTag = null;
  
  /**
   * Constructs a ImageTag instance with no specific attributes.
   */
  public ImageTag() {
    super(TAG_NAME);
  }
  /**
   * Constructs a ImageTag instance with specific attributes.
   * @param name The tag name.
   */
  public ImageTag(String name) {
    super(name);
  }


  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Inserts a image into the document. Images may be places absolutely on the page or " +
      "relative to its parent item. The image gets loaded by the resource loader, which is " +
      "specified at the document handler.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.CLASS)
      .addDefinition(CommonAttributes.STYLE)
      .addDefinition(CommonAttributes.IMAGE_SOURCE)
      .addDefinition(CommonAttributes.IMAGE_X)
      .addDefinition(CommonAttributes.IMAGE_Y)
      .addDefinition(CommonAttributes.IMAGE_WIDTH)
      .addDefinition(CommonAttributes.IMAGE_HEIGHT)
      .addDefinition(CommonAttributes.SCALE_TO_FIT)
      .addDefinition(CommonAttributes.IMAGE_ROTATE)
      .addDefinition(CommonAttributes.IMAGE_PAGE)
      .addDefinition(CommonAttributes.IMAGE_HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonAttributes.IMAGE_VERTICAL_ALIGNMENT)
      .addDefinition(CommonAttributes.IMAGE_TEXTWRAP)
      .addDefinition(CommonAttributes.IMAGE_UNDERLYING)
      .addDefinition(CommonAttributes.IMAGE_DRAW_FOREGROUND);
  }
  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.IMAGE_SOURCE)
      .addDefinition(CommonStyleAttributes.IMAGE_X)
      .addDefinition(CommonStyleAttributes.IMAGE_Y)
      .addDefinition(CommonStyleAttributes.IMAGE_WIDTH)
      .addDefinition(CommonStyleAttributes.IMAGE_HEIGHT)
      .addDefinition(CommonStyleAttributes.SCALE_TO_FIT)
      .addDefinition(CommonStyleAttributes.IMAGE_ROTATE)
      .addDefinition(CommonStyleAttributes.IMAGE_HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.IMAGE_VERTICAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.IMAGE_TEXTWRAP)
      .addDefinition(CommonStyleAttributes.IMAGE_UNDERLYING)
      .addDefinition(CommonStyleAttributes.IMAGE_DRAW_FOREGROUND);
  }

  /**
   * Gets the image's page position.
   * @return The image's page position.
   */
  public PagePosition getPosition() {
    return pos;
  }

  /**
   * Gets the image width.
   * @return The image width.
   */
  public float getWidth() {
    return width;
  }
  
  /**
   * Gets the image height.
   * @return The image height.
   */
  public float getHeight() {
    return height;
  }
  
  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    
    // determining the surrounding phrase holder
    phraseHolder = documentHandler.latestPhraseHolder();
    // determining the optional surrounding sticky tag
    stickyTag = (StickyTag) documentHandler.latestOfType(StickyTag.TAG_NAME);
  }
  
  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the image item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    // setting the attributes
    float x = getDimensionAttribute(CommonAttributes.IMAGE_X, true, CommonStyleAttributes.IMAGE_X);
    float y = getDimensionAttribute(CommonAttributes.IMAGE_Y, true, CommonStyleAttributes.IMAGE_Y);
    pos = new PagePosition(x, y);
    width = getDimensionAttribute(CommonAttributes.IMAGE_WIDTH, true, CommonStyleAttributes.IMAGE_WIDTH);
    height = getDimensionAttribute(CommonAttributes.IMAGE_HEIGHT, true, CommonStyleAttributes.IMAGE_HEIGHT);
    if (isAttributeSupported(CommonAttributes.SCALE_TO_FIT)) {
      scaleToFit = getBooleanAttribute(CommonAttributes.SCALE_TO_FIT, false, true, CommonStyleAttributes.SCALE_TO_FIT);
    }
    rotate = getFloatAttribute(CommonAttributes.IMAGE_ROTATE, true, CommonStyleAttributes.IMAGE_ROTATE);
    if (isAttributeSupported(CommonAttributes.IMAGE_SOURCE)) {
      sourceAttr = getStringAttribute(CommonAttributes.IMAGE_SOURCE, false, CommonStyleAttributes.IMAGE_SOURCE);
    }
    if (isAttributeSupported(CommonStyleAttributes.IMAGE_PAGE)) {
      pageAttr = getStringAttribute(CommonStyleAttributes.IMAGE_PAGE, true, CommonStyleAttributes.IMAGE_PAGE);
    }

    if (isAttributeDefined(CommonAttributes.IMAGE_HORIZONTAL_ALIGNMENT, CommonStyleAttributes.IMAGE_HORIZONTAL_ALIGNMENT)) {
      hAlign = getHorizontalAlignmentAttribute(CommonAttributes.IMAGE_HORIZONTAL_ALIGNMENT, true, CommonStyleAttributes.IMAGE_HORIZONTAL_ALIGNMENT);
    }
    if (isAttributeDefined(CommonAttributes.IMAGE_VERTICAL_ALIGNMENT, CommonStyleAttributes.IMAGE_VERTICAL_ALIGNMENT)) {
      vAlign = getVerticalAlignmentAttribute(CommonAttributes.IMAGE_VERTICAL_ALIGNMENT, true, CommonStyleAttributes.IMAGE_VERTICAL_ALIGNMENT);
    }
    if (getBooleanAttribute(CommonAttributes.IMAGE_TEXTWRAP, true, CommonStyleAttributes.IMAGE_TEXTWRAP)) {
      hAlign |= Image.TEXTWRAP;
    }
    if (getBooleanAttribute(CommonAttributes.IMAGE_UNDERLYING, true, CommonStyleAttributes.IMAGE_UNDERLYING)) {
      hAlign |= Image.UNDERLYING;
    }
  }

  /**
   * Processes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    Image image = getImage();
    
    if (image == null) {
      return;
    }
    
    float width = this.width;
    float height = this.height;
    if ((width > 0) && (height > 0)) {
      if (scaleToFit) {
        image.scaleToFit(width, height);
      } else {
        image.scaleAbsolute(width, height);
      }
    } else if (width > 0) {
      float pw = image.getPlainWidth();
      float ph = image.getPlainHeight();
      image.scaleAbsolute(width, (width / pw) * ph);
    } else if (height > 0) {
      float pw = image.getPlainWidth();
      float ph = image.getPlainHeight();
      image.scaleAbsolute((height / ph) * pw, height);
    } else {
      //added by Kiran. If both width and height are not specified
      //then use the image dimensions
      image.setWidthPercentage(0);
    }
    if (rotate != 0.0F) {
      image.setRotation((float)Math.PI * rotate / 180);
    }
    width = image.getScaledWidth();
    height = image.getScaledHeight();
    
    Document document = documentHandler.getDocument();
    Rectangle pageSize = document.getPageSize();
    if (!isAttributeDefined(CommonAttributes.IMAGE_Y, CommonStyleAttributes.IMAGE_Y)) {
      switch (vAlign) {
        case Element.ALIGN_TOP:
          pos.setY(pageSize.getHeight() - document.bottomMargin() - height);
          break;
        case Element.ALIGN_BOTTOM:
          pos.setY(document.topMargin());
          break;
        case Element.ALIGN_MIDDLE:
          pos.setY(pageSize.getHeight() / 2.0F - height / 2.0F);
          break;
        default:
          pos.setY(document.topMargin());
          break;
      }
    }
    
    if (isAttributeDefined(CommonAttributes.IMAGE_Y, CommonStyleAttributes.IMAGE_Y) || 
        isAttributeDefined(CommonAttributes.IMAGE_VERTICAL_ALIGNMENT, CommonStyleAttributes.IMAGE_VERTICAL_ALIGNMENT)) {
      if (!isAttributeDefined(CommonAttributes.IMAGE_X, CommonStyleAttributes.IMAGE_X)) {
        switch (hAlign) {
          case Element.ALIGN_LEFT:
            pos.setX(document.leftMargin());
            break;
          case Element.ALIGN_RIGHT:
            pos.setX(pageSize.getWidth() - document.rightMargin() - width);
            break;
          case Element.ALIGN_CENTER:
            pos.setX(pageSize.getWidth() / 2.0F - width / 2.0F);
            break;
          default:
            pos.setX(document.leftMargin());
            break;
        }
      }
      
      boolean drawForeground = getBooleanAttribute(CommonAttributes.IMAGE_DRAW_FOREGROUND, true, true, CommonStyleAttributes.IMAGE_DRAW_FOREGROUND);
      if (stickyTag != null) {
        // registering this tag with the sticky tag.
        stickyTag.addStickyElementHandler(new ImagePageEventHandler(image, drawForeground));
      } else {
        // performing image output
        PdfWriter documentWriter = (PdfWriter) documentHandler.getDocumentWriter();
        if (drawForeground) {
          // drawing image here and now!
          insertImage(documentWriter, document, image, true);
        } else {
          // drawing image here and now!
          insertImage(documentWriter, document, image, false);
        }
      }
      return;
    }
    
    if (stickyTag != null) {
      log.warn("Usage with sticky tag only works for absolutely positioned images!");
    }
    ElementContainer elementContainer = getElementContainer();
    if (isAttributeDefined(CommonAttributes.IMAGE_HORIZONTAL_ALIGNMENT, CommonStyleAttributes.IMAGE_HORIZONTAL_ALIGNMENT)) {
      // setting horizontal alignment
      image.setAlignment(hAlign);
      elementContainer.addElement(this, image);
    } else {
      Chunk imgChunk = new Chunk(image, 0.0F, 0.0F);
      //Phrase imgPhrase = new Phrase(image.scaledHeight(), imgChunk);
      if (phraseHolder != null) {
        phraseHolder.addChunk(this, imgChunk);
      } else {
        elementContainer.addElement(this, imgChunk);
      }
    }
  }

  
  /**
   * Inserts the given image into the document.
   * @param writer The pdf writer.
   * @param document The document to add the image to.
   * @param image The image to insert.
   * @param foreground Tells whether to place the image in the foreground or the background.
   */
  protected void insertImage(PdfWriter writer, Document document, Image image, boolean foreground) {
    try {
      // calculating the correct image position
      Rectangle pageSize = document.getPageSize();
      float x = pos.calcX(pageSize.getRight(), image.getScaledWidth());
      float y = pos.calcY(pageSize.getTop(), image.getScaledHeight());
      // placing the image absolutely into the page
      image.setAbsolutePosition(x, y);
      PdfContentByte cb = null;
      if (foreground) {
        cb = writer.getDirectContent();
      } else {
        cb = writer.getDirectContentUnder();
      }
      cb.addImage(image);      
      
    } catch (DocumentException ex) {
      ex.printStackTrace();
      throw new RuntimeException(ex.getMessage());
    }
  }
  
}
