/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import java.io.IOException;
import java.io.StringReader;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.AttributeDefinition;
import org.ujac.print.DocumentHandlerException;
import org.ujac.util.xml.XmlUtils;

/**
 * Name: ImportTag<br>
 * Description: Handles the &lt;import&gt; tags.
 * 
 * @author lauerc
 */
public class ImportTag extends BaseDocumentTag {

  /** Definition of the 'source' attribute. */
  private static final AttributeDefinition SOURCE = new AttributeDefinition(TagAttributes.ATTR_SOURCE, 
      AttributeDefinition.TYPE_STRING, true, "The location of the document to import. This document " +
      "is loaded using the ResourceLoader, specified at the " +
      "DocumentHandler.");

  /** The item's name. */
  public static final String TAG_NAME = "import";
  
  /** The source attribute. */
  private Object source = null;

  /**
   * Constructs a ImportTag instance with no specific attributes.
   */
  public ImportTag() {
    super(TAG_NAME);
  }


  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Imports a document part from an external source, this document gets loaded by the " + 
      "resource loader, which is specified at the document handler.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(SOURCE);
  }

  /**
   * @see org.ujac.print.BaseDocumentTag#isVirtualContainer()
   */
  public boolean isVirtualContainer() {
    return true;
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the import item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    // getting attributes
    this.source = evalAttribute(SOURCE);

    try {
      // performing import    
      XMLReader contentXmlReader = XmlUtils.createXmlReader(documentHandler.getXmlReaderClass());
      contentXmlReader.setContentHandler(documentHandler);
      
      // disabling DTD/Schema based XML validation to avoid performance problems
      // and problems on systems that doesn't have internet access. 
      XmlUtils.disableEntityResolver(contentXmlReader);
      
      byte[] importData = null;
      if (source instanceof byte[]) {
        importData = (byte[]) source;
      } else {
        importData = documentHandler.loadResource(source.toString());
      }
      String importText = new String(importData);

      // avoiding that the internally processed stuff is copied too.
      documentHandler.disableTemplateCopyWriter();
      // parsing the imported document
      contentXmlReader.parse(new InputSource(new StringReader(importText)));
      // allowing the copying of XML events again.
      documentHandler.enableTemplateCopyWriter();
    } catch (SAXException ex) {
      Throwable cause = ex.getException();
      String at = "";
      if (cause instanceof DocumentHandlerException) {
        DocumentHandlerException dx = (DocumentHandlerException) cause;
        at = " at line " + dx.getLocator().getLineNumber();
      }
      throw new DocumentHandlerException(locator(), "Import of template '" + source + "' failed" + at + ":" + ex.getMessage(), ex);
    } catch (IOException ex) {
      throw new DocumentHandlerException(locator(), "Import of template '" + source + "' failed" + ex.getMessage(), ex);
    }
  }

}
