/*
 * Copyright (C) 2003, 2004 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import java.util.ArrayList;
import java.util.List;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.ChildDefinition;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.ItemHolder;
import org.ujac.print.TagAttributeException;

/**
 * Name: ItemsTag<br>
 * Description: A class handling simple items, added to an surrounding tag, which implements the ItemHolder interface.
 * 
 * @author lauerc
 */
public class ItemsTag extends BaseDocumentTag implements ItemHolder {
  
  /** The item's name. */
  public static final String TAG_NAME = "items";
  
  /** The item holder. */
  private ItemHolder itemHolder = null;
  /** The item list. */
  private List items = null;

  /**
   * Constructs a ItemsTag instance with no specific attributes.
   */
  public ItemsTag() {
    super(TAG_NAME);
  }

  /**
   * @see org.ujac.print.BaseDocumentTag#getDescription()
   */
  public String getDescription() {
    return "Aggregates a list of items and adds these list to the surrounding item holder. Known item holders tags are: parameter.";
  }
  
  /**
   * @see org.ujac.print.BaseDocumentTag#buildSupportedAttributes()
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.RENDERED);
  }
  
  /**
   * @see org.ujac.print.BaseDocumentTag#buildSupportedChilds()
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(ItemTag.class, 0, ChildDefinition.UNLIMITED));
  }
  
  /**
   * Tells whether textual content is allowed for the tag or not.
   * @return true if the item is allowed to contain textual content, else false.
   */
  public boolean isTextBodyAllowed() {
    return false;
  }
  
  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // getting parents
    this.itemHolder = (ItemHolder) documentHandler.latestOfType(ItemHolder.class);
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    // resetting items list
    items = null;
  }
  
  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }
    
    itemHolder.addItem(items);
  }

  /**
   * @see org.ujac.print.ItemHolder#addItem(java.lang.Object)
   */
  public void addItem(Object item) throws TagAttributeException {
    // parsing item value
    if (items == null) {
      items = new ArrayList();
    }
    items.add(item);
  }

  /**
   * Parses a value from the given text according to the rules of the item holder's container.
   * @param text The text to parse.
   * @return The parsed value.
   * @exception TagAttributeException In case the adding of the item has been failed.
   */
  public Object parseItem(String text) throws TagAttributeException {
    return itemHolder.parseItem(text);
  }
  
}
