/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.JavascriptContainer;

import org.ujac.util.template.TemplateException;

import com.lowagie.text.pdf.PdfWriter;

/**
 * Name: JavascriptTag<br>
 * Description: Adds javascript code to the document.
 * 
 * @author lauerc
 */
public class JavascriptTag extends BaseDocumentTag {

  /** The item's name. */
  public static final String TAG_NAME = "javascript";

  /**
   * Constructs a JavascriptTag instance with no specific attributes.
   */
  public JavascriptTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Adds javascript code to the surrounding element.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.RENDERED);
  }

  /**
   * Tells whether textual content is allowed for the tag or not.
   * @return true if the item is allowed to contain textual content, else false.
   */
  public boolean isTextBodyAllowed() {
    return true;
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the macro-param item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }
    
    // getting javascript code from tag body 
    String code = getContent();
    try {
      // evaluating code.
      code = documentHandler.executeTemplate(code);
      // adding evaluated code to document
      JavascriptContainer javascriptContainer = (JavascriptContainer) documentHandler.latestOfType(JavascriptContainer.class);
      if (javascriptContainer == null) {
        throw new DocumentHandlerException(locator(), "Unable to locate javascript container.");
      }
      PdfWriter pdfWriter = (PdfWriter) documentHandler.getDocumentWriter();
      pdfWriter.addJavaScript(code);
    } catch (TemplateException ex) {
      throw new DocumentHandlerException(locator(), "Failed to evaluate script code: '" + code + "': " + ex.getMessage(), ex);
    }
  }

}
