/*
 * Copyright (C) 2003 - 2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.ChildDefinition;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.CommonAttributesHolder;
import org.ujac.print.DocumentFont;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.ElementContainer;
import org.ujac.print.FontHolder;
import org.ujac.print.PhraseHolder;

import com.lowagie.text.Chunk;
import com.lowagie.text.Element;
import com.lowagie.text.Font;
import com.lowagie.text.List;
import com.lowagie.text.ListItem;
import com.lowagie.text.Phrase;

/**
 * Name: ListItemTag<br>
 * Description: Class handling list-item tags.
 * 
 * @author lauerc
 */
public class ListItemTag extends BaseElementTag implements ElementContainer, PhraseHolder, FontHolder {

  /** The item's name. */
  public static final String TAG_NAME = "list-item";

  /** The surrounding list tag. */
  private ListTag listTag = null;
  /**
   * Changed by Kiran. Enhanced this ListItem to contain a sub list also.
   * The list-item.
   */
  private ListItem item = null;
  
  /** The fixed vertical leading distance. */
  private float leading = DEFAULT_LEADING;
  /** The line spacing attribute. */
  private float lineSpacing = DEFAULT_LINE_SPACING;
  /** Tells whether or not the item's contents should be kept together on one page. */
  private boolean keepTogether = false;
  /** The spacing-before attribute. */
  private float spacingBefore = 0.0f;
  /** The spacing-after attribute. */
  private float spacingAfter = 0.0f;
  /** The horizontal alignment of the item. */ 
  private int hAlign = 0;

  /** The phrase that holds the item text. */
  private Phrase phrase = null;
  
  /**
   * Constructs a ListItemTag instance with no specific attributes.
   */
  public ListItemTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Adds an item the the surrounding &lt;list&gt; tag.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.CLASS)
      .addDefinition(CommonAttributes.STYLE)
      .addDefinition(CommonAttributes.FONT)
      .addDefinition(CommonAttributes.LEADING)
      .addDefinition(CommonAttributes.KEEP_TOGETHER)
      .addDefinition(CommonAttributes.LINE_SPACING)
      .addDefinition(CommonAttributes.SPACING_BEFORE)
      .addDefinition(CommonAttributes.SPACING_AFTER)
      .addDefinition(CommonAttributes.HORIZONTAL_ALIGNMENT);
  }
  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.FONT_NAME)
      .addDefinition(CommonStyleAttributes.LEADING)
      .addDefinition(CommonStyleAttributes.KEEP_TOGETHER)
      .addDefinition(CommonStyleAttributes.LINE_SPACING)
      .addDefinition(CommonStyleAttributes.SPACING_BEFORE)
      .addDefinition(CommonStyleAttributes.SPACING_AFTER)
      .addDefinition(CommonStyleAttributes.HORIZONTAL_ALIGNMENT);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(BaseFontTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(MacroTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(IfTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(ElseTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(LogTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(SwitchTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(ForeachTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(PhraseTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(NumberFormatTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(DateFormatTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ImageTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(SpaceTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(NewLineTag.class, 0, ChildDefinition.UNLIMITED))
      // added by Kiran    
      .addDefinition(new ChildDefinition(ListTag.class, 0, ChildDefinition.UNLIMITED));
  }

  /**
   * Tells whether or not this tag does respect style attributes or not.
   * @return always <code>true</code>.
   */
  public boolean isStyleable() {
    return true;
  }

  /**
   * Tells whether textual content is allowed for the tag or not.
   * @return true if the item is allowed to contain textual content, else false.
   */
  public boolean isTextBodyAllowed() {
    return true;
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    
    // determining the latest surrounding list tag
    listTag = (ListTag) documentHandler.latestOfType(ListTag.class);
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    this.item = null;
    this.phrase = null;

    if (!isValid()) {
      return;
    }

    // getting attributes
    CommonAttributesHolder commonAttributesHolder = getCommonAttributesHolder();
    if (isAttributeDefined(CommonAttributes.LEADING, CommonStyleAttributes.LEADING)) {
      this.leading = getDimensionAttribute(CommonAttributes.LEADING, true, CommonStyleAttributes.LEADING);
    } else {
      this.leading = commonAttributesHolder.getLeading();
    }    
    if (isAttributeDefined(CommonAttributes.LINE_SPACING, CommonStyleAttributes.LINE_SPACING)) {
      this.lineSpacing = getDimensionAttribute(CommonAttributes.LINE_SPACING, true, CommonStyleAttributes.LINE_SPACING);
    } else {
      this.lineSpacing = commonAttributesHolder.getLineSpacing();
    }
    this.keepTogether = getBooleanAttribute(CommonAttributes.KEEP_TOGETHER, false, true, CommonStyleAttributes.KEEP_TOGETHER);
    this.spacingBefore = getDimensionAttribute(CommonAttributes.SPACING_BEFORE, 0.0f, true, CommonStyleAttributes.SPACING_BEFORE);
    this.spacingAfter = getDimensionAttribute(CommonAttributes.SPACING_AFTER, 0.0f, true, CommonStyleAttributes.SPACING_AFTER);
    
    // Added by Kiran
    this.hAlign = getHorizontalAlignmentAttribute(CommonAttributes.HORIZONTAL_ALIGNMENT, true, CommonStyleAttributes.HORIZONTAL_ALIGNMENT);
  }
  
  /**
   * Gets the actual content.
   * This method is intended for testing purposes. 
   * @return The last content.
   */
  public String getActualContent() {
    if (phrase == null) {
      return super.getActualContent();
    }
    return getPhraseContent(phrase);
  }

  /**
   * Flushes the content buffer.
   * @exception DocumentHandlerException Thrown in case something went wrong while flushing the content buffer.
   */
  public void flushContent() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    if (phrase == null) {
      createPhrase();
    }
    if (isIgnoreFlush()) {
      // don't flush twice!
      return;
    }

    if (isTrimNextHead()) {
      trimContentHead();
      setTrimNextHead(false);
    }
    if (isItemClosed() && isTrimBody()) {
      trimContentTail();
    }

    Phrase nestedPhrase = addChunk(phrase, getLeading(), getLineSpacing());
    if (nestedPhrase != null) {
      phrase = nestedPhrase;
    }
    resetContent();
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    flushContent();

    
    //Added by Kiran
    if (item == null) {
      createListItem();
    }
    listTag.addItem(item);
  }

  /**
   * Adds the given chunk to the root phrase.
   * @param item The document item.
   * @param chunk The chunk to add to the root phrase.
   * @param font The font to use.
   * @exception DocumentHandlerException In case the document processing failed.
   */
  public void addChunk(BaseDocumentTag item, String chunk, DocumentFont font) 
    throws DocumentHandlerException {
    if (phrase == null) {
      createPhrase();
    }
    addChunk(item, phrase, chunk, font, getLeading(), getLineSpacing());
  }

  /**
   * Adds the given chunk to the root phrase.
   * @param item The document item.
   * @param chunk The chunk to add to the root phrase.
   */
  public void addChunk(BaseDocumentTag item, Chunk chunk) {
    if (phrase == null) {
      createPhrase();
    }
    addChunk(phrase, chunk, getLeading(), getLineSpacing());
  }


  /**
   * Gets the fixed vertical leading distance.
   * @return The current fixed vertical leading distance.
   */
  public float getLeading() {
    return leading;
  }

  /**
   * Gets the line spacing.
   * @return The current line spacing.
   */
  public float getLineSpacing() {
    return lineSpacing;
  }
  
  /**
   * Adds a element to the container.
   * @param item The document item.
   * @param element The element to be added.
   * @exception DocumentHandlerException If something went wrong when adding the element.
   */
  public void addElement(BaseDocumentTag item, Element element) throws DocumentHandlerException {
    if (phrase == null) {
      createPhrase();
    }
    phrase.add(element);    
  }

  /**
   * Tells whether the element container is a top level container or not.
   * @return true if the element container is a top level container, else false.
   */
  public boolean isTopLevel() {
    return false;
  }

  /**
   * Creates a new phrase.
   */
  private void createPhrase() {
    phrase = new Phrase(0.0F);
  }

  
  /**
   * Added by Kiran
   * Adds a sublist to the list-item.
   * @param sublist The sublist to add. 
   */
  public void addSubList(List sublist) {
    if (item == null) {
      createListItem();
    }
    item.add(sublist);
  }
  
  /**
   * Creates an list item.
   */
  private void createListItem() {
    Font font = getFont().getFont();
    //Added by Kiran
    item = new ListItem("", font);
    item.setAlignment(hAlign);
    //End Added by Kiran
    item.add(phrase);
    item.setSpacingBefore(spacingBefore);
    item.setSpacingAfter(spacingAfter);
    //item.setLeading(leading, lineSpacing);
    item.setLeading(phrase.getLeading());
    item.setKeepTogether(keepTogether);
  }

}
