/*
 * Copyright (C) 2003 - 2006 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import java.util.List;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.ChildDefinition;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.DocumentFont;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.ElementContainer;
import org.ujac.print.PhraseHolder;
import org.ujac.print.TagAttributeException;

import org.ujac.util.template.TemplateException;

import com.lowagie.text.Chunk;
import com.lowagie.text.Element;
import com.lowagie.text.Image;
import com.lowagie.text.Phrase;

/**
 * Name: ListSymbolTag<br>
 * Description: A class handling the list-symbol tag.
 * 
 * @author lauerc
 */
public class ListSymbolTag extends BaseElementTag implements ElementContainer, PhraseHolder {

  /** The item's name. */
  public static final String TAG_NAME = "list-symbol";
  
  /** The surrounding list tag. */
  private ListTag listTag = null;
  /** The nested chunk. */
  private Chunk nestedChunk = null;
  
  /**
   * Constructs a ListSymbolTag instance with no specific attributes.
   */
  public ListSymbolTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Specifies the list symbol at the surroundng list.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return null;
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(ImageTag.class, 0, 1));
  }
  
  /**
   * Tells whether textual content is allowed for the tag or not.
   * @return true if the item is allowed to contain textual content, else false.
   */
  public boolean isTextBodyAllowed() {
    return true;
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    
    // determining the latest surrounding list tag
    listTag = (ListTag) documentHandler.latestOfType(ListTag.class);
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    Chunk listSymbol = nestedChunk;
    if (listSymbol == null) {
      String symbol = null;
      try {
        symbol = getDocumentHandler().executeTemplate(getContent());
      } catch (TemplateException ex) {
        throw new TagAttributeException(locator(), "Attribute evaluation failed: " + ex.getMessage(), ex);
      }
      if (symbol == null) {
        symbol = "";
      }
      Phrase symbolPhrase = addChunk(this, new Phrase(0.0F), symbol, getFont(), getLeading(), getLineSpacing());
      List symbolChunks = symbolPhrase.getChunks();
      int numSymbolChunks = symbolChunks.size();
      if (numSymbolChunks >= 1) {
        listSymbol = (Chunk) symbolChunks.get(0);
      } else {
        throw new DocumentHandlerException(locator(), "No list symbol found.");
      }
    }
    
    listTag.setListSymbol(listSymbol);
  }

  /**
   * @see org.ujac.print.ElementContainer#addElement(org.ujac.print.BaseDocumentTag, com.lowagie.text.Element)
   */
  public void addElement(BaseDocumentTag item, Element element) throws DocumentHandlerException {
    if (element instanceof Image) {
      this.nestedChunk = new Chunk((Image) element, 0.0F, 0.0F);
      return;
    }
    
    if (element instanceof Chunk) {
      this.nestedChunk = (Chunk) element;
    } else {
      throw new DocumentHandlerException(locator(), "Only images are allowed within list-symbol tag.");
    }
  }

  /**
   * Tells whether the element container is a top level container or not.
   * @return true if the element container is a top level container, else false.
   */
  public boolean isTopLevel() {
    return false;
  }

  /**
   * @see org.ujac.print.PhraseHolder#addChunk(org.ujac.print.BaseDocumentTag, java.lang.String, com.lowagie.text.Font)
   */
  public void addChunk(BaseDocumentTag item, String chunk, DocumentFont font) throws DocumentHandlerException {
    this.nestedChunk = new Chunk(chunk, font.getFont());
    float textRise = font.getTextRise();
    if (textRise != 0.0F) {
      nestedChunk.setTextRise(textRise);
    }
  }

  /**
   * @see org.ujac.print.PhraseHolder#addChunk(org.ujac.print.BaseDocumentTag, com.lowagie.text.Chunk)
   */
  public void addChunk(BaseDocumentTag item, Chunk chunk) {
    this.nestedChunk = chunk;
  }

  /**
   * Gets the fixed vertical leading distance.
   * @return The current fixed vertical leading distance.
   */
  public float getLeading() {
    return DEFAULT_LEADING;
  }

  /**
   * Gets the line spacing.
   * @return The current line spacing.
   */
  public float getLineSpacing() {
    return listTag.getLineSpacing();
  }

}
