/*
 * Copyright (C) 2003 - 2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.ChildDefinition;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.ElementContainer;
import org.ujac.print.FontHolder;
import org.ujac.print.TagAttributeException;

import com.lowagie.text.Chunk;
import com.lowagie.text.GreekList;
import com.lowagie.text.List;
import com.lowagie.text.ListItem;
import com.lowagie.text.RomanList;
import com.lowagie.text.ZapfDingbatsList;
import com.lowagie.text.ZapfDingbatsNumberList;

/**
 * Name: ListTag<br>
 * Description: A class handling list tags.
 * 
 * @author lauerc
 */
public class ListTag extends BaseElementTag implements FontHolder {

  /** The default symbol indentation distance. */
  public static final float DEFAULT_SYMBOL_INDENT = 15.0F;

  /** Definition of the 'numbered' attribute. */
  public static final AttributeDefinition NUMBERED = new AttributeDefinition(TagAttributes.ATTR_NUMBERED, AttributeDefinition.TYPE_BOOLEAN, false, 
      "Tells whether the list items are numbered or not.");
  /** Definition of the 'lettered' attribute. */
  public static final AttributeDefinition LETTERED = new AttributeDefinition(TagAttributes.ATTR_LETTERED, AttributeDefinition.TYPE_BOOLEAN, false, 
      "Tells whether the list items are lettered or not.");
  /** Definition of the 'symbol-indent' attribute. */
  public static final AttributeDefinition SYMBOL_INDENT = new AttributeDefinition(TagAttributes.ATTR_SYMBOL_INDENT, AttributeDefinition.TYPE_INT, false, 
      "Specifies the symbol indention.");
  /** Definition of the 'list-type' attribute. */
  public static final AttributeDefinition LIST_TYPE = new AttributeDefinition(TagAttributes.ATTR_LIST_TYPE, AttributeDefinition.TYPE_STRING, false,
      "Specifies the type of list.The listtype can be only 'list', 'roman', 'greek' or 'zapfdingbats'.");
  /** Definition of the 'itemstart' attribute. */
  public static final AttributeDefinition LIST_ITEMSTART = new AttributeDefinition(TagAttributes.ATTR_LIST_ITEMSTART, AttributeDefinition.TYPE_INT, false,
      "Specifies the numbering at which the list item should start.");
  /** Definition of the 'type' attribute. */
  public static final AttributeDefinition TYPE = new AttributeDefinition(TagAttributes.ATTR_TYPE, AttributeDefinition.TYPE_STRING, false,
      "Specifies the character to be used in the list." +
      "If the font used supports this char then it will be" +
      " displayed else it will not.");

  /** The item's name. */
  public static final String TAG_NAME = "list";

  /** The line spacing attribute. */
  private float lineSpacing = 0.0f;

  /** The list. */
  private List list = null;
  /** The surrounding list tag. */
  private ListTag listTag = null;
  /**
   * Added by Kiran.
   * The surrounding list tag.
   */
  private ListItemTag listItemTag = null;
  /**
   * Added by Kiran
   * The list type. The listtype can be only 'list', 'roman', 'greek' or 'zapfdingbats'.
   */
  private String listType = null;
  /** The char to be used in List Item bullet. */
  private char listChar = 0;
  /** The list numbering starts at. */
  private int listStartAt = 1;

  /**
   * Constructs a ListTag instance with no specific attributes.
   */
  public ListTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Adds an item to the surroundng list.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.CLASS)
      .addDefinition(CommonAttributes.STYLE)
      .addDefinition(CommonAttributes.FONT)
      .addDefinition(CommonAttributes.LINE_SPACING)
      .addDefinition(NUMBERED)
      .addDefinition(LETTERED)
      .addDefinition(SYMBOL_INDENT)
      .addDefinition(LIST_TYPE)
      .addDefinition(LIST_ITEMSTART)
      .addDefinition(TYPE);
  }
  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.FONT_NAME)
      .addDefinition(CommonStyleAttributes.LINE_SPACING)
      .addDefinition(NUMBERED)
      .addDefinition(LETTERED)
      .addDefinition(SYMBOL_INDENT);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(BaseFontTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(MacroTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(IfTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ElseTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(LogTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(SwitchTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ForeachTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ListTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ListSymbolTag.class, 0, 1))
      .addDefinition(new ChildDefinition(ListItemTag.class, 0, ChildDefinition.UNLIMITED));
  }

  /**
   * Tells whether or not this tag does respect style attributes or not.
   * @return always <code>true</code>.
   */
  public boolean isStyleable() {
    return true;
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    
    //Added by Kiran
    listItemTag = (ListItemTag) documentHandler.latestOfType(ListItemTag.class);

    if (listItemTag == null) {
      // determining the latest surrounding list tag
      listTag = (ListTag) documentHandler.latestOfType(ListTag.class);
    }
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    // getting attributes
    if (isAttributeDefined(CommonAttributes.LINE_SPACING, CommonStyleAttributes.LINE_SPACING)) {
      this.lineSpacing = getDimensionAttribute(CommonAttributes.LINE_SPACING, DEFAULT_LINE_SPACING, true, CommonStyleAttributes.LINE_SPACING);
    } else {
      if (listTag != null) {
        this.lineSpacing = listTag.getLineSpacing();
      } else {
        ElementContainer elementContainer = getElementContainer();
        this.lineSpacing = elementContainer.getLineSpacing();
      }
    }
    
    //Added by Kiran Krishnamurthy
    this.listType = getStringAttribute(LIST_TYPE, "list", true, null);
    this.listChar = this.getCharAttribute("type");
    this.listStartAt = this.getIntegerAttribute(LIST_ITEMSTART, 1, true, null);

    this.list = new List(
      getBooleanAttribute(NUMBERED, true, NUMBERED),
      getBooleanAttribute(LETTERED, true, LETTERED),
      getDimensionAttribute(SYMBOL_INDENT, DEFAULT_SYMBOL_INDENT, true, SYMBOL_INDENT)
    );

  }


  /**
   * Added by Kiran
   * Gets the value for the given attribute's name.
   * @param name The attribute's name.
   * @param defaultValue The default value in case the attribute was not defined.
   * @return The determined value.
   * @exception TagAttributeException if the item contains an invalid value or an required attribute was not defined.
   */
  public char getCharAttribute(String name)
    throws TagAttributeException {
    char listChar = 0;
    String attrValue = (String) evalAttribute(name);

    if (attrValue != null) {
      try {
        listChar = (char)Integer.valueOf(attrValue).intValue();
      } catch(NumberFormatException nfe) {
        listChar = getUnicodeChar(attrValue);
        //canDisplayChar(listChar);
      }
    }
    return listChar;
  }

  /**
   * Added by Kiran
   * Creates a char based on the unicode string given. '\ucc345'
   * The same can be achieved by doing String.getBytes("UTF-8")
   * but since we cannot change the encoding dynamically its better to get the
   * hex representation of the string and let the font take care of it.
   * @param s The unicode string.
   * @return The parsed unicode character.
   */
  public char getUnicodeChar(String s) {
    int i = 0;
    int len = s.length();
    char c = 0;
    while (i < len) {
      c = s.charAt(i++);
      if (c=='\\') {
        if (i < len) {
          c = s.charAt(i++);
          if (c == 'u') {
            c = (char) Integer.parseInt(s.substring(i,i+4),16);
            //i += 4;
            break;
          }
        }
      } else {
        c = s.charAt(0);
      }
    }
    return c;
  }


  /**
   * Added by Kiran
   * Creates a list based on the type given.
   * The listtype can be only 'list', 'roman', 'greek' or 'zapfdingbats'.
   * @param sublist The sublist to add.
   * @exception DocumentHandlerException In case and unsupported list type was specified.
   */
  public void setupList() throws DocumentHandlerException {
    if ("list".equals(listType)) {
      this.list = new List(
          getBooleanAttribute(NUMBERED, true, NUMBERED),
          getBooleanAttribute(LETTERED, true, LETTERED),
          getFloatAttribute(SYMBOL_INDENT, DEFAULT_SYMBOL_INDENT, true, SYMBOL_INDENT)
      );
      this.list.setFirst(this.listStartAt);
      this.list.setListSymbol(new Character(this.listChar).toString());
    } else if ("roman".equals(listType)) {
      this.list = new RomanList(false, getIntegerAttribute(SYMBOL_INDENT, 15, true, SYMBOL_INDENT));
      this.list.setFirst(this.listStartAt);
    } else if ("greek".equals(listType)) {
      this.list = new GreekList(true, getIntegerAttribute(SYMBOL_INDENT, 15, true, SYMBOL_INDENT));
      this.list.setFirst(this.listStartAt);
    } else if ("zapfdingbats".equals(listType)) {
      if (getBooleanAttribute(NUMBERED, true, NUMBERED)) {
        this.list = new ZapfDingbatsNumberList(
            this.listChar, 
            getIntegerAttribute(SYMBOL_INDENT, 15, true, SYMBOL_INDENT)
        );
        this.list.setFirst(this.listStartAt);
      } else {
        this.list = new ZapfDingbatsList(
        this.listChar, getIntegerAttribute(SYMBOL_INDENT, 15, true, SYMBOL_INDENT));
        this.list.setFirst(this.listStartAt);
      }
    } else {
      throw new TagAttributeException(
          locator(), "Unsupported listType '" + listType +
          "'. The listtype can be only 'list', 'roman', 'greek' or 'zapfdingbats'."
      );
    }
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    //Added by Kiran
    if (listItemTag != null) {
      listItemTag.addSubList(list);
    } else if (listTag != null) {
      listTag.addSubList(list);
    } else {
      ElementContainer elementContainer = getElementContainer();
      elementContainer.addElement(this, list);
    }
  }

  /**
   * Sets the list symbol.
   * @param symbol The chunk to use as symbol. 
   */
  public void setListSymbol(Chunk symbol) {
    list.setListSymbol(symbol);
  } 

  /**
   * Adds a sublist to the list.
   * @param sublist The sublist to add. 
   */
  public void addSubList(List sublist) {
    list.add(sublist);
  } 

  /**
   * Adds an item to the list.
   * @param item The item to add. 
   */
  public void addItem(ListItem item) {
    list.add(item);
  }

  /**
   * Gets the line spacing.
   * @return The current line spacing.
   */
  public float getLineSpacing() {
    return lineSpacing;
  }

}
