/*
 * Copyright (C) 2005 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.TagAttributeException;

import org.ujac.util.exi.ExpressionException;
import org.ujac.util.template.TemplateException;

/**
 * Name: LogTag<br>
 * Description: A class handling &lt;log&gt; tags.
 * 
 * @author lauerc
 */
public class LogTag extends BaseDocumentTag {

  /** Definition of the 'level' attribute. */
  private static final AttributeDefinition LEVEL = new AttributeDefinition(TagAttributes.ATTR_LEVEL, 
      new String[] {"debug", "info", "warn", "error", "fatal"}, true, "The log level.");
  /** Definition of the 'level' attribute. */
  private static final AttributeDefinition CONDITION = CommonAttributes.CONDITION.cloneAttrDef().setRequired(false);
  
  /** The item's name. */
  public static final String TAG_NAME = "log";

  /** The condition. */
  private String condition = null;

  /**
   * Constructs a LogTag instance with no specific attributes.
   */
  public LogTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Writing the tag's content to log.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(LEVEL)
      .addDefinition(CONDITION);
  }
  
  /**
   * Tells whether textual content is allowed for the tag or not.
   * @return true if the item is allowed to contain textual content, else false.
   */
  public boolean isTextBodyAllowed() {
    return true;
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // setting the containers.
    this.condition = getStringAttribute(CONDITION, false, null);
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    if (condition != null) {
      try {
        // evaluating condition
        if (!documentHandler.evalBoolean(condition)) {
          // condition not fulfilled, skipping log
          return;
        }
      } catch (ExpressionException ex) {
        throw new TagAttributeException(locator(), "Attribute evaluation failed: " + ex.getMessage(), ex);
      }
    }
    
    String content = null;
    try {
      content = documentHandler.executeTemplate(getContent());
    } catch (TemplateException ex) {
      throw new TagAttributeException(locator(), "Body evaluation failed: " + ex.getMessage(), ex);
    }
    String level = getStringAttribute(LEVEL, true, null);
    
    if ("debug".equals(level)) {
      log.debug(content);
      return;
    }
    if ("info".equals(level)) {
      log.info(content);
      return;
    }
    if ("warn".equals(level)) {
      log.warn(content);
      return;
    }
    if ("error".equals(level)) {
      log.error(content);
      return;
    }
    if ("fatal".equals(level)) {
      log.fatal(content);
      return;
    }
  }

}
