/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import java.util.Map;
import java.util.List;
import java.util.HashMap;
import java.util.ArrayList;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.Condition;
import org.ujac.print.DocumentEvent;
import org.ujac.print.CellContainer;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.DocumentEventRecorder;
import org.ujac.print.ElementContainer;
import org.ujac.print.ParameterDefHolder;
import org.ujac.print.AttributeDefinition;
import org.ujac.print.ParameterDefinition;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.PdfCellContainer;
import org.ujac.print.PdfTableHolder;

import com.lowagie.text.Cell;
import com.lowagie.text.Element;
import com.lowagie.text.pdf.PdfPCell;

/**
 * Name: MacroDefTag<br>
 * Description: A tag defining macros.
 * 
 * @author lauerc
 */
public class MacroDefTag 
  extends BaseDocumentTag 
  implements ParameterDefHolder, DocumentEventRecorder, CellContainer, PdfCellContainer, 
             ElementContainer, Condition {

  /** Definition of the 'macro-name' attribute. */
  private static final AttributeDefinition MACRO_NAME = new AttributeDefinition(TagAttributes.ATTR_NAME, 
      AttributeDefinition.TYPE_STRING, true, "The name of the macro to define.");

  /** The item's name. */
  public static final String TAG_NAME = "macro-def";

  /** The name attribute. */
  protected String name = null;
  /** The parameter definition map. */
  private Map parameterDefs = null;
  /** The document event list. */
  private List documentEvents = new ArrayList();

  /**
   * Constructs a MacroDefTag instance with no specific attributes.
   */
  public MacroDefTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Defines a document macro block. This macro can be used in serveral parts of the " +
      "document using the 'macro' item.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(MACRO_NAME);
  }

  /**
   * Is the item virtual container, such as if, else or foreach.
   * @return true if the item is such a container, else false.
   */
  public boolean isVirtualContainer() {
    return true;
  }

  /**
   * Gets the document event list.
   * @return The document event list.
   */
  public List getDocumentEvents() {
    return documentEvents;
  }

  /**
   * Gets the parameter definition map.
   * @return The parameter definition map.
   */
  public Map getParameterDefs() {
    return parameterDefs;
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
  
    // getting attributes
    if (isAttributeSupported(MACRO_NAME)) {
      this.name = getStringAttribute(MACRO_NAME, false, null);
    }
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the image item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    documentHandler.pushEventRecorder(this);

    if (!isValid()) {
      return;
    }

    // creating parameter map
    parameterDefs = new HashMap();    
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the macro-param item.
   */
  public void closeItem() throws DocumentHandlerException {
    documentHandler.popEventRecorder();

    if (!isValid()) {
      return;
    }
   
    // registering macro definition
    documentHandler.registerMacroDefinition(this.name, this);
    // registering custom tag
    Map customAttributes = new HashMap();
    customAttributes.put(TagAttributes.ATTR_NAME, this.name);
    documentHandler.getTagFactory().registerTag(this.name, new CustomTagBuilder(MacroTag.TAG_NAME, this.name, customAttributes));
  }

  /**
   * Sets a parameter definition. 
   * @param parameterDef  The parameter definition to set.
   */
  public void setParameterDef(ParameterDefinition parameterDef) {
    parameterDefs.put(parameterDef.getName(), parameterDef);
  }

  /**
   * @see org.ujac.print.DocumentEventRecorder#recordEvent(org.ujac.print.DocumentEvent)
   */
  public void recordEvent(DocumentEvent event) {
    BaseDocumentTag item = event.getItem();
    if ((item != this) &&
        !(item instanceof ParameterDefTag)) {
      documentEvents.add(event);
    }
  }

  /**
   * Adds a cell to the container.
   * @param item The document item.
   * @param cell The cell to be added.
   * @exception DocumentHandlerException If something went wrong when adding the cell.
   */
  public void addCell(BaseDocumentTag item, Cell cell) throws DocumentHandlerException {
  }

  /**
   * Gets the number of remaining columns of the current row.
   * @return The number of remaining column to the end of the current row.
   */
  public int getRemainingColumns() {
    return 0; 
  }

  /**
   * Adds a PDF cell to the container.
   * @param item The document item.
   * @param cell The PDF cell to be added.
   * @exception DocumentHandlerException If something went wrong when adding the cell.
   */
  public void addCell(BaseDocumentTag item, PdfPCell cell) throws DocumentHandlerException {
  }

  /**
   * Gets the number of remaining columns of the current row.
   * @return The number of remaining column to the end of the current row.
   */
  public int getRemainingPdfColumns() {
    return 0; 
  }


  /**
   * Gets the PDF table holder.
   * @return The component that really holds a PDF table.
   */
  public PdfTableHolder getPdfTable() {
    return null;
  }

  /**
   * Adds a element to the container.
   * @param item The document item.
   * @param element The element to be added.
   * @exception DocumentHandlerException If something went wrong when adding the element.
   */
  public void addElement(BaseDocumentTag item, Element element) throws DocumentHandlerException {
  }

  /**
   * Tells whether the element container is a top level container or not.
   * @return true if the element container is a top level container, else false.
   */
  public boolean isTopLevel() {
    return false;
  }

  /**
   * Gets the line spacing.
   * @return The current line spacing.
   */
  public float getLineSpacing() {
    return DEFAULT_LINE_SPACING;
  }

  /**
   * Gets the leading.
   * @return The current leading.
   */
  public float getLeading() {
    return DEFAULT_LEADING;
  }

  /**
   * @see org.ujac.print.Condition#isTrue()
   */
  public boolean isTrue() {
    return false;
  }

  /**
   * Gets the cell attributes.
   * @return The cell attributes.
   */
  public CellAttributes getCellAttributes() {
    return null;
  }

}
