/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import java.util.Map;
import java.util.Set;
import java.util.List;
import java.util.HashMap;
import java.util.Iterator;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.DocumentEvent;
import org.ujac.print.ChildDefinition;
import org.ujac.print.DocumentTagInterceptor;
import org.ujac.print.ParameterHolder;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.AttributeDefinition;
import org.ujac.print.ParameterDefinition;
import org.ujac.print.TagAttributeException;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.TagBuilder;

import org.ujac.util.template.TemplateContext;
import org.ujac.util.template.TemplateException;

/**
 * Name: MacroTag<br>
 * Description: A tag executing macros of the class.
 * 
 * @author lauerc
 */
public class MacroTag extends BaseDocumentTag implements ParameterHolder {

  /** Definition of the 'macro-name' attribute. */
  private static final AttributeDefinition MACRO_NAME = new AttributeDefinition(TagAttributes.ATTR_NAME, 
      AttributeDefinition.TYPE_STRING, true, "The name of the macro definition to use.");

  /** The item's name. */
  public static final String TAG_NAME = "macro";

  /** The macro definition item. */
  private MacroDefTag macroDef = null;
  /** The name attribute. */
  private String name = null;
  /** The parameter map. */
  private Map parameters = null;

  /**
   * Constructs a MacroTag instance with no specific attributes.
   */
  public MacroTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Uses a document macro, this macro must have been specified previously by a 'macro-def' item.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.RENDERED)
      .addDefinition(MACRO_NAME);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(ParameterTag.class, 0, ChildDefinition.UNLIMITED));
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();

    // getting attributes
    //this.supportedAttributeMap = new AttributeDefinitionMap(supportedAttributes);
    Map customAttributes = getCustomAttributes();
    if (customAttributes != null) {
      this.name = (String) customAttributes.get(TagAttributes.ATTR_NAME);
      if (name == null) {
        this.name = (String) customAttributes.get(TagBuilder.ATTR_TAG_NAME);
        if (name != null) {
          customAttributes.put(TagAttributes.ATTR_NAME, name);
        }
      }
    }
    this.name = getStringAttribute(MACRO_NAME, false, null);
    // getting the macro
    macroDef = documentHandler.lookupMacroDefinition(name);
    if (macroDef == null) {
      throw new DocumentHandlerException(locator(), "No definition found for macro '" + name + "'.");
    }
    // registering parameters as attributes at macro tag
    Iterator iterParamDefs = macroDef.getParameterDefs().entrySet().iterator();
    while (iterParamDefs.hasNext()) {
      Map.Entry pdEntry = (Map.Entry) iterParamDefs.next();
      ParameterDefinition pd = (ParameterDefinition) pdEntry.getValue();
      this.addAttributeDefinition(new AttributeDefinition(
        pd.getName(),
        pd.getType(),
        !pd.isOptional(),
        pd.getDescription()
      ));
    }
    
    super.checkAttributes();
  }

  /**
   * @see org.ujac.print.BaseDocumentTag#checkAttributes()
   */
  protected void checkAttributes() throws TagAttributeException {
    //super.checkAttributes();
  }
  
  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the image item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }
    // creating parameter map
    parameters = new HashMap();    
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the macro-param item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }
    // playing macro
    List documentEvents = macroDef.getDocumentEvents();
    int numEvents = documentEvents.size();
    TemplateContext oldTemplateContext = null;
    Map parameters = this.parameters;
    Map parameterDefs = macroDef.getParameterDefs();
    Set paramDefSet = parameterDefs.entrySet();
    Iterator iterParamDefs = paramDefSet.iterator();
    while (iterParamDefs.hasNext()) {
      Map.Entry pdEntry = (Map.Entry) iterParamDefs.next();
      ParameterDefinition pd = (ParameterDefinition) pdEntry.getValue();
      String pdName = pd.getName();
      if (isAttributeDefined(pdName)) {
        // evaluating attribute
        String attr = getRawAttribute(pdName);
        Object value = attr;
        if ("${body}".equals(attr)) {
          try {
            value = documentHandler.executeTemplate(attr);
          } catch (TemplateException ex) {
            throw new TagAttributeException(locator(), "Attribute evaluation failed: " + ex.getMessage(), ex);
          }
        }
        parameters.put(pdName, value);
      }
      if (!parameters.containsKey(pdName)) {
        if (!pd.isOptional()) {
          throw new TagAttributeException(locator(), "Required attribute '" + pd.getName() + "' was not defined for macro '" + name + "'.");
        }
        parameters.put(pdName, pd.getDefaultValue());
      }
    }
    if (parameters.size() > 0) {
      oldTemplateContext = documentHandler.getTemplateContext();
      TemplateContext localTemplateContext = new TemplateContext(oldTemplateContext);
      localTemplateContext.putAll(parameters);
      documentHandler.setTemplateContext(localTemplateContext);
    }
    
    DocumentTagInterceptor tagInterceptor = documentHandler.getTagInterceptor();
    
    // playing document events
    for (int i = 0; i < numEvents; i++) {
      DocumentEvent event = (DocumentEvent) documentEvents.get(i);
      BaseDocumentTag item = event.getItem();
      switch (event.getType()) {
        case DocumentEvent.OPEN:
          // opening item
          BaseDocumentTag topItem = item.getParentItem();
          if (topItem == this) {
            topItem = topItem.getParentItem();
          }
          if (topItem != null) {
            topItem.flushContent();
          }
          // re-initializing item
          item.preInitialize();
          item.initialize();

          if ((tagInterceptor == null) || tagInterceptor.beforeOpen(item)) {
            // opening the item.
            item.openItem();
            if (tagInterceptor != null) {
              tagInterceptor.afterOpen(item);
            }
          }
          // putting the item on top of the stack.
          documentHandler.pushItem(item);
          break;
        case DocumentEvent.CHARACTERS:
          String content = event.getContent();
          if (content.length() > 0) {
            item.setItemOpened(false);
            item.addContent(content);
            if (i == (numEvents - 1)) {
              // flushing content of last item
              item.flushContent();
            }
          }
          break;
        case DocumentEvent.CLOSE:
          // closing item
          item.setItemClosed(true);
          item.flushContent();
          
          if ((tagInterceptor == null) || tagInterceptor.beforeClose(item)) {
            item.closeItem();
            if (tagInterceptor != null) {
              tagInterceptor.afterClose(item);
            }
          }
          documentHandler.popItem();
          break;
          
        default:
          throw new DocumentHandlerException(locator(), "Encountered illegal event type '" + event.getType() + "!");
      }
    }
    
    if (oldTemplateContext != null) {
      // resetting template context
      documentHandler.setTemplateContext(oldTemplateContext);
    }
  }
 
  /**
   * Sets a parameter. 
   * @param name  The parameter name.
   * @param value  The parameter value.
   */
  public void setParameter(String name, Object value) {
    parameters.put(name, value);
  }
  

  /**
   * Gets the definition of the given parameter. 
   * @param name  The parameter name.
   * @return  The parameter value.
   */
  public ParameterDefinition getParameterDefinition(String name) {
    return (ParameterDefinition) macroDef.getParameterDefs().get(name);
  }
}
