/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.DocumentHandlerException;

import com.lowagie.text.Document;

/**
 * Name: MetaTag<br>
 * Description: A class handling meta tags.
 * 
 * @author lauerc
 */
public class MetaTag extends BaseDocumentTag {

  /** Definition of the 'title' attribute. */
  private static final AttributeDefinition TITLE = new AttributeDefinition(TagAttributes.ATTR_TITLE,
      AttributeDefinition.TYPE_STRING, false, "The title of the document.");
  /** Definition of the 'subject' attribute. */
  private static final AttributeDefinition SUBJECT = new AttributeDefinition(TagAttributes.ATTR_SUBJECT, 
      AttributeDefinition.TYPE_STRING, false, "The subject of the document.");
  /** Definition of the 'keywords' attribute. */
  private static final AttributeDefinition KEYWORDS = new AttributeDefinition(TagAttributes.ATTR_KEYWORDS, 
      AttributeDefinition.TYPE_STRING, false, "The subject of the document.");
  /** Definition of the 'author' attribute. */
  private static final AttributeDefinition AUTHOR = new AttributeDefinition(TagAttributes.ATTR_AUTHOR, 
      AttributeDefinition.TYPE_STRING, false, "The document's author.");
  /** Definition of the 'creator' attribute. */
  private static final AttributeDefinition CREATOR = new AttributeDefinition(TagAttributes.ATTR_CREATOR, 
      AttributeDefinition.TYPE_STRING, false, "The document's creator.");
  /** Definition of the 'creation-date' attribute. */
  private static final AttributeDefinition CREATION_DATE = new AttributeDefinition(TagAttributes.ATTR_CREATION_DATE, 
      AttributeDefinition.TYPE_STRING, false, "Tells whether to add the creation date to the document or not.");
  
  /** The item's name. */
  public static final String TAG_NAME = "meta";

  /**
   * Constructs a MetaTag instance with no specific attributes.
   */
  public MetaTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Adds meta informations to the document." ;
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(TITLE)    
      .addDefinition(SUBJECT)
      .addDefinition(KEYWORDS)
      .addDefinition(AUTHOR)
      .addDefinition(CREATOR)
      .addDefinition(CREATION_DATE);
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
  }
  
  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    // setting the meta data 
    Document doc = getDocumentHandler().getDocument();
    if (isAttributeDefined(TITLE)) {
      doc.addTitle(getTextAttribute(TITLE, true, null));
    }
    if (isAttributeDefined(SUBJECT)) {
      doc.addSubject(getStringAttribute(SUBJECT, true, null));
    }
    if (isAttributeDefined(KEYWORDS)) {
      doc.addKeywords(getStringAttribute(KEYWORDS, true, null));
    }
    if (isAttributeDefined(AUTHOR)) {
      doc.addAuthor(getStringAttribute(AUTHOR, true, null));
    }
    if (isAttributeDefined(CREATOR)) {
      doc.addCreator(getStringAttribute(CREATOR, true, null));
    }
    if (getBooleanAttribute(CREATION_DATE, true, null)) {
      doc.addCreationDate();
    }
  }
}
