/*
 * Copyright (C) 2003 - 2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import java.awt.Color;

import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.ChildDefinition;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.AttributeDefinition;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.NewPageHandler;

import com.lowagie.text.PageSize;
import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.PdfPageEvent;

/**
 * Name: NewPageTag<br>
 * Description: Item to force a page break at the document.<br>
 * 
 * @author lauerc
 */
public class NewPageTag extends BaseDocumentTag {

  /** Definition of the 'logical-pagenumbering' attribute. */
  private static final AttributeDefinition LOGICAL_PAGENUMBERING = new AttributeDefinition(TagAttributes.ATTR_LOGICAL_PAGENUMBERING, AttributeDefinition.TYPE_INT, false,
      "The Logical Page Numbering style of the generated PDF (0: arabic numerals, 1: uppercase roman numerals, 2: lowercase roman numerals" +
      ", 3: uppercase letters, 4: lowercase letters, 5: empty (No logical page numbers are generated but fixed text may still exist)).");
  /** Definition of the 'logical-pagetext' attribute. */
  private static final AttributeDefinition LOGICAL_PAGETEXT = new AttributeDefinition(TagAttributes.ATTR_LOGICAL_PAGETEXT, AttributeDefinition.TYPE_STRING, false,
      "The Logical Page Numbering style text of the generated PDF.");

  /** The item's name. */
  public static final String TAG_NAME = "new-page";

  /** The new page handler. */
  protected NewPageHandler newPageHandler = null;
  /** Holds the queued event handlers. */
  private Map queuedEventHandlers = new HashMap();

  /**
   * Constructs a NewPageTag instance with no specific attributes.
   */
  public NewPageTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Starts a new page.";
  }

  /**
   * @see org.ujac.print.BaseDocumentTag#buildSupportedAttributes()
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes() 
      .addDefinition(CommonAttributes.RENDERED)
      .addDefinition(CommonAttributes.PAGE_SIZE)
      .addDefinition(CommonAttributes.PAGE_WIDTH)
      .addDefinition(CommonAttributes.PAGE_HEIGHT)
      .addDefinition(CommonAttributes.ROTATE)
      .addDefinition(CommonAttributes.MARGIN_LEFT)
      .addDefinition(CommonAttributes.MARGIN_RIGHT)
      .addDefinition(CommonAttributes.MARGIN_TOP)
      .addDefinition(CommonAttributes.MARGIN_BOTTOM)
      .addDefinition(CommonAttributes.BACKGROUND_COLOR)
      .addDefinition(LOGICAL_PAGENUMBERING)
      .addDefinition(LOGICAL_PAGETEXT);
  }
  /**
   * @see org.ujac.print.BaseDocumentTag#buildSupportedAttributes()
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.PAGE_SIZE)
      .addDefinition(CommonStyleAttributes.PAGE_WIDTH)
      .addDefinition(CommonStyleAttributes.PAGE_HEIGHT)
      .addDefinition(CommonStyleAttributes.ROTATE)
      .addDefinition(CommonStyleAttributes.MARGIN_LEFT)
      .addDefinition(CommonStyleAttributes.MARGIN_RIGHT)
      .addDefinition(CommonStyleAttributes.MARGIN_TOP)
      .addDefinition(CommonStyleAttributes.MARGIN_BOTTOM)
      .addDefinition(CommonStyleAttributes.BACKGROUND_COLOR)
      .addDefinition(LOGICAL_PAGENUMBERING)
      .addDefinition(LOGICAL_PAGETEXT);
  }

  /**
   * @see org.ujac.print.BaseDocumentTag#buildSupportedChilds()
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(BaseFontTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(MacroTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(IfTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ElseTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(LogTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(SwitchTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(HeaderTag.class, 0, 1))
      .addDefinition(new ChildDefinition(FooterTag.class, 0, 1))
      .addDefinition(new ChildDefinition(StickyTag.class, 0, 1));
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // setting the new page handler
    newPageHandler = (NewPageHandler) documentHandler.latestOfType(NewPageHandler.class);
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }
    
    queuedEventHandlers.clear();

    if (isAttributeDefined(CommonAttributes.LOGICAL_PAGENUMBERING)) {
      // Gets the value for the Logical Page Numbering Attribute.
      int logicalPageNumberStyle = this.getIntegerAttribute(CommonAttributes.LOGICAL_PAGENUMBERING, true, null);
      // setting the page number style at document handler
      documentHandler.setLogicalPageNumberStyle(logicalPageNumberStyle);
    }
    
    if (isAttributeDefined(CommonAttributes.LOGICAL_PAGETEXT)) {
      // Gets the value for the Logical Page Numbering Text Attribute.
      String logicalPageNumberText = this.getStringAttribute(CommonAttributes.LOGICAL_PAGETEXT, "", true, null);
      // setting the page number text at document handler
      documentHandler.setLogicalPageNumberText(logicalPageNumberText);
    }
  }
  
  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while closing the foreach item.
   */
  public void closeItem() throws DocumentHandlerException {

    if (!isValid()) {
      return;
    }

    Rectangle size = null;
    String sizeAttr = getStringAttribute(CommonAttributes.PAGE_SIZE, true, CommonStyleAttributes.PAGE_SIZE);
    if (sizeAttr != null) {
      try {
        Field sizeField = PageSize.class.getField(sizeAttr);
        size = (Rectangle) sizeField.get(null);
      } catch (Exception e) {
        throw new DocumentHandlerException(locator(), "Unable to set page size: " + e.getMessage(), e);
      }
    } else {
      if ((isAttributeDefined(CommonAttributes.PAGE_WIDTH, CommonStyleAttributes.PAGE_WIDTH)) && 
          (isAttributeDefined(CommonAttributes.PAGE_HEIGHT, CommonStyleAttributes.PAGE_HEIGHT))) {
        float width = getDimensionAttribute(CommonAttributes.PAGE_WIDTH, true, CommonStyleAttributes.PAGE_WIDTH);
        float height = getDimensionAttribute(CommonAttributes.PAGE_HEIGHT, true, CommonStyleAttributes.PAGE_HEIGHT);
        size = new Rectangle(width, height);
      }
    }

    String rotateAttr = getStringAttribute(CommonAttributes.PAGE_ROTATE, true, CommonStyleAttributes.PAGE_ROTATE);
    boolean rotate = false;
    if (rotateAttr != null) {
      rotate = new Boolean(rotateAttr).booleanValue(); 
    }
    
    Color bgColor = getColorAttribute(CommonAttributes.BACKGROUND_COLOR, true, CommonStyleAttributes.BACKGROUND_COLOR);
    
    boolean marginsDefined = isAttributeDefined(CommonAttributes.MARGIN_LEFT, CommonStyleAttributes.MARGIN_LEFT) &&
      isAttributeDefined(CommonAttributes.MARGIN_RIGHT, CommonStyleAttributes.MARGIN_RIGHT) &&
      isAttributeDefined(CommonAttributes.MARGIN_TOP, CommonStyleAttributes.MARGIN_TOP) &&
      isAttributeDefined(CommonAttributes.MARGIN_BOTTOM, CommonStyleAttributes.MARGIN_BOTTOM);
    float marginLeft = getFloatAttribute(CommonAttributes.MARGIN_LEFT, true, CommonStyleAttributes.MARGIN_LEFT);
    float marginRight = getFloatAttribute(CommonAttributes.MARGIN_RIGHT, true, CommonStyleAttributes.MARGIN_RIGHT);
    float marginTop = getFloatAttribute(CommonAttributes.MARGIN_TOP, true, CommonStyleAttributes.MARGIN_TOP);
    float marginBottom = getFloatAttribute(CommonAttributes.MARGIN_BOTTOM, true, CommonStyleAttributes.MARGIN_BOTTOM);
    
    if (size == null) {
      size = new Rectangle(documentHandler.getDocument().getPageSize());
    }
    
    if ((rotateAttr != null) && rotate) {
      // rotating document size.
      size = size.rotate();
    }
    if (bgColor != null) {
      size.setBackgroundColor(bgColor);
    }

    Rectangle margins = null;
    if (marginsDefined) {
      margins = new Rectangle(documentHandler.getDocument().getPageSize());
      margins.setBorderWidthLeft(marginLeft);
      margins.setBorderWidthRight(marginRight);
      margins.setBorderWidthTop(marginTop);
      margins.setBorderWidthBottom(marginBottom);
    }
    // letting the NewPageHandler perform the new-page request
    this.newPageHandler.handleNewPage(size, margins);
    // registering the queued page event handlers with the document handler
    Iterator iterQueuedEventHandlers = queuedEventHandlers.entrySet().iterator();
    while (iterQueuedEventHandlers.hasNext()) {
      Map.Entry entry = (Map.Entry) iterQueuedEventHandlers.next();
      documentHandler.setPageEventHandler(entry.getKey(), (PdfPageEvent) entry.getValue());
    }
  }

  
  /**
   * Queues the given page event handler for registration to the document 
   * event handler map, after the new-page has been performed. 
   * @param key The event handler key.
   * @param handler The event handler.
   */
  public void queuePageEventHandler(Object key, PdfPageEvent handler) {
    queuedEventHandlers.put(key,  handler);
  }
  
}
