/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import java.util.List;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.AttributeDefinition;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.DocumentPageEventHandler;
import org.ujac.print.OutlineItem;
import org.ujac.print.PdfHelper;
import org.ujac.print.TagAttributeException;
import org.ujac.print.TocItem;

import org.ujac.util.BeanUtils;

import org.ujac.util.template.TemplateException;

import com.lowagie.text.DocumentException;
import com.lowagie.text.Rectangle;
import com.lowagie.text.Document;

import com.lowagie.text.pdf.PdfPCell;
import com.lowagie.text.pdf.PdfPCellEvent;
import com.lowagie.text.pdf.PdfWriter;
import com.lowagie.text.pdf.PdfOutline;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfDestination;

/**
 * Name: OutlineTag<br>
 * Description: Item handler for PDF outlines.
 * 
 * @author lauerc
 */
public class OutlineTag extends BaseDocumentTag implements PdfPCellEvent {

  /** Definition of the 'name' attribute. */
  private static final AttributeDefinition NAME = CommonAttributes.NAME.cloneAttrDef(
      "The name of the outline, needed to link outlines with " +
    "each other and to build anchors which can be referenced from TOC."
  ).setRequired(false);
  /** Definition of the 'parent' attribute. */
  private static final AttributeDefinition PARENT = new AttributeDefinition(TagAttributes.ATTR_PARENT, 
      AttributeDefinition.TYPE_NAME, false, "The name of the parent outline.");
  /** Definition of the 'x' attribute. */
  private static final AttributeDefinition X = new AttributeDefinition(TagAttributes.ATTR_X, 
      AttributeDefinition.TYPE_DIMENSION, false, "The absolute horizontal position to jump to.");
  /** Definition of the 'y' attribute. */
  private static final AttributeDefinition Y = new AttributeDefinition(TagAttributes.ATTR_Y, 
      AttributeDefinition.TYPE_DIMENSION, false, "The absolute horizontal position to jump to.");
  /** Definition of the 'toc-number' attribute. */
  private static final AttributeDefinition TOC_NUMBER = new AttributeDefinition(TagAttributes.ATTR_TOC_NUMBER, 
      AttributeDefinition.TYPE_STRING, false, "The number to view in the table of contents.");
  
  /** The item's name. */
  public static final String TAG_NAME = "outline";

  /** The surrounding tag. */
  protected BaseDocumentTag parentTag = null; 
  /** The number to view in the table of contents. */
  private String tocNumber = null;
  /** The toc relevance. */
  private String tocRelevance = null;
  
  /**
   * Constructs a OutlineTag instance with no specific attributes.
   */
  public OutlineTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Adds a outline to the document.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.RENDERED)
      .addDefinition(NAME)
      .addDefinition(PARENT)
      .addDefinition(X)
      .addDefinition(Y)
      .addDefinition(TOC_NUMBER)
      .addDefinition(CommonAttributes.TOC_RELEVANCE);
  }
  
  /**
   * Tells whether textual content is allowed for the tag or not.
   * @return true if the item is allowed to contain textual content, else false.
   */
  public boolean isTextBodyAllowed() {
    return true;
  }
  
  /**
   * @see org.ujac.print.BaseDocumentTag#initialize()
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    
    // getting parent tag
    this.parentTag = documentHandler.latestRealTag();
  }
  

  /**
   * @see org.ujac.print.BaseDocumentTag#openItem()
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();
    
    tocNumber = getStringAttribute(TOC_NUMBER, null, true, null);
    tocRelevance = getStringAttribute(CommonAttributes.TOC_RELEVANCE, true, CommonStyleAttributes.TOC_RELEVANCE);
  }
  
  /**
   * @see org.ujac.print.BaseDocumentTag#closeItem()
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }
    
    if (parentTag instanceof BaseCellTag) {
      // waiting for the cell to be layouted
      ((BaseCellTag) parentTag).addCellEventHandler(this);
      return;
    }
    if (parentTag instanceof ParagraphTag) {
      // waiting for the paragraph to be layouted
      documentHandler.setPageEventHandler(parentTag, new OutlinePageEventHandler());
      return;
    }
    
    try {
      Document document = getDocumentHandler().getDocument();
      PdfWriter pdfWriter = (PdfWriter) getDocumentHandler().getDocumentWriter();
  
      float y = 0.0F;
      if (isAttributeDefined(Y)) {
        y = getDimensionAttribute(Y, true, null);
      } else {
        // trying to determine the current vertical position on page.
        y = PdfHelper.getCurrentVerticalPosition(pdfWriter) + document.bottomMargin() + document.bottom();
      }
    
      createOutline(y);
    } catch (DocumentException ex) {
      throw new DocumentHandlerException(locator(), "Unable to create outline: " + ex.getMessage(), ex);
    }
  }

  /**
   * Creates the outline at the given vertical position
   * @param y The vertical position of the outline.
   * @throws DocumentHandlerException
   */
  protected void createOutline(float y) throws DocumentHandlerException {
    PdfWriter pdfWriter = (PdfWriter) getDocumentHandler().getDocumentWriter();
    PdfContentByte cb = pdfWriter.getDirectContent();
    
    PdfOutline parent = cb.getRootOutline();
    if (isAttributeDefined(PARENT)) {
      parent = findOutline(parent, getStringAttribute(PARENT, true, null));
    }

    PdfDestination destination = null;
    float x = 0.0F;
    if (isAttributeDefined(X)) {
      x = getDimensionAttribute(X, true, null);
      destination = new PdfDestination(PdfDestination.XYZ, x, y, 0.0F);
    } else {
      destination = new PdfDestination(PdfDestination.FITH, y);
    }
    
    String title = null;
    try {
      title = documentHandler.executeTemplate(getContent());
    } catch (TemplateException ex) {
      throw new TagAttributeException(locator(), "Attribute evaluation failed: " + ex.getMessage(), ex);
    }
    
    PdfOutline outline = new PdfOutline(parent, destination, title);
    
    String name = null;
    if (isAttributeDefined(NAME)) {
      name = getStringAttribute(NAME, true, null);
      outline.setTag(name);
      if (documentHandler.isDestinationsSupported()) {
        cb.addOutline(outline, name);

        // creating anchor for outline
        cb.localDestination(name, destination);
      }
    }

    OutlineItem outlineItem = new OutlineItem(
        name, documentHandler.getPageNumber(pdfWriter), 
        x, y, parent.getTag(), title
    );
    documentHandler.addOutline(outlineItem);
    
    if ((tocNumber != null) || (tocRelevance != null)) {
      if (!BeanUtils.equals(tocRelevance, "index")) {
        if (tocNumber == null) {
          throw new TagAttributeException(locator(), "In case toc relevance includes table of contents, the attribute 'toc-number' has to be defined for tag '" + TAG_NAME + "'.");
        }
      }
      TocItem tocItem = new TocItem(
          1, tocNumber, title, tocRelevance != null ? tocRelevance : "all",
          documentHandler.getPageNumber(pdfWriter), y
      );
      tocItem.setOutline(outlineItem);
      documentHandler.addTocItem(tocItem);
    }
    
  }
  
  /**
   * Finds the outline with the given name in the outline tree below the given root outline.
   * @param rootOutline The root outline. 
   * @param name The name of the desired outline.
   * @return The detected outline or null.
   */
  private PdfOutline findOutline(PdfOutline rootOutline, String name) {
    String tag = rootOutline.getTag();
    if ((tag != null) && tag.equals(name)) {
      return rootOutline;
    }
    List childs = rootOutline.getKids();
    if (childs == null) {
      return null;
    }
    int numChilds = childs.size();
    for (int i = 0; i < numChilds; i++) {
      PdfOutline result = findOutline((PdfOutline) childs.get(i), name);
      if (result != null) {
        return result;
      }
    }
    return null;
  }

  /**
   * @see com.lowagie.text.pdf.PdfPCellEvent#cellLayout(com.lowagie.text.pdf.PdfPCell, com.lowagie.text.Rectangle, com.lowagie.text.pdf.PdfContentByte[])
   */
  public void cellLayout(PdfPCell cell, Rectangle position, PdfContentByte[] canvases) {
    try {
      createOutline(position.getTop());
    } catch (DocumentHandlerException ex) {
      ex.printStackTrace();
    }
  }

  
  /**
   * Name: OutlinePageEventHandler<br>
   * Description: Determines vertical positions and page numbers for outlines.<br>
   */
  public class OutlinePageEventHandler extends DocumentPageEventHandler {

    /** Tells whether or not the outline has been created. */
    private boolean outlineCreated = false;
    
    /**
     * Tells whether this event handler is temporary, and should be removed after the 
     * first usage or not.
     * @return true, if this handler should be executed only once, else false.
     */
    public boolean isTemporary() {
      return true;
    }
    
    /**
     * @see com.lowagie.text.pdf.PdfPageEventHelper#onParagraph(com.lowagie.text.pdf.PdfWriter, com.lowagie.text.Document, float)
     */
    public void onParagraph(PdfWriter writer, Document document, float position) {
      if (!outlineCreated && (parentTag instanceof ParagraphTag)) {
        try {
          createOutline(position);
          // making sure the outline is created only once!
          outlineCreated = true;
        } catch (DocumentHandlerException ex) {
          ex.printStackTrace();
        }
      }
    }
  }

}
