/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import java.awt.Color;

import java.lang.reflect.Field;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.DocumentHandlerException;

import com.lowagie.text.Document;
import com.lowagie.text.PageSize;
import com.lowagie.text.Rectangle;

/**
 * Name: PageFormatTag<br>
 * Description: A class handling &lt;page-format&gt; tags.
 * 
 * @author lauerc
 */
public class PageFormatTag extends BaseDocumentTag {

  /** The item's name. */
  public static final String TAG_NAME = "page-format";

  /** The document item. */
  protected DocumentTag documentItem = null;
  
  /**
   * Constructs a PageFormatTag instance with no specific attributes.
   */
  public PageFormatTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Defines the page format.";
  }

  /**
   * @see org.ujac.print.BaseDocumentTag#buildSupportedAttributes()
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.PAGE_SIZE)
      .addDefinition(CommonAttributes.PAGE_WIDTH)
      .addDefinition(CommonAttributes.PAGE_HEIGHT)
      .addDefinition(CommonAttributes.PAGE_ROTATE)
      .addDefinition(CommonAttributes.MARGIN_LEFT)
      .addDefinition(CommonAttributes.MARGIN_RIGHT)
      .addDefinition(CommonAttributes.MARGIN_TOP)
      .addDefinition(CommonAttributes.MARGIN_BOTTOM)
      .addDefinition(CommonAttributes.BACKGROUND_COLOR);
  }
  /**
   * @see org.ujac.print.BaseDocumentTag#buildSupportedAttributes()
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.PAGE_SIZE)
      .addDefinition(CommonStyleAttributes.PAGE_WIDTH)
      .addDefinition(CommonStyleAttributes.PAGE_HEIGHT)
      .addDefinition(CommonStyleAttributes.PAGE_ROTATE)
      .addDefinition(CommonStyleAttributes.MARGIN_LEFT)
      .addDefinition(CommonStyleAttributes.MARGIN_RIGHT)
      .addDefinition(CommonStyleAttributes.MARGIN_TOP)
      .addDefinition(CommonStyleAttributes.MARGIN_BOTTOM)
      .addDefinition(CommonStyleAttributes.BACKGROUND_COLOR);
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // setting the conditions
    documentItem = (DocumentTag) documentHandler.latestOfType(DocumentTag.TAG_NAME);
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the &lt;page-format&gt; item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    if (documentItem.getDocument(false) != null) {
      throw new DocumentHandlerException(locator(), "Page format already defined!");
    }
    
    Rectangle size = null;
    String sizeAttr = getStringAttribute(CommonAttributes.PAGE_SIZE, true, CommonStyleAttributes.PAGE_SIZE);
    if (sizeAttr != null) {
      try {
        Field sizeField = PageSize.class.getField(sizeAttr);
        size = (Rectangle) sizeField.get(null);
      } catch (Exception e) {
        throw new DocumentHandlerException(locator(), "Unable to set page size: " + e.getMessage(), e);
      }
    } else {
      if (isAttributeDefined(CommonAttributes.PAGE_WIDTH, CommonStyleAttributes.PAGE_WIDTH) && isAttributeDefined(CommonAttributes.PAGE_HEIGHT, CommonStyleAttributes.PAGE_HEIGHT)) {
        float width = getDimensionAttribute(CommonAttributes.PAGE_WIDTH, true, CommonStyleAttributes.PAGE_WIDTH);
        float height = getDimensionAttribute(CommonAttributes.PAGE_HEIGHT, true, CommonStyleAttributes.PAGE_HEIGHT);
        size = new Rectangle(width, height);
      }
    }

    if (size != null) {
      String rotateAttr = getStringAttribute(CommonAttributes.PAGE_ROTATE, true, CommonStyleAttributes.PAGE_ROTATE);
      boolean rotate = false;
      if (rotateAttr != null) {
        rotate = new Boolean(rotateAttr).booleanValue(); 
      }
      
      Color bgColor = getColorAttribute(CommonAttributes.BACKGROUND_COLOR, true, CommonStyleAttributes.BACKGROUND_COLOR);
      
      float marginLeft = getFloatAttribute(CommonAttributes.MARGIN_LEFT, true, CommonStyleAttributes.MARGIN_LEFT);
      float marginRight = getFloatAttribute(CommonAttributes.MARGIN_RIGHT, true, CommonStyleAttributes.MARGIN_RIGHT);
      float marginTop = getFloatAttribute(CommonAttributes.MARGIN_TOP, true, CommonStyleAttributes.MARGIN_TOP);
      float marginBottom = getFloatAttribute(CommonAttributes.MARGIN_BOTTOM, true, CommonStyleAttributes.MARGIN_BOTTOM);
      
      if (rotate) {
        // rotating document size.
        size = size.rotate();
      }
      if (bgColor != null) {
        size.setBackgroundColor(bgColor);
      }
      
      // creating document
      Document document = new Document(size, marginLeft, marginRight, marginTop, marginBottom);
      // setting document at document handler
      documentHandler.setDocument(document);
      
      // registering the document with the surrounding document tag
      documentItem.setDocument(document);
    }
  }
  
  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while closing the &lt;page-format&gt; tag.
   */
  public void closeItem() throws DocumentHandlerException {
  }

}
