/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.ParameterDefHolder;
import org.ujac.print.ParameterDefinition;
import org.ujac.util.UjacTypes;

/**
 * Name: ParameterDefTag<br>
 * Description: A class handling ParameterDefinitions.
 * 
 * @author lauerc
 */
public class ParameterDefTag extends BaseDocumentTag {

  /** Definition of the 'list' attribute. */
  private static final AttributeDefinition LIST_FLAG = new AttributeDefinition(TagAttributes.ATTR_LIST, 
      AttributeDefinition.TYPE_BOOLEAN, false, "The list flag.");
  /** Definition of the 'optional' attribute. */
  private static final AttributeDefinition OPTIONAL_FLAG = new AttributeDefinition(TagAttributes.ATTR_OPTIONAL, 
      AttributeDefinition.TYPE_BOOLEAN, false, "The optional flag.");
  /** Definition of the 'default-value' attribute. */
  private static final AttributeDefinition DEFAULT_VALUE = new AttributeDefinition(TagAttributes.ATTR_DEFAULT_VALUE, 
      AttributeDefinition.TYPE_STRING, false, "The default value, this only makes sense if the " +
      "optional flag is defined as 'true'.");
  /** Definition of the 'type' attribute. */
  private static final AttributeDefinition PARAM_TYPE = new AttributeDefinition(TagAttributes.ATTR_TYPE, 
      new String[] {
        UjacTypes.TYPE_NAME_INT, "integer", UjacTypes.TYPE_NAME_FLOAT, UjacTypes.TYPE_NAME_LONG, UjacTypes.TYPE_NAME_DOUBLE,
        UjacTypes.TYPE_NAME_DATE, UjacTypes.TYPE_NAME_TIME, UjacTypes.TYPE_NAME_TIMESTAMP, UjacTypes.TYPE_NAME_STRING,
        "text", UjacTypes.TYPE_NAME_OBJECT
      }, 
      false, "The attribute type.");
  /** Definition of the 'description' attribute. */
  private static final AttributeDefinition PARAM_DESCRIPTION = new AttributeDefinition(TagAttributes.ATTR_DESCRIPTION, 
      AttributeDefinition.TYPE_STRING, false, "A description for the paremeter.");

  /** The item's name. */
  public static final String TAG_NAME = "parameter-def";
  
  /** The macro item. */
  private ParameterDefHolder parameterDefHolder = null;
  /** The name attribute. */
  private String name = null;
  /** The type attribute. */
  private String typeAttr = null;
  /** The type ID. */
  private int type = -1;
  /** The list flag. */
  private boolean list = false;
  /** The optional flag. */
  private boolean optional = false;
  /** The default value. */
  private Object defaultValue = null;
  /** The parameter description. */
  private String description = null;

  /**
   * Constructs a ParameterDefTag instance with no specific attributes.
   */
  public ParameterDefTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Adds a parameter definition to its surrounding parameter definition holder." +
      "This could be a macro definition for example.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.PARAM_NAME)
      .addDefinition(LIST_FLAG)
      .addDefinition(OPTIONAL_FLAG)
      .addDefinition(DEFAULT_VALUE)
      .addDefinition(PARAM_TYPE)
      .addDefinition(PARAM_DESCRIPTION);
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // getting parents
    this.parameterDefHolder = (ParameterDefHolder) documentHandler.latestOfType(ParameterDefHolder.class);
    if (parameterDefHolder == null) {
      throw new DocumentHandlerException(locator(), "Item '" + getName() + "' is not supported here.");
    }
    // getting attributes
    this.name = getStringAttribute(CommonAttributes.PARAM_NAME, false, null);
    this.typeAttr = getStringAttribute(PARAM_TYPE, UjacTypes.TYPE_NAME_STRING, false, null).toLowerCase();
    if (UjacTypes.TYPE_NAME_STRING.equals(typeAttr) || "text".equals(typeAttr)) {
      type = UjacTypes.TYPE_STRING;
    } else if (UjacTypes.TYPE_NAME_BOOLEAN.equals(typeAttr)) {
      type = UjacTypes.TYPE_BOOLEAN;
    } else if (UjacTypes.TYPE_NAME_INT.equals(typeAttr) || "integer".equals(typeAttr)) {
      type = UjacTypes.TYPE_INT;
    } else if (UjacTypes.TYPE_NAME_FLOAT.equals(typeAttr)) {
      type = UjacTypes.TYPE_FLOAT;
    } else if (UjacTypes.TYPE_NAME_DOUBLE.equals(typeAttr)) {
      type = UjacTypes.TYPE_DOUBLE;
    } else if (UjacTypes.TYPE_NAME_DATE.equals(typeAttr)) {
      type = UjacTypes.TYPE_DATE;
    } else if (UjacTypes.TYPE_NAME_TIME.equals(typeAttr)) {
      type = UjacTypes.TYPE_TIME;
    } else if (UjacTypes.TYPE_NAME_TIMESTAMP.equals(typeAttr)) {
      type = UjacTypes.TYPE_TIMESTAMP;
    } else if (UjacTypes.TYPE_NAME_OBJECT.equals(typeAttr)) {
      type = UjacTypes.TYPE_OBJECT;
    } else {
      throw new DocumentHandlerException(locator(), "Detected unsupported object type '" + this.typeAttr + "' at attribute '" + this.name + "'.");
    }
    
    String attr = getStringAttribute(DEFAULT_VALUE, false, null);
    this.defaultValue = typeCastValue(attr, typeAttr);

    if (isAttributeDefined(OPTIONAL_FLAG)) {
      this.optional = getBooleanAttribute(OPTIONAL_FLAG, false, null);
    } else {
      this.optional = true;
    }
    this.description = getStringAttribute(PARAM_DESCRIPTION, true, null);
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the macro-param item.
   */
  public void closeItem() throws DocumentHandlerException {
    // adding evaluated value to the surrounding macro item.
    parameterDefHolder.setParameterDef(new ParameterDefinition(
      this.name, this.type, this.list, 
      this.optional, this.defaultValue, 
      description
    ));
  }
}
