/*
 * Copyright (C) 2003 - 2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import java.util.ArrayList;
import java.util.List;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.ChildDefinition;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.ItemHolder;
import org.ujac.print.ParameterHolder;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.ParameterDefinition;
import org.ujac.print.TagAttributeException;
import org.ujac.print.DocumentHandlerException;

import org.ujac.util.UjacTypes;
import org.ujac.util.exi.ExpressionException;

import org.ujac.util.template.TemplateException;

/**
 * Name: ParameterTag<br>
 * Description: A class handling &lt;parameter&gt; tags.
 * @author lauerc
 */
public class ParameterTag extends BaseDocumentTag implements ItemHolder {

  /** The item's name. */
  public static final String TAG_NAME = "parameter";

  /** The macro item. */
  private ParameterHolder parameterHolder = null;
  /** The name attribute. */
  private String name = null;
  /** The item list. */
  private List items = null;

  /**
   * Constructs a ParameterTag instance with no specific attributes.
   */
  public ParameterTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Adds a parameter to its surrounding parameter holder." +
      "This could be a macro for example. There must be a according parameter definition for " +
      "each parameter. The parameter value is determined from the item's body.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.PARAM_NAME);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(ItemsTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(ItemTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(IfTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ElseTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ForeachTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(SwitchTag.class, 0, ChildDefinition.UNLIMITED));
  }

  /**
   * Tells whether textual content is allowed for the tag or not.
   * @return true if the item is allowed to contain textual content, else false.
   */
  public boolean isTextBodyAllowed() {
    return true;
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // getting parents
    this.parameterHolder = (ParameterHolder) documentHandler.latestOfType(ParameterHolder.class);
    if (parameterHolder == null) {
      throw new DocumentHandlerException(locator(), "Item '" + getName() + "' is not supported here.");
    }
    // getting attributes
    this.name = getStringAttribute(CommonAttributes.PARAM_NAME, false, null);
  }
  
  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    // resetting items list
    items = null;
  }


  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the macro-param item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }
    
    // checking parameter definition
    ParameterDefinition paramDef = parameterHolder.getParameterDefinition(this.name);
    if (paramDef == null) {
      throw new TagAttributeException(locator(), "Detected unsupported parameter '" + this.name + "'.");
    }
    String expr = getContent();
    Object value = null;
    
    if (items != null) {
      value = items;
    } else if (paramDef.isList()) {
      value = expr;
    } else {
      try {
        // evaluating content
        switch (paramDef.getType()) {
          case UjacTypes.TYPE_BOOLEAN:
            value = new Boolean(documentHandler.evalBoolean(expr));
            break;
    
          case UjacTypes.TYPE_INT:
            value = new Integer(documentHandler.evalInt(expr));
            break;
    
          case UjacTypes.TYPE_FLOAT:
            value = new Float(documentHandler.evalFloat(expr));
            break;
    
          case UjacTypes.TYPE_DOUBLE:
            value = new Double(documentHandler.evalDouble(expr));
            break;
    
          case UjacTypes.TYPE_DATE:
            value = documentHandler.evalDate(expr);
            break;
    
          case UjacTypes.TYPE_STRING:
          case ParameterDefinition.TYPE_COLOR:
          case ParameterDefinition.TYPE_FONT:
          case ParameterDefinition.TYPE_FORMAT:
            if (!paramDef.isList()) {
              value = documentHandler.executeTemplate(expr);
              break;
            }
    
          default:
            value = documentHandler.evalObject(expr);
            break;
        }
      } catch (TemplateException ex) {
        throw new TagAttributeException(locator(), "Attribute evaluation failed: " + ex.getMessage(), ex);
      }
    }
    // adding evaluated value to the surrounding macro item.
    parameterHolder.setParameter(this.name, value);
  }

  /**
   * @see org.ujac.print.ItemHolder#addItem(java.lang.Object)
   */
  public void addItem(Object item) throws TagAttributeException {
    // parsing item value
    if (items == null) {
      items = new ArrayList();
    }
    items.add(item);
  }

  /**
   * Parses a value from the given text according to the rules of the item holder's container.
   * @param text The text to parse.
   * @return The parsed value.
   * @exception TagAttributeException In case the adding of the item has been failed.
   */
  public Object parseItem(String text) throws TagAttributeException {

    ParameterDefinition paramDef = parameterHolder.getParameterDefinition(this.name);
    if (paramDef == null) {
      throw new TagAttributeException(locator(), "Detected unsupported parameter '" + this.name + "'.");
    }

    try {
      // evaluating content
      Object value = null;
      try {
        switch (paramDef.getType()) {
          case UjacTypes.TYPE_BOOLEAN:
            value = new Boolean(documentHandler.evalBoolean(text));
            break;
    
          case UjacTypes.TYPE_INT:
            value = new Integer(documentHandler.evalInt(text));
            break;
    
          case UjacTypes.TYPE_FLOAT:
            value = new Float(documentHandler.evalFloat(text));
            break;
    
          case UjacTypes.TYPE_DOUBLE:
            value = new Double(documentHandler.evalDouble(text));
            break;
    
          case UjacTypes.TYPE_DATE:
            value = documentHandler.evalDate(text);
            break;
    
          case UjacTypes.TYPE_STRING:
          case ParameterDefinition.TYPE_FONT:
          case ParameterDefinition.TYPE_FORMAT:
            value = documentHandler.evalString(text);
            break;
  
          case ParameterDefinition.TYPE_COLOR:
            value = colorFromString(name, text, true);
            break;
    
          default:
            value = documentHandler.evalObject(text);
            break;
        }
      } catch (ExpressionException ex) {
        throw new TagAttributeException(locator(), "Attribute evaluation failed: " + ex.getMessage(), ex);
      }
      return value;
    } catch (DocumentHandlerException ex) {
      throw new TagAttributeException(ex.getLocator(), ex.getMessage(), ex.getOriginalCause());
    }
  }

}
