/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import java.io.IOException;

import org.ujac.print.DocumentHandler;

import com.lowagie.text.BadElementException;
import com.lowagie.text.Image;
import com.lowagie.text.Rectangle;

import com.lowagie.text.pdf.PdfImportedPage;
import com.lowagie.text.pdf.PdfReader;
import com.lowagie.text.pdf.PdfWriter;

/**
 * Name: PdfImageBuilder<br>
 * Description: ImageBuilder implementation for PDF format.
 * 
 * @author lauerc
 */
public class PdfImageBuilder implements ImageBuilder {

  /**
   * Checks, if the implementation is capable of handling the given image data or not.
   * @param imageData The image data to handle.
   * @return true if this implementation can build a image from the given data, else false.
   */
  public boolean checkFormat(byte[] imageData) {
    if (imageData.length < 5) {
      return false;
    }
    return ((imageData[0] == '%') &&
            (imageData[1] == 'P') &&
            (imageData[2] == 'D') &&
            (imageData[3] == 'F') &&
            (imageData[4] == '-'));
  }
  
  /**
   * Builds an image from the given data.
   * @param documentHandler The document handler.
   * @param imageData The image data to handle.
   * @param path The path, where the image is located at the resoure loader.
   * @param width The desired output width for the image (zero if unspecified).
   * @param height The desired output height for the image (zero if unspecified).
   * @param page The number of the page to use as image.
   * @return The generated image.
   * @exception IOException In case the image creation failed.
   */
  public Image buildImage(DocumentHandler documentHandler, byte[] imageData, 
                          String path, float width, float height, int page) 
    throws IOException {
    
    try {
      PdfWriter pdfWriter = (PdfWriter) documentHandler.getDocumentWriter();
      PdfReader reader = new PdfReader(imageData);
      if ((page < 0) || (page > reader.getNumberOfPages())) {
        throw new IOException("Page number for imported Pdf-Image is out of range: " + page);
      }
    
      // reading first page from given page.
      PdfImportedPage importedPage = pdfWriter.getImportedPage(reader, page);

      // fixing image position:
      // in case the document border has been cutted at the bottom and the 
      // left edge, iText doesn't take care of it. I think this is a bug.
      // In case this bug in iText will is fixed sometimes in the future,
      // the following lines have to get removed!
      Rectangle bbox = importedPage.getBoundingBox();
      float left = bbox.getLeft();
      float bottom = bbox.getBottom();
      if ((left != 0.0F) || (bottom != 0.0F)) {
        importedPage.setMatrix(1, 0, 0, 1, -left, -bottom);
      }
      
      // creating image from PDF.
      return Image.getInstance(importedPage);
    } catch (BadElementException ex) {
      throw new IOException("Unable to create PDF image: " + ex.getMessage());
    }
  }

}
