/*
 * Copyright (C) 2005 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.ChildDefinition;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.FontHolder;
import org.ujac.print.PdfCellContainer;
import org.ujac.print.PdfTableHolder;
import org.ujac.print.StyleHolder;

import com.lowagie.text.pdf.PdfPCell;
import com.lowagie.text.pdf.PdfPTable;

/**
 * Name: PdfTableRowTag<br>
 * Description: A class handling &lt;table-row&gt; tags. This tag enables  
 *   an easier control of the table cells, because it automatically adds
 *   missing table fields which helps to avoid messages like 
 *   'the latest row ist not filled comletely ...'.<br>
 * 
 * @author lauerc
 */
public class PdfTableRowTag extends BaseDocumentTag implements PdfCellContainer, FontHolder, StyleHolder {

  /** The item's name. */
  public static final String TAG_NAME = "table-row";

  /** The current cell attributes. */
  private CellAttributes cellAttributes = null;

  /** The table item. */
  private PdfTableTag tableItem = null;
  /** The cell counter for the row. */
  private int numCells = 0;
    
  /**
   * Constructs a PdfTableRowTag instance with no specific attributes.
   */
  public PdfTableRowTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Controls the cells of a single row for the surrounding table. It automatically " +
        "fills up the row with empty cells in case not all cells, definef by the table layout " +
        "have been defined inside the row.";
  }
  
  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.RENDERED)
      .addDefinition(CommonAttributes.CLASS)
      .addDefinition(CommonAttributes.STYLE)
      .addDefinition(CommonAttributes.FONT)
      .addDefinition(CommonAttributes.LEADING)
      .addDefinition(CommonAttributes.LINE_SPACING)
      .addDefinition(CommonAttributes.BACKGROUND_COLOR)
      .addDefinition(CommonAttributes.BORDER_STYLE)
      .addDefinition(CommonAttributes.BORDER_WIDTH)
      .addDefinition(CommonAttributes.BORDER_COLOR)
      .addDefinition(CommonAttributes.HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonAttributes.VERTICAL_ALIGNMENT)
      .addDefinition(CommonAttributes.PADDING)
      .addDefinition(CommonAttributes.PADDING_LEFT)
      .addDefinition(CommonAttributes.PADDING_RIGHT)
      .addDefinition(CommonAttributes.PADDING_TOP)
      .addDefinition(CommonAttributes.PADDING_BOTTOM)
      .addDefinition(CommonAttributes.CELL_HEIGHT)
      .addDefinition(CommonAttributes.NO_WRAP);
  }
  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.FONT_NAME)
      .addDefinition(CommonStyleAttributes.LEADING)
      .addDefinition(CommonStyleAttributes.LINE_SPACING)
      .addDefinition(CommonStyleAttributes.BACKGROUND_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_STYLE)
      .addDefinition(CommonStyleAttributes.BORDER_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_LEFT_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_RIGHT_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_TOP_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_BOTTOM_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_LEFT_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_RIGHT_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_TOP_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_BOTTOM_COLOR)
      .addDefinition(CommonStyleAttributes.HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.VERTICAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.PADDING)
      .addDefinition(CommonStyleAttributes.PADDING_LEFT)
      .addDefinition(CommonStyleAttributes.PADDING_RIGHT)
      .addDefinition(CommonStyleAttributes.PADDING_TOP)
      .addDefinition(CommonStyleAttributes.PADDING_BOTTOM)
      .addDefinition(CommonStyleAttributes.CELL_HEIGHT)
      .addDefinition(CommonStyleAttributes.NO_WRAP);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(BaseFontTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(MacroTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(IfTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(ElseTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(LogTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(SwitchTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(ForeachTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(PdfCellTag.class, 0, ChildDefinition.UNLIMITED));
  }

  /**
   * Tells whether or not this tag does respect style attributes or not.
   * @return always <code>true</code>.
   */
  public boolean isStyleable() {
    return true;
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // getting the table item
    this.tableItem = (PdfTableTag) documentHandler.latestOfType(PdfTableTag.class);
    if (tableItem == null) {
      throw new DocumentHandlerException(locator(), "The table-header item must be located below a table!");
    }
  }
  
  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    numCells = 0;

    if (!isValid()) {
      return;
    }

    // inherriting the surrounding cell's attributes
    cellAttributes = new CellAttributes(getCommonAttributesHolder());
    cellAttributes.parseRowAttributes(this, tableItem.getCellAttributes());
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while closing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    // adding table to latest element container
    int numRemainingCells = tableItem.getNumColumns() - numCells;
    if (numRemainingCells > 0) {
      PdfPCell filler = new PdfPCell();
      cellAttributes.setupCell(filler);
      filler.setColspan(numRemainingCells);
      tableItem.addCell(this, filler);
    }
  }

  /**
   * Adds a PDF cell to the container.
   * @param item The document item.
   * @param cell The PDF cell to be added.
   * @exception DocumentHandlerException If something went wrong when adding the cell.
   */
  public void addCell(BaseDocumentTag item, PdfPCell cell) throws DocumentHandlerException {
    // incrementing header cell counter
    numCells += Math.max(cell.getColspan(), 1);
    if (numCells > tableItem.getNumColumns()) {
      throw new DocumentHandlerException(locator(), "The row size exceeds the number of table columns.");
    }
    // adding cell to table
    tableItem.addCell(item, cell);
  }
  
  /**
   * Gets the number of remaining columns of the current row.
   * @return The number of remaining column to the end of the current row.
   */
  public int getRemainingPdfColumns() {
    int tableColumns = tableItem.getNumColumns();
    return tableColumns - (numCells % tableColumns); 
  }

  /**
   * Gets the PDF table holder.
   * @return The component that really holds a PDF table.
   */
  public PdfTableHolder getPdfTable() {
    return tableItem;
  }

  /**
   * Gets the iText table.
   * @return The iText table.
   */
  public PdfPTable getTable() {
    return tableItem.getTable();
  }

  /**
   * Gets the cell attributes.
   * @return The cell attributes.
   */
  public CellAttributes getCellAttributes() {
    return cellAttributes;
  }
  
  /**
   * Gets the line spacing.
   * @return The current line spacing.
   */
  public float getLineSpacing() {
    if (cellAttributes != null) {
      return cellAttributes.getLineSpacing();
    }
    return DEFAULT_LINE_SPACING;
  }
  
  /**
   * Gets the leading.
   * @return The current leading.
   */
  public float getLeading() {
    if (cellAttributes != null) {
      return cellAttributes.getLeading();
    }
    return DEFAULT_LEADING;
  }

}