/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.AttributeDefinition;
import org.ujac.print.DocumentHandlerException;

/**
 * Name: NewPageTag<br>
 * Description: Tag to force a page break at the document.<br>
 * 
 * @author lauerc
 */
public class PdfTableSplitTag extends BaseDocumentTag {

  /** Definition of the 'new-page' attribute. */
  private static final AttributeDefinition NEW_PAGE = new AttributeDefinition(TagAttributes.ATTR_NEW_PAGE, 
      AttributeDefinition.TYPE_BOOLEAN, false, "Flag telling whether to add a page break before continuing the " +
      "table or not.");
  /** Definition of the 'keep-header' attribute. */
  private static final AttributeDefinition KEEP_HEADER = new AttributeDefinition(TagAttributes.ATTR_KEEP_HEADER, 
      AttributeDefinition.TYPE_BOOLEAN, false, "Flag telling whether to keep the currently defined header " +
      "of the table or to reset the header.");

  /** The item's name. */
  public static final String TAG_NAME = "table-split";

  /** The table item. */
  private PdfTableTag tableItem = null;
  /** The new-page attribute. */
  private boolean newPage = false;
  /** The keep-header attribute. */
  private boolean keepHeader = false;

  /**
   * Constructs PdfTableSplitTag BoldTag instance with no specific attributes.
   */
  public PdfTableSplitTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Splits the surrounding table. This can be useful, if you like to break up a table or " +
      "want to continue the table on the next page.";
  }

  /**
   * @see org.ujac.print.BaseDocumentTag#buildSupportedAttributes()
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.RENDERED)
      .addDefinition(NEW_PAGE) 
      .addDefinition(KEEP_HEADER);
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // setting the parent item
    this.tableItem = (PdfTableTag) documentHandler.latestOfType(PdfTableTag.TAG_NAME);
    // setting the attributes
    this.newPage = getBooleanAttribute(NEW_PAGE, false, null);
    this.keepHeader = getBooleanAttribute(KEEP_HEADER, false, null);
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    tableItem.splitTable(newPage, keepHeader);
  }
  
  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while closing the foreach item.
   */
  public void closeItem() throws DocumentHandlerException {
  }
}
