/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import java.util.List;
import java.util.ArrayList;
import java.util.StringTokenizer;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.ChildDefinition;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.DocumentHandler;
import org.ujac.print.ElementContainer;
import org.ujac.print.FontHolder;
import org.ujac.print.PdfCellContainer;
import org.ujac.print.PdfHelper;
import org.ujac.print.PdfTableHolder;
import org.ujac.print.StyleHolder;
import org.ujac.print.TagAttributeException;
import org.ujac.print.DocumentHandlerException;
import org.ujac.util.exi.ExpressionException;

import org.xml.sax.Locator;

import com.lowagie.text.DocumentException;

import com.lowagie.text.pdf.PdfPCell;
import com.lowagie.text.pdf.PdfPTable;
import com.lowagie.text.pdf.PdfWriter;

/**
 * Name: PdfTableTag<br>
 * Description: An item handling PDF tables.<br>
 * 
 * @author lauerc
 */
public class PdfTableTag extends BaseElementTag implements PdfCellContainer, PdfTableHolder, FontHolder, StyleHolder {

  /** The item's name. */
  public static final String TAG_NAME = "table";

  /** Definition of the 'bgcolor' attribute. */
  private static final AttributeDefinition BACKGROUND_COLOR =
      CommonAttributes.BACKGROUND_COLOR.cloneAttrDef("Defines the default background color for the nested cells.");
  /** Definition of the 'padding-bottom' style attribute. */
  private static final AttributeDefinition BACKGROUND_COLOR_STYLE =
      CommonStyleAttributes.BACKGROUND_COLOR.cloneAttrDef("Defines the default background color for the nested cells.");

  /** Definition of the 'border-style' attribute. */
  private static final AttributeDefinition BORDER_STYLE =
      CommonAttributes.BORDER_STYLE.cloneAttrDef("Defines the default border type for the nested cells.");
  /** Definition of the 'border-style' style attribute. */
  private static final AttributeDefinition BORDER_STYLE_STYLE =
      CommonStyleAttributes.BORDER_STYLE.cloneAttrDef("Defines the default border type for the nested cells.");

  /** Definition of the 'border-style' attribute. */
  private static final AttributeDefinition BORDER_WIDTH =
      CommonAttributes.BORDER_WIDTH.cloneAttrDef("Defines the default border width for the nested cells.");
  /** Definition of the 'border-style' style attribute. */
  private static final AttributeDefinition BORDER_WIDTH_STYLE =
      CommonStyleAttributes.BORDER_WIDTH.cloneAttrDef("Defines the default border width for the nested cells.");

  /** Definition of the 'border-color' attribute. */
  private static final AttributeDefinition BORDER_COLOR =
      CommonAttributes.BORDER_COLOR.cloneAttrDef("Defines the default border color for the nested cells.");
  /** Definition of the 'border-color' style attribute. */
  private static final AttributeDefinition BORDER_COLOR_STYLE =
      CommonStyleAttributes.BORDER_COLOR.cloneAttrDef("Defines the default border color for the nested cells.");

  /** Definition of the 'padding' attribute. */
  private static final AttributeDefinition PADDING =
      CommonAttributes.PADDING.cloneAttrDef("Defines the default padding for the nested cells.");
  /** Definition of the 'padding' style attribute. */
  private static final AttributeDefinition PADDING_STYLE =
      CommonStyleAttributes.PADDING.cloneAttrDef("Defines the default padding for the nested cells.");

  /** Definition of the 'padding-left' attribute. */
  private static final AttributeDefinition PADDING_LEFT =
      CommonAttributes.PADDING_LEFT.cloneAttrDef("Defines the default left padding for the nested cells.");
  /** Definition of the 'padding-left' style attribute. */
  private static final AttributeDefinition PADDING_LEFT_STYLE =
      CommonStyleAttributes.PADDING_LEFT.cloneAttrDef("Defines the default left padding for the nested cells.");

  /** Definition of the 'padding-right' attribute. */
  private static final AttributeDefinition PADDING_RIGHT =
      CommonAttributes.PADDING_RIGHT.cloneAttrDef("Defines the default right padding for the nested cells.");
  /** Definition of the 'padding-right' style attribute. */
  private static final AttributeDefinition PADDING_RIGHT_STYLE =
      CommonStyleAttributes.PADDING_RIGHT.cloneAttrDef("Defines the default right padding for the nested cells.");

  /** Definition of the 'padding-top' attribute. */
  private static final AttributeDefinition PADDING_TOP =
      CommonAttributes.PADDING_TOP.cloneAttrDef("Defines the default top padding for the nested cells.");
  /** Definition of the 'padding-top' style attribute. */
  private static final AttributeDefinition PADDING_TOP_STYLE =
      CommonStyleAttributes.PADDING_TOP.cloneAttrDef("Defines the default top padding for the nested cells.");

  /** Definition of the 'padding-bottom' attribute. */
  private static final AttributeDefinition PADDING_BOTTOM =
      CommonAttributes.PADDING_BOTTOM.cloneAttrDef("Defines the default bottom padding for the nested cells.");
  /** Definition of the 'padding-bottom' style attribute. */
  private static final AttributeDefinition PADDING_BOTTOM_STYLE =
      CommonStyleAttributes.PADDING_BOTTOM.cloneAttrDef("Defines the default bottom padding for the nested cells.");

  /** 
   * The maximum number of lines for a table segment.
   * In case this size is reached, an automnatic table
   * split is performed to keep the memory consumption low.  
   */
  public static final int MAX_SEGMENT_SIZE = 100;

  /** The latest pdf cell container. */
  private PdfCellContainer pdfCellContainer = null;
  /** The current cell attributes. */
  private CellAttributes cellAttributes = null;

  /** The iText table. */
  private PdfPTable table = null;
  /** The latest child item. */
  private BaseDocumentTag latestChild = null;
  /** The header item list. */
  private List headerItems = new ArrayList();

  /** The number of cells, printed in the current output segment. */
  private int numCells = 0;
  /** The number of rows, printed in the current output segment. */
  private int numRows = 0;
  /** The number of table columns. */
  private int columns = 0;
  /** The width of the whole table. */
  private float width = 0.0f;
  /** The widths attribute. */
  private String widthsAttr = null;
  /** The widths of the table columns. */
  private float[] widths = null;
  /** The horizontal alignment of the table. */ 
  private int hAlign = 0;
  /** Tells whether to split rows or not. */
  private boolean splitRows = false;
  /** 
   * Tells whether to split rows late or not. 
   * If true the row will only split if it's the first one in an empty page.
   */
  private boolean splitLate = true;

  /** The spacing-before attribute. */
  private float spacingBefore = 0.0f;
  /** The spacing-after attribute. */
  private float spacingAfter = 0.0f;
  /** Tells whether or not the table's contents should be kept together on one page. */
  private boolean keepTogether = false;

  /**
   * Constructs a PdfTableTag instance with no specific attributes.
   */
  public PdfTableTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Adds a table to its parent item. Tables are very useful if you want to layout your " +
      "document nicely.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.CLASS)
      .addDefinition(CommonAttributes.STYLE)
      .addDefinition(CommonAttributes.FONT)
      .addDefinition(CommonAttributes.LEADING)
      .addDefinition(CommonAttributes.LINE_SPACING)
      .addDefinition(CommonAttributes.TABLE_COLUMNS)
      .addDefinition(CommonAttributes.TABLE_WIDTHS)
      .addDefinition(CommonAttributes.TABLE_WIDTH)
      .addDefinition(CommonAttributes.HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonAttributes.CELL_HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonAttributes.CELL_VERTICAL_ALIGNMENT)
      .addDefinition(BACKGROUND_COLOR)
      .addDefinition(BORDER_STYLE)
      .addDefinition(BORDER_WIDTH)
      .addDefinition(BORDER_COLOR)
      .addDefinition(PADDING)
      .addDefinition(PADDING_LEFT)
      .addDefinition(PADDING_RIGHT)
      .addDefinition(PADDING_TOP)
      .addDefinition(PADDING_BOTTOM)
      .addDefinition(CommonAttributes.TABLE_SPLIT_ROWS)
      .addDefinition(CommonAttributes.TABLE_SPLIT_LATE)
      .addDefinition(CommonAttributes.SPACING_BEFORE)
      .addDefinition(CommonAttributes.SPACING_AFTER)
      .addDefinition(CommonAttributes.KEEP_TOGETHER);
  }

  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.FONT_NAME)
      .addDefinition(CommonStyleAttributes.LEADING)
      .addDefinition(CommonStyleAttributes.LINE_SPACING)
      .addDefinition(CommonStyleAttributes.TABLE_WIDTH)
      .addDefinition(CommonStyleAttributes.HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.CELL_HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.CELL_VERTICAL_ALIGNMENT)
      .addDefinition(BACKGROUND_COLOR_STYLE)
      .addDefinition(BORDER_STYLE_STYLE)
      .addDefinition(BORDER_WIDTH_STYLE)
      .addDefinition(BORDER_COLOR_STYLE)
      .addDefinition(PADDING_STYLE)
      .addDefinition(PADDING_LEFT_STYLE)
      .addDefinition(PADDING_RIGHT_STYLE)
      .addDefinition(PADDING_TOP_STYLE)
      .addDefinition(PADDING_BOTTOM_STYLE)
      .addDefinition(CommonStyleAttributes.TABLE_SPLIT_ROWS)
      .addDefinition(CommonStyleAttributes.TABLE_SPLIT_LATE)
      .addDefinition(CommonStyleAttributes.SPACING_BEFORE)
      .addDefinition(CommonStyleAttributes.SPACING_AFTER)
      .addDefinition(CommonStyleAttributes.KEEP_TOGETHER);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(BaseFontTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(MacroTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(IfTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(ElseTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(LogTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(SwitchTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(ForeachTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(TocTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(IndexTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(PdfTableHeaderTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(PdfTableRowTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(PdfCellTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(PdfTableSplitTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(PdfCellBlockTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ImportTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(OutlineTag.class, 0, ChildDefinition.UNLIMITED));
  }

  /**
   * Tells whether or not this tag does respect style attributes or not.
   * @return always <code>true</code>.
   */
  public boolean isStyleable() {
    return true;
  }

  /**
   * Gets the number of columns, defined for the table.
   * @return The number of columns that have been defined for the table.
   */
  public int getNumColumns() {
    return columns;
  }
  
  /**
   * Gets the number of content rows, that have been added to the current table so far.
   * @return The number of content rows, that have been added to the current table so far.
   */
  public int getNumContentRows() {
    if (table == null) {
      return 0;
    }
    return table.size() - table.getHeaderRows();
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    
    // getting latest pdf cell container
    pdfCellContainer = documentHandler.latestPdfCellContainer();
  }
  
  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    // inherriting the surrounding cell's attributes
    cellAttributes = new CellAttributes(getCommonAttributesHolder());
    
    CellAttributes parentAttributes = null;
    if (pdfCellContainer != null) {
      parentAttributes = pdfCellContainer.getCellAttributes();
    }
    cellAttributes.parseTableAttributes(this, parentAttributes);
    
    columns = getIntegerAttribute(CommonAttributes.TABLE_COLUMNS, true, null);
    widthsAttr = getStringAttribute(CommonAttributes.TABLE_WIDTHS, true, null);
    width = getFloatAttribute(CommonAttributes.TABLE_WIDTH, 100.0F, true, CommonStyleAttributes.TABLE_WIDTH);
    spacingBefore = getDimensionAttribute(CommonAttributes.SPACING_BEFORE, 0.0f, true, CommonStyleAttributes.SPACING_BEFORE);
    spacingAfter = getDimensionAttribute(CommonAttributes.SPACING_AFTER, 0.0f, true, CommonStyleAttributes.SPACING_AFTER);
    keepTogether = getBooleanAttribute(CommonAttributes.KEEP_TOGETHER, false, true, CommonStyleAttributes.KEEP_TOGETHER);

    if (widthsAttr != null) {
      StringTokenizer tkn = null;
      try {
        tkn = new StringTokenizer(documentHandler.evalString(widthsAttr), ",");
      } catch (ExpressionException ex) {
        throw new TagAttributeException(locator(), "Attribute evaluation failed: " + ex.getMessage(), ex);
      }
      
      columns = tkn.countTokens();
      widths = new float[columns];
      int idx = 0;
      while (tkn.hasMoreTokens()) {
        String token = tkn.nextToken();
        try {
          widths[idx] = Float.parseFloat(token);
          ++idx;
        } catch (NumberFormatException ex) {
          throw new TagAttributeException(locator(), "Attribute widths must hold a list of floating point numbers, separated by ',' (was '" + token + "').", ex);
        }
      }
    } else if (isAttributeDefined(CommonAttributes.TABLE_COLUMNS)) {
      widths = new float[columns];
      float averageWidth = 100.0f / columns;
      for (int i = 0; i < columns; i++) {
        widths[i] = averageWidth;
      }
    } else {
      throw new TagAttributeException(locator(), "Eighter attribute '" + TagAttributes.ATTR_WIDTHS + "' or '" + 
                                      TagAttributes.ATTR_COLUMNS + "' has to be defined for tag <" + getName() + ">.");
    }
    hAlign = getHorizontalAlignmentAttribute(CommonAttributes.HORIZONTAL_ALIGNMENT, true, CommonStyleAttributes.HORIZONTAL_ALIGNMENT);

    splitRows = getBooleanAttribute(CommonAttributes.TABLE_SPLIT_ROWS, false, CommonStyleAttributes.TABLE_SPLIT_ROWS);
    splitLate = getBooleanAttribute(CommonAttributes.TABLE_SPLIT_LATE, false, CommonStyleAttributes.TABLE_SPLIT_LATE);

    if (columns == 0) {
      throw new TagAttributeException(locator(), "Tables have to hold at least one column!");
    }

    openItem(true);
  }
  
  /**
   * Opens the item.
   * @param resetData tells whether to reset some specific data or not (don't blame for this silly comment).
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem(boolean resetData) throws DocumentHandlerException {

    if (resetData) {
      // resetting cell counter
      numCells = 0;
      // clearing header items
      headerItems.clear();
    }
    
    if (!isValid()) {
      return;
    }

    // creating the table  
    table = new PdfPTable(columns);
    
    // configuring the table
    if (width > 0.0f) {
      table.setWidthPercentage(width);
    }
    if (widths != null) {
      try {
        // setting table widths
        table.setWidths(widths);
      } catch (DocumentException ex) {
        throw new DocumentHandlerException(
            locator(), "Invalid table widths:'" + 
            getStringAttribute(CommonAttributes.TABLE_WIDTHS, true, null) + 
            "': " + ex.getMessage(), ex
        );
      }
    }
    // setting the horizntal aligmnemt
    table.setHorizontalAlignment(hAlign);
    // setting the split-rows flag
    table.setSplitRows(splitRows);
    table.setSplitLate(splitLate);
    
    if (resetData) {
      // defining the spacing-before for the first table segment
      table.setSpacingBefore(spacingBefore);
    }
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while closing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    // flushes the current table segment
    flushTableSegment(true);
  }

  /**
   * Flushes the current table segment.
   * @param lastSegment Tells whether this is the last segment or not.
   * @exception DocumentHandlerException Thrown in case something went wrong while performing the table output.
   */
  private void flushTableSegment(boolean lastSegment) throws DocumentHandlerException {
    // checking if the last row was filled completely
    if ((numCells % columns) > 0) {
      Locator locator = locator();
      if (latestChild != null) {
        locator = latestChild.locator();
      }
      throw new DocumentHandlerException(locator, "The last row is not filled completely, check the table cells!");
    }
    
    if (lastSegment) {
      // defining the spacing-after for the last table segment
      table.setSpacingAfter(spacingAfter);
    }
    if (keepTogether) {
      if (!fitsPage()) {
        documentHandler.getDocument().newPage();
      }
    }
    // adding table to latest element container
    ElementContainer elementContainer = getElementContainer();
    elementContainer.addElement(this, table);

    // resetting cell & row counter
    numCells = 0;
    numRows = 0;
  }

  /**
   * Adds a cell to the table.
   * @param item The tag which performed the cell adding.
   * @param cell The cell to be added.
   * @exception DocumentHandlerException If something went wrong when adding the cell.
   */
  public void addCell(BaseDocumentTag item, PdfPCell cell) throws DocumentHandlerException {
    // setting the last child item
    latestChild = item;
    // incrementing header cell counter
    numCells += Math.max(cell.getColspan(), 1);
    // adding cell to table
    table.addCell(cell);

    if (numCells % columns == 0) {
      ++numRows;
      if (!keepTogether && (numRows >= MAX_SEGMENT_SIZE)) {
        // performing a table split here to keep the memory consumption low
        splitTable(false, true);
      }
    }
  }

  /**
   * Gets the number of remaining columns of the current row.
   * @return The number of remaining column to the end of the current row.
   */
  public int getRemainingPdfColumns() {
    return columns - (numCells % columns); 
  }

  /**
   * Gets the PDF table holder.
   * @return The component that really holds a PDF table.
   */
  public PdfTableHolder getPdfTable() {
    return this;
  }

  /**
   * Adds a header cell to the table.
   * @param item The tag which performed the cell adding.
   * @param cell The header cell to be added.
   * @exception DocumentHandlerException If something went wrong when adding the header cell.
   */
  public void addHeaderCell(BaseDocumentTag item, PdfPCell cell) throws DocumentHandlerException {
    // adding cell as usual
    addCell(item, cell);
    // adding cell to header list
    headerItems.add(cell);
  }
  
  /**
   * Gets the iText table.
   * @return The iText table.
   */
  public PdfPTable getTable() {
    return table;
  }

  /**
   * Checks whether the table fits the current page or not. 
   * @return true if there's room enough for the table on the current page, else false.
   * @throws DocumentHandlerException In case something went wrong.
   */
  public boolean fitsPage() throws DocumentHandlerException {
    DocumentHandler documentHandler = getDocumentHandler();
    PdfWriter writer = (PdfWriter) documentHandler.getDocumentWriter();
    try {
      return PdfHelper.fitsPage(writer, documentHandler.getDocument(), getTable());
    } catch (DocumentException ex) {
      throw new DocumentHandlerException(locator(), "Fits page check failed: " + ex.getMessage(), ex);
    }
  }

  /**
   * Splits the table vertically.
   * @param newPage Tells whether to start a new page or not.
   * @param keepHeader Tells whether to keep the old header or not.
   * @return true if a split has been really performed, else false.
   * @exception DocumentHandlerException If something went wrong while splitting the table.
   */
  public boolean splitTable(boolean newPage, boolean keepHeader) throws DocumentHandlerException {
    if (!isValid()) {
      return false;
    }

    if (keepHeader && !newPage && (this.numCells < 1)) {
      // nothing 2 do!
      return false;
    }

    int numHeaderRows = table.getHeaderRows();
    // flushes the current table segment
    flushTableSegment(false);

    if (newPage) {
      try {
        documentHandler.getDocument(true).newPage();
      } catch (Exception ex) {
        throw new DocumentHandlerException(locator(), "Unable to open a new page: " + ex.getMessage(), ex);
      }
    }
    // starts the new table
    openItem(false);
    if (keepHeader) {
      if (!newPage) {
        table.setSkipFirstHeader(true);
      }
      int numHeaderItems = headerItems.size();
      for (int i = 0; i < numHeaderItems; i++) {
        PdfPCell headerItem = (PdfPCell) headerItems.get(i);
        table.addCell(headerItem);
      }
      table.setHeaderRows(numHeaderRows);
    } else {
      // clearing header items
      headerItems.clear();
    }
    return true;
  }

  /**
   * Gets the cell attributes.
   * @return The cell attributes.
   */
  public CellAttributes getCellAttributes() {
    return cellAttributes;
  }
  
  /**
   * Gets the line spacing.
   * @return The current line spacing.
   */
  public float getLineSpacing() {
    if (cellAttributes != null) {
      return cellAttributes.getLineSpacing();
    }
    return DEFAULT_LINE_SPACING;
  }
  
  /**
   * Gets the leading.
   * @return The current leading.
   */
  public float getLeading() {
    if (cellAttributes != null) {
      return cellAttributes.getLeading();
    }
    return DEFAULT_LEADING;
  }

}
