/*
 * Copyright (C) 2003 - 2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.CommonAttributesHolder;
import org.ujac.print.DocumentFont;
import org.ujac.print.FontHolder;
import org.ujac.print.PhraseHolder;
import org.ujac.print.ChildDefinition;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.ElementContainer;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.StyleHolder;

import com.lowagie.text.Chunk;
import com.lowagie.text.Phrase;
import com.lowagie.text.Element;
import com.lowagie.text.SplitCharacter;
import com.lowagie.text.pdf.PdfChunk;

/**
 * Name: PhraseTag<br>
 * Description: A class handling &lt;phrase&gt; tags.<br>
 * 
 * @author lauerc
 */
public class PhraseTag extends BaseElementTag implements ElementContainer, PhraseHolder, FontHolder, StyleHolder {

  /** The item's name. */
  public static final String TAG_NAME = "phrase";

  /** Split character implementation that allows no automatical split at any character. */
  public static final SplitCharacter NO_SPLIT = new SplitCharacter() {
    /**
     * Disallowing automatical split for all tested characters.
     * @param start the lower limit of <CODE>cc</CODE> inclusive
     * @param current the pointer to the character in <CODE>cc</CODE>
     * @param end the upper limit of <CODE>cc</CODE> exclusive
     * @param cc an array of characters at least <CODE>end</CODE> sized
     * @param ck an array of <CODE>PdfChunk</CODE>.
     * @return Always <CODE>false</CODE>!
     */
    public boolean isSplitCharacter(int start, int current, int end, char cc[], PdfChunk ck[]) {
      return false;
    }
  };
  
  /** 
   * The absolute leading, to be added to the variable leading amount, 
   * resulting from the line spacing factor.
   */
  private float leading = DEFAULT_LEADING;
  /** The line spacing factor. */
  private float lineSpacing = DEFAULT_LINE_SPACING;
  /** The rotation angle. */
  private float rotate = 0.0f;
  /** 
   * Tells whether or not to disallow to automatical wrap the contents of 
   * the phrase around multiple lines. Setting this attribute to true
   * makes the tag to ignore all font manipulations that are defined
   * within the contents. 
   */
  private boolean noWrap = false;
  /** The phrase. */
  private Phrase phrase = null;

  /**
   * Constructs a PhraseTag instance with no specific attributes.
   */
  public PhraseTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Adds a phrase to its parent item.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.CLASS)
      .addDefinition(CommonAttributes.STYLE)
      .addDefinition(CommonAttributes.FONT)
      .addDefinition(CommonAttributes.LEADING)
      .addDefinition(CommonAttributes.LINE_SPACING)
      .addDefinition(CommonAttributes.ROTATE)
      .addDefinition(CommonAttributes.TRIM_BODY)
      .addDefinition(CommonAttributes.NO_WRAP)
      .addDefinition(CommonAttributes.BACKGROUND_COLOR);
  }

  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.FONT_NAME)
      .addDefinition(CommonStyleAttributes.LEADING)
      .addDefinition(CommonStyleAttributes.LINE_SPACING)
      .addDefinition(CommonStyleAttributes.ROTATE)
      .addDefinition(CommonStyleAttributes.NO_WRAP)
      .addDefinition(CommonStyleAttributes.BACKGROUND_COLOR);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(BaseFontTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(MacroTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(IfTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(ElseTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(LogTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(SwitchTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(ForeachTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(PhraseTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(NumberFormatTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(DateFormatTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(AnchorTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(AnnotationTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ChartTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ImageTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(SpaceTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(NewLineTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BarcodeTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(DatamatrixTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BarcodePdf417Tag.class, 0, ChildDefinition.UNLIMITED));
  }

  /**
   * Tells whether or not this tag does respect style attributes or not.
   * @return always <code>true</code>.
   */
  public boolean isStyleable() {
    return true;
  }

  /**
   * Tells whether textual content is allowed for the tag or not.
   * @return true if the item is allowed to contain textual content, else false.
   */
  public boolean isTextBodyAllowed() {
    return true;
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    this.phrase = null;

    if (!isValid()) {
      return;
    }
    
    // getting attributes
    CommonAttributesHolder commonAttributesHolder = getCommonAttributesHolder();
    if (isAttributeDefined(CommonAttributes.LEADING, CommonStyleAttributes.LEADING)) {
      this.leading = getDimensionAttribute(CommonAttributes.LEADING, true, CommonStyleAttributes.LEADING);
    } else {
      this.leading = commonAttributesHolder.getLeading();
    }    
    if (isAttributeDefined(CommonAttributes.LINE_SPACING, CommonStyleAttributes.LINE_SPACING)) {
      this.lineSpacing = getDimensionAttribute(CommonAttributes.LINE_SPACING, true, CommonStyleAttributes.LINE_SPACING);
    } else {
      this.lineSpacing = commonAttributesHolder.getLineSpacing();
    }
    this.rotate = getFloatAttribute(CommonAttributes.ROTATE, 0.0f, true, CommonStyleAttributes.ROTATE);
    this.noWrap = getBooleanAttribute(CommonAttributes.NO_WRAP, false, true, CommonStyleAttributes.NO_WRAP);
    if (isAttributeSupported(CommonAttributes.TRIM_BODY)) {
      setTrimBody(getBooleanAttribute(CommonAttributes.TRIM_BODY, true, true, null));
    }
  }

  /**
   * Gets the actual content.
   * This method is intended for testing purposes. 
   * @return The last content.
   */
  public String getActualContent() {
    if (phrase == null) {
      return super.getActualContent();
    }
    return getPhraseContent(phrase);
  }
  
  /**
   * Flushes the content buffer.
   * @exception DocumentHandlerException Thrown in case something went wrong while flushing the content buffer.
   */
  public void flushContent() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    if (phrase == null) {
      createPhrase();
    }
    if (isIgnoreFlush()) {
      // don't flush twice!
      return;
    }

    if (isTrimNextHead()) {
      trimContentHead();
      setTrimNextHead(false);
    }
    if (isItemClosed() && isTrimBody()) {
      trimContentTail();
    }

    Phrase nestedPhrase = addChunk(phrase, getLeading(), getLineSpacing());
    if (nestedPhrase != null) {
      phrase = nestedPhrase;
    }
    resetContent();
  }

  /**
   * Processes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    flushContent();
    
    if (noWrap && (phrase != null)) {
      int numChunks = phrase.size();
      if (numChunks > 1) {
        StringBuffer contentBuf = new StringBuffer();
        for (int i = 0; i < numChunks; i++) {
          Chunk chunk = (Chunk) phrase.get(i);
          contentBuf.append(chunk.getContent());
        }
        Chunk chunk = (Chunk) phrase.get(0);
        chunk = new Chunk(contentBuf.toString(), chunk.getFont());
        chunk.setSplitCharacter(NO_SPLIT);
        phrase.clear();
        phrase.add(chunk);
      } else if (numChunks == 1) {
        Chunk chunk = (Chunk) phrase.get(0);
        chunk.setSplitCharacter(NO_SPLIT);
      }
    }
    
    if (rotate != 0.0F) {
      phrase = rotatePhrase(phrase, rotate);
    }
    ElementContainer elementContainer = getElementContainer();
    elementContainer.addElement(this, phrase);
  }

  /**
   * Adds the given chunk to the root phrase.
   * @param item The document item.
   * @param chunk The chunk to add to the root phrase.
   * @param font The font to use.
   * @exception DocumentHandlerException In case the document processing failed.
   */
  public void addChunk(BaseDocumentTag item, String chunk, DocumentFont font) 
    throws DocumentHandlerException {
    if (phrase == null) {
      createPhrase();
    }
    addChunk(item, phrase, chunk, font, getLeading(), getLineSpacing());
  }

  /**
   * Adds the given chunk to the root phrase.
   * @param item The document item.
   * @param chunk The chunk to add to the root phrase.
   * @exception DocumentHandlerException In case the document processing failed.
   */
  public void addChunk(BaseDocumentTag item, Chunk chunk)
    throws DocumentHandlerException {
    
    if (phrase == null) {
      createPhrase();
    }
    addChunk(phrase, chunk, getLeading(), getLineSpacing());
  }

  /**
   * Gets the fixed vertical leading distance.
   * @return The current fixed vertical leading distance.
   */
  public float getLeading() {
    return leading;
  }

  /**
   * Gets the line spacing.
   * @return The current line spacing.
   */
  public float getLineSpacing() {
    return lineSpacing;
  }

  /**
   * Adds a element to the container.
   * @param item The document item.
   * @param element The element to be added.
   * @exception DocumentHandlerException If something went wrong when adding the element.
   */
  public void addElement(BaseDocumentTag item, Element element)
    throws DocumentHandlerException {
    
    if (phrase == null) {
      createPhrase();
    }
    phrase.add(element);
    if (element instanceof Phrase) {
      extendLeading(phrase, (Phrase) element);
    }
  }

  /**
   * Tells whether the element container is a top level container or not.
   * @return true if the element container is a top level container, else false.
   */
  public boolean isTopLevel() {
    return false;
  }

  /**
   * Creates a new phrase.
   */
  private void createPhrase() {
    phrase = new Phrase(0.0F);
  }

}
