/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import java.text.Format;

import java.util.ArrayList;
import java.util.List;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.ChildDefinition;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.Condition;
import org.ujac.print.DocumentFont;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.FontHolder;
import org.ujac.print.StyleHolder;

import org.ujac.util.BeanUtils;
import org.ujac.util.UjacTypes;

import org.ujac.util.table.Column;
import org.ujac.util.table.LayoutHints;
import org.ujac.util.table.Row;
import org.ujac.util.table.TableConstants;
import org.ujac.util.table.TableException;

import org.ujac.util.template.TemplateException;
import org.ujac.util.text.FormatHelper;

import com.lowagie.text.Element;
import com.lowagie.text.Phrase;

import com.lowagie.text.pdf.PdfPCell;

/**
 * Name: PrintRowTag<br>
 * Description: A class printing the contents of the current row of the surrounding 
 *    print-table tag.<br>
 *              
 * @author lauerc
 */
public class PrintRowTag 
  extends BaseDocumentTag 
  implements FontHolder, Condition, RowCellContainer, StyleHolder {

  /** The item's name. */
  public static final String TAG_NAME = "print-row";

  /** The surrounding print-table tag. */
  protected PrintTableTag rowContainer = null;
  /** The current cell attributes. */
  private CellAttributes cellAttributes = null;
  
  /** The rotation angle. */
  protected float rotate = 0.0f;
  
  /** The registered row cells. */
  protected RowCellTag[] registeredRowCells = null;
  /** The extra rows to print before the data row. */
  protected List extraRowsBefore = null;
  /** The extra rows to print after the data row. */
  protected List extraRowsAfter = null;

  /**
   * Constructs a PrintRowTag instance with no specific attributes.
   */
  public PrintRowTag() {
    super(TAG_NAME);
  }
  /**
   * Constructs a PrintRowTag instance with specific attributes.
   * @param name The tag name.
   */
  public PrintRowTag(String name) {
    super(name);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Prints the contents of normal table rows.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.RENDERED)
      .addDefinition(CommonAttributes.CLASS)
      .addDefinition(CommonAttributes.STYLE)
      .addDefinition(CommonAttributes.FONT)
      .addDefinition(CommonAttributes.LEADING)
      .addDefinition(CommonAttributes.LINE_SPACING)
      .addDefinition(CommonAttributes.BACKGROUND_COLOR)
      .addDefinition(CommonAttributes.BORDER_STYLE)
      .addDefinition(CommonAttributes.BORDER_WIDTH)
      .addDefinition(CommonAttributes.BORDER_COLOR)
      .addDefinition(CommonAttributes.HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonAttributes.VERTICAL_ALIGNMENT)
      .addDefinition(CommonAttributes.PADDING)
      .addDefinition(CommonAttributes.PADDING_LEFT)
      .addDefinition(CommonAttributes.PADDING_RIGHT)
      .addDefinition(CommonAttributes.PADDING_TOP)
      .addDefinition(CommonAttributes.PADDING_BOTTOM)
      .addDefinition(CommonAttributes.CELL_HEIGHT)
      .addDefinition(CommonAttributes.ROTATE)
      .addDefinition(CommonAttributes.NO_WRAP);
  }

  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.FONT_NAME)
      .addDefinition(CommonStyleAttributes.LEADING)
      .addDefinition(CommonStyleAttributes.LINE_SPACING)
      .addDefinition(CommonStyleAttributes.BACKGROUND_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_STYLE)
      .addDefinition(CommonStyleAttributes.BORDER_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_LEFT_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_RIGHT_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_TOP_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_BOTTOM_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_LEFT_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_RIGHT_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_TOP_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_BOTTOM_COLOR)
      .addDefinition(CommonStyleAttributes.HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.VERTICAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.PADDING)
      .addDefinition(CommonStyleAttributes.PADDING_LEFT)
      .addDefinition(CommonStyleAttributes.PADDING_RIGHT)
      .addDefinition(CommonStyleAttributes.PADDING_TOP)
      .addDefinition(CommonStyleAttributes.PADDING_BOTTOM)
      .addDefinition(CommonStyleAttributes.CELL_HEIGHT)
      .addDefinition(CommonStyleAttributes.ROTATE)
      .addDefinition(CommonStyleAttributes.NO_WRAP);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(IfTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ElseTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(LogTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(SwitchTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(ForeachTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(BaseFontTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(MacroTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(ExtraRowTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(RowCellTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(SetPropertyTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(TogglePropertyTag.class, 0, ChildDefinition.UNLIMITED));
  }

  /**
   * Tells whether or not this tag does respect style attributes or not.
   * @return always <code>true</code>.
   */
  public boolean isStyleable() {
    return true;
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // setting the cell container.
    rowContainer = (PrintTableTag) documentHandler.latestOfType(PrintTableTag.class);
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the foreach item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }
    
    // inherriting the surrounding cell's attributes
    cellAttributes = new CellAttributes(getCommonAttributesHolder());
    cellAttributes.parseRowAttributes(this, rowContainer.getCellAttributes());
   
    rotate = getFloatAttribute(CommonAttributes.ROTATE, 0.0f, true, CommonStyleAttributes.ROTATE);

    registeredRowCells = new RowCellTag[rowContainer.getNumColumns()];
    extraRowsBefore = null;
    extraRowsAfter = null;
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while closing the foreach item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    if (isTrue()) {
      Row currentRow = (Row) rowContainer.getCurrentValue();

      // performing output of leading extra rows.
      if (extraRowsBefore != null) {
        List extraRows = extraRowsBefore;
        int numExtraRows = extraRows.size();
        for (int i = 0; i < numExtraRows; i++) {
          ExtraRowTag extraRow = (ExtraRowTag) extraRows.get(i);
          extraRow.outputRow();
        }
      }
      
      // performing output of data row.
      outputRow(currentRow);

      // performing output of trailing extra rows.
      if (extraRowsAfter != null) {
        List extraRows = extraRowsAfter;
        int numExtraRows = extraRows.size();
        for (int i = 0; i < numExtraRows; i++) {
          ExtraRowTag extraRow = (ExtraRowTag) extraRows.get(i);
          extraRow.outputRow();
        }
      }
    }
  }
  
  /**
   * Inserts a extra row.
   * @param extraRow The extra row to insert.
   * @param insertDirection The insert direction.
   */
  public void insertExtraRow(ExtraRowTag extraRow, int insertDirection) {
    List extraRowList = null;
    if (insertDirection == PrintTableTag.DIRECTION_BEFORE) {
      if (extraRowsBefore == null) {
        extraRowsBefore = new ArrayList();
      }
      extraRowList = extraRowsBefore;
    } else {
      if (extraRowsAfter == null) {
        extraRowsAfter = new ArrayList();
      }
      extraRowList = extraRowsAfter;
    }
    extraRowList.add(extraRow);
  }
  
  /**
   * Performs the row output.
   * @param currentRow The row to print.
   * @exception DocumentHandlerException Thrown in case something went wrong while closing the foreach item.
   */
  public void outputRow(Row currentRow) throws DocumentHandlerException {

    try {
      // creating the cells
      Column[] columns = rowContainer.getVisibleColumns();
      for (int i = 0; i < columns.length; i++) {
        Column column = columns[i];
        
        int columnIdx = column.getIndex();
        
        int colspan = 1;
        boolean fillRow = false;
        CellAttributes attrs = (CellAttributes) cellAttributes.clone();
        int hAlign = -1;
        if (columnIdx != -1) {
          switch (currentRow.getAlign(columnIdx)) {
          case LayoutHints.LEFT:
            hAlign = Element.ALIGN_LEFT;
            break;
          case LayoutHints.RIGHT:
            hAlign = Element.ALIGN_RIGHT;
            break;
          case LayoutHints.CENTER:
            hAlign = Element.ALIGN_CENTER;
            break;
          default:
            hAlign = Element.ALIGN_LEFT;
            break;
          }
        }
        attrs.setHAlign(hAlign);
        
        DocumentFont font = getFont();
        float rotate = this.rotate;
        String formatString = null;
        
        PdfPCell cell = null;
        RowCellTag rowCell = registeredRowCells[i]; 

        if (rowCell != null) {
          CellAttributes rowCellAttributes = rowCell.getCellAttributes();
          attrs.merge(rowCellAttributes);
          if (rowCell.getCell() != null) {
            cell = rowCell.getCell();
            colspan = cell.getColspan();
            fillRow = rowCellAttributes.isFillRow();
            if (fillRow) {
              colspan = rowContainer.getRemainingPdfColumns();
            }
            attrs.setColspan(colspan);
            attrs.setupCell(cell);
          } else {
            colspan = rowCellAttributes.getColspan();
            fillRow = rowCellAttributes.isFillRow();
            if (fillRow) {
              colspan = rowContainer.getRemainingPdfColumns();
            }
            font = rowCell.getFont();
            formatString = rowCell.getFormat();
            rotate = rowCell.getRotate();
          }
        }

        if (cell == null) {
          String cellContent = null;
          if ((columnIdx != -1) && currentRow.isVisible(columnIdx)) {
            if (formatString == null) {
              String defaultContent = null;
              if (isHeaderRow()) {
                if (column != null) {
                  defaultContent = column.getLayoutHints().getHeader();
                }
              } else {
                defaultContent = rowContainer.getDefaultContent(i);
              }
              if (!BeanUtils.isEmpty(defaultContent)) {
                try {
                  cellContent = documentHandler.executeTemplate(defaultContent);
                } catch (TemplateException ex) {
                  throw new DocumentHandlerException(locator(), "Failed to evaluate default cell content: " + ex.getMessage(), ex);
                }
              } else {
                // using standard formatting.
                cellContent = currentRow.formatValue(columnIdx);
              }
            } else {
              FormatHelper formatHelper = documentHandler.getFormatHelper();
              Format format = documentHandler.lookupFormat(formatString);
              Object cellObject = currentRow.getObject(columnIdx);
              if (format == null) {
                switch (column.getType()) {
                  case UjacTypes.TYPE_INT: 
                  case UjacTypes.TYPE_LONG: 
                  case UjacTypes.TYPE_FLOAT:
                  case UjacTypes.TYPE_DOUBLE:
                    format = formatHelper.createNumberFormat(formatString);
                    break;
                  case UjacTypes.TYPE_DATE:
                  case UjacTypes.TYPE_TIME:
                  case UjacTypes.TYPE_TIMESTAMP:
                    format = formatHelper.createDateFormat(formatString);
                    break;
                }
                // registering font at document handler
                documentHandler.registerFormat(formatString, format);
              }              
              // formatting the cell's content
              cellContent = format.format(cellObject);
            }
          } else {
            String defaultContent = null;
            if (isHeaderRow()) {
              if (column != null) {
                defaultContent = column.getLayoutHints().getHeader();
              }
            } else {
              defaultContent = rowContainer.getDefaultContent(i);
            }
            if (!BeanUtils.isEmpty(defaultContent)) {
              try {
                cellContent = documentHandler.executeTemplate(defaultContent);
              } catch (TemplateException ex) {
                throw new DocumentHandlerException(locator(), "Failed to evaluate default cell content: " + ex.getMessage(), ex);
              }
            }
          }
          if ((cellContent == null) && isHeaderRow()) {
            cellContent = column.getLayoutHints().getHeader();
          }
          Phrase cellPhrase = this.addChunk(
              this, new Phrase(0.0F), cellContent, 
              font, attrs.getLeading(), 
              attrs.getLineSpacing()
          );
          if (rotate != 0.0F) {
            cellPhrase = rotatePhrase(cellPhrase, rotate);
          }
          cell = new PdfPCell(cellPhrase);
        
          // configuring the cell
          attrs.setLeading(0.0F);
          attrs.setColspan(colspan);
          attrs.setupCell(cell);
        }
        if (colspan > 1) {
          // colspan has been defined, skipping the spanned columns
          i += colspan - 1;
        }
        // adding cell to latest cell container
        if (isHeaderRow()) {
          rowContainer.addHeaderCell(this, cell);
        } else {
          rowContainer.addCell(this, cell);
        }
      }
    } catch (TableException ex) {
      throw new DocumentHandlerException(locator(), "Failed to access table cell: " + ex.getMessage(), ex);
    }
  }

  /**
   * Registers the given cell style for the specified column name.
   * @param rowCell The row cell to register.
   * @exception DocumentHandlerException In case the row-cell registration failed.
   */
  public void registerRowCell(RowCellTag rowCell) throws DocumentHandlerException {
    int index = rowCell.getColumnIndex();
    if (index < 0) {
      index = this.rowContainer.getColumnIndex(rowCell.getColumnName());
    }
    if (index < 0) {
      throw new DocumentHandlerException(locator(), "Unable to register row-cell for undefined column '" + rowCell.getColumnName() + "'.");
    }
    if (index >= this.registeredRowCells.length) {
      throw new DocumentHandlerException(locator(), "Unable to register row-cell: index " + index + " is out of range.");
    }
    this.registeredRowCells[index] = rowCell;
  }

  /**
   * @see org.ujac.print.Condition#isTrue()
   */
  public boolean isTrue() {
    Row currentRow = (Row) rowContainer.getCurrentValue();
    return (currentRow != null) && TableConstants.ROW_TYPE_NORMAL.equals(currentRow.getType());
  }
  
  /**
   * Tells whether the row is a header row or not.
   * @return true in case the row is a header row, else false.
   */
  protected boolean isHeaderRow() {
    return false;
  }

  /**
   * Gets the cell attributes.
   * @return The cell attributes.
   */
  public CellAttributes getCellAttributes() {
    return cellAttributes;
  }
  
}
