/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.ChildDefinition;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.Condition;
import org.ujac.print.DocumentHandler;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.ElementContainer;
import org.ujac.print.FontHolder;
import org.ujac.print.PdfCellContainer;
import org.ujac.print.PdfHelper;
import org.ujac.print.PdfTableHolder;
import org.ujac.print.StyleHolder;

import org.ujac.util.UjacTypes;
import org.ujac.util.table.Column;
import org.ujac.util.table.ColumnNotDefinedException;
import org.ujac.util.table.DefaultColumn;
import org.ujac.util.table.GroupRow;
import org.ujac.util.table.LayoutHints;
import org.ujac.util.table.Row;
import org.ujac.util.table.Table;
import org.ujac.util.table.TableConstants;
import org.ujac.util.table.TableException;
import org.ujac.util.template.TemplateContext;
import org.ujac.util.text.FormatHelper;
import org.ujac.web.tag.PrintTableTag.DummyColumn;

import org.xml.sax.Locator;

import com.lowagie.text.DocumentException;

import com.lowagie.text.pdf.PdfPCell;
import com.lowagie.text.pdf.PdfPTable;
import com.lowagie.text.pdf.PdfWriter;

/**
 * Name: PrintTableTag<br>
 * Description: A class iterating over a given table and repeats the 
 *    nested body for each element of the table.<br>
 * 
 * @author lauerc
 */
public class PrintTableTag 
  extends BaseLoopTag 
  implements PdfCellContainer, PdfTableHolder, FontHolder, Condition, StyleHolder {

  /**
   * Name: DummyColumn<br>
   * Description: A class for dummy columns which are added through the insert-column tag.
   */
  public class DummyColumn extends DefaultColumn {

    /** The serial version UID. */
    private static final long serialVersionUID = 3618696418123134000L;

    /**
     * Constructs a DummyColumn instance with specific attributes.
     * @param name The column name.
     */
    protected DummyColumn(String name) {
      super(null, name, -1);
    }
    
    /**
     * Writes the object's data to the given stream.
     * @param s The stream to write to
     * @exception IOException In case the data output failed.
     */
    private void writeObject(ObjectOutputStream s) throws IOException {
      writeData(s);
    }
    /**
     * Reads the object's data from the given stream.
     * @param s The stream to read from.
     * @exception IOException In case the data reading failed.
     * @exception ClassNotFoundException In case the class to deserialize could not be found
     */
    private void readObject(ObjectInputStream s) throws IOException, ClassNotFoundException  {
      readData(s);
    }
    
  }


  /** Definition of the 'print-header' attribute. */
  private static final AttributeDefinition PRINT_HEADER = new AttributeDefinition(TagAttributes.ATTR_PRINT_HEADER, 
      AttributeDefinition.TYPE_BOOLEAN, false, "Tells whether to print the header row or not.");
  /** Definition of the 'level-filter' attribute. */
  private static final AttributeDefinition LEVEL_FILTER = new AttributeDefinition(TagAttributes.ATTR_LEVEL_FILTER, 
      AttributeDefinition.TYPE_INT, false, "The maximum level of importance to support, any column " +
      "with a higher level than the specified one is hidden.");

  /** Constant for the insert direction 'before'. */
  public static final int DIRECTION_BEFORE = 1;
  /** Constant for the insert direction 'after'. */
  public static final int DIRECTION_AFTER = 2;
  
  /** The item's name. */
  public static final String TAG_NAME = "print-table";
  
  /** 
   * The maximum number of lines for a table segment.
   * In case this size is reached, an automnatic table
   * split is performed to keep the memory consumption low.  
   */
  public static final int MAX_SEGMENT_SIZE = 100;

  /** The current cell attributes. */
  private CellAttributes cellAttributes = null;

  /** The latest child item. */
  private BaseDocumentTag latestChild = null;
  /** The header item list. */
  private List headerItems = new ArrayList();
  /** The table to print. */
  private Table table = null;
  /** The iText table. */
  private PdfPTable pTable = null;
  /** The number of cells, printed in the current output segment. */
  private int numCells = 0;
  /** The number of rows, printed in the current output segment. */
  private int numRows = 0;
  /** The width of the whole table. */
  private float width = 0.0f;
  /** The horizontal alignment of the table. */ 
  private int hAlign = 0;
  /** Tells whether to split rows or not. */
  private boolean splitRows = false;
  /** 
   * Tells whether to split rows late or not. 
   * If true the row will only split if it's the first one in an empty page.
   */
  private boolean splitLate = true;

  /** The spacing-before attribute. */
  private float spacingBefore = 0.0f;
  /** The spacing-after attribute. */
  private float spacingAfter = 0.0f;
  
  /** The maximum level of importance to support. */
  private int levelFilter = -1;

  /** The column list. */
  private List columnList = new ArrayList();
  /** The default cell content list. */
  private List defaultCellContents = new ArrayList();
  /** The column index. */
  private Map columnIndex = null;
  /** The table columns. */
  private Column[] columns = null;
  /** The widths of the columns. */
  private float[] widths = null;
  /** Tells whether the output has already been started or not. */ 
  protected boolean outputStarted = false;

  /** The print header flag. */
  private boolean printHeader = true;
  /** Tells whether currently a block is open or not. */
  private boolean blockOpen = false;
  
  /** The cleared flag for column definitions. */
  private boolean columnDefinitionsCleared = false;
  
  /**
   * Constructs a PrintTableTag instance with no specific attributes.
   */
  public PrintTableTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Executes the nested items for each row of the specified table. Creates a table " +
      "according to the column layout hints of the table.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.CLASS)
      .addDefinition(CommonAttributes.STYLE)
      .addDefinition(CommonAttributes.FONT)
      .addDefinition(CommonAttributes.LOOP_VARIABLE)
      .addDefinition(CommonAttributes.SEQUENCE)
      .addDefinition(CommonAttributes.TABLE_WIDTH)
      .addDefinition(CommonAttributes.HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonAttributes.VERTICAL_ALIGNMENT)
      .addDefinition(CommonAttributes.LEADING)
      .addDefinition(CommonAttributes.LINE_SPACING)
      .addDefinition(CommonAttributes.CELL_HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonAttributes.CELL_VERTICAL_ALIGNMENT)
      .addDefinition(CommonAttributes.BACKGROUND_COLOR)
      .addDefinition(CommonAttributes.BORDER_STYLE)
      .addDefinition(CommonAttributes.BORDER_WIDTH)
      .addDefinition(CommonAttributes.BORDER_COLOR)
      .addDefinition(CommonAttributes.PADDING)
      .addDefinition(CommonAttributes.PADDING_LEFT)
      .addDefinition(CommonAttributes.PADDING_RIGHT)
      .addDefinition(CommonAttributes.PADDING_TOP)
      .addDefinition(CommonAttributes.PADDING_BOTTOM)
      .addDefinition(CommonAttributes.NO_WRAP)
      .addDefinition(PRINT_HEADER)
      .addDefinition(CommonAttributes.SPACING_BEFORE)
      .addDefinition(CommonAttributes.SPACING_AFTER)
      .addDefinition(CommonAttributes.TABLE_SPLIT_ROWS)
      .addDefinition(CommonAttributes.TABLE_SPLIT_LATE)
      .addDefinition(LEVEL_FILTER); 
  }

  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.FONT_NAME)
      .addDefinition(CommonStyleAttributes.TABLE_WIDTH)
      .addDefinition(CommonStyleAttributes.HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.VERTICAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.LEADING)
      .addDefinition(CommonStyleAttributes.LINE_SPACING)
      .addDefinition(CommonStyleAttributes.CELL_HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.CELL_VERTICAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.BACKGROUND_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_STYLE)
      .addDefinition(CommonStyleAttributes.BORDER_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_COLOR)
      .addDefinition(CommonStyleAttributes.PADDING)
      .addDefinition(CommonStyleAttributes.PADDING_LEFT)
      .addDefinition(CommonStyleAttributes.PADDING_RIGHT)
      .addDefinition(CommonStyleAttributes.PADDING_TOP)
      .addDefinition(CommonStyleAttributes.PADDING_BOTTOM)
      .addDefinition(CommonStyleAttributes.NO_WRAP)
      .addDefinition(PRINT_HEADER)
      .addDefinition(CommonStyleAttributes.SPACING_BEFORE)
      .addDefinition(CommonStyleAttributes.SPACING_AFTER)
      .addDefinition(CommonStyleAttributes.TABLE_SPLIT_ROWS)
      .addDefinition(CommonStyleAttributes.TABLE_SPLIT_LATE)
      .addDefinition(LEVEL_FILTER);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(BaseFontTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(MacroTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(IfTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(ElseTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(LogTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(SwitchTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(InsertColumnTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(AlterColumnTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(RemoveColumnTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(DefineColumnTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(PrintRowTag.class, 0, ChildDefinition.UNLIMITED));
  }

  /**
   * Tells whether or not this tag does respect style attributes or not.
   * @return always <code>true</code>.
   */
  public boolean isStyleable() {
    return true;
  }
  
  /**
   * Tells whether textual content is allowed for the tag or not.
   * @return true if the item is allowed to contain textual content, else false.
   */
  public boolean isTextBodyAllowed() {
    return false;
  }

  /**
   * Gets all visible columns.
   * @return An array holding all visible columns.
   */
  public Column[] getVisibleColumns() {
    if (columns == null) {
      FormatHelper formatHelper = documentHandler.getFormatHelper();
      int numColumns = columnList.size();
      columns = new Column[numColumns];
      widths = new float[numColumns];
      for (int i = 0; i < numColumns; i++) {
        Column column = (Column) columnList.get(i);
        columns[i] = column;
        widths[i] = column.getLayoutHints().getWidth();
        if (column.getFormat() == null) {
          switch (column.getType()) {
          case UjacTypes.TYPE_INT: 
          case UjacTypes.TYPE_LONG: 
            column.setFormat(formatHelper.getIntegerFormat());
            break;
          case UjacTypes.TYPE_FLOAT: 
          case UjacTypes.TYPE_DOUBLE: 
            column.setFormat(formatHelper.getDoubleFormat());
            break;
          case UjacTypes.TYPE_DATE: 
            column.setFormat(formatHelper.getDateFormat());
            break;
          case UjacTypes.TYPE_TIME: 
            column.setFormat(formatHelper.getTimeFormat());
            break;
          case UjacTypes.TYPE_TIMESTAMP: 
            column.setFormat(formatHelper.getTimestampFormat());
            break;
          }
        }
      }
    }
    return columns;
  }
  
  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the foreach item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    // resetting the column index
    columnIndex = null;

    outputStarted = false;

    if (!isValid()) {
      return;
    }

    // inherriting the surrounding cell's attributes
    cellAttributes = new CellAttributes(getCommonAttributesHolder());
    cellAttributes.parseTableAttributes(this, null);

    // getting the table to print
    try {
      table = (Table) getSequenceHolder();
    } catch (ClassCastException ex) {
      throw new DocumentHandlerException(locator(), "Only tables are allowed as sequences at print-table tag!");
    }
    if (table == null) {
      throw new DocumentHandlerException(locator(), "The attribute '" + TagAttributes.ATTR_SEQUENCE + "' doesn't resolve to a table, check your document properties!");
    }
    // using FormatHelper from documentHandler 
    table.setFormatHelper(documentHandler.getFormatHelper());

    // clearing the (old) column list
    columnList.clear();
    columns = null;
    
    // setting the attributes
    printHeader = getBooleanAttribute(PRINT_HEADER, true, true, null);
    levelFilter = getIntegerAttribute(LEVEL_FILTER, -1, true, null);
    width = getFloatAttribute(CommonAttributes.TABLE_WIDTH, 100.0F, true, CommonStyleAttributes.TABLE_WIDTH);
    Column[] columns = table.getVisibleColumns(levelFilter);
    for (int i = 0; i < columns.length; i++) {
      columnList.add(columns[i]);
      defaultCellContents.add(null);
    }
    
    hAlign = getHorizontalAlignmentAttribute(CommonAttributes.HORIZONTAL_ALIGNMENT, true, CommonStyleAttributes.HORIZONTAL_ALIGNMENT);

    splitRows = getBooleanAttribute(CommonAttributes.TABLE_SPLIT_ROWS, false, CommonStyleAttributes.TABLE_SPLIT_ROWS);
    splitLate = getBooleanAttribute(CommonAttributes.TABLE_SPLIT_LATE, false, CommonStyleAttributes.TABLE_SPLIT_LATE);

    spacingBefore = getDimensionAttribute(CommonAttributes.SPACING_BEFORE, 0.0f, true, CommonStyleAttributes.SPACING_BEFORE);
    spacingAfter = getDimensionAttribute(CommonAttributes.SPACING_AFTER, 0.0f, true, CommonStyleAttributes.SPACING_AFTER);
  }

  /**
   * Opens the item.
   * @param resetData tells whether to reset some specific data or not (don't blame for this silly comment).
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem(boolean resetData) throws DocumentHandlerException {

    if (resetData) {
      // resetting cell counter
      numCells = 0;
      // clearing header items
      headerItems.clear();
      outputStarted = true;
    }
    
    if (!isValid()) {
      return;
    }

    // creating the table  
    pTable = new PdfPTable(columns.length);
    
    // configuring the table
    if (width > 0.0f) {
      pTable.setWidthPercentage(width);
    }
    try {
      // setting table widths
      pTable.setWidths(widths);
    } catch (DocumentException ex) {
      throw new DocumentHandlerException(locator(), "Invalid table widths:'" + 
        widths.toString() + "': " + ex.getMessage(), ex);
    }
    // setting the horizntal aligmnemt
    pTable.setHorizontalAlignment(hAlign);
    // setting the split-rows flag
    pTable.setSplitRows(splitRows);
    pTable.setSplitLate(splitLate);

    if (resetData) {
      // defining the spacing-before for the first table segment
      pTable.setSpacingBefore(spacingBefore);
    }
  }
  
  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while closing the foreach item.
   */
  public void closeItem() throws DocumentHandlerException {
    
    // looping nested items
    super.closeItem();

    if (!isValid()) {
      return;
    }

    if (pTable == null) {
      return;
    }
    
    // flushes the current table segment
    flushTableSegment(true);
  }

  /**
   * Flushes the current table segment.
   * @param lastSegment Tells whether this is the last segment or not.
   * @exception DocumentHandlerException Thrown in case something went wrong while performing the table output.
   */
  private void flushTableSegment(boolean lastSegment) throws DocumentHandlerException {
    // checking if the last row was filled completely
    if ((numCells % columns.length) > 0) {
      Locator locator = locator();
      if (latestChild != null) {
        locator = latestChild.locator();
      }
      throw new DocumentHandlerException(locator, "The last row is not filled completely, check the table cells!");
    }
    
    if (lastSegment) {
      // defining the spacing-after for the last table segment
      pTable.setSpacingAfter(spacingAfter);
    }

    // adding table to latest element container
    ElementContainer elementContainer = getElementContainer();
    elementContainer.addElement(this, pTable);

    // resetting cell & row counter
    numCells = 0;
    numRows = 0;
  }
  
  /**
   * Pre-processes the child items of the foreach item.
   * @return Tells, whether or not to repeat the child items, returning false skips the execution
   *   for the current item.
   * @exception DocumentHandlerException Thrown in case something went wrong.
   */
  protected boolean preRepeatChildItems() throws DocumentHandlerException {
    Row row = (Row) getCurrentValue();
    if (row != null) {
      boolean reallyBreaked = false;
      if (row.isStartsBlock()) {
        blockOpen = true;        
      }
      if (row.isStartsBlock() && (getNumContentRows() > 0)) {
        // splitting the table at block start.
        reallyBreaked = splitTable(row.isStartNewPage(), true);
      } else {
        if (row.isStartNewPage()) {
          // starting a new page
          reallyBreaked = splitTable(true, true);
        }
      }
      if (row.isStartNewPage() && !reallyBreaked) {
        try {
          documentHandler.getDocument(true).newPage();
        } catch (Exception ex) {
          throw new DocumentHandlerException(locator(), "Unable to open a new page: " + ex.getMessage(), ex);
        }
      }
    }
    
    return true;
  }

  /**
   * Post-processes the child items of the foreach item.
   * @exception DocumentHandlerException Thrown in case something went wrong.
   */
  protected void postRepeatChildItems() throws DocumentHandlerException {
    Row row = (Row) getCurrentValue();
    if (row.isEndsBlock()) {
      // Checking if the iText table fits into the current page
      if (!fitsPage()) {
        try {
          documentHandler.getDocument().newPage();
        } catch (Exception ex) {
          throw new DocumentHandlerException(locator(), "Unable to start a new page: " + ex.getMessage(), ex);
        }
      }
      // splitting the table after closing the block.
      splitTable(false, true);
      blockOpen = false;
    }
  }
  
  /**
   * Method called before the child items are printed.
   * @exception DocumentHandlerException Thrown in case something went wrong.
   */
  protected void preLoopChildItems() throws DocumentHandlerException {
    
    if (!printHeader) {
      return;
    }
    
    String loopVariable = getLoopVariable();
    String loopIdxVariable = getLoopIdxVariable();
    Row headerRow = table.getHeaderRow();
    int numColumns = table.getColumnCount();
    try {
      for (int i = 0; i < numColumns; i++) {
        LayoutHints layoutHints = table.getColumn(i).getLayoutHints();
        if (layoutHints != null) {
          headerRow.setString(i, layoutHints.getHeader());
        }
      }
      TemplateContext ctx = getDocumentHandler().getTemplateContext();
      ctx.setProperty(loopVariable, headerRow);
      setCurrentValue(headerRow);
      ctx.setProperty(loopIdxVariable, new Integer(-1));
    } catch (TableException ex) {
      throw new DocumentHandlerException(locator(), "Table access failure: " + ex.getMessage(), ex);
    }
    repeatChildItems();
  }
  
  /**
   * Method called after the child items are printed.
   * @exception DocumentHandlerException Thrown in case something went wrong.
   */
  protected void postLoopChildItems() throws DocumentHandlerException {
    
    Row currentRow = (Row) getCurrentValue();
    if ((currentRow != null) && TableConstants.ROW_TYPE_FOOTER.equals(currentRow.getType())) {
      // real footer has been printed, nothing to do
      return;
    }
    
    // printing artificial footer row
    String loopVariable = getLoopVariable();
    String loopIdxVariable = getLoopIdxVariable();
    Row footerRow = new GroupRow(table, TableConstants.ROW_TYPE_FOOTER, null);
    int numColumns = table.getColumnCount();
    try {
      if (TableConstants.ROW_TYPE_HEADER.equals(currentRow.getType())) {
        for (int i = 0; i < numColumns; i++) {
          footerRow.setObject(i, null);
          footerRow.setVisible(i, false);
        }
      } else {
        for (int i = 0; i < numColumns; i++) {
          footerRow.setObject(i, currentRow.getObject(i));
          footerRow.setVisible(i, false);
        }
      }
      TemplateContext ctx = getDocumentHandler().getTemplateContext();
      ctx.setProperty(loopVariable, footerRow);
      setCurrentValue(footerRow);
      ctx.setProperty(loopIdxVariable, new Integer(-1));
    } catch (TableException ex) {
      throw new DocumentHandlerException(locator(), "Table access failure: " + ex.getMessage(), ex);
    }
    repeatChildItems();
  }

  /**
   * Gets the default contents for the given column.
   * @param idx The index of the column to get the default contents for.
   * @return The index of the column.
   */
  protected String getDefaultContent(int idx) {
    return (String) defaultCellContents.get(idx);
  }

  /**
   * Checks whether the table output has been started or not.
   * @return true if the table output has been started, else false.
   */
  public boolean isOutputStarted() {
    return outputStarted;
  }
  
  /**
   * Adds a cell to the table.
   * @param item The tag which performed the cell adding.
   * @param cell The cell to be added.
   * @exception DocumentHandlerException If something went wrong when adding the cell.
   */
  public void addCell(BaseDocumentTag item, PdfPCell cell) throws DocumentHandlerException {
    if (!isOutputStarted()) {
      openItem(true);
    }
    
    // setting the last child item
    latestChild = item;
    // incrementing header cell counter
    numCells += Math.max(cell.getColspan(), 1);
    // adding cell to table
    pTable.addCell(cell);

    if (numCells % getVisibleColumns().length == 0) {
      ++numRows;
      if (!blockOpen && (numRows >= MAX_SEGMENT_SIZE)) {
        // performing a table split here to keep the memory consumption low
        splitTable(false, true);
      }
    }
  }
  
  /**
   * Gets the number of remaining columns of the current row.
   * @return The number of remaining column to the end of the current row.
   */
  public int getRemainingPdfColumns() {
    Column[] columns = getVisibleColumns();
    int numColumns = columns.length;
    return numColumns - (numCells % numColumns); 
  }

  /**
   * Gets the PDF table holder.
   * @return The component that really holds a PDF table.
   */
  public PdfTableHolder getPdfTable() {
    return this;
  }

  /**
   * Adds a header cell to the table.
   * @param item The tag which performed the cell adding.
   * @param cell The header cell to be added.
   * @exception DocumentHandlerException If something went wrong when adding the header cell.
   */
  public void addHeaderCell(BaseDocumentTag item, PdfPCell cell) throws DocumentHandlerException {
    // adding cell as usual
    addCell(item, cell);
    // adding cell to header list
    headerItems.add(cell);
  }

  /**
   * Checks whether the table fits the current page or not. 
   * @return true if there's room enough for the table on the current page, else false.
   * @throws DocumentHandlerException In case something went wrong.
   */
  public boolean fitsPage() throws DocumentHandlerException {
    DocumentHandler documentHandler = getDocumentHandler();
    PdfWriter writer = (PdfWriter) documentHandler.getDocumentWriter();
    try {
      return PdfHelper.fitsPage(writer, documentHandler.getDocument(), getTable());
    } catch (DocumentException ex) {
      throw new DocumentHandlerException(locator(), "Fits page check failed: " + ex.getMessage(), ex);
    }
  }

  /**
   * Splits the table vertically.
   * @param newPage Tells whether to start a new page or not.
   * @param keepHeader Tells whether to keep the old header or not.
   * @return true if a split has been really performed, else false.
   * @exception DocumentHandlerException If something went wrong while splitting the table.
   */
  public boolean splitTable(boolean newPage, boolean keepHeader) throws DocumentHandlerException {
    if (!isValid()) {
      return false;
    }
    
    if (numCells == 0) {
      // nothing to split yet
      return false;
    }

    int numHeaderRows = pTable.getHeaderRows();

    // flushes the current table segment
    flushTableSegment(false);

    if (newPage) {
      try {
        documentHandler.getDocument(true).newPage();
      } catch (Exception ex) {
        throw new DocumentHandlerException(locator(), "Unable to open a new page: " + ex.getMessage(), ex);
      }
    }
    // starts the new table
    openItem(false);
    if (keepHeader) {
      if (!newPage) {
        pTable.setSkipFirstHeader(true);
      }
      int numHeaderItems = headerItems.size();
      for (int i = 0; i < numHeaderItems; i++) {
        PdfPCell headerItem = (PdfPCell) headerItems.get(i);
        pTable.addCell(headerItem);
      }
      pTable.setHeaderRows(numHeaderRows);
    } else {
      // clearing header items
      headerItems.clear();
    }
    return true;
  }
  
  /**
   * Defines a column for the table.
   * @param name The name of the column.
   * @param width The width of the column.
   * @param hAlign The horizontal alignment of the column.
   * @param title The title of the column.
   * @param defaultContents The default contents to use for this column.
   * @exception DocumentHandlerException In case the given reference column does not exist. 
   */
  public void defineColumn(String name, float width, int hAlign, String title, String defaultContents) 
    throws DocumentHandlerException {
    
    if (columns != null) {
      return;
    }
    if (!columnDefinitionsCleared) {
      // clearing column definition list
      columnList.clear();
      defaultCellContents.clear();
      columnDefinitionsCleared = true;
    }
    
    // finding reference column
    int numColumns = columnList.size();
   
    Column col = null;
    try {
      col = table.getColumn(name);
    } catch (ColumnNotDefinedException ex) {
      // column not defined yet
    }
    if (col != null) {
      // column already exists
      LayoutHints lh = col.getLayoutHints();
      if (lh != null) {
        lh.setWidth(width);
        lh.setHeader(title);
        lh.setHeaderAlign(hAlign);
        lh.setVisible(true);
      } else {
        col.setLayoutHints(new LayoutHints(
            width,
            title,
            hAlign
        ));
      }
      // checking if column already exists in visible column list ...
      for (int i = 0; i < numColumns; i++) {
        Column tmpCol = (Column) columnList.get(i);
        if (!tmpCol.getName().equals(name)) {
          continue;
        }
        // just redefinining default context for existing column definition
        defaultCellContents.set(i, defaultContents);
        return;
      }
      // Column doesn't exit yet, adding new column definition
      columnList.add(col);
      defaultCellContents.add(defaultContents);
      return;
    }
    Column newCol = new DummyColumn(name);
    newCol.setLayoutHints(new LayoutHints(
        width,
        title,
        hAlign
    ));
  
    columnList.add(newCol);
    defaultCellContents.add(defaultContents);
  }

  /**
   * Inserts a column into the table.
   * @param name The name of the new column.
   * @param width The width of the new column.
   * @param hAlign The horizontal alignment of the new column.
   * @param title The title of the column.
   * @param reference The column at which to insert the new column.
   * @param direction The insert direction.
   * @param defaultContents The default contents to use for this column.
   * @see #DIRECTION_BEFORE
   * @see #DIRECTION_AFTER
   * @exception DocumentHandlerException In case the given reference column does not exist. 
   */
  public void insertColumn(String name, float width, int hAlign, String title,
                           String reference, int direction, String defaultContents) 
    throws DocumentHandlerException {
    
    if (columns != null) {
      return;
    }
    
    // finding reference column
    int numColumns = columnList.size();
    int coldx = -1;
    int referenceIdx = -1;
    for (int i = 0; i < numColumns; i++) {
      Column col = (Column) columnList.get(i);
      if (col.getName().equals(name)) {
        coldx = i;
        if (referenceIdx != -1) {
          break;
        }
      }
      if (col.getName().equals(reference)) {
        referenceIdx = i;
        if (coldx != -1) {
          break;
        }
      }
    }
    if (coldx != -1) {
      // This column already exists, silently ignoring it
      return;
    }
    if (referenceIdx == -1) {
      throw new DocumentHandlerException(locator(), "The column name '" + reference + "' does not exist.");
    }
    
    if (direction == DIRECTION_AFTER) {
      referenceIdx += 1;
    }
    
    Column newCol = new DummyColumn(name);
    newCol.setLayoutHints(new LayoutHints(
      width,
      title,
      hAlign
    ));

    if (referenceIdx >= numColumns) {
      columnList.add(newCol);
      defaultCellContents.add(defaultContents);
    } else {
      columnList.add(referenceIdx, newCol);
      defaultCellContents.add(referenceIdx, defaultContents);
    }
  }

  /**
   * Alters a column at the table.
   * @param name The name of the new column.
   * @param width The width of the new column.
   * @param hAlign The horizontal alignment of the new column.
   * @param title The title of the column.
   * @param defaultContents The default contents to use for this column.
   * @exception DocumentHandlerException In case the given reference column does not exist. 
   */
  public void alterColumn(String name, float width, int hAlign, String title, String defaultContents) 
    throws DocumentHandlerException {
    
    if (columns != null) {
      return;
    }
    
    // finding reference column
    int numColumns = columnList.size();
    int coldx = -1;
    for (int i = 0; i < numColumns; i++) {
      Column col = (Column) columnList.get(i);
      defaultCellContents.set(i, defaultContents);
      if (col.getName().equals(name)) {
        coldx = i;
        LayoutHints layoutHints = col.getLayoutHints();
        if (layoutHints == null) {
          layoutHints = new LayoutHints();
          col.setLayoutHints(layoutHints);
        }
        if (width >= 0.0) {
          layoutHints.setWidth(width);
        }
        if (hAlign >= 0) {
          layoutHints.setAlign(hAlign);
        }
        if (title != null) {
          layoutHints.setHeader(title);
        }
        break;
      }
    }
    if (coldx < 0) {
      throw new DocumentHandlerException(locator(), "The column name '" + name + "' is undefined.");
    }
    
  }

  /**
   * Removes a column from the table.
   * @param name The name of the new column.
   * @exception DocumentHandlerException In case the given column does not exist. 
   */
  public void removeColumn(String name) throws DocumentHandlerException {
    if (columns != null) {
      return;
    }
    
    // finding column
    int numColumns = columnList.size();
    for (int i = 0; i < numColumns; i++) {
      Column col = (Column) columnList.get(i);
      if (col.getName().equals(name)) {
        columnList.remove(i);
        return;
      }
    }
    throw new DocumentHandlerException(locator(), "The column name '" + name + "', which was requested to remove, does not exist.");
  }
  
  /**
   * Gets the PDF table.
   * @return The PDF table.
   */
  protected PdfPTable getTable() {
    return pTable;
  }

  /**
   * Gets the number of rows, printed so far within the current row segment.
   * @return The number of rows, printed so far within the current row segment.
   */
  protected int getNumRows() {
    return numRows;
  }
  
  /**
   * Gets the number of columns by row to print.
   * @return The number of columns by row.
   */
  public int getNumColumns() {
    return columnList.size();
  }
  
  /**
   * Gets the number of content rows, that have been added to the current table so far.
   * @return The number of content rows, that have been added to the current table so far.
   */
  public int getNumContentRows() {
    if (pTable == null) {
      return 0;
    }
    return pTable.size() - pTable.getHeaderRows();
  }
  
  /**
   * Gets the index of the given column.
   * @param columnName The name of the column to get the index for.
   * @return The index of the column.
   */
  protected int getColumnIndex(String columnName) {
    Column[] columns = getVisibleColumns();
    int numColumns = columns.length;
    if (this.columnIndex == null) {
      this.columnIndex = new HashMap();
      for (int i = 0; i < numColumns; i++) {
        Column col = columns[i];
        columnIndex.put(col.getName(), new Integer(i));
      }
    }
    Integer idx = (Integer) columnIndex.get(columnName);
    if (idx == null) {
      return -1;
    }
    return idx.intValue();
  }

  /**
   * Gets the cell attributes.
   * @return The cell attributes.
   */
  public CellAttributes getCellAttributes() {
    return cellAttributes;
  }
  
  /**
   * Gets the line spacing.
   * @return The current line spacing.
   */
  public float getLineSpacing() {
    if (cellAttributes != null) {
      return cellAttributes.getLineSpacing();
    }
    return DEFAULT_LINE_SPACING;
  }
  
  /**
   * Gets the leading.
   * @return The current leading.
   */
  public float getLeading() {
    if (cellAttributes != null) {
      return cellAttributes.getLeading();
    }
    return DEFAULT_LEADING;
  }

}
