/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import java.io.IOException;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.DocumentHandlerException;

import com.lowagie.text.DocumentException;
import com.lowagie.text.Font;

/**
 * Name: RegisterFontTag<br>
 * Description: Handler class for the 'register-font' tag.
 * 
 * @author lauerc
 */
public class RegisterFontTag extends BaseDocumentTag {

  /** Definition of the 'source' attribute. */
  private static final AttributeDefinition SOURCE = new AttributeDefinition(TagAttributes.ATTR_SOURCE, 
      AttributeDefinition.TYPE_STRING, true, "The location of the TTF file at the resource loader.");
  /** Definition of the 'embedded' attribute. */
  private static final AttributeDefinition EMBEDDED = new AttributeDefinition("embedded", 
      AttributeDefinition.TYPE_BOOLEAN, false, "The font file needs to be embedded or not.");
  /** Definition of the 'cached' attribute. */
  private static final AttributeDefinition CACHED = new AttributeDefinition("cached", 
      AttributeDefinition.TYPE_BOOLEAN, false, "The font file needs to be cached or not.");

  /** The item's name. */
  public static final String TAG_NAME = "register-font";
  
  /** The font family. */
  private String family = null;
  /** The location of the font. */
  private Object source = null;
  /** The font style. */
  private int style = Font.NORMAL;
  //Added by Kiran
  /** embed the font into the PDF. */
  private boolean embedded = true;
  /** should the font be cached and reused. */
  private boolean cached = true; 
  
  /**
   * Constructs a RegisterFontTag instance with no specific attributes.
   */
  public RegisterFontTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Registers the specified TTF font." ;
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.FONT_FAMILY)
      .addDefinition(SOURCE)
      .addDefinition(EMBEDDED)   
      .addDefinition(CACHED)   
      .addDefinition(CommonAttributes.FONT_STYLE);
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // getting attributes
    this.family = getStringAttribute(CommonAttributes.FONT_FAMILY, true, null);
    this.source = evalAttribute(SOURCE);
    String styleAttr = getStringAttribute(CommonAttributes.FONT_STYLE, true, null);
    if (styleAttr != null) {
      this.style = Font.getStyleValue(styleAttr);
    }
    //Added by Kiran
    this.embedded = getBooleanAttribute(EMBEDDED, true, true, null);
    this.cached = getBooleanAttribute(CACHED, true, true, null);
  }
  
  /**
   * @see org.ujac.print.BaseDocumentTag#closeItem()
   */
  public void closeItem() throws DocumentHandlerException {
    /*
     * Added by Kiran
     * The following 3 lines of code is to fix the problem when Register-font tag
     * is used inside an If tag.
     */ 
    if (!isValid()) { 
      return; 
    } 

    try {
      // loading the font 
      byte[] fontData = null;
      if (source instanceof byte[]) {
        fontData = (byte[]) source;
      } else {
        fontData = documentHandler.loadResource(source.toString());
      }
      // registering font at document handler
      //Modified by Kiran
      documentHandler.registerFont(family, fontData, style, embedded, cached);
    } catch (IOException ex) {
      throw new DocumentHandlerException(locator(), "Error while loading font from location '" +
        source + "': " + ex.getMessage(), ex);
    } catch (DocumentException ex) {
      throw new DocumentHandlerException(locator(), "Error while loading font from location '" +
        source + "': " + ex.getMessage(), ex);
    }
  }
  
}
