/*
 * Copyright (C) 2007 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.DocumentTagFactory;

/**
 * Name: RegisterTagTag<br>
 * Description: Handler class for the 'register-tag' tag.
 * 
 * @author lauerc
 */
public class RegisterTagTag extends BaseDocumentTag {

  /** Definition of the 'tag-class' attribute. */
  private static final AttributeDefinition TAG_NAME_ATTR = 
      CommonAttributes.NAME.cloneAttrDef("The name of the tag to register.");
  /** Definition of the 'tag-class' attribute. */
  private static final AttributeDefinition TAG_CLASS = new AttributeDefinition("tag-class", 
      AttributeDefinition.TYPE_STRING, true, "The fully qualified tag implementation class name.");

  /** The item's name. */
  public static final String TAG_NAME = "register-tag";
  
  /**
   * Constructs a RegisterTagTag instance with no specific attributes.
   */
  public RegisterTagTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Registers the specified tag class." ;
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(TAG_NAME_ATTR)
      .addDefinition(TAG_CLASS);
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
  }
  
  /**
   * @see org.ujac.print.BaseDocumentTag#closeItem()
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) { 
      return; 
    } 

    String tagName = getStringAttribute(TAG_NAME_ATTR, true, null); 
    String tagClassName = getStringAttribute(TAG_CLASS, true, null); 
    try {
      // getting class for the given tag class name.
      Class tagClass = Class.forName(tagClassName);
      // registering the tag with the tag factory
      DocumentTagFactory tagFactory = getDocumentHandler().getTagFactory();
      tagFactory.registerTag(tagName, tagClass);
    } catch (ClassNotFoundException ex) {
      throw new DocumentHandlerException(locator(), "Unable to register tag implementation class '" +
          tagClassName + "', check class path.");
    }
  }
  
}
