/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.DocumentHandlerException;
import org.ujac.util.text.FormatHelper;

/**
 * Name: ResourceBundleTag<br>
 * Description: A class handling resource bundles.<br>
 * 
 * @author lauerc
 */
public class ResourceBundleTag extends BaseDocumentTag {

  /** Definition of the 'source' attribute. */
  private static final AttributeDefinition BUNDLE_NAME =  
      CommonAttributes.NAME.cloneAttrDef("The name of the resource bundle.");
  /** Definition of the 'source' attribute. */
  private static final AttributeDefinition BUNDLE_SOURCE = new AttributeDefinition(TagAttributes.ATTR_SOURCE, 
      AttributeDefinition.TYPE_STRING, false, "The location of the resource bundle at the resource loader.");

  
  /** The item's name. */
  public static final String TAG_NAME = "resource-bundle";

  /** The bundle name. */
  private String name = null;
  /** The bundle source. */
  private String source = null;
    
  /**
   * Constructs a ResourceBundleTag instance with no specific attributes.
   */
  public ResourceBundleTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Loads a localized resource bundle through the resource loader.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(BUNDLE_NAME)
      .addDefinition(BUNDLE_SOURCE);
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
  }
  
  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    // getting attributes
    name = getStringAttribute(BUNDLE_NAME, false, null);
    source = getStringAttribute(BUNDLE_SOURCE, true, null);

    FormatHelper formatHelper = documentHandler.getFormatHelper();
    Locale locale = formatHelper.getLocale();
    try {
      // loading bundle from resource loader
      ResourceBundle bundle = ResourceBundle.getBundle(source, locale);
      // setting bundle at document properties
      documentHandler.getTemplateContext().setProperty(name, bundle);
    } catch (MissingResourceException e) {
      throw new DocumentHandlerException(locator(), "Unable to load resource bundle '" + source + "': " + e.getMessage(), e);
    }
  }
   
}
