/*
 * Copyright (C) 2003 - 2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.FontHolder;

/**
 * Name: RowCellTag<br>
 * Description: Handler class for the tag 'cell-style'.
 * 
 * @author lauerc
 */
public class RowCellTag extends BaseCellTag implements FontHolder {

  /** The item's name. */
  public static final String TAG_NAME = "row-cell";

  /** Definition of the 'rotate' attribute. */
  private static final AttributeDefinition CELL_COLUMN_NAME = 
      CommonAttributes.COLUMN_NAME.cloneAttrDef("The name of the column to define the attributes for.");

  /** Definition of the 'rotate' attribute. */
  private static final AttributeDefinition CELL_COLUMN_INDEX = 
      CommonAttributes.COLUMN_INDEX.cloneAttrDef("The index of the column to define the attributes for.");

  /** The surrounding RowCellTag container. */
  private RowCellContainer rowCellContainer = null;
  
  /** The column name. */
  protected String name = null;
  /** The column index. */
  protected int index = -1;
  
  /** The format. */
  private String format = null;
  /** The rotation angle. */
  private float rotate = 0.0f;
  
  /**
   * Constructs a RowCellTag instance with no specific attributes.
   */
  public RowCellTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Specifies the style of a cell for a specific column within the surrounding " +
      "print-row tag.";
  }
  
  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CELL_COLUMN_NAME)
      .addDefinition(CELL_COLUMN_INDEX)
      .addDefinition(CommonAttributes.FORMAT);
  }

  /**
   * Gets the format.
   * @return The format string.
   */
  public String getFormat() {
    return format;
  }

  /**
   * Gets the rotation angle.
   * @return The rotation angle.
   */
  public float getRotate() {
    return rotate;
  }
  
  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // setting the cell container.
    this.rowCellContainer = (RowCellContainer) documentHandler.latestOfType(RowCellContainer.class);
    if (rowCellContainer == null) {
      throw new DocumentHandlerException(locator(), "Unable to determine surrounding container for " + TAG_NAME + " tag.");
    }
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    // getting attributes
    this.name = getStringAttribute(CommonAttributes.COLUMN_NAME, true, null);
    this.index = getIntegerAttribute(CommonAttributes.COLUMN_INDEX, -1, true, null);
    this.rotate = getFloatAttribute(CommonAttributes.ROTATE, 0.0f, true, CommonStyleAttributes.ROTATE);
    this.format = getStringAttribute(CommonAttributes.FORMAT, true, null);
  }

  /**
   * Gets the parent attributes.
   * @return The parent attributes.
   */
  protected CellAttributes getParentAttributes() {
    return rowCellContainer.getCellAttributes();
  }
  
  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while closing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }
    super.closeItem();
    itemClosed = true;

    // registering the cell style
    rowCellContainer.registerRowCell(this);
  }

  /**
   * Gets the column name.
   * @return The column name.
   */
  public String getColumnName() {
    return name;
  }
  
  /**
   * Gets the column index.
   * @return The column index.
   */
  public int getColumnIndex() {
    return index;
  }
  
}
