
/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.ChildDefinition;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.CommonAttributesHolder;
import org.ujac.print.DocumentFont;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.ElementContainer;
import org.ujac.print.NewPageHandler;
import org.ujac.print.StyleHolder;
import org.ujac.print.TocItem;

import com.lowagie.text.Chunk;
import com.lowagie.text.Document;
import com.lowagie.text.Element;
import com.lowagie.text.Font;
import com.lowagie.text.Paragraph;
import com.lowagie.text.Rectangle;
import com.lowagie.text.Section;

/**
 * Name: SectionTag<br>
 * Description: Handler class for the ;lt;section&gt; tag.
 * 
 * @author lauerc
 */
public class SectionTag extends BaseElementTag implements ElementContainer, StyleHolder, NewPageHandler {

  /** The item's name. */
  public static final String TAG_NAME = "section";
  
  /** Definition of the 'font' attribute. */
  private static final AttributeDefinition SECTION_FONT = 
      CommonAttributes.FONT.cloneAttrDef("The name of the font definition to use for the section contents.");
  /** Definition of the 'font-name' style attribute. */
  private static final AttributeDefinition SECTION_FONT_NAME_STYLE = 
      CommonStyleAttributes.FONT_NAME.cloneAttrDef("The name of the font definition to use for the section contents.");

  /** Definition of the 'spacing-before' attribute. */
  private static final AttributeDefinition SECTION_SPACING_BEFORE = 
      CommonAttributes.SPACING_BEFORE.cloneAttrDef("The vertical spacing before the chapter title.");
  /** Definition of the 'spacing-before' attribute. */
  private static final AttributeDefinition SECTION_SPACING_BEFORE_STYLE = 
      CommonStyleAttributes.SPACING_BEFORE.cloneAttrDef("The vertical spacing before the chapter title.");

  /** Definition of the 'spacing-after' attribute. */
  private static final AttributeDefinition SECTION_SPACING_AFTER = 
      CommonAttributes.SPACING_AFTER.cloneAttrDef("The vertical spacing after the chapter title.");
  /** Definition of the 'spacing-after' attribute. */
  private static final AttributeDefinition SECTION_SPACING_AFTER_STYLE = 
      CommonStyleAttributes.SPACING_AFTER.cloneAttrDef("The vertical spacing after the chapter title.");

  /** The surrounding section tag. */
  private SectionTag parentSection;
  /** The title font. */
  private DocumentFont titleFont = null;
  /** The iText section element. */
  protected Section section = null;
  /** The vertical spacing before the section title. */
  private float spacingBefore = 0.0F;
  /** The vertical spacing after the section title. */
  private float spacingAfter = 0.0F;
  /** The fixed vertical leading distance. */
  private float leading = 0.0F;
  /** The line spacing attribute. */
  private float lineSpacing = DEFAULT_LINE_SPACING;
  /** The toc relevance. */
  protected String tocRelevance = null;

  /**
   * Constructs a SectionTag instance with no specific attributes.
   */
  public SectionTag() {
    super(TAG_NAME);
  }
  /**
   * Constructs a SectionTag instance with specific attributes.
   * @param name The tag name.
   */
  public SectionTag(String name) {
    super(name);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Adds a chapter to the document.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.CLASS)
      .addDefinition(CommonAttributes.STYLE)
      .addDefinition(CommonAttributes.SECTION_NUMBER_DEPTH)
      .addDefinition(CommonAttributes.INDENTATION)
      .addDefinition(CommonAttributes.SECTION_TITLE)
      .addDefinition(CommonAttributes.SECTION_TITLE_FONT)
      .addDefinition(SECTION_FONT)
      .addDefinition(SECTION_SPACING_BEFORE)
      .addDefinition(SECTION_SPACING_AFTER)
      .addDefinition(CommonAttributes.LEADING)
      .addDefinition(CommonAttributes.LINE_SPACING)
      .addDefinition(CommonAttributes.TOC_RELEVANCE);
  }

  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.INDENTATION)
      .addDefinition(CommonStyleAttributes.SECTION_TITLE_FONT)
      .addDefinition(SECTION_FONT_NAME_STYLE)
      .addDefinition(SECTION_SPACING_BEFORE_STYLE)
      .addDefinition(SECTION_SPACING_AFTER_STYLE)
      .addDefinition(CommonStyleAttributes.LEADING)
      .addDefinition(CommonStyleAttributes.LINE_SPACING)
      .addDefinition(CommonAttributes.TOC_RELEVANCE);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(SectionTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BaseFontTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(MacroTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(IfTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(ElseTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(LogTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(SwitchTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(ForeachTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(PhraseTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(ParagraphTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(TableTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(PdfTableTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(AnchorTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(AnnotationTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ImageTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BarcodeTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(DatamatrixTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BarcodePdf417Tag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ListTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(SetPropertyTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(DefineTableTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(AddRowTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(AssertTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(NewLineTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(NewPageTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(OutlineTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ImportTag.class, 0, ChildDefinition.UNLIMITED));
  }

  /**
   * Tells whether or not this tag does respect style attributes or not.
   * @return always <code>true</code>.
   */
  public boolean isStyleable() {
    return true;
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // getting the surrounding section tag
    this.parentSection = (SectionTag) documentHandler.latestOfType(SectionTag.class);
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    // handling the title font
    String titleFontName = getStringAttribute(CommonAttributes.SECTION_TITLE_FONT, true, CommonAttributes.SECTION_TITLE_FONT);
    if (titleFontName == null) {
      titleFont = getLocalFont();
    } else {
      titleFont = documentHandler.getFont(titleFontName);
    }

    this.spacingBefore = getDimensionAttribute(CommonAttributes.SPACING_BEFORE, true, CommonStyleAttributes.SPACING_BEFORE);
    this.spacingAfter = getDimensionAttribute(CommonAttributes.SPACING_AFTER, true, CommonStyleAttributes.SPACING_AFTER);
    CommonAttributesHolder commonAttributesHolder = getCommonAttributesHolder();
    if (isAttributeDefined(CommonAttributes.LEADING, CommonStyleAttributes.LEADING)) {
      this.leading = getDimensionAttribute(CommonAttributes.LEADING, true, CommonStyleAttributes.LEADING);
    } else {
      this.leading = commonAttributesHolder.getLeading();
    }    
    if (isAttributeDefined(CommonAttributes.LINE_SPACING, CommonStyleAttributes.LINE_SPACING)) {
      this.lineSpacing = getDimensionAttribute(CommonAttributes.LINE_SPACING, true, CommonStyleAttributes.LINE_SPACING);
    } else {
      this.lineSpacing = commonAttributesHolder.getLineSpacing();
    }
    
    tocRelevance = getStringAttribute(CommonAttributes.TOC_RELEVANCE, "all", true, CommonStyleAttributes.TOC_RELEVANCE);
    
    // creating the section
    createSection();

    if (isAttributeDefined(CommonAttributes.INDENTATION, CommonStyleAttributes.INDENTATION)) {
      this.section.setIndentation(getDimensionAttribute(CommonAttributes.INDENTATION, 0.0f, true, CommonStyleAttributes.INDENTATION));
    }
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    if (spacingAfter > 0.0F) {
      Paragraph sap = new Paragraph(0.0F);
      sap.setSpacingAfter(spacingAfter);
      section.add(sap);
    }
  }
  
  /**
   * Creates the iText section element.
   * @exception DocumentHandlerException In case the creation of the section failed.
   */
  protected void createSection() throws DocumentHandlerException {
    String titleStr = getTextAttribute(CommonAttributes.SECTION_TITLE, true, null);
    DocumentFont titleDocFont = getTitleFont();
    Font titleFont = null;
    if (titleDocFont != null) {
      titleFont = titleDocFont.getFont();
    } else {
      titleFont = new Font();
    }
    Paragraph title = new Paragraph(titleStr, titleFont);
    // workaround for the missing multipliedLeading (aka line-spacing) support in iText
    extendLeading(title, titleFont, getLeading(), getLineSpacing());
    Section sec = parentSection.getSection();
    int numberDepth = getIntegerAttribute(CommonAttributes.SECTION_NUMBER_DEPTH, 2, true, null);
    if (spacingBefore != 0.0F) {
      Paragraph sbp = new Paragraph(0.0F);
      sbp.setSpacingBefore(spacingBefore);
      sec.add(sbp);
    }
    title.setSpacingAfter(getSpacingAfter());
    this.section = sec.addSection(title, numberDepth);
    section.setNumberStyle(sec.getNumberStyle());
    String number = ((Chunk) section.getTitle().getChunks().get(0)).getContent().trim();
    getDocumentHandler().addTocItem(new TocItem(numberDepth, number, titleStr, tocRelevance));
  }

  /**
   * Gets the section.
   * @return The section item.
   */
  public Section getSection() {
    return this.section;
  }

  /**
   * @see org.ujac.print.ElementContainer#addElement(org.ujac.print.BaseDocumentTag, com.lowagie.text.Element)
   */
  public void addElement(BaseDocumentTag item, Element element) throws DocumentHandlerException {
    // workaround for the missing multipliedLeading (aka line-spacing) support in iText
    if (item instanceof ParagraphTag) {
      ParagraphTag tag = (ParagraphTag) item;
      float leading = tag.getLeading();
      float lineSpacing = tag.getLineSpacing();
      extendLeading((Paragraph) element, item.getFont().getFont(), leading, lineSpacing);
    }
    this.section.add(element);
  }

  /**
   * Tells whether the element container is a top level container or not.
   * @return true if the element container is a top level container, else false.
   */
  public boolean isTopLevel() {
    return false;
  }

  /**
   * Gets the fixed vertical leading distance.
   * @return The current fixed vertical leading distance.
   */
  public float getLeading() {
    return leading;
  }

  /**
   * Gets the line spacing.
   * @return The current line spacing.
   */
  public float getLineSpacing() {
    return lineSpacing;
  }

  /**
   * Getter method for the the property spacingBefore.
   * @return The current value of property spacingBefore.
   */
  public float getSpacingBefore() {
    return spacingBefore;
  }

  /**
   * Getter method for the the property spacingAfter.
   * @return The current value of property spacingAfter.
   */
  public float getSpacingAfter() {
    return spacingAfter;
  }

  /**
   * Getter method for the the property titleFont.
   * @return The current value of property titleFont.
   */
  public DocumentFont getTitleFont() {
    return titleFont;
  }

  /**
   * Handles a new page request.
   * @param size The new document size, null if the size should remain the same.
   * @param margins The new document margins, null if the margins should remain the same.
   * @return A reference to the document. 
   * @exception DocumentHandlerException In case something went wrong.
   */
  public Document handleNewPage(Rectangle size, Rectangle margins) 
    throws DocumentHandlerException {
    
    this.section.add(Chunk.NEXTPAGE);

    return documentHandler.getDocument();
  }
  
}
