/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import java.text.DateFormat;
import java.text.Format;
import java.text.NumberFormat;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.DocumentHandlerException;

import org.ujac.util.text.FormatHelper;

/**
 * Name: SetFormatTag<br>
 * Description: Class handling the set-format tag.
 * 
 * @author lauerc
 */
public class SetFormatTag extends BaseDocumentTag {

  /** Definition of the 'data-type' attribute. */
  private static final AttributeDefinition DATA_TYPE = new AttributeDefinition(TagAttributes.ATTR_TYPE,
      new String [] {"int", "float", "long", "double", "String", "Date", "Time", "Timestamp", "Object"}, 
      true, "The type for which to define the format, supported types are: " + 
      "int, long, float, double, String, Date, Time, Timestamp and Object.");
  /** Definition of the 'format' attribute. */
  private static final AttributeDefinition FORMAT = new AttributeDefinition(TagAttributes.ATTR_FORMAT, 
      AttributeDefinition.TYPE_STRING, false,  "The format for the specified type. For more info see " +
      "JAVA SDK documentation for types java.text.DecimalFormat and java.text.SimpleDateFormat.");

  /** The item's name. */
  public static final String TAG_NAME = "set-format";

  /** The type name. */
  private String type = null;
  /** The format string. */
  private String formatString = null;
    
  /**
   * Constructs a SetFormatTag instance with no specific attributes.
   */
  public SetFormatTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Defines the output format for the given data type." ;
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(DATA_TYPE)    
      .addDefinition(FORMAT);
  }

  /**
   * @see org.ujac.print.BaseDocumentTag#isStructureChecksDisabled()
   */
  public boolean isStructureChecksDisabled() {
    return true;
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // getting attributes
    type = getStringAttribute(DATA_TYPE, false, null);
    formatString = getStringAttribute(FORMAT, true, null);
  }
  
  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    FormatHelper formatHelper = documentHandler.getFormatHelper();
    Format format = documentHandler.lookupFormat(formatString);
    if (format == null) {
      if (type.equalsIgnoreCase("int")) {
        format = formatHelper.setIntegerFormat(formatString);
      } else if (type.equalsIgnoreCase("double")) {
        format = formatHelper.setDoubleFormat(formatString);
      } else if (type.equalsIgnoreCase("Date")) {
        format = formatHelper.setDateFormat(formatString);
      } else if (type.equalsIgnoreCase("Time")) {
        format = formatHelper.setTimeFormat(formatString);
      } else if (type.equalsIgnoreCase("Timestamp")) {
        format = formatHelper.setTimestampFormat(formatString);
      }
      // registering font at document handler
      documentHandler.registerFormat(formatString, format);
    } else {
      // setting font at expression interpreter
      if (type.equalsIgnoreCase("int")) {
        formatHelper.setIntegerFormat((NumberFormat) format);
      } else if (type.equalsIgnoreCase("double")) {
        formatHelper.setDoubleFormat((NumberFormat) format);
      } else if (type.equalsIgnoreCase("Date")) {
        formatHelper.setDateFormat((DateFormat) format);
      } else if (type.equalsIgnoreCase("Time")) {
        formatHelper.setTimeFormat((DateFormat) format);
      } else if (type.equalsIgnoreCase("Timestamp")) {
        formatHelper.setTimestampFormat((DateFormat) format);
      }
    }
  }
   
}
