/*
 * Copyright (C) 2006 - 2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.DocumentHandlerException;

import com.lowagie.text.Font;

/**
 * Name: SpanTag<br>
 * Description: A class handling Span items.<br>
 * 
 * @author lauerc
 */
public class SpanTag extends BaseFontTag {

  /** The item's name. */
  public static final String TAG_NAME = "span";
   
  /**
   * Constructs a SpanTag instance with no specific attributes.
   */
  public SpanTag() {
    super(TAG_NAME);
  }


  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Defines the style for the nested tags.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.CLASS)
      .addDefinition(CommonAttributes.STYLE)
      .addDefinition(CommonAttributes.FONT)
      .addDefinition(CommonAttributes.FONT_COLOR)
      .addDefinition(CommonAttributes.FONT_FAMILY)
      .addDefinition(CommonAttributes.FONT_ENCODING)
      .addDefinition(CommonAttributes.FONT_SIZE)
      .addDefinition(CommonAttributes.FONT_STYLE)
      .addDefinition(CommonAttributes.TEXT_RISE);
  }
  
  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.FONT_NAME)
      .addDefinition(CommonStyleAttributes.FONT_COLOR)
      .addDefinition(CommonStyleAttributes.FONT_FAMILY)
      .addDefinition(CommonStyleAttributes.FONT_ENCODING)
      .addDefinition(CommonStyleAttributes.FONT_SIZE)
      .addDefinition(CommonStyleAttributes.FONT_STYLE)
      .addDefinition(CommonStyleAttributes.TEXT_RISE);
  }
  
  /**
   * @see org.ujac.print.BaseDocumentTag#isStyleable()
   */
  public boolean isStyleable() {
    return true;
  }

  /**
   * Is the item virtual container, such as if, else or foreach.
   * @return true if the item is such a container, else false.
   */
  public boolean isVirtualContainer() {
    return true;
  }
  
  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    // getting attributes
    String fontDef = getStringAttribute(CommonAttributes.FONT, true, CommonStyleAttributes.FONT_NAME);
    if (fontDef != null) {
      font = documentHandler.getFont(fontDef);
    }
    if (font != null) {
      this.family = font.getFamily();
      this.encoding = font.getEncoding();
      this.size = font.getSize();
      this.textRise = font.getTextRise();
      this.color = font.getColor();
    }
    
    color = getColorAttribute(CommonAttributes.FONT_COLOR, this.color, true, CommonStyleAttributes.FONT_COLOR);
    family = getStringAttribute(CommonAttributes.FONT_FAMILY, family, true, CommonStyleAttributes.FONT_FAMILY);
    encoding = getStringAttribute(CommonAttributes.FONT_ENCODING, encoding, true, CommonStyleAttributes.FONT_ENCODING);
    size = getDimensionAttribute(CommonAttributes.FONT_SIZE, size, true, CommonStyleAttributes.FONT_SIZE);
    String styleAttribute = getStringAttribute(CommonAttributes.FONT_STYLE, true, CommonStyleAttributes.FONT_STYLE);
    if (styleAttribute != null) {
      style = Font.getStyleValue(styleAttribute);
    } else {
      // for compatibility with old style handling
      styleAttribute = getStringAttribute(CommonAttributes.STYLE, true, null);
      if (styleAttribute != null) {
        style = Font.getStyleValue(styleAttribute);
      }
    }
    textRise = getDimensionAttribute(CommonAttributes.TEXT_RISE, textRise, true, CommonStyleAttributes.TEXT_RISE);

    // configuring font
    font = createFont(family, encoding, size, style, textRise, color);
  }

}
