/*
 * Copyright (C) 2003 - 2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import java.util.ArrayList;
import java.util.List;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.ChildDefinition;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.DocumentPageEventHandler;

import com.lowagie.text.Document;

import com.lowagie.text.pdf.PdfWriter;

/**
 * Name: StickyTag<br>
 * Description: A class marking sticky areas.
 * 
 * @author lauerc
 */
public class StickyTag extends BaseDocumentTag {

  /** Definition of the 'name' attribute. */
  private static final AttributeDefinition STICKY_NAME = CommonAttributes.NAME.cloneAttrDef("The name of the sticky area.")
      .setRequired(false);
  
  /** The item's name. */
  public static final String TAG_NAME = "sticky";

  /**
   * Name: StickyPageEventHandler<br>
   * Description: The page event handler, handling the output of absolutely positioned elements.
   */
  class StickyPageEventHandler extends DocumentPageEventHandler {
    
    /**
     * We override the onEndPage method.
     * @param writer The document writer.
     * @param document The iText document.
     */
    public void onEndPage(PdfWriter writer, Document document) {
      List imageHandlers = getStickyElementHandlers();
      int numHandlers = imageHandlers.size();
      for (int i = 0; i < numHandlers; i++) {
        ((DocumentPageEventHandler) imageHandlers.get(i)).onEndPage(writer, document);
      }
    }
    
    /**
     * We override the onCloseDocument method.
     * @param writer The document writer.
     * @param document The iText document.
     */
    public void onCloseDocument(PdfWriter writer, Document document) {
      List imageHandlers = getStickyElementHandlers();
      int numHandlers = imageHandlers.size();
      for (int i = 0; i < numHandlers; i++) {
        ((DocumentPageEventHandler) imageHandlers.get(i)).onCloseDocument(writer, document);
      }
    }

    /**
     * Tells whether this event handler is temporary, and should be removed after the 
     * first usage or not.
     * @return true, if this handler should be executed only once, else false.
     */
    public boolean isTemporary() {
      return false;
    }
  }

  /** The new-page tag this tag may be nested within. */
  protected NewPageTag newPageTag = null;
  /** The list of background image handlers. */
  private List stickyElementHandlers = new ArrayList();
  
  /**
   * Constructs a StickyTag instance with no specific attributes.
   */
  public StickyTag() {
    super(TAG_NAME);
  }


  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Defines a sticky area. This area can contain background images and other " +
        "absolutely positioned elements.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(STICKY_NAME);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(IfTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ElseTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(LogTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(SwitchTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BackgroundImageTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BoxTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ImageTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BarcodeTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(DatamatrixTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BarcodePdf417Tag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(BaseFontTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(MacroTag.class, 0, ChildDefinition.UNLIMITED));
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // determining the supprounding tags
    this.newPageTag = (NewPageTag) documentHandler.latestOfType(NewPageTag.TAG_NAME);
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }
    
    // clearing background image handlers
    stickyElementHandlers.clear();
    
    super.openItem();
  }
  
  /**
   * @see org.ujac.print.BaseDocumentTag#closeItem()
   */
  public void closeItem() throws DocumentHandlerException {
    super.closeItem();

    if (!isValid()) {
      return;
    }
    
    String key = "sticky-" + getStringAttribute(STICKY_NAME, "default", true, null);
    if (isInNewPageTag()) {
      newPageTag.queuePageEventHandler(key, new StickyPageEventHandler());
    } else {
      documentHandler.setPageEventHandler(key, new StickyPageEventHandler());
    }
  }

  /**
   * Adds a sticky element handler.
   * @param handler The sticky element handler to add.
   */
  public void addStickyElementHandler(DocumentPageEventHandler handler) {
    stickyElementHandlers.add(handler);
  }
  
  /**
   * Getter method for the the property stickyElementHandlers.
   * @return The current value of property stickyElementHandlers.
   */
  public List getStickyElementHandlers() {
    return stickyElementHandlers;
  }
  
  /**
   * Tells whether this tag is embedded within a new-page tag or not.
   * @return true if a surrounding new-page tag exists, else false.
   */
  protected boolean isInNewPageTag() {
    return this.newPageTag != null;
  }

}
