/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.AttributeHandler;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.StyleParser;
import org.ujac.util.CollectionUtils;

/**
 * Name: StyleDefItem<br>
 * Description: A class which handles style definitions.<br>
 * 
 * @author lauerc
 */
public class StyleDefTag extends BaseDocumentTag {

  /** The item's name. */
  public static final String TAG_NAME = "style-def";
    
  /** Definition of the 'font' attribute. */
  private static final AttributeDefinition STYLE_DEF_NAME = 
      CommonAttributes.NAME.cloneAttrDef(
          "The name of the style definition. In case you'd like to define the" +
          "style for all occurrences of a tag, put the tag name here. In case " +
          "the style should be valid for all tags, specify the tag name after a " +
          "leading '.' character. Otherwise append the name to the tag name " +
          "you'd like to define separated by a '.' character (example: cell.normal)."
      );
  
  /**
   * Constructs a StyleDefTag instance with no specific attributes.
   */
  public StyleDefTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Defines a style for the document. The style model aims to be similar to Cascading Style Sheets " +
        "well known in HTML.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(STYLE_DEF_NAME)
      .addDefinition(CommonAttributes.STYLE)
      .addDefinition(CommonAttributes.FONT)
      .addDefinition(CommonAttributes.BACKGROUND_COLOR)
      .addDefinition(CommonAttributes.BORDER_STYLE)
      .addDefinition(CommonAttributes.BORDER_WIDTH)
      .addDefinition(CommonAttributes.BORDER_COLOR)
      .addDefinition(CommonAttributes.HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonAttributes.VERTICAL_ALIGNMENT)
      .addDefinition(CommonAttributes.CELL_HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonAttributes.CELL_VERTICAL_ALIGNMENT)
      .addDefinition(CommonAttributes.PADDING)
      .addDefinition(CommonAttributes.PADDING_LEFT)
      .addDefinition(CommonAttributes.PADDING_RIGHT)
      .addDefinition(CommonAttributes.PADDING_TOP)
      .addDefinition(CommonAttributes.PADDING_BOTTOM)
      .addDefinition(CommonAttributes.LINE_SPACING)
      .addDefinition(CommonAttributes.LEADING)
      .addDefinition(CommonAttributes.SPACING_BEFORE)
      .addDefinition(CommonAttributes.SPACING_AFTER)
      .addDefinition(CommonAttributes.TEXT_RISE);
  }
  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonStyleAttributes.BACKGROUND_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_STYLE)
      .addDefinition(CommonStyleAttributes.BORDER_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_LEFT_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_RIGHT_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_TOP_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_BOTTOM_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_LEFT_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_RIGHT_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_TOP_COLOR)
      .addDefinition(CommonStyleAttributes.BORDER_BOTTOM_COLOR)
      .addDefinition(CommonStyleAttributes.HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.VERTICAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.CELL_HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.CELL_VERTICAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.PADDING)
      .addDefinition(CommonStyleAttributes.PADDING_LEFT)
      .addDefinition(CommonStyleAttributes.PADDING_RIGHT)
      .addDefinition(CommonStyleAttributes.PADDING_TOP)
      .addDefinition(CommonStyleAttributes.PADDING_BOTTOM)
      .addDefinition(CommonStyleAttributes.LINE_SPACING)
      .addDefinition(CommonStyleAttributes.LEADING)
      .addDefinition(CommonStyleAttributes.SPACING_BEFORE)
      .addDefinition(CommonStyleAttributes.SPACING_AFTER)
      .addDefinition(CommonStyleAttributes.TEXT_RISE)
      .addDefinition(CommonStyleAttributes.FONT_NAME)
      .addDefinition(CommonStyleAttributes.FONT_COLOR)
      .addDefinition(CommonStyleAttributes.FONT_ENCODING)
      .addDefinition(CommonStyleAttributes.FONT_FAMILY)
      .addDefinition(CommonStyleAttributes.FONT_SIZE)
      .addDefinition(CommonStyleAttributes.FONT_STYLE);
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
  }
  
  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    // getting attributes
    String name = getStringAttribute(CommonAttributes.NAME, false, null);
    Map styleMap = new HashMap();
    if (!name.startsWith(".") && (name.indexOf('.') != -1)) {
      int dotIdx = name.indexOf('.');
      String parentName = name.substring(dotIdx + 1);
      Map parentStyle = documentHandler.getStyle(parentName);
      if (parentStyle != null) {
        styleMap.putAll(parentStyle);
      }
    }
    
    CollectionUtils.put(styleMap, CommonStyleAttributes.FONT_NAME.getName(), getStringAttribute(CommonAttributes.FONT, true, CommonStyleAttributes.FONT_NAME));
    CollectionUtils.put(styleMap, CommonStyleAttributes.BACKGROUND_COLOR.getName(), getColorAttribute(CommonAttributes.BACKGROUND_COLOR, true, CommonStyleAttributes.BACKGROUND_COLOR));
    if (isAttributeDefined(CommonAttributes.BORDER_STYLE, CommonStyleAttributes.BORDER_STYLE)) {
      CollectionUtils.put(styleMap, CommonStyleAttributes.BORDER_STYLE.getName(), new Integer(getBorderAttribute(CommonAttributes.BORDER_STYLE, true, CommonStyleAttributes.BORDER_STYLE)));
    }

    if (isAttributeDefined(CommonAttributes.HORIZONTAL_ALIGNMENT, CommonStyleAttributes.HORIZONTAL_ALIGNMENT)) {
      CollectionUtils.put(styleMap, CommonStyleAttributes.HORIZONTAL_ALIGNMENT.getName(), new Integer(getHorizontalAlignmentAttribute(CommonAttributes.HORIZONTAL_ALIGNMENT, true, CommonStyleAttributes.HORIZONTAL_ALIGNMENT)));
    }
    if (isAttributeDefined(CommonAttributes.VERTICAL_ALIGNMENT, CommonStyleAttributes.VERTICAL_ALIGNMENT)) {
      CollectionUtils.put(styleMap, CommonStyleAttributes.VERTICAL_ALIGNMENT.getName(), new Integer(getVerticalAlignmentAttribute(CommonAttributes.VERTICAL_ALIGNMENT, true, CommonStyleAttributes.VERTICAL_ALIGNMENT)));
    }

    if (isAttributeDefined(CommonAttributes.CELL_HORIZONTAL_ALIGNMENT, CommonStyleAttributes.CELL_HORIZONTAL_ALIGNMENT)) {
      CollectionUtils.put(styleMap, CommonStyleAttributes.CELL_HORIZONTAL_ALIGNMENT.getName(), new Integer(getHorizontalAlignmentAttribute(CommonAttributes.CELL_HORIZONTAL_ALIGNMENT, true, CommonStyleAttributes.CELL_HORIZONTAL_ALIGNMENT)));
    }
    if (isAttributeDefined(CommonAttributes.CELL_VERTICAL_ALIGNMENT, CommonStyleAttributes.CELL_VERTICAL_ALIGNMENT)) {
      CollectionUtils.put(styleMap, CommonStyleAttributes.CELL_VERTICAL_ALIGNMENT.getName(), new Integer(getVerticalAlignmentAttribute(CommonAttributes.CELL_VERTICAL_ALIGNMENT, true, CommonStyleAttributes.CELL_VERTICAL_ALIGNMENT)));
    }
    if (isAttributeDefined(CommonAttributes.TEXT_RISE, CommonStyleAttributes.TEXT_RISE)) {
      CollectionUtils.put(styleMap, CommonStyleAttributes.TEXT_RISE.getName(), new Float(getDimensionAttribute(CommonAttributes.TEXT_RISE, true, CommonStyleAttributes.TEXT_RISE)));
    }
    
    String styleDef = getStringAttribute(CommonAttributes.STYLE, true, null);
    if (styleDef != null) {
      StyleParser styleParser = documentHandler.getStyleParser();
      styleParser.parseStyle(locator(), styleDef, supportedStyleAttributeMap, styleMap);
    }
    
    Object value = null;
    AttributeHandler attrHandler = documentHandler.getAttributeHandler();

    AttributeDefinitionMap attrDefs = buildSupportedStyleAttributes();
    List attrs = attrDefs.getSortedAttributes();
    int numAttrs = attrs.size();
    for (int i = 0; i < numAttrs; i++) {
      AttributeDefinition attrDef = (AttributeDefinition) attrs.get(i);
      value = getStyleAttribute(attrDef.getName());
      if (value != null) {
        value = attrHandler.typeCastAttributeValue(locator(), attrDef, attrDef.getName(), value);
        CollectionUtils.put(styleMap, attrDef.getName(), value);
      }
    }

    attrDefs = buildSupportedAttributes();
    attrs = attrDefs.getSortedAttributes();
    numAttrs = attrs.size();
    for (int i = 0; i < numAttrs; i++) {
      AttributeDefinition attrDef = (AttributeDefinition) attrs.get(i);
      String attrName = attrDef.getName();
      if (TagAttributes.ATTR_NAME.equals(attrName) || TagAttributes.ATTR_STYLE.equals(attrName)) {
        continue;
      }
      value = evalAttribute(attrName);
      if (value != null) {
        value = attrHandler.typeCastAttributeValue(locator(), attrDef, attrName, value);
        if (TagAttributes.ATTR_FONT.equals(attrName)) {
          CollectionUtils.put(styleMap, CommonStyleAttributes.FONT_NAME.getName(), value);
          continue;
        }
        if (TagAttributes.ATTR_BACKGROUND_COLOR.equals(attrName)) {
          CollectionUtils.put(styleMap, CommonStyleAttributes.BACKGROUND_COLOR.getName(), value);
          continue;
        }
        if (TagAttributes.ATTR_BORDER_STYLE.equals(attrName)) {
          CollectionUtils.put(styleMap, CommonStyleAttributes.BORDER_STYLE.getName(), value);
          continue;
        }
        if (TagAttributes.ATTR_BORDER_WIDTH.equals(attrName)) {
          CollectionUtils.put(styleMap, CommonStyleAttributes.BORDER_WIDTH.getName(), value);
          continue;
        }
        if (TagAttributes.ATTR_BORDER_COLOR.equals(attrName)) {
          CollectionUtils.put(styleMap, CommonStyleAttributes.BORDER_COLOR.getName(), value);
          continue;
        }
        if (TagAttributes.ATTR_HORIZONTAL_ALIGNMENT.equals(attrName)) {
          CollectionUtils.put(styleMap, CommonStyleAttributes.HORIZONTAL_ALIGNMENT.getName(), value);
          continue;
        }
        if (TagAttributes.ATTR_VERTICAL_ALIGNMENT.equals(attrName)) {
          CollectionUtils.put(styleMap, CommonStyleAttributes.VERTICAL_ALIGNMENT.getName(), value);
          continue;
        }
        if (TagAttributes.ATTR_CELL_HORIZONTAL_ALIGNMENT.equals(attrName)) {
          CollectionUtils.put(styleMap, CommonStyleAttributes.CELL_HORIZONTAL_ALIGNMENT.getName(), value);
          continue;
        }
        if (TagAttributes.ATTR_CELL_VERTICAL_ALIGNMENT.equals(attrName)) {
          CollectionUtils.put(styleMap, CommonStyleAttributes.CELL_VERTICAL_ALIGNMENT.getName(), value);
          continue;
        }
        if (TagAttributes.ATTR_PADDING.equals(attrName)) {
          CollectionUtils.put(styleMap, CommonStyleAttributes.PADDING.getName(), value);
          continue;
        }
        if (TagAttributes.ATTR_PADDING_LEFT.equals(attrName)) {
          CollectionUtils.put(styleMap, CommonStyleAttributes.PADDING_LEFT.getName(), value);
          continue;
        }
        if (TagAttributes.ATTR_PADDING_RIGHT.equals(attrName)) {
          CollectionUtils.put(styleMap, CommonStyleAttributes.PADDING_RIGHT.getName(), value);
          continue;
        }
        if (TagAttributes.ATTR_PADDING_TOP.equals(attrName)) {
          CollectionUtils.put(styleMap, CommonStyleAttributes.PADDING_TOP.getName(), value);
          continue;
        }
        if (TagAttributes.ATTR_PADDING_BOTTOM.equals(attrName)) {
          CollectionUtils.put(styleMap, CommonStyleAttributes.PADDING_BOTTOM.getName(), value);
          continue;
        }
        if (TagAttributes.ATTR_LINE_SPACING.equals(attrName)) {
          CollectionUtils.put(styleMap, CommonStyleAttributes.LINE_SPACING.getName(), value);
          continue;
        }
        if (TagAttributes.ATTR_LEADING.equals(attrName)) {
          CollectionUtils.put(styleMap, CommonStyleAttributes.LEADING.getName(), value);
          continue;
        }
        if (TagAttributes.ATTR_SPACING_BEFORE.equals(attrName)) {
          CollectionUtils.put(styleMap, CommonStyleAttributes.SPACING_BEFORE.getName(), value);
          continue;
        }
        if (TagAttributes.ATTR_SPACING_AFTER.equals(attrName)) {
          CollectionUtils.put(styleMap, CommonStyleAttributes.SPACING_AFTER.getName(), value);
          continue;
        }
        if (TagAttributes.ATTR_TEXT_RISE.equals(attrName)) {
          CollectionUtils.put(styleMap, CommonStyleAttributes.TEXT_RISE.getName(), value);
          continue;
        }
      }
    }

    // registering style at document handler
    documentHandler.registerStyle(name, styleMap);
  }

}
