/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import java.awt.Graphics2D;

import java.awt.print.PageFormat;
import java.awt.print.Paper;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InterruptedIOException;

import java.net.MalformedURLException;
import java.net.URL;

import org.apache.batik.bridge.BridgeContext;
import org.apache.batik.bridge.DocumentLoader;
import org.apache.batik.bridge.UserAgent;
import org.apache.batik.bridge.UserAgentAdapter;

import org.apache.batik.dom.svg.SAXSVGDocumentFactory;
import org.apache.batik.dom.svg.SVGDOMImplementation;
import org.apache.batik.dom.svg.SVGDocumentFactory;
import org.apache.batik.dom.svg.SVGOMDocument;

import org.apache.batik.transcoder.TranscoderException;
import org.apache.batik.transcoder.TranscoderInput;
import org.apache.batik.transcoder.XMLAbstractTranscoder;

import org.apache.batik.transcoder.print.PrintTranscoder;

import org.apache.batik.util.MimeTypeConstants;
import org.apache.batik.util.ParsedURL;

import org.ujac.print.DocumentHandler;
import org.ujac.util.xml.XmlUtils;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import com.lowagie.text.BadElementException;
import com.lowagie.text.Image;

import com.lowagie.text.pdf.FontMapper;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfTemplate;
import com.lowagie.text.pdf.PdfWriter;

/**
 * Name: SvgImageBuilder<br>
 * Description: ImageBuilder implementation for SVG format.
 * 
 * @author lauerc
 */
public class SvgImageBuilder extends BaseImageBuilder {

  /** The SVG document type ID. */
  public static final char[] SVG_DOC_TYPE_ID = "-//W3C//DTD SVG".toCharArray();

  /**
   * Name: SvgUserAgent<br>
   * Description: A specific batik UserAgent implementation.
   */
  private class SvgUserAgent extends UserAgentAdapter {
    
    /** The document handler. */
    private DocumentHandler documentHandler;
    
    /**
     * Constructs a UserAgent instance with specific attributes.
     * @param documentHandler The document handler.
     */
    protected SvgUserAgent(DocumentHandler documentHandler) {
      this.documentHandler = documentHandler;
    }
    
    /**
     * @see org.apache.batik.bridge.UserAgentAdapter#getXMLParserClassName()
     */
    public String getXMLParserClassName() {
      return documentHandler.getXmlReaderClass();
    }
    
  }
  
  /**
   * Name: DocumentFactory<br>
   * Description: A specific batik SVGDocumentFactory implementation.
   */
  private class DocumentFactory extends SAXSVGDocumentFactory {

    /** The document handler. */
    private DocumentHandler documentHandler;
    /** The path, where the image is located at the resoure loader. */
    private String path;
    
    /**
     * Constructs a DocumentFactory instance with specific attributes.
     * @param documentHandler The document handler.
     * @param path The path, where the image is located at the resoure loader.
     */
    public DocumentFactory(DocumentHandler documentHandler, String path) {
      super(documentHandler.getXmlReaderClass());
      this.documentHandler = documentHandler;
      this.path = path;
    }

    /**
     * Creates a SVG Document instance.<br>
     * This method supports gzipped sources.
     * @param uri The document URI.
     * @return The created document.
     * @exception IOException if an error occured while reading the document.
     */
    public Document createDocument(String uri) throws IOException {
      ParsedURL purl = new ParsedURL(uri);
      InputStream is = null;
      if (uri.startsWith("file:") && (path != null)) {
        String resourceLocation = path + uri.substring(5);
        is = new ByteArrayInputStream(documentHandler.loadResource(resourceLocation));
      } else {
        is = purl.openStream(MimeTypeConstants.MIME_TYPES_SVG);
      }
      InputSource isrc = new InputSource(is);
      // now looking for a charset encoding in the content type such
      // as "image/svg+xml; charset=iso8859-1" this is not official
      // for image/svg+xml yet! only for text/xml and maybe
      // for application/xml
      String contentType = purl.getContentType();
      int cindex = -1;
      if (contentType != null) {
        contentType = contentType.toLowerCase();
        cindex = contentType.indexOf(HTTP_CHARSET);
      }
      if (cindex != -1) {
        int i = cindex + HTTP_CHARSET.length();
        int eqIdx = contentType.indexOf('=', i);
        if (eqIdx != -1) {
          eqIdx++; // no one is interested in the equals sign...
          String charset;
          // The patch had ',' as the terminator but I suspect
          // that is the delimiter between possible charsets,
          // but if another 'attribute' were in the accept header
          // charset would be terminated by a ';'. So I look
          // for both and take to closer of the two.
          int idx = contentType.indexOf(',', eqIdx);
          int semiIdx = contentType.indexOf(';', eqIdx);
          if ((semiIdx != -1) && ((semiIdx < idx) || (idx == -1))) {
            idx = semiIdx;
          }
          if (idx != -1) {
            charset = contentType.substring(eqIdx, idx);
          } else {
            charset = contentType.substring(eqIdx);
          }
          isrc.setEncoding(charset.trim());
        }
      }
      isrc.setSystemId(uri);
      Document doc = super.createDocument(SVGDOMImplementation.SVG_NAMESPACE_URI, "svg", uri, isrc);
      try {
        ((SVGOMDocument) doc).setURLObject(new URL(purl.toString()));
      } catch (MalformedURLException mue) {
        // Not very likely to happen given we already opened the stream.
        throw new IOException("Malformed URL: " + uri);
      }
      return doc;
    }
    
    /**
     * @see org.apache.batik.dom.util.SAXDocumentFactory#createDocument(org.xml.sax.InputSource)
     */
    protected Document createDocument(InputSource is) throws IOException {
      try {
        XMLReader parser = XmlUtils.createXmlReader(parserClassName);

        parser.setContentHandler(this);
        parser.setDTDHandler(this);
        parser.setEntityResolver(this);
        parser.setErrorHandler((errorHandler == null) ? this : errorHandler);

        parser.setFeature("http://xml.org/sax/features/namespaces", true);
        parser.setFeature("http://xml.org/sax/features/namespace-prefixes", true);
        parser.setFeature("http://xml.org/sax/features/validation", isValidating);
        parser.setProperty("http://xml.org/sax/properties/lexical-handler", this);
        parser.parse(is);
      } catch (SAXException e) {
        Exception ex = e.getException();
        if (ex != null && ex instanceof InterruptedIOException) {
          throw (InterruptedIOException)ex;
        }
        throw new IOException(e.getMessage());
      }

      currentNode = null;
      Document ret = document;
      document = null;
      return ret;
    }

  }

  /**
   * Name: SvgDocumentLoader<br>
   * Description: A specific batik DocumentLoader implementation.
   */
  private class SvgDocumentLoader extends DocumentLoader {
    
    /**
     * Constructs a new SvgDocumentLoader with the specified XML parser.
     * @param userAgent The user agent to use
     * @param factory The document factory to use. 
     */
    protected SvgDocumentLoader(UserAgent userAgent, SVGDocumentFactory factory) {
      super(userAgent);
      this.documentFactory = factory;
    }

  }
  
  /**
   * Name: ImageTranscoder<br>
   * Description: A specific batik PrintTranscoder implementation.
   */
  private class ImageTranscoder extends PrintTranscoder {
    
    /** The document factory. */
    private SVGDocumentFactory documentFactory;
    /** The user agent. */
    private UserAgent userAgent;
    
    /**
     * Constructs a ImageTranscoder instance with specific attributes.
     * @param documentFactory The document factory.
     * @param userAgent The user agent.
     */
    protected ImageTranscoder(SVGDocumentFactory documentFactory, UserAgent userAgent) {
      this.userAgent = userAgent;
      this.documentFactory = documentFactory;
    }
    
    /**
     * @see org.apache.batik.transcoder.SVGAbstractTranscoder#createBridgeContext()
     */
    protected BridgeContext createBridgeContext() {
      return new BridgeContext(userAgent, new SvgDocumentLoader(userAgent, documentFactory));
    }
    
  }
  
  /**
   * Checks, if the implementation is capable of handling the given image data or not.
   * @param imageData The image data to handle.
   * @return true if this implementation can build a image from the given data, else false.
   */
  public boolean checkFormat(byte[] imageData) {
    
    if (!isXml(imageData)) {
      try {
        imageData = uncompress(imageData);
        if (!isXml(imageData)) {
          return false;
        }
      } catch (IOException ex) {
        // no zip stream
        return false;
      }
    }

    int maxIdx = Math.max(imageData.length, 1024);
    int docTypeLength = SVG_DOC_TYPE_ID.length;
    
   outer:
    for (int i = 5; i < maxIdx; i++) {
      if (imageData[i] == SVG_DOC_TYPE_ID[0]) {
        if ((docTypeLength + i) > maxIdx) {
          // not possible
          return false;
        }
        for (int j = 1; j < docTypeLength; j++) {
          if (imageData[j + i] != SVG_DOC_TYPE_ID[j]) {
            continue outer;
          }
        }
        // seems, we've found what we're looking for
        return true;
      }
    }
    
    return false;
  }
  
  /**
   * Builds an image from the given data.
   * @param documentHandler The document handler.
   * @param imageData The image data to handle.
   * @param path The path, where the image is located at the resoure loader.
   * @param width The desired output width for the image (zero if unspecified).
   * @param height The desired output height for the image (zero if unspecified).
   * @param page The number of the page to use as image, useless for this image builder.
   * @return The generated image.
   * @exception IOException In case the image creation failed.
   */
  public Image buildImage(DocumentHandler documentHandler, byte[] imageData, 
                          String path, float width, float height, int page) 
    throws IOException {

    try {
      imageData = uncompress(imageData);
    } catch (IOException ex) {
      // not compressed
    }
    Graphics2D graphics = null;
    try {
      // first loading DOM tree of SVG document
      UserAgent userAgent = new SvgUserAgent(documentHandler);
      SVGDocumentFactory documentFactory = new DocumentFactory(documentHandler, path);
      org.w3c.dom.Document svgDoc = new SvgDocumentLoader(userAgent, documentFactory)
        .loadDocument("file:image.svg", new ByteArrayInputStream(imageData));
      // parsing the original size of the SVG image
      NodeList roots = svgDoc.getElementsByTagName("svg");
      Node root = roots.item(0);
      NamedNodeMap attrs = root.getAttributes();
      Node widthNode = attrs.getNamedItem("width");
      float imageWidth = Float.parseFloat(widthNode.getChildNodes().item(0).getNodeValue());
      Node heightNode = attrs.getNamedItem("height");
      float imageHeight = Float.parseFloat(heightNode.getChildNodes().item(0).getNodeValue());
      
      PdfWriter writer = (PdfWriter) documentHandler.getDocumentWriter();
      PdfContentByte cb = writer.getDirectContent();

      PdfTemplate tp = cb.createTemplate(imageWidth, imageHeight);
    
      FontMapper fontMapper = documentHandler.getFontMapper();
      graphics = tp.createGraphics(imageWidth, imageHeight, fontMapper);

      //long startTime = System.currentTimeMillis();
      PrintTranscoder prm = new ImageTranscoder(documentFactory, userAgent);

      String xmlReaderClassName = documentHandler.getXmlReaderClass();
      if (xmlReaderClassName != null) {
        prm.addTranscodingHint(XMLAbstractTranscoder.KEY_XML_PARSER_CLASSNAME, xmlReaderClassName);
      }
      final StringBuffer errorMsg = new StringBuffer();
      prm.setErrorHandler(new org.apache.batik.transcoder.DefaultErrorHandler() {
        /**
         * @see org.apache.batik.transcoder.DefaultErrorHandler#error(org.apache.batik.transcoder.TranscoderException)
         */
        public void error(TranscoderException ex) throws TranscoderException {
          errorMsg.append(ex.getMessage());
          ex.printStackTrace();
          super.error(ex);
        }
        /**
         * @see org.apache.batik.transcoder.DefaultErrorHandler#fatalError(org.apache.batik.transcoder.TranscoderException)
         */
        public void fatalError(TranscoderException ex) throws TranscoderException {
          errorMsg.append(ex.getMessage());
          ex.printStackTrace();
          super.fatalError(ex);
        }
        /**
         * @see org.apache.batik.transcoder.DefaultErrorHandler#warning(org.apache.batik.transcoder.TranscoderException)
         */
        public void warning(TranscoderException ex) throws TranscoderException {
          super.warning(ex);
        }
      });
      TranscoderInput ti = new TranscoderInput(svgDoc);
      prm.transcode(ti, null);
      //long endTime = System.currentTimeMillis();
      //System.out.println("Needed " + (endTime - startTime) + "ms to transcode SVG image");

      //startTime = System.currentTimeMillis();
      PageFormat pg = new PageFormat();
      Paper pp = new Paper();
      pp.setSize(width, height);
      pp.setImageableArea(0, 0, imageWidth, imageHeight);
      pg.setPaper(pp);
      prm.print(graphics, pg, 0);
      //endTime = System.currentTimeMillis();
      //System.out.println("Needed " + (endTime - startTime) + "ms to render SVG image");

      if (errorMsg.length() >  0) {
        // error occurred
        throw new IOException(errorMsg.toString());
      }
      
      // creating image from PDF.
      return Image.getInstance(tp);
    } catch (BadElementException ex) {
      throw new IOException("Unable to create image from SVG document: " + ex.getMessage());
    } finally {
      if (graphics != null) {
        // disposing graphics context, important to make future output to 
        // pdf content byte work.
        graphics.dispose();
      }
    }
  }

}
