/*
 * Copyright (C) 2003 - 2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.ChildDefinition;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.DocumentHandlerException;

/**
 * Name: SwitchTag<br>
 * Description: A class handling 'switch' items.<br>
 * 
 * @author lauerc
 */
public class SwitchTag extends BaseDocumentTag {

  /** Definition of the 'value' attribute. */
  private static final AttributeDefinition VALUE = new AttributeDefinition(TagAttributes.ATTR_VALUE, 
      AttributeDefinition.TYPE_STRING, true, "Expression serving the value to check against the nested " +
      "case items.");

  /** The item's name. */
  public static final String TAG_NAME = "switch";

  /** The value of the switch item. */
  private String value = null;
  /** The flag indicating whether a match was already found or not. */
  private boolean matchFound = true;

  /**
   * Constructs a SwitchTag instance with no specific attributes.
   */
  public SwitchTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Specifies the value that nested 'case' items compare their value to.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(VALUE);
  }
  /**
   * @see org.ujac.print.BaseDocumentTag#buildSupportedChilds()
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(CaseTag.class, 1, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(DefaultTag.class, 0, 1));
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    // resetting matchFound flag
    matchFound = false;
    if (isValid()) {
      // getting value attribute
      value = getStringAttribute(VALUE, true, null);
    }
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while closing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
  }

  /**
   * Checks if the given value equals the 
   * @param value The value to compare.
   * @return true if the given value matches the value of this instance, else false.
   */
  public boolean equalsValue(String value) {
    boolean result = this.value.equals(value);
    if (result) {
      this.matchFound = true;
    }
    return result;
  }

  /**
   * Checks whether a match was already found or not. 
   * @return true if a match was previously found, else false. 
   */
  public boolean isMatchFound() {
    return matchFound;
  }
}
