/*
 * Copyright (C) 2003 - 2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import java.util.StringTokenizer;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.CellContainer;
import org.ujac.print.ChildDefinition;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.ElementContainer;
import org.ujac.print.FontHolder;
import org.ujac.print.StyleHolder;
import org.ujac.print.TagAttributeException;
import org.ujac.print.DocumentHandlerException;

import org.ujac.util.exi.ExpressionException;

import com.lowagie.text.Cell;
import com.lowagie.text.Table;
import com.lowagie.text.BadElementException;

/**
 * Name: TableTag<br>
 * Description: An item handling tables.<br>
 * 
 * @author lauerc
 */
public class TableTag extends BaseElementTag implements CellContainer, FontHolder, StyleHolder {
  
  /** The item's name. */
  public static final String TAG_NAME = "table";

  /** The current cell attributes. */
  private CellAttributes cellAttributes = null;

  /** The iText table. */
  private Table table = null;
  /** The number of cells. */
  private int numCells = 0;
  /** The number of table columns. */
  private int columns = 0;
  /** The width of the whole table. */
  private float width = 0.0f;
  /** The widths attribute. */
  private String widthsAttr = null;
  /** The widths of the table columns. */
  private float[] widths = null;
  /** The horizontal alignment of the table. */ 
  private int hAlign = 0;

  /**
   * Constructs a TableTag instance with no specific attributes.
   */
  public TableTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Adds a table to its parent item. Tables are very useful if you want to layout your " +
      "document nicely.";
  }


  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.CLASS)
      .addDefinition(CommonAttributes.STYLE)
      .addDefinition(CommonAttributes.FONT)
      .addDefinition(CommonAttributes.TABLE_COLUMNS)
      .addDefinition(CommonAttributes.TABLE_WIDTHS)
      .addDefinition(CommonAttributes.TABLE_WIDTH)
      .addDefinition(CommonAttributes.TABLE_HALIGN);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(BaseFontTag.class, 0, ChildDefinition.UNLIMITED)) 
      .addDefinition(new ChildDefinition(MacroTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(IfTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ElseTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(LogTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(SwitchTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ForeachTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(TocTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(IndexTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(CellTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ImportTag.class, 0, ChildDefinition.UNLIMITED)); 
  }

  /**
   * Tells whether or not this tag does respect style attributes or not.
   * @return always <code>true</code>.
   */
  public boolean isStyleable() {
    return true;
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    // inherriting the surrounding cell's attributes
    cellAttributes = new CellAttributes(getCommonAttributesHolder());
    cellAttributes.parseAttributes(this, null);

    // setting the attributes
    columns = getIntegerAttribute(CommonAttributes.TABLE_COLUMNS, true, null);
    widthsAttr = getStringAttribute(CommonAttributes.TABLE_WIDTHS, false, null);
    width = getFloatAttribute(CommonAttributes.TABLE_WIDTH, true, CommonStyleAttributes.TABLE_WIDTH);

    if (widthsAttr != null) {
      if (columns > 0) {
        throw new TagAttributeException(locator(), "Specify eigther widths or columns attribute for tables, not both!");
      }
      StringTokenizer tkn = null;
      try {
        tkn = new StringTokenizer(documentHandler.evalString(widthsAttr), ",");
      } catch (ExpressionException ex) {
        throw new TagAttributeException(locator(), "Attribute evaluation failed: " + ex.getMessage(), ex);
      }
      
      columns = tkn.countTokens();
      widths = new float[columns];
      int idx = 0;
      while (tkn.hasMoreTokens()) {
        String token = tkn.nextToken();
        try {
          widths[idx] = Float.parseFloat(token);
          ++idx;
        } catch (NumberFormatException ex) {
          throw new TagAttributeException(locator(), "Attribute widths must hold a list of floating point numbers, separated by ',' (was '" + token + "').", ex);
        }
      }
    } else {
      widths = new float[columns];
      float averageWidth = 100.0f / columns;
      for (int i = 0; i < columns; i++) {
        widths[i] = averageWidth;
      }
    }
    hAlign = getHorizontalAlignmentAttribute(CommonAttributes.HORIZONTAL_ALIGNMENT, true, CommonStyleAttributes.HORIZONTAL_ALIGNMENT);
    
    // creating the table    
    try {
      table = new Table(columns);
    } catch (BadElementException ex) {
      throw new DocumentHandlerException(locator(), "Unable to create table: " + ex.getMessage(), ex);
    }

    // configuring the table
    if (width > 0.0f) {
      table.setWidth(width);
    }
    if (widths != null) {
      try {
        // setting table widths
        table.setWidths(widths);
      } catch (BadElementException ex) {
        throw new DocumentHandlerException(
            locator(), "Invalid table widths: '" + 
            getStringAttribute(CommonAttributes.TABLE_WIDTHS, true, null) + "': " + 
            ex.getMessage(), ex
        );
      }
    }
    // setting the horizntal aligmnemt
    table.setAlignment(hAlign);
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while closing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    // adding table to latest element container
    ElementContainer elementContainer = getElementContainer();
    elementContainer.addElement(this, table);
  }

  /**
   * Adds a cell to the container.
   * @param item The document item.
   * @param cell The cell to be added.
   * @exception DocumentHandlerException If something went wrong when adding the cell.
   */
  public void addCell(BaseDocumentTag item, Cell cell) throws DocumentHandlerException {
    // incrementing header cell counter
    numCells += Math.max(cell.getColspan(), 1);
    // adding cell to table
    table.addCell(cell);
  }
  
  /**
   * Gets the number of remaining columns of the current row.
   * @return The number of remaining column to the end of the current row.
   */
  public int getRemainingColumns() {
    return columns - (numCells % columns); 
  }

  /**
   * Gets the iText table.
   * @return The iText table.
   */
  public Table getTable() {
    return table;
  }

  /**
   * Gets the cell attributes.
   * @return The cell attributes.
   */
  public CellAttributes getCellAttributes() {
    return cellAttributes;
  }

}
