/*
 * Copyright (C) 2005 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import java.awt.Color;
import java.awt.Font;

import org.ujac.chart.ChartItemPosition;
import org.ujac.chart.TextChartItem;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.DocumentFont;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.ParameterHolder;
import org.ujac.print.TagAttributeException;

import org.ujac.util.template.TemplateException;

import com.lowagie.text.Element;

/**
 * Name: TextChartItemTag<br>
 * Description: A class handling &lt;text-chart-item&gt; tags.<br>.
 * 
 * @author lauerc
 */
public class TextChartItemTag  extends BaseDocumentTag {

  /** The item's name. */
  public static final String TAG_NAME = "text-chart-item";

  /** The macro item. */
  private ParameterHolder parameterHolder = null;
  /** The name attribute. */
  private String name = null;
  /** The font name. */
  private String fontName = null;
  /** The horizontal alignment of the text item. */ 
  private int hAlign = 0;
  /** The vertical alignment of the text item. */ 
  private int vAlign = 0;
  /** The text alignment. */ 
  private int textAlign = 0;
 /** The rotation flag. */ 
  private boolean rotate = false;
  
  /**
   * Constructs a TextChartItemTag instance with no specific attributes.
   */
  public TextChartItemTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Adds a text item parameter to its surrounding chart tag.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.RENDERED)
      .addDefinition(CommonAttributes.NAME)
      .addDefinition(CommonAttributes.FONT)
      .addDefinition(CommonAttributes.HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonAttributes.VERTICAL_ALIGNMENT)
      .addDefinition(CommonAttributes.TEXT_ALIGNMENT)
      .addDefinition(CommonAttributes.ROTATE_FLAG);
  }
  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.FONT_NAME)
      .addDefinition(CommonStyleAttributes.HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.VERTICAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.TEXT_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.ROTATE_FLAG);
  }

  /**
   * Tells whether textual content is allowed for the tag or not.
   * @return true if the item is allowed to contain textual content, else false.
   */
  public boolean isTextBodyAllowed() {
    return true;
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // getting parents
    this.parameterHolder = (ParameterHolder) documentHandler.latestOfType(ParameterHolder.class);
    if (parameterHolder == null) {
      throw new DocumentHandlerException(locator(), "Item '" + getName() + "' is not supported here.");
    }
    // getting attributes
    this.name = getStringAttribute(CommonAttributes.NAME, false, null);
  }
  
  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    // getting attributes
    this.fontName = getStringAttribute(CommonAttributes.FONT, true, CommonStyleAttributes.FONT_NAME);
    this.hAlign = getHorizontalAlignmentAttribute(CommonAttributes.HORIZONTAL_ALIGNMENT, true, CommonStyleAttributes.HORIZONTAL_ALIGNMENT);
    this.vAlign = getVerticalAlignmentAttribute(CommonAttributes.VERTICAL_ALIGNMENT, true, CommonStyleAttributes.VERTICAL_ALIGNMENT);
    this.textAlign = getHorizontalAlignmentAttribute(CommonAttributes.TEXT_ALIGNMENT, true, CommonStyleAttributes.TEXT_ALIGNMENT);
    if (!isAttributeDefined(CommonAttributes.TEXT_ALIGNMENT, CommonStyleAttributes.TEXT_ALIGNMENT)) {
      this.textAlign = getHorizontalAlignmentAttribute(CommonAttributes.HORIZONTAL_ALIGNMENT, true, CommonStyleAttributes.HORIZONTAL_ALIGNMENT);
    }
    this.rotate = getBooleanAttribute(CommonAttributes.ROTATE, true, CommonStyleAttributes.ROTATE);
  }


  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the macro-param item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }
    
    // checking parameter definition
    String expr = getContent();
    Object value = null;
    
    try {
       String text = documentHandler.executeTemplate(expr);
       Font textFont = null;
       Color textColor = null;
       DocumentFont docFont = null;
       if (fontName != null) {
         docFont = documentHandler.getFont(fontName);
       } else {
         docFont = getFont();
       }
       if (docFont != null) {
         textFont = documentHandler.getAwtFont(docFont);
         textColor = docFont.getColor();
       }
       
       short hAlign = 0;
       switch (this.hAlign) {
         case Element.ALIGN_LEFT:
           hAlign = ChartItemPosition.LEFT;
           break;
         case Element.ALIGN_CENTER:
           hAlign = ChartItemPosition.CENTER;
           break;
         case Element.ALIGN_RIGHT:
           hAlign = ChartItemPosition.RIGHT;
           break;
         default:
           hAlign = ChartItemPosition.LEFT;
           break;
       }

       short vAlign = 0;
       switch (this.vAlign) {
         case Element.ALIGN_TOP:
           vAlign = ChartItemPosition.TOP;
           break;
         case Element.ALIGN_MIDDLE:
           vAlign = ChartItemPosition.MIDDLE;
           break;
         case Element.ALIGN_BOTTOM:
           vAlign = ChartItemPosition.BOTTOM;
           break;
         default:
           vAlign = ChartItemPosition.TOP;
           break;
       }

       value = new TextChartItem(
           new ChartItemPosition(hAlign, vAlign), 
           textAlign, text, textFont, textColor
       ).setRotate(rotate);
    } catch (TemplateException ex) {
      throw new TagAttributeException(locator(), "Attribute evaluation failed: " + ex.getMessage(), ex);
    }
    // adding evaluated value to the surrounding macro item.
    parameterHolder.setParameter(this.name, value);
  }

}
