/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.CellContainer;
import org.ujac.print.Condition;
import org.ujac.print.DocumentFont;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.ElementContainer;
import org.ujac.print.PdfCellContainer;
import org.ujac.print.PdfTableHolder;
import org.ujac.print.PhraseHolder;
import org.ujac.print.TocItem;

import org.ujac.util.template.TemplateException;

import com.lowagie.text.Cell;
import com.lowagie.text.Chunk;
import com.lowagie.text.Element;
import com.lowagie.text.Phrase;
import com.lowagie.text.pdf.PdfPCell;

/**
 * Name: TocTag<br>
 * Description: Handler class for the &lt;toc&gt; tag.<br/>
 *   This tag loops over the table of contents entries, which have been
 *   registered during the main document processing step.
 * 
 * @author lauerc
 */
public class TocTag
  extends BaseLoopTag 
  implements Condition, ElementContainer, CellContainer, PdfCellContainer, PhraseHolder {

  /** The item's name. */
  public static final String TAG_NAME = "toc";

  /** Definition of the 'loop-variable' attribute. */
  private static final AttributeDefinition TOC_LOOP_VARIABLE = 
      CommonAttributes.LOOP_VARIABLE.cloneAttrDef(
          "The loop variable, holding the current TOC entry. " + 
          "Together with the loop variable there " +
          "will be a property available called variable + 'Idx', " +
          "which holds the index of the current element from the sequence."
       );
   
  /** The cell container. */
  protected CellContainer cellContainer = null;
  /** The PDF cell container. */
  protected PdfCellContainer pdfCellContainer = null;
  /** The phrase holder. */
  protected PhraseHolder phraseHolder = null;
  
  /** The phrase which takes the content. */
  protected Phrase phrase = null;

  /** Indicates, whether elements or cells have been added through this condition or not. */
  protected boolean childsAdded = false;

  /**
   * Constructs a TocTag instance with no specific attributes.
   */
  public TocTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "This tag loops over the table of contents entries, which have been " +
      "registered during the main document processing step.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(TOC_LOOP_VARIABLE);
  }

  /**
   * Is the item virtual container, such as if, else or foreach.
   * @return true if the item is such a container, else false.
   */
  public boolean isVirtualContainer() {
    return true;
  }
  
  /**
   * Tells whether textual content is allowed for the tag or not.
   * @return true if the item is allowed to contain textual content, else false.
   */
  public boolean isTextBodyAllowed() {
    return true;
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // setting the containers.
    this.cellContainer = documentHandler.latestCellContainer();
    this.pdfCellContainer = documentHandler.latestPdfCellContainer();
    this.phraseHolder = documentHandler.latestPhraseHolder();
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the foreach item.
   */
  public void openItem() throws DocumentHandlerException {
    // defining the sequence iterator
    this.sequenceHolder = documentHandler.getTocItems();
    
    // letting the super class do the rest!
    super.openItem();
  }

  /**
   * Pre-processes the child items of the foreach item.
   * @return Tells, whether or not to repeat the child items, returning false skips the execution
   *   for the current item.
   * @exception DocumentHandlerException Thrown in case something went wrong.
   */
  protected boolean preRepeatChildItems() throws DocumentHandlerException {
    TocItem currentItem = (TocItem) getCurrentValue();
    String currentRelevance = currentItem.getTocRelevance();
    if ("all".equals(currentRelevance)) {
      return true;
    }
    return "toc".equals(currentRelevance);
  }

  /**
   * Adds a element to the container.
   * @param item The document item.
   * @param element The element to be added.
   * @exception DocumentHandlerException If something went wrong when adding the element.
   */
  public void addElement(BaseDocumentTag item, Element element) throws DocumentHandlerException {
    if (isTrue()) {
      flushContent();
      ElementContainer elementContainer = getElementContainer();
      elementContainer.addElement(item, element);
      childsAdded = true;
    }
  }

  /**
   * Tells whether the element container is a top level container or not.
   * @return true if the element container is a top level container, else false.
   */
  public boolean isTopLevel() {
    ElementContainer elementContainer = getElementContainer();
    return elementContainer.isTopLevel();
  }

  /**
   * Adds a cell to the container.
   * @param item The document item.
   * @param cell The cell to be added.
   * @exception DocumentHandlerException If something went wrong when adding the cell.
   */
  public void addCell(BaseDocumentTag item, Cell cell) throws DocumentHandlerException {
    if (isTrue()) {
      cellContainer.addCell(item, cell);
      childsAdded = true;
    }
  }

  /**
   * Gets the number of remaining columns of the current row.
   * @return The number of remaining column to the end of the current row.
   */
  public int getRemainingColumns() {
    return cellContainer.getRemainingColumns(); 
  }

  /**
   * Adds a cell to the container.
   * @param item The document item.
   * @param cell The PDF cell to be added.
   * @exception DocumentHandlerException If something went wrong when adding the cell.
   */
  public void addCell(BaseDocumentTag item, PdfPCell cell) throws DocumentHandlerException {
    if (isTrue()) {
      pdfCellContainer.addCell(item, cell);
      childsAdded = true;
    }
  }

  /**
   * Gets the number of remaining columns of the current row.
   * @return The number of remaining column to the end of the current row.
   */
  public int getRemainingPdfColumns() {
    return pdfCellContainer.getRemainingPdfColumns(); 
  }

  /**
   * Gets the PDF table holder.
   * @return The component that really holds a PDF table.
   */
  public PdfTableHolder getPdfTable() {
    return pdfCellContainer.getPdfTable();
  }

  /**
   * Adds the given chunk to the root phrase.
   * @param item The document item.
   * @param chunk The chunk to add to the root phrase.
   * @param font The font to use.
   * @exception DocumentHandlerException In case the document processing failed.
   */
  public void addChunk(BaseDocumentTag item, String chunk, DocumentFont font) 
    throws DocumentHandlerException {
    
    if (phraseHolder == null) {
      return;
    }
    phraseHolder.addChunk(item, chunk, font);
  }

  /**
   * Adds the given chunk to the root phrase.
   * @param item The document item.
   * @param chunk The chunk to add to the root phrase.
   * @exception DocumentHandlerException In case the document processing failed.
   */
  public void addChunk(BaseDocumentTag item, Chunk chunk) throws DocumentHandlerException {
    if (phraseHolder == null) {
      return;
    }
    phraseHolder.addChunk(item, chunk);
  }

  /**
   * Gets the fixed vertical leading distance.
   * @return The current fixed vertical leading distance.
   */
  public float getLeading() {
    if (phraseHolder == null) {
      return DEFAULT_LEADING;
    }
    return phraseHolder.getLeading();
  }

  /**
   * Gets the line spacing.
   * @return The current line spacing.
   */
  public float getLineSpacing() {
    if (phraseHolder == null) {
      ElementContainer elementContainer = getElementContainer();
      if (elementContainer == null) {
        return DEFAULT_LINE_SPACING;
      }
      return elementContainer.getLineSpacing();
    }
    return phraseHolder.getLineSpacing();
  }

  /**
   * Flushes the content buffer.
   * @exception DocumentHandlerException If something went wrong while flushing the content buffer.
   */
  public void flushContent() throws DocumentHandlerException {
    if (!isTrue()) {
      return;
    }

    if ((phraseHolder != null) && (phrase == null)) {
      phrase = new Phrase(0.0F);
    }
    if (isIgnoreFlush()) {
      // don't flush twice!
      return;
    }

    if (isTrimNextHead()) {
      trimContentHead();
      setTrimNextHead(false);
    }
    if (isItemClosed() && isTrimBody()) {
      trimContentTail();
    }

    if (phraseHolder != null) {
      if (content == null) {
        // no sense to go on!
        return;
      }
      try {
        String content = documentHandler.executeTemplate(this.getContent());
        phraseHolder.addChunk(this, content, getFont());
      } catch (TemplateException ex) {
        throw new DocumentHandlerException(locator(), "Content evaluation failed: " + ex.getMessage(), ex);
      }
    } else {
      Phrase nestedPhrase = addChunk(phrase, 0.0F, getLineSpacing());
      if (nestedPhrase != null) {
        phrase = nestedPhrase;
      }
    }
    resetContent();
  }

  /**
   * Gets the cell container.
   * @return The cell container.
   */
  public CellContainer getCellContainer() {
    return cellContainer;
  }

  /**
   * Gets the PDF cell container.
   * @return The PDF cell container.
   */
  public PdfCellContainer getPdfCellContainer() {
    return pdfCellContainer;
  }

  /**
   * Gets the cell attributes.
   * @return The cell attributes.
   */
  public CellAttributes getCellAttributes() {
    return pdfCellContainer.getCellAttributes();
  }

}
