/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.DocumentHandlerException;

import org.ujac.util.template.TemplateContext;

/**
 * Name: SetPropertyTag<br>
 * Description: Class handling the toggle-property tag.
 * 
 * @author lauerc
 */
public class TogglePropertyTag extends BaseDocumentTag {

  /** The item's name. */
  public static final String TAG_NAME = "toggle-property";

  /** Definition of the 'font' attribute. */
  private static final AttributeDefinition PROPERTY_NAME = 
      CommonAttributes.PROPERTY_NAME.cloneAttrDef("The name of the property to toggle.");

  /** The property name. */
  private String name = null;
  /** The property type. */
  private String type = null;
    
  /**
   * Constructs a TogglePropertyTag instance with no specific attributes.
   */
  public TogglePropertyTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Toggles the value for a specific property. Each time this tag is executed " +
        "it changes its value from a to b or vice versa.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(PROPERTY_NAME)    
      .addDefinition(CommonAttributes.PROPERTY_TYPE) 
      .addDefinition(CommonAttributes.PROPERTY_A)
      .addDefinition(CommonAttributes.PROPERTY_B);
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // getting attributes
    name = getStringAttribute(CommonAttributes.PROPERTY_NAME, false, null);
    type = getStringAttribute(CommonAttributes.PROPERTY_TYPE, false, null);
  }
  
  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    // getting the property value
    Object a = typeCastValue(getStringAttribute(CommonAttributes.PROPERTY_A, false, null), type);
    Object b = typeCastValue(getStringAttribute(CommonAttributes.PROPERTY_B, false, null), type);
    
    // toggeling the document property
    TemplateContext ctx = documentHandler.getTemplateContext();
    Object curVal = ctx.getProperty(name);
    if ((curVal == null) || (curVal.equals(b))) {
      ctx.setProperty(name, a);
    } else {
      ctx.setProperty(name, b);
    }

  }
 
}
