/*
 * Copyright (C) 2004 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.DocumentHandlerException;

import com.lowagie.text.pdf.PdfAction;
import com.lowagie.text.pdf.PdfDestination;
import com.lowagie.text.pdf.PdfWriter;

/**
 * Name: ViewerPreferencesTag<br>
 * Description: A class defining the viewer preferences for the generated document.
 * 
 * @author lauerc
 */
public class ViewerPreferencesTag extends BaseDocumentTag {

  /** Definition of the 'layout' attribute. */
  private static final AttributeDefinition PAGE_LAYOUT = new AttributeDefinition(TagAttributes.ATTR_PAGE_LAYOUT, 
      new String[] {"SinglePage", "OneColumn", "TwoColumnLeft", "TwoColumnRight"}, false, 
      "Defines the page layout.");
  /** Definition of the 'page-mode' attribute. */
  private static final AttributeDefinition PAGE_MODE = new AttributeDefinition(TagAttributes.ATTR_PAGE_MODE, 
      new String[] {"UseNone", "UseOutlines", "UseThumbs", "UseOutlines|UseThumbs"}, false, 
      "Defines the page mode for non full screen viewer mode.");    
  /** Definition of the 'non-full-screen-page-mode' attribute. */
  private static final AttributeDefinition NON_FULL_SCREEN_PAGE_MODE = new AttributeDefinition(TagAttributes.ATTR_NON_FULL_SCREEN_PAGE_MODE, 
      new String[] {"SinglePage", "OneColumn", "TwoColumnLeft", "TwoColumnRight"}, false, 
      "Defines the page mode for non full screen mode.");    
  /** Definition of the 'full-screen' attribute. */
  private static final AttributeDefinition FULL_SCREEN = new AttributeDefinition(TagAttributes.ATTR_FULL_SCREEN, 
      AttributeDefinition.TYPE_BOOLEAN, false, "Tells whether or not to display the document in full screen mode.");
  /** Definition of the 'hide-toolbar' attribute. */
  private static final AttributeDefinition HIDE_TOOLBAR = new AttributeDefinition(TagAttributes.ATTR_HIDE_TOOLBAR, 
      AttributeDefinition.TYPE_BOOLEAN, false, "Tells whether or not to hide the viewer's tool bar.");
  /** Definition of the 'hioe-menubar' attribute. */
  private static final AttributeDefinition HIDE_MENUBAR = new AttributeDefinition(TagAttributes.ATTR_HIDE_MENUBAR, 
      AttributeDefinition.TYPE_BOOLEAN, false, "Tells whether or not to hide the viewer's menu bar.");
  /** Definition of the 'hode-window-ui' attribute. */
  private static final AttributeDefinition HIDE_WINDOW_UI = new AttributeDefinition(TagAttributes.ATTR_HIDE_WINDOW_UI, 
      AttributeDefinition.TYPE_BOOLEAN, false, "Tells whether or not to hide the viewer's user interface.");
  /** Definition of the 'zoom' attribute. */
  private static final AttributeDefinition ZOOM = new AttributeDefinition(TagAttributes.ATTR_ZOOM, 
      AttributeDefinition.TYPE_FLOAT, false, "Tells what zoom level needs to be set when pdf is opened. " +
      "Any value given is multiplied by 100. So the % should be 0.0 to 1.0");
  /** Definition of the 'fit-window' attribute. */
  private static final AttributeDefinition FIT_WINDOW = new AttributeDefinition(TagAttributes.ATTR_FIT_WINDOW, 
      AttributeDefinition.TYPE_BOOLEAN, false, "Tells whether or not to use the fit window option.");
  /** Definition of the 'center-window' attribute. */
  private static final AttributeDefinition CENTER_WINDOW = new AttributeDefinition(TagAttributes.ATTR_CENTER_WINDOW, 
      AttributeDefinition.TYPE_BOOLEAN, false, "Tells whether or not to use the center window option.");
  /** Definition of the 'direction' attribute. */
  private static final AttributeDefinition DIRECTION = new AttributeDefinition(TagAttributes.ATTR_DIRECTION, 
      new String[] { "L2R", "R2L"}, false, 
      "Defines the view direction.");    
  /** Definition of the 'doc-title' attribute. */
  private static final AttributeDefinition DISPLAY_DOC_TITLE = new AttributeDefinition(TagAttributes.ATTR_DISPLAY_DOC_TITLE, 
      AttributeDefinition.TYPE_BOOLEAN, false, "Tells whether or not to display the document title.");

  
  /** The item's name. */
  public static final String TAG_NAME = "viewer-preferences";

  /**
   * Constructs a ViewerPreferencesTag instance with no specific attributes.
   */
  public ViewerPreferencesTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Defines the viewer preferences for the generated document." ;
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(PAGE_LAYOUT)    
      .addDefinition(PAGE_MODE)    
      .addDefinition(NON_FULL_SCREEN_PAGE_MODE)    
      .addDefinition(FULL_SCREEN)
      .addDefinition(HIDE_TOOLBAR)
      .addDefinition(HIDE_MENUBAR)
      .addDefinition(HIDE_WINDOW_UI)
      .addDefinition(ZOOM)
      .addDefinition(FIT_WINDOW)
      .addDefinition(CENTER_WINDOW)
      .addDefinition(DIRECTION)    
      .addDefinition(DISPLAY_DOC_TITLE);
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
  }
  
  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }

    // setting the meta data
    PdfWriter writer = (PdfWriter) getDocumentHandler().getDocumentWriter();
    int viewerPreferences = 0;
    if (isAttributeDefined(PAGE_LAYOUT)) {
      String attr = getStringAttribute(PAGE_LAYOUT, true, null);
      if ("SinglePage".equals(attr)) {
        viewerPreferences |= PdfWriter.PageLayoutSinglePage;
      } else if ("OneColumn".equals(attr)) {
        viewerPreferences |= PdfWriter.PageLayoutOneColumn;
      } else if ("TwoColumnLeft".equals(attr)) {
        viewerPreferences |= PdfWriter.PageLayoutTwoColumnLeft;
      } else if ("TwoColumnRight".equals(attr)) {
        viewerPreferences |= PdfWriter.PageLayoutTwoColumnRight;
      }
    }
    if (isAttributeDefined(PAGE_MODE)) {
      String attr = getStringAttribute(PAGE_MODE, true, null);
      if ("UseNone".equals(attr)) {
        viewerPreferences |= PdfWriter.PageModeUseNone;
      } else if ("UseOutlines".equals(attr)) {
        viewerPreferences |= PdfWriter.PageModeUseOutlines;
      } else if ("UseThumbs".equals(attr)) {
        viewerPreferences |= PdfWriter.PageModeUseThumbs;
      } else if ("UseOutlines|UseThumbs".equals(attr)) {
        viewerPreferences |= PdfWriter.PageModeUseOutlines | PdfWriter.PageModeUseThumbs;
      }
    }
    if (isAttributeDefined(NON_FULL_SCREEN_PAGE_MODE)) {
      String attr = getStringAttribute(NON_FULL_SCREEN_PAGE_MODE, true, null);
      if ("UseNone".equals(attr)) {
        viewerPreferences |= PdfWriter.NonFullScreenPageModeUseNone;
      } else if ("UseOutlines".equals(attr)) {
        viewerPreferences |= PdfWriter.NonFullScreenPageModeUseOutlines;
      } else if ("UseThumbs".equals(attr)) {
        viewerPreferences |= PdfWriter.NonFullScreenPageModeUseThumbs;
      } else if ("UseOutlines|UseThumbs".equals(attr)) {
        viewerPreferences |= PdfWriter.NonFullScreenPageModeUseOutlines | PdfWriter.NonFullScreenPageModeUseThumbs;
      }
    }
    if (isAttributeDefined(FULL_SCREEN)) {
      if (getBooleanAttribute(FULL_SCREEN, true, null)) {
        viewerPreferences |= PdfWriter.PageModeFullScreen;
      }
    }
    if (isAttributeDefined(HIDE_TOOLBAR)) {
      if (getBooleanAttribute(HIDE_TOOLBAR, true, null)) {
        viewerPreferences |= PdfWriter.HideToolbar;
      }
    }
    if (isAttributeDefined(HIDE_MENUBAR)) {
      if (getBooleanAttribute(HIDE_MENUBAR, true, null)) {
        viewerPreferences |= PdfWriter.HideMenubar;
      }
    }
    if (isAttributeDefined(HIDE_WINDOW_UI)) {
      if (getBooleanAttribute(HIDE_WINDOW_UI, true, null)) {
        viewerPreferences |= PdfWriter.HideWindowUI;
      }
    }
    if (isAttributeDefined(FIT_WINDOW)) {
      if (getBooleanAttribute(FIT_WINDOW, true, null)) {
        viewerPreferences |= PdfWriter.FitWindow;
      }
    }
    if (isAttributeDefined(CENTER_WINDOW)) {
      if (getBooleanAttribute(CENTER_WINDOW, true, null)) {
        viewerPreferences |= PdfWriter.CenterWindow;
      }
    }
    if (isAttributeDefined(DIRECTION)) {
      String attr = getStringAttribute(NON_FULL_SCREEN_PAGE_MODE, true, null);
      if ("L2R".equals(attr)) {
        viewerPreferences |= PdfWriter.DirectionL2R;
      } else if ("R2L".equals(attr)) {
        viewerPreferences |= PdfWriter.DirectionR2L;
      }
    }
    if (isAttributeDefined(DISPLAY_DOC_TITLE)) {
      if (getBooleanAttribute(DISPLAY_DOC_TITLE, true, null)) {
        viewerPreferences |= PdfWriter.DisplayDocTitle;
      }
    }
    if (isAttributeDefined(ZOOM)) {
      float zoomLevel = getFloatAttribute(ZOOM, 0F, true, null);
      writer.setOpenAction(
          PdfAction.gotoLocalPage(
              1, new PdfDestination(PdfDestination.XYZ, -1, -1, zoomLevel), writer
          )
      );
    }
    // finally setting the viewer preferences
    writer.setViewerPreferences(viewerPreferences);
  }

}
