/*
   * Copyright (C) 2005 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.DocumentFont;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.ElementContainer;
import org.ujac.print.FontHolder;
import org.ujac.print.PagePosition;
import org.ujac.print.PhraseHolder;

import com.lowagie.text.BadElementException;
import com.lowagie.text.Chunk;
import com.lowagie.text.Document;
import com.lowagie.text.Element;
import com.lowagie.text.Image;
import com.lowagie.text.DocumentException;
import com.lowagie.text.Phrase;
import com.lowagie.text.Rectangle;

import com.lowagie.text.pdf.PdfTemplate;
import com.lowagie.text.pdf.PdfWriter;
import com.lowagie.text.pdf.PdfContentByte;

/**
 * Name: WatermarkTag<br>
 * Description: A class handling &lt;watermark&gt; tags.<br>
 * 
 * @author lauerc
 */
public class WatermarkTag extends BackgroundImageTag implements ElementContainer, PhraseHolder, FontHolder {

  /** The item's name. */
  public static final String TAG_NAME = "watermark";

  /** Definition of the 'source' attribute. */
  private static final AttributeDefinition WATERMARK_IMAGE_SOURCE =
      CommonAttributes.IMAGE_SOURCE.cloneAttrDef().setRequired(false);
  
  /** 
   * The absolute leading, to be added to the variable leading amount, 
   * resulting from the line spacing factor.
   */
  private float leading = DEFAULT_LEADING;
  /** The line spacing factor. */
  private float lineSpacing = DEFAULT_LINE_SPACING;
  /** The phrase. */
  private Phrase phrase = null;

  /**
   * Constructs a WatermarkTag instance with no specific attributes.
   */
  public WatermarkTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Inserts a watermark into the document. Watermarks might be images, which  gets loaded by the " +
      "resource loader, which is specified at the document handler, or are generated from the text, defined " +
      "by the tag's body.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()    
      .addDefinition(CommonAttributes.FONT)
      .addDefinition(WATERMARK_IMAGE_SOURCE)
      .addDefinition(CommonAttributes.IMAGE_DRAW_FOREGROUND);
  }

  /**
   * Gets the list of supported style attributes.
   * @return The supported style attributes.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.FONT_NAME)
      .addDefinition(CommonStyleAttributes.IMAGE_X)
      .addDefinition(CommonStyleAttributes.IMAGE_Y)
      .addDefinition(CommonStyleAttributes.IMAGE_WIDTH)
      .addDefinition(CommonStyleAttributes.IMAGE_HEIGHT)
      .addDefinition(CommonStyleAttributes.SCALE_TO_FIT)
      .addDefinition(CommonStyleAttributes.HORIZONTAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.VERTICAL_ALIGNMENT)
      .addDefinition(CommonStyleAttributes.IMAGE_DRAW_FOREGROUND);
  }

  /**
   * Tells whether textual content is allowed for the tag or not.
   * @return true if the item is allowed to contain textual content, else false.
   */
  public boolean isTextBodyAllowed() {
    return true;
  }
  
  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
  }
  
  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the image item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();

    if (!isValid()) {
      return;
    }
  }

  /**
   * Processes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    flushContent();

    super.closeItem();
  }

  /**
   * Gets the image to insert into the document.
   * @return The created image.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public Image getImage() throws DocumentHandlerException {
    if (sourceAttr != null) {
      return super.getImage();
    }
    if (phrase == null) {
      throw new DocumentHandlerException(locator(), "The watermark tag requires eighter a image source definition or a watermark text, defined by the tag body.");
    }
    try {
      PdfContentByte cb = ((PdfWriter) documentHandler.getDocumentWriter(true)).getDirectContent();
      float phraseWidth = getPhraseWidth(phrase);
      float phraseHeight = getPhraseHeight(phrase);
      PdfTemplate template = cb.createTemplate(phraseWidth, phraseHeight);
      template.beginText();
      // TODO: Fix calculation of baseline offset!
      printPhrase(template, phrase, 0.0F, phraseHeight * 0.25F);
      template.endText();
      Image img = Image.getInstance(template);
      return img;
    } catch (BadElementException ex) {
      throw new DocumentHandlerException(locator(), "Failed to rotate phrase: " + ex.getMessage(), ex);
    }
  }
  
  /**
   * Inserts the given image into the document.
   * @param writer The pdf writer.
   * @param document The document to add the image to.
   * @param image The image to insert.
   */
  protected void insertImage(PdfWriter writer, Document document, Image image) {
    try {
      // calculating the correct image position
      Rectangle pageSize = document.getPageSize();
      PagePosition pos = getPosition();
      float x = pos.calcX(pageSize.getRight(), image.getScaledWidth());
      float y = pos.calcY(pageSize.getTop(), image.getScaledHeight());
      // placing the image absolutely into the page
      image.setAbsolutePosition(x, y);
      boolean drawForeground = getBooleanAttribute(CommonAttributes.IMAGE_DRAW_FOREGROUND, false, true, CommonStyleAttributes.IMAGE_DRAW_FOREGROUND);
      PdfContentByte cb = null;
      if (drawForeground) {
        cb = writer.getDirectContent();
      } else {
        cb = writer.getDirectContentUnder();
      }
      cb.addImage(image);      
    } catch (DocumentException ex) {
      log.error("Watermark output failed: " + ex.getMessage(), ex);
      throw new RuntimeException(ex.getMessage());
    } catch (DocumentHandlerException ex) {
      log.error("Watermark output failed: " + ex.getMessage(), ex);
      throw new RuntimeException(ex.getMessage());
    }
  }

  /**
   * Gets the actual content.
   * This method is intended for testing purposes. 
   * @return The last content.
   */
  public String getActualContent() {
    if (phrase == null) {
      return super.getActualContent();
    }
    return getPhraseContent(phrase);
  }

  /**
   * Flushes the content buffer.
   * @exception DocumentHandlerException Thrown in case something went wrong while flushing the content buffer.
   */
  public void flushContent() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    if (phrase == null) {
      createPhrase();
    }
    if (isIgnoreFlush()) {
      // don't flush twice!
      return;
    }

    if (isTrimNextHead()) {
      trimContentHead();
      setTrimNextHead(false);
    }
    if (isItemClosed() && isTrimBody()) {
      trimContentTail();
    }

    Phrase nestedPhrase = addChunk(phrase, getLeading(), getLineSpacing());
    if (nestedPhrase != null) {
      phrase = nestedPhrase;
    }
    resetContent();
  }

  /**
   * Adds the given chunk to the root phrase.
   * @param item The document item.
   * @param chunk The chunk to add to the root phrase.
   * @param font The font to use.
   * @exception DocumentHandlerException In case the document processing failed.
   */
  public void addChunk(BaseDocumentTag item, String chunk, DocumentFont font) 
    throws DocumentHandlerException {
    if (phrase == null) {
      createPhrase();
    }
    addChunk(item, phrase, chunk, font, getLeading(), getLineSpacing());
  }

  /**
   * Adds the given chunk to the root phrase.
   * @param item The document item.
   * @param chunk The chunk to add to the root phrase.
   * @exception DocumentHandlerException In case the document processing failed.
   */
  public void addChunk(BaseDocumentTag item, Chunk chunk)
    throws DocumentHandlerException {
    
    if (phrase == null) {
      createPhrase();
    }
    addChunk(phrase, chunk, getLeading(), getLineSpacing());
  }

  /**
   * Gets the fixed vertical leading distance.
   * @return The current fixed vertical leading distance.
   */
  public float getLeading() {
    return leading;
  }

  /**
   * Gets the line spacing.
   * @return The current line spacing.
   */
  public float getLineSpacing() {
    return lineSpacing;
  }

  /**
   * Adds a element to the container.
   * @param item The document item.
   * @param element The element to be added.
   * @exception DocumentHandlerException If something went wrong when adding the element.
   */
  public void addElement(BaseDocumentTag item, Element element)
    throws DocumentHandlerException {
    
    if (phrase == null) {
      createPhrase();
    }
    phrase.add(element);
    if (element instanceof Phrase) {
      extendLeading(phrase, (Phrase) element);
    }
  }

  /**
   * Tells whether the element container is a top level container or not.
   * @return true if the element container is a top level container, else false.
   */
  public boolean isTopLevel() {
    return false;
  }

  /**
   * Creates a new phrase.
   */
  private void createPhrase() {
    phrase = new Phrase(0.0F);
  }

}
