/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag.acroform;

import java.awt.Color;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.ChildDefinition;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.DocumentFont;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.ElementContainer;
import org.ujac.print.JavascriptContainer;
import org.ujac.print.TagAttributeException;
import org.ujac.print.tag.BaseElementTag;
import org.ujac.print.tag.CommonAttributes;
import org.ujac.print.tag.CommonStyleAttributes;
import org.ujac.print.tag.JavascriptTag;
import org.ujac.print.tag.TagAttributes;

import org.ujac.util.BeanUtils;

import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.BaseField;
import com.lowagie.text.pdf.PdfAcroForm;
import com.lowagie.text.pdf.PdfAction;
import com.lowagie.text.pdf.PdfAnnotation;
import com.lowagie.text.pdf.PdfBorderDictionary;
import com.lowagie.text.pdf.PdfFormField;
import com.lowagie.text.pdf.PdfWriter;

/**
 * Name: BaseAcroFieldTag<br>
 * Description: Base class for AcroField tag implementations.
 * 
 * @author lauerc
 */
public abstract class BaseAcroFieldTag
  extends BaseElementTag
  implements JavascriptContainer {

  /** Definition of the 'name' attribute. */
  public static final AttributeDefinition FIELD_NAME = new AttributeDefinition(TagAttributes.ATTR_NAME, 
      AttributeDefinition.TYPE_NAME, true, "The field name.");
  /** Definition of the 'value' attribute. */
  public static final AttributeDefinition FIELD_VALUE = new AttributeDefinition(TagAttributes.ATTR_VALUE, 
      AttributeDefinition.TYPE_STRING, false, "The field's value.");
  /** Definition of the 'status' attribute. */
  public static final AttributeDefinition FIELD_STATUS = new AttributeDefinition(TagAttributes.ATTR_STATUS, 
      AttributeDefinition.TYPE_BOOLEAN, false, "The initial status (true/false).");
  /** Definition of the 'x' attribute. */
  public static final AttributeDefinition FIELD_X = new AttributeDefinition(TagAttributes.ATTR_X, 
      AttributeDefinition.TYPE_DIMENSION, false, "The field's horizontal position on the current page.");
  /** Definition of the 'y' attribute. */
  public static final AttributeDefinition FIELD_Y = new AttributeDefinition(TagAttributes.ATTR_Y, 
      AttributeDefinition.TYPE_DIMENSION, false, "The field's vertical position on the current page.");
  /** Definition of the 'width' attribute. */
  public static final AttributeDefinition FIELD_WIDTH = new AttributeDefinition(TagAttributes.ATTR_WIDTH, 
      AttributeDefinition.TYPE_DIMENSION, false, "The field's width.");
  /** Definition of the 'height' attribute. */
  public static final AttributeDefinition FIELD_HEIGHT = new AttributeDefinition(TagAttributes.ATTR_HEIGHT, 
      AttributeDefinition.TYPE_DIMENSION, false, "The field's heigth.");
  /** Definition of the 'editable' attribute. */
  public static final AttributeDefinition FIELD_EDITABLE = new AttributeDefinition(TagAttributes.ATTR_EDITABLE, 
      AttributeDefinition.TYPE_BOOLEAN, false, "Tells whether the field should be editable or not, default is true.");
  /** Definition of the 'border-type' attribute. */
  public static final AttributeDefinition BORDER_TYPE = new AttributeDefinition(TagAttributes.ATTR_BORDER_TYPE, 
      new String[] {"inset", "beveled", "dashed", "solid", "underline"}, false, "The field's border type.");
  /** Definition of the 'form-url' attribute. */
  public static final AttributeDefinition FORM_URL = new AttributeDefinition(TagAttributes.ATTR_URL, 
      AttributeDefinition.TYPE_STRING, false, "The URL to post the form values to.");

  /** The field name. */
  protected String fieldName = null; 
  /** The horizontal position. */
  protected float x = 0.0F;
  /** The vertical position. */
  protected float y = 0.0F;
  /** The field's width. */
  protected float width = 0.0F;
  /** The field's height. */
  protected float height = 0.0F;
  /** The border width. */
  protected float borderWidth = BaseField.BORDER_WIDTH_THIN;
  /** The border color. */
  protected Color borderColor = null;
  /** The border type. */
  protected int borderType = PdfBorderDictionary.STYLE_SOLID;
  /** The javascript attribute, filled by the nested javascript tag. */
  protected String javascript = null;
  
  /**
   * Constructs a BaseAcroFieldTag instance with specific attributes.
   * @param name The tag name.
   */
  public BaseAcroFieldTag(String name) {
    super(name);
  }
  
  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Adds an Acro Field to the document.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.RENDERED)
      .addDefinition(FIELD_NAME)
      .addDefinition(FIELD_X)
      .addDefinition(FIELD_Y)
      .addDefinition(FIELD_WIDTH)
      .addDefinition(FIELD_HEIGHT)
      .addDefinition(CommonAttributes.BACKGROUND_COLOR)
      .addDefinition(BORDER_TYPE)
      .addDefinition(CommonAttributes.BORDER_WIDTH)
      .addDefinition(CommonAttributes.BORDER_COLOR);
  }

  /**
   * Gets the map of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(FIELD_X)
      .addDefinition(FIELD_Y)
      .addDefinition(FIELD_WIDTH)
      .addDefinition(FIELD_HEIGHT)
      .addDefinition(CommonStyleAttributes.BACKGROUND_COLOR)
      .addDefinition(BORDER_TYPE)
      .addDefinition(CommonStyleAttributes.BORDER_WIDTH)
      .addDefinition(CommonStyleAttributes.BORDER_COLOR);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(JavascriptTag.class, 0, 1));
  }
  
  /**
   * Tells whether textual content is allowed for the tag or not.
   * @return true if the item is allowed to contain textual content, else false.
   */
  public boolean isTextBodyAllowed() {
    return true;
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();
    
    if (!isValid()) {
      return;
    }

    // getting common attributes
    if (isAttributeSupported(FIELD_NAME)) {
      this.fieldName = getStringAttribute(FIELD_NAME, true, null);
    }
    if (isAttributeSupported(FIELD_X)) {
      this.x = getDimensionAttribute(FIELD_X, true, FIELD_X);
    }
    if (isAttributeSupported(FIELD_Y)) {
      this.y = getDimensionAttribute(FIELD_Y, true, FIELD_Y);
    }
    if (isAttributeSupported(FIELD_WIDTH)) {
      this.width = getDimensionAttribute(FIELD_WIDTH, true, FIELD_WIDTH);
    }
    if (isAttributeSupported(FIELD_HEIGHT)) {
      this.height = getDimensionAttribute(FIELD_HEIGHT, true, FIELD_HEIGHT);
    }
    
    if (isAttributeSupported(BORDER_TYPE)) {
      String strBorderType = getStringAttribute(BORDER_TYPE, "solid", true, BORDER_TYPE);
      this.borderType = borderTypeFromString(strBorderType);
    }
    if (isAttributeSupported(CommonAttributes.BORDER_WIDTH)) {
      this.borderWidth = getDimensionAttribute(CommonAttributes.BORDER_WIDTH, BaseField.BORDER_WIDTH_THIN, true, CommonStyleAttributes.BORDER_WIDTH);
    }
    if (isAttributeSupported(CommonAttributes.BORDER_COLOR)) {
      this.borderColor = getColorAttribute(CommonAttributes.BORDER_COLOR, DEFAULT_FONT_COLOR, true, CommonStyleAttributes.BORDER_COLOR);
    }
  }
  
  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }

    if (isVisibleElement()) {
      if (!isAttributeDefined(FIELD_X) && !isAttributeDefined(FIELD_Y)) {
        ElementContainer elementContainer = getElementContainer();
        elementContainer.addElement(this, new FloatingFormElement(this, getPdfWriter(), width, height));
      } else {
        render(getPdfWriter(), null);
      }
    } else {
      addFormField(getAcroForm());
    }
  }
  
  /**
   * Gets the AcroForm component for the current document. 
   * @return The AcroForm component.
   */
  public PdfAcroForm getAcroForm() {
    PdfWriter writer = (PdfWriter) documentHandler.getDocumentWriter();
    return writer.getAcroForm();
  }

  /**
   * Gets the PdfWriter component for the current document. 
   * @return The PdfWriter component.
   */
  public PdfWriter getPdfWriter() {
    PdfWriter writer = (PdfWriter) documentHandler.getDocumentWriter();
    return writer;
  }

  /**
   * @see org.ujac.print.JavascriptContainer#addJavaScript(java.lang.String)
   */
  public void addJavaScript(String code) {
    this.setJavascript(code);
  }

  /**
   * Adds a field to the given AcroForm. 
   * @param form The AcroForm to add the field to.
   * @exception DocumentHandlerException In case something went wrong while adding the form field.
   */
  public abstract PdfFormField addFormField(PdfAcroForm form) throws DocumentHandlerException;

  /**
   * Tells whether or not this element is visible to the user.
   * @return true if to render the element.
   */
  public boolean isVisibleElement() {
    return true;
  }
  
  /**
   * Renders the form field into the current document.
   * @param writer The writer to use.
   * @param fieldPosition The field position (may be null).
   * @return The rendered form field.
   * @exception DocumentHandlerException In case something bad happened.
   */
  public PdfFormField render(PdfWriter writer, Rectangle fieldPosition) throws DocumentHandlerException {
    PdfFormField formField = addFormField(getAcroForm());
    if (fieldPosition != null) {
      formField.setWidget(fieldPosition, PdfAnnotation.HIGHLIGHT_INVERT);
    }
    if (formField != null) {
      writer.addAnnotation(formField);
    }
    return formField;
  }
  
  /**
   * Applies the specified javascript to the given form field.
   * @param formField The form field to handle
   */
  protected void applyJavaScript(PdfFormField formField) {
    if (!BeanUtils.isEmpty(javascript)) {
      PdfAction action = PdfAction.javaScript(javascript, getPdfWriter());  
      formField.setAction(action);
    }
  }
  
  protected Rectangle getFieldDimensions() {
    if (isAttributeDefined(FIELD_X) || isAttributeDefined(FIELD_Y)) {
      return new Rectangle(x, y, x + width, y + height);
    }
    return new Rectangle(width, height);
  }
  
  /**
   * Gets the border width for the field.
   * @return The border width.
   */
  protected float getBorderWidth() {
    return borderWidth;
  }
  
  /**
   * Gets the border color for the field.
   * @return The border color.
   */
  protected Color getBorderColor() {
    return borderColor;
  }
  
  /**
   * Gets the border type for the field.
   * @return The border type.
   */
  protected int getBorderType() {
    return borderType;
  }

  /**
   * Gets the text color for the field.
   * @return The text color.
   */
  protected Color getTextColor() {
    DocumentFont font = getFont();
    return font.getColor();
  }

  /**
   * Getter method for the the property javascript.
   * @return The current value of property javascript.
   */
  public String getJavascript() {
    return javascript;
  }
  /**
   * Setter method for the the property javascript.
   * @param javascript The value to set for the property javascript.
   */
  public void setJavascript(String javascript) {
    this.javascript = javascript;
  }

  /**
   * Gets the border type from the given String.
   * @param borderType The String to get the style ID for.
   * @return The according border style ID.
   * @throws TagAttributeException In case the given style is invalid.
   */
  protected int borderTypeFromString(String borderType) throws TagAttributeException {
    if (borderType == null) {
      return PdfBorderDictionary.STYLE_SOLID;
    }
    String borderTypeLower = borderType.toLowerCase();
    if ("inset".equals(borderTypeLower)) {
      return PdfBorderDictionary.STYLE_INSET;
    } else if ("beveled".equals(borderTypeLower)) {
      return PdfBorderDictionary.STYLE_BEVELED;
    } else if ("dashed".equals(borderTypeLower)) {
      return PdfBorderDictionary.STYLE_DASHED;
    } else if ("solid".equals(borderTypeLower)) {
      return PdfBorderDictionary.STYLE_SOLID;
    } else if ("underline".equals(borderTypeLower)) {
      return PdfBorderDictionary.STYLE_UNDERLINE;
    }
    throw new TagAttributeException(locator(), "Invalid border type '" + borderTypeLower + "'");
  }

}
