/*
 * Copyright (C) 2003 - 2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag.acroform;

import java.io.IOException;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.DocumentFont;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.tag.CommonAttributes;
import org.ujac.print.tag.CommonStyleAttributes;

import org.ujac.util.template.TemplateException;

import com.lowagie.text.DocumentException;
import com.lowagie.text.Rectangle;

import com.lowagie.text.pdf.BaseField;
import com.lowagie.text.pdf.BaseFont;
import com.lowagie.text.pdf.PdfAcroForm;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfFormField;
import com.lowagie.text.pdf.PdfWriter;
import com.lowagie.text.pdf.RadioCheckField;

/**
 * Name: CheckboxAcroFieldTag<br>
 * Description: Tag implementation for AcroForm check boxes.
 * 
 * @author lauerc
 */
public class CheckBoxAcroFieldTag extends BaseAcroFieldTag {

  /** The item's name. */
  public static final String TAG_NAME = "check-box";

  /** The field value. */
  protected String fieldValue = null; 
  /** The field status. */
  protected boolean status = false;

  /**
   * Constructs a CheckBoxAcroFieldTag instance with no specific attributes.
   */
  public CheckBoxAcroFieldTag() {
    super(TAG_NAME);
  }


  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Adds a check box to the documents AcroForm.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.CLASS)
      .addDefinition(CommonAttributes.STYLE)
      .addDefinition(FIELD_VALUE)
      .addDefinition(FIELD_STATUS)
      .addDefinition(CommonAttributes.FONT);
  }

  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.FONT_NAME);
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();
    
    if (!isValid()) {
      return;
    }

    // getting specific attributes
    this.fieldValue = getStringAttribute(FIELD_VALUE, true, null);
    this.status = getBooleanAttribute(FIELD_STATUS, true, null);
  }
 
  /**
   * Adds a field to the given AcroForm. 
   * @param form The AcroForm to add the field to.
   * @exception DocumentHandlerException in case something went wrong.
   */
  public PdfFormField addFormField(PdfAcroForm form) throws DocumentHandlerException {

    // creating the check box
    try {
      DocumentFont font = getFont();
      BaseFont bf = font.getFont().getBaseFont();
      PdfWriter pdfWriter = getPdfWriter();
      RadioCheckField cf = new RadioCheckField(pdfWriter, getFieldDimensions(), fieldName, fieldValue);
      cf.setChecked(status);
      cf.setFont(bf);
      int rotation = documentHandler.getDocument().getPageSize().getRotation();
      cf.setRotation(rotation);
      cf.setFontSize(font.getSize());
      cf.setBorderStyle(getBorderType());
      cf.setBorderWidth(getBorderWidth());
      cf.setBorderColor(getBorderColor());
      cf.setBackgroundColor(getBackgroundColor());
      cf.setTextColor(getTextColor());
      cf.setOptions(BaseField.VISIBLE);
      cf.setCheckType(RadioCheckField.TYPE_CHECK);
      PdfFormField ff = cf.getCheckField();
      applyJavaScript(ff);
      return ff;
    } catch (DocumentException ex) {
      throw new DocumentHandlerException(locator(), "Unable to add FormField: " + ex.getMessage(), ex);
    } catch (IOException ex) {
      throw new DocumentHandlerException(locator(), "Unable to add FormField: " + ex.getMessage(), ex);
    }
/*
    // drawing label for the radio button
    String label = null;
    try {
      label = documentHandler.executeTemplate(getContent());
    } catch (TemplateException ex) {
      throw new DocumentHandlerException(locator(), "Content evaluation failed: " + ex.getMessage(), ex);
    }

    DocumentFont docFont = getFont();
    PdfContentByte cb = getPdfWriter().getDirectContent();
    cb.saveState();
    cb.beginText();
    cb.setFontAndSize(docFont.getFont().getBaseFont(), docFont.getSize());
    cb.moveText(x + width * 1.2F, y + borderWidth);
    cb.showText(label);
    cb.endText();
    cb.restoreState();
*/
  }

  
  /**
   * Renders the form field into the current document.
   * @param writer The writer to use.
   * @param fieldPosition The field position (may be null).
   * @return The rendered form field.
   * @exception DocumentHandlerException In case something bad happened.
   */
  public PdfFormField render(PdfWriter writer, Rectangle fieldPosition) throws DocumentHandlerException {
    PdfFormField formField = super.render(writer, fieldPosition);
    
    // drawing label for the radio button
    String label = null;
    try {
      label = documentHandler.executeTemplate(getContent());
    } catch (TemplateException ex) {
      throw new DocumentHandlerException(locator(), "Content evaluation failed: " + ex.getMessage(), ex);
    }

    DocumentFont docFont = getFont();
    PdfContentByte cb = writer.getDirectContent();
    cb.saveState();
    cb.beginText();
    cb.setFontAndSize(docFont.getFont().getBaseFont(), docFont.getSize());
    float x = this.x;
    float y = this.y;
    if (fieldPosition != null) {
      x = fieldPosition.getLeft();
      y = fieldPosition.getBottom();
    }
    cb.moveText(x + width * 1.2F, y + borderWidth);
    cb.showText(label);
    cb.endText();
    cb.restoreState();

    return formField;
  }

}
