/*
 * Copyright (C) 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.print.tag.acroform;

import com.lowagie.text.Rectangle;

import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfPCell;
import com.lowagie.text.pdf.PdfPCellEvent;
import com.lowagie.text.pdf.PdfPTable;
import com.lowagie.text.pdf.PdfWriter;

/**
 * This class is an enhancement to the iText libary. iText doesn't support the
 * relative placement of form elements. This class is a workaround: We place a
 * table in the size of the desired form element in the document and register a
 * PdfPCellEvent listener. The listener is called when the table is positioned
 * in the document. We change the position of the form to the position of the
 * table. That this works we need to use the following order:<br>
 * Create a form element:<br>
 * <code>TextField tf = new TextField(pdfWriter, new Rectangle(x, y, x + width, y + height), fieldName);</code>
 * <br>
 * <code>...do the layout of field...</code><br>
 * <br>
 * Create the FloatingFormElement:<br>
 * <code>FloatingFormElement element = new FloatingFormElement(tf.getTextField(), pdfWriter);</code>
 * <br>
 * <br>
 * Add the form element to reserve a place:<br>
 * <code>document.add(element);</code><br>
 * <br>
 * The annotation is added automaticly.<br>
 * 
 * @author pschweizer
 */
public class FloatingFormElement extends PdfPTable implements PdfPCellEvent {

  private BaseAcroFieldTag tag;
  private PdfWriter writer;

  public FloatingFormElement(BaseAcroFieldTag tag, PdfWriter writer, float width, float height) {
    super(1);
    this.tag = tag;
    this.writer = writer;
    this.setHorizontalAlignment(PdfPTable.ALIGN_LEFT);
    float[] widths = {width};
    try {
      this.setTotalWidth(widths);
    } catch (Exception e) {
    }
    this.setLockedWidth(true);
    PdfPCell cell = new PdfPCell();
    cell.setFixedHeight(height);
    cell.setCellEvent(this);
    cell.setBorder(PdfPCell.NO_BORDER);
    this.addCell(cell);
  }

  /**
   * Will be called by the table, no need to call this method by user.
   */
  public void cellLayout(PdfPCell cell, Rectangle pos, PdfContentByte[] canvases) {
    try {
      Rectangle fieldPosition = new Rectangle(
          pos.getLeft(), pos.getBottom() - 5, 
          pos.getRight(), pos.getTop() - 5
      );
      // this.field.setBox( fieldPosition );
      // PdfFormField ff = getPdfFormField();
      this.tag.render(this.writer, fieldPosition);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}
