/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag.acroform;

import java.awt.Color;

import java.io.IOException;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.DocumentFont;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.tag.CommonAttributes;
import org.ujac.print.tag.CommonStyleAttributes;

import org.ujac.util.template.TemplateException;

import com.lowagie.text.DocumentException;
import com.lowagie.text.Rectangle;

import com.lowagie.text.pdf.BaseField;
import com.lowagie.text.pdf.BaseFont;
import com.lowagie.text.pdf.PdfAcroForm;
import com.lowagie.text.pdf.PdfBorderDictionary;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfFormField;
import com.lowagie.text.pdf.PdfWriter;
import com.lowagie.text.pdf.RadioCheckField;

/**
 * Name: CheckboxAcroFieldTag<br>
 * Description: Tag implementation for AcroForm radio buttons.
 * 
 * @author lauerc
 */
public class RadioButtonAcroFieldTag extends BaseAcroFieldTag {

  /** The item's name. */
  public static final String TAG_NAME = "radio-button";

  /** The surrounding radio group. */
  protected RadioGroupAcroFieldTag radioButtonGroup = null;
  /** The field value. */
  protected String fieldValue = null; 
  /** The border style. */
  protected int borderStyle = PdfBorderDictionary.STYLE_INSET;

  /**
   * Constructs a RadioButtonAcroFieldTag instance with no specific attributes.
   */
  public RadioButtonAcroFieldTag() {
    super(TAG_NAME);
  }


  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Adds a radio button to the documents AcroForm.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(FORM_URL)
      .addDefinition(CommonAttributes.FONT)
      .addDefinition(FIELD_VALUE)
      .removeDefinition(FIELD_NAME);
  }
  
  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.FONT_NAME);
  }

  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
    // getting parents
    this.radioButtonGroup = (RadioGroupAcroFieldTag) documentHandler.latestOfType(RadioGroupAcroFieldTag.class);
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();
    
    if (!isValid()) {
      return;
    }

    // getting field value
    this.fieldValue = getStringAttribute(FIELD_VALUE, true, null);
    
    // setting the local font
    if (isAttributeSupported(CommonAttributes.FONT)) {
      setLocalFont(getStringAttribute(CommonAttributes.FONT, true, CommonStyleAttributes.FONT_NAME));
    }

    // getting specific attributes
    if (isAttributeDefined(BORDER_TYPE, BORDER_TYPE)) {
      String strBorderStyle = getStringAttribute(BORDER_TYPE, "solid", true, BORDER_TYPE);
      this.borderStyle = borderTypeFromString(strBorderStyle);
    } else {
      this.borderType = radioButtonGroup.getBorderType();
    }

    if (isAttributeDefined(CommonAttributes.BORDER_WIDTH, CommonStyleAttributes.BORDER_WIDTH)) {
      this.borderWidth = getDimensionAttribute(CommonAttributes.BORDER_WIDTH, BaseField.BORDER_WIDTH_THIN, true, CommonStyleAttributes.BORDER_WIDTH);
    } else {
      this.borderWidth = radioButtonGroup.getBorderWidth();
    }
    
    if (isAttributeDefined(CommonAttributes.BORDER_COLOR, CommonStyleAttributes.BORDER_COLOR)) {
      this.borderColor = getColorAttribute(CommonAttributes.BORDER_COLOR, Color.lightGray, true, CommonStyleAttributes.BORDER_COLOR);
    } else {
      this.borderColor = radioButtonGroup.getBorderColor();
    }
    
    if (isAttributeDefined(CommonAttributes.BACKGROUND_COLOR, CommonStyleAttributes.BACKGROUND_COLOR)) {
      setBackgroundColor(getColorAttribute(CommonAttributes.BACKGROUND_COLOR, null, true, CommonStyleAttributes.BACKGROUND_COLOR));
    } else {
      setBackgroundColor(radioButtonGroup.getBackgroundColor());
    }
  }
 
  /**
   * Adds a field to the given AcroForm. 
   * @param form The AcroForm to add the field to.
   * @exception DocumentHandlerException In case something went wrong while adding the form field.
   */
  public PdfFormField addFormField(PdfAcroForm form) throws DocumentHandlerException {

    // creating the radio button
    PdfWriter pdfWriter = (PdfWriter) getDocumentHandler().getDocumentWriter();
    try {
      DocumentFont font = getFont();
      BaseFont bf = font.getFont().getBaseFont();
      RadioCheckField cf = new RadioCheckField(pdfWriter, getFieldDimensions(), radioButtonGroup.getRadioGroupName(), fieldValue);
      cf.setChecked(fieldValue.equals(radioButtonGroup.getRadioValue()));
      cf.setFont(bf);
      int rotation = documentHandler.getDocument().getPageSize().getRotation();
      cf.setRotation(rotation);
      cf.setFontSize(font.getSize());
      cf.setBorderStyle(getBorderStyle());
      cf.setBorderWidth(getBorderWidth());
      cf.setBorderColor(getBorderColor());
      cf.setBackgroundColor(getBackgroundColor());
      cf.setTextColor(getTextColor());
      cf.setOptions(BaseField.VISIBLE);
      cf.setCheckType(RadioCheckField.TYPE_CIRCLE);
      PdfFormField ff = cf.getRadioField();
      applyJavaScript(ff);
      radioButtonGroup.getRadioGroup().addKid(ff);
      return ff;
    } catch (DocumentException ex) {
      throw new DocumentHandlerException(locator(), "Unable to add FormField: " + ex.getMessage(), ex);
    } catch (IOException ex) {
      throw new DocumentHandlerException(locator(), "Unable to add FormField: " + ex.getMessage(), ex);
    }
  }

  public PdfFormField render(PdfWriter writer, Rectangle fieldPosition) throws DocumentHandlerException {
    PdfFormField formField = super.render(writer, fieldPosition);
    
    // drawing label for the radio button
    String label = null;
    try {
      label = documentHandler.executeTemplate(getContent());
    } catch (TemplateException ex) {
      throw new DocumentHandlerException(locator(), "Content evaluation failed: " + ex.getMessage(), ex);
    }

    DocumentFont docFont = getFont();
    PdfContentByte cb = writer.getDirectContent();
    cb.saveState();
    cb.beginText();
    cb.setFontAndSize(docFont.getFont().getBaseFont(), docFont.getSize());
    float x = this.x;
    float y = this.y;
    if (fieldPosition != null) {
      x = fieldPosition.getLeft();
      y = fieldPosition.getBottom();
    }
    cb.moveText(x + width * 1.2F, y + borderWidth);
    cb.showText(label);
    cb.endText();
    cb.restoreState();

    writer.addAnnotation(formField);

    return formField;
  }

  /**
   * Gets the border width for the field.
   * @return The border width.
   */
  protected float getBorderWidth() {
    return borderWidth;
  }
  
  /**
   * Gets the border color for the field.
   * @return The border color.
   */
  protected Color getBorderColor() {
    return borderColor;
  }
  
  /**
   * Gets the border style for the field.
   * @return The border width.
   */
  protected int getBorderStyle() {
    return borderStyle;
  }

  /**
   * Gets the text color for the field.
   * @return The text color.
   */
  protected Color getTextColor() {
    return null;
  }
  
}
