/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag.acroform;

import java.awt.Color;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.ChildDefinition;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.DocumentFont;
import org.ujac.print.DocumentHandlerException;

import org.ujac.print.tag.CommonAttributes;
import org.ujac.print.tag.CommonStyleAttributes;

import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.BaseField;
import com.lowagie.text.pdf.PdfAcroForm;
import com.lowagie.text.pdf.PdfFormField;
import com.lowagie.text.pdf.RadioCheckField;

/**
 * Name: CheckboxAcroFieldTag<br>
 * Description: Tag implementation for AcroForm button groups.
 * 
 * @author lauerc
 */
public class RadioGroupAcroFieldTag extends BaseAcroFieldTag {

  /** The item's name. */
  public static final String TAG_NAME = "radio-group";

  /** The radio group. */
  protected PdfFormField radioGroup = null;
  /** The field value. */
  protected String fieldValue = null; 

  /**
   * Constructs a RadioGroupAcroFieldTag instance with no specific attributes.
   */
  public RadioGroupAcroFieldTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Adds a radio button group to the documents AcroForm.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.CLASS)
      .addDefinition(CommonAttributes.STYLE)
      .addDefinition(FIELD_VALUE)
      .addDefinition(CommonAttributes.FONT);
  }
  
  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.FONT_NAME);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(RadioButtonAcroFieldTag.class, 0, ChildDefinition.UNLIMITED));
  }

  public boolean isVirtualContainer() {
    return true;
  }
  
  public boolean isStructureChecksDisabled() {
    return true;
  }
  
  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();
    
    if (!isValid()) {
      return;
    }

    // getting specific attributes
    this.fieldValue = getStringAttribute(FIELD_VALUE, true, null);
    this.borderWidth = getDimensionAttribute(CommonAttributes.BORDER_WIDTH, BaseField.BORDER_WIDTH_THIN, true, CommonStyleAttributes.BORDER_WIDTH);
    this.borderColor = getColorAttribute(CommonAttributes.BORDER_COLOR, Color.lightGray, true, CommonStyleAttributes.BORDER_COLOR);

    // creating the radio group
    RadioCheckField bt = new RadioCheckField(getPdfWriter(), new Rectangle(0, 0, 0, 0), fieldName, fieldValue);
    bt.setOptions(BaseField.VISIBLE);
    DocumentFont font = getLocalFont();
    if (isAttributeDefined(CommonAttributes.FONT, CommonStyleAttributes.FONT_NAME)) {
      bt.setFont(font.getFont().getBaseFont());
      bt.setFontSize(font.getSize());
    }
    radioGroup = bt.getRadioGroup(false, false);
  }

  /**
   * Adds a field to the given AcroForm. 
   * @param form The AcroForm to add the field to.
   * @exception DocumentHandlerException In case something went wrong while adding the form field.
   */
  public PdfFormField addFormField(PdfAcroForm form) throws DocumentHandlerException {
    getPdfWriter().addAnnotation(radioGroup);
    return null;
 }

  /**
   * Tells whether or not this element is visible to the user.
   * @return true if to render the element.
   */
  public boolean isVisibleElement() {
    return false;
  }

  /**
   * Gets the radio group. 
   * @return The radio group.
   */
  public PdfFormField getRadioGroup() {
    return radioGroup;
  }

  /**
   * Gets the radio group name. 
   * @return The radio group name.
   */
  public String getRadioGroupName() {
    return fieldName;
  }

  /**
   * Gets the radio value. 
   * @return The radio value.
   */
  public String getRadioValue() {
    return fieldValue;
  }

}
