/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag.acroform;

import java.io.IOException;

import java.util.ArrayList;
import java.util.List;

import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.ChildDefinition;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.DocumentFont;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.TagAttributeException;

import org.ujac.print.tag.CommonAttributes;
import org.ujac.print.tag.CommonStyleAttributes;
import org.ujac.print.tag.ElseTag;
import org.ujac.print.tag.ForeachTag;
import org.ujac.print.tag.IfTag;
import org.ujac.print.tag.LogTag;
import org.ujac.print.tag.SwitchTag;

import com.lowagie.text.DocumentException;

import com.lowagie.text.pdf.BaseField;
import com.lowagie.text.pdf.BaseFont;
import com.lowagie.text.pdf.PdfAcroForm;
import com.lowagie.text.pdf.PdfFormField;
import com.lowagie.text.pdf.PdfWriter;
import com.lowagie.text.pdf.TextField;

/**
 * Name: CheckboxAcroFieldTag<br>
 * Description: Tag implementation for AcroForm select lists.
 * 
 * @author lauerc
 */
public class SelectListAcroFieldTag extends BaseAcroFieldTag implements OptionHolder {

  /** The item's name. */
  public static final String TAG_NAME = "select-list";

  /** The field value. */
  protected String fieldValue = null; 
  
  /** The list of nested options. */
  private List options = new ArrayList();
  
  /**
   * Constructs a SelectListAcroFieldTag instance with no specific attributes.
   */
  public SelectListAcroFieldTag() {
    super(TAG_NAME);
  }


  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Adds a select list to the documents AcroForm.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.CLASS)
      .addDefinition(CommonAttributes.STYLE)
      .addDefinition(FIELD_VALUE)
      .addDefinition(FORM_URL)
      .addDefinition(CommonAttributes.FONT);
  }
  
  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.FONT_NAME);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(OptionTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(IfTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ElseTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(LogTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(SwitchTag.class, 0, ChildDefinition.UNLIMITED))
      .addDefinition(new ChildDefinition(ForeachTag.class, 0, ChildDefinition.UNLIMITED));
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();
    
    this.options.clear();
    
    if (!isValid()) {
      return;
    }

    // getting specific attributes
    this.fieldValue = getStringAttribute(FIELD_VALUE, true, null);
  }
 
  /**
   * Adds a field to the given AcroForm. 
   * @param form The AcroForm to add the field to.
   * @exception DocumentHandlerException In case something went wrong while adding the form field.
   */
  public PdfFormField addFormField(PdfAcroForm form) throws DocumentHandlerException {
    int numOptions = options.size();
    String[] choices = new String[numOptions];
    for (int i = 0; i < numOptions; i++) {
      Option option = (Option) options.get(i);
      choices[i] = option.getText();
    }
    String[] choicesExports = new String[numOptions];
    for (int i = 0; i < numOptions; i++) {
      Option option = (Option) options.get(i);
      choicesExports[i] = option.getValue();
    }

    try {
      DocumentFont font = getFont();
      BaseFont bf = font.getFont().getBaseFont();
      PdfWriter pdfWriter = getPdfWriter();
      TextField tf = new TextField(pdfWriter, getFieldDimensions(), fieldName);
      tf.setFont(bf);
      int rotation = documentHandler.getDocument().getPageSize().getRotation();
      tf.setRotation(rotation);
      tf.setFontSize(font.getSize());
      tf.setText(fieldValue);
      tf.setBorderStyle(getBorderType());
      tf.setBorderWidth(getBorderWidth());
      tf.setBorderColor(getBorderColor());
      tf.setBackgroundColor(getBackgroundColor());
      tf.setTextColor(getTextColor());
      tf.setChoices(choices);
      tf.setChoices(choicesExports);
      tf.setOptions(BaseField.VISIBLE | BaseField.COMB | BaseField.MULTILINE);
      PdfFormField ff = tf.getListField();
      applyJavaScript(ff);
      return ff;
    } catch (DocumentException ex) {
      throw new DocumentHandlerException(locator(), "Unable to add FormField: " + ex.getMessage(), ex);
    } catch (IOException ex) {
      throw new DocumentHandlerException(locator(), "Unable to add FormField: " + ex.getMessage(), ex);
    }
  }

  /**
   * Adds the given option to the option holder's container.
   * @param option The option to add.
   * @exception TagAttributeException In case the adding of the option has been failed.
   */
  public void addOption(Option option) throws TagAttributeException {
    this.options.add(option);
  }

}
