/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag.acroform;

import java.io.IOException;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.DocumentFont;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.TagAttributeException;
import org.ujac.print.tag.CommonAttributes;
import org.ujac.print.tag.CommonStyleAttributes;
import org.ujac.print.tag.TagAttributes;

import org.ujac.util.template.TemplateException;

import com.lowagie.text.DocumentException;
import com.lowagie.text.Element;

import com.lowagie.text.pdf.BaseField;
import com.lowagie.text.pdf.BaseFont;
import com.lowagie.text.pdf.PdfAcroForm;
import com.lowagie.text.pdf.PdfFormField;
import com.lowagie.text.pdf.PdfWriter;
import com.lowagie.text.pdf.TextField;

/**
 * Name: CheckboxAcroFieldTag<br>
 * Description: Tag implementation for AcroForm text field.
 * 
 * @author lauerc
 */
public class TextFieldAcroFieldTag extends BaseAcroFieldTag {

  /** Definition of the 'type' attribute. */
  public static final AttributeDefinition FIELD_TYPE= new AttributeDefinition(TagAttributes.ATTR_TYPE, 
      new String[] {"simple", "multiline", "password"}, false, "The field type. Default is 'simple'.");
  /** Definition of the 'text-align' attribute. */
  public static final AttributeDefinition TEXT_ALIGN= new AttributeDefinition(TagAttributes.ATTR_TEXT_ALIGNMENT,
      new String[] {"left","center","right"}, false, "The field text alignment.  Default is 'left'.");

  /** The item's name. */
  public static final String TAG_NAME = "text-field";

  /** The field type. */
  protected String type = null;
  /** The text alignment */
  protected int alignment = Element.ALIGN_LEFT;

  /**
   * Constructs a TextFieldAcroFieldTag instance with no specific attributes.
   */
  public TextFieldAcroFieldTag() {
    super(TAG_NAME);
  }


  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Adds a text field to the documents AcroForm.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(CommonAttributes.CLASS)
      .addDefinition(CommonAttributes.STYLE)
      .addDefinition(FIELD_TYPE)
      .addDefinition(CommonAttributes.FONT)
      .addDefinition(TEXT_ALIGN);
  }
  
  /**
   * Gets the list of supported style attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedStyleAttributes() {
    return super.buildSupportedStyleAttributes()
      .addDefinition(CommonStyleAttributes.FONT_NAME)
      .addDefinition(TEXT_ALIGN);
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();
    
    if (!isValid()) {
      return;
    }

    // getting specific attributes
    this.type = getStringAttribute(FIELD_TYPE, "simple", true, null);
    this.alignment = textAlignmentFromString(getStringAttribute(TEXT_ALIGN, "left", true, null));
  }
 
  /**
   * Adds a field to the given AcroForm. 
   * @param form The AcroForm to add the field to.
   * @exception DocumentHandlerException In case something went wrong while adding the form field.
   */
  public PdfFormField addFormField(PdfAcroForm form) throws DocumentHandlerException {
    String text = null;
    try {
      text = documentHandler.executeTemplate(getContent());
    } catch (TemplateException ex) {
      throw new DocumentHandlerException(locator(), "Content evaluation failed: " + ex.getMessage(), ex);
    }

    try {
      DocumentFont font = getFont();
      BaseFont bf = font.getFont().getBaseFont();
      PdfWriter pdfWriter = getPdfWriter();
      TextField tf = new TextField(pdfWriter, getFieldDimensions(), fieldName);
      tf.setFont(bf);
      int rotation = documentHandler.getDocument().getPageSize().getRotation();
      tf.setRotation(rotation);
      tf.setFontSize(font.getSize());
      tf.setText(text);
      tf.setBorderStyle(getBorderType());
      tf.setBorderWidth(getBorderWidth());
      tf.setBorderColor(getBorderColor());
      tf.setBackgroundColor(getBackgroundColor());
      tf.setTextColor(getTextColor());
      tf.setAlignment(alignment);

      if ("multiline".equals(type)) {
        tf.setOptions(BaseField.VISIBLE | BaseField.MULTILINE);
      } else if ("password".equals(type)) { 
        tf.setOptions(BaseField.VISIBLE | BaseField.PASSWORD);
      } else { 
        tf.setOptions(BaseField.VISIBLE);
      }
      PdfFormField ff = tf.getTextField();
      applyJavaScript(ff);
      return ff;
    } catch (DocumentException ex) {
      throw new DocumentHandlerException(locator(), "Unable to add FormField: " + ex.getMessage(), ex);
    } catch (IOException ex) {
      throw new DocumentHandlerException(locator(), "Unable to add FormField: " + ex.getMessage(), ex);
    }
  }
  
  /**
   * Gets the text alignment type from the given String.
   * @param alignment The String to get the text alignment for.
   * @return The according text alignment.
   * @throws TagAttributeException In case the given text alignment is invalid.
   */
  protected int textAlignmentFromString(String alignment) throws TagAttributeException {
    if (alignment == null) {
      return Element.ALIGN_LEFT;
    }
    alignment = alignment.toLowerCase();
    if ("left".equals(alignment)) {
      return Element.ALIGN_LEFT;
    } else if ("center".equals(alignment)) {
      return Element.ALIGN_CENTER;
    } else if ("right".equals(alignment)) {
      return Element.ALIGN_RIGHT;
    }
    throw new TagAttributeException(locator(), "Invalid text alignment '" + alignment + "'");
  }

}
