/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag.graphics;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.DocumentHandlerException;

import org.ujac.print.tag.TagAttributes;

import com.lowagie.text.pdf.PdfContentByte;

/**
 * Name: DrawCircleTag<br>
 * Description: Implementation of the &lt;draw-circle&gt; tag.
 * 
 * @author lauerc
 */
public class DrawCircleTag extends BaseGraphicsTag {

  /** Definition of the 'radius' attribute. */
  private static final AttributeDefinition RADIUS = new AttributeDefinition(TagAttributes.ATTR_RADIUS, 
      AttributeDefinition.TYPE_DIMENSION, true,"The radius of the circle to draw.");
  /** Definition of the 'x' attribute. */
  private static final AttributeDefinition CIRCLE_X = 
      X.cloneAttrDef("The horizontal position of the center of the circle."); 
  /** Definition of the 'y' attribute. */
  private static final AttributeDefinition CIRCLE_Y =  
      Y.cloneAttrDef("The vertical position of the center of the circle.");

  /** The item's name. */
  public static final String TAG_NAME = "draw-circle";
  
  /**
   * Constructs a DrawCircleTag instance with no specific attributes.
   */
  public DrawCircleTag() {
    super(TAG_NAME);
  }


  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Draws a circle.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(FILL)
      .addDefinition(CIRCLE_X)
      .addDefinition(CIRCLE_Y)
      .addDefinition(RADIUS);
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();
    
    if (!isValid()) {
      return;
    }

    // getting the coordinates
    boolean fill = getBooleanAttribute(FILL, false, true, null);
    float x = getDimensionAttribute(CIRCLE_X, true, null);
    float y = getDimensionAttribute(CIRCLE_Y, true, null);
    float radius = getDimensionAttribute(RADIUS, true, null);

    // drawing the circle
    PdfContentByte cb = this.graphicsTag.getContentByte();
    cb.circle(x, y, radius);
    if (fill) {
      cb.fillStroke();
    } else {
      cb.stroke();
    }
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }
  }

}
