/*
 * Copyright (C) 2003 - 2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag.graphics;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.TagAttributeException;

import com.lowagie.text.pdf.PdfContentByte;

/**
 * Name: DrawCurveTag<br>
 * Description: Implementation of the &lt;draw-curve&gt; tag.
 * 
 * @author lauerc
 */
public class DrawCurveTag extends BaseGraphicsTag {

  /** The item's name. */
  public static final String TAG_NAME = "draw-curve";
  
  /** Definition of the 'x1' attribute. */
  private static final AttributeDefinition CURVE_X1 = 
      X1.cloneAttrDef("The horizontal position of the start point."); 
  /** Definition of the 'y1' attribute. */
  private static final AttributeDefinition CURVE_Y1 = 
      Y1.cloneAttrDef("The vertical position of the start point."); 

  /** Definition of the 'x1' attribute. */
  private static final AttributeDefinition CURVE_X2 = 
      X2.cloneAttrDef("The horizontal position of the first control point."); 
  /** Definition of the 'y1' attribute. */
  private static final AttributeDefinition CURVE_Y2 = 
      Y2.cloneAttrDef("TThe vertical position of the first control point."); 
    
  /** Definition of the 'x1' attribute. */
  private static final AttributeDefinition CURVE_X3 = 
      X3.cloneAttrDef("The horizontal position of the second control point."); 
  /** Definition of the 'y1' attribute. */
  private static final AttributeDefinition CURVE_Y3 = 
      Y3.cloneAttrDef("TThe vertical position of the second control point."); 

  /** Definition of the 'x1' attribute. */
  private static final AttributeDefinition CURVE_X4 = 
      X4.cloneAttrDef("The horizontal position of the end point."); 
  /** Definition of the 'y1' attribute. */
  private static final AttributeDefinition CURVE_Y4 = 
      Y4.cloneAttrDef("The vertical position of the end point."); 

  /**
   * Constructs a DrawCurveTag instance with no specific attributes.
   */
  public DrawCurveTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Draws a rectangle.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(FILL)
      .addDefinition(CURVE_X1)
      .addDefinition(CURVE_Y1)
      .addDefinition(CURVE_X2)
      .addDefinition(CURVE_Y2)
      .addDefinition(CURVE_X3)
      .addDefinition(CURVE_Y3)
      .addDefinition(CURVE_X4)
      .addDefinition(CURVE_Y4);
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();
    
    if (!isValid()) {
      return;
    }

    // getting the coordinates
    boolean fill = getBooleanAttribute(FILL, false, true, null);
    float x1 = getDimensionAttribute(X1, true, null);
    float y1 = getDimensionAttribute(Y1, true, null);
    float x2 = getDimensionAttribute(Y2, -1, true, null);
    float y2 = getDimensionAttribute(Y2, -1, true, null);
    float x3 = getDimensionAttribute(X3, -1, true, null);
    float y3 = getDimensionAttribute(Y3, -1, true, null);
    float x4 = getDimensionAttribute(X4, true, null);
    float y4 = getDimensionAttribute(Y4, true, null);

    // drawing the line
    PdfContentByte cb = this.graphicsTag.getContentByte();
    cb.moveTo(x1, y1);
    if ((x2 < 0) || (y2 < 0)) {
      if ((x3 < 0) || (y3 < 0)) {
        throw new TagAttributeException(locator(), "Eigther the first or second control point have to be defined for curves.");
      }
      cb.curveTo(x3, y3, x4, y4);
    } else if ((x3 < 0) || (y3 < 0)) {
      cb.curveFromTo(x2, y2, x4, y4);
    } else {
      cb.curveTo(x2, y2, x3, y3, x4, y4);
    }
    if (fill) {
      cb.fillStroke();
    } else {
      cb.stroke();
    }
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }
  }

}
