/*
 * Copyright (C) 2003 - 2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag.graphics;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.DocumentHandlerException;

import com.lowagie.text.pdf.PdfContentByte;

/**
 * Name: DrawEllipseTag<br>
 * Description: Implementation of the &lt;draw-ellipse&gt; tag.
 * 
 * @author lauerc
 */
public class DrawEllipseTag extends BaseGraphicsTag {

  /** The item's name. */
  public static final String TAG_NAME = "draw-ellipse";
  
  /** Definition of the 'fill' attribute. */
  private static final AttributeDefinition ELLIPSE_FILL = 
      FILL.cloneAttrDef("Tells whether or not to fill the ellipse, default is false.");
  
  /** Definition of the 'x1' attribute. */
  private static final AttributeDefinition ELLIPSE_X1 = 
      X1.cloneAttrDef("The horizontal position of the start point of the ellipse."); 
  /** Definition of the 'y1' attribute. */
  private static final AttributeDefinition ELLIPSE_Y1 = 
      Y1.cloneAttrDef("The vertical position of the start point of the ellipse."); 
    
  /** Definition of the 'x2' attribute. */
  private static final AttributeDefinition ELLIPSE_X2 = 
      X2.cloneAttrDef("The horizontal position of the end point of the ellipse."); 
  /** Definition of the 'y2' attribute. */
  private static final AttributeDefinition ELLIPSE_Y2 = 
      Y2.cloneAttrDef("The vertical position of the end point of the ellipse."); 

  
  /**
   * Constructs a DrawEllipseTag instance with no specific attributes.
   */
  public DrawEllipseTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Draws a ellipse.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(ELLIPSE_FILL)
      .addDefinition(ELLIPSE_X1)
      .addDefinition(ELLIPSE_Y1)
      .addDefinition(ELLIPSE_X2)
      .addDefinition(ELLIPSE_Y2);
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();
    
    if (!isValid()) {
      return;
    }

    // getting the coordinates
    boolean fill = getBooleanAttribute(FILL, false, true, null);
    float x1 = getDimensionAttribute(X1, true, null);
    float y1 = getDimensionAttribute(Y1, true, null);
    float x2 = getDimensionAttribute(X2, true, null);
    float y2 = getDimensionAttribute(Y2, true, null);

    // drawing the line
    PdfContentByte cb = this.graphicsTag.getContentByte();
    cb.ellipse(x1, y1, x2, y2);
    if (fill) {
      cb.fillStroke();
    } else {
      cb.stroke();
    }
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }
  }

}
