/*
 * Copyright (C) 2003 - 2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag.graphics;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.DocumentHandlerException;

import org.ujac.print.tag.TagAttributes;

import com.lowagie.text.pdf.PdfContentByte;

/**
 * Name: DrawRectangleTag<br>
 * Description: Implementation of the &lt;draw-rectangle&gt; tag.
 * 
 * @author lauerc
 */
public class DrawRectangleTag extends BaseGraphicsTag {

  /** Definition of the 'radius' attribute. */
  private static final AttributeDefinition RECT_FILL = 
      FILL.cloneAttrDef("Tells whether or not to fill the rectangle, default is false.");
  /** Definition of the 'x1' attribute. */
  private static final AttributeDefinition RECT_X = 
      X.cloneAttrDef("The horizontal position of the start point of the rectangle."); 
  /** Definition of the 'y1' attribute. */
  private static final AttributeDefinition RECT_Y = 
      Y.cloneAttrDef("The vertical position of the start point of the rectangle."); 

  /** Definition of the 'width' attribute. */
  protected static final AttributeDefinition RECT_WIDTH = new AttributeDefinition(TagAttributes.ATTR_WIDTH, 
      AttributeDefinition.TYPE_DIMENSION, true, "The width of the rectangle.");
  /** Definition of the 'height' attribute. */
  protected static final AttributeDefinition RECT_HEIGHT = new AttributeDefinition(TagAttributes.ATTR_HEIGHT, 
      AttributeDefinition.TYPE_DIMENSION, true, "The height of the rectangle.");
  
  /** The item's name. */
  public static final String TAG_NAME = "draw-rectangle";

  
  /**
   * Constructs a DrawRectangleTag instance with no specific attributes.
   */
  public DrawRectangleTag() {
    super(TAG_NAME);
  }

  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Draws a rectangle.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(RECT_FILL)
      .addDefinition(RECT_X)
      .addDefinition(RECT_Y)
      .addDefinition(RECT_WIDTH)
      .addDefinition(RECT_HEIGHT);
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();
    
    if (!isValid()) {
      return;
    }

    // getting the coordinates
    boolean fill = getBooleanAttribute(FILL, false, true, null);
    float x = getDimensionAttribute(X, true, X);
    float y = getDimensionAttribute(Y, true, Y);
    float width = getDimensionAttribute(RECT_WIDTH, true, RECT_WIDTH);
    float height = getDimensionAttribute(RECT_HEIGHT, true, RECT_HEIGHT);

    // drawing the line
    PdfContentByte cb = this.graphicsTag.getContentByte();
    cb.rectangle(x, y, width, height);
    if (fill) {
      cb.fillStroke();
    } else {
      cb.stroke();
    }
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }
  }

}
