/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.tag.graphics;

import java.awt.Color;

import org.ujac.print.AttributeDefinition;
import org.ujac.print.AttributeDefinitionMap;
import org.ujac.print.BaseDocumentTag;
import org.ujac.print.ChildDefinition;
import org.ujac.print.ChildDefinitionMap;
import org.ujac.print.DocumentHandlerException;

import org.ujac.print.tag.TagAttributes;

import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfWriter;

/**
 * Name: GraphicsTag<br>
 * Description: Implementation of the &lt;graphics&gt; tag.
 * 
 * @author lauerc
 */
public class GraphicsTag extends BaseDocumentTag {

  /** Definition of the 'stroke-color' attribute. */
  protected static final AttributeDefinition STROKE_COLOR = new AttributeDefinition(TagAttributes.ATTR_STROKE_COLOR, 
      AttributeDefinition.TYPE_COLOR, false, "The stroke color for graphical operations.");
  /** Definition of the 'fill-color' attribute. */
  protected static final AttributeDefinition FILL_COLOR = new AttributeDefinition(TagAttributes.ATTR_FILL_COLOR, 
      AttributeDefinition.TYPE_COLOR, false, "The fill color for graphical operations.");
  /** Definition of the 'line-width' attribute. */
  protected static final AttributeDefinition LINE_WIDTH = new AttributeDefinition(TagAttributes.ATTR_LINE_WIDTH, 
      AttributeDefinition.TYPE_DIMENSION, false, "The line width for graphical operations.");
  /** Definition of the 'draw-foreground' attribute. */
  protected static final AttributeDefinition DRAW_FOREGROUND = new AttributeDefinition(TagAttributes.ATTR_DRAW_FOREGROUND, 
      AttributeDefinition.TYPE_BOOLEAN, false, "Tells whether to draw the image in the foreground " +
      "or in the background. The default value for this attribute is true.");

  /** The item's name. */
  public static final String TAG_NAME = "graphics";
  
  /** The stroke color. */
  private Color strokeColor = null;
  /** The fill color. */
  private Color fillColor = null;
  /** The line width. */
  private float lineWidth = 1.0F;
  /** The draw foreground flag. */
  private boolean drawForeground = true;

  /** The PDF content byte to perform graphical informations on. */
  private PdfContentByte cb = null;
  
  /**
   * Constructs a GraphicsTag instance with no specific attributes.
   */
  public GraphicsTag() {
    super(TAG_NAME);
  }


  /**
   * Gets a brief description for the item.
   * @return The item's description.
   */
  public String getDescription() {
    return "Defines a graphics area.";
  }

  /**
   * Gets the list of supported attributes.
   * @return The attribute definitions.
   */
  protected AttributeDefinitionMap buildSupportedAttributes() {
    return super.buildSupportedAttributes()
      .addDefinition(STROKE_COLOR)
      .addDefinition(FILL_COLOR)
      .addDefinition(LINE_WIDTH)
      .addDefinition(DRAW_FOREGROUND);
  }

  /**
   * Gets the list of supported childs.
   * @return The child definitions.
   */
  public ChildDefinitionMap buildSupportedChilds() {
    return super.buildSupportedChilds()
      .addDefinition(new ChildDefinition(BaseGraphicsTag.class, 0, ChildDefinition.UNLIMITED));
  }


  /**
   * Tells whether textual content is allowed for the tag or not.
   * @return true if the item is allowed to contain textual content, else false.
   */
  public boolean isTextBodyAllowed() {
    return false;
  }
  
  /**
   * Initializes the item. 
   * @exception DocumentHandlerException If something went badly wrong.
   */
  public void initialize() throws DocumentHandlerException {
    super.initialize();
  }

  /**
   * Opens the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while opening the document item.
   */
  public void openItem() throws DocumentHandlerException {
    super.openItem();
    
    if (!isValid()) {
      return;
    }

    this.strokeColor = getColorAttribute(STROKE_COLOR, Color.black, true, null);
    this.fillColor = getColorAttribute(FILL_COLOR, Color.black, true, null);
    this.lineWidth = getDimensionAttribute(LINE_WIDTH, 1.0F, true, null);
    this.drawForeground = getBooleanAttribute(DRAW_FOREGROUND, true, true, null);
    
    if (drawForeground) {
      this.cb = ((PdfWriter) documentHandler.getDocumentWriter()).getDirectContent();
    } else {
      this.cb = ((PdfWriter) documentHandler.getDocumentWriter()).getDirectContentUnder();
    }

    // storing initial state
    cb.saveState();
    
    // configuring content byte
    cb.setColorStroke(strokeColor);
    cb.setColorFill(fillColor);
    cb.setLineWidth(lineWidth);
  }

  /**
   * Closes the item.
   * @exception DocumentHandlerException Thrown in case something went wrong while processing the document item.
   */
  public void closeItem() throws DocumentHandlerException {
    if (!isValid()) {
      return;
    }
    // restoring initial state
    cb.restoreState();
  }

  /**
   * Gets the content byte to execute graphical operations.
   * @return The content byte.
   */
  public PdfContentByte getContentByte() {
    return cb;
  }
  
  /**
   * Sets the current stroke color.
   * @param color The stroke color to set.
   */
  public void setStrokeColor(Color color) {
    if (color == null) {
      cb.resetCMYKColorStroke();
    } else {
      cb.setColorStroke(color);
    }
  }

  /**
   * Sets the current fill color.
   * @param color The fill color to set.
   */
  public void setFillColor(Color color) {
    if (color == null) {
      cb.resetCMYKColorFill();
    } else {
      cb.setColorFill(color);
    }
  }

  /**
   * Sets the current line width.
   * @param width The line width to set.
   */
  public void setLineWidth(float width) {
    cb.setLineWidth(width);
  }
  
  /**
   * Sets the current line cap.
   * @param lineCap The line cap to set.
   */
  public void setLineCap(int lineCap) {
    cb.setLineCap(lineCap);
  }

  /**
   * Sets the current line join.
   * @param lineJoin The line join to set.
   */
  public void setLineJoin(int lineJoin) {
    cb.setLineJoin(lineJoin);
  }

  /**
   * Sets the current line dash.
   * @param phase The value of the phase.
   * @param unitsOn The number of units that must be on.
   * @param unitsOff The number of units that must be off.
   */
  public void setLineDash(float phase, float unitsOn, float unitsOff) {
    if (unitsOff > 0.0F) {
      cb.setLineDash(unitsOn, unitsOn, phase);
    } else if (unitsOn > 0.0F) {
      cb.setLineDash(unitsOn, phase);
    } else {
      cb.setLineDash(phase);
    }
  }
  
}
