/*
 * Copyright (C) 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.test;

import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;

import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import org.ujac.print.AbstractTagTest;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.DocumentPrinter;
import org.ujac.print.TestTagInterceptor;

import org.ujac.util.io.FileResourceLoader;
import org.ujac.util.io.ResourceLoader;

import org.ujac.util.template.DefaultTemplateInterpreterFactory;
import org.ujac.util.template.TemplateInterpreter;

import org.ujac.util.text.FormatHelper;

import junit.framework.TestCase;


/**
 * Name: BasePrintTests<br>
 * Description: A base class for print tests.
 * 
 * @author lauerc
 */
public class BasePrintTest extends TestCase {

  /** The template interpreter to test. */ 
  protected TemplateInterpreter templateInterpreter = null; 
  /** The format helper to use. */ 
  protected FormatHelper formatHelper = null;
  
  /** The document printer to use. */
  protected DocumentPrinter docPrinter = null;
  /** The document properties to use. */
  protected Map docProperties = null;
  /** The resource loader. */
  protected ResourceLoader resourceLoader = null;
  /** The document tag interceptor for testing purposes. */
  private TestTagInterceptor testTagInterceptor = null;

  /**
   * Sets up the test data.
   * @exception Exception In case a problem occured during the set up.
   */
  protected void setUp() throws Exception {
    super.setUp();
    // creating format helper for international locale
    this.formatHelper = FormatHelper.createInstance(Locale.US);
    // creating expression interpreter
    templateInterpreter = new DefaultTemplateInterpreterFactory().createTemplateInterpreter();

    this.docProperties = new HashMap();
    this.docPrinter = new DocumentPrinter();
    this.docPrinter.setProperties(this.docProperties);
    this.docPrinter.setTemplateInterpreter(this.templateInterpreter);
    this.resourceLoader = new FileResourceLoader("./");
    docPrinter.setResourceLoader(resourceLoader);
    docPrinter.setTemplateInterpreter(templateInterpreter);
    
    testTagInterceptor = new TestTagInterceptor();
    docPrinter.setTagInterceptor(testTagInterceptor);
  }

  /**
   * Sets the document template source.
   * @param location The location of the document template file.
   * @throws IOException In case the document template file could not be opened.
   */
  protected void setTemplateSource(String location) throws IOException {
    this.docPrinter.setTemplateSource(new FileReader(location));
  }

  /**
   * Loads the properties file from the given location.
   * @param location The location of the properties file to read.
   * @throws IOException In case the properties file could not be loaded.
   */
  protected void loadProperties(String location) throws IOException {
    this.docProperties = DocumentPrinter.loadProperties(location);
    this.docPrinter.setProperties(this.docProperties);
  }

  /**
   * Prints the document to the given destination.
   * @param destFile The destination file name.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  protected void printDocument(String destFile) throws DocumentHandlerException, IOException {
    printDocument(destFile, false);
  }

  /**
   * Prints the document to the given destination.
   * @param destFile The destination file name.
   * @param supportToc Tells whether or not to support the output of a leading table of contents.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  protected void printDocument(String destFile, boolean supportToc) throws DocumentHandlerException, IOException {
    this.docPrinter.printDocument(new FileOutputStream(destFile), supportToc);
  }
  
  /**
   * Adds a single test to the test tag interceptor.
   * @param tagId The ID of the tag to test. 
   * @param test The test to perform.
   */
  protected void addTagTest(String tagId, AbstractTagTest test) {
    this.testTagInterceptor.addTest(tagId, test);
  }
  /**
   * Adds a single test to the test tag interceptor.
   * @param tagId The ID of the tag to test. 
   * @param test The test to perform.
   */
  protected void clearTagTests() {
    this.testTagInterceptor.clearTests();
  }
  
  /**
   * Tells, whether or not all test have been passed.
   * @return true in case all tests have been performed successfully, else falses.
   */
  protected boolean isAllTagTestsPassed() {
    return this.testTagInterceptor.isAllTestsPassed();
  }

}
