/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.test;

import java.awt.Color;
import java.awt.Font;

import java.io.IOException;

import java.sql.Time;
import java.sql.Timestamp;

import java.text.DecimalFormatSymbols;
import java.text.ParseException;

import java.util.ArrayList;
import java.util.List;

import org.ujac.chart.Chart;
import org.ujac.chart.ChartAttributes;
import org.ujac.chart.ChartException;
import org.ujac.chart.ChartFactory;
import org.ujac.chart.ChartItemPosition;
import org.ujac.chart.ChartModel;
import org.ujac.chart.DefaultChartModel;
import org.ujac.chart.TextChartItem;
import org.ujac.print.AbstractTagTest;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.tag.BoxTag;
import org.ujac.print.tag.CellAttributes;
import org.ujac.print.tag.ImageTag;
import org.ujac.print.tag.MacroTag;
import org.ujac.print.tag.PdfCellTag;


import org.ujac.util.StopWatch;
import org.ujac.util.UjacTypes;

import org.ujac.util.table.DataTable;
import org.ujac.util.table.LayoutHints;
import org.ujac.util.table.Row;
import org.ujac.util.table.Table;
import org.ujac.util.table.TableException;
import org.ujac.util.text.FormatHelper;

import com.lowagie.text.BadElementException;
import com.lowagie.text.Image;
import com.lowagie.text.Jpeg;

/**
 * Name: DocumentPrinterTest<br>
 * Description: Common test cases for the document printer.
 *   Some of these test cases only check whether the PDF generation work, 
 *   they do not prove the correctness of the results! Other test cases
 *   are performing automated tests, based on the DocumentTagInterceptor 
 *   concept.
 * 
 * @author lauerc
 */
public class DocumentPrinterTest extends BasePrintTest {
  
  /**
   * Name: ChartTestTuple<br>
   * Description: A class for chart test values.
   */
  public class ChartTestTuple {      

    /** The first value. */
    private double value;
    /** The second value. */
    private double value2;
    /** The third value. */
    private double value3;
    /** The label. */
    private String description;
    
    /**
     * Constructs a ChartTestTuple instance with specific attributes.
     * @param value The first value.
     * @param value2 The second value.
     * @param value3 The third value.
     * @param description The label
     */
    public ChartTestTuple(double value,double value2, double value3, String description) {
      this.value = value;
      this.value2 = value2;
      this.value3 = value3;
      this.description = description;
    }

    /**
     * Getter method for the the property description.
     * @return The current value of property description.
     */
    public String getDescription() {
      return description;
    }
    
    /**
     * Getter method for the the property value.
     * @return The current value of property value.
     */
    public double getValue() {
      return value;
    }
    
    /**
     * Getter method for the the property value2.
     * @return The current value of property value2.
     */
    public double getValue2() {
      return value2;
    }
    
    /**
     * Getter method for the the property value3.
     * @return The current value of property value3.
     */
    public double getValue3() {
      return value3;
    }

  }

  /**
   * Name: Person<br>
   * Description: Test bean.
   */
  public class Person {
    
    /** The last name. */
    private String name = null;
    /** The first name. */
    private String firstName = null;
    /** The sex ('m' | 'f'). */
    private String sex = "m";
    
    /**
     * Constructs a Person instance with specific attributes.
     */
    public Person() {
    }

    /**
     * Constructs a Person instance with specific attributes.
     * @param name The last name to set.
     * @param firstName The first name to set.
     * @param sex The sex to set.
     */
    public Person(String name, String firstName, String sex) {
      this.name = name;
      this.firstName = firstName;
      this.sex = sex;
    }

    /**
     * Gets the last name.
     * @return The last name
     */
    public String getName() {
      return name;
    }
    /**
     * Sets the last name.
     * @param name The last name to set.
     */
    public void setName(String name) {
      this.name = name;
    }

    /**
     * Gets the first name.
     * @return The first name
     */
    public String getFirstName() {
      return firstName;
    }
    /**
     * Sets the first name.
     * @param firstName The first name to set.
     */
    public void setFirstName(String firstName) {
      this.firstName = firstName;
    }

    /**
     * Gets the sex.
     * @return The last name
     */
    public String getSex() {
      return sex;
    }
    /**
     * Sets the sex.
     * @param sex The sex to set.
     */
    public void setSex(String sex) {
      this.sex = sex;
    }
    
  }

  /**
   * Sets up the test data.
   * @exception Exception In case a problem occured during the set up.
   */
  protected void setUp() throws Exception {
    super.setUp();
  }

  /**
   * Tests barcode output.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testBarcode() throws DocumentHandlerException, IOException {
    setTemplateSource("test/print/barcode-test.xml");
    printDocument("test/barcode-test.pdf");
  }

  /**
   * Tests image output.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   * @exception BadElementException In case the Jpeg handling failed.
   */
  public void testImages() throws DocumentHandlerException, IOException, BadElementException {
    loadProperties("test/print/image-test.properties");
    byte[] ujacLogo = resourceLoader.loadResource("test/print/ujac-logo.jpg");
    docProperties.put("ujacLogo", ujacLogo);
    Jpeg itextJpg = new Jpeg(ujacLogo);
    docProperties.put("itextJpg", itextJpg);
    setTemplateSource("test/print/image-test.xml");
    
    addTagTest("jpg", new AbstractTagTest() {
      int jpgRef = -1;
      public void performTest() throws DocumentHandlerException {
        ImageTag tag = (ImageTag) this.getTag();
        Image img = tag.getImage();
        Long ref = img.getMySerialId();
        int refId = ref.intValue();
        //System.out.println("jpg ref: " + refId);
        if (jpgRef != -1) {
          assertEquals(jpgRef, refId);
        }
        jpgRef = refId;
      }
    });

    addTagTest("itextJpg", new AbstractTagTest() {
      int jpgRef = -1;
      public void performTest() throws DocumentHandlerException {
        ImageTag tag = (ImageTag) this.getTag();
        Image img = tag.getImage();
        Long ref = img.getMySerialId();
        int refId = ref.intValue();
        //System.out.println("jpg ref: " + refId);
        if (jpgRef != -1) {
          assertEquals(jpgRef, refId);
        }
        jpgRef = refId;
      }
    });

    addTagTest("pdf", new AbstractTagTest() {
      int pdfRef = -1;
      public void performTest() throws DocumentHandlerException {
        ImageTag tag = (ImageTag) this.getTag();
        Image img = tag.getImage();
        Long ref = img.getMySerialId();
        int refId = ref.intValue();
        //System.out.println("pdf ref: " + refId);
        if (pdfRef != -1) {
          assertEquals(pdfRef, refId);
        }
        pdfRef = refId;
      }
    });

    addTagTest("svg", new AbstractTagTest() {
      int svgRef = -1;
      public void performTest() throws DocumentHandlerException {
        ImageTag tag = (ImageTag) this.getTag();
        Image img = tag.getImage();
        Long ref = img.getMySerialId();
        int refId = ref.intValue();
        //System.out.println("svg ref: " + refId);
        if (svgRef != -1) {
          assertEquals(svgRef, refId);
        }
        svgRef = refId;
      }
    });

    addTagTest("svgz", new AbstractTagTest() {
      int svgzRef = -1;
      public void performTest() throws DocumentHandlerException {
        ImageTag tag = (ImageTag) this.getTag();
        Image img = tag.getImage();
        Long ref = img.getMySerialId();
        int refId = ref.intValue();
        //System.out.println("svgz ref: " + refId);
        if (svgzRef != -1) {
          assertEquals(svgzRef, refId);
        }
        svgzRef = refId;
      }
    });
    
    printDocument("test/image-test.pdf");

    assertTrue(isAllTagTestsPassed());
  }
  
  /**
   * Tests usage of overlays.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testOverlay() throws DocumentHandlerException, IOException {
    //loadProperties("test/print/overlay-test.properties");
    setTemplateSource("test/print/overlay-test.xml");
    printDocument("test/overlay-test.pdf");
  }
  
  /**
   * Tests chart output.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testCharts() throws DocumentHandlerException, IOException {
    loadProperties("test/print/chart-test.properties");
    setTemplateSource("test/print/chart-test.xml");
    printDocument("test/chart-test.pdf");
    
    List chartValues = new ArrayList();
    chartValues.add(new ChartTestTuple(12.43, 15.32, 14.0, "OBM"));
    chartValues.add(new ChartTestTuple(432.32, 545.22, 43.34, "TexMEX"));
    chartValues.add(new ChartTestTuple(243.46, 435.22, 343.54, "Orbit"));
    chartValues.add(new ChartTestTuple(322.75, 315.22, 373.46, "YesNo"));
    chartValues.add(new ChartTestTuple(342.12, 845.22, 553.76, "TicTac"));
    chartValues.add(new ChartTestTuple(922.22, 655.22, 46.83, "PostPhone"));
    
    docProperties.put("values", chartValues);
    setTemplateSource("test/print/chart-test.xml");
    printDocument("test/chart-test3.pdf");

    setTemplateSource("test/print/chart-test2.xml");
    printDocument("test/chart-test2.pdf");
  }
  
  /**
   * Tests chart instance output.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   * @exception ChartException In case the chart processing failed.
   */
  public void testChartInstance() throws DocumentHandlerException, IOException, ChartException {
    setTemplateSource("test/print/chart-instance-test.xml");

    // testing chart instance
    String title = "JAVA Application Server Market Share";
    ChartModel cm = new DefaultChartModel();
    cm.setTitle(new TextChartItem(new ChartItemPosition(ChartItemPosition.CENTER, ChartItemPosition.TOP), 
                title, new Font("Helvetica", Font.HANGING_BASELINE | Font.BOLD, 20)));
    //cm.setMaxValue(200);
    cm.setMinValue(0);
    List chartData = new ArrayList();
    chartData.add(new double[] {4, 5, 7});
    chartData.add(new double[] {30, 25, 20});
    chartData.add(new double[] {3, 2, 1});
    chartData.add(new double[] {4, 5, 7});
    chartData.add(new double[] {18, 22, 25});
    chartData.add(new double[] {3, 3, 2});
    chartData.add(new double[] {4, 2, 1});
    chartData.add(new double[] {6, 10, 13});
    chartData.add(new double[] {4, 3, 2});
    chartData.add(new double[] {1, 2, 1});
    chartData.add(new double[] {3, 4, 4});
    chartData.add(new double[] {6, 4, 3});
    chartData.add(new double[] {4, 4, 6});
    chartData.add(new double[] {7, 7, 6});
    chartData.add(new double[] {3, 2, 2});
    cm.setChartData(chartData);
    
    ChartAttributes attrs = new ChartAttributes();
    List colors = new ArrayList();
    colors.add(Color.blue);
    colors.add(Color.green);
    colors.add(Color.red);
    colors.add(Color.orange);
    colors.add(Color.yellow);
    colors.add(Color.magenta);
    colors.add(Color.cyan);
    attrs.setList(Chart.ATTR_SEGMENT_COLORS, colors);
    
    List descriptions = new ArrayList();
    descriptions.add("Apache Tomcat");
    descriptions.add("BEA WebLogic");
    descriptions.add("Borland Enterprise Server");
    descriptions.add("Caucho Resin");
    descriptions.add("IBM Websphere");
    descriptions.add("IONA Orbix ASP");
    descriptions.add("Ironflare Orion");
    descriptions.add("JBoss AS");
    descriptions.add("Macromedia JRun");
    descriptions.add("Novell exteNd");
    descriptions.add("Oracle Application Server");
    descriptions.add("Pramati Server");
    descriptions.add("SAP Web Application Server");
    descriptions.add("Sun ONE Application Server");
    descriptions.add("Sybase EAServer");
    cm.setSegmentLabels(descriptions);

    descriptions = new ArrayList();
    descriptions.add("2002");
    descriptions.add("2003");
    descriptions.add("2004");
    cm.setAreaLabels(descriptions);
    
    attrs.setString(Chart.ATTR_DESCRIPTION_PATTERN, "%s (%v Parts, %p%%)");
    attrs.setFloat(Chart.ATTR_BORDER_WIDTH, 1.0F);
    attrs.setFloat(Chart.ATTR_DESCRIPTION_LINE_WIDTH, 2.0F);
    attrs.setFloat(Chart.ATTR_LINE_WIDTH, 5.0F);
    attrs.setBoolean(Chart.ATTR_VIEW_LEGEND, true);
    attrs.setBoolean(Chart.ATTR_FILL_AREA, true);
    attrs.setFloat(Chart.ATTR_VALUE_STEP_SIZE, 10);
    attrs.setString(Chart.ATTR_SEGMENT_MARK_STYLE, "border");
    
    Chart chart = ChartFactory.createChart("line-2d", cm, attrs);
    docProperties.put("chartInstance", chart);
    printDocument("test/chart-instance-test.pdf");
  }

  /**
   * Tests whitespace handling output.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testWhitespaces() throws DocumentHandlerException, IOException {
    loadProperties("test/print/whitespace-test.properties");
    setTemplateSource("test/print/whitespace-test.xml");
    addTagTest("ctcc", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        String content = getContent();
        assertFalse(content.charAt(0) == ' ');
      }
    });
    printDocument("test/whitespace-test.pdf");
  }
  
  /**
   * Tests font handling.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testFont() throws DocumentHandlerException, IOException {
    loadProperties("test/print/font-test.properties");
    setTemplateSource("test/print/font-test.xml");
    printDocument("test/font-test.pdf");
  }

  /**
   * Tests header/footer usage.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testHeaderFooter() throws DocumentHandlerException, IOException {
    loadProperties("test/print/header-footer-test.properties");
    setTemplateSource("test/print/header-footer-test.xml");
    printDocument("test/header-footer-test.pdf");
  }

  /**
   * Tests list output.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testList() throws DocumentHandlerException, IOException {
    loadProperties("test/print/list-test.properties");
    setTemplateSource("test/print/list-test.xml");
    printDocument("test/list-test.pdf");
  }

  /**
   * Tests internationalization.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testI18n() throws DocumentHandlerException, IOException {
    loadProperties("test/print/i18n.properties");
    setTemplateSource("test/print/i18n-test.xml");
    printDocument("test/i18n-test.pdf");
  }
  
  /**
   * Tests custom tag handling.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testCustomTags() throws DocumentHandlerException, IOException {
    docPrinter.loadModule("test/print/mymodule.xml");
    setTemplateSource("test/print/custom-tag-test.xml");
    printDocument("test/custom-tag-test.pdf");
  }

  /**
   * Tests anchor handling.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testAnchor() throws DocumentHandlerException, IOException {
    setTemplateSource("test/print/anchor-test.xml");
    printDocument("test/anchor-test.pdf");
  }

  /**
   * Tests chapter and section output.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testSection() throws DocumentHandlerException, IOException {
    setTemplateSource("test/print/section-test.xml");
    printDocument("test/section-test.pdf");
  }

  /**
   * Tests column text usage.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testColumnText() throws DocumentHandlerException, IOException {
    setTemplateSource("test/print/column-text-test.xml");
    printDocument("test/column-text-test.pdf");
  }

  /**
   * Tests TOC handling.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testToc() throws DocumentHandlerException, IOException {
    setTemplateSource("test/print/toc-test2.xml");
    printDocument("test/toc-test2.pdf", false);

    StopWatch sw = new StopWatch();
    setTemplateSource("test/print/toc-test.xml");
    docPrinter.setPageNumberingStartsAfterHead(true);
    printDocument("test/toc-test.pdf", true);
    System.out.println("Needed " + sw.stop() + " to print document without numbered leading TOC area.");
    
    sw.reset();
    setTemplateSource("test/print/toc-test3.xml");
    docPrinter.setPageNumberingStartsAfterHead(false);
    printDocument("test/toc-test3.pdf", true);
    System.out.println("Needed " + sw.stop() + " to print document with numbered leading TOC area.");

    sw.reset();
    setTemplateSource("test/print/toc-test4.xml");
    docPrinter.setPageNumberingStartsAfterHead(true);
    printDocument("test/toc-test4.pdf", true);
    System.out.println("Needed " + sw.stop() + " to print document with numbered leading TOC area.");

    sw.reset();
    setTemplateSource("test/print/toc-test5.xml");
    docPrinter.setPageNumberingStartsAfterHead(true);
    printDocument("test/toc-test5.pdf", true);
    System.out.println("Needed " + sw.stop() + " to print document with numbered leading TOC area.");
  }

  /**
   * Tests AcroForm output.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testAcroForm() throws DocumentHandlerException, IOException {
    setTemplateSource("test/print/acro-form-test.xml");
    printDocument("test/acro-form-test.pdf");
  }

  /**
   * Tests graphic output.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testGraphics() throws DocumentHandlerException, IOException {
    setTemplateSource("test/print/graphics-test.xml");
    printDocument("test/graphics-test.pdf");
  }

  /**
   * Tests handling of vertical spacings.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testVerticalSpacing() throws DocumentHandlerException, IOException {
    setTemplateSource("test/print/vertical-spacing-test.xml");
    printDocument("test/vertical-spacing-test.pdf");
  }

  /**
   * Tests color handling.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testColor() throws DocumentHandlerException, IOException {
    setTemplateSource("test/print/color-test.xml");
    printDocument("test/color-test.pdf");
  }

  /**
   * Tests handling of sticky tags.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testSticky() throws DocumentHandlerException, IOException {
    setTemplateSource("test/print/sticky-test.xml");
    addTagTest("tableBox", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        BoxTag tag = (BoxTag) this.tag;
        CellAttributes ca = tag.getLocalCellAttributes();
        assertEquals(new Float(80.0F), new Float(ca.getFixedHeight()));
      }
    });
    addTagTest("boxTableCell", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        PdfCellTag tag = (PdfCellTag) this.tag;
        CellAttributes ca = tag.getLocalCellAttributes();
        assertEquals(new Float(-1.0F), new Float(ca.getFixedHeight()));
        assertEquals("Test", getContent());
      }
    });
    printDocument("test/sticky-test.pdf");
    assertTrue(isAllTagTestsPassed());
  }

  /**
   * Tests usage of viewer preferences.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testViewerPreferences() throws DocumentHandlerException, IOException {
    setTemplateSource("test/print/viewer-preferences-test.xml");
    printDocument("test/viewer-preferences-test.pdf");
  }

  /**
   * Tests handling of segments.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testSegments() throws DocumentHandlerException, IOException {
    setTemplateSource("test/print/segment-test.xml");
    printDocument("test/segment-test.pdf");
  }

  /**
   * Tests usage of document encryption.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testEncryption() throws DocumentHandlerException, IOException {
    setTemplateSource("test/print/encryption-test.xml");
    printDocument("test/encryption-test.pdf");
  }

  /**
   * Tests usage of watermarks.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testWatermark() throws DocumentHandlerException, IOException {
    setTemplateSource("test/print/watermark-test.xml");
    printDocument("test/watermark-test.pdf");
  }

  /**
   * Tests log output.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testLog() throws DocumentHandlerException, IOException {
    setTemplateSource("test/print/log-test.xml");
    printDocument("test/log-test.pdf");
  }

  /**
   * Tests the &lt;define-table&gt; tag.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testDefineTable() throws DocumentHandlerException, IOException {
    setTemplateSource("test/print/define-table-test.xml");
    printDocument("test/define-table-test.pdf");
  }

  /**
   * Tests the &lt;style-def&gt; tag.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testStyle() throws DocumentHandlerException, IOException {
    setTemplateSource("test/print/style-test.xml");
    addTagTest("staticCell1", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        assertEquals("Karl Meier", getContent());
        assertEquals(new Color(0, 127, 255), getColorStyleAttribute("background-color"));
        assertEquals(new Float(2.0F), new Float(getFloatStyleAttribute("padding-top")));
        assertEquals(new Float(5.0F), new Float(getFloatStyleAttribute("padding-bottom")));
      }
    });
    addTagTest("staticCell2", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        assertEquals("Michael Walter", getContent());
        assertNull(getColorStyleAttribute("background-color"));
        assertEquals(new Float(2.0F), new Float(getFloatStyleAttribute("padding-top")));
        assertEquals(new Float(5.0F), new Float(getFloatStyleAttribute("padding-bottom")));
      }
    });
    printDocument("test/style-test.pdf");
    assertTrue(isAllTagTestsPassed());
  }

  /**
   * Tests the &lt;set-format&gt; tag.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception ParseException In case a date could not be parsed.
   * @exception IOException In case something went wrong during output.
   * @exception TableException In case a table processing error occurred.
   */
  public void testSetFormat() throws DocumentHandlerException, IOException, ParseException, TableException {
    setTemplateSource("test/print/set-format-test.xml");
    FormatHelper fh = FormatHelper.createInstance();
    docProperties.put("date", fh.getIsoDateFormat().parse("2003-02-24"));
    docProperties.put("time", new Time(fh.getIsoTimeFormat().parse("22:38:47").getTime()));
    docProperties.put("timestamp", new Timestamp(fh.getIsoTimestampFormat().parse("2003-02-24 22:38:47").getTime()));
    docProperties.put("int", new Integer(12));
    docProperties.put("long", new Long(1432425345452L));
    docProperties.put("double", new Double(1763.515645));

    Table data = new DataTable();
    int dateColID = data.addColumn("date", UjacTypes.TYPE_DATE, new LayoutHints(10, "date")).getIndex();
    int timeColID = data.addColumn("time", UjacTypes.TYPE_TIME, new LayoutHints(10, "time")).getIndex();
    int timestampColID = data.addColumn("timestamp", UjacTypes.TYPE_TIMESTAMP, new LayoutHints(10, "timestamp")).getIndex();
    int intColID = data.addColumn("int", UjacTypes.TYPE_INT, new LayoutHints(10, "int")).getIndex();
    int longColID = data.addColumn("long", UjacTypes.TYPE_LONG, new LayoutHints(10, "long")).getIndex();
    int doubleColID = data.addColumn("double", UjacTypes.TYPE_DOUBLE, new LayoutHints(10, "double")).getIndex();
    Row row = data.addRow();
    row.setDate(dateColID, fh.getIsoDateFormat().parse("2003-02-24"));
    row.setTime(timeColID, new Time(fh.getIsoTimeFormat().parse("22:38:47").getTime()));
    row.setTimestamp(timestampColID, new Timestamp(fh.getIsoTimestampFormat().parse("2003-02-24 22:38:47").getTime()));
    row.setInt(intColID, 12);
    row.setLong(longColID, 1432425345452L);
    row.setDouble(doubleColID, 1763.515645);
    docProperties.put("table", data);
    printDocument("test/set-format-test.pdf");
  }

  /**
   * Tests the &lt;set-property&gt; tag.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception ParseException In case a date could not be parsed.
   * @exception IOException In case something went wrong during output.
   * @exception TableException In case a table processing error occurred.
   */
  public void testSetProperty() throws DocumentHandlerException, IOException, ParseException, TableException {
    final char decSeparator = new DecimalFormatSymbols().getDecimalSeparator();
    setTemplateSource("test/print/set-property-test.xml");
    addTagTest("iCell", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        assertEquals("88", getContent());
      }
    });
    addTagTest("lCell", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        assertEquals("88", getContent());
      }
    });
    addTagTest("fCell", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        assertEquals("88" + decSeparator + "000", getContent());
      }
    });
    addTagTest("dCell", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        assertEquals("88" + decSeparator + "000", getContent());
      }
    });
    addTagTest("dateCell", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        assertEquals("08.08.2008", getContent());
      }
    });
    addTagTest("timeCell", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        assertEquals("08:08:08", getContent());
      }
    });
    addTagTest("timestampCell", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        assertEquals("08.08.2008 08:08:08", getContent());
      }
    });
    printDocument("test/set-property-test.pdf");
    assertTrue(isAllTagTestsPassed());
  }
  
  /**
   * Tests the &lt;set-format&gt; tag.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testSpecialCharacters() throws DocumentHandlerException, IOException {
    docProperties.put("double", new Double(1763.515645));
    docProperties.put("ltgt", "<->");
    setTemplateSource("test/print/special-characters-test.xml");
    docProperties.put("text", "&lt;test&gt;");
    docProperties.put("containsBackslash", "2134234 & & ..... \\");
    this.docPrinter.setTranslateEscapeSequences(false);
    printDocument("test/special-characters-test.pdf");
  }

  /**
   * Tests conditional tag evaluation.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testConditions() throws DocumentHandlerException, IOException {
    setTemplateSource("test/print/condition-test.xml");
    
    docProperties.put("helma", new Person("Meier", "Helma", "f"));
    docProperties.put("fritz", new Person("Meier", "Fritz", "m"));

    addTagTest("helma1", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        if (getTag() instanceof MacroTag) {
          // skipping macro
          return;
        }
        assertEquals("Hello Mrs. Helma Meier", getContent());
      }
    });

/*
    addTagTest("staticCell2", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        assertEquals("Michael Walter", getContent());
        assertNull(getColorStyleAttribute("background-color"));
        assertEquals(new Float(2.0F), new Float(getFloatStyleAttribute("padding-top")));
        assertEquals(new Float(5.0F), new Float(getFloatStyleAttribute("padding-bottom")));
      }
    });
*/
    printDocument("test/condition-test.pdf");
    assertTrue(isAllTagTestsPassed());
  }

}
