/*
 * Copyright (C) 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.print.test;

import java.awt.Color;

import java.io.IOException;

import org.ujac.print.AbstractTagTest;
import org.ujac.print.DocumentHandlerException;
import org.ujac.print.tag.BaseCellTag;
import org.ujac.print.tag.CellAttributes;

import org.ujac.util.UjacTypes;
import org.ujac.util.table.DataTable;
import org.ujac.util.table.LayoutHints;
import org.ujac.util.table.Row;
import org.ujac.util.table.Table;
import org.ujac.util.table.TableException;


/**
 * Name: TableTest<br>
 * Description: Table related test cases for the document printer.
 * 
 * @author lauerc
 */
public class TableTest extends BasePrintTest {

  /**
   * Sets up the test data.
   * @exception Exception In case a problem occured during the set up.
   */
  protected void setUp() throws Exception {
    super.setUp();

    addTagTest("name1", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        assertEquals("Karl Meier", getContent());
      }
    });
    addTagTest("name2", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        assertEquals("Michael Walter", getContent());
      }
    });
    addTagTest("name3", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        assertEquals("Udo Becker", getContent());
      }
    });
    addTagTest("name4", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        assertEquals("Konrad Fritz", getContent());
      }
    });
  }
  

  /**
   * Tests table layout.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testTableLayout() 
    throws DocumentHandlerException, IOException {

    clearTagTests();
    
    addTagTest("colored", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        assertEquals("colored border", getContent());
        BaseCellTag ct = (BaseCellTag) tag;
        CellAttributes attrs = ct.getCellAttributes();
        // testing border widths
        assertEquals(new Float(1.0F), new Float(attrs.getBorderLeftWidth()));
        assertEquals(new Float(2.0F), new Float(attrs.getBorderRightWidth()));
        assertEquals(new Float(3.0F), new Float(attrs.getBorderTopWidth()));
        assertEquals(new Float(4.0F), new Float(attrs.getBorderBottomWidth()));

        // testing border colors
        assertEquals(new Color(0, 0, 255), attrs.getBorderLeftColor());
        assertEquals(new Color(0, 255, 0), attrs.getBorderRightColor());
        assertEquals(new Color(255, 255, 0), attrs.getBorderTopColor());
        assertEquals(new Color(255, 0, 0), attrs.getBorderBottomColor());
      }
    });

    addTagTest("funky", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        assertEquals("funky border", getContent());
        BaseCellTag ct = (BaseCellTag) tag;
        CellAttributes attrs = ct.getCellAttributes();
        // testing border widths
        assertEquals(new Float(1.0F), new Float(attrs.getBorderLeftWidth()));
        assertEquals(new Float(2.0F), new Float(attrs.getBorderRightWidth()));
        assertEquals(new Float(1.0F), new Float(attrs.getBorderTopWidth()));
        assertEquals(new Float(2.0F), new Float(attrs.getBorderBottomWidth()));

        // testing border colors
        assertEquals(new Color(0, 0, 255), attrs.getBorderLeftColor());
        assertEquals(new Color(0, 255, 0), attrs.getBorderRightColor());
        assertEquals(new Color(255, 255, 0), attrs.getBorderTopColor());
        assertEquals(new Color(255, 0, 0), attrs.getBorderBottomColor());
      }
    });

    setTemplateSource("test/print/table-test.xml");
    printDocument("test/table-test.pdf");
  }

  /**
   * Tests static table output.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   */
  public void testStaticTable() 
    throws DocumentHandlerException, IOException {
    
    setTemplateSource("test/print/static-table-test.xml");
    addTagTest("city1", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        assertEquals("Hasenhausen", getContent());
      }
    });
    addTagTest("street2", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        assertEquals("Ulmenweg 29", getContent());
      }
    });
    addTagTest("zip3", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        assertEquals("34375", getContent());
      }
    });
    addTagTest("dateOfBirth4", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        assertEquals("02.02.1971", getContent());
      }
    });
    printDocument("test/static-table-test.pdf");
    assertTrue(isAllTagTestsPassed());
  }

  /**
   * Tests table output with static layout.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   * @exception TableException In case the table processing failed.
   */
  public void testStaticTableLayout()
    throws DocumentHandlerException, IOException, TableException {
    
    loadProperties("test/print/table-test.properties");

    setTemplateSource("test/print/static-table-layout-test.xml");
    addTagTest("city1", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        assertEquals("Hasenhausen", getContent());
      }
    });
    addTagTest("street2", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        assertEquals("Ulmenweg 29", getContent());
      }
    });
    addTagTest("zip3", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        assertEquals("34375", getContent());
      }
    });
    addTagTest("dateOfBirth4", new AbstractTagTest() {
      public void performTest() throws DocumentHandlerException {
        assertEquals("02.02.1971", getContent());
      }
    });
    printDocument("test/static-table-layout-test.pdf");
    assertTrue(isAllTagTestsPassed());
  }

  /**
   * Tests table output with static layout.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   * @exception TableException In case the table processing failed.
   */
  public void testDynamicTable()
    throws DocumentHandlerException, IOException, TableException {
    
    loadProperties("test/print/table-test.properties");

    setTemplateSource("test/print/dynamic-table-test.xml");
    printDocument("test/dynamic-table-test.pdf");
    assertTrue(isAllTagTestsPassed());
  }

  /**
   * Tests table output using the define column tag.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   * @exception TableException In case the table processing failed.
   */
  public void testDefineColumn()
    throws DocumentHandlerException, IOException, TableException {
    
    loadProperties("test/print/table-test.properties");

    setTemplateSource("test/print/define-column-test.xml");
    printDocument("test/define-column-test.pdf");
    //assertTrue(isAllTagTestsPassed());
  }
  
  /**
   * Tests table output performance.
   * @exception DocumentHandlerException If something went wrong at document handling.
   * @exception IOException In case something went wrong during output.
   * @exception TableException In case the table processing failed.
   */
  public void testTableOutputPerformances() throws DocumentHandlerException, IOException, TableException {
    loadProperties("test/print/table-test.properties");

    clearTagTests();
    
    //long start = System.currentTimeMillis();
    Table persons = new DataTable();
    persons.addColumn("id", UjacTypes.TYPE_INT)
      .setLayoutHints(new LayoutHints(15, "ID", LayoutHints.RIGHT))
      .setFormat("######");
    persons.addColumn("lastName", UjacTypes.TYPE_STRING).setLayoutHints(new LayoutHints(20, "Name"));
    persons.addColumn("firstName", UjacTypes.TYPE_STRING).setLayoutHints(new LayoutHints(20, "First Name"));
    persons.addColumn("address", UjacTypes.TYPE_STRING).setLayoutHints(new LayoutHints(20, "Address"));
    persons.addColumn("zipCode", UjacTypes.TYPE_INT).setLayoutHints(new LayoutHints(15, "Zip Code"));
    persons.addColumn("city", UjacTypes.TYPE_STRING).setLayoutHints(new LayoutHints(20, "City"));
    persons.addColumn("dateOfBirth", UjacTypes.TYPE_STRING).setLayoutHints(new LayoutHints(20, "Date of birth", LayoutHints.CENTER));
    for (int i = 0; i < 1000; i++) {
      Row row = persons.addRow();
      row.setInt(0, i + 1);
      row.setString(1, "Meyer");
      row.setString(2, "Curt");
      row.setString(3, "Zoostr. 12");
      row.setInt(4, 13432);
      row.setString(5, "Cornvalley");
      row.setString(6, "01.12.1961");
    }
    //long end = System.currentTimeMillis();
    //System.out.println("Took " + (end - start) + "ms to fill data table.");
    docProperties.put("persons", persons);
    setTemplateSource("test/print/dynamic-table-test.xml");

    //start = System.currentTimeMillis();
    printDocument("test/big-table-test.pdf");
    //end = System.currentTimeMillis();
    //System.out.println("Took " + (end - start) + "ms to print document.");
  }

}
