/*
 * Copyright (C) 2004-2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Name: CollectionUtils<br>
 * Description: A class providing methods to ease the handling of collections.
 * 
 * @author lauerc
 */
public class CollectionUtils {
  
  /**
   * Probes whether the given collection is empty or not.
   * @param collection The collection to test.
   * @return true if the given collection is null or contains no elements
   */
  public static final boolean isEmpty(Collection collection) {
    return (collection == null) || (collection.size() < 1);
  }

  /**
   * Gets the size of the given collection.
   * @param collection The collection to to get the size for.
   * @return The size of the given collection or -1 in case null was given.
   */
  public static final int getSize(Collection collection) {
    if (collection == null) {
      return -1;
    }
    return collection.size();
  }

  /**
   * Probes whether the given map contains the given key or not.
   * @param collection The collection to check the presence of the element for.
   * @param element The collection to to test.
   * @return true if the given map contains the given key, else false.
   */
  public static final boolean contains(Collection collection, Object element) {
    return (collection != null) && (element != null) && collection.contains(element);
  }

  /**
   * Clears the given collection.
   * @param collection The collection to clear.
   */
  public static final void clear(Collection collection) {
    if (collection == null) {
      return;
    }
    collection.clear();
  }

  
  /**
   * Probes whether the given map is empty or not.
   * @param map The map to test.
   * @return true if the given map is null or contains no elements
   */
  public static final boolean isEmpty(Map map) {
    return (map == null) || (map.size() < 1);
  }

  /**
   * Gets the size of the given map.
   * @param map The map to to get the size for.
   * @return The size of the given map or -1 in case null was given.
   */
  public static final int getSize(Map map) {
    if (map == null) {
      return -1;
    }
    return map.size();
  }

  /**
   * Probes whether the given map contains the given key or not.
   * @param map The map to check the presence of the key for.
   * @param key The key to to test.
   * @return true if the given map contains the given key, else false.
   */
  public static final boolean containsKey(Map map, Object key) {
    return (map != null) && (key != null) && map.containsKey(key);
  }

  /**
   * Probes whether the given map contains the given value or not.
   * @param map The map to check the presence of the value for.
   * @param value The value to to test.
   * @return true if the given map contains the given value, else false.
   */
  public static final boolean containsValue(Map map, Object value) {
    return (map != null) && map.containsValue(value);
  }

  /**
   * Clears the given map.
   * @param map The map to clear.
   */
  public static final void clear(Map map) {
    if (map == null) {
      return;
    }
    map.clear();
  }

  
  /**
   * Creates an ArrayList instance and adds the given object to it.
   * @param obj The object to add to the List.
   * @return The created List instance.
   */
  public static final List createArrayList(Object obj) {
    List list = new ArrayList();
    list.add(obj);
    return list;
  }
  /**
   * Creates an ArrayList instance and adds the given objects to it.
   * @param objs The objects to add to the List.
   * @return The created List instance.
   */
  public static final List createArrayList(Object[] objs) {
    List list = new ArrayList();
    for (int i = 0; i < objs.length; i++) {
      list.add(objs[i]);
    }
    return list;
  }
  
  /**
   * Creates an ArrayList instance and adds the given objects to it.
   * @param list The list to sort.
   * @param fieldName The name of the field by which to sort. 
   */
  public static final void sortList(List list, String fieldName) {
    Collections.sort(list, new BeanComparator(fieldName));
  }
  /**
   * Creates an ArrayList instance and adds the given objects to it.
   * @param list The list to sort.
   * @param compareTuple The name of the field plus the direction by which to sort. 
   */
  public static final void sortList(List list, CompareTuple compareTuple) {
    Collections.sort(list, new BeanComparator(compareTuple));
  }
  /**
   * Creates an ArrayList instance and adds the given objects to it.
   * @param list The list to sort.
   * @param fieldNames The names of the fields by which to sort. 
   */
  public static final void sortList(List list, String[] fieldNames) {
    Collections.sort(list, new BeanComparator(fieldNames));
  }
  /**
   * Creates an ArrayList instance and adds the given objects to it.
   * @param list The list to sort.
   * @param compareTuples The names of the field plus the directions by which to sort. 
   */
  public static final void sortList(List list, CompareTuple[] compareTuples) {
    Collections.sort(list, new BeanComparator(compareTuples));
  }

  /**
   * Creates an HashSet instance and adds the given object to it.
   * @param obj The object to add to the Set.
   * @return The created Set instance.
   */
  public static final Set createHashSet(Object obj) {
    Set set = new HashSet();
    set.add(obj);
    return set;
  }
  /**
   * Creates an HashSet instance and adds the given objects to it.
   * @param objs The objects to add to the List.
   * @return The created Set instance.
   */
  public static final Set createHashSet(Object[] objs) {
    Set set = new HashSet();
    for (int i = 0; i < objs.length; i++) {
      set.add(objs[i]);
    }
    return set;
  }

  /**
   * Creates a HashMap instance and adds the given key/value pair to it.
   * @param key The key for the value to add.
   * @param value The value to add.
   * @return The created Map instance.
   */
  public static final Map createHashMap(Object key, Object value) {
    Map map = new HashMap();
    put(map, key, value);
    return map;
  }

  /**
   * Puts the given key/value pair into the given map.
   * @param map The map to set the key/value pair to.
   * @param key The key to associate the value with.
   * @param value The value to add.
   * @return The map instance.
   */
  public static final Map put(Map map, Object key, Object value) {
    if (map == null) {
      return null;
    }
    if ((key != null) && (value != null)) {
      map.put(key, value);
    }
    return map;
  }

  /**
   * Puts the given key/value pair into the given map.
   * @param map The map to add the given other map's entries to.
   * @param other The map to merge with the given map.
   * @return The map instance.
   */
  public static final Map putAll(Map map, Map other) {
    if ((map == null) || (other == null)) {
      return map;
    }
    map.putAll(other);
    return map;
  }

  /**
   * Gets a value according to the given key from the given map.
   * @param map The map to get the value from.
   * @param key The key to get the value for.
   * @return value The associated value.
   */
  public static final Object get(Map map, Object key) {
    if ((map == null) || (key == null)) {
      return null;
    }
    return map.get(key);
  }

  /**
   * Gets a value according to the given key from the given map.
   * @param map The map to get the value from.
   * @param key The key to get the value for.
   * @return value The associated value.
   */
  public static final Object putAll(Map map, Object key) {
    if ((map == null) || (key == null)) {
      return null;
    }
    return map.get(key);
  }

}
