/*
 * Copyright (C) 2005 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util;

import java.text.DateFormat;
import java.text.SimpleDateFormat;

import java.util.Calendar;
import java.util.Date;

/**
 * Name: DateUtils<br>
 * Description: Provides calendar and date utilities for more convenient date handling.
 * 
 * @author lauerc
 */
public class DateUtils {

  /**
   * Gets the current date.
   * @return The current date with all time values set to minimum.
   */
  public static final Date today() {
    Calendar cal = Calendar.getInstance();
    cal.set(Calendar.MILLISECOND, cal.getActualMinimum(Calendar.MILLISECOND));
    cal.set(Calendar.SECOND, cal.getActualMinimum(Calendar.SECOND));
    cal.set(Calendar.MINUTE, cal.getActualMinimum(Calendar.MINUTE));
    cal.set(Calendar.HOUR_OF_DAY, cal.getActualMinimum(Calendar.HOUR_OF_DAY));
    return cal.getTime();
  }

  /**
   * Gets the current date -1 day.
   * @return The current date -1 day with all time values set to minimum.
   */
  public static final Date yesterday() {
    Calendar cal = Calendar.getInstance();
    cal.set(Calendar.MILLISECOND, cal.getActualMinimum(Calendar.MILLISECOND));
    cal.set(Calendar.SECOND, cal.getActualMinimum(Calendar.SECOND));
    cal.set(Calendar.MINUTE, cal.getActualMinimum(Calendar.MINUTE));
    cal.set(Calendar.HOUR_OF_DAY, cal.getActualMinimum(Calendar.HOUR_OF_DAY));
    cal.add(Calendar.DATE, -1);
    return cal.getTime();
  }

  /**
   * Gets the current time.
   * @return The current time with milliseconds set to minimum.
   */
  public static final Date now() {
    return now(Calendar.SECOND);
  }
  
  /**
   * Gets the current time.
   * @param precision The ID of the most precise value of interest.
   * @see Calendar#MILLISECOND
   * @see Calendar#SECOND
   * @see Calendar#MINUTE
   * @see Calendar#HOUR
   * @return The current timestamp with all values more acurate than the 
   *  passed field ID set to minimum.
   */
  public static final Date now(int precision) {
    Calendar cal = Calendar.getInstance();
    
    if (precision == Calendar.MILLISECOND) {
      return cal.getTime();
    }
    cal.set(Calendar.MILLISECOND, cal.getActualMinimum(Calendar.MILLISECOND));

    if (precision == Calendar.SECOND) {
      return cal.getTime();
    }
    cal.set(Calendar.SECOND, cal.getActualMinimum(Calendar.SECOND));
    
    if (precision == Calendar.MINUTE) {
      return cal.getTime();
    }
    cal.set(Calendar.MINUTE, cal.getActualMinimum(Calendar.MINUTE));

    if ((precision == Calendar.HOUR) || (precision == Calendar.HOUR_OF_DAY)) {
      return cal.getTime();
    }
    cal.set(Calendar.HOUR_OF_DAY, cal.getActualMinimum(Calendar.HOUR_OF_DAY));
    
    return cal.getTime();
  }
  
  /**
   * Creates a Calendar instance for the given arguments.
   * @param year The year to set.
   * @param month The month to set.
   * @param day The day to set.
   * @return The created calendar instance.
   */
  public static final Calendar createCalendar(int year, int month, int day) {
    Calendar cal = createCalendar(Calendar.DATE);
    setYear(cal, year);
    setMonth(cal, month);
    setDay(cal, day);
    return cal;
  }
  
  /**
   * Creates a Calendar instance for the given arguments.
   * @param year The year to set.
   * @param month The month to set.
   * @param day The day to set.
   * @param hour The hour to set.
   * @param minute The minute to set.
   * @param second The second to set.
   * @return The created calendar instance.
   */
  public static final Calendar createCalendar(int year, int month, int day, int hour, 
                                              int minute, int second) {
    Calendar cal = createCalendar(Calendar.SECOND);
    setYear(cal, year);
    setMonth(cal, month);
    setDay(cal, day);
    setHour(cal, hour);
    setMinute(cal, minute);
    setSecond(cal, second);
    return cal;
  }
  
  /**
   * Creates a Calendar instance for the given arguments.
   * @param year The year to set.
   * @param month The month to set.
   * @param day The day to set.
   * @param hour The hour to set.
   * @param minute The minute to set.
   * @param second The second to set.
   * @param millisecond The millisecond to set.
   * @return The created calendar instance.
   */
  public static final Calendar createCalendar(int year, int month, int day, int hour, 
                                              int minute, int second, int millisecond) {
    Calendar cal = createCalendar(Calendar.MILLISECOND);
    setYear(cal, year);
    setMonth(cal, month);
    setDay(cal, day);
    setHour(cal, hour);
    setMinute(cal, minute);
    setSecond(cal, second);
    setMillisecond(cal, millisecond);
    return cal;
  }
  
  /**
   * Creates a Calendar instance for the given date.
   * @param date The date to set for the new calendar.
   * @return The created calendar instance.
   */
  public static final Calendar createCalendar(Date date) {
    Calendar cal = Calendar.getInstance();
    if (date != null) {
      cal.setTime(date);
    }
    return cal;
  }
  
  /**
   * Creates a Calendar instance for the current time.
   * @param precision The ID of the most precise value of interest.
   * @see Calendar#MILLISECOND
   * @see Calendar#SECOND
   * @see Calendar#MINUTE
   * @see Calendar#HOUR
   * @return The created calendar instance with all values more acurate 
   *  than the passed field ID set to minimum.
   */
  public static final Calendar createCalendar(int precision) {
    return createCalendar(null, precision);
  }

  /**
   * Creates a Calendar instance for the given date.
   * @param date The date to set for the new calendar.
   * @param precision The ID of the most precise value of interest.
   * @see Calendar#MILLISECOND
   * @see Calendar#SECOND
   * @see Calendar#MINUTE
   * @see Calendar#HOUR
   * @return The created calendar instance with all values more acurate 
   *  than the passed field ID set to minimum.
   */
  public static final Calendar createCalendar(Date date, int precision) {
    Calendar cal = Calendar.getInstance();
    if (date != null) {
      cal.setTime(date);
    }
    
    if (precision == Calendar.MILLISECOND) {
      return cal;
    }
    cal.set(Calendar.MILLISECOND, cal.getActualMinimum(Calendar.MILLISECOND));

    if (precision == Calendar.SECOND) {
      return cal;
    }
    cal.set(Calendar.SECOND, cal.getActualMinimum(Calendar.SECOND));
    
    if (precision == Calendar.MINUTE) {
      return cal;
    }
    cal.set(Calendar.MINUTE, cal.getActualMinimum(Calendar.MINUTE));

    if ((precision == Calendar.HOUR) || (precision == Calendar.HOUR_OF_DAY)) {
      return cal;
    }
    cal.set(Calendar.HOUR_OF_DAY, cal.getActualMinimum(Calendar.HOUR_OF_DAY));
    
    return cal;
  }
  
  /**
   * Creates a Date instance for the given arguments.
   * @param year The year to set.
   * @param month The month to set.
   * @param day The day to set.
   * @return The created date instance.
   */
  public static final Date createDate(int year, int month, int day) {
    Calendar cal = createCalendar(year, month, day);
    return cal.getTime();
  }
  
  /**
   * Creates a Date instance for the given arguments.
   * @param year The year to set.
   * @param month The month to set.
   * @param day The day to set.
   * @param hour The hour to set.
   * @param minute The minute to set.
   * @param second The second to set.
   * @return The created date instance.
   */
  public static final Date createDate(int year, int month, int day, int hour, 
                                      int minute, int second) {
    Calendar cal = createCalendar(year, month, day, hour, minute, second);
    return cal.getTime();
  }
  
  /**
   * Creates a Date instance for the given arguments.
   * @param year The year to set.
   * @param month The month to set.
   * @param day The day to set.
   * @param hour The hour to set.
   * @param minute The minute to set.
   * @param second The second to set.
   * @param millisecond The millisecond to set.
   * @return The created date instance.
   */
  public static final Date createDate(int year, int month, int day, int hour, 
                                      int minute, int second, int millisecond) {
    Calendar cal = createCalendar(year, month, day, hour, minute, second, millisecond);
    return cal.getTime();
  }

  /**
   * Gets the day of month from the given calendar. 
   * @param cal The calendar to get the current day of month from.
   * @return The determined day of month.
   */
  public static final int getDay(Calendar cal) {
    return cal.get(Calendar.DAY_OF_MONTH);
  }
  /**
   * Gets the day of month from the given date. 
   * @param date The date to get the current day of month from.
   * @return The determined day of month.
   */
  public static final int getDay(Date date) {
    return getDay(createCalendar(date));
  }
  /**
   * Sets the day of month at the given calendar. 
   * @param cal The calendar to set the current day of month at.
   * @param day The day of month to set for the calendar.
   */
  public static final void setDay(Calendar cal, int day) {
    int min = cal.getActualMinimum(Calendar.DAY_OF_MONTH);
    int max = cal.getActualMaximum(Calendar.DAY_OF_MONTH);
    if (day < min || day > max) {
      DateFormat df = new SimpleDateFormat("yyyy-MM-dd");
      throw new RuntimeException("Encountered illegal day value " + day + " for the date " + df.format(cal.getTime()));
    }
    cal.set(Calendar.DAY_OF_MONTH, day);
  }
  /**
   * Sets the day of month at the given date. 
   * @param date The calendar to set the current day of month at.
   * @param day The day of month to set for the date.
   * @return The resulting date.
   */
  public static final Date setDay(Date date, int day) {
    Calendar cal = createCalendar(date);
    setDay(cal, day);
    return cal.getTime();
  }
  /**
   * Increments the given calendar by the given amount of days. 
   * @param cal The calendar to alter.
   * @param days The number of days to add to the calendar.
   */
  public static final void incrDay(Calendar cal, int days) {
    cal.add(Calendar.DAY_OF_MONTH, days);
  }
  /**
   * Increments the given date by the given amount of days. 
   * @param date The date to alter.
   * @param days The number of days to add to the date.
   * @return The resulting date.
   */
  public static final Date incrDay(Date date, int days) {
    Calendar cal = createCalendar(date);
    incrDay(cal, days);
    return cal.getTime();
  }

  /**
   * Gets the month from the given calendar. Since the Calendar's month
   * index is zero based, which feels not very natural, the served 
   * value is incremented by 1 (so: January = 1, February = 2, ...).
   * @param cal The calendar to get the current month from.
   * @return The determined month.
   */
  public static final int getMonth(Calendar cal) {
    return cal.get(Calendar.MONTH) + 1;
  }
  /**
   * Gets the month from the given date. Since the date's month
   * index is zero based, which feels not very natural, the served 
   * value is incremented by 1 (so: January = 1, February = 2, ...).
   * @param date The date to get the current month from.
   * @return The determined month.
   */
  public static final int getMonth(Date date) {
    return getMonth(createCalendar(date));
  }
  /**
   * Sets the month at the given calendar. The expected month index 
   * is 1 based which feels more natural than the Calendar's 
   * values (zero based). 
   * @param cal The calendar to set the current month at.
   * @param month The month to set for the calendar.
   */
  public static final void setMonth(Calendar cal, int month) {
    int calMonth = month - 1;
    int min = cal.getActualMinimum(Calendar.MONTH);
    int max = cal.getActualMaximum(Calendar.MONTH);
    if (calMonth < min || calMonth > max) {
      DateFormat df = new SimpleDateFormat("yyyy-MM-dd");
      throw new RuntimeException("Encountered illegal month value " + month + " for the date " + df.format(cal.getTime()));
    }
    Date prevDate = cal.getTime();
    cal.set(Calendar.MONTH, calMonth);
    if (cal.get(Calendar.MONTH) != calMonth) {
      DateFormat df = new SimpleDateFormat("yyyy-MM-dd");
      throw new RuntimeException("Failed to set month value " + month + " for the date " + df.format(prevDate));
    }
  }
  /**
   * Sets the month at the given date. The expected month index 
   * is 1 based which feels more natural than the date's values (zero based). 
   * @param date The calendar to set the current month at.
   * @param month The month to set to the date.
   * @return The resulting date.
   */
  public static final Date setMonth(Date date, int month) {
    Calendar cal = createCalendar(date);
    setMonth(cal, month);
    return cal.getTime();
  }
  /**
   * Increments the given calendar by the given amount of months. 
   * @param cal The calendar to alter.
   * @param months The number of months to add to the calendar.
   */
  public static final void incrMonth(Calendar cal, int months) {
    cal.add(Calendar.MONTH, months);
  }
  /**
   * Increments the given date by the given amount of months. 
   * @param date The date to alter.
   * @param months The number of months to add to the date.
   * @return The resulting date.
   */
  public static final Date incrMonth(Date date, int months) {
    Calendar cal = createCalendar(date);
    incrMonth(cal, months);
    return cal.getTime();
  }

  /**
   * Gets the year from the given calendar. 
   * @param cal The calendar to get the current year from.
   * @return The determined year.
   */
  public static final int getYear(Calendar cal) {
    return cal.get(Calendar.YEAR);
  }
  /**
   * Gets the year from the given date. 
   * @param date The date to get the current year from.
   * @return The determined year.
   */
  public static final int getYear(Date date) {
    return getYear(createCalendar(date));
  }
  /**
   * Sets the year at the given calendar. 
   * @param cal The calendar to set the current year at.
   * @param year The year to set for the calendar.
   */
  public static final void setYear(Calendar cal, int year) {
    int min = cal.getActualMinimum(Calendar.YEAR);
    int max = cal.getActualMaximum(Calendar.YEAR);
    if (year < min || year > max) {
      DateFormat df = new SimpleDateFormat("yyyy-MM-dd");
      throw new RuntimeException("Encountered illegal year value " + year + " for the date " + df.format(cal.getTime()));
    }
    cal.set(Calendar.YEAR, year);
  }
  /**
   * Sets the year at the given date. 
   * @param date The date to set the current year at.
   * @param year The year to set for the date.
   * @return The resulting date.
   */
  public static final Date setYear(Date date, int year) {
    Calendar cal = createCalendar(date);
    setYear(cal, year);
    return cal.getTime();
  }
  /**
   * Increments the given calendar by the given amount of years. 
   * @param cal The calendar to alter.
   * @param years The number of years to add to the calendar.
   */
  public static final void incrYear(Calendar cal, int years) {
    cal.add(Calendar.YEAR, years);
  }
  /**
   * Increments the given date by the given amount of years. 
   * @param date The calendar to set the current year at.
   * @param year The year to set for the date.
   * @return The resulting date.
   */
  public static final Date incrYear(Date date, int year) {
    Calendar cal = createCalendar(date);
    incrYear(cal, year);
    return cal.getTime();
  }
  
  /**
   * Gets the millisecond from the given calendar. 
   * @param cal The calendar to get the current millisecond from.
   * @return The determined millisecond.
   */
  public static final int getHour(Calendar cal) {
    return cal.get(Calendar.HOUR_OF_DAY);
  }
  /**
   * Gets the millisecond from the given date. 
   * @param date The date to get the current millisecond from.
   * @return The determined millisecond.
   */
  public static final int getHour(Date date) {
    Calendar cal = createCalendar(date);
    return getHour(cal);
  }
  /**
   * Sets the millisecond at the given calendar. 
   * @param cal The calendar to set the current millisecond at.
   * @param hour The hour to set for the calendar.
   */
  public static final void setHour(Calendar cal, int hour) {
    int min = cal.getActualMinimum(Calendar.HOUR_OF_DAY);
    int max = cal.getActualMaximum(Calendar.HOUR_OF_DAY);
    if (hour < min || hour > max) {
      DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
      throw new RuntimeException("Encountered illegal hour value " + hour + " for the date " + df.format(cal.getTime()));
    }
    cal.set(Calendar.HOUR_OF_DAY, hour);
  }
  /**
   * Sets the millisecond at the given date. 
   * @param date The date to set the current millisecond at.
   * @param hour The hour to set for the calendar.
   * @return The resulting date.
   */
  public static final Date setHour(Date date, int hour) {
    Calendar cal = createCalendar(date);
    setHour(cal, hour);
    return cal.getTime();
  }
  /**
   * Increments the given calendar by the given amount of hours. 
   * @param cal The calendar to alter.
   * @param hours The number of hours to add to the calendar.
   */
  public static final void incrHour(Calendar cal, int hours) {
    cal.add(Calendar.HOUR_OF_DAY, hours);
  }
  /**
   * Increments the given date by the given amount of hours. 
   * @param date The date to alter.
   * @param hours The number of hours to add to the calendar.
   * @return The resulting date.
   */
  public static final Date incrHour(Date date, int hours) {
    Calendar cal = createCalendar(date);
    incrHour(cal, hours);
    return cal.getTime();
  }
  
  /**
   * Gets the minute from the given calendar. 
   * @param cal The calendar to get the current minute from.
   * @return The determined minute.
   */
  public static final int getMinute(Calendar cal) {
    return cal.get(Calendar.MINUTE);
  }
  /**
   * Gets the minute from the given date. 
   * @param date The date to get the current minute from.
   * @return The determined minute.
   */
  public static final int getMinute(Date date) {
    Calendar cal = createCalendar(date);
    return getMinute(cal);
  }
  /**
   * Sets the minute at the given calendar. 
   * @param cal The calendar to set the current minute at.
   * @param minute The minute to set for the calendar.
   */
  public static final void setMinute(Calendar cal, int minute) {
    int min = cal.getActualMinimum(Calendar.MINUTE);
    int max = cal.getActualMaximum(Calendar.MINUTE);
    if (minute < min || minute > max) {
      DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
      throw new RuntimeException("Encountered illegal minute value " + minute + " for the date " + df.format(cal.getTime()));
    }
    cal.set(Calendar.MINUTE, minute);
  }
  /**
   * Sets the minute at the given date. 
   * @param date The date to set the current minute at.
   * @param minute The minute to set for the date.
   * @return The resulting date.
   */
  public static final Date setMinute(Date date, int minute) {
    Calendar cal = createCalendar(date);
    setMinute(cal, minute);
    return cal.getTime();
  }
  /**
   * Increments the given calendar by the given amount of minutes. 
   * @param cal The calendar to alter.
   * @param minutes The number of minutes to add to the calendar.
   */
  public static final void incrMinute(Calendar cal, int minutes) {
    cal.add(Calendar.MINUTE, minutes);
  }
  /**
   * Increments the given date by the given amount of minutes. 
   * @param date The date to alter.
   * @param minutes The number of minutes to add to the date.
   * @return The resulting date.
   */
  public static final Date incrMinute(Date date, int minutes) {
    Calendar cal = createCalendar(date);
    incrMinute(cal, minutes);
    return cal.getTime();
  }
  
  /**
   * Gets the second from the given calendar. 
   * @param cal The calendar to get the current second from.
   * @return The determined second.
   */
  public static final int getSecond(Calendar cal) {
    return cal.get(Calendar.SECOND);
  }
  /**
   * Gets the second from the given date. 
   * @param date The date to get the current second from.
   * @return The determined second.
   */
  public static final int getSecond(Date date) {
    Calendar cal = createCalendar(date);
    return getSecond(cal);
  }
  /**
   * Sets the second at the given calendar. 
   * @param cal The calendar to set the current second at.
   * @param second The second to set for the calendar.
   */
  public static final void setSecond(Calendar cal, int second) {
    int min = cal.getActualMinimum(Calendar.SECOND);
    int max = cal.getActualMaximum(Calendar.SECOND);
    if (second < min || second > max) {
      DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
      throw new RuntimeException("Encountered illegal second value " + second + " for the date " + df.format(cal.getTime()));
    }
    cal.set(Calendar.SECOND, second);
  }
  /**
   * Sets the second at the given date. 
   * @param date The date to set the current second at.
   * @param second The second to set for the date.
   * @return The resulting date.
   */
  public static final Date setSecond(Date date, int second) {
    Calendar cal = createCalendar(date);
    setSecond(cal, second);
    return cal.getTime();
  }
  /**
   * Increments the given calendar by the given amount of seconds. 
   * @param cal The calendar to alter.
   * @param seconds The number of seconds to add to the calendar.
   */
  public static final void incrSecond(Calendar cal, int seconds) {
    cal.add(Calendar.SECOND, seconds);
  }
  /**
   * Increments the given date by the given amount of seconds. 
   * @param date The date to alter.
   * @param seconds The number of seconds to add to the calendar.
   * @return The resulting date.
   */
  public static final Date incrSecond(Date date, int seconds) {
    Calendar cal = createCalendar(date);
    incrSecond(cal, seconds);
    return cal.getTime();
  }

  /**
   * Gets the millisecond from the given calendar. 
   * @param cal The calendar to get the current millisecond from.
   * @return The determined millisecond.
   */
  public static final int getMillisecond(Calendar cal) {
    return cal.get(Calendar.MILLISECOND);
  }
  /**
   * Gets the millisecond from the given calendar. 
   * @param date The date to get the current millisecond from.
   * @return The determined millisecond.
   */
  public static final int getMillisecond(Date date) {
    Calendar cal = createCalendar(date);
    return getMillisecond(cal);
  }
  /**
   * Sets the millisecond at the given calendar. 
   * @param cal The calendar to set the current millisecond at.
   * @param millisecond The millisecond to set for the calendar.
   */
  public static final void setMillisecond(Calendar cal, int millisecond) {
    int min = cal.getActualMinimum(Calendar.MILLISECOND);
    int max = cal.getActualMaximum(Calendar.MILLISECOND);
    if (millisecond < min || millisecond > max) {
      DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
      throw new RuntimeException("Encountered illegal millisecond value " + millisecond + " for the date " + df.format(cal.getTime()));
    }
    cal.set(Calendar.MILLISECOND, millisecond);
  }
  /**
   * Sets the millisecond at the given calendar. 
   * @param date The date to set the current millisecond at.
   * @param millisecond The millisecond to set for the calendar.
   * @return The resulting date.
   */
  public static final Date setMillisecond(Date date, int millisecond) {
    Calendar cal = createCalendar(date);
    setMillisecond(cal, millisecond);
    return cal.getTime();
  }
  /**
   * Increments the given calendar by the given amount of milliseconds. 
   * @param cal The calendar to alter.
   * @param milliseconds The number of milliseconds to add to the calendar.
   */
  public static final void incrMillisecond(Calendar cal, int milliseconds) {
    cal.add(Calendar.MILLISECOND, milliseconds);
  }
  /**
   * Increments the given calendar by the given amount of milliseconds. 
   * @param date The date to alter.
   * @param milliseconds The number of milliseconds to add to the calendar.
   * @return The resulting date.
   */
  public static final Date incrMillisecond(Date date, int milliseconds) {
    Calendar cal = createCalendar(date);
    incrMillisecond(cal, milliseconds);
    return cal.getTime();
  }
  
  /**
   * Gets the minimum of the given two date values. 
   * @param a The first date.
   * @param b The second date.
   * @return The minimum of a and b.
   */
  public static final Date min(Date a, Date b) {
    if ((a == null) || (b == null)) {
      return null;
    }
    if (a.after(b)) {
      return b;
    }
    return a;
  }

  /**
   * Gets the minimum of the given two date values. 
   * @param a The first date.
   * @param b The second date.
   * @return The minimum of a and b.
   */
  public static final Date max(Date a, Date b) {
    if (a == null) {
      return b;
    }
    if (b == null) {
      return a;
    }
    if (a.after(b)) {
      return b;
    }
    return a;
  }
  
  /**
   * Gets the begin of year for the given date.
   * @param d The date to process.
   * @return The january 1st in the year of the given date.
   */
  public static final Date beginOfYear(Date d) {
    Calendar cal = createCalendar(d, Calendar.DATE);
    // determining begin of year for the given date
    cal.set(Calendar.DAY_OF_MONTH, 1);
    cal.set(Calendar.MONTH, Calendar.JANUARY);
    return cal.getTime();
  }
  
  /**
   * Gets the end of year for the given date.
   * @param d The date to process.
   * @return The january 1st in the year of the given date.
   */
  public static final Date endOfYear(Date d) {
    Calendar cal = createCalendar(d, Calendar.DATE);
    // determining end of year for the given date
    cal.set(Calendar.MONTH, Calendar.DECEMBER);
    cal.set(Calendar.DAY_OF_MONTH, cal.getActualMaximum(Calendar.DAY_OF_MONTH));
    return cal.getTime();
  }
  
  /**
   * Gets the previous ultimo for the given date.
   * @param d The date to process.
   * @return In case the given date equals to the end of a month this date is returned 
   *    else the end of the previous month is returned.
   */
  public static final Date previousUltimo(Date d) {
    Calendar cal = createCalendar(d, Calendar.DATE);
    // determining previous ultimo
    if (cal.get(Calendar.DAY_OF_MONTH) == cal.getActualMaximum(Calendar.DAY_OF_MONTH)) {
      return cal.getTime();
    }
    cal.set(Calendar.DAY_OF_MONTH, 1);
    cal.add(Calendar.MONTH, -1);
    cal.set(Calendar.DAY_OF_MONTH, cal.getActualMaximum(Calendar.DAY_OF_MONTH));
    return cal.getTime();
  }
  
  /**
   * Gets the next ultimo for the given date.
   * @param d The date to process.
   * @return Always returns the end of the current month.
   */
  public static final Date nextUltimo(Date d) {
    Calendar cal = createCalendar(d, Calendar.DATE);
    // determining previous ultimo
    cal.set(Calendar.DAY_OF_MONTH, cal.getActualMaximum(Calendar.DAY_OF_MONTH));
    return cal.getTime();
  }

}
