/*
 * Copyright (C) 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util;

import java.sql.Time;
import java.sql.Timestamp;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.ujac.util.text.FormatHelper;

import org.ujac.util.types.BooleanType;
import org.ujac.util.types.DataType;
import org.ujac.util.types.DateType;
import org.ujac.util.types.DoubleType;
import org.ujac.util.types.FloatType;
import org.ujac.util.types.IntegerType;
import org.ujac.util.types.LongType;
import org.ujac.util.types.ShortType;
import org.ujac.util.types.StringType;
import org.ujac.util.types.TimeType;
import org.ujac.util.types.TimestampType;

/**
 * Name: DefaultTypeConverter<br>
 * Description: The default type converter implementation.
 * 
 * @author lauerc
 */
public class DefaultTypeConverter implements TypeConverter {

  /** The data type registry. */
  private Map dataTypeRegistry = new HashMap(); 
  /** The type ID map. */
  private DataType[] typeIdMap = new DataType[UjacTypes.TYPE_OBJECT + 1]; 
  /** The format helper to use. */
  private FormatHelper formatHelper = null;
  
  /**
   * Constructs a DefaultTypeConverter instance with no specific attributes.
   */
  public DefaultTypeConverter() {
    this(FormatHelper.createInstance());
  }

  /**
   * Constructs a DefaultTypeConverter instance with specific attributes.
   * @param formatHelper The format helper to use.
   */
  public DefaultTypeConverter(FormatHelper formatHelper) {
    this.formatHelper = formatHelper;
    registerDataTypes();
  }

  /**
   * Registers the data types.
   */
  protected void registerDataTypes() {
    registerDataType(String.class, new StringType());

    DataType dataType = new BooleanType();
    registerDataType(Boolean.class, dataType);
    registerDataType(boolean.class, dataType);

    dataType = new IntegerType();
    registerDataType(Integer.class, dataType);
    registerDataType(int.class, dataType);
    
    dataType = new LongType();
    registerDataType(Long.class, dataType);
    registerDataType(long.class, dataType);
    
    dataType = new ShortType();
    registerDataType(Short.class, dataType);
    registerDataType(short.class, dataType);
    
    dataType = new DoubleType();
    registerDataType(Double.class, dataType);
    registerDataType(double.class, dataType);

    dataType = new FloatType();
    registerDataType(Float.class, dataType);
    registerDataType(float.class, dataType);

    registerDataType(Date.class, new DateType());
    registerDataType(Time.class, new TimeType());
    registerDataType(Timestamp.class, new TimestampType());
  }

  /**
   * Registers the data type.
   * @param clazz The type to handle.
   * @param typeHandler The type handler.
   */
  public void registerDataType(Class clazz, DataType typeHandler) {
    dataTypeRegistry.put(clazz, typeHandler);
    typeIdMap[typeHandler.getTypeId()] = typeHandler;
  }

  /**
   * Gets the data type for the given class.
   * @param clazz The type to get the handler for.
   * @return The registered type handler.
   */
  public DataType getDataType(Class clazz) {
    return (DataType) dataTypeRegistry.get(clazz);
  }

  /**
   * Gets the data type for the given type ID.
   * @param typeId The ID of the type to get the handler for.
   * @return The registered type handler.
   */
  public DataType getDataType(int typeId) {
    return typeIdMap[typeId];
  }

  /**
   * Converts the given value into an instance of this type.
   * @param clazz The type class.
   * @param value The value to convert.
   * @return The according instance.
   * @exception TypeConverterException In case the conversion failed.
   */
  public Object convertObject(Class clazz, Object value)
    throws TypeConverterException {
    
    DataType typeHandler = getDataType(clazz);
    if (typeHandler == null) {
      if (clazz.isInstance(value)) {
        return value;
      }
      throw new TypeConverterException("No type handler registerd for class " + clazz.getName());
    }
    return typeHandler.convertObject(value, formatHelper);
  }

  /**
   * Converts the given value into an instance of this type.
   * @param typeId The type ID.
   * @param value The value to convert.
   * @return The according instance.
   * @exception TypeConverterException In case the conversion failed.
   */
  public Object convertObject(int typeId, Object value)
    throws TypeConverterException {
    
    DataType typeHandler = getDataType(typeId);
    if (typeHandler == null) {
      throw new TypeConverterException("No type handler registerd for type ID " + typeId);
    }
    return typeHandler.convertObject(value, formatHelper);
  }

  /**
   * FormatsConverts the given value into an instance of this type.
   * @param clazz The type class.
   * @param value The value to convert. 
   * @return The formatted value.
   * @exception TypeConverterException In case the conversion failed.
   */
  public String formatValue(Class clazz, Object value)
    throws TypeConverterException {
    
    if (value == null) {
      return null;
    }
    
    DataType typeHandler = getDataType(clazz);
    if (typeHandler == null) {
      throw new TypeConverterException("No type handler registerd for class " + clazz.getName());
    }
    return typeHandler.formatValue(value, formatHelper);
  }

  /**
   * FormatsConverts the given value into an instance of this type.
   * @param typeId The type ID.
   * @param value The value to convert. 
   * @return The formatted value.
   * @exception TypeConverterException In case the conversion failed.
   */
  public String formatValue(int typeId, Object value)
    throws TypeConverterException {
    
    if (value == null) {
      return null;
    }
    
    DataType typeHandler = getDataType(typeId);
    if (typeHandler == null) {
      throw new TypeConverterException("No type handler registerd for type ID " + typeId);
    }
    return typeHandler.formatValue(value, formatHelper);
  }
}
