/*
 * Copyright (C) 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util;

/**
 * Name: StringUtils<br>
 * Description: Class providing common utility methods for String operations.
 * 
 * @author lauerc
 */
public class StringUtils {

  /**
   * Checks if the given string is null or has zero length.
   * @param value The string to check.
   * @return true if the given string is null or has zero length.
   */
  public static final boolean isEmpty(String value) {
    return value == null || value.length() == 0;
  }
  
  /**
   * Gets the length of the given String.
   * @param text The String to get the length from.
   * @return The length of the given String.
   */
  public static final int getLength(String text) {
    if (text == null) {
      return 0;
    }
    return text.length();
  }
  
  /**
   * Gets the upper case String from the given String.
   * @param text The String to get the upper case String from.
   * @return The upper case string of the given String or null.
   */
  public static final String getUpperCase(String text) {
    if (text == null) {
      return null;
    }
    return text.toUpperCase();
  }
  
  /**
   * Gets the lower case String from the given String.
   * @param text The String to get the lower case String from.
   * @return The lower case string of the given String or null.
   */
  public static final String getLowerCase(String text) {
    if (text == null) {
      return null;
    }
    return text.toLowerCase();
  }

  /**
   * Raises the first character of the given String.
   * @param text The String to capitalize.
   * @return The capitalized String or null.
   */
  public static final String capitalize(String text) {
    if (isEmpty(text)) {
      return text;
    }
    int len = text.length();
    if (len == 1) {
      return text.toUpperCase();
    }
    return new StringBuffer(len).append(Character.toUpperCase(text.charAt(0))).append(text.substring(1)).toString();
  }
  
  /**
   * Checks if the given objects are equal.
   * @param a The object to compare with object b.
   * @param b The object to compare with object a.
   * @return true if the objects are equal or are both null.
   */
  public static final boolean equals(String a, String b) {
    return BeanUtils.equals(a, b);
  }
  /**
   * Checks if the given objects are equal.
   * @param a The object to compare with object b.
   * @param b The object to compare with object a.
   * @param caseSensitive Tells whether or not to compare string values case sensitive.
   * @return true if the objects are equal or are both null.
   */
  public static final boolean equals(String a, String b, boolean caseSensitive) {
    if ((a == null) || (b == null)) {
      return a == b;
    }
    if (caseSensitive) {
      return a.equals(b);
    }
    return a.equalsIgnoreCase(b);
  }
  
  /**
   * Checks if the given objects are likewise.
   * @param a The object to compare with object b.
   * @param b The object to compare with object a.
   * @return true if a is like b or are both null.
   */
  public static final boolean like(String a, String b) {
    if ((a == null) && (b == null)) {
      return true;
    }
    if (a == null) {
      return false;
    }
    if (b == null) {
      return false;
    }
    int operandLength = b.length();
    if (operandLength < 1) {
      return false;
    }
    if (b.charAt(0) == '%') {
      if (b.charAt(operandLength - 1) == '%') {
        String compareText = b.substring(1, operandLength - 1);
        return a.indexOf(compareText) >= 0;
      }
      return a.endsWith(b.substring(1));
    } else if (b.charAt(operandLength - 1) == '%') {
      String compareText = b.substring(0, operandLength - 1);
      return a.startsWith(compareText);
    }
    // equal check
    return a.equals(b);
  }
  /**
   * Checks if the given objects are likewise.
   * @param a The object to compare with object b.
   * @param b The object to compare with object a.
   * @param caseSensitive Tells whether or not to compare string values case sensitive.
   * @return true if a is like b or are both null.
   */
  public static final boolean like(String a, String b, boolean caseSensitive) {
    if ((a == null) && (b == null)) {
      return true;
    }
    if (a == null) {
      return false;
    }
    if (b == null) {
      return false;
    }
    int operandLength = b.length();
    if (operandLength < 1) {
      return false;
    }
    if (b.charAt(0) == '%') {
      if (b.charAt(operandLength - 1) == '%') {
        String compareText = b.substring(1, operandLength - 1);
        if (caseSensitive) {
          return a.indexOf(compareText) >= 0;
        }
        compareText = compareText.toLowerCase();
        return a.toLowerCase().indexOf(compareText) >= 0;
      }
      if (caseSensitive) {
        return a.endsWith(b.substring(1));
      }
      return a.toLowerCase().endsWith(b.substring(1).toLowerCase());
    } else if (b.charAt(operandLength - 1) == '%') {
      String compareText = b.substring(0, operandLength - 1);
      if (caseSensitive) {
        return a.startsWith(compareText);
      }
      compareText = compareText.toLowerCase();
      return a.toLowerCase().startsWith(compareText);
    }
    // equal check
    if (caseSensitive) {
      return a.equals(b);
    }
    return a.equalsIgnoreCase(b);
  }

  /**
   * Trims off the heading and trailing white spaces from the given string.
   * @param string the string to trim.
   * @return The trimmed string.
   */
  public static final String trim(String string) {
    if (string == null) {
      // no sense to go on!
      return null;
    }
    return string.trim();
  }

  /**
   * Trims off the heading white spaces from the given string.
   * @param string the string to trim.
   * @return The trimmed string.
   */
  public static final String trimHead(String string) {
    if (string == null) {
      // no sense to go on!
      return null;
    }
    int offset = 0;
    int length = string.length();
    for (int i = 0; i < length; i++) {
      if (!Character.isWhitespace(string.charAt(i))) {
        if (i > 0) {
          offset = i;
        }
        break;
      }
    }
    if (offset == 0) {
      return string;
    }
    return string.substring(offset, length);
  }

  /**
   * Trims off the trailing white spaces from the content buffer.
   * @param string the string to trim.
   * @return The trimmed string.
   */
  public static final String trimTail(String string) {
    if (string == null) {
      // no sense to go on!
      return string;
    }
    int length = string.length();
    while ((length > 0) && (Character.isWhitespace(string.charAt(length - 1)))) {
      --length;
    }
    if (length == string.length()) {
      return string;
    }
    return string.substring(0, length);
  }

}
