/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.db;

import java.sql.Types;

/**
 * Name: AttributeDefinition<br>
 * Description: A class holding the definition of a class attribute.<br>
 * 
 * @author lauerc
 */
public class AttributeDefinition {

  /** The attribute name. */
  private String name;
  
  /** The attribute type. */
  private String type = null;
  /** The primitive type name. */
  private String primitiveType = null;
  /** The fully qualified type name. */
  private String fullyQualifiedType = null;
  
  /** The name of the attribute's getter method. */
  private String getter;
  /** The name of the attribute's setter method. */
  private String setter;

  /** The column name. */
  private String columnName;
  /** The SQL type. */
  private int sqlType;
  /** The primary key flag. */
  private boolean primaryKey;
  /** The foreign key flag. */
  private boolean foreignKey;
  
  /**
   * Constructs an AttributeDefinition instance with specific attributes.
   * @param name The attribute name.
   * @param type The attribute type.
   */
  public AttributeDefinition(String name, String type) {
    setName(name);
    setType(type);
  }

  /**
   * Constructs an AttributeDefinition instance with specific attributes.
   * @param name The attribute name.
   * @param columnName The column name.
   * @param sqlType The sql type.
   * @param columnSize The size of the SQL type.
   * @param decimalDigits The number of decimal digits.
   * @param primaryKey The primary key flag.
   * @param foreignKey The foreign key flag.
   * @param dataTypeMapper The data type mapper.
   */
  public AttributeDefinition(String name, String columnName, int sqlType,
                             int columnSize, int decimalDigits,
                             boolean primaryKey, boolean foreignKey, 
                             DataTypeMapper dataTypeMapper) {
    setName(name);
    setType(dataTypeMapper.getJavaTypeName(sqlType, columnSize, decimalDigits, false, false));
    setPrimitiveType(dataTypeMapper.getJavaTypeName(sqlType, columnSize, decimalDigits, true, false));
    setFullyQualifiedType(dataTypeMapper.getJavaTypeName(sqlType, columnSize, decimalDigits, false, true));
    
    this.columnName = columnName;
    this.sqlType = sqlType;
    this.primaryKey = primaryKey;
    this.foreignKey = foreignKey;
  }
  
  
  /**
   * Gets the current attribute name.
   * @return The attribute name.
   */
  public String getName() {
    return name;
  }
  /**
   * Sets the attribute name.
   * @param name The new attribute name.
   */
  private void setName(String name) {
    this.name = name;
    if ((name != null) && (name.length() > 0)) {
      this.getter = new StringBuffer("get").append(Character.toUpperCase(name.charAt(0))).append(name.substring(1)).toString();
      this.setter = new StringBuffer("set").append(Character.toUpperCase(name.charAt(0))).append(name.substring(1)).toString();
    }
  }

  
  /**
   * Gets the current attribute type.
   * @return The attribute type.
   */
  public String getType() {
    return type;
  }
  /**
   * Sets the attribute name type.
   * @param type The new attribute type.
   */
  public void setType(String type) {
    this.type = type;
  }

  /**
   * Getter method for the the property fullyQualifiedType.
   * @return The current value of property fullyQualifiedType.
   */
  public String getFullyQualifiedType() {
    return fullyQualifiedType;
  }
  /**
   * Setter method for the the property fullyQualifiedType.
   * @param fullyQualifiedType The value to set for the property fullyQualifiedType.
   */
  public void setFullyQualifiedType(String fullyQualifiedType) {
    this.fullyQualifiedType = fullyQualifiedType;
  }

  /**
   * Getter method for the the property primitiveType.
   * @return The current value of property primitiveType.
   */
  public String getPrimitiveType() {
    return primitiveType;
  }
  /**
   * Setter method for the the property primitiveType.
   * @param primitiveType The value to set for the property primitiveType.
   */
  public void setPrimitiveType(String primitiveType) {
    this.primitiveType = primitiveType;
  }

  
  /**
   * Gets the name of the getter method for the attribute.
   * @return The getter method's name.
   */
  public String getGetter() {
    return getter;
  }

  /**
   * Gets the name of the setter method for the attribute.
   * @return The setter method's name.
   */
  public String getSetter() {
    return setter;
  }

  /**
   * Gets the attribute's column name.
   * @return The column name.
   */
  public String getColumnName() {
    return columnName;
  }

  /**
   * Gets the attribute's SQL type.
   * @return The SQL type.
   */
  public int getSqlType() {
    return sqlType;
  }
  /**
   * Gets the attribute's SQL type name.
   * @return The SQL type name.
   */
  public String getSqlTypeName() {
    return getSqlTypeName(sqlType);
  }
  /**
   * Is the attribute a primary key?
   * @return The primary key flag.
   */
  public boolean isPrimaryKey() {
    return primaryKey;
  }

  /**
   * Is the attribute a foreign key?
   * @return The foreign key flag.
   */
  public boolean isForeignKey() {
    return foreignKey;
  }

  /**
   * Gets the name of the given SQL type
   * @param sqlType The SQL type ID from java.sql.Types.
   * @return The SQL type name.
   */
  public static final String getSqlTypeName(int sqlType) {
    switch (sqlType) {
      case Types.ARRAY: return "ARRAY";
      case Types.BIGINT: return "BIGINT";
      case Types.BINARY: return "BINARY";
      case Types.BIT: return "BIT";
      case Types.CHAR: return "CHAR";
      case Types.CLOB: return "CLOB";
      //case Types.DATALINK: return "DATALINK";
      case Types.DATE: return "DATE";
      case Types.DECIMAL: return "DECIMAL";
      case Types.DISTINCT: return "DISTINCT";
      case Types.DOUBLE: return "DOUBLE";
      case Types.FLOAT: return "FLOAT";
      case Types.INTEGER: return "INTEGER";
      case Types.JAVA_OBJECT: return "JAVA_OBJECT";
      case Types.LONGVARBINARY: return "LONGVARBINARY";
      case Types.LONGVARCHAR: return "LONGVARCHAR";
      case Types.NUMERIC: return "NUMERIC";
      case Types.OTHER: return "OTHER";
      case Types.REAL: return "REAL";
      case Types.REF: return "REF";
      case Types.SMALLINT: return "SMALLINT";
      case Types.STRUCT: return "STRUCT";
      case Types.TIME: return "TIME";
      case Types.TIMESTAMP: return "TIMESTAMP";
      case Types.TINYINT: return "TINYINT";
      case Types.VARBINARY: return "VARBINARY";
      case Types.VARCHAR: return "VARCHAR";
      default: return null;
    }
  }

  /**
   * Returns a string representation of the object. 
   * @return  a string representation of the object.
   */
  public String toString() {
    StringBuffer buf = new StringBuffer("{");
    buf.append("java name: ")
      .append(name)
      .append(", java type: ")
      .append(type)
      .append(", column name: ")
      .append(columnName)
      .append(", SQL type: ")
      .append(getSqlTypeName(sqlType))
      .append(", is primary key: ")
      .append(primaryKey)
      .append(", is foreign key: ")
      .append(foreignKey)
    	.append("}");
    return buf.toString();
  }

}
