/*
 * Copyright (C) 2003, 2004, 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.db;

import java.sql.ResultSet;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.DriverManager;
import java.sql.DatabaseMetaData;

import java.util.Set;
import java.util.HashSet;
import java.util.StringTokenizer;


/**
 * Name: DatabaseSchemaReader<br>
 * Description: Reads informations from database schema using DatabaseMetaData.<br>
 * 
 * @author lauerc
 */
public class DatabaseSchemaReader {

  /** The database connection. */
  private Connection cxn = null;
  /** The driver class name. */
  private String driver;
  /** The JDBC URL. */
  private String url;
  /** The database user. */
  private String user;
  /** The user's password. */
  private String passwd;
  /** The data type mapper. */
  private DataTypeMapper dataTypeMapper = null;
  
  /**
   * Constructs a DatabaseSchemaReader with specific attributes.
   * @param cxn  The database connection to use.
   */
  public DatabaseSchemaReader(Connection cxn) {
    this(cxn, new DefaultDataTypeMapper());
  }
  
  /**
   * Constructs a DatabaseSchemaReader with specific attributes.
   * @param cxn  The database connection to use.
   * @param dataTypeMapper The data type mapper to use.
   */
  public DatabaseSchemaReader(Connection cxn, DataTypeMapper dataTypeMapper) {
    this.cxn = cxn;
    this.dataTypeMapper = dataTypeMapper;
   }

  /**
   * Constructs a DatabaseSchemaReader with specific attributes.
   * @param driver  The driver class name.
   * @param url  The JDBC URL.
   * @param user  The database user.
   * @param passwd  The user's password.
   */
  public DatabaseSchemaReader(String driver, String url, String user, String passwd) {
    this(driver, url, user, passwd, new DefaultDataTypeMapper());
  }

  /**
   * Constructs a DatabaseSchemaReader with specific attributes.
   * @param driver  The driver class name.
   * @param url  The JDBC URL.
   * @param user  The database user.
   * @param passwd  The user's password.
   * @param dataTypeMapper The data type mapper to use.
   */
  public DatabaseSchemaReader(String driver, String url, String user, String passwd, 
                              DataTypeMapper dataTypeMapper) {
    this.driver = driver;
    this.url = url;
    this.user = user;
    this.passwd = passwd;
    this.dataTypeMapper = dataTypeMapper;
  }

  /**
   * Gets the definition for the given table name.
   * @param tableName  The name of the desired table.
   * @return  The definition of the desired table.
   * @exception SQLException In case a JDBC problem occurred.
   */
  public TableDefinition getTableDefinition(String tableName) throws SQLException {
    
    TableDefinition tableDef = new TableDefinition(tableName);
    
    boolean maintainConnection = false;
    if (cxn == null) {
      maintainConnection = true;
      cxn = getConnection();
    }
    try {
      DatabaseMetaData meta = cxn.getMetaData();
      String catalog = cxn.getCatalog();
      Set primaryKeySet = new HashSet();
      ResultSet primaryKeys = meta.getPrimaryKeys(catalog, tableDef.getSchema(), tableDef.getName());
      while (primaryKeys.next()) {
        String columnName = primaryKeys.getString(4);
        primaryKeySet.add(columnName);
      }
      primaryKeys.close();
      
      Set foreignKeySet = new HashSet();
      ResultSet foreignKeys = meta.getImportedKeys(catalog, tableDef.getSchema(), tableDef.getName());
      while (foreignKeys.next()) {
        String columnName = foreignKeys.getString(8);
        foreignKeySet.add(columnName);
      }
      foreignKeys.close();
      
      ResultSet columns = meta.getColumns(catalog, tableDef.getSchema(), tableDef.getName(), "%");
      while (columns.next()) {
        String columnName = columns.getString(4);
        int columnType = columns.getInt(5);
        int columnSize = columns.getInt(7);
        int decimalDigits = columns.getInt(9);
        String javaName = getJavaName(columnName);
        tableDef.addAttribute(new AttributeDefinition(javaName, columnName, columnType,
                                                       columnSize, decimalDigits,
                                                       primaryKeySet.contains(columnName),
                                                       foreignKeySet.contains(columnName),
                                                       dataTypeMapper));
      }
      columns.close();
    } finally {
      if (maintainConnection) {
        cleanup(cxn);
        cxn = null;
      }
    }
    return tableDef;
  }

  /**
   * Gets the java attribute name for the given column name.
   * @param columnName The column name.
   * @return The java attribute name.
   */
  public static final String getJavaName(String columnName) {
    StringTokenizer tkn = new StringTokenizer(columnName.toLowerCase(), "_", true);
    StringBuffer nameBuf = new StringBuffer();
    while (tkn.hasMoreTokens()) {
      String token = tkn.nextToken();
      if (token.charAt(0) == '_') {
        if (tkn.hasMoreTokens()) {
          token = tkn.nextToken();
          if (nameBuf.length() > 0) {
            nameBuf.append(Character.toUpperCase(token.charAt(0))).append(token.substring(1));
          } else {
            nameBuf.append(token);
          }
        }
        continue;
      }
      nameBuf.append(token);
    }
    return nameBuf.toString();
  }

  /**
   * Gets the JDBC connection.
   * @return The JDBC connection.
     * @exception SQLException In case a JDBC problem occurred.
   */
  private Connection getConnection() throws SQLException {
    try {
      Class.forName(driver);
    } catch (ClassNotFoundException e) {
      throw new SQLException("The driver class '" + driver + "' could not be found in the class path.");
    }
    return DriverManager.getConnection(url, user, passwd);
  }

  /**
   * Cleans up the JDBC stuff.
   * @param cxn The JDBC conntection.
   */
  private void cleanup(Connection cxn) {
    try {
      if (cxn != null) {
        cxn.close();
      }
    } catch (SQLException e) {
      e.printStackTrace();
    }
  }

  /**
   * The main method for testing purposes.
   * @param args The command line arguments.
   */
  public static void main(String[] args) {
    if (args.length < 5) {
      System.err.println("Usage: org.ujac.util.codegen.DatabaseSchemaReader <driver> <url> <user> <passwd> <table name>");
      System.exit(1);
    }

    String driver = args[0];
    String url = args[1];
    String user = args[2];
    String passwd = args[3];
    String tableName = args[4];
    
    DatabaseSchemaReader reader = new DatabaseSchemaReader(driver, url, user, passwd);
    try {
      TableDefinition tableDef = reader.getTableDefinition(tableName);
      System.out.println(tableDef);
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }
}
