/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.db;

import java.sql.Types;

/**
 * Name: DefaultDataTypeMapper<br>
 * Description: Default implementation of the interface DataTypeMapper.
 * 
 * @author lauerc
 */
public class DefaultDataTypeMapper implements DataTypeMapper {

  /**
   * Gets the JAVA type of the given SQL type.
   * @param sqlType The SQL type ID from java.sql.Types.
   * @param columnSize The size of the SQL type.
   * @param decimalDigits The number of decimal digits.
   * @return The SQL type name.
   * @see #JAVA_TYPE_OBJECT
   * @see #JAVA_TYPE_INT
   * @see #JAVA_TYPE_LONG
   * @see #JAVA_TYPE_FLOAT
   * @see #JAVA_TYPE_DOUBLE
   * @see #JAVA_TYPE_BOOLEAN
   * @see #JAVA_TYPE_STRING
   * @see #JAVA_TYPE_DATE
   * @see #JAVA_TYPE_TIME
   * @see #JAVA_TYPE_TIMESTAMP
   */
  public int getJavaType(int sqlType, int columnSize, int decimalDigits) {
    switch (sqlType) {
      case Types.BIGINT: return JAVA_TYPE_LONG;
      case Types.CHAR: return JAVA_TYPE_STRING;
      case Types.CLOB: return JAVA_TYPE_OBJECT;
      case Types.DATE: return JAVA_TYPE_DATE;
      case Types.DECIMAL: 
        if (columnSize > 0) {
          return JAVA_TYPE_DOUBLE;
        }
        if (decimalDigits > 9) {
          return JAVA_TYPE_LONG;
        }
        return JAVA_TYPE_INT;

      case Types.DOUBLE: return JAVA_TYPE_DOUBLE;
      case Types.FLOAT: return JAVA_TYPE_FLOAT;
      case Types.INTEGER: return JAVA_TYPE_INT;
      case Types.JAVA_OBJECT: return JAVA_TYPE_OBJECT;
      case Types.LONGVARCHAR: return JAVA_TYPE_STRING;
      case Types.NUMERIC: 
        if (decimalDigits > 9) {
          return JAVA_TYPE_LONG;
        }
        return JAVA_TYPE_INT;
      case Types.REAL: return JAVA_TYPE_FLOAT;
      case Types.SMALLINT: return JAVA_TYPE_INT;

      case Types.TIME: return JAVA_TYPE_TIME;
      case Types.TIMESTAMP: return JAVA_TYPE_TIMESTAMP;
      case Types.TINYINT: return JAVA_TYPE_INT;
      case Types.VARCHAR: return JAVA_TYPE_STRING;
   
      default:
        throw new RuntimeException("Not supported type: " + getSqlTypeName(sqlType));
    }
  }
  
  /**
   * Gets the name of the given SQL type
   * @param sqlType The SQL type ID from java.sql.Types.
   * @param columnSize The size of the SQL type.
   * @param decimalDigits The number of decimal digits.
   * @param allowPrimitive If true, uses the simple types (int, long, float, double, ...) for
   *   numeric values, otherwise returns the wrapper classed for these types.
   * @param fullyQualified If true, returns the fully qualified class names for system classes, 
   *   else returns the system class names without java.lang package prefix. 
   * @return The SQL type name.
   */
  public String getJavaTypeName(int sqlType, int columnSize, int decimalDigits, 
                                boolean allowPrimitive, boolean fullyQualified) {
    switch (sqlType) {
      case Types.BIGINT: return getLongType(allowPrimitive, fullyQualified);
      case Types.CHAR: return getStringType(fullyQualified);
      case Types.CLOB: return getStringType(fullyQualified);
      case Types.DATE: return "java.sql.Date";
      case Types.DECIMAL: 
        if (columnSize > 0) {
          return getDoubleType(allowPrimitive, fullyQualified);
        }
        if (decimalDigits > 9) {
          return getLongType(allowPrimitive, fullyQualified);
        }
        return getIntType(allowPrimitive, fullyQualified);
      
      case Types.DOUBLE: return getDoubleType(allowPrimitive, fullyQualified);
      case Types.FLOAT: return getFloatType(allowPrimitive, fullyQualified);
      case Types.INTEGER: return getIntType(allowPrimitive, fullyQualified);
      case Types.JAVA_OBJECT: return getObjectType(fullyQualified);
      case Types.LONGVARCHAR: return getStringType(fullyQualified);
      case Types.NUMERIC: 
        if (decimalDigits > 9) {
          return getLongType(allowPrimitive, fullyQualified);
        }
        return getIntType(allowPrimitive, fullyQualified);

      case Types.REAL: return getFloatType(allowPrimitive, fullyQualified);
      case Types.SMALLINT: return getIntType(allowPrimitive, fullyQualified);

      case Types.TIME: return "java.sql.Time";
      case Types.TIMESTAMP: return "java.sql.Timestamp";
      case Types.TINYINT: return getIntType(allowPrimitive, fullyQualified);
      case Types.VARCHAR: return getStringType(fullyQualified);
 
      default:
        throw new RuntimeException("Not supported type: " + getSqlTypeName(sqlType));
    }
  }

  /**
   * Gets the type name for String values in consideration of the given flags.
   * @param fullyQualified If true, returns the fully qualified class names for system classes, 
   *   else returns the system class names without java.lang package prefix. 
   * @return The SQL type name.
   */
  private String getStringType(boolean fullyQualified) {
    if (fullyQualified) {
      return "java.lang.String";
    }
    return "String";
  }

  /**
   * Gets the type name for String values in consideration of the given flags.
   * @param fullyQualified If true, returns the fully qualified class names for system classes, 
   *   else returns the system class names without java.lang package prefix. 
   * @return The SQL type name.
   */
  private String getObjectType(boolean fullyQualified) {
    if (fullyQualified) {
      return "java.lang.Object";
    }
    return "Object";
  }

  /**
   * Gets the type name for integer values in consideration of the given flags.
   * @param allowPrimitive If true, uses the simple types (int, long, float, double, ...) for
   *   numeric values, otherwise returns the wrapper classed for these types.
   * @param fullyQualified If true, returns the fully qualified class names for system classes, 
   *   else returns the system class names without java.lang package prefix. 
   * @return The SQL type name.
   */
  private String getIntType(boolean allowPrimitive, boolean fullyQualified) {
    if (allowPrimitive) {
      return "int";
    }
    if (fullyQualified) {
      return "java.lang.Integer";
    }
    return "Integer";
  }

  /**
   * Gets the type name for long values in consideration of the given flags.
   * @param allowPrimitive If true, uses the simple types (int, long, float, double, ...) for
   *   numeric values, otherwise returns the wrapper classed for these types.
   * @param fullyQualified If true, returns the fully qualified class names for system classes, 
   *   else returns the system class names without java.lang package prefix. 
   * @return The SQL type name.
   */
  private String getLongType(boolean allowPrimitive, boolean fullyQualified) {
    if (allowPrimitive) {
      return "long";
    }
    if (fullyQualified) {
      return "java.lang.Long";
    }
    return "Long";
  }
  
  /**
   * Gets the type name for long values in consideration of the given flags.
   * @param allowPrimitive If true, uses the simple types (int, long, float, double, ...) for
   *   numeric values, otherwise returns the wrapper classed for these types.
   * @param fullyQualified If true, returns the fully qualified class names for system classes, 
   *   else returns the system class names without java.lang package prefix. 
   * @return The SQL type name.
   */
  private String getFloatType(boolean allowPrimitive, boolean fullyQualified) {
    if (allowPrimitive) {
      return "float";
    }
    if (fullyQualified) {
      return "java.lang.Float";
    }
    return "Float";
  }

  /**
   * Gets the type name for long values in consideration of the given flags.
   * @param allowPrimitive If true, uses the simple types (int, long, float, double, ...) for
   *   numeric values, otherwise returns the wrapper classed for these types.
   * @param fullyQualified If true, returns the fully qualified class names for system classes, 
   *   else returns the system class names without java.lang package prefix. 
   * @return The SQL type name.
   */
  private String getDoubleType(boolean allowPrimitive, boolean fullyQualified) {
    if (allowPrimitive) {
      return "double";
    }
    if (fullyQualified) {
      return "java.lang.Double";
    }
    return "Double";
  }

  /**
   * Gets the name of the given SQL type
   * @param sqlType The SQL type ID from java.sql.Types.
   * @return The SQL type name.
   */
  public String getSqlTypeName(int sqlType) {
    switch (sqlType) {
      case Types.ARRAY: return "ARRAY";
      case Types.BIGINT: return "BIGINT";
      case Types.BINARY: return "BINARY";
      case Types.BIT: return "BIT";
      case 16: /*Types.BOOLEAN*/ return "BOOLEAN";
      case Types.CHAR: return "CHAR";
      case Types.CLOB: return "CLOB";
      //case Types.DATALINK: return "DATALINK";
      case Types.DATE: return "DATE";
      case Types.DECIMAL: return "DECIMAL";
      case Types.DISTINCT: return "DISTINCT";
      case Types.DOUBLE: return "DOUBLE";
      case Types.FLOAT: return "FLOAT";
      case Types.INTEGER: return "INTEGER";
      case Types.JAVA_OBJECT: return "JAVA_OBJECT";
      case Types.LONGVARBINARY: return "LONGVARBINARY";
      case Types.LONGVARCHAR: return "LONGVARCHAR";
      case Types.NUMERIC: return "NUMERIC";
      case Types.OTHER: return "OTHER";
      case Types.REAL: return "REAL";
      case Types.REF: return "REF";
      case Types.SMALLINT: return "SMALLINT";
      case Types.STRUCT: return "STRUCT";
      case Types.TIME: return "TIME";
      case Types.TIMESTAMP: return "TIMESTAMP";
      case Types.TINYINT: return "TINYINT";
      case Types.VARBINARY: return "VARBINARY";
      case Types.VARCHAR: return "VARCHAR";
      default: return null;
    }
  }
}
