/*
 * Copyright (C) 2003 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.db;

import java.util.Map;
import java.util.List;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.StringTokenizer;


/**
 * Name: TableDefinition.java<br>
 * Description: Description of class.<br>
 * 
 * @author lauerc
 */
public class TableDefinition {

  /** The table name. */
  private String name;
  /** The schema the table is located at. */
  private String schema;
  /** The attribute list. */
  private List attributes = new ArrayList();
  /** The attribute name map. */
  private Map attributeNameMap = new HashMap();
  /** The attribute column map. */
  private Map attributeColumnMap = new HashMap();
  
  /**
   * Constructs a TableDefinition instance with specific attributes.
   * @param name The table name.
   */
  public TableDefinition(String name) {
    StringTokenizer nameTkn = new StringTokenizer(name, ".");
    String first = nameTkn.nextToken();
    if (nameTkn.hasMoreTokens()) {
      this.schema = first;
      this.name = nameTkn.nextToken();
    } else {
      this.name = first;
    }
  }

  /**
   * Constructs a TableDefinition instance with specific attributes.
   * @param name The table name.
   * @param schema The schema, the table is located at.
   */
  public TableDefinition(String name, String schema) {
    this.name = name;
    this.schema = schema;
  }

  /**
   * Gets the table name.
   * @return The table name.
   */
  public String getName() {
    return name;
  }

  /**
   * Gets the schema, the table is located at.
   * @return The schema name.
   */
  public String getSchema() {
    return schema;
  }
	
  /**
   * Adds an attribute.
   * @param attribute The attribute definition.
   */
  public void addAttribute(AttributeDefinition attribute) {
    attributes.add(attribute);
    attributeNameMap.put(attribute.getName(), attribute);
    attributeColumnMap.put(attribute.getColumnName(), attribute);
  }
  
  /**
   * Gets an attribute by its name.
   * @param name The attribute name.
   * @return The determined attribute definition. 
   */
  public AttributeDefinition getAttributeByName(String name) {
    return (AttributeDefinition) attributeNameMap.get(name);
  }

  /**
   * Gets an attribute by its column name.
   * @param columnName The attribute column name.
   * @return The determined attribute definition. 
   */
  public AttributeDefinition getAttributeByColumn(String columnName) {
    return (AttributeDefinition) attributeColumnMap.get(columnName);
  }

  /**
   * Clears the attribute list.
   */
  public void clearAttributes() {
    attributes.clear();
  }

  /**
   * Gets the attribute list.
   * @return The attribute definition list.
   */
  public List getAttributes() {
    return attributes;
  }
  
  /**
   * Gets the number of primary keys.
   * @return The number of primary keys.
   */
  public int getNumPrimaryKeys() {
    int numPrimaryKeys = 0;
    int numAttributes = attributes.size();
    for (int i = 0; i < numAttributes; i++) {
      AttributeDefinition attr = (AttributeDefinition) attributes.get(i);
      if (attr.isPrimaryKey()) {
        ++numPrimaryKeys;
      }
    }
    return numPrimaryKeys;
  }
  
  /**
   * Gets the primary key attributes.
   * @return The primary key attributes.
   */
  public AttributeDefinition[] getPrimaryKeys() {
    int numPrimaryKeys = getNumPrimaryKeys();
    AttributeDefinition[] pkAttrs = new AttributeDefinition[numPrimaryKeys];
    int pkIdx = 0;
    int numAttributes = attributes.size();
    for (int i = 0; i < numAttributes; i++) {
      AttributeDefinition attr = (AttributeDefinition) attributes.get(i);
      if (attr.isPrimaryKey()) {
        pkAttrs[pkIdx] = attr;
        ++pkIdx;
      }
    }
    return pkAttrs;
  }

  /**
   * Returns a string representation of the object. 
   * @return  a string representation of the object.
   */
  public String toString() {
    StringBuffer buf = new StringBuffer();
    buf.append("table name: '")
      .append(name);
    if (schema != null) {
      buf.append("' at schema: '")
        .append(schema);
    }
    buf.append("'\nattributes:");
    
    int numAttributes = attributes.size();
    for (int i=0; i<numAttributes; i++) {
      AttributeDefinition attribute = (AttributeDefinition) attributes.get(i);
      buf.append("\n  ").append(attribute);
    }
    
    return buf.toString();
  }
}
