/*
 * Copyright (C) 2003, 2004 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.exi;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.FileOutputStream;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.HashMap;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Map;

import org.ujac.print.DocumentHandlerException;
import org.ujac.print.DocumentPrinter;

/**
 * Name: DocumentationGenerator<br>
 * Description: A class generating the documentation for the type handlers
 *    according to the meta data, provided through the ExpressionType and
 *    ExpressionOperation interfaces.
 * 
 * @author lauerc
 */
public class DocumentationGenerator {

  /**
   * Name: TypeNameComparator<br>
   * Description: Compares ExpressionType instances by their type class names.
   */
  private static class TypeNameComparator implements Comparator {

    /**
     * Constructs a TypeNameComparator instance with no specific attributes.
     */
    public TypeNameComparator() {
    }

    /**
     * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
     */
    public int compare(Object o1, Object o2) {
      ExpressionType item1 = (ExpressionType) o1;
      ExpressionType item2 = (ExpressionType) o2;
      return item1.getType().getName().compareTo(item2.getType().getName());
    }
    
  }
  
  /** The tag factory. */
  private ExpressionInterpreter exi = null;
  
  /**
   * Constructs a DocumentationGenerator instance with specific attributes.
   * @param exi The expression interpreter to use.
   */
  public DocumentationGenerator(ExpressionInterpreter exi) {
    this.exi = exi;
  }
  
  /**
   * Generates the documentation using the given documentaion template.
   * @param documentationTempleteStream  The documentation template stream.
   * @param outputStream  The output stream.
   * @exception DocumentHandlerException If something went wrong while generating the documentation.
   * @exception IOException In case the writing of the document or the reading of the template has been failed.
   */
  private void generateDocumentation(InputStream documentationTempleteStream, 
                                     OutputStream outputStream) 
    throws DocumentHandlerException, IOException {
   
    List typeHandlers = new ArrayList();
    Iterator typeIter = exi.getTypeHandlers().iterator();
    while (typeIter.hasNext()) {
      ExpressionType typeHandler = (ExpressionType) typeIter.next();
      typeHandlers.add(typeHandler);      
    }
    // sorting the item list
    Collections.sort(typeHandlers, new TypeNameComparator());
    
    Map properties = new HashMap();
    properties.put("typeHandlers", typeHandlers);
    DocumentPrinter docPrinter = new DocumentPrinter(documentationTempleteStream, properties);
    docPrinter.printDocument(outputStream);
  }

  /**
   * The main method for command line access.
   * @param args The command line attributes.
   */
  public static void main(String[] args) {
    if (args.length > 1) {
      System.err.println("usage: org.ujac.util.exi.DocumentationGenerator [<output file> <expression interpreter class>]");
      System.exit(1);
    }
    try {
      OutputStream outputStream = null;
      if (args.length < 1)  {
        outputStream = System.out;
      } else {
        outputStream = new FileOutputStream(args[0]);
        if (args.length > 1) {
          System.setProperty(ExpressionInterpreter.ATTRIBUTE_NAME, args[1]);
        }
      }
      ExpressionInterpreter exi = ExpressionInterpreter.createInstance();
      DocumentationGenerator dg = new DocumentationGenerator(exi);
      InputStream documentationTempleteStream = DocumentationGenerator.class.getClassLoader().getResourceAsStream("org/ujac/util/exi/documentation-template.xml");
      dg.generateDocumentation(documentationTempleteStream, outputStream);
    } catch (IOException ex) {
      ex.printStackTrace();
    } catch (DocumentHandlerException ex) {
      ex.printStackTrace();
    }
  }
}
