/*
 * Copyright (C) 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.exi;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.ujac.util.text.FormatHelper;

/**
 * Name: ExpressionContext<br>
 * Description: A class holding all relevant data, needed to execute expressions.
 * 
 * @author lauerc
 */
public class ExpressionContext implements Map, Serializable {

  /** The serial version UID. */
  private static final long serialVersionUID = 3546084661601252665L;
  
  /** The properties to use for expression evaluation. */
  private Map properties;
  /** 
   * The bean used to retrieve parameter values, if the
   * parameter value didn't exist in the params map.
   */
  private Object bean;
  /** The format helper to use. */
  private FormatHelper formatHelper;
  
  /**
   * Constructs a ExpressionContext instance with no specific attributes.
   */
  public ExpressionContext() {
    this(new HashMap(), null, null);
  }
  
  /**
   * Constructs a ExpressionContext instance with specific attributes.
   * @param properties The properties to use for expression evaluation. 
   */
  public ExpressionContext(Map properties) {
    this(properties, null, null);
  }

  /**
   * Constructs a ExpressionContext instance with specific attributes.
   * @param properties The properties to use for expression evaluation. 
   * @param formatHelper The format helper to use. 
   */
  public ExpressionContext(Map properties, FormatHelper formatHelper) {
    this(properties, null, formatHelper);
  }

  /**
   * Constructs a ExpressionContext instance with specific attributes.
   * @param properties The properties to use for expression evaluation. 
   * @param bean The bean used to retrieve parameter values, if the
   *   parameter value didn't exist in the localData map.
   * @param formatHelper The format helper to use. 
   */
  public ExpressionContext(Map properties, Object bean, FormatHelper formatHelper) {
    this.properties = properties;
    this.bean = bean;
    if (formatHelper == null) {
      this.formatHelper = FormatHelper.createInstance();
    } else {
      this.formatHelper = formatHelper;
    }
  }

  /**
   * Constructs a ExpressionContext instance with specific attributes.
   * @param ctx The context to clone. 
   */
  public ExpressionContext(ExpressionContext ctx) {
    this.properties = new HashMap(ctx.properties);
    this.bean = ctx.getBean();
    this.formatHelper = ctx.getFormatHelper();
  }

  /**
   * Gets a property from the property map.
   * @param name The property name.
   * @return The value of the desired property or null.
   */
  public Object getProperty(Object name) {
    return properties.get(name);
  }
  /**
   * Sets a property at the property map.
   * @param name The property name.
   * @param value The value to set.
   * @return The old value of the desired property or null.
   */
  public Object setProperty(Object name, Object value) {
    return this.properties.put(name, value);
  }

  /**
   * Getter method for the the property bean.
   * @return The current value of property bean.
   */
  public Object getBean() {
    return bean;
  }
  /**
   * Setter method for the the property bean.
   * @param bean The value to set for the property bean.
   */
  public void setBean(Object bean) {
    this.bean = bean;
  }

  /**
   * Getter method for the the property formatHelper.
   * @return The current value of property formatHelper.
   */
  public FormatHelper getFormatHelper() {
    return formatHelper;
  }
  /**
   * Setter method for the the property formatHelper.
   * @param formatHelper The value to set for the property formatHelper.
   */
  public void setFormatHelper(FormatHelper formatHelper) {
    this.formatHelper = formatHelper;
  }

  /**
   * @see java.util.Map#size()
   */
  public int size() {
    return properties.size();
  }

  /**
   * @see java.util.Map#isEmpty()
   */
  public boolean isEmpty() {
    return properties.isEmpty();
  }

  /**
   * @see java.util.Map#containsKey(java.lang.Object)
   */
  public boolean containsKey(Object key) {
    return properties.containsKey(key);
  }

  /**
   * @see java.util.Map#containsValue(java.lang.Object)
   */
  public boolean containsValue(Object value) {
    return properties.containsValue(value);
  }

  /**
   * @see java.util.Map#get(java.lang.Object)
   */
  public Object get(Object key) {
    return getProperty(key);
  }

  /**
   * @see java.util.Map#put(java.lang.Object, java.lang.Object)
   */
  public Object put(Object key, Object value) {
    return setProperty(key, value);
  }
  
  /**
   * @see java.util.Map#remove(java.lang.Object)
   */
  public Object remove(Object key) {
    return properties.remove(key);
  }

  /**
   * @see java.util.Map#putAll(java.util.Map)
   */
  public void putAll(Map properties) {
    if (properties == null) {
      return;
    }
    Iterator iterProperties = properties.entrySet().iterator();
    while (iterProperties.hasNext()) {
      Map.Entry entry = (Map.Entry) iterProperties.next();
      setProperty(entry.getKey(), entry.getValue());
    }
  }

  /**
   * @see java.util.Map#clear()
   */
  public void clear() {
    properties.clear();
  }

  /**
   * @see java.util.Map#keySet()
   */
  public Set keySet() {
    return properties.keySet();
  }

  /**
   * @see java.util.Map#values()
   */
  public Collection values() {
    return properties.values();
  }

  /**
   * @see java.util.Map#entrySet()
   */
  public Set entrySet() {
    return properties.entrySet();
  }

  /**
   * Writes the object's data to the given stream.
   * @param s The stream to write to
   * @exception IOException In case the data output failed.
   */
  protected void writeData(ObjectOutputStream s) throws IOException {
    s.writeObject(properties);
    s.writeObject(bean);
    s.writeObject(formatHelper);
  }
  /**
   * Reads the object's data from the given stream.
   * @param s The stream to read from.
   * @exception IOException In case the data reading failed.
   * @exception ClassNotFoundException In case the class to deserialize could not be found
   */
  protected void readData(ObjectInputStream s) throws IOException, ClassNotFoundException  {
    properties = (Map) s.readObject();
    bean = s.readObject();
    formatHelper = (FormatHelper) s.readObject();
  }

  /**
   * Writes the object's data to the given stream.
   * @param s The stream to write to
   * @exception IOException In case the data output failed.
   */
  private void writeObject(ObjectOutputStream s) throws IOException {
    writeData(s);
  }
  /**
   * Reads the object's data from the given stream.
   * @param s The stream to read from.
   * @exception IOException In case the data reading failed.
   * @exception ClassNotFoundException In case the class to deserialize could not be found
   */
  private void readObject(ObjectInputStream s) throws IOException, ClassNotFoundException  {
    readData(s);
  }

}
