/*
 * Copyright (C) 2003 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.exi;


/**
 * Title:       ExpressionTuple<br>
 * Description: A class holding expression tuples: object, operation and operand.<br>
 * 
 * @author lauerc
 */
public class ExpressionTuple extends Operand {

  /** The serial version UID. */
  private static final long serialVersionUID = 3256718498527850545L;
  
  /** The object which to call. */
  private Operand object;
  /** The operation for the expression. */
  private Operation operation;
  /** The object which to call. */
  private Operand operand;
  /** Tells whether the expression was explicitely encapsulated or not. */
  private boolean explitelyEncapsulated;
  
  /** The result of the expression. */
  private Object result;
	
  /**
   * Default constructor for ExpressionTuple.
   */
  public ExpressionTuple() {
  }
  
  /**
   * Constructs an ExpressionTuple instance with specific arguments.
   * @param source The textual source for the expression.
   * @param absolutePosition The absolute start position of the expression within the source.
   * @param relativePosition The relative start position of the expression within the source.
   * @param length The length of the expression.
   */
  public ExpressionTuple(char[] source, int absolutePosition, int relativePosition, int length) {
    super(source, absolutePosition, relativePosition, length);
  }

  /**
   * Constructs an ExpressionTuple instance with specific arguments.
   * Copies the source, totalPosition and totalLength from the outer expression.
   * @param outer The outer expression.
   */
  public ExpressionTuple(ExpressionTuple outer) {
    super(outer);
  }
  
  /**
   * @see org.ujac.util.exi.Operand#isSimple()
   */
  public boolean isSimple() {
    return false;
  }

  /**
   * Returns the object.
   * @return The object to call.
   */
  public Operand getObject() {
    return object;
  }
  /**
   * Sets the object.
   * @param object The object to set
   */
  public void setObject(Operand object) {
    this.object = object;
  }

  /**
   * Returns the operand.
   * @return The operand to use for the call.
   */
  public Operand getOperand() {
    return operand;
  }
  /**
   * Sets the operand.
   * @param operand The operand to set
   */
  public void setOperand(Operand operand) {
    this.operand = operand;
  }

  /**
   * Gets the explitelyEncapsulated flag.
   * @return The current value of the explitelyEncapsulated flag.
   */
  public boolean isExplitelyEncapsulated() {
    return explitelyEncapsulated;
  }
  /**
   * Sets the explitelyEncapsulated flag.
   * @param explitelyEncapsulated The value for the explitelyEncapsulated flag to set.
   */
  public void setExplitelyEncapsulated(boolean explitelyEncapsulated) {
    this.explitelyEncapsulated = explitelyEncapsulated;
  }

  /**
   * Returns the operation.
   * @return The operation to execute.
   */
  public Operation getOperation() {
    return operation;
  }
  /**
   * Sets the operation.
   * @param operation The operation to set
   */
  public void setOperation(Operation operation) {
    this.operation = operation;
  }

  /**
   * Returns the result.
   * @return Object
   */
  public Object getResult() {
    return result;
  }
  /**
   * Sets the result.
   * @param result The result to set
   */
  public void setResult(Object result) {
    this.result = result;
  }

  /**
   * Gets the operand value.
   * @return the value of the operand.
   */
  public Object getValue() {
    return result;
  }
  
  /**
   * Converts expression tuple to String.
   *@return The objects content as a String.
   */
  public String toString() {
    StringBuffer content = new StringBuffer("${");
    //content.append("objectVariable='").append(objectVariable.toString()).append("'");
    content.append(object);

    
    if (operation != null) {
      content.append(" ").append(operation.toString());
    }

    if (operand != null) {
      content.append(" ").append(operand.toString());
    }

    if (result != null) {
      if (content.length() <= 2) {
        content.append("'").append(result).append("'");
      }
    }

    return content.append('}').toString();
  }
  
  /**
   * @see java.lang.Object#clone()
   */
  public Object clone() {
    ExpressionTuple tuple = (ExpressionTuple) super.clone();
    tuple.setObject((Operand) object.clone());
    /** The operation for the expression. */
    if (operand != null) {
      tuple.setOperand((Operand) operand.clone());
    }
    return tuple;
  }

}
