/*
 * Copyright (C) 2003 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.exi;

import java.io.Serializable;

/**
 * Name: Operand<br>
 * Description: A common implementation for operands.<br>
 * 
 * @author lauerc
 */
public abstract class Operand implements Serializable, Cloneable {

  /** The expression source. */
  private char[] source;
  /** The tuple's relative position in the source string. */
  private int relativePosition;
  /** The tuple's absolute position in the source string. */
  private int absolutePosition;
  /** The tuple's length. */
  private int length;
  /** The position of the whole expression, this differs for nested expressions. */
  private int totalPosition;
  /** The length of the whole expression, this differs for nested expressions. */
  private int totalLength;
  /** The forced type. */
  private ExpressionType forcedType = null;

  /**
   * Constructs an Operand instance with no specific arguments.
   */
  public Operand() {
  }
  
  /**
   * Constructs an Operand instance with specific arguments.
   * @param source The textual source for the expression.
   * @param absolutePosition The absolute start position of the expression within the source.
   * @param relativePosition The relative start position of the expression within the source.
   * @param length The length of the expression.
   */
  public Operand(char[] source, int absolutePosition, int relativePosition, int length) {
    this.source = source;
    this.absolutePosition = absolutePosition;
    this.relativePosition = relativePosition;
    this.length = length;
    this.totalPosition = absolutePosition;
    this.totalLength = length;
  }

  /**
   * Constructs an Operand instance with specific arguments.
   * Copies the source, totalPosition and totalLength from the outer expression.
   * @param outer The outer expression.
   */
  public Operand(Operand outer) {
    this.source = outer.getSource();
    this.totalPosition = outer.getTotalPosition();
    this.totalLength = outer.getTotalLength();
  }
  
  /**
   * Tells whether the operand is a simple one or not.
   * @return true if the operand is instance of class VariableOperand else false.
   */
  public abstract boolean isSimple();

  /**
   * Gets the source of the current expression part.
   * @return The code of the current expression part.
   */
  public String getCode() {
    return new String(source, absolutePosition, length);
  }

  /**
   * Gets the source.
   * @return The source of the complete expression.
   */
  public char[] getSource() {
    return source;
  }
  /**
   * Sets the source.
   * @param source The source to set
   */
  public void setSource(char[] source) {
    this.source = source;
  }

  /**
   * Gets the tuple's absolute position in the source string.
   * @return The tuple's absolute position in the source string.
   */
  public int getAbsolutePosition() {
    return absolutePosition;
  }
  /**
   * Sets the tuple's absolute position in the source string.
   * @param absolutePosition The absolute position to set.
   */
  public void setAbsolutePosition(int absolutePosition) {
    this.absolutePosition = absolutePosition;
  }

  /**
   * Gets the tuple's relative position in the source string.
   * @return The tuple's relative position in the source string.
   */
  public int getRelativePosition() {
    return relativePosition;
  }
  /**
   * Sets the tuple's relative position in the source string.
   * @param relativePosition The relative position to set.
   */
  public void setRelativePosition(int relativePosition) {
    this.relativePosition = relativePosition;
  }

  /**
   * Gets the tuple's length.
   * @return The tuple's length.
   */
  public int getLength() {
    return length;
  }
  /**
   * Sets the tuple's length.
   * @param length The length to set.
   */
  public void setLength(int length) {
    this.length = length;
  }

  /**
   * Gets the tuple's total position in the source string.
   * @return The tuple's total position in the source string.
   */
  public int getTotalPosition() {
    return totalPosition;
  }
  /**
   * Sets the tuple's total position in the source string.
   * @param totalPosition The total position to set.
   */
  public void setTotalPosition(int totalPosition) {
    this.totalPosition = totalPosition;
  }

  /**
   * Gets the tuple's total length.
   * @return The tuple's length.
   */
  public int getTotalLength() {
    return totalLength;
  }
  /**
   * Sets the tuple's total length.
   * @param totalLength The total length to set.
   */
  public void setTotalLength(int totalLength) {
    this.totalLength = totalLength;
  }
  
  /**
   * Gets the operand value.
   * @return the value of the operand.
   */
  public abstract Object getValue();

  /**
   * Getter method for the the property forcedType.
   * @return The current value of property forcedType.
   */
  public ExpressionType getForcedType() {
    return forcedType;
  }
  /**
   * Setter method for the the property forcedType.
   * @param forcedType The value to set for the property forcedType.
   */
  public void setForcedType(ExpressionType forcedType) {
    this.forcedType = forcedType;
  }
  
  /**
   * @see java.lang.Object#clone()
   */
  public Object clone() {
    try {
      return super.clone();
    } catch (CloneNotSupportedException ex) {
      throw new RuntimeException("Failed to clone Operand instance: " + ex.getMessage(), ex);
    }
  }

}
