/*
 * Copyright (C) 2003, 2004, 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.exi.test;

import java.sql.Time;
import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.ParseException;

import java.util.Calendar;

import org.ujac.util.DateUtils;
import org.ujac.util.exi.ExpressionException;

/**
 * Name: DateTypeTest<br>
 * Description: A class testing the DateType class.
 * 
 * @author lauerc
 */
public class DateTypeTest extends BaseExpressionTest {

  /** The date format to use. */
  private DateFormat df = null;
  
  /**
   * @see junit.framework.TestCase#setUp()
   */
  protected void setUp() throws Exception {
    super.setUp();

    Calendar cal = Calendar.getInstance();
    cal.set(Calendar.DATE, 1);
    cal.set(Calendar.MONTH, Calendar.MAY);
    cal.set(Calendar.YEAR, 1973);
    cal.set(Calendar.HOUR_OF_DAY, 03);
    cal.set(Calendar.MINUTE, 38);
    cal.set(Calendar.SECOND, 0);
    cal.set(Calendar.MILLISECOND, cal.getMinimum(Calendar.MILLISECOND));
    
    properties.put("birthday", DateUtils.createCalendar(cal.getTime(), Calendar.DATE).getTime());
    properties.put("birth_timestamp", new Timestamp(cal.getTime().getTime()));
    properties.put("birth_time", new Time(cal.getTime().getTime()));

    df = formatHelper.getIsoDateFormat();
  }

  /**
   * Tests the 'getDay' operation.
   * @throws ExpressionException In case an expression evaluation fails.
   */
  public void testGetDay() throws ExpressionException {
    assertTrue(exi.evalInt("${birthday getDay}", properties, formatHelper) == 1);
  }

  /**
   * Tests the 'getMonth' operation.
   * @throws ExpressionException In case an expression evaluation fails.
   */
  public void testGetMonth() throws ExpressionException {
    assertTrue(exi.evalInt("${birthday getMonth}", properties, formatHelper) == 5);
  }

  /**
   * Tests the 'getear' operation.
   * @throws ExpressionException In case an expression evaluation fails.
   */
  public void testGetYear() throws ExpressionException {
    assertTrue(exi.evalInt("${birthday getYear}", properties, formatHelper) == 1973);
  }

  /**
   * Tests the '==' operation.
   * @throws ExpressionException In case an expression evaluation fails.
   */
  public void testEqual() throws ExpressionException {
    assertTrue(exi.evalBoolean("${birthday == '1973-05-01'}", properties, formatHelper));
    assertTrue(exi.evalBoolean("${birth_time == '03:38:00'}", properties, formatHelper));
    assertTrue(exi.evalBoolean("${birth_timestamp == '1973-05-01 03:38:00'}", properties, formatHelper));
  }

  /**
   * Tests the '!=' operation.
   * @throws ExpressionException In case an expression evaluation fails.
   */
  public void testNotEqual() throws ExpressionException {
    assertTrue(exi.evalBoolean("${birthday != '1973-05-02'}", properties, formatHelper));
    assertTrue(exi.evalBoolean("${birth_time != '03:38:12'}", properties, formatHelper));
    assertTrue(exi.evalBoolean("${birth_timestamp != '1972-05-01 03:38:12'}", properties, formatHelper));
  }

  /**
   * Tests the '>' operation.
   * @throws ExpressionException In case an expression evaluation fails.
   */
  public void testGreater() throws ExpressionException {
    assertTrue(exi.evalBoolean("${birthday > '1973-04-30'}", properties, formatHelper));
    assertTrue(exi.evalBoolean("${birth_time > '03:37:12'}", properties, formatHelper));
    assertTrue(exi.evalBoolean("${birth_timestamp > '1973-05-01 03:37:12'}", properties, formatHelper));
  }

  /**
   * Tests the '<' operation.
   * @throws ExpressionException In case an expression evaluation fails.
   */
  public void testLess() throws ExpressionException {
    assertTrue(exi.evalBoolean("${birthday < '1973-05-02'}", properties, formatHelper));
    assertTrue(exi.evalBoolean("${birth_time < '03:38:30'}", properties, formatHelper));
    assertTrue(exi.evalBoolean("${birth_timestamp < '1973-05-01 03:38:30'}", properties, formatHelper));
  }

  /**
   * Tests the 'format' operation.
   * @throws ExpressionException In case an expression evaluation fails.
   */
  public void testFormat() throws ExpressionException {
    assertEquals("01.05.1973", exi.evalString("${birthday format 'dd.MM.yyyy'}", properties, formatHelper));
    assertEquals("03:38:00", exi.evalString("${birth_time format 'HH:mm:ss'}", properties, formatHelper));
    assertEquals("01.05.1973 03:38:00", exi.evalString("${birth_timestamp format 'dd.MM.yyyy HH:mm:ss'}", properties, formatHelper));
  }

  /**
   * Tests the increment operations.
   * @throws ExpressionException In case an expression evaluation fails.
   */
  public void testIncrement() throws ExpressionException, ParseException {
    assertEquals(exi.evalDate("${birthday incrYear 1}", properties, formatHelper), df.parse("1974-05-01"));
    assertEquals(exi.evalDate("${birthday incrMonth 1}", properties, formatHelper), df.parse("1973-06-01"));
    assertEquals(exi.evalDate("${birthday incrDay 1}", properties, formatHelper), df.parse("1973-05-02"));
  }

  /**
   * Tests the increment operations.
   * @throws ExpressionException In case an expression evaluation fails.
   */
  public void testDecrement() throws ExpressionException, ParseException {
    assertEquals(exi.evalDate("${birthday decrYear 1}", properties, formatHelper), df.parse("1972-05-01"));
    assertEquals(exi.evalDate("${birthday decrMonth 1}", properties, formatHelper), df.parse("1973-04-01"));
    assertEquals(exi.evalDate("${birthday decrDay 1}", properties, formatHelper), df.parse("1973-04-30"));
  }

  /**
   * Tests the set operations.
   * @throws ExpressionException In case an expression evaluation fails.
   */
  public void testSet() throws ExpressionException, ParseException {
    assertEquals(exi.evalDate("${birthday setYear 1975}", properties, formatHelper), df.parse("1975-05-01"));
    assertEquals(exi.evalDate("${birthday setMonth 06}", properties, formatHelper), df.parse("1973-06-01"));
    assertEquals(exi.evalDate("${birthday setDay 15}", properties, formatHelper), df.parse("1973-05-15"));
  }

  /**
   * Tests the beginOfYear operation.
   * @throws ExpressionException In case an expression evaluation fails.
   */
  public void testBeginOfYear() throws ExpressionException, ParseException {
    assertEquals(exi.evalDate("${birthday beginOfYear}", properties, formatHelper), df.parse("1973-01-01"));
    assertEquals(exi.evalDate("${${birthday decrYear 1} beginOfYear}", properties, formatHelper), df.parse("1972-01-01"));
  }

  /**
   * Tests the endOfYear operation.
   * @throws ExpressionException In case an expression evaluation fails.
   */
  public void testEndOfYear() throws ExpressionException, ParseException {
    assertEquals(exi.evalDate("${birthday endOfYear}", properties, formatHelper), df.parse("1973-12-31"));
  }

}
