/*
 * Copyright (C) 2004, 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.exi.test;

import org.ujac.util.exi.ExpressionException;

/**
 * Name: NumberTypeTest<br>
 * Description: A class testing numeric operations.
 * 
 * @author lauerc
 */
public class NumberTypeTest extends BaseExpressionTest {

  /**
   * @see junit.framework.TestCase#setUp()
   */
  protected void setUp() throws Exception {
    super.setUp();

    properties.put("ten", new Integer(10));  
    properties.put("negativeInt", new Integer(-13));  
    properties.put("negativeFloat", new Float(-13.0F));  
    properties.put("negativeDouble", new Double(-13.0));

    properties.put("byte1", new Byte((byte) 8));
    properties.put("byte2", new Byte((byte) 15));
  }

  /**
   * Tests type casts.
   * @throws ExpressionException In case an expression evaluation fails.
   */
  public void testTypeCast() throws ExpressionException {
    assertEquals(exi.evalInt("${(int) 1.5}", properties, formatHelper), 1);
    assertEquals(exi.evalDouble("${(double) 1 + 2.5 + 3.5}", properties, formatHelper), 7.0, 0.0);
  }

  /**
   * Tests the '+' operation.
   * @throws ExpressionException In case an expression evaluation fails.
   */
  public void testPlus() throws ExpressionException {
    assertEquals(exi.evalInt("${1 + 2 + 3}", properties, formatHelper), 6);
    assertEquals(exi.evalDouble("${1.0 + 2.5 + 3.5}", properties, formatHelper), 7.0, 0.0);
  }

  /**
   * Tests the '-' operation.
   * @throws ExpressionException In case an expression evaluation fails.
   */
  public void testMinus() throws ExpressionException {
    assertEquals(exi.evalInt("${5 - 2 - 3}", properties, formatHelper), 0);
    assertEquals(exi.evalDouble("${5.7 - 2.5 - 3.2}", properties, formatHelper), 0.0, 0.0);
  }

  /**
   * Tests the '*' operation.
   * @throws ExpressionException In case an expression evaluation fails.
   */
  public void testMmul() throws ExpressionException {
    assertEquals(exi.evalInt("${5 * 2 * 3}", properties, formatHelper), 30);
    assertEquals(exi.evalDouble("${2.5 * 3.5 * 1.5}", properties, formatHelper), 13.125, 0.0);
  }

  /**
   * Tests the '/' operation.
   * @throws ExpressionException In case an expression evaluation fails.
   */
  public void testDiv() throws ExpressionException {
    assertEquals(exi.evalInt("${30 / 2 / 5}", properties, formatHelper), 3);
    assertEquals(exi.evalDouble("${20.0 / 5.0 / 2.5}", properties, formatHelper), 1.6, 0.0);
  }

  /**
   * Tests the precedence of operators.
   * @throws ExpressionException In case an expression evaluation fails.
   */
  public void testPrecedenceRules() throws ExpressionException {
    assertTrue(exi.evalBoolean("${${ten isDefined} && ten == 10}", properties, formatHelper));
    assertEquals(exi.evalInt("${30 + 10 * 2 + 5}", properties, formatHelper), 55);
    assertEquals(exi.evalInt("${30 + 2 * 5}", properties, formatHelper), 40);
    assertEquals(exi.evalDouble("${30 * 5.0 + 2 * 5.0}", properties, formatHelper), 160.0, 0.0);
    assertEquals(exi.evalDouble("${30.0 + 8.0 * 2.0 + 12.5}", properties, formatHelper), 58.5, 0.0);
    assertEquals(exi.evalDouble("${30.0 + 7.5 * 2.0 / 5.0}", properties, formatHelper), 33.0, 0.0);

    assertEquals(exi.evalInt("${1 * 4 / 2 + 3 }", properties, formatHelper), 5);
    assertEquals(exi.evalInt("${10 * ${1 + 7 - 2} / 3}", properties, formatHelper), 20);
    assertEquals(exi.evalInt("${7 + 28 * 3 - 5 + ${ten} * 5 + 8 + 4 * 23}", properties, formatHelper), 236);
    assertEquals(exi.evalInt("${10 + 1 * 4 / 2 + 3}", properties, formatHelper), 15);
    assertEquals(exi.evalInt("${10 + 1 * 4 / 2 + 3}", properties, formatHelper), 15);

    assertEquals(exi.evalInt("${${10.0 - 5.0 + 9.0} / 70 * 100}", properties, formatHelper), 20);
  }

  /**
   * Tests the usage of negative constants.
   * @throws ExpressionException In case an expression evaluation fails.
   */
  public void testNegatives() throws ExpressionException {
    assertEquals(exi.evalFloat("${-negativeFloat}", properties, formatHelper), 13.0F, 0.0F);
    assertEquals(exi.evalInt("${-negativeInt+10}", properties, formatHelper), 23);
    assertEquals(exi.evalDouble("${-5.7-2.3}", properties, formatHelper), -8.0, 0.0);
    assertEquals(exi.evalDouble("${-5.7 - 2.3}", properties, formatHelper), -8.0, 0.0);
    assertEquals(exi.evalInt("${5 * -1}", properties, formatHelper), -5);
    assertEquals(exi.evalInt("${5*-1}", properties, formatHelper), -5);
  }

  /**
   * Tests comparison operators for numeric values.
   * @throws ExpressionException In case an expression evaluation fails.
   */
  public void testComparisonOperators() throws ExpressionException {
    assertTrue(exi.evalBoolean("${ten==10}", properties, formatHelper));
    assertTrue(exi.evalBoolean("${ten<=10}", properties, formatHelper));
    assertTrue(exi.evalBoolean("${ten>=10}", properties, formatHelper));
    assertTrue(exi.evalBoolean("${ten<11}", properties, formatHelper));
    assertFalse(exi.evalBoolean("${ten>10}", properties, formatHelper));
    
    assertTrue(exi.evalBoolean("${ten == 10}", properties, formatHelper));
    assertTrue(exi.evalBoolean("${ten <= 10}", properties, formatHelper));
    assertTrue(exi.evalBoolean("${ten >= 10}", properties, formatHelper));
    assertTrue(exi.evalBoolean("${ten < 11}", properties, formatHelper));
    assertFalse(exi.evalBoolean("${ten > 10}", properties, formatHelper));
   }

  /**
   * Tests the usage of byte values in expressions.
   * @throws ExpressionException In case an expression evaluation fails.
   */
  public void testBytes() throws ExpressionException {
    assertTrue(exi.evalBoolean("${byte1 == 0x08}", properties, formatHelper));
    assertTrue(exi.evalBoolean("${byte2 == 0x0F}", properties, formatHelper));
    assertTrue(exi.evalBoolean("${byte1 != 0x0F}", properties, formatHelper));
    assertTrue(exi.evalBoolean("${byte2 != 0x08}", properties, formatHelper));
  }

}
