/*
 * Copyright (C) 2004 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.exi.type;

import java.text.Format;

import java.util.Calendar;
import java.util.Date;

import org.ujac.util.DateUtils;
import org.ujac.util.exi.BaseExpressionOperation;
import org.ujac.util.exi.ExpressionContext;
import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.ExpressionInterpreter;
import org.ujac.util.exi.ExpressionOperation;
import org.ujac.util.exi.ExpressionTuple;
import org.ujac.util.exi.NoOperandException;
import org.ujac.util.exi.Operand;
import org.ujac.util.exi.OperandException;

import org.ujac.util.text.FormatHelper;

/**
 * Name: BaseDateType<br>
 * Description: A base type handler class for Date values.
 * 
 * @author lauerc
 */
public abstract class BaseDateType extends BaseType {

  /**
   * The equal compare operation for Dates.
   */
  public class EqualOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      Date operandValue = evalDate(operand, ctx);
      // equal check
      return new Boolean(compareDates(((Date) expr.getObject().getValue()), operandValue) == 0);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Compares the object with the operand for equality.";
    }
  }
  
  /**
   * The 'not equal' compare operation for Dates.
   */
  public class NotEqualOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      Date operandValue = evalDate(operand, ctx);
      // not equal check
      return new Boolean(compareDates(((Date) expr.getObject().getValue()), operandValue) != 0);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Compares the object with the operand for inequality.";
    }
  }

  /**
   * The 'greater' compare operation for Dates.
   */
  public class GreaterOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      Date operandValue = evalDate(operand, ctx);
      // greater check
      return new Boolean(compareDates(((Date) expr.getObject().getValue()), operandValue) > 0);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the object is greater than the operand.";
    }
  }

  /**
   * The 'less' compare operation for Dates.
   */
  public class LessOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      Date operandValue = evalDate(operand, ctx);
      // less check
      return new Boolean(compareDates(((Date) expr.getObject().getValue()), operandValue) < 0);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the object is smaller than the operand.";
    }
  }

  /**
   * The 'greater equal' compare operation for Dates.
   */
  public class GreaterEqualOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      Date operandValue = evalDate(operand, ctx);
      // greater equal check
      return new Boolean(compareDates(((Date) expr.getObject().getValue()), operandValue) >= 0);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the object is greater than or equal to the operand.";
    }
  }

  /**
   * The 'less equal' compare operation for Dates.
   */
  public class LessEqualOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      Date operandValue = evalDate(operand, ctx);
      // less equal check
      return new Boolean(compareDates(((Date) expr.getObject().getValue()), operandValue) <= 0);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the object is smaller than or equal to the operand.";
    }
  }
  
  /**
   * The 'increment year' operation for Dates.
   */
  public class IncrementYearOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      int operandValue = interpreter.evalIntOperand(operand, ctx, false);
      // increasing year
      return DateUtils.incrYear((Date) expr.getObject().getValue(), operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Increments the value of the object's year by the amount, specified by the operand.";
    }
  }
  
  /**
   * The 'decrement year' operation for Dates.
   */
  public class DecrementYearOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      int operandValue = interpreter.evalIntOperand(operand, ctx, false);
      // decreasing year
      return DateUtils.incrYear((Date) expr.getObject().getValue(), -operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Decrements the value of the object's year by the amount, specified by the operand.";
    }
  }

  /**
   * The 'set year' operation for Dates.
   */
  public class SetYearOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      int operandValue = interpreter.evalIntOperand(operand, ctx, false);
      // setting year
      return DateUtils.setYear((Date) expr.getObject().getValue(), operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Sets the value of the object's year to the value, specified by the operand.";
    }
  }
    
  /**
   * The 'increment month' operation for Dates.
   */
  public class IncrementMonthOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      int operandValue = interpreter.evalIntOperand(operand, ctx, false);
      // increasing month
      return DateUtils.incrMonth((Date) expr.getObject().getValue(), operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Increments the value of the object's month by the amount, specified by the operand.";
    }
  }
  
  /**
   * The 'decrement month' operation for Dates.
   */
  public class DecrementMonthOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      int operandValue = interpreter.evalIntOperand(operand, ctx, false);
      // decreasing month
      return DateUtils.incrMonth((Date) expr.getObject().getValue(), -operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Decrements the value of the object's month by the amount, specified by the operand.";
    }
  }

  /**
   * The 'set month' operation for Dates.
   */
  public class SetMonthOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      int operandValue = interpreter.evalIntOperand(operand, ctx, false);
      // setting month
      return DateUtils.setMonth((Date) expr.getObject().getValue(), operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Sets the value of the object's month to the value, specified by the operand.";
    }
  }

  /**
   * The 'increment day' operation for Dates.
   */
  public class IncrementDayOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      int operandValue = interpreter.evalIntOperand(operand, ctx, false);
      // increasing month
      return DateUtils.incrDay((Date) expr.getObject().getValue(), operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Increments the value of the object's day by the amount, specified by the operand.";
    }
  }
  
  /**
   * The 'decrement day' operation for Dates.
   */
  public class DecrementDayOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      int operandValue = interpreter.evalIntOperand(operand, ctx, false);
      // decreasing month
      return DateUtils.incrDay((Date) expr.getObject().getValue(), -operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Decrements the value of the object's day by the amount, specified by the operand.";
    }
  }
  /**
   * The 'set day' operation for Dates.
   */
  public class SetDayOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      int operandValue = interpreter.evalIntOperand(operand, ctx, false);
      // setting day
      return DateUtils.setDay((Date) expr.getObject().getValue(), operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Sets the value of the object's day to the value, specified by the operand.";
    }
  }
  

  /**
   * The 'min' operation for Dates.
   */
  public class MinOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      Date operandValue = evalDate(operand, ctx);
      // determining minimum
      return DateUtils.min((Date) expr.getObject().getValue(), operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the minimum of the object and the operand value.";
    }
  }

  /**
   * The 'max' operation for Dates.
   */
  public class MaxOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      Date operandValue = evalDate(operand, ctx);
      // determining maximum
      return DateUtils.min((Date) expr.getObject().getValue(), operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the maximum of the object and the operand value.";
    }
  }

  /**
   * The 'begin of year' operation for Dates.
   */
  public class BeginOfYearOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new NoOperandException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // determining begin of year for the given date
      return DateUtils.beginOfYear((Date) expr.getObject().getValue());
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the begin of year (January, 1st) for the date, specified by the object.";
    }
  }

  /**
   * The 'end of year' operation for Dates.
   */
  public class EndOfYearOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new NoOperandException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // determining begin of year for the given date
      return DateUtils.endOfYear((Date) expr.getObject().getValue());
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the end of year (December, 31st) for the date, specified by the object.";
    }
  }

  /**
   * The 'previous ultimo' operation for Dates.
   */
  public class PreviousUltimoOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new NoOperandException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // determining previous ultimo
      return DateUtils.previousUltimo((Date) expr.getObject().getValue());
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the previous ultimo of the date, specified by the object. The ultimo date is the last day of a month. " +
          "In case the day is the 15th of April, the resulting object will be the 31th of March.";
    }
  }

  /**
   * The 'next ultimo' operation for Dates.
   */
  public class NextUltimoOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new NoOperandException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // determining next ultimo
      return DateUtils.nextUltimo((Date) expr.getObject().getValue());
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the next ultimo of the date, specified by the object. The ultimo date is the last day of a month. " +
          "In case the day is the 15th of April, the resulting object will be the 30th of April.";
    }
  }

  /**
   * The 'get year' operation for Dates.
   */
  public class GetYearOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new NoOperandException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting the year from the date instance
      return new Integer(DateUtils.getYear((Date) expr.getObject().getValue()));
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the year field from the date, specified by the object.";
    }
  }

  /**
   * The 'get month' operation for Dates.
   */
  public class GetMonthOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new NoOperandException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting the month from the date instance
      return new Integer(DateUtils.getMonth((Date) expr.getObject().getValue()));
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the month field from the date, specified by the object.";
    }
  }

  /**
   * The 'get day' operation for Dates.
   */
  public class GetDayOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new NoOperandException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting the day from the date instance
      return new Integer(DateUtils.getDay((Date) expr.getObject().getValue()));
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the day field from the date, specified by the object.";
    }
  }

  /**
   * The reference operation for Dates.
   */
  public class ReferenceOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException(
            "No operand given for operation: " + expr.getOperation() + 
            " on object " + expr.getObject() + "!"
        );         
      }
      // getting operand
      String operandValue = interpreter.evalStringOperand(operand, ctx);

      Calendar cal = DateUtils.createCalendar((Date) expr.getObject().getValue());

      if ("year".equals(operandValue)) {
        return new Integer(DateUtils.getYear(cal));
      } else if ("month".equals(operandValue)) {
        return new Integer(DateUtils.getMonth(cal));
      } else if ("day".equals(operandValue)) {
        return new Integer(DateUtils.getDay(cal));
      } else if ("hour".equals(operandValue)) {
        return new Integer(DateUtils.getHour(cal));
      } else if ("minute".equals(operandValue)) {
        return new Integer(DateUtils.getMinute(cal));
      } else if ("second".equals(operandValue)) {
        return new Integer(DateUtils.getSecond(cal));
      } else if ("millisecond".equals(operandValue)) {
        return new Integer(DateUtils.getMillisecond(cal));
      } else {
        throw new OperandException("Unknown property '" + operandValue + "' for date values at expression '" + expr.getCode() + "'.");
      }
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Accesses the fields of the date by the given operand name.\nValid names are:\n" +
          "\t year: Get the year field of the date\n " +
          "\t month: Get the month field of the date\n " +
          "\t day: Get the day field of the date\n " +
          "\t hour: Get the hour of day field of the date\n " +
          "\t minute: Get the minute field of the date\n " +
          "\t second: Get the second field of the date\n " +
          "\t millisecond: Get the millisecond field of the date";
    }
  }
  
  /**
   * The 'format' operation for Dates.
   */
  public class FormatOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      Object operandValue = operand.getValue();
      // determining format
      Format format = null;
      FormatHelper formatHelper = ctx.getFormatHelper();
      if (operandValue instanceof String) {
        String formatName = (String) operandValue;
        format = formatHelper.lookupFormat(formatName);
        if (format == null) {
          format = formatHelper.createDateFormat(formatName);
          formatHelper.registerFormat(formatName, format);
        }
      }
      return format.format(expr.getObject().getValue());
    }

    
    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Formats the object using the format, specified by the operand.";
    }
  }

  /**
   * Evaluates the given operand to a Date value.
   * @param operand The operand to evaluate. 
   * @param ctx The expression context.
   * @return The evaluated date.
   * @exception ExpressionException In case the evaluation failed.
   */
  protected abstract Date evalDate(Operand operand, ExpressionContext ctx) 
    throws ExpressionException;
  
  /**
   * Executes the toString operation.
   * @param expr The expression, holding the object to get as string.
   * @param formatHelper The format helper to use.
   * @return The textual representation of the expression value.
   */
  protected abstract String execToString(ExpressionTuple expr, FormatHelper formatHelper);
  
  /**
   * Compares the given date instances.
   * @param object The object to compare.
   * @param operand The operand to compare.
   * @return 1 in case the object is bigger than the operand, 
   *         0 if object and operand equals
   *        -1 in case the object is smaller than the operand. 
   */
  protected abstract int compareDates(Date object, Date operand);
  
  /**
   * Constructs a BaseNumberType instance with specific attributes.
   * @param interpreter The expression interpreter
   */
  public BaseDateType(ExpressionInterpreter interpreter) {
    super(interpreter);
    // adding operations for this type.
    ExpressionOperation op = new EqualOperation();
    addOperation("==", op);
    addOperation("eq", op);
    op = new GreaterOperation();
    addOperation(">", op);
    addOperation("gt", op);
    op = new GreaterEqualOperation();
    addOperation(">=", op);
    addOperation("ge", op);
    op = new LessOperation();
    addOperation("<", op);
    addOperation("lt", op);
    op = new LessEqualOperation();
    addOperation("<=", op);
    addOperation("le", op);
    op = new ReferenceOperation();
    addOperation(".", op);
    addOperation("->", op);

    op = new BeginOfYearOperation();
    addOperation("beginOfYear", op);
    op = new EndOfYearOperation();
    addOperation("endOfYear", op);

    addOperation("min", new MinOperation());
    addOperation("max", new MaxOperation());
    addOperation("prevUltimo", new PreviousUltimoOperation());
    addOperation("nextUltimo", new NextUltimoOperation());
    addOperation("incrDay", new IncrementDayOperation());
    addOperation("incrMonth", new IncrementMonthOperation());
    addOperation("incrYear", new IncrementYearOperation());
    addOperation("decrDay", new DecrementDayOperation());
    addOperation("decrMonth", new DecrementMonthOperation());
    addOperation("decrYear", new DecrementYearOperation());
    addOperation("getDay", new GetDayOperation());
    addOperation("getMonth", new GetMonthOperation());
    addOperation("getYear", new GetYearOperation());
    addOperation("setDay", new SetDayOperation());
    addOperation("setMonth", new SetMonthOperation());
    addOperation("setYear", new SetYearOperation());

    addOperation("format", new FormatOperation());
  }
  
}
