/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.exi.type;

import java.util.Map;
import java.util.HashMap;
import java.util.Set;

import org.ujac.util.exi.BaseExpressionOperation;
import org.ujac.util.exi.ExpressionContext;
import org.ujac.util.exi.NoOperandException;
import org.ujac.util.exi.Operand;
import org.ujac.util.exi.OperandException;
import org.ujac.util.exi.OperandNotSupportedException;
import org.ujac.util.exi.Operation;
import org.ujac.util.exi.ExpressionType;
import org.ujac.util.exi.ExpressionTuple;
import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.ExpressionOperation;
import org.ujac.util.exi.ExpressionInterpreter;
import org.ujac.util.exi.TypeCastException;
import org.ujac.util.text.FormatHelper;


/**
 * Name: BaseType<br>
 * Description: A base implementation for expression type handlers.<br>
 * 
 * @author lauerc
 */
abstract public class BaseType implements ExpressionType {

  /**
   * Checks the existance of the given object.
   */
  public class ObjectDefinedOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {

      return new Boolean(expr.getObject().getValue() != null);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Tells whether the object is defined or not.";
    }
  }
  
  /**
   * Checks the existance of the given object.
   */
  public class ObjectNotDefinedOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {

      return new Boolean(expr.getObject().getValue() == null);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the object is not defined.";
    }
  }

  /**
   * Determines the textual representation of the given object.
   */
  public class ToStringOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {

      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new OperandNotSupportedException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }

      return execToString(expr, ctx.getFormatHelper());
    }
    
    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Serves a textual representation of the object.";
    }
  }
  
  /**
   * The equal compare operation.
   */
  public class EqualOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
      
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      Object operandValue = interpreter.evalOperand(operand, ctx);
      
      Object objectValue = expr.getObject().getValue();
      if ((objectValue == null) || (operandValue == null)) {
        return new Boolean(objectValue == operandValue);
      }
      // equal check
      return new Boolean(objectValue.equals(operandValue));
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Compares the object with the operand for equality.";
    }
  }
  
  /**
   * The 'not equal' compare operation.
   */
  public class NotEqualOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
      
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      Object operandValue = interpreter.evalOperand(operand, ctx);
      
      Object objectValue = expr.getObject().getValue();
      if ((objectValue == null) || (operandValue == null)) {
        return new Boolean(objectValue != operandValue);
      }
      // not equal check
      return new Boolean(!objectValue.equals(operandValue));
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Compares the object with the operand for inequality.";
    }
  }

  /**
   * The 'instanceof' operation.
   */
  public class InstanceofOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
      
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      String operandValue = interpreter.evalStringOperand(operand, ctx);
      
      Object objectValue = expr.getObject().getValue();
      if (objectValue == null) {
        return new Boolean(false);
      }
      
      try {
        Class operandClass = Class.forName(operandValue);
        // instanceof check
        return new Boolean(operandClass.isInstance(objectValue));
      } catch (ClassNotFoundException ex) {
        throw new OperandException("No class definition found for operand '" + operandValue + "' at operation: " + 
                                   expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Compares the object with the operand for inequality.";
    }
  }

  /** The expression interpreter. */
  protected ExpressionInterpreter interpreter;
  /** The operation map. */
  protected Map operations = new HashMap();

  /**
   * Executes the toString operation.
   * @param expr The expression, holding the object to get as string.
   * @param formatHelper The format helper to use.
   * @return The textual representation of the expression value.
   */
  protected String execToString(ExpressionTuple expr, FormatHelper formatHelper) {
    Object value = expr.getObject().getValue();
    if (value == null) {
      return "null";
    }
    return value.toString();
  }
  
  /**
   * Constructs a BaseType instace with specific attributes.
   * @param interpreter The expression interpreter.
   */
  public BaseType(ExpressionInterpreter interpreter) {
    this.interpreter = interpreter;
    // adding global operations
    addOperation("isDefined", new ObjectDefinedOperation());
    addOperation("notDefined", new ObjectNotDefinedOperation());
    addOperation("toString", new ToStringOperation());
    ExpressionOperation op = new EqualOperation();
    addOperation("==", op);
    addOperation("eq", op);
    op = new NotEqualOperation();
    addOperation("!=", op);
    addOperation("ne", op);
    addOperation("instanceof", new InstanceofOperation());
  }

  /**
   * @see org.ujac.util.exi.ExpressionType#getOperation(org.ujac.util.exi.Operation)
   */
  public ExpressionOperation getOperation(Operation operation) {
    return (ExpressionOperation) operations.get(operation.getName());
  }

  /**
   * Gets all registered operation names.
   * @return The names of the operations, registerd with this type handler.
   */
  public Set getOperationNames() {
    return operations.keySet();
  }

  /**
   * Gets all registered operations.
   * @return The operations, registerd with this type handler.
   */
  public Map getOperations() {
    return operations;
  }

  /**
   * Adds an operation.
   * @param name The name of the operation.
   * @param operation The operation.
   */
  public void addOperation(String name, ExpressionOperation operation) {
    operations.put(name, operation);
  }

  /**
   * Removes an operation.
   * @param name The name of the operation.
   */
  public void removeOperation(String name) {
    operations.remove(name);
  }

  /**
   * Removes all operations.
   */
  public void removeAllOperations() {
    operations.clear();
  }

  /**
   * Evaluates the given expression tuple.
   * @param expr The expression tuple to evaluate. 
   * @param ctx The expression context.
   * @return The result of the tuple evaluation.
   * @exception ExpressionException If the evaluation failed.
   */
  public Object evalTuple(ExpressionTuple expr, ExpressionContext ctx)
    throws ExpressionException {
    
    ExpressionOperation op = getOperation(expr.getOperation());
    if (op == null) {
      if (expr.getObject().getValue() == null) {
        throw new ExpressionException("Unable to execute operation '" + expr.getOperation() + "' on null value at expression '" + expr.getCode() + "'.");
      }
      throw new ExpressionException("Operation '" + expr.getOperation() + "' is not supported for type " + getType().getName() + " at expression '" + expr.getCode() + "'.");
    }
    return op.evaluate(expr, ctx);
  }

  /**
   * Gets the type name of the expression type.
   * @return The name of the class the expression type is valid for.
   */
  public String getTypeName() {
    Class type = getType();
    if (type.isArray()) {
      return type.getComponentType().getName() + "[]";
    }
    return type.getName();
  }

  /**
   * Gets the alias of the expression type.
   * @return The type alias.
   */
  public String getAlias() {
    return null;
  }

  /**
   * Gets a description for the operation.
   * @return The item's description.
   */
  public String getDescription() {
    return "A type handler for " + getTypeName() + " values.";
  }

  /**
   * Casts the given value into the handler-specific type.
   * @param value The value to cast.
   * @return The transformed value.
   * @exception TypeCastException In case the type cast failed.
   */
  public Object typeCast(Object value) throws TypeCastException {
    throw new TypeCastException("Type casts are not supported for type " + getType() + "!");
  }

}
