/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.exi.type;

import org.ujac.util.exi.BaseExpressionOperation;
import org.ujac.util.exi.ConditionResultHolder;
import org.ujac.util.exi.ExpressionContext;
import org.ujac.util.exi.Operand;
import org.ujac.util.exi.ExpressionTuple;
import org.ujac.util.exi.NoOperandException;
import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.ExpressionOperation;
import org.ujac.util.exi.ExpressionInterpreter;


/**
 * Name: BooleanType<br>
 * Description: An expression type handler for boolean values.<br>
 * 
 * @author lauerc
 */
public class BooleanType extends BaseType {

  /**
   * The equal compare operation for Booleans.
   */
  public class EqualOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + 
                                     expr.getOperation() + 
                                     " on object " + 
                                     expr.getObject() + "!");         
      }
      // getting operand
      boolean operandValue = interpreter.evalBooleanOperand(operand, ctx);
      // equal check
      return new Boolean(((Boolean) expr.getObject().getValue()).booleanValue() == operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Compares the object with the operand for equality.";
    }
  }

  /**
   * The 'not equal' compare operation for Booleans.
   */
  public class NotEqualOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + 
                                     expr.getOperation() + " on object " + 
                                     expr.getObject() + "!");         
      }
      // getting operand
      boolean operandValue = interpreter.evalBooleanOperand(operand, ctx);
      // not equal check
      return new Boolean(((Boolean) expr.getObject().getValue()).booleanValue() != operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Compares the object with the operand for inequality.";
    }
  }
  
  /**
   * The 'not' operation for Booleans.
   */
  public class NotOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new NoOperandException("No operand supported for operation: " + 
                                     expr.getOperation() + 
                                     " on object " + 
                                     expr.getObject() + "!");         
      }
      boolean value = ((Boolean) expr.getObject().getValue()).booleanValue();
      // returning negated value
      return new Boolean(!value);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Toggles the object's value: true becomes false and vice versa.";
    }
  }

  /**
   * The 'or' operation for Booleans.
   */
  public class OrOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + 
                                     expr.getOperation() + 
                                     " on object " + 
                                     expr.getObject() + "!");         
      }
      boolean value = ((Boolean) expr.getObject().getValue()).booleanValue();
      if (value) {
        // no further check required
        return Boolean.TRUE;
      }
      // returning operand value
      return new Boolean(interpreter.evalBooleanOperand(operand, ctx));
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Combines the object and the operand with the 'or' operation.";
    }
  }

  /**
   * The 'and' operation for Booleans.
   */
  public class AndOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + 
                                     expr.getOperation() + 
                                     " on object " + 
                                     expr.getObject() + "!");         
      }
      boolean value = ((Boolean) expr.getObject().getValue()).booleanValue();
      if (!value) {
        // no further check required
        return Boolean.FALSE;
      }
      // returning operand value
      return new Boolean(interpreter.evalBooleanOperand(operand, ctx));
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Combines the object and the operand with the 'and' operation.";
    }
  }

  /**
   * The '?' operation for Booleans.
   */
  public class ConditionOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + 
                                     expr.getOperation() + 
                                     " on object " + 
                                     expr.getObject() + "!");         
      }
      boolean value = ((Boolean) expr.getObject().getValue()).booleanValue();
      if (!value) {
        // no further check required
        return new ConditionResultHolder(false, null);
      }
      // returning operand value
      return new ConditionResultHolder(true, interpreter.evalOperand(operand, ctx));
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Combines the object and the operand with the 'and' operation.";
    }
  }

  /**
   * Constructs a BooleanType instance with no specific arguments.
   * @param interpreter
   */
  public BooleanType(ExpressionInterpreter interpreter) {
    super(interpreter);
    ExpressionOperation op = new EqualOperation();
    addOperation("==", op);
    addOperation("eq", op);
    op = new NotEqualOperation();
    addOperation("!=", op);
    addOperation("ne", op);
    op = new NotOperation();
    addOperation("not", op);
    op = new AndOperation();
    addOperation("&&", op);
    addOperation("and", op);
    op = new OrOperation();
    addOperation("||", op);
    addOperation("or", op);
    op = new ConditionOperation();
    addOperation("?", op);
  }

  /**
   * @see org.ujac.util.exi.ExpressionType#getType()
   */
  public Class getType() {
    return Boolean.class;
  }

  /**
   * Gets the alias of the expression type.
   * @return The tpye alias.
   */
  public String getAlias() {
    return "boolean";
  }

}
