/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.exi.type;

import java.text.Format;

import org.ujac.util.exi.BaseExpressionOperation;
import org.ujac.util.exi.ExpressionContext;
import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.ExpressionInterpreter;
import org.ujac.util.exi.ExpressionOperation;
import org.ujac.util.exi.ExpressionTuple;
import org.ujac.util.exi.NoOperandException;
import org.ujac.util.exi.Operand;

import org.ujac.util.text.FormatHelper;

/**
 * Name: CharType<br>
 * Description: An expression type handler for char values.<br>
 * 
 * @author lauerc
 */
public class CharType extends BaseType {

  /**
   * The equal compare operation for Chars.
   */
  public class EqualOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      char operandValue = interpreter.evalCharOperand(operand, ctx);
      // less equal check
      return new Boolean(((Character) expr.getObject().getValue()).charValue() == operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Compares the object with the operand for equality.";
    }
  }
  
  /**
   * The 'not equal' compare operation for Chars.
   */
  public class NotEqualOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      char operandValue = interpreter.evalCharOperand(operand, ctx);
      // less equal check
      return new Boolean(((Character) expr.getObject().getValue()).charValue() != operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Compares the object with the operand for inequality.";
    }
  }

  /**
   * The 'greater' compare operation for Chars.
   */
  public class GreaterOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      char operandValue = interpreter.evalCharOperand(operand, ctx);
      // less equal check
      return new Boolean(((Character) expr.getObject().getValue()).charValue() > operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the object is greater than the operand.";
    }
  }

  /**
   * The 'less' compare operation for Chars.
   */
  public class LessOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      char operandValue = interpreter.evalCharOperand(operand, ctx);
      // less equal check
      return new Boolean(((Character) expr.getObject().getValue()).charValue() < operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the object is smaller than the operand.";
    }
  }

  /**
   * The 'greater equal' compare operation for Chars.
   */
  public class GreaterEqualOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      char operandValue = interpreter.evalCharOperand(operand, ctx);
      // less equal check
      return new Boolean(((Character) expr.getObject().getValue()).charValue() >= operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the object is greater than or equal to the operand.";
    }
  }

  /**
   * The 'less equal' compare operation for Chars.
   */
  public class LessEqualOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      char operandValue = interpreter.evalCharOperand(operand, ctx);
      // less equal check
      return new Boolean(((Character) expr.getObject().getValue()).charValue() <= operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the object is smaller than or equal to the operand.";
    }
  }

  /**
   * The 'add' operation for Chars.
   */
  public class AddOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      String operandValue = interpreter.evalStringOperand(operand, ctx);
      // sum of object and operand
      StringBuffer buf = new StringBuffer();
      buf.append(((Character) expr.getObject().getValue()).charValue());
      buf.append(operandValue);
      return buf.toString();
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Calculates the sum of the object and the operand value.";
    }
  }

  /**
   * The 'min' operation for Chars.
   */
  public class MinOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      char operandValue = interpreter.evalCharOperand(operand, ctx);
      // determining minimum
      Character object = (Character) (expr.getObject().getValue());
      if (object.charValue() > operandValue) {
        return object;
      }
      return new Character(operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the minimum of the object and the operand value.";
    }
  }

  /**
   * The 'max' operation for Chars.
   */
  public class MaxOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      char operandValue = interpreter.evalCharOperand(operand, ctx);
      // determining minimum
      Character object = (Character) (expr.getObject().getValue());
      if (object.charValue() > operandValue) {
        return object;
      }
      return new Character(operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the maximum of the object and the operand value.";
    }
  }

  /**
   * Executes the toString operation.
   * @param expr The expression, holding the object to get as string.
   * @param formatHelper The format helper to use.
   * @return The textual representation of the expression value.
   */
  protected String execToString(ExpressionTuple expr, FormatHelper formatHelper) {
    Number value = (Number) expr.getObject().getValue();
    Format format = formatHelper.getIntegerFormat();
    if (format != null) {
      return format.format(value);
    }
    return value.toString();
  }
  
  /**
   * Constructs a CharType instance with no specific arguments.
   * @param interpreter
   */
  public CharType(ExpressionInterpreter interpreter) {
    super(interpreter);
    // adding operations for this type.
    ExpressionOperation op = new EqualOperation();
    addOperation("==", op);
    addOperation("eq", op);
    op = new NotEqualOperation();
    addOperation("!=", op);
    addOperation("ne", op);
    op = new GreaterOperation();
    addOperation(">", op);
    addOperation("gt", op);
    op = new GreaterEqualOperation();
    addOperation(">=", op);
    addOperation("ge", op);
    op = new LessOperation();
    addOperation("<", op);
    addOperation("lt", op);
    op = new LessEqualOperation();
    addOperation("<=", op);
    addOperation("le", op);
    op = new AddOperation();
    addOperation("+", op);
    addOperation("concat", op);
    
    addOperation("min", new MinOperation());
    addOperation("max", new MaxOperation());
  }

  /**
   * @see org.ujac.util.exi.ExpressionType#getType()
   */
  public Class getType() {
    return Character.class;
  }

  /**
   * Gets the alias of the expression type.
   * @return The tpye alias.
   */
  public String getAlias() {
    return "char";
  }

}
