/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.exi.type;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Collection;

import org.ujac.util.BeanException;
import org.ujac.util.BeanUtils;

import org.ujac.util.exi.BaseExpressionOperation;
import org.ujac.util.exi.ExpressionContext;
import org.ujac.util.exi.Operand;
import org.ujac.util.exi.ExpressionTuple;
import org.ujac.util.exi.NoOperandException;
import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.ExpressionOperation;
import org.ujac.util.exi.ExpressionInterpreter;
import org.ujac.util.exi.OperandNotSupportedException;

/**
 * Name: CollectionType<br>
 * Description: An expression type handler for Collection values.
 * 
 * @author lauerc
 */
public class CollectionType extends BaseType {

  /**
   * The 'isEmpty' operation for Collections.
   */
  public class IsEmptyOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new OperandNotSupportedException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }

      Collection col = (Collection) (expr.getObject().getValue());
      return new Boolean(col.isEmpty());
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Tells whether the collection is empty or not.";
    }
  }

  /**
   * The 'notEmpty' operation for Collections.
   */
  public class NotEmptyOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new OperandNotSupportedException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }

      Collection col = (Collection) (expr.getObject().getValue());
      return new Boolean(!col.isEmpty());
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the collection is not empty.";
    }
  }

  /**
   * The 'size' operation for Collections.
   */
  public class SizeOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new OperandNotSupportedException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }

      Collection col = (Collection) (expr.getObject().getValue());
      return new Integer(col.size());
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Determines the size of the collection.";
    }
  }

  /**
   * The 'contains' operation for Collections.
   */
  public class ContainsOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      Object operandValue = interpreter.evalOperand(operand, ctx);

      Collection col = (Collection) (expr.getObject().getValue());
      return new Boolean(col.contains(operandValue));
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the collection contains the operand value.";
    }
  }
  
  /**
   * The 'notContains' operation for Collections.
   */
  public class NotContainsOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      Object operandValue = interpreter.evalOperand(operand, ctx);

      Collection col = (Collection) (expr.getObject().getValue());
      return new Boolean(!col.contains(operandValue));
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the collection not contains the operand value.";
    }
  }

  /**
   * The 'get' operation for Collections.
   */
  public class GetOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
      
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      int operandValue = interpreter.evalIntOperand(operand, ctx, false);

      try {
        List list = (List) (expr.getObject().getValue());
        return list.get(operandValue);
      } catch (ClassCastException ex) {
        throw new ExpressionException("The operation " + expr.getOperation() + " is only supported for List-type collections."); 
      }
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets an element from the collection. This operation is only implemented for List instances";
    }
  }

  /**
   * Name: BeanComparator<br>
   * Description: Compares generic beans.
   */
  private static class BeanComparator implements Comparator {

    /** The name of the field to compare. */ 
    private String fieldName = null;
    
    /**
     * Constructs a BeanComparator instance with specific attributes.
     * @param fieldName The name of the field to compare.
     */
    public BeanComparator(String fieldName) {
      this.fieldName = fieldName;
    }
    
    /**
     * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
     */
    public int compare(Object o1, Object o2) {
      Comparable c1 = null;
      Comparable c2 = null;
      if (fieldName != null) {
        try {
          c1 = (Comparable) BeanUtils.getProperty(o1, fieldName);
          c2 = (Comparable) BeanUtils.getProperty(o2, fieldName);
        } catch (BeanException ex) {
          throw new RuntimeException(ex.getMessage(), ex);
        }
      } else {
        c1 = (Comparable) o1;
        c2 = (Comparable) o2;
      }
      return c1.compareTo(c2);
    }
    
  }

  /**
   * The 'sort' operation for Collections.
   */
  public class SortOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
      
      String sortByField = null;
      Operand operand = expr.getOperand();
      if (operand != null) {
        // getting operand
        Object sortByValue = interpreter.evalOperand(operand, ctx).toString();
        if (sortByValue != null) {
          sortByField = sortByValue.toString();   
        }
      }

      // sorting the item list
      Collection list = (Collection) (expr.getObject().getValue());
      List sortList = new ArrayList();
      sortList.addAll(list);
      Collections.sort(sortList, new BeanComparator(sortByField));
      return sortList;
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Sorts the collection. If an operand is given, it sorts the collection " +
          "according to the bean field name the operand specifies.";
    }
  }

  /**
   * Constructs a CollectionType instance with specific attributes.
   * @param interpreter The expression interpreter.
   */
  public CollectionType(ExpressionInterpreter interpreter) {
    super(interpreter);
    // adding operations for this type.
    ExpressionOperation op = new IsEmptyOperation();
    addOperation("isEmpty", op);
    op = new NotEmptyOperation();
    addOperation("notEmpty", op);
    op = new SizeOperation();
    addOperation("size", op);
    op = new ContainsOperation();
    addOperation("contains", op);
    op = new NotContainsOperation();
    addOperation("notContains", op);
    op = new GetOperation();
    addOperation(".", op);
    addOperation("[]", op);
    addOperation("get", op);
    op = new SortOperation();
    addOperation("sort", op);
  }

  /**
   * @see org.ujac.util.exi.ExpressionType#getType()
   */
  public Class getType() {
    return Collection.class;
  }

}
