/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.exi.type;

import org.ujac.util.BeanException;
import org.ujac.util.BeanUtils;
import org.ujac.util.exi.BaseExpressionOperation;
import org.ujac.util.exi.ExpressionContext;
import org.ujac.util.exi.Operand;
import org.ujac.util.exi.ExpressionTuple;
import org.ujac.util.exi.NoOperandException;
import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.ExpressionOperation;
import org.ujac.util.exi.ExpressionInterpreter;
import org.ujac.util.exi.OperandException;


/**
 * Name: DefaultType<br>
 * Description: The default expression type handler. This one is used if 
 *    no other handler is applicable<br>
 * 
 * @author lauerc
 */
public class DefaultType extends BaseType {

  /**
   * The reference operation for beans.
   */
  public class ReferenceOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {

      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting the property name.
      String operandValue = interpreter.evalStringOperand(operand, ctx);
      
      Operand object = expr.getObject();
      Object objectValue = object.getValue();
      if (objectValue == null) {
        // Avoiding NullPointerException
        throw new OperandException("Unable to resolve reference " + expr.getObject() + " for null value at expression " + expr.toString() + "!");         
      }
      try {
        // getting the desired property
        return BeanUtils.getProperty(objectValue, operandValue);
      } catch (BeanException ex) {
        throw new OperandException("Failed to resolve property '" + operandValue + " at expression '" + expr.getCode() + "': " + ex.getMessage(), ex);
      }
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Accesses the object's property specified by the operand.";
    }
  }

  /**
   * The property defined check operation for beans.
   */
  public class PropertyDefinedOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {

      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      String operandValue = interpreter.evalStringOperand(operand, ctx);
      Object objectValue = expr.getObject().getValue();
      Class objectClass = objectValue.getClass();

      try {
        BeanUtils.getPropertyType(objectClass, operandValue, false);
        return Boolean.TRUE;
      } catch (BeanException ex) {
        return Boolean.FALSE;
      }
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks it the object contains a property that matches the name, specified by the operand.";
    }
  }

  /**
   * Constructs a DefaultType instance with no specific arguments.
   * @param interpreter The expression interpreter.
   */
  public DefaultType(ExpressionInterpreter interpreter) {
    super(interpreter);
    // adding operations for this type.
    ExpressionOperation ref = new ReferenceOperation(); 
    addOperation(".", ref);
    addOperation("->", ref);
    addOperation("[]", ref);
    addOperation("getField", ref);
    addOperation("propertyDefined", new PropertyDefinedOperation());
  }

  /**
   * @see org.ujac.util.exi.ExpressionType#getType()
   */
  public Class getType() {
    return Object.class;
  }

}
