/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.exi.type;

import java.text.Format;

import org.ujac.util.exi.BaseExpressionOperation;
import org.ujac.util.exi.ExpressionContext;
import org.ujac.util.exi.Operand;
import org.ujac.util.exi.ExpressionTuple;
import org.ujac.util.exi.NoOperandException;
import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.ExpressionOperation;
import org.ujac.util.exi.ExpressionInterpreter;
import org.ujac.util.exi.OperandNotSupportedException;
import org.ujac.util.exi.TypeCastException;

import org.ujac.util.text.FormatHelper;


/**
 * Name: LongType<br>
 * Description: An expression type handler for long values.<br>
 * 
 * @author lauerc
 */
public class LongType extends BaseNumberType {

  /**
   * The equal compare operation for Longs.
   */
  public class EqualOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      long operandValue = interpreter.evalLongOperand(operand, ctx, false);
      // equal check
      return new Boolean(((Number) expr.getObject().getValue()).longValue() == operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Compares the object with the operand for equality.";
    }
  }
	
  /**
   * The 'not equal' compare operation for Longs.
   */
  public class NotEqualOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      long operandValue = interpreter.evalLongOperand(operand, ctx, false);
      // not equal check
      return new Boolean(((Number) expr.getObject().getValue()).longValue() != operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Compares the object with the operand for inequality.";
    }
  }

  /**
   * The 'greater' compare operation for Longs.
   */
  public class GreaterOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      long operandValue = interpreter.evalLongOperand(operand, ctx, false);
      // greater check
      return new Boolean(((Number) expr.getObject().getValue()).longValue() > operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the object is greater than the operand.";
    }
  }

  /**
   * The 'less' compare operation for Longs.
   */
  public class LessOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      long operandValue = interpreter.evalLongOperand(operand, ctx, false);
      // less check
      return new Boolean(((Number) expr.getObject().getValue()).longValue() < operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the object is smaller than the operand.";
    }
  }

  /**
   * The 'greater equal' compare operation for Longs.
   */
  public class GreaterEqualOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      long operandValue = interpreter.evalLongOperand(operand, ctx, false);
      // greater equal check
      return new Boolean(((Number) expr.getObject().getValue()).longValue() >= operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the object is greater than or equal to the operand.";
    }
  }

  /**
   * The 'less equal' compare operation for Longs.
   */
  public class LessEqualOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      long operandValue = interpreter.evalLongOperand(operand, ctx, false);
      // less equal check
      return new Boolean(((Number) expr.getObject().getValue()).longValue() <= operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the object is smaller than or equal to the operand.";
    }
  }

  /**
   * The 'add' operation for Longs.
   */
  public class AddOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      long operandValue = interpreter.evalLongOperand(operand, ctx, false);
      // sum of object and operand
      return new Long(((Number) expr.getObject().getValue()).longValue() + operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Calculates the sum of the object and the operand value.";
    }
  }

  /**
   * The 'sub' operation for Longs.
   */
  public class SubOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      long operandValue = interpreter.evalLongOperand(operand, ctx, false);
      // difference of object and operand
      return new Long(((Number) expr.getObject().getValue()).longValue() - operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Calculates the difference of the object and the operand value.";
    }
  }

  /**
   * The 'mul' operation for Longs.
   */
  public class MulOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      long operandValue = interpreter.evalLongOperand(operand, ctx, false);
      // product of object and operand
      return new Long(((Number) expr.getObject().getValue()).longValue() * operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Calculates the product of the object and the operand value.";
    }
  }

  /**
   * The 'div' operation for Longs.
   */
  public class DivOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      long operandValue = interpreter.evalLongOperand(operand, ctx, false);
      // division result of object through operand
      return new Long(((Number) expr.getObject().getValue()).longValue() / operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Calculates the quotient of the object and the operand value.";
    }
  }

  /**
   * The 'mod' operation for Longs.
   */
  public class ModOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
    throws ExpressionException {
      
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      long operandValue = interpreter.evalLongOperand(operand, ctx, false);
      // division result of object through operand
      return new Long(((Number) expr.getObject().getValue()).longValue() % operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Calculates the modulo of the object and the operand value.";
    }
  }

  /**
   * The 'min' operation for Longs.
   */
  public class MinOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      long operandValue = interpreter.evalLongOperand(operand, ctx, false);
      // determining minimum
      Number object = (Number) (expr.getObject().getValue());
      if (object.longValue() < operandValue) {
        return object;
      }
      return new Long(operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the minimum of the object and the operand value.";
    }
  }

  /**
   * The 'max' operation for Longs.
   */
  public class MaxOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      long operandValue = interpreter.evalLongOperand(operand, ctx, false);
      // determining minimum
      Number object = (Number) (expr.getObject().getValue());
      if (object.longValue() > operandValue) {
        return object;
      }
      return new Long(operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the maximum of the object and the operand value.";
    }
  }

  /**
   * The 'abs' operation for Longs.
   */
  public class AbsOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new OperandNotSupportedException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // determining absolute value
      Number object = (Number) (expr.getObject().getValue());
      return new Long(Math.abs(object.longValue()));
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Determines the object's absolute value.";
    }
  }
  
  /**
   * Executes the toString operation.
   * @param expr The expression, holding the object to get as string.
   * @param formatHelper The format helper to use.
   * @return The textual representation of the expression value.
   */
  protected String execToString(ExpressionTuple expr, FormatHelper formatHelper) {
    Number value = (Number) expr.getObject().getValue();
    Format format = formatHelper.getIntegerFormat();
    if (format != null) {
      return format.format(value);
    }
    return value.toString();
  }
  
  /**
   * Constructs a LongType instance with no specific arguments.
   * @param interpreter
   */
  public LongType(ExpressionInterpreter interpreter) {
    super(interpreter);
    // adding operations for this type.
    ExpressionOperation op = new EqualOperation();
    addOperation("==", op);
    addOperation("eq", op);
    op = new NotEqualOperation();
    addOperation("!=", op);
    addOperation("ne", op);
    op = new GreaterOperation();
    addOperation(">", op);
    addOperation("gt", op);
    op = new GreaterEqualOperation();
    addOperation(">=", op);
    addOperation("ge", op);
    op = new LessOperation();
    addOperation("<", op);
    addOperation("lt", op);
    op = new LessEqualOperation();
    addOperation("<=", op);
    addOperation("le", op);
    op = new AddOperation();
    addOperation("+", op);
    addOperation("add", op);
    op = new SubOperation();
    addOperation("-", op);
    addOperation("sub", op);
    op = new MulOperation();
    addOperation("*", op);
    addOperation("mul", op);
    op = new DivOperation();
    addOperation("/", op);
    addOperation("div", op);
    op = new ModOperation();
    addOperation("%", op);
    addOperation("mod", op);
    
    addOperation("min", new MinOperation());
    addOperation("max", new MaxOperation());
    addOperation("abs", new AbsOperation());
  }

  /**
   * @see org.ujac.util.exi.ExpressionType#getType()
   */
  public Class getType() {
    return Long.class;
  }

  /**
   * Gets the alias of the expression type.
   * @return The tpye alias.
   */
  public String getAlias() {
    return "long";
  }

  /**
   * Gets the negative value for the given number.
   * @param number The number to negate.
   * @return The negated value of the given number.
   */
  public Number getNegative(Number number) {
    return new Long(number.longValue() * -1L);
  }

  /**
   * Casts the given value into the handler-specific type.
   * @param value The value to cast.
   * @return The transformed value.
   * @exception TypeCastException In case the type cast failed.
   */
  public Object typeCast(Object value) throws TypeCastException {
    if (value instanceof String) {
      String strValue = (String) value;
      try {
        long longVal = Long.parseLong(strValue);
        return new Long(longVal);
      } catch (NumberFormatException ex) {
        throw new TypeCastException("Unable to parse long value from '" + strValue + "'.");
      }
    }
    if (value instanceof Number) {
      return new Long(((Number) value).longValue());
    }
    throw new TypeCastException("Unable to transform value '" + value + "' into a long.");
  }

}
