/*
 * Copyright (C) 2004 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.exi.type;

import org.ujac.util.exi.BaseExpressionOperation;
import org.ujac.util.exi.ExpressionContext;
import org.ujac.util.exi.NullType;
import org.ujac.util.exi.Operand;
import org.ujac.util.exi.ExpressionTuple;
import org.ujac.util.exi.NoOperandException;
import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.ExpressionOperation;
import org.ujac.util.exi.ExpressionInterpreter;
import org.ujac.util.exi.OperandException;
import org.ujac.util.exi.OperandNotSupportedException;

/**
 * Name: NullValueType<br>
 * Description: The expression type handler for null values. This one is used if the object value 
 *    is null<br>
 * 
 * @author lauerc
 */
public class NullValueType extends BaseType {

  /**
   * The reference operation for beans.
   */
  public class ReferenceOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {

      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " +
                                     expr.getOperation() + " on object " + 
                                     expr.getObject() + "!");         
      }
      
      // Avoiding NullPointerException
      throw new OperandException("Unable to resolve reference " +
                                 expr.getObject() + " for null value at expression " + 
                                 expr.toString() + "!");         
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Accesses the object's property specified by the operand.";
    }
  }

  /**
   * The property defined check operation for beans.
   */
  public class PropertyDefinedOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {

      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " +
                                     expr.getOperation() + " on object " + 
                                     expr.getObject() + "!");         
      }
      return Boolean.FALSE;
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks it the object contains a property that matches the name, specified by the operand.";
    }
  }

  /**
   * The 'isEmpty' operation for arrays.
   */
  public class IsEmptyOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
      
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new OperandNotSupportedException("No operand supported for operation: " + 
                                               expr.getOperation() + " on object " + 
                                               expr.getObject() + "!");         
      }
      return Boolean.TRUE;
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Tells whether the object is empty or not.";
    }
  }

  /**
   * The 'notEmpty' operation for arrays.
   */
  public class NotEmptyOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
      
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new OperandNotSupportedException("No operand supported for operation: " + 
                                               expr.getOperation() + " on object " + 
                                               expr.getObject() + "!");         
      }
      return Boolean.FALSE;
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the object is not empty.";
    }
  }

  /**
   * The 'size' operation for arrays.
   */
  public class SizeOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
      
      return new Integer(0);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Determines the size of the object (always 0).";
    }
  }

  /**
   * The 'format' operation for Numbers.
   */
  public class FormatOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " +
                                     expr.getOperation() + " on object " + 
                                     expr.getObject() + "!");         
      }
      return "";
    }

    
    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Formats the object using the format, specified by the operand (always '').";
    }
  }

  /**
   * The equal compare operation for Doubles.
   */
  public class EqualOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + 
                                     expr.getOperation() + 
                                     " on object " + 
                                     expr.getObject() + "!");         
      }
      // getting operand
      Object operandValue = interpreter.evalOperand(operand, ctx);
      if (operandValue instanceof Number) {
        return new Boolean(((Number) operandValue).doubleValue() == 0.0);
      }
      // equal check
      return new Boolean(expr.getObject().getValue() == operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Compares the object with the operand for equality.";
    }
  }
    
  /**
   * The 'not equal' compare operation for Doubles.
   */
  public class NotEqualOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + 
                                     expr.getOperation() + 
                                     " on object " + 
                                     expr.getObject() + "!");         
      }
      // getting operand
      Object operandValue = interpreter.evalOperand(operand, ctx);
      // equal check
      return new Boolean(expr.getObject().getValue() != operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Compares the object with the operand for inequality.";
    }
  }

  /**
   * The 'greater' compare operation for Doubles.
   */
  public class GreaterOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + 
                                     expr.getOperation() + 
                                     " on object " + 
                                     expr.getObject() + "!");         
      }
      // always false!
      return Boolean.FALSE;
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the object is greater than the operand.";
    }
  }

  /**
   * The 'less' compare operation for Doubles.
   */
  public class LessOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + 
                                     expr.getOperation() + 
                                     " on object " + 
                                     expr.getObject() + "!");         
      }
      // getting operand
      Object operandValue = interpreter.evalOperand(operand, ctx);
      // equal check
      return new Boolean(operandValue != null);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the object is smaller than the operand.";
    }
  }

  /**
   * The 'greater equal' compare operation for Doubles.
   */
  public class GreaterEqualOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + 
                                     expr.getOperation() + 
                                     " on object " + 
                                     expr.getObject() + "!");         
      }
      // getting operand
      Object operandValue = interpreter.evalOperand(operand, ctx);
      // equal check
      return new Boolean(operandValue == null);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the object is greater than or equal to the operand.";
    }
  }

  /**
   * The 'less equal' compare operation for Doubles.
   */
  public class LessEqualOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + 
                                     expr.getOperation() + 
                                     " on object " + 
                                     expr.getObject() + "!");         
      }
      // equal check
      return Boolean.TRUE;
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the object is smaller than or equal to the operand.";
    }
  }

  /**
   * The 'add' operation for null values.
   */
  public class AddOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + 
                                     expr.getOperation() + 
                                     " on object " + 
                                     expr.getObject() + "!");         
      }
      // getting operand
      double operandValue = interpreter.evalDoubleOperand(operand, ctx, false);
      // sum of object and operand
      return new Double(operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Calculates the sum of the object and the operand value.";
    }
  }

  /**
   * The 'sub' operation for null values.
   */
  public class SubOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + 
                                     expr.getOperation() + 
                                     " on object " + 
                                     expr.getObject() + "!");         
      }
      // getting operand
      double operandValue = interpreter.evalDoubleOperand(operand, ctx, false);
      // difference of object and operand
      return new Double(-operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Calculates the difference of the object and the operand value.";
    }
  }

  /**
   * The 'mul' operation for null values.
   */
  public class MulOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + 
                                     expr.getOperation() + 
                                     " on object " + 
                                     expr.getObject() + "!");         
      }
      // product of object and operand
      return new Double(0.0);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Calculates the product of the object and the operand value.";
    }
  }

  /**
   * The 'div' operation for null values.
   */
  public class DivOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + 
                                     expr.getOperation() + 
                                     " on object " + 
                                     expr.getObject() + "!");         
      }
      return new Double(0.0);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Calculates the quotient of the object and the operand value.";
    }
  }

  /**
   * The 'min' operation for null values.
   */
  public class MinOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + 
                                     expr.getOperation() + 
                                     " on object " + 
                                     expr.getObject() + "!");         
      }
      // getting operand
      double operandValue = interpreter.evalDoubleOperand(operand, ctx, false);
      // determining minimum
      if (operandValue < 0.0) {
        return new Double(0.0);
      }
      return new Double(0.0);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the minimum of the object and the operand value.";
    }
  }

  /**
   * The 'max' operation for null values.
   */
  public class MaxOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + 
                                     expr.getOperation() + 
                                     " on object " + 
                                     expr.getObject() + "!");         
      }
      // getting operand
      double operandValue = interpreter.evalDoubleOperand(operand, ctx, false);
      return new Double(operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the maximum of the object and the operand value.";
    }
  }
  
  /**
   * The 'abs' operation for null values.
   */
  public class AbsOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new OperandNotSupportedException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // determining absolute value
      return new Double(0.0);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Determines the object's absolute value.";
    }
  }

  
  /**
   * Constructs a NullValueType instance with no specific arguments.
   * @param interpreter The expression interpreter.
   */
  public NullValueType(ExpressionInterpreter interpreter) {
    super(interpreter);
    // adding operations for this type.
    ExpressionOperation op = new ReferenceOperation(); 
    addOperation(".", op);
    addOperation("->", op);
    addOperation("[]", op);
    addOperation("get", op);
    addOperation("getField", op);
    addOperation("propertyDefined", new PropertyDefinedOperation());
    op = new IsEmptyOperation();
    addOperation("isEmpty", op);
    addOperation("noRows", op);
    op = new NotEmptyOperation();
    addOperation("notEmpty", op);
    addOperation("hasRows", op);
    op = new SizeOperation();
    addOperation("size", op);
    addOperation("rowCount", op);
    addOperation("format", new FormatOperation());
    
    op = new EqualOperation();
    addOperation("==", op);
    addOperation("eq", op);
    op = new NotEqualOperation();
    addOperation("!=", op);
    addOperation("ne", op);
    op = new GreaterOperation();
    addOperation(">", op);
    addOperation("gt", op);
    op = new GreaterEqualOperation();
    addOperation(">=", op);
    addOperation("ge", op);
    op = new LessOperation();
    addOperation("<", op);
    addOperation("lt", op);
    op = new LessEqualOperation();
    addOperation("<=", op);
    addOperation("le", op);

    op = new AddOperation();
    addOperation("+", op);
    addOperation("add", op);
    op = new SubOperation();
    addOperation("-", op);
    addOperation("sub", op);
    op = new MulOperation();
    addOperation("*", op);
    addOperation("mul", op);
    op = new DivOperation();
    addOperation("/", op);
    addOperation("div", op);
   
    addOperation("min", new MinOperation());
    addOperation("max", new MaxOperation());
    addOperation("abs", new AbsOperation());
  }
  
  /**
   * @see org.ujac.util.exi.ExpressionType#getType()
   */
  public Class getType() {
    return NullType.class;
  }

  /**
   * Gets the type name of the expression type.
   * @return The name of the class the expression type is valid for.
   */
  public String getTypeName() {
    return "null Reference";
  }

  /**
   * Gets a description for the type handler.
   * @return The type handler description.
   */
  public String getDescription() {
    return "A special handler for null values to avoid some error messages, " +
        "resulting from not declared operations at the default handler.";
  }

}
