/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.exi.type;

import java.util.ResourceBundle;

import org.ujac.util.exi.BaseExpressionOperation;
import org.ujac.util.exi.ExpressionContext;
import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.ExpressionInterpreter;
import org.ujac.util.exi.ExpressionOperation;
import org.ujac.util.exi.ExpressionTuple;
import org.ujac.util.exi.NoOperandException;
import org.ujac.util.exi.Operand;
import org.ujac.util.exi.OperandNotSupportedException;

/**
 * Name: ResourceBundleType<br>
 * Description: An expression type handler for ResourceBundle values.
 * 
 * @author lauerc
 */
public class ResourceBundleType extends BaseType {

  /**
   * The 'isEmpty' operation for ResourceBundles.
   */
  public class IsEmptyOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new OperandNotSupportedException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }

      ResourceBundle bundle = (ResourceBundle) (expr.getObject().getValue());
      return new Boolean(!bundle.getKeys().hasMoreElements());
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Tells whether the bundle is empty or not.";
    }
  }

  /**
   * The 'notEmpty' operation for ResourceBundles.
   */
  public class NotEmptyOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new OperandNotSupportedException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }

      ResourceBundle bundle = (ResourceBundle) (expr.getObject().getValue());
      return new Boolean(bundle.getKeys().hasMoreElements());
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the bundle is not empty.";
    }
  }

  /**
   * The 'containsKey' operation for ResourceBundles.
   */
  public class ContainsKeyOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      String operandValue = interpreter.evalStringOperand(operand, ctx);

      ResourceBundle bundle = (ResourceBundle) (expr.getObject().getValue());
      return new Boolean(bundle.getString(operandValue) != null);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the bundle contains the key, specified by the operand value.";
    }
  }
  
  /**
   * The 'notContainsKey' operation for ResourceBundles.
   */
  public class NotContainsKeyOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      String operandValue = interpreter.evalStringOperand(operand, ctx);

      ResourceBundle bundle = (ResourceBundle) (expr.getObject().getValue());
      return new Boolean(bundle.getString(operandValue) == null);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the bundle not contains the key, specified by the operand.";
    }
  }

  /**
   * The 'get' operation for ResourceBundles.
   */
  public class GetOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      String operandValue = interpreter.evalStringOperand(operand, ctx);

      ResourceBundle bundle = (ResourceBundle) (expr.getObject().getValue());
      return bundle.getString(operandValue);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets an element from the bundle by its name.";
    }
  }

  /**
   * Constructs a ResourceBundleType instance with specific attributes.
   * @param interpreter The expression interpreter.
   */
  public ResourceBundleType(ExpressionInterpreter interpreter) {
    super(interpreter);
    // adding operations for this type.
    ExpressionOperation op = new IsEmptyOperation();
    addOperation("isEmpty", op);
    op = new NotEmptyOperation();
    addOperation("notEmpty", op);
    op = new ContainsKeyOperation();
    addOperation("containsKey", op);
    op = new NotContainsKeyOperation();
    addOperation("notContainsKey", op);
    op = new GetOperation();
    addOperation(".", op);
    addOperation("[]", op);
    addOperation("get", op);
  }

  /**
   * @see org.ujac.util.exi.ExpressionType#getType()
   */
  public Class getType() {
    return ResourceBundle.class;
  }

}
