/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.exi.type;

import org.ujac.util.BeanUtils;

import org.ujac.util.exi.BaseExpressionOperation;
import org.ujac.util.exi.ExpressionContext;
import org.ujac.util.exi.Operand;
import org.ujac.util.exi.ExpressionTuple;
import org.ujac.util.exi.NoOperandException;
import org.ujac.util.exi.ExpressionOperation;
import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.ExpressionInterpreter;
import org.ujac.util.exi.OperandException;
import org.ujac.util.exi.OperandNotSupportedException;

import org.ujac.util.table.Column;
import org.ujac.util.table.GroupRow;
import org.ujac.util.table.OperationNotSupportedException;
import org.ujac.util.table.Row;
import org.ujac.util.table.ColumnNotDefinedException;
import org.ujac.util.table.Table;
import org.ujac.util.table.TypeMismatchException;

/**
 * Name: TableType<br>
 * Description: An expression type handler for DataTable values.<br>
 * 
 * @author lauerc
 */
public class RowType extends BaseType {

  /**
   * The 'hasRows' operation for Rows.
   */
  public class HasRowsOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new OperandNotSupportedException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }

      Row row = (Row) (expr.getObject().getValue());
      return new Boolean(!row.getTable().isEmpty());
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the table that holds the row, has rows.";
    }
  }

  /**
   * The 'noRows' operation for Rows.
   */
  public class NoRowsOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new OperandNotSupportedException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }

      Row row = (Row) (expr.getObject().getValue());
      return new Boolean(row.getTable().isEmpty());
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the table that holds the row, has no rows.";
    }
  }

  /**
   * The 'rowCount' operation for Rows.
   */
  public class RowCountOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new OperandNotSupportedException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }

      try {
        Row row = (Row) (expr.getObject().getValue());
        return new Integer(row.getTable().getRowCount());
      } catch (OperationNotSupportedException ex) {
        throw new ExpressionException(ex.getMessage());
      }
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the number of rows from the table that holds the row.";
    }
  }

  /**
   * The 'columnCount' operation for Rows.
   */
  public class ColumnCountOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new OperandNotSupportedException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }

      Row row = (Row) (expr.getObject().getValue());
      return new Integer(row.getTable().getColumnCount());
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the number of columns from the table that holds the row.";
    }
  }

  /**
   * The 'hasColumn' operation for Rows.
   */
  public class HasColumnOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      String operandValue = interpreter.evalStringOperand(operand, ctx);

      Row row = (Row) (expr.getObject().getValue());
      try {
        return new Boolean(row.getTable().getColumn(operandValue) != null);
      } catch (ColumnNotDefinedException ex) {
        return Boolean.FALSE;
      }
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks it the table that holds the row contains a column that matches the name, specified by the operand.";
    }
  }
  
  /**
   * The reference operation for Rows.
   */
  public class ReferenceOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      String operandValue = interpreter.evalStringOperand(operand, ctx);
      if (operandValue == null) {
        throw new OperandException("Row index evaluated to null value: '" + operand + "'");
      }
      
      Row row = (Row) (expr.getObject().getValue());
      try {
        Table table = row.getTable();
        int colIdx = table.getColumnIndex(operandValue);
        return row.getObject(colIdx);
      } catch (ColumnNotDefinedException e) {
        if ("rowType".equals(operandValue)) {
          return row.getType();
        }
        if ("groupName".equals(operandValue)) {
          if (row instanceof GroupRow) {
            GroupRow gr = (GroupRow) row;
            return gr.getGroupName();
          }
          return null;
        }
        if ("startNewPage".equals(operandValue)) {
          return new Boolean(row.isStartNewPage());
        }
        if ("startsBlock".equals(operandValue)) {
          return new Boolean(row.isStartsBlock());
        }
        if ("endsBlock".equals(operandValue)) {
          return new Boolean(row.isEndsBlock());
        }
        
        StringBuffer msg = new StringBuffer("Unknown column '" + operandValue + "' for table at expression '" + expr.getCode() + "'. ");
        msg.append("Valid columns are: ");
        Table table = row.getTable();
        int numColumns = table.getColumnCount();
        try {
          for (int i = 0; i < numColumns; i++) {
            if (i > 0) {
              msg.append(", ");
            }
            msg.append(table.getColumn(i).getName());
          }
        } catch (ColumnNotDefinedException ex) {
          // this should never happen!
        }
        msg.append(".");
        throw new OperandException(msg.toString());
      }
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Accesses the field specified by the operand at this row.";
    }
  }
  
  /**
   * The formatValue operation for Rows.
   */
  public class FormatValueOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      String operandValue = interpreter.evalStringOperand(operand, ctx);

      Row row = (Row) (expr.getObject().getValue());
      Table table = row.getTable();
      try {
        if (BeanUtils.equals(operandValue, "groupName") && (row instanceof GroupRow)) {
          GroupRow gr = (GroupRow) row;
          return gr.getGroupName();
        }
        if (operandValue == null) {
          throw new OperandException("No valid operand given for operation: " + 
                                     expr.getOperation() + " on object " + 
                                     expr.getObject() + "! Quotes forgotten?");         
        }
        Column col = table.getColumn(operandValue);
        int colIdx = col.getIndex();
        if (!row.isVisible(colIdx)) {
          return "";
        }
        String result = row.getString(colIdx);
        if (result == null) {
          return "-";
        }
        return result;
      } catch (ColumnNotDefinedException e) {
        StringBuffer msg = new StringBuffer("Unknown column '" + operandValue + "' for table at expression '" + expr.getCode() + "'. ");
        msg.append("Valid columns are: ");
        int numColumns = table.getColumnCount();
        try {
          for (int i = 0; i < numColumns; i++) {
            if (i > 0) {
              msg.append(", ");
            }
            msg.append(table.getColumn(i).getName());
          }
        } catch (ColumnNotDefinedException ex) {
          // this should never happen!
        }
        msg.append(".");
        throw new OperandException(msg.toString());
      } catch (TypeMismatchException e) {
        throw new OperandException("Unable to format value of column '" + operandValue + "'.");
      }
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the formatted value of the field specified by the operand at this row.";
    }
  }
  /**
   * The row type operation for Rows.
   */
  public class RowTypeOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
    throws ExpressionException {
      
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new NoOperandException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }

      Row row = (Row) (expr.getObject().getValue());
      return row.getType();
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the type of the row.";
    }
  }

  /**
   * The field visible check operation for Rows.
   */
  public class IsVisibleOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
      
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      String operandValue = interpreter.evalStringOperand(operand, ctx);

      Row row = (Row) (expr.getObject().getValue());
      try {
        return new Boolean(row.isVisible(operandValue));
      } catch (ColumnNotDefinedException e) {
        throw new OperandException("Unknown column '" + operandValue + "' for table at expression '" + expr.getCode() + "'.");
      }
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks whether the field, specified by the operand is visible or not.";
    }
  }
  
  /**
   * Constructs a TableType instance with no specific arguments.
   * @param interpreter
   */
  public RowType(ExpressionInterpreter interpreter) {
    super(interpreter);
    // adding operations for this type.
    ExpressionOperation op = new ReferenceOperation();
    addOperation(".", op);
    addOperation("[]", op);
    addOperation("getField", op);
    op = new FormatValueOperation();
    addOperation("formatValue", op);
    op = new HasRowsOperation();
    addOperation("hasRows", op);
    op = new NoRowsOperation();
    addOperation("noRows", op);
    op = new RowCountOperation();
    addOperation("rowCount", op);
    op = new ColumnCountOperation();
    addOperation("columnCount", op);
    op = new HasColumnOperation();
    addOperation("hasColumn", op);
    op = new RowTypeOperation();
    addOperation("rowType", op);
    op = new IsVisibleOperation();
    addOperation("isVisible", op);
    op = new IsVisibleOperation();
    addOperation("isVisible", op);
  }

  /**
   * @see org.ujac.util.exi.ExpressionType#getType()
   */
  public Class getType() {
    return Row.class;
  }

}
