/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.exi.type;

import java.util.StringTokenizer;

import org.ujac.util.BeanUtils;
import org.ujac.util.StringUtils;
import org.ujac.util.exi.BaseExpressionOperation;
import org.ujac.util.exi.ExpressionContext;
import org.ujac.util.exi.Operand;
import org.ujac.util.exi.ExpressionTuple;
import org.ujac.util.exi.NoOperandException;
import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.ExpressionOperation;
import org.ujac.util.exi.ExpressionInterpreter;
import org.ujac.util.exi.OperandException;
import org.ujac.util.exi.OperandNotSupportedException;


/**
 * Name: StringType<br>
 * Description: An expression type handler for String values.<br>
 * 
 * @author lauerc
 */
public class StringType extends BaseType {

  /**
   * The equal compare operation for Strings.
   */
  public class EqualOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      String operandValue = interpreter.evalStringOperand(operand, ctx);
      // equal check
      return new Boolean(((String) expr.getObject().getValue()).equals(operandValue));
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Compares the object with the operand for equality.";
    }
  }
	
  /**
   * The 'not equal' compare operation for Strings.
   */
  public class NotEqualOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      String operandValue = interpreter.evalStringOperand(operand, ctx);
      // not equal check
      return new Boolean(!((String) expr.getObject().getValue()).equals(operandValue));
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Compares the object with the operand for inequality.";
    }
  }

  /**
   * The like compare operation for Strings.
   */
  public class LikeOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      String operandValue = interpreter.evalStringOperand(operand, ctx);
      int operandLength = operandValue.length();
      if (operandLength < 1) {
        return Boolean.FALSE;
      }
      return new Boolean(StringUtils.like(((String) expr.getObject().getValue()), operandValue));
    }
  }
  
  /**
   * The notLike compare operation for Strings.
   */
  public class NotLikeOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      String operandValue = interpreter.evalStringOperand(operand, ctx);
      int operandLength = operandValue.length();
      if (operandLength < 1) {
        return Boolean.TRUE;
      }
      if (operandValue.charAt(0) == '%') {
        if (operandValue.charAt(operandLength - 1) == '%') {
          String compareText = operandValue.substring(1, operandLength - 1);
          return new Boolean(((String) expr.getObject().getValue()).indexOf(compareText) < 0);
        }
        return new Boolean(!((String) expr.getObject().getValue()).endsWith(operandValue.substring(1)));
      } else if (operandValue.charAt(operandLength - 1) == '%') {
        String compareText = operandValue.substring(0, operandLength - 1);
        return new Boolean(!((String) expr.getObject().getValue()).startsWith(compareText));
      }
      // equal check
      return new Boolean(((String) expr.getObject().getValue()).indexOf(operandValue) < 0);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Compares the object with the operand for similarity. This operation behaves " +
          "similar to the 'like' operation, known from SQL. In case the operand starts and ends " +
          "with '%', text within the wildcard marks has to occur somewhere within the object. " +
          "In case the operand only starts with '%', the object has to end with the operand text. " +
          "In case the operand only ends with '%', the object has to start with the operand text. " +
          "If none of the cases above matches, this operation behaves like the 'equal' operation.";
    }
  }

  /**
   * The 'concat' operation for Strings.
   */
  public class ConcatOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      String operandValue = interpreter.evalStringOperand(operand, ctx);
      // sum of object and operand
      return new StringBuffer((String) (expr.getObject().getValue())).append(operandValue).toString();
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Concatenates the object and the operand.";
    }
  }

  /**
   * The 'trim' operation for Strings.
   */
  public class TrimOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new NoOperandException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting the length of the String.
      return ((String) expr.getObject().getValue()).trim();
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Trims the leading and trailing white spaces from the object.";
    }
  }

  /**
   * The length operation for Strings.
   */
  public class LengthOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new NoOperandException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting the length of the String.
      return new Integer(((String) expr.getObject().getValue()).length());
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the length of the object.";
    }
  }

  /**
   * The lowerCase operation for Strings.
   */
  public class LowerCaseOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new NoOperandException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting the lower case form of the String.
      return ((String) expr.getObject().getValue()).toLowerCase();
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the object as lower case string.";
    }
  }

  /**
   * The upperCase operation for Strings.
   */
  public class UpperCaseOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new NoOperandException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting the upper case form of the String.
      return ((String) expr.getObject().getValue()).toUpperCase();
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the object as upper case string.";
    }
  }

  /**
   * The capitalize operation for Strings.
   */
  public class CapitalizeOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new NoOperandException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting the upper case form of the String.
      return StringUtils.capitalize(((String) expr.getObject().getValue()));
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Raises the first character of the given String.";
    }
  }
  
  /**
   * The 'isEmpty' operation for Strings.
   */
  public class IsEmptyOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
      
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new OperandNotSupportedException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // checking if the length of the String is zero
      return new Boolean(((String) expr.getObject().getValue()).length() == 0);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the String has zero length.";
    }
  }

  /**
   * The 'notEmpty' operation for Strings.
   */
  public class NotEmptyOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
      
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new OperandNotSupportedException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // checking if the length of the String is zero
      return new Boolean(((String) expr.getObject().getValue()).length() > 0);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the String has no zero length.";
    }
  }

  /**
   * The 'substring' operation for Strings.
   */
  public class SubstringOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
      
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new OperandNotSupportedException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      String obj = (String) expr.getObject().getValue();
      // parsing operand
      int tIdx = 0;
      int start = 0;
      int end = 0;
      StringTokenizer tkn = new StringTokenizer(BeanUtils.toString(operand.getValue()), ":", true);
      while (tkn.hasMoreTokens()) {
        String t = tkn.nextToken();
        if (t.charAt(0) == ':') {
          ++tIdx;
          if (tIdx == 1) {
            ++tIdx;
          }
          continue;
        }
        if (tIdx == 0) {
          try {
            start = Integer.parseInt(t);
          } catch (NumberFormatException ex) {
            throw new OperandException("Illegal operand '" + operand.getValue() + "' for operation " + expr.getOperation() + " on object " + expr.getObject() + ", expected numeric values.");  
          }
        }
        if (tIdx == 2) {
          try {
            end = Integer.parseInt(t);
          } catch (NumberFormatException ex) {
            throw new OperandException("Illegal operand '" + operand.getValue() + "' for operation " + expr.getOperation() + " on object " + expr.getObject() + ", expected numeric values.");  
          }
        }
        ++tIdx;
      }
      if (end > 0) {
        return obj.substring(start, end);
      }
      return obj.substring(start);
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Returns a substring of the object the operation is applied to. The operand can be given in the forms '&lt;start&gt;', '&lt;start&gt;:&lt;length&gt;', ':length'.";
    }
  }
  
  /**
   * Constructs a StringType instance with specific arguments.
   * @param interpreter The expression interpreter.
   */
  public StringType(ExpressionInterpreter interpreter) {
    super(interpreter);
    // adding operations for this type.
    ExpressionOperation op = new EqualOperation();
    addOperation("==", op);
    addOperation("eq", op);
    op = new NotEqualOperation();
    addOperation("!=", op);
    addOperation("ne", op);
    op = new LikeOperation();
    addOperation("like", op);
    op = new NotLikeOperation();
    addOperation("notLike", op);
    op = new ConcatOperation();
    addOperation("+", op);
    addOperation("concat", op);
    addOperation("trim", new TrimOperation());
    addOperation("length", new LengthOperation());
    addOperation("lowerCase", new LowerCaseOperation());
    addOperation("upperCase", new UpperCaseOperation());
    addOperation("capitalize", new CapitalizeOperation());
    addOperation("isEmpty", new IsEmptyOperation());
    addOperation("notEmpty", new NotEmptyOperation());
    addOperation("substring", new SubstringOperation());
  }

  /**
   * @see org.ujac.util.exi.ExpressionType#getType()
   */
  public Class getType() {
    return String.class;
  }

  /**
   * Gets the alias of the expression type.
   * @return The tpye alias.
   */
  public String getAlias() {
    return "String";
  }

}
