/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.exi.type;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.ujac.util.UjacTypes;
import org.ujac.util.exi.BaseExpressionOperation;
import org.ujac.util.exi.ExpressionContext;
import org.ujac.util.exi.Operand;
import org.ujac.util.exi.ExpressionTuple;
import org.ujac.util.exi.NoOperandException;
import org.ujac.util.exi.ExpressionOperation;
import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.ExpressionInterpreter;
import org.ujac.util.exi.OperandException;
import org.ujac.util.exi.OperandNotSupportedException;

import org.ujac.util.table.Column;
import org.ujac.util.table.DataTable;
import org.ujac.util.table.LayoutHints;
import org.ujac.util.table.OperationNotSupportedException;
import org.ujac.util.table.Row;
import org.ujac.util.table.Table;
import org.ujac.util.table.RowNotDefinedException;
import org.ujac.util.table.ColumnNotDefinedException;
import org.ujac.util.table.TableException;
import org.ujac.util.table.TypeMismatchException;

/**
 * Name: TableType<br>
 * Description: An expression type handler for DataTable values.<br>
 * 
 * @author lauerc
 */
public class TableType extends BaseType {

  /**
   * The 'hasRows' operation for Tables.
   */
  public class HasRowsOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new OperandNotSupportedException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      
      Table table = (Table) (expr.getObject().getValue());
      return new Boolean(!table.isEmpty());
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the table, specified by the object has rows.";
    }
  }

  /**
   * The 'noRows' operation for Tables.
   */
  public class NoRowsOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new OperandNotSupportedException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }

      Table table = (Table) (expr.getObject().getValue());
      return new Boolean(table.isEmpty());
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks if the table, specified by the object has no rows.";
    }
  }

  /**
   * The 'rowCount' operation for Tables.
   */
  public class RowCountOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new OperandNotSupportedException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }

      try {
        Table table = (Table) (expr.getObject().getValue());
        return new Integer(table.getRowCount());
       } catch (OperationNotSupportedException ex) {
         throw new ExpressionException(ex.getMessage());
       }
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the number of rows from the table, specified by the object.";
    }
  }

  /**
   * The 'columnCount' operation for Tables.
   */
  public class ColumnCountOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new OperandNotSupportedException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }

      Table table = (Table) (expr.getObject().getValue());
      return new Integer(table.getColumnCount());
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the number of columns from the table, specified by the object.";
    }
  }

  /**
   * The 'columns' operation for Tables.
   */
  public class ColumnsOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new OperandNotSupportedException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }

      try {
        Table table = (Table) (expr.getObject().getValue());
        int numColumns = table.getColumnCount();
        List columns = new ArrayList();
        for (int i = 0; i < numColumns; i++) {
          columns.add(table.getColumn(i));
        }
        return columns;
      } catch (TableException ex) {
        throw new ExpressionException("Table processing failed: " + ex.getMessage(), ex);
      }
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the number of columns from the table, specified by the object.";
    }
  }

  /**
   * The 'visibleColumns' operation for Tables.
   */
  public class VisibleColumnsOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Table table = (Table) (expr.getObject().getValue());

      Operand operand = expr.getOperand();
      if (operand != null) {
        // getting operand
        int level = interpreter.evalIntOperand(operand, ctx, false);
        return table.getVisibleColumns(level);
      }
      return table.getVisibleColumns();
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the number of columns from the table, specified by the object.";
    }
  }

  /**
   * The 'hasColumn' operation for Tables.
   */
  public class HasColumnOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      String operandValue = interpreter.evalStringOperand(operand, ctx);

      Table table = (Table) (expr.getObject().getValue());
      try {
        return new Boolean(table.getColumn(operandValue) != null);
      } catch (ColumnNotDefinedException ex) {
        return Boolean.FALSE;
      }
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Checks it the table contains a column that matches the name, specified by the operand.";
    }
  }
  
  /**
   * The reference operation for Tables.
   */
  public class ReferenceOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      String operandValue = interpreter.evalStringOperand(operand, ctx);

      Table table = (Table) (expr.getObject().getValue());
      try {
        return table.getObject(0, operandValue);
      } catch (RowNotDefinedException e) {
        throw new ExpressionException("No row defined for table at expression '" + expr.getCode() + "'.");
      } catch (ColumnNotDefinedException e) {
        StringBuffer msg = new StringBuffer("Unknown column '" + operandValue + "' for table at expression '" + expr.getCode() + "'. ");
        msg.append("Valid columns are: ");
        int numColumns = table.getColumnCount();
        try {
          for (int i = 0; i < numColumns; i++) {
            if (i > 0) {
              msg.append(", ");
            }
            msg.append(table.getColumn(i).getName());
          }
        } catch (ColumnNotDefinedException ex) {
          // this should never happen!
        }
        msg.append(".");
        throw new OperandException(msg.toString());
      }
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Accesses the field specified by the operand at the first row of the table.";
    }
  }

  /**
   * The 'getRow' operation for Tables.
   */
  public class GetRowOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      int operandValue = interpreter.evalIntOperand(operand, ctx, false);

      Table table = (Table) (expr.getObject().getValue());
      try {
        return table.getRow(operandValue);
      } catch (OperationNotSupportedException e) {
        throw new ExpressionException("Table accessed at expression '" + expr.getCode() + "' does not support the 'getRow' operation.");
      } catch (RowNotDefinedException e) {
        throw new ExpressionException("Row #" + operandValue + " is not defined for table at expression '" + expr.getCode() + "'.");
      }
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Accesses the field specified by the operand at the first row of the table.";
    }
  }
  
  /**
   * The formatValue operation for Rows.
   */
  public class FormatValueOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand == null) {
        throw new NoOperandException("No operand given for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }
      // getting operand
      String operandValue = interpreter.evalStringOperand(operand, ctx);

      Table table = (Table) (expr.getObject().getValue());
      try {
        return table.getString(0, operandValue);
      } catch (RowNotDefinedException e) {
        throw new ExpressionException("No row defined for table at expression '" + expr.getCode() + "'.");
      } catch (ColumnNotDefinedException e) {
        StringBuffer msg = new StringBuffer("Unknown column '" + operandValue + "' for table at expression '" + expr.getCode() + "'. ");
        msg.append("Valid columns are: ");
        int numColumns = table.getColumnCount();
        try {
          for (int i = 0; i < numColumns; i++) {
            if (i > 0) {
              msg.append(", ");
            }
            msg.append(table.getColumn(i).getName());
          }
        } catch (ColumnNotDefinedException ex) {
          // this should never happen!
        }
        msg.append(".");
        throw new OperandException(msg.toString());
      } catch (TypeMismatchException e) {
        throw new OperandException("Unable to format value of column '" + operandValue + "'.");
      }
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Gets the formatted value of the field specified by the operand at this row.";
    }
  }

  /**
   * The rotate operation for Tables.
   */
  public class RotateOperation extends BaseExpressionOperation {
    /**
     * Evaluates the given values.
     * @param expr The expression tuple to process. 
     * @param ctx The expression context.
     * @return The result of the tuple evaluation.
     * @exception ExpressionException If the evaluation failed.
     */
    public Object evaluate(ExpressionTuple expr, ExpressionContext ctx)
      throws ExpressionException {
        
      Operand operand = expr.getOperand();
      if (operand != null) {
        throw new NoOperandException("No operand supported for operation: " + expr.getOperation() + " on object " + expr.getObject() + "!");         
      }

      Table table = (Table) (expr.getObject().getValue());
      try {
        Column[] visibleColumns = table.getVisibleColumns();

        Table rotatedTable = new DataTable(table.getFormatHelper());
        Column rCol = rotatedTable.addColumn("header", UjacTypes.TYPE_OBJECT);
        rCol.setLayoutHints(new LayoutHints(10, null));
        for (int i = 0; i < visibleColumns.length; i++) {
          Row rRow = rotatedTable.addRow();
          rRow.setString(rCol.getIndex(), visibleColumns[i].getLayoutHints().getHeader());
        }

        int rowIdx = 0;
        Iterator iterTable = table.iterator();
        while (iterTable.hasNext()) {
          Row row = (Row) iterTable.next();
          rCol = rotatedTable.addColumn("row" + rowIdx, UjacTypes.TYPE_OBJECT);
          rCol.setLayoutHints(new LayoutHints(10, null));
          for (int i = 0; i < visibleColumns.length; i++) {
            Row rRow = rotatedTable.getRow(i);
            rRow.setObject(rCol.getIndex(), row.getObject(i));
          }
          ++rowIdx;
        }
        return rotatedTable;
      } catch (TableException e) {
        throw new ExpressionException("Failed to rotate table at expression '" + expr.getCode() + "'.");
      }
    }

    /**
     * Gets a description for the operation.
     * @return The item's description.
     */
    public String getDescription() {
      return "Rotates the table, specified by the object. It serves a table, that holds the same values as " +
          "the original table, but rotated by 90 degrees. The first column of the rotated table holds the " +
          "title row of the original table.";
    }
  }

  /**
   * Constructs a TableType instance with no specific arguments.
   * @param interpreter
   */
  public TableType(ExpressionInterpreter interpreter) {
    super(interpreter);
    // adding operations for this type.
    ExpressionOperation op = new ReferenceOperation();
    addOperation(".", op);
    addOperation("[]", op);
    addOperation("getField", op);
    op = new GetRowOperation();
    addOperation("getRow", op);
    op = new FormatValueOperation();
    addOperation("formatValue", op);
    op = new HasRowsOperation();
    addOperation("hasRows", op);
    addOperation("notEmpty", op);
    op = new NoRowsOperation();
    addOperation("noRows", op);
    addOperation("isEmpty", op);
    op = new RowCountOperation();
    addOperation("rowCount", op);
    addOperation("size", op);
    op = new ColumnCountOperation();
    addOperation("columnCount", op);
    op = new ColumnsOperation();
    addOperation("columns", op);
    op = new VisibleColumnsOperation();
    addOperation("visibleColumns", op);
    op = new HasColumnOperation();
    addOperation("hasColumn", op);
    op = new RotateOperation();
    addOperation("rotate", op);
  }

  /**
   * @see org.ujac.util.exi.ExpressionType#getType()
   */
  public Class getType() {
    return Table.class;
  }

}
