/*
 * Copyright (C) 2004, 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.exi.type;

import java.sql.Timestamp;

import java.text.Format;

import java.util.Calendar;
import java.util.Date;

import org.ujac.util.exi.ExpressionContext;
import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.ExpressionInterpreter;
import org.ujac.util.exi.ExpressionTuple;
import org.ujac.util.exi.Operand;

import org.ujac.util.text.FormatHelper;

/**
 * Name: TimestampType<br>
 * Description: An expression type handler for Timestamp values.<br>
 * 
 * @author lauerc
 */
public class TimestampType extends BaseDateType {

  /**
   * Executes the toString operation.
   * @param expr The expression, holding the object to get as string.
   * @param formatHelper The format helper to use.
   * @return The textual representation of the expression value.
   */
  protected String execToString(ExpressionTuple expr, FormatHelper formatHelper) {
    Timestamp value = (Timestamp) expr.getObject().getValue();
    Format format = formatHelper.getTimestampFormat();
    if (format != null) {
      return format.format(value);
    }
    return value.toString();
  }

  /**
   * Evaluates the given operand to a Date value.
   * @param operand The operand to evaluate. 
   * @param ctx The expression context.
   * @return The evaluated Date value.
   * @exception ExpressionException In case the evaluation failed.
   */
  protected Date evalDate(Operand operand, ExpressionContext ctx)
    throws ExpressionException {
    
    return interpreter.evalTimestampOperand(operand, ctx, false);    
  }

  /**
   * Compares the given date instances.
   * @param object The object to compare.
   * @param operand The operand to compare.
   * @return 1 in case the object is bigger than the operand, 
   *         0 if object and operand equals
   *        -1 in case the object is smaller than the operand. 
   */
  protected int compareDates(Date object, Date operand) {
    Calendar objectCal = Calendar.getInstance();
    objectCal.setTime(object);
    Calendar operandCal = Calendar.getInstance();
    operandCal.setTime(operand);

    int objectYear = objectCal.get(Calendar.YEAR);
    int operandYear = operandCal.get(Calendar.YEAR);
    
    int objectMonth = objectCal.get(Calendar.MONTH);
    int operandMonth = operandCal.get(Calendar.MONTH);

    int objectDay = objectCal.get(Calendar.DAY_OF_MONTH);
    int operandDay = operandCal.get(Calendar.DAY_OF_MONTH);

    int objectHour = objectCal.get(Calendar.HOUR_OF_DAY);
    int operandHour = operandCal.get(Calendar.HOUR_OF_DAY);

    int objectMinute = objectCal.get(Calendar.MINUTE);
    int operandMinute = operandCal.get(Calendar.MINUTE);

    int objectSecond = objectCal.get(Calendar.SECOND);
    int operandSecond = operandCal.get(Calendar.SECOND);

    if (objectYear > operandYear) {
      return 1;
    }
    if (objectYear < operandYear) {
      return -1;
    }
      
    if (objectMonth > operandMonth) {
      return 1;
    }
    if (objectMonth < operandMonth) {
      return -1;
    }

    if (objectDay > operandDay) {
      return 1;
    }
    if (objectDay < operandDay) {
      return -1;
    }

    if (objectHour > operandHour) {
      return 1;
    }
    if (objectHour < operandHour) {
      return -1;
    }

    if (objectMinute > operandMinute) {
      return 1;
    }
    if (objectMinute < operandMinute) {
      return -1;
    }

    if (objectSecond > operandSecond) {
      return 1;
    }
    if (objectSecond < operandSecond) {
      return -1;
    }

    return 0;
  }

  /**
   * Constructs a TimeType instance with specific arguments.
   * @param interpreter The expression interpreter.
   */
  public TimestampType(ExpressionInterpreter interpreter) {
    super(interpreter);
  }

  /**
   * @see org.ujac.util.exi.ExpressionType#getType()
   */
  public Class getType() {
    return Timestamp.class;
  }

  /**
   * Gets the alias of the expression type.
   * @return The tpye alias.
   */
  public String getAlias() {
    return "Timestamp";
  }

}
