/*
 * Copyright (C) 2003 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.io;

import java.io.IOException;

import java.util.HashMap;
import java.util.Map;

/**
 * Name: MultiResourceLoader<br>
 * Description: A resource loader which caches the resources it loads
 * by a defined real ResourceLoader to reduce I/O.
 * 
 * @author lauerc
 */
public class CachedResourceLoader implements ResourceLoader {

  /** The resource loader. */
  private ResourceLoader resourceLoader = null;
  /** The resource cache. */
  private Map resourceCache = new HashMap();
  
  /**
   * Constructs a CachedResourceLoader instance with specific attributes.
   * @param resourceLoader The real resource loader.
   */
  public CachedResourceLoader(ResourceLoader resourceLoader) {
    this.resourceLoader = resourceLoader;
  }
  
  /**
   * First checks the resource cache until the desired resource is really loaded
   * by the defined loader.
   * @see org.ujac.util.io.ResourceLoader#loadResource(java.lang.String)
   */
  public byte[] loadResource(String location) throws IOException {
    // first making cache lookup
    byte[] resource = (byte[]) resourceCache.get(location);
    if (resource != null) {
      return resource;
    }
    // really loading resource
    resource = resourceLoader.loadResource(location);
    // adding resource to cache.
    resourceCache.put(location, resource);
    return resource;
  }
  
  /**
   * Checks whether or not the desired resource exists. 
   * @param location The location of the resource.
   * @return true in case the given resource could be located, else false.
   */
  public boolean resourceExists(String location) {
    // first making cache lookup
    byte[] resource = (byte[]) resourceCache.get(location);
    if (resource != null) {
      return true;
    }
    // really loading resource
    return resourceLoader.resourceExists(location);
  }
  
  /**
   * Clears the resource cache.
   */
  public void clearCache() {
    resourceCache.clear();
  }
  
}
