/*
 * Copyright (C) 2006 by Kiran Krishnamurthy, Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.io;

import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.IOException;

import java.net.URL;

import java.util.Enumeration;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Name: ClassResourceLoader<br>
 * Description: A resource loader using the current thread's context class loader.
 * 
 * @author kmurthy
 */
public class ClassResourceLoader implements ResourceLoader {
  
  /** The log. */
  private static final Log log = LogFactory.getLog("ClassResourceLoader");

  /** 
   * @see org.ujac.util.io.ResourceLoader#loadResource(java.lang.String)
   * @author kmurthy
   */
  public byte[] loadResource(String location) throws IOException {
    URL url = null;
    ClassLoader loader = Thread.currentThread().getContextClassLoader();
    if (loader == null) {
      loader = ClassResourceLoader.class.getClassLoader();
    }
    Enumeration resources = loader.getResources(location);
    if (resources.hasMoreElements()) {
      url = (URL) resources.nextElement();
      if (resources.hasMoreElements()) {
        log.warn("*** WARNING ***");
        log.warn("There are multiple " + location
            + " files in classpath at the following locations:");
        log.warn(url.getPath());
        while (resources.hasMoreElements()) {
          log.warn(((URL) resources.nextElement()).getPath());
        }
      }
      log.debug("Loading " + location + " from: " + url.getPath());
    } else {
      // TODO Refine exception class choice
      throw new IOException("Could not find the resource: " + location
          + " in the classpath");
    }
    return loadFile(url);
  }
  
  /**
   * Loads a file from the given URL.
   * @param url The url to load.
   * @return The loaded contents.
   * @author kmurthy
   */
  private static byte[] loadFile(URL url) {
    FileInputStream fis = null;
    ByteArrayOutputStream baos = null;
    try {
      fis = new FileInputStream(url.getPath());
      baos = new ByteArrayOutputStream();
      byte[] bytes = new byte[1024];
      for (int size = fis.read(bytes); size > -1; size = fis.read(bytes)) {
        baos.write(bytes, 0, size);
        bytes = new byte[1024];
      }
      baos.flush();
    } catch (Exception e) {
      log.error("Failed to load resource.", e);
      e.printStackTrace();
    } finally {
      try {
        baos.close();
        fis.close();
      } catch (IOException ioe) {
        log.warn("Error occured when closing the input stream " + ioe.getMessage());
      }
    }
    return baos.toByteArray();
  }
  
  /**
   * @see org.ujac.util.io.ResourceLoader#resourceExists(java.lang.String)
   */
  public boolean resourceExists(String location) {
    try {
      ClassLoader loader = Thread.currentThread().getContextClassLoader();
      if (loader == null) {
        loader = ClassResourceLoader.class.getClassLoader();
      }
      Enumeration resources = loader.getResources(location);
      return resources.hasMoreElements();
    } catch (IOException ex) {
      log.error("Failed to check resource.", ex);
    }
    return false;
  }
  
}
