/*
 * Copyright (C) 2003, 2004 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.io;

import java.io.File;
import java.io.IOException;

/**
 * Title:       FileResourceLoader<br>
 * Description: A resource loader which loads resources from a given directory.<br>
 * 
 * @author lauerc
 */
public class FileResourceLoader implements ResourceLoader {

  /** The I/O buffer size. */
  public static final int IO_BUFFER_SIZE = 2048;
  /** The root directory from which to load the resources. */
  private File rootDirectory = null;

  /**
   * Constructs a FileResourceLoader instance with specific attributes.
   * @param rootPath The root directory location.
   * @exception IOException If the given rootDirectory does not exist or is no directory. 
   */
  public FileResourceLoader(String rootPath) throws IOException {
    this(new File(rootPath));
  }
  
  /**
   * Constructs a FileResourceLoader instance with specific attributes.
   * @param rootDirectory The root directory.
   * @exception IOException If the given rootDirectory does not exist or is no directory. 
   */
  public FileResourceLoader(File rootDirectory) throws IOException {
    if (rootDirectory == null) {
      throw new IOException("The root directory mustn't be null.");
    }
    if (!rootDirectory.exists()) {
      throw new IOException("The root directory '" + rootDirectory.getCanonicalPath() + "' doesn't exist.");
    }
    if (!rootDirectory.isDirectory()) {
      throw new IOException("The root directory '" + rootDirectory.getCanonicalPath() + "' is no directory.");
    }
    this.rootDirectory = rootDirectory;
  }

  /**
   * Loads a binary resource from the file system.
   * @param location The location of the resource.
   * @return The loaded resource.
   * @exception IOException In case the loading of the resource failed.
   */
  public byte[] loadResource(String location) throws IOException {
    if ((location == null) || (location.length() == 0)) {
      throw new IOException("The given resource location must not be null and non empty.");
    }
    String absolutePath = null;
    if (location.charAt(0) == '/') {
      absolutePath = rootDirectory.getCanonicalPath() + location;
    } else {
      absolutePath = rootDirectory.getCanonicalPath() + "/" + location;
    }
    return FileUtils.loadFile(new File(absolutePath));
  }
  
  /**
   * Checks whether or not the desired resource exists. 
   * @param location The location of the resource.
   * @return true in case the given resource could be located, else false.
   */
  public boolean resourceExists(String location) {
    if ((location == null) || (location.length() == 0)) {
      return false;
    }
    String absolutePath = null;
    try {
      if (location.charAt(0) == '/') {
        absolutePath = rootDirectory.getCanonicalPath() + location;
      } else {
        absolutePath = rootDirectory.getCanonicalPath() + "/" + location;
      }
    } catch (IOException ex) {
      // resource loading failed, resource doesn't seem to exist ...
      return false;
    }
    return new File(absolutePath).exists();
  }
  
}
