/*
 * Copyright (C) 2003 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.io;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * Name: MultiResourceLoader<br>
 * Description: A resource loader loading from multiple nested loaders.
 * It tries all nested loaders in the order they've beem added until the
 * desired resource has been loaded.
 * 
 * @author lauerc
 */
public class MultiResourceLoader implements ResourceLoader {

  /** The resource loader list. */
  private List resourceLoaders = new ArrayList();

  /**
   * Adds a resource loader.
   * @param resourceLoader The resource loader to add.
   */
  public void addResourceLoader(ResourceLoader resourceLoader) {
    if (resourceLoaders == null) {
      resourceLoaders = new ArrayList();
    }
    resourceLoaders.add(resourceLoader);
  }

  /**
   * Loads the resource according to the given location from the nested loaders.
   * Tries all loaders in the order they've beem added until the desired resource
   * has been loaded.
   * @see org.ujac.util.io.ResourceLoader#loadResource(java.lang.String)
   */
  public byte[] loadResource(String location) throws IOException {
    int numResourceLoaders = resourceLoaders.size();
    if (numResourceLoaders == 0) {
      throw new IOException("No resource loaders specified!");
    }
    for (int i=0; i<numResourceLoaders; i++) {
      ResourceLoader resourceLoader = (ResourceLoader) resourceLoaders.get(i);
      try {
        return resourceLoader.loadResource(location);
      } catch (IOException ex) {
        if (i >= numResourceLoaders-1) {
          throw ex;
        }
      }
    }
    return null;
  }


  /**
   * Checks whether or not the desired resource exists. 
   * @param location The location of the resource.
   * @return true in case the given resource could be located, else false.
   */
  public boolean resourceExists(String location) {
    int numResourceLoaders = resourceLoaders.size();
    if (numResourceLoaders == 0) {
      return false;
    }
    for (int i=0; i<numResourceLoaders; i++) {
      ResourceLoader resourceLoader = (ResourceLoader) resourceLoaders.get(i);
      if (resourceLoader.resourceExists(location)) {
        return true;
      }
    }
    return false;
  }

}
