/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.table;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.ujac.util.BeanUtils;
import org.ujac.util.TypeConverter;

import org.ujac.util.text.FormatHelper;


/**
 * Name: BaseTable<br>
 * Description: Basic table implementation.
 * 
 * @author lauerc
 */
public abstract class BaseTable implements Table {

  /** The format helper. */
  private FormatHelper formatHelper = null;
  /** The type converter. */
  private TypeConverter typeConverter = null;
  /** The columns of the table. */
  protected List columns = new ArrayList();
  /** The columns of the table. */
  protected Map columnMap = new HashMap();

  /**
   * Constructs a BaseTable instance with no specific arguments.
   */
  public BaseTable() {
    this(null);
  }

  /**
   * Constructs a BaseTable instance with no specific arguments.
   * @param formatHelper The format helper to use.
   */
  public BaseTable(FormatHelper formatHelper) {
    if (formatHelper == null) {
      this.formatHelper = FormatHelper.createInstance();
    } else {
      this.formatHelper = formatHelper;
    }
  }

  /**
   * Getter method for the the property formatHelper.
   * @return The current value of property formatHelper.
   */
  public FormatHelper getFormatHelper() {
    return formatHelper;
  }
  /**
   * Setter method for the the property formatHelper.
   * @param formatHelper The value to set for the property formatHelper.
   */
  public void setFormatHelper(FormatHelper formatHelper) {
    this.formatHelper = formatHelper;
  }

  /**
   * Getter method for the the property typeConverter.
   * @return The current value of property typeConverter.
   */
  public TypeConverter getTypeConverter() {
    return typeConverter;
  }
  /**
   * Setter method for the the property typeConverter.
   * @param typeConverter The value to set for the property typeConverter.
   */
  public void setTypeConverter(TypeConverter typeConverter) {
    this.typeConverter = typeConverter;
  }

  /**
   * Gets the current number of columns.
   * @return The number of columns.
   */
  public int getColumnCount() {
    return columns.size();
  }

  /**
   * Adds a column to the table
   * @param name The column name.
   * @param type The column type.
   * @return The new column.
   * @exception ColumnAlreadyDefinedException In case the column was already defined.
   * @exception OperationNotSupportedException In case the operation is not supported.
   */
  public Column addColumn(String name, int type) 
    throws ColumnAlreadyDefinedException, OperationNotSupportedException {
    
    Column col = (Column) columnMap.get(name);
    if (col != null) {
      throw new ColumnAlreadyDefinedException("A column with name '" + name + "' already exists.");
    }
    col = new DefaultColumn(this, name, type);
    // adding new column to column map
    columnMap.put(name, col);
    // adding new column to column list
    columns.add(col);
    int idx = columns.size() - 1;
    col.setIndex(idx);
    col.setDataIndex(idx);
    return col;
  }

  /**
   * Adds a column to the table
   * @param name The column name.
   * @param key Tells whether or not the column defines a key value.
   * @param type The column type.
   * @return The new column.
   * @exception ColumnAlreadyDefinedException In case the column was already defined.
   */
  public Column addColumn(String name, boolean key, int type) 
    throws ColumnAlreadyDefinedException {
    
    Column col = (Column) columnMap.get(name);
    if (col != null) {
      throw new ColumnAlreadyDefinedException("A column with name '" + name + "' already exists.");
    }
    col = new DefaultColumn(this, name, key, type);
    // adding new column to column map
    columnMap.put(name, col);
    // adding new column to column list
    columns.add(col);
    int idx = columns.size() - 1;
    col.setIndex(idx);
    col.setDataIndex(idx);
    return col;
  }

  /** 
   * Gets the column at the given index.
   * @param index The column index.
   * @return The desired column.
   * @exception ColumnNotDefinedException If the column does not exist.
   */
  public Column getColumn(int index) throws ColumnNotDefinedException {
    try {
      return (Column) columns.get(index);
    } catch (IndexOutOfBoundsException ex) {
      throw new ColumnNotDefinedException("Column at index " + index + " does not exist.");
    }
  }

  /** 
   * Gets the column with the given name.
   * @param name The column name.
   * @return The desired column.
   * @exception ColumnNotDefinedException If the column does not exist.
   */
  public Column getColumn(String name) throws ColumnNotDefinedException {
    Column col = (Column) columnMap.get(name);
    if (col == null) {
      throw new ColumnNotDefinedException("Column '" + name + "' does not exist.");
    }
    return col;
  }

  /** 
   * Gets the index of the given column name.
   * @param name The column name
   * @return int The index of the column.
   * @exception ColumnNotDefinedException If the column does not exist.
   */
  public int getColumnIndex(String name) throws ColumnNotDefinedException {
    Column col = getColumn(name);
    return col.getIndex();
  }

  /** 
   * Renames a column.
   * @param column The column to rename.
   * @param name The column name to set.
   * @exception ColumnAlreadyDefinedException In case a column with that name already exists.
   */
  public void renameColumn(Column column, String name) throws ColumnAlreadyDefinedException {
    if (BeanUtils.equals(name, column.getName())) {
      // name equals current name, nothing 2 do!
      return;
    }
    if (columnMap.get(name) != null) {
      throw new ColumnAlreadyDefinedException("Renaming of column '" + column.getName() + 
                                              "' failed: A column with name '" + name + "' already exists.");
    }
    columnMap.put(name, column);
    columnMap.remove(column.getName());
    ((DefaultColumn) column).setName(name);
  }

  /**
   * Orders the sequence of report columns according to the list of given column names.
   * @param columnNames The list of column names after which to order the columns.
   * @exception ColumnNotDefinedException In case a column from the given order list
   *  does not exist. 
   * @exception ColumnAlreadyDefinedException In case a column was defined more than 
   *  once at the given order list.
   */
  public void orderColumns(String[] columnNames) 
    throws ColumnNotDefinedException, ColumnAlreadyDefinedException {
    
    int numColumns = columns.size();
    Map columnMap = (Map) ((HashMap) this.columnMap).clone();
    
    Set newColumnNames = new HashSet();
    List newColumns = new ArrayList(numColumns);
    for (int i = 0; i < columnNames.length; i++) {
      String name = columnNames[i];
      Column col = (Column) columnMap.get(name);
      if (col == null) {
        if (newColumnNames.contains(name)) {
          throw new ColumnAlreadyDefinedException("The column '" + name + "' was specified in the given column order list more than once.");
        }
        throw new ColumnNotDefinedException("The column '" + name + "', specified in the given column order list does not exist.");
      }
      newColumns.add(col);
      newColumnNames.add(name);
      columnMap.remove(name);
    }
    if (columnMap.size() > 0) {
      // adding the remaining column according to their original order
      Column[] rest = new Column[columnMap.size()];
      int idx = 0;
      Iterator iterRest = columnMap.entrySet().iterator();
      while (iterRest.hasNext()) {
        rest[idx] = ((Column) ((Map.Entry) iterRest.next()).getValue());
        ++idx;
      }
      Arrays.sort(rest, new Comparator() {
        public int compare(Object a, Object b) {
          int idxA = ((Column) a).getIndex();
          int idxB = ((Column) b).getIndex();
          if (idxA > idxB) {
            return 1;
          }
          if (idxA < idxB) {
            return -1;
          }
          return 0;
        }
      });
      // finally adding the rest
      for (int i = 0; i < rest.length; i++) {
        Column col = rest[i];
        newColumns.add(col);
      }
    }

    // redefining the column indices
    for (int i = 0; i < numColumns; i++) {
      Column col = (Column) newColumns.get(i);
      col.setIndex(i);
    }

    // OK, ordering successfuly completed
    this.columns = newColumns;
  }
  
  /**
   * Gets all visible columns.
   * @return An array holding all visible columns.
   */
  public Column[] getVisibleColumns() {
    int numColumns = getColumnCount();
    Column[] columns = new Column[numColumns];
    int idx = 0;
    try {
      for (int i = 0; i < numColumns; i++) {
        Column column = getColumn(i);
        LayoutHints layoutHints = column.getLayoutHints();
        if ((layoutHints != null) && layoutHints.isVisible()) {
          columns[idx] = column;
          ++idx;
        }
      }
    } catch (ColumnNotDefinedException ex) {
      // this should never happen!
      throw new RuntimeException(ex.getMessage());
    }
    Column[] visibleColumns = new Column[idx];
    if (idx > 0) {
      System.arraycopy(columns, 0, visibleColumns, 0, idx);
    }
    return visibleColumns;
  }

  /**
   * Gets all visible columns.
   * @param levelFilter The maximum level of importance to support.
   * @return An array holding all visible columns.
   */
  public Column[] getVisibleColumns(int levelFilter) {
    int numColumns = getColumnCount();
    Column[] columns = new Column[numColumns];
    int idx = 0;
    try {
      for (int i = 0; i < numColumns; i++) {
        Column column = getColumn(i);
        LayoutHints layoutHints = column.getLayoutHints();
        if ((layoutHints != null) && layoutHints.isVisible()) {
          if ((levelFilter >= 0) && (layoutHints.getLevel() > levelFilter)) {
            // filtering unsupported levels
            continue;
          }
          columns[idx] = column;
          ++idx;
        }
      }
    } catch (ColumnNotDefinedException ex) {
      // this should never happen!
      throw new RuntimeException(ex.getMessage());
    }
    Column[] visibleColumns = new Column[idx];
    if (idx > 0) {
      System.arraycopy(columns, 0, visibleColumns, 0, idx);
    }
    return visibleColumns;
  }

  /**
   * Copies the table.
   * @return A copy holding the filtered contents of the handled table.
   * @exception TableException In case an unexpected failure occurred.
   */
  public Table copyTable() throws TableException {
    Table copy = new DataTable(this.getFormatHelper());
    
    // copying columns
    int numColumns = getColumnCount();
    for (int i = 0; i < numColumns; i++) {
      Column col = getColumn(i);
      copy.addColumn(col.getName(), col.getType())
        .setFormat(col.getFormat())
        .setLayoutHints(col.getLayoutHints())
        .setKey(col.isKey())
        .setAlias(col.getAlias())
        .setDataIndex(col.getDataIndex());
    }
    
    // copying rows
    Iterator iterRows = iterator();
    while (iterRows.hasNext()) {
      Row row = (Row) iterRows.next();
      copy.addRow(row);
    }
    return copy;
  }

  /**
   * Gets the distinct values of the specified column.
   * @param columnIdx The index of the desired column.
   * @return A set of distinct values.
   * @exception TableException In case an unexpected failure occurred.
   */
  public Set getDistinctValues(int columnIdx) throws TableException {
    Set distinctValues = new HashSet();
    Iterator iterRows = iterator();
    while (iterRows.hasNext()) {
      Row row = (Row) iterRows.next();
      distinctValues.add(row.getObject(columnIdx));
    }
    return distinctValues;
  }
  /**
   * Gets the distinct values of the specified column.
   * @param columnName The name of the desired column.
   * @return A set of distinct values.
   * @exception TableException In case an unexpected failure occurred.
   */
  public Set getDistinctValues(String columnName) throws TableException {
    int columnIdx = getColumnIndex(columnName);
    return getDistinctValues(columnIdx);
  }

  
  /**
   * Writes the object's data to the given stream.
   * @param s The stream to write to
   * @exception IOException In case the data output failed.
   */
  protected void writeData(ObjectOutputStream s) throws IOException {
    s.writeObject(formatHelper);
    s.writeObject(columns);
    s.writeObject(columnMap);
  }
  /**
   * Reads the object's data from the given stream.
   * @param s The stream to read from.
   * @exception IOException In case the data reading failed.
   * @exception ClassNotFoundException In case the class to deserialize could not be found
   */
  protected void readData(ObjectInputStream s) throws IOException, ClassNotFoundException  {
    formatHelper = (FormatHelper) s.readObject();
    columns = (List) s.readObject();
    columnMap = (Map) s.readObject();
  }

}
