/*
 * Copyright (C) 2004 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.table;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import java.sql.Time;
import java.sql.Timestamp;

import java.util.Date;
import java.util.List;

import org.ujac.util.text.FormatHelper;

/**
 * Name: BeanTable<br>
 * Description: A table implementation acting as an interface for a list of beans.
 * 
 * @author lauerc
 */
public class BeanTable extends DataTable {
  
  /** The serial version UID. */
  static final long serialVersionUID = -6301105392789353319L;

  /**
   * Constructs a BeanTable instance with specific attributes.
   * @param itemList The list of beans to fill the table with.
   * @exception TableException In case no bean class could be determined
   *   from the given list. 
   */
  public BeanTable(List itemList) throws TableException {
    this(null, itemList, null);
  }
  
  /**
   * Constructs a BeanTable instance with specific attributes.
   * @param formatHelper The format helper to use.
   * @param itemList The list of beans to fill the table with.
   * @exception TableException In case no bean class could be determined
   *   from the given list. 
   */
  public BeanTable(FormatHelper formatHelper, List itemList) throws TableException {
    this(formatHelper, itemList, null);
  }

  /**
   * Constructs a BeanTable instance with specific attributes.
   * @param itemList The list of beans to fill the table with.
   * @param columnFilter The column set filter. 
   * @exception TableException In case no bean class could be determined
   *   from the given list. 
   */
  public BeanTable(List itemList, ColumnFilter columnFilter) throws TableException {
    this(null, itemList, columnFilter);
  }
  
  /**
   * Constructs a BeanTable instance with specific attributes.
   * @param formatHelper The format helper to use.
   * @param itemList The list of beans to fill the table with.
   * @param columnFilter The column set filter. 
   * @exception TableException In case no bean class could be determined
   *   from the given list. 
   */
  public BeanTable(FormatHelper formatHelper, List itemList, ColumnFilter columnFilter)
    throws TableException {
    
    super(formatHelper);
    if ((itemList == null) || (itemList.size() < 1)) {
      throw new TableException("Unable to determine bean class from empty list.");
    }
    Object firstItem = itemList.get(0);
    if (firstItem == null) {
      throw new TableException("Unable to determine bean class from given list.");
    }
    
    setupColumns(firstItem.getClass(), columnFilter);
    
    // copying the data from the bean list into the data table structure
    Object[] getterArgs = new Object[0];
    Object[] columns = this.columns.toArray();
    int numColumns = columns.length;
    int numItems = itemList.size();
    try {
      for (int i = 0; i < numItems; i++) {
        Object bean = itemList.get(i);
        Row row = addRow();
        for (int j = 0; j < numColumns; j++) {
          BeanColumn col = (BeanColumn) columns[j];
          Method getter = col.getGetter();
          Object value = getter.invoke(bean, getterArgs);
          if (value == null) {
            continue;
          }
          switch (col.getType()) {
            case TYPE_BOOLEAN:
              row.setBoolean(j, ((Boolean) value).booleanValue());
              break;
            case TYPE_FLOAT:
              row.setFloat(j, ((Number) value).floatValue());
              break;
            case TYPE_DOUBLE:
              row.setDouble(j, ((Number) value).doubleValue());
              break;
            case TYPE_INT:
              row.setInt(j, ((Number) value).intValue());
              break;
            case TYPE_LONG:
              row.setLong(j, ((Number) value).longValue());
              break;
            case TYPE_STRING:
              row.setString(j, (String) value);
              break;
            case TYPE_DATE:
              row.setDate(j, (Date) value);
              break;
            case TYPE_TIME:
              row.setTime(j, (Time) value);
              break;
            case TYPE_TIMESTAMP:
              row.setTimestamp(j, (Timestamp) value);
              break;
            default: // assuming type Object
              row.setObject(j, value);
              break;
          }
        }
      }
    } catch (IllegalAccessException ex) {
      throw new TableException("Unable to import data from bean list:" + ex.getMessage());
    } catch (InvocationTargetException ex) {
      throw new TableException("Unable to import data from bean list:" + ex.getMessage());
    }
  }
  
  
  /**
   * Constructs a BeanTable instance with specific attributes.
   * @param dataType The type of beans to fill the table with.
   */
  public BeanTable(Class dataType) {
    this(null, dataType, null);
  }
  
  /**
   * Constructs a BeanTable instance with specific attributes.
   * @param formatHelper The format helper to use.
   * @param dataType The type of beans to fill the table with.
   */
  public BeanTable(FormatHelper formatHelper, Class dataType) {
    this(formatHelper, dataType, null);
  }
  
  /**
   * Constructs a BeanTable instance with specific attributes.
   * @param formatHelper The format helper to use.
   * @param dataType The type of beans to fill the table with.
   */
  public BeanTable(Class dataType, ColumnFilter columnFilter) {
    this(null, dataType, columnFilter);
  }
  
  /**
   * Constructs a BeanTable instance with specific attributes.
   * @param formatHelper The format helper to use.
   * @param dataType The daty type to use.
   * @param columnFilter The column set filter. 
   */
  public BeanTable(FormatHelper formatHelper, Class dataType, ColumnFilter columnFilter) {
    super(formatHelper);
    setupColumns(dataType, columnFilter);
  }

  /**
   * Sets up the columns of the bean table.
   * @param dataClass The data class to use.
   * @param columnFilter The column set filter. 
   */
  private void setupColumns(Class dataClass, ColumnFilter columnFilter) {
    
    try {
      // determining getter methods from data class
      Method[] methods = dataClass.getMethods();
      for (int i = 0; i < methods.length; i++) {
        Method method = methods[i];
        Class returnType = method.getReturnType();
        if (returnType == null) {
          // getter metods have to have a return type defined.
          continue;
        }
        
        String methodName = method.getName();
        if (methodName.startsWith("get")) {
          if (method.getParameterTypes().length > 0) {
            // legal getters must have no parameters
            continue;
          }
          String columnName = Character.toLowerCase(methodName.charAt(3)) + methodName.substring(4);
          if ((columnFilter != null) && !columnFilter.isValid(columnName)) {
            continue;
          }
          if ("class".equals(columnName)) {
            continue;
          }
          int type = getColumnType(method.getReturnType());
          if (type > 0) {
            addColumn(columnName, type, method);
          }
          continue;
        }
        
        if (methodName.startsWith("is")) {
          if (method.getParameterTypes().length > 0) {
            // legal getters must have no parameters
            continue;
          }
          String columnName = Character.toLowerCase(methodName.charAt(2)) + methodName.substring(3);
          if ((columnFilter != null) && !columnFilter.isValid(columnName)) {
            continue;
          }
          int type = getColumnType(method.getReturnType());
          if (type > 0) {
            addColumn(columnName, type, method);
          }
          continue;
        }
      }
    } catch (ColumnAlreadyDefinedException ex) {
      // this should never happen
      throw new RuntimeException(ex.getMessage());
    }
  }
  
  /**
   * Gets the column type from the given data type.
   * @param dataType The class to get the type for.
   * @return The determined column type.
   */
  private int getColumnType(Class dataType) {
    if (boolean.class.isAssignableFrom(dataType)) {
      return TYPE_BOOLEAN;
    }
    if (Boolean.class.isAssignableFrom(dataType)) {
      return TYPE_BOOLEAN;
    }
    if (int.class.isAssignableFrom(dataType)) {
      return TYPE_INT;
    }
    if (Integer.class.isAssignableFrom(dataType)) {
      return TYPE_INT;
    }
    if (long.class.isAssignableFrom(dataType)) {
      return TYPE_LONG;
    }
    if (Long.class.isAssignableFrom(dataType)) {
      return TYPE_LONG;
    }
    if (float.class.isAssignableFrom(dataType)) {
      return TYPE_FLOAT;
    }
    if (Float.class.isAssignableFrom(dataType)) {
      return TYPE_FLOAT;
    }
    if (double.class.isAssignableFrom(dataType)) {
      return TYPE_DOUBLE;
    }
    if (Double.class.isAssignableFrom(dataType)) {
      return TYPE_DOUBLE;
    }
    if (String.class.isAssignableFrom(dataType)) {
      return TYPE_STRING;
    }
    if (Time.class.isAssignableFrom(dataType)) {
      return TYPE_TIME;
    }
    if (Timestamp.class.isAssignableFrom(dataType)) {
      return TYPE_TIMESTAMP;
    }
    if (Date.class.isAssignableFrom(dataType)) {
      return TYPE_DATE;
    }
    return TYPE_OBJECT;
  }

  /**
   * Adds a column to the table
   * @param name The column name.
   * @param type The column type.
   * @param getter The getter method.
   * @return The new column.
   * @exception ColumnAlreadyDefinedException In case the column was already defined.
   */
  public Column addColumn(String name, int type, Method getter) 
    throws ColumnAlreadyDefinedException {
   
    Column col = (Column) columnMap.get(name);
    if (col != null) {
      throw new ColumnAlreadyDefinedException("A column with name '" + name + "' already exists.");
    }
    col = new BeanColumn(this, name, type, getter);
    // adding new column to column map
    columnMap.put(name, col);
    // adding new column to column list
    columns.add(col);
    int idx = columns.size() - 1;
    col.setIndex(idx);
    col.setDataIndex(idx);
    return col;
  }

  /**
   * Writes the object's data to the given stream.
   * @param s The stream to write to
   * @exception IOException In case the data output failed.
   */
  private void writeObject(ObjectOutputStream s) throws IOException {
    super.writeData(s);
  }

  /**
   * Reads the object's data from the given stream.
   * @param s The stream to read from.
   * @exception IOException In case the data reading failed.
   * @exception ClassNotFoundException In case the class to deserialize could not be found
   */
  private void readObject(ObjectInputStream s) throws IOException, ClassNotFoundException  {
    super.readData(s);
  }

}
