/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.table;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import java.text.DateFormat;
import java.text.Format;
import java.text.NumberFormat;
import java.text.ParseException;

import java.util.Date;

import org.ujac.util.BeanUtils;
import org.ujac.util.UjacTypes;

/**
 * Name: Row<br>
 * Description: A class holding row data.<br>
 * 
 * @author lauerc
 */
public class DataRow extends BaseRow implements TableConstants, Row {

  /** The serial version UID. */
  static final long serialVersionUID = -8435793368278149790L;

  /** The index of the row. */
  private int row = -1;
  /** The row type. */
  private String type = ROW_TYPE_NORMAL;
  
  /**
   * Constructs a DataRow instance with specific arguments.
   * @param table The table which contains this row.
   * @param rowIdx The index of this row.
   */
  protected DataRow(Table table, int rowIdx) {
    super(table);
    this.row = rowIdx;
  }

  /**
   * Constructs a DataRow instance with specific attributes.
   * @param table The row's table.
   * @param rowIdx The index of this row.
   * @param srcRow the row to copy.
   */
  public DataRow(Table table, int rowIdx, Row srcRow) {
    super(table, srcRow);
    this.row = rowIdx;
  }
  
  /**
   * Gets the row type.
   * @return The row type.
   */
  public String getType() {
    return type;
  }
  /**
   * Sets the row type.
   * @param type The row type to set.
   */
  public void setType(String type) {
    this.type = type;
  }
  
  /**
   * Gets the table the row is member of.
   * @return The table that contains this row.
   */
  public Table getTable() {
    return table;
  }
  
  /**
   * Gets the colum index.
   * @return The column index.
   */
  public int getIndex() {
    return row;
  }
  /**
   * Sets the colum index.
   * @param index The new column index.
   */
  protected void setIndex(int index) {
    this.row = index;
  }

  /**
   * Expands the length of the row to the number of columns of the table.
   */
  protected void expandRow() {
    int columnCount = table.getColumnCount();
    int elementCount = rowData.size();
    for (int i = elementCount; i < columnCount; i++) {
      rowData.add(null);
    }
  }

  /**
   * Removes the given column from the row data.
   * @param columnIdx The index of the column to remove. 
   * @exception ColumnNotDefinedException In case the column to remove does not exist.
   */
  protected void removeColumn(int columnIdx) throws ColumnNotDefinedException {
    Column column = table.getColumn(columnIdx);
    int dataIdx = column.getDataIndex();
    rowData.remove(dataIdx);
  }

  /**
   * Gets the String value from the given row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public String getString(String columnName) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnName);
    return getString(column.getIndex());
  }
  /**
   * Gets the String value from the given row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public String getString(int columnIdx) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnIdx);
    int dataIdx = column.getDataIndex();
    Object value = rowData.get(dataIdx);
    if (value == null) {
      return null;
    }
    Format fmt = getFormat(column.getName());
    switch (column.getType()) {
      case UjacTypes.TYPE_STRING:
        if ((fmt != null) && !(value instanceof String)) {
          return fmt.format(value);
        }
        return value.toString();
        
      case UjacTypes.TYPE_INT:
      case UjacTypes.TYPE_LONG:
      case UjacTypes.TYPE_FLOAT:
      case UjacTypes.TYPE_DOUBLE:
      case UjacTypes.TYPE_BOOLEAN:
      case UjacTypes.TYPE_DATE:
      case UjacTypes.TYPE_TIME:
      case UjacTypes.TYPE_TIMESTAMP:
        if (fmt != null) {
          return fmt.format(value);
        }
        return value.toString();
        
      case UjacTypes.TYPE_OBJECT:
        if (fmt != null) {
          return fmt.format(value);
        }
        return value.toString();
        
      default: 
        throw new TypeMismatchException("Unable to convert a variable of type " + column.getTypeName() + " to String.");
    }
  }
  /**
   * Sets the String value at the given row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public void setString(String columnName, String value) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnName);
    setString(column.getIndex(), value);
  }
  /**
   * Sets the String value at the given row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public void setString(int columnIdx, String value) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnIdx);
    int dataIdx = column.getDataIndex();
    if (value == null) {
      rowData.set(dataIdx, null);
    }

    switch (column.getType()) {
      case UjacTypes.TYPE_OBJECT:
      case UjacTypes.TYPE_STRING:
        rowData.set(dataIdx, value);
        break;
      case UjacTypes.TYPE_DATE:
        try {
          rowData.set(dataIdx, getFormat(column.getName()).parseObject(value));
        } catch (ParseException ex) {
          throw new TypeMismatchException("Unable to parse date value from String '" + value +"'.");
        }
        break;
      case UjacTypes.TYPE_TIME:
        try {
          rowData.set(dataIdx, getFormat(column.getName()).parseObject(value));
        } catch (ParseException ex) {
          throw new TypeMismatchException("Unable to parse date value from String '" + value +"'.");
        }
        break;
      case UjacTypes.TYPE_TIMESTAMP:
        try {
          rowData.set(dataIdx, getFormat(column.getName()).parseObject(value));
        } catch (ParseException ex) {
          throw new TypeMismatchException("Unable to parse date value from String '" + value +"'.");
        }
        break;
      case UjacTypes.TYPE_INT:
        try {
          NumberFormat fmt = (NumberFormat) getFormat(column.getName());
          if (fmt != null) {
            rowData.set(dataIdx, fmt.parseObject(value));
          } else {
            rowData.set(dataIdx, new Integer(value));
          }
        } catch (ParseException ex) {
          throw new TypeMismatchException("Unable to parse integer value from String '" + value +"'.");
        }
        break;
      case UjacTypes.TYPE_LONG:
        try {
          NumberFormat fmt = (NumberFormat) getFormat(column.getName());
          if (fmt != null) {
            rowData.set(dataIdx, fmt.parseObject(value));
          } else {
            rowData.set(dataIdx, new Long(value));
          }
        } catch (ParseException ex) {
          throw new TypeMismatchException("Unable to parse long value from String '" + value +"'.");
        }
        break;
      case UjacTypes.TYPE_FLOAT:
        try {
          NumberFormat fmt = (NumberFormat) getFormat(column.getName());
          if (fmt != null) {
            rowData.set(dataIdx, fmt.parseObject(value));
          } else {
            rowData.set(dataIdx, new Float(value));
          }
        } catch (ParseException ex) {
          throw new TypeMismatchException("Unable to parse float value from String '" + value +"'.");
        }
        break;
      case UjacTypes.TYPE_DOUBLE:
        try {
          NumberFormat fmt = (NumberFormat) getFormat(column.getName());
          if (fmt != null) {
            rowData.set(dataIdx, fmt.parseObject(value));
          } else {
            rowData.set(dataIdx, new Double(value));
          }
        } catch (ParseException ex) {
          throw new TypeMismatchException("Unable to parse double value from String '" + value +"'.");
        }
        break;
      default: 
        throw new TypeMismatchException("Unable to write String value to row type " + column.getTypeName() + ".");
    }
  }

  /**
   * Gets the int value from the given row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public int getInt(String columnName) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnName);
    return getInt(column.getIndex());
  }
  /**
   * Gets the integer value from the given row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public int getInt(int columnIdx) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnIdx);
    int dataIdx = column.getDataIndex();
    Object value = rowData.get(dataIdx);
    if (value == null) {
      return 0;
    }
    if (column.getType() != UjacTypes.TYPE_INT) {
      throw new TypeMismatchException("Unable to convert a variable of type " + column.getTypeName() + " to int.");
    }
    return ((Number) value).intValue(); 
  }
  /**
   * Sets the int value at the given row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public void setInt(String columnName, int value) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnName);
    setInt(column.getIndex(), value);
  }
  /**
   * Sets the integer value at the given row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public void setInt(int columnIdx, int value) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnIdx);
    int dataIdx = column.getDataIndex();
    switch (column.getType()) {
      case UjacTypes.TYPE_OBJECT:
      case UjacTypes.TYPE_INT:
      case UjacTypes.TYPE_LONG:
      case UjacTypes.TYPE_FLOAT:
      case UjacTypes.TYPE_DOUBLE:
      case UjacTypes.TYPE_STRING:
      default: 
        Integer integerVal = new Integer(value);
        rowData.set(dataIdx, integerVal);
        break;
    }
  }

  /**
   * Gets the long value from the given row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public long getLong(String columnName) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnName);
    return getLong(column.getIndex());
  }
  /**
   * Gets the long value from the given row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public long getLong(int columnIdx) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnIdx);
    int dataIdx = column.getDataIndex();
    Object value = rowData.get(dataIdx);
    if (value == null) {
      return 0L;
    }
    switch (column.getType()) {
      case UjacTypes.TYPE_LONG:
        return ((Number) value).longValue(); 
      case UjacTypes.TYPE_INT:
        return ((Number) value).longValue();
      default: 
        throw new TypeMismatchException("Unable to convert a variable of type " + column.getTypeName() + " to long.");
    }
  }
  /**
   * Sets the long value at the given row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public void setLong(String columnName, long value) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnName);
    setLong(column.getIndex(), value);
  }
  /**
   * Sets the long value at the given row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public void setLong(int columnIdx, long value) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnIdx);
    int dataIdx = column.getDataIndex();
    switch (column.getType()) {
      case UjacTypes.TYPE_OBJECT:
      case UjacTypes.TYPE_INT:
      case UjacTypes.TYPE_LONG:
      case UjacTypes.TYPE_FLOAT:
      case UjacTypes.TYPE_DOUBLE:
      case UjacTypes.TYPE_STRING:
      default: 
        Long longVal = new Long(value);
        rowData.set(dataIdx, longVal);
        break;
    }
  }

  /**
   * Gets the float value from the given row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public float getFloat(String columnName) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnName);
    return getFloat(column.getIndex());
  }
  /**
   * Gets the float value from the given row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public float getFloat(int columnIdx) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnIdx);
    int dataIdx = column.getDataIndex();
    Object value = rowData.get(dataIdx);
    if (value == null) {
      return Float.NaN;
    }
    switch (column.getType()) {
      case TYPE_INT:
      case TYPE_LONG:
      case TYPE_FLOAT:
      case TYPE_DOUBLE:
        return ((Number) value).floatValue(); 
      default:
        throw new TypeMismatchException("Unable to convert a variable of type " + column.getTypeName() + " to double.");
    }
  }
  /**
   * Sets the float value at the given row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public void setFloat(String columnName, float value) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnName);
    setDouble(column.getIndex(), value);
  }
  /**
   * Sets the float value at the given row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public void setFloat(int columnIdx, float value) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnIdx);
    int dataIdx = column.getDataIndex();
    switch (column.getType()) {
      case UjacTypes.TYPE_OBJECT:
      case UjacTypes.TYPE_INT:
      case UjacTypes.TYPE_LONG:
      case UjacTypes.TYPE_FLOAT:
      case UjacTypes.TYPE_DOUBLE:
      case UjacTypes.TYPE_STRING:
      default: 
        Double doubleVal = new Double(value);
        rowData.set(dataIdx, doubleVal);
        break;
    }
  }

  /**
   * Gets the double value from the given row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public double getDouble(String columnName) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnName);
    return getDouble(column.getIndex());
  }
  /**
   * Gets the double value from the given row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public double getDouble(int columnIdx) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnIdx);
    int dataIdx = column.getDataIndex();
    Object value = rowData.get(dataIdx);
    if (value == null) {
      return Double.NaN;
    }
    switch (column.getType()) {
      case TYPE_INT:
      case TYPE_LONG:
      case TYPE_FLOAT:
      case TYPE_DOUBLE:
        return ((Number) value).doubleValue(); 
      default:
        throw new TypeMismatchException("Unable to convert a variable of type " + column.getTypeName() + " to double.");
    }
  }
  /**
   * Sets the double value at the given row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public void setDouble(String columnName, double value) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnName);
    setDouble(column.getIndex(), value);
  }
  /**
   * Sets the double value at the given row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public void setDouble(int columnIdx, double value) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnIdx);
    int dataIdx = column.getDataIndex();
    switch (column.getType()) {
      case UjacTypes.TYPE_OBJECT:
      case UjacTypes.TYPE_INT:
      case UjacTypes.TYPE_LONG:
      case UjacTypes.TYPE_FLOAT:
      case UjacTypes.TYPE_DOUBLE:
      case UjacTypes.TYPE_STRING:
      default: 
        Double doubleVal = new Double(value);
        rowData.set(dataIdx, doubleVal);
        break;
    }
  }

  /**
   * Gets the boolean value from the given row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public boolean getBoolean(String columnName) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnName);
    return getBoolean(column.getIndex());
  }
  /**
   * Gets the boolean value from the given row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public boolean getBoolean(int columnIdx) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnIdx);
    int dataIdx = column.getDataIndex();
    Object value = rowData.get(dataIdx);
    if (value == null) {
      return false;
    }
    switch (column.getType()) {
      case UjacTypes.TYPE_BOOLEAN:
        return ((Boolean) value).booleanValue(); 
      case UjacTypes.TYPE_INT:
        return ((Number) value).intValue() > 0 ? true : false;
      case UjacTypes.TYPE_LONG:
        return ((Number) value).longValue() > 0 ? true : false;
      case UjacTypes.TYPE_STRING:
        String strVal = (String) value;
        return strVal.equalsIgnoreCase("true") || strVal.equalsIgnoreCase("y") || strVal.equalsIgnoreCase("j") ? true : false; 
      case UjacTypes.TYPE_OBJECT:
        if (value instanceof Boolean) {
          return ((Boolean) value).booleanValue();
        }
        return new Boolean(value.toString()).booleanValue();
      default: 
        throw new TypeMismatchException("Unable to convert a variable of type " + column.getTypeName() + " to boolean.");
    }
  }
  /**
   * Sets the boolean value at the given row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public void setBoolean(String columnName, boolean value) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnName);
    setBoolean(column.getIndex(), value);
  }
  /**
   * Sets the boolean value at the given row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public void setBoolean(int columnIdx, boolean value) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnIdx);
    int dataIdx = column.getDataIndex();
    switch (column.getType()) {
      case UjacTypes.TYPE_OBJECT:
      case UjacTypes.TYPE_BOOLEAN:
        Boolean boolVal = value ? Boolean.TRUE : Boolean.FALSE;
        rowData.set(dataIdx, boolVal);
        break; 
      case UjacTypes.TYPE_INT:
        Integer intVal = value ? new Integer(1) : new Integer(0);
        rowData.set(dataIdx, intVal);
        break; 
      case UjacTypes.TYPE_LONG:
        Long longVal = value ? new Long(1) : new Long(0);
        rowData.set(dataIdx, longVal);
        break; 
      case UjacTypes.TYPE_STRING:
        String strVal = new Boolean(value).toString();
        rowData.set(dataIdx, strVal);
        break;
      default: 
        throw new TypeMismatchException("Unable write boolean value to row type " + column.getTypeName() + ".");
    }
  }

  /**
   * Gets the Date value from the given row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public Date getDate(String columnName) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnName);
    return getDate(column.getIndex());
  }
  /**
   * Gets the Date value from the given row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public Date getDate(int columnIdx) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnIdx);
    int dataIdx = column.getDataIndex();
    Object value = rowData.get(dataIdx);
    if (value == null) {
      return null;
    }
    switch (column.getType()) {
      case UjacTypes.TYPE_DATE:
      case UjacTypes.TYPE_TIME:
      case UjacTypes.TYPE_TIMESTAMP:
        return (Date) value;
      case UjacTypes.TYPE_STRING:
        try {
          return (Date) getFormat(column.getName()).parseObject((String) value);
        } catch (ParseException e) {
          throw new TypeMismatchException("Unable to convert a variable of type " + column.getTypeName() + " to Date: " + e.getMessage());
        }
      case UjacTypes.TYPE_OBJECT:
        if (value instanceof Date) {
          return (Date) value;
        }
        try {
          return (Date) getFormat(column.getName()).parseObject(value.toString());
        } catch (ParseException e) {
          throw new TypeMismatchException("Unable to convert a variable of type " + column.getTypeName() + " to Date: " + e.getMessage());
        }
      default:  
        throw new TypeMismatchException("Unable to convert a variable of type " + column.getTypeName() + " to Date.");
    }
  }
  /**
   * Sets the Date value at the given row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public void setDate(String columnName, Date value) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnName);
    setDate(column.getIndex(), value);
  }
  /**
   * Sets the Date value at the given row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public void setDate(int columnIdx, Date value) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnIdx);
    int dataIdx = column.getDataIndex();
    if (value == null) {
      rowData.set(dataIdx, null);
    }

    switch (column.getType()) {
      case UjacTypes.TYPE_OBJECT:
      case UjacTypes.TYPE_DATE:
      case UjacTypes.TYPE_TIME:
      case UjacTypes.TYPE_TIMESTAMP:
        rowData.set(dataIdx, value);
        break; 
      case UjacTypes.TYPE_STRING:
        DateFormat fmt = (DateFormat) getFormat(column.getName());
        String strVal = null;
        if (fmt != null) {
          strVal = fmt.format(value);
        } else {
          strVal = value.toString();
        }
        rowData.set(dataIdx, strVal);
        break;
      default: 
        throw new TypeMismatchException("Unable write Date value to row type " + column.getTypeName() + ".");
    }
  }

  /**
   * Gets the Time value from the given row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public Date getTime(String columnName) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnName);
    return getTime(column.getIndex());
  }
  /**
   * Gets the Time value from the given row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public Date getTime(int columnIdx) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnIdx);
    int dataIdx = column.getDataIndex();
    Object value = rowData.get(dataIdx);
    if (value == null) {
      return null;
    }
    switch (column.getType()) {
      case UjacTypes.TYPE_DATE:
      case UjacTypes.TYPE_TIME:
      case UjacTypes.TYPE_TIMESTAMP:
        return (Date) value;
      case UjacTypes.TYPE_STRING:
        try {
          return (Date) getFormat(column.getName()).parseObject((String) value);
        } catch (ParseException e) {
          throw new TypeMismatchException("Unable to convert a variable of type " + column.getTypeName() + " to Time: " + e.getMessage());
        }
      case UjacTypes.TYPE_OBJECT:
        if (value instanceof Date) {
          return (Date) value;
        }
        try {
          return (Date) getFormat(column.getName()).parseObject(value.toString());
        } catch (ParseException e) {
          throw new TypeMismatchException("Unable to convert a variable of type " + column.getTypeName() + " to Time: " + e.getMessage());
        }
      default:  
        throw new TypeMismatchException("Unable to convert a variable of type " + column.getTypeName() + " to Date.");
    }
  }
  /**
   * Sets the Time value at the given row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public void setTime(String columnName, Date value) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnName);
    setTime(column.getIndex(), value);
  }
  /**
   * Sets the Time value at the given row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public void setTime(int columnIdx, Date value) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnIdx);
    int dataIdx = column.getDataIndex();
    if (value == null) {
      rowData.set(dataIdx, null);
    }
    
    switch (column.getType()) {
      case UjacTypes.TYPE_OBJECT:
      case UjacTypes.TYPE_DATE:
      case UjacTypes.TYPE_TIME:
      case UjacTypes.TYPE_TIMESTAMP:
        rowData.set(dataIdx, value);
        break; 
      case UjacTypes.TYPE_STRING:
        DateFormat fmt = (DateFormat) getFormat(column.getName());
        String strVal = null;
        if (fmt != null) {
          strVal = fmt.format(value);
        } else {
          strVal = value.toString();
        }
        rowData.set(dataIdx, strVal);
        break;
      default: 
        throw new TypeMismatchException("Unable write Time value to row type " + column.getTypeName() + ".");
    }
  }

  /**
   * Gets the Timestamp value from the given row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public Date getTimestamp(String columnName) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnName);
    return getTimestamp(column.getIndex());
  }
  /**
   * Gets the Timestamp value from the given row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public Date getTimestamp(int columnIdx) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnIdx);
    int dataIdx = column.getDataIndex();
    Object value = rowData.get(dataIdx);
    if (value == null) {
      return null;
    }
    switch (column.getType()) {
      case UjacTypes.TYPE_DATE:
      case UjacTypes.TYPE_TIME:
      case UjacTypes.TYPE_TIMESTAMP:
        return (Date) value;
      case UjacTypes.TYPE_STRING:
        try {
          return (Date) getFormat(column.getName()).parseObject((String) value);
        } catch (ParseException e) {
          throw new TypeMismatchException("Unable to convert a variable of type " + column.getTypeName() + " to Date: " + e.getMessage());
        }
      case UjacTypes.TYPE_OBJECT:
        if (value instanceof Date) {
          return (Date) value;
        }
        try {
          return (Date) getFormat(column.getName()).parseObject(value.toString());
        } catch (ParseException e) {
          throw new TypeMismatchException("Unable to convert a variable of type " + column.getTypeName() + " to Date: " + e.getMessage());
        }
      default:  
        throw new TypeMismatchException("Unable to convert a variable of type " + column.getTypeName() + " to Date.");
    }
  }
  /**
   * Sets the Timestamp value at the given row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public void setTimestamp(String columnName, Date value) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnName);
    setTimestamp(column.getIndex(), value);
  }
  /**
   * Sets the Timestamp value at the given row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public void setTimestamp(int columnIdx, Date value) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnIdx);
    int dataIdx = column.getDataIndex();
    if (value == null) {
      rowData.set(dataIdx, null);
    }

    switch (column.getType()) {
      case UjacTypes.TYPE_OBJECT:
      case UjacTypes.TYPE_DATE:
      case UjacTypes.TYPE_TIME:
      case UjacTypes.TYPE_TIMESTAMP:
        rowData.set(dataIdx, value);
        break; 
      case UjacTypes.TYPE_STRING:
        DateFormat fmt = (DateFormat) getFormat(column.getName());
        String strVal = null;
        if (fmt != null) {
          strVal = fmt.format(value);
        } else {
          strVal = value.toString();
        }
        rowData.set(dataIdx, strVal);
        break;
      default: 
        throw new TypeMismatchException("Unable write Timestamp value to row type " + column.getTypeName() + ".");
    }
  }

  /**
   * Gets the Object value from the given row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   */
  public Object getObject(String columnName) throws ColumnNotDefinedException {
    Column column = table.getColumn(columnName);
    return getObject(column.getIndex());
  }
  /**
   * Gets the Object value from the given row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   */
  public Object getObject(int columnIdx) throws ColumnNotDefinedException {
    Column column = table.getColumn(columnIdx);
    int dataIdx = column.getDataIndex();
    Object value = rowData.get(dataIdx);
    if (value == null) {
      return null;
    }
    return value;
  }
  /**
   * Sets the Object value at the given row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public void setObject(String columnName, Object value) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnName);
    setObject(column.getIndex(), value);
  }
  /**
   * Sets the Object value at the given row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  public void setObject(int columnIdx, Object value) throws ColumnNotDefinedException, TypeMismatchException {
    Column column = table.getColumn(columnIdx);
    int dataIdx = column.getDataIndex();
    if (value == null) {
      rowData.set(dataIdx, null);
      return;
    }

    switch (column.getType()) {
      case UjacTypes.TYPE_STRING:
        rowData.set(dataIdx, value.toString());
        break;
      case UjacTypes.TYPE_DATE:
        try {
          if (value instanceof Date) {
            rowData.set(dataIdx, value);
          } else {
            String strValue = value.toString();
            if (BeanUtils.isEmpty(strValue)) {
              rowData.set(dataIdx, null);
            } else {
              rowData.set(dataIdx, getFormat(column.getName()).parseObject(strValue));
            }
          }
        } catch (ParseException ex) {
          throw new TypeMismatchException("Unable to parse date value from Object '" + value +"'.");
        }
        break;
      case UjacTypes.TYPE_TIME:
        try {
          if (value instanceof Date) {
            rowData.set(dataIdx, value);
          } else {
            String strValue = value.toString();
            if (BeanUtils.isEmpty(strValue)) {
              rowData.set(dataIdx, null);
            } else {
              rowData.set(dataIdx, getFormat(column.getName()).parseObject(strValue));
            }
          }
        } catch (ParseException ex) {
          throw new TypeMismatchException("Unable to parse date value from Object '" + value +"'.");
        }
        break;
      case UjacTypes.TYPE_TIMESTAMP:
        try {
          if (value instanceof Date) {
            rowData.set(dataIdx, value);
          } else {
            String strValue = value.toString();
            if (BeanUtils.isEmpty(strValue)) {
              rowData.set(dataIdx, null);
            } else {
              rowData.set(dataIdx, getFormat(column.getName()).parseObject(strValue));
            }
          }
        } catch (ParseException ex) {
          throw new TypeMismatchException("Unable to parse date value from Object '" + value +"'.");
        }
        break;
      case UjacTypes.TYPE_INT:
        try {
          NumberFormat fmt = (NumberFormat) getFormat(column.getName());
          if (value instanceof Number) {
            rowData.set(dataIdx, value);
          } else {
            String strValue = value.toString();
            if (BeanUtils.isEmpty(strValue)) {
              rowData.set(dataIdx, null);
            } else {
              if (fmt != null) {
                rowData.set(dataIdx, fmt.parseObject(strValue));
              } else {
                rowData.set(dataIdx, new Integer(strValue));
              }
            }
          }
        } catch (ParseException ex) {
          throw new TypeMismatchException("Unable to parse integer value from Object '" + value +"'.");
        }
        break;
      case UjacTypes.TYPE_LONG:
        try {
          NumberFormat fmt = (NumberFormat) getFormat(column.getName());
          if (value instanceof Number) {
            rowData.set(dataIdx, value);
          } else {
            String strValue = value.toString();
            if (BeanUtils.isEmpty(strValue)) {
              rowData.set(dataIdx, null);
            } else {
              if (fmt != null) {
                rowData.set(dataIdx, fmt.parseObject(strValue));
              } else {
                rowData.set(dataIdx, new Long(strValue));
              }
            }
          }
        } catch (ParseException ex) {
          throw new TypeMismatchException("Unable to parse long value from Object '" + value +"'.");
        }
        break;
      case UjacTypes.TYPE_FLOAT:
        try {
          NumberFormat fmt = (NumberFormat) getFormat(column.getName());
          if (value instanceof Number) {
            rowData.set(dataIdx, value);
          } else {
            String strValue = value.toString();
            if (BeanUtils.isEmpty(strValue)) {
              rowData.set(dataIdx, null);
            } else {
              if (fmt != null) {
                rowData.set(dataIdx, fmt.parseObject(strValue));
              } else {
                rowData.set(dataIdx, new Float(strValue));
              }
            }
          }
        } catch (ParseException ex) {
          throw new TypeMismatchException("Unable to parse float value from Object '" + value +"'.");
        }
        break;
      case UjacTypes.TYPE_DOUBLE:
        try {
          NumberFormat fmt = (NumberFormat) getFormat(column.getName());
          if (value instanceof Number) {
            rowData.set(dataIdx, value);
          } else {
            String strValue = value.toString();
            if (BeanUtils.isEmpty(strValue)) {
              rowData.set(dataIdx, null);
            } else {
              if (fmt != null) {
                rowData.set(dataIdx, fmt.parseObject(strValue));
              } else {
                rowData.set(dataIdx, new Double(strValue));
              }
            }
          }
        } catch (ParseException ex) {
          throw new TypeMismatchException("Unable to parse double value from Object '" + value +"'.");
        }
        break;
      case UjacTypes.TYPE_BOOLEAN:
        if (value instanceof Boolean) {
          rowData.set(dataIdx, value);
        } else {
          rowData.set(dataIdx, new Boolean(value.toString()));
        }
        break;
      case UjacTypes.TYPE_OBJECT:
        rowData.set(dataIdx, value);
        break;
      default: 
        throw new TypeMismatchException("Unable to write Object value to row type " + column.getTypeName() + ".");
    }
  }

  /**
   * Writes the object's data to the given stream.
   * @param s The stream to write to
   * @exception IOException In case the data output failed.
   */
  protected void writeData(ObjectOutputStream s) throws IOException {
    super.writeData(s);
    s.writeInt(row);
    s.writeObject(type);
  }
  /**
   * Reads the object's data from the given stream.
   * @param s The stream to read from.
   * @exception IOException In case the data reading failed.
   * @exception ClassNotFoundException In case the class to deserialize could not be found
   */
  protected void readData(ObjectInputStream s) throws IOException, ClassNotFoundException  {
    super.readData(s);
    row = s.readInt();
    type = (String) s.readObject();
  }

  /**
   * Writes the object's data to the given stream.
   * @param s The stream to write to
   * @exception IOException In case the data output failed.
   */
  private void writeObject(ObjectOutputStream s) throws IOException {
    writeData(s);
  }
  /**
   * Reads the object's data from the given stream.
   * @param s The stream to read from.
   * @exception IOException In case the data reading failed.
   * @exception ClassNotFoundException In case the class to deserialize could not be found
   */
  private void readObject(ObjectInputStream s) throws IOException, ClassNotFoundException  {
    readData(s);
  }
  
}
