/*
 * Copyright (C) 2003, 2004, 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.table;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.Date;
import java.util.List;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Collections;

import org.ujac.util.DefaultTypeConverter;
import org.ujac.util.TypeConverter;
import org.ujac.util.TypeConverterException;
import org.ujac.util.UjacTypes;

import org.ujac.util.text.FormatHelper;

/**
 * Name: DataTable<br>
 * Description: A class holding table data.<br>
 * 
 * @author lauerc
 */
public class DataTable extends BaseTable implements Serializable {

  /** The serial version UID. */
  static final long serialVersionUID = 2494694894996180895L;

  /** The rows of the table. */
  private List rows = new ArrayList();

  /**
   * Constructs a DataTable instance with no specific arguments.
   */
  public DataTable() {
    super();
  }

  /**
   * Constructs a DataTable instance with no specific arguments.
   * @param formatHelper The format helper to use.
   */
  public DataTable(FormatHelper formatHelper) {
    super(formatHelper);
  }

  /**
   * Adds a column to the table
   * @param name The column name.
   * @param type The column type.
   * @return The new column.
   * @exception ColumnAlreadyDefinedException In case the column was already defined.
   * @exception OperationNotSupportedException In case the operation is not supported.
   */
  public Column addColumn(String name, int type) 
    throws ColumnAlreadyDefinedException, OperationNotSupportedException {
    
    Column col = super.addColumn(name, type);
    // expanding all existing rows
    List rows = this.rows;
    int numRows = rows.size();
    for (int i = 0; i < numRows; i++) {
      ((DataRow) rows.get(i)).expandRow();
    }
    return col;    
  }

  /**
   * Adds a column to the table
   * @param name The column name.
   * @param type The column type.
   * @param layoutHints The layout hints for the column.
   * @return The new column.
   * @exception ColumnAlreadyDefinedException In case the column was already defined.
   * @exception OperationNotSupportedException In case the operation is not supported.
   */
  public Column addColumn(String name, int type, LayoutHints layoutHints) 
    throws ColumnAlreadyDefinedException, OperationNotSupportedException {
   
    Column col = addColumn(name, type);
    col.setLayoutHints(layoutHints);
    return col;
  }

  /**
   * Adds a column of type String to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @throws TableException If an error occured.
   */
  public Table addString(String name, String value) throws TableException {
    Column col = addColumn(name, UjacTypes.TYPE_STRING);
    if (getRowCount() < 1) {
      addRow();
    }
    Row row = getRow(0);
    ((DataRow) row).expandRow();
    row.setString(col.getIndex(), value);    
    return this;
  }

  /**
   * Adds a column of type int to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @throws TableException If an error occured.
   */
  public Table addInt(String name, int value) throws TableException { 
    Column col = addColumn(name, UjacTypes.TYPE_INT);
    if (getRowCount() < 1) {
      addRow();
    }
    Row row = getRow(0);
    ((DataRow) row).expandRow();
    row.setInt(col.getIndex(), value);    
    return this;
  }

  /**
   * Adds a column of type long to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @throws TableException If an error occured.
   */
  public Table addLong(String name, long value) throws TableException { 
    Column col = addColumn(name, UjacTypes.TYPE_LONG);
    if (getRowCount() < 1) {
      addRow();
    }
    Row row = getRow(0);
    ((DataRow) row).expandRow();
    row.setLong(col.getIndex(), value);    
    return this;
  }

  /**
   * Adds a column of type float to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @throws TableException If an error occured.
   */
  public Table addFloat(String name, float value) throws TableException { 
    Column col = addColumn(name, UjacTypes.TYPE_FLOAT);
    if (getRowCount() < 1) {
      addRow();
    }
    Row row = getRow(0);
    ((DataRow) row).expandRow();
    row.setFloat(col.getIndex(), value);    
    return this;
  }

  /**
   * Adds a column of type double to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @throws TableException If an error occured.
   */
  public Table addDouble(String name, double value) throws TableException { 
    Column col = addColumn(name, UjacTypes.TYPE_DOUBLE);
    if (getRowCount() < 1) {
      addRow();
    }
    Row row = getRow(0);
    ((DataRow) row).expandRow();
    row.setDouble(col.getIndex(), value);    
    return this;
  }

  /**
   * Adds a column of type boolean to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @throws TableException If an error occured.
   */
  public Table addBoolean(String name, boolean value) throws TableException { 
    Column col = addColumn(name, UjacTypes.TYPE_BOOLEAN);
    if (getRowCount() < 1) {
      addRow();
    }
    Row row = getRow(0);
    ((DataRow) row).expandRow();
    row.setBoolean(col.getIndex(), value);    
    return this;
  }

  /**
   * Adds a column of type Date to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @throws TableException If an error occured.
   */
  public Table addDate(String name, Date value) throws TableException { 
    Column col = addColumn(name, UjacTypes.TYPE_DATE);
    if (getRowCount() < 1) {
      addRow();
    }
    Row row = getRow(0);
    ((DataRow) row).expandRow();
    row.setDate(col.getIndex(), value);    
    return this;
  }

  /**
   * Adds a column of type Date to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @throws TableException If an error occured.
   */
  public Table addTime(String name, Date value) throws TableException { 
    Column col = addColumn(name, UjacTypes.TYPE_TIME);
    if (getRowCount() < 1) {
      addRow();
    }
    Row row = getRow(0);
    ((DataRow) row).expandRow();
    row.setTime(col.getIndex(), value);    
    return this;    
  }

  /**
   * Adds a column of type Date to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @throws TableException If an error occured.
   */
  public Table addTimestamp(String name, Date value) throws TableException { 
    Column col = addColumn(name, UjacTypes.TYPE_TIMESTAMP);
    if (getRowCount() < 1) {
      addRow();
    }
    Row row = getRow(0);
    ((DataRow) row).expandRow();
    row.setTimestamp(col.getIndex(), value);    
    return this;    
  }

  /**
   * Checks if the table is empty.
   * @return true if the table has no rows, else false.
   */
  public boolean isEmpty() {
    return rows.size() == 0;
  }

  /**
   * Gets the current number of rows.
   * @return The number of rows.
   */
  public int getRowCount() {
    return rows.size();
  }

  /**
   * Adds a row to the table
   * @return The new row.
   */
  public Row addRow() {
    // increasing row count
    DataRow newRow = new DataRow(this, rows.size());
    rows.add(newRow);
    return newRow;
  }
  
  /**
   * Adds a row to the table
   * @param srcRow The row to copy.
   * @return The new row.
   * @exception OperationNotSupportedException In case the operation is not supported.
   */
  public Row addRow(Row srcRow) throws OperationNotSupportedException {
    DataRow newRow = new DataRow(this, rows.size(), srcRow);
    rows.add(newRow);
    return newRow;
  }

  /**
   * Gets a row from the table
   * @param rowIdx The index of the desired row.
   * @return The new row.
   * @exception RowNotDefinedException In case the requested row does not exist.
   */
  public Row getRow(int rowIdx) throws RowNotDefinedException {
    if (rowIdx >= getRowCount()) {
      throw new RowNotDefinedException("The row with index " + rowIdx + " does not exist.");
    }
    return (Row) rows.get(rowIdx);
  }

  /**
   * Removes a row from the table
   * @param rowIdx The index of the row to remove.
   * @exception RowNotDefinedException In case the requested row does not exist.
   */
  public void removeRow(int rowIdx) throws RowNotDefinedException {
    if (rowIdx >= getRowCount()) {
      throw new RowNotDefinedException("The row with index " + rowIdx + " does not exist.");
    }
    rows.remove(rowIdx);
  }
  
  /**
   * Gets the header row.
   * @return The header row for the table.
   */
  public Row getHeaderRow() {
    HeaderRow row = new HeaderRow(this);
    return row;
  }
  
  /**
   * Returns an iterator over the rows in this table in proper sequence.
   * @return an iterator over the rows in this table in proper sequence.
   */
  public Iterator iterator() {
    return new DataTableIterator(this); 
  }
  
  /**
   * Gets the String value from the given row at the given column.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the String conversion failed.
   */
  public String getString(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    Row row = getRow(rowIdx);
    return row.getString(columnIdx);
  }
  /**
   * Gets the String value from the given row at the given column.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the String conversion failed.
   */
  public String getString(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    Column col = getColumn(columnName);
    return getString(rowIdx, col.getIndex());
  }
  /**
   * Sets the String value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the String conversion failed.
   */
  public void setString(int rowIdx, int columnIdx, String value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    Row row = getRow(rowIdx);
    row.setString(columnIdx, value);
  }
  /**
   * Sets the String value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the String conversion failed.
   */
  public void setString(int rowIdx, String columnName, String value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    Column col = getColumn(columnName);
    setString(rowIdx, col.getIndex(), value);
  }

  /**
   * Gets the integer value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the int conversion failed.
   */
  public int getInt(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    Row row = getRow(rowIdx);
    return row.getInt(columnIdx);
  }
  /**
   * Gets the integer value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the int conversion failed.
   */
  public int getInt(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    Column col = getColumn(columnName);
    return getInt(rowIdx, col.getIndex());
  }
  /**
   * Sets the integer value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the int conversion failed.
   */
  public void setInt(int rowIdx, int columnIdx, int value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    Row row = getRow(rowIdx);
    row.setInt(columnIdx, value);
  }
  /**
   * Sets the integer value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the int conversion failed.
   */
  public void setInt(int rowIdx, String columnName, int value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    setInt(rowIdx, columnIdx, value); 
  }

  /**
   * Gets the long value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the long conversion failed.
   */
  public long getLong(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    Row row = getRow(rowIdx);
    return row.getLong(columnIdx);
  }
  /**
   * Gets the long value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the long conversion failed.
   */
  public long getLong(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    return getLong(rowIdx, columnIdx); 
  }
  /**
   * Sets the long value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the long conversion failed.
   */
  public void setLong(int rowIdx, int columnIdx, long value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    Row row = getRow(rowIdx);
    row.setLong(columnIdx, value);
  }
  /**
   * Sets the integer value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the long conversion failed.
   */
  public void setLong(int rowIdx, String columnName, long value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    setLong(rowIdx, columnIdx, value); 
  }

  /**
   * Gets the float value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the float conversion failed.
   */
  public float getFloat(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    Row row = getRow(rowIdx);
    return row.getFloat(columnIdx);
  }
  /**
   * Gets the float value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the float conversion failed.
   */
  public float getFloat(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    return getFloat(rowIdx, columnIdx); 
  }
  /**
   * Sets the float value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the float conversion failed.
   */
  public void setFloat(int rowIdx, int columnIdx, float value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    Row row = getRow(rowIdx);
    row.setFloat(columnIdx, value);
  }
  /**
   * Sets the float value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the float conversion failed.
   */
  public void setFloat(int rowIdx, String columnName, float value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    setFloat(rowIdx, columnIdx, value); 
  }

  /**
   * Gets the double value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the double conversion failed.
   */
  public double getDouble(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    Row row = getRow(rowIdx);
    return row.getDouble(columnIdx);
  }
  /**
   * Gets the double value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the double conversion failed.
   */
  public double getDouble(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    return getDouble(rowIdx, columnIdx); 
  }
  /**
   * Sets the double value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the double conversion failed.
   */
  public void setDouble(int rowIdx, int columnIdx, double value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    Row row = getRow(rowIdx);
    row.setDouble(columnIdx, value);
  }
  /**
   * Sets the double value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the double conversion failed.
   */
  public void setDouble(int rowIdx, String columnName, double value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    setDouble(rowIdx, columnIdx, value); 
  }

  /**
   * Gets the boolean value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the boolean conversion failed.
   */
  public boolean getBoolean(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    Row row = getRow(rowIdx);
    return row.getBoolean(columnIdx);
  }
  /**
   * Gets the boolean value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the boolean conversion failed.
   */
  public boolean getBoolean(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    return getBoolean(rowIdx, columnIdx); 
  }
  /**
   * Sets the boolean value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the boolean conversion failed.
   */
  public void setBoolean(int rowIdx, int columnIdx, boolean value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    Row row = getRow(rowIdx);
    row.setBoolean(columnIdx, value);
  }
  /**
   * Sets the boolean value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the boolean conversion failed.
   */
  public void setBoolean(int rowIdx, String columnName, boolean value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    setBoolean(rowIdx, columnIdx, value); 
  }

  /**
   * Gets the Date value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the date conversion failed.
   */
  public Date getDate(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    Row row = getRow(rowIdx);
    return row.getDate(columnIdx);
  }
  /**
   * Gets the Date value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the date conversion failed.
   */
  public Date getDate(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    return getDate(rowIdx, columnIdx); 
  }
  /**
   * Sets the Date value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the date conversion failed.
   */
  public void setDate(int rowIdx, int columnIdx, Date value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    Row row = getRow(rowIdx);
    row.setDate(columnIdx, value);
  }
  /**
   * Sets the Time value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the date conversion failed.
   */
  public void setDate(int rowIdx, String columnName, Date value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    setDate(rowIdx, columnIdx, value); 
  }

  /**
   * Gets the Time value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the date conversion failed.
   */
  public Date getTime(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    Row row = getRow(rowIdx);
    return row.getTime(columnIdx);
  }
  /**
   * Gets the Date value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the time conversion failed.
   */
  public Date getTime(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    return getDate(rowIdx, columnIdx); 
  }
  /**
   * Sets the Date value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the time conversion failed.
   */
  public void setTime(int rowIdx, int columnIdx, Date value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    Row row = getRow(rowIdx);
    row.setTime(columnIdx, value);
  }
  /**
   * Sets the Time value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the time conversion failed.
   */
  public void setTime(int rowIdx, String columnName, Date value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    setTime(rowIdx, columnIdx, value); 
  }

  /**
   * Gets the Timestamp value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the timestamp conversion failed.
   */
  public Date getTimestamp(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    Row row = getRow(rowIdx);
    return row.getTimestamp(columnIdx);
  }
  /**
   * Gets the Timestamp value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the timestamp conversion failed.
   */
  public Date getTimestamp(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    return getTimestamp(rowIdx, columnIdx); 
  }
  /**
   * Sets the Timestamp value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the timestamp conversion failed.
   */
  public void setTimestamp(int rowIdx, int columnIdx, Date value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    Row row = getRow(rowIdx);
    row.setTimestamp(columnIdx, value);
  }
  /**
   * Sets the Timestamp value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the timestamp conversion failed.
   */
  public void setTimestamp(int rowIdx, String columnName, Date value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    setDate(rowIdx, columnIdx, value); 
  }

  /**
   * Gets the Object value from the given row at the given column.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   */
  public Object getObject(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException {
    Row row = getRow(rowIdx);
    return row.getObject(columnIdx); 
  }
  /**
   * Gets the Object value from the given row at the given column.
   * @param rowIdx The index of the desired row.
   * @param columnName The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   */
  public Object getObject(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException {
    int columnIdx = getColumnIndex(columnName);
    return getObject(rowIdx, columnIdx); 
  }
  /**
   * Sets the Object value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the object conversion failed.
   */
  public void setObject(int rowIdx, int columnIdx, Object value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    Row row = getRow(rowIdx);
    row.setObject(columnIdx, value);
  }
  /**
   * Sets the Object value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the object conversion failed.
   */
  public void setObject(int rowIdx, String columnName, Object value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    setObject(rowIdx, columnIdx, value); 
  }

  /**
   * Sorts the table according to the given compare item definitions.
   * @param compareColumns The column definition for the sort process.
   */
  public void sort(CompareItem[] compareColumns) {
    Comparator rowComparator = new RowComparator(compareColumns);
    Collections.sort(rows, rowComparator);
  }

  /**
   * Sorts the table according to the given column and direction.
   * @param columnIdx The column to sort by.
   * @param direction The sort directon.
   */
  public void sort(int columnIdx, int direction) {
    sort(new CompareItem[] {new CompareItem(columnIdx, direction)});
  }

  /**
   * Swaps the rows with the given two indices.
   * @param a The index of the first row.
   * @param b The index of the second row.
   */  
  public void swapRows(int a, int b) {
    if (a == b) {
      return;
    }
    DataRow rowA = (DataRow) rows.get(a);
    DataRow rowB = (DataRow) rows.get(b);
    rows.set(a, rowB);
    rows.set(b, rowA);
  }

  /**
   * Changes the type of the given column.
   * @param columnIdx The index of the column which has been changed.
   * @param typeId The new column type ID.
   * @exception ColumnNotDefinedException In case the desired column does not exist.
   * @exception TypeConverterException In case the object conversion failed.
   */
  public void setColumnType(int columnIdx, int typeId)
    throws ColumnNotDefinedException, TypeConverterException {
    
    Column column = getColumn(columnIdx);
    int dataIdx = column.getDataIndex();
    int oldType = column.getType();
    if (oldType == typeId) {
      // nothing 2 do
      return;
    }
    column.setType(typeId);
    
    TypeConverter tc = getTypeConverter();
    if (tc == null) {
      tc = new DefaultTypeConverter();
    }
    try {
      int numRows = rows.size();
      for (int i = 0; i < numRows; i++) {
        Row row = (Row) rows.get(i);
        row.setObject(columnIdx, tc.convertObject(typeId, row.getObject(dataIdx)));
      }
    } catch (TableException ex) {
      throw new TypeConverterException("Type conversion failed: " + ex.getMessage(), ex);
    }
  }

  /**
   * Changes the type of the given column.
   * @param columnName The name of the column which has been changed.
   * @param typeId The new column type ID.
   * @exception ColumnNotDefinedException In case the desired column does not exist.
   * @exception TypeConverterException In case the object conversion failed.
   */
  public void setColumnType(String columnName, int typeId)
    throws ColumnNotDefinedException, TypeConverterException {
    
    Column column = getColumn(columnName);
    setColumnType(column.getIndex(), typeId);
  }

  
  /**
   * Converts the table to String.
   * @return the textual representation of the table.
   */
  public String toString() {
 
    StringBuffer buf = new StringBuffer();
    int numCols = columns.size();

    try {
      // first determining the widths of the columns
      int totalWidth = 1;
      int[] widths = new int[numCols];
      for (int i = 0; i < numCols; i++) {
        Column col = (Column) columns.get(i);
        widths[i] = computeColumnWidth(i, col);
        totalWidth += widths[i]+1;
      }
      
      for (int i = 0; i < totalWidth; i++) {
        buf.append('-');
      }
      buf.append("\n|");
      for (int i = 0; i < numCols; i++) {
        if (i > 0) {
          buf.append('|');
        }
        Column col = (Column) columns.get(i);
        printField(buf, col.getName() + ":" + col.getTypeName(), widths[i]);
      }
      buf.append("|\n");
      
      for (int k = 0; k < totalWidth; k++) {
        buf.append('-');
      }
      buf.append('\n');
      
      int numRows = getRowCount();
      for (int row = 0; row < numRows; row++) {
        if (row > 0) {
          buf.append('\n');
        }
        buf.append('|');
        
        for (int j = 0; j < numCols; j++) {
          if (j > 0) {
            buf.append('|');
          }
          printField(buf, getString(row, j), widths[j]);
        }
        buf.append('|');
      }
      
      buf.append('\n');
      for (int k = 0; k < totalWidth; k++) {
        buf.append('-');
      }

    } catch (RowNotDefinedException e) {
      e.printStackTrace();
    } catch (ColumnNotDefinedException e) {
      e.printStackTrace();
    } catch (TypeMismatchException e) {
      e.printStackTrace();
    }
    
    return buf.toString();
  }
  
  /**
   * Computes the width of one column.
   * @param colId The ID of the column.
   * @param col The definition of the column.
   * @return int The width of the column in characters.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException The desired column was not defined.
   * @exception TypeMismatchException A type mismatch problem has occured.
   */
  private int computeColumnWidth(int colId, Column col) 
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    
    int width = col.getName().length() + col.getTypeName().length() + 1;
    int numRows = getRowCount();
    String val = null;
    int valLength = 0;
    for (int row=0; row<numRows; row++) {
      val = getString(row, colId);
      if (val == null) {
        continue;
      }
      valLength = val.length();
      width = valLength > width ? valLength : width;
    }
    return width;
  }

  /**
   * Prints a field to the given string buffer.
   * @param buf The string buffer.
   * @param value The textual value.
   * @param columnWidth The width of the column in characters.
   */
  private void printField(StringBuffer buf, String value, int columnWidth) {
    buf.append(value);
    int len = 0; 
    if (value != null) {
      len = value.length(); 
    } else {
      len = 4;
    }
    if (len >= columnWidth) {
      return;
    }
    
    for (int i=len; i<columnWidth; i++) {
      buf.append(' ');
    }
  }
  
  /**
   * The main method for testing purposes.
   * @param args The command line arguments.
   */
  public static final void main(String[] args) {
    try {
      DataTable table = new DataTable();
      int nameID = table.addColumn("name", UjacTypes.TYPE_STRING).getIndex();
      int firstNameID = table.addColumn("firstName", UjacTypes.TYPE_STRING).getIndex();
      int addressID = table.addColumn("address", UjacTypes.TYPE_STRING).getIndex();
      int zipCodeID = table.addColumn("zipCode", UjacTypes.TYPE_INT).getIndex();
      int cityID = table.addColumn("city", UjacTypes.TYPE_STRING).getIndex();
      int birthdayID = table.addColumn("birthday", UjacTypes.TYPE_DATE).getIndex();
      
      Row row = table.addRow();
      row.setString(nameID, "Pan");
      row.setString(firstNameID, "Peter");
      row.setString(addressID, "Zwiebelgasse 12");
      row.setInt(zipCodeID, 12345);
      row.setString(cityID, "Wichtelhausen");
      row.setString(birthdayID, "20.02.1971");

      int ageID = table.addColumn("age", UjacTypes.TYPE_INT).getIndex();

      row = table.addRow();
      row.setString(nameID, "Kunz");
      row.setString(firstNameID, "Heiner");
      row.setString(addressID, "Wurtsweg 23");
      row.setInt(zipCodeID, 12345);
      row.setString(cityID, "Wichtelhausen");
      row.setString(birthdayID, "24.03.1964");
      row.setInt(ageID, 39);
      
      System.out.println(table.toString());

      Iterator iter = table.iterator();
      while (iter.hasNext()) {
        DataRow r = (DataRow) iter.next();
        System.out.println(r.getString("name"));
      }
    } catch (OperationNotSupportedException e) {
      e.printStackTrace();
    } catch (ColumnAlreadyDefinedException e) {
      e.printStackTrace();
    } catch (ColumnNotDefinedException e) {
      e.printStackTrace();
    } catch (TypeMismatchException e) {
      e.printStackTrace();
    }
  }

  /**
   * Writes the object's data to the given stream.
   * @param s The stream to write to
   * @exception IOException In case the data output failed.
   */
  protected void writeData(ObjectOutputStream s) throws IOException {
    super.writeData(s);
    s.writeObject(rows);
  }
  /**
   * Reads the object's data from the given stream.
   * @param s The stream to read from.
   * @exception IOException In case the data reading failed.
   * @exception ClassNotFoundException In case the class to deserialize could not be found
   */
  protected void readData(ObjectInputStream s) throws IOException, ClassNotFoundException  {
    super.readData(s);
    rows = (List) s.readObject();
  }

  /**
   * Writes the object's data to the given stream.
   * @param s The stream to write to
   * @exception IOException In case the data output failed.
   */
  private void writeObject(ObjectOutputStream s) throws IOException {
    writeData(s);
  }
  /**
   * Reads the object's data from the given stream.
   * @param s The stream to read from.
   * @exception IOException In case the data reading failed.
   * @exception ClassNotFoundException In case the class to deserialize could not be found
   */
  private void readObject(ObjectInputStream s) throws IOException, ClassNotFoundException  {
    readData(s);
  }

}
