/*
 * Copyright (C) 2004 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.table;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

import java.text.Format;

import org.ujac.util.text.FormatHelper;

/**
 * Name: DefaultColumn<br>
 * Description: Default column implementation.
 * 
 * @author lauerc
 */
public class DefaultColumn implements Column, Serializable {

  /** The serial version UID. */
  static final long serialVersionUID = -7084479108164808073L;

  /** The column's table. */
  private Table table = null;
  /** The column name. */
  private String name;
  /** Tells whether or not the field defines a key value, defaults to false. */
  private boolean key = false;
  /** The column type. */
  private int type;
  /** The column alias. */
  private String alias;
  /** The column format. */
  private Format format;
  /** The column index. */
  private int index = -1;
  /** The data index. */
  private int dataIndex = -1;
  /** The layout hints. */
  private LayoutHints layoutHints;
  
  /**
   * Constructs a DefaultColumn instance with specific arguments.
   * @param table The column's table.
   * @param name The column name.
   * @param type The column type.
   */
  protected DefaultColumn(Table table, String name, int type) {
    this.table = table;
    this.name = name;
    this.type = type;
  }
  
  /**
   * Constructs a DefaultColumn instance with specific arguments.
   * @param table The column's table.
   * @param name The column name.
   * @param key Tells whether or not the column defines a key value.
   * @param type The column type.
   */
  protected DefaultColumn(Table table, String name, boolean key, int type) {
    this.table = table;
    this.name = name;
    this.key = key;
    this.type = type;
  }

  /**
   * Gets the colum name.
   * @return The column name.
   */
  public String getName() {
    return name;
  }
  /**
   * Sets the colum name.
   * @param name The new column name.
   */
  protected void setName(String name) {
    this.name = name;
  }

  /**
   * Tells wheter this column defines a key value or not.
   * @return true if this column defines a key value, else false.
   */
  public boolean isKey() {
    return key;
  }
  /**
   * Sets the key flag.
   * @param key The key flag to set.
   * @return A reference to this instance for more convinience.
   */
  public Column setKey(boolean key) {
    this.key = key;
    return this;
  }

  /**
   * Gets the colum name.
   * @return The column type.
   */
  public int getType() {
    return type;
  }
  /**
   * Sets the colum type.
   * @param type The new column type.
   */
  public void setType(int type) {
    this.type = type;
  }

  /**
   * Gets the column alias.
   * @return The column alias.
   */
  public String getAlias() {
    return alias;
  }
  /**
   * Sets the colum alias.
   * @param alias The new column alias.
   * @return A reference to this instance for more convinience.
   */
  public Column setAlias(String alias) {
    this.alias = alias;
    return this;
  }

  /**
   * Gets the colum format.
   * @return The column format.
   */
  public Format getFormat() {
    if (format != null) {
      return format;
    }
    
    if (table == null) {
      return null;
    }
    
    FormatHelper fh = table.getFormatHelper();
    switch (type) {
      case TYPE_INT:
      case TYPE_LONG:
        return fh.getIntegerFormat();
      case TYPE_FLOAT:
      case TYPE_DOUBLE:
        return fh.getDoubleFormat();
      case TYPE_DATE:
        return fh.getDateFormat();
      case TYPE_TIME:
        return fh.getTimeFormat();
      case TYPE_TIMESTAMP:
        return fh.getTimestampFormat();
      default:
        return null;
    }
  }
  /**
   * Sets the colum format.
   * @param format The new column format.
   * @return A reference to this instance for more convinience.
   */
  public Column setFormat(Format format) {
    this.format = format;
    return this;
  }
  /**
   * Sets the colum format.
   * @param format The format string to use.
   * @return A reference to this instance for more convinience.
   */
  public Column setFormat(String format) {
    switch (type) {
      case TYPE_INT:
      case TYPE_LONG:
      case TYPE_FLOAT:
      case TYPE_DOUBLE:
        this.format = table.getFormatHelper().createNumberFormat(format);
        break;
      
      case TYPE_DATE:
      case TYPE_TIME:
      case TYPE_TIMESTAMP:
        this.format = table.getFormatHelper().createDateFormat(format);
        break;
    }
    return this;
  }
  
  /**
   * Gets the colum index.
   * @return The column index.
   */
  public int getIndex() {
    return index;
  }
  /**
   * Sets the colum index.
   * @param index The new column index.
   * @return A reference to this instance for more convinience.
   */
  public Column setIndex(int index) {
    this.index = index;
    return this;
  }
  
  /**
   * Gets the data index.
   * @return The data index.
   */
  public int getDataIndex() {
    return dataIndex;
  }
  /**
   * Sets the data index.
   * @param dataIndex The new data index.
   * @return A reference to this instance for more convinience.
   */
  public Column setDataIndex(int dataIndex) {
    this.dataIndex = dataIndex;
    return this;
  }

  /**
   * Gets the layout hints.
   * @return Returns the layout hints.
   */
  public LayoutHints getLayoutHints() {
    return layoutHints;
  }
  /**
   * Sets the layout hints.
   * @param layoutHints The layout hints to set.
   * @return A reference to this instance for more convinience.
   */
  public Column setLayoutHints(LayoutHints layoutHints) {
    this.layoutHints = layoutHints;
    return this;
  }

  /**
   * Gets the name of the column type.
   * @return the name of the type or UNKNOWN.
   */
  public String getTypeName() {
    switch (type) {
      case TYPE_STRING: 
        return "String";
      case TYPE_INT: 
        return "int";
      case TYPE_LONG: 
        return "long";
      case TYPE_FLOAT: 
        return "float";
      case TYPE_DOUBLE: 
        return "double";
      case TYPE_BOOLEAN: 
        return "boolean";
      case TYPE_DATE: 
        return "Date";
      case TYPE_TIME: 
        return "Time";
      case TYPE_TIMESTAMP: 
        return "Timestamp";
      default:
        return "UNKNOWN";
    }
  }
  
  /**
   * @see java.lang.Object#toString()
   */
  public String toString() {
    return "{name=" + name + ", type=" + getTypeName() + ", alias=" + alias + ", index=" + index + ", layoutHints=" + layoutHints + "}"; 
  }

  /**
   * Writes the object's data to the given stream.
   * @param s The stream to write to
   * @exception IOException In case the data output failed.
   */
  protected void writeData(ObjectOutputStream s) throws IOException {
    s.writeObject(table);
    s.writeObject(name);
    s.writeBoolean(key);
    s.writeInt(type);
    s.writeObject(alias);
    s.writeObject(format);
    s.writeInt(index);
    s.writeInt(dataIndex);
    s.writeObject(layoutHints);
  }
  /**
   * Reads the object's data from the given stream.
   * @param s The stream to read from.
   * @exception IOException In case the data reading failed.
   * @exception ClassNotFoundException In case the class to deserialize could not be found
   */
  protected void readData(ObjectInputStream s) throws IOException, ClassNotFoundException  {
    table = (Table) s.readObject();
    name = (String) s.readObject();
    key = s.readBoolean();
    type = s.readInt();
    alias = (String) s.readObject();
    format = (Format) s.readObject();
    index = s.readInt();
    dataIndex = s.readInt();
    layoutHints = (LayoutHints) s.readObject();
  }

  /**
   * Writes the object's data to the given stream.
   * @param s The stream to write to
   * @exception IOException In case the data output failed.
   */
  private void writeObject(ObjectOutputStream s) throws IOException {
    writeData(s);
  }
  /**
   * Reads the object's data from the given stream.
   * @param s The stream to read from.
   * @exception IOException In case the data reading failed.
   * @exception ClassNotFoundException In case the class to deserialize could not be found
   */
  private void readObject(ObjectInputStream s) throws IOException, ClassNotFoundException  {
    readData(s);
  }

}
