/*
 * Copyright (C) 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.table;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import java.util.Date;
import java.util.Iterator;

import org.ujac.util.TypeConverter;
import org.ujac.util.TypeConverterException;

import org.ujac.util.text.FormatHelper;

/**
 * Name: FilteredTable<br>
 * Description: A table implementation filtering nested tables according to 
 *   a given table definition.
 * 
 * @author lauerc
 */
public class FilteredTable
  extends BaseTable
  implements Table {

  /** The serial version UID. */
  static final long serialVersionUID = 2135433454353128079L;

  /** The data table. */
  private Table dataTable = null;
  /** The report functions .*/
  private Column[] columns = null;
  /** The row filter to use .*/
  private RowFilterRule filterRule = null;
  /** The first valid row. */
  private Row firstRow = null;
  
  /**
   * Constructs a FilteredTable instance with specific attributes.
   * @param dataTable The data table.
   * @exception TableException In case a table handling failure occurred.
   */
  public FilteredTable(Table dataTable)
    throws TableException {

    this.dataTable = dataTable;
    int numColumns = dataTable.getColumnCount();
    this.columns = new Column[numColumns];
    for (int i = 0; i < numColumns; i++) {
      columns[i] = dataTable.getColumn(i);
    }
  }

  /**
   * Constructs a FilteredTable instance with specific attributes.
   * @param dataTable The data table.
   * @param filterRule The filter rule to apply
   * @exception TableException In case a table handling failure occurred.
   */
  public FilteredTable(Table dataTable, RowFilterRule filterRule)
    throws TableException {

    this(dataTable);
    setFilterRule(filterRule);
  }
  
  /**
   * Sets the filter rule.
   * @param filterRule The row filter rule to set.
   * @exception TableException In case a table handling failure occurred.
   */
  public void setFilterRule(RowFilterRule filterRule)
    throws TableException {
      
    this.filterRule = filterRule;
    filterRule.setup(dataTable);
    
    // determining the first row
    firstRow = null;
    Iterator iter = iterator();
    if (iter.hasNext()) {
      firstRow = (Row) iter.next();
    }
  }
  /**
   * Gets the filter rule.
   * @return rowFilter The current filter rule.
   */
  public RowFilterRule getFilterRule() {
    return this.filterRule;
  }

  /**
   * Getter method for the the property formatHelper.
   * @return The current value of property formatHelper.
   */
  public FormatHelper getFormatHelper() {
    return dataTable.getFormatHelper();
  }
  /**
   * Setter method for the the property formatHelper.
   * @param formatHelper The value to set for the property formatHelper.
   */
  public void setFormatHelper(FormatHelper formatHelper) {
    dataTable.setFormatHelper(formatHelper);
  }

  /**
   * Getter method for the the property typeConverter.
   * @return The current value of property typeConverter.
   */
  public TypeConverter getTypeConverter() {
    return dataTable.getTypeConverter();
  }
  /**
   * Setter method for the the property typeConverter.
   * @param typeConverter The value to set for the property typeConverter.
   */
  public void setTypeConverter(TypeConverter typeConverter) {
    dataTable.setTypeConverter(typeConverter);
  }

  /**
   * @see org.ujac.util.table.Table#getColumn(int)
   */
  public Column getColumn(int index) throws ColumnNotDefinedException {
    return dataTable.getColumn(index);
  }

  /**
   * @see org.ujac.util.table.Table#getColumn(java.lang.String)
   */
  public Column getColumn(String name) throws ColumnNotDefinedException {
    return dataTable.getColumn(name);
  }

  /**
   * @see org.ujac.util.table.Table#getColumnIndex(java.lang.String)
   */
  public int getColumnIndex(String name) throws ColumnNotDefinedException {
    return dataTable.getColumnIndex(name);
  }

  /**
   * Sets the new order for the report columns according to the list of given
   * column names.
   * @param columnNames The list of column names after which to order the columns.
   * @exception ColumnNotDefinedException In case a column from the given order list
   *  does not exist. 
   * @exception ColumnAlreadyDefinedException In case a column was defined more than 
   *  once at the given order list.
   */
  public void orderColumns(String[] columnNames) 
    throws ColumnNotDefinedException, ColumnAlreadyDefinedException {
    
    this.dataTable.orderColumns(columnNames);
  }

  /**
   * Gets all visible columns.
   * @return An array holding all visible columns.
   */
  public Column[] getVisibleColumns() {
    return this.dataTable.getVisibleColumns();
  }

  /**
   * Gets all visible columns.
   * @param levelFilter The maximum level of importance to support.
   * @return An array holding all visible columns.
   */
  public Column[] getVisibleColumns(int levelFilter) {
    return this.dataTable.getVisibleColumns(levelFilter);
  }

  /**
   * @see org.ujac.util.table.Table#getHeaderRow()
   */
  public Row getHeaderRow() {
    return dataTable.getHeaderRow();
  }
  
  /**
   * Returns an iterator over the rows of this report in proper sequence.
   * @return an iterator over the rows of this report in proper sequence.
   */
  public Iterator iterator() {
    // creating iterator
    return new FilteredTableIterator(dataTable, filterRule); 
  }

  /**
   * Gets the current number of columns.
   * @return The number of columns.
   */
  public int getColumnCount() {
    return dataTable.getColumnCount();
  }

  /**
   * Getter method for the the property dataTable.
   * @return The current value of property dataTable.
   */
  protected Table getDataTable() {
    return dataTable;
  }


  /**
   * @see org.ujac.util.table.Table#addColumn(java.lang.String, int)
   */
  public Column addColumn(String name, int type) throws OperationNotSupportedException {
    throwNotSupportedException();
    return null;
  }

  /**
   * @see org.ujac.util.table.Table#addColumn(java.lang.String, int, LayoutHints)
   */
  public Column addColumn(String name, int type, LayoutHints layoutHints) throws OperationNotSupportedException {
    throwNotSupportedException();
    return null;
  }

  /**
   * @see org.ujac.util.table.Table#addString(java.lang.String, java.lang.String)
   */
  public Table addString(String name, String value) throws TableException {
    throwNotSupportedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#addInt(java.lang.String, int)
   */
  public Table addInt(String name, int value) throws TableException {
    throwNotSupportedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#addLong(java.lang.String, long)
   */
  public Table addLong(String name, long value) throws TableException {
    throwNotSupportedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#addFloat(java.lang.String, float)
   */
  public Table addFloat(String name, float value) throws TableException {
    throwNotSupportedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#addDouble(java.lang.String, double)
   */
  public Table addDouble(String name, double value) throws TableException {
    throwNotSupportedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#addBoolean(java.lang.String, boolean)
   */
  public Table addBoolean(String name, boolean value) throws TableException {
    throwNotSupportedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#addDate(java.lang.String, java.util.Date)
   */
  public Table addDate(String name, Date value) throws TableException {
    throwNotSupportedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#addTime(java.lang.String, java.util.Date)
   */
  public Table addTime(String name, Date value) throws TableException {
    throwNotSupportedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#addTimestamp(java.lang.String, java.util.Date)
   */
  public Table addTimestamp(String name, Date value) throws TableException {
    throwNotSupportedException();
    return null;
  }

  /**
   * Checks if the table is empty.
   * @return true if the table has no rows, else false.
   */
  public boolean isEmpty() {
    return firstRow != null;
  }
  
  /**
   * @see org.ujac.util.table.Table#getRowCount()
   */
  public int getRowCount() throws OperationNotSupportedException {
    throwNotSupportedException();
    return 0;
  }
  /**
   * @see org.ujac.util.table.Table#addRow()
   */
  public Row addRow() throws OperationNotSupportedException {
    throwNotSupportedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#addRow(org.ujac.util.table.Row)
   */
  public Row addRow(Row srcRow) throws OperationNotSupportedException {
    throwNotSupportedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#getRow(int)
   */
  public Row getRow(int rowIdx) throws RowNotDefinedException, OperationNotSupportedException {
    throwNotSupportedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#removeRow(int)
   */
  public void removeRow(int rowIdx) throws RowNotDefinedException, OperationNotSupportedException {
    throwNotSupportedException();
  }


  /**
   * @see org.ujac.util.table.Table#getString(int, int)
   */
  public String getString(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      return firstRow.getString(columnIdx);
    }
    throwDirectRowAccessNotAllowedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#getString(int, java.lang.String)
   */
  public String getString(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      return firstRow.getString(columnName);
    }
    throwDirectRowAccessNotAllowedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#setString(int, int, java.lang.String)
   */
  public void setString(int rowIdx, int columnIdx, String value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      firstRow.setString(columnIdx, value);
    }
    throwDirectRowAccessNotAllowedException();
  }
  /**
   * @see org.ujac.util.table.Table#setString(int, java.lang.String, java.lang.String)
   */
  public void setString(int rowIdx, String columnName, String value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      firstRow.setString(columnName, value);
    }
    throwDirectRowAccessNotAllowedException();
  }

  /**
   * @see org.ujac.util.table.Table#getInt(int, int)
   */
  public int getInt(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      return firstRow.getInt(columnIdx);
    }
    throwDirectRowAccessNotAllowedException();
    return 0;
  }
  /**
   * @see org.ujac.util.table.Table#getInt(int, java.lang.String)
   */
  public int getInt(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      return firstRow.getInt(columnName);
    }
    throwDirectRowAccessNotAllowedException();
    return 0;
  }
  /**
   * @see org.ujac.util.table.Table#setInt(int, int, int)
   */
  public void setInt(int rowIdx, int columnIdx, int value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      firstRow.setInt(columnIdx, value);
    }
    throwDirectRowAccessNotAllowedException();
  }
  /**
   * @see org.ujac.util.table.Table#setInt(int, java.lang.String, int)
   */
  public void setInt(int rowIdx, String columnName, int value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      firstRow.setInt(columnName, value);
    }
    throwDirectRowAccessNotAllowedException();
  }

  /**
   * @see org.ujac.util.table.Table#getLong(int, int)
   */
  public long getLong(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      return firstRow.getLong(columnIdx);
    }
    throwDirectRowAccessNotAllowedException();
    return 0L;
  }
  /**
   * @see org.ujac.util.table.Table#getLong(int, java.lang.String)
   */
  public long getLong(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      return firstRow.getLong(columnName);
    }
    throwDirectRowAccessNotAllowedException();
    return 0L;
  }
  /**
   * @see org.ujac.util.table.Table#setLong(int, int, long)
   */
  public void setLong(int rowIdx, int columnIdx, long value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      firstRow.setLong(columnIdx, value);
    }
    throwDirectRowAccessNotAllowedException();
  }
  /**
   * @see org.ujac.util.table.Table#setLong(int, java.lang.String, long)
   */
  public void setLong(int rowIdx, String columnName, long value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      firstRow.setLong(columnName, value);
    }
    throwDirectRowAccessNotAllowedException();
  }

  /**
   * @see org.ujac.util.table.Table#getFloat(int, int)
   */
  public float getFloat(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      return firstRow.getFloat(columnIdx);
    }
    throwDirectRowAccessNotAllowedException();
    return 0F;
  }
  /**
   * @see org.ujac.util.table.Table#getFloat(int, java.lang.String)
   */
  public float getFloat(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      return firstRow.getFloat(columnName);
    }
    throwDirectRowAccessNotAllowedException();
    return 0F;
  }
  /**
   * @see org.ujac.util.table.Table#setFloat(int, int, float)
   */
  public void setFloat(int rowIdx, int columnIdx, float value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      firstRow.setFloat(columnIdx, value);
    }
    throwDirectRowAccessNotAllowedException();
  }
  /**
   * @see org.ujac.util.table.Table#setFloat(int, java.lang.String, float)
   */
  public void setFloat(int rowIdx, String columnName, float value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      firstRow.setFloat(columnName, value);
    }
    throwDirectRowAccessNotAllowedException();
  }

  /**
   * @see org.ujac.util.table.Table#getDouble(int, int)
   */
  public double getDouble(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      return firstRow.getDouble(columnIdx);
    }
    throwDirectRowAccessNotAllowedException();
    return 0.0;
  }
  /**
   * @see org.ujac.util.table.Table#getDouble(int, java.lang.String)
   */
  public double getDouble(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      return firstRow.getDouble(columnName);
    }
    throwDirectRowAccessNotAllowedException();
    return 0.0;
  }
  /**
   * @see org.ujac.util.table.Table#setDouble(int, int, double)
   */
  public void setDouble(int rowIdx, int columnIdx, double value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      firstRow.setDouble(columnIdx, value);
    }
    throwDirectRowAccessNotAllowedException();
  }
  /**
   * @see org.ujac.util.table.Table#setDouble(int, java.lang.String, double)
   */
  public void setDouble(int rowIdx, String columnName, double value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      firstRow.setDouble(columnName, value);
    }
    throwDirectRowAccessNotAllowedException();
  }

  /**
   * @see org.ujac.util.table.Table#getBoolean(int, int)
   */
  public boolean getBoolean(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      return firstRow.getBoolean(columnIdx);
    }
    throwDirectRowAccessNotAllowedException();
    return false;
  }
  /**
   * @see org.ujac.util.table.Table#getBoolean(int, java.lang.String)
   */
  public boolean getBoolean(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      return firstRow.getBoolean(columnName);
    }
    throwDirectRowAccessNotAllowedException();
    return false;
  }
  /**
   * @see org.ujac.util.table.Table#setBoolean(int, int, boolean)
   */
  public void setBoolean(int rowIdx, int columnIdx, boolean value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      firstRow.setBoolean(columnIdx, value);
    }
    throwDirectRowAccessNotAllowedException();
  }
  /**
   * @see org.ujac.util.table.Table#setBoolean(int, java.lang.String, boolean)
   */
  public void setBoolean(int rowIdx, String columnName, boolean value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      firstRow.setBoolean(columnName, value);
    }
    throwDirectRowAccessNotAllowedException();
  }

  /**
   * @see org.ujac.util.table.Table#getDate(int, int)
   */
  public Date getDate(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      return firstRow.getDate(columnIdx);
    }
    throwDirectRowAccessNotAllowedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#getDate(int, java.lang.String)
   */
  public Date getDate(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      return firstRow.getDate(columnName);
    }
    throwDirectRowAccessNotAllowedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#setDate(int, int, java.util.Date)
   */
  public void setDate(int rowIdx, int columnIdx, Date value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      firstRow.setDate(columnIdx, value);
    }
    throwDirectRowAccessNotAllowedException();
  }
  /**
   * @see org.ujac.util.table.Table#setDate(int, java.lang.String, java.util.Date)
   */
  public void setDate(int rowIdx, String columnName, Date value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      firstRow.setDate(columnName, value);
    }
    throwDirectRowAccessNotAllowedException();
  }

  /**
   * @see org.ujac.util.table.Table#getTime(int, int)
   */
  public Date getTime(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      return firstRow.getTime(columnIdx);
    }
    throwDirectRowAccessNotAllowedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#getTime(int, java.lang.String)
   */
  public Date getTime(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      return firstRow.getTime(columnName);
    }
    throwDirectRowAccessNotAllowedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#setTime(int, int, java.util.Date)
   */
  public void setTime(int rowIdx, int columnIdx, Date value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      firstRow.setTime(columnIdx, value);
    }
    throwDirectRowAccessNotAllowedException();
  }
  /**
   * @see org.ujac.util.table.Table#setTime(int, java.lang.String, java.util.Date)
   */
  public void setTime(int rowIdx, String columnName, Date value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      firstRow.setTime(columnName, value);
    }
    throwDirectRowAccessNotAllowedException();
  }

  /**
   * @see org.ujac.util.table.Table#getTimestamp(int, int)
   */
  public Date getTimestamp(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      return firstRow.getTimestamp(columnIdx);
    }
    throwDirectRowAccessNotAllowedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#getTimestamp(int, java.lang.String)
   */
  public Date getTimestamp(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      return firstRow.getTimestamp(columnName);
    }
    throwDirectRowAccessNotAllowedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#setTimestamp(int, int, java.util.Date)
   */
  public void setTimestamp(int rowIdx, int columnIdx, Date value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      firstRow.setTimestamp(columnIdx, value);
    }
    throwDirectRowAccessNotAllowedException();
  }
  /**
   * @see org.ujac.util.table.Table#setTimestamp(int, java.lang.String, java.util.Date)
   */
  public void setTimestamp(int rowIdx, String columnName, Date value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      firstRow.setTimestamp(columnName, value);
    }
    throwDirectRowAccessNotAllowedException();
  }

  /**
   * @see org.ujac.util.table.Table#getObject(int, int)
   */
  public Object getObject(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException {
    if (rowIdx == 0) {
      return firstRow.getObject(columnIdx);
    }
    throwDirectRowAccessNotAllowedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#getObject(int, java.lang.String)
   */
  public Object getObject(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException {
    if (rowIdx == 0) {
      return firstRow.getObject(columnName);
    }
    throwDirectRowAccessNotAllowedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#setObject(int, int, java.lang.Object)
   */
  public void setObject(int rowIdx, int columnIdx, Object value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      firstRow.setObject(columnIdx, value);
    }
    throwDirectRowAccessNotAllowedException();
  }
  /**
   * @see org.ujac.util.table.Table#setObject(int, java.lang.String, java.lang.Object)
   */
  public void setObject(int rowIdx, String columnName, Object value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    if (rowIdx == 0) {
      firstRow.setObject(columnName, value);
    }
    throwDirectRowAccessNotAllowedException();
  }


  /**
   * @see org.ujac.util.table.Table#sort(org.ujac.util.table.CompareItem[])
   */
  public void sort(CompareItem[] compareColumns) throws OperationNotSupportedException {
    throwNotSupportedException();
  }


  /**
   * @see org.ujac.util.table.Table#sort(int, int)
   */
  public void sort(int columnIdx, int direction) throws OperationNotSupportedException {
    throwNotSupportedException();
  }

  /**
   * Changes the type of the given column.
   * @param columnIdx The index of the column which has been changed.
   * @param typeId The new column type ID.
   * @exception ColumnNotDefinedException In case the desired column does not exist.
   * @exception TypeConverterException In case the object conversion failed.
   */
  public void setColumnType(int columnIdx, int typeId)
    throws ColumnNotDefinedException, TypeConverterException {
    
    dataTable.setColumnType(columnIdx, typeId);
  }

  /**
   * Changes the type of the given column.
   * @param columnName The name of the column which has been changed.
   * @param typeId The new column type ID.
   * @exception ColumnNotDefinedException In case the desired column does not exist.
   * @exception TypeConverterException In case the object conversion failed.
   */
  public void setColumnType(String columnName, int typeId)
    throws ColumnNotDefinedException, TypeConverterException {
    
    dataTable.setColumnType(columnName, typeId);
  }

  
  /**
   * Throws an exception, telling that the operation is not supported.
   * @throws OperationNotSupportedException Thrown to tell that the operation is not supported.
   */
  private void throwNotSupportedException() throws OperationNotSupportedException {
    throw new OperationNotSupportedException("This operation is not supported for filtered tables.");
  }
  /**
   * Throws an runtime exception, telling that direct row access is not supported for filtered tables.
   * @throws RuntimeException Thrown to tell that the operation is not supported.
   */
  private void throwDirectRowAccessNotAllowedException() throws RuntimeException {
    throw new RuntimeException("Direct row access is not supported for filtered tables.");
  }
  
  /**
   * Writes the object's data to the given stream.
   * @param s The stream to write to
   * @exception IOException In case the data output failed.
   */
  private void writeObject(ObjectOutputStream s) throws IOException {
    s.writeObject(dataTable);
    s.writeObject(columns);
  }
  /**
   * Reads the object's data from the given stream.
   * @param s The stream to read from.
   * @exception IOException In case the data reading failed.
   * @exception ClassNotFoundException In case the class to deserialize could not be found
   */
  private void readObject(ObjectInputStream s) throws IOException, ClassNotFoundException  {
    dataTable = (Table) s.readObject();
    columns = (Column[]) s.readObject();
  }

}
