/*
 * Copyright (C) 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.table;

import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Name: FilteredTableIterator<br>
 * Description: An iterator for the class FilteredTable.
 * 
 * @author lauerc
 */
public class FilteredTableIterator implements Iterator {

  /** The table to iterate. */
  private Table table = null;
  /** The filter rule to use. */
  private RowFilterRule filterRule = null;

  /** The table iterator, used to iterate over the given table. */
  private Iterator tableIterator = null;
  /** The prefetched data row. */
  private Row prefetchedRow = null;

  /**
   * Constructs a FilteredTableIterator instance with specific attributes.
   * @param table The table to iterate.
   * @param filterRule The filter rule to use.
   */
  protected FilteredTableIterator(Table table, RowFilterRule filterRule) {
    this.table = table;
    this.filterRule = filterRule;
    this.tableIterator = table.iterator();

    // prefetching first row
    prefetchRow();
  }
  
  /**
   * @see java.util.Iterator#hasNext()
   */
  public boolean hasNext() {
    return prefetchedRow != null;
  }

  /**
   * @see java.util.Iterator#next()
   */
  public Object next() {
    Row currentRow = prefetchedRow;
    if (currentRow == null) {
      throw new NoSuchElementException();
    }
    prefetchedRow = null;
    
    // prefetching next row
    prefetchRow();
    
    return currentRow;
  }
  
  /**
   * Prefetches the next valid row.
   */
  private void prefetchRow() {
    try {
      while (tableIterator.hasNext()) {
        Row row = (Row) tableIterator.next();
        if (filterRule.test(row)) {
          prefetchedRow = row;
          break;
        }
      }
    } catch (TableException ex) {
      throw new RuntimeException("Unexpected error while prefetching next valid row.", ex);
    }
  }

  /**
   * @see java.util.Iterator#remove()
   */
  public void remove() {
    throw new IllegalStateException("Removing rows is not permitted for filtered tables!");
  }

}
