/*
 * Copyright (C) 2003-2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.table;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

/**
 * Name: LayoutHints<br>
 * Description: A class holding layout hints for table columns.
 * 
 * @author lauerc
 */
public class LayoutHints implements Serializable {

  /** The serial version UID. */
  static final long serialVersionUID = -5722131385432713833L;

  /** Constant for left alignment. */
  public static final int LEFT = 0;
  /** Constant for left alignment. */
  public static final int CENTER = 1;
  /** Constant for left alignment. */
  public static final int RIGHT = 2;
  
  /** The visible width of the column. */
  private float width = 0.0F;
  /** The column header. */
  private String header = null;
  /** The column alignment. */
  private int align = LEFT;
  /** The column header alignment. */
  private int headerAlign = LEFT;
  /** The visibility flag. */
  private boolean visible = true;
  /** The level of importance, the higher the value the less important it is. */
  private int level = 0;
    
  /**
   * Constructs a LayoutHints instance with no specific attributes.
   */
  public LayoutHints() {
  }

  /**
   * Constructs a LayoutHints instance with specific attributes.
   * @param width The column width.
   * @param title The column title.
   */
  public LayoutHints(float width, String title) {
    this(width, title, LEFT, true);
  }

  /**
   * Constructs a LayoutHints instance with specific attributes.
   * @param width The column width.
   * @param title The column title.
   * @param align The column alignment (LEFT|CENTER|RIGHT).
   */
  public LayoutHints(float width, String title, int align) {
    this(width, title, align, true);
  }

  /**
   * Constructs a LayoutHints instance with specific attributes.
   * @param width The column width.
   * @param header The column title.
   * @param align The column alignment (LEFT|CENTER|RIGHT).
   * @param visible The visibility flag.
   */
  public LayoutHints(float width, String header, int align, boolean visible) {
    this.width = width;
    this.header = header;
    this.align = align;
    this.headerAlign = align;
    this.visible = visible;
  }


  /**
   * Getter method for the the property align.
   * @return The current value of property align.
   */
  public int getAlign() {
    return align;
  }
  /**
   * Gets the textual representation of the column alignment.
   * @return The textual representation of the current column alignment.
   */
  public String getAlignText() {
    return TableHelper.getAlignmentName(align);
  }
  /**
   * Setter method for the the property align.
   * @param align The value to set for the property align.
   * @return The reference to this instance. 
   */
  public LayoutHints setAlign(int align) {
    this.align = align;
    return this;
  }

  /**
   * Getter method for the the property headerAlign.
   * @return The current value of property headerAlign.
   */
  public int getHeaderAlign() {
    return headerAlign;
  }
  /**
   * Gets the textual representation of the column header alignment.
   * @return The textual representation of the current column header alignment.
   */
  public String getHeaderAlignText() {
    return TableHelper.getAlignmentName(headerAlign);
  }
  /**
   * Setter method for the the property headerAlign.
   * @param headerAlign The value to set for the property headerAlign.
   * @return The reference to this instance. 
   */
  public LayoutHints setHeaderAlign(int headerAlign) {
    this.headerAlign = headerAlign;
    return this;
  }

  /**
   * Getter method for the the property title.
   * @return The current value of property title.
   */
  public String getHeader() {
    return header;
  }
  /**
   * Setter method for the the property title.
   * @param header The value to set for the property header.
   * @return The reference to this instance. 
   */
  public LayoutHints setHeader(String header) {
    this.header = header;
    return this;
  }

  /**
   * Getter method for the the property visible.
   * @return The current value of property visible.
   */
  public boolean isVisible() {
    return visible;
  }
  /**
   * Setter method for the the property visible.
   * @param visible The value to set for the property visible.
   * @return The reference to this instance. 
   */
  public LayoutHints setVisible(boolean visible) {
    this.visible = visible;
    return this;
  }

  /**
   * Gets the level of importance.
   * @return The current level of importance, the higher the value the less important it is.
   */
  public int getLevel() {
    return level;
  }
  /**
   * Sets the level of importance.
   * @param level The level of importance, the higher the value the less important it is.
   * @return A reference to the altered FormField instance. 
   */
  public LayoutHints setLevel(int level) {
    this.level = level;
    return this;
  }

  /**
   * Getter method for the the property width.
   * @return The current value of property width.
   */
  public float getWidth() {
    return width;
  }
  /**
   * Setter method for the the property width.
   * @param width The value to set for the property width.
   * @return The reference to this instance. 
   */
  public LayoutHints setWidth(float width) {
    this.width = width;
    return this;
  }

  /**
   * Writes the object's data to the given stream.
   * @param s The stream to write to
   * @exception IOException In case the data output failed.
   */
  private void writeObject(ObjectOutputStream s) throws IOException {
    s.writeFloat(width);
    s.writeObject(header);
    s.writeInt(align);
    s.writeInt(headerAlign);
    s.writeBoolean(visible);
    s.writeInt(level);
  }
  /**
   * Reads the object's data from the given stream.
   * @param s The stream to read from.
   * @exception IOException In case the data reading failed.
   * @exception ClassNotFoundException In case the class to deserialize could not be found
   */
  private void readObject(ObjectInputStream s) throws IOException, ClassNotFoundException  {
    width = s.readFloat();
    header = (String) s.readObject();
    align = s.readInt();
    headerAlign = s.readInt();
    visible = s.readBoolean();
    level = s.readInt();
  }

}
