/*
 * Copyright (C) 2005 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.table;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.ujac.util.BeanUtils;

/**
 * Name: PivotTable<br>
 * Description: A table implementation, generating pivot tables out of 
 *   ordered data tables.
 * 
 * @author lauerc
 */
public class PivotTable extends DataTable {

  /** The serial version UID. */
  private static final long serialVersionUID = -6469000018376336834L;

  /** Constant for function type 'SUM'. */
  public static final int FUNCTION_SUM = 1;
  /** Constant for function type 'COUNT'. */
  public static final int FUNCTION_COUNT = 2;
  
  /**
   * Constructs a PivotTable instance with specific attributes.
   * @param dataTable The data table.
   * @param groupColumns The group column list.
   * @param columnKey The column key.
   * @param dataKey The data key.
   * @param function The function to use.
   * @exception TableException In case a table access failure occurs.
   */
  public PivotTable(Table dataTable, String groupColumns[], String columnKey, String dataKey, FunctionExecutor function)
    throws TableException {
    
    int numGroupColumns = groupColumns.length;
    int[] srcGroupColumnIdxs = new int[numGroupColumns];
    int[] dstGroupColumnIdxs = new int[numGroupColumns];
    for (int i = 0; i < numGroupColumns; i++) {
      String colName = groupColumns[i];
      Column col = dataTable.getColumn(colName);
      srcGroupColumnIdxs[i] = col.getIndex();
      dstGroupColumnIdxs[i] = addColumn(colName, col.isKey(), col.getType()).setLayoutHints(col.getLayoutHints()).getIndex();
    }
    
    Column colCol = dataTable.getColumn(columnKey);
    int columnColumnIdx = colCol.getIndex();

    Column dataCol = dataTable.getColumn(dataKey);
    int dataColumnIdx = dataCol.getIndex();

    int sumColumnIdx = -1;

    List sumList = new ArrayList();
    double groupResult = 0.0;
    Object curGroupVal = null;
    Row curRow = null;
    boolean first = true;
    Iterator dataTableIter = dataTable.iterator();
    while (dataTableIter.hasNext()) {
      Row row = (Row) dataTableIter.next();
      Object groupVal = null;
      StringBuffer grpBuf = new StringBuffer();
      for (int i = 0; i < numGroupColumns; i++) {
        groupVal = row.getObject(srcGroupColumnIdxs[i]);
        grpBuf.append(groupVal).append(";");
      }
      groupVal = grpBuf.toString();
        
      String colName = row.getString(columnColumnIdx);

      if ((first) || !BeanUtils.equals(groupVal, curGroupVal)) {
        if (!first) {
          if (function.integersGuaranteed()) {
            sumList.add(new Integer((int) groupResult));
          } else {
            sumList.add(new Double(groupResult));
          }
        }
        first = false;
        curGroupVal = groupVal;
        groupResult = 0.0;
        curRow = addRow();
        
        for (int i = 0; i < numGroupColumns; i++) {
          groupVal = row.getObject(srcGroupColumnIdxs[i]);
          curRow.setObject(dstGroupColumnIdxs[i], groupVal);
        }
      }

      Column curCol = null;
      try {
        curCol = getColumn(colName);
      } catch (ColumnNotDefinedException ex) {
        if (function.integersGuaranteed()) {
          curCol = addColumn(colName, TYPE_INT, new LayoutHints(10, colName));
        } else {
          curCol = addColumn(colName, TYPE_DOUBLE, new LayoutHints(10, colName));
        }
      }
      
      Object value = row.getObject(dataColumnIdx);
      groupResult = function.execute(groupResult, value);
      
      double curVal = curRow.getDouble(curCol.getIndex());
      curVal = function.execute(curVal, value);
      
      if (function.integersGuaranteed()) {
        curRow.setInt(curCol.getIndex(), (int) curVal);
      } else {
        curRow.setDouble(curCol.getIndex(), curVal);
      }
    }
    if (function.integersGuaranteed()) {
      sumList.add(new Integer((int) groupResult));
    } else {
      sumList.add(new Double(groupResult));
    }

    if (function.integersGuaranteed()) {
      sumColumnIdx = addColumn("sum", TYPE_INT, new LayoutHints(10, "Sum")).getIndex();
    } else {
      sumColumnIdx = addColumn("sum", TYPE_DOUBLE, new LayoutHints(10, "Sum")).getIndex();
    }
    int numSums = sumList.size();
    for (int i = 0; i < numSums; i++) {
      Row row = getRow(i);
      row.setObject(sumColumnIdx, sumList.get(i));
    }
  }
  
}
