/*
 * Copyright (C) 2003, 2004, 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.table;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

import java.text.Format;

import java.util.ArrayList;
import java.util.List;

/**
 * Name: ReportFunction<br>
 * Description: A class describing and managing report funtions.
 * 
 * @author lauerc
 */
public class ReportFunction implements Cloneable, TableConstants, Serializable {

  /** The serial version UID. */
  static final long serialVersionUID = -6712139086971422640L;

  /** The name of the column to apply the function on. */
  private String columnName = null;
  /** Tells whether or not to calculate the total result. */
  private boolean calculateTotalResult = false;
  /** The function executor. */
  private FunctionExecutor executor = null;
  /** The column index. */
  private int columnIdx = -1;
  /** The report. */
  private ReportTable report = null;
  /** The overal result. */
  private double totalResult = 0.0;
  /** The report groups to apply the function on. */
  private List groupNames = new ArrayList();
  /** The report groups to apply the function on. */
  private ReportGroup[] groups = null;
  /** The output format of the function result. */
  private Format format = null;
  /** The alignment for the function result. */
  private int align = LayoutHints.RIGHT;
  
  /**
   * Constructs a ReportFunction instance with specific attributes.
   * @param executor The function executor.
   * @param columnName The column name.
   * @param calculateTotalResult Tells whether or not to calculate the total result.
   */
  public ReportFunction(String columnName, FunctionExecutor executor, boolean calculateTotalResult) {
    this.columnName = columnName;
    this.executor = executor;
    this.calculateTotalResult = calculateTotalResult;
  }

  /**
   * Constructs a ReportFunction instance with specific attributes.
   * @param executor The function executor.
   * @param columnName The column name.
   * @param calculateTotalResult Tells whether or not to calculate the total result.
   * @param groups The report groups to apply the function on.
   */
  public ReportFunction(String columnName, FunctionExecutor executor, 
                        boolean calculateTotalResult, String[] groups) {
    this.columnName = columnName;
    this.executor = executor;
    this.calculateTotalResult = calculateTotalResult;
    if (groups != null) {
      for (int i = 0; i < groups.length; i++) {
        addGroup(groups[i]);
      }
    }
  }

  /**
   * Constructs a ReportFunction instance with specific attributes.
   * @param executor The function executor.
   * @param columnName The column name.
   * @param calculateTotalResult Tells whether or not to calculate the total result.
   * @param groups The report groups to apply the function on.
   */
  public ReportFunction(String columnName, FunctionExecutor executor, 
                        boolean calculateTotalResult, List groups) {
    this.columnName = columnName;
    this.executor = executor;
    this.calculateTotalResult = calculateTotalResult;
    if (groups != null) {
      int numGroups = groups.size();
      for (int i = 0; i < numGroups; i++) {
        addGroup((String) groups.get(i));
      }
    }
  }

  /**
   * Gets the column name.
   * @return The name of the column to apply the function on.
   */
  public String getColumnName() {
    return columnName;
  }

  /**
   * Checks whether or not to calculate the total result.
   * @return true: calculate the global result, else don't calculate.
   */
  public boolean isCalculateTotalResult() {
    return calculateTotalResult;
  }
  
  /**
   * Getter method for the the property columnIdx.
   * @return The current value of property columnIdx.
   */
  public int getColumnIdx() {
    return columnIdx;
  }
  /**
   * Setter method for the the property columnIdx.
   * @param columnIdx The value to set for the property columnIdx.
   */
  protected void setColumnIdx(int columnIdx) {
    this.columnIdx = columnIdx;
  }

  /**
   * Getter method for the the property executor.
   * @return The current value of property executor.
   */
  protected FunctionExecutor getExecutor() {
    return executor;
  }

  /**
   * Getter method for the the property totalResult.
   * @return The current value of property totalResult.
   */
  public double getTotalResult() {
    return totalResult;
  }

  /**
   * Adds a column to group by.
   * @param groupName The name of the column to group by.
   */
  public void addGroup(String groupName) {
    this.groupNames.add(groupName);
  }

  /**
   * Getter method for the the property align.
   * @return The current value of property align.
   */
  public int getAlign() {
    return align;
  }
  /**
   * Setter method for the the property align.
   * @param align The value to set for the property align.
   */
  public void setAlign(int align) {
    this.align = align;
  }

  /**
   * Getter method for the the property format.
   * @return The current value of property format.
   */
  public Format getFormat() {
    return format;
  }
  /**
   * Setter method for the the property format.
   * @param format The value to set for the property format.
   */
  public void setFormat(Format format) {
    this.format = format;
  }

  /**
   * Initializes the report function.
   * @param report The report to set.
   * @param allGroups An array providing all available report groups.
   */
  protected void init(ReportTable report, ReportGroup[] allGroups) {
    this.report = report;
    this.totalResult = 0.0;
    
    if (columnIdx < 0) {
      try {
        columnIdx = report.getColumnIndex(columnName);
      } catch (ColumnNotDefinedException ex) {
        // this should never happen!
        throw new TableIterationException(ex.getMessage());
      }
    }
    
    int numGroupNames = groupNames.size();
    this.groups = new ReportGroup[numGroupNames];
    int groupIdx = 0;
    for (int i = 0; i < numGroupNames; i++) {
      String groupName = (String) groupNames.get(i);
      ReportGroup group = null;
      for (int j = 0; j < allGroups.length; j++) {
        group = allGroups[j];
        if ((group != null) && group.getColumnName().equals(groupName)) {
          groups[groupIdx] = group;
          ++groupIdx;
          break;
        }
        group = null; 
      }
      // checking group coverage
      if (group == null) {
        throw new TableIterationException("Detected undefined group '" + groupName + "'");
      }
    }
    
    if (this.format == null) {
      if (executor.integersGuaranteed()) {
        this.format = report.getDataTable().getFormatHelper().getIntegerFormat();
      } else  {
        try {
          this.format = report.getDataTable().getColumn(columnIdx).getFormat();
          if (format == null) {
            switch (report.getColumn(columnIdx).getType()) {
              case TYPE_INT:
              case TYPE_LONG:
                this.format = report.getDataTable().getFormatHelper().getIntegerFormat();
                break;
                
              case TYPE_FLOAT:
              case TYPE_DOUBLE:
                this.format = report.getDataTable().getFormatHelper().getDoubleFormat();
                break;
            }
          }
        } catch (ColumnNotDefinedException ex) {
          // this should never happen!
          throw new TableIterationException(ex.getMessage());
        }
      }
    }
  }

  /**
   * Applies the function for the given operand on the given current value.
   * @param operand The operand to apply.
   * @exception TypeMismatchException In case the given operand type was invalid.
   */
  public void apply(Object operand) throws TypeMismatchException {
    if (calculateTotalResult) {
      // applying executor on total result
      totalResult = executor.execute(totalResult, operand);
    }
    if (groups != null) {
      // applying executor on groups
      for (int i = 0; i < groups.length; i++) {
        ReportGroup group = groups[i];
        group.applyFunction(columnIdx, executor, operand);
      }
    }
  }
  
  /**
   * @see java.lang.Object#clone()
   */
  protected Object clone() {
    ReportFunction clone = null;
    try {
      clone = (ReportFunction) super.clone();
    } catch (CloneNotSupportedException ex) {
      throw new RuntimeException("Failed to clone Operand instance: " + ex.getMessage(), ex);
    }
    clone.groupNames = (List) ((ArrayList) groupNames).clone();
    return clone;
  }

  /**
   * Writes the object's data to the given stream.
   * @param s The stream to write to
   * @exception IOException In case the data output failed.
   */
  private void writeObject(ObjectOutputStream s) throws IOException {
    s.writeObject(columnName);
    s.writeBoolean(calculateTotalResult);
    s.writeObject(executor);
    s.writeInt(columnIdx);
    s.writeObject(report);
    s.writeDouble(totalResult);
    s.writeObject(groupNames);
    s.writeObject(groups);
    s.writeObject(format);
    s.writeInt(align);
  }
  /**
   * Reads the object's data from the given stream.
   * @param s The stream to read from.
   * @exception IOException In case the data reading failed.
   * @exception ClassNotFoundException In case the class to deserialize could not be found
   */
  private void readObject(ObjectInputStream s) throws IOException, ClassNotFoundException  {
    columnName = (String) s.readObject();
    calculateTotalResult = s.readBoolean();
    executor = (FunctionExecutor) s.readObject();
    columnIdx = s.readInt();
    report = (ReportTable) s.readObject();
    totalResult = s.readDouble();
    groupNames = (List) s.readObject();
    groups = (ReportGroup[]) s.readObject();
    format = (Format) s.readObject();
    align = s.readInt();
  }

}
