/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.table;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

/**
 * Name: ReportGroup<br>
 * Description: A class describing and managing report groups.
 * 
 * @author lauerc
 */
public class ReportGroup implements Cloneable, Serializable {

  /** The serial version UID. */
  static final long serialVersionUID = 3181590254515459952L;

  /** The name of the column to apply the function on. */
  private String columnName = null;
  /** Tells whether the group title row shall be shown. */
  private boolean viewTitleRow;
  /** Tells whether or not the results shall be shown in the title row. */
  private boolean resultsInTitleRow;
  /**
   * Tells whether or not to hide the group column at succeeding rows.
   * If true, only the first row is shown the succeeding ones are hidden.
   */
  private boolean hideAtSucceedingRows;
  /**
   * Tells whether or not to start a new page if a change of the value
   * at the grouping column is detected.
   */
  private boolean startNewPage;
  /**
   * Tells whether or not to try to keep the grouped rows together on one page.
   */
  private boolean keepTogether;
  /** 
   * Enforces the visibility of a result row for the group. 
   * This flag will make the ReportIterator create a result row even if there's 
   * no result or the result is printed in title row (requested by Marc Laubie).
   */
  private boolean enforceResultRow;
  /** The column index. */
  private int columnIdx = -1;
  /** The report. */
  private ReportTable report = null;
  /** The group result. */
  private double[] results = null;
  /** Tells whether functions have been applied on this group or not. */
  private boolean[] functionsApplied = null;
  /** The current title row. */
  private Row titleRow = null;
  
  /**
   * Constructs a ReportGroup instance with specific attributes.
   * @param columnName The column name.
   */
  public ReportGroup(String columnName) {
    this.columnName = columnName;
  }

  /**
   * Constructs a ReportGroup instance with specific attributes.
   * @param columnName The column name.
   * @param viewTitleRow The view title row flag.
   * @param resultsInTitleRow The results in title row flag.
   * @param hideAtSucceedingRows Tells whether or not to hide the grouped column at
   *    succeeding rows. If true, only the first row is shown the succeeding ones are
   *    hidden.
   * @param startNewPage Tells whether or not to start a new page if a change of the
   *    value at the grouping column is detected.
   */
  public ReportGroup(String columnName, boolean viewTitleRow, boolean resultsInTitleRow,
                     boolean hideAtSucceedingRows, boolean startNewPage) {
    this(columnName, viewTitleRow, resultsInTitleRow, hideAtSucceedingRows, startNewPage, false);
  }

  /**
   * Constructs a ReportGroup instance with specific attributes.
   * @param columnName The column name.
   * @param viewTitleRow The view title row flag.
   * @param resultsInTitleRow The results in title row flag.
   * @param hideAtSucceedingRows Tells whether or not to hide the grouped column at
   *    succeeding rows. If true, only the first row is shown the succeeding ones are
   *    hidden.
   * @param startNewPage Tells whether or not to start a new page if a change of the
   *    value at the grouping column is detected.
   * @param keepTogether Tells whether or not to try to keep the grouped rows 
   *    together on one page.
   */
  public ReportGroup(String columnName, boolean viewTitleRow, boolean resultsInTitleRow,
                     boolean hideAtSucceedingRows, boolean startNewPage, boolean keepTogether) {
    this.columnName = columnName;
    this.viewTitleRow = viewTitleRow;
    this.resultsInTitleRow = resultsInTitleRow;
    this.hideAtSucceedingRows = hideAtSucceedingRows;
    this.startNewPage = startNewPage;
    this.keepTogether = keepTogether;
  }

  /**
   * Gets the column name.
   * @return The name of the column to apply the function on.
   */
  public String getColumnName() {
    return columnName;
  }

  /**
   * Getter method for the the property viewTitleRow.
   * @return The current value of property viewTitleRow.
   */
  public boolean isViewTitleRow() {
    return viewTitleRow;
  }
  /**
   * Setter method for the the property viewTitleRow.
   * @param viewTitleRow The value to set for the property viewTitleRow.
   * @return The reference to this instance. 
   */
  public ReportGroup setViewTitleRow(boolean viewTitleRow) {
    this.viewTitleRow = viewTitleRow;
    return this;
  }

  /**
   * Getter method for the the property resultsInTitleRow.
   * @return The current value of property resultsInTitleRow.
   */
  public boolean isResultsInTitleRow() {
    return resultsInTitleRow;
  }
  /**
   * Setter method for the the property resultsInTitleRow.
   * @param resultsInTitleRow The value to set for the property resultsInTitleRow.
   * @return The reference to this instance. 
   */
  public ReportGroup setResultsInTitleRow(boolean resultsInTitleRow) {
    this.resultsInTitleRow = resultsInTitleRow;
    return this;
  }

  /**
   * Getter method for the the property hideAtSucceedingRows.
   * @return The current value of property hideAtSucceedingRows.
   */
  public boolean isHideAtSucceedingRows() {
    return hideAtSucceedingRows;
  }
  /**
   * Setter method for the the property hideAtSucceedingRows.
   * @param hideAtSucceedingRows The value to set for the property hideAtSucceedingRows.
   * @return The reference to this instance. 
   */
  public ReportGroup setHideAtSucceedingRows(boolean hideAtSucceedingRows) {
    this.hideAtSucceedingRows = hideAtSucceedingRows;
    return this;
  }

  /**
   * Getter method for the the property startNewPage.
   * @return The current value of property startNewPage.
   */
  public boolean isStartNewPage() {
    return startNewPage;
  }
  /**
   * Setter method for the the property startNewPage.
   * @param startNewPage The value to set for the property startNewPage.
   * @return The reference to this instance. 
   */
  public ReportGroup setStartNewPage(boolean startNewPage) {
    this.startNewPage = startNewPage;
    return this;
  }
  
  /**
   * Getter method for the the property keepTogether.
   * @return The current value of property keepTogether.
   */
  public boolean isKeepTogether() {
    return keepTogether;
  }
  /**
   * Setter method for the the property keepTogether.
   * @param keepTogether The value to set for the property keepTogether.
   * @return The reference to this instance. 
   */
  public ReportGroup setKeepTogether(boolean keepTogether) {
    this.keepTogether = keepTogether;
    return this;
  }
  
  /**
   * Getter method for the the property enforceResultRow.
   * @return The current value of property enforceResultRow.
   */
  public boolean isEnforceResultRow() {
    return enforceResultRow;
  }
  /**
   * Setter method for the the property enforceResultRow.
   * @param enforceResultRow The value to set for the property enforceResultRow.
   * @return The reference to this instance. 
   */
  public ReportGroup setEnforceResultRow(boolean enforceResultRow) {
    this.enforceResultRow = enforceResultRow;
    return this;
  }

  /**
   * Getter method for the the property columnIdx.
   * @return The current value of property columnIdx.
   */
  public int getColumnIdx() {
    return columnIdx;
  }
  /**
   * Setter method for the the property columnIdx.
   * @param columnIdx The value to set for the property columnIdx.
   * @return The reference to this instance. 
   */
  protected ReportGroup setColumnIdx(int columnIdx) {
    this.columnIdx = columnIdx;
    return this;
  }
  
  /**
   * Getter method for the the property groupResult.
   * @return The current value of property groupResult.
   */
  public double[] getGroupResults() {
    return results;
  }

  /**
   * Getter method for the the property groupResult.
   * @param columnId The column ID for which to retrieve the result.
   * @return The current value of property groupResult.
   */
  public double getGroupResult(int columnId) {
    return results[columnId];
  }

  /**
   * Getter method for the the property functionsApplied.
   * @return The current value of property functionsApplied.
   */
  protected boolean isFunctionsApplied() {
    for (int i = 0; i < functionsApplied.length; i++) {
      if (functionsApplied[i]) {
        return true;
      }
    }
    return false;
  }

  /**
   * Getter method for the the property functionsApplied.
   * @param columnId The column ID for which to set the result.
   * @return The current value of property functionsApplied.
   */
  protected boolean isFunctionsApplied(int columnId) {
    return functionsApplied[columnId];
  }
  
  /**
   * Applies the given report function executor on the given column.
   * @param columnId The column ID for which to set the result.
   * @param executor The report function executor to apply.
   * @param operand The value to set for the property groupResult.
   * @exception TypeMismatchException In case the given operand type was invalid.
   */
  protected void applyFunction(int columnId, FunctionExecutor executor, Object operand) 
    throws TypeMismatchException {
    results[columnId] = executor.execute(results[columnId], operand);
    functionsApplied[columnId] = true;
  }

  /**
   * Getter method for the the property titleRow.
   * @return The current value of property titleRow.
   */
  protected Row getTitleRow() {
    return titleRow;
  }
  /**
   * Setter method for the the property titleRow.
   * @param titleRow The value to set for the property titleRow.
   */
  protected void setTitleRow(Row titleRow) {
    this.titleRow = titleRow;
  }

  
  /**
   * Initializes the report group.
   * @param report The report to set.
   */
  protected void init(ReportTable report) {
    this.report = report;
    this.results = new double[report.getColumnCount()];
    this.functionsApplied = new boolean[report.getColumnCount()];
   }

  /**
   * Resets the report group.
   * Sets all result values to zero.
   */
  protected void reset() {
    for (int i = 0; i < results.length; i++) {
      results[i] = 0.0;
      functionsApplied[i] = false;
    }
  }
  
  /**
   * @see java.lang.Object#clone()
   */
  protected Object clone() {
    ReportGroup clone = null;
    try {
      clone = (ReportGroup) super.clone();
    } catch (CloneNotSupportedException ex) {
      throw new RuntimeException("Failed to clone ReportGroup instance: " + ex.getMessage(), ex);
    }
    return clone;
  }
  
  /**
   * @see java.lang.Object#toString()
   */
  public String toString() {
    return "{columnName=" + columnName + ", viewTitleRow=" + viewTitleRow + 
      ", resultsInTitleRow=" + resultsInTitleRow +
      ", hideAtSucceedingRows=" + hideAtSucceedingRows +
      ", startNewPage=" + startNewPage +
      ", keepTogether=" + keepTogether + 
      ", enforceResultRow=" + enforceResultRow + 
      ", columnIdx=" + columnIdx + "}";
  }

  /**
   * Writes the object's data to the given stream.
   * @param s The stream to write to
   * @exception IOException In case the data output failed.
   */
  private void writeObject(ObjectOutputStream s) throws IOException {
    s.writeObject(columnName);
    s.writeBoolean(viewTitleRow);
    s.writeBoolean(resultsInTitleRow);
    s.writeBoolean(hideAtSucceedingRows);
    s.writeBoolean(startNewPage);
    s.writeBoolean(keepTogether);
    s.writeBoolean(enforceResultRow);
    s.writeInt(columnIdx);
    s.writeObject(report);
    s.writeObject(results);
    s.writeObject(functionsApplied);
    s.writeObject(titleRow);
  }
  /**
   * Reads the object's data from the given stream.
   * @param s The stream to read from.
   * @exception IOException In case the data reading failed.
   * @exception ClassNotFoundException In case the class to deserialize could not be found
   */
  private void readObject(ObjectInputStream s) throws IOException, ClassNotFoundException  {
    columnName = (String) s.readObject();
    viewTitleRow = s.readBoolean();
    resultsInTitleRow = s.readBoolean();
    hideAtSucceedingRows = s.readBoolean();
    startNewPage = s.readBoolean();
    keepTogether = s.readBoolean();
    enforceResultRow = s.readBoolean();
    columnIdx = s.readInt();
    report = (ReportTable) s.readObject();
    results = (double[]) s.readObject();
    functionsApplied = (boolean[]) s.readObject();
    titleRow = (Row) s.readObject();
  }

}
