/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.table;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.NoSuchElementException;

/**
 * Name: ReportTableIterator<br>
 * Description: A iterator class for report tables.<br>
 * 
 * @author lauerc
 */
public class ReportIterator implements Iterator {

  /** The report functions. */
  private ReportFunction[] functions = null;
  /** The report groups. */
  private ReportGroup[] groups = null;
  /** The table to process. */
  private Table table;
  /** The table iterator, used to iterate over the given table. */
  private Iterator tableIterator;
  /** The group start row. */
  private Row groupStartRow = null;
  /** The last data row. */
  private Row lastDataRow = null;
  /** The prefetched data row. */
  private Row prefetchedRow = null;
  /** The row buffer. */
  private List rowBuffer = new LinkedList();
  /** The index of the current row. */
  private int currentRowIdx = -1;
  
  /**
   * Contructs a ReportIterator intance with specific attributes.
   * @param report The report we want to iterate.
   * @param functions The report functions.
   * @param groups The report groups.
   */
  protected ReportIterator(ReportTable report, ReportFunction[] functions, ReportGroup[] groups) {
    this.functions = functions;
    this.groups = groups;
    this.table = report.getDataTable();
    this.tableIterator = table.iterator();
  }

  /**
   * @see java.util.Iterator#hasNext()
   */
  public boolean hasNext() {
    return tableIterator.hasNext() || !rowBuffer.isEmpty() || (prefetchedRow != null);
  }

  /**
   * @see java.util.Iterator#next()
   */
  public Object next() {
    if (!isRowBufferEmpty()) {
      return fetchRow();
    }
    
    Row row = prefetchedRow;
    if (row == null) {
      row = (Row) tableIterator.next();
      ++currentRowIdx;
    } else {
      prefetchedRow = null;
    }
    return processNextRow(row); 
  }

  /**
   * Processes the next report row.
   * @param nextRow The next row according to the table iterator or row buffer.
   * @return The next row to print.
   */
  private Row processNextRow(Row nextRow) {
    
    try {
      ReportFunction[] functions = this.functions;
      int firstChangedGroupIdx = -1;
      int lastChangedGroupIdx = -1;
      boolean blockStartAllowed = true; //currentRowIdx > 0;
      boolean pageBreakAllowed = currentRowIdx > 0;
      do {
        if (groupStartRow == null) {
          firstChangedGroupIdx = 0;
          groupStartRow = nextRow;
          for (int i = 0; i < groups.length; i++) {
            ReportGroup group = groups[i];
            if (group == null) {
              continue;
            }
            if (group.isViewTitleRow()) {
              // Building title row(s) ...
              Row groupTitleRow = new GroupRow(table, TableConstants.ROW_TYPE_GROUP_TITLE, group.getColumnName());
              // defining block start if neccessary
              if (group.isKeepTogether() && blockStartAllowed) {
                groupTitleRow.setStartsBlock(true);
                blockStartAllowed = false;
              }
              // defining page break if neccessary
              if (group.isStartNewPage() && pageBreakAllowed) {
                groupTitleRow.setStartNewPage(true);
                pageBreakAllowed = false;
              }
              // defining group start
              if (group.isKeepTogether() && blockStartAllowed) {
                groupStartRow.setStartsBlock(true);
                blockStartAllowed = false;
              }
              int numColumns = table.getColumnCount();
              for (int j = 0; j < numColumns; j++) {
                // copying data from new row
                groupTitleRow.setObject(j, nextRow.getObject(j));
                if (j == group.getColumnIdx()) {
                  groupTitleRow.setVisible(j, true);
                }
              }
              // setting new group start row
              group.setTitleRow(groupTitleRow);
              groupStartRow = groupTitleRow;
              // pushing title row to row buffer
              pushRow(groupTitleRow);
              // checking whether or not to hide the grouped column at succeeding rows
              if (group.isHideAtSucceedingRows()) {
                // hiding field!
                //nextRow.setVisible(group.getColumnIdx(), false);
              }
            } else {
              // defining page break if neccessary
              if (group.isStartNewPage() && pageBreakAllowed) {
                groupStartRow.setStartNewPage(true);
                pageBreakAllowed = false;
              }
              // defining group start
              if (group.isKeepTogether() && blockStartAllowed) {
                groupStartRow.setStartsBlock(true);
                blockStartAllowed = false;
              }
            }
          }
        } else {
          // checking if group has changed
          ReportGroup[] groups = this.groups;
          firstChangedGroupIdx = -1;

          // comparing current row with group start row
          boolean firstResultsInTitleGroupClosed = false;
          int firstGroupIdx = -1;
          for (int i = 0; i < groups.length; i++) {
            ReportGroup group = groups[i];
            if (group != null) {
              if (firstGroupIdx < 0) {
                firstGroupIdx = i;
              }
              Object nro = nextRow.getObject(i);
              Object sro = groupStartRow.getObject(i);
              boolean equal = false;
              if ((nro == null) || (sro == null)) {
                equal = (nro == sro);
              } else {
                equal = nextRow.getObject(i).equals(groupStartRow.getObject(i));
              }
              if (!equal) {
                firstChangedGroupIdx = i;
                firstResultsInTitleGroupClosed = (i == firstGroupIdx) && group.isResultsInTitleRow();
                break;
              }
            }
          }
  
          lastChangedGroupIdx = firstChangedGroupIdx;
          if (firstChangedGroupIdx >= 0) {
            Row blockEndRow = lastDataRow;
            for (int j = groups.length - 1; j >= firstChangedGroupIdx; j--) {
              ReportGroup group = groups[j];
              if (group == null) {
                continue;
              }

              lastChangedGroupIdx = Math.max(lastChangedGroupIdx, j);
              if (!group.isFunctionsApplied()) {
                if (group.isEnforceResultRow()) {
                  // enforcing visibility of result row
                  blockEndRow = enforceResultRow(group);
                } else {
                  // pushing row on row buffer
                  if (group.isKeepTogether()) {
                    //pushRow(blockEndRow);
                  }
                }
                // defining block end if neccessary
                if (group.isKeepTogether()) {
                  blockEndRow.setEndsBlock(true);
                  blockStartAllowed = true;
                }
                // no function result to output
                continue;
              }
              // Group has results!
              double[] results = group.getGroupResults();
              Row groupResultRow = null;
              if (group.isResultsInTitleRow()) {
                groupResultRow = group.getTitleRow();
                group.setTitleRow(null);
                if (group.isEnforceResultRow()) {
                  // enforcing visibility of result row
                  blockEndRow = enforceResultRow(group);
                }
              } else {
                groupResultRow = new GroupRow(table, TableConstants.ROW_TYPE_GROUP, group.getColumnName());
                blockEndRow = groupResultRow;
              }
              for (int i = 0; i < results.length; i++) {
                if (group.isFunctionsApplied(i)) {
                  ReportFunction function = functions[i];
                  groupResultRow.setDouble(i, group.getGroupResult(i));
                  groupResultRow.setVisible(i, true);
                  groupResultRow.setFormat(i, function.getFormat());
                  groupResultRow.setAlign(i, function.getAlign());
                } else {
                  // copying data from last row
                  groupResultRow.setObject(i, lastDataRow.getObject(i));
                }
              }
              // defining block end if neccessary
              if ((group != null) && group.isKeepTogether()) {
                blockEndRow.setEndsBlock(true);
                blockStartAllowed = true;
              }
              // resetting group
              group.reset();
              if (group.isResultsInTitleRow()) {
                if (group.isKeepTogether()) {
                  blockEndRow = groupResultRow;
                }
              } else {
                // pushing result group to row buffer
                pushRow(groupResultRow);
              }
            }
  
            // setting new group start row
            groupStartRow = nextRow;
  
            if (firstResultsInTitleGroupClosed) {
              // remembering current row as prefetched row and letting 
              // the table printing component print this group
              prefetchedRow = nextRow;
              groupStartRow = null;
              // output row is the first from queue!
              return fetchRow();
            }
            
            for (int j = firstChangedGroupIdx; j <= lastChangedGroupIdx; j++) {
              ReportGroup group = groups[j];
              if (group == null) {
                continue;
              }
              if (!group.isViewTitleRow()) {
                // defining page break if neccessary
                if (group.isStartNewPage() && pageBreakAllowed) {
                  groupStartRow.setStartNewPage(true);
                  pageBreakAllowed = false;
                }
                // defining group start
                if (group.isKeepTogether() && blockStartAllowed) {
                  groupStartRow.setStartsBlock(true);
                  blockStartAllowed = false;
                }
                continue;
              }
              // Building title row ...
              Row groupTitleRow = new GroupRow(table, TableConstants.ROW_TYPE_GROUP_TITLE, group.getColumnName());
              // defining block start if neccessary
              if (blockStartAllowed && group.isKeepTogether()) {
                groupTitleRow.setStartsBlock(true);
                blockStartAllowed = false;
              }
              // defining page break if neccessary
              if (group.isStartNewPage() && pageBreakAllowed) {
                groupTitleRow.setStartNewPage(true);
                pageBreakAllowed = false;
              }
              // defining group start
              if (group.isKeepTogether() && blockStartAllowed) {
                groupStartRow.setStartsBlock(true);
                blockStartAllowed = false;
              }
              int numColumns = table.getColumnCount();
              for (int i = 0; i < numColumns; i++) {
                // copying data from new row
                groupTitleRow.setObject(i, nextRow.getObject(i));
                if (i == group.getColumnIdx()) {
                  groupTitleRow.setVisible(i, true);
                }
              }
              // setting new group start row
              group.setTitleRow(groupTitleRow);
              groupStartRow = groupTitleRow;
              // pushing title row to row buffer
              pushRow(groupTitleRow);
              // checking whether or not to hide the grouped column at succeeding rows
              if (group.isHideAtSucceedingRows()) {
                // hiding field!
                //nextRow.setVisible(group.getColumnIdx(), false);
              }
            }
          }
        }
  
        // applying functions for the row data
        for (int i = 0; i < functions.length; i++) {
          ReportFunction function = functions[i];
          if (function != null) {
            function.apply(nextRow.getObject(i));
          }
        }

        // checking whether or not to hide the grouped column at succeeding rows
        for (int i = 0; i < groups.length; i++) {
          ReportGroup group = groups[i];
          if (group != null) {
            int groupColIdx = group.getColumnIdx();
            if (group.isHideAtSucceedingRows() && ((firstChangedGroupIdx == -1) || (groupColIdx < firstChangedGroupIdx))) {
              // hiding field!
              nextRow.setVisible(group.getColumnIdx(), false);
            }
          }
        }
        
        // pushing new row to row buffer
        pushRow(nextRow);
        
        this.lastDataRow = nextRow;
        
        if (!tableIterator.hasNext()) {
          Row blockEndRow = lastDataRow;
          for (int i = groups.length - 1; i >= 0; i--) {
            ReportGroup group = groups[i];
            if (group != null) {
              if (!group.isFunctionsApplied()) {
                if (group.isEnforceResultRow()) {
                  // enforcing visibility of result row
                  blockEndRow = enforceResultRow(group);
                }
                // defining block end if neccessary
                if (group.isKeepTogether()) {
                  blockEndRow.setEndsBlock(true);
                  blockStartAllowed = true;
                }
                // no function result to output
                continue;
              }
              
              // Group has results!
              double[] results = group.getGroupResults();
              Row groupResultRow = null;
              if (group.isResultsInTitleRow()) {
                groupResultRow = group.getTitleRow();
                group.setTitleRow(null);
                if (group.isEnforceResultRow()) {
                  // enforcing visibility of result row
                  blockEndRow = enforceResultRow(group);
                }
              } else {
                groupResultRow = new GroupRow(table, TableConstants.ROW_TYPE_GROUP, group.getColumnName());
                blockEndRow = groupResultRow;
              }
              for (int j = 0; j < results.length; j++) {
                if (group.isFunctionsApplied(j)) {
                  ReportFunction function = functions[j];
                  groupResultRow.setDouble(j, group.getGroupResult(j));
                  groupResultRow.setVisible(j, true);
                  groupResultRow.setFormat(j, function.getFormat());
                  groupResultRow.setAlign(j, function.getAlign());
                } else {
                  // copying data from last row
                  groupResultRow.setObject(j, lastDataRow.getObject(j));
                }
              }
              // resetting group
              group.reset();
              if (!group.isResultsInTitleRow()) {
                // pushing result group to row buffer
                pushRow(groupResultRow);
              }
            }
            // defining block end if neccessary
            if ((group != null) && group.isKeepTogether()) {
              blockEndRow.setEndsBlock(true);
              blockStartAllowed = true;
            }
          }
  
          boolean resultsCalculated = false;
          for (int i = 0; i < functions.length; i++) {
            ReportFunction f = functions[i];
            if (f == null) {
              continue;
            }
            if (f.isCalculateTotalResult()) {
              resultsCalculated = true;
              break;
            }
          }
          
          if (resultsCalculated) {
            // Building footer row ...
            DataRow footerRow = new DataRow(table, -1);
            footerRow.setType(TableConstants.ROW_TYPE_FOOTER);
            for (int i = 0; i < functions.length; i++) {
              ReportFunction function = functions[i];
              if ((function != null) && function.isCalculateTotalResult()) {
                // copying data from last row
                footerRow.setDouble(i, function.getTotalResult());
                footerRow.setFormat(i, function.getFormat());
                footerRow.setAlign(i, function.getAlign());
              } else {
                // copying data from last row
                footerRow.setObject(i, lastDataRow.getObject(i));
                footerRow.setVisible(i, false);
              }
            }
            // pushing footer row to row buffer
            pushRow(footerRow);
          }
          return fetchRow();
        }
        if (hasGroupsWithResultsInTitle() || !blockStartAllowed) {
          nextRow = (Row) tableIterator.next();
          ++currentRowIdx;
        } else {
          return fetchRow();
        }
      } while (true);
    } catch (Exception ex) {
      throw new TableIterationException("Report iteration failed: " + ex.getMessage(), ex);
    }
  }

  /**
   * Enforces a result rest row.
   * @param group The group for which to enforce the result set row.
   * @return The created group row.
   * @throws TableException In case the row processing failed.
   */
  private Row enforceResultRow(ReportGroup group) throws TableException {
    // enforcing visibility of result row
    Row enforcedResultRow = new GroupRow(table, TableConstants.ROW_TYPE_GROUP, group.getColumnName());
    int numColumns = table.getColumnCount();
    for (int k = 0; k < numColumns; k++) {
      // copying data from new row
      enforcedResultRow.setObject(k, lastDataRow.getObject(k));
      if (k == group.getColumnIdx()) {
        enforcedResultRow.setVisible(k, false);
      }
    }
    // pushing result group to row buffer
    pushRow(enforcedResultRow);
    return enforcedResultRow;
  }
  
  /**
   * Checks whether a unsatisfied group with results in the title row exists. 
   * @return true, if a group was detected, else false.
   */
  private boolean hasGroupsWithResultsInTitle() {
    for (int i = 0; i < groups.length; i++) {
      ReportGroup group = groups[i];
      if (group != null) {
        if (group.isResultsInTitleRow() && (group.getTitleRow() != null)) {
          return true;
        }
      }
    }
    return false;
  }
  
  /**
   * @see java.util.Iterator#remove()
   */
  public void remove() {
    throw new IllegalStateException("Removing rows is not permitted for reports!");
  }
  
  /**
   * Checks if the row stack is empty.
   * @return true if the row stack is empty, else false.
   */
  private boolean isRowBufferEmpty() {
    return (rowBuffer.size() == 0);
  }

  /**
   * Adds the given element to the row stack.
   * @param row The row to put on top of the row stack.
   */
  private void pushRow(Row row) {
    rowBuffer.add(row);
  }

  /**
   * Removes the last element from the row stack.
   * @return The last element from the row stack.
   * @exception NoSuchElementException In case the row stack is empty.
   */
  private Row fetchRow() throws NoSuchElementException {
    int lastIdx = rowBuffer.size() - 1;
    if (lastIdx < 0) {
      throw new NoSuchElementException("Row buffer is empty!");
    }
    return (Row) rowBuffer.remove(0);
  }
}
