/*
 * Copyright (C) 2003, 2004, 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.table;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import java.util.Arrays;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.ujac.util.TypeConverter;
import org.ujac.util.TypeConverterException;

import org.ujac.util.text.FormatHelper;

/**
 * Name: ReportTable<br>
 * Description: A table implementation serving simple reports
 *   with groups, functions, etc.
 * 
 * @author lauerc
 */
public class ReportTable
  extends BaseTable
  implements Table {

  /** The serial version UID. */
  static final long serialVersionUID = 2769570291019778079L;

  /** The data table. */
  private Table dataTable = null;
  /** The report functions .*/
  private Column[] columns = null;
  /** The report functions .*/
  private ReportFunction[] functions = null;
  /** The report groups .*/
  private ReportGroup[] groups = null;

  /**
   * Constructs a ReportTable instance with specific attributes.
   * @param dataTable The data table.
   */
  public ReportTable(Table dataTable) {
    this.dataTable = dataTable;
    int numColumns = dataTable.getColumnCount();
    this.columns = new Column[numColumns];
    try {
      for (int i = 0; i < numColumns; i++) {
        columns[i] = dataTable.getColumn(i);
      }
    } catch (ColumnNotDefinedException ex) {
      // this should never happen!
      ex.printStackTrace();
      throw new RuntimeException(ex.getMessage());
    }
    this.functions = new ReportFunction[numColumns];
    this.groups = new ReportGroup[numColumns];
  }
  
  /**
   * Getter method for the the property formatHelper.
   * @return The current value of property formatHelper.
   */
  public FormatHelper getFormatHelper() {
    return dataTable.getFormatHelper();
  }
  /**
   * Setter method for the the property formatHelper.
   * @param formatHelper The value to set for the property formatHelper.
   */
  public void setFormatHelper(FormatHelper formatHelper) {
    dataTable.setFormatHelper(formatHelper);
  }

  /**
   * Getter method for the the property typeConverter.
   * @return The current value of property typeConverter.
   */
  public TypeConverter getTypeConverter() {
    return dataTable.getTypeConverter();
  }
  /**
   * Setter method for the the property typeConverter.
   * @param typeConverter The value to set for the property typeConverter.
   */
  public void setTypeConverter(TypeConverter typeConverter) {
    dataTable.setTypeConverter(typeConverter);
  }

  /**
   * @see org.ujac.util.table.Table#getColumn(int)
   */
  public Column getColumn(int index) throws ColumnNotDefinedException {
    return dataTable.getColumn(index);
  }

  /**
   * @see org.ujac.util.table.Table#getColumn(java.lang.String)
   */
  public Column getColumn(String name) throws ColumnNotDefinedException {
    return dataTable.getColumn(name);
  }

  /**
   * @see org.ujac.util.table.Table#getColumnIndex(java.lang.String)
   */
  public int getColumnIndex(String name) throws ColumnNotDefinedException {
    return dataTable.getColumnIndex(name);
  }

  /**
   * Sets the new order for the report columns according to the list of given
   * column names.
   * @param columnNames The list of column names after which to order the columns.
   * @exception ColumnNotDefinedException In case a column from the given order list
   *  does not exist. 
   * @exception ColumnAlreadyDefinedException In case a column was defined more than 
   *  once at the given order list.
   */
  public void orderColumns(String[] columnNames) 
    throws ColumnNotDefinedException, ColumnAlreadyDefinedException {
    
    Map columnMap = new HashMap();
    for (int i = 0; i < columns.length; i++) {
      Column col = columns[i];
      String name = col.getName();
      columnMap.put(name, col);
    }
    
    Set newColumnNames = new HashSet();
    Column[] newColumns = new Column[columns.length];
    for (int i = 0; i < columnNames.length; i++) {
      String name = columnNames[i];
      Column col = (Column) columnMap.get(name);
      if (col == null) {
        if (newColumnNames.contains(name)) {
          throw new ColumnAlreadyDefinedException("The column '" + name + "' was specified in the given column order list more than once.");
        }
        throw new ColumnNotDefinedException("The column '" + name + "', specified in the given column order list does not exist.");
      }
      newColumns[i] = col;
      newColumnNames.add(name);
      columnMap.remove(name);
    }
    if (columnMap.size() > 0) {
      // adding the remaining column according to their original order
      Column[] rest = new Column[columnMap.size()];
      int idx = 0;
      Iterator iterRest = columnMap.entrySet().iterator();
      while (iterRest.hasNext()) {
        rest[idx] = ((Column) ((Map.Entry) iterRest.next()).getValue());
        ++idx;
      }
      Arrays.sort(rest, new Comparator() {
        public int compare(Object a, Object b) {
          int idxA = ((Column) a).getIndex();
          int idxB = ((Column) b).getIndex();
          if (idxA > idxB) {
            return 1;
          }
          if (idxA < idxB) {
            return -1;
          }
          return 0;
        }
      });
      // finally adding the rest
      int nextIdx = columnNames.length;
      for (int i = 0; i < rest.length; i++) {
        Column col = rest[i];
        newColumns[nextIdx] = col;
        ++nextIdx;
      }      
    }

    // OK, ordering successfuly completed
    this.columns = newColumns;
  }

  /**
   * Gets all visible columns.
   * @return An array holding all visible columns.
   */
  public Column[] getVisibleColumns() {
    int numVisibleColumns = 0;
    for (int i = 0; i < columns.length; i++) {
      Column col = columns[i];
      LayoutHints lh = col.getLayoutHints();
      if ((lh != null) && lh.isVisible()) {
        ++numVisibleColumns;
      }
    }
    int nextIdx = 0;
    Column[] visibleColumns = new Column[numVisibleColumns];
    for (int i = 0; i < columns.length; i++) {
      Column col = columns[i];
      LayoutHints lh = col.getLayoutHints();
      if ((lh != null) && lh.isVisible()) {
        visibleColumns[nextIdx] = col;
        ++nextIdx;
      }
    }
    return visibleColumns;
  }

  /**
   * Gets all visible columns.
   * @param levelFilter The maximum level of importance to support.
   * @return An array holding all visible columns.
   */
  public Column[] getVisibleColumns(int levelFilter) {
    int numVisibleColumns = 0;
    for (int i = 0; i < columns.length; i++) {
      Column col = columns[i];
      LayoutHints lh = col.getLayoutHints();
      if ((lh != null) && lh.isVisible()) {
        if ((levelFilter >= 0) && (lh.getLevel() > levelFilter)) {
          // filtering unsupported levels
          continue;
        }
        ++numVisibleColumns;
      }
    }
    int nextIdx = 0;
    Column[] visibleColumns = new Column[numVisibleColumns];
    for (int i = 0; i < columns.length; i++) {
      Column col = columns[i];
      LayoutHints lh = col.getLayoutHints();
      if ((lh != null) && lh.isVisible()) {
        visibleColumns[nextIdx] = col;
        ++nextIdx;
      }
    }
    return visibleColumns;
  }

  /**
   * @see org.ujac.util.table.Table#getHeaderRow()
   */
  public Row getHeaderRow() {
    return dataTable.getHeaderRow();
  }
  
  /**
   * Returns an iterator over the rows of this report in proper sequence.
   * @return an iterator over the rows of this report in proper sequence.
   */
  public Iterator iterator() {
    // cloning report groups
    ReportGroup[] groups = new ReportGroup[this.groups.length];
    for (int i = 0; i < groups.length; i++) {
      ReportGroup g = this.groups[i];
      if (g != null) {
        groups[i] = (ReportGroup) g.clone();
        groups[i].init(this);
      }
    }
    // cloning report functions
    ReportFunction[] functions = new ReportFunction[this.functions.length];
    for (int i = 0; i < functions.length; i++) {
      ReportFunction f = this.functions[i];
      if (f != null) {
        functions[i] = (ReportFunction) f.clone();
        functions[i].init(this, groups);
       }
    }
    // creating iterator
    return new ReportIterator(this, functions, groups); 
  }

  /**
   * Adds a group to the report.
   * @param group The report group to add.
   * @exception ColumnNotDefinedException In case the data table doesn't contain a column 
   *   named like the group's column name.
   */
  public void addGroup(ReportGroup group) throws ColumnNotDefinedException {
    int groupIdx = dataTable.getColumnIndex(group.getColumnName());
    group.setColumnIdx(groupIdx);
    groups[groupIdx] = group;
  }

  /**
   * Adds a column function to the report.
   * @param function The report function to add.
   * @exception ColumnNotDefinedException In case the data table doesn't contain a column 
   *   named like the function's column name.
   */
  public void addFunction(ReportFunction function) throws ColumnNotDefinedException {
    int functionIdx = dataTable.getColumnIndex(function.getColumnName());
    function.setColumnIdx(functionIdx);
    functions[functionIdx] = function;
  }

  /**
   * Gets the current number of columns.
   * @return The number of columns.
   */
  public int getColumnCount() {
    return dataTable.getColumnCount();
  }

  /**
   * Getter method for the the property functions.
   * @return The current value of property functions.
   */
  public ReportFunction[] getFunctions() {
    return functions;
  }

  /**
   * Getter method for the the property groups.
   * @return The current value of property groups.
   */
  public ReportGroup[] getGroups() {
    return groups;
  }

  /**
   * Getter method for the the property dataTable.
   * @return The current value of property dataTable.
   */
  protected Table getDataTable() {
    return dataTable;
  }


  /**
   * @see org.ujac.util.table.Table#addColumn(java.lang.String, int)
   */
  public Column addColumn(String name, int type) throws OperationNotSupportedException {
    throwNotSupportedException();
    return null;
  }

  /**
   * @see org.ujac.util.table.Table#addColumn(java.lang.String, int, LayoutHints)
   */
  public Column addColumn(String name, int type, LayoutHints layoutHints) throws OperationNotSupportedException {
    throwNotSupportedException();
    return null;
  }

  /**
   * @see org.ujac.util.table.Table#addString(java.lang.String, java.lang.String)
   */
  public Table addString(String name, String value) throws TableException {
    throwNotSupportedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#addInt(java.lang.String, int)
   */
  public Table addInt(String name, int value) throws TableException {
    throwNotSupportedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#addLong(java.lang.String, long)
   */
  public Table addLong(String name, long value) throws TableException {
    throwNotSupportedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#addFloat(java.lang.String, float)
   */
  public Table addFloat(String name, float value) throws TableException {
    throwNotSupportedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#addDouble(java.lang.String, double)
   */
  public Table addDouble(String name, double value) throws TableException {
    throwNotSupportedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#addBoolean(java.lang.String, boolean)
   */
  public Table addBoolean(String name, boolean value) throws TableException {
    throwNotSupportedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#addDate(java.lang.String, java.util.Date)
   */
  public Table addDate(String name, Date value) throws TableException {
    throwNotSupportedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#addTime(java.lang.String, java.util.Date)
   */
  public Table addTime(String name, Date value) throws TableException {
    throwNotSupportedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#addTimestamp(java.lang.String, java.util.Date)
   */
  public Table addTimestamp(String name, Date value) throws TableException {
    throwNotSupportedException();
    return null;
  }

  /**
   * Checks if the table is empty.
   * @return true if the table has no rows, else false.
   */
  public boolean isEmpty() {
    return dataTable.isEmpty();
  }
  
  /**
   * @see org.ujac.util.table.Table#getRowCount()
   */
  public int getRowCount() throws OperationNotSupportedException {
    throwNotSupportedException();
    return 0;
  }
  /**
   * @see org.ujac.util.table.Table#addRow()
   */
  public Row addRow() throws OperationNotSupportedException {
    throwNotSupportedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#addRow(org.ujac.util.table.Row)
   */
  public Row addRow(Row srcRow) throws OperationNotSupportedException {
    throwNotSupportedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#getRow(int)
   */
  public Row getRow(int rowIdx) throws RowNotDefinedException, OperationNotSupportedException {
    throwNotSupportedException();
    return null;
  }
  /**
   * @see org.ujac.util.table.Table#removeRow(int)
   */
  public void removeRow(int rowIdx) throws RowNotDefinedException, OperationNotSupportedException {
    throwNotSupportedException();
  }


  /**
   * @see org.ujac.util.table.Table#getString(int, int)
   */
  public String getString(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    return dataTable.getString(rowIdx, columnIdx);
  }
  /**
   * @see org.ujac.util.table.Table#getString(int, java.lang.String)
   */
  public String getString(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    return dataTable.getString(rowIdx, columnName);
  }
  /**
   * @see org.ujac.util.table.Table#setString(int, int, java.lang.String)
   */
  public void setString(int rowIdx, int columnIdx, String value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    dataTable.setString(rowIdx, columnIdx, value);
  }
  /**
   * @see org.ujac.util.table.Table#setString(int, java.lang.String, java.lang.String)
   */
  public void setString(int rowIdx, String columnName, String value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    dataTable.setString(rowIdx, columnName, value);
  }

  /**
   * @see org.ujac.util.table.Table#getInt(int, int)
   */
  public int getInt(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    return dataTable.getInt(rowIdx, columnIdx);
  }
  /**
   * @see org.ujac.util.table.Table#getInt(int, java.lang.String)
   */
  public int getInt(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    return dataTable.getInt(rowIdx, columnName);
  }
  /**
   * @see org.ujac.util.table.Table#setInt(int, int, int)
   */
  public void setInt(int rowIdx, int columnIdx, int value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    dataTable.setInt(rowIdx, columnIdx, value);
  }
  /**
   * @see org.ujac.util.table.Table#setInt(int, java.lang.String, int)
   */
  public void setInt(int rowIdx, String columnName, int value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    dataTable.setInt(rowIdx, columnName, value);
  }

  /**
   * @see org.ujac.util.table.Table#getLong(int, int)
   */
  public long getLong(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    return dataTable.getLong(rowIdx, columnIdx);
  }
  /**
   * @see org.ujac.util.table.Table#getLong(int, java.lang.String)
   */
  public long getLong(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    return dataTable.getLong(rowIdx, columnName);
  }
  /**
   * @see org.ujac.util.table.Table#setLong(int, int, long)
   */
  public void setLong(int rowIdx, int columnIdx, long value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    dataTable.setLong(rowIdx, columnIdx, value);
  }
  /**
   * @see org.ujac.util.table.Table#setLong(int, java.lang.String, long)
   */
  public void setLong(int rowIdx, String columnName, long value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    dataTable.setLong(rowIdx, columnName, value);
  }

  /**
   * @see org.ujac.util.table.Table#getFloat(int, int)
   */
  public float getFloat(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    return dataTable.getFloat(rowIdx, columnIdx);
  }
  /**
   * @see org.ujac.util.table.Table#getFloat(int, java.lang.String)
   */
  public float getFloat(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    return dataTable.getFloat(rowIdx, columnName);
  }
  /**
   * @see org.ujac.util.table.Table#setFloat(int, int, float)
   */
  public void setFloat(int rowIdx, int columnIdx, float value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    dataTable.setFloat(rowIdx, columnIdx, value);
  }
  /**
   * @see org.ujac.util.table.Table#setFloat(int, java.lang.String, float)
   */
  public void setFloat(int rowIdx, String columnName, float value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    dataTable.setFloat(rowIdx, columnName, value);
  }

  /**
   * @see org.ujac.util.table.Table#getDouble(int, int)
   */
  public double getDouble(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    return dataTable.getDouble(rowIdx, columnIdx);
  }
  /**
   * @see org.ujac.util.table.Table#getDouble(int, java.lang.String)
   */
  public double getDouble(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    return dataTable.getDouble(rowIdx, columnName);
  }
  /**
   * @see org.ujac.util.table.Table#setDouble(int, int, double)
   */
  public void setDouble(int rowIdx, int columnIdx, double value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    dataTable.setDouble(rowIdx, columnIdx, value);
  }
  /**
   * @see org.ujac.util.table.Table#setDouble(int, java.lang.String, double)
   */
  public void setDouble(int rowIdx, String columnName, double value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    dataTable.setDouble(rowIdx, columnName, value);
  }

  /**
   * @see org.ujac.util.table.Table#getBoolean(int, int)
   */
  public boolean getBoolean(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    return dataTable.getBoolean(rowIdx, columnIdx);
  }
  /**
   * @see org.ujac.util.table.Table#getBoolean(int, java.lang.String)
   */
  public boolean getBoolean(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    return dataTable.getBoolean(rowIdx, columnName);
  }
  /**
   * @see org.ujac.util.table.Table#setBoolean(int, int, boolean)
   */
  public void setBoolean(int rowIdx, int columnIdx, boolean value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    dataTable.setBoolean(rowIdx, columnIdx, value);
  }
  /**
   * @see org.ujac.util.table.Table#setBoolean(int, java.lang.String, boolean)
   */
  public void setBoolean(int rowIdx, String columnName, boolean value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    dataTable.setBoolean(rowIdx, columnName, value);
  }

  /**
   * @see org.ujac.util.table.Table#getDate(int, int)
   */
  public Date getDate(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    return dataTable.getDate(rowIdx, columnIdx);
  }
  /**
   * @see org.ujac.util.table.Table#getDate(int, java.lang.String)
   */
  public Date getDate(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    return dataTable.getDate(rowIdx, columnName);
  }
  /**
   * @see org.ujac.util.table.Table#setDate(int, int, java.util.Date)
   */
  public void setDate(int rowIdx, int columnIdx, Date value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    dataTable.setDate(rowIdx, columnIdx, value);
  }
  /**
   * @see org.ujac.util.table.Table#setDate(int, java.lang.String, java.util.Date)
   */
  public void setDate(int rowIdx, String columnName, Date value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    dataTable.setDate(rowIdx, columnName, value);
  }

  /**
   * @see org.ujac.util.table.Table#getTime(int, int)
   */
  public Date getTime(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    return dataTable.getTime(rowIdx, columnIdx);
  }
  /**
   * @see org.ujac.util.table.Table#getTime(int, java.lang.String)
   */
  public Date getTime(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    return dataTable.getTime(rowIdx, columnName);
  }
  /**
   * @see org.ujac.util.table.Table#setTime(int, int, java.util.Date)
   */
  public void setTime(int rowIdx, int columnIdx, Date value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    dataTable.setTime(rowIdx, columnIdx, value);
  }
  /**
   * @see org.ujac.util.table.Table#setTime(int, java.lang.String, java.util.Date)
   */
  public void setTime(int rowIdx, String columnName, Date value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    dataTable.setTime(rowIdx, columnName, value);
  }

  /**
   * @see org.ujac.util.table.Table#getTimestamp(int, int)
   */
  public Date getTimestamp(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    return dataTable.getTimestamp(rowIdx, columnIdx);
  }
  /**
   * @see org.ujac.util.table.Table#getTimestamp(int, java.lang.String)
   */
  public Date getTimestamp(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    return dataTable.getTimestamp(rowIdx, columnName);
  }
  /**
   * @see org.ujac.util.table.Table#setTimestamp(int, int, java.util.Date)
   */
  public void setTimestamp(int rowIdx, int columnIdx, Date value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    dataTable.setTimestamp(rowIdx, columnIdx, value);
  }
  /**
   * @see org.ujac.util.table.Table#setTimestamp(int, java.lang.String, java.util.Date)
   */
  public void setTimestamp(int rowIdx, String columnName, Date value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    dataTable.setTimestamp(rowIdx, columnName, value);
  }

  /**
   * @see org.ujac.util.table.Table#getObject(int, int)
   */
  public Object getObject(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException {
    return dataTable.getObject(rowIdx, columnIdx);
  }
  /**
   * @see org.ujac.util.table.Table#getObject(int, java.lang.String)
   */
  public Object getObject(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException {
    return dataTable.getObject(rowIdx, columnName);
  }
  /**
   * @see org.ujac.util.table.Table#setObject(int, int, java.lang.Object)
   */
  public void setObject(int rowIdx, int columnIdx, Object value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    dataTable.setObject(rowIdx, columnIdx, value);
  }
  /**
   * @see org.ujac.util.table.Table#setObject(int, java.lang.String, java.lang.Object)
   */
  public void setObject(int rowIdx, String columnName, Object value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    dataTable.setObject(rowIdx, columnName, value);
  }


  /**
   * @see org.ujac.util.table.Table#sort(org.ujac.util.table.CompareItem[])
   */
  public void sort(CompareItem[] compareColumns) throws OperationNotSupportedException {
    throwNotSupportedException();
  }


  /**
   * @see org.ujac.util.table.Table#sort(int, int)
   */
  public void sort(int columnIdx, int direction) throws OperationNotSupportedException {
    throwNotSupportedException();
  }

  /**
   * Changes the type of the given column.
   * @param columnIdx The index of the column which has been changed.
   * @param typeId The new column type ID.
   * @exception ColumnNotDefinedException In case the desired column does not exist.
   * @exception TypeConverterException In case the object conversion failed.
   */
  public void setColumnType(int columnIdx, int typeId)
    throws ColumnNotDefinedException, TypeConverterException {
    
    dataTable.setColumnType(columnIdx, typeId);
  }

  /**
   * Changes the type of the given column.
   * @param columnName The name of the column which has been changed.
   * @param typeId The new column type ID.
   * @exception ColumnNotDefinedException In case the desired column does not exist.
   * @exception TypeConverterException In case the object conversion failed.
   */
  public void setColumnType(String columnName, int typeId)
    throws ColumnNotDefinedException, TypeConverterException {
    
    dataTable.setColumnType(columnName, typeId);
  }

  
  /**
   * Throws an exception, telling that the operation is not supported.
   * @throws OperationNotSupportedException Thrown to tell that the operation is not supported.
   */
  private void throwNotSupportedException() throws OperationNotSupportedException {
    throw new OperationNotSupportedException("This operation is not supported for report tables.");
  }
  
  /**
   * Writes the object's data to the given stream.
   * @param s The stream to write to
   * @exception IOException In case the data output failed.
   */
  private void writeObject(ObjectOutputStream s) throws IOException {
    s.writeObject(dataTable);
    s.writeObject(columns);
    s.writeObject(functions);
    s.writeObject(groups);
  }
  /**
   * Reads the object's data from the given stream.
   * @param s The stream to read from.
   * @exception IOException In case the data reading failed.
   * @exception ClassNotFoundException In case the class to deserialize could not be found
   */
  private void readObject(ObjectInputStream s) throws IOException, ClassNotFoundException  {
    dataTable = (Table) s.readObject();
    columns = (Column[]) s.readObject();
    functions = (ReportFunction[]) s.readObject();
    groups = (ReportGroup[]) s.readObject();
  }

}
