/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.table;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.ResultSetMetaData;

import org.ujac.util.UjacTypes;
import org.ujac.util.db.DataTypeMapper;
import org.ujac.util.db.DefaultDataTypeMapper;

/**
 * Name: ResultSetReader<br>
 * Description: Reads tables from JDBC result sets.<br>
 * 
 * @author lauerc
 */
public class ResultSetReader {

  /**
   * Reads a table from a given result set.
   * @param rs The result set.
   * @return The according table.
   * @throws SQLException If a database problem occured.
   * @throws TableException If a table problem occured.
   */
  public static final DataTable readTable(ResultSet rs) throws SQLException, TableException {
    return readTable(rs, new DefaultDataTypeMapper());
  }
  
  /**
   * Reads a table from a given result set.
   * @param rs The result set.
   * @param dataTypeMapper The data type mapper to use.
   * @return The according table.
   * @throws SQLException If a database problem occured.
   * @throws TableException If a table problem occured.
   */
  public static final DataTable readTable(ResultSet rs, DataTypeMapper dataTypeMapper) 
    throws SQLException, TableException {
    
    DataTable table = new DataTable();
    ResultSetMetaData meta = rs.getMetaData();
    int numColumns = meta.getColumnCount();
    
    // defining table columns
    int[] columnTypes = new int[numColumns];
    int columnType = 0;
    String columnName = null;
    for (int i = 1; i <= numColumns; i++) {
      columnName = meta.getColumnName(i);
      int javaType = dataTypeMapper.getJavaType(meta.getColumnType(i), meta.getScale(i), meta.getPrecision(i));
      int alignment = LayoutHints.LEFT;
      switch (javaType) {
        case DataTypeMapper.JAVA_TYPE_OBJECT: 
          columnType = UjacTypes.TYPE_OBJECT;
          break;
        case DataTypeMapper.JAVA_TYPE_INT: 
          columnType = UjacTypes.TYPE_INT;
          alignment = LayoutHints.RIGHT;
          break;
        case DataTypeMapper.JAVA_TYPE_LONG: 
          columnType = UjacTypes.TYPE_LONG;
          alignment = LayoutHints.RIGHT;
          break;
        case DataTypeMapper.JAVA_TYPE_FLOAT: 
          columnType = UjacTypes.TYPE_FLOAT;
          alignment = LayoutHints.RIGHT;
          break;
        case DataTypeMapper.JAVA_TYPE_DOUBLE: 
          columnType = UjacTypes.TYPE_DOUBLE;
          alignment = LayoutHints.RIGHT;
          break;
        case DataTypeMapper.JAVA_TYPE_BOOLEAN: 
          columnType = UjacTypes.TYPE_BOOLEAN;
          break;
        case DataTypeMapper.JAVA_TYPE_STRING: 
          columnType = UjacTypes.TYPE_STRING;
          break;
        case DataTypeMapper.JAVA_TYPE_DATE: 
          columnType = UjacTypes.TYPE_DATE;
          break;
        case DataTypeMapper.JAVA_TYPE_TIME: 
          columnType = UjacTypes.TYPE_TIME;
          break;
        case DataTypeMapper.JAVA_TYPE_TIMESTAMP: 
          columnType = UjacTypes.TYPE_TIMESTAMP;
          break;
           
        default:
          throw new TableException("Unsupported SQL type '" + meta.getColumnTypeName(i) + "' at column '" + columnName + "'.");
      }
      columnTypes[i-1] = columnType;
      Column col = table.addColumn(columnName, columnType);
      col.setLayoutHints(new LayoutHints(10, meta.getColumnName(i), alignment, true));
    }
    
    // copying rows
    while (rs.next()) {
      Row row = table.addRow();
      for (int i=0; i<numColumns; i++) {
        switch (columnTypes[i]) {
          case UjacTypes.TYPE_BOOLEAN:
            row.setBoolean(i, rs.getBoolean(i+1));
            break;

          case UjacTypes.TYPE_INT:
            row.setInt(i, rs.getInt(i+1));
            break;

          case UjacTypes.TYPE_LONG:
            row.setLong(i, rs.getLong(i+1));
            break;

          case UjacTypes.TYPE_FLOAT:
            row.setFloat(i, rs.getFloat(i+1));
            break;

          case UjacTypes.TYPE_DOUBLE:
            row.setDouble(i, rs.getDouble(i+1));
            break;

          case UjacTypes.TYPE_STRING:
            row.setString(i, rs.getString(i+1));
            break;

          case UjacTypes.TYPE_DATE:
            row.setDate(i, rs.getDate(i+1));
            break;

          case UjacTypes.TYPE_TIME:
            row.setTime(i, rs.getTime(i+1));
            break;

          case UjacTypes.TYPE_TIMESTAMP:
            row.setTimestamp(i, rs.getTimestamp(i+1));
            break;

          default:
            row.setObject(i, rs.getObject(i+1));
            break;
        }
      }
    }
     
    return table;
  }

}
