/*
 * Copyright (C) 2003, 2004, 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.table;

import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

import java.util.Date;
import java.util.Iterator;

import org.ujac.util.DefaultTypeConverter;
import org.ujac.util.TypeConverter;
import org.ujac.util.TypeConverterException;
import org.ujac.util.UjacTypes;

import org.ujac.util.db.DataTypeMapper;
import org.ujac.util.db.DefaultDataTypeMapper;
import org.ujac.util.text.FormatHelper;

/**
 * Name: ResultSetTable<br>
 * Description: A table implementation, serving data retrieved from a result set.<br>
 * 
 * @author lauerc
 */
public class ResultSetTable extends BaseTable {

  /** The result set. */
  private ResultSet resultSet = null;
  /** The data type mapper. */
  private DataTypeMapper dataTypeMapper = null;
  /** The index of the current row. */
  private int currentRowIdx = 0;
  /** The index of the current row. */
  private int currentResultSetRowIdx = 0;
  /** The index of the current row. */
  private Row currentRow = null;
  
  /**
   * Constructs a DataTable instance with specific arguments.
   * @param resultSet The result set to read.
   * @throws SQLException If a database problem occured.
   * @throws TableException If a table problem occured.
   */
  public ResultSetTable(ResultSet resultSet)
    throws SQLException, TableException {
    
    this(resultSet, FormatHelper.createInstance());
  }

  /**
   * Constructs a DataTable instance with specific arguments.
   * @param resultSet The result set to read.
   * @param formatHelper The format helper to use.
   * @throws SQLException If a database problem occured.
   * @throws TableException If a table problem occured.
   */
  public ResultSetTable(ResultSet resultSet, FormatHelper formatHelper)
    throws SQLException, TableException {

    this(resultSet, formatHelper, new DefaultDataTypeMapper());
  }

  /**
   * Constructs a DataTable instance with no specific arguments.
   * @param resultSet The result set to read.
   * @param formatHelper The format helper to use.
   * @param dataTypeMapper The data type mapper to use.
   * @exception SQLException If a database problem occured.
   * @exception TableException If a table problem occured.
   */
  public ResultSetTable(ResultSet resultSet, FormatHelper formatHelper, DataTypeMapper dataTypeMapper)
    throws SQLException, TableException {
    
    super(formatHelper);
    this.resultSet = resultSet;
    this.dataTypeMapper = dataTypeMapper;

    // initializes the table.
    init();
  }
  
  /**
   * Initializes the 
   * @exception SQLException If a database problem occured.
   * @exception TableException If a table problem occured.
   */
  private void init()
    throws SQLException, TableException {

    ResultSetMetaData rms = resultSet.getMetaData();
    int numColumns = rms.getColumnCount();
    for (int i = 1; i <= numColumns; i++) {
      String name = rms.getColumnName(i);
      String description = rms.getColumnLabel(i);
      int type = rms.getColumnType(i);
      int scale = rms.getScale(i);
      int precision = rms.getPrecision(i);
      int columnType = 0;
      int columnAlign = LayoutHints.LEFT;
      switch (dataTypeMapper.getJavaType(type, scale, precision)) {
        case DataTypeMapper.JAVA_TYPE_OBJECT: 
          columnType = UjacTypes.TYPE_OBJECT;
          columnAlign = LayoutHints.LEFT;
          break;
        case DataTypeMapper.JAVA_TYPE_INT: 
          columnType = UjacTypes.TYPE_LONG;
          columnAlign = LayoutHints.RIGHT;
          break;
        case DataTypeMapper.JAVA_TYPE_LONG: 
          columnType = UjacTypes.TYPE_LONG;
          columnAlign = LayoutHints.RIGHT;
          break;
        case DataTypeMapper.JAVA_TYPE_FLOAT: 
          columnType = UjacTypes.TYPE_FLOAT;
          columnAlign = LayoutHints.RIGHT;
          break;
        case DataTypeMapper.JAVA_TYPE_DOUBLE: 
          columnType = UjacTypes.TYPE_DOUBLE;
          columnAlign = LayoutHints.RIGHT;
          break;
        case DataTypeMapper.JAVA_TYPE_BOOLEAN: 
          columnType = UjacTypes.TYPE_BOOLEAN;
          columnAlign = LayoutHints.LEFT;
          break;
        case DataTypeMapper.JAVA_TYPE_STRING: 
          columnType = UjacTypes.TYPE_STRING;
          columnAlign = LayoutHints.LEFT;
          break;
        case DataTypeMapper.JAVA_TYPE_DATE: 
          columnType = UjacTypes.TYPE_DATE;
          columnAlign = LayoutHints.LEFT;
          break;
        case DataTypeMapper.JAVA_TYPE_TIME: 
          columnType = UjacTypes.TYPE_TIME;
          columnAlign = LayoutHints.LEFT;
          break;
        case DataTypeMapper.JAVA_TYPE_TIMESTAMP: 
          columnType = UjacTypes.TYPE_TIMESTAMP;
          columnAlign = LayoutHints.LEFT;
          break;
        default:
          throw new TableException("Unsupported SQL type '" + rms.getColumnTypeName(i) + "' at column '" + name + "'.");
      }
      // adding new column to column map
      Column col = super.addColumn(name, columnType);
      // creating layout hints
      LayoutHints lh = new LayoutHints(10, description, columnAlign);
      col.setLayoutHints(lh);
    }
  }

  /**
   * Adds a column to the table
   * @param name The column name.
   * @param type The column type.
   * @return The new column.
   * @exception OperationNotSupportedException This method is not supported for this implementation.
   */
  public Column addColumn(String name, int type) throws OperationNotSupportedException {
    throw new OperationNotSupportedException("Adding columns is not allowed for ResultSetTables.");    
  }
  
  /**
   * Adds a column to the table
   * @param name The column name.
   * @param type The column type.
   * @param layoutHints The layout hints for the column.
   * @return The new column.
   * @exception OperationNotSupportedException In case the operation is not supported.
   */
  public Column addColumn(String name, int type, LayoutHints layoutHints)
    throws OperationNotSupportedException {
    
    throw new OperationNotSupportedException("Adding columns is not allowed for ResultSetTables.");    
  }

  /**
   * Adds a column of type String to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @exception OperationNotSupportedException This method is not supported for this implementation.
   */
  public Table addString(String name, String value) throws OperationNotSupportedException {
    throw new OperationNotSupportedException("Adding columns is not allowed for ResultSetTables.");    
  }

  /**
   * Adds a column of type int to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @exception TableException This method is not supported for this implementation.
   */
  public Table addInt(String name, int value) throws TableException { 
    throw new OperationNotSupportedException("Adding columns is not allowed for ResultSetTables.");    
  }

  /**
   * Adds a column of type long to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @exception TableException This method is not supported for this implementation.
   */
  public Table addLong(String name, long value) throws TableException { 
    throw new OperationNotSupportedException("Adding columns is not allowed for ResultSetTables.");    
  }

  /**
   * Adds a column of type float to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @exception TableException This method is not supported for this implementation.
   */
  public Table addFloat(String name, float value) throws TableException { 
    throw new OperationNotSupportedException("Adding columns is not allowed for ResultSetTables.");    
  }

  /**
   * Adds a column of type double to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @exception TableException This method is not supported for this implementation.
   */
  public Table addDouble(String name, double value) throws TableException { 
    throw new OperationNotSupportedException("Adding columns is not allowed for ResultSetTables.");    
  }

  /**
   * Adds a column of type boolean to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @exception TableException This method is not supported for this implementation.
   */
  public Table addBoolean(String name, boolean value) throws TableException { 
    throw new OperationNotSupportedException("Adding columns is not allowed for ResultSetTables.");    
  }

  /**
   * Adds a column of type Date to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @exception TableException This method is not supported for this implementation.
   */
  public Table addDate(String name, Date value) throws TableException { 
    throw new OperationNotSupportedException("Adding columns is not allowed for ResultSetTables.");    
  }

  /**
   * Adds a column of type Date to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @exception TableException This method is not supported for this implementation.
   */
  public Table addTime(String name, Date value) throws TableException { 
    throw new OperationNotSupportedException("Adding columns is not allowed for ResultSetTables.");    
  }

  /**
   * Adds a column of type Date to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @exception TableException This method is not supported for this implementation.
   */
  public Table addTimestamp(String name, Date value) throws TableException { 
    throw new OperationNotSupportedException("Adding columns is not allowed for ResultSetTables.");    
  }

  /**
   * Checks if the table is empty.
   * @return true if the table has no rows, else false.
   */
  public boolean isEmpty() {
    return (currentRow == null) && !checkNextRow();
  }

  /**
   * Gets the current number of rows. Always return -1, because retrieving the number of 
   * result records is a problem for result sets.
   * @return The number of rows.
   */
  public int getRowCount() {
    return -1;
  }

  /**
   * Adds a row to the table
   * @return The new row.
   * @exception OperationNotSupportedException This method is not supported for this implementation.
   */
  public Row addRow() throws OperationNotSupportedException {
    throw new OperationNotSupportedException("Adding rows is not allowed for ResultSetTables.");    
  }
  /**
   * @see org.ujac.util.table.Table#addRow(org.ujac.util.table.Row)
   */
  public Row addRow(Row srcRow) throws OperationNotSupportedException {
    throw new OperationNotSupportedException("Adding rows is not allowed for ResultSetTables.");    
  }

  /**
   * Gets a row from the table
   * @param rowIdx The index of the desired row.
   * @return The new row.
   * @exception RowNotDefinedException In case the requested row does not exist.
   */
  public Row getRow(int rowIdx) throws RowNotDefinedException {
    setCurrentRow(rowIdx + 1);
    return currentRow;
  }

  /**
   * Removes a row from the table
   * @param rowIdx The index of the row to remove.
   * @exception OperationNotSupportedException This method is not supported for this implementation.
   */
  public void removeRow(int rowIdx) throws OperationNotSupportedException {
    throw new OperationNotSupportedException("Removing rows is not allowed for ResultSetTables.");    
  }
  
  /**
   * Gets the header row.
   * @return The header row for the table.
   */
  public Row getHeaderRow() {
    HeaderRow row = new HeaderRow(this);
    return row;
  }
  
  /**
   * Returns an iterator over the rows in this table in proper sequence.
   * @return an iterator over the rows in this table in proper sequence.
   */
  public Iterator iterator() {
    return new ResultSetTableIterator(this); 
  }
  
  /**
   * Gets the String value from the given row at the given column.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the String conversion failed.
   */
  public String getString(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    setCurrentRow(rowIdx + 1);
    return currentRow.getString(columnIdx);
  }
  /**
   * Gets the String value from the given row at the given column.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the String conversion failed.
   */
  public String getString(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    Column col = getColumn(columnName);
    return getString(rowIdx, col.getIndex());
  }
  /**
   * Sets the String value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the String conversion failed.
   */
  public void setString(int rowIdx, int columnIdx, String value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    throw new RuntimeException("Manipulating values is not allowed for ResultSetTables.");    
  }
  /**
   * Sets the String value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the String conversion failed.
   */
  public void setString(int rowIdx, String columnName, String value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    Column col = getColumn(columnName);
    setString(rowIdx, col.getIndex(), value);
  }

  /**
   * Gets the integer value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the int conversion failed.
   */
  public int getInt(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    setCurrentRow(rowIdx + 1);
    return currentRow.getInt(columnIdx);
  }
  /**
   * Gets the integer value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the int conversion failed.
   */
  public int getInt(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    Column col = getColumn(columnName);
    return getInt(rowIdx, col.getIndex());
  }
  /**
   * Sets the integer value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the int conversion failed.
   */
  public void setInt(int rowIdx, int columnIdx, int value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    throw new RuntimeException("Manipulating values is not allowed for ResultSetTables.");    
  }
  /**
   * Sets the integer value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the int conversion failed.
   */
  public void setInt(int rowIdx, String columnName, int value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    setInt(rowIdx, columnIdx, value); 
  }

  /**
   * Gets the long value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the long conversion failed.
   */
  public long getLong(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    setCurrentRow(rowIdx + 1);
    return currentRow.getLong(columnIdx);
  }
  /**
   * Gets the long value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the long conversion failed.
   */
  public long getLong(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    return getLong(rowIdx, columnIdx); 
  }
  /**
   * Sets the long value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the long conversion failed.
   */
  public void setLong(int rowIdx, int columnIdx, long value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    throw new RuntimeException("Manipulating values is not allowed for ResultSetTables.");    
  }
  /**
   * Sets the integer value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the long conversion failed.
   */
  public void setLong(int rowIdx, String columnName, long value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    setLong(rowIdx, columnIdx, value); 
  }

  /**
   * Gets the float value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the float conversion failed.
   */
  public float getFloat(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    setCurrentRow(rowIdx + 1);
    return currentRow.getFloat(columnIdx);
  }
  /**
   * Gets the double value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the float conversion failed.
   */
  public float getFloat(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    return getFloat(rowIdx, columnIdx); 
  }
  /**
   * Sets the float value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the float conversion failed.
   */
  public void setFloat(int rowIdx, int columnIdx, float value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    throw new RuntimeException("Manipulating values is not allowed for ResultSetTables.");    
  }
  /**
   * Sets the float value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the float conversion failed.
   */
  public void setFloat(int rowIdx, String columnName, float value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    setFloat(rowIdx, columnIdx, value); 
  }

  /**
   * Gets the double value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the double conversion failed.
   */
  public double getDouble(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    setCurrentRow(rowIdx + 1);
    return currentRow.getDouble(columnIdx);
  }
  /**
   * Gets the double value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the double conversion failed.
   */
  public double getDouble(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    return getDouble(rowIdx, columnIdx); 
  }
  /**
   * Sets the double value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the double conversion failed.
   */
  public void setDouble(int rowIdx, int columnIdx, double value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    throw new RuntimeException("Manipulating values is not allowed for ResultSetTables.");    
  }
  /**
   * Sets the double value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the double conversion failed.
   */
  public void setDouble(int rowIdx, String columnName, double value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    setDouble(rowIdx, columnIdx, value); 
  }

  /**
   * Gets the boolean value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the boolean conversion failed.
   */
  public boolean getBoolean(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    setCurrentRow(rowIdx + 1);
    return currentRow.getBoolean(columnIdx);
  }
  /**
   * Gets the boolean value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the boolean conversion failed.
   */
  public boolean getBoolean(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    return getBoolean(rowIdx, columnIdx); 
  }
  /**
   * Sets the boolean value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the boolean conversion failed.
   */
  public void setBoolean(int rowIdx, int columnIdx, boolean value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    throw new RuntimeException("Manipulating values is not allowed for ResultSetTables.");    
  }
  /**
   * Sets the boolean value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the boolean conversion failed.
   */
  public void setBoolean(int rowIdx, String columnName, boolean value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    setBoolean(rowIdx, columnIdx, value); 
  }

  /**
   * Gets the Date value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the date conversion failed.
   */
  public Date getDate(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    setCurrentRow(rowIdx + 1);
    return currentRow.getDate(columnIdx);
  }
  /**
   * Gets the Date value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the date conversion failed.
   */
  public Date getDate(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    return getDate(rowIdx, columnIdx); 
  }
  /**
   * Sets the Date value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the date conversion failed.
   */
  public void setDate(int rowIdx, int columnIdx, Date value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    throw new RuntimeException("Manipulating values is not allowed for ResultSetTables.");    
  }
  /**
   * Sets the Date value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the date conversion failed.
   */
  public void setDate(int rowIdx, String columnName, Date value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    setDate(rowIdx, columnIdx, value); 
  }

  /**
   * Gets the Time value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the date conversion failed.
   */
  public Date getTime(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    setCurrentRow(rowIdx + 1);
    return currentRow.getTime(columnIdx);
  }
  /**
   * Gets the Time value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the time conversion failed.
   */
  public Date getTime(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    return getDate(rowIdx, columnIdx); 
  }
  /**
   * Sets the Time value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the time conversion failed.
   */
  public void setTime(int rowIdx, int columnIdx, Date value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    throw new RuntimeException("Manipulating values is not allowed for ResultSetTables.");    
  }
  /**
   * Sets the Time value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the time conversion failed.
   */
  public void setTime(int rowIdx, String columnName, Date value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    setTime(rowIdx, columnIdx, value); 
  }

  /**
   * Gets the Timestamp value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the timestamp conversion failed.
   */
  public Date getTimestamp(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    setCurrentRow(rowIdx + 1);
    return currentRow.getTimestamp(columnIdx);
  }
  /**
   * Gets the Timestamp value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the timestamp conversion failed.
   */
  public Date getTimestamp(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    return getTimestamp(rowIdx, columnIdx); 
  }
  /**
   * Sets the Timestamp value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the timestamp conversion failed.
   */
  public void setTimestamp(int rowIdx, int columnIdx, Date value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    throw new RuntimeException("Manipulating values is not allowed for ResultSetTables.");    
  }
  /**
   * Sets the Timestamp value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the timestamp conversion failed.
   */
  public void setTimestamp(int rowIdx, String columnName, Date value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    setDate(rowIdx, columnIdx, value); 
  }

  /**
   * Gets the Object value from the given row at the given column.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   */
  public Object getObject(int rowIdx, int columnIdx) throws RowNotDefinedException, ColumnNotDefinedException {
    setCurrentRow(rowIdx + 1);
    return currentRow.getObject(columnIdx);
  }
  /**
   * Gets the Object value from the given row at the given column.
   * @param rowIdx The index of the desired row.
   * @param columnName The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   */
  public Object getObject(int rowIdx, String columnName) throws RowNotDefinedException, ColumnNotDefinedException {
    int columnIdx = getColumnIndex(columnName);
    return getObject(rowIdx, columnIdx); 
  }
  /**
   * Sets the Object value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the object conversion failed.
   */
  public void setObject(int rowIdx, int columnIdx, Object value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    throw new RuntimeException("Manipulating values is not allowed for ResultSetTables.");    
  }
  /**
   * Sets the Object value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the object conversion failed.
   */
  public void setObject(int rowIdx, String columnName, Object value) throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    int columnIdx = getColumnIndex(columnName);
    setObject(rowIdx, columnIdx, value); 
  }

  /**
   * Checks, if the next row exists in the result set or not.
   * @return true if the database cursor has been moved to the next row successfuly, else false.
   */
  protected boolean checkNextRow() {
    if (currentRowIdx == currentResultSetRowIdx) {
      try {
        if (!resultSet.next()) {
          return false;
        }
      } catch (SQLException ex) {
        return false;
      }
      ++currentResultSetRowIdx;
    }
    return true;
  }
  
  /**
   * Sets the current row.
   * @param rowIdx The index of the row to move to.
   * @throws RowNotDefinedException In case the move request failed.
   */
  private void setCurrentRow(int rowIdx) throws RowNotDefinedException {
    if (rowIdx == currentRowIdx) {
      return;
    }
    try {
      int resultSetDelta = rowIdx - currentResultSetRowIdx;

      // maintaining cursor position
      if (resultSetDelta == 1) {
        if (!resultSet.next()) {
          throw new RowNotDefinedException("No more rows available: moving to row #" + rowIdx + " failed.");
        }
      } else if (resultSetDelta > 0) {
        if (!resultSet.relative(resultSetDelta)) {
          throw new RowNotDefinedException("No more rows available: moving to row #" + rowIdx + " failed.");
        }
      }
      currentResultSetRowIdx = rowIdx;
      currentRowIdx = rowIdx;

      TypeConverter typeConverter = getTypeConverter();
      
      // fetching data from current row
      currentRow = new DataRow(this, rowIdx - 1);
      int numColumns = getColumnCount();
      Object value = null;
      for (int i = 0; i < numColumns; i++) {
        Column col = getColumn(i);
        value = resultSet.getObject(col.getDataIndex() + 1);
        if (typeConverter != null) {
          value = typeConverter.convertObject(col.getType(), value);
        }
        currentRow.setObject(i, value);
      }
      currentRowIdx = rowIdx;
    } catch (SQLException ex) {
      throw new RowNotDefinedException("No more rows available: " + ex.getMessage());
    } catch (TypeConverterException ex) {
      throw new RowNotDefinedException("TypeConversion failed: " + ex.getMessage(), ex);
    } catch (TableException ex) {
      throw new RowNotDefinedException("No more rows available: " + ex.getMessage());
    }
  }
  
 
  /**
   * Converts the table to String.
   * @return the textual representation of the table.
   */
  public String toString() {
 
    StringBuffer buf = new StringBuffer();
    int numCols = getColumnCount();

    try {
      // first determining the widths of the columns
      int totalWidth = 1;
      int[] widths = new int[numCols];
      for (int i = 0; i < numCols; i++) {
        Column col = (Column) columns.get(i);
        widths[i] = computeColumnWidth(i, col);
        totalWidth += widths[i]+1;
      }
      
      for (int i = 0; i < totalWidth; i++) {
        buf.append('-');
      }
      buf.append("\n|");
      for (int i = 0; i < numCols; i++) {
        if (i > 0) {
          buf.append('|');
        }
        Column col = (Column) columns.get(i);
        printField(buf, col.getName() + ":" + col.getTypeName(), widths[i]);
      }
      buf.append("|\n");
      
      for (int k = 0; k < totalWidth; k++) {
        buf.append('-');
      }
      buf.append('\n');
      
      int numRows = getRowCount();
      for (int row = 0; row < numRows; row++) {
        if (row > 0) {
          buf.append('\n');
        }
        buf.append('|');
        
        for (int j = 0; j < numCols; j++) {
          if (j > 0) {
            buf.append('|');
          }
          printField(buf, getString(row, j), widths[j]);
        }
        buf.append('|');
      }
      
      buf.append('\n');
      for (int k = 0; k < totalWidth; k++) {
        buf.append('-');
      }

    } catch (RowNotDefinedException e) {
      e.printStackTrace();
    } catch (ColumnNotDefinedException e) {
      e.printStackTrace();
    } catch (TypeMismatchException e) {
      e.printStackTrace();
    }
    
    return buf.toString();
  }
  
  /**
   * Computes the width of one column.
   * @param colId The ID of the column.
   * @param col The definition of the column.
   * @return int The width of the column in characters.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException The desired column was not defined.
   * @exception TypeMismatchException A type mismatch problem has occured.
   */
  private int computeColumnWidth(int colId, Column col) 
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException {
    
    int width = col.getName().length() + col.getTypeName().length() + 1;
    int numRows = getRowCount();
    String val = null;
    int valLength = 0;
    for (int row=0; row<numRows; row++) {
      val = getString(row, colId);
      if (val == null) {
        continue;
      }
      valLength = val.length();
      width = valLength > width ? valLength : width;
    }
    return width;
  }

  /**
   * Prints a field to the given string buffer.
   * @param buf The string buffer.
   * @param value The textual value.
   * @param columnWidth The width of the column in characters.
   */
  private void printField(StringBuffer buf, String value, int columnWidth) {
    buf.append(value);
    int len = 0; 
    if (value != null) {
      len = value.length(); 
    } else {
      len = 4;
    }
    if (len >= columnWidth) {
      return;
    }
    
    for (int i=len; i<columnWidth; i++) {
      buf.append(' ');
    }
  }

  /**
   * Sorts the table according to the given compare item definitions.
   * @param compareColumns The column definition for the sort process.
   * @exception OperationNotSupportedException In case the operation is not supported.
   */
  public void sort(CompareItem[] compareColumns) throws OperationNotSupportedException {
    throw new OperationNotSupportedException("Sorting tables is not allowed for ResultSetTables.");    
  }

  /**
   * Sorts the table according to the given column and direction.
   * @param columnIdx The column to sort by.
   * @param direction The sort directon.
   * @exception OperationNotSupportedException In case the operation is not supported.
   */
  public void sort(int columnIdx, int direction) throws OperationNotSupportedException {
    throw new OperationNotSupportedException("Sorting tables is not allowed for ResultSetTables.");    
  }


  /**
   * Changes the type of the given column.
   * @param columnIdx The index of the column which has been changed.
   * @param typeId The new column type ID.
   * @exception ColumnNotDefinedException In case the desired column does not exist.
   * @exception TypeConverterException In case the object conversion failed.
   */
  public void setColumnType(int columnIdx, int typeId)
    throws ColumnNotDefinedException, TypeConverterException {
    
    Column column = getColumn(columnIdx);
    int oldType = column.getType();
    if (oldType == typeId) {
      // nothing 2 do
      return;
    }
    column.setType(typeId);

    if (getTypeConverter() == null) {
      setTypeConverter(new DefaultTypeConverter());
    }
  }

  /**
   * Changes the type of the given column.
   * @param columnName The name of the column which has been changed.
   * @param typeId The new column type ID.
   * @exception ColumnNotDefinedException In case the desired column does not exist.
   * @exception TypeConverterException In case the object conversion failed.
   */
  public void setColumnType(String columnName, int typeId)
    throws ColumnNotDefinedException, TypeConverterException {
    
    Column column = getColumn(columnName);
    setColumnType(column.getIndex(), typeId);
  }

}
