/*
 * Copyright (C) 2003 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.table;

import java.text.Format;
import java.util.Date;
import java.util.List;

/**
 * Name: Row<br>
 * Description: An interface for table rows.
 * 
 * @author lauerc
 */
public interface Row {
  
  /**
   * Gets the row type.
   * @return The row type.
   */
  String getType();
  
  /**
   * Gets the table which holds this row.
   * @return The table that contains this row.
   */
  Table getTable();

  /**
   * Gets the list of elements from this row.
   * @return The row data as a List.
   */
  List asList();

  /**
   * Gets the colum index.
   * @return The column index.
   */
  int getIndex();
  
  /**
   * Gets the String value from the given row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  String getString(String columnName) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Gets the String value from the given row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  String getString(int columnIdx) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the String value at the given row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the date conversion failed.
   */
  void setString(String columnName, String value) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the String value at the given row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  void setString(int columnIdx, String value) throws ColumnNotDefinedException, TypeMismatchException;
  
  /**
   * Gets the int value from the given row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  int getInt(String columnName) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Gets the integer value from the given row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  int getInt(int columnIdx) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the int value at the given row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  void setInt(String columnName, int value) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the integer value at the given row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  void setInt(int columnIdx, int value) throws ColumnNotDefinedException, TypeMismatchException;
  
  /**
   * Gets the long value from the given row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  long getLong(String columnName) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Gets the long value from the given row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  long getLong(int columnIdx) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the long value at the given row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  void setLong(String columnName, long value) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the long value at the given row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  void setLong(int columnIdx, long value) throws ColumnNotDefinedException, TypeMismatchException;
  
  /**
   * Gets the float value from the given row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the float conversion failed.
   */
  float getFloat(int columnIdx) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Gets the float value from the given row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the float conversion failed.
   */
  float getFloat(String columnName) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the float value at the given row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the float conversion failed.
   */
  void setFloat(int columnIdx, float value) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the float value at the given row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the float conversion failed.
   */
  void setFloat(String columnName, float value) throws ColumnNotDefinedException, TypeMismatchException;

  /**
   * Gets the double value from the given row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  double getDouble(String columnName) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Gets the double value from the given row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  double getDouble(int columnIdx) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the double value at the given row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  void setDouble(String columnName, double value) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the double value at the given row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  void setDouble(int columnIdx, double value) throws ColumnNotDefinedException, TypeMismatchException;
  
  /**
   * Gets the boolean value from the given row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  boolean getBoolean(String columnName) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Gets the boolean value from the given row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  boolean getBoolean(int columnIdx) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the boolean value at the given row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  void setBoolean(String columnName, boolean value) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the boolean value at the given row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  void setBoolean(int columnIdx, boolean value) throws ColumnNotDefinedException, TypeMismatchException;
  
  /**
   * Gets the Date value from the given row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  Date getDate(String columnName) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Gets the Date value from the given row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  Date getDate(int columnIdx) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the Date value at the given row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  void setDate(String columnName, Date value) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the Date value at the given row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  void setDate(int columnIdx, Date value) throws ColumnNotDefinedException, TypeMismatchException;
  
  /**
   * Gets the Time value from the given row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  Date getTime(String columnName) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Gets the Time value from the given row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  Date getTime(int columnIdx) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the Time value at the given row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  void setTime(String columnName, Date value) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the Time value at the given row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  void setTime(int columnIdx, Date value) throws ColumnNotDefinedException, TypeMismatchException;
  
  /**
   * Gets the Timestamp value from the given row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  Date getTimestamp(String columnName) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Gets the Timestamp value from the given row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  Date getTimestamp(int columnIdx) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the Timestamp value at the given row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  void setTimestamp(String columnName, Date value) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the Timestamp value at the given row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  void setTimestamp(int columnIdx, Date value) throws ColumnNotDefinedException, TypeMismatchException;
  
  /**
   * Gets the Object value from the given row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   */
  Object getObject(String columnName) throws ColumnNotDefinedException;
  /**
   * Gets the Object value from the given row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   */
  Object getObject(int columnIdx) throws ColumnNotDefinedException;
  /**
   * Sets the Object value at the given row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  void setObject(String columnName, Object value) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the Object value at the given row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  void setObject(int columnIdx, Object value) throws ColumnNotDefinedException, TypeMismatchException;

  /**
   * Formats the given column's value for output.
   * @param columnName The name of the desired column.
   * @return The formatted value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  String formatValue(String columnName) throws ColumnNotDefinedException, TypeMismatchException;
  /**
   * Formats the given column's value for output.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the data conversion failed.
   */
  String formatValue(int columnIdx) throws ColumnNotDefinedException, TypeMismatchException;

  /**
   * Checks whether the given field is visible or not.
   * @param columnName The name of the desired column.
   * @return The visibility flag.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   */
  boolean isVisible(String columnName) throws ColumnNotDefinedException;
  /**
   * Checks whether the given field is visible or not.
   * @param columnIdx The index of the desired column.
   * @return The visibility flag.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   */
  boolean isVisible(int columnIdx) throws ColumnNotDefinedException;
  /**
   * Sets the visibility flag for the given field.
   * @param columnName The name of the desired column.
   * @param visible The visibility flag.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   */
  void setVisible(String columnName, boolean visible) throws ColumnNotDefinedException;
  /**
   * Sets the visibility flag for the given field.
   * @param columnIdx The index of the desired column.
   * @param visible The visibility flag.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   */
  void setVisible(int columnIdx, boolean visible) throws ColumnNotDefinedException;
  
  /**
   * Gets the format of the given field.
   * @param columnName The name of the desired column.
   * @return The field's format.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   */
  public Format getFormat(String columnName) throws ColumnNotDefinedException;
  /**
   * Gets the format of the given field.
   * @param columnIdx The index of the desired column.
   * @return The field's format.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   */
  public Format getFormat(int columnIdx) throws ColumnNotDefinedException;
  /**
   * Sets the row specific format for the given field.
   * @param columnName The name of the desired column.
   * @param format The format to set.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   */
  public void setFormat(String columnName, Format format) throws ColumnNotDefinedException;
  /**
   * Sets the row specific format for the given field.
   * @param columnIdx The index of the desired column.
   * @param format The format to set.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   */
  public void setFormat(int columnIdx, Format format) throws ColumnNotDefinedException;

  /**
   * Gets the alignment of the given field.
   * @param columnName The name of the desired column.
   * @return The field's alignment.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   */
  public int getAlign(String columnName) throws ColumnNotDefinedException;
  /**
   * Gets the alignment of the given field.
   * @param columnIdx The index of the desired column.
   * @return The field's alignment.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   */
  public int getAlign(int columnIdx) throws ColumnNotDefinedException;
  /**
   * Sets the row specific alignment for the given field.
   * @param columnName The name of the desired column.
   * @param alignment The format to set.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   */
  void setAlign(String columnName, int alignment) throws ColumnNotDefinedException;
  /**
   * Sets the row specific alignment for the given field.
   * @param columnIdx The index of the desired column.
   * @param alignment The format to set.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   */
  void setAlign(int columnIdx, int alignment) throws ColumnNotDefinedException;

  /**
   * Tells whether or not to start a new page before printing this row.
   * @return true, if a page break is required here, else false.
   */
  boolean isStartNewPage();
  /**
   * Sets the startNewPage flag.
   * @param startNewPage The new value for the startNewPage attribute.
   */
  void setStartNewPage(boolean startNewPage);
  
  /**
   * Tells whether or not this row starts a new row block.
   * @return true, if this row starts a new row block, else false.
   */
  boolean isStartsBlock();
  /**
   * Sets the startsBlock flag.
   * @param startsBlock The new value for the startsBlock attribute.
   */
  void setStartsBlock(boolean startsBlock);
  
  /**
   * Tells whether or not this row ends the current row block.
   * @return true, if this row ends the current row block, else false.
   */
  boolean isEndsBlock();
  /**
   * Sets the endsBlock flag.
   * @param endsBlock The new value for the endsBlock attribute.
   */
  void setEndsBlock(boolean endsBlock);

}