/*
 * Copyright (C) 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.table;

import java.util.ArrayList;
import java.util.List;

/**
 * Name: RowFilterRuleSet<br>
 * Description: A set of rules which combines table row rules.
 * 
 * @author lauerc
 */
public class RowFilterRuleSet implements RowFilterRule {

  /** The connector type 'AND'. */
  public static final Integer AND = new Integer(1);
  /** The connector type 'OR'. */
  public static final Integer OR = new Integer(2);
  
  /** The value to compare the column value with. */
  private List rules = new ArrayList();
  
  /**
   * Constructs a RowFilterRuleSet instance with specific attributes.
   * @param firstRule The first rule to adds.
   */
  public RowFilterRuleSet(RowFilterRule firstRule) {
    this.rules.add(firstRule);
  }

  /**
   * Adds another rule to the connector.
   * @param connector The connector (AND/OR).
   * @param rule The rule to add
   * @return A reference to this class to provide a more fluent interface. 
   */
  public RowFilterRuleSet add(Integer connector, RowFilterRule rule) {
    rules.add(connector);
    rules.add(rule);
    return this;
  }
  
  /** 
   * Sets up the rule. 
   * @param table The table for which set up the rule.
   * @exception TableException In case a table handling failure occurred.
   */
  public void setup(Table table) throws TableException {
    int numRules = rules.size();
    for (int i = 0; i < numRules; i++) {
      RowFilterRule nextRule = (RowFilterRule) rules.get(i);
      nextRule.setup(table);
      // skipping next connector
      ++i;
    }
  }
  
  /**
   * @see org.ujac.util.table.RowFilterRule#test(org.ujac.util.table.Row)
   */
  public boolean test(Row row) throws TableException {
    int numRules = rules.size();
    boolean result = ((RowFilterRule) rules.get(0)).test(row);
    for (int i = 1; i < numRules; i++) {
      Integer connector = (Integer) rules.get(i++);
      RowFilterRule nextRule = (RowFilterRule) rules.get(i);
      if (AND.equals(connector)) {
        result = result && nextRule.test(row);
      } else if (OR.equals(connector)) {
        result = result || nextRule.test(row);
      }
    }
    return result;
  }

}
