/*
 * Copyright (C) 2003, 2004, 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.table;

import java.util.Date;
import java.util.Iterator;
import java.util.Set;

import org.ujac.util.TypeConverter;
import org.ujac.util.TypeConverterException;

import org.ujac.util.text.FormatHelper;

/**
 * Name: Table<br>
 * Description: A common interface for tables.
 * 
 * @author lauerc
 */
public interface Table extends TableConstants {

  /**
   * Getter method for the the property formatHelper.
   * @return The current value of property formatHelper.
   */
  FormatHelper getFormatHelper();
  /**
   * Setter method for the the property formatHelper.
   * @param formatHelper The value to set for the property formatHelper.
   */
  void setFormatHelper(FormatHelper formatHelper);

  /**
   * Getter method for the the property typeConverter.
   * @return The current value of property typeConverter.
   */
  TypeConverter getTypeConverter();
  /**
   * Setter method for the the property typeConverter.
   * @param typeConverter The value to set for the property typeConverter.
   */
  void setTypeConverter(TypeConverter typeConverter);

  /**
   * Gets the current number of columns.
   * @return The number of columns.
   */
  int getColumnCount();
  /** 
   * Gets the column at the given index.
   * @param index The column index.
   * @return The desired column.
   * @exception ColumnNotDefinedException If the column does not exist.
   */
  Column getColumn(int index) throws ColumnNotDefinedException;
  /** 
   * Gets the column with the given name.
   * @param name The column name.
   * @return The desired column.
   * @exception ColumnNotDefinedException If the column does not exist.
   */
  Column getColumn(String name) throws ColumnNotDefinedException;
  /** 
   * Gets the index of the given column name.
   * @param name The column name
   * @return int The index of the column.
   * @exception ColumnNotDefinedException If the column does not exist.
   */
  int getColumnIndex(String name) throws ColumnNotDefinedException;
  /** 
   * Renames a column.
   * @param column The column to rename.
   * @param name The column name to set.
   * @exception ColumnAlreadyDefinedException In case a column with that name already exists.
   */
  void renameColumn(Column column, String name) throws ColumnAlreadyDefinedException;
  /**
   * Orders the sequence of report columns according to the list of given column names.
   * @param columnNames The list of column names after which to order the columns.
   * @exception ColumnNotDefinedException In case a column from the given order list
   *  does not exist. 
   * @exception ColumnAlreadyDefinedException In case a column was defined more than 
   *  once at the given order list.
   */
  void orderColumns(String[] columnNames) 
    throws ColumnNotDefinedException, ColumnAlreadyDefinedException;
  /**
   * Gets all visible columns.
   * @return An array holding all visible columns.
   */
  Column[] getVisibleColumns();
  /**
   * Gets all visible columns.
   * @param levelFilter The maximum level of importance to support.
   * @return An array holding all visible columns.
   */
  Column[] getVisibleColumns(int levelFilter);

  /**
   * Gets the header row.
   * @return The header row for the table.
   */
  Row getHeaderRow();
  /**
   * Returns an iterator over the rows in this table in proper sequence.
   * @return an iterator over the rows in this table in proper sequence.
   */
  Iterator iterator();

  /**
   * Adds a column to the table
   * @param name The column name.
   * @param type The column type.
   * @return The new column.
   * @exception ColumnAlreadyDefinedException In case the column was already defined.
   * @exception OperationNotSupportedException In case the operation is not supported.
   */
  Column addColumn(String name, int type) 
    throws ColumnAlreadyDefinedException, OperationNotSupportedException;

  /**
   * Adds a column to the table
   * @param name The column name.
   * @param type The column type.
   * @param layoutHints The layout hints for the column.
   * @return The new column.
   * @exception ColumnAlreadyDefinedException In case the column was already defined.
   * @exception OperationNotSupportedException In case the operation is not supported.
   */
  Column addColumn(String name, int type, LayoutHints layoutHints) 
    throws ColumnAlreadyDefinedException, OperationNotSupportedException;

  /**
   * Adds a column of type String to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @throws TableException If an error occured.
   */
  Table addString(String name, String value) throws TableException;

  /**
   * Adds a column of type int to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @throws TableException If an error occured.
   */
  Table addInt(String name, int value) throws TableException;

  /**
   * Adds a column of type long to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @throws TableException If an error occured.
   */
  Table addLong(String name, long value) throws TableException;

  /**
   * Adds a column of type float to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @throws TableException If an error occured.
   */
  Table addFloat(String name, float value) throws TableException;

  /**
   * Adds a column of type double to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @throws TableException If an error occured.
   */
  Table addDouble(String name, double value) throws TableException;

  /**
   * Adds a column of type boolean to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @throws TableException If an error occured.
   */
  Table addBoolean(String name, boolean value) throws TableException;

  /**
   * Adds a column of type Date to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @throws TableException If an error occured.
   */
  Table addDate(String name, Date value) throws TableException;

  /**
   * Adds a column of type Date to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @throws TableException If an error occured.
   */
  Table addTime(String name, Date value) throws TableException;

  /**
   * Adds a column of type Date to the table and sets its initial value at the first row.
   * @param name The column name.
   * @param value The initial value at the first row.
   * @return A reference to this table.
   * @throws TableException If an error occured.
   */
  Table addTimestamp(String name, Date value) throws TableException;

  /**
   * Checks if the table is empty.
   * @return true if the table has no rows, else false.
   */
  boolean isEmpty();
  /**
   * Gets the current number of rows.
   * @return The number of rows.
   * @exception OperationNotSupportedException In case the operation is not supported.
   */
  int getRowCount() throws OperationNotSupportedException;
  /**
   * Adds a row to the table
   * @return The new row.
   * @exception OperationNotSupportedException In case the operation is not supported.
   */
  Row addRow() throws OperationNotSupportedException;
  /**
   * Adds a row to the table
   * @param srcRow The row to copy.
   * @return The new row.
   * @exception OperationNotSupportedException In case the operation is not supported.
   */
  Row addRow(Row srcRow) throws OperationNotSupportedException;

  /**
   * Gets a row from the table
   * @param rowIdx The index of the desired row.
   * @return The new row.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception OperationNotSupportedException In case the operation is not supported.
   */
  Row getRow(int rowIdx) throws RowNotDefinedException, OperationNotSupportedException;

  /**
   * Removes a row from the table
   * @param rowIdx The index of the row to remove.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception OperationNotSupportedException In case the operation is not supported.
   */
  void removeRow(int rowIdx) throws RowNotDefinedException, OperationNotSupportedException;
  
  /**
   * Gets the String value from the given row at the given column.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the String conversion failed.
   */
  String getString(int rowIdx, int columnIdx)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Gets the String value from the given row at the given column.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the String conversion failed.
   */
  String getString(int rowIdx, String columnName)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the String value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the String conversion failed.
   */
  void setString(int rowIdx, int columnIdx, String value)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the String value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the String conversion failed.
   */
  void setString(int rowIdx, String columnName, String value)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;

  /**
   * Gets the integer value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the int conversion failed.
   */
  int getInt(int rowIdx, int columnIdx)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Gets the integer value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the int conversion failed.
   */
  int getInt(int rowIdx, String columnName)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the integer value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the int conversion failed.
   */
  void setInt(int rowIdx, int columnIdx, int value)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the integer value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the int conversion failed.
   */
  void setInt(int rowIdx, String columnName, int value)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;

  /**
   * Gets the long value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the long conversion failed.
   */
  long getLong(int rowIdx, int columnIdx)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Gets the long value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the long conversion failed.
   */
  long getLong(int rowIdx, String columnName)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the long value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the long conversion failed.
   */
  void setLong(int rowIdx, int columnIdx, long value)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the integer value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the long conversion failed.
   */
  void setLong(int rowIdx, String columnName, long value)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;

  /**
   * Gets the float value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the float conversion failed.
   */
  float getFloat(int rowIdx, int columnIdx)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Gets the float value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the float conversion failed.
   */
  float getFloat(int rowIdx, String columnName)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the float value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the float conversion failed.
   */
  void setFloat(int rowIdx, int columnIdx, float value)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the float value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the float conversion failed.
   */
  void setFloat(int rowIdx, String columnName, float value)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;

  /**
   * Gets the double value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the double conversion failed.
   */
  double getDouble(int rowIdx, int columnIdx)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Gets the double value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the double conversion failed.
   */
  double getDouble(int rowIdx, String columnName)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the double value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the double conversion failed.
   */
  void setDouble(int rowIdx, int columnIdx, double value)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the double value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the double conversion failed.
   */
  void setDouble(int rowIdx, String columnName, double value)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  
  /**
   * Gets the boolean value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the boolean conversion failed.
   */
  boolean getBoolean(int rowIdx, int columnIdx)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Gets the boolean value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the boolean conversion failed.
   */
  boolean getBoolean(int rowIdx, String columnName)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the boolean value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the boolean conversion failed.
   */
  void setBoolean(int rowIdx, int columnIdx, boolean value)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the boolean value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the boolean conversion failed.
   */
  void setBoolean(int rowIdx, String columnName, boolean value)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;

  /**
   * Gets the Date value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the date conversion failed.
   */
  Date getDate(int rowIdx, int columnIdx)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Gets the Date value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the date conversion failed.
   */
  Date getDate(int rowIdx, String columnName)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the Date value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the date conversion failed.
   */
  void setDate(int rowIdx, int columnIdx, Date value) 
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the Time value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the date conversion failed.
   */
  void setDate(int rowIdx, String columnName, Date value)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;

  /**
   * Gets the Time value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the date conversion failed.
   */
  Date getTime(int rowIdx, int columnIdx)
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Gets the Date value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the time conversion failed.
   */
  Date getTime(int rowIdx, String columnName) 
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the Date value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the time conversion failed.
   */
  void setTime(int rowIdx, int columnIdx, Date value) 
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the Time value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the time conversion failed.
   */
  void setTime(int rowIdx, String columnName, Date value) 
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;

  /**
   * Gets the Timestamp value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the timestamp conversion failed.
   */
  Date getTimestamp(int rowIdx, int columnIdx) 
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Gets the Timestamp value from the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the timestamp conversion failed.
   */
  Date getTimestamp(int rowIdx, String columnName) 
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the Timestamp value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the timestamp conversion failed.
   */
  void setTimestamp(int rowIdx, int columnIdx, Date value) 
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the Timestamp value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the timestamp conversion failed.
   */
  void setTimestamp(int rowIdx, String columnName, Date value) 
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;

  /**
   * Gets the Object value from the given row at the given column.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   */
  Object getObject(int rowIdx, int columnIdx) 
    throws RowNotDefinedException, ColumnNotDefinedException;
  /**
   * Gets the Object value from the given row at the given column.
   * @param rowIdx The index of the desired row.
   * @param columnName The index of the desired column.
   * @return The detected value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   */
  Object getObject(int rowIdx, String columnName) 
    throws RowNotDefinedException, ColumnNotDefinedException;
  /**
   * Sets the Object value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnIdx The index of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the object conversion failed.
   */
  void setObject(int rowIdx, int columnIdx, Object value) 
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;
  /**
   * Sets the Object value at the given row.
   * @param rowIdx The index of the desired row.
   * @param columnName The name of the desired column.
   * @param value The new value.
   * @exception RowNotDefinedException In case the requested row does not exist.
   * @exception ColumnNotDefinedException In case the requested column does not exist.
   * @exception TypeMismatchException In case the object conversion failed.
   */
  void setObject(int rowIdx, String columnName, Object value) 
    throws RowNotDefinedException, ColumnNotDefinedException, TypeMismatchException;

  /**
   * Sorts the table according to the given compare item definitions.
   * @param compareColumns The column definition for the sort process.
   * @exception OperationNotSupportedException In case the operation is not supported.
   */
  void sort(CompareItem[] compareColumns) throws OperationNotSupportedException;
  /**
   * Sorts the table according to the given column and direction.
   * @param columnIdx The column to sort by.
   * @param direction The sort directon.
   * @exception OperationNotSupportedException In case the operation is not supported.
   */
  void sort(int columnIdx, int direction) throws OperationNotSupportedException;

  /**
   * Changes the type of the given column.
   * @param columnIdx The index of the column which has been changed.
   * @param typeId The new column type ID.
   * @exception ColumnNotDefinedException In case the desired column does not exist.
   * @exception TypeConverterException In case the object conversion failed.
   */
  void setColumnType(int columnIdx, int typeId)
    throws ColumnNotDefinedException, TypeConverterException;

  /**
   * Changes the type of the given column.
   * @param columnName The name of the column which has been changed.
   * @param typeId The new column type ID.
   * @exception ColumnNotDefinedException In case the desired column does not exist.
   * @exception TypeConverterException In case the object conversion failed.
   */
  void setColumnType(String columnName, int typeId)
    throws ColumnNotDefinedException, TypeConverterException;


  /**
   * Copies the table.
   * @return A copy holding the filtered contents of the handled table.
   * @exception TableException In case an unexpected failure occurred.
   */
  Table copyTable() throws TableException;
  
  /**
   * Gets the distinct values of the specified column.
   * @param columnIdx The index of the desired column.
   * @return A set of distinct values.
   * @exception TableException In case an unexpected failure occurred.
   */
  Set getDistinctValues(int columnIdx) throws TableException;
  
  /**
   * Gets the distinct values of the specified column.
   * @param columnName The name of the desired column.
   * @return A set of distinct values.
   * @exception TableException In case an unexpected failure occurred.
   */
  Set getDistinctValues(String columnName) throws TableException;

}