/*
 * Copyright (C) 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.table;

import java.io.IOException;
import java.io.PrintWriter;
import java.text.Format;
import java.util.Iterator;
import java.util.Locale;

import org.ujac.util.BeanUtils;
import org.ujac.util.UjacTypes;
import org.ujac.util.text.FormatHelper;
import org.ujac.util.xml.BaseObjectSerializer;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/**
 * Name: TableSerializer<br>
 * Description: A class serializing/deserializing tables to/from XML.
 * 
 * @author lauerc
 */
public class TableSerializer extends BaseObjectSerializer {

  /**
   * Constructs a TableSerializer instance with no specific attributes.
   */
  public TableSerializer() {
  }

  /**
   * Constructs a TableSerializer instance with specific attributes.
   * @param encoding The encoding to use.
   */
  public TableSerializer(String encoding) {
    super(encoding);
  }
  
  /** Constant for tag 'table'. */
  public static final String TAG_TABLE = "table";
  /** Constant for tag 'columns'. */
  public static final String TAG_COLUMNS = "columns";
  /** Constant for tag 'column'. */
  public static final String TAG_COLUMN = "column";
  /** Constant for tag 'header'. */
  public static final String TAG_HEADER = "header";
  /** Constant for tag 'layout-hints'. */
  public static final String TAG_LAYOUT_HINTS = "layout-hints";
  /** Constant for tag 'rows'. */
  public static final String TAG_ROWS = "rows";
  /** Constant for tag 'row'. */
  public static final String TAG_ROW = "row";
  /** Constant for tag 'cell'. */
  public static final String TAG_CELL = "cell";
  
  /** Constant for attribute 'name'. */
  public static final String ATTR_NAME = "name";
  /** Constant for attribute 'type'. */
  public static final String ATTR_TYPE = "type";
  /** Constant for attribute 'alias'. */
  public static final String ATTR_ALIAS = "alias";
  /** Constant for attribute 'width'. */
  public static final String ATTR_WIDTH = "width";
  /** Constant for attribute 'align'. */
  public static final String ATTR_ALIGN = "align";
  /** Constant for attribute 'header-align'. */
  public static final String ATTR_HEADER_ALIGN = "header-align";
  /** Constant for attribute 'level'. */
  public static final String ATTR_LEVEL = "level";
  /** Constant for attribute 'column'. */
  public static final String ATTR_COLUMN = "column";
  
  /**
   * Writes an object to the given print writer.
   * @param writer The writer to write the WebForm to.
   * @param obj The instance to write.
   * @exception IOException In case an I/O failure occurred.
   */
  public void write(PrintWriter writer, Object obj) throws IOException {
    super.write(writer, obj);
    
    Table table = (Table) obj;
    indent(writer, 0);
    writeOpenTag(writer, TAG_TABLE, false, true);
    int numColumns = table.getColumnCount();
    try {
      indent(writer, 1);
      writeOpenTag(writer, TAG_COLUMNS, false, true);
      Format[] origFormats = new Format[numColumns];
      FormatHelper fh = table.getFormatHelper();
      for (int i = 0; i < numColumns; i++) {
        Column col = table.getColumn(i);
        // remembering original formats
        origFormats[i] = col.getFormat();
        indent(writer, 2);
        writeOpenTag(writer, TAG_COLUMN, true, false);
        writeAttribute(writer, ATTR_NAME, col.getName());
        writeAttribute(writer, ATTR_TYPE, col.getTypeName());
        String alias = col.getAlias();
        if (alias != null) {
          writeAttribute(writer, ATTR_ALIAS, alias);
        }
        switch (col.getType()) {
          case UjacTypes.TYPE_INT:
          case UjacTypes.TYPE_SHORT:
          case UjacTypes.TYPE_LONG:
            col.setFormat(fh.getDefaultIntegerFormat(Locale.US));
            break;
          case UjacTypes.TYPE_DOUBLE:
          case UjacTypes.TYPE_FLOAT:
            col.setFormat(fh.getDefaultDoubleFormat(Locale.US));
            break;
          case UjacTypes.TYPE_DATE:
            col.setFormat(fh.getIsoDateFormat());
            break;
          case UjacTypes.TYPE_TIME:
            col.setFormat(fh.getIsoTimeFormat());
            break;
          case UjacTypes.TYPE_TIMESTAMP:
            col.setFormat(fh.getIsoTimestampFormat());
            break;
        }
        closeOpenTag(writer, false, true);
        LayoutHints lh = col.getLayoutHints();
        if (lh != null) {
          indent(writer, 3);
          writeOpenTag(writer, TAG_LAYOUT_HINTS, true, false);
          writeAttribute(writer, ATTR_WIDTH, Float.toString(lh.getWidth()));
          writeAttribute(writer, ATTR_ALIGN, lh.getAlignText());
          writeAttribute(writer, ATTR_HEADER_ALIGN, lh.getHeaderAlignText());
          writeAttribute(writer, ATTR_LEVEL, Integer.toString(lh.getLevel()));
          
          String header = lh.getHeader();
          if (BeanUtils.isEmpty(header)) {
            closeOpenTag(writer, true, true);
          } else {
            closeOpenTag(writer, false, true);

            indent(writer, 4);
            writeTag(writer, TAG_HEADER, header, true);
            
            indent(writer, 3);
            writeCloseTag(writer, TAG_LAYOUT_HINTS, true);
          }
        }
        indent(writer, 2);
        writeCloseTag(writer, TAG_COLUMN, true);
      }
      indent(writer, 1);
      writeCloseTag(writer, TAG_COLUMNS, true);

      indent(writer, 1);
      writeOpenTag(writer, TAG_ROWS, false, true);

      Iterator iterRows = table.iterator();
      while (iterRows.hasNext()) {
        Row row = (Row) iterRows.next();

        indent(writer, 2);
        writeOpenTag(writer, TAG_ROW, false, true);

        for (int i = 0; i < numColumns; i++) {
          String value = row.getString(i);
          if (value == null) {
            continue;
          }
          Column col = table.getColumn(i);
          int type = col.getType();
          
          indent(writer, 3);
          writeOpenTag(writer, TAG_CELL, true, false);
          writeAttribute(writer, ATTR_COLUMN, col.getName());
          closeOpenTag(writer, false, false);
          switch (type) {
            case UjacTypes.TYPE_STRING:
            case UjacTypes.TYPE_OBJECT:
              writeTagBody(writer, row.getString(i), true);
              break;
              
            default:
              writeTagBody(writer, row.getString(i), false);
              break;
          }
          writeCloseTag(writer, TAG_CELL, true);
        }
        
        indent(writer, 2);
        writeCloseTag(writer, TAG_ROW, true);
      }
      
      indent(writer, 1);
      writeCloseTag(writer, TAG_ROWS, true);

      // restoring original formats
      for (int i = 0; i < numColumns; i++) {
        Column col = table.getColumn(i);
        col.setFormat(origFormats[i]);
      }

    } catch (TableException ex) {
      throw new RuntimeException("Unable to serialize table: " + ex.getMessage(), ex);
    }
    indent(writer, 0);
    writeCloseTag(writer, TAG_TABLE, false);
  }
  

  /**
   * @see org.ujac.util.xml.BaseObjectSerializer#startElement(org.ujac.util.xml.BaseObjectSerializer.StackElement, org.ujac.util.xml.BaseObjectSerializer.StackElement, java.lang.Object)
   */
  public void startElement(StackElement localElement,
                           StackElement parentElement, Object root)
    throws SAXException {
    
    Attributes localAttrs = localElement.getAttributes();
    
    String name = localElement.getName();
    if (TAG_TABLE.equals(name)) {
      Table table = new DataTable();
      localElement.setData(table);
    } else if (TAG_COLUMNS.equals(name)) {
      localElement.setData(parentElement.getData());
    } else if (TAG_COLUMN.equals(name)) {
      Table table = (Table) parentElement.getData();
      FormatHelper fh = table.getFormatHelper();
      String colName = localAttrs.getValue(ATTR_NAME);
      String colTypeName = localAttrs.getValue(ATTR_TYPE);
      try {
        int colType = TableHelper.getColumnTypeId(colTypeName);
        Column col = table.addColumn(colName, colType);
        String colAlias = localAttrs.getValue(ATTR_ALIAS);
        if (!BeanUtils.isEmpty(colAlias)) {
          col.setAlias(colAlias);
        }
        switch (col.getType()) {
          case UjacTypes.TYPE_INT:
          case UjacTypes.TYPE_SHORT:
          case UjacTypes.TYPE_LONG:
            col.setFormat(fh.getDefaultIntegerFormat(Locale.US));
            break;
          case UjacTypes.TYPE_DOUBLE:
          case UjacTypes.TYPE_FLOAT:
            col.setFormat(fh.getDefaultDoubleFormat(Locale.US));
            break;
          case UjacTypes.TYPE_DATE:
            col.setFormat(fh.getIsoDateFormat());
            break;
          case UjacTypes.TYPE_TIME:
            col.setFormat(fh.getIsoTimeFormat());
            break;
          case UjacTypes.TYPE_TIMESTAMP:
            col.setFormat(fh.getIsoTimestampFormat());
            break;
        }
        localElement.setData(col);
      } catch (TableException ex) {
        throw new SAXException("Failed to handle attributes of column '" + colName + "': " + ex.getMessage(), ex);
      }
    } else if (TAG_LAYOUT_HINTS.equals(name)) {
      Column col = (Column) parentElement.getData();
      String strWidth = localAttrs.getValue(ATTR_WIDTH);
      String strAlign = localAttrs.getValue(ATTR_ALIGN);
      String strHeaderAlign = localAttrs.getValue(ATTR_HEADER_ALIGN);
      String strLevel = localAttrs.getValue(ATTR_LEVEL);
      try {
        float width = Float.parseFloat(strWidth);
        int align = TableHelper.getAlignmentId(strAlign);
        LayoutHints lh = new LayoutHints(width, null, align);
        int headerAlign = TableHelper.getAlignmentId(strHeaderAlign);
        lh.setHeaderAlign(headerAlign);
        int level = Integer.parseInt(strLevel);
        lh.setLevel(level);
        col.setLayoutHints(lh);
        localElement.setData(lh);
      } catch (NumberFormatException ex) {
        throw new SAXException("Failed to handle layout hints of column '" + col.getName() + "': " + ex.getMessage(), ex);
      }
    } else if (TAG_ROWS.equals(name)) {
      localElement.setData(parentElement.getData());
    } else if (TAG_ROW.equals(name)) {
      Table table = (Table) parentElement.getData();
      try {
        localElement.setData(table.addRow());
      } catch (TableException ex) {
        throw new SAXException("Failed to add row to table: " + ex.getMessage(), ex);
      }
    }
  }

  /**
   * @see org.ujac.util.xml.BaseObjectSerializer#endElement(org.ujac.util.xml.BaseObjectSerializer.StackElement, java.lang.String, org.ujac.util.xml.BaseObjectSerializer.StackElement, java.lang.Object)
   */
  public void endElement(StackElement localElement, String content,
                         StackElement parentElement, Object root)
    throws SAXException {
    
    String name = localElement.getName();
    if (TAG_HEADER.equals(name)) {
      LayoutHints lh = (LayoutHints) parentElement.getData();
      lh.setHeader(content.trim());
    } else if (TAG_CELL.equals(name)) {
      Row row = (Row) parentElement.getData();
      String colName = localElement.getAttributes().getValue(ATTR_COLUMN);
      try {
        row.setString(colName, content);
      } catch (TableException ex) {
        try {
          throw new SAXException(
              "Failed to set value '" + content + "' for column '" + colName + 
              "' with type " + row.getTable().getColumn(colName).getTypeName() + 
              ": " + ex.getMessage(), ex
          );
        } catch (ColumnNotDefinedException cnex) {
          throw new SAXException(
              "Failed to set value '" + content + "' for undefined column '" + colName + 
              "': " + ex.getMessage(), ex
          );
        }
      }
    } else if (TAG_TABLE.equals(name)) {
      Table table = (Table) localElement.getData();
      FormatHelper fh = table.getFormatHelper();
      // restoring default formats
      int numColumns = table.getColumnCount();
      try {
        for (int i = 0; i < numColumns; i++) {
          Column col = table.getColumn(i);
          switch (col.getType()) {
            case UjacTypes.TYPE_INT:
            case UjacTypes.TYPE_SHORT:
            case UjacTypes.TYPE_LONG:
              col.setFormat(fh.getIntegerFormat());
              break;
            case UjacTypes.TYPE_DOUBLE:
            case UjacTypes.TYPE_FLOAT:
              col.setFormat(fh.getDoubleFormat());
              break;
            case UjacTypes.TYPE_DATE:
              col.setFormat(fh.getDateFormat());
              break;
            case UjacTypes.TYPE_TIME:
              col.setFormat(fh.getTimeFormat());
              break;
            case UjacTypes.TYPE_TIMESTAMP:
              col.setFormat(fh.getTimestampFormat());
              break;
          }
        }
      } catch (TableException ex) {
        throw new SAXException("Failed to restore default column formats: " + ex.getMessage(), ex);
      }
    }
  }
  
}
