/*
 * Copyright (C) 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.table.test;

import org.ujac.util.UjacTypes;

import org.ujac.util.table.CountFunctionExecutor;
import org.ujac.util.table.DataTable;
import org.ujac.util.table.LayoutHints;
import org.ujac.util.table.PivotTable;
import org.ujac.util.table.Row;
import org.ujac.util.table.SumFunctionExecutor;
import org.ujac.util.table.Table;
import org.ujac.util.table.TableException;

import junit.framework.TestCase;

/**
 * Name: PivotTableTest<br>
 * Description: A JUnit test for pivot tables.
 * 
 * @author lauerc
 */
public class PivotTableTest extends TestCase {

  /** The source table. */
  private Table dataTable;
  
  /**
   * Constructs a PivotTableTest instance with specific attributes.
   */
  public PivotTableTest() {
    super();
  }
  
  /**
   * Constructs a PivotTableTest instance with specific attributes.
   * @param name The name of the test case.
   */
  public PivotTableTest(String name) {
    super(name);
  }
  
  /**
   * Sets up the test case.
   * @see junit.framework.TestCase#setUp()
   */
  protected void setUp() throws Exception {
    super.setUp();
    
    this.dataTable = new DataTable();
    dataTable.addColumn("department", UjacTypes.TYPE_STRING, new LayoutHints(15, "Department"));
    dataTable.addColumn("group", UjacTypes.TYPE_STRING, new LayoutHints(15, "Group"));
    dataTable.addColumn("name", UjacTypes.TYPE_STRING, new LayoutHints(20, "Name"));
    dataTable.addColumn("value", UjacTypes.TYPE_DOUBLE, new LayoutHints(20, "Value"));
    
    Row row = dataTable.addRow();
    row.setString(0, "Dep1");
    row.setString(1, "Group1");
    row.setString(2, "Harry");
    row.setDouble(3, 100);

    row = dataTable.addRow();
    row.setString(0, "Dep1");
    row.setString(1, "Group2");
    row.setString(2, "Eric");
    row.setDouble(3, 430);

    row = dataTable.addRow();
    row.setString(0, "Dep1");
    row.setString(1, "Group2");
    row.setString(2, "Eric");
    row.setDouble(3, 321);
    
    row = dataTable.addRow();
    row.setString(0, "Dep1");
    row.setString(1, "Group2");
    row.setString(2, "Willie");
    row.setDouble(3, 130);

    row = dataTable.addRow();
    row.setString(0, "Dep1");
    row.setString(1, "Group2");
    row.setString(2, "Carl");
    row.setDouble(3, 331);

    row = dataTable.addRow();
    row.setString(0, "Dep2");
    row.setString(1, "Group1");
    row.setString(2, "Harry");
    row.setDouble(3, 430);

    row = dataTable.addRow();
    row.setString(0, "Dep2");
    row.setString(1, "Group2");
    row.setString(2, "Willie");
    row.setDouble(3, 130);

    row = dataTable.addRow();
    row.setString(0, "Dep2");
    row.setString(1, "Group2");
    row.setString(2, "Curt");
    row.setDouble(3, 123);

    row = dataTable.addRow();
    row.setString(0, "Dep3");
    row.setString(1, "Group1");
    row.setString(2, "Eric");
    row.setDouble(3, 232);

    row = dataTable.addRow();
    row.setString(0, "Dep4");
    row.setString(1, "Group1");
    row.setString(2, "Willie");
    row.setDouble(3, 534);
  }

  /**
   * Tests the creation of a PivotTable.
   * @throws TableException In case the data table access fails.
   */
  public void testPivotTableSum() throws TableException {
    Table table = new PivotTable(
        dataTable, 
        new String[] {"department", "group"}, 
        "name", "value",
        new SumFunctionExecutor() 
    );
    assertEquals(6, table.getRowCount());
    assertEquals(100, table.getRow(0).getDouble("sum"), 0.0);
    assertEquals(1212, table.getRow(1).getDouble("sum"), 0.0);
    assertEquals(430, table.getRow(2).getDouble("sum"), 0.0);
    assertEquals(253, table.getRow(3).getDouble("sum"), 0.0);
    assertEquals(232, table.getRow(4).getDouble("sum"), 0.0);
    assertEquals(534, table.getRow(5).getDouble("sum"), 0.0);
  }

  /**
   * Tests the creation of a PivotTable.
   * @throws TableException In case the data table access fails.
   */
  public void testPivotTableSumSingleGroup() throws TableException {
    Table table = new PivotTable(
        dataTable, 
        new String[] {"department"}, 
        "name", "value",
        new SumFunctionExecutor() 
    );
    assertEquals(4, table.getRowCount());
    assertEquals(1312, table.getRow(0).getDouble("sum"), 0.0);
    assertEquals(683, table.getRow(1).getDouble("sum"), 0.0);
    assertEquals(232, table.getRow(2).getDouble("sum"), 0.0);
    assertEquals(534, table.getRow(3).getDouble("sum"), 0.0);
  }

  /**
   * Tests the creation of a PivotTable.
   * @throws TableException In case the data table access fails.
   */
  public void testPivotTableCount() throws TableException {
    Table table = new PivotTable(
        dataTable, 
        new String[] {"department", "group"}, 
        "name", "value",
        new CountFunctionExecutor()
    );
    assertEquals(6, table.getRowCount());
    assertEquals(1, table.getRow(0).getInt("sum"));
    assertEquals(4, table.getRow(1).getInt("sum"));
    assertEquals(1, table.getRow(2).getInt("sum"));
    assertEquals(2, table.getRow(3).getInt("sum"));
    assertEquals(1, table.getRow(4).getInt("sum"));
    assertEquals(1, table.getRow(5).getInt("sum"));
  }
  
  /**
   * Tests the creation of a PivotTable.
   * @throws TableException In case the data table access fails.
   */
  public void testPivotTableCountSingleGroup() throws TableException {
    Table table = new PivotTable(
        dataTable, 
        new String[] {"department"}, 
        "name", "value",
        new CountFunctionExecutor()
    );
    assertEquals(4, table.getRowCount());
    assertEquals(5, table.getRow(0).getInt("sum"));
    assertEquals(3, table.getRow(1).getInt("sum"));
    assertEquals(1, table.getRow(2).getInt("sum"));
    assertEquals(1, table.getRow(3).getInt("sum"));
  }

}
