/*
 * Copyright (C) 2005 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.template;

import java.io.Writer;

import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.ExpressionTuple;
import org.ujac.util.exi.Operand;

/**
 * Name: AssertToken<br>
 * Description: Represents an _assert_ statement.
 * 
 * @author lauerc
 */
public class AssertToken extends TemplateToken {
  
  /** The serial version UID. */
  private static final long serialVersionUID = 3257562897587450673L;
  
  /** The assertion check. */
  private Operand assertionCheck = null;
  
  /**
   * Constructs a AssertToken instance with no specific attributes.
   */
  public AssertToken() {
  }
  
  /**
   * Gets the token name. 
   * @return The token name.
   */
  public String getTokenName() {
    return "assert";
  }
  
  /**
   * Initializes the token.
   * @param interpreter The template interpreter.
   * @param source The source code.
   * @param position The start position.
   * @param length The length of the interresting area.
   * @exception TemplateException In case the initialization failed.
   */
  public void initialize(BaseTemplateInterpreter interpreter, char[] source, int position, int length)
    throws TemplateException {
    
    super.initialize(interpreter, source, position, length);
    int endPos = parseArguments();
    setLength(endPos - position);
  }

  /**
   * Parses the arguments of the token.
   * @return The position after the argument list.
   * @exception StatementException In case the argument parsing failed.
   */
  public int parseArguments() throws StatementException {
    String tokenName = getTokenName();
    if (tokenName == null) {
      return position;
    }
    
    int curPos = position + tokenName.length() + 2;
    int endPos = position + getLength();
    char[] source = this.source;
    boolean inside = false;
    int argStart = -1;

    for (int i = curPos; i <= endPos; i++) {
      char c = source[i];
      switch (c) {
        case '(':
          inside = true;
          break;
          
        case ')':
          if (inside) {
            if (argStart != -1) {
              StringBuffer eb = new StringBuffer("${")
                .append(source, argStart, i - argStart)
                .append("}");
              try {
                ExpressionTuple et = interpreter.parseExpr(eb.toString());
                setArgument(et, 0);
                argStart = -1;
              } catch (ExpressionException ex) {
                throw new StatementException("Syntax error at position " + i + ": invalid condition statement.", ex);
              }
            }
            return i;
          }
          throw new StatementException("Syntax error at position " + i + ": unbalanced parethesis.");

        default:
          if (Character.isWhitespace(c)) {
            continue;
          }
          if (argStart < 0) {
            argStart = i;
          }
          break;
      }
      
    }
    throw new StatementException("Syntax error at position " + endPos + ": unbalanced parethesis.");
  }
  
  /**
   * Sets an argument.
   * @param argument The argument to set.
   * @param idx The argument index.
   * @exception StatementException In case the given argument is not valid.
   */
  public void setArgument(Operand argument, int idx) throws StatementException {
    switch (idx) {
      case 0:
        this.assertionCheck = argument;
        break;
        
      default:
        throw new StatementException("The assert statement may take one argument only.");
    }
  }

  /**
   * Excecutes the statement token.
   * @param writer The writer to write to.
   * @param ctx The template context.
   * @exception ExpressionException If the token could not be successfuly executed.
   */
  public void execute(Writer writer, TemplateContext ctx)
    throws ExpressionException {

    if (!interpreter.evalBooleanExpr(assertionCheck, ctx)) {
      throw new AssertionException("Assertion failed: " + assertionCheck + "!");
    }
  }
  
  /**
   * @see java.lang.Object#toString()
   */
  public String toString() {
    return getIndention() + "_assert_(" + getArgumentsAsString() + ")\n"; 
  }
  
  /**
   * Clones the token for caching reasons.
   * @param interpreter The interpreter to use.
   * @param parent The actual parent in the cloned object tree.
   * @return The clone.
   */
  protected TemplateToken cloneToken(BaseTemplateInterpreter interpreter, TemplateTokenContainer parent) {
    StaticToken clone =  (StaticToken) clone();
    clone.interpreter = interpreter;
    clone.parent = parent;
    return clone;
  }
  
  /**
   * @see java.lang.Object#clone()
   */
  public Object clone() {
    try {
      return super.clone();
    } catch (CloneNotSupportedException ex) {
      throw new RuntimeException("Failed to clone template token: " + ex.getMessage(), ex);
    }
  }

}
