/*
 * Copyright (C) 2005 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.template;

import java.io.IOException;
import java.io.Writer;

import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.ExpressionTuple;

/**
 * Name: BaseConditionToken<br>
 * Description: A base class for condition tokens.
 * 
 * @author lauerc
 */
public abstract class BaseConditionToken extends TemplateTokenContainer {

  /**
   * Constructs a BaseConditionToken instance with no specific attributes.
   */
  public BaseConditionToken() {
  }

  /**
   * Parses the arguments of the token.
   * @return The position after the argument list.
   * @exception StatementException In case the argument parsing failed.
   */
  public int parseArguments() throws StatementException {
    String tokenName = getTokenName();
    if (tokenName == null) {
      return position;
    }
    
    int curPos = position + tokenName.length() + 2;
    int endPos = position + getLength();
    char[] source = this.source;
    boolean inside = false;
    int argStart = -1;
    for (int i = curPos; i <= endPos; i++) {
      char c = source[i];
      switch (c) {
        case '(':
          inside = true;
          break;
          
        case ')':
          if (inside) {
            if (argStart != -1) {
              StringBuffer eb = new StringBuffer("${")
                .append(source, argStart, i - argStart)
                .append("}");
              try {
                ExpressionTuple et = interpreter.parseExpr(eb.toString());
                setArgument(et, 0);
                argStart = -1;
              } catch (ExpressionException ex) {
                throw new StatementException("Syntax error at position " + i + ": invalid condition statement.", ex);
              }
            }
            return i;
          }
          throw new StatementException("Syntax error at position " + i + ": unbalanced parethesis.");

        default:
          if (Character.isWhitespace(c)) {
            continue;
          }
          if (argStart < 0) {
            argStart = i;
          }
          break;
      }
      
    }
    throw new StatementException("Syntax error at position " + endPos + ": unbalanced parethesis.");
  }

  /**
   * Excecutes the statement token.
   * @param writer The writer to write to.
   * @param ctx The template context.
   * @exception TemplateException If the token could not be successfuly executed.
   * @exception IOException In case an output failure occurred.
   */
  public void execute(Writer writer, TemplateContext ctx)
    throws TemplateException, IOException {

    if (isTrue(ctx)) {
      int numTokens = getTokenCount();
      for (int i = 0; i < numTokens; i++) {
        TemplateToken subToken = getToken(i);
        subToken.execute(writer, ctx);
      }
    }
  }

  /**
   * Tells whether or not the condition is true.
   * @param ctx The template context.
   * @return true or false.
   * @exception ExpressionException If the token could not be successfuly executed.
   */
  public abstract boolean isTrue(TemplateContext ctx) throws ExpressionException;

  /**
   * Tells whether or not the condition bas been resolved so far.
   * @return true or false.
   */
  public boolean isResolved() {
    return getRootCondition().isResolved();
  }
  /**
   * Tells whether or not the condition bas been resolved so far.
   * @param resolved The value to seet.
   */
  public void setResolved(boolean resolved) {
    getRootCondition().setResolved(resolved);
  }


  /**
   * Gets the previous condition token in the parent's child list. 
   * @return The condition token.
   * @exception StatementException In case the latest token was no condition token.
   */
  protected BaseConditionToken getPreviousCondition() throws StatementException {
    TemplateTokenContainer parent = getParent();
    int numParentChilds = parent.getTokenCount();
    if (numParentChilds < 2) {
      throw new StatementException("The _" + getTokenName() + "_ statement must directly follow a condition statement!");
    }
    try {
      TemplateToken token = parent.getToken(numParentChilds - 2);
      return (BaseConditionToken) token;
    } catch (ClassCastException ex) {
      throw new StatementException("The _" + getTokenName() + "_ statement must directly follow a condition statement!");
    }
  }
  
  /**
   * Gets the root condition.
   * @return The root condition of the current condition.
   */
  public abstract BaseConditionToken getRootCondition();
  
}
