/*
 * Copyright (C) 2005 - 2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.template;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;

import java.sql.Time;
import java.sql.Timestamp;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.ExpressionInterpreter;
import org.ujac.util.exi.ExpressionTuple;
import org.ujac.util.exi.Operand;

/**
 * Name: TemplateInterpreter<br>
 * Description: Description of the class.
 * 
 * @author lauerc
 */
public abstract class BaseTemplateInterpreter implements TemplateInterpreter {

  /** The expression interpeter to use. */
  protected ExpressionInterpreter exi = null;
  /** The template token registry (_if_, _foreach_, _loop_, ...) */
  protected Map tokenTypes = new HashMap();
  /** The expression filter. */
  protected ExpressionFilter expressionFilter = null;
 

  /**
   * Constructs a TemplateInterpreter instance with specific attributes.
   */
  public BaseTemplateInterpreter() {
    this.exi = ExpressionInterpreter.createInstance();
    registerTokenTypes();
  }

  /**
   * Constructs a TemplateInterpreter instance with specific attributes.
   * @param exi The expression interpreter to use.
   */
  public BaseTemplateInterpreter(ExpressionInterpreter exi) {
    this.exi = exi;
    registerTokenTypes();
  }
  
  /**
   * Constructs a TemplateInterpreter instance with specific attributes.
   * @param exi The expression interpreter to use.
   * @param tokenTypes The template token registry (_if_, _foreach_, _loop_, ...)
   */
  protected BaseTemplateInterpreter(ExpressionInterpreter exi, Map tokenTypes) {
    this.exi = exi;
    this.tokenTypes = tokenTypes;
  }
  
  /**
   * Registers the token types.
   */
  protected void registerTokenTypes() {
    // defining statement types
    tokenTypes.put("if", IfToken.class); 
    tokenTypes.put("else", ElseToken.class); 
    tokenTypes.put("elseif", ElseifToken.class); 

    tokenTypes.put("foreach", ForeachToken.class); 

    tokenTypes.put("procedure", ProcedureToken.class); 
    
    tokenTypes.put("callproc", CallprocToken.class);
    
    tokenTypes.put("define", DefineToken.class);
    
    tokenTypes.put("assert", AssertToken.class);

    tokenTypes.put("log", LogToken.class);
  }
  
  /**
   * Parses the given template source code.
   * @param source The source to parse.
   * @return The parsed statement.
   * @exception TemplateException If the expression could not be
   *   successfuly resolved
   * @exception IOException In case a I/O problem occured while writing the
   *   expression result.
   */
  public TemplateRoot parse(String source) 
    throws TemplateException, IOException {
   
    if (source == null) {
      return null;
    }
    char[] code = source.toCharArray();
    return parse(code, 0, code.length);
  }
  
  /**
   * Parses the given template source code.
   * @param source The source to parse.
   * @param offset The start position within the source.
   * @param length The length of the area to parse.
   * @return The parsed statement.
   * @exception TemplateException If the expression could not be
   *   successfuly resolved
   * @exception IOException In case a I/O problem occured while writing the
   *   expression result.
   */
  public TemplateRoot parse(char[] source, int offset, int length) 
    throws TemplateException, IOException {
   
    TemplateRoot root = new TemplateRoot(source, offset, length);
    parse(root);
    return root;
  }

  /**
   * Parses the given template token.
   * @param tokenContainer The token container to fill.
   * @return The continue position for the next parse step.
   * @exception TemplateException If the expression could not be
   *   successfuly resolved
   * @exception IOException In case a I/O problem occured while writing the
   *   expression result.
   */
  public abstract int parse(TemplateTokenContainer tokenContainer) 
    throws TemplateException, IOException;
 
  /**
   * Executes the given template.
   * @param template The source code of the template to execute.
   * @param ctx The template context to use.
   * @return The template execution result.
   * @exception TemplateException If the an expression could not be
   *   successfuly resolved
   */
  public String execute(String template, TemplateContext ctx)
    throws TemplateException {
    
    try {
      TemplateRoot templateRoot = parse(template);
      if (templateRoot == null) {
        return null;
      }
      StringWriter writer = new StringWriter();
      execute(templateRoot, writer, ctx);
      return writer.toString();
    } catch (IOException ex) {
      // This should never happen!
      throw new RuntimeException(ex.getMessage());
    }
  }

  /**
   * Executes the given template.
   * @param template The source code of the template to execute.
   * @param writer The writer to output the result to.
   * @param ctx The template context to use.
   * @exception TemplateException If the an expression could not be
   *   successfuly resolved
   * @exception IOException If something went wrong while writing the 
   *   result to the given writer.
   */
  public void execute(String template, Writer writer, TemplateContext ctx)
    throws TemplateException, IOException {
    
    TemplateRoot templateRoot = parse(template);
    execute(templateRoot, writer, ctx);
  }
  
  /**
   * Executes the given template.
   * @param template The root element of the template parse result.
   * @param writer The writer to output the result to.
   * @param ctx The template context to use.
   * @exception TemplateException If the an expression could not be
   *   successfuly resolved
   * @exception IOException If something went wrong while writing the 
   *   result to the given writer.
   */
  public void execute(TemplateRoot template, Writer writer, TemplateContext ctx)
    throws TemplateException, IOException {
    
    template.execute(writer, ctx);
  }
  
  /**
   * Parses an expression from the given source code range.
   * @param source The source code, to be parsed.
   * @return The parsed expression.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public final ExpressionTuple parseExpr(String source) 
    throws ExpressionException {
  
    return exi.parseExpr(source);
  }

  /**
   * Parses an expression from the given source code range.
   * @param source The source code, to be parsed.
   * @param offset The offset from which to parse the expression string. 
   * @param lastPos The last position to which to parse the expression string. 
   * @return The parsed expression.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public final ExpressionTuple parseExpr(char[] source, int offset, int lastPos) 
    throws ExpressionException {
  
    return exi.parseExpr(source, offset, lastPos);
  }
  
  /**
   * Evaluates an expression as a boolean value.
   * @param operand The operand to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public final boolean evalBooleanExpr(Operand operand, TemplateContext ctx)
    throws ExpressionException {

    return exi.evalBooleanOperand(operand, ctx);
  }
  /**
   * Evaluates an expression as a boolean value.
   * @param expression The expression to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public final boolean evalBooleanExpr(String expression, TemplateContext ctx)
    throws ExpressionException {
    
    return exi.evalBoolean(expression, ctx);
  }

  /**
   * Evaluates an expression as an int value.
   * @param operand The operand to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public int evalIntExpr(Operand operand, TemplateContext ctx)
    throws ExpressionException {
    
    return exi.evalIntOperand(operand, ctx, false);
  }
  /**
   * Evaluates an expression as an int value.
   * @param expression The expression to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public int evalIntExpr(String expression, TemplateContext ctx)
    throws ExpressionException {
    
    return exi.evalInt(expression, ctx);
  }

  /**
   * Evaluates an expression as a long value.
   * @param operand The operand to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public long evalLongExpr(Operand operand, TemplateContext ctx)
    throws ExpressionException {
    
    return exi.evalLongOperand(operand, ctx, false);
  }
  /**
   * Evaluates an expression as a long value.
   * @param expression The expression to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public long evalLongExpr(String expression, TemplateContext ctx)
    throws ExpressionException {
    
    return exi.evalLong(expression, ctx);
  }

  /**
   * Evaluates an expression as a float value.
   * @param operand The operand to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public float evalFloatExpr(Operand operand, TemplateContext ctx)
    throws ExpressionException {
    
    return exi.evalFloatOperand(operand, ctx, false);
  }
  /**
   * Evaluates an expression as a float value.
   * @param expression The expression to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public float evalFloatExpr(String expression, TemplateContext ctx)
    throws ExpressionException {
    
    return exi.evalFloat(expression, ctx);
  }

  /**
   * Evaluates an expression as a double value.
   * @param operand The operand to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public double evalDoubleExpr(Operand operand, TemplateContext ctx)
    throws ExpressionException {
    
    return exi.evalDoubleOperand(operand, ctx, false);
  }
  /**
   * Evaluates an expression as a double value.
   * @param expression The expression to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public double evalDoubleExpr(String expression, TemplateContext ctx)
    throws ExpressionException {
    
    return exi.evalDouble(expression, ctx);
  }

  /**
   * Evaluates an expression as a Date value.
   * @param operand The operand to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public Date evalDateExpr(Operand operand, TemplateContext ctx)
    throws ExpressionException {
    
    return exi.evalDateOperand(operand, ctx, false);
  }
  /**
   * Evaluates an expression as a Date value.
   * @param expression The expression to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public Date evalDateExpr(String expression, TemplateContext ctx)
    throws ExpressionException {
    
    return exi.evalDate(expression, ctx);
  }

  /**
   * Evaluates an expression as a Time value.
   * @param operand The operand to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public Time evalTimeExpr(Operand operand, TemplateContext ctx)
    throws ExpressionException {
    
    return exi.evalTimeOperand(operand, ctx, false);
  }
  /**
   * Evaluates an expression as a Time value.
   * @param expression The expression to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public Time evalTimeExpr(String expression, TemplateContext ctx)
    throws ExpressionException {
    
    return exi.evalTime(expression, ctx);
  }

  /**
   * Evaluates an expression as a Timestamp value.
   * @param operand The operand to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public Timestamp evalTimestampExpr(Operand operand, TemplateContext ctx)
    throws ExpressionException {
    
    return exi.evalTimestampOperand(operand, ctx, false);
  }
  /**
   * Evaluates an expression as a Timestamp value.
   * @param expression The expression to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public Timestamp evalTimestampExpr(String expression, TemplateContext ctx)
    throws ExpressionException {
    
    return exi.evalTimestamp(expression, ctx);
  }

  /**
   * Evaluates an expression as a String value.
   * @param operand The operand to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public String evalStringExpr(Operand operand, TemplateContext ctx)
    throws ExpressionException {
    
    return exi.evalStringOperand(operand, ctx);
  }
  /**
   * Evaluates an expression as a String value.
   * @param expression The expression to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public String evalStringExpr(String expression, TemplateContext ctx)
    throws ExpressionException {
    
    return exi.evalString(expression, ctx);
  }

  /**
   * Evaluates an expression as an Object.
   * @param operand The operand to evaluate.
   * @param ctx The template context.
   * @return The evaluated object.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public final Object evalObjectExpr(Operand operand, TemplateContext ctx)
    throws ExpressionException {

    return exi.evalOperand(operand, ctx);
  }
  /**
   * Evaluates an expression as an Object.
   * @param expression The expression to evaluate.
   * @param ctx The template context.
   * @return The evaluated object.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  public final Object evalObjectExpr(String expression, TemplateContext ctx)
    throws ExpressionException {

    return exi.evalObject(expression, ctx);
  }

  /**
   * Gets the token type that matches the given text.
   * @param tokenName The token name.
   * @return Integer The detected type or null.
   * @exception StatementException In case the instantiation failed.
   */
  public final TemplateToken createToken(String tokenName) throws StatementException {
    Class tc = getTokenType(tokenName);
    if (tc == null) {
      return null;
    }
    try {
      return (TemplateToken) tc.newInstance();
    } catch (IllegalAccessException ex) {
      throw new StatementException("template token instantiation failed.", ex);
    } catch (InstantiationException ex) {
      throw new StatementException("template token instantiation failed.", ex);
    }
  }

  /**
   * Gets the statement type that matches the given name.
   * @param tokenName The token name.
   * @return Integer The detected type or null.
   */
  public final Class getTokenType(String tokenName) {
    return (Class) tokenTypes.get(tokenName);
  }

  /**
   * Getter method for the the property expressionFilter.
   * @return The current value of property expressionFilter.
   */
  public ExpressionFilter getExpressionFilter() {
    return expressionFilter;
  }
  /**
   * Setter method for the the property expressionFilter.
   * @param expressionFilter The value to set for the property expressionFilter.
   */
  public void setExpressionFilter(ExpressionFilter expressionFilter) {
    this.expressionFilter = expressionFilter;
  }

}
