/*
 * Copyright (C) 2005 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.template;

import java.io.IOException;
import java.io.Writer;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.ujac.util.CollectionUtils;

import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.Operand;

/**
 * Name: CallprocToken<br>
 * Description: Represents a _callproc_ statement.
 * 
 * @author lauerc
 */
public class CallprocToken extends TemplateToken {
  
  /** The serial version UID. */
  private static final long serialVersionUID = 3762530096761942838L;
  
  /** The procedure name. */
  private String procName = null;
  /** The procedure arguments. */
  private List arguments = null;
  
  /**
   * Constructs a CallprocToken instance with no specific attributes.
   */
  public CallprocToken() {
  }

  /**
   * Gets the token name. 
   * @return The token name.
   */
  public String getTokenName() {
    return "callproc";
  }
  
  /**
   * Initializes the token.
   * @param interpreter The template interpreter.
   * @param source The source code.
   * @param position The start position.
   * @param length The length of the interresting area.
   * @exception TemplateException In case the initialization failed.
   */
  public void initialize(BaseTemplateInterpreter interpreter, char[] source, int position, int length)
    throws TemplateException {
    
    super.initialize(interpreter, source, position, length);
    int endPos = parseArguments();
    setLength(endPos - position);
  }

  /**
   * Sets an argument.
   * @param argument The argument to set.
   * @param idx The argument index.
   */
  public void setArgument(Operand argument, int idx) {
    if (idx == 0) {
      this.procName = argument.getValue().toString();
      return;
    }
    if (arguments == null) {
      arguments = new ArrayList();
    }
    arguments.add(argument);
  }

  /**
   * Getter method for the the property arguments.
   * @return The current value of property arguments.
   */
  public List getArguments() {
    return arguments;
  }

  /**
   * Excecutes the statement token.
   * @param writer The writer to write to.
   * @param ctx The template context.
   * @exception TemplateException If the token could not be successfuly executed.
   * @exception IOException In case an output failure occurred.
   */
  public void execute(Writer writer, TemplateContext ctx)
    throws TemplateException, IOException {

    // looking up procedure info
    Map procedureRegistry = getParent().getProcedureRegistry();
    ProcedureToken proc = (ProcedureToken) procedureRegistry.get(procName);
    if (proc == null) {
      throw new ExpressionException("No procedure registered with name '" + procName + "'.");
    }
    
    // building argument list
    int numArgs = CollectionUtils.getSize(arguments);
    List argValues = new ArrayList(numArgs);
    for (int i = 0; i < numArgs; i++) {
      Operand arg = (Operand) arguments.get(i);
      argValues.add(interpreter.evalObjectExpr(arg, ctx));
    }
    // calling procedure
    proc.execute(argValues, writer, ctx);
  }
  
  /**
   * @see java.lang.Object#toString()
   */
  public String toString() {
    return getIndention() + "_callproc_ " + procName + "(" + getArgumentsAsString() + ")\n"; 
  }

  /**
   * Gets the argument list as string.
   * @return The arguements as a comma separated text.
   */
  protected String getArgumentsAsString() {
    StringBuffer ab = new StringBuffer();
    int numArguments = arguments.size();
    List argumentNames = getArguments();
    for (int i = 0; i < numArguments; i++) {
      if (i > 0) {
        ab.append(", ");
      }
      ab.append(argumentNames.get(i));
    }
    return ab.toString();
  }

  /**
   * Clones the token for caching reasons.
   * @param interpreter The interpreter to use.
   * @param parent The actual parent in the cloned object tree.
   * @return The clone.
   */
  protected TemplateToken cloneToken(BaseTemplateInterpreter interpreter, TemplateTokenContainer parent) {
    CallprocToken clone = (CallprocToken) clone();
    clone.interpreter = interpreter;
    clone.parent = parent;
    int numOperands = CollectionUtils.getSize(clone.arguments);
    for (int i = 0; i < numOperands; i++) {
      clone.arguments.set(i, ((Operand) clone.arguments.get(i)).clone());
    }
    return clone;
  }
  
  /**
   * @see java.lang.Object#clone()
   */
  public Object clone() {
    try {
      return super.clone();
    } catch (CloneNotSupportedException ex) {
      throw new RuntimeException("Failed to clone template token: " + ex.getMessage(), ex);
    }
  }

}
