/*
 * Copyright (C) 2005 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.template;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;

import org.ujac.util.exi.ExpressionInterpreter;
import org.ujac.util.exi.ExpressionTuple;

/**
 * Name: DefaultTemplateInterpreter<br>
 * Description: Default TemplateInterpreter implementation.
 * 
 * @author lauerc
 */
public class DefaultTemplateInterpreter extends BaseTemplateInterpreter {

  /**
   * Constructs a TemplateInterpreter instance with specific attributes.
   */
  public DefaultTemplateInterpreter() {
  }

  /**
   * Constructs a TemplateInterpreter instance with specific attributes.
   * @param exi The expression interpreter to use.
   */
  public DefaultTemplateInterpreter(ExpressionInterpreter exi) {
    super(exi);
  }
  
  /**
   * Parses the given template source code.
   * @param source The source to parse.
   * @return The parsed statement.
   * @exception TemplateException If the expression could not be
   *   successfuly resolved
   * @exception IOException In case a I/O problem occured while writing the
   *   expression result.
   */
  public TemplateRoot parse(String source) 
    throws TemplateException, IOException {
   
    if (source == null) {
      return null;
    }
    char[] code = source.toCharArray();
    return parse(code);
  }
  
  /**
   * Parses the given template source code.
   * @param source The source to parse.
   * @return The parsed statement.
   * @exception TemplateException If the expression could not be
   *   successfuly resolved
   * @exception IOException In case a I/O problem occured while writing the
   *   expression result.
   */
  public TemplateRoot parse(char[] source) 
    throws TemplateException, IOException {
   
    TemplateRoot root = new TemplateRoot(source, 0, source.length);
    parse(root);
    return root;
  }

  /**
   * Parses the given template token.
   * @param tokenContainer The token container to fill.
   * @return The continue position for the next parse step.
   * @exception TemplateException If the expression could not be
   *   successfuly resolved
   * @exception IOException In case a I/O problem occured while writing the
   *   expression result.
   */
  public int parse(TemplateTokenContainer tokenContainer) 
    throws TemplateException, IOException {
    
    char[] source = tokenContainer.getSource();
    int pos = tokenContainer.getPosition();
    int length = tokenContainer.getLength();
    int maxPos = pos + length;
    
    int staticStart = -1;
  mainLoop:
    for (int i = tokenContainer.getBodyStart(); i < maxPos; i++) {
      char c = source[i];
      switch (c) {
        case '_':
          for (int ti = i + 1; ti < maxPos; ti++) {
            c = source[ti];
            switch (c) {
              case '_':
                if (staticStart >= 0) {
                  // adding static text
                  StaticToken st = new StaticToken(this, source, staticStart, i - staticStart);
                  tokenContainer.addToken(st);
                  staticStart = -1;
                }
                // detected token !?
                String stn = new String(source, i + 1, ti - i - 1);
                if (stn.length() == 0) {
                  // adding saving isolated underscore
                  StaticToken st = new StaticToken(this, source, i, 1);
                  tokenContainer.addToken(st);
                  staticStart = -1;
                  continue mainLoop;
                }
                TemplateToken tkn = createToken(stn);
                if (tkn == null) {
                  if (tokenContainer.isLegalAncestor(stn)) {
                    tokenContainer.close(i, ti);
                    return ti;
                  }
                  staticStart = i;
                  continue mainLoop;
                }
                if (tkn.isBodyToken()) {
                  if (tokenContainer.isLegalAncestor(tkn)) {
                    tokenContainer.close(i, ti);
                    tokenContainer.getParent().addToken(tkn);
                  } else {
                    tokenContainer.addToken(tkn);
                  }
                  // initializing token
                  tkn.initialize(this, source, i, maxPos - i);
                  TemplateTokenContainer bodyToken = (TemplateTokenContainer) tkn;
                  // parsing statement body
                  i = parse(bodyToken);
                  if (tokenContainer.isClosed()) {
                    return i;
                  }
                } else {
                  // initializing token
                  tokenContainer.addToken(tkn);
                  tkn.initialize(this, source, i, maxPos - i);
                  i = tkn.getEndPosition();
                }
                continue mainLoop;
                
              case '$':
                // illegal character
                break;
                
              default:
                if (Character.isWhitespace(c)) {
                  // illegal character
                  break;
                }
            }
          }
          break;
          
        case '$':
          if ((source.length > i + 1) && (source[i + 1] == '{')) {
            if (staticStart >= 0) {
              // adding static text
              StaticToken st = new StaticToken(this, source, staticStart, i - staticStart);
              tokenContainer.addToken(st);
              staticStart = -1;
            }
            if (expressionFilter != null) {
              int nextPos = expressionFilter.filterExpression(source, i, maxPos);
              if (nextPos > 0) {
                StaticToken st = new StaticToken(this, source, i, nextPos - i);
                tokenContainer.addToken(st);
                staticStart = -1;
                i = nextPos - 1;
                break;
              }
            }
            // parsing expression
            ExpressionTuple expr = parseExpr(source, i, maxPos - 1);
            //i = expr.getPosition() + expr.getLength();
            i += (expr.getRelativePosition() + expr.getLength());
            tokenContainer.addToken(new ExpressionToken(this, source, i, expr.getLength(), expr));
            break;
          }
          
        default:
          if (staticStart < 0) {
            staticStart = i;
          }
          break;
      }
    }
    if (staticStart >= 0) {
      // adding static text
      StaticToken st = new StaticToken(this, source, staticStart, maxPos - staticStart);
      tokenContainer.addToken(st);
      staticStart = -1;
    }

    return tokenContainer.getEndPosition();
  }
 
  /**
   * Executes the given template.
   * @param template The source code of the template to execute.
   * @param ctx The template context to use.
   * @return The template execution result.
   * @exception TemplateException If the an expression could not be
   *   successfuly resolved
   */
  public String execute(String template, TemplateContext ctx)
    throws TemplateException {
    
    try {
      TemplateRoot templateRoot = parse(template);
      if (templateRoot == null) {
        return null;
      }
      StringWriter writer = new StringWriter();
      execute(templateRoot, writer, ctx);
      return writer.toString();
    } catch (IOException ex) {
      // This should never happen!
      throw new RuntimeException(ex.getMessage());
    }
  }

  /**
   * Executes the given template.
   * @param template The source code of the template to execute.
   * @param writer The writer to output the result to.
   * @param ctx The template context to use.
   * @exception TemplateException If the an expression could not be
   *   successfuly resolved
   * @exception IOException If something went wrong while writing the 
   *   result to the given writer.
   */
  public void execute(String template, Writer writer, TemplateContext ctx)
    throws TemplateException, IOException {
    
    TemplateRoot templateRoot = parse(template);
    execute(templateRoot, writer, ctx);
  }
  
  /**
   * Executes the given template.
   * @param template The root element of the template parse result.
   * @param writer The writer to output the result to.
   * @param ctx The template context to use.
   * @exception TemplateException If the an expression could not be
   *   successfuly resolved
   * @exception IOException If something went wrong while writing the 
   *   result to the given writer.
   */
  public void execute(TemplateRoot template, Writer writer, TemplateContext ctx)
    throws TemplateException, IOException {
    
    template.execute(writer, ctx);
  }

  /**
   * @see java.lang.Object#clone()
   */
  public Object clone() {
    //return new DefaultTemplateInterpreter(exi, (Map) ((HashMap) tokenTypes).clone());
    DefaultTemplateInterpreter ti = null;
    try {
      ti = (DefaultTemplateInterpreter) super.clone();
    } catch (CloneNotSupportedException ex) {
      throw new RuntimeException("Failed to clone Operand instance: " + ex.getMessage(), ex);
    }
    return ti;
  }

}
