/*
 * Copyright (C) 2005 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.template;

import java.io.Writer;

import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.Operand;

/**
 * Name: DefineToken<br>
 * Description: Represents a _define_ statement.
 * 
 * @author lauerc
 */
public class DefineToken extends TemplateToken {
  
  /** The serial version UID. */
  private static final long serialVersionUID = 3256441421497250873L;
  
  /** The property name. */
  private String propertyName = null;
  /** The property value. */
  private Operand value = null;
  /** Tells whether this is a global property or not. */
  private boolean global = false;
  
  /**
   * Constructs a DefineToken instance with no specific attributes.
   */
  public DefineToken() {
  }

  /**
   * Gets the token name. 
   * @return The token name.
   */
  public String getTokenName() {
    return "define";
  }
  
  /**
   * Initializes the token.
   * @param interpreter The template interpreter.
   * @param source The source code.
   * @param position The start position.
   * @param length The length of the interresting area.
   * @exception TemplateException In case the initialization failed.
   */
  public void initialize(BaseTemplateInterpreter interpreter, char[] source, int position, int length)
    throws TemplateException {
    
    super.initialize(interpreter, source, position, length);
    int endPos = parseArguments();
    setLength(endPos - position);
  }

  /**
   * Sets an argument.
   * @param argument The argument to set.
   * @param idx The argument index.
   * @exception StatementException In case the given argument is not valid.
   */
  public void setArgument(Operand argument, int idx)
    throws StatementException {
    
    switch (idx) {
      case 0:
        this.propertyName = argument.getValue().toString();
        break;

      case 1:
        this.value = argument;
        break;
        
      case 2:
        this.global = "global".equals(argument.getValue());
        break;
        
      default:
        throw new StatementException("The log statement may take maximum three argument only.");
    }
  }

  /**
   * Excecutes the statement token.
   * @param writer The writer to write to.
   * @param ctx The template context.
   * @exception ExpressionException If the token could not be successfuly executed.
   */
  public void execute(Writer writer, TemplateContext ctx)
    throws ExpressionException {

    if (global) {
      ctx.setGlobalProperty(propertyName, interpreter.evalObjectExpr(value, ctx));
    } else {
      ctx.setProperty(propertyName, interpreter.evalObjectExpr(value, ctx));
    }
  }

  
  /**
   * @see java.lang.Object#toString()
   */
  public String toString() {
    return getIndention() + "_define_(" + getArgumentsAsString() + ")\n"; 
  }

  /**
   * Clones the token for caching reasons.
   * @param interpreter The interpreter to use.
   * @param parent The actual parent in the cloned object tree.
   * @return The clone.
   */
  protected TemplateToken cloneToken(BaseTemplateInterpreter interpreter, TemplateTokenContainer parent) {
    DefineToken clone = (DefineToken) clone();
    clone.interpreter = interpreter;
    clone.parent = parent;
    clone.value = (Operand) this.value.clone();
    return clone;
  }
  
  /**
   * @see java.lang.Object#clone()
   */
  public Object clone() {
    try {
      return super.clone();
    } catch (CloneNotSupportedException ex) {
      throw new RuntimeException("Failed to clone template token: " + ex.getMessage(), ex);
    }
  }

}
