/*
 * Copyright (C) 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.template;

import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.Operand;

/**
 * Name: ElseToken<br>
 * Description: Represents an _else_ statement.
 * 
 * @author lauerc
 */
public class ElseToken extends BaseConditionToken {

  /** The serial version UID. */
  private static final long serialVersionUID = 3258134673849659958L;

  /** The ancestor rules. */
  public static final TokenAncestor[] ancestorRules = new TokenAncestor[] {
    new TokenAncestor("endif")
  };

  /** The root condition token. */
  private BaseConditionToken rootCondition = null;
  
  /**
   * Constructs a ElseToken instance with no specific attributes.
   */
  public ElseToken() {
  }
  
  /**
   * Gets the token name. 
   * @return The token name.
   */
  public String getTokenName() {
    return "else";
  }
  
  /**
   * Gets the ancestor rules for this template.
   * @return An array of template ancestors.
   */
  public TokenAncestor[] getAncestorRules() {
    return ancestorRules;
  }

  /**
   * Initializes the token.
   * @param interpreter The template interpreter.
   * @param source The source code.
   * @param position The start position.
   * @param length The length of the interresting area.
   * @exception TemplateException In case the initialization failed.
   */
  public void initialize(BaseTemplateInterpreter interpreter, char[] source, int position, int length)
    throws TemplateException {
    
    super.initialize(interpreter, source, position, length);
    setBodyStart(position + 6);
    rootCondition = getPreviousCondition().getRootCondition();
  }

  /**
   * Sets an argument.
   * @param argument The argument to set.
   * @param idx The argument index.
   * @exception StatementException In case the given argument is not valid.
   */
  public void setArgument(Operand argument, int idx) throws StatementException {
    throw new StatementException("The else statement doesn't allow any argument.");
  }

  /**
   * Tells whether or not the condition is true.
   * @param ctx The template context.
   * @return true or false.
   * @exception ExpressionException If the token could not be successfuly executed.
   */
  public boolean isTrue(TemplateContext ctx) throws ExpressionException {
    return !rootCondition.isResolved();
  }

  /**
   * Gets the root condition.
   * @return The root condition of the current condition.
   */
  public BaseConditionToken getRootCondition() {
    return rootCondition;
  }

  /**
   * @see java.lang.Object#toString()
   */
  public String toString() {
    return getIndention() + "_else_\n" + super.toString();
  }

  /**
   * Clones the token for caching reasons.
   * @param interpreter The interpreter to use.
   * @param parent The actual parent in the cloned object tree.
   * @return The clone.
   */
  protected TemplateToken cloneToken(BaseTemplateInterpreter interpreter, TemplateTokenContainer parent) {
    ElseToken clone = null;
    try {
      clone = (ElseToken) super.clone();
    } catch (CloneNotSupportedException ex) {
      throw new RuntimeException("Failed to clone template token: " + ex.getMessage(), ex);
    }
    clone.interpreter = interpreter;
    clone.parent = parent;
    try {
      clone.rootCondition = clone.getPreviousCondition().getRootCondition();
    } catch (StatementException ex) {
      throw new RuntimeException("Failed to set root condition: " + ex.getMessage(), ex);
    }
    clone.cloneChildTokens(procedureRegistry, this);
    return clone;
  }

}
