/*
 * Copyright (C) 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.template;

import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.ExpressionTuple;
import org.ujac.util.exi.Operand;

/**
 * Name: ElseifToken<br>
 * Description: Represents an _elseif_ statement.
 * 
 * @author lauerc
 */
public class ElseifToken extends BaseConditionToken {

  /** The serial version UID. */
  private static final long serialVersionUID = 3257572793259669817L;

  /** The ancestor rules. */
  public static final TokenAncestor[] ancestorRules = new TokenAncestor[] {
    new TokenAncestor("elseif", true, true), 
    new TokenAncestor("else", true),
    new TokenAncestor("endif")
  };
  
  /** The condition. */
  private ExpressionTuple condition = null;
  /** The root condition token. */
  private BaseConditionToken rootCondition = null;

  /**
   * Constructs a ElseifToken instance with no specific attributes.
   */
  public ElseifToken() {
  }

  /**
   * Gets the token name. 
   * @return The token name.
   */
  public String getTokenName() {
    return "elseif";
  }
  
  /**
   * Gets the ancestor rules for this template.
   * @return An array of template ancestors.
   */
  public TokenAncestor[] getAncestorRules() {
    return ancestorRules;
  }

  /**
   * Initializes the token.
   * @param interpreter The template interpreter.
   * @param source The source code.
   * @param position The start position.
   * @param length The length of the interresting area.
   * @exception TemplateException In case the initialization failed.
   */
  public void initialize(BaseTemplateInterpreter interpreter, char[] source, int position, int length)
    throws TemplateException {
    
    super.initialize(interpreter, source, position, length);
    setBodyStart(parseArguments() + 1);
    rootCondition = getPreviousCondition().getRootCondition();
  }
  
  /**
   * Sets an argument.
   * @param argument The argument to set.
   * @param idx The argument index.
   */
  public void setArgument(Operand argument, int idx) {
    condition = (ExpressionTuple) argument;
  }


  /**
   * Tells whether or not the condition is true.
   * @param ctx The template context.
   * @return true or false.
   * @exception ExpressionException If the token could not be successfuly executed.
   */
  public boolean isTrue(TemplateContext ctx) throws ExpressionException {
    if (rootCondition.isResolved()) {
      return false;
    }
    
    boolean cond = interpreter.evalBooleanExpr(condition, ctx);
    rootCondition.setResolved(cond);
    return cond;
  }

  /**
   * Gets the root condition.
   * @return The root condition of the current condition.
   */
  public BaseConditionToken getRootCondition() {
    return rootCondition;
  }

  /**
   * @see java.lang.Object#toString()
   */
  public String toString() {
    return getIndention() + "_elseif_(" + getArgumentsAsString() + ")\n" + super.toString(); 
  }

  /**
   * @see org.ujac.util.template.TemplateToken#getArgumentsAsString()
   */
  protected String getArgumentsAsString() {
    return condition.toString();
  }

  /**
   * Clones the token for caching reasons.
   * @param interpreter The interpreter to use.
   * @param parent The actual parent in the cloned object tree.
   * @return The clone.
   */
  protected TemplateToken cloneToken(BaseTemplateInterpreter interpreter, TemplateTokenContainer parent) {
    ElseifToken clone = null;
    try {
      clone = (ElseifToken) super.clone();
    } catch (CloneNotSupportedException ex) {
      throw new RuntimeException("Failed to clone template token: " + ex.getMessage(), ex);
    }
    clone.interpreter = interpreter;
    clone.parent = parent;
    clone.condition = (ExpressionTuple) this.condition.clone();
    try {
      clone.rootCondition = clone.getPreviousCondition().getRootCondition();
    } catch (StatementException ex) {
      throw new RuntimeException("Failed to set root condition: " + ex.getMessage(), ex);
    }
    return clone;
  }

}
