/*
 * Copyright (C) 2005 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.template;

import java.io.IOException;
import java.io.Writer;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.Operand;
import org.ujac.util.exi.SequenceIndex;
import org.ujac.util.table.Table;

/**
 * Name: ForeachToken<br>
 * Description: Represents a _foreach_ statement.
 * 
 * @author lauerc
 */
public class ForeachToken extends TemplateTokenContainer {

  /** The serial version UID. */
  private static final long serialVersionUID = 3256999947785286963L;

  /** The ancestor rules. */
  public static final TokenAncestor[] ancestorRules = new TokenAncestor[] {
    new TokenAncestor("endfor")
  };

  /** The iterator method's attribute declaration. */
  private static final Class[] ITERATOR_ATTRIBUTES = {};
  /** The iterator method's parameter list. */
  private static final Object[] ITERATOR_PARAMETERS = {};

  /** The loop variable name. */
  private String loopVariable = null;
  /** The operand, holding the sequence. */
  private Operand sequence = null;
  /** 
   * The even style and odd style are used alternating as value of the 
   * property named by the concatenation of the loopVariable name and 'Style'.
   */
  private String evenStyle = null;
  /** 
   * The odd style and even style are used alternating as value of the 
   * property named by the concatenation of the loopVariable name and 'Style'.
   */
  private String oddStyle = null;
  
  /**
   * Constructs a ForeachToken instance with no specific attributes.
   */
  public ForeachToken() {
  }

  /**
   * Gets the token name. 
   * @return The token name.
   */
  public String getTokenName() {
    return "foreach";
  }
  
  /**
   * Gets the ancestor rules for this template.
   * @return An array of template ancestors.
   */
  public TokenAncestor[] getAncestorRules() {
    return ancestorRules;
  }

  /**
   * Initializes the token.
   * @param interpreter The template interpreter.
   * @param source The source code.
   * @param position The start position.
   * @param length The length of the interresting area.
   * @exception TemplateException In case the initialization failed.
   */
  public void initialize(BaseTemplateInterpreter interpreter, char[] source, int position, int length)
    throws TemplateException {
    
    super.initialize(interpreter, source, position, length);
    setBodyStart(parseArguments() + 1);
  }

  /**
   * Sets an argument.
   * @param argument The argument to set.
   * @param idx The argument index.
   * @exception StatementException In case the given argument is not valid.
   */
  public void setArgument(Operand argument, int idx) 
    throws StatementException {
    
    switch (idx) {
      case 0:
        loopVariable = argument.getValue().toString();
        break;

      case 1:
        sequence = argument;
        break;

      case 2:
        evenStyle = argument.getValue().toString();
        break;

      case 3:
        oddStyle = argument.getValue().toString();
        break;
        
      default:
        throw new StatementException("The log statement may take maximum four argument only.");
    }
  }

  /**
   * Excecutes the statement token.
   * @param writer The writer to write to.
   * @param ctx The template context.
   * @exception TemplateException If the token could not be successfuly executed.
   * @exception IOException In case an output failure occurred.
   */
  public void execute(Writer writer, TemplateContext ctx)
    throws TemplateException, IOException {

    Iterator iter = getSequenceIterator(ctx);
    if (iter == null) {
      return;
    }
    
    String loopIndexVariable = loopVariable + "Idx";
    String loopStyleVariable = loopVariable + "Style";
    SequenceIndex idx = new SequenceIndex(0);
    ctx.setProperty(loopIndexVariable, idx);
    while (iter.hasNext()) {
      Object loopValue = iter.next();
      ctx.setProperty(loopVariable, loopValue);
      if (idx.intValue() % 2 == 0) {
        ctx.setProperty(loopStyleVariable, evenStyle);
      } else {
        ctx.setProperty(loopStyleVariable, oddStyle);
      }
      int numTokens = getTokenCount();
      for (int i = 0; i < numTokens; i++) {
        TemplateToken subToken = getToken(i);
        subToken.execute(writer, ctx);
      }
      idx.increment();
    }
  }

  /**
   * Gets the iterator for the given 
   * @param ctx The template context.
   * @return The evaluated iterator.
   * @throws ExpressionException In case the expression evaluation failed.
   */
  private Iterator getSequenceIterator(TemplateContext ctx) throws ExpressionException {
    Object sequenceHolder = interpreter.evalObjectExpr(sequence, ctx);

    if (sequenceHolder == null) {
      return null;
    }

    if (sequenceHolder instanceof Collection) {
      return ((Collection) sequenceHolder).iterator();
    } else if (sequenceHolder instanceof Iterator) {
      return (Iterator) sequenceHolder;
    } else if (sequenceHolder instanceof Map) {
      return ((Map) sequenceHolder).keySet().iterator();
    } else if (sequenceHolder instanceof Table) {
      return ((Table) sequenceHolder).iterator();
    } else if (sequenceHolder instanceof Object[]) {
      List sequenceList = Arrays.asList((Object[]) sequenceHolder);
      return sequenceList.iterator();
    } else if (sequenceHolder instanceof String) {
      // sequence can be a string like 1-10 which means, we will iterate from 1 to 10 in steps of 1.
      StringTokenizer strTkn = new StringTokenizer(sequenceHolder.toString(), "-");
      if (strTkn.countTokens() == 2) {
        String strFrom = strTkn.nextToken();
        String strTo = strTkn.nextToken();
        
        int from = -1;
        try {
          from = Integer.parseInt(strFrom);
        } catch (NumberFormatException ex) {
          throw new StatementException("The _foreach_ sequence doesn't evaluate to a valid sequence.", ex);
        }

        int to = -1;
        try {
          to = Integer.parseInt(strTo);
        } catch (NumberFormatException ex) {
          throw new StatementException("The _foreach_ sequence doesn't evaluate to a valid sequence.", ex);
        }
        
        List sequence = new ArrayList();
        for (int i = from;  i <= to; i++) {
          sequence.add(new Integer(i));
        }
        return sequence.iterator();
      }
      // sequence can be a string like 1,2,3,4,5 which means, we will iterate over the list of comma separated values.
      strTkn = new StringTokenizer(sequenceHolder.toString(), ",");
      if (strTkn.countTokens() < 2) {
        throw new StatementException("The _foreach_ sequence doesn't evaluate to a valid sequence.");
      }
      List sequence = new ArrayList();
      while (strTkn.hasMoreTokens()) {
        sequence.add(strTkn.nextToken());
      }
      return sequence.iterator();
    } else {
      try {
        Method iteratorMethod = sequenceHolder.getClass().getMethod("iterator", ITERATOR_ATTRIBUTES);
        return (Iterator) iteratorMethod.invoke(sequenceHolder, ITERATOR_PARAMETERS);
      } catch (SecurityException ex) {
        throw new StatementException("The _foreach_ sequence doesn't evaluate to a valid sequence.", ex);
      } catch (NoSuchMethodException ex) {
        throw new StatementException("The _foreach_ sequence doesn't evaluate to a valid sequence.", ex);
      } catch (ClassCastException ex) {
        throw new StatementException("The _foreach_ sequence doesn't evaluate to a valid sequence.", ex);
      } catch (IllegalArgumentException ex) {
        throw new StatementException("The _foreach_ sequence doesn't evaluate to a valid sequence.", ex);
      } catch (IllegalAccessException ex) {
        throw new StatementException("The _foreach_ sequence doesn't evaluate to a valid sequence.", ex);
      } catch (InvocationTargetException ex) {
        throw new StatementException("The _foreach_ sequence doesn't evaluate to a valid sequence.", ex);
      }
    }
  }

  /**
   * @see java.lang.Object#toString()
   */
  public String toString() {
    return getIndention() + "_foreach_(" + getArgumentsAsString() + ")\n" + super.toString(); 
  }

  /**
   * @see org.ujac.util.template.TemplateToken#getArgumentsAsString()
   */
  protected String getArgumentsAsString() {
    return loopVariable + ", " + sequence.toString();
  }

  /**
   * Clones the token for caching reasons.
   * @param interpreter The interpreter to use.
   * @param parent The actual parent in the cloned object tree.
   * @return The clone.
   */
  protected TemplateToken cloneToken(BaseTemplateInterpreter interpreter, TemplateTokenContainer parent) {
    ForeachToken clone = (ForeachToken) clone();
    clone.interpreter = interpreter;
    clone.parent = parent;
    clone.sequence = (Operand) this.sequence.clone();
    clone.cloneChildTokens(procedureRegistry, this);
    return clone;
  }
  
  /**
   * @see java.lang.Object#clone()
   */
  public Object clone() {
    try {
      return super.clone();
    } catch (CloneNotSupportedException ex) {
      throw new RuntimeException("Failed to clone template token: " + ex.getMessage(), ex);
    }
  }

}
