/*
 * Copyright (C) 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.template;

import java.io.IOException;
import java.io.Writer;

import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.ExpressionTuple;
import org.ujac.util.exi.Operand;

/**
 * Name: IfToken<br>
 * Description: Represents an _if_ statement.
 * 
 * @author lauerc
 */
public class IfToken extends BaseConditionToken {

  /** The serial version UID. */
  private static final long serialVersionUID = 4050482317929690416L;

  /** The ancestor rules. */
  public static final TokenAncestor[] ancestorRules = new TokenAncestor[] {
    new TokenAncestor("elseif", true, true), 
    new TokenAncestor("else", true),
    new TokenAncestor("endif")
  };

  /** The condition. */
  private ExpressionTuple condition = null;
  /** Tells whether or not this condition or one of its ancestors has been resolved. */
  private boolean resolved = false;
  
  /**
   * Constructs a IfToken instance with no specific attributes.
   */
  public IfToken() {
  }
  
  /**
   * Gets the token name. 
   * @return The token name.
   */
  public String getTokenName() {
    return "if";
  }
  
  /**
   * Gets the ancestor rules for this template.
   * @return An array of template ancestors.
   */
  public TokenAncestor[] getAncestorRules() {
    return ancestorRules;
  }

  /**
   * Initializes the token.
   * @param interpreter The template interpreter.
   * @param source The source code.
   * @param position The start position.
   * @param length The length of the interresting area.
   * @exception TemplateException In case the initialization failed.
   */
  public void initialize(BaseTemplateInterpreter interpreter, char[] source, int position, int length)
    throws TemplateException {
    
    super.initialize(interpreter, source, position, length);
    setBodyStart(parseArguments() + 1);
  }

  /**
   * Sets an argument.
   * @param argument The argument to set.
   * @param idx The argument index.
   */
  public void setArgument(Operand argument, int idx) {
    condition = (ExpressionTuple) argument;
  }

  /**
   * Excecutes the statement token.
   * @param writer The writer to write to.
   * @param ctx The template context.
   * @exception TemplateException If the token could not be successfuly executed.
   * @exception IOException In case an output failure occurred.
   */
  public void execute(Writer writer, TemplateContext ctx)
    throws TemplateException, IOException {

    this.resolved = false;
    super.execute(writer, ctx);
  }

  /**
   * Tells whether or not the condition is true.
   * @param ctx The template context.
   * @return true or false.
   * @exception ExpressionException If the token could not be successfuly executed.
   */
  public boolean isTrue(TemplateContext ctx) throws ExpressionException {
    boolean cond = interpreter.evalBooleanExpr(condition, ctx);
    setResolved(cond);
    return cond;
  }

  /**
   * @see java.lang.Object#toString()
   */
  public String toString() {
    return getIndention() + "_if_(" + getArgumentsAsString() + ")\n" + super.toString();
  }

  /**
   * @see org.ujac.util.template.TemplateToken#getArgumentsAsString()
   */
  protected String getArgumentsAsString() {
    return condition.toString();
  }
  
  /**
   * Tells whether or not the condition bas been resolved so far.
   * @return true or false.
   */
  public boolean isResolved() {
    return resolved;
  }
  /**
   * Tells whether or not the condition bas been resolved so far.
   * @param resolved The value to seet.
   */
  public void setResolved(boolean resolved) {
    this.resolved = resolved;
  }

  /**
   * Gets the root condition.
   * @return a reference to this instance.
   */
  public BaseConditionToken getRootCondition() {
    return this;
  }
  
  /**
   * Clones the token for caching reasons.
   * @param interpreter The interpreter to use.
   * @param parent The actual parent in the cloned object tree.
   * @return The clone.
   */
  protected TemplateToken cloneToken(BaseTemplateInterpreter interpreter, TemplateTokenContainer parent) {
    IfToken clone = null;
    try {
      clone = (IfToken) super.clone();
    } catch (CloneNotSupportedException ex) {
      throw new RuntimeException("Failed to clone template token: " + ex.getMessage(), ex);
    }
    clone.interpreter = interpreter;
    clone.parent = parent;
    clone.condition = (ExpressionTuple) this.condition.clone();
    clone.resolved = false;
    clone.cloneChildTokens(procedureRegistry, this);
    return clone;
  }
  
}
