/*
 * Copyright (C) 2005 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.template;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.ujac.util.BeanUtils;

import org.ujac.util.exi.Operand;

/**
 * Name: AssertToken<br>
 * Description: Represents a _log_ statement.
 * 
 * @author lauerc
 */
public class LogToken extends TemplateToken {
  
  /** The serial version UID. */
  private static final long serialVersionUID = 3688503294611895861L;
  
  /** The log levels. */
  private static final Map LOG_LEVELS = new HashMap();
  /** The debug level. */
  private static final int DEBUG = 1;
  /** The info level. */
  private static final int INFO = 2;
  /** The warn level. */
  private static final int WARN = 3;
  /** The error level. */
  private static final int ERROR = 4;
  /** The fatal level. */
  private static final int FATAL = 5;
  static {
    LOG_LEVELS.put("debug", new Integer(DEBUG));
    LOG_LEVELS.put("DEBUG", new Integer(DEBUG));
    LOG_LEVELS.put("info", new Integer(INFO));
    LOG_LEVELS.put("INFO", new Integer(INFO));
    LOG_LEVELS.put("warn", new Integer(WARN));
    LOG_LEVELS.put("WARN", new Integer(WARN));
    LOG_LEVELS.put("error", new Integer(ERROR));
    LOG_LEVELS.put("ERROR", new Integer(ERROR));
    LOG_LEVELS.put("fatal", new Integer(FATAL));
    LOG_LEVELS.put("FATAL", new Integer(FATAL));
  }
  
  /** The log level. */
  private Operand logLevel = null;
  /** The log message. */
  private Operand logMessage = null;
  /** The log message template. */
  private TemplateRoot logMessageTemplate = null;
  
  /**
   * Constructs a AssertToken instance with no specific attributes.
   */
  public LogToken() {
  }
  
  /**
   * Gets the token name. 
   * @return The token name.
   */
  public String getTokenName() {
    return "log";
  }
  
  /**
   * Initializes the token.
   * @param interpreter The template interpreter.
   * @param source The source code.
   * @param position The start position.
   * @param length The length of the interresting area.
   * @exception TemplateException In case the initialization failed.
   */
  public void initialize(BaseTemplateInterpreter interpreter, char[] source, int position, int length)
    throws TemplateException {
    
    super.initialize(interpreter, source, position, length);
    int endPos = parseArguments();
    setLength(endPos - position);
    try {
      this.logMessageTemplate = interpreter.parse(BeanUtils.toString(this.logMessage.getValue()));
    } catch (IOException ex) {
      throw new StatementException("_log_ parameter parsing failed: " + ex.getMessage(), ex);
    }
  }

  /**
   * Sets an argument.
   * @param argument The argument to set.
   * @param idx The argument index.
   * @exception StatementException In case the given argument is not valid.
   */
  public void setArgument(Operand argument, int idx) throws StatementException {
    switch (idx) {
      case 0:
        this.logLevel = argument;
        break;

      case 1:
        this.logMessage = argument;
        break;

      default:
        throw new StatementException("The log statement may take two argument only.");
    }
  }

  /**
   * Excecutes the statement token.
   * @param writer The writer to write to.
   * @param ctx The template context.
   * @exception TemplateException If the token could not be successfuly executed.
   * @exception IOException In case an output failure occurred.
   */
  public void execute(Writer writer, TemplateContext ctx)
    throws TemplateException, IOException {

    Log log = LogFactory.getLog("TemplateInterpreter");
    String logLevel = BeanUtils.toString(this.logLevel.getValue());
    StringWriter sw = new StringWriter();
    interpreter.execute(logMessageTemplate, sw, ctx);
    String logMessage = sw.toString();
    Integer levelId = (Integer) LOG_LEVELS.get(logLevel);
    switch (levelId.intValue()) {
      case DEBUG:
        log.debug(logMessage);
        break;
      case INFO:
        log.info(logMessage);
        break;
      case WARN:
        log.warn(logMessage);
        break;
      case ERROR:
        log.error(logMessage);
        break;
      case FATAL:
        log.fatal(logMessage);
        break;
      default:
        log.info(logMessage);
        break;
    }
  }
  
  /**
   * @see java.lang.Object#toString()
   */
  public String toString() {
    return getIndention() + "_log_(" + getArgumentsAsString() + ")\n"; 
  }

  /**
   * Clones the token for caching reasons.
   * @param interpreter The interpreter to use.
   * @param parent The actual parent in the cloned object tree.
   * @return The clone.
   */
  protected TemplateToken cloneToken(BaseTemplateInterpreter interpreter, TemplateTokenContainer parent) {
    LogToken clone = (LogToken) clone();
    clone.interpreter = interpreter;
    clone.parent = parent;
    clone.logLevel = (Operand) this.logLevel.clone();
    clone.logMessage = (Operand) this.logMessage.clone();
    clone.logMessageTemplate = (TemplateRoot) clone.logMessageTemplate.cloneToken(interpreter, parent);
    return clone;
  }
  
  /**
   * @see java.lang.Object#clone()
   */
  public Object clone() {
    try {
      return super.clone();
    } catch (CloneNotSupportedException ex) {
      throw new RuntimeException("Failed to clone template token: " + ex.getMessage(), ex);
    }
  }

}
