/*
 * Copyright (C) 2005 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.template;

import java.io.IOException;
import java.io.Writer;

import java.util.ArrayList;
import java.util.List;

import org.ujac.util.CollectionUtils;
import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.Operand;

/**
 * Name: ProcedureToken<br>
 * Description: Represents a _procedure_ statement.
 * 
 * @author lauerc
 */
public class ProcedureToken extends TemplateTokenContainer {

  /** The serial version UID. */
  private static final long serialVersionUID = 3257291348297135926L;

  /** The ancestor rules. */
  public static final TokenAncestor[] ancestorRules = new TokenAncestor[] {
    new TokenAncestor("endproc")
  };

  /** The name of the procedure. */
  private String name = null;
  /** The argument list for the procedure. */
  private List arguments = null;

  /**
   * Constructs a ProcedureToken instance with no specific attributes.
   */
  public ProcedureToken() {
  }
  
  /**
   * Gets the token name. 
   * @return The token name.
   */
  public String getTokenName() {
    return "procedure";
  }

  /**
   * Initializes the token.
   * @param interpreter The template interpreter.
   * @param source The source code.
   * @param position The start position.
   * @param length The length of the interresting area.
   * @exception TemplateException In case the initialization failed.
   */
  public void initialize(BaseTemplateInterpreter interpreter, char[] source, int position, int length)
    throws TemplateException {
    
    super.initialize(interpreter, source, position, length);
    
    String tokenName = getTokenName();
    int curPos = position + tokenName.length() + 2;
    int endPos = position + length;
    int nameStart = -1;
    int nameEnd = -1;

   parseLoop:
    for (int i = curPos; i <= endPos; i++) {
      char c = source[i];
      if (c == '(') {
        // this is the end!
        if (nameEnd < 0) {
          nameEnd = i - 1;
        }
        break parseLoop;
      }        

      if (Character.isWhitespace(c)) {
        if ((nameStart > 0) && (nameEnd < 0)) {
          nameEnd = i - 1;
        }
        continue;
      }
      if (nameStart < 0) {
        nameStart = i;
        continue;
      }
    }
    this.name = new String(source, nameStart, nameEnd - nameStart + 1);
    setBodyStart(parseArguments(nameEnd + 1) + 1);
    procedureRegistry.put(getName(), this);
  }

  /**
   * Getter method for the the property name.
   * @return The current value of property name.
   */
  public String getName() {
    return name;
  }
  
  /**
   * Gets the ancestor rules for this template.
   * @return An array of template ancestors.
   */
  public TokenAncestor[] getAncestorRules() {
    return ancestorRules;
  }

  /**
   * Checks whether this token is valid for the given container.
   * @param container The container that wants to add the token.
   * @return true if the token may be added to the container, else false.
   */
  public boolean checkContainer(TemplateTokenContainer container) {
    this.procedureRegistry = container.getProcedureRegistry();
    return false;
  }

  /**
   * Sets an argument.
   * @param argument The argument to set.
   * @param idx The argument index.
   */
  public void setArgument(Operand argument, int idx) {
    if (arguments == null) {
      arguments = new ArrayList();
    }
    arguments.add(argument.getValue());
  }

  /**
   * Getter method for the the property arguments.
   * @return The current value of property arguments.
   */
  public List getArguments() {
    return arguments;
  }

  /**
   * Excecutes the statement token.
   * @param writer The writer to write to.
   * @param ctx The template context.
   * @exception ExpressionException If the token could not be successfuly executed.
   * @exception IOException In case an output failure occurred.
   */
  public void execute(Writer writer, TemplateContext ctx)
    throws ExpressionException, IOException {
    
    throw new ExpressionException("Use method execute(List, Writer, ExpressionContext) instead!");
  }

  /**
   * Calls the procedure. 
   * @param arguments The arguments to use for the procedure call.
   * @param writer The writer, to which to write the result.
   * @param ctx The template context.
   * @exception IOException In case an I/O problem occurred.
   * @exception TemplateException If something went wrong.
   */
  protected void execute(List arguments, Writer writer, TemplateContext ctx)
    throws IOException, TemplateException {
    
    // cloning parameter list
    TemplateContext procedureCtx = new TemplateContext(ctx);
    
    // setting the procedure arguments
    int numArguments = arguments.size();
    List argumentNames = getArguments();
    if (numArguments != argumentNames.size()) {
      throw new ExpressionException("Invalid number of argument list given for call of procedure " + 
                                    this + ".");
    }
    for (int i = 0; i < numArguments; i++) {
      procedureCtx.setProperty(argumentNames.get(i), arguments.get(i));
    }
    
    // executing the procedure code
    int numTokens = getTokenCount();
    for (int i = 0; i < numTokens; i++) {
      TemplateToken subToken = getToken(i);
      subToken.execute(writer, procedureCtx);
    }
  }

  /**
   * @see java.lang.Object#toString()
   */
  public String toString() {
    return getIndention() + "_procedure_ " + getName() + "(" + getArgumentsAsString() + ")\n" + super.toString();
  }

  /**
   * Gets the argument list as string.
   * @return The arguements as a comma separated text.
   */
  protected String getArgumentsAsString() {
    StringBuffer ab = new StringBuffer();
    int numArguments = arguments.size();
    List argumentNames = getArguments();
    for (int i = 0; i < numArguments; i++) {
      if (i > 0) {
        ab.append(", ");
      }
      ab.append(argumentNames.get(i));
    }
    return ab.toString();
  }

  /**
   * Clones the token for caching reasons.
   * @param interpreter The interpreter to use.
   * @param parent The actual parent in the cloned object tree.
   * @return The clone.
   */
  protected TemplateToken cloneToken(BaseTemplateInterpreter interpreter, TemplateTokenContainer parent) {
    ProcedureToken clone = (ProcedureToken) clone();
    clone.interpreter = interpreter;
    clone.parent = parent;
    int numOperands = CollectionUtils.getSize(clone.arguments);
    for (int i = 0; i < numOperands; i++) {
      clone.arguments.set(i, ((Operand) clone.arguments.get(i)).clone());
    }
    clone.cloneChildTokens(procedureRegistry, this);
    clone.procedureRegistry.put(clone.getName(), this);
    return clone;
  }
  
  /**
   * @see java.lang.Object#clone()
   */
  public Object clone() {
    try {
      return super.clone();
    } catch (CloneNotSupportedException ex) {
      throw new RuntimeException("Failed to clone template token: " + ex.getMessage(), ex);
    }
  }
  
}
