/*
 * Copyright (C) 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.template;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.ujac.util.CollectionUtils;
import org.ujac.util.exi.ExpressionContext;
import org.ujac.util.text.FormatHelper;

/**
 * Name: TemplateContext<br>
 * Description: A class holding all relevant data, needed to execute templates.
 * 
 * @author lauerc
 */
public class TemplateContext extends ExpressionContext {

  /** The serial version UID. */
  private static final long serialVersionUID = 3257285816496567609L;
  
  /** The global properties. */
  private Map globalProperties = null;
  
  /**
   * Constructs a TemplateContext instance with specific attributes.
   * @param properties The properties to use for expression evaluation. 
   */
  public TemplateContext(Map properties) {
    super(properties);
    globalProperties = new HashMap();
  }

  /**
   * Constructs a TemplateContext instance with specific attributes.
   * @param properties The properties to use for expression evaluation. 
   * @param formatHelper The format helper to use. 
   */
  public TemplateContext(Map properties, FormatHelper formatHelper) {
    super(properties, formatHelper);
    globalProperties = new HashMap();
  }

  /**
   * Constructs a ExpressionContext instance with specific attributes.
   * @param properties The properties to use for expression evaluation. 
   * @param bean The bean used to retrieve parameter values, if the
   *   parameter value didn't exist in the localData map.
   * @param formatHelper The format helper to use. 
   */
  public TemplateContext(Map properties, Object bean, FormatHelper formatHelper) {
    super(properties, bean, formatHelper);
    globalProperties = new HashMap();
  }

  /**
   * Constructs a TemplateContext instance with specific attributes.
   * @param ctx The context to clone. 
   */
  public TemplateContext(TemplateContext ctx) {
    super(ctx);
    this.globalProperties = ctx.globalProperties;
  }

  /**
   * Gets a property from the property map.
   * @param name The property name.
   * @return The value of the desired property or null.
   */
  public Object getProperty(Object name) {
    Object value = globalProperties.get(name);
    if (value != null) {
      return value;
    }
    return super.getProperty(name);
  }
  /**
   * Setter method for the the property localData.
   * @param name The property name.
   * @param value The value to set.
   * @return The old value of the property to set.
   */
  public Object setProperty(Object name, Object value) {
    if (globalProperties.containsKey(name)) {
      return globalProperties.put(name, value);
    }
    return super.setProperty(name, value);
  }
  /**
   * Setter method for the the property localData.
   * @param name The property name.
   * @param value The value to set.
   */
  public void setGlobalProperty(Object name, Object value) {
    this.globalProperties.put(name, value);
  }

  /**
   * @see java.util.Map#size()
   */
  public int size() {
    return super.size() + CollectionUtils.getSize(globalProperties);
  }

  /**
   * @see java.util.Map#isEmpty()
   */
  public boolean isEmpty() {
    return super.isEmpty() && CollectionUtils.isEmpty(globalProperties);
  }

  /**
   * @see java.util.Map#containsKey(java.lang.Object)
   */
  public boolean containsKey(Object key) {
    return super.containsKey(key) || CollectionUtils.containsKey(globalProperties, key);
  }

  /**
   * @see java.util.Map#containsValue(java.lang.Object)
   */
  public boolean containsValue(Object value) {
    return super.containsValue(value) || CollectionUtils.containsValue(globalProperties, value);
  }

  /**
   * @see java.util.Map#get(java.lang.Object)
   */
  public Object get(Object key) {
    return getProperty(key);
  }

  /**
   * @see java.util.Map#put(java.lang.Object, java.lang.Object)
   */
  public Object put(Object key, Object value) {
    return setProperty(key, value);
  }
  
  /**
   * @see java.util.Map#remove(java.lang.Object)
   */
  public Object remove(Object key) {
    Object removed = globalProperties.remove(key);
    if (removed != null) {
      return removed;
    }
    return super.remove(key);
  }

  /**
   * @see java.util.Map#clear()
   */
  public void clear() {
    super.clear();
    CollectionUtils.clear(globalProperties);
  }

  /**
   * @see java.util.Map#keySet()
   */
  public Set keySet() {
    if (globalProperties == null) {
      return super.keySet();
    }
    Set keySet = new HashSet(super.keySet());
    keySet.addAll(globalProperties.keySet());
    return keySet;
  }

  /**
   * @see java.util.Map#values()
   */
  public Collection values() {
    if (globalProperties == null) {
      return super.values();
    }
    List values = new ArrayList(super.values());
    values.addAll(globalProperties.values());
    return values;
  }

  /**
   * @see java.util.Map#entrySet()
   */
  public Set entrySet() {
    if (globalProperties == null) {
      return super.entrySet();
    }
    Set entrySet = new HashSet(super.entrySet());
    entrySet.addAll(globalProperties.entrySet());
    return entrySet;
  }
  
  
  /**
   * Writes the object's data to the given stream.
   * @param s The stream to write to
   * @exception IOException In case the data output failed.
   */
  private void writeObject(ObjectOutputStream s) throws IOException {
    super.writeData(s);
    s.writeObject(globalProperties);
  }
  /**
   * Reads the object's data from the given stream.
   * @param s The stream to read from.
   * @exception IOException In case the data reading failed.
   * @exception ClassNotFoundException In case the class to deserialize could not be found
   */
  private void readObject(ObjectInputStream s) throws IOException, ClassNotFoundException  {
    super.readData(s);
    globalProperties = (Map) s.readObject();
  }

}
