/*
 * Copyright (C) 2005 - 2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.template;

import java.io.IOException;
import java.io.Writer;

import java.sql.Time;
import java.sql.Timestamp;

import java.util.Date;

import org.ujac.util.exi.ExpressionException;

/**
 * Name: TemplateInterpreter<br>
 * Description: Description of the class.
 * 
 * @author lauerc
 */
public interface TemplateInterpreter {

  /**
   * Parses the given template source code.
   * @param source The source to parse.
   * @return The parsed statement.
   * @exception TemplateException If the expression could not be
   *   successfuly resolved
   * @exception IOException In case a I/O problem occured while writing the
   *   expression result.
   */
  TemplateRoot parse(String source) 
    throws TemplateException, IOException;
 
  /**
   * Executes the given template.
   * @param template The source code of the template to execute.
   * @param ctx The template context to use.
   * @return The template execution result.
   * @exception TemplateException If the an expression could not be
   *   successfuly resolved
   */
  String execute(String template, TemplateContext ctx)
    throws TemplateException;

  /**
   * Executes the given template.
   * @param template The source code of the template to execute.
   * @param writer The writer to output the result to.
   * @param ctx The template context to use.
   * @exception TemplateException If the an expression could not be
   *   successfuly resolved
   * @exception IOException If something went wrong while writing the 
   *   result to the given writer.
   */
  void execute(String template, Writer writer, TemplateContext ctx)
    throws TemplateException, IOException;
  
  /**
   * Executes the given template.
   * @param template The root element of the template parse result.
   * @param writer The writer to output the result to.
   * @param ctx The template context to use.
   * @exception TemplateException If the an expression could not be
   *   successfuly resolved
   * @exception IOException If something went wrong while writing the 
   *   result to the given writer.
   */
  void execute(TemplateRoot template, Writer writer, TemplateContext ctx)
    throws TemplateException, IOException;
  
  /**
   * Evaluates an expression as a boolean value.
   * @param expression The expression to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  boolean evalBooleanExpr(String expression, TemplateContext ctx)
    throws ExpressionException;

  /**
   * Evaluates an expression as an int value.
   * @param expression The expression to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  int evalIntExpr(String expression, TemplateContext ctx)
    throws ExpressionException;

  /**
   * Evaluates an expression as a long value.
   * @param expression The expression to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  long evalLongExpr(String expression, TemplateContext ctx)
    throws ExpressionException;

  /**
   * Evaluates an expression as a float value.
   * @param expression The expression to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  float evalFloatExpr(String expression, TemplateContext ctx)
    throws ExpressionException;

  /**
   * Evaluates an expression as a double value.
   * @param expression The expression to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  double evalDoubleExpr(String expression, TemplateContext ctx)
    throws ExpressionException;

  /**
   * Evaluates an expression as a Date value.
   * @param expression The expression to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  Date evalDateExpr(String expression, TemplateContext ctx)
    throws ExpressionException;

  /**
   * Evaluates an expression as a Time value.
   * @param expression The expression to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  Time evalTimeExpr(String expression, TemplateContext ctx)
    throws ExpressionException;

  /**
   * Evaluates an expression as a Timestamp value.
   * @param expression The expression to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  Timestamp evalTimestampExpr(String expression, TemplateContext ctx)
    throws ExpressionException;

  /**
   * Evaluates an expression as a String value.
   * @param expression The expression to evaluate.
   * @param ctx The template context.
   * @return The evaluated boolean.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  String evalStringExpr(String expression, TemplateContext ctx)
    throws ExpressionException;

  /**
   * Evaluates an expression as an Object.
   * @param expression The expression to evaluate.
   * @param ctx The template context.
   * @return The evaluated object.
   * @exception ExpressionException If the expression could not be successfuly resolved
   */
  Object evalObjectExpr(String expression, TemplateContext ctx)
    throws ExpressionException;

  /**
   * Getter method for the the property expressionFilter.
   * @return The current value of property expressionFilter.
   */
  public ExpressionFilter getExpressionFilter();
  /**
   * Setter method for the the property expressionFilter.
   * @param expressionFilter The value to set for the property expressionFilter.
   */
  public void setExpressionFilter(ExpressionFilter expressionFilter);

}
