/*
 * Copyright (C) 2005 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.template;

import java.io.IOException;
import java.io.Serializable;
import java.io.Writer;

import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.Operand;

/**
 * Name: TemplateToken<br>
 * Description: A base class for template tokens.
 * 
 * @author lauerc
 */
public abstract class TemplateToken implements Serializable, Cloneable {

  /** The template interpreter. */
  protected BaseTemplateInterpreter interpreter = null;
  /** The parent token. */
  protected TemplateTokenContainer parent = null;
  /** The statement level. */
  private int level = 0;
  /** The source code. */
  protected char[] source = null;
  /** The position of the statement token in the expression. */
  protected int position;
  /** The length of the statement token. */
  private int length;
  /** The next legal ancestor idx. */
  protected int nextLegalAncestor = 0;
  
  /**
   * Constructs a TemplateToken instance with no specific attributes.
   */
  public TemplateToken() {
  }

  /**
   * Constructs a TemplateToken instance with specific attributes.
   * @param source The source code.
   * @param position The statement's position in the source code.
   * @param length The statement length.
   */
  public TemplateToken(char[] source, int position, int length) {
    this.source = source;
    this.position = position;
    this.length = length;
  }

  /**
   * Gets the token name. 
   * @return The token name.
   */
  public String getTokenName() {
    return null;
  }
  
  /**
   * Gets the ancestor rules for this template.
   * @return An array of template ancestors.
   */
  public TokenAncestor[] getAncestorRules() {
    return null;
  }

  /**
   * Checks whether this token is valid for the given container.
   * @param container The container that wants to add the token.
   * @return true if the token may be added to the container, else false.
   */
  public boolean checkContainer(TemplateTokenContainer container) {
    return true;
  }
  
  /**
   * Getter method for the the property parent.
   * @return The current value of property parent.
   */
  public TemplateTokenContainer getParent() {
    return parent;
  }
  /**
   * Setter method for the the property parent.
   * @param parent The value to set for the property parent.
   */
  public void setParent(TemplateTokenContainer parent) {
    this.parent = parent;
  }


  /**
   * Getter method for the the property level.
   * @return The current value of property level.
   */
  public int getLevel() {
    return level;
  }
  /**
   * Setter method for the the property level.
   * @param level The value to set for the property level.
   */
  public void setLevel(int level) {
    this.level = level;
  }

  /**
   * Tells whether or this token has a body. 
   * @return true if the token has a body, else false.
   */
  public boolean isBodyToken() {
    return false;
  }
  
  /**
   * Tells whether or the given token is a legal ancestor.
   * @param token The token to test.
   * @return true if the token is a legal ancestor, else false.
   */
  public boolean isLegalAncestor(String token) {
    TokenAncestor[] ancestorRules = getAncestorRules();
    if (ancestorRules == null) {
      return false;
    }
    for (int i = nextLegalAncestor; i < ancestorRules.length; i++) {
      TokenAncestor ancestor = ancestorRules[i];
      if (ancestor.getToken().equals(token)) {
        return true;
      }
    }
    return false;
  }
  
  /**
   * Tells whether or the given token is a legal ancestor.
   * @param token The token to test.
   * @return true if the token is a legal ancestor, else false.
   */
  public boolean isLegalAncestor(TemplateToken token) {
    return isLegalAncestor(token.getTokenName());
  }
  
  /**
   * Parses the arguments of the token.
   * @return The position after the argument list.
   * @exception ExpressionException In case the argument parsing failed.
   */
  public int parseArguments() throws ExpressionException {
    String tokenName = getTokenName();
    if (tokenName == null) {
      return position;
    }
    
    int curPos = position + tokenName.length() + 2;
    return parseArguments(curPos);
  }

  /**
   * Parses the arguments of the token.
   * @param offset The position fron which to parse the source.
   * @return The position after the argument list.
   * @exception ExpressionException In case the argument parsing failed.
   */
  protected int parseArguments(int offset) throws ExpressionException {

    int argIdx = 0;
    char[] source = this.source;
    int endPos = source.length - 1;

    int parenthesisLevel = 0;
   parseLoop:
    for (int i = offset; i <= endPos; i++) {
      char c = source[i];
      if (c == '(') {
        ++parenthesisLevel;
      } else if (c == ')') {
        --parenthesisLevel;
        if (parenthesisLevel == 0) {
          endPos = i - 1;
          break parseLoop;
        }
      }
    }

    if (parenthesisLevel != 0) {
      throw new StatementException("Syntax error at position " + offset + ": unbalanced parethesis.");
    }
    
    int curPos = offset + 1;
    for (int i = curPos; i <= endPos; i++) {
      char c = source[i];
      if (c == ',' || Character.isWhitespace(c)) {
        // ignoring comma as optional parameter separator
        continue;
      }
      // parsing constant
      Operand operand = parseConstant(i, endPos, argIdx);
      ++argIdx;
      i += operand.getRelativePosition() + operand.getLength();
    }
    return endPos + 1;
  }

  /**
   * Parses a constant and adds the result to the argument list.
   * @param argStart The statement's position in the source code.
   * @param argEnd The end position.
   * @param argIdx The argument index.
   * @return The resulting constant operand.
   * @exception ExpressionException In case the parsing failed.
   */
  private Operand parseConstant(int argStart, int argEnd, int argIdx) throws ExpressionException {
    Operand operand = interpreter.exi.parseOperand(source, argStart, argEnd, true);
    setArgument(operand, argIdx);
    return operand;
    
    /*
    boolean floatingPoint = false;
    boolean legalDigit = false;
  parseLoop:
    for (int i = argStart; i < argEnd; i++) {
      char c = source[i];
      switch (c) {
        case '-':
          if (i != argStart) {
            legalDigit = false;
            break parseLoop;
          }
          break;

        case '.':
          if (legalDigit) {
            if (!floatingPoint) {
              floatingPoint = true;
              break;
            }
            throw new StatementException("Syntax error at position " + i + ": invalid numeric constant.");
          }
          break;
          
        default:
          if (Character.isDigit(c)) {
            legalDigit = true;
          } else {
            legalDigit = false;
            break parseLoop;
          }
      }
    }
    
    ConstantOperand constant = new ConstantOperand(source, argStart, argEnd - argStart);
    String constantText = new String(source, argStart, argEnd - argStart);
    if (!legalDigit) {
      constant.setObject(constantText);
    } else {
      try {
        if (floatingPoint) {
          constant.setObject(new Double(Double.parseDouble(constantText)));
        } else {
          constant.setObject(new Integer(Integer.parseInt(constantText)));
        }
      } catch (NumberFormatException ex) {
        throw new StatementException("Syntax error at position " + argStart + ": invalid numeric constant.");
      }
    }
    setArgument(constant, argIdx);
    return constant;
    */
  }
  
  /**
   * Initializes the token.
   * @param interpreter The template interpreter.
   * @param source The source code.
   * @param position The start position.
   * @param length The length of the interresting area.
   * @exception TemplateException In case the initialization failed.
   */
  public void initialize(BaseTemplateInterpreter interpreter, char[] source, int position, int length)
    throws TemplateException {
    
    this.interpreter = interpreter;
    this.source = source;
    this.position = position;
    this.length = length;
  }

  /**
   * Gets the source code.
   * @return The source code.
   */
  public char[] getSource() {
    return source;
  }
  
  /**
   * Gets the length of the statement token.
   * @return The length of the statement token.
   */
  public int getLength() {
    return length;
  }
  /**
   * Setter method for the the property length.
   * @param length The value to set for the property length.
   */
  protected void setLength(int length) {
    this.length = length;
  }

  /**
   * Gets the position of the statement token in the expression.
   * @return The position of the statement token.
   */
  public int getPosition() {
    return position;
  }

  /**
   * Gets the end position of the statement token in the expression.
   * @return The end position of the statement token.
   */
  public int getEndPosition() {
    return position + length;
  }
  
  /**
   * Excecutes the statement token.
   * @param writer The writer to write to.
   * @param ctx The template context.
   * @exception TemplateException If the token could not be successfuly executed.
   * @exception IOException In case an output failure occurred.
   */
  public abstract void execute(Writer writer, TemplateContext ctx)
    throws TemplateException, IOException;
    
  /**
   * Sets an argument.
   * @param argument The argument to set.
   * @param idx The argument index.
   * @exception StatementException In case the given argument is not valid.
   */
  public abstract void setArgument(Operand argument, int idx) throws StatementException;
  
  /**
   * Gets the indention for prettier output.
   * @return The indention text.
   */
  protected String getIndention() {
    if (level == 0) {
      return "";
    }
    StringBuffer ib = new StringBuffer(level);
    for (int i = 0; i < level; i++) {
      ib.append('\t');
    }
    return ib.toString();
  }
  
  /**
   * Gets the argument list as string.
   * @return The arguements as a comma separated text.
   */
  protected String getArgumentsAsString() {
    return "";
  }

  /**
   * Clones the token for caching reasons.
   * @param interpreter The interpreter to use.
   * @param parent The actual parent in the cloned object tree.
   * @return The clone.
   */
  protected abstract TemplateToken cloneToken(BaseTemplateInterpreter interpreter, TemplateTokenContainer parent);

}
