/*
 * Copyright (C) 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.template;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.ujac.util.CollectionUtils;

/**
 * Name: TemplateTokenContainer<br>
 * Description: A base class for statement token containers.
 * 
 * @author lauerc
 */
public abstract class TemplateTokenContainer extends TemplateToken {

  /** The procedure registry. */
  protected Map procedureRegistry = null;

  /** The list of nested tokens. */
  private List tokens = new ArrayList();
  /** The start index of the container's body source. */
  private int bodyStart = -1;
  /** The end index of the container's body source. */
  private int bodyEnd = -1;
  /** Tells whether or not the container has been closed. */
  private boolean closed = false;
  
  /**
   * Constructs a TemplateTokenContainer instance with no specific attributes.
   */
  public TemplateTokenContainer() {
  }

  /**
   * Tells whether or not this token has a body. 
   * @return true if the token has a body, else false.
   */
  public boolean isBodyToken() {
    return true;
  }

  /**
   * Tells whether or not the container has been closed. 
   * @return true if the the container has been closed, else false.
   */
  public boolean isClosed() {
    return closed;
  }
  
  /**
   * Closes the statement token container.
   * @param bodyEnd The body end to set.
   * @param statementEnd The statment end to set.
   */
  public void close(int bodyEnd, int statementEnd) {
    this.closed = true;
    this.bodyEnd = bodyEnd;
    setLength(statementEnd - position + 1);
  }

  /**
   * Gets the current number of tokens.
   * @return The token list size.
   */
  public final int getTokenCount() {
    return tokens.size();
  }

  /**
   * Gets a token from the token list.
   * @param idx The token index.
   * @return The according element from the token list.
   */
  public final TemplateToken getToken(int idx) {
    return (TemplateToken) tokens.get(idx);
  }

  /**
   * Adds a token to the token list.
   * @param token The token to add.
   */
  public final void addToken(TemplateToken token) {
    token.setParent(this);
    token.setLevel(getLevel() + 1);
    if (token.isBodyToken()) {
      TemplateTokenContainer childContainer = (TemplateTokenContainer) token;
      childContainer.setProcedureRegistry(this.procedureRegistry);
    }
    if (token.checkContainer(this)) {
      tokens.add(token);
    }
  }

  /**
   * Clears the token list.
   */
  public final void clearTokens() {
    tokens.clear();
  }

  /**
   * Getter method for the the property bodyStart.
   * @return The current value of property bodyStart.
   */
  public int getBodyStart() {
    return bodyStart;
  }
  /**
   * Setter method for the the property bodyStart.
   * @param bodyStart The value to set for the property bodyStart.
   */
  public void setBodyStart(int bodyStart) {
    this.bodyStart = bodyStart;
  }

  /**
   * Getter method for the the property bodyEnd.
   * @return The current value of property bodyEnd.
   */
  public int getBodyEnd() {
    return bodyEnd;
  }
  
  /**
   * @see java.lang.Object#toString()
   */
  public String toString() {
    StringBuffer b = new StringBuffer();
    int numTokens = getTokenCount();
    for (int i = 0; i < numTokens; i++) {
      TemplateToken t = getToken(i);
      b.append(t);
    }
    return b.toString(); 
  }
  
  /**
   * Gets the procedure registry.
   * @return The procedure registry.
   */
  public Map getProcedureRegistry() {
    return procedureRegistry;
  }
  
  /**
   * Sets the procedure registry.
   * @param registry The procedure registry to set.
   */
  public void setProcedureRegistry(Map registry) {
    this.procedureRegistry = registry;
  }

  /**
   * Clones the childs of this token.
   * @param procedureRegistry The procedure registry to use.
   * @param parent The parent token container.
   */
  protected void cloneChildTokens(Map procedureRegistry, TemplateTokenContainer parent) {
    this.procedureRegistry = procedureRegistry;
    int numChilds = CollectionUtils.getSize(tokens);
    for (int i = 0; i < numChilds; i++) {
      TemplateToken clonedToken = ((TemplateToken) tokens.get(i)).cloneToken(interpreter, parent);
      clonedToken.setLevel(getLevel() + 1);
      tokens.set(i, clonedToken);
      if (clonedToken.isBodyToken()) {
        TemplateTokenContainer childContainer = (TemplateTokenContainer) clonedToken;
        childContainer.setProcedureRegistry(this.procedureRegistry);
      }
    }
  }
}
