/*
 * Copyright (C) 2003, 2004, 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.template.test;

import java.util.ArrayList;
import java.util.List;

import org.ujac.util.template.TemplateException;

/**
 * Name: TemplateTest<br>
 * Description: A class for statement expression tests.
 * 
 * @author lauerc
 */
public class ProcedureTest extends BaseTemplateTest {

  /**
   * A class for tree nodes
   */
  public static class TreeNode {
    
    /** The node name. */
    private String name = null;
    /** The child list. */
    private List childs = null;
    
    /**
     * Constructs a TreeNode instance with specific attributes.
     * @param name The name of the node.
     */
    public TreeNode(String name) {
      this.name = name;
    }
    
    /**
     * Gets the name of the node.
     * @return The name of the node.
     */
    public String getName() {
      return name;
    }
    
    /**
     * Gets the childs of the node.
     * @return The childs of the node.
     */
    public List getChilds() {
      return childs;
    }
    
    /**
     * Adds a child to the node.
     * @param child The child to add.
     */
    public void addChild(TreeNode child) {
      if (childs == null) {
        childs = new ArrayList();
      }
      childs.add(child);
    }
    
  }
  
  /**
   * @see junit.framework.TestCase#setUp()
   */
  protected void setUp() throws Exception {
    super.setUp();

    // setting up test environment
    ArrayList numbers = new ArrayList();
    numbers.add(new Integer(12));
    numbers.add(new Integer(43));
    numbers.add(new Integer(23));
    numbers.add(new Integer(14));
    numbers.add(new Integer(76));
    numbers.add(new Integer(64));
    numbers.add(new Integer(36));
    ctx.setProperty("numbers", numbers);
    
    // building simple tree structure for recursion test
    TreeNode root = new TreeNode("root");
    TreeNode child1 = new TreeNode("child1");
    TreeNode child2 = new TreeNode("child2");
    TreeNode child3 = new TreeNode("child3");
    root.addChild(child1);
    root.addChild(child2);
    root.addChild(child3);
    child1.addChild(new TreeNode("grandChild1"));
    TreeNode grandChild2 = new TreeNode("grandChild2");
    child2.addChild(grandChild2);
    child3.addChild(new TreeNode("grandChild3"));
    grandChild2.addChild(new TreeNode("grandGrandChild2"));
    ctx.setProperty("rootNode", root);
    
  }

  /**
   * Tests statements executing procedures.
   * @throws TemplateException In case the template execution failed.
   */
  public void testProcedures() throws TemplateException {
    String result = interpreter.execute(
      "_procedure_ printNumber(idx, value)" +
      "_if_(idx!=0),_endif_#${idx}:${value}" +
      "_endproc_" +
      "_foreach_(number ${numbers})" +
      "_callproc_(printNumber, ${numberIdx}, ${number})" +
      "_endfor_",
      ctx
    );
    assertEquals("#0:12,#1:43,#2:23,#3:14,#4:76,#5:64,#6:36", result);
  }
  
  /**
   * Tests statements executing recursive procedures.
   * @throws TemplateException In case the template execution failed.
   */
  public void testRecursion() throws TemplateException {
    // tests recursion
    String result = interpreter.execute(
        "_procedure_ printItem(item, separator)" +
        "_define_(count, ${count + 1})" +
        "${item.name} #${count}\n" +
        "_if_(item.childs isDefined)" +
        "_foreach_(child ${item.childs})" +
        "${separator}_callproc_(printItem, ${child}, ${separator + ' '})" +
        "_endfor_" +
        "_endif_" +
        "_endproc_" +
        "" +
        "_define_(count, 0)" +
        "_callproc_(printItem, ${rootNode}, ' ')",
        ctx
    );
    assertEquals("root #1\n child1 #2\n  grandChild1 #3\n child2 #2\n  grandChild2 #3\n   grandGrandChild2 #4\n child3 #2\n  grandChild3 #3\n", result);
  }
  
  /**
   * Tests statements executing recursive procedures.
   * @throws TemplateException In case the template execution failed.
   */
  public void testRecursionGlobal() throws TemplateException {
    // The same but with global counter variable
    String result = interpreter.execute(
      "_procedure_ printItem(item, separator)" +
      "_define_(count, ${count + 1})" +
      "${item.name} #${count}\n" +
      "_if_(item.childs isDefined)" +
      "_foreach_(child ${item.childs})" +
      "${separator}_callproc_(printItem, ${child}, ${separator + ' '})" +
      "_endfor_" +
      "_endif_" +
      "_endproc_" +
      "" +
      "_define_(count, 0, global)" +
      "_callproc_(printItem, ${rootNode}, ' ')",
      ctx
    );
    assertEquals("root #1\n child1 #2\n  grandChild1 #3\n child2 #4\n  grandChild2 #5\n   grandGrandChild2 #6\n child3 #7\n  grandChild3 #8\n", result);
  }

}
