/*
 * Copyright (C) 2003, 2004, 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.template.test;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.ujac.util.template.TemplateException;

/**
 * Name: TemplateTest<br>
 * Description: Defines main template interpreter test cases.
 * 
 * @author lauerc
 */
public class TemplateTest extends BaseTemplateTest {

  /**
   * @see junit.framework.TestCase#setUp()
   */
  protected void setUp() throws Exception {
    super.setUp();

    ctx.setProperty("string", new String("Hallo, das ist ein Test"));
    ctx.setProperty("number", new Integer(10));
    ctx.setProperty("test", "${${testdatum}.year}");
    ctx.setProperty("test2", "${testdatum decrMonth 11}");
    ctx.setProperty("exception", new Exception("Test message."));

    List numbers = new ArrayList();
    numbers.add("one");
    numbers.add("two");
    numbers.add("three");
    numbers.add("four");
    numbers.add("five");
    numbers.add("six");
    numbers.add("seven");
    numbers.add("eight");
    numbers.add("nine");
    numbers.add("ten");
    numbers.add("eleven");
    ctx.setProperty("numberList", numbers);

    List hugeList = new ArrayList();
    for (int i = 0; i < 10000; i++) {
      hugeList.add(new Integer(i));
    }
    ctx.setProperty("hugeList", hugeList);

    Map numberMap = new TreeMap();
    int numNumbers = numbers.size();
    for (int i = 0; i < numNumbers; i++) {
      numberMap.put(numbers.get(i), new Integer(i + 1));
    }
    ctx.setProperty("numberMap", numberMap);
 
  }

  /**
   * Tests the execution of templates, containing underscores.
   * @exception TemplateException In case the template execution failed.
   */
  public void testUnderscores() throws TemplateException {
    String result = interpreter.execute(
      "_test_1_2_3_4_",
      ctx
    );
    assertEquals("_test_1_2_3_4_", result);

    result = interpreter.execute(
      "test_1_2_3_4.tst",
      ctx
    );
    assertEquals("test_1_2_3_4.tst", result);

    result = interpreter.execute(
      "_if_(number == 10)_434_121_else_1234__endif_",
      ctx
    );
    assertEquals("_434_121", result);
  }
  /**
   * Tests the execution of _foreach_ statements.
   * @exception TemplateException In case the template execution failed.
   */
  public void testForeach() throws TemplateException {
    String result = interpreter.execute(
        "_foreach_(number ${numberList})" +
        "_if_(numberIdx != 0),_endif__if_(numberIdx == 0)zero_elseif_(numberIdx == 1)one_elseif_(numberIdx == 2)two_elseif_(numberIdx == 3)three_elseif_(numberIdx == 4)four_elseif_(numberIdx == 5)five_elseif_(numberIdx == 6)six_else__if_(numberIdx==7)seven_elseif_(numberIdx==8)eight_elseif_(numberIdx==9)nine_else_${numberIdx}_endif__endif_: ${numberMap.${number}}" +
        "__endfor_",
        ctx
      );
    assertEquals("zero: 1_,one: 2_,two: 3_,three: 4_,four: 5_,five: 6_,six: 7_,seven: 8_,eight: 9_,nine: 10_,10: 11_", result);

    result = interpreter.execute(
      "_foreach_(number ${numberList})" +
      "_if_(numberIdx != 0),_endif_${numberIdx}:${number} = ${numberMap.${number}}" +
      "__endfor_",
      ctx
    );
    assertEquals("0:one = 1_,1:two = 2_,2:three = 3_,3:four = 4_,4:five = 5_,5:six = 6_,6:seven = 7_,7:eight = 8_,8:nine = 9_,9:ten = 10_,10:eleven = 11_", result);

    result = interpreter.execute("_foreach_(test ${nullIsNull})${test},_endfor_", ctx);
    assertEquals(result, "");
  }

  /**
   * Extended tests for the execution of _foreach_ statements.
   * @exception TemplateException In case the template execution failed.
   */
  public void testExtendedForeach() throws TemplateException {
    // parsing statement
    String result = interpreter.execute(
      "_foreach_(number ${numberList} even odd)" +
      "_if_(numberIdx != 0),_endif_<${numberStyle}>${numberIdx}:${number}=${numberMap.${number}}</${numberStyle}>" +
      "_endfor_",
      ctx
    );
    assertEquals(result, "<even>0:one=1</even>,<odd>1:two=2</odd>,<even>2:three=3</even>,<odd>3:four=4</odd>,<even>4:five=5</even>,<odd>5:six=6</odd>,<even>6:seven=7</even>,<odd>7:eight=8</odd>,<even>8:nine=9</even>,<odd>9:ten=10</odd>,<even>10:eleven=11</even>");
  }

  /**
   * Extended tests for the execution of _foreach_ statements.
   * @exception TemplateException In case the template execution failed.
   */
  public void testMapEntriesLoop() throws TemplateException {
    // parsing statement
    String result = interpreter.execute(
      "_foreach_(numberEntry ${numberMap entries})" +
      "_if_(numberEntryIdx != 0),_endif_${numberEntryIdx}:${numberEntry.key}=${numberEntry.value}" +
      "_endfor_",
      ctx
    );
    assertEquals("0:eight=8,1:eleven=11,2:five=5,3:four=4,4:nine=9,5:one=1,6:seven=7,7:six=6,8:ten=10,9:three=3,10:two=2", result);
  }

  /**
   * Tests the _if_ _elseif_ _else_ _endif_ statement.
   * @exception TemplateException In case the template execution failed.
   */
  public void testIfElseifElseEndif() throws TemplateException {
    String result = interpreter.execute(
      "_if_(number > 100)greater one hundred" +
      "_elseif_(number > 10)greater ten" +
      "_elseif_(number > 1)greater one" +
      "_else_(number > 1)zero or less" +
      "_endif_",
      ctx
    );
    assertEquals(result, "greater one");
  }

  /**
   * Tests the _define_ statement.
   * @exception TemplateException In case the template execution failed.
   */
  public void testDefine() throws TemplateException {
    String result = interpreter.execute(
      "_define_(defTest 100)" +
      "${defTest}",
      ctx
    );
    assertEquals(result, "100");

    result = interpreter.execute(
      "_define_(defTest (double) 100)" +
      "${defTest}",
      ctx
    );
    assertEquals("100.00", result);
  }

  /**
   * Tests the _log_ statement.
   * @exception TemplateException In case the template execution failed.
   */
  public void testLog() throws TemplateException {
    // parsing statement
    String result = interpreter.execute(
      "_foreach_(number ${numberList} even odd)" +
      "_log_('info', 'Number #${numberIdx}: ${number}')" +
      "_endfor_",
      ctx
    );
    assertEquals("", result);
  }

  /**
   * Tests the _assert_ statement.
   * @exception TemplateException In case the template execution failed.
   */
  public void testAssert() throws TemplateException {
    // parsing statement
    String result = interpreter.execute(
      "_assert_(${numberList notEmpty})" +
      "_assert_(numberList[0] == 'one')" +
      "_log_('info', 'numberList is not empty')",
      ctx
    );
    assertEquals("", result);
  }

}
