/*
 * Copyright (C) 2007 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.test;

import java.util.List;

import org.ujac.util.BeanUtils;
import org.ujac.util.CollectionUtils;
import org.ujac.util.CompareTuple;

import junit.framework.TestCase;

/**
 * Name: StringUtilsTest<br>
 * Description: Tests for the StringUtils class.
 * 
 * @author lauerc
 */
public class CollectionUtilsTest extends TestCase {
  
  /**  The list to process. */
  private List list = null;
  
  /**
   * Data container for sorting tests.
   */
  public class Person {
    
    /** The ID. */
    private int id;
    /** The name. */
    private String name;
    /** The first name. */
    private String firstName;
    
    /**
     * Constructs a Person instance with specific attributes.
     * @param id The initial ID.
     * @param name The initial name.
     * @param firstName The initial first name.
     */
    public Person(int id, String name, String firstName) {
      this.id = id;
      this.name = name;
      this.firstName = firstName;
    }

    /**
     * Getter method for the the property id.
     * @return The current value of property id.
     */
    public int getId() {
      return id;
    }

    /**
     * Getter method for the the property name.
     * @return The current value of property name.
     */
    public String getName() {
      return name;
    }

    /**
     * Getter method for the the property firstName.
     * @return The current value of property firstName.
     */
    public String getFirstName() {
      return firstName;
    }
    
    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
      Person otherPerson = (Person) obj;
      return id == otherPerson.id && 
          BeanUtils.equals(name, otherPerson.name) && 
          BeanUtils.equals(firstName, otherPerson.firstName);
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
      return (Integer.toString(id) + "_" + name + "_" + firstName).hashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
      return "{id: " + id + ", name: " + name + ", firstName: " + firstName + "}";
    }
    
  }
  
  /**
   * @see junit.framework.TestCase#setUp()
   */
  protected void setUp() throws Exception {
    list = CollectionUtils.createArrayList(new Person[] {
        new Person(1, "Urpils", "Hansi"),
        new Person(2, "Urpils", "Sigrid"),
        new Person(3, "Becker", "Hilde"),
        new Person(4, "Meier", "Kurt"),
        new Person(5, "Becker", "Heinz"),
    });
  }
  
  /**
   * Tests ascending list sorting.
   * @throws Exception In case something went wrong.
   */
  public void testSortListAscending() throws Exception {
    CollectionUtils.sortList(list, new String[] {"name", "firstName"});
    assertEquals("Becker, Heinz", ((Person) list.get(0)).getName() + ", " + ((Person) list.get(0)).getFirstName());
    assertEquals("Becker, Hilde", ((Person) list.get(1)).getName() + ", " + ((Person) list.get(1)).getFirstName());
    assertEquals("Meier, Kurt", ((Person) list.get(2)).getName() + ", " + ((Person) list.get(2)).getFirstName());
    assertEquals("Urpils, Hansi", ((Person) list.get(3)).getName() + ", " + ((Person) list.get(3)).getFirstName());
    assertEquals("Urpils, Sigrid", ((Person) list.get(4)).getName() + ", " + ((Person) list.get(4)).getFirstName());
  }
  
  /**
   * Tests ascending list sorting.
   * @throws Exception In case something went wrong.
   */
  public void testSortListDescending() throws Exception {
    CollectionUtils.sortList(list, new CompareTuple[] {
        new CompareTuple("name", CompareTuple.DESCENDING), 
        new CompareTuple("firstName", CompareTuple.DESCENDING)
    });
    assertEquals("Urpils, Sigrid", ((Person) list.get(0)).getName() + ", " + ((Person) list.get(0)).getFirstName());
    assertEquals("Urpils, Hansi", ((Person) list.get(1)).getName() + ", " + ((Person) list.get(1)).getFirstName());
    assertEquals("Meier, Kurt", ((Person) list.get(2)).getName() + ", " + ((Person) list.get(2)).getFirstName());
    assertEquals("Becker, Hilde", ((Person) list.get(3)).getName() + ", " + ((Person) list.get(3)).getFirstName());
    assertEquals("Becker, Heinz", ((Person) list.get(4)).getName() + ", " + ((Person) list.get(4)).getFirstName());
  }
  
  /**
   * Tests list sorting with mixed directions.
   * @throws Exception In case something went wrong.
   */
  public void testSortListMixedDirections() throws Exception {
    CollectionUtils.sortList(list, new CompareTuple[] {
        new CompareTuple("name", CompareTuple.ASCENDING), 
        new CompareTuple("firstName", CompareTuple.DESCENDING)
    });
    assertEquals("Becker, Hilde", ((Person) list.get(0)).getName() + ", " + ((Person) list.get(0)).getFirstName());
    assertEquals("Becker, Heinz", ((Person) list.get(1)).getName() + ", " + ((Person) list.get(1)).getFirstName());
    assertEquals("Meier, Kurt", ((Person) list.get(2)).getName() + ", " + ((Person) list.get(2)).getFirstName());
    assertEquals("Urpils, Sigrid", ((Person) list.get(3)).getName() + ", " + ((Person) list.get(3)).getFirstName());
    assertEquals("Urpils, Hansi", ((Person) list.get(4)).getName() + ", " + ((Person) list.get(4)).getFirstName());
  }
  
  /**
   * Tests ascending list sorting.
   * @throws Exception In case something went wrong.
   */
  public void testContains() throws Exception {
    assertTrue(CollectionUtils.contains(list, new Person(1, "Urpils", "Hansi")));
    assertFalse(CollectionUtils.contains(list, new Person(1, "Urpils", "Sigrid")));
  }

}
