/*
 * Copyright (C) 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.test;

import java.text.DateFormat;
import java.text.SimpleDateFormat;

import java.util.Calendar;
import java.util.Date;

import org.ujac.util.DateUtils;

import junit.framework.TestCase;

/**
 * Name: DateUtilsTest<br>
 * Description: Tests for the DateUtils class.
 * 
 * @author lauerc
 */
public class DateUtilsTest extends TestCase {
  
  /** The date format to use. */
  private DateFormat df = null;
  /** The precise date format to use. */
  private DateFormat preciseDf = null;
  
  /**
   * @see junit.framework.TestCase#setUp()
   */
  protected void setUp() throws Exception {
    df = new SimpleDateFormat("yyyy-MM-dd");
    preciseDf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
  }

  /**
   * Tests the createDate method.
   * @throws Exception In case something went wrong.
   */
  public void testCreateDate() throws Exception {
    Date d = DateUtils.createDate(2008, 02, 29);
    assertEquals("2008-02-29", df.format(d));

    Date d2 = DateUtils.createDate(2008, 02, 29, 8, 30, 1);
    assertEquals("2008-02-29 08:30:01", preciseDf.format(d2));
  }
  
  /**
   * Tests the createCalendar method.
   * @throws Exception In case something went wrong.
   */
  public void testCreateCalendar() throws Exception {
    Calendar cal = DateUtils.createCalendar(2008, 02, 29);
    assertEquals(df.parse("2008-02-29"), cal.getTime());

    Calendar cal2 = DateUtils.createCalendar(2008, 02, 29, 8, 30, 1);
    assertEquals(preciseDf.parse("2008-02-29 08:30:01"), cal2.getTime());
  }

  /**
   * Tests boundary checks for the createDate method.
   * @throws Exception In case something went wrong.
   */
  public void testCreateChecks() throws Exception {
    try {
      // testing check for invalid day input
      DateUtils.createDate(2008, 02, -1);
      assertTrue(false);
    } catch (RuntimeException ex) {
      assertTrue(true);
    }
    try {
      // testing check for invalid day input
      DateUtils.createDate(2008, 02, 30);
      assertTrue(false);
    } catch (RuntimeException ex) {
      assertTrue(true);
    }

    try {
      // testing check for invalid month input
      DateUtils.createDate(2008, -1, 29);
      assertTrue(false);
    } catch (RuntimeException ex) {
      assertTrue(true);
    }
    try {
      // testing check for invalid day input
      DateUtils.createDate(2008, 13, 29);
      assertTrue(false);
    } catch (RuntimeException ex) {
      assertTrue(true);
    }
    
    try {
      // testing check for invalid hour input
      DateUtils.createDate(2008, 02, 29, -1, 10, 10);
      assertTrue(false);
    } catch (RuntimeException ex) {
      assertTrue(true);
    }
    try {
      // testing check for invalid hour input
      DateUtils.createDate(2008, 02, 29, 24, 10, 10);
      assertTrue(false);
    } catch (RuntimeException ex) {
      assertTrue(true);
    }

    try {
      // testing check for invalid minute input
      DateUtils.createDate(2008, 02, 29, 8, -1, 10);
      assertTrue(false);
    } catch (RuntimeException ex) {
      assertTrue(true);
    }
    try {
      // testing check for invalid minute input
      DateUtils.createDate(2008, 02, 29, 8, 60, 10);
      assertTrue(false);
    } catch (RuntimeException ex) {
      assertTrue(true);
    }
  }
  
  /**
   * Tests the setDay method.
   * @throws Exception In case something went wrong.
   */
  public void testSetDay() throws Exception {
    Date date = df.parse("2008-04-30");
    try {
      // testing check for invalid day input
      DateUtils.setDay(date, -1);
      assertTrue(false);
    } catch (RuntimeException ex) {
      assertTrue(true);
    }
    try {
      // testing check for invalid day input
      DateUtils.setDay(date, 31);
      assertTrue(false);
    } catch (RuntimeException ex) {
      assertTrue(true);
    }
    date = DateUtils.setDay(date, 1);
    assertEquals(df.parse("2008-04-01"), date);
  }
  
  /**
   * Tests the setMonth method.
   * @throws Exception In case something went wrong.
   */
  public void testSetMonth() throws Exception {
    Date date = df.parse("2008-04-30");
    try {
      // testing check for invalid month input
      DateUtils.setMonth(date, -1);
      assertTrue(false);
    } catch (RuntimeException ex) {
      assertTrue(true);
    }
    try {
      // testing check for invalid month input
      DateUtils.setMonth(date, 31);
      assertTrue(false);
    } catch (RuntimeException ex) {
      assertTrue(true);
    }
    try {
      date = DateUtils.setMonth(date, 2);
      assertTrue(false);
    } catch (RuntimeException ex) {
      assertTrue(true);
    }
    date = DateUtils.setMonth(date, 6);
    assertEquals(df.parse("2008-06-30"), date);
  }

  /**
   * Tests the setYear method.
   * @throws Exception In case something went wrong.
   */
  public void testSetYear() throws Exception {
    Date date = df.parse("2008-04-30");
    try {
      // testing check for invalid year input
      DateUtils.setYear(date, 0);
      assertTrue(false);
    } catch (RuntimeException ex) {
      assertTrue(true);
    }
    try {
      // testing check for invalid year input
      DateUtils.setYear(date, 292278995);
      assertTrue(false);
    } catch (RuntimeException ex) {
      assertTrue(true);
    }
    date = DateUtils.setYear(date, 1999);
    assertEquals(df.parse("1999-04-30"), date);
  }

  /**
   * Tests the setHour method.
   * @throws Exception In case something went wrong.
   */
  public void testSetHour() throws Exception {
    Date date = preciseDf.parse("2008-04-30 08:15:00");
    try {
      // testing check for invalid hour input
      DateUtils.setHour(date, -1);
      assertTrue(false);
    } catch (RuntimeException ex) {
      assertTrue(true);
    }
    try {
      // testing check for invalid hour input
      DateUtils.setHour(date, 24);
      assertTrue(false);
    } catch (RuntimeException ex) {
      assertTrue(true);
    }
    date = DateUtils.setHour(date, 23);
    assertEquals(preciseDf.parse("2008-04-30 23:15:00"), date);
  }
  /**
   * Tests the incrHour method.
   * @throws Exception In case something went wrong.
   */
  public void testIncrHour() throws Exception {
    Date date = preciseDf.parse("2008-04-30 08:15:00");
    date = DateUtils.incrHour(date, 23);
    assertEquals(preciseDf.parse("2008-05-01 07:15:00"), date);
  }

  /**
   * Tests the setMinute method.
   * @throws Exception In case something went wrong.
   */
  public void testSetMinute() throws Exception {
    Date date = preciseDf.parse("2008-04-30 08:15:00");
    try {
      // testing check for invalid minute input
      DateUtils.setMinute(date, -1);
      assertTrue(false);
    } catch (RuntimeException ex) {
      assertTrue(true);
    }
    try {
      // testing check for invalid minute input
      DateUtils.setMinute(date, 60);
      assertTrue(false);
    } catch (RuntimeException ex) {
      assertTrue(true);
    }
    date = DateUtils.setMinute(date, 59);
    assertEquals(preciseDf.parse("2008-04-30 08:59:00"), date);
  }
  /**
   * Tests the incrMinute method.
   * @throws Exception In case something went wrong.
   */
  public void testIncrMinute() throws Exception {
    Date date = preciseDf.parse("2008-04-30 08:15:00");
    date = DateUtils.incrMinute(date, 46);
    assertEquals(preciseDf.parse("2008-04-30 09:01:00"), date);
  }

  /**
   * Tests the setSecond method.
   * @throws Exception In case something went wrong.
   */
  public void testSetSecond() throws Exception {
    Date date = preciseDf.parse("2008-04-30 08:15:00");
    try {
      // testing check for invalid second input
      DateUtils.setSecond(date, -1);
      assertTrue(false);
    } catch (RuntimeException ex) {
      assertTrue(true);
    }
    try {
      // testing check for invalid second input
      DateUtils.setSecond(date, 60);
      assertTrue(false);
    } catch (RuntimeException ex) {
      assertTrue(true);
    }
    date = DateUtils.setSecond(date, 59);
    assertEquals(preciseDf.parse("2008-04-30 08:15:59"), date);
  }
  /**
   * Tests the incrSecond method.
   * @throws Exception In case something went wrong.
   */
  public void testIncrSecond() throws Exception {
    Date date = preciseDf.parse("2008-04-30 08:15:00");
    date = DateUtils.incrSecond(date, 141);
    assertEquals(preciseDf.parse("2008-04-30 08:17:21"), date);
  }
  /**
   * Tests the yesterday method.
   * @throws Exception In case something went wrong.
   */
  public void testYesterday() throws Exception {
    Calendar cal = Calendar.getInstance();
    cal.add(Calendar.DATE, -1);
    String strYesterday = df.format(cal.getTime());
    Date yesterday = df.parse(strYesterday);
    assertEquals(yesterday, DateUtils.yesterday());
  }
  
  /**
   * Tests the beginOfYear method.
   * @throws Exception In case something went wrong.
   */
  public void testBeginOfYear() throws Exception {
    Date date = preciseDf.parse("2008-04-30 08:15:00");
    date = DateUtils.beginOfYear(date);
    assertEquals(df.parse("2008-01-01"), date);
  }
  
  /**
   * Tests the endOfYear method.
   * @throws Exception In case something went wrong.
   */
  public void testEndOfYear() throws Exception {
    Date date = preciseDf.parse("2008-04-30 08:15:00");
    date = DateUtils.endOfYear(date);
    assertEquals(df.parse("2008-12-31"), date);
  }
  
  /**
   * Tests the nextUltimo method.
   * @throws Exception In case something went wrong.
   */
  public void testNextUltimo() throws Exception {
    Date date = preciseDf.parse("2008-04-30 08:15:00");
    date = DateUtils.nextUltimo(date);
    assertEquals(df.parse("2008-04-30"), date);

    Date date2 = preciseDf.parse("2008-04-20 08:15:00");
    date2 = DateUtils.nextUltimo(date2);
    assertEquals(df.parse("2008-04-30"), date2);
  }
  
  /**
   * Tests the previousUltimo method.
   * @throws Exception In case something went wrong.
   */
  public void testPreviousUltimo() throws Exception {
    Date date = preciseDf.parse("2008-04-30 08:15:00");
    date = DateUtils.previousUltimo(date);
    assertEquals(df.parse("2008-04-30"), date);

    Date date2 = preciseDf.parse("2008-04-20 08:15:00");
    date2 = DateUtils.previousUltimo(date2);
    assertEquals(df.parse("2008-03-31"), date2);
  }

}
