/*
 * Copyright (C) 2003, 2004, 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.text;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import java.math.BigDecimal;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.FieldPosition;

/**
 * Name: BigDecimalFormat<br>
 * Description: A number format class, using the BigDecimal class to perform the rounding.
 * 
 * @author lauerc
 */
public class BigDecimalFormat extends DecimalFormat {

  /** The serial version UID. */
  private static final long serialVersionUID = 3690478034742620467L;

  /** The rounding mode. */
  private int roundingMode = BigDecimal.ROUND_HALF_UP;

  /**
   * Constructs a BigDecimalFormat instance with no specific attributes.
   */
  public BigDecimalFormat() {
  }

  /**
   * Constructs a BigDecimalFormat instance with specific attributes.
   * @param pattern The format pattern.
   */
  public BigDecimalFormat(String pattern) {
    super(pattern);
  }

  /**
   * Constructs a BigDecimalFormat instance with specific attributes.
   * @param pattern The format pattern.
   * @param roundingMode The initial rounding mode.
   */
  public BigDecimalFormat(String pattern, int roundingMode) {
    super(pattern);
    this.roundingMode = roundingMode;
  }

  /**
   * Constructs a BigDecimalFormat instance with specific attributes.
   * @param pattern The format pattern.
   * @param symbols The format symbols.
   */
  public BigDecimalFormat (String pattern, DecimalFormatSymbols symbols) {
    super(pattern, symbols);
  }

  /**
   * Constructs a BigDecimalFormat instance with specific attributes.
   * @param pattern The format pattern.
   * @param symbols The format symbols.
   * @param roundingMode The initial rounding mode.
   */
  public BigDecimalFormat (String pattern, DecimalFormatSymbols symbols, int roundingMode) {
    super(pattern, symbols);
    this.roundingMode = roundingMode;
  }

  /**
   * @see java.text.NumberFormat#format(double, java.lang.StringBuffer, java.text.FieldPosition)
   */
  public StringBuffer format(double number, StringBuffer result, FieldPosition fieldPosition) {
    int maxFractionDigits = this.getMaximumFractionDigits();
    try {
      BigDecimal bd = new BigDecimal(number).setScale(maxFractionDigits, roundingMode);
      return super.format(bd.doubleValue(), result, fieldPosition);
    } catch (NumberFormatException ex) {
      return super.format(number, result, fieldPosition);
    }
  }
  
  /**
   * Gets the rounding mode.
   * @return The current rounding mode.
   */
  public int getRoundingModeId() {
    return roundingMode;
  }
  /**
   * Sets the rounding mode.
   * @param roundingMode The rounding mode to set.
   */
  public void setRoundingModeId(int roundingMode) {
    this.roundingMode = roundingMode;
  }
  
  /**
   * Writes the object's data to the given stream.
   * @param s The stream to write to
   * @exception IOException In case the data output failed.
   */
  private void writeObject(ObjectOutputStream s) throws IOException {
    s.defaultWriteObject();
  }
  /**
   * Reads the object's data from the given stream.
   * @param s The stream to read from.
   * @exception IOException In case the data reading failed.
   * @exception ClassNotFoundException In case the class to deserialize could not be found
   */
  private void readObject(ObjectInputStream s) throws IOException, ClassNotFoundException  {
    s.defaultReadObject();
  }

}
