/*
 * Copyright (C) 2003, 2004, 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.util.text;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

import java.sql.Time;
import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.DecimalFormatSymbols;
import java.text.Format;
import java.text.NumberFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;

import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

/**
 * Name: FormatHelper<br>
 * Description: A helper class handling formatting objects.
 * 
 * @author lauerc
 */
public class FormatHelper implements Serializable {

  /** The serial version UID. */
  private static final long serialVersionUID = 4050204120063750192L;

  /**
   * Constant for the format helper attribute name, needed to store the 
   * format helper in the session or request.
   */ 
  public static final String ATTRIBUTE_NAME = "org.ujac.format.helper";

  /** Constant for the ISO 8601 date format pattern. */
  public static final String ISO_DATE_PATTERN = "yyyy-MM-dd";
  /** Constant for the ISO 8601 time format pattern. */
  public static final String ISO_TIME_PATTERN = "HH:mm:ss";
  /** Constant for the ISO 8601 timestamp format pattern. */
  public static final String ISO_TIMESTAMP_PATTERN = "yyyy-MM-dd HH:mm:ss";
  
  /** The format helper to use as prototype. */
  private static FormatHelper prototype = null;

  /** The locale. */
  protected Locale locale = null;

  /** The date format. */
  protected DateFormat dateFormat = null;
  /** The time format. */
  protected DateFormat timeFormat = null;
  /** The timestamp format. */
  protected DateFormat timestampFormat = null;
  /** The integer format. */
  protected NumberFormat integerFormat = null;
  /** The floating point format. */
  protected NumberFormat doubleFormat = null;

  /** The format map. */
  private Map formatMap = new HashMap();

  /**
   * Creates an FormatHelper instance according to the registered prototype. 
   * @return The created FormatHelper instance.
   */
  public static FormatHelper createInstance() {
    return createInstance(Locale.getDefault());
  }

  /**
   * Creates an FormatHelper instance according to the registered prototype. 
   * @return The created FormatHelper instance.
   */
  public static FormatHelper createIsoInstance() {
    FormatHelper fh = createInstance(Locale.US);
    fh.setDateFormat(ISO_DATE_PATTERN);
    fh.setTimeFormat(ISO_TIME_PATTERN);
    fh.setTimestampFormat(ISO_TIMESTAMP_PATTERN);
    fh.setIntegerFormat("");
    fh.setDoubleFormat("");
    return fh;
  }

  /**
   * Creates an FormatHelper instance according to the registered prototype.
   * @param locale The locale to use.
   * @return The created FormatHelper instance.
   */
  public static FormatHelper createInstance(Locale locale) {
    if (prototype == null) {
      String clazzName = System.getProperty(ATTRIBUTE_NAME);
      if (clazzName != null) {
        try {
          FormatHelper fh = (FormatHelper) Class.forName(clazzName).newInstance();
          if (locale.equals(Locale.getDefault())) {
            return fh;
          }
          return fh.getInstance(locale);
        } catch (ClassNotFoundException ex) {
          throw new RuntimeException("Unable to create FormatHelper instance from class '" + clazzName + "': " + ex.getMessage());
        } catch (InstantiationException ex) {
          throw new RuntimeException("Unable to create FormatHelper instance from class '" + clazzName + "': " + ex.getMessage());
        } catch (IllegalAccessException ex) {
          throw new RuntimeException("Unable to create FormatHelper instance from class '" + clazzName + "': " + ex.getMessage());
        } catch (ClassCastException ex) {
          throw new RuntimeException("Unable to create FormatHelper instance from class '" + clazzName + "': " + ex.getMessage());
        }
      }
      return new FormatHelper(locale);
    }
    return prototype.getInstance(locale);
  }
  
  /**
   * Defines the format helper prototype.
   * @param prototype The prototype to define.
   */
  public static void setPrototype(FormatHelper prototype) {
    FormatHelper.prototype = prototype;
  }

  /**
   * Constructs a FormatHelper instance with no specific attributes.
   */
  public FormatHelper() {
    this.locale = Locale.getDefault();
  }

  /**
   * Constructs a FormatHelper instance with specific attributes.
   * @param locale The initial locale.
   */
  public FormatHelper(Locale locale) {
    this.locale = locale;
  }

  /**
   * Gets an instance of the FormatHelper for the given locale.
   * @param locale The locale to use.
   * @return The created format helper instance.
   */
  public FormatHelper getInstance(Locale locale) {
    return new FormatHelper(locale);
  }

  /**
   * Gets the used locale.
   * @return The locale, used for output formats.
   */
  public Locale getLocale() {
    return locale;
  }

  /**
   * Gets the current integer format.
   * @return The current integer format.
   */
  public NumberFormat getIntegerFormat() {
    if (integerFormat == null) {
      integerFormat = getDefaultIntegerFormat(locale);
    }
    return integerFormat;
  }
  /**
   * Sets the integer format.
   * @param integerFormat The integerFormat to set
   */
  public void setIntegerFormat(NumberFormat integerFormat) {
    this.integerFormat = integerFormat;
  }
  /**
   * Sets the integer format.
   * @param formatPattern The format pattern to use.
   * @return The created format.
   */
  public NumberFormat setIntegerFormat(String formatPattern) {
    NumberFormat format = (NumberFormat) lookupFormat(formatPattern);
    if (format == null) {
      format = createNumberFormat(formatPattern);
      registerFormat(formatPattern, format);
    }
    this.integerFormat = format;
    return this.integerFormat;
  }

  /**
   * Gets the current double format.
   * @return The current double format.
   */
  public NumberFormat getDoubleFormat() {
    if (doubleFormat == null) {
      doubleFormat = getDefaultDoubleFormat(locale);
    }
    return doubleFormat;
  }
  /**
   * Sets the double format.
   * @param doubleFormat The doubleFormat to set
   */
  public void setDoubleFormat(NumberFormat doubleFormat) {
    this.doubleFormat = doubleFormat;
  }
  /**
   * Sets the double format.
   * @param formatPattern The format pattern to use.
   * @return The created format.
   */
  public NumberFormat setDoubleFormat(String formatPattern) {
    NumberFormat format = (NumberFormat) lookupFormat(formatPattern);
    if (format == null) {
      format = createNumberFormat(formatPattern);
      registerFormat(formatPattern, format);
    }
    this.doubleFormat = format;
    return this.doubleFormat;
  }

  /**
   * Gets the current date format.
   * @return The current date format.
   */
  public DateFormat getDateFormat() {
    if (dateFormat == null) {
      dateFormat = getDefaultDateFormat(locale);
    }
    return dateFormat;
  }
  /**
   * Sets the date format.
   * @param dateFormat The date format to set
   */
  public void setDateFormat(DateFormat dateFormat) {
    this.dateFormat = dateFormat;
  }
  /**
   * Sets the date format.
   * @param formatPattern The format pattern to use.
   * @return The created format.
   */
  public DateFormat setDateFormat(String formatPattern) {
    DateFormat format = (DateFormat) lookupFormat(formatPattern);
    if (format == null) {
      format = createDateFormat(formatPattern);
      registerFormat(formatPattern, format);
    }
    this.dateFormat = format;
    return this.dateFormat;
  }

  /**
   * Gets the current time format.
   * @return The current time format.
   */
  public DateFormat getTimeFormat() {
    if (timeFormat == null) {
      timeFormat = getDefaultTimeFormat(locale);
    }
    return timeFormat;
  }
  /**
   * Sets the timeFormat.
   * @param timeFormat The timeFormat to set
   */
  public void setTimeFormat(DateFormat timeFormat) {
    this.timeFormat = timeFormat;
  }
  /**
   * Sets the time format.
   * @param formatPattern The format pattern to use.
   * @return The created format.
   */
  public DateFormat setTimeFormat(String formatPattern) {
    DateFormat format = (DateFormat) lookupFormat(formatPattern);
    if (format == null) {
      format = createDateFormat(formatPattern);
      registerFormat(formatPattern, format);
    }
    this.timeFormat = format;
    return this.timeFormat;
  }

  /**
   * Gets the current timestamp format.
   * @return The current timestamp format.
   */
  public DateFormat getTimestampFormat() {
    if (timestampFormat == null) {
      timestampFormat = getDefaultTimestampFormat(locale);
    }
    return timestampFormat;
  }
  /**
   * Sets the timestampFormat.
   * @param timestampFormat The timestampFormat to set
   */
  public void setTimestampFormat(DateFormat timestampFormat) {
    this.timestampFormat = timestampFormat;
  }
  /**
   * Sets the timestamp format.
   * @param formatPattern The format pattern to use.
   * @return The created format.
   */
  public DateFormat setTimestampFormat(String formatPattern) {
    DateFormat format = (DateFormat) lookupFormat(formatPattern);
    if (format == null) {
      format = createDateFormat(formatPattern);
      registerFormat(formatPattern, format);
    }
    this.timestampFormat = format;
    return this.timestampFormat;
  }
  
  /**
   * Creates a number format for the current locale.
   * @param formatPattern The format pattern to use.
   * @return The created format.
   */
  public NumberFormat createNumberFormat(String formatPattern) {
    NumberFormat format = new SmartDecimalFormat(formatPattern, new DecimalFormatSymbols(locale));
    registerFormat(formatPattern, format);
    return format;
  }

  /**
   * Creates a date format for the current locale.
   * @param formatPattern The format pattern to use.
   * @return The created format.
   */
  public DateFormat createDateFormat(String formatPattern) {
    DateFormat format = new SimpleDateFormat(formatPattern, locale);
    registerFormat(formatPattern, format);
    return format;
  }


  /**
   * Gets the default integer format for the given locale.
   * @param locale The locale to use.
   * @return The default integer format for the given locale.
   */
  public NumberFormat getDefaultIntegerFormat(Locale locale) {
    return new SmartDecimalFormat("###,###", new DecimalFormatSymbols(locale));
  }

  /**
   * Gets the default double format for the given locale.
   * @param locale The locale to use.
   * @return The default double format for the given locale.
   */
  public NumberFormat getDefaultDoubleFormat(Locale locale) {
    return new SmartDecimalFormat("###,##0.00", new DecimalFormatSymbols(locale));
  }

  /**
   * Gets the default date format for the given locale.
   * @param locale The locale to use.
   * @return The default date format for the given locale.
   */
  public DateFormat getDefaultDateFormat(Locale locale) {
    return DateFormat.getDateInstance(DateFormat.DEFAULT, locale);
  }

  /**
   * Gets the default time format for the given locale.
   * @param locale The locale to use.
   * @return The default time format for the given locale.
   */
  public DateFormat getDefaultTimeFormat(Locale locale) {
    return DateFormat.getTimeInstance(DateFormat.DEFAULT, locale);
  }

  /**
   * Gets the default timestamp format for the given locale.
   * @param locale The locale to use.
   * @return The default timestamp format for the given locale.
   */
  public DateFormat getDefaultTimestampFormat(Locale locale) {
    return DateFormat.getDateTimeInstance(DateFormat.DEFAULT, DateFormat.DEFAULT, locale);
  }

  /**
   * Gets the date format according to the ISO 8601.
   * @return The ISO 8601 date format.
   */
  public DateFormat getIsoDateFormat() {
    DateFormat format = (DateFormat) lookupFormat(ISO_DATE_PATTERN);
    if (format == null) {
      format = createDateFormat(ISO_DATE_PATTERN);
      registerFormat(ISO_DATE_PATTERN, format);
    }
    return format;
  }

  /**
   * Gets the time format according to the ISO 8601.
   * @return The ISO 8601 time format.
   */
  public DateFormat getIsoTimeFormat() {
    DateFormat format = (DateFormat) lookupFormat(ISO_TIME_PATTERN);
    if (format == null) {
      format = createDateFormat(ISO_TIME_PATTERN);
      registerFormat(ISO_TIME_PATTERN, format);
    }
    return format;
  }

  /**
   * Gets the timestamp format according to the ISO 8601.
   * @return The ISO 8601 time stamp format.
   */
  public DateFormat getIsoTimestampFormat() {
    DateFormat format = (DateFormat) lookupFormat(ISO_TIMESTAMP_PATTERN);
    if (format == null) {
      format = createDateFormat(ISO_TIMESTAMP_PATTERN);
      registerFormat(ISO_TIMESTAMP_PATTERN, format);
    }
    return format;
  }

  /**
   * Registers the given format instance for the also given format string.
   * @param formatString The format string.
   * @param formatInstance The format instance.
   */
  public void registerFormat(String formatString, Format formatInstance) {
    formatMap.put(formatString, formatInstance);
  }
  /**
   * Looks up the macro definition for the given macro name.
   * @param formatString The format string.
   * @return The according format instance or null.
   */
  public Format lookupFormat(String formatString) {
    return (Format) formatMap.get(formatString);
  }
  
  /**
   * Parses an integer from the given text using the current integer format.
   * @param text The text to parse.
   * @return The parsed int.
   * @throws ParseException In case the number parsing failed.
   * @see #getIntegerFormat() 
   */
  public int parseInt(String text) throws ParseException {
    NumberFormat nf = getIntegerFormat();
    return nf.parse(text).intValue();
  }
  /**
   * Formats the given long as String using the current integer format.
   * @param value The value to format.
   * @return The formatted number.
   * @see #getIntegerFormat() 
   */
  public String formatInt(int value) {
    NumberFormat nf = getIntegerFormat();
    return nf.format(value);
  }
  
  /**
   * Parses a long from the given text using the current integer format.
   * @param text The text to parse.
   * @return The parsed long.
   * @throws ParseException In case the number parsing failed.
   * @see #getIntegerFormat() 
   */
  public long parseLong(String text) throws ParseException {
    NumberFormat nf = getIntegerFormat();
    return nf.parse(text).longValue();
  }
  /**
   * Formats the given long as String using the current integer format.
   * @param value The value to format.
   * @return The formatted number.
   * @see #getIntegerFormat() 
   */
  public String formatLong(long value) {
    NumberFormat nf = getIntegerFormat();
    return nf.format(value);
  }

  /**
   * Parses a short from the given text using the current integer format.
   * @param text The text to parse.
   * @return The parsed short.
   * @throws ParseException In case the number parsing failed.
   * @see #getIntegerFormat() 
   */
  public short parseShort(String text) throws ParseException {
    NumberFormat nf = getIntegerFormat();
    return nf.parse(text).shortValue();
  }
  /**
   * Formats the given short as String using the current integer format.
   * @param value The value to format.
   * @return The formatted number.
   * @see #getIntegerFormat() 
   */
  public String formatShort(short value) {
    NumberFormat nf = getIntegerFormat();
    return nf.format(value);
  }
  
  /**
   * Parses a double from the given text using the current double format.
   * @param text The text to parse.
   * @return The parsed double.
   * @throws ParseException In case the number parsing failed.
   * @see #getDoubleFormat() 
   */
  public long parseDouble(String text) throws ParseException {
    NumberFormat nf = getDoubleFormat();
    return nf.parse(text).longValue();
  }
  /**
   * Formats the given double as String using the current double format.
   * @param value The value to format.
   * @return The formatted number.
   * @see #getDoubleFormat() 
   */
  public String formatDouble(double value) {
    NumberFormat nf = getDoubleFormat();
    return nf.format(value);
  }

  /**
   * Parses a float from the given text using the current double format.
   * @param text The text to parse.
   * @return The parsed float.
   * @throws ParseException In case the number parsing failed.
   * @see #getDoubleFormat() 
   */
  public long parseFloat(String text) throws ParseException {
    NumberFormat nf = getDoubleFormat();
    return nf.parse(text).longValue();
  }
  /**
   * Formats the given float as String using the current double format.
   * @param value The value to format.
   * @return The formatted number.
   * @see #getDoubleFormat() 
   */
  public String formatFloat(float value) {
    NumberFormat nf = getDoubleFormat();
    return nf.format(value);
  }

  /**
   * Parses a date from the given text using the current date format.
   * @param text The text to parse.
   * @return The parsed text.
   * @throws ParseException In case the date parsing failed.
   * @see #getDateFormat() 
   */
  public Date parseDate(String text) throws ParseException {
    DateFormat df = getDateFormat();
    return df.parse(text);
  }
  /**
   * Formats the given date as String using the current date format.
   * @param date The date to format.
   * @return The formatted date.
   */
  public String formatDate(Date date) {
    DateFormat df = getDateFormat();
    return df.format(date);
  }
  
  /**
   * Parses a time from the given text using the current time format.
   * @param text The text to parse.
   * @return The parsed time.
   * @throws ParseException In case the time parsing failed.
   * @see #getTimeFormat() 
   */
  public Time parseTime(String text) throws ParseException {
    DateFormat df = getTimeFormat();
    return new Time(df.parse(text).getTime());
  }
  /**
   * Formats the given time as String using the current time format.
   * @param date The date to format.
   * @return The formatted time.
   */
  public String formatTime(Date date) {
    DateFormat df = getTimeFormat();
    return df.format(date);
  }
  
  /**
   * Parses a time stamp from the given text using the current timestamp format.
   * @param text The text to parse.
   * @return The parsed time stamp.
   * @throws ParseException In case the time parsing failed.
   * @see #getTimestampFormat() 
   */
  public Timestamp parseTimestamp(String text) throws ParseException {
    DateFormat df = getTimestampFormat();
    return new Timestamp(df.parse(text).getTime());
  }
  /**
   * Formats the given date as String using the current time format.
   * @param date The date to format.
   * @return The formatted time.
   */
  public String formatTimestamp(Date date) {
    DateFormat df = getTimestampFormat();
    return df.format(date);
  }

  /**
   * @see java.lang.Object#clone()
   */
  public Object clone() {
    FormatHelper clone = null;
    try {
      clone = (FormatHelper) super.clone();
    } catch (CloneNotSupportedException ex) {
      throw new RuntimeException("Failed to clone Operand instance: " + ex.getMessage(), ex);
    }
    clone.formatMap = new HashMap();
    return clone;
  }

  /**
   * Writes the object's data to the given stream.
   * @param s The stream to write to
   * @exception IOException In case the data output failed.
   */
  protected void writeData(ObjectOutputStream s) throws IOException {
    s.writeObject(locale);
    s.writeObject(dateFormat);
    s.writeObject(timeFormat);
    s.writeObject(timestampFormat);
    s.writeObject(integerFormat);
    s.writeObject(doubleFormat);
    s.writeObject(formatMap);
  }
  /**
   * Reads the object's data from the given stream.
   * @param s The stream to read from.
   * @exception IOException In case the data reading failed.
   * @exception ClassNotFoundException In case the class to deserialize could not be found
   */
  protected void readData(ObjectInputStream s) throws IOException, ClassNotFoundException  {
    locale = (Locale) s.readObject();
    dateFormat = (DateFormat) s.readObject();
    timeFormat = (DateFormat) s.readObject();
    timestampFormat = (DateFormat) s.readObject();
    integerFormat = (NumberFormat) s.readObject();
    doubleFormat = (NumberFormat) s.readObject();
    formatMap = (Map) s.readObject();
  }

  /**
   * Writes the object's data to the given stream.
   * @param s The stream to write to
   * @exception IOException In case the data output failed.
   */
  private void writeObject(ObjectOutputStream s) throws IOException {
    writeData(s);
  }
  /**
   * Reads the object's data from the given stream.
   * @param s The stream to read from.
   * @exception IOException In case the data reading failed.
   * @exception ClassNotFoundException In case the class to deserialize could not be found
   */
  private void readObject(ObjectInputStream s) throws IOException, ClassNotFoundException  {
    readData(s);
  }

}
