/*
 * Copyright (C) 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.text;

import java.util.StringTokenizer;

import org.ujac.util.BeanUtils;

/**
 * Name: SimplePatternMatcher<br>
 * Description: A simple pattern matcher implementation.<br/>
 * This implementation allows patterns, beginning with a
 * wildcard ('*'), ends with a wildcard or both of each.<br/>
 * In case no wild card is defined, tested texts have to
 * exactly match the pattern.
 * 
 * @author lauerc
 */
public class SimplePatternMatcher implements PatternMatcher {

  /** Constant for exact match mode. */
  public static final int MODE_EXACT = 0;
  /** Constant for exact match mode. */
  public static final int MODE_ENDS_WITH = 1;
  /** Constant for exact match mode. */
  public static final int MODE_STARTS_WITH = 2;
  /** Constant for exact match mode. */
  public static final int MODE_INBETWEEN = 3;
  
  /**
   * Name: PatternDefinition<br>
   * Description: Defines a pattern.
   */
  public class PatternDefinition {

    /** The match mode. */
    private int matchMode = MODE_EXACT;
    /** The pattern to match. */
    private String pattern = null;
    
    /**
     * Constructs a PatternDefinition instance with specific attributes.
     * @param matchMode The match mode.
     * @param pattern The pattern String.
     */
    public PatternDefinition(int matchMode, String pattern) {
      this.matchMode = matchMode;
      this.pattern = pattern;
    }

    /**
     * Getter method for the the property matchMode.
     * @return The current value of property matchMode.
     */
    public int getMatchMode() {
      return matchMode;
    }
    /**
     * Setter method for the the property matchMode.
     * @param matchMode The value to set for the property matchMode.
     */
    public void setMatchMode(int matchMode) {
      this.matchMode = matchMode;
    }

    /**
     * Getter method for the the property pattern.
     * @return The current value of property pattern.
     */
    public String getPattern() {
      return pattern;
    }
    /**
     * Setter method for the the property pattern.
     * @param pattern The value to set for the property pattern.
     */
    public void setPattern(String pattern) {
      this.pattern = pattern;
    }

  }
  
  /** The defined patterns. */
  private PatternDefinition[] patterns = null;
  
  
  /**
   * Constructs a PatternMatcher instance with specific attributes.
   * @param pattern The pattern to match.
   */
  public SimplePatternMatcher(String pattern) {
    StringTokenizer tkn = new StringTokenizer(pattern, "|");
    int numPatterns = tkn.countTokens();
    this.patterns = new PatternDefinition[numPatterns];
    
    for (int i = 0; i < numPatterns; i++) {
      String t = tkn.nextToken();
      if (BeanUtils.isEmpty(t)) {
        this.patterns[i] = new PatternDefinition(MODE_INBETWEEN, null);
        continue;
      }
      if (t.startsWith("*")) {
        if (t.endsWith("*")) {
          if (t.length() > 1) {
            this.patterns[i] = new PatternDefinition(MODE_INBETWEEN, t.substring(1, t.length() - 1));
          } else {
            this.patterns[i] = new PatternDefinition(MODE_INBETWEEN, null);
          }
          continue;
        }
        this.patterns[i] = new PatternDefinition(MODE_ENDS_WITH, t.substring(1));
        continue;
      } else if (t.endsWith("*")) {
        this.patterns[i] = new PatternDefinition(MODE_STARTS_WITH, t.substring(0, t.length() - 1));
        continue;
      }
      this.patterns[i] = new PatternDefinition(MODE_EXACT, t);
    }
  }
  
  /**
   * Test whether the given text matches the pattern.
   * @param text The text to test.
   * @return true if the text matches the pattern, else false.
   */
  public boolean matches(String text) {
    if ((text == null) || (patterns == null)) {
      return false;
    }
    
    for (int i = 0; i < patterns.length; i++) {
      PatternDefinition pd = patterns[i];
      String pattern = pd.getPattern();
      switch (pd.getMatchMode()) {
        case MODE_EXACT:
          // exact match
          if (text.equals(pattern)) {
            return true;
          }
          break;
          
        case MODE_ENDS_WITH:
          // ends with
          if (text.endsWith(pattern)) {
            return true;
          }
          break;
  
        case MODE_STARTS_WITH:
          // starts with
          if (text.startsWith(pattern)) {
            return true;
          }
          break;
  
        case MODE_INBETWEEN:
          if (pattern == null) {
            return true;
          }
          // somewhere inbetween
          if (text.indexOf(pattern) != -1) {
            return true;
          }
          break;
      }
    }
    
    // text didn't match pattern
    return false;
  }

}
