/*
 * Copyright (C) 2005-2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.xml;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.Reader;
import java.io.Writer;

import java.util.ArrayList;
import java.util.List;


import org.ujac.util.BeanUtils;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.helpers.DefaultHandler;

/**
 * Name: BaseObjectSerializer<br>
 * Description: A base class for classes which read/write objects from/to XML documents.
 * 
 * @author lauerc
 */
public abstract class BaseObjectSerializer {

  /** The encoding to use. */
  private String encoding = "UTF-8";
  
  /** The XML Reader to use. */
  private XMLReader xmlReader;
  /** The object handler. */
  private ObjectHandler contentHandler = new ObjectHandler();
  /** The parent serializer. */
  protected BaseObjectSerializer parentSerializer;
  /** The root level. */
  private int rootLevel = 0;
  
  /**
   * Constructs a BaseObjectSerializer instance with no specific attributes.
   */
  public BaseObjectSerializer() {
  }

  /**
   * Constructs a BaseObjectSerializer instance with specific attributes.
   * @param encoding The encoding to use.
   */
  public BaseObjectSerializer(String encoding) {
    if (encoding != null) {
      this.encoding = encoding;
    }
  }
  
  /**
   * Gets the parsed root object.
   * @return The parsed root object.
   */
  public Object getRootObject() {
    return contentHandler.getRoot();
  }

  /**
   * Getter method for the the property rootLevel.
   * @return The current value of property rootLevel.
   */
  public int getRootLevel() {
    return rootLevel;
  }
  /**
   * Setter method for the the property rootLevel.
   * @param rootLevel The value to set for the property rootLevel.
   */
  public void setRootLevel(int rootLevel) {
    this.rootLevel = rootLevel;
  }

  /**
   * Parses an object from the given input stream.
   * @param is The stream to read the object from.
   * @return The parsed WebForm.
   * @exception SAXException In case the parsing failed.
   * @exception IOException In case an I/O failure occurred or the parsing failed.
   */
  public Object read(InputStream is) throws SAXException, IOException {
    InputSource formSource = new InputSource(is);
    return read(formSource);
  }
  
  /**
   * Parses an object from the given reader.
   * @param reader The reader to read the object from.
   * @return The parsed object.
   * @exception SAXException In case the parsing failed.
   * @exception IOException In case an I/O failure occurred or the parsing failed.
   */
  public Object read(Reader reader) throws SAXException, IOException {
    InputSource formSource = new InputSource(reader);
    return read(formSource);
  }
  
  /**
   * Parses an instance from the given input source.
   * @param src The input source to read the WebForm from.
   * @return The parsed WebForm.
   * @exception SAXException In case the parsing failed.
   * @exception IOException In case an I/O failure occurred.
   */
  public Object read(InputSource src) throws SAXException, IOException {
    xmlReader = XmlUtils.createXmlReader(null);
    xmlReader.setContentHandler(contentHandler);
    xmlReader.parse(src);
    return contentHandler.getRoot();
  }

  /**
   * Writes an object to the given output stream.
   * @param os The stream to write the object to.
   * @param obj The instance to write.
   * @exception IOException In case an I/O failure occurred.
   */
  public void write(OutputStream os, Object obj) throws IOException {
    OutputStreamWriter osw = new OutputStreamWriter(os);
    write(osw, obj);
    osw.flush();
  }
  
  /**
   * Writes an object to the given writer.
   * @param writer The writer to write the WebForm to.
   * @param obj The instance to write.
   * @exception IOException In case an I/O failure occurred.
   */
  public void write(Writer writer, Object obj) throws IOException {
    write(new PrintWriter(writer), obj);
  }
  
  /**
   * Writes an object to the given print writer.
   * @param writer The writer to write the WebForm to.
   * @param obj The instance to write.
   * @exception IOException In case an I/O failure occurred.
   */
  public void write(PrintWriter writer, Object obj) throws IOException {
    if (rootLevel == 0) {
      writer.println("<?xml version=\"1.0\" encoding=\"" + getEncoding() + "\"?>");
    }
  }

  /**
   * Writes a tag body.
   * @param writer The writer to write to.
   * @param value The value to write.
   * @param cData Tells whether or not wrap value into CDATA.
   */
  public void writeTagBody(PrintWriter writer, Object value, boolean cData) {
    if (cData) {
      writer.print("<![CDATA[");
    }
    writer.print(value);
    if (cData) {
      writer.print("]]>");
    }
  }

  /**
   * Writes a XML attribute to the given writer.
   * In case the given value is null, the output of the attribute is skipped.
   * @param writer The writer to write to.
   * @param name The name of the attribute to write.
   * @param value The attribute's value.
   */
  public void writeAttribute(PrintWriter writer, String name, String value) {
    if (value == null) {
      return;
    }
    writer.print(" ");
    writer.print(name);
    writer.print("=\"");
    writer.print(value);
    writer.print("\"");
  }

  /**
   * Writes a XML attribute to the given writer.
   * In case the given value is null, the output of the attribute is skipped.
   * @param writer The writer to write to.
   * @param name The name of the attribute to write.
   * @param value The attribute's value.
   */
  public void writeAttribute(PrintWriter writer, String name, boolean value) {
    if (!value) {
      return;
    }
    writer.print(" ");
    writer.print(name);
    writer.print("=\"");
    writer.print(value);
    writer.print("=\"");
  }

  /**
   * Writes a XML attribute to the given writer.
   * In case the given value is null, the output of the attribute is skipped.
   * @param writer The writer to write to.
   * @param name The name of the attribute to write.
   * @param value The attribute's value.
   */
  public void writeAttribute(PrintWriter writer, String name, int value) {
    writer.print(" ");
    writer.print(name);
    writer.print("=\"");
    writer.print(value);
    writer.print("=\"");
  }

  /**
   * Writes the defined indention to the given print writer.
   * @param writer The writer to write the WebForm to.
   * @param level The indention level.
   */
  public void indent(PrintWriter writer, int level) {
    int absoluteLevel = rootLevel + level;
    for (int i = 0; i < absoluteLevel; i++) {
      writer.write("\t");
    }
  }

  /**
   * Starts a new line.
   * @param writer The writer to write the WebForm to.
   */
  public void startNewLine(PrintWriter writer) {
    writer.write("\n");
  }
  
  /**
   * Writes an open tag to the given writer.
   * @param writer The writer to write to.
   * @param name The name of the tag to open.
   * @param leaveOpen Tells to leave the tag open.
   * @param newLine Tells to start a new line after the tag.
   */
  public void writeOpenTag(PrintWriter writer, String name, boolean leaveOpen, boolean newLine) {
    writer.print("<");
    writer.print(name);
    if (leaveOpen) {
      return;
    }
    if (newLine) {
      writer.println(">");
    } else {
      writer.print(">");
    }
  }
  /**
   * Closes an open tag to the given writer.
   * @param writer The writer to write to.
   * @param bodyLess Tells whether or not to close a bodyless tag.
   * @param newLine Tells to start a new line after the tag.
   */
  public void closeOpenTag(PrintWriter writer, boolean bodyLess, boolean newLine) {
    if (bodyLess) {
      writer.print("/");
    }
    if (newLine) {
      writer.println(">");
    } else {
      writer.print(">");
    }
  }

  /**
   * Writes a close tag to the given writer.
   * @param writer The writer to write to.
   * @param name The name of the tag to open.
   * @param newLine Tells to start a new line after the tag.
   */
  public void writeCloseTag(PrintWriter writer, String name, boolean newLine) {
    writer.print("</");
    writer.print(name);
    if (newLine) {
      writer.println(">");
      return;
    }
    writer.print(">");
  }

  /**
   * Writes a XML tag to the given writer.
   * In case the given value is null, the tag output is skipped.
   * @param writer The writer to write to.
   * @param name The name of the tag to write.
   * @param value The attribute's value.
   * @param isCData Tells whether or not the body should be written as CDATA.
   */
  public void writeTag(PrintWriter writer, String name, Object value, boolean isCData) {
    if (value == null) {
      return;
    }
    writer.print("<");
    writer.print(name);
    writer.print(">");
    if (isCData) {
      writer.print("<![CDATA[");
    }
    writer.print(value);
    if (isCData) {
      writer.print("]]>");
    }
    writer.print("</");
    writer.print(name);
    writer.println(">");
  }

  /**
   * Getter method for the the property encoding.
   * @return The current value of property encoding.
   */
  public String getEncoding() {
    return encoding;
  }
  /**
   * Setter method for the the property encoding.
   * @param encoding The value to set for the property encoding.
   */
  public void setEncoding(String encoding) {
    this.encoding = encoding;
  }
  
  
  /**
   * Name: StackElement<br>
   * Description: A class holding stack elements.
   */
  public class StackElement {
    
    /** The element name. */
    private String name = null;
    /** The tag attributes. */
    private Attributes attributes = null;
    /** The hierarchy level. */
    private int level = -1;
    /** The element data. */
    private Object data = null;
    /** The complex type definition. */
    private ComplexTypeDefinition typeDefinition = null;
    
    /**
     * Constructs a StackElement instance with specific attributes.
     * @param name The element name.
     * @param attributes The tag attributes.
     * @param level The initial level.
     */
    public StackElement(String name, Attributes attributes, int level) {
      this.name = name;
      this.attributes = attributes;
      this.level = level;
    }
    
    /**
     * Getter method for the the property name.
     * @return The current value of property name.
     */
    public String getName() {
      return name;
    }

    /**
     * Gets the tag attributes.
     * @return The current tag attributes.
     */
    public Attributes getAttributes() {
      return attributes;
    }

    /**
     * Getter method for the the property level.
     * @return The current value of property level.
     */
    public int getLevel() {
      return level;
    }

    /**
     * Getter method for the the property data.
     * @return The current value of property data.
     */
    public Object getData() {
      return data;
    }
    /**
     * Setter method for the the property data.
     * @param data The value to set for the property data.
     */
    public void setData(Object data) {
      this.data = data;
    }

    /**
     * Getter method for the the property typeDefinition.
     * @return The current value of property typeDefinition.
     */
    public ComplexTypeDefinition getTypeDefinition() {
      return typeDefinition;
    }
    /**
     * Setter method for the the property typeDefinition.
     * @param typeDefinition The value to set for the property typeDefinition.
     */
    public void setTypeDefinition(ComplexTypeDefinition typeDefinition) {
      this.typeDefinition = typeDefinition;
    }

  }
  

  /**
   * Name: ObjectHandler<br>
   * Description: Description of the class.
   */
  public class ObjectHandler extends DefaultHandler {

    /** The root object. */
    private Object root = null;

    /** The content buffer. */
    private StringBuffer contentBuf = new StringBuffer();
    /** The element stack. */
    private List elementStack = null;

    /**
     * Constructs a ReportDefinitionHandler instance with specific attributes.
     */
    public ObjectHandler() {
    }
    
    /**
     * Gets the root object.
     * @return The root object of the pars results.
     */
    public Object getRoot() {
      return root;
    }

    /**
     * @see org.xml.sax.helpers.DefaultHandler#startDocument()
     */
    public void startDocument() throws SAXException {
      root = null;
      this.elementStack = new ArrayList();
    }
    
    /**
     * @see org.xml.sax.helpers.DefaultHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
     */
    public void startElement(String uri, String localName, String qName, Attributes attributes)
      throws SAXException {
      
      // resetting content buffer
      contentBuf.setLength(0);
      StackElement parentElement = topElement();
      StackElement localElement = new StackElement(localName, new AttributesImpl(attributes), elementStack.size());
      BaseObjectSerializer.this.startElement(localElement, parentElement, root);
      if (parentElement == null) {
        root = localElement.getData();
      }
      pushElement(localElement);
    }
    
    /**
     * @see org.xml.sax.helpers.DefaultHandler#characters(char[], int, int)
     */
    public void characters(char[] ch, int start, int length) throws SAXException {
      contentBuf.append(ch, start, length);
    }
    
    /**
     * @see org.xml.sax.helpers.DefaultHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
     */
    public void endElement(String uri, String localName, String qName) throws SAXException {
      StackElement localElement = popElement();
      StackElement parentElement = topElement();
      
      BaseObjectSerializer.this.endElement(localElement, contentBuf.toString(), parentElement, root);

      if (elementStack.isEmpty() && (parentSerializer != null)) {
        xmlReader.setContentHandler(parentSerializer.contentHandler);
      }
    }
    
    public void endDocument() throws SAXException {
      super.endDocument();
    }

    /**
     * Pops an element from the stack.
     * @return The removed element.
     */
    private StackElement popElement() {
      int currentIdx = elementStack.size() - 1;
      return (StackElement) elementStack.remove(currentIdx);
    }

    /**
     * Gets the top element from the stack.
     * @return The top element.
     */
    private StackElement topElement() {
      int currentIdx = elementStack.size() - 1;
      if (currentIdx < 0) {
        return null;
      }
      return (StackElement) elementStack.get(currentIdx);
    }

    /**
     * Pushes an element on the stack.
     * @param element The new element.
     */
    private void pushElement(StackElement element) {
      elementStack.add(element);
    }

  }
  
  /**
   * Delegates the XML parsing to the given serializer.
   * @param delegate The serializer to delegat the job to.
   * @throws SAXException In case the parsing failed.
   */
  public void delegateHandler(BaseObjectSerializer delegate) throws SAXException {
    delegate.contentHandler.startDocument();
    delegate.parentSerializer = this;
    delegate.xmlReader = this.xmlReader;
    this.xmlReader.setContentHandler(delegate.contentHandler);
  }

  /**
   * Starts the given local element.
   * @param localElement The element to process.
   * @param parentElement The parent element if present, else null.
   * @param root The root object of the data structure.
   * @throws SAXException In case the parsing failed.
   */
  public abstract void startElement(StackElement localElement, StackElement parentElement, Object root)
    throws SAXException;
  
  /**
   * Ends the given local element.
   * @param localElement The element to process.
   * @param content The content of the local element.
   * @param parentElement The parent element if present, else null.
   * @param root The root object of the data structure.
   * @throws SAXException In case the parsing failed.
   */
  public abstract void endElement(StackElement localElement, String content, StackElement parentElement, Object root)
    throws SAXException;

  
  /**
   * Parses an integer value from the given String.
   * @param string The String to parse.
   * @param defaultValue The default value.
   * @return The parsed data type.
   */
  protected int parseIntValue(String string, int defaultValue) {
    if (BeanUtils.isEmpty(string)) {
      return defaultValue;
    }
    return Integer.parseInt(string);
  }

  /**
   * Parses the given boolean value.
   * @param string The String to parse.
   * @param defaultValue The default value. 
   * @return The parsed boolean value.
   */
  protected boolean parseBooleanValue(String string, boolean defaultValue) {
    if (BeanUtils.isEmpty(string)) {
      return defaultValue;
    }
    return new Boolean(string).booleanValue();
  }

}
