/*
 * Copyright (C) 2003 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.xml;

import java.io.IOException;
import java.io.StringReader;

import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * Name: XmlUtils<br>
 * Description: Class providing common utility methods for XML parsing issues.
 * 
 * @author lauerc
 */
public class XmlUtils {

  /** The default SAX driver class name. */
  public static final String DEFAULT_SAX_DRIVER = "org.apache.crimson.parser.XMLReaderImpl";
  
  /**
   * Creates a XMLReader instance according to the given driver class name.
   * In case a null value is passed for attribute <code>xmlReaderClass</code>, 
   * it first tries to create an instance from system defaults. If this attempt
   * fails, it tries to use the default driver class.
   * @param xmlReaderClass The name of the SAX driver class to use.
   * @return The created XMLReader instance.
   * @exception SAXException If no default XMLReader class
   *  can be identified and instantiated or the class cannot be
   *  loaded, instantiated, and cast to XMLReader. 
   * @see #DEFAULT_SAX_DRIVER
   */
  public static final XMLReader createXmlReader(String xmlReaderClass) throws SAXException {
    if (xmlReaderClass != null) {
      // using the user defined implementation ...
      return XMLReaderFactory.createXMLReader(xmlReaderClass);
    }
    try {
      // trying to create an instance from system defaults ...
      return XMLReaderFactory.createXMLReader();
    } catch (SAXException ex) {
      // trying to use the internally defined default parser ...
      return XMLReaderFactory.createXMLReader(DEFAULT_SAX_DRIVER);
    }
  }
  
  /**
   * Disables entity resolving for the given XMLReader component.
   * This avoids problems for usage behind a firewall and should increase the
   * performance because of lesser network transfers.
   * @param xmlReader The XMLReader to manipulate
   */
  public static final void disableEntityResolver(XMLReader xmlReader) {
    // disabling DTD/Schema based XML validation to avoid performance problems
    // and problems on systems that doesn't have internet access. 
    xmlReader.setEntityResolver(new EntityResolver() {
      public InputSource resolveEntity(String publicId, String systemId) throws SAXException, IOException {
        return new InputSource(new StringReader("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>"));
      }
    });
  }
}
