/*
 * Copyright (C) 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.util.xml.test;

import java.io.StringReader;
import java.io.StringWriter;

import java.util.ArrayList;
import java.util.List;

import org.ujac.util.CollectionUtils;

import org.ujac.util.xml.ComplexTypeDefinition;
import org.ujac.util.xml.DynamicObjectSerializer;
import org.ujac.util.xml.ElementDefinition;

import junit.framework.TestCase;

/**
 * Name: DynamicObjectSerializerTest<br>
 * Description: Test cases for the DynamicObjectSerializer component.
 * 
 * @author lauerc
 */
public class DynamicObjectSerializerTest extends TestCase {
  
  /** The XML serializer. */
  private DynamicObjectSerializer os = null;
  
  /** The object to serialize. */
  private Object personToSerialize = null;
  /** The XML structure to parse. */
  private String personToParse = null;

  /** The addresses to serialize. */
  private List addressesToSerialize = null;
  /** The XML structure to parse. */
  private String addressesToParse = null;
  
  /**
   * @see junit.framework.TestCase#setUp()
   */
  protected void setUp() throws Exception {
    super.setUp();
    os = new DynamicObjectSerializer();
    os.setComplexType(new ComplexTypeDefinition("person", Person.class));
    os.setComplexType(new ComplexTypeDefinition("address", Address.class));
    
    Address a = new Address();
    a.setAddress("Street 1");
    a.setZipCode(new Integer(1111));
    a.setCity("Nimmerland"); 
    a.setCountry("Nimmerland");

    Person p = new Person();
    p.setMainAddress(a);
    p.setAddresses(CollectionUtils.createArrayList(a));
    p.setName("Pan");
    p.setFirstName("Peter");
    p.setSex("M");

    personToSerialize = p;
    
    personToParse = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
      "<person firstName=\"Peter\" name=\"Pan\" sex=\"M\">\n" +
      "\t<addresses>\n" +
      "\t\t<address address=\"Street 1\" city=\"Nimmerland\" country=\"Nimmerland\" zipCode=\"1111\"/>\n" +
      "\t</addresses>\n" +
      "\t<mainAddress address=\"Street 1\" city=\"Nimmerland\" country=\"Nimmerland\" zipCode=\"1111\"/>\n" + 
      "</person>\n";
    
    
    addressesToSerialize = new ArrayList();
    addressesToSerialize.add(new Address("Paterplatz 1", new Integer(23237), "Klosterhausen", "Germany"));
    addressesToSerialize.add(new Address("Natterweg 8", new Integer(43243), "Ringelheim", "Germany"));
    addressesToSerialize.add(new Address("Turbogasse 116", new Integer(34523), "Ganzschnell", "Germany"));
    addressesToSerialize.add(new Address("Fahrweg 1", new Integer(23453), "Urlauben", "Germany"));
    
    addressesToParse = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
      "<addresses>\n" +
      "\t<address address=\"Paterplatz 1\" city=\"Klosterhausen\" country=\"Germany\" zipCode=\"23237\"/>\n" +
      "\t<address address=\"Natterweg 8\" city=\"Ringelheim\" country=\"Germany\" zipCode=\"43243\"/>\n" +
      "\t<address address=\"Turbogasse 116\" city=\"Ganzschnell\" country=\"Germany\" zipCode=\"34523\"/>\n" +
      "\t<address address=\"Fahrweg 1\" city=\"Urlauben\" country=\"Germany\" zipCode=\"23453\"/>\n" +
      "</addresses>\n";
  }
  
  /**
   * Tests attribute name conversion.
   */
  public void testAttributeNameConversion() {
    String attributeName = "test-name";
    String propertyName = os.attributeName2PropertyName(attributeName);
    assertEquals("testName", propertyName);
    String backAgain = os.propertyName2AttributeName(propertyName);
    assertEquals(attributeName, backAgain);

    attributeName = "test-name1234-blah-ter121";
    propertyName = os.attributeName2PropertyName(attributeName);
    assertEquals("testName1234BlahTer121", propertyName);
    backAgain = os.propertyName2AttributeName(propertyName);
    assertEquals(attributeName, backAgain);
  }
  
  /**
   * Tests the serializer.
   * @throws Exception  In case serialization/deserialization failed.
   */
  public void testSerializer() throws Exception {
    StringWriter sw = new StringWriter();
    os.write(sw, personToSerialize);
    assertEquals(personToParse, sw.toString());
  }

  /**
   * Tests the parser.
   * @throws Exception  In case serialization/deserialization failed.
   */
  public void testParser() throws Exception {
    StringReader sr = new StringReader(personToParse);
    Object parsed = os.read(sr);
    System.out.println("toSerialize:\n" + personToSerialize);
    System.out.println("parsed:\n" + parsed);
    assertEquals(parsed, personToSerialize);
  }

  /**
   * Tests the serializer.
   * @throws Exception  In case serialization/deserialization failed.
   */
  public void testCollectionSerializer() throws Exception {
    StringWriter sw = new StringWriter();
    os.setRootElement(new ElementDefinition("addresses", List.class));
    os.write(sw, addressesToSerialize);
    assertEquals(addressesToParse, sw.toString());
  }

  /**
   * Tests the serializer.
   * @throws Exception  In case serialization/deserialization failed.
   */
  public void testCollectionParser() throws Exception {
    StringReader sr = new StringReader(addressesToParse);
    os.setRootElement(new ElementDefinition("addresses", List.class));
    Object parsed = os.read(sr);
    System.out.println("toSerialize:\n" + addressesToSerialize);
    System.out.println("parsed:\n" + parsed);
    assertEquals(parsed, addressesToSerialize);
  }
  
  /**
   * Runs all test cases.
   * @param args The command line arguments.
   */
  public static void main(String[] args) {
    new DynamicObjectSerializerTest().run();
  }
  
}
