/*
 * Copyright (C) 2005 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.web.servlet;

import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ujac.print.DocumentPrinter;

import org.ujac.util.UjacException;

import org.ujac.util.io.ClassPathResourceLoader;
import org.ujac.util.io.FileUtils;
import org.ujac.util.io.ResourceLoader;

import org.ujac.util.template.TemplateInterpreter;
import org.ujac.util.text.FormatHelper;

/**
 * Name: ServletDocumentPrinter<br>
 * Description: A helper class that eases the usage of the DocumentPrinter within servlets.
 * 
 * @author lauerc
 */
public class ServletDocumentPrinter {

  /** The real document printer. */
  private DocumentPrinter documentPrinter = null;
  
  /**
   * Constructs a ServletDocumentPrinter instance with no specific attributes.
   * @param templateSource The template source which is loaded from the class path.
   * @param properties The document's properties.
   * @exception IOException In case the template could not be read.
   */
  public ServletDocumentPrinter(String templateSource, Map properties)
    throws IOException {

    // loading template from resource stream
    ResourceLoader resourceLoader = new ClassPathResourceLoader(getClass().getClassLoader());
    byte[] templateBytes = resourceLoader.loadResource(templateSource);
    InputStream is = new ByteArrayInputStream(templateBytes);

    // creating DocumentPrinter instance
    this.documentPrinter = new DocumentPrinter(is, properties);

    // using classpath resource loader as default
    this.documentPrinter.setResourceLoader(resourceLoader);
  }

  /**
   * Constructs a ServletDocumentPrinter instance with no specific attributes.
   * @param templateStream The template input stream.
   * @param properties The document's properties.
   * @exception IOException In case the template could not be read.
   */
  public ServletDocumentPrinter(InputStream templateStream, Map properties) 
    throws IOException {

    // loading template from resource stream
    ResourceLoader resourceLoader = new ClassPathResourceLoader(getClass().getClassLoader());

    // creating DocumentPrinter instance
    this.documentPrinter = new DocumentPrinter(templateStream, properties);
    
    // using classpath resource loader as default
    this.documentPrinter.setResourceLoader(resourceLoader);
  }

  /**
   * Gets the document printer instance.
   * @return The document printer instance.
   */
  public DocumentPrinter getDocumentPrinter() {
    return documentPrinter;
  }

  /**
   * Generates a PDF and writes it to the servlet output stream.
   * @param request The HTTP request.
   * @param response The HTTP response.
   * @exception IOException In case an I/O problem occurred.
   * @exception UjacException In case an UJAC related problem occurred.
   */
  public void printDocument(HttpServletRequest request, HttpServletResponse response) 
    throws IOException, UjacException {
    
    printDocument(request, response, false);
  }

  /**
   * Generates a PDF and writes it to the servlet output stream.
   * @param request The HTTP request.
   * @param response The HTTP response.
   * @param supportToc Tells whether or not to support the output of a leading table of contents.
   * @exception IOException In case an I/O problem occurred.
   * @exception UjacException In case an UJAC related problem occurred.
   */
  public void printDocument(HttpServletRequest request, HttpServletResponse response, boolean supportToc) 
    throws IOException, UjacException {
    
    File tempFile = null;
    FileOutputStream fos = null;
    try {
      // configuring document printer
      TemplateInterpreter interpreter = TemplateInterpreterHelper.getTemplateInterpreter(request);
      documentPrinter.setTemplateInterpreter(interpreter);
      FormatHelper fh = TemplateInterpreterHelper.getFormatHelper(request);
      documentPrinter.setFormatHelper(fh);
      
      // creating temporary file, to keep memory consumption low
      tempFile = File.createTempFile("ujac_print", "pdf");
      fos = new FileOutputStream(tempFile);
      BufferedOutputStream os = new BufferedOutputStream(fos);
      
      // printing document to temporary file
      documentPrinter.printDocument(os, supportToc);
      os.close();
      fos.close();
      fos = null;
      
      // writing PDF to servlet response
      long pdfSize = tempFile.length();
      response.setContentLength((int) pdfSize);
      response.setContentType("application/pdf");
      FileUtils.copyToStream(tempFile, response.getOutputStream());
      
    } finally {
      // cleanin up streams
      if (fos != null) {
        fos.close();
      }

      // removing temporary file again 
      if (!tempFile.delete()) {
        // direct removal failed, marking to delete it on exit.
        tempFile.deleteOnExit();
      }
    }
  }

}
