/*
 * Copyright (C) 2005 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.web.servlet;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import javax.xml.transform.Result;

import org.ujac.print.DocumentTransformHelper;

import org.ujac.util.UjacException;

import org.ujac.util.io.ClassPathResourceLoader;
import org.ujac.util.io.FileUtils;
import org.ujac.util.io.ResourceLoader;

import org.ujac.util.template.TemplateInterpreter;
import org.ujac.util.text.FormatHelper;

/**
 * Name: ServletDocumentTransformHelper<br>
 * Description: A helper class that eases the output of the DocumentTransformHelper within servlets.
 * 
 * @author lauerc
 */
public class ServletDocumentTransformHelper {

  /** The real document transform helper. */
  private DocumentTransformHelper documentTransformHelper = null;
  
  /** The temporary file to write to. */
  File tempFile = null;
  /** The temporary file output stream to write to. */
  FileOutputStream fos = null;

  /**
   * Constructs a ServletDocumentPrinter instance with no specific attributes.
   */
  public ServletDocumentTransformHelper() {
    this.documentTransformHelper = new DocumentTransformHelper();
  }

  /**
   * Gets the document transform helper instance.
   * @return The document transform helper instance.
   */
  public DocumentTransformHelper getTransformHelper() {
    return documentTransformHelper;
  }
  
  /**
   * Generates a PDF and writes it to the servlet output stream.
   * @param request The HTTP request.
   * @param supportToc Tells whether or not to support the output of a leading table of contents.
   *    Notice: if you use this option, the main document will be completely generated in memory, 
   *    this may lead to problems if you process huge documents.
   * @return The transformation result.
   * @exception IOException In case an I/O problem occurred.
   * @exception UjacException In case an UJAC related problem occurred.
   */
  public Result prepare(HttpServletRequest request, boolean supportToc) 
    throws IOException, UjacException {
    
    this.tempFile = null;
    FileOutputStream fos = null;
    try {
      // configuring document printer
      TemplateInterpreter interpreter = TemplateInterpreterHelper.getTemplateInterpreter(request);
      documentTransformHelper.setTemplateInterpreter(interpreter);
      FormatHelper fh = TemplateInterpreterHelper.getFormatHelper(request);
      documentTransformHelper.setFormatHelper(fh);
      
      // generating document
      if (documentTransformHelper.getResourceLoader() == null) {
        ResourceLoader resourceLoader = new ClassPathResourceLoader(getClass().getClassLoader());
        documentTransformHelper.setResourceLoader(resourceLoader);
      }
      
      // creating temporary file, to keep memory consumption low
      tempFile = File.createTempFile("ujac_print", "pdf");
      fos = new FileOutputStream(tempFile);
      BufferedOutputStream os = new BufferedOutputStream(fos);
      
      // printing document to temporary file
      return documentTransformHelper.prepare(os, supportToc);
     
    } catch (IOException ex) {
      // cleanin up streams
      if (fos != null) {
        fos.close();
      }

      // removing temporary file again 
      if ((tempFile != null) && !tempFile.delete()) {
        // direct removal failed, marking to delete it on exit.
        tempFile.deleteOnExit();
      }
      throw ex;
    } catch (UjacException ex) {
      // cleanin up streams
      if (fos != null) {
        fos.close();
      }

      // removing temporary file again 
      if ((tempFile != null) && !tempFile.delete()) {
        // direct removal failed, marking to delete it on exit.
        tempFile.deleteOnExit();
      }
      throw ex;
    }
  }

  /**
   * Finishes the document processing.
   * @param response The HTTP response to write the output to.
   * @exception IOException In case something went wrong during output.
   * @exception UjacException In case an UJAC related problem occurred.
   */
  public void finish(HttpServletResponse response)
    throws UjacException, IOException {
  
    try {
      documentTransformHelper.finish();

      // writing PDF to servlet response
      long pdfSize = tempFile.length();
      response.setContentLength((int) pdfSize);
      response.setContentType("application/pdf");
      FileUtils.copyToStream(tempFile, response.getOutputStream());
    } finally {
      // cleanin up streams
      if (fos != null) {
        fos.close();
      }

      // removing temporary file again 
      if ((tempFile != null) && !tempFile.delete()) {
        // direct removal failed, marking to delete it on exit.
        tempFile.deleteOnExit();
      }
    }
  }

}
