/*
 * Copyright (C) 2003, 2004 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.web.servlet;

import java.sql.Timestamp;

import java.util.Date;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.Tag;

import org.ujac.util.template.DefaultTemplateInterpreterFactory;
import org.ujac.util.template.TemplateContext;
import org.ujac.util.template.TemplateInterpreter;
import org.ujac.util.template.TemplateInterpreterFactory;
import org.ujac.util.text.FormatHelper;


/**
 * Name: TemplateInterpreterHelper<br>
 * Description: Eases the usage of the TemplateInterpreter in servlet environments.
 * 
 * @author lauerc
 */
public class TemplateInterpreterHelper {

  /** Constant for the TemplateInterpreter attribute name. */ 
  public static final String ATTR_TEMPLATE_INTERPRETER_FACTORY = TemplateInterpreterFactory.ATTRIBUTE_NAME;
  /** Constant for the FormatHelper attribute name. */ 
  public static final String ATTR_FORMAT_HELPER = FormatHelper.ATTRIBUTE_NAME;
  /** Constant for the ExpressionInterpreter properties attribute name. */ 
  public static final String ATTR_EXPRESSION_PROPERTIES = "ujac-expression-properties";
  
  /**
   * Gets the TemplateInterpreterFactory instance for the given request.
   * @param request The request at which the TemplateInterpreterFactory may be attached.
   * @return The TemplateInterpreterFactory instance for the given request.
   */
  public static TemplateInterpreterFactory getTemplateInterpreterFactory(HttpServletRequest request) {
    TemplateInterpreterFactory factory = (TemplateInterpreterFactory) request.getAttribute(ATTR_TEMPLATE_INTERPRETER_FACTORY);
    if (factory == null) {
      factory = new DefaultTemplateInterpreterFactory();
      request.setAttribute(ATTR_TEMPLATE_INTERPRETER_FACTORY, factory);
    }
    return factory;
  }
  /**
   * Gets the TemplateInterpreterFactory instance for the given request.
   * @param pageContext The page context at which the TemplateInterpreterFactory may be attached.
   * @return The TemplateInterpreterFactory instance for the given request.
   */
  public static TemplateInterpreterFactory getTemplateInterpreterFactory(PageContext pageContext) {
    TemplateInterpreterFactory factory = (TemplateInterpreterFactory) pageContext.findAttribute(ATTR_TEMPLATE_INTERPRETER_FACTORY);
    if (factory == null) {
      factory = new DefaultTemplateInterpreterFactory();
      pageContext.setAttribute(ATTR_TEMPLATE_INTERPRETER_FACTORY, factory, PageContext.PAGE_SCOPE);
    }
    return factory;
  }
  
  /**
   * Gets the TemplateInterpreter instance for the given request.
   * @param request The request at which the TemplateInterpreter may be attached.
   * @return The TemplateInterpreter instance for the given request.
   */
  public static TemplateInterpreter getTemplateInterpreter(HttpServletRequest request) {
    return getTemplateInterpreterFactory(request).createTemplateInterpreter();
  }
  /**
   * Gets the TemplateInterpreter instance for the given request.
   * @param pageContext The page context at which the TemplateInterpreter may be attached.
   * @return The TemplateInterpreter instance for the given request.
   */
  public static TemplateInterpreter getTemplateInterpreter(PageContext pageContext) {
    return getTemplateInterpreterFactory(pageContext).createTemplateInterpreter();
  }
  
  /**
   * Gets the FormatHelper instance for the given request.
   * @param request The request at which the ExpressionInterpreter may be attached.
   * @return The ExpressionInterpreter instance for the given request.
   */
  public static FormatHelper getFormatHelper(HttpServletRequest request) {
    FormatHelper formatHelper = (FormatHelper) request.getAttribute(ATTR_FORMAT_HELPER);
    if (formatHelper == null) {
      formatHelper = FormatHelper.createInstance(request.getLocale());
      request.setAttribute(ATTR_FORMAT_HELPER, formatHelper);
    }
    return formatHelper;
  }
  /**
   * Gets the ExpressionInterpreter instance for the given request.
   * @param pageContext The page context at which the ExpressionInterpreter may be attached.
   * @return The ExpressionInterpreter instance for the given request.
   */
  public static FormatHelper getFormatHelper(PageContext pageContext) {
    FormatHelper formatHelper = (FormatHelper) pageContext.findAttribute(ATTR_FORMAT_HELPER);
    if (formatHelper == null) {
      formatHelper = FormatHelper.createInstance(pageContext.getRequest().getLocale());
      pageContext.setAttribute(ATTR_FORMAT_HELPER, formatHelper, PageContext.PAGE_SCOPE);
    }
    return formatHelper;
  }
  /**
   * Gets the ExpressionInterpreter properties for the given request.
   * @param request The request at which the properties may be attached.
   * @return The ExpressionInterpreter properties for the given request.
   */
  public static Map getExpressionProperties(HttpServletRequest request) {
    Map props = (Map) request.getAttribute(ATTR_EXPRESSION_PROPERTIES);
    if (props == null) {
      props = new HashMap();

      // filling in session attributes
      HttpSession session = request.getSession();
      Enumeration enumAttributes = session.getAttributeNames();
      while (enumAttributes.hasMoreElements()) {
        String name = (String) enumAttributes.nextElement();
        props.put(name, session.getAttribute(name)); 
      }
      // filling in request attributes
      enumAttributes = request.getAttributeNames();
      while (enumAttributes.hasMoreElements()) {
        String name = (String) enumAttributes.nextElement();
        props.put(name, request.getAttribute(name)); 
      }
      
      // defining generated properties at request scope
      request.setAttribute(ATTR_EXPRESSION_PROPERTIES, props);
    }
    return props;
  }
  /**
   * Gets the ExpressionInterpreter properties for the given request.
   * @param pageContext The page context at which the ExpressionInterpreter may be attached.
   * @return The ExpressionInterpreter properties for the given request.
   */
  public static Map getExpressionProperties(PageContext pageContext) {
    Map props = (Map) pageContext.findAttribute(ATTR_EXPRESSION_PROPERTIES);
    if (props == null) {
      props = new HashMap();

      // filling in session attributes
      Enumeration enumAttributes = pageContext.getAttributeNamesInScope(PageContext.SESSION_SCOPE);
      while (enumAttributes.hasMoreElements()) {
        String name = (String) enumAttributes.nextElement();
        props.put(name, pageContext.getAttribute(name, PageContext.SESSION_SCOPE)); 
      }
      // filling in page attributes
      enumAttributes = pageContext.getAttributeNamesInScope(PageContext.PAGE_SCOPE);
      while (enumAttributes.hasMoreElements()) {
        String name = (String) enumAttributes.nextElement();
        props.put(name, pageContext.getAttribute(name, PageContext.PAGE_SCOPE)); 
      }
      // filling in request attributes
      enumAttributes = pageContext.getAttributeNamesInScope(PageContext.REQUEST_SCOPE);
      while (enumAttributes.hasMoreElements()) {
        String name = (String) enumAttributes.nextElement();
        props.put(name, pageContext.getAttribute(name, PageContext.REQUEST_SCOPE)); 
      }
      
      // defining generated properties at page scope
      pageContext.setAttribute(ATTR_EXPRESSION_PROPERTIES, props, PageContext.PAGE_SCOPE);
    }
    addDefaultProperties(props);
    return props;
  }

  /**
   * Creates a new template context.
   * @param request The request at which the properties may be attached.
   * @return The ExpressionInterpreter properties for the given request.
   */
  public static TemplateContext createTemplateContext(HttpServletRequest request) {
    Map properties = getExpressionProperties(request);
    FormatHelper formatHelper = getFormatHelper(request);
    return getTemplateInterpreterFactory(request).createTemplateContext(properties, formatHelper);
  }

  /**
   * Creates a new template context.
   * @param pageContext The page context at which the ExpressionInterpreter may be attached.
   * @return The ExpressionInterpreter properties for the given request.
   */
  public static TemplateContext createTemplateContext(PageContext pageContext) {
    Map properties = getExpressionProperties(pageContext);
    FormatHelper formatHelper = getFormatHelper(pageContext);
    return getTemplateInterpreterFactory(pageContext).createTemplateContext(properties, formatHelper);
  }
  
  /**
   * Creates a new template context.
   * @param pageContext The page context at which the ExpressionInterpreter may be attached.
   * @param tag The tag where to find the template context. 
   * @return The ExpressionInterpreter properties for the given request.
   */
  public static TemplateContext getTemplateContext(PageContext pageContext, Tag tag) {
    TemplateContext ctx = (TemplateContext) pageContext.getAttribute("TEMPLATE_CONTEXT", PageContext.PAGE_SCOPE);
    if (ctx != null) {
      return ctx;
    }
    // no template context found, creating new one
    ctx = createTemplateContext(pageContext);
    pageContext.setAttribute("TEMPLATE_CONTEXT", ctx, PageContext.PAGE_SCOPE);
    return ctx;
  }

  /**
   * Adds default properties.
   * @param properties The property map to maintain.
   */
  protected static void addDefaultProperties(Map properties) {
    Date currentDate = new Date();
    properties.put("today", currentDate);
    properties.put("now", new Timestamp(currentDate.getTime()));
  }

}
