/*
 * Copyright (C) 2004, 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.web.tag;

import java.io.IOException;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;

import javax.servlet.jsp.tagext.BodyTagSupport;
import javax.servlet.jsp.tagext.Tag;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.ujac.util.exi.ExpressionException;

import org.ujac.util.template.TemplateContext;
import org.ujac.util.template.TemplateInterpreter;

import org.ujac.web.servlet.TemplateContextHolder;
import org.ujac.web.servlet.TemplateInterpreterHelper;

/**
 * Name: BaseTag<br>
 * Description: Base JSP custom tag implementation which provides a smart way to render attributes for generated HTML tags.
 * 
 * @author lauerc
 */
public abstract class BaseTag extends BodyTagSupport implements TemplateContextHolder {
  
  /** The extra attribute list. */
  private Map tagAttributes = null;

  /** The template context. */
  private TemplateContext templateContext = null;

  /** The log writer. */
  protected Log log = null;

  /**
   * Writes the given attributes to the JSP writer.
   * @param ctx The page context.
   * @param writer The JSP writer to output to.
   * @exception IOException In case the JSP output failed.
   */
  protected void writeAttributes(PageContext ctx, JspWriter writer) 
    throws IOException {
    
    TemplateInterpreter interpreter = TemplateInterpreterHelper.getTemplateInterpreter(ctx);
    TemplateContext templateCtx = getTemplateContext();
    Map.Entry entry = null;
    try {
      if (tagAttributes != null) {
        Iterator entryIter = tagAttributes.entrySet().iterator();
        while (entryIter.hasNext()) {
          entry = (Map.Entry) entryIter.next();
          writer.print(" ");
          writer.print(entry.getKey());
          writer.print("=\"");
          writer.print(interpreter.evalStringExpr((String) entry.getValue(), templateCtx));
          writer.print("\"");
        }
      }
    } catch (ExpressionException ex) {
      throw new IOException("Unable to evaluate value value '" + entry.getValue() + "' for attribute '" + entry.getKey() + "': " + ex.getMessage());
    }
  }
  
  /**
   * Getter method for the the property tagAttributes.
   * @return The current value of property tagAttributes.
   */
  public Map getTagAttributes() {
    return tagAttributes;
  }
  /**
   * Gets an tag attribute parameter.
   * @param name The name of the tag attribute to get.
   * @return The tag attribute link.
   */
  public String getTagAttribute(String name) {
    if (tagAttributes == null) {
      return null;
    }
    return (String) tagAttributes.get(name);
  }
  /**
   * Sets an tag attribute.
   * @param name The name of the tag attribute.
   * @param url the action link.
   */
  public void setTagAttribute(String name, String url) {
    if (tagAttributes == null) {
      this.tagAttributes = new HashMap();
    }
    tagAttributes.put(name, url);
  }
  /**
   * Removes an tag attribute.
   * @param name The name of the tag attribute.
   */
  public void removeTagAttribute(String name) {
    if (tagAttributes == null) {
      return;
    }
    tagAttributes.remove(name);
  }
  /**
   * Removes all tag attributes.
   */
  public void clearTagAttributes() {
    if (tagAttributes == null) {
      return;
    }
    tagAttributes.clear();
  }
  
  /**
   * @see javax.servlet.jsp.tagext.Tag#doStartTag()
   */
  public int doStartTag() throws JspException {
    this.log = LogFactory.getLog(getClass());
    this.templateContext = null;
    return super.doStartTag();
  }
  
  /**
   * @see javax.servlet.jsp.tagext.BodyTagSupport#release()
   */
  public void release() {
    super.release();
    this.tagAttributes = null;
    this.templateContext = null;
  }
  
  /**
   * Gets the parent tag that matches the given class.
   * @param parentClass The class of the desrired parent tag.
   * @return The detected parent tag or null.
   */
  protected Tag getParent(Class parentClass) {
    Tag parent = getParent();
    while (parent != null) {
      if (parentClass.isInstance(parent)) {
        return parent;
      }
      parent = parent.getParent();
    }
    return null;
  }
  
  /**
   * Gets the template context.
   * @return The associated template context.
   */
  public TemplateContext getTemplateContext() {
    if (templateContext != null) {
      return templateContext;
    }
    return TemplateInterpreterHelper.getTemplateContext(pageContext, this);
  }
  /**
   * Sets an attribute, both in template context and request.
   * @param key The attribute key.
   * @param value The attribute value to set.
   * @return The old value.
   */
  public Object setAttribute(String key, Object value) {
    TemplateContext ctx = getTemplateContext();
    if ((key == null) || (value == null)) {
      pageContext.removeAttribute(key, PageContext.REQUEST_SCOPE);
      return null;
    }
    Object oldValue = ctx.setProperty(key, value);
    pageContext.setAttribute(key, value, PageContext.REQUEST_SCOPE);
    return oldValue;
  }

}
