/*
 * Copyright (C) 2007 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.web.tag;

import javax.servlet.jsp.JspException;

import javax.servlet.jsp.tagext.BodyContent;

import org.ujac.util.table.Column;
import org.ujac.util.table.LayoutHints;
import org.ujac.util.table.Table;
import org.ujac.util.table.TableException;

/**
 * Name: DefineColumnTag<br>
 * Description: A custom tag defining columns for surrounding print-table tags.
 * 
 * @author lauerc
 */
public class DefineColumnTag extends BaseTag {

  /** The serial version UID. */
  private static final long serialVersionUID = 3257852086476551222L;

  /** The column name. */
  private String name;
  /** The column width. */
  private String width;
  /** The column title. */
  private String title;
  /** The horizontal alignment. */
  private String halign;
  /** The name of the action to call from this column. */
  private String action;
  /** 
   * The javascript code to execute when calling the link. 
   * This code is evaluated dynamically by the expression interpreter,
   * the called URL is held by the property 'ACTION_URL', so a reasonable
   * example would be "openLink('${ACTION_URL}')".
   */
  private String script = null;

  /**
   * Getter method for the the property halign.
   * @return The current value of property halign.
   */
  public String getHalign() {
    return halign;
  }
  /**
   * Setter method for the the property halign.
   * @param halign The value to set for the property halign.
   */
  public void setHalign(String halign) {
    this.halign = halign.toLowerCase();
  }

  /**
   * Getter method for the the property name.
   * @return The current value of property name.
   */
  public String getName() {
    return name;
  }
  /**
   * Setter method for the the property name.
   * @param name The value to set for the property name.
   */
  public void setName(String name) {
    this.name = name;
  }

  /**
   * Getter method for the the property width.
   * @return The current value of property width.
   */
  public String getWidth() {
    return width;
  }
  /**
   * Setter method for the the property width.
   * @param width The value to set for the property width.
   */
  public void setWidth(String width) {
    this.width = width;
  }

  /**
   * Getter method for the the property title.
   * @return The current value of property title.
   */
  public String getTitle() {
    return title;
  }
  /**
   * Setter method for the the property title.
   * @param title The value to set for the property title.
   */
  public void setTitle(String title) {
    this.title = title;
  }

  /**
   * Getter method for the the property action.
   * @return The current value of property action.
   */
  public String getAction() {
    return action;
  }
  /**
   * Setter method for the the property action.
   * @param action The value to set for the property action.
   */
  public void setAction(String action) {
    this.action = action;
  }
  
  /**
   * Setter method for the the property script.
   * @param script The javascript code to execute.
   */
  public void setScript(String script) {
    this.script = script;
  }

  /**
   * @see javax.servlet.jsp.tagext.BodyTagSupport#doStartTag()
   */
  public int doStartTag() throws JspException {
    super.doStartTag();
    return EVAL_BODY_BUFFERED;
  }

  /**
   * @see javax.servlet.jsp.tagext.TagSupport#doEndTag()
   */
  public int doEndTag() throws JspException {

    String body = null;
    BodyContent bc = this.bodyContent;
    if (bc != null) {
      body = bc.getString();
    }
    
    PrintTableTag printTableTag = null;
    try {
      printTableTag = (PrintTableTag) getParent(PrintTableTag.class);
    } catch (ClassCastException ex) {
      throw new JspException("Parent tag of row-cell tag has to be print-row tag.");
    }
    
    if (printTableTag.isOutputStarted()) {
      // output has already been started, skipping insertion
      return EVAL_BODY_INCLUDE;
    }
    
    float width = -1.0F;
    if (this.width != null) {
      width = Float.parseFloat(this.width);
    }
    int hAlign = -1;
    if (this.halign != null) {
      if ("left".equals(this.halign)) {
        hAlign = LayoutHints.LEFT;
      } else if ("right".equals(this.halign)) {
        hAlign = LayoutHints.RIGHT;
      } else if ("center".equals(this.halign)) {
        hAlign = LayoutHints.CENTER;
      } else {
        throw new JspException("Unsupported alignment type '" + this.halign + "'.");
      }
    }
    
    if (action != null) {
      try {
        StringBuffer bodyBuf = new StringBuffer("<a href=\"");
        StringBuffer actionBuf = null;
        if (script != null) {
          actionBuf = new StringBuffer();
        } else {
          actionBuf = bodyBuf;
        }
        // adding parameters for key fields
        int numKeys = 0;
        Table table = printTableTag.getTable();
        String loopVariableName = printTableTag.getLoopVariable();
        int numColumns = table.getColumnCount();
        for (int i = 0; i < numColumns; i++) {
          Column col = table.getColumn(i);
          if (col.isKey()) {
            if (numKeys == 0) {
              actionBuf.append("?");
            } else {
              actionBuf.append("&");
            }
            String fieldName = col.getAlias();
            if (fieldName == null) {
              fieldName = col.getName();
            }
            actionBuf.append(fieldName)
              .append("=${")
              .append(loopVariableName)
              .append(" formatValue '")
              .append(col.getName())
              .append("'}");
          }
        }
        if (script != null) {
          int actionUrlIdx = script.indexOf("${ACTION_URL}");
          if (actionUrlIdx != -1) {
            bodyBuf.append(script.substring(0, actionUrlIdx))
              .append(actionBuf.toString())
              .append(script.substring(actionUrlIdx + 13));
          } else {
            bodyBuf.append(actionBuf.toString());
          }
        }

        bodyBuf.append("\">")
          .append(body)
          .append("</a>");
        body = bodyBuf.toString();
      } catch (TableException ex) {
        throw new JspException("Table access failure: " + ex.getMessage(), ex);
      }
    }
    
    // inserting column into table
    printTableTag.defineColumn(name, width, hAlign, title, body);
    return EVAL_BODY_INCLUDE;
  }

  /**
   * @see javax.servlet.jsp.tagext.BodyTagSupport#release()
   */
  public void release() {
    super.release();
    this.name = null;
    this.width = null;
    this.title = null;
    this.halign = null;
    this.action = null;
  }

}
