/*
 * Copyright (C) 2003, 2004, 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */

package org.ujac.web.tag;

import java.io.IOException;

import java.util.Locale;
import java.util.Map;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;

import javax.servlet.jsp.tagext.BodyTag;
import javax.servlet.jsp.tagext.BodyContent;
import javax.servlet.jsp.tagext.BodyTagSupport;
import javax.servlet.jsp.tagext.Tag;


import org.ujac.util.exi.ExpressionException;

import org.ujac.util.template.TemplateContext;
import org.ujac.util.template.TemplateInterpreter;
import org.ujac.util.text.FormatHelper;

import org.ujac.web.servlet.TemplateInterpreterHelper;

/**
 * Name: EvalTag<br>
 * Description: Evaluates the tags body using the expression interpreter.<br>
 * 
 * @author lauerc
 */
public class EvalTag extends BodyTagSupport {

  /** The serial version UID. */
  private static final long serialVersionUID = 3257003254792466483L;

  /** The bean to access (for convenience). */
  protected String bean = null;
  /** Tells, whether a ustom format has been defined or not. */
  protected boolean customFormat = false;
  /** The integer format. */
  protected String integerFormat = null;
  /** The double format. */
  protected String doubleFormat = null;
  /** The date format. */
  protected String dateFormat = null;
  /** The time format. */
  protected String timeFormat = null;
  /** The timestamp format. */
  protected String timestampFormat = null;

  /** The template interpreter. */
  private TemplateInterpreter interpreter = null;
  /** The format helper. */
  private FormatHelper formatHelper = null;
  /** The expression properties. */
  private Map properties = null;

  /**
   * Sets the bean name to access. This optional attribute allows shorter expressions
   * @param bean The name of the default bean.
   */
  public void setBean(String bean) {
    this.bean = bean;
  }

  /**
   * Sets the format for printing integer values.
   * @param format The format string to set.
   */
  public void setIntegerFormat(String format) {
    this.customFormat = true;
    this.integerFormat = format;
  }

  /**
   * Sets the format for printing double values.
   * @param format The format string to set.
   */
  public void setDoubleFormat(String format) {
    this.customFormat = true;
    this.doubleFormat = format;
  }

  /**
   * Sets the format for printing Date values.
   * @param format The format string to set.
   */
  public void setDateFormat(String format) {
    this.customFormat = true;
    this.dateFormat = format;
  }

  /**
   * Sets the format for printing Time values.
   * @param format The format string to set.
   */
  public void setTimeFormat(String format) {
    this.customFormat = true;
    this.timeFormat = format;
  }

  /**
   * Sets the format for printing Timestamp values.
   * @param format The format string to set.
   */
  public void setTimestampFormat(String format) {
    this.customFormat = true;
    this.timestampFormat = format;
  }

  
  /**
   * Getter method for the the property templateInterpreter.
   * @return The current value of property templateInterpreter.
   */
  public TemplateInterpreter getTemplateInterpreter() {
    return interpreter;
  }
  /**
   * Setter method for the the property templateInterpreter.
   * @param interpreter The value to set for the property templateInterpreter.
   */
  public void setTemplateInterpreter(TemplateInterpreter interpreter) {
    this.interpreter = interpreter;
  }

  /**
   * Getter method for the the property formatHelper.
   * @return The current value of property formatHelper.
   */
  public FormatHelper getFormatHelper() {
    return formatHelper;
  }
  /**
   * Setter method for the the property formatHelper.
   * @param formatHelper The value to set for the property formatHelper.
   */
  public void setFormatHelper(FormatHelper formatHelper) {
    this.formatHelper = formatHelper;
  }

  /**
   * Getter method for the the property properties.
   * @return The current value of property properties.
   */
  public Map getProperties() {
    return properties;
  }

  /**
   * Gets the bean instance according to the bean attribute.
   * @return The value of bean property from the properties map.
   */
  public Object getBeanInstance() {
    if (properties == null) {
      return null;
    }
    return properties.get(bean);
  }

  /**
   * @see javax.servlet.jsp.tagext.Tag#doStartTag()
   */
  public int doStartTag() throws JspException {
    return BodyTag.EVAL_BODY_BUFFERED;
  }

  /**
   * @see javax.servlet.jsp.tagext.BodyTag#doInitBody()
   */
  public void doInitBody() throws JspException {
    PageContext ctx = this.pageContext;
    this.interpreter = TemplateInterpreterHelper.getTemplateInterpreter(ctx);
    this.formatHelper = TemplateInterpreterHelper.getFormatHelper(ctx);
    if (customFormat) {
      // configuring custom format helper
      formatHelper = (FormatHelper) formatHelper.clone();
      if (integerFormat != null) {
        formatHelper.setIntegerFormat(integerFormat);
      }
      if (doubleFormat != null) {
        formatHelper.setDoubleFormat(doubleFormat);
      }
      if (dateFormat != null) {
        formatHelper.setDateFormat(dateFormat);
      }
      if (timeFormat != null) {
        formatHelper.setTimeFormat(timeFormat);
      }
      if (timestampFormat != null) {
        formatHelper.setTimestampFormat(timestampFormat);
      }
    }
    this.properties = TemplateInterpreterHelper.getExpressionProperties(ctx);
  }

  /**
   * @see javax.servlet.jsp.tagext.IterationTag#doAfterBody()
   */
  public int doAfterBody() throws JspException {
    BodyContent bc = this.bodyContent;
    String body = bc.getString();
    try {
      // evaluating body
      TemplateContext ctx = TemplateInterpreterHelper.getTemplateContext(pageContext, this);
      ctx.setBean(getBeanInstance());
      interpreter.execute(body, getPreviousOut(), ctx);
    } catch (ExpressionException e) {
      e.printStackTrace();
      throw new JspException("Error in expression while evaluating body: " + e.getMessage());
    } catch (IOException e) {
      e.printStackTrace();
      throw new JspException("I/O error while evaluating body: " + e.getMessage());
    } catch (Throwable e) {
      e.printStackTrace();
      throw new JspException("Error while evaluating body: " + e.getMessage());
    }
    return EVAL_PAGE;
  }

  /**
   * @see javax.servlet.jsp.tagext.Tag#doEndTag()
   */
  public int doEndTag() throws JspException {
    return Tag.EVAL_PAGE;
  }

  /**
   * @see javax.servlet.jsp.tagext.Tag#release()
   */
  public void release() {
    super.release();
    this.bean = null;
    this.customFormat = false;
    this.integerFormat = null;
    this.doubleFormat = null;
    this.dateFormat = null;
    this.timeFormat = null;
    this.timestampFormat = null;
    this.interpreter = null;
    this.formatHelper = null;
    this.properties = null;
  }

  /**
   * Gets the locale from the HTTP request.
   * @return The user agent's locale.
   */
  protected Locale getLocale() {
    return pageContext.getRequest().getLocale();
  }

}
