/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.web.tag;

import java.io.IOException;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;

import org.ujac.util.table.Column;
import org.ujac.util.table.LayoutHints;
import org.ujac.util.table.Row;
import org.ujac.util.table.TableException;

/**
 * Name: ExtraRowHolder<br>
 * Description: A class holding extra-row data.
 * 
 * @author lauerc
 */
public class ExtraRowHolder {

  /** The surrounding print-table tag. */
  protected PrintTableTag rowContainer = null;

  /** The general style class. */
  private String style = null;
  /** The row height. */
  private String height = null;
  /** The vertical alignment. */
  private String valign = null;
  /** The left padding. */ 
  private float paddingLeft = -1.0F;
  /** The right padding. */ 
  private float paddingRight = -1.0F;
  /** The top padding. */ 
  private float paddingTop = -1.0F;
  /** The bottom padding. */ 
  private float paddingBottom = -1.0F;

  /** The registered row cells. */
  protected RowCellHolder[] registeredRowCells = null;

  /**
   * Constructs a ExtraRowHolder instance with specific attributes.
   * @param rowContainer The row container.
   * @param registeredRowCells The registerd row cells.
   * @param style The row style.
   * @param height The row height.
   * @param valign The vertical alignment.
   * @param paddingLeft The left padding.
   * @param paddingRight The right padding.
   * @param paddingTop The top padding.
   * @param paddingBottom The bottom padding.
   */
  public ExtraRowHolder(PrintTableTag rowContainer, RowCellHolder[] registeredRowCells,
                        String style, String height, String valign, float paddingLeft, 
                        float paddingRight, float paddingTop, float paddingBottom) {
    this.rowContainer = rowContainer;
    this.registeredRowCells = registeredRowCells;
    this.style = style;
    this.height = height;
    this.valign = valign;
    this.paddingLeft = paddingLeft;
    this.paddingRight = paddingRight;
    this.paddingTop = paddingTop;
    this.paddingBottom = paddingBottom;
  }

  /**
   * Gets the registered row cells.
   * @return The registered row cells.
   */
  public RowCellHolder[] getRowCells() {
    return registeredRowCells;
  }
  
  /**
   * Performs the row output.
   * @param writer The JSP writer to use.
   * @exception JspException Thrown in case something went wrong while closing the foreach item.
   */
  public void outputRow(JspWriter writer) throws JspException {
    try {
      JspWriter out = writer;

      out.print("  <tr");
      String style = this.style;
      if (style != null) {
        out.print(" class=\"" + style + "\"");
      }
      out.println(">");

      Row row = rowContainer.getCurrentRow();
      Column[] columns = rowContainer.getVisibleColumns();
      for (int i = 0; i < columns.length; i++) {
        Column column = columns[i];
        int columnIdx = column.getIndex();
        
        LayoutHints layoutHints = column.getLayoutHints();
        RowCellHolder rowCell = registeredRowCells[i];
        String width = null;
        String height = this.height;
        style = this.style;
        String valign = this.valign;
        String halign = null;
        if (columnIdx != -1) {
          switch (row.getAlign(columnIdx)) {
            case LayoutHints.LEFT:
              halign = "left";
              break;
            case LayoutHints.RIGHT:
              halign = "right";
              break;
            case LayoutHints.CENTER:
              halign = "center";
              break;
            default: // assuming left alignment
              halign = "left";
              break;
          }
        }
        
        float paddingLeft = this.paddingLeft;
        float paddingRight = this.paddingRight;
        float paddingTop = this.paddingTop;
        float paddingBottom = this.paddingBottom;
        String content = null;
        int colspan = 1;
        if (rowCell != null) {
          width = rowCell.getWidth();
          if (rowCell.getHeight() != null) {
            height = rowCell.getHeight();
          }
          if (rowCell.getStyle() != null) {
            style = rowCell.getStyle();
          }
          if (rowCell.getValign() != null) {
            valign = rowCell.getValign();
          }
          String tmpHalign = rowCell.getHalign();
          if (tmpHalign != null) {
            halign = tmpHalign;
          }
          if (rowCell.getPaddingLeft() >= 0.0F) {
            paddingLeft = rowCell.getPaddingLeft();
          }
          if (rowCell.getPaddingRight() >= 0.0F) {
            paddingRight = rowCell.getPaddingRight();
          }
          if (rowCell.getPaddingTop() >= 0.0F) {
            paddingTop = rowCell.getPaddingTop();
          }
          if (rowCell.getPaddingBottom() >= 0.0F) {
            paddingBottom = rowCell.getPaddingBottom();
          }
          if (rowCell.hasContent()) {
            content = rowContainer.evalTemplate(rowCell.getContent());
          }
          if (rowCell.getColspan() > 0) {
            colspan = rowCell.getColspan();
          }
          if (rowCell.isFillRow()) {
            colspan = columns.length - i;
          }
        }
        String cellStyle = style;
        if ((width == null) && (layoutHints.getWidth() > 0)) {
          width = Integer.toString((int) layoutHints.getWidth()) + '%';
        }
        if (content == null) {
          if ((columnIdx != -1) && row.isVisible(columnIdx)) {
            content = row.getString(columnIdx);
          } else {
            content = "&nbsp;";
          }
        }
        // printing cell
        out.print("    <td");
        out.print(getCellAttributes(
            cellStyle, width, height, halign, valign, colspan, 
            paddingLeft, paddingRight, paddingTop, paddingBottom
        ));
        out.print(">");
        out.print(content);
        out.println("</td>");
        if (colspan > 1) {
          i += colspan - 1;
        }
      }
      out.println("  </tr>");
    } catch (IOException ex) {
      throw new JspException("Unable to print table row: " + ex.getMessage(), ex);
    } catch (TableException ex) {
      throw new JspException("Unable to print table row: " + ex.getMessage(), ex);
    } catch (ClassCastException ex) {
      throw new JspException("Parent tag of row-cell tag has to be print-row tag.");
    }
  }

  /**
   * Produces a td/th tag attribute list according to the given attributes.
   * @param styleClass The style class.
   * @param width The field width.
   * @param height The field height.
   * @param hAlign The horizontal alignment.
   * @param vAlign The vertical alignment.
   * @param colspan The column span.
   * @param paddingLeft The left padding.
   * @param paddingRight The right padding.
   * @param paddingTop The top padding.
   * @param paddingBottom The bottom padding.
   * @return The generated attribute list string.
   */
  protected String getCellAttributes(String styleClass, String width, String height, 
                                     String hAlign, String vAlign, int colspan,
                                     float paddingLeft, float paddingRight, 
                                     float paddingTop, float paddingBottom) {
    boolean stylesDefined = false;
    StringBuffer styleBuffer = new StringBuffer(" style=\"");
    StringBuffer attributesBuffer = new StringBuffer();
    if (styleClass != null) {
      attributesBuffer.append(" class=\"" + styleClass + "\"");
    }
    if (hAlign != null) {
      stylesDefined = true;
      styleBuffer.append("text-align:" + hAlign + ";");
      attributesBuffer.append(" align=\"" + hAlign + "\"");
    }
    if (vAlign != null) {
      stylesDefined = true;
      styleBuffer.append("vertical-align:" + vAlign + ";");
      attributesBuffer.append(" valign=\"" + vAlign + "\"");
    }
    if (width != null) {
      stylesDefined = true;
      styleBuffer.append("width:" + width);
      if (!width.endsWith("%")) {
        styleBuffer.append("px");
      }
      styleBuffer.append(";");
      attributesBuffer.append(" width=\"" + width + "\"");
    }
    if (height != null) {
      stylesDefined = true;
      styleBuffer.append("height:" + height);
      if (!height.endsWith("%")) {
        styleBuffer.append("px");
      }
      styleBuffer.append(";");
      attributesBuffer.append(" height=\"" + height + "\"");
    }
    if (paddingLeft > 0) {
      stylesDefined = true;
      styleBuffer.append("padding-left:" + paddingLeft);
      styleBuffer.append("px;");
    }
    if (paddingRight > 0) {
      stylesDefined = true;
      styleBuffer.append("padding-right:" + paddingRight);
      styleBuffer.append("px;");
    }
    if (paddingTop > 0) {
      stylesDefined = true;
      styleBuffer.append("padding-top:" + paddingTop);
      styleBuffer.append("px;");
    }
    if (paddingBottom > 0) {
      stylesDefined = true;
      styleBuffer.append("padding-bottom:" + paddingBottom);
      styleBuffer.append("px;");
    }
    if (colspan > 1) {
      attributesBuffer.append(" colspan=\"" + colspan + "\"");
    }
    styleBuffer.append("\"");
    if (stylesDefined) {
      attributesBuffer.append(styleBuffer.toString());
    }
    return attributesBuffer.toString();
  }

  /**
   * Getter method for the the property style.
   * @return The current value of property style.
   */
  public String getStyle() {
    return style;
  }

  /**
   * Getter method for the the property height.
   * @return The current value of property height.
   */
  public String getHeight() {
    return height;
  }

  /**
   * Getter method for the the property valign.
   * @return The current value of property valign.
   */
  public String getValign() {
    return valign;
  }

  /**
   * Getter method for the the property paddingLeft.
   * @return The current value of property paddingLeft.
   */
  public float getPaddingLeft() {
    return paddingLeft;
  }

  /**
   * Getter method for the the property paddingRight.
   * @return The current value of property paddingRight.
   */
  public float getPaddingRight() {
    return paddingRight;
  }

  /**
   * Getter method for the the property paddingTop.
   * @return The current value of property paddingTop.
   */
  public float getPaddingTop() {
    return paddingTop;
  }

  /**
   * Getter method for the the property paddingBottom.
   * @return The current value of property paddingBottom.
   */
  public float getPaddingBottom() {
    return paddingBottom;
  }

}
