/*
 * Copyright (C) 2003, 2004, 2005 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.web.tag;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.IterationTag;
import javax.servlet.jsp.tagext.Tag;

import org.ujac.util.UjacException;
import org.ujac.util.exi.ExpressionException;
import org.ujac.util.exi.SequenceIndex;

import org.ujac.util.template.TemplateContext;
import org.ujac.util.template.TemplateException;
import org.ujac.util.template.TemplateInterpreter;

import org.ujac.util.text.FormatHelper;

import org.ujac.web.servlet.TemplateContextHolder;
import org.ujac.web.servlet.TemplateInterpreterHelper;

/**
 * Name: ForeachTag<br>
 * Description: A custom tag class iterating over the sequence, specified by a given expression.
 * 
 * @author lauerc
 */
public class ForeachTag implements IterationTag, TemplateContextHolder {

  /** The default even style. */
  private static final String DEFAULT_EVEN_STYLE = "even";
  /** The default odd style. */
  private static final String DEFAULT_ODD_STYLE = "odd";

  /** The iterator method's attribute declaration. */
  private static final Class[] iteratorAttributes = {};
  /** The iterator method's parameter list. */
  private static final Object[] iteratorParameters = {};

  /** The page context. */
  private PageContext pageContext = null;
  /** The parent tag. */
  private Tag parent = null;

  /** The loop variable. */
  private String loopVariable = null;
  /** The loop index variable. */
  private String loopIdxVariable = null;
  /** The loop style variable. */
  private String loopStyleVariable = null;
  /** The sequence expression. */
  private String sequence = null;

  /** The even style. */
  private String evenStyle = DEFAULT_EVEN_STYLE;
  /** The odd style. */
  private String oddStyle = DEFAULT_ODD_STYLE;
  
  /** Tells, whether a ustom format has been defined or not. */
  protected boolean customFormat = false;
  /** The integer format. */
  protected String integerFormat = null;
  /** The double format. */
  protected String doubleFormat = null;
  /** The date format. */
  protected String dateFormat = null;
  /** The time format. */
  protected String timeFormat = null;
  /** The timestamp format. */
  protected String timestampFormat = null;

  /** The template interpreter. */
  private TemplateInterpreter interpreter = null;
  /** The template context. */
  private TemplateContext templateContext = null;
  
  /** The sequence holder. */
  private Object sequenceHolder = null;
  /** The sequence iterator. */
  private Iterator sequenceIterator = null;
  /** The iteration index. */
  private SequenceIndex iterationIdx = null;
  /** The current value. */
  private Object currentValue = null;

  /** The old loop value. */
  private Object oldLoopValue = null;
  /** The old loop index. */
  private Object oldLoopIndex = null;
  /** The old loop style. */
  private Object oldLoopStyle = null;
  
  /**
   * Getter method for the the property loopVariable.
   * @return The current value of property loopVariable.
   */
  public String getLoopVariable() {
    return loopVariable;
  }
  /**
   * Setter method for the the property loopVariable.
   * @param loopVariable The value to set for the property loopVariable.
   */
  public void setLoopVariable(String loopVariable) {
    this.loopVariable = loopVariable;
    this.loopIdxVariable = loopVariable + "Idx";
    this.loopStyleVariable = loopVariable + "Style";
  }

  /**
   * Getter method for the the property sequence.
   * @return The current value of property sequence.
   */
  public String getSequence() {
    return sequence;
  }
  /**
   * Setter method for the the property sequence.
   * @param sequence The value to set for the property sequence.
   */
  public void setSequence(String sequence) {
    this.sequence = sequence;
  }

  /**
   * Getter method for the the property evenStyle.
   * @return The current value of property evenStyle.
   */
  public String getEvenStyle() {
    return evenStyle;
  }
  /**
   * Setter method for the the property evenStyle.
   * @param evenStyle The value to set for the property evenStyle.
   */
  public void setEvenStyle(String evenStyle) {
    this.evenStyle = evenStyle;
  }
  
  /**
   * Getter method for the the property oddStyle.
   * @return The current value of property oddStyle.
   */
  public String getOddStyle() {
    return oddStyle;
  }
  /**
   * Setter method for the the property oddStyle.
   * @param oddStyle The value to set for the property oddStyle.
   */
  public void setOddStyle(String oddStyle) {
    this.oddStyle = oddStyle;
  }
  
  /**
   * Getter method for the the property currentValue.
   * @return The current value of property currentValue.
   */
  public Object getCurrentValue() {
    return currentValue;
  }

  /**
   * Getter method for the the property templateInterpreter.
   * @return The current value of property templateInterpreter.
   */
  public TemplateInterpreter getTemplateInterpreter() {
    return interpreter;
  }

  /**
   * Getter method for the the property iterationIdx.
   * @return The current value of property iterationIdx.
   */
  public int getIterationIdx() {
    if (iterationIdx == null) {
      return -1;
    }
    return iterationIdx.intValue();
  }


  /**
   * Getter method for the the property sequenceHolder.
   * @return The current value of property sequenceHolder.
   */
  public Object getSequenceHolder() {
    return sequenceHolder;
  }

  /**
   * Getter method for the the property sequenceIterator.
   * @return The current value of property sequenceIterator.
   */
  public Iterator getSequenceIterator() {
    return sequenceIterator;
  }


  /**
   * Sets the format for printing integer values.
   * @param format The format string to set.
   */
  public void setIntegerFormat(String format) {
    this.customFormat = true;
    this.integerFormat = format;
  }

  /**
   * Sets the format for printing double values.
   * @param format The format string to set.
   */
  public void setDoubleFormat(String format) {
    this.customFormat = true;
    this.doubleFormat = format;
  }

  /**
   * Sets the format for printing Date values.
   * @param format The format string to set.
   */
  public void setDateFormat(String format) {
    this.customFormat = true;
    this.dateFormat = format;
  }

  /**
   * Sets the format for printing Time values.
   * @param format The format string to set.
   */
  public void setTimeFormat(String format) {
    this.customFormat = true;
    this.timeFormat = format;
  }

  /**
   * Sets the format for printing Timestamp values.
   * @param format The format string to set.
   */
  public void setTimestampFormat(String format) {
    this.customFormat = true;
    this.timestampFormat = format;
  }

  /**
   * Gets the page context.
   * @return The current page context.
   */
  public PageContext getPageContext() {
    return this.pageContext;
  }
  /**
   * @see javax.servlet.jsp.tagext.Tag#setPageContext(javax.servlet.jsp.PageContext)
   */
  public void setPageContext(PageContext pageContext) {
    this.pageContext = pageContext;    
  }
  
  /**
   * @see javax.servlet.jsp.tagext.Tag#setParent(javax.servlet.jsp.tagext.Tag)
   */
  public void setParent(Tag parent) {
    this.parent = parent;    
  }

  /**
   * @see javax.servlet.jsp.tagext.Tag#getParent()
   */
  public Tag getParent() {
    return parent;
  }

  /**
   * Evaluates the given expression.
   * @param text The text to evaluate.
   * @return The evaluation result.
   * @exception JspException In case the evaluation has failed.
   */
  protected String evalTemplate(String text) throws JspException {
    try {
      return getTemplateInterpreter().execute(text, templateContext);
    } catch (TemplateException ex) {
      throw new JspException("Unable to excecute template: " + ex, ex);
    }
  }
  
  /**
   * @see javax.servlet.jsp.tagext.Tag#doStartTag()
   */
  public int doStartTag() throws JspException {
    PageContext ctx = this.pageContext;
    
    // initializing expression interpreter
    this.templateContext = null;
    this.interpreter = TemplateInterpreterHelper.getTemplateInterpreter(ctx);
    this.templateContext = getTemplateContext();
    if (customFormat) {
      // configuring custom format helper
      FormatHelper formatHelper = (FormatHelper) templateContext.getFormatHelper().clone();
      if (integerFormat != null) {
        formatHelper.setIntegerFormat(integerFormat);
      }
      if (doubleFormat != null) {
        formatHelper.setDoubleFormat(doubleFormat);
      }
      if (dateFormat != null) {
        formatHelper.setDateFormat(dateFormat);
      }
      if (timeFormat != null) {
        formatHelper.setTimeFormat(timeFormat);
      }
      if (timestampFormat != null) {
        formatHelper.setTimestampFormat(timestampFormat);
      }
      templateContext.setFormatHelper(formatHelper);
    }
    
    try {
      sequenceHolder = evalSequenceHolder();
    } catch (ExpressionException e) {
      throw new JspException("Failed to evaluate sequence expression: " + e.getMessage(), e);
    } catch (Throwable e) {
      throw new JspException("Failed to evaluate sequence expression: " + e.getMessage(), e);
    }

    if (sequenceHolder == null) {
      return SKIP_BODY;
    }
    
    if (sequenceHolder instanceof Collection) {
      sequenceIterator = ((Collection) sequenceHolder).iterator();
    } else if (sequenceHolder instanceof Iterator) {
      sequenceIterator = (Iterator) sequenceHolder;
    } else if (sequenceHolder instanceof Map) {
      sequenceIterator = ((Map) sequenceHolder).keySet().iterator();
    } else if (sequenceHolder instanceof String) {
      // sequence can be a string like 1-10 which means, we will iterate from 1 to 10 in steps of 1.
      StringTokenizer strTkn = new StringTokenizer(sequenceHolder.toString(), "-");
      if (strTkn.countTokens() == 2) {
        String strFrom = strTkn.nextToken();
        String strTo = strTkn.nextToken();
        
        int from = -1;
        try {
          from = Integer.parseInt(strFrom);
        } catch (NumberFormatException ex) {
          throw new JspException("The attribute 'sequence' at item 'foreach' doesn't evaluate to a valid sequence.", ex);
        }

        int to = -1;
        try {
          to = Integer.parseInt(strTo);
        } catch (NumberFormatException ex) {
          throw new JspException("The attribute 'sequence' at item 'foreach' doesn't evaluate to a valid sequence.", ex);
        }
        
        List sequence = new ArrayList();
        for (int i = from;  i <= to; i++) {
          sequence.add(new Integer(i));
        }
        sequenceIterator = sequence.iterator();
      } else {
        // sequence can be a string like 1,2,3,4,5 which means, we will iterate over the list of comma separated values.
        strTkn = new StringTokenizer(sequenceHolder.toString(), ",");
        if (strTkn.countTokens() < 2) {
          throw new JspException("The attribute 'sequence' at item 'foreach' doesn't evaluate to a valid sequence.");
        }
        List sequence = new ArrayList();
        while (strTkn.hasMoreTokens()) {
          sequence.add(strTkn.nextToken());
        }
        sequenceIterator = sequence.iterator();
      }
    } else if (sequenceHolder instanceof Object[]) {
      List sequenceList = Arrays.asList((Object[]) sequenceHolder);
      sequenceIterator = sequenceList.iterator();
    } else {
      try {
        Method iteratorMethod = sequenceHolder.getClass().getMethod("iterator", iteratorAttributes);
        sequenceIterator = (Iterator) iteratorMethod.invoke(sequenceHolder, iteratorParameters);
      } catch (SecurityException ex) {
        throw new JspException("The attribute 'sequence' at item 'foreach' doesn't evaluate to a valid sequence.", ex);
      } catch (NoSuchMethodException ex) {
        throw new JspException("The attribute 'sequence' at item 'foreach' doesn't evaluate to a valid sequence.", ex);
      } catch (ClassCastException ex) {
        throw new JspException("The attribute 'sequence' at item 'foreach' doesn't evaluate to a valid sequence.", ex);
      } catch (IllegalArgumentException ex) {
        throw new JspException("The attribute 'sequence' at item 'foreach' doesn't evaluate to a valid sequence.", ex);
      } catch (IllegalAccessException ex) {
        throw new JspException("The attribute 'sequence' at item 'foreach' doesn't evaluate to a valid sequence.", ex);
      } catch (InvocationTargetException ex) {
        throw new JspException("The attribute 'sequence' at item 'foreach' doesn't evaluate to a valid sequence.", ex);
      }
    }

    // resetting iteration index
    this.iterationIdx = null;
    
    this.currentValue = null;
    if (sequenceIterator.hasNext()) {
      this.currentValue = sequenceIterator.next();
      this.iterationIdx = new SequenceIndex();
    }
    
    if (currentValue == null) {
      return SKIP_BODY;
    }
    this.oldLoopValue = setAttribute(loopVariable, currentValue);
    this.oldLoopIndex = setAttribute(loopIdxVariable, iterationIdx);
    this.oldLoopStyle = setAttribute(loopStyleVariable, evenStyle);
    return EVAL_BODY_INCLUDE;
  }

  /**
   * Evaluates the sequence holder.
   * @return The sequence holder.
   * @throws UjacException In case the evaluation failed.
   */
  protected Object evalSequenceHolder() throws UjacException {
    return interpreter.evalObjectExpr(sequence, templateContext);
  }
  
  /**
   * @see javax.servlet.jsp.tagext.IterationTag#doAfterBody()
   */
  public int doAfterBody() throws JspException {
    if (!sequenceIterator.hasNext()) {
      return SKIP_BODY;
    }
    
    this.currentValue = sequenceIterator.next();
    this.iterationIdx.increment();
    setAttribute(loopVariable, currentValue);
    setAttribute(loopStyleVariable, isEvenIteration() ? evenStyle : oddStyle);

    return EVAL_BODY_AGAIN;
  }
  
  /**
   * @see javax.servlet.jsp.tagext.Tag#doEndTag()
   */
  public int doEndTag() throws JspException {
    setAttribute(loopVariable, oldLoopValue);
    setAttribute(loopIdxVariable, oldLoopIndex);
    setAttribute(loopStyleVariable, oldLoopStyle);
    return EVAL_PAGE;
  }

  /**
   * @see javax.servlet.jsp.tagext.Tag#release()
   */
  public void release() {
    this.loopVariable = null;
    this.loopIdxVariable = null;
    this.sequence = null;
    this.evenStyle = DEFAULT_EVEN_STYLE;
    this.oddStyle = DEFAULT_ODD_STYLE;
    
    this.sequenceHolder = null;    
    this.sequenceIterator = null;    
    this.currentValue = null;
    this.iterationIdx = null;
    this.templateContext = null;    
  }

  /**
   * Gets the template context.
   * @return The template context.
   */
  public TemplateContext getTemplateContext() {
    if (templateContext != null) {
      return templateContext;
    }
    return TemplateInterpreterHelper.getTemplateContext(pageContext, this);
  }
  /**
   * Sets an attribute, both in template context and request.
   * @param key The attribute key.
   * @param value The attribute value to set.
   * @return The old value.
   */
  public Object setAttribute(String key, Object value) {
    TemplateContext ctx = getTemplateContext();
    if ((key == null) || (value == null)) {
      return null;
    }
    Object oldValue = ctx.setProperty(key, value);
    pageContext.setAttribute(key, value, PageContext.REQUEST_SCOPE);
    return oldValue;
  }
  /**
   * Gets an attribute from template context.
   * @param key The attribute key.
   * @return The current value.
   */
  public Object getAttribute(String key) {
    TemplateContext ctx = getTemplateContext();
    if (key == null) {
      return null;
    }
    return ctx.getProperty(key);
  }

  /**
   * Checks, whether the current iteration step is a even one or not.
   * @return true, if the current iteration step is a even one, else false.
   */
  public boolean isEvenIteration() {
    if (iterationIdx == null) {
      return false;
    }
    return (iterationIdx.intValue() % 2) == 0;
  }

}
