/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.web.tag;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.jsp.JspException;

import org.ujac.util.BeanUtils;
import org.ujac.util.table.Column;
import org.ujac.util.table.GroupRow;
import org.ujac.util.table.LayoutHints;
import org.ujac.util.table.Row;
import org.ujac.util.table.TableConstants;
import org.ujac.util.table.TableException;

/**
 * Name: PrintFooterRowTag<br>
 * Description: A custom tag class printing footer rows.
 * 
 * @author lauerc
 */
public class PrintFooterRowTag extends PrintRowTag {

  /** The serial version UID. */
  private static final long serialVersionUID = 3691043175145944371L;

  /** The 'forceFooter' attribute. */
  private boolean forceFooter = false;
  
  /** The footer row. */
  private Row footerRow = null;


  /**
   * Getter method for the the property forceFooter.
   * @return The current value of property forceFooter.
   */
  public boolean isForceFooter() {
    return forceFooter;
  }
  /**
   * Setter method for the the property forceFooter.
   * @param forceFooter The value to set for the property forceFooter.
   */
  public void setForceFooter(boolean forceFooter) {
    this.forceFooter = forceFooter;
  }
  
  /**
   * Checks the row type.
   * @param row The row to check.
   * @return true if the row is valid for output, else false.
   */
  protected boolean checkRowType(Row row) {
    if ((row == null) || !TableConstants.ROW_TYPE_FOOTER.equals(row.getType())) {
      if (forceFooter && !rowContainer.getSequenceIterator().hasNext()) {
        row = new GroupRow(rowContainer.getTable(), TableConstants.ROW_TYPE_FOOTER, null);
      } else {
        return false;
      }
    }
    footerRow = row;
    rowContainer.getTemplateContext().setProperty(rowContainer.getLoopVariable(), row);
    return true;
  }

  /**
   * Performs the row output.
   * @exception JspException Thrown in case something went wrong while closing the foreach item.
   */
  protected void outputRow() throws JspException {
    try {
      PrintTableTag parent = getRowContainer();
      String template = parent.getTemplate();

      // processing style
      String rowStyle = alternateRowStyle();
      
      if (BeanUtils.isEmpty(template)) {
        super.outputRow();
      } else {
        try {
          setAttribute(PrintTableTag.ATTRIBUTE_TABLE_ROW_STYLE, rowStyle);
          setAttribute(PrintTableTag.ATTRIBUTE_TABLE_ROW_HEIGHT, getHeight());

          List actualCells = new ArrayList();
          Row row = footerRow;
          setAttribute(PrintTableTag.ATTRIBUTE_TABLE_ROW_TYPE, row.getType());
          Column[] columns = parent.getVisibleColumns();
          for (int i = 0; i < columns.length; i++) {
            Column column = columns[i];
            int columnIdx = column.getIndex();
            
            boolean visible = true;
            if (columnIdx >= 0) {
              visible = row.isVisible(columnIdx);
            }
            LayoutHints layoutHints = column.getLayoutHints();
            RowCellHolder rowCell = registeredRowCells[i];
            String width = null;
            String height = this.getHeight();
            String valign = this.getValign();
            String halign = null;
            if (columnIdx != -1) {
              switch (row.getAlign(columnIdx)) {
                case LayoutHints.RIGHT:
                  halign = "right";
                  break;
                case LayoutHints.CENTER:
                  halign = "center";
                  break;
                default:
                  halign = "left";
                  break;
              }
            }
 
            String cellStyle = rowStyle;
            float paddingLeft = this.getPaddingLeft();
            float paddingRight = this.getPaddingRight();
            float paddingTop = this.getPaddingTop();
            float paddingBottom = this.getPaddingBottom();
            
            String content = null;
            int colspan = 1;
            if (rowCell != null) {
              visible = true;
              width = rowCell.getWidth();
              if (rowCell.getHeight() != null) {
                height = rowCell.getHeight();
              }
              if (rowCell.getStyle() != null) {
                cellStyle = rowCell.getStyle();
              }
              if (rowCell.getValign() != null) {
                valign = rowCell.getValign();
              }
              String tmpHalign = rowCell.getHalign();
              if (tmpHalign != null) {
                halign = tmpHalign;
              }
              if (rowCell.getPaddingLeft() >= 0.0F) {
                paddingLeft = rowCell.getPaddingLeft();
              }
              if (rowCell.getPaddingRight() >= 0.0F) {
                paddingRight = rowCell.getPaddingRight();
              }
              if (rowCell.getPaddingTop() >= 0.0F) {
                paddingTop = rowCell.getPaddingTop();
              }
              if (rowCell.getPaddingBottom() >= 0.0F) {
                paddingBottom = rowCell.getPaddingBottom();
              }
              if (rowCell.hasContent()) {
                content = rowContainer.evalTemplate(rowCell.getContent());
              }
              if (rowCell.getColspan() > 0) {
                colspan = rowCell.getColspan();
              }
              if (rowCell.isFillRow()) {
                colspan = columns.length - i;
              }
            }
            if ((width == null) && (layoutHints.getWidth() > 0)) {
              width = Integer.toString((int) layoutHints.getWidth());
            }
            if (content == null) {
              if ((columnIdx != -1) && row.isVisible(columnIdx)) {
                content = row.getString(columnIdx);
              } else {
                content = parent.getDefaultContent(i);
                if (content != null) {
                  content = rowContainer.evalTemplate(content);
                }
              }
            }
            if (content == null) {
              content = "&nbsp;";
            }
            
            // setting cell
            CellData cellData = new CellData(column.getName(), content, colspan);
            cellData.setVisible(visible);
            cellData.setStyle(cellStyle);
            cellData.setWidth(width);
            cellData.setHeight(height);
            cellData.setHalign(halign);
            cellData.setValign(valign);
            cellData.setPaddingLeft((int) paddingLeft);
            cellData.setPaddingRight((int) paddingRight);
            cellData.setPaddingTop((int) paddingTop);
            cellData.setPaddingBottom((int) paddingBottom);
            actualCells.add(cellData);
            if (colspan > 1) {
              i += colspan - 1;
            }
          }

          setAttribute(PrintTableTag.ATTRIBUTE_TABLE_CELLS, actualCells);

          pageContext.include(template + "-footer.jsp");
          parent.setFooterPrinted(true);
        } catch (ServletException ex) {
          throw new JspException("Failed to include template file '" + template + "-body.jsp'.");
        }
      }
    } catch (IOException ex) {
      throw new JspException("Unable to print table row: " + ex.getMessage(), ex);
    } catch (TableException ex) {
      throw new JspException("Unable to print table row: " + ex.getMessage(), ex);
    } catch (ClassCastException ex) {
      throw new JspException("Type cast problem detected: " + ex.getMessage(), ex);
    }
  }
  
  /**
   * @see org.ujac.web.tag.PrintRowTag#release()
   */
  public void release() {
    super.release();
    this.forceFooter = false;
  }

}
