/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn't download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.web.tag;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;

import org.ujac.util.BeanUtils;

import org.ujac.util.table.Column;
import org.ujac.util.table.LayoutHints;
import org.ujac.util.table.Row;
import org.ujac.util.table.TableException;

/**
 * Name: PrintHeaderRowTag<br>
 * Description: A custom tag class printing header rows.
 * 
 * @author lauerc
 */
public class PrintHeaderRowTag extends PrintRowTag {
  
  /** The serial version UID. */
  private static final long serialVersionUID = 3688790280065267248L;

  /**
   * Checks the row type.
   * @param row The row to check.
   * @return true if the row is valid for output, else false.
   */
  protected boolean checkRowType(Row row) {
    if (rowContainer.isHeaderPrinted()) {
      return false;
    }
    return true;
  }
  
  /**
   * @see javax.servlet.jsp.tagext.BodyTagSupport#doEndTag()
   */
  public int doEndTag() throws JspException {
    if (isSkipped()) {
      setSkipped(false);
      return EVAL_PAGE;
    }

    // performing output of leading extra rows.
    if (extraRowsBefore != null) {
      List extraRows = extraRowsBefore;
      int numExtraRows = extraRows.size();
      for (int i = 0; i < numExtraRows; i++) {
        ExtraRowHolder extraRow = (ExtraRowHolder) extraRows.get(i);
        extraRow.outputRow(pageContext.getOut());
      }
    }
    
    // performing output of data row.
    outputRow();

    // performing output of trailing extra rows.
    if (extraRowsAfter != null) {
      List extraRows = extraRowsAfter;
      int numExtraRows = extraRows.size();
      for (int i = 0; i < numExtraRows; i++) {
        ExtraRowHolder extraRow = (ExtraRowHolder) extraRows.get(i);
        extraRow.outputRow(pageContext.getOut());
      }
    }
    rowContainer.setHeaderPrinted(true);
    return EVAL_PAGE;
  }

  /**
   * Performs the row output.
   * @exception JspException Thrown in case something went wrong while closing the foreach item.
   */
  protected void outputRow() throws JspException {
    try {
      PrintTableTag parent = getRowContainer();
      String template = parent.getTemplate();
      String style = this.getStyle();

      if (BeanUtils.isEmpty(template)) {
        JspWriter out = pageContext.getOut();
  
        out.print("  <thead><tr");
        if (style != null) {
          out.print(" class=\"" + style + "\"");
        }
        out.println(">");
  
        Row row = parent.getCurrentRow();
        Column[] columns = parent.getVisibleColumns();
        for (int i = 0; i < columns.length; i++) {
          Column column = columns[i];
          int columnIdx = column.getIndex();
          
          LayoutHints layoutHints = column.getLayoutHints();
          RowCellHolder rowCell = registeredRowCells[i];
          String width = null;
          String height = this.getHeight();
          style = this.getStyle();
          String valign = this.getValign();
          String halign = null;
          if (columnIdx != -1) {
            int align = layoutHints.getAlign();
            if (row != null) {
              align = row.getAlign(columnIdx);
            }
            switch (align) {
              case LayoutHints.LEFT:
                halign = "left";
                break;
              case LayoutHints.RIGHT:
                halign = "right";
                break;
              case LayoutHints.CENTER:
                halign = "center";
                break;
              default:
                halign = "left";
                break;
            }
          }
          
          float paddingLeft = this.getPaddingLeft();
          float paddingRight = this.getPaddingRight();
          float paddingTop = this.getPaddingTop();
          float paddingBottom = this.getPaddingBottom();
          String content = null;
          int colspan = 1;
          if (rowCell != null) {
            width = rowCell.getWidth();
            if (rowCell.getHeight() != null) {
              height = rowCell.getHeight();
            }
            if (rowCell.getStyle() != null) {
              style = rowCell.getStyle();
            }
            if (rowCell.getValign() != null) {
              valign = rowCell.getValign();
            }
            String tmpHalign = rowCell.getHalign();
            if (tmpHalign != null) {
              halign = tmpHalign;
            }
            if (rowCell.getPaddingLeft() >= 0.0F) {
              paddingLeft = rowCell.getPaddingLeft();
            }
            if (rowCell.getPaddingRight() >= 0.0F) {
              paddingRight = rowCell.getPaddingRight();
            }
            if (rowCell.getPaddingTop() >= 0.0F) {
              paddingTop = rowCell.getPaddingTop();
            }
            if (rowCell.getPaddingBottom() >= 0.0F) {
              paddingBottom = rowCell.getPaddingBottom();
            }
            if (rowCell.hasContent()) {
              content = rowContainer.evalTemplate(rowCell.getContent());
            }
            if (rowCell.getColspan() > 0) {
              colspan = rowCell.getColspan();
            }
            if (rowCell.isFillRow()) {
              colspan = columns.length - i;
            }
          }
          String cellStyle = style;
          if ((width == null) && (layoutHints.getWidth() > 0)) {
            width = Integer.toString((int) layoutHints.getWidth()) + '%';
          }
          if (content == null) {
            boolean visible = layoutHints.isVisible();
            if ((row != null) && (columnIdx >= 0)) {
              visible = row.isVisible(columnIdx);
            }
            if ((columnIdx != -1) && visible) {
              content = column.getLayoutHints().getHeader();
            } else {
              content = "&nbsp;";
            }
          }
          // printing cell
          out.print("    <th");
          out.print(getCellAttributes(
              cellStyle, width, height, halign, valign, colspan, 
              paddingLeft, paddingRight, paddingTop, paddingBottom
          ));
          out.print(">");
          out.print(content);
          out.println("</th>");
          if (colspan > 1) {
            i += colspan - 1;
          }
        }
        out.println("  </tr></thead>");
        out.println("  <tbody>");
      } else {

        try {
          setAttribute(PrintTableTag.ATTRIBUTE_TABLE_ROW_STYLE, style);
          setAttribute(PrintTableTag.ATTRIBUTE_TABLE_ROW_HEIGHT, getHeight());

          float totalWidth = 0.0F;
          List actualCells = new ArrayList();
          Row row = parent.getCurrentRow();
          setAttribute(PrintTableTag.ATTRIBUTE_TABLE_ROW_TYPE, "header");
          Column[] columns = parent.getVisibleColumns();
          for (int i = 0; i < columns.length; i++) {
            Column column = columns[i];
            int columnIdx = column.getIndex();
            
            LayoutHints layoutHints = column.getLayoutHints();
            RowCellHolder rowCell = registeredRowCells[i];
            String width = null;
            String height = this.getHeight();
            style = this.getStyle();
            String valign = this.getValign();
            String halign = null;
            if (columnIdx != -1) {
              int align = layoutHints.getAlign();
              if (row != null) {
                align = row.getAlign(columnIdx);
              }
              switch (align) {
                case LayoutHints.LEFT:
                  halign = "left";
                  break;
                case LayoutHints.RIGHT:
                  halign = "right";
                  break;
                case LayoutHints.CENTER:
                  halign = "center";
                  break;
                default:
                  halign = "left";
                  break;
              }
            }
            
            float paddingLeft = this.getPaddingLeft();
            float paddingRight = this.getPaddingRight();
            float paddingTop = this.getPaddingTop();
            float paddingBottom = this.getPaddingBottom();
            String content = null;
            int colspan = 1;
            if (rowCell != null) {
              width = rowCell.getWidth();
              if (rowCell.getHeight() != null) {
                height = rowCell.getHeight();
              }
              if (rowCell.getStyle() != null) {
                style = rowCell.getStyle();
              }
              if (rowCell.getValign() != null) {
                valign = rowCell.getValign();
              }
              String tmpHalign = rowCell.getHalign();
              if (tmpHalign != null) {
                halign = tmpHalign;
              }
              if (rowCell.getPaddingLeft() >= 0.0F) {
                paddingLeft = rowCell.getPaddingLeft();
              }
              if (rowCell.getPaddingRight() >= 0.0F) {
                paddingRight = rowCell.getPaddingRight();
              }
              if (rowCell.getPaddingTop() >= 0.0F) {
                paddingTop = rowCell.getPaddingTop();
              }
              if (rowCell.getPaddingBottom() >= 0.0F) {
                paddingBottom = rowCell.getPaddingBottom();
              }
              if (rowCell.hasContent()) {
                content = rowContainer.evalTemplate(rowCell.getContent());
              }
              if (rowCell.getColspan() > 0) {
                colspan = rowCell.getColspan();
              }
              if (rowCell.isFillRow()) {
                colspan = columns.length - i;
              }
            }
            String cellStyle = style;
            if ((width == null) && (layoutHints.getWidth() > 0)) {
              width = Integer.toString((int) layoutHints.getWidth());
            }
            boolean visible = layoutHints.isVisible();
            if (content == null) {
              if ((row != null) && (columnIdx >= 0)) {
                visible = row.isVisible(columnIdx);
              }
              content = column.getLayoutHints().getHeader();
            }

            // setting cell
            CellData cellData = new CellData(column.getName(), content, colspan);
            cellData.setVisible(visible);
            cellData.setStyle(cellStyle);
            cellData.setWidth(width);
            cellData.setHeight(height);
            cellData.setHalign(halign);
            cellData.setValign(valign);
            cellData.setPaddingLeft((int) paddingLeft);
            cellData.setPaddingRight((int) paddingRight);
            cellData.setPaddingTop((int) paddingTop);
            cellData.setPaddingBottom((int) paddingBottom);
            actualCells.add(cellData);
            if (colspan > 1) {
              i += colspan - 1;
            }
            totalWidth += layoutHints.getWidth();
          }

          setAttribute(PrintTableTag.ATTRIBUTE_TABLE_CELLS, actualCells);
          setAttribute(PrintTableTag.ATTRIBUTE_TABLE_WIDTH, new Float(totalWidth));

          pageContext.include(template + "-header.jsp");
        } catch (ServletException ex) {
          log.error("Failed to include template file '" + template + "-body.jsp': " + ex.getMessage(), ex);
          throw new JspException("Failed to include template file '" + template + "-body.jsp'.");
        }
      }
    } catch (IOException ex) {
      throw new JspException("Unable to print table row: " + ex.getMessage(), ex);
    } catch (TableException ex) {
      throw new JspException("Unable to print table row: " + ex.getMessage(), ex);
    } catch (ClassCastException ex) {
      throw new JspException("Parent tag of row-cell tag has to be print-row tag.");
    }
  }
  
}
