/*
 * Copyright (C) 2003 - 2008 by Christian Lauer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you didn'tin download this code from the following link, you should check if
 * you aren't using an obsolete version:
 * http://sourceforge.net/projects/ujac
 */
 
package org.ujac.web.tag;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;

import org.ujac.util.BeanUtils;
import org.ujac.util.CollectionUtils;
import org.ujac.util.StringUtils;
import org.ujac.util.UjacException;
import org.ujac.util.UjacTypes;

import org.ujac.util.table.BeanTable;
import org.ujac.util.table.Column;
import org.ujac.util.table.ColumnNotDefinedException;
import org.ujac.util.table.DataTable;
import org.ujac.util.table.DefaultColumn;
import org.ujac.util.table.LayoutHints;
import org.ujac.util.table.Row;
import org.ujac.util.table.Table;
import org.ujac.util.table.TableConstants;

import org.ujac.util.template.TemplateContext;
import org.ujac.util.text.FormatHelper;

/**
 * Name: PrintTableTag<br>
 * Description: A custom tag class for convinient output of tables.
 * 
 * @author lauerc
 */
public class PrintTableTag extends ForeachTag {
  
  /**
   * Name: DummyColumn<br>
   * Description: A class for dummy columns which are added through the insert-column tag.
   */
  public class DummyColumn extends DefaultColumn {

    /** The serial version UID. */
    private static final long serialVersionUID = 3618696418123134000L;

    /**
     * Constructs a DummyColumn instance with specific attributes.
     * @param name The column name.
     */
    protected DummyColumn(String name) {
      super(null, name, -1);
    }
    
    /**
     * Writes the object's data to the given stream.
     * @param s The stream to write to
     * @exception IOException In case the data output failed.
     */
    private void writeObject(ObjectOutputStream s) throws IOException {
      writeData(s);
    }
    /**
     * Reads the object's data from the given stream.
     * @param s The stream to read from.
     * @exception IOException In case the data reading failed.
     * @exception ClassNotFoundException In case the class to deserialize could not be found
     */
    private void readObject(ObjectInputStream s) throws IOException, ClassNotFoundException  {
      readData(s);
    }

  }

  /** Constant for the insert direction 'before'. */
  public static final int DIRECTION_BEFORE = 1;
  /** Constant for the insert direction 'after'. */
  public static final int DIRECTION_AFTER = 2;

  /** Constant for the _TABLE_ROW_STYLE attribute. */
  public static final String ATTRIBUTE_TABLE_ROW_STYLE = "_TABLE_ROW_STYLE";
  /** Constant for the _TABLE_ROW_TYPE attribute. */
  public static final String ATTRIBUTE_TABLE_ROW_TYPE = "_TABLE_ROW_TYPE";
  /** Constant for the _TABLE_ROW_HEIGHT attribute. */
  public static final String ATTRIBUTE_TABLE_ROW_HEIGHT = "_TABLE_ROW_HEIGHT";
  /** Constant for the _TABLE_CELLS attribute. */
  public static final String ATTRIBUTE_TABLE_CELLS = "_TABLE_CELLS";
  /** Constant for the _TABLE_WIDTH attribute. */
  public static final String ATTRIBUTE_TABLE_WIDTH = "_TABLE_WIDTH";

  /** The table ID. */
  private String id = null;
  /** The table width. */
  private String width = null;
  /** The table height. */
  private String height = null;
  /** The table border. */
  private int border = 0;
  /** The cell padding. */
  private int cellpadding = 0;
  /** The cell spacing. */
  private int cellspacing = 0;

  /** The maximum level of importance to support. */
  private int levelFilter = -1;
  /** The template tag. */
  private String template = null;

  /** The table. */
  private Table table = null;
  /** The column list. */
  private List columnList = new ArrayList();
  /** The default cell content list. */
  private List defaultCellContents = new ArrayList();
  /** The column index. */
  private Map columnIndex = null;
  /** The table columns. */
  private Column[] columns = null;
  /** The column widths. */
  private float[] widths = null;

  /** The 'header printed' flag. */
  private boolean headerPrinted = false;
  /** The 'footer printed' flag. */
  private boolean footerPrinted = false;

  /** The column attribute map. */
  private Map columnAttributeMaps = null;

  /** The old table width. */
  private Object oldTableWidth = null;
  /** The old table cells. */
  private Object oldTableCells = null;
  /** The old table row style. */
  private Object oldTableRowStyle = null;
  /** The old table row height. */
  private Object oldTableRowHeight = null;
  /** The old table row type. */
  private Object oldTableRowType = null;
  
  /** The cleared flag for column definitions. */
  private boolean columnDefinitionsCleared = false;

  /**
   * Getter method for the the property border.
   * @return The current value of property border.
   */
  public int getBorder() {
    return border;
  }
  /**
   * Setter method for the the property border.
   * @param border The value to set for the property border.
   */
  public void setBorder(int border) {
    this.border = border;
  }

  /**
   * Getter method for the the property id.
   * @return The current value of property id.
   */
  public String getId() {
    return id;
  }
  /**
   * Setter method for the the property id.
   * @param id The value to set for the property id.
   */
  public void setId(String id) {
    this.id = id;
  }

  /**
   * Getter method for the the property width.
   * @return The current value of property width.
   */
  public String getWidth() {
    return width;
  }
  /**
   * Setter method for the the property width.
   * @param width The value to set for the property width.
   */
  public void setWidth(String width) {
    this.width = width;
  }

  /**
   * Getter method for the the property height.
   * @return The current value of property height.
   */
  public String getHeight() {
    return height;
  }
  /**
   * Setter method for the the property height.
   * @param height The value to set for the property height.
   */
  public void setHeight(String height) {
    this.height = height;
  }

  /**
   * Getter method for the the property cellpadding.
   * @return The current value of property cellpadding.
   */
  public int getCellpadding() {
    return cellpadding;
  }
  /**
   * Setter method for the the property cellpadding.
   * @param cellpadding The value to set for the property cellpadding.
   */
  public void setCellpadding(int cellpadding) {
    this.cellpadding = cellpadding;
  }

  /**
   * Getter method for the the property cellspacing.
   * @return The current value of property cellspacing.
   */
  public int getCellspacing() {
    return cellspacing;
  }
  /**
   * Setter method for the the property cellspacing.
   * @param cellspacing The value to set for the property cellspacing.
   */
  public void setCellspacing(int cellspacing) {
    this.cellspacing = cellspacing;
  }
  
  /**
   * Getter method for the the property levelFilter.
   * @return The current value of property levelFilter.
   */
  public int getLevelFilter() {
    return levelFilter;
  }
  /**
   * Setter method for the the property levelFilter.
   * @param levelFilter The value to set for the property levelFilter.
   */
  public void setLevelFilter(int levelFilter) {
    this.levelFilter = levelFilter;
  }

  /**
   * Getter method for the the property template.
   * @return The current value of the property template.
   */
  public String getTemplate() {
    return template;
  }
  /**
   * Setter method for the the property template.
   * @param template The value to set for the property template.
   */
  public void setTemplate(String template) {
    this.template = template;
  }

  /**
   * Getter method for the the property headerPrinted.
   * @return The current value of property headerPrinted.
   */
  public boolean isHeaderPrinted() {
    return headerPrinted;
  }
  /**
   * Setter method for the the property headerPrinted.
   * @param headerPrinted The value to set for the property headerPrinted.
   */
  public void setHeaderPrinted(boolean headerPrinted) {
    this.headerPrinted = headerPrinted;
  }

  /**
   * Getter method for the the property footerPrinted.
   * @return The current value of property footerPrinted.
   */
  public boolean isFooterPrinted() {
    return footerPrinted;
  }
  /**
   * Setter method for the the property footerPrinted.
   * @param footerPrinted The value to set for the property footerPrinted.
   */
  public void setFooterPrinted(boolean footerPrinted) {
    this.footerPrinted = footerPrinted;
  }

  /**
   * Gets all visible columns.
   * @return An array holding all visible columns.
   */
  public Column[] getVisibleColumns() {
    if (columns == null) {
      FormatHelper formatHelper = getTemplateContext().getFormatHelper();
      int numColumns = columnList.size();
      columns = new Column[numColumns];
      widths = new float[numColumns];
      for (int i = 0; i < numColumns; i++) {
        Column column = (Column) columnList.get(i);
        columns[i] = column;
        widths[i] = column.getLayoutHints().getWidth();
        if (column.getFormat() == null) {
          switch (column.getType()) {
          case UjacTypes.TYPE_INT: 
          case UjacTypes.TYPE_LONG: 
            column.setFormat(formatHelper.getIntegerFormat());
            break;
          case UjacTypes.TYPE_FLOAT: 
          case UjacTypes.TYPE_DOUBLE: 
            column.setFormat(formatHelper.getDoubleFormat());
            break;
          case UjacTypes.TYPE_DATE: 
            column.setFormat(formatHelper.getDateFormat());
            break;
          case UjacTypes.TYPE_TIME: 
            column.setFormat(formatHelper.getTimeFormat());
            break;
          case UjacTypes.TYPE_TIMESTAMP: 
            column.setFormat(formatHelper.getTimestampFormat());
            break;
          }
        }
      }
    }
    return columns;
  }
  
  /**
   * Gets the number of columns by row to print.
   * @return The number of columns by row.
   */
  protected int getNumColumns() {
    return columnList.size();
  }
  
  /**
   * Gets the index of the given column.
   * @param columnName The name of the column to get the index for.
   * @return The index of the column.
   */
  protected int getColumnIndex(String columnName) {
    Column[] columns = getVisibleColumns();
    int numColumns = columns.length;
    if (this.columnIndex == null) {
      this.columnIndex = new HashMap();
      for (int i = 0; i < numColumns; i++) {
        Column col = columns[i];
        columnIndex.put(col.getName(), new Integer(i));
      }
    }
    Integer idx = (Integer) columnIndex.get(columnName);
    if (idx == null) {
      return -1;
    }
    return idx.intValue();
  }
  
  /**
   * Gets the default contents for the given column.
   * @param idx The index of the column to get the default contents for.
   * @return The index of the column.
   */
  protected String getDefaultContent(int idx) {
    return (String) defaultCellContents.get(idx);
  }

  /**
   * Checks whether the table output has been started or not.
   * @return true if the table output has been started, else false.
   */
  public boolean isOutputStarted() {
    return (columns != null);
  }
  
  /**
   * Getter method for the the property table.
   * @return The current value of property table.
   */
  public Table getTable() {
    return table;
  }

  /**
   * Getter method for the the property widths.
   * @return The current value of property widths.
   */
  public float[] getWidths() {
    return widths;
  }

  /**
   * Gets the current row.
   * @return The current row.
   */
  protected Row getCurrentRow() {
    return (Row) getCurrentValue();
  }
  
  /**
   * Evaluates the sequence holder.
   * @return The sequence holder.
   * @throws UjacException In case the evaluation failed.
   */
  protected Object evalSequenceHolder() throws UjacException {
    Object seqHolder = super.evalSequenceHolder();
    TemplateContext tctx = getTemplateContext();
    if (seqHolder instanceof Table) {
      this.table = (Table) seqHolder;
    } else if (seqHolder instanceof List) {
      FormatHelper formatHelper = tctx.getFormatHelper();
      List items = (List) seqHolder;
      if (CollectionUtils.isEmpty(items)) {
        // creating empty table.
        this.table = new DataTable();
      } else {
        // creating bean table from list
        this.table = new BeanTable(formatHelper, items);
      }
    } else {
      throw new UjacException("The sequence '" + getSequence() + "' doesn't evaluate to a table.");
    }
    return this.table;
  }
  
  /**
   * @see javax.servlet.jsp.tagext.Tag#doStartTag()
   */
  public int doStartTag() throws JspException {
    super.doStartTag();

    // resetting the column index
    this.columnIndex = null;
    this.columnAttributeMaps = null;
    this.headerPrinted = false;
    this.footerPrinted = false;
    this.columnDefinitionsCleared = false;
    
    setAttribute("_TABLE", table);
    setAttribute("_TABLE_COLUMNS", table);
    
    Column[] columns = table.getVisibleColumns(levelFilter);
    for (int i = 0; i < columns.length; i++) {
      columnList.add(columns[i]);
      defaultCellContents.add(null);
    }

    // saving old table attriutes
    oldTableRowStyle = getAttribute(ATTRIBUTE_TABLE_ROW_STYLE);
    oldTableRowHeight = getAttribute(ATTRIBUTE_TABLE_ROW_HEIGHT);
    oldTableRowType = getAttribute(ATTRIBUTE_TABLE_ROW_TYPE);
    oldTableCells = getAttribute(ATTRIBUTE_TABLE_CELLS);
    oldTableWidth = getAttribute(ATTRIBUTE_TABLE_WIDTH);

    // printing table head
    try {
      PageContext ctx = getPageContext();
      if (BeanUtils.isEmpty(template)) {
        JspWriter out = ctx.getOut();
        out.print("<table border=\"" + border + "\" cellspacing=\"" +
            "" + cellspacing + "\" cellpadding=\"" + cellpadding + "\"");
        if (id != null) {
          out.print(" id=\"" + id + "\"");
        }
        if (width != null) {
          out.print(" width=\"" + width + "\"");
        }
        if (height != null) {
          out.print(" height=\"" + height + "\"");
        }
        out.println(">");
      }
    } catch (IOException ex) {
      throw new JspException("Unable to print table head: " + ex.getMessage(), ex);
    }
    return EVAL_BODY_INCLUDE;
  }

  /**
   * @see javax.servlet.jsp.tagext.Tag#doEndTag()
   */
  public int doEndTag() throws JspException {
    super.doEndTag();
    
    String evenStyle = getEvenStyle(); 
    String oddStyle = getOddStyle();
    
    // closing table
    try {
      PageContext ctx = getPageContext();
      if (BeanUtils.isEmpty(template)) {
        JspWriter out = ctx.getOut();
        out.println("  </tbody>");
        out.println("</table>");
        
        if (id != null) {
          String capitalizedId = StringUtils.capitalize(id);
          String tableVar = id;
          String tableBodyVar = id + "Body";
          Column[] columns = getVisibleColumns();
          out.println("<script type=\"text/javascript\" language=\"JavaScript\"><!--");
          out.print("var ");
          out.print(tableVar);
          out.println(" = null;");
          out.print("var ");
          out.print(tableBodyVar);
          out.println(" = null;");
          out.println();
          out.print("function get");
          out.print(capitalizedId);
          out.println("() {");
          out.print("  if (");
          out.print(tableVar);
          out.println(" == null) {");
          out.print("    ");
          out.print(tableVar);
          out.print(" = document.getElementById('");
          out.print(id);
          out.println("');");
          out.print("    ");
          out.print(tableBodyVar);
          out.print(" = ");
          out.print(tableVar);
          out.println(".getElementsByTagName('tbody').item(0);");
          out.println("  }");
          out.print("  return ");
          out.print(tableVar);
          out.println(";");
          out.println("}");
          out.println();
          out.print("function add");
          out.print(capitalizedId);
          out.print("Row(");
          for (int i = 0; i < columns.length; i++) {
            if (i > 0) {
              out.print(", ");
            }
            out.print(columns[i].getName());
          }
          out.println(") {");
          out.print("  if (");
          out.print(tableVar);
          out.println(" == null) {");
          out.print("    get");
          out.print(capitalizedId);
          out.println("();");
          out.println("  }");
          out.print("  var tableBody = ");
          out.print(tableBodyVar);
          out.println(";");
          out.println("  var rows = tableBody.rows;");
          out.println("  var td = null;");
          out.println("  var tr = document.createElement('tr');");
          out.println("  if (rows.length % 2 == 0) {");
          out.print("    tr.setAttribute('class', '");
          out.print(evenStyle);
          out.println("');");
          out.println("  } else {");
          out.print("    tr.setAttribute('class', '");
          out.print(oddStyle);
          out.println("');");
          out.println("  }");
          for (int i = 0; i < columns.length; i++) {
            Column col = columns[i];
            String colName = col.getName();
            out.println("  td = document.createElement('td');");
            out.print("  if (");
            out.print(colName);
            out.print(" instanceof Element)");
            out.print(" td.appendChild(");
            out.print(colName);
            out.print("); else td.appendChild(document.createTextNode(");
            out.print(colName);
            out.println("));");
            List colAttrs = (List) columnAttributeMaps.get(colName);
            if (colAttrs != null) {
              int numColAttrs = colAttrs.size();
              for (int j = 0; j < numColAttrs; j++) {
                MarkupAttribute attr = (MarkupAttribute) colAttrs.get(j);
                out.print("  td.setAttribute('");
                out.print(attr.getName());
                out.print("', '");
                out.print(attr.getValue());
                out.println("');");
              }
            }
            out.println("  tr.appendChild(td);");
          }
          out.println("  tableBody.appendChild(tr);");
          out.print("}");
          out.println("//-->");
          out.println("</script>");
        }
      } else {
        if (!isFooterPrinted()) {
          Row row = getCurrentRow();
          if ((row == null) || !TableConstants.ROW_TYPE_FOOTER.equals(row.getType())) {
            try {
              ctx.include(template + "-footer.jsp");
            } catch (ServletException ex) {
              throw new JspException("Failed to include template file '" + template + "-footer.jsp'.");
            }
          }
        }
      }
    } catch (IOException ex) {
      throw new JspException("Unable to print closing table tag: " + ex.getMessage(), ex);
    }
    
    // forcing release call
    release();
    
    // restoring old table attriutes
    setAttribute(ATTRIBUTE_TABLE_ROW_STYLE, oldTableRowStyle);
    setAttribute(ATTRIBUTE_TABLE_ROW_HEIGHT, oldTableRowHeight);
    setAttribute(ATTRIBUTE_TABLE_ROW_TYPE, oldTableRowType);
    setAttribute(ATTRIBUTE_TABLE_CELLS, oldTableCells);
    setAttribute(ATTRIBUTE_TABLE_WIDTH, oldTableWidth);
    
    return EVAL_PAGE;
  }

  /**
   * @see javax.servlet.jsp.tagext.Tag#release()
   */
  public void release() {
    super.release();
    this.columnList.clear();
    this.defaultCellContents.clear();
    this.id = null;
    this.width = null;
    this.columns = null;
    this.headerPrinted = false;
    this.border = 0;
    this.template = null;
    this.columnDefinitionsCleared = false;
  }

  /**
   * Defines a column for the table.
   * @param name The name of the column.
   * @param width The width of the column.
   * @param hAlign The horizontal alignment of the column.
   * @param title The title of the column.
   * @param defaultContents The default contents to use for this column.
   * @exception JspException In case the given reference column does not exist. 
   */
  public void defineColumn(String name, float width, int hAlign, String title, String defaultContents) 
    throws JspException {
    
    if (columns != null) {
      return;
    }
    if (!columnDefinitionsCleared) {
      // clearing column definition list
      columnList.clear();
      defaultCellContents.clear();
      columnDefinitionsCleared = true;
    }
    
    // finding reference column
    int numColumns = columnList.size();
   
    Column col = null;
    try {
      col = table.getColumn(name);
    } catch (ColumnNotDefinedException ex) {
      // column not defined yet
    }
    if (col != null) {
      // column already exists
      LayoutHints lh = col.getLayoutHints();
      if (lh != null) {
        lh.setWidth(width);
        lh.setHeader(title);
        lh.setHeaderAlign(hAlign);
        lh.setVisible(true);
      } else {
        col.setLayoutHints(new LayoutHints(
            width,
            title,
            hAlign
        ));
      }
      // checking if column already exists in visible column list ...
      for (int i = 0; i < numColumns; i++) {
        Column tmpCol = (Column) columnList.get(i);
        if (!tmpCol.getName().equals(name)) {
          continue;
        }
        // just redefinining default context for existing column definition
        defaultCellContents.set(i, defaultContents);
        return;
      }
      // Column doesn't exit yet, adding new column definition
      columnList.add(col);
      defaultCellContents.add(defaultContents);
      return;
    }
    Column newCol = new DummyColumn(name);
    newCol.setLayoutHints(new LayoutHints(
        width,
        title,
        hAlign
    ));
  
    columnList.add(newCol);
    defaultCellContents.add(defaultContents);
  }

  /**
   * Inserts a column into the table.
   * @param name The name of the new column.
   * @param width The width of the new column.
   * @param hAlign The horizontal alignment of the new column.
   * @param title The title of the column.
   * @param reference The column at which to insert the new column.
   * @param direction The insert direction. In case null was given
   *   as reference, <code>DIRECTION_BEFORE</code> inserts before the
   *   first colummn, <code>DIRECTION_AFTER</code> appends behind the
   *   last column.
   * @see #DIRECTION_BEFORE
   * @see #DIRECTION_AFTER
   * @param defaultContents The default contents to use for this column.
   * @exception JspException In case the given reference column does not exist. 
   */
  public void insertColumn(String name, float width, int hAlign, String title,
                           String reference, int direction, String defaultContents) 
    throws JspException {
    
    if (columns != null) {
      return;
    }
    
    // finding reference column
    int numColumns = columnList.size();
    int coldx = -1;
    int referenceIdx = -1;
    if (reference != null) {
      for (int i = 0; i < numColumns; i++) {
        Column col = (Column) columnList.get(i);
        if (col.getName().equals(name)) {
          coldx = i;
          if (referenceIdx != -1) {
            break;
          }
        }
        if (col.getName().equals(reference)) {
          referenceIdx = i;
          if (coldx != -1) {
            break;
          }
        }
      }
      if (coldx != -1) {
        // This column already exists, just altering the layout hints.
        Column col = (Column) columnList.get(coldx);
        col.setLayoutHints(new LayoutHints(
            width,
            title,
            hAlign
        ));
        return;
      }
      if (referenceIdx == -1) {
        throw new JspException("The column name '" + reference + "' does not exist.");
      }
      if (direction == DIRECTION_AFTER) {
        referenceIdx += 1;
      }
    } else {
      if (direction == DIRECTION_AFTER) {
        referenceIdx = numColumns;
      } else {
        referenceIdx = 0;
      }
      
    }
    
    Column col = null;
    try {
      col = table.getColumn(name);
    } catch (ColumnNotDefinedException ex) {
      // column not defined yet
    }
    if (col != null) {
      // column already exists
      LayoutHints lh = col.getLayoutHints();
      if (lh != null) {
        lh.setWidth(width);
        lh.setHeader(title);
        lh.setHeaderAlign(hAlign);
        lh.setVisible(true);
      } else {
        col.setLayoutHints(new LayoutHints(
            width,
            title,
            hAlign
        ));
      }
      // checking if column already exists in visible column list ...
      for (int i = 0; i < numColumns; i++) {
        Column tmpCol = (Column) columnList.get(i);
        if (!tmpCol.getName().equals(name)) {
          continue;
        }
        // just redefinining default context for existing column definition
        defaultCellContents.set(i, defaultContents);
        return;
      }
      // Column doesn't exit yet, adding new column definition
      columnList.add(col);
      defaultCellContents.add(defaultContents);
      return;
    }
    Column newCol = new DummyColumn(name);
    newCol.setLayoutHints(new LayoutHints(
        width,
        title,
        hAlign
    ));
  
    if (referenceIdx >= numColumns) {
      columnList.add(newCol);
      defaultCellContents.add(defaultContents);
    } else {
      columnList.add(referenceIdx, newCol);
      defaultCellContents.add(referenceIdx, defaultContents);
    }
  }

  /**
   * Alters a column at the table.
   * @param name The name of the new column.
   * @param width The width of the new column.
   * @param hAlign The horizontal alignment of the new column.
   * @param title The title of the column.
   * @exception JspException In case the given reference column does not exist. 
   */
  public void alterColumn(String name, float width, int hAlign, String title) 
    throws JspException {
    
    if (columns != null) {
      return;
    }
    
    // finding reference column
    int numColumns = columnList.size();
    int coldx = -1;
    for (int i = 0; i < numColumns; i++) {
      Column col = (Column) columnList.get(i);
      if (col.getName().equals(name)) {
        coldx = i;
        LayoutHints layoutHints = col.getLayoutHints();
        if (layoutHints == null) {
          layoutHints = new LayoutHints();
          col.setLayoutHints(layoutHints);
        }
        if (width >= 0.0) {
          layoutHints.setWidth(width);
        }
        if (hAlign >= 0) {
          layoutHints.setAlign(hAlign);
        }
        if (title != null) {
          layoutHints.setHeader(title);
        }
        break;
      }
    }
    if (coldx < 0) {
      throw new JspException("The column name '" + name + "' is undefined.");
    }
    
  }

  /**
   * Removes a column from the table.
   * @param name The name of the new column.
   * @exception JspException In case the given column does not exist. 
   */
  public void removeColumn(String name) throws JspException {
    if (columns != null) {
      return;
    }
    
    // finding column
    int numColumns = columnList.size();
    for (int i = 0; i < numColumns; i++) {
      Column col = (Column) columnList.get(i);
      if (col.getName().equals(name)) {
        columnList.remove(i);
        return;
      }
    }
    throw new JspException("The column name '" + name + "', which was requested to remove, does not exist.");
  }
  
  /**
   * Tells whether or not the column styles had been defined so far.
   * @return true in case the column styles had been defined so far, else false.
   */
  protected boolean isColumnStylesDefined() {
    return columnAttributeMaps != null;
  }
  
  /**
   * Sets the column styles. 
   * @param column The column.
   * @param columnAttributes The column attribute list.
   */
  protected void setColumnAttributes(Column column, List columnAttributes) {
    if (columnAttributeMaps == null) {
      columnAttributeMaps = new HashMap();
    }
    columnAttributeMaps.put(column.getName(), columnAttributes);
  }

}
